/* TuComposer OS independent code
**
** $VER: CreateEnvelope.c 31.22 (28.7.99)
**
** (C) 1999 by Sebastian Vater (known as Basty/Seasons).
**
*/

#include <exec/types.h>
#include <tucomposer/tucomposer.h>
#include <clib/tucomposer_protos.h>

#ifdef __cplusplus
extern "C" {
#endif

void CreateNone ( WORD *EnvData, ULONG NumPoints, ULONG Scale, ULONG YOffset, ULONG ScaleType );
void CreateSine ( WORD *EnvData, ULONG NumPoints, ULONG Scale, ULONG YOffset, ULONG ScaleType );
void CreateCoSine ( WORD *EnvData, ULONG NumPoints, ULONG Scale, ULONG YOffset, ULONG ScaleType );
void CreateRamp ( WORD *EnvData, ULONG NumPoints, ULONG Scale, ULONG YOffset, ULONG ScaleType );
void CreateTriangle ( WORD *EnvData, ULONG NumPoints, ULONG Scale, ULONG YOffset, ULONG ScaleType );
void CreateSquare ( WORD *EnvData, ULONG NumPoints, ULONG Scale, ULONG YOffset, ULONG ScaleType );
void CreateSawtooth ( WORD *EnvData, ULONG NumPoints, ULONG Scale, ULONG YOffset, ULONG ScaleType );

#ifdef __cplusplus
}
#endif

extern struct TuComposerBase *TuCBase;

APTR EnvTypeTable[] = {
  CreateNone,
  CreateSine,
  CreateCoSine,
  CreateRamp,
  CreateSquare,
  CreateTriangle,
  CreateSawtooth
};

struct TuComposerInstrEnvelope *CreateEnvelope ( unsigned long NumPoints,
    unsigned long Type, unsigned long Scale, long YOffset,
    unsigned long Nodes )
{
  struct TuComposerInstrEnvelope *EnvStr = NULL;

  if ( NumPoints == 0L )
    NumPoints = DEFAULT_ENVPOINTS;

  if ( NumPoints < 0x10000 )
  {
    ULONG ScaleType;

    ScaleType = Scale & SCALE_INVERT;
    Scale &= ~(SCALE_INVERT);

    if ( Scale > MAX_SCALE )
      Scale = MAX_SCALE;

    if ( Type > ENVL_LAST )
      Type = ENVL_EMPTY;

    if ( (EnvStr = (struct TuComposerInstrEnvelope *) AllocPMem ( sizeof (struct TuComposerInstrEnvelope))))
    {
      WORD *EnvData;

      if ( (EnvData = (WORD *) AllocPMem ( NumPoints * sizeof (WORD ))))
      {
        void (**CreateEnvType)( WORD *EnvData, ULONG NumPoints, ULONG Scale, ULONG YOffset, ULONG ScaleType );

        EnvStr->tcie_Data = EnvData;
        EnvStr->tcie_NumPoints = NumPoints;

        EnvStr->tcie_LowestValue = -Scale;
        EnvStr->tcie_HighestValue = Scale;

        EnvStr->tcie_Tempo = DEFAULT_ENVTEMPO;

        InitNodeList ( (struct List *) &(EnvStr->tcie_UnknownChunks));

        (APTR) CreateEnvType = (APTR) &(EnvTypeTable);
        CreateEnvType[Type] ( EnvData, NumPoints, Scale, YOffset, ScaleType );

        if ( Nodes )
        {
          UWORD *EnvNode;

          if ( Nodes == 1 )
            Nodes++;

          if ( (EnvNode = (UWORD *) AllocPMem ( Nodes * sizeof (UWORD))))
          {
            ULONG NodeDiv, NodeMod, NodeVal = 0L, NodeCnt = 0L, i;

            EnvStr->tcie_NodePoints = EnvNode;
            EnvStr->tcie_NumNodes = Nodes;

            if ( Nodes > NumPoints )
              Nodes = NumPoints;

            NodeDiv = NumPoints / Nodes;
            NodeMod = NumPoints % Nodes;

            for (i = Nodes; i > 0; i--)
            {
              *EnvNode++ = NodeVal;

              NodeVal += NodeDiv;
              NodeCnt += NodeMod;

              if ( NodeCnt >= Nodes )
              {
                NodeCnt -= Nodes;
                NodeVal++;
              }
            }

            *--EnvNode = NumPoints - 1;
          } else
          {
            FreeEnvelope ( EnvStr );

            EnvStr = NULL;
          }
        }
      } else
      {
        FreePMem ( EnvStr, sizeof (struct TuComposerInstrEnvelope ));

        EnvStr = NULL;
      }
    }
  }

  return ( EnvStr );
}

void CreateNone ( WORD *EnvData, ULONG NumPoints, ULONG Scale, ULONG YOffset, ULONG ScaleType )
{
  ULONG i;

  for (i = NumPoints; i > 0; i--)
  {
    *EnvData++ = YOffset;
  }
}

void CreateSine ( WORD *EnvData, ULONG NumPoints, ULONG Scale, ULONG YOffset, ULONG ScaleType )
{
  ULONG i, SineDiv, SineMod, SinePos = 0L, SineCnt = 0L;
  LONG SineVal = 0L;
  WORD *SineTable = TuCBase->tcb_SineTable;

  SineDiv = 360 / NumPoints;
  SineMod = 360 % NumPoints;

  for (i = NumPoints; i > 0; i--)
  {
    SineVal = SineTable[SinePos];

    if ( ScaleType )
      SineVal = -SineVal;

    SinePos += SineDiv;

    SineVal *= (LONG) Scale;
    SineVal /= 32767;
    SineVal += YOffset;

    SineCnt += SineMod;

    if ( SineCnt >= NumPoints )
    {
      SineCnt -= NumPoints;
      SinePos++;
    }

    *EnvData++ = SineVal;
  }
}

void CreateCoSine ( WORD *EnvData, ULONG NumPoints, ULONG Scale, ULONG YOffset, ULONG ScaleType )
{
  ULONG i, SineDiv, SineMod, SineCnt = 0L;
  LONG SinePos = 90L, SineVal = 0L;
  WORD *SineTable = TuCBase->tcb_SineTable;

  SineDiv = 360 / NumPoints;
  SineMod = 360 % NumPoints;

  for (i = NumPoints; i > 0; i--)
  {
    SineVal = SineTable[SinePos];

    if ( ScaleType )
      SineVal = -SineVal;

    if ( (SinePos -= SineDiv) < 0)
      SinePos += 360;

    SineVal *= (LONG) Scale;
    SineVal /= 32767;
    SineVal += YOffset;

    SineCnt += SineMod;

    if ( SineCnt >= NumPoints )
    {
      SineCnt -= NumPoints;
      SinePos--;

      if ( SinePos < 0 )
        SinePos += 360;
    }

    *EnvData++ = SineVal;
  }
}

void CreateRamp ( WORD *EnvData, ULONG NumPoints, ULONG Scale, ULONG YOffset, ULONG ScaleType )
{
  ULONG i, StartScale = -Scale, Points, ScaleDiv, ScaleMod, ScaleCnt = 0L, RampVal;

  if ( (Points = NumPoints >> 1) == 0L )
    Points = 1;

  ScaleDiv = Scale / Points;
  ScaleMod = Scale % Points;

  for (i = NumPoints; i > 0; i--)
  {
    RampVal = StartScale;

    StartScale += ScaleDiv;
    ScaleCnt += ScaleMod;

    if ( ScaleCnt >= NumPoints )
    {
      ScaleCnt -= NumPoints;
      StartScale++;
    }

    if ( ScaleType )
      RampVal = -RampVal;

    RampVal += YOffset;

    *EnvData++ = RampVal;
  }
}

void CreateSquare ( WORD *EnvData, ULONG NumPoints, ULONG Scale, ULONG YOffset, ULONG ScaleType )
{
  ULONG i, j, SquareVal;

  if ( ScaleType )
    Scale = -Scale;

  for (i = 2; i > 0; i--)
  {
    Scale = -Scale;

    SquareVal = (Scale + YOffset);

    for (j = NumPoints / 2; j > 0; j--)
    {
      *EnvData++ = SquareVal;
    }
  }
}

void CreateTriangle ( WORD *EnvData, ULONG NumPoints, ULONG Scale, ULONG YOffset, ULONG ScaleType )
{
  ULONG i, TriangleVal, TrianglePos = 0L, TriangleDownPos, Points, ScaleDiv, ScaleMod, ScaleCnt = 0L;

  if ( (Points = NumPoints >> 2) == 0L )
    Points = 1;

  ScaleDiv = Scale / Points;
  ScaleMod = Scale % Points;

  TriangleDownPos = NumPoints - Points;

  for (i = NumPoints; i > 0; i--)
  {
    TriangleVal = TrianglePos;

    if ( TriangleDownPos >= i )
    {
      if ( TriangleDownPos == i )
      {
        ScaleCnt += ScaleMod;
        ScaleDiv = -ScaleDiv;
      }

      if ( Points >= i )
      {
        ScaleCnt += ScaleMod;
        ScaleDiv = -ScaleDiv;
        TriangleDownPos = 0L;
      }

      TrianglePos += ScaleDiv;
      ScaleCnt += ScaleMod;

      if ( ScaleCnt >= NumPoints )
      {
        ScaleCnt -= NumPoints;
        TrianglePos--;
      }
    } else
    {
      TrianglePos += ScaleDiv;
      ScaleCnt += ScaleMod;

      if ( ScaleCnt >= NumPoints )
      {
        ScaleCnt -= NumPoints;
        TrianglePos++;
      }
    }

    if ( ScaleType )
      TriangleVal = -TriangleVal;

    TriangleVal += YOffset;

    *EnvData++ = TriangleVal;
  }
}

void CreateSawtooth ( WORD *EnvData, ULONG NumPoints, ULONG Scale, ULONG YOffset, ULONG ScaleType )
{
  ULONG i, SawtoothVal, SawtoothPos = Scale, SawtoothDownPos, Points, ScaleDiv, ScaleMod, ScaleCnt = 0L;

  SawtoothDownPos = NumPoints >> 1;

  if ( (Points = NumPoints >> 2) == 0L )
    Points = 1;

  ScaleDiv = -(Scale / Points);
  ScaleMod = Scale % Points;

  for (i = NumPoints; i > 0; i--)
  {
    SawtoothVal = SawtoothPos;

    if ( SawtoothDownPos >= i )
    {
      if ( SawtoothDownPos == i )
      {
        ScaleCnt += ScaleMod;
        ScaleDiv = -ScaleDiv;
      }

      SawtoothPos += ScaleDiv;
      ScaleCnt += ScaleMod;

      if ( ScaleCnt >= NumPoints )
      {
        ScaleCnt -= NumPoints;
        SawtoothPos++;
      }
    } else
    {
      SawtoothPos += ScaleDiv;
      ScaleCnt += ScaleMod;

      if ( ScaleCnt >= NumPoints )
      {
        ScaleCnt -= NumPoints;
        SawtoothPos--;
      }
    }

    if ( ScaleType )
      SawtoothVal = -SawtoothVal;

    SawtoothVal += YOffset;

    *EnvData++ = SawtoothVal;
  }
}
