/* TuComposer OS independent code
**
** $VER: LoadEnvelope.c 31.22 (9.2.00)
**
** (C) 2000 by Sebastian Vater (known as Basty/Seasons).
**
*/

#include <exec/types.h>
#include <tucomposer/tucomposer.h>
#include <clib/dos_protos.h>
#include <clib/tucomposer_protos.h>

#ifdef __cplusplus
extern "C" {
#endif

#ifndef MOTOROLA
  UWORD ReadMWord ( UBYTE *Buffer );
  void WriteMWord ( UBYTE *Buffer, UWORD Value );

  ULONG ReadMLong ( UBYTE *Buffer );
  void WriteMLong ( UBYTE *Buffer, ULONG Value );
#else
  #define ReadMWord( Buffer ) (*(UWORD *) (Buffer))
  #define WriteMWord( Buffer, Value ) (*(UWORD *) (Buffer)) = Value

  #define ReadMLong( Buffer ) (*(ULONG *) (Buffer))
  #define WriteMLong( Buffer, Value ) (*(ULONG *) (Buffer)) = Value
#endif

#ifdef __cplusplus
}
#endif

struct TuComposerInstrEnvelope *LoadEnvelope ( struct CFHandle *CFHandle,
    struct TuComposerInstrEnvelope *EnvStr, unsigned long NumEnv )
{
  struct IFFHeader IFF_Header;
  struct IFF_EHDR EHDR;
  struct TuComposerInstrEnvelope *TmpEnv;

#ifndef MOTOROLA
  WORD *EnvData;
  UWORD *EnvNode;
#endif

  ULONG Length, IFFSize;
  ULONG Warning, Invalid = 0L;

  for (;;)
  {
    if ( (CRead ( CFHandle, &IFF_Header, sizeof (struct IFFHeader)) != sizeof (struct IFFHeader)) )
      return ( NULL );

    if ( (ReadMLong ( (UBYTE *) &IFF_Header.ifh_FORMId[0] ) != 'FORM') || ( (Length = ReadMLong ( (UBYTE *) &IFF_Header.ifh_Length[0]) ) < 4) || ((ReadMLong ( (UBYTE *) &IFF_Header.ifh_TypeId[0] ) >> 8) != 'TCE') )
      return ( NULL );

    Length -= 4;

    if (!( NumEnv-- ))
      break;

    if ( CSeek ( CFHandle, Length, OFFSET_CURRENT ) == 0xFFFFFFFF )
      return ( NULL );
  }

  if ( (ReadMLong ( (UBYTE *) &IFF_Header.ifh_TypeId[0] ) != 'TCE1' ) )
  {
    SetIoErr ( ERROR_ACTION_NOT_KNOWN );

    return ( NULL );
  }

  if (!( TmpEnv = CreateEnvelope ( 1L, ENVL_EMPTY, 0L, 0L, 0L )))
    goto LoadTCEHeaderErr;

  do
  {
    struct IFFChunk IFF_Chunk;
    struct UnknownChunk *UnkChunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      goto LoadTCEHeaderErr;

    IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0]);
    IFFLength = IFFSize;

    IFFSize += (IFFSize & 1);

    switch ( ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] ) )
    {
      case 'EHDR' :
      {
        if ( Invalid )
          goto TCEIFFDamaged;

        if ( (CRead ( CFHandle, &(EHDR.ehdr_Version), (sizeof (struct IFF_EHDR) - sizeof (struct IFFChunk))) != (sizeof (struct IFF_EHDR) - sizeof (struct IFFChunk))))
          goto LoadTCEHeaderErr;

        Warning = 0L;

        if ( (EHDR.ehdr_Version > EHDR_VERSION) )
          Warning = ERROR_ACTION_NOT_KNOWN;

        TmpEnv->tcie_Flags = ReadMWord ( (UBYTE *) &(EHDR.ehdr_Flags) );
        TmpEnv->tcie_Tempo = ReadMWord ( (UBYTE *) &(EHDR.ehdr_Tempo) );
        TmpEnv->tcie_SustainStart = ReadMWord ( (UBYTE *) &(EHDR.ehdr_SustainStart) );
        TmpEnv->tcie_SustainEnd = ReadMWord ( (UBYTE *) &(EHDR.ehdr_SustainEnd) );
        TmpEnv->tcie_SustainCount = ReadMWord ( (UBYTE *) &(EHDR.ehdr_SustainCount) );
        TmpEnv->tcie_LoopStart = ReadMWord ( (UBYTE *) &(EHDR.ehdr_LoopStart) );
        TmpEnv->tcie_LoopEnd = ReadMWord ( (UBYTE *) &(EHDR.ehdr_LoopEnd) );
        TmpEnv->tcie_LoopCount = ReadMWord ( (UBYTE *) &(EHDR.ehdr_LoopCount) );
        TmpEnv->tcie_LowestValue = ReadMWord ( (UBYTE *) &(EHDR.ehdr_LowestValue) );
        TmpEnv->tcie_HighestValue = ReadMWord ( (UBYTE *) &(EHDR.ehdr_HighestValue) );

        Invalid = ~Invalid;

        IFFLength = IFFSize - (sizeof (struct MIFF_EHDR) - sizeof (struct IFFChunk));

        break;
      }
      case 'BODY' :
      {
        if ( ReSizeEnvelope ( TmpEnv, IFFLength / sizeof (WORD)))
          goto LoadTCEHeaderErr;

        if ( CRead ( CFHandle, TmpEnv->tcie_Data, IFFLength ) != IFFLength )
          goto LoadTCEHeaderErr;

        IFFLength &= 1;

        break;
      }
      case 'NODE' :
      {
        if ( TmpEnv->tcie_NodePoints )
          goto TCEIFFDamaged;

        if (!( TmpEnv->tcie_NodePoints = (UWORD *) AllocPMem ( IFFLength )))
          goto LoadTCEHeaderErr;

        TmpEnv->tcie_NumNodes = IFFLength / sizeof (UWORD);

        if ( CRead ( CFHandle, TmpEnv->tcie_NodePoints, IFFLength ) != IFFLength )
          goto LoadTCEHeaderErr;

        IFFLength &= 1;

        break;
      }
      default :
      {
        if (!( UnkChunk = AllocUnknownChunk ( NULL, ReadMLong ( (UBYTE *) &(IFF_Chunk.iff_Identifier[0])), IFFLength )))
          goto LoadTCEHeaderErr;

        AddEnvelopeUnknownChunk ( TmpEnv, UnkChunk );

        if ( (CRead ( CFHandle, &(UnkChunk->uc_Data), IFFSize ) != IFFSize ) )
          goto LoadTCEHeaderErr;

        IFFLength &= 1;

        break;
      }
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
      {
LoadTCEHeaderErr:

        FreeEnvelope ( TmpEnv );

        return ( NULL );
      }
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      goto TCEIFFDamaged;

  } while ( (Length -= IFFSize) );

  if (!( Invalid ))
  {
TCEIFFDamaged:

    SetIoErr ( ERROR_INVALID_RESIDENT_LIBRARY );

    FreeEnvelope ( TmpEnv );

    return ( NULL );
  }

#ifndef MOTOROLA
  EnvData = TmpEnv->tcie_Data;

  for ( Length = TmpEnv->tcie_NumPoints; Length > 0; Length--)
  {
    *EnvData++ = ReadMWord ( (UBYTE *) EnvData);
  }

  EnvNode = TmpEnv->tcie_NodePoints;

  for ( Length = TmpEnv->tcie_NumNodes; Length > 0; Length--)
  {
    *EnvNode++ = ReadMWord ( (UBYTE *) EnvNode);
  }
#endif

  if (!( EnvStr ))
    EnvStr = TmpEnv;

  ReMapEnvelope ( TmpEnv, EnvStr );

  FreeEnvelope ( TmpEnv );

  SetIoErr ( Warning );

  return ( EnvStr );
}
