/* TuComposer OS independent code
**
** $VER: SaveEnvelope.c 31.22 (10.2.00)
**
** (C) 2000 by Sebastian Vater (known as Basty/Seasons).
**
*/

#include <exec/types.h>
#include <tucomposer/tucomposer.h>
#include <clib/tucomposer_protos.h>

#define SAVESAMPLE_BUFFER 4096

#ifdef __cplusplus
extern "C" {
#endif

#ifndef MOTOROLA
  UWORD ReadMWord ( UBYTE *Buffer );
  void WriteMWord ( UBYTE *Buffer, UWORD Value );

  ULONG ReadMLong ( UBYTE *Buffer );
  void WriteMLong ( UBYTE *Buffer, ULONG Value );
#else
  #define ReadMWord( Buffer ) (*(UWORD *) (Buffer))
  #define WriteMWord( Buffer, Value ) (*(UWORD *) (Buffer)) = Value

  #define ReadMLong( Buffer ) (*(ULONG *) (Buffer))
  #define WriteMLong( Buffer, Value ) (*(ULONG *) (Buffer)) = Value
#endif

ULONG SaveChunk ( struct CFHandle *CFHandle, struct IFFChunk *IFF_Chunk, unsigned long ChunkName, unsigned long ChunkLen, ULONG *ChunkLenBuf, unsigned long ErrorCode );

#ifdef __cplusplus
}
#endif

ULONG SaveEnvelope ( struct CFHandle *CFHandle,
    struct TuComposerInstrEnvelope *EnvStr )
{
  struct IFFHeader IFF_Header;
  struct IFFChunk IFF_Chunk;
  struct IFF_EHDR EHDR;
  struct UnknownChunk *UnkChunk;
  WORD *EnvData = EnvStr->tcie_Data;
  UWORD *EnvNode = EnvStr->tcie_NodePoints;
#ifndef MOTOROLA
  WORD *CopyData;
  UWORD *CopyNode;
  ULONG WriteLeft;
#endif
  ULONG TCESize = sizeof (ULONG), FORMSize;
  ULONG TCELenPos;
  ULONG FaultCode, EnvLen;

  if ( (TCELenPos = CSeek ( CFHandle, 0L, OFFSET_CURRENT )) == 0xFFFFFFFF )
    return ( ERROR_INSTR_IOERR );

  TCELenPos += sizeof (ULONG);

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0]), 'FORM' );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), TCESize );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_TypeId[0]), 'TCE1' );

  if ( (CWrite ( CFHandle, &IFF_Header, sizeof (struct IFFHeader )) != sizeof (struct IFFHeader )))
    return ( ERROR_INSTR_IOERR );

  EHDR.ehdr_Version = EHDR_VERSION;
  EHDR.ehdr_Revision = EHDR_REVISION;

  WriteMWord ( (UBYTE *) &(EHDR.ehdr_Flags), EnvStr->tcie_Flags );
  WriteMWord ( (UBYTE *) &(EHDR.ehdr_Tempo), EnvStr->tcie_Tempo );
  WriteMWord ( (UBYTE *) &(EHDR.ehdr_SustainStart), EnvStr->tcie_SustainStart );
  WriteMWord ( (UBYTE *) &(EHDR.ehdr_SustainEnd), EnvStr->tcie_SustainEnd );
  WriteMWord ( (UBYTE *) &(EHDR.ehdr_SustainCount), EnvStr->tcie_SustainCount );
  WriteMWord ( (UBYTE *) &(EHDR.ehdr_LoopStart), EnvStr->tcie_LoopStart );
  WriteMWord ( (UBYTE *) &(EHDR.ehdr_LoopEnd), EnvStr->tcie_LoopEnd );
  WriteMWord ( (UBYTE *) &(EHDR.ehdr_LoopCount), EnvStr->tcie_LoopCount );
  WriteMWord ( (UBYTE *) &(EHDR.ehdr_LowestValue), EnvStr->tcie_LowestValue );
  WriteMWord ( (UBYTE *) &(EHDR.ehdr_HighestValue), EnvStr->tcie_HighestValue );

  if ( (FaultCode = SaveChunk ( CFHandle, (struct IFFChunk *) &EHDR, 'EHDR', sizeof (struct IFF_EHDR), (ULONG *) &TCESize, ERROR_INSTR_IOERR )))
    return ( FaultCode );

  EnvLen = EnvStr->tcie_NumPoints;
  FORMSize = EnvLen * sizeof (WORD);

  WriteMLong ( (UBYTE *) &(IFF_Chunk.iff_Identifier[0]), 'BODY' );
  WriteMLong ( (UBYTE *) &(IFF_Chunk.iff_Length[0]), FORMSize );

  if ( (CWrite ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk )) != sizeof (struct IFFChunk )))
    return ( ERROR_INSTR_IOERR );

  FORMSize += (FORMSize & 1);
  TCESize += sizeof (struct IFFChunk) + FORMSize;

#ifndef MOTOROLA
  CopyData = AllocSampleData ( SAVESAMPLE_BUFFER * sizeof (WORD));

  do
  {
    WORD *Buffer = CopyData;
    ULONG i;

    WriteLeft = SAVESAMPLE_BUFFER;

    if ( EnvLen < SAVESAMPLE_BUFFER )
      WriteLeft = EnvLen;

    for (i = WriteLeft; i > 0; i--)
    {
      *Buffer++ = ReadMWord ( (UBYTE *) EnvData++ );
    }

    if ( CWrite ( CFHandle, CopyData, (WriteLeft * sizeof (WORD))) != (WriteLeft * sizeof (WORD)))
    {
      FreeSampleData ( CopyData, SAVESAMPLE_BUFFER * sizeof (WORD));

      return ( ERROR_INSTR_IOERR );
    }
  } while ( (EnvLen -= WriteLeft) );

  FreeSampleData ( CopyData, SAVESAMPLE_BUFFER * sizeof (WORD));

  if ( (EnvStr->tcie_NumPoints * sizeof (WORD)) & 1 )
  {
    *(ULONG *) &(IFF_Chunk.iff_Identifier[0]) = 0L;

    if ( (CWrite ( CFHandle, &IFF_Chunk, 1 )) != 1 )
      return ( ERROR_INSTR_IOERR );
  }
#else
  if ( (CWrite ( CFHandle, EnvStr->tcie_Data, FORMSize )) != FORMSize )
    return ( ERROR_INSTR_IOERR );
#endif

  if ( EnvNode )
  {
    EnvLen = EnvStr->tcie_NumNodes;
    FORMSize = EnvLen * sizeof (UWORD);

    WriteMLong ( (UBYTE *) &(IFF_Chunk.iff_Identifier[0]), 'NODE' );
    WriteMLong ( (UBYTE *) &(IFF_Chunk.iff_Length[0]), FORMSize );

    if ( (CWrite ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk )) != sizeof (struct IFFChunk )))
      return ( ERROR_INSTR_IOERR );

    FORMSize += (FORMSize & 1);
    TCESize += sizeof (struct IFFChunk) + FORMSize;

#ifndef MOTOROLA
    CopyNode = (UWORD *) AllocSampleData ( SAVESAMPLE_BUFFER * sizeof (UWORD));

    do
    {
      UWORD *Buffer = CopyNode;
      ULONG i;

      WriteLeft = SAVESAMPLE_BUFFER;

      if ( EnvLen < SAVESAMPLE_BUFFER )
        WriteLeft = EnvLen;

      for (i = WriteLeft; i > 0; i--)
      {
        *Buffer++ = ReadMWord ( (UBYTE *) EnvNode++ );
      }

      if ( CWrite ( CFHandle, CopyData, (WriteLeft * sizeof (WORD))) != (WriteLeft * sizeof (UWORD)))
      {
        FreeSampleData ( CopyData, SAVESAMPLE_BUFFER * sizeof (UWORD));

        return ( ERROR_INSTR_IOERR );
      }
    } while ( (EnvLen -= WriteLeft) );

    FreeSampleData ( CopyData, SAVESAMPLE_BUFFER * sizeof (UWORD));

    if ( (EnvStr->tcie_NumNodes * sizeof (UWORD)) & 1 )
    {
      *(ULONG *) &(IFF_Chunk.iff_Identifier[0]) = 0L;

      if ( (CWrite ( CFHandle, &IFF_Chunk, 1 )) != 1 )
        return ( ERROR_INSTR_IOERR );
    }
#else
    if ( (CWrite ( CFHandle, EnvStr->tcie_NodePoints, FORMSize )) != FORMSize )
      return ( ERROR_INSTR_IOERR );
#endif
  }

  UnkChunk = (struct UnknownChunk *) &(EnvStr->tcie_UnknownChunks.mlh_Head);

  for (;;)
  {
    UnkChunk = (struct UnknownChunk *) UnkChunk->uc_Node.mln_Succ;

    if (!( UnkChunk->uc_Node.mln_Succ ))
      break;

    if ( (FaultCode = SaveChunk ( CFHandle, (struct IFFChunk *) &UnkChunk, ReadMLong ( (UBYTE *) &(UnkChunk->uc_Identifier[0])), ReadMLong ( (UBYTE *) &(UnkChunk->uc_Length[0])), &TCESize, ERROR_INSTR_IOERR )))
      return ( FaultCode );
  }

  if ( CSeek ( CFHandle, TCELenPos, OFFSET_BEGINNING ) == 0xFFFFFFFF )
    return ( ERROR_INSTR_IOERR );

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), TCESize );

  if ( CWrite ( CFHandle, &(IFF_Header.ifh_Length[0]), sizeof (ULONG)) != sizeof (ULONG))
    return ( ERROR_INSTR_IOERR );

  return ( 0L );
}

ULONG SaveChunk ( struct CFHandle *CFHandle, struct IFFChunk *IFF_Chunk, unsigned long ChunkName, unsigned long ChunkLen, ULONG *ChunkLenBuf, unsigned long ErrorCode )
{
  ChunkLen -= sizeof (struct IFFChunk );

  if ( ChunkLen )
  {
    WriteMLong ( (UBYTE *) &(IFF_Chunk->iff_Identifier[0]), ChunkName );
    WriteMLong ( (UBYTE *) &(IFF_Chunk->iff_Length[0]), ChunkLen );

    if ( (CWrite ( CFHandle, IFF_Chunk, (ChunkLen + sizeof (struct IFFChunk)))) != (ChunkLen + sizeof (struct IFFChunk)))
      return ( ErrorCode );

    if ( ChunkLen & 1 )
    {
      *(ULONG *) &(IFF_Chunk->iff_Identifier[0]) = 0L;

      if ( (CWrite ( CFHandle, &IFF_Chunk, 1 )) != 1 )
        return ( ErrorCode );

      ChunkLen++;
    }

    *ChunkLenBuf += (ChunkLen + sizeof (struct IFFChunk));
  }

  return ( 0L );
}
