/* TuComposer OS independent code
**
** $VER: SplitEnvelope.c 31.22 (29.7.99)
**
** (C) 1999 by Sebastian Vater (known as Basty/Seasons).
**
*/

#include <stddef.h>
#include <string.h>
#include <exec/types.h>
#include <tucomposer/tucomposer.h>
#include <clib/tucomposer_protos.h>

struct TuComposerInstrEnvelope *SplitEnvelope (
    struct TuComposerInstrEnvelope *SrcEnvStr,
    struct TuComposerInstrEnvelope *DestEnvStr, unsigned long Point )
{
  if ( SrcEnvStr )
  {
    UWORD *NodePoints;
    ULONG Length, SplitLen;

    Length = SrcEnvStr->tcie_NumPoints;

    if ( (Point >= Length) || (Point == 0L) )
      return ( NULL );

    SplitLen = Length - Point;

    if ( DestEnvStr )
    {
      if ( ReSizeEnvelope ( DestEnvStr, SplitLen ))
        return ( NULL );

    } else
    {
      if (!( DestEnvStr = CreateEnvelope ( SplitLen, ENVL_EMPTY, 0L, 0L, 0L )))
        return ( NULL );

    }

    memcpy ( DestEnvStr->tcie_Data, SrcEnvStr->tcie_Data + Point, SplitLen * sizeof (WORD));

    ReSizeEnvelope ( SrcEnvStr, Point - 1);

    DestEnvStr->tcie_Flags = SrcEnvStr->tcie_Flags;
    memcpy ( &(SrcEnvStr->tcie_Tempo), &(DestEnvStr->tcie_Tempo), offsetof (struct TuComposerInstrEnvelope, tcie_UnknownChunks) - offsetof (struct TuComposerInstrEnvelope, tcie_Tempo));

    if ( (SrcEnvStr->tcie_SustainStart >= Point) || (SrcEnvStr->tcie_SustainEnd >= Point ))
    {
      SrcEnvStr->tcie_SustainStart = 0;
      SrcEnvStr->tcie_SustainEnd = 0;
      SrcEnvStr->tcie_SustainCount = 0;
      SrcEnvStr->tcie_Flags &= ~(TCIEF_Sustain|TCIEF_SustainPingpong);
    }

    if ( (SrcEnvStr->tcie_LoopStart >= Point) || (SrcEnvStr->tcie_LoopEnd >= Point ))
    {
      SrcEnvStr->tcie_LoopStart = 0;
      SrcEnvStr->tcie_LoopEnd = 0;
      SrcEnvStr->tcie_LoopCount = 0;
      SrcEnvStr->tcie_Flags &= ~(TCIEF_Loop|TCIEF_Pingpong);
    }

    if ( (DestEnvStr->tcie_SustainStart < Point) || (DestEnvStr->tcie_SustainEnd < Point ))
    {
      DestEnvStr->tcie_SustainStart = 0;
      DestEnvStr->tcie_SustainEnd = 0;
      DestEnvStr->tcie_SustainCount = 0;
      DestEnvStr->tcie_Flags &= ~(TCIEF_Sustain|TCIEF_SustainPingpong);
    }

    if ( (DestEnvStr->tcie_LoopStart < Point) || (DestEnvStr->tcie_LoopEnd < Point ))
    {
      DestEnvStr->tcie_LoopStart = 0;
      DestEnvStr->tcie_LoopEnd = 0;
      DestEnvStr->tcie_LoopCount = 0;
      DestEnvStr->tcie_Flags &= ~(TCIEF_Loop|TCIEF_Pingpong);
    }

    FreePMem ( DestEnvStr->tcie_NodePoints, DestEnvStr->tcie_NumNodes );

    DestEnvStr->tcie_NodePoints = NULL;
    DestEnvStr->tcie_NumNodes = 0;

    if ( (NodePoints = SrcEnvStr->tcie_NodePoints ))
    {
      ULONG NumNodes = SrcEnvStr->tcie_NumNodes, i;
      ULONG NewLen = 0L;

      for (i = NumNodes; i > 0; i--)
      {
        NewLen++;

        if ( *NodePoints++ >= Point )
          break;
      }

      if ( i > 1 )
      {
        if ( (NodePoints = (UWORD *) AllocPMem ( NewLen * sizeof (UWORD))))
        {
          UWORD *OldNodePoints = SrcEnvStr->tcie_NodePoints + NewLen;
          UWORD *NewNodePoints;

          memcpy ( NodePoints, SrcEnvStr->tcie_Data, NewLen * sizeof (UWORD));

          NodePoints[NewLen - 1] = Point - 2;

          if ( (NewNodePoints = (UWORD *) AllocPMem ( (NumNodes - NewLen) * sizeof (UWORD))))
          {
            UWORD EnvNode;

            DestEnvStr->tcie_NodePoints = NewNodePoints;
            DestEnvStr->tcie_NumNodes = (NumNodes - NewLen);

            for (i = (NumNodes - NewLen); i > 0; i--)
            {
              EnvNode = *OldNodePoints++ - Point;
              *NewNodePoints++ = EnvNode;
            }

            NewNodePoints[-1] = SplitLen - 1;

            *DestEnvStr->tcie_NodePoints = 0;

            FreePMem ( SrcEnvStr->tcie_NodePoints, NumNodes * sizeof (UWORD));

            SrcEnvStr->tcie_NodePoints = NodePoints;
            SrcEnvStr->tcie_NumNodes = NewLen;

            return ( DestEnvStr );
          }

          FreePMem ( NodePoints, NewLen * sizeof (UWORD));
        }

        FreeEnvelope ( DestEnvStr );

        DestEnvStr = NULL;
      }
    }

    return ( DestEnvStr );
  }

  return ( NULL );
}
