/* TuComposer OS independent code
**
** $VER: InternalMixer.c 31.22 (3.8.99)
**
** (C) 1999 by Sebastian Vater (known as Basty/Seasons).
**
*/

#include <exec/types.h>
#include <exec/libraries.h>
#include <tucomposer/tucomposer.h>
#include <tucomposer/external/mixer.h>
#include <clib/tucomposer_protos.h>

static struct TuComposerExtLibBase *OpenExtLib ( struct TuComposerExtLibBase *ExtLibBase );
static BPTR CloseExtLib ( struct TuComposerExtLibBase *ExtLibBase );
static BPTR ExtLib ( void );
static ULONG InternalMixerUpdate ( union FlagPass *FlagPassStr,
    unsigned long FlagPassType, unsigned long FlagNum );
static struct MixerAllocStruct *InternalMixerAlloc ( void );
static ULONG InternalMixerFree ( struct MixerAllocStruct *MixAllocStr );
static ULONG InternalMixerInit ( struct MixerAllocStruct *MixAllocStr,
    struct SoundDeviceInfo *ExtSoundDevStr, union FlagPass *FlagPassStr );
static ULONG InternalMixerChangeRate ( struct MixerAllocStruct *MixAllocStr,
    unsigned long NewMixRate );
static ULONG InternalMixerChangeTempo ( struct MixerAllocStruct *MixAllocStr,
    unsigned long NewTempo );
static ULONG InternalMixerChangeVolTable ( struct MixerAllocStruct *MixAllocStr,
    unsigned long Amplify, unsigned long LeftVol, unsigned long RightVol,
    unsigned long NumChannels );
static void InternalMixerGetChannel ( struct MixerAllocStruct *MixAllocStr,
    struct MixerStruct *MixStr, unsigned long Channel );
static void InternalMixerSetChannel ( struct MixerAllocStruct *MixAllocStr,
    struct MixerStruct *MixStr, unsigned long Channel );
static void InternalMixerSetChannelVolPanPitch ( struct MixerAllocStruct
    *MixAllocStr, struct MixerStruct *MixStr, unsigned long Channel );
static void InternalMixerSetChannelPosRepFlags ( struct MixerAllocStruct
    *MixAllocStr, struct MixerStruct *MixStr, unsigned long Channel );
static void InternalMixerDoMix ( struct MixerAllocStruct *MixAllocStr,
    LONG *MixBuffer );

extern struct MixerInfo InternalMixerBase;
static struct InternalMixerFlagStr InternalMixerSaveFlags;
static STRPTR SaveButton2 [];
static APTR InternalMixerExtFuncList [];
static APTR InternalMixerFuncList [];

static UBYTE InternalMixerName[] = "internal_mixer.library\0";
static UBYTE InternalMixerLoadID[] = "Internal Mixer\0";
static UBYTE InternalMixerIDString[] = "Internal Mixer Lib v31.00 (03 Aug 99) by Basty/Seasons.\0";

struct TuComposerExtList InternalMixer =
{
  {NULL, NULL},
  EXTLF_Intern|EXTLF_Loader|EXTLF_Saver, 0,
  (struct TuComposerExtLibBase *) &InternalMixerBase,
  NULL,
  (STRPTR) &InternalMixerName,
  (STRPTR) &InternalMixerLoadID,
  NULL,
  (struct TuComposerExtFlagStr *) &InternalMixerSaveFlags,
  0
};

struct MixerInfo InternalMixerBase =
{
  {{{NULL, NULL, NT_LIBRARY, 0, (STRPTR) &InternalMixerName},
      LIBF_SUMUSED, 0,
      0, sizeof (struct MixerInfo), /* NegSize, PosSize */
      EXTLIB_MIXER_VERSION,
      EXTLIB_REVISION,
      (STRPTR) &InternalMixerIDString,
      NULL, 0},
    NULL,
    (struct ExtFuncList *) &InternalMixerExtFuncList,
    NULL,
    (struct TuComposerExtList *) &InternalMixer},
  (struct MixerFunc *) &InternalMixerFuncList,
  MIF_StereoSupport|MIF_Surround, 0,
  22050,
  512,
  65536,
  1000, 256000,
  16, 32768,
  65535
};

static UBYTE SaveText0[] = "True 16-bit mixing\0";
static UBYTE SaveText1[] = "Stereo output\0";
static UBYTE SaveText2[] = "Interpolation\0";
static UBYTE SaveText21[] = "Disabled\0";
static UBYTE SaveText22[] = "Fast\0";
static UBYTE SaveText23[] = "Effective\0";
static UBYTE SaveText24[] = "Full\0";

static struct InternalMixerFlagStr {
  UWORD  tefs_NumEntries;
  UWORD  tefs_StructSize;
  union FlagEntryStruct tefs_Flag0;
  union FlagEntryStruct tefs_Flag1;
  union FlagEntryStruct tefs_Flag2;
} InternalMixerSaveFlags =
{
  3, sizeof (InternalMixerSaveFlags),
  {(STRPTR) &SaveText0, EXTFLAG_BOOLEAN, TEFSF_Update, 0, 0L, 0L, 0xFFFFFFFFL},
  {(STRPTR) &SaveText1, EXTFLAG_BOOLEAN, TEFSF_Update, 0, 0L, 0L, 0xFFFFFFFFL},
  {(STRPTR) &SaveButton2, EXTFLAG_BUTTON, TEFSF_Update, 1+4, 0L, 0L, 1L}
};

static STRPTR SaveButton2[] =
{
  SaveText2,
  SaveText21,
  SaveText22,
  SaveText23,
  SaveText24
};

static APTR InternalMixerExtFuncList[] =
{
  OpenExtLib,
  CloseExtLib,
  ExtLib,
  ExtLib,
  ExtLib,
  InternalMixerUpdate,
  ExtLib,
  ExtLib
};

static APTR InternalMixerFuncList[] =
{
  InternalMixerAlloc,
  InternalMixerFree,
  InternalMixerInit,
  InternalMixerChangeRate,
  InternalMixerChangeTempo,
  InternalMixerChangeVolTable,
  InternalMixerGetChannel,
  InternalMixerSetChannel,
  InternalMixerSetChannelVolPanPitch,
  InternalMixerSetChannelPosRepFlags,
  InternalMixerDoMix,
};

static struct TuComposerExtLibBase *OpenExtLib ( struct TuComposerExtLibBase *ExtLibBase )
{
  return ( NULL );
}

static BPTR CloseExtLib ( struct TuComposerExtLibBase *ExtLibBase )
{
  return ( NULL );
}

static BPTR ExtLib ( void )
{
  return ( NULL );
}

static ULONG InternalMixerUpdate ( union FlagPass *FlagPassStr,
    unsigned long FlagPassType, unsigned long FlagNum )
{
  return ( 0L );
}

static struct MixerAllocStruct *InternalMixerAlloc ( void )
{
  return ( NULL );
}

static ULONG InternalMixerFree ( struct MixerAllocStruct *MixAllocStr )
{
  return ( 0L );
}

static ULONG InternalMixerInit ( struct MixerAllocStruct *MixAllocStr,
    struct SoundDeviceInfo *ExtSoundDevStr, union FlagPass *FlagPassStr )
{
  return ( 0L );
}
static ULONG InternalMixerChangeRate ( struct MixerAllocStruct *MixAllocStr,
    unsigned long NewMixRate )
{
  return ( 0L );
}

static ULONG InternalMixerChangeTempo ( struct MixerAllocStruct *MixAllocStr,
    unsigned long NewTempo )
{
  return ( 0L );
}

static ULONG InternalMixerChangeVolTable ( struct MixerAllocStruct *MixAllocStr,
    unsigned long Amplify, unsigned long LeftVol, unsigned long RightVol,
    unsigned long NumChannels )
{
  return ( 0L );
}

static void InternalMixerGetChannel ( struct MixerAllocStruct *MixAllocStr,
    struct MixerStruct *MixStr, unsigned long Channel )
{

}

static void InternalMixerSetChannel ( struct MixerAllocStruct *MixAllocStr,
    struct MixerStruct *MixStr, unsigned long Channel )
{

}

static void InternalMixerSetChannelVolPanPitch ( struct MixerAllocStruct
    *MixAllocStr, struct MixerStruct *MixStr, unsigned long Channel )
{

}

static void InternalMixerSetChannelPosRepFlags ( struct MixerAllocStruct
    *MixAllocStr, struct MixerStruct *MixStr, unsigned long Channel )
{

}

static void InternalMixerDoMix ( struct MixerAllocStruct *MixAllocStr,
    LONG *MixBuffer )
{

}
