/* TuComposer AmigaOS dependent code
**
** $VER: InternalTCI.c 31.00 (7.2.00)
**
** (C) 2000 by Sebastian Vater (known as Basty/Seasons).
**
*/

#include <stddef.h>
#include <string.h>
#include <exec/types.h>
#include <exec/libraries.h>
#include <exec/lists.h>
#include <exec/nodes.h>
#include <tucomposer/tucomposer.h>
#include <tucomposer/synth.h>
#include <tucomposer/external/external.h>
#include <tucomposer/external/instr.h>
#include <proto/dos.h>
#include <proto/tucomposer.h>

#include "compiler.h"

#define SAVESAMPLE_BUFFER 4096

#ifdef __cplusplus
extern "C" {
#endif

#ifndef MOTOROLA
  static UWORD ReadMWord ( UBYTE *Buffer );
  static void WriteMWord ( UBYTE *Buffer, UWORD Value );

  static ULONG ReadMLong ( UBYTE *Buffer );
  static void WriteMLong ( UBYTE *Buffer, ULONG Value );
#else
  #define ReadMWord( Buffer ) (*(UWORD *) (Buffer))
  #define WriteMWord( Buffer, Value ) (*(UWORD *) (Buffer)) = Value

  #define ReadMLong( Buffer ) (*(ULONG *) (Buffer))
  #define WriteMLong( Buffer, Value ) (*(ULONG *) (Buffer)) = Value
#endif

static struct TuComposerExtLibBase *InternalTCIOpenExtLib ( void );
static BPTR InternalTCICloseExtLib ( void );
static BPTR ExtLib ( void );
static ULONG IdentifyTCI ( struct CFHandle *CFHandle );
static struct TuComposerInstr *LoadTCI ( struct CFHandle *CFHandle,
    struct TuComposerInstr *InstrStr, union FlagPass *FlagPassStr,
    unsigned long NumInstr );
static ULONG SaveTCI ( struct CFHandle *CFHandle,
    struct TuComposerInstr *InstrStr, union FlagPass *FlagPassStr );
static ULONG GetTCIInfo ( struct CFHandle *CFHandle,
    struct InstrInfo *InstrInfo, unsigned long NumInstr );
static ULONG GetNumTCIs ( struct CFHandle *CFHandle );

static ULONG LoadTCISamp ( struct CFHandle *CFHandle, struct TuComposerInstr *InstrStr, union FlagPass *FlagPassStr, unsigned long Length );
static ULONG LoadSampSmpl ( struct CFHandle *CFHandle, struct TuComposerInstr *InstrStr, union FlagPass *FlagPassStr, unsigned long Length );
static ULONG LoadSmplSnth ( struct CFHandle *CFHandle, struct TuComposerSample *SampleStr, union FlagPass *FlagPassStr, unsigned long Length );
static ULONG LoadSnthWfrm ( struct CFHandle *CFHandle, struct TuComposerSynth *SynthStr, unsigned long Length );
static ULONG LoadWfrmWave ( struct CFHandle *CFHandle, struct TuComposerSynth *SynthStr, unsigned long Length );
static ULONG LoadSnthStab ( struct CFHandle *CFHandle, struct TuComposerSynth *SynthStr, unsigned long Length );
static ULONG LoadStabSmbl ( struct CFHandle *CFHandle, struct TuComposerSynth *SynthStr, unsigned long Length );

static ULONG LoadTCIEnvl ( struct CFHandle *CFHandle, struct TuComposerModule *ModuleStr, unsigned long Length );
static ULONG LoadEnvlEnvd ( struct CFHandle *CFHandle, struct TuComposerModule *ModuleStr, unsigned long Length );

static ULONG LoadTCIArpg ( struct CFHandle *CFHandle, struct TuComposerModule *ModuleStr, unsigned long Length );
static ULONG LoadArpgArpe ( struct CFHandle *CFHandle, struct TuComposerInstrArpeggio *ArpStr, unsigned long Length );

static ULONG SaveTCIChunk ( struct CFHandle *CFHandle, struct IFFChunk *IFF_Chunk, unsigned long ChunkName, unsigned long ChunkLen, ULONG *ChunkLenBuf, unsigned long ErrorCode );
static ULONG SaveTCIName ( struct CFHandle *CFHandle, STRPTR Name, unsigned long ChunkName, ULONG *ChunkLenBuf, unsigned long ErrorCode );

static ULONG SaveInstSmpl ( struct CFHandle *CFHandle, struct TuComposerInstrEnvelope **InstrEnvStr, struct TuComposerSample **InstrSmpStr, struct TuComposerSample *SampleStr, ULONG *ChunkLenBuf, union FlagPass *FlagPassStr );
static ULONG SaveSmplSnth ( struct CFHandle *CFHandle, struct TuComposerSynth *SynthStr, ULONG *ChunkLenBuf, union FlagPass *FlagPassStr );
static ULONG SaveSnthWave ( struct CFHandle *CFHandle, struct TuComposerSynthWave *WaveformStr, ULONG *ChunkLenBuf );
static ULONG SaveSnthSmbl ( struct CFHandle *CFHandle, struct TuComposerSynthSymbolTable *SymbolStr, ULONG *ChunkLenBuf );

static ULONG SaveTCIEnvd ( struct CFHandle *CFHandle, struct TuComposerInstrEnvelope *EnvStr, ULONG *ChunkLenBuf );

static ULONG SaveTCIKbrd ( struct CFHandle *CFHandle, struct TuComposerInstrKeyboard *KeybrdStr, ULONG *ChunkLenBuf );

static ULONG SaveTCIArpg ( struct CFHandle *CFHandle, struct TuComposerInstrArpeggio *ArpStr, ULONG *ChunkLenBuf );
static ULONG SaveArpgArpe ( struct CFHandle *CFHandle, struct TuComposerInstrArpeggio *ArpStr, ULONG *ChunkLenBuf );

static ULONG GetInstrEnvelopeNum ( struct TuComposerInstrEnvelope **InstrEnvStr, struct TuComposerInstrEnvelope *EnvStr );

static void DeltaCrunch8 ( BYTE **SampleDataPos, BYTE *Buffer, BYTE *DeltaPos, ULONG Length );
static void DeltaCrunch16 ( WORD **SampleDataPos, WORD *Buffer, WORD *DeltaPos, ULONG Length );
static void DeltaCrunch32 ( LONG **SampleDataPos, LONG *Buffer, LONG *DeltaPos, ULONG Length );
static void DeltaCrunchX ( LONG **SampleDataPos, LONG *Buffer, LONG *DeltaPos, ULONG BitDepth, ULONG Length );

#ifdef __cplusplus
}
#endif

extern struct InstrLibrary InternalTCIBase;
extern struct InternalTCILoadFlagStr InternalTCILoadFlags;
extern struct InternalTCISaveFlagStr InternalTCISaveFlags;
extern STRPTR LoadButton3 [];
extern STRPTR SaveButton1 [];

static UBYTE InternalTCIName[] = "internal_tci.library\0";
static UBYTE InternalTCILoadID[] = "TuComposer Instrument (TCI)\0";
static UBYTE InternalTCIIDString[] = "Internal TCI Lib Version 1.00 (v31) (07 Feb 00) by Basty/Seasons.\0";

struct TuComposerExtList InternalTCI =
{
  {NULL, NULL},
  EXTLF_Intern|EXTLF_Loader|EXTLF_Saver, 0,
  (struct TuComposerExtLibBase *) &InternalTCIBase,
  NULL,
  (STRPTR) &InternalTCIName,
  (STRPTR) &InternalTCILoadID,
  (struct TuComposerExtFlagStr *) &InternalTCILoadFlags,
  (struct TuComposerExtFlagStr *) &InternalTCISaveFlags,
  0
};

#include "InternalTCI_library.h"

static UBYTE LoadText0[] = "Load samples\0";
static UBYTE LoadText1[] = "Sample directory\0";
static UBYTE LoadText1Default[] = "ST-01:\0";
static UBYTE LoadText2[] = "Load synth code symbols\0";
static UBYTE LoadText3[] = "Envelopes\0";
static UBYTE LoadText31[] = "Add to end\0";
static UBYTE LoadText32[] = "Add to beginning\0";
static UBYTE LoadText33[] = "Ignore\0";

static UBYTE SaveText0[] = "Save samples\0";
static UBYTE SaveText1[] = "Delta packing\0";
static UBYTE SaveText11[] = "All\0";
static UBYTE SaveText12[] = "None\0";
static UBYTE SaveText13[] = "Partial (4, 8, 12, 16, 20, 24, 28 & 32 bit)\0";
static UBYTE SaveText2[] = "Save synth code symbols\0";
static UBYTE SaveText3[] = "Save envelopes\0";

static struct InternalTCILoadFlagStr {
  UWORD  tefs_NumEntries;
  UWORD  tefs_StructSize;
  union FlagEntryBooleanStruct tefs_Flag0;
  union FlagEntryDirectoryStruct tefs_Flag1;
  union FlagEntryBooleanStruct tefs_Flag2;
  union FlagEntryButtonStruct tefs_Flag3;
} InternalTCILoadFlags =
{
  4, sizeof (InternalTCILoadFlags),
  {(STRPTR) &LoadText0, EXTFLAG_BOOLEAN, 0, 0, 0L, 0L, 0xFFFFFFFFL},
  {(STRPTR) &LoadText1, EXTFLAG_DIRECTORY, 0, 256, 0L, 0L, (STRPTR) &LoadText1Default},
  {(STRPTR) &LoadText2, EXTFLAG_BOOLEAN, 0, 0, 0L, 0L, 0xFFFFFFFFL},
  {(STRPTR *) &LoadButton3, EXTFLAG_BUTTON, 0, 1+3, 0L, 0L, 1L}
};

static struct InternalTCISaveFlagStr {
  UWORD  tefs_NumEntries;
  UWORD  tefs_StructSize;
  union FlagEntryBooleanStruct tefs_Flag0;
  union FlagEntryButtonStruct tefs_Flag1;
  union FlagEntryBooleanStruct tefs_Flag2;
  union FlagEntryBooleanStruct tefs_Flag3;
} InternalTCISaveFlags =
{
  4, sizeof (InternalTCISaveFlags),
  {(STRPTR) &SaveText0, EXTFLAG_BOOLEAN, 0, 0, 0L, 0L, 0xFFFFFFFFL},
  {(STRPTR *) &SaveButton1, EXTFLAG_BUTTON, 0, 1+3, 0L, 0L, 1L},
  {(STRPTR) &SaveText2, EXTFLAG_BOOLEAN, 0, 0, 0L, 0L, 0xFFFFFFFFL},
  {(STRPTR) &SaveText3, EXTFLAG_BOOLEAN, 0, 0, 0L, 0L, 0xFFFFFFFFL}
};

static STRPTR LoadButton3[] =
{
  LoadText3,
  LoadText31,
  LoadText32,
  LoadText33,
};

static STRPTR SaveButton1[] =
{
  SaveText1,
  SaveText11,
  SaveText12,
  SaveText13,
};

static struct TuComposerExtLibBase *InternalTCIOpenExtLib ( void )
{
  InternalTCIBase.insl_Library.extb_LibNode.lib_OpenCnt++;

  return ( (struct TuComposerExtLibBase *) &InternalTCIBase );
}

static BPTR InternalTCICloseExtLib ( void )
{
  InternalTCIBase.insl_Library.extb_LibNode.lib_OpenCnt--;

  return ( NULL );
}

static BPTR ExtLib ( void )
{
  return ( NULL );
}

static ULONG IdentifyTCI ( struct CFHandle *CFHandle )
{
  struct IFFHeader ID_IFF;

  if ( (CRead ( CFHandle, &ID_IFF, sizeof (struct IFFHeader)) != sizeof (struct IFFHeader)) )
    return ( 0L );

  if ( (ReadMLong ( (UBYTE *) &ID_IFF.ifh_FORMId[0] ) != 'FORM') || ( ReadMLong ( (UBYTE *) &ID_IFF.ifh_Length[0] ) < 4) || ( (ReadMLong ( (UBYTE *) &ID_IFF.ifh_TypeId[0]) >> 8) != 'TCI' ) )
    return ( 0L );

  return ( -1L );
}

static struct TuComposerInstr *LoadTCI ( struct CFHandle *CFHandle,
    struct TuComposerInstr *InstrStr, union FlagPass *FlagPassStr,
    ULONG NumInstr )
{
  struct IFFHeader IFF_Header;
  struct IFF_IHDR IHDR;
  struct TuComposerModule *ModuleStr;
  struct TuComposerInstr *TmpInstr;
  struct TuComposerSample *SampleStr;
  ULONG Length, IFFSize;
  ULONG Warning, Invalid = 0L;

  for (;;)
  {
    if ( (CRead ( CFHandle, &IFF_Header, sizeof (struct IFFHeader)) != sizeof (struct IFFHeader)) )
      return ( NULL );

    if ( (ReadMLong ( (UBYTE *) &IFF_Header.ifh_FORMId[0] ) != 'FORM') || ( (Length = ReadMLong ( (UBYTE *) &IFF_Header.ifh_Length[0]) ) < 4) || ((ReadMLong ( (UBYTE *) &IFF_Header.ifh_TypeId[0] ) >> 8) != 'TCI') )
      return ( NULL );

    Length -= 4;

    if (!( NumInstr-- ))
      break;

    if ( CSeek ( CFHandle, Length, OFFSET_CURRENT ) == 0xFFFFFFFF )
      return ( NULL );
  }

  if ( (ReadMLong ( (UBYTE *) &IFF_Header.ifh_TypeId[0] ) != 'TCI1' ) )
  {
    SetIoErr ( ERROR_ACTION_NOT_KNOWN );

    return ( NULL );
  }

  if (!( ModuleStr = AllocTCModule ( 1L )) )
    return ( NULL );

  if (!( TmpInstr = AllocInstr ( 0L )) )
  {
    FreeTCModule ( ModuleStr );

    return ( NULL );
  }

  AddInstr ( ModuleStr, TmpInstr );

  do
  {
    struct IFFChunk IFF_Chunk;
    struct UnknownChunk *UnkChunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      goto LoadTCIHeaderErr;

    IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0]);
    IFFLength = IFFSize;

    IFFSize += (IFFSize & 1);

    switch ( ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] ) )
    {
      case 'FILE' :
      {
        if ( TmpInstr->tci_FileName )
          break;

        if (!( TmpInstr->tci_FileName = (STRPTR) AllocPMem ( IFFLength + 1 )))
        {
GetTCIError:
          FreeTCModule ( ModuleStr );

          return ( NULL );
        }

        if ( (CRead ( CFHandle, TmpInstr->tci_FileName, IFFLength ) != IFFLength) )
          goto GetTCIError;

        IFFLength &= 1;

        break;
      }
      case 'NAME' :
      {
        if ( TmpInstr->tci_Name )
          break;

        if (!( TmpInstr->tci_Name = (STRPTR) AllocPMem ( IFFLength + 1 )))
          goto GetTCIError;

        if ( (CRead ( CFHandle, TmpInstr->tci_Name, IFFLength ) != IFFLength) )
          goto GetTCIError;

        IFFLength &= 1;

        break;
      }
      case 'IHDR' :
      {
        if ( Invalid )
          goto TCIIFFDamaged;

        if ( (CRead ( CFHandle, &(IHDR.ihdr_Version), (sizeof (struct IFF_IHDR) - sizeof (struct IFFChunk))) != (sizeof (struct IFF_IHDR) - sizeof (struct IFFChunk))))
          goto LoadTCIHeaderErr;

        Warning = 0L;

        if ( (IHDR.ihdr_Version > IHDR_VERSION) )
          Warning = ERROR_ACTION_NOT_KNOWN;

        *(ULONG *) &(TmpInstr->tci_VolumeEnvelope) = ReadMWord ( (UBYTE *) &(IHDR.ihdr_VolumeEnvNum));
        *(ULONG *) &(TmpInstr->tci_PanningEnvelope) = ReadMWord ( (UBYTE *) &(IHDR.ihdr_PanningEnvNum));
        *(ULONG *) &(TmpInstr->tci_SlideEnvelope) = ReadMWord ( (UBYTE *) &(IHDR.ihdr_SlideEnvNum));
        *(ULONG *) &(TmpInstr->tci_VibratoEnvelope) = ReadMWord ( (UBYTE *) &(IHDR.ihdr_VibratoEnvNum));
        *(ULONG *) &(TmpInstr->tci_TremoloEnvelope) = ReadMWord ( (UBYTE *) &(IHDR.ihdr_TremoloEnvNum));
        *(ULONG *) &(TmpInstr->tci_PannoloEnvelope) = ReadMWord ( (UBYTE *) &(IHDR.ihdr_PannoloEnvNum));
        *(ULONG *) &(TmpInstr->tci_ChannoloEnvelope) = ReadMWord ( (UBYTE *) &(IHDR.ihdr_ChannoloEnvNum));
        *(ULONG *) &(TmpInstr->tci_SpenoloEnvelope) = ReadMWord ( (UBYTE *) &(IHDR.ihdr_SpenoloEnvNum));
        *(ULONG *) &(TmpInstr->tci_ArpeggioStructure) = ReadMWord ( (UBYTE *) &(IHDR.ihdr_ArpeggioDefNum));
        *(ULONG *) &(TmpInstr->tci_KeyboardDefs) = ReadMWord ( (UBYTE *) &(IHDR.ihdr_KeyboardDefNum));

        memcpy ( &(TmpInstr->tci_NumSamples), &(IHDR.ihdr_NumSamples), (sizeof (struct IFF_IHDR) - offsetof (struct IFF_IHDR, ihdr_NumSamples)));

#ifndef MOTOROLA
        TmpInstr->tci_VolumeSwing = ReadMWord ( (UBYTE *) &(TmpInstr->tci_VolumeSwing) );
        TmpInstr->tci_PanningSwing = ReadMWord ( (UBYTE *) &(TmpInstr->tci_PanningSwing) );
        TmpInstr->tci_PitchSwing = ReadMLong ( (UBYTE *) &(TmpInstr->tci_PitchSwing) );
        TmpInstr->tci_PitchPanSeparation = ReadMWord ( (UBYTE *) &(TmpInstr->tci_PitchPanSeparation) );
        TmpInstr->tci_EnvUsageFlags = ReadMWord ( (UBYTE *) &(TmpInstr->tci_EnvUsageFlags) );
        TmpInstr->tci_EnvProcFlags = ReadMWord ( (UBYTE *) &(TmpInstr->tci_EnvProcFlags) );
        TmpInstr->tci_EnvRetrigFlags = ReadMWord ( (UBYTE *) &(TmpInstr->tci_EnvRetrigFlags) );
        TmpInstr->tci_EnvRandomFlags = ReadMWord ( (UBYTE *) &(TmpInstr->tci_EnvRandomFlags) );
        TmpInstr->tci_EnvRndDelayFlags = ReadMWord ( (UBYTE *) &(TmpInstr->tci_EnvRndDelayFlags) );
        TmpInstr->tci_FadeOut = ReadMWord ( (UBYTE *) &(TmpInstr->tci_FadeOut) );
        TmpInstr->tci_Hold = ReadMWord ( (UBYTE *) &(TmpInstr->tci_Hold) );
        TmpInstr->tci_Decay = ReadMWord ( (UBYTE *) &(TmpInstr->tci_Decay) );
#endif

        Invalid = ~Invalid;

        IFFLength = IFFSize - (sizeof (struct IFF_IHDR) - sizeof (struct IFFChunk));

        break;
      }
      case 'KBRD' :
      {
        struct TuComposerInstrKeyboard *KeybrdStr;

#ifndef MOTOROLA
        struct KeyboardEntry *KeybrdEntry;
        ULONG i;
#endif

        if (!( KeybrdStr = AllocKeyboard ()))
          goto GetTCIError;

        AddKeyboard ( ModuleStr, KeybrdStr );

        if ( CRead ( CFHandle, &(KeybrdStr->tcik_NoteEntry[0]), (sizeof (struct IFF_KBRD) - sizeof (struct IFFChunk))) != (sizeof (struct IFF_KBRD) - sizeof (struct IFFChunk)))
          goto GetTCIError;

#ifndef MOTOROLA
        KeybrdEntry = (struct KeyboardEntry *) &(KeybrdStr->tcik_NoteEntry[0]);

        for (i = NUMOCTAVES * NUMNOTES; i > 0; i--)
        {
          KeybrdEntry->tcik_Sample_C_0 = ReadMWord ( (UBYTE *) &(KeybrdEntry->tcik_Sample_C_0));

          KeybrdEntry++;
        }
#endif
        IFFLength = IFFSize - (sizeof (struct IFF_KBRD) - sizeof (struct IFFChunk));

        break;
      }
      case 'FORM' :
      {
        ULONG FaultCode;

        if ( (CRead ( CFHandle, &IFF_Header, 4 ) != 4 ) )
          goto LoadTCIHeaderErr;

        switch ( ReadMLong ( (UBYTE *) &IFF_Header.ifh_FORMId[0] ) )
        {
          case 'SAMP' :
          {
            if ( (FaultCode = LoadTCISamp ( CFHandle, TmpInstr, FlagPassStr, IFFSize )))
            {
SetFaultCode:

              SetIoErr ( FaultCode );

              FreeTCModule ( ModuleStr );

              return ( NULL );
            }

            IFFLength = 0L;

            break;
          }
          case 'ENVL' :
          {
            if ( FlagPassStr[3].flg_Value == 3 )
              break;

            if ( (FaultCode = LoadTCIEnvl ( CFHandle, ModuleStr, IFFSize )))
              goto SetFaultCode;

            IFFLength = 0L;

            break;
          }
          case 'ARPG' :
          {
            if ( FlagPassStr[3].flg_Value == 3 )
              break;

            if ( (FaultCode = LoadTCIArpg ( CFHandle, ModuleStr, IFFSize )))
              goto SetFaultCode;

            IFFLength = 0L;

            break;
          }
          default :
          {
            if ( CSeek ( CFHandle, -4, OFFSET_CURRENT ) == 0xFFFFFFFF )
              goto LoadTCIHeaderErr;

            goto AllocUnkChunk;
          }
        }

        break;
      }
      default :
      {
AllocUnkChunk:

        if (!( UnkChunk = AllocUnknownChunk ( NULL, ReadMLong ( (UBYTE *) &(IFF_Chunk.iff_Identifier[0])), IFFLength )))
          goto LoadTCIHeaderErr;

        AddInstrUnknownChunk ( TmpInstr, UnkChunk );

        if ( (CRead ( CFHandle, &(UnkChunk->uc_Data), IFFLength ) != IFFLength ) )
          goto LoadTCIHeaderErr;

        IFFLength &= 1;

        break;
      }
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
      {
LoadTCIHeaderErr:

        FreeTCModule ( ModuleStr );

        return ( NULL );
      }
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      goto TCIIFFDamaged;

  } while ( (Length -= IFFSize) );

  if (!( Invalid ))
  {
TCIIFFDamaged:

    SetIoErr ( ERROR_INVALID_RESIDENT_LIBRARY );

    FreeTCModule ( ModuleStr );

    return ( NULL );
  }

  TmpInstr->tci_VolumeEnvelope = GetEnvelopeAddr ( ModuleStr, *(ULONG *) &(TmpInstr->tci_VolumeEnvelope));
  TmpInstr->tci_PanningEnvelope = GetEnvelopeAddr ( ModuleStr, *(ULONG *) &(TmpInstr->tci_PanningEnvelope));
  TmpInstr->tci_SlideEnvelope = GetEnvelopeAddr ( ModuleStr, *(ULONG *) &(TmpInstr->tci_SlideEnvelope));
  TmpInstr->tci_VibratoEnvelope = GetEnvelopeAddr ( ModuleStr, *(ULONG *) &(TmpInstr->tci_VibratoEnvelope));
  TmpInstr->tci_TremoloEnvelope = GetEnvelopeAddr ( ModuleStr, *(ULONG *) &(TmpInstr->tci_TremoloEnvelope));
  TmpInstr->tci_PannoloEnvelope = GetEnvelopeAddr ( ModuleStr, *(ULONG *) &(TmpInstr->tci_PannoloEnvelope));
  TmpInstr->tci_ChannoloEnvelope = GetEnvelopeAddr ( ModuleStr, *(ULONG *) &(TmpInstr->tci_ChannoloEnvelope));
  TmpInstr->tci_SpenoloEnvelope = GetEnvelopeAddr ( ModuleStr, *(ULONG *) &(TmpInstr->tci_SpenoloEnvelope));
  TmpInstr->tci_ArpeggioStructure = GetArpeggioAddr ( ModuleStr, *(ULONG *) &(TmpInstr->tci_ArpeggioStructure));
  TmpInstr->tci_KeyboardDefs = GetKeyboardAddr ( ModuleStr, *(ULONG *) &(TmpInstr->tci_KeyboardDefs));

  SampleStr = (struct TuComposerSample *) &(TmpInstr->tci_Samples.mlh_Head);

  for (;;)
  {
    SampleStr = (struct TuComposerSample *) SampleStr->tcs_Node.mln_Succ;

    if (!( SampleStr->tcs_Node.mln_Succ ))
      break;

    if ( SampleStr->tcs_Flags & TCSF_Redirect )
    {
      struct TuComposerSample *SampleScan;
      ULONG Cnt = *(ULONG *) &(SampleStr->tcs_UserData);

      SampleScan = (struct TuComposerSample *) &(TmpInstr->tci_Samples.mlh_Head);

      for (;;)
      {
        SampleScan = (struct TuComposerSample *) SampleScan->tcs_Node.mln_Succ;

        if (!( SampleScan->tcs_Node.mln_Succ ))
          break;

        if ( --Cnt )
          continue;

        SampleStr->tcs_Data = SampleScan->tcs_Data;
        SampleStr->tcs_Length = ((SampleStr->tcs_Samples * SampleStr->tcs_BitDepth) + 7UL) >> 3UL;
        SampleStr->tcs_UserData = NULL;

        break;
      }

      SampleStr->tcs_AutoVibratoEnv = GetEnvelopeAddr ( ModuleStr, *(ULONG *) &(SampleStr->tcs_AutoVibratoEnv));
      SampleStr->tcs_AutoTremoloEnv = GetEnvelopeAddr ( ModuleStr, *(ULONG *) &(SampleStr->tcs_AutoTremoloEnv));
      SampleStr->tcs_AutoPannoloEnv = GetEnvelopeAddr ( ModuleStr, *(ULONG *) &(SampleStr->tcs_AutoPannoloEnv));
    }
  }

  switch ( FlagPassStr[3].flg_Value )
  {
    case 1 :
    {
      AddInstrEnvelopes ( InstrStr, ENVF_Tail );

      break;
    }
    case 2 :
    {
      AddInstrEnvelopes ( InstrStr, 0L );

      break;
    }
  }

  if (!( InstrStr ))
    InstrStr = TmpInstr;

  ReMapInstr ( TmpInstr, InstrStr );

  FreeTCModule ( ModuleStr );

  SetIoErr ( Warning );

  return ( InstrStr );
}

static ULONG LoadTCISamp ( struct CFHandle *CFHandle, struct TuComposerInstr *InstrStr, union FlagPass *FlagPassStr, unsigned long Length )
{
  struct IFFHeader IFF_Header;
  ULONG IFFSize;

  if ( Length < 4 )
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  Length -= 4;

  do
  {
    struct IFFChunk IFF_Chunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      return ( IoErr ());

    if ( (IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0] )) & 1 )
      IFFSize++;

    IFFLength = IFFSize;

    if ( (ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] ) ) == 'FORM' )
    {
      ULONG FaultCode;

      if ( (CRead ( CFHandle, &IFF_Header, 4 ) != 4 ) )
        return ( IoErr ());

      if ( (ReadMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0])) ) == 'SMPL' )
      {
        if ( (FaultCode = LoadSampSmpl ( CFHandle, InstrStr, FlagPassStr, IFFSize )))
          return ( FaultCode );

        IFFLength = 0L;
      }
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
        return ( IoErr ());
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      return ( ERROR_INVALID_RESIDENT_LIBRARY );

  } while ( (Length -= IFFSize) );

  return ( 0L );
}

static ULONG LoadSampSmpl ( struct CFHandle *CFHandle, struct TuComposerInstr *InstrStr, union FlagPass *FlagPassStr, unsigned long Length )
{
  struct IFFHeader IFF_Header;
  struct IFF_SMPH SMPH;
  struct IFF_SMPR SMPR;
  struct TuComposerSample *SampleStr;
  ULONG Invalid = 0L, IFFSize;

  if ( Length < 4 )
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  Length -= 4;

  if (!( SampleStr = AllocSample ( NULL, 0L )))
    return ( IoErr ());

  if ( AddSample ( InstrStr, SampleStr ))
    return ( ERROR_TOO_MANY_LEVELS );


  do
  {
    struct IFFChunk IFF_Chunk;
    struct UnknownChunk *UnkChunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      return ( IoErr ());

    IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0]);
    IFFLength = IFFSize;

    IFFSize += (IFFSize & 1);

    switch ( ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] ) )
    {
      case 'FILE' :
      {
        if ( SampleStr->tcs_FileName )
          break;

        if (!( SampleStr->tcs_FileName = (STRPTR) AllocPMem ( IFFLength + 1 )))
          return ( IoErr ());

        if ( (CRead ( CFHandle, SampleStr->tcs_FileName, IFFLength ) != IFFLength) )
          return ( IoErr ());

        IFFLength &= 1;

        break;
      }
      case 'NAME' :
      {
        if ( SampleStr->tcs_Name )
          break;

        if (!( SampleStr->tcs_Name = (STRPTR) AllocPMem ( IFFLength + 1 )))
          return ( IoErr ());

        if ( (CRead ( CFHandle, SampleStr->tcs_Name, IFFLength ) != IFFLength) )
          return ( IoErr ());

        IFFLength &= 1;

        break;
      }
      case 'SMPH' :
      {
        if ( Invalid )
          return ( ERROR_INVALID_RESIDENT_LIBRARY );

        if ( (CRead ( CFHandle, &(SMPH.smph_Samples), (sizeof (struct IFF_SMPH) - sizeof (struct IFFChunk))) != (sizeof (struct IFF_SMPH) - sizeof (struct IFFChunk))))
          return ( IoErr ());

        *(ULONG *) &(SampleStr->tcs_AutoVibratoEnv) = ReadMWord ( (UBYTE *) &(SMPH.smph_AutoVibratoNum));
        *(ULONG *) &(SampleStr->tcs_AutoTremoloEnv) = ReadMWord ( (UBYTE *) &(SMPH.smph_AutoTremoloNum));
        *(ULONG *) &(SampleStr->tcs_AutoPannoloEnv) = ReadMWord ( (UBYTE *) &(SMPH.smph_AutoPannoloNum));

        memcpy ( &(SampleStr->tcs_Samples), &(SMPH.smph_Samples), (offsetof (struct IFF_SMPH, smph_AutoVibratoNum) - sizeof (struct IFFChunk)));

#ifndef MOTOROLA
        SampleStr->tcs_Samples = ReadMLong ( (UBYTE *) &(SampleStr->tcs_Samples) );
        SampleStr->tcs_Repeat = ReadMLong ( (UBYTE *) &(SampleStr->tcs_Repeat) );
        SampleStr->tcs_RepLen = ReadMLong ( (UBYTE *) &(SampleStr->tcs_RepLen) );
        SampleStr->tcs_RepCount = ReadMLong ( (UBYTE *) &(SampleStr->tcs_RepCount) );
        SampleStr->tcs_SustainRepeat = ReadMLong ( (UBYTE *) &(SampleStr->tcs_SustainRepeat) );
        SampleStr->tcs_SustainRepLen = ReadMLong ( (UBYTE *) &(SampleStr->tcs_SustainRepLen) );
        SampleStr->tcs_SustainRepCount = ReadMLong ( (UBYTE *) &(SampleStr->tcs_SustainRepCount) );
        SampleStr->tcs_Rate = ReadMLong ( (UBYTE *) &(SampleStr->tcs_Rate) );
        SampleStr->tcs_StartOffset = ReadMLong ( (UBYTE *) &(SampleStr->tcs_StartOffset) );
        SampleStr->tcs_LowerRateLimit = ReadMLong ( (UBYTE *) &(SampleStr->tcs_LowerRateLimit) );
        SampleStr->tcs_UpperRateLimit = ReadMLong ( (UBYTE *) &(SampleStr->tcs_UpperRateLimit) );
#endif

        memcpy ( &(SampleStr->tcs_EnvUsageFlags), &(SMPH.smph_EnvUsageFlags), (sizeof (struct IFF_SMPH) - offsetof (struct IFF_SMPH, smph_EnvUsageFlags)));

#ifndef MOTOROLA
        SampleStr->tcs_VibratoSweep = ReadMWord ( (UBYTE *) &(SampleStr->tcs_VibratoSweep) );
        SampleStr->tcs_TremoloSweep = ReadMWord ( (UBYTE *) &(SampleStr->tcs_TremoloSweep) );
        SampleStr->tcs_PannoloSweep = ReadMWord ( (UBYTE *) &(SampleStr->tcs_PannoloSweep) );
#endif

        Invalid = ~Invalid;

        IFFLength = IFFSize - (sizeof (struct IFF_SMPH) - sizeof (struct IFFChunk));

        break;
      }
      case 'BODY' :
      {
        if ( SampleStr->tcs_Length )
          return ( ERROR_INVALID_RESIDENT_LIBRARY );

        SampleStr->tcs_Length = ~SampleStr->tcs_Length;

        if ( FlagPassStr[0].flg_Value )
        {
          SampleStr->tcs_Length = IFFLength;

          if (!( SampleStr->tcs_Data = AllocSampleData ( IFFLength )))
            return ( IoErr ());

          if ( CRead ( CFHandle, SampleStr->tcs_Data, IFFLength ) != IFFLength )
            return ( IoErr ());

          IFFLength &= 1;
        }

        break;
      }
      case 'SMPR' :
      {
        if ( SampleStr->tcs_Length )
          return ( ERROR_INVALID_RESIDENT_LIBRARY );

        SampleStr->tcs_Length = ~SampleStr->tcs_Length;

        if ( CRead ( CFHandle, &(SMPR.smpr_SampleNum), (sizeof (struct IFF_SMPR) - sizeof (struct IFFChunk))) != (sizeof (struct IFF_SMPR) - sizeof (struct IFFChunk)))
          return ( IoErr ());

        *(ULONG *) &(SampleStr->tcs_UserData) = ReadMLong ( (UBYTE *) &(SMPR.smpr_SampleNum));

        IFFLength &= 1;

        break;
      }
      case 'FORM' :
      {
        ULONG FaultCode;

        if ( (CRead ( CFHandle, &IFF_Header, 4 ) != 4 ) )
          return ( IoErr ());

        if ( (ReadMLong ( (UBYTE *) &IFF_Header.ifh_FORMId[0] ) == 'SNTH' ))
        {
          if ( (FaultCode = LoadSmplSnth ( CFHandle, SampleStr, FlagPassStr, IFFSize )))
            return ( FaultCode );

          IFFLength = 0L;
        } else
        {
          if ( CSeek ( CFHandle, -4, OFFSET_CURRENT ) == 0xFFFFFFFF )
            return ( FaultCode );

          goto AllocUnkChunk;
        }

        break;
      }
      default :
      {
AllocUnkChunk:

        if (!( UnkChunk = AllocUnknownChunk ( NULL, ReadMLong ( (UBYTE *) &(IFF_Chunk.iff_Identifier[0])), IFFLength )))
          return ( IoErr ());

        AddSampleUnknownChunk ( SampleStr, UnkChunk );

        if ( (CRead ( CFHandle, &(UnkChunk->uc_Data), IFFLength ) != IFFLength ) )
          return ( IoErr ());

        IFFLength &= 1;

        break;
      }
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
        return ( IoErr ());
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      return ( ERROR_INVALID_RESIDENT_LIBRARY );

  } while ( (Length -= IFFSize) );

  if (!( Invalid ))
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  if (!(*(ULONG *) &(SampleStr->tcs_UserData)) )
  {
    if ( SampleStr->tcs_Samples && ( SampleStr->tcs_Length == 0L ) )
    {
      struct CFHandle *LoadSmpHandle;
      struct TuComposerExtList *SmpExtList;
      union FlagPass *SmpFlagPassStr = NULL;
      BPTR OldLock;
      BPTR SmpDirLock;
      ULONG SmpDefaultSize = 0L;
      ULONG Error;

      if (!( SmpDirLock = Lock ( (STRPTR) FlagPassStr[1].flg_Pointer, SHARED_LOCK )))
        return ( IoErr ());

      if (!( OldLock = CurrentDir ( SmpDirLock )))
      {
        Error = IoErr ();

        UnLock ( SmpDirLock );

        return ( Error );
      }

      if (!( LoadSmpHandle = COpen ( SampleStr->tcs_FileName, MODE_OLDFILE, NULL )))
      {
        Error = IoErr ();

        CurrentDir ( OldLock );
        UnLock ( SmpDirLock );

        return ( Error );
      }

      if ( ( SmpExtList = IdentifySample ( LoadSmpHandle )))
      {
        if ( SmpExtList->extl_LoadFlagStr )
          SmpDefaultSize = SmpExtList->extl_LoadFlagStr->tefs_NumEntries;

        if ( SmpDefaultSize )
        {
          union FlagEntryStruct *FlagEntry = (union FlagEntryStruct *) &(SmpExtList->extl_LoadFlagStr->tefs_Flag);
          union FlagPass *SmpStoreFlagPass = SmpFlagPassStr;
          ULONG i;

          if (!( SmpFlagPassStr = (union FlagPass *) AllocPMem ( SmpDefaultSize * sizeof (LONG))))
          {
            Error = IoErr ();

            CClose ( LoadSmpHandle );

            CurrentDir ( OldLock );
            UnLock ( SmpDirLock );

            return ( Error );
          }

          for (i = SmpDefaultSize; i > 0; i--)
          {
            (*SmpStoreFlagPass++).flg_Value = FlagEntry->tefs_FlagInteger.tefs_FlagDefault;

            FlagEntry++;
          }
        }

        SampleStr = LoadSample ( LoadSmpHandle, SampleStr, SmpExtList, SmpFlagPassStr, 0L );
      } else
      {
        SampleStr = LoadSampleData ( LoadSmpHandle, SampleStr, SampleStr->tcs_Length );
      }

      Error = IoErr ();

      FreePMem ( SmpFlagPassStr, SmpDefaultSize );

      CClose ( LoadSmpHandle );

      CurrentDir ( OldLock );
      UnLock ( SmpDirLock );

      if (!( SampleStr ))
        return ( Error );

    } else
    {
#ifndef MOTOROLA
      if ( SampleStr->tcs_BitDepth != 8 )
      {
        ULONG i;

        if ( SampleStr->tcs_BitDepth == 16 )
        {
          WORD *SampleData = (WORD *) SampleStr->tcs_Data;

          for ( i = ((SampleStr->tcs_Samples + 3UL) >> 2UL); i > 0; i--)
          {
            *SampleData++ = ReadMWord ( (UBYTE *) SampleData );
            *SampleData++ = ReadMWord ( (UBYTE *) SampleData );
            *SampleData++ = ReadMWord ( (UBYTE *) SampleData );
            *SampleData++ = ReadMWord ( (UBYTE *) SampleData );
          }
        } else
        {
          LONG *SampleData = (LONG *) SampleStr->tcs_Data;

          for ( i = ((SampleStr->tcs_Samples + 1UL) >> 1UL); i > 0; i--)
          {
            *SampleData++ = ReadMLong ( (UBYTE *) SampleData );
            *SampleData++ = ReadMLong ( (UBYTE *) SampleData );
          }
        }
      }
#endif

      if ( SampleStr->tcs_Flags & SMPLF_Delta )
        DecrunchSample ( SampleStr, 0L );
    }

    SampleStr->tcs_Flags &= ~(TCSF_Redirect);
  }

  return ( 0L );
}

static ULONG LoadSmplSnth ( struct CFHandle *CFHandle, struct TuComposerSample *SampleStr, union FlagPass *FlagPassStr, unsigned long Length )
{
  struct IFFHeader IFF_Header;
  struct MIFF_YHDR YHDR;
  struct TuComposerSynth *SynthStr;
  ULONG IFFSize;
  UBYTE Invalid = 0;

  if ( Length < 4 )
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  Length -= 4;

  if (!( SynthStr = AllocSynth ( 1L, 0L, 0L )))
    return ( IoErr ());

  AssignSynth ( SampleStr, SynthStr );

  do
  {
    struct IFFChunk IFF_Chunk;
    struct UnknownChunk *UnkChunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      return ( IoErr ());

    IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0]);
    IFFLength = IFFSize;

    IFFSize += (IFFSize & 1);

    switch ( ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] ) )
    {
      case 'FILE' :
      {
        if ( SynthStr->tcy_FileName )
          break;

        if (!( SynthStr->tcy_FileName = (STRPTR) AllocPMem ( IFFLength + 1 )))
          return ( IoErr ());

        if ( (CRead ( CFHandle, SynthStr->tcy_FileName, IFFLength ) != IFFLength) )
          return ( IoErr ());

        IFFLength &= 1;

        break;
      }
      case 'NAME' :
      {
        if ( SynthStr->tcy_Name )
          break;

        if (!( SynthStr->tcy_Name = (STRPTR) AllocPMem ( IFFLength + 1 )))
          return ( IoErr ());

        if ( (CRead ( CFHandle, SynthStr->tcy_Name, IFFLength ) != IFFLength) )
          return ( IoErr ());

        IFFLength &= 1;

        break;
      }
      case 'YHDR' :
      {
        if ( Invalid & 1 )
          return ( ERROR_INVALID_RESIDENT_LIBRARY );

        if ( (CRead ( CFHandle, &(YHDR.myhdr_NumWaves), (sizeof (struct MIFF_YHDR) - sizeof (struct IFFChunk))) != (sizeof (struct MIFF_YHDR) - sizeof (struct IFFChunk))))
          return ( IoErr ());

        memcpy ( &(SynthStr->tcy_VolumePos), &(YHDR.myhdr_VolumePos), (sizeof (struct MIFF_YHDR) - offsetof (struct MIFF_YHDR, myhdr_VolumePos)));

#ifndef MOTOROLA
        SynthStr->tcy_VolumePos = ReadMWord ( (UBYTE *) &(SynthStr->tcy_VolumePos) );
        SynthStr->tcy_PanningPos = ReadMWord ( (UBYTE *) &(SynthStr->tcy_PanningPos) );
        SynthStr->tcy_SlidePos = ReadMWord ( (UBYTE *) &(SynthStr->tcy_SlidePos) );
        SynthStr->tcy_SpecialPos = ReadMWord ( (UBYTE *) &(SynthStr->tcy_SpecialPos) );
        SynthStr->tcy_VolumeSusPos = ReadMWord ( (UBYTE *) &(SynthStr->tcy_VolumeSusPos) );
        SynthStr->tcy_PanningSusPos = ReadMWord ( (UBYTE *) &(SynthStr->tcy_PanningSusPos) );
        SynthStr->tcy_SlideSusPos = ReadMWord ( (UBYTE *) &(SynthStr->tcy_SlideSusPos) );
        SynthStr->tcy_SpecialSusPos = ReadMWord ( (UBYTE *) &(SynthStr->tcy_SpecialSusPos) );
        SynthStr->tcy_VolNNAPos = ReadMWord ( (UBYTE *) &(SynthStr->tcy_VolNNAPos) );
        SynthStr->tcy_PanNNAPos = ReadMWord ( (UBYTE *) &(SynthStr->tcy_PanNNAPos) );
        SynthStr->tcy_SlideNNAPos = ReadMWord ( (UBYTE *) &(SynthStr->tcy_SlideNNAPos) );
        SynthStr->tcy_SpecialNNAPos = ReadMWord ( (UBYTE *) &(SynthStr->tcy_SpecialNNAPos) );
        SynthStr->tcy_VolDNAPos = ReadMWord ( (UBYTE *) &(SynthStr->tcy_VolDNAPos) );
        SynthStr->tcy_PanDNAPos = ReadMWord ( (UBYTE *) &(SynthStr->tcy_PanDNAPos) );
        SynthStr->tcy_SlideDNAPos = ReadMWord ( (UBYTE *) &(SynthStr->tcy_SlideDNAPos) );
        SynthStr->tcy_SpecialDNAPos = ReadMWord ( (UBYTE *) &(SynthStr->tcy_SpecialDNAPos) );
        SynthStr->tcy_Variable0 = ReadMWord ( (UBYTE *) &(SynthStr->tcy_Variable0) );
        SynthStr->tcy_Variable1 = ReadMWord ( (UBYTE *) &(SynthStr->tcy_Variable1) );
        SynthStr->tcy_Variable2 = ReadMWord ( (UBYTE *) &(SynthStr->tcy_Variable2) );
        SynthStr->tcy_Variable3 = ReadMWord ( (UBYTE *) &(SynthStr->tcy_Variable3) );
        SynthStr->tcy_Variable4 = ReadMWord ( (UBYTE *) &(SynthStr->tcy_Variable4) );
        SynthStr->tcy_Variable5 = ReadMWord ( (UBYTE *) &(SynthStr->tcy_Variable5) );
        SynthStr->tcy_Variable6 = ReadMWord ( (UBYTE *) &(SynthStr->tcy_Variable6) );
        SynthStr->tcy_Variable7 = ReadMWord ( (UBYTE *) &(SynthStr->tcy_Variable7) );
        SynthStr->tcy_Variable8 = ReadMWord ( (UBYTE *) &(SynthStr->tcy_Variable8) );
        SynthStr->tcy_Variable9 = ReadMWord ( (UBYTE *) &(SynthStr->tcy_Variable9) );
        SynthStr->tcy_VariableA = ReadMWord ( (UBYTE *) &(SynthStr->tcy_VariableA) );
        SynthStr->tcy_VariableB = ReadMWord ( (UBYTE *) &(SynthStr->tcy_VariableB) );
        SynthStr->tcy_VariableC = ReadMWord ( (UBYTE *) &(SynthStr->tcy_VariableC) );
        SynthStr->tcy_VariableD = ReadMWord ( (UBYTE *) &(SynthStr->tcy_VariableD) );
        SynthStr->tcy_VariableE = ReadMWord ( (UBYTE *) &(SynthStr->tcy_VariableE) );
        SynthStr->tcy_VariableF = ReadMWord ( (UBYTE *) &(SynthStr->tcy_VariableF) );
        SynthStr->tcy_VolConVar = ReadMWord ( (UBYTE *) &(SynthStr->tcy_VolConVar) );
        SynthStr->tcy_PanConVar = ReadMWord ( (UBYTE *) &(SynthStr->tcy_PanConVar) );
        SynthStr->tcy_SlideConVar = ReadMWord ( (UBYTE *) &(SynthStr->tcy_SlideConVar) );
        SynthStr->tcy_SpecialConVar = ReadMWord ( (UBYTE *) &(SynthStr->tcy_SpecialConVar) );
        SynthStr->tcy_VarKeepMask = ReadMWord ( (UBYTE *) &(SynthStr->tcy_VarKeepMask) );
#endif

        Invalid |= 1;

        IFFLength = IFFSize - (sizeof (struct MIFF_YHDR) - sizeof (struct IFFChunk));

        break;
      }
      case 'CODE' :
      {
        struct TuComposerSynthCode *SynthCodeStr;

        if ( Invalid & 2 )
          return ( ERROR_INVALID_RESIDENT_LIBRARY );

        if (!( SynthCodeStr = AllocSynthCode ( IFFSize / sizeof (struct TuComposerSynthCode) )))
          return ( IoErr ());

        AssignSynthCode ( SynthStr, SynthCodeStr, IFFSize / sizeof (struct TuComposerSynthCode));

        if ( CRead ( CFHandle, SynthCodeStr, IFFSize ) != IFFSize )
          return ( IoErr ());

#ifndef MOTOROLA
        for (IFFLength = SynthStr->tcy_CodeEntries; IFFLength > 0; IFFLength--)
        {
          SynthCodeStr->tcyc_InstData = ReadMWord ( (UBYTE *) &(SynthCodeStr->tcyc_InstData) );

          SynthCodeStr++;
        }
#endif
        IFFLength &= 1;

        Invalid |= 2;

        break;
      }
      case 'FORM' :
      {
        ULONG FaultCode;

        if ( (CRead ( CFHandle, &IFF_Header, 4 ) != 4 ) )
          return ( IoErr ());

        switch ( ReadMLong ( (UBYTE *) &IFF_Header.ifh_FORMId[0] ) )
        {
          case 'WFRM' :
          {
            if ( (FaultCode = LoadSnthWfrm ( CFHandle, SynthStr, IFFSize )))
              return ( FaultCode );

            IFFLength = 0L;

            break;
          }
          case 'STAB' :
          {
            if (!( FlagPassStr[2].flg_Value ))
              break;

            if ( (FaultCode = LoadSnthStab ( CFHandle, SynthStr, IFFSize )))
              return ( FaultCode );

            IFFLength = 0L;

            break;
          }
          default :
          {
            if ( CSeek ( CFHandle, -4, OFFSET_CURRENT ) == 0xFFFFFFFF )
              return ( FaultCode );

            goto AllocUnkChunk;
          }
        }

        break;
      }
      default :
      {
AllocUnkChunk:

        if (!( UnkChunk = AllocUnknownChunk ( NULL, ReadMLong ( (UBYTE *) &(IFF_Chunk.iff_Identifier[0])), IFFLength )))
          return ( IoErr ());

        AddSynthUnknownChunk ( SynthStr, UnkChunk );

        if ( (CRead ( CFHandle, &(UnkChunk->uc_Data), IFFLength ) != IFFLength ) )
          return ( IoErr ());

        IFFLength &= 1;

        break;
      }
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
        return ( IoErr ());
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      return ( ERROR_INVALID_RESIDENT_LIBRARY );

  } while ( (Length -= IFFSize) );

  if ( Invalid < 3 )
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  return ( 0L );
}

static ULONG LoadSnthWfrm ( struct CFHandle *CFHandle, struct TuComposerSynth *SynthStr, unsigned long Length )
{
  struct IFFHeader IFF_Header;
  ULONG IFFSize;

  if ( Length < 4 )
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  Length -= 4;

  do
  {
    struct IFFChunk IFF_Chunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      return ( IoErr ());

    if ( (IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0] )) & 1 )
      IFFSize++;

    IFFLength = IFFSize;

    if ( (ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] ) ) == 'FORM' )
    {
      ULONG FaultCode;

      if ( (CRead ( CFHandle, &IFF_Header, 4 ) != 4 ) )
        return ( IoErr ());

      if ( (ReadMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0])) ) == 'WAVE' )
      {
        if ( (FaultCode = LoadWfrmWave ( CFHandle, SynthStr, IFFSize )))
          return ( FaultCode );

        IFFLength = 0L;
      }
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
        return ( IoErr ());
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      return ( ERROR_INVALID_RESIDENT_LIBRARY );

  } while ( (Length -= IFFSize) );

  return ( 0L );
}

static ULONG LoadWfrmWave ( struct CFHandle *CFHandle, struct TuComposerSynth *SynthStr, unsigned long Length )
{
  struct IFF_WHDR WHDR;
  struct TuComposerSynthWave *WaveformStr;
  ULONG IFFSize;
  UBYTE Invalid = 0;

  if ( Length < 4 )
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  Length -= 4;

  if (!( WaveformStr = AllocWaveform ( 1L )))
    return ( IoErr ());

  if ( AddWaveform ( SynthStr, WaveformStr ))
    return ( ERROR_TOO_MANY_LEVELS );

  do
  {
    struct IFFChunk IFF_Chunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      return ( IoErr ());

    IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0]);
    IFFLength = IFFSize;

    IFFSize += (IFFSize & 1);

    switch ( ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] ) )
    {
      case 'FILE' :
      {
        if ( WaveformStr->tcyw_FileName )
          break;

        if (!( WaveformStr->tcyw_FileName = (STRPTR) AllocPMem ( IFFLength + 1 )))
          return ( IoErr ());

        if ( (CRead ( CFHandle, WaveformStr->tcyw_FileName, IFFLength ) != IFFLength) )
          return ( IoErr ());

        IFFLength &= 1;

        break;
      }
      case 'NAME' :
      {
        if ( WaveformStr->tcyw_Name )
          break;

        if (!( WaveformStr->tcyw_Name = (STRPTR) AllocPMem ( IFFLength + 1 )))
          return ( IoErr ());

        if ( (CRead ( CFHandle, WaveformStr->tcyw_Name, IFFLength ) != IFFLength) )
          return ( IoErr ());

        IFFLength &= 1;

        break;
      }
      case 'WHDR' :
      {
        if ( Invalid & 1 )
          return ( ERROR_INVALID_RESIDENT_LIBRARY );

        if ( (CRead ( CFHandle, &(WHDR.whdr_Repeat), (sizeof (struct IFF_WHDR) - sizeof (struct IFFChunk))) != (sizeof (struct IFF_WHDR) - sizeof (struct IFFChunk))))
          return ( IoErr ());

        WaveformStr->tcyw_Repeat = ReadMLong ( (UBYTE *) &(WHDR.whdr_Repeat) );
        WaveformStr->tcyw_RepLen = ReadMLong ( (UBYTE *) &(WHDR.whdr_RepLen) );
        WaveformStr->tcyw_Flags = ReadMWord ( (UBYTE *) &(WHDR.whdr_Flags) );

        Invalid |= 1;

        IFFLength = IFFSize - (sizeof (struct IFF_WHDR) - sizeof (struct IFFChunk));

        break;
      }
      case 'BODY' :
      {
        WORD *WaveData;

        if ( Invalid & 2 )
          return ( ERROR_INVALID_RESIDENT_LIBRARY );

        if (!( WaveData = (WORD *) AllocPMem ( IFFLength )))
          return ( IoErr ());

        FreePMem ( WaveformStr->tcyw_WaveData, 2L );

        WaveformStr->tcyw_WaveData = WaveData;
        WaveformStr->tcyw_Length = IFFLength;

        if ( CRead ( CFHandle, WaveData, IFFLength ) != IFFLength )
          return ( IoErr ());

        Invalid |= 2;

        IFFLength &= 1;

        break;
      }
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
        return ( IoErr ());
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      return ( ERROR_INVALID_RESIDENT_LIBRARY );

  } while ( (Length -= IFFSize) );

  if ( Invalid < 3 )
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  Length = WaveformStr->tcyw_Length;

  if (!( WaveformStr->tcyw_Flags & TCYWF_8Bit ))
    Length >>= 1UL;

  WaveformStr->tcyw_Samples = Length;

#ifndef MOTOROLA
  if (!( WaveformStr->tcyw_Flags & TCYWF_8Bit ))
  {
    WORD *WaveData = WaveformStr->tcyw_WaveData;
    ULONG i;

    for (i = ((WaveformStr->tcyw_Samples + 3UL) >> 2UL); i > 0; i--)
    {
      *WaveData++ = ReadMWord ( (UBYTE *) WaveData);
      *WaveData++ = ReadMWord ( (UBYTE *) WaveData);
      *WaveData++ = ReadMWord ( (UBYTE *) WaveData);
      *WaveData++ = ReadMWord ( (UBYTE *) WaveData);
    }
  }
#endif

  return ( 0L );
}

static ULONG LoadSnthStab ( struct CFHandle *CFHandle, struct TuComposerSynth *SynthStr, unsigned long Length )
{
  struct IFFHeader IFF_Header;
  ULONG IFFSize;

  if ( Length < 4 )
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  Length -= 4;

  do
  {
    struct IFFChunk IFF_Chunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      return ( IoErr ());

    if ( (IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0] )) & 1 )
      IFFSize++;

    IFFLength = IFFSize;

    if ( (ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] ) ) == 'FORM' )
    {
      ULONG FaultCode;

      if ( (CRead ( CFHandle, &IFF_Header, 4 ) != 4 ) )
        return ( IoErr ());

      if ( (ReadMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0])) ) == 'SMBL' )
      {
        if ( (FaultCode = LoadStabSmbl ( CFHandle, SynthStr, IFFSize )))
          return ( FaultCode );

        IFFLength = 0L;
      }
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
        return ( IoErr ());
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      return ( ERROR_INVALID_RESIDENT_LIBRARY );

  } while ( (Length -= IFFSize) );

  return ( 0L );
}

static ULONG LoadStabSmbl ( struct CFHandle *CFHandle, struct TuComposerSynth *SynthStr, unsigned long Length )
{
  struct IFF_SREF SREF;
  struct TuComposerSynthSymbolTable *SymbolStr;
  ULONG Invalid = 0L, IFFSize;

  if ( Length < 4 )
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  Length -= 4;

  if (!( SymbolStr = AllocSymbol ( "UNNAMED" )))
    return ( IoErr ());

  AddSymbol ( SynthStr, SymbolStr );

  do
  {
    struct IFFChunk IFF_Chunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      return ( IoErr ());

    IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0]);
    IFFLength = IFFSize;

    IFFSize += (IFFSize & 1);

    switch ( ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] ) )
    {
      case 'NAME' :
      {
        STRPTR SymbolName;

        if (!( SymbolName = (STRPTR) AllocPMem ( IFFLength + 1 )))
          return ( IoErr ());

        if ( (CRead ( CFHandle, SymbolName, IFFLength ) != IFFLength) )
          return ( IoErr ());

        if ( AssignSymbol ( SymbolStr, SymbolName ))
          return ( IoErr ());

        IFFLength &= 1;

        break;
      }
      case 'SREF' :
      {
        if ( Invalid )
          return ( ERROR_INVALID_RESIDENT_LIBRARY );

        if ( (CRead ( CFHandle, &(SREF.sref_SymbolValue), (sizeof (struct IFF_SREF) - sizeof (struct IFFChunk))) != (sizeof (struct IFF_SREF) - sizeof (struct IFFChunk))))
          return ( IoErr ());

        SymbolStr->tsst_SymbolValue = ReadMWord ( (UBYTE *) &(SREF.sref_SymbolValue) );
        SymbolStr->tsst_FirstLine = ReadMWord ( (UBYTE *) &(SREF.sref_FirstLine) );
        SymbolStr->tsst_LastLine = ReadMWord ( (UBYTE *) &(SREF.sref_LastLine) );
        SymbolStr->tsst_Type = SREF.sref_Type;
        SymbolStr->tsst_Flags = SREF.sref_Flags;

        Invalid = ~Invalid;

        IFFLength = IFFSize - (sizeof (struct IFF_SREF) - sizeof (struct IFFChunk));

        break;
      }
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
        return ( IoErr ());
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      return ( ERROR_INVALID_RESIDENT_LIBRARY );

  } while ( (Length -= IFFSize) );

  if ( Invalid )
    return ( 0L );

  return ( ERROR_INVALID_RESIDENT_LIBRARY );
}

static ULONG LoadTCIEnvl ( struct CFHandle *CFHandle, struct TuComposerModule *ModuleStr, unsigned long Length )
{
  struct IFFHeader IFF_Header;
  ULONG IFFSize;

  if ( Length < 4 )
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  Length -= 4;

  do
  {
    struct IFFChunk IFF_Chunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      return ( IoErr ());

    if ( (IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0] )) & 1 )
      IFFSize++;

    IFFLength = IFFSize;

    if ( (ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] ) ) == 'FORM' )
    {
      ULONG FaultCode;

      if ( (CRead ( CFHandle, &IFF_Header, 4 ) != 4 ) )
        return ( IoErr ());

      if ( (ReadMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0])) ) == 'ENVD' )
      {
        if ( (FaultCode = LoadEnvlEnvd ( CFHandle, ModuleStr, IFFSize )))
          return ( FaultCode );

        IFFLength = 0L;
      }
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
        return ( IoErr ());
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      return ( ERROR_INVALID_RESIDENT_LIBRARY );

  } while ( (Length -= IFFSize) );

  return ( 0L );
}

static ULONG LoadEnvlEnvd ( struct CFHandle *CFHandle, struct TuComposerModule *ModuleStr, unsigned long Length )
{
  struct MIFF_EHDR EHDR;
  struct TuComposerInstrEnvelope *EnvStr;

#ifndef MOTOROLA
  WORD *EnvData;
  UWORD *EnvNode;
#endif

  ULONG Invalid = 0L, IFFSize;

  if ( Length < 4 )
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  Length -= 4;

  if (!( EnvStr = CreateEnvelope ( 1L, ENVL_EMPTY, 0L, 0L, 0L )))
    return ( IoErr ());

  AddEnvelope ( ModuleStr, EnvStr );

  do
  {
    struct IFFChunk IFF_Chunk;
    struct UnknownChunk *UnkChunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      return ( IoErr ());

    IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0]);
    IFFLength = IFFSize;

    IFFSize += (IFFSize & 1);

    switch ( ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] ) )
    {
      case 'EHDR' :
      {
        if ( Invalid )
          return ( ERROR_INVALID_RESIDENT_LIBRARY );

        if ( (CRead ( CFHandle, &(EHDR.mehdr_Flags), (sizeof (struct MIFF_EHDR) - sizeof (struct IFFChunk))) != (sizeof (struct MIFF_EHDR) - sizeof (struct IFFChunk))))
          return ( IoErr ());

        EnvStr->tcie_Flags = ReadMWord ( (UBYTE *) &(EHDR.mehdr_Flags) );
        EnvStr->tcie_Tempo = ReadMWord ( (UBYTE *) &(EHDR.mehdr_Tempo) );
        EnvStr->tcie_SustainStart = ReadMWord ( (UBYTE *) &(EHDR.mehdr_SustainStart) );
        EnvStr->tcie_SustainEnd = ReadMWord ( (UBYTE *) &(EHDR.mehdr_SustainEnd) );
        EnvStr->tcie_SustainCount = ReadMWord ( (UBYTE *) &(EHDR.mehdr_SustainCount) );
        EnvStr->tcie_LoopStart = ReadMWord ( (UBYTE *) &(EHDR.mehdr_LoopStart) );
        EnvStr->tcie_LoopEnd = ReadMWord ( (UBYTE *) &(EHDR.mehdr_LoopEnd) );
        EnvStr->tcie_LoopCount = ReadMWord ( (UBYTE *) &(EHDR.mehdr_LoopCount) );
        EnvStr->tcie_LowestValue = ReadMWord ( (UBYTE *) &(EHDR.mehdr_LowestValue) );
        EnvStr->tcie_HighestValue = ReadMWord ( (UBYTE *) &(EHDR.mehdr_HighestValue) );

        Invalid = ~Invalid;

        IFFLength = IFFSize - (sizeof (struct MIFF_EHDR) - sizeof (struct IFFChunk));

        break;
      }
      case 'BODY' :
      {
        if ( ReSizeEnvelope ( EnvStr, IFFLength / sizeof (WORD)))
          return ( IoErr ());

        if ( CRead ( CFHandle, EnvStr->tcie_Data, IFFLength ) != IFFLength )
          return ( IoErr ());

        IFFLength &= 1;

        break;
      }
      case 'NODE' :
      {
        if ( EnvStr->tcie_NodePoints )
          return ( ERROR_INVALID_RESIDENT_LIBRARY );

        if (!( EnvStr->tcie_NodePoints = (UWORD *) AllocPMem ( IFFLength )))
          return ( IoErr ());

        EnvStr->tcie_NumNodes = IFFLength / sizeof (UWORD);

        if ( CRead ( CFHandle, EnvStr->tcie_NodePoints, IFFLength ) != IFFLength )
          return ( IoErr ());

        IFFLength &= 1;

        break;
      }
      default :
      {
        if (!( UnkChunk = AllocUnknownChunk ( NULL, ReadMLong ( (UBYTE *) &(IFF_Chunk.iff_Identifier[0])), IFFLength )))
          return ( IoErr ());

        AddEnvelopeUnknownChunk ( EnvStr, UnkChunk );

        if ( (CRead ( CFHandle, &(UnkChunk->uc_Data), IFFSize ) != IFFSize ) )
          return ( IoErr ());

        IFFLength &= 1;

        break;
      }
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
        return ( IoErr ());
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      return ( ERROR_INVALID_RESIDENT_LIBRARY );

  } while ( (Length -= IFFSize) );

  if (!( Invalid ))
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

#ifndef MOTOROLA
  EnvData = EnvStr->tcie_Data;

  for ( Length = EnvStr->tcie_NumPoints; Length > 0; Length--)
  {
    *EnvData++ = ReadMWord ( (UBYTE *) EnvData);
  }

  EnvNode = EnvStr->tcie_NodePoints;

  for ( Length = EnvStr->tcie_NumNodes; Length > 0; Length--)
  {
    *EnvNode++ = ReadMWord ( (UBYTE *) EnvNode);
  }
#endif

  return ( 0L );
}

static ULONG LoadTCIArpg ( struct CFHandle *CFHandle, struct TuComposerModule *ModuleStr, unsigned long Length )
{
  struct IFFHeader IFF_Header;
  struct IFF_AHDR AHDR;
  struct TuComposerInstrArpeggio *ArpStr;

#ifndef MOTOROLA
  struct TuComposerInstrArpeggioData *ArpData;
#endif

  ULONG Invalid = 0L, IFFSize;

  if ( Length < 4 )
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  Length -= 4;

  if (!( ArpStr = AllocArpeggio ( 1L )))
    return ( IoErr ());

  AddArpeggio ( ModuleStr, ArpStr );

  do
  {
    struct IFFChunk IFF_Chunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      return ( IoErr ());

    IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0]);
    IFFLength = IFFSize;

    IFFSize += (IFFSize & 1);

    switch ( ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] ) )
    {
      case 'AHDR' :
      {
        if ( Invalid )
          return ( ERROR_INVALID_RESIDENT_LIBRARY );

        if ( (CRead ( CFHandle, &(AHDR.ahdr_Length), (sizeof (struct IFF_AHDR) - sizeof (struct IFFChunk))) != (sizeof (struct IFF_AHDR) - sizeof (struct IFFChunk))))
          return ( IoErr ());

        ArpStr->tcia_Flags = ReadMWord ( (UBYTE *) &(AHDR.ahdr_Flags) );
        ArpStr->tcia_SustainStart = ReadMWord ( (UBYTE *) &(AHDR.ahdr_SustainStart) );
        ArpStr->tcia_SustainEnd = ReadMWord ( (UBYTE *) &(AHDR.ahdr_SustainEnd) );
        ArpStr->tcia_SustainCount = ReadMWord ( (UBYTE *) &(AHDR.ahdr_SustainCount) );
        ArpStr->tcia_LoopStart = ReadMWord ( (UBYTE *) &(AHDR.ahdr_LoopStart) );
        ArpStr->tcia_LoopEnd = ReadMWord ( (UBYTE *) &(AHDR.ahdr_LoopEnd) );
        ArpStr->tcia_LoopCount = ReadMWord ( (UBYTE *) &(AHDR.ahdr_LoopCount) );

        if ( ReSizeArpeggio ( ArpStr, ReadMWord ( (UBYTE *) &(AHDR.ahdr_Length))))
          return ( IoErr ());

        Invalid = ~Invalid;

        IFFLength = IFFSize - (sizeof (struct IFF_AHDR) - sizeof (struct IFFChunk));

        break;
      }
      case 'FORM' :
      {
        ULONG FaultCode;

        if ( (CRead ( CFHandle, &IFF_Header, 4 ) != 4 ) )
          return ( IoErr ());

        if ( (ReadMLong ( (UBYTE *) &IFF_Header.ifh_FORMId[0] ) == 'ARPE' ))
        {
          if ( (FaultCode = LoadArpgArpe ( CFHandle, ArpStr, IFFSize )))
            return ( FaultCode );

          IFFLength = 0L;
        }

        break;
      }
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
        return ( IoErr ());
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      return ( ERROR_INVALID_RESIDENT_LIBRARY );

  } while ( (Length -= IFFSize) );

  if (!( Invalid ))
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

#ifndef MOTOROLA
  ArpData = ArpStr->tcia_Data;

  for ( Length = ArpStr->tcia_Entries; Length > 0; Length--)
  {
    ArpData->tciad_DataWord = ReadMWord ( (UBYTE *) &(ArpData->tciad_DataWord));
    ArpData->tciad_DataWord2 = ReadMWord ( (UBYTE *) &(ArpData->tciad_DataWord2));
    ArpData->tciad_DataWord3 = ReadMWord ( (UBYTE *) &(ArpData->tciad_DataWord3));
    ArpData->tciad_DataWord4 = ReadMWord ( (UBYTE *) &(ArpData->tciad_DataWord4));

    ArpData++;
  }
#endif

  return ( 0L );
}

static ULONG LoadArpgArpe ( struct CFHandle *CFHandle, struct TuComposerInstrArpeggio *ArpStr, unsigned long Length )
{
  ULONG IFFSize, NumTick = 0L;

  if ( Length < 4 )
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  Length -= 4;

  do
  {
    struct IFFChunk IFF_Chunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      return ( IoErr ());

    if ( (IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0] )) & 1 )
      IFFSize++;

    IFFLength = IFFSize;

    if ( (ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0])) == 'ARPE' )
    {
      if ( NumTick >= ArpStr->tcia_Entries )
      {
        if ( AddArpeggioTick ( ArpStr, 1L ))
          return ( IoErr ());
      }

      if ( CRead ( CFHandle, &(ArpStr->tcia_Data[NumTick]), sizeof (struct TuComposerInstrArpeggioData)) != sizeof (struct TuComposerInstrArpeggioData))
        return ( IoErr ());

      NumTick++;

      IFFLength = IFFSize - sizeof (struct TuComposerInstrArpeggioData);
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
        return ( IoErr ());
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      return ( ERROR_INVALID_RESIDENT_LIBRARY );

  } while ( (Length -= IFFSize) );

  return ( 0L );
}

static ULONG SaveTCI ( struct CFHandle *CFHandle,
    struct TuComposerInstr *InstrStr, union FlagPass *FlagPassStr )
{
  struct IFFHeader IFF_Header;
  struct IFF_IHDR IHDR;
  struct TuComposerSample **InstrSmpStr;
  struct TuComposerInstrEnvelope **InstrEnvStr;
  struct TuComposerInstrKeyboard *KeybrdStr;
  struct TuComposerInstrArpeggio *ArpStr;
  struct UnknownChunk *UnkChunk;
  ULONG TCISize = sizeof (ULONG), FORMSize;
  ULONG TCILenPos, FORMLenPos;
  ULONG FaultCode;

  if ( (TCILenPos = CSeek ( CFHandle, 0L, OFFSET_CURRENT )) == 0xFFFFFFFF )
    return ( ERROR_INSTR_IOERR );

  TCILenPos += sizeof (ULONG);

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0]), 'FORM' );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), TCISize );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_TypeId[0]), 'TCI1' );

  if ( (CWrite ( CFHandle, &IFF_Header, sizeof (struct IFFHeader )) != sizeof (struct IFFHeader )))
    return ( ERROR_INSTR_IOERR );

  if ( (FaultCode = SaveTCIName ( CFHandle, InstrStr->tci_FileName, 'FILE', (ULONG *) &TCISize, ERROR_INSTR_IOERR )))
    return ( FaultCode );

  if ( (FaultCode = SaveTCIName ( CFHandle, InstrStr->tci_Name, 'NAME', (ULONG *) &TCISize, ERROR_INSTR_IOERR )))
    return ( FaultCode );

  if (!( InstrEnvStr = GetInstrEnvelopes ( InstrStr )))
    return ( ERROR_INSTR_NOMEM );

  IHDR.ihdr_Version = IHDR_VERSION;
  IHDR.ihdr_Revision = IHDR_REVISION;

  WriteMWord ( (UBYTE *) &(IHDR.ihdr_VolumeEnvNum), GetInstrEnvelopeNum ( InstrEnvStr, InstrStr->tci_VolumeEnvelope ));
  WriteMWord ( (UBYTE *) &(IHDR.ihdr_PanningEnvNum), GetInstrEnvelopeNum ( InstrEnvStr, InstrStr->tci_PanningEnvelope ));
  WriteMWord ( (UBYTE *) &(IHDR.ihdr_SlideEnvNum), GetInstrEnvelopeNum ( InstrEnvStr, InstrStr->tci_SlideEnvelope ));
  WriteMWord ( (UBYTE *) &(IHDR.ihdr_VibratoEnvNum), GetInstrEnvelopeNum ( InstrEnvStr, InstrStr->tci_VibratoEnvelope ));
  WriteMWord ( (UBYTE *) &(IHDR.ihdr_TremoloEnvNum), GetInstrEnvelopeNum ( InstrEnvStr, InstrStr->tci_TremoloEnvelope ));
  WriteMWord ( (UBYTE *) &(IHDR.ihdr_PannoloEnvNum), GetInstrEnvelopeNum ( InstrEnvStr, InstrStr->tci_PannoloEnvelope ));
  WriteMWord ( (UBYTE *) &(IHDR.ihdr_ChannoloEnvNum), GetInstrEnvelopeNum ( InstrEnvStr, InstrStr->tci_ChannoloEnvelope ));
  WriteMWord ( (UBYTE *) &(IHDR.ihdr_SpenoloEnvNum), GetInstrEnvelopeNum ( InstrEnvStr, InstrStr->tci_SpenoloEnvelope ));

  WriteMWord ( (UBYTE *) &(IHDR.ihdr_ArpeggioDefNum), (InstrStr->tci_ArpeggioStructure ? 1 : 0));
  WriteMWord ( (UBYTE *) &(IHDR.ihdr_ArpeggioDefNum), (InstrStr->tci_KeyboardDefs ? 1 : 0));

  memcpy ( &(IHDR.ihdr_NumSamples), &(InstrStr->tci_NumSamples), (sizeof (struct IFF_IHDR) - offsetof (struct IFF_IHDR, ihdr_NumSamples )));

  IHDR.ihdr_CompFlags &= ~TCICF_SeparateSamples;

#ifndef MOTOROLA
  WriteMWord ( (UBYTE *) &(IHDR.ihdr_VolumeSwing), IHDR.ihdr_VolumeSwing );
  WriteMWord ( (UBYTE *) &(IHDR.ihdr_PanningSwing), IHDR.ihdr_PanningSwing );
  WriteMLong ( (UBYTE *) &(IHDR.ihdr_PitchSwing), IHDR.ihdr_PitchSwing );
  WriteMWord ( (UBYTE *) &(IHDR.ihdr_PitchPanSep), IHDR.ihdr_PitchPanSep );
  WriteMWord ( (UBYTE *) &(IHDR.ihdr_EnvUsageFlags), IHDR.ihdr_EnvUsageFlags );
  WriteMWord ( (UBYTE *) &(IHDR.ihdr_EnvProcFlags), IHDR.ihdr_EnvProcFlags );
  WriteMWord ( (UBYTE *) &(IHDR.ihdr_EnvRetrigFlags), IHDR.ihdr_EnvRetrigFlags );
  WriteMWord ( (UBYTE *) &(IHDR.ihdr_EnvRandomFlags), IHDR.ihdr_EnvRandomFlags );
  WriteMWord ( (UBYTE *) &(IHDR.ihdr_EnvRndDelayFlags), IHDR.ihdr_EnvRndDelayFlags );
  WriteMWord ( (UBYTE *) &(IHDR.ihdr_FadeOut), IHDR.ihdr_FadeOut );
  WriteMWord ( (UBYTE *) &(IHDR.ihdr_Hold), IHDR.ihdr_Hold );
  WriteMWord ( (UBYTE *) &(IHDR.ihdr_Decay), IHDR.ihdr_Decay );
#endif

  if ( (FaultCode = SaveTCIChunk ( CFHandle, (struct IFFChunk *) &IHDR, 'IHDR', sizeof (struct IFF_IHDR), (ULONG *) &TCISize, ERROR_INSTR_IOERR )))
    return ( FaultCode );

  if (!( InstrSmpStr = GetInstrSamples ( InstrStr )))
    return ( ERROR_INSTR_NOMEM );

  if ( *InstrSmpStr )
  {
    struct TuComposerSample **SampleList = InstrSmpStr;
    struct TuComposerSample *SampleStr;

    TCISize += (sizeof (struct IFFHeader) - sizeof (ULONG));
    FORMSize = sizeof (ULONG);

    if ( (FORMLenPos = CSeek ( CFHandle, 0L, OFFSET_CURRENT )) == 0xFFFFFFFF )
    {
SaveTCISmpIOErr:

      FreeInstrEnvelopes ( InstrEnvStr );
      FreeInstrSamples ( InstrSmpStr );

      return ( ERROR_INSTR_IOERR );
    }

    FORMLenPos += sizeof (ULONG);

    WriteMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0]), 'FORM' );
    WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), FORMSize );
    WriteMLong ( (UBYTE *) &(IFF_Header.ifh_TypeId[0]), 'SAMP' );

    if ( (CWrite ( CFHandle, &IFF_Header, sizeof (struct IFFHeader )) != sizeof (struct IFFHeader )))
      goto SaveTCISmpIOErr;

    for (;;)
    {
      if (!( SampleStr = *SampleList++ ))
        break;

      if ( (FaultCode = SaveInstSmpl ( CFHandle, InstrEnvStr, InstrSmpStr, SampleStr, (ULONG *) &FORMSize, FlagPassStr )))
      {
        FreeInstrEnvelopes ( InstrEnvStr );
        FreeInstrSamples ( InstrSmpStr );

        return ( FaultCode );
      }
    }

    if ( (FORMLenPos = CSeek ( CFHandle, FORMLenPos, OFFSET_BEGINNING )) == 0xFFFFFFFF )
      goto SaveTCISmpIOErr;

    WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), FORMSize );

    if ( CWrite ( CFHandle, &(IFF_Header.ifh_Length[0]), sizeof (ULONG)) != sizeof (ULONG))
      goto SaveTCISmpIOErr;

    if ( CSeek ( CFHandle, FORMLenPos, OFFSET_BEGINNING ) == 0xFFFFFFFF )
      goto SaveTCISmpIOErr;

    TCISize += FORMSize;
  }

  FreeInstrSamples ( InstrSmpStr );

  if ( FlagPassStr[3].flg_Value )
  {
    if ( *InstrEnvStr )
    {
      struct TuComposerInstrEnvelope **EnvList = InstrEnvStr;
      struct TuComposerInstrEnvelope *EnvStr;

      TCISize += (sizeof (struct IFFHeader) - sizeof (ULONG));
      FORMSize = sizeof (ULONG);

      if ( (FORMLenPos = CSeek ( CFHandle, 0L, OFFSET_CURRENT )) == 0xFFFFFFFF )
        return ( ERROR_INSTR_IOERR );

      FORMLenPos += sizeof (ULONG);

      WriteMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0]), 'FORM' );
      WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), FORMSize );
      WriteMLong ( (UBYTE *) &(IFF_Header.ifh_TypeId[0]), 'ENVL' );

      if ( (CWrite ( CFHandle, &IFF_Header, sizeof (struct IFFHeader )) != sizeof (struct IFFHeader )))
      {
SaveTCIEnvIOErr:
        FreeInstrEnvelopes ( InstrEnvStr );

        return ( ERROR_INSTR_IOERR );
      }

      for (;;)
      {
        if (!( EnvStr = *EnvList++ ))
          break;

        if ( (FaultCode = SaveTCIEnvd ( CFHandle, EnvStr, (ULONG *) &FORMSize )))
        {
          FreeInstrEnvelopes ( InstrEnvStr );

          return ( FaultCode );
        }
      }

      if ( (FORMLenPos = CSeek ( CFHandle, FORMLenPos, OFFSET_BEGINNING )) == 0xFFFFFFFF )
        goto SaveTCIEnvIOErr;

      WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), FORMSize );

      if ( CWrite ( CFHandle, &(IFF_Header.ifh_Length[0]), sizeof (ULONG)) != sizeof (ULONG))
        goto SaveTCIEnvIOErr;

      if ( CSeek ( CFHandle, FORMLenPos, OFFSET_BEGINNING ) == 0xFFFFFFFF )
        goto SaveTCIEnvIOErr;

      TCISize += FORMSize;
    }
  }

  FreeInstrEnvelopes ( InstrEnvStr );

  if ( InstrStr->tci_KeyboardDefs )
  {
    if (!( KeybrdStr = GetInstrKeyboard ( InstrStr )))
      return ( IoErr ());

    if ( CheckKeyboard ( KeybrdStr ))
    {
      if ( (FaultCode = SaveTCIKbrd ( CFHandle, KeybrdStr, (ULONG *) &TCISize )))
      {
        FreeKeyboard ( KeybrdStr );

        return ( FaultCode );
      }
    }

    FreeKeyboard ( KeybrdStr );
  }

  if ( (ArpStr = InstrStr->tci_ArpeggioStructure) )
  {
    if ( (FaultCode = SaveTCIArpg ( CFHandle, ArpStr, (ULONG *) &TCISize )))
      return ( FaultCode );
  }

  UnkChunk = (struct UnknownChunk *) &(InstrStr->tci_UnknownChunks.mlh_Head);

  for (;;)
  {
    UnkChunk = (struct UnknownChunk *) UnkChunk->uc_Node.mln_Succ;

    if (!( UnkChunk->uc_Node.mln_Succ ))
      break;

    if ( (FaultCode = SaveTCIChunk ( CFHandle, (struct IFFChunk *) &UnkChunk, ReadMLong ( (UBYTE *) &(UnkChunk->uc_Identifier[0])), ReadMLong ( (UBYTE *) &(UnkChunk->uc_Length[0])), &TCISize, ERROR_INSTR_IOERR )))
      return ( FaultCode );
  }

  if ( CSeek ( CFHandle, TCILenPos, OFFSET_BEGINNING ) == 0xFFFFFFFF )
    return ( ERROR_INSTR_IOERR );

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), TCISize );

  if ( CWrite ( CFHandle, &(IFF_Header.ifh_Length[0]), sizeof (ULONG)) != sizeof (ULONG))
    return ( ERROR_INSTR_IOERR );

  return ( 0L );
}

static ULONG SaveTCIChunk ( struct CFHandle *CFHandle, struct IFFChunk *IFF_Chunk, unsigned long ChunkName, unsigned long ChunkLen, ULONG *ChunkLenBuf, unsigned long ErrorCode )
{
  ChunkLen -= sizeof (struct IFFChunk );

  if ( ChunkLen )
  {
    WriteMLong ( (UBYTE *) &(IFF_Chunk->iff_Identifier[0]), ChunkName );
    WriteMLong ( (UBYTE *) &(IFF_Chunk->iff_Length[0]), ChunkLen );

    if ( (CWrite ( CFHandle, IFF_Chunk, (ChunkLen + sizeof (struct IFFChunk)))) != (ChunkLen + sizeof (struct IFFChunk)))
      return ( ErrorCode );

    if ( ChunkLen & 1 )
    {
      *(ULONG *) &(IFF_Chunk->iff_Identifier[0]) = 0L;

      if ( (CWrite ( CFHandle, &IFF_Chunk, 1 )) != 1 )
        return ( ErrorCode );

      ChunkLen++;
    }

    *ChunkLenBuf += (ChunkLen + sizeof (struct IFFChunk));
  }

  return ( 0L );
}

static ULONG SaveTCIName ( struct CFHandle *CFHandle, STRPTR Name, unsigned long ChunkName, ULONG *ChunkLenBuf, unsigned long ErrorCode )
{
  struct IFFChunk IFF_Chunk;

  if ( Name )
  {
    ULONG Length = strlen ( Name );

    WriteMLong ( (UBYTE *) &(IFF_Chunk.iff_Identifier[0]), ChunkName );
    WriteMLong ( (UBYTE *) &(IFF_Chunk.iff_Length[0]), Length );

    Length += (Length & 1);

    if ( Length )
    {
      if ( (CWrite ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk))) != sizeof (struct IFFChunk))
        return ( ErrorCode );

      if ( (CWrite ( CFHandle, Name, Length )) != Length )
        return ( ErrorCode );

      *ChunkLenBuf += (Length + sizeof (struct IFFChunk));
    }
  }

  return ( 0L );
}

static ULONG SaveInstSmpl ( struct CFHandle *CFHandle, struct TuComposerInstrEnvelope **InstrEnvStr, struct TuComposerSample **InstrSmpStr, struct TuComposerSample *SampleStr, ULONG *ChunkLenBuf, union FlagPass *FlagPassStr )
{
  struct IFFHeader IFF_Header;
  struct IFFChunk IFF_Chunk;
  struct IFF_SMPH SMPH;
  struct IFF_SMPR SMPR;
  struct UnknownChunk *UnkChunk;
  ULONG SmplSize = sizeof (ULONG), FORMSize;
  ULONG SmplLenPos;
  ULONG FaultCode;

  if ( (SmplLenPos = CSeek ( CFHandle, 0L, OFFSET_CURRENT )) == 0xFFFFFFFF )
    return ( ERROR_SAMPLE_IOERR );

  SmplLenPos += sizeof (ULONG);

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0]), 'FORM' );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), SmplSize );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_TypeId[0]), 'SMPL' );

  if ( (CWrite ( CFHandle, &IFF_Header, sizeof (struct IFFHeader )) != sizeof (struct IFFHeader )))
    return ( ERROR_SAMPLE_IOERR );

  if ( (FaultCode = SaveTCIName ( CFHandle, SampleStr->tcs_FileName, 'FILE', (ULONG *) &SmplSize, ERROR_SAMPLE_IOERR )))
    return ( FaultCode );

  if ( (FaultCode = SaveTCIName ( CFHandle, SampleStr->tcs_Name, 'NAME', (ULONG *) &SmplSize, ERROR_SAMPLE_IOERR )))
    return ( FaultCode );

  memcpy ( &(SMPH.smph_Samples), &(SampleStr->tcs_Samples), (offsetof (struct IFF_SMPH, smph_AutoVibratoNum) - sizeof (struct IFFChunk)));

#ifndef MOTOROLA
  WriteMLong ( (UBYTE *) &(SMPH.smph_Samples), SMPH.smph_Samples );
  WriteMLong ( (UBYTE *) &(SMPH.smph_Repeat), SMPH.smph_Repeat );
  WriteMLong ( (UBYTE *) &(SMPH.smph_RepLen), SMPH.smph_RepLen );
  WriteMLong ( (UBYTE *) &(SMPH.smph_RepCount), SMPH.smph_RepCount );
  WriteMLong ( (UBYTE *) &(SMPH.smph_SustainRepeat), SMPH.smph_SustainRepeat );
  WriteMLong ( (UBYTE *) &(SMPH.smph_SustainRepLen), SMPH.smph_SustainRepLen );
  WriteMLong ( (UBYTE *) &(SMPH.smph_SustainRepCount), SMPH.smph_SustainRepCount );
  WriteMLong ( (UBYTE *) &(SMPH.smph_Rate), SMPH.smph_Rate );
  WriteMLong ( (UBYTE *) &(SMPH.smph_StartOffset), SMPH.smph_StartOffset );
  WriteMLong ( (UBYTE *) &(SMPH.smph_LowerRateLimit), SMPH.smph_LowerRateLimit );
  WriteMLong ( (UBYTE *) &(SMPH.smph_UpperRateLimit), SMPH.smph_UpperRateLimit );
#endif

  WriteMWord ( (UBYTE *) &(SMPH.smph_AutoVibratoNum), GetInstrEnvelopeNum ( InstrEnvStr, SampleStr->tcs_AutoVibratoEnv ));
  WriteMWord ( (UBYTE *) &(SMPH.smph_AutoTremoloNum), GetInstrEnvelopeNum ( InstrEnvStr, SampleStr->tcs_AutoTremoloEnv ));
  WriteMWord ( (UBYTE *) &(SMPH.smph_AutoPannoloNum), GetInstrEnvelopeNum ( InstrEnvStr, SampleStr->tcs_AutoPannoloEnv ));

  memcpy ( &(SMPH.smph_EnvUsageFlags), &(SampleStr->tcs_EnvUsageFlags), (sizeof (struct IFF_SMPH) - offsetof (struct IFF_SMPH, smph_EnvUsageFlags )));

#ifndef MOTOROLA
  WriteMWord ( (UBYTE *) &(SMPH.smph_VibratoSweep), SMPH.smph_VibratoSweep );
  WriteMWord ( (UBYTE *) &(SMPH.smph_TremoloSweep), SMPH.smph_TremoloSweep );
  WriteMWord ( (UBYTE *) &(SMPH.smph_PannoloSweep), SMPH.smph_PannoloSweep );
#endif

  SMPH.smph_Flags &= ~TCSF_Redirect;

  if ( SampleStr->tcs_Flags & TCSF_Redirect )
  {
    struct TuComposerSample *OrigSampleStr;
    ULONG NumSample = 0L;

    for (;;)
    {
      if (!( OrigSampleStr = *InstrSmpStr++ ))
        break;

      NumSample++;

      if ( OrigSampleStr->tcs_Flags & TCSF_Redirect )
        continue;

      if ( OrigSampleStr->tcs_Data == SampleStr->tcs_Data )
        goto FoundOriginSample;
    }

    goto SaveSampleNormal;

FoundOriginSample:

    if ( (FaultCode = SaveTCIChunk ( CFHandle, (struct IFFChunk *) &SMPH, 'SMPH', sizeof (struct IFF_SMPH), (ULONG *) &SmplSize, ERROR_SAMPLE_IOERR )))
      return ( FaultCode );

    WriteMLong ( (UBYTE *) &(SMPR.smpr_SampleNum), NumSample );

    if ( (FaultCode = SaveTCIChunk ( CFHandle, (struct IFFChunk *) &SMPR, 'SMPR', sizeof (struct IFF_SMPR), (ULONG *) &SmplSize, ERROR_SAMPLE_IOERR )))
      return ( FaultCode );
  } else
  {
SaveSampleNormal:

    if ( SampleStr->tcs_Samples && FlagPassStr[0].flg_Value )
    {
      switch ( FlagPassStr[1].flg_Value )
      {
        case 1 :
        {
          SMPH.smph_Flags |= SMPLF_Delta;

          break;
        }
        case 3 :
        {
          if (!( SampleStr->tcs_BitDepth & 4 ))
            SMPH.smph_Flags |= SMPLF_Delta;

          break;
        }
      }

      if ( (FaultCode = SaveTCIChunk ( CFHandle, (struct IFFChunk *) &SMPH, 'SMPH', sizeof (struct IFF_SMPH), (ULONG *) &SmplSize, ERROR_SAMPLE_IOERR )))
        return ( FaultCode );

      FORMSize = SampleStr->tcs_Length;

      WriteMLong ( (UBYTE *) &(IFF_Chunk.iff_Identifier[0]), 'BODY' );
      WriteMLong ( (UBYTE *) &(IFF_Chunk.iff_Length[0]), FORMSize );

      if ( (CWrite ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk )) != sizeof (struct IFFChunk )))
        return ( ERROR_SAMPLE_IOERR );

      FORMSize += (FORMSize & 1);
      SmplSize += sizeof (struct IFFChunk) + FORMSize;

      if ( SMPH.smph_Flags & SMPLF_Delta )
      {
        WORD *SampleData = SampleStr->tcs_Data;
        WORD *CopyData = AllocSampleData ( SAVESAMPLE_BUFFER * sizeof (LONG));
        ULONG WriteLeft, WriteChunk = (((SAVESAMPLE_BUFFER * SampleStr->tcs_BitDepth) + 7UL) >> 3UL), WriteSmp = SAVESAMPLE_BUFFER;
        ULONG SmpLen = SampleStr->tcs_Length, SmpSize = SampleStr->tcs_Samples;
        LONG  Delta = 0L;

        do
        {
          WriteLeft = WriteChunk;

          if ( SmpLen < WriteChunk )
          {
            WriteLeft = (((SmpSize * SampleStr->tcs_BitDepth) + 7UL) >> 3UL);
            WriteSmp = SmpSize;
          }

          switch ( SampleStr->tcs_BitDepth )
          {
            case 8 :
            {
              DeltaCrunch8 ( (BYTE **) &SampleData, (BYTE *) CopyData, (BYTE *) &Delta, WriteSmp );

              break;
            }
            case 16 :
            {
              DeltaCrunch16 ( (WORD **) &SampleData, CopyData, (WORD *) &Delta, WriteSmp );

              break;
            }
            case 32 :
            {
              DeltaCrunch32 ( (LONG **) &SampleData, (LONG *) CopyData, (LONG *) &Delta, WriteSmp );

              break;
            }
            default :
            {
              DeltaCrunchX ( (LONG **) &SampleData, (LONG *) CopyData, (LONG *) &Delta, SampleStr->tcs_BitDepth, WriteSmp );

              break;
            }
          }

          if ( CWrite ( CFHandle, CopyData, WriteLeft ) != WriteLeft )
          {
            FreeSampleData ( CopyData, SAVESAMPLE_BUFFER * sizeof (LONG));

            return ( ERROR_SAMPLE_IOERR );
          }

          SmpSize -= WriteSmp;
        } while ( (SmpLen -= WriteLeft) );

        FreeSampleData ( CopyData, SAVESAMPLE_BUFFER * sizeof (LONG));

        if ( SampleStr->tcs_Length & 1 )
        {
          *(ULONG *) &(IFF_Chunk.iff_Identifier[0]) = 0L;

          if ( (CWrite ( CFHandle, &IFF_Chunk, 1 )) != 1 )
            return ( ERROR_SAMPLE_IOERR );
        }
      } else
      {
        BYTE *SampleData = (BYTE *) SampleStr->tcs_Data;
        ULONG SmpLen = SampleStr->tcs_Samples;

#ifndef MOTOROLA
        if ( SampleStr->tcs_BitDepth == 8 )
        {
          if ( SmpLen & 1 )
            SampleData[SmpLen] = 0;

          if ( (CWrite ( CFHandle, SampleData, FORMSize )) != FORMSize )
            return ( ERROR_SAMPLE_IOERR );
        } else
        {
          WORD *CopyData = AllocSampleData ( SAVESAMPLE_BUFFER * sizeof (ULONG));
          ULONG WriteLeft;

          do
          {
            ULONG i;

            WriteLeft = SAVESAMPLE_BUFFER;

            if ( SmpLen < SAVESAMPLE_BUFFER )
              WriteLeft = SmpLen;

            if ( SampleStr->tcs_BitDepth == 16 )
            {
              WORD *Buffer = CopyData;

              for (i = WriteLeft; i > 0; i--)
              {
                *Buffer++ = ReadMWord ( (UBYTE *) SampleData++ );
              }

              if ( CWrite ( CFHandle, CopyData, (WriteLeft * sizeof (WORD))) != (WriteLeft * sizeof (WORD)))
              {
                FreeSampleData ( CopyData, SAVESAMPLE_BUFFER * sizeof (ULONG));

                return ( ERROR_SAMPLE_IOERR );
              }
            } else
            {
              LONG *Buffer = (LONG *) CopyData;

              for (i = WriteLeft; i > 0; i--)
              {
                *Buffer++ = ReadMLong ( (UBYTE *) SampleData++ );
              }

              if ( CWrite ( CFHandle, CopyData, (WriteLeft * sizeof (LONG))) != (WriteLeft * sizeof (LONG)))
              {
                FreeSampleData ( CopyData, SAVESAMPLE_BUFFER * sizeof (ULONG));

                return ( ERROR_SAMPLE_IOERR );
              }
            }
          } while ( (SmpLen -= WriteLeft) );

          FreeSampleData ( CopyData, SAVESAMPLE_BUFFER * sizeof (ULONG));

          if ( SampleStr->tcs_Length & 1 )
          {
            *(ULONG *) &(IFF_Chunk.iff_Identifier[0]) = 0L;

            if ( (CWrite ( CFHandle, &IFF_Chunk, 1 )) != 1 )
              return ( ERROR_SAMPLE_IOERR );
          }
        }
#else
        if ( (CWrite ( CFHandle, SampleData, FORMSize )) != FORMSize )
          return ( ERROR_SAMPLE_IOERR );
#endif
      }
    } else
    {
      if ( (FaultCode = SaveTCIChunk ( CFHandle, (struct IFFChunk *) &SMPH, 'SMPH', sizeof (struct IFF_SMPH), (ULONG *) &SmplSize, ERROR_SAMPLE_IOERR )))
        return ( FaultCode );
    }
  }

  if ( SampleStr->tcs_Synth )
  {
    if ( (FaultCode = SaveSmplSnth ( CFHandle, SampleStr->tcs_Synth, (ULONG *) &SmplSize, FlagPassStr )))
      return ( FaultCode );
  }

  UnkChunk = (struct UnknownChunk *) &(SampleStr->tcs_UnknownChunks.mlh_Head);

  for (;;)
  {
    UnkChunk = (struct UnknownChunk *) UnkChunk->uc_Node.mln_Succ;

    if (!( UnkChunk->uc_Node.mln_Succ ))
      break;

    if ( (FaultCode = SaveTCIChunk ( CFHandle, (struct IFFChunk *) &UnkChunk, ReadMLong ( (UBYTE *) &(UnkChunk->uc_Identifier[0])), ReadMLong ( (UBYTE *) &(UnkChunk->uc_Length[0])), &SmplSize, ERROR_SAMPLE_IOERR )))
      return ( FaultCode );
  }

  if ( (SmplLenPos = CSeek ( CFHandle, SmplLenPos, OFFSET_BEGINNING )) == 0xFFFFFFFF )
    return ( ERROR_SAMPLE_IOERR );

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), SmplSize );

  if ( CWrite ( CFHandle, &(IFF_Header.ifh_Length[0]), sizeof (ULONG)) != sizeof (ULONG))
    return ( ERROR_SAMPLE_IOERR );

  if ( CSeek ( CFHandle, SmplLenPos, OFFSET_BEGINNING ) == 0xFFFFFFFF )
    return ( ERROR_SAMPLE_IOERR );

  *ChunkLenBuf += SmplSize + (sizeof (struct IFFHeader) - sizeof (ULONG));

  return ( 0L );
}

static ULONG SaveSmplSnth ( struct CFHandle *CFHandle, struct TuComposerSynth *SynthStr, ULONG *ChunkLenBuf, union FlagPass *FlagPassStr )
{
  struct IFFHeader IFF_Header;
  struct IFFChunk IFF_Chunk;
  struct MIFF_YHDR YHDR;
  struct TuComposerSynthWave *WaveformStr;
  struct TuComposerSynthSymbolTable *SymbolStr;
  struct UnknownChunk *UnkChunk;
  ULONG SnthSize = sizeof (ULONG), FORMSize;
  ULONG SnthLenPos, FORMLenPos;
  ULONG FaultCode;

  if ( (SnthLenPos = CSeek ( CFHandle, 0L, OFFSET_CURRENT )) == 0xFFFFFFFF )
    return ( ERROR_SYNTH_IOERR );

  SnthLenPos += sizeof (ULONG);

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0]), 'FORM' );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), SnthSize );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_TypeId[0]), 'SNTH' );

  if ( (CWrite ( CFHandle, &IFF_Header, sizeof (struct IFFHeader )) != sizeof (struct IFFHeader )))
    return ( ERROR_SYNTH_IOERR );

  if ( (FaultCode = SaveTCIName ( CFHandle, SynthStr->tcy_FileName, 'FILE', (ULONG *) &SnthSize, ERROR_SYNTH_IOERR )))
    return ( FaultCode );

  if ( (FaultCode = SaveTCIName ( CFHandle, SynthStr->tcy_Name, 'NAME', (ULONG *) &SnthSize, ERROR_SYNTH_IOERR )))
    return ( FaultCode );

  WriteMWord ( (UBYTE *) &(YHDR.myhdr_NumWaves), GetNumWaveforms ( SynthStr ));

  memcpy ( &(YHDR.myhdr_VolumePos), &(SynthStr->tcy_VolumePos), (sizeof (struct MIFF_YHDR) - sizeof (struct IFFChunk)));

#ifndef MOTOROLA
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_VolumePos), YHDR.myhdr_VolumePos );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_PanningPos), YHDR.myhdr_PanningPos );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_SlidePos), YHDR.myhdr_SlidePos );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_SpecialPos), YHDR.myhdr_SpecialPos );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_VolumeSusPos), YHDR.myhdr_VolumeSusPos );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_PanningSusPos), YHDR.myhdr_PanningSusPos );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_SlideSusPos), YHDR.myhdr_SlideSusPos );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_SpecialSusPos), YHDR.myhdr_SpecialSusPos );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_VolNNAPos), YHDR.myhdr_VolNNAPos );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_PanNNAPos), YHDR.myhdr_PanNNAPos );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_SlideNNAPos), YHDR.myhdr_SlideNNAPos );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_SpecialNNAPos), YHDR.myhdr_SpecialNNAPos );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_VolDNAPos), YHDR.myhdr_VolDNAPos );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_PanDNAPos), YHDR.myhdr_PanDNAPos );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_SlideDNAPos), YHDR.myhdr_SlideDNAPos );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_SpecialDNAPos), YHDR.myhdr_SpecialDNAPos );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_Variable0), YHDR.myhdr_Variable0 );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_Variable1), YHDR.myhdr_Variable1 );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_Variable2), YHDR.myhdr_Variable2 );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_Variable3), YHDR.myhdr_Variable3 );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_Variable4), YHDR.myhdr_Variable4 );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_Variable5), YHDR.myhdr_Variable5 );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_Variable6), YHDR.myhdr_Variable6 );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_Variable7), YHDR.myhdr_Variable7 );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_Variable8), YHDR.myhdr_Variable8 );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_Variable9), YHDR.myhdr_Variable9 );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_VariableA), YHDR.myhdr_VariableA );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_VariableB), YHDR.myhdr_VariableB );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_VariableC), YHDR.myhdr_VariableC );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_VariableD), YHDR.myhdr_VariableD );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_VariableE), YHDR.myhdr_VariableE );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_VariableF), YHDR.myhdr_VariableF );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_VolConVar), YHDR.myhdr_VolConVar );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_PanConVar), YHDR.myhdr_PanConVar );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_SlideConVar), YHDR.myhdr_SlideConVar );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_SpecialConVar), YHDR.myhdr_SpecialConVar );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_VarKeepMask), YHDR.myhdr_VarKeepMask );
#endif

  if ( (FaultCode = SaveTCIChunk ( CFHandle, (struct IFFChunk *) &YHDR, 'YHDR', sizeof (struct MIFF_YHDR), (ULONG *) &SnthSize, ERROR_SYNTH_IOERR )))
    return ( FaultCode );

  WaveformStr = (struct TuComposerSynthWave *) &(SynthStr->tcy_Waves.mlh_Head);

  if ( WaveformStr->tcyw_Node.mln_Succ->mln_Succ )
  {
    SnthSize += (sizeof (struct IFFHeader) - sizeof (ULONG));
    FORMSize = sizeof (ULONG);

    if ( (FORMLenPos = CSeek ( CFHandle, 0L, OFFSET_CURRENT )) == 0xFFFFFFFF )
      return ( ERROR_SYNTH_IOERR );

    FORMLenPos += sizeof (ULONG);

    WriteMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0]), 'FORM' );
    WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), FORMSize );
    WriteMLong ( (UBYTE *) &(IFF_Header.ifh_TypeId[0]), 'WFRM' );

    if ( (CWrite ( CFHandle, &IFF_Header, sizeof (struct IFFHeader )) != sizeof (struct IFFHeader )))
      return ( ERROR_SYNTH_IOERR );

    for (;;)
    {
      WaveformStr = (struct TuComposerSynthWave *) WaveformStr->tcyw_Node.mln_Succ;

      if (!( WaveformStr->tcyw_Node.mln_Succ ))
        break;

      if ( (FaultCode = SaveSnthWave ( CFHandle, WaveformStr, (ULONG *) &FORMSize )))
        return ( FaultCode );
    }

    if ( (FORMLenPos = CSeek ( CFHandle, FORMLenPos, OFFSET_BEGINNING )) == 0xFFFFFFFF )
      return ( ERROR_SYNTH_IOERR );

    WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), FORMSize );

    if ( CWrite ( CFHandle, &(IFF_Header.ifh_Length[0]), sizeof (ULONG)) != sizeof (ULONG))
      return ( ERROR_SYNTH_IOERR );

    if ( CSeek ( CFHandle, FORMLenPos, OFFSET_BEGINNING ) == 0xFFFFFFFF )
      return ( ERROR_SYNTH_IOERR );

    SnthSize += FORMSize;
  }

  FORMSize = SynthStr->tcy_CodeEntries * sizeof (struct TuComposerSynthCode);

  WriteMLong ( (UBYTE *) &(IFF_Chunk.iff_Identifier[0]), 'CODE' );
  WriteMLong ( (UBYTE *) &(IFF_Chunk.iff_Length[0]), FORMSize );

  if ( (CWrite ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk )) != sizeof (struct IFFChunk )))
    return ( ERROR_SYNTH_IOERR );

  FORMSize += (FORMSize & 1);
  SnthSize += sizeof (struct IFFChunk) + FORMSize;

  if ( (CWrite ( CFHandle, SynthStr->tcy_SynthCode, FORMSize )) != FORMSize )
    return ( ERROR_SYNTH_IOERR );

  if ( FlagPassStr[2].flg_Value )
  {
    SymbolStr = (struct TuComposerSynthSymbolTable *) &(SynthStr->tcy_Symbol.mlh_Head);

    if ( SymbolStr->tsst_Node.mln_Succ->mln_Succ )
    {
      SnthSize += (sizeof (struct IFFHeader) - sizeof (ULONG));
      FORMSize = sizeof (ULONG);

      if ( (FORMLenPos = CSeek ( CFHandle, 0L, OFFSET_CURRENT )) == 0xFFFFFFFF )
        return ( ERROR_SYNTH_IOERR );

      FORMLenPos += sizeof (ULONG);

      WriteMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0]), 'FORM' );
      WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), FORMSize );
      WriteMLong ( (UBYTE *) &(IFF_Header.ifh_TypeId[0]), 'STAB' );

      if ( (CWrite ( CFHandle, &IFF_Header, sizeof (struct IFFHeader )) != sizeof (struct IFFHeader )))
        return ( ERROR_SYNTH_IOERR );

      for (;;)
      {
        SymbolStr = (struct TuComposerSynthSymbolTable *) SymbolStr->tsst_Node.mln_Succ;

        if (!( SymbolStr->tsst_Node.mln_Succ ))
          break;

        if ( (FaultCode = SaveSnthSmbl ( CFHandle, SymbolStr, (ULONG *) &FORMSize )))
          return ( FaultCode );
      }

      if ( (FORMLenPos = CSeek ( CFHandle, FORMLenPos, OFFSET_BEGINNING )) == 0xFFFFFFFF )
        return ( ERROR_SYNTH_IOERR );

      WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), FORMSize );

      if ( CWrite ( CFHandle, &(IFF_Header.ifh_Length[0]), sizeof (ULONG)) != sizeof (ULONG))
        return ( ERROR_SYNTH_IOERR );

      if ( CSeek ( CFHandle, FORMLenPos, OFFSET_BEGINNING ) == 0xFFFFFFFF )
        return ( ERROR_SYNTH_IOERR );

      SnthSize += FORMSize;
    }
  }

  UnkChunk = (struct UnknownChunk *) &(SynthStr->tcy_UnknownChunks.mlh_Head);

  for (;;)
  {
    UnkChunk = (struct UnknownChunk *) UnkChunk->uc_Node.mln_Succ;

    if (!( UnkChunk->uc_Node.mln_Succ ))
      break;

    if ( (FaultCode = SaveTCIChunk ( CFHandle, (struct IFFChunk *) &UnkChunk, ReadMLong ( (UBYTE *) &(UnkChunk->uc_Identifier[0])), ReadMLong ( (UBYTE *) &(UnkChunk->uc_Length[0])), &SnthSize, ERROR_SYNTH_IOERR )))
      return ( FaultCode );
  }

  if ( (SnthLenPos = CSeek ( CFHandle, SnthLenPos, OFFSET_BEGINNING )) == 0xFFFFFFFF )
    return ( ERROR_SYNTH_IOERR );

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), SnthSize );

  if ( CWrite ( CFHandle, &(IFF_Header.ifh_Length[0]), sizeof (ULONG)) != sizeof (ULONG))
    return ( ERROR_SYNTH_IOERR );

  if ( CSeek ( CFHandle, SnthLenPos, OFFSET_BEGINNING ) == 0xFFFFFFFF )
    return ( ERROR_SYNTH_IOERR );

  *ChunkLenBuf += SnthSize + (sizeof (struct IFFHeader) - sizeof (ULONG));

  return ( 0L );
}

static ULONG SaveSnthWave ( struct CFHandle *CFHandle, struct TuComposerSynthWave *WaveformStr, ULONG *ChunkLenBuf )
{
  struct IFFHeader IFF_Header;
  struct IFFChunk IFF_Chunk;
  struct IFF_WHDR WHDR;
  BYTE *WaveData = (BYTE *) WaveformStr->tcyw_WaveData;
  ULONG WaveSize = sizeof (ULONG), FORMSize;
  ULONG WaveLenPos;
  ULONG FaultCode, WaveLen;

  if ( (WaveLenPos = CSeek ( CFHandle, 0L, OFFSET_CURRENT )) == 0xFFFFFFFF )
    return ( ERROR_SYNTH_IOERR );

  WaveLenPos += sizeof (ULONG);

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0]), 'FORM' );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), WaveSize );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_TypeId[0]), 'WAVE' );

  if ( (CWrite ( CFHandle, &IFF_Header, sizeof (struct IFFHeader )) != sizeof (struct IFFHeader )))
    return ( ERROR_SYNTH_IOERR );

  if ( (FaultCode = SaveTCIName ( CFHandle, WaveformStr->tcyw_FileName, 'FILE', (ULONG *) &WaveSize, ERROR_SYNTH_IOERR )))
    return ( FaultCode );

  if ( (FaultCode = SaveTCIName ( CFHandle, WaveformStr->tcyw_Name, 'NAME', (ULONG *) &WaveSize, ERROR_SYNTH_IOERR )))
    return ( FaultCode );

   memcpy ( &(WHDR.whdr_Repeat), &(WaveformStr->tcyw_Repeat), (sizeof (struct IFF_WHDR) - sizeof (struct IFFChunk)));

#ifndef MOTOROLA
   WriteMLong ( (UBYTE *) &(WHDR.whdr_Repeat), WHDR.whdr_Repeat );
   WriteMLong ( (UBYTE *) &(WHDR.whdr_RepLen), WHDR.whdr_RepLen );
   WriteMWord ( (UBYTE *) &(WHDR.whdr_Flags), WHDR.whdr_Flags );
#endif

  if ( (FaultCode = SaveTCIChunk ( CFHandle, (struct IFFChunk *) &WHDR, 'WHDR', sizeof (struct IFF_WHDR), (ULONG *) &WaveSize, ERROR_SYNTH_IOERR )))
    return ( FaultCode );

  FORMSize = WaveformStr->tcyw_Length;
  WaveLen = WaveformStr->tcyw_Samples;

  WriteMLong ( (UBYTE *) &(IFF_Chunk.iff_Identifier[0]), 'BODY' );
  WriteMLong ( (UBYTE *) &(IFF_Chunk.iff_Length[0]), FORMSize );

  if ( (CWrite ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk )) != sizeof (struct IFFChunk )))
    return ( ERROR_SYNTH_IOERR );

  FORMSize += (FORMSize & 1);
  WaveSize += sizeof (struct IFFChunk) + FORMSize;

#ifndef MOTOROLA
  if ( WaveformStr->tcyw_Flags & TCYWF_8Bit )
  {
    WaveLen = WaveformStr->tcyw_Length;

    if ( WaveLen & 1 )
      WaveData[WaveLen] = 0;

    if ( (CWrite ( CFHandle, WaveData, FORMSize )) != FORMSize )
      return ( ERROR_SYNTH_IOERR );
  } else
  {
    WORD *CopyData = AllocSampleData ( SAVESAMPLE_BUFFER * sizeof (WORD));
    ULONG WriteLeft;

    do
    {
      WORD *Buffer = CopyData;
      ULONG i;

      WriteLeft = SAVESAMPLE_BUFFER;

      if ( WaveLen < SAVESAMPLE_BUFFER )
        WriteLeft = WaveLen;

      for (i = WriteLeft; i > 0; i--)
      {
        *Buffer++ = ReadMWord ( (UBYTE *) WaveData++ );
      }

      if ( CWrite ( CFHandle, CopyData, (WriteLeft * sizeof (WORD))) != (WriteLeft * sizeof (WORD)))
      {
        FreeSampleData ( CopyData, SAVESAMPLE_BUFFER * sizeof (WORD));

        return ( ERROR_SYNTH_IOERR );
      }
    } while ( (WaveLen -= WriteLeft) );

    FreeSampleData ( CopyData, SAVESAMPLE_BUFFER * sizeof (WORD));

    if ( WaveformStr->tcyw_Length & 1 )
    {
      *(ULONG *) &(IFF_Chunk.iff_Identifier[0]) = 0L;

      if ( (CWrite ( CFHandle, &IFF_Chunk, 1 )) != 1 )
        return ( ERROR_SYNTH_IOERR );
    }
  }
#else
  if ( (CWrite ( CFHandle, WaveData, FORMSize )) != FORMSize )
    return ( ERROR_SYNTH_IOERR );
#endif

  if ( (WaveLenPos = CSeek ( CFHandle, WaveLenPos, OFFSET_BEGINNING )) == 0xFFFFFFFF )
    return ( ERROR_SYNTH_IOERR );

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), WaveSize );

  if ( CWrite ( CFHandle, &(IFF_Header.ifh_Length[0]), sizeof (ULONG)) != sizeof (ULONG))
    return ( ERROR_SYNTH_IOERR );

  if ( CSeek ( CFHandle, WaveLenPos, OFFSET_BEGINNING ) == 0xFFFFFFFF )
    return ( ERROR_SYNTH_IOERR );

  *ChunkLenBuf += WaveSize + (sizeof (struct IFFHeader) - sizeof (ULONG));

  return ( 0L );
}

static ULONG SaveSnthSmbl ( struct CFHandle *CFHandle, struct TuComposerSynthSymbolTable *SymbolStr, ULONG *ChunkLenBuf )
{
  struct IFFHeader IFF_Header;
  struct IFF_SREF SREF;
  ULONG SmblSize = sizeof (ULONG);
  ULONG SmblLenPos;
  ULONG FaultCode;

  if ( (SmblLenPos = CSeek ( CFHandle, 0L, OFFSET_CURRENT )) == 0xFFFFFFFF )
    return ( ERROR_SYNTH_IOERR );

  SmblLenPos += sizeof (ULONG);

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0]), 'FORM' );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), SmblSize );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_TypeId[0]), 'SMBL' );

  if ( (CWrite ( CFHandle, &IFF_Header, sizeof (struct IFFHeader )) != sizeof (struct IFFHeader )))
    return ( ERROR_SYNTH_IOERR );

  if ( (FaultCode = SaveTCIName ( CFHandle, SymbolStr->tsst_SymbolName, 'NAME', (ULONG *) &SmblSize, ERROR_SYNTH_IOERR )))
    return ( FaultCode );

  memcpy ( &(SREF.sref_SymbolValue), &(SymbolStr->tsst_SymbolValue), (sizeof (struct IFF_SREF) - sizeof (struct IFFChunk)));

#ifndef MOTOROLA
  WriteMWord ( (UBYTE *) &(SREF.sref_SymbolValue), SREF.sref_SymbolValue );
  WriteMWord ( (UBYTE *) &(SREF.sref_FirstLine), SREF.sref_FirstLine );
  WriteMWord ( (UBYTE *) &(SREF.sref_LastLine), SREF.sref_LastLine );
#endif

  if ( (FaultCode = SaveTCIChunk ( CFHandle, (struct IFFChunk *) &SREF, 'SREF', sizeof (struct IFF_SREF), (ULONG *) &SmblSize, ERROR_SYNTH_IOERR )))
    return ( FaultCode );

  if ( (SmblLenPos = CSeek ( CFHandle, SmblLenPos, OFFSET_BEGINNING )) == 0xFFFFFFFF )
    return ( ERROR_SYNTH_IOERR );

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), SmblSize );

  if ( CWrite ( CFHandle, &(IFF_Header.ifh_Length[0]), sizeof (ULONG)) != sizeof (ULONG))
    return ( ERROR_SYNTH_IOERR );

  if ( CSeek ( CFHandle, SmblLenPos, OFFSET_BEGINNING ) == 0xFFFFFFFF )
    return ( ERROR_SYNTH_IOERR );

  *ChunkLenBuf += SmblSize + (sizeof (struct IFFHeader) - sizeof (ULONG));

  return ( 0L );
}

static ULONG SaveTCIEnvd ( struct CFHandle *CFHandle, struct TuComposerInstrEnvelope *EnvStr, ULONG *ChunkLenBuf )
{
  struct IFFHeader IFF_Header;
  struct IFFChunk IFF_Chunk;
  struct MIFF_EHDR EHDR;
  struct UnknownChunk *UnkChunk;
  WORD *EnvData = EnvStr->tcie_Data;
  UWORD *EnvNode = EnvStr->tcie_NodePoints;
#ifndef MOTOROLA
  WORD *CopyData;
  UWORD *CopyNode;
  ULONG WriteLeft;
#endif
  ULONG EnvdSize = sizeof (ULONG), FORMSize;
  ULONG EnvdLenPos;
  ULONG FaultCode, EnvLen;

  if ( (EnvdLenPos = CSeek ( CFHandle, 0L, OFFSET_CURRENT )) == 0xFFFFFFFF )
    return ( ERROR_INSTR_IOERR );

  EnvdLenPos += sizeof (ULONG);

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0]), 'FORM' );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), EnvdSize );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_TypeId[0]), 'ENVD' );

  if ( (CWrite ( CFHandle, &IFF_Header, sizeof (struct IFFHeader )) != sizeof (struct IFFHeader )))
    return ( ERROR_INSTR_IOERR );

  WriteMWord ( (UBYTE *) &(EHDR.mehdr_Flags), EnvStr->tcie_Flags );
  WriteMWord ( (UBYTE *) &(EHDR.mehdr_Tempo), EnvStr->tcie_Tempo );
  WriteMWord ( (UBYTE *) &(EHDR.mehdr_SustainStart), EnvStr->tcie_SustainStart );
  WriteMWord ( (UBYTE *) &(EHDR.mehdr_SustainEnd), EnvStr->tcie_SustainEnd );
  WriteMWord ( (UBYTE *) &(EHDR.mehdr_SustainCount), EnvStr->tcie_SustainCount );
  WriteMWord ( (UBYTE *) &(EHDR.mehdr_LoopStart), EnvStr->tcie_LoopStart );
  WriteMWord ( (UBYTE *) &(EHDR.mehdr_LoopEnd), EnvStr->tcie_LoopEnd );
  WriteMWord ( (UBYTE *) &(EHDR.mehdr_LoopCount), EnvStr->tcie_LoopCount );
  WriteMWord ( (UBYTE *) &(EHDR.mehdr_LowestValue), EnvStr->tcie_LowestValue );
  WriteMWord ( (UBYTE *) &(EHDR.mehdr_HighestValue), EnvStr->tcie_HighestValue );

  if ( (FaultCode = SaveTCIChunk ( CFHandle, (struct IFFChunk *) &EHDR, 'EHDR', sizeof (struct MIFF_EHDR), (ULONG *) &EnvdSize, ERROR_INSTR_IOERR )))
    return ( FaultCode );

  EnvLen = EnvStr->tcie_NumPoints;
  FORMSize = EnvLen * sizeof (WORD);

  WriteMLong ( (UBYTE *) &(IFF_Chunk.iff_Identifier[0]), 'BODY' );
  WriteMLong ( (UBYTE *) &(IFF_Chunk.iff_Length[0]), FORMSize );

  if ( (CWrite ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk )) != sizeof (struct IFFChunk )))
    return ( ERROR_INSTR_IOERR );

  FORMSize += (FORMSize & 1);
  EnvdSize += sizeof (struct IFFChunk) + FORMSize;

#ifndef MOTOROLA
  CopyData = AllocSampleData ( SAVESAMPLE_BUFFER * sizeof (WORD));

  do
  {
    WORD *Buffer = CopyData;
    ULONG i;

    WriteLeft = SAVESAMPLE_BUFFER;

    if ( EnvLen < SAVESAMPLE_BUFFER )
      WriteLeft = EnvLen;

    for (i = WriteLeft; i > 0; i--)
    {
      *Buffer++ = ReadMWord ( (UBYTE *) EnvData++ );
    }

    if ( CWrite ( CFHandle, CopyData, (WriteLeft * sizeof (WORD))) != (WriteLeft * sizeof (WORD)))
    {
      FreeSampleData ( CopyData, SAVESAMPLE_BUFFER * sizeof (WORD));

      return ( ERROR_INSTR_IOERR );
    }
  } while ( (EnvLen -= WriteLeft) );

  FreeSampleData ( CopyData, SAVESAMPLE_BUFFER * sizeof (WORD));

  if ( (EnvStr->tcie_NumPoints * sizeof (WORD)) & 1 )
  {
    *(ULONG *) &(IFF_Chunk.iff_Identifier[0]) = 0L;

    if ( (CWrite ( CFHandle, &IFF_Chunk, 1 )) != 1 )
      return ( ERROR_INSTR_IOERR );
  }
#else
  if ( (CWrite ( CFHandle, EnvStr->tcie_Data, FORMSize )) != FORMSize )
    return ( ERROR_INSTR_IOERR );
#endif

  if ( EnvNode )
  {
    EnvLen = EnvStr->tcie_NumNodes;
    FORMSize = EnvLen * sizeof (UWORD);

    WriteMLong ( (UBYTE *) &(IFF_Chunk.iff_Identifier[0]), 'NODE' );
    WriteMLong ( (UBYTE *) &(IFF_Chunk.iff_Length[0]), FORMSize );

    if ( (CWrite ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk )) != sizeof (struct IFFChunk )))
      return ( ERROR_INSTR_IOERR );

    FORMSize += (FORMSize & 1);
    EnvdSize += sizeof (struct IFFChunk) + FORMSize;

#ifndef MOTOROLA
    CopyNode = (UWORD *) AllocSampleData ( SAVESAMPLE_BUFFER * sizeof (UWORD));

    do
    {
      UWORD *Buffer = CopyNode;
      ULONG i;

      WriteLeft = SAVESAMPLE_BUFFER;

      if ( EnvLen < SAVESAMPLE_BUFFER )
        WriteLeft = EnvLen;

      for (i = WriteLeft; i > 0; i--)
      {
        *Buffer++ = ReadMWord ( (UBYTE *) EnvNode++ );
      }

      if ( CWrite ( CFHandle, CopyData, (WriteLeft * sizeof (WORD))) != (WriteLeft * sizeof (UWORD)))
      {
        FreeSampleData ( CopyData, SAVESAMPLE_BUFFER * sizeof (UWORD));

        return ( ERROR_INSTR_IOERR );
      }
    } while ( (EnvLen -= WriteLeft) );

    FreeSampleData ( CopyData, SAVESAMPLE_BUFFER * sizeof (UWORD));

    if ( (EnvStr->tcie_NumNodes * sizeof (UWORD)) & 1 )
    {
      *(ULONG *) &(IFF_Chunk.iff_Identifier[0]) = 0L;

      if ( (CWrite ( CFHandle, &IFF_Chunk, 1 )) != 1 )
        return ( ERROR_INSTR_IOERR );
    }
#else
    if ( (CWrite ( CFHandle, EnvStr->tcie_NodePoints, FORMSize )) != FORMSize )
      return ( ERROR_INSTR_IOERR );
#endif
  }

  UnkChunk = (struct UnknownChunk *) &(EnvStr->tcie_UnknownChunks.mlh_Head);

  for (;;)
  {
    UnkChunk = (struct UnknownChunk *) UnkChunk->uc_Node.mln_Succ;

    if (!( UnkChunk->uc_Node.mln_Succ ))
      break;

    if ( (FaultCode = SaveTCIChunk ( CFHandle, (struct IFFChunk *) &UnkChunk, ReadMLong ( (UBYTE *) &(UnkChunk->uc_Identifier[0])), ReadMLong ( (UBYTE *) &(UnkChunk->uc_Length[0])), &EnvdSize, ERROR_INSTR_IOERR )))
      return ( FaultCode );
  }

  if ( (EnvdLenPos = CSeek ( CFHandle, EnvdLenPos, OFFSET_BEGINNING )) == 0xFFFFFFFF )
    return ( ERROR_INSTR_IOERR );

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), EnvdSize );

  if ( CWrite ( CFHandle, &(IFF_Header.ifh_Length[0]), sizeof (ULONG)) != sizeof (ULONG))
    return ( ERROR_INSTR_IOERR );

  if ( CSeek ( CFHandle, EnvdLenPos, OFFSET_BEGINNING ) == 0xFFFFFFFF )
    return ( ERROR_INSTR_IOERR );

  *ChunkLenBuf += EnvdSize + (sizeof (struct IFFHeader) - sizeof (ULONG));

  return ( 0L );
}

static ULONG SaveTCIKbrd ( struct CFHandle *CFHandle, struct TuComposerInstrKeyboard *KeybrdStr, ULONG *ChunkLenBuf )
{
  struct IFF_KBRD KBRD;
#ifndef MOTOROLA
  struct KeyboardEntry *KeybrdEntry;
  ULONG i;
#endif

  memcpy ( &(KBRD.kbrd_Keys[0]), &(KeybrdStr->tcik_NoteEntry[0]), (sizeof (struct IFF_KBRD) - sizeof (struct IFFChunk)));

#ifndef MOTOROLA
  KeybrdEntry = (struct KeyboardEntry *) &(KBRD.kbrd_Keys[0]);

  for (i = NUMOCTAVES * NUMNOTES; i > 0; i--)
  {
    WriteMWord ( (UBYTE *) &(KeybrdEntry->tcik_Sample_C_0), KeybrdEntry->tcik_Sample_C_0 );

    KeybrdEntry++;
  }
#endif

  return ( SaveTCIChunk ( CFHandle, (struct IFFChunk *) &KBRD, 'KBRD', sizeof (struct IFF_KBRD), (ULONG *) ChunkLenBuf, ERROR_INSTR_IOERR ));
}

static ULONG SaveTCIArpg ( struct CFHandle *CFHandle, struct TuComposerInstrArpeggio *ArpStr, ULONG *ChunkLenBuf )
{
  struct IFFHeader IFF_Header;
  struct IFF_AHDR AHDR;
  ULONG ArpgSize = sizeof (ULONG), FORMSize;
  ULONG ArpgLenPos, FORMLenPos;
  ULONG FaultCode;

  if ( (ArpgLenPos = CSeek ( CFHandle, 0L, OFFSET_CURRENT )) == 0xFFFFFFFF )
    return ( ERROR_INSTR_IOERR );

  ArpgLenPos += sizeof (ULONG);

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0]), 'FORM' );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), ArpgSize );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_TypeId[0]), 'ARPG' );

  if ( (CWrite ( CFHandle, &IFF_Header, sizeof (struct IFFHeader )) != sizeof (struct IFFHeader )))
    return ( ERROR_INSTR_IOERR );

  WriteMWord ( (UBYTE *) &(AHDR.ahdr_Length), ArpStr->tcia_Entries );
  WriteMWord ( (UBYTE *) &(AHDR.ahdr_Flags), ArpStr->tcia_Flags );
  WriteMWord ( (UBYTE *) &(AHDR.ahdr_SustainStart), ArpStr->tcia_SustainStart );
  WriteMWord ( (UBYTE *) &(AHDR.ahdr_SustainEnd), ArpStr->tcia_SustainEnd );
  WriteMWord ( (UBYTE *) &(AHDR.ahdr_SustainCount), ArpStr->tcia_SustainCount );
  WriteMWord ( (UBYTE *) &(AHDR.ahdr_LoopStart), ArpStr->tcia_LoopStart );
  WriteMWord ( (UBYTE *) &(AHDR.ahdr_LoopEnd), ArpStr->tcia_LoopEnd );

  if ( (FaultCode = SaveTCIChunk ( CFHandle, (struct IFFChunk *) &AHDR, 'AHDR', sizeof (struct IFF_AHDR), (ULONG *) &ArpgSize, ERROR_INSTR_IOERR )))
    return ( FaultCode );

  ArpgSize += (sizeof (struct IFFHeader) - sizeof (ULONG));
  FORMSize = sizeof (ULONG);

  if ( (FORMLenPos = CSeek ( CFHandle, 0L, OFFSET_CURRENT )) == 0xFFFFFFFF )
    return ( ERROR_INSTR_IOERR );

  FORMLenPos += sizeof (ULONG);

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0]), 'FORM' );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), FORMSize );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_TypeId[0]), 'ARPE' );

  if ( (CWrite ( CFHandle, &IFF_Header, sizeof (struct IFFHeader )) != sizeof (struct IFFHeader )))
    return ( ERROR_INSTR_IOERR );

  if ( (FaultCode = SaveArpgArpe ( CFHandle, ArpStr, (ULONG *) &FORMSize )))
    return ( FaultCode );

  if ( (FORMLenPos = CSeek ( CFHandle, FORMLenPos, OFFSET_BEGINNING )) == 0xFFFFFFFF )
    return ( ERROR_INSTR_IOERR );

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), FORMSize );

  if ( CWrite ( CFHandle, &(IFF_Header.ifh_Length[0]), sizeof (ULONG)) != sizeof (ULONG))
    return ( ERROR_INSTR_IOERR );

  if ( CSeek ( CFHandle, FORMLenPos, OFFSET_BEGINNING ) == 0xFFFFFFFF )
    return ( ERROR_INSTR_IOERR );

  ArpgSize += FORMSize;

  if ( (ArpgLenPos = CSeek ( CFHandle, ArpgLenPos, OFFSET_BEGINNING )) == 0xFFFFFFFF )
    return ( ERROR_INSTR_IOERR );

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), ArpgSize );

  if ( CWrite ( CFHandle, &(IFF_Header.ifh_Length[0]), sizeof (ULONG)) != sizeof (ULONG))
    return ( ERROR_INSTR_IOERR );

  if ( CSeek ( CFHandle, ArpgLenPos, OFFSET_BEGINNING ) == 0xFFFFFFFF )
    return ( ERROR_INSTR_IOERR );

  *ChunkLenBuf += ArpgSize + (sizeof (struct IFFHeader) - sizeof (ULONG));

  return ( 0L );
}

static ULONG SaveArpgArpe ( struct CFHandle *CFHandle, struct TuComposerInstrArpeggio *ArpStr, ULONG *ChunkLenBuf )
{
  struct IFF_ARPE ARPE;
  struct TuComposerInstrArpeggioData *ArpData = ArpStr->tcia_Data;
  ULONG FaultCode, i;

  for (i = ArpStr->tcia_Entries; i > 0; i--)
  {
    memcpy ( &(ARPE.arpe_Tone), ArpData, (sizeof (struct IFF_ARPE) - sizeof (struct IFFChunk )));

#ifndef MOTOROLA
    WriteMWord ( (UBYTE *) &(ARPE.arpe_DataWord), ARPE.arpe_DataWord );
    WriteMWord ( (UBYTE *) &(ARPE.arpe_DataWord2), ARPE.arpe_DataWord2 );
    WriteMWord ( (UBYTE *) &(ARPE.arpe_DataWord3), ARPE.arpe_DataWord3 );
    WriteMWord ( (UBYTE *) &(ARPE.arpe_DataWord4), ARPE.arpe_DataWord4 );
#endif

    if ( (FaultCode = SaveTCIChunk ( CFHandle, (struct IFFChunk *) &ARPE, 'ARPE', sizeof (struct IFF_ARPE), (ULONG *) ChunkLenBuf, ERROR_INSTR_IOERR )))
      return ( FaultCode );

    ArpData++;
  }

  return ( 0L );
}

static ULONG GetTCIInfo ( struct CFHandle *CFHandle,
    struct InstrInfo *InstrInfo, unsigned long NumInstr )
{
  struct IFFHeader IFF_Header;
  ULONG Length, IFFSize;
  UBYTE ScanFlags = 0;

  for (;;)
  {
    if ( (CRead ( CFHandle, &IFF_Header, sizeof (struct IFFHeader)) != sizeof (struct IFFHeader)) )
      return ( 0L );

    if ( (ReadMLong ( (UBYTE *) &IFF_Header.ifh_FORMId[0] ) != 'FORM') || ( (Length = ReadMLong ( (UBYTE *) &IFF_Header.ifh_Length[0]) ) < 4) || ((ReadMLong ( (UBYTE *) &IFF_Header.ifh_TypeId[0] ) >> 8) != 'TCI') )
      return ( 0L );

    Length -= 4;

    if (!( NumInstr-- ))
      break;

    if ( CSeek ( CFHandle, Length, OFFSET_CURRENT ) == 0xFFFFFFFF )
      return ( 0L );
  }

  do
  {
    struct IFFChunk IFF_Chunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      return ( 0L );

    if ( (IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0] )) & 1 )
      IFFSize++;

    IFFLength = IFFSize;

    switch ( ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] ) )
    {
      case 'FILE' :
      {
        if ( ScanFlags & 0x01 )
          break;

        if (!( InstrInfo->ii_FileName = (STRPTR) AllocPMem ( ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0] ) + 1) ))
          goto GetTCIError;

        if ( (CRead ( CFHandle, InstrInfo->ii_FileName, IFFLength ) != IFFLength) )
          goto GetTCIError;

        IFFLength = 0L;
        ScanFlags |= 0x01;

        break;
      }
      case 'NAME' :
      {
        if ( ScanFlags & 0x02 )
          break;

        if (!( InstrInfo->ii_Name = (STRPTR) AllocPMem ( ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0] ) + 1) ))
          goto GetTCIError;

        if ( (CRead ( CFHandle, InstrInfo->ii_Name, IFFLength ) != IFFLength) )
          goto GetTCIError;

        IFFLength = 0L;
        ScanFlags |= 0x02;

        break;
      }
      case 'IHDR' :
      {
        if ( ScanFlags & 0x04 )
          break;

        if ( (CRead ( CFHandle, &(InstrInfo->ii_Version), (sizeof (struct InstrInfo) - sizeof (struct IFFChunk))) != (sizeof (struct InstrInfo) - sizeof (struct IFFChunk))) )
          goto GetTCIError;

#ifndef MOTOROLA
        InstrInfo->ii_VolumeEnvelope = ReadMWord ( (UBYTE *) &(InstrInfo->ii_VolumeEnvelope));
        InstrInfo->ii_PanningEnvelope = ReadMWord ( (UBYTE *) &(InstrInfo->ii_PanningEnvelope));
        InstrInfo->ii_SlideEnvelope = ReadMWord ( (UBYTE *) &(InstrInfo->ii_SlideEnvelope));
        InstrInfo->ii_VibratoEnvelope = ReadMWord ( (UBYTE *) &(InstrInfo->ii_VibratoEnvelope));
        InstrInfo->ii_TremoloEnvelope = ReadMWord ( (UBYTE *) &(InstrInfo->ii_TremoloEnvelope));
        InstrInfo->ii_PannoloEnvelope = ReadMWord ( (UBYTE *) &(InstrInfo->ii_PannoloEnvelope));
        InstrInfo->ii_ChannoloEnvelope = ReadMWord ( (UBYTE *) &(InstrInfo->ii_ChannoloEnvelope));
        InstrInfo->ii_SpenoloEnvelope = ReadMWord ( (UBYTE *) &(InstrInfo->ii_SpenoloEnvelope));
        InstrInfo->ii_ArpeggioStructure = ReadMWord ( (UBYTE *) &(InstrInfo->ii_ArpeggioStructure));
        InstrInfo->ii_KeyboardDefs = ReadMWord ( (UBYTE *) &(InstrInfo->ii_KeyboardDefs));
        InstrInfo->ii_VolumeSwing = ReadMWord ( (UBYTE *) &(InstrInfo->ii_VolumeSwing));
        InstrInfo->ii_PanningSwing = ReadMWord ( (UBYTE *) &(InstrInfo->ii_PanningSwing));
        InstrInfo->ii_PitchSwing = ReadMLong ( (UBYTE *) &(InstrInfo->ii_PitchSwing));
        InstrInfo->ii_PitchPanSeparation = ReadMWord ( (UBYTE *) &(InstrInfo->ii_PitchPanSeparation));
        InstrInfo->ii_EnvUsageFlags = ReadMWord ( (UBYTE *) &(InstrInfo->ii_EnvUsageFlags));
        InstrInfo->ii_EnvProcFlags = ReadMWord ( (UBYTE *) &(InstrInfo->ii_EnvProcFlags));
        InstrInfo->ii_EnvRandomFlags = ReadMWord ( (UBYTE *) &(InstrInfo->ii_EnvRandomFlags));
        InstrInfo->ii_EnvRndDelayFlags = ReadMWord ( (UBYTE *) &(InstrInfo->ii_EnvRndDelayFlags));
        InstrInfo->ii_FadeOut = ReadMWord ( (UBYTE *) &(InstrInfo->ii_FadeOut));
        InstrInfo->ii_Hold = ReadMWord ( (UBYTE *) &(InstrInfo->ii_Hold));
        InstrInfo->ii_Decay = ReadMWord ( (UBYTE *) &(InstrInfo->ii_Decay));
#endif

        IFFLength -= (sizeof (struct InstrInfo) - sizeof (struct IFFChunk));
        ScanFlags |= 0x04;

        break;
      }
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
      {
GetTCIError:
        FreeNameStr ( InstrInfo->ii_FileName );
        FreeNameStr ( InstrInfo->ii_Name );

        return ( 0L );
      }
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      goto GetTCIError;

  } while ( (Length -= IFFSize) );

  if (!( ScanFlags & 0x04 ))
    goto GetTCIError;

  return ( -1L );
}

static ULONG GetNumTCIs ( struct CFHandle *CFHandle )
{
  struct IFFHeader IFF_Header;
  ULONG NumTCIs = 0L;

  for (;;)
  {
    ULONG Length;

    if ( (CRead ( CFHandle, &IFF_Header, sizeof (struct IFFHeader)) != sizeof (struct IFFHeader)) )
      break;

    if ( (ReadMLong ( (UBYTE *) &IFF_Header.ifh_FORMId[0] ) != 'FORM') || ( (Length = ReadMLong ( (UBYTE *) &IFF_Header.ifh_Length[0]) ) < 4) )
      break;

    if ( CSeek ( CFHandle, Length - 4, OFFSET_CURRENT ) == 0xFFFFFFFF )
      break;

    if ( ((ReadMLong ( (UBYTE *) &IFF_Header.ifh_TypeId[0] ) >> 8) != 'TCI') )
      continue;

    NumTCIs++;
  }

  return ( NumTCIs );
}

static ULONG GetInstrEnvelopeNum ( struct TuComposerInstrEnvelope **InstrEnvStr, struct TuComposerInstrEnvelope *EnvStr )
{
  ULONG Count = 0L;

  if ( EnvStr )
  {
    for (;;)
    {
      struct TuComposerInstrEnvelope *EnvScan;

      if (!( EnvScan = *InstrEnvStr++ ))
      {
        Count = 0L;

        break;
      }

      Count++;

      if ( EnvScan == EnvStr )
        break;
    }
  }

  return ( Count );
}

static void DeltaCrunch8 ( BYTE **SampleDataPos, BYTE *Buffer, BYTE *DeltaPos, ULONG Length )
{
  BYTE *SampleData = *SampleDataPos;
  ULONG i;
  BYTE Delta = *DeltaPos;

  for (i = Length; i > 0; i--)
  {
    BYTE Sample = *SampleData++;

    Sample -= Delta;
    *Buffer++ = Sample;

    Delta += Sample;
  }

  *SampleDataPos = SampleData;
  *DeltaPos = Delta;
}

static void DeltaCrunch16 ( WORD **SampleDataPos, WORD *Buffer, WORD *DeltaPos, ULONG Length )
{
  WORD *SampleData = *SampleDataPos;
  ULONG i;
  WORD Delta = *DeltaPos;

  for (i = Length; i > 0; i--)
  {
    WORD Sample = *SampleData++;

    Sample -= Delta;
    WriteMWord ( (UBYTE *) Buffer++, Sample );

    Delta += Sample;
  }

  *SampleDataPos = SampleData;
  *DeltaPos = Delta;
}

static void DeltaCrunch32 ( LONG **SampleDataPos, LONG *Buffer, LONG *DeltaPos, ULONG Length )
{
  LONG *SampleData = *SampleDataPos;
  ULONG i;
  LONG  Delta = *DeltaPos;

  for (i = Length; i > 0; i--)
  {
    LONG Sample = *SampleData++;

    Sample -= Delta;
    WriteMLong ( (UBYTE *) Buffer++, Sample );

    Delta += Sample;
  }

  *SampleDataPos = SampleData;
  *DeltaPos = Delta;
}

static void DeltaCrunchX ( LONG **SampleDataPos, LONG *Buffer, LONG *DeltaPos, ULONG BitDepth, ULONG Length )
{
  LONG *SampleData = *SampleDataPos;
  ULONG i;
  LONG  Delta = *DeltaPos;
  ULONG Bit = 0L;

  for (i = Length; i > 0; i--)
  {
    ULONG Sample;

    if ( (Bit + BitDepth) < 32 )
    {
      Sample = (*SampleData << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
    } else
    {
      Sample = *SampleData++ << Bit;
      Sample |= (*SampleData & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
    }

    Sample -= Delta;

    Sample &= ~((1UL << (32UL - BitDepth)) - 1UL );

    WriteMLong ( (UBYTE *) Buffer, ReadMLong ( (UBYTE *) Buffer ) | (Sample >> Bit));

    if ( (Bit + BitDepth) >= 32 )
    {
      Buffer++;
      WriteMLong ( (UBYTE *) Buffer, (Sample << (32UL - Bit)));

      Bit -= 32;
    }

    Bit += BitDepth;

    Delta += Sample;
  }

  *SampleDataPos = SampleData;
  *DeltaPos = Delta;
}

#ifndef MOTOROLA

static UWORD ReadMWord ( UBYTE *Buffer )
{
  UWORD MotorolaWord;

  MotorolaWord = *Buffer++ << 8U;
  MotorolaWord += *Buffer;

  return ( MotorolaWord );
}

static void WriteMWord ( UBYTE *Buffer, UWORD Value )
{
  *Buffer++ = Value >> 8U;
  *Buffer = Value;
}

static ULONG ReadMLong ( UBYTE *Buffer )
{
  ULONG MotorolaLong;

  MotorolaLong = *Buffer++;
  MotorolaLong <<= 8UL;
  MotorolaLong += *Buffer++;
  MotorolaLong <<= 8UL;
  MotorolaLong += *Buffer++;
  MotorolaLong <<= 8UL;
  MotorolaLong += *Buffer;

  return ( MotorolaLong );
}

static void WriteMLong ( UBYTE *Buffer, ULONG Value )
{
  *Buffer++ = Value >> 24UL;
  *Buffer++ = Value >> 16UL;
  *Buffer++ = Value >> 8UL;
  *Buffer = Value;
}

#endif
