/* TuComposer AmigaOS dependent code
**
** $VER: InternalTCT.c 31.00 (9.2.00)
**
** (C) 2000 by Sebastian Vater (known as Basty/Seasons).
**
*/

#include <stddef.h>
#include <string.h>
#include <exec/types.h>
#include <exec/libraries.h>
#include <exec/lists.h>
#include <exec/nodes.h>
#include <tucomposer/tucomposer.h>
#include <tucomposer/track.h>
#include <tucomposer/external/external.h>
#include <tucomposer/external/track.h>
#include <proto/dos.h>
#include <proto/tucomposer.h>

#include "compiler.h"

#ifdef __cplusplus
extern "C" {
#endif

#ifndef MOTOROLA
  static UWORD ReadMWord ( UBYTE *Buffer );
  static void WriteMWord ( UBYTE *Buffer, UWORD Value );

  static ULONG ReadMLong ( UBYTE *Buffer );
  static void WriteMLong ( UBYTE *Buffer, ULONG Value );
#else
  #define ReadMWord( Buffer ) (*(UWORD *) (Buffer))
  #define WriteMWord( Buffer, Value ) (*(UWORD *) (Buffer)) = Value

  #define ReadMLong( Buffer ) (*(ULONG *) (Buffer))
  #define WriteMLong( Buffer, Value ) (*(ULONG *) (Buffer)) = Value
#endif

static struct TuComposerExtLibBase *InternalTCTOpenExtLib ( void );
static BPTR InternalTCTCloseExtLib ( void );
static BPTR ExtLib ( void );
static ULONG IdentifyTCT ( struct CFHandle *CFHandle );
static struct TuComposerTrack *LoadTCT ( struct CFHandle *CFHandle,
    struct TuComposerTrack *TrackStr, union FlagPass *FlagPassStr,
    unsigned long NumTrack );
static ULONG SaveTCT ( struct CFHandle *CFHandle,
    struct TuComposerTrack *TrackStr, union FlagPass *FlagPassStr );
static ULONG GetTCTInfo ( struct CFHandle *CFHandle,
    struct TrackInfo *TrackInfo, unsigned long NumTrack );
static ULONG GetNumTCTs ( struct CFHandle *CFHandle );

static ULONG SaveTCTChunk ( struct CFHandle *CFHandle, struct IFFChunk *IFF_Chunk, unsigned long ChunkName, unsigned long ChunkLen, ULONG *ChunkLenBuf, unsigned long ErrorCode );
static ULONG SaveTCTName ( struct CFHandle *CFHandle, STRPTR Name, unsigned long ChunkName, ULONG *ChunkLenBuf, unsigned long ErrorCode );

#ifdef __cplusplus
}
#endif

extern struct TrackLibrary InternalTCTBase;

static UBYTE InternalTCTName[] = "internal_tct.library\0";
static UBYTE InternalTCTLoadID[] = "TuComposer Track (TCT)\0";
static UBYTE InternalTCTIDString[] = "Internal TCT Lib Version 1.00 (v31) (09 Feb 00) by Basty/Seasons.\0";

struct TuComposerExtList InternalTCT =
{
  {NULL, NULL},
  EXTLF_Intern|EXTLF_Loader|EXTLF_Saver, 0,
  (struct TuComposerExtLibBase *) &InternalTCTBase,
  NULL,
  (STRPTR) &InternalTCTName,
  (STRPTR) &InternalTCTLoadID,
  NULL,
  NULL,
  0
};

#include "InternalTCT_library.h"

static struct TuComposerExtLibBase *InternalTCTOpenExtLib ( void )
{
  InternalTCTBase.trkl_Library.extb_LibNode.lib_OpenCnt++;

  return ( (struct TuComposerExtLibBase *) &InternalTCTBase );
}

static BPTR InternalTCTCloseExtLib ( void )
{
  InternalTCTBase.trkl_Library.extb_LibNode.lib_OpenCnt--;

  return ( NULL );
}

static BPTR ExtLib ( void )
{
  return ( NULL );
}

static ULONG IdentifyTCT ( struct CFHandle *CFHandle )
{
  struct IFFHeader ID_IFF;

  if ( (CRead ( CFHandle, &ID_IFF, sizeof (struct IFFHeader)) != sizeof (struct IFFHeader)) )
    return ( 0L );

  if ( (ReadMLong ( (UBYTE *) &ID_IFF.ifh_FORMId[0] ) != 'FORM') || ( ReadMLong ( (UBYTE *) &ID_IFF.ifh_Length[0] ) < 4) || ( (ReadMLong ( (UBYTE *) &ID_IFF.ifh_TypeId[0]) >> 8) != 'TCT' ) )
    return ( 0L );

  return ( -1L );
}

static struct TuComposerTrack *LoadTCT ( struct CFHandle *CFHandle,
    struct TuComposerTrack *TrackStr, union FlagPass *FlagPassStr,
    ULONG NumTrack )
{
  struct IFFHeader IFF_Header;
  struct IFF_THDR THDR;
  struct TuComposerPacked PackedStr;
  struct UnknownChunk *UnkChunk;
  ULONG Length, IFFSize;
  ULONG Warning, Invalid = 0L;

  memset ( &(PackedStr.tcp_FileName), 0, sizeof (struct TuComposerPacked));

  InitNodeList ( (struct List *) &(PackedStr.tcp_UnknownChunks));

  for (;;)
  {
    if ( (CRead ( CFHandle, &IFF_Header, sizeof (struct IFFHeader)) != sizeof (struct IFFHeader)) )
      return ( NULL );

    if ( (ReadMLong ( (UBYTE *) &IFF_Header.ifh_FORMId[0] ) != 'FORM') || ( (Length = ReadMLong ( (UBYTE *) &IFF_Header.ifh_Length[0]) ) < 4) || ((ReadMLong ( (UBYTE *) &IFF_Header.ifh_TypeId[0] ) >> 8) != 'TCT') )
      return ( NULL );

    Length -= 4;

    if (!( NumTrack-- ))
      break;

    if ( CSeek ( CFHandle, Length, OFFSET_CURRENT ) == 0xFFFFFFFF )
      return ( NULL );
  }

  if ( (ReadMLong ( (UBYTE *) &IFF_Header.ifh_TypeId[0] ) != 'TCT1' ) )
  {
    SetIoErr ( ERROR_ACTION_NOT_KNOWN );

    return ( NULL );
  }

  do
  {
    struct IFFChunk IFF_Chunk;
    struct UnknownChunk *UnkChunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      goto LoadTCTHeaderErr;

    IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0]);
    IFFLength = IFFSize;

    IFFSize += (IFFSize & 1);

    switch ( ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] ) )
    {
      case 'FILE' :
      {
        if ( PackedStr.tcp_FileName )
          break;

        if (!( PackedStr.tcp_FileName = (STRPTR) AllocPMem ( IFFLength + 1 )))
          goto LoadTCTHeaderErr;

        if ( (CRead ( CFHandle, PackedStr.tcp_FileName, IFFLength ) != IFFLength) )
          goto LoadTCTHeaderErr;

        IFFLength &= 1;

        break;
      }
      case 'NAME' :
      {
        if ( PackedStr.tcp_Name )
          break;

        if (!( PackedStr.tcp_Name = (STRPTR) AllocPMem ( IFFLength + 1 )))
          goto LoadTCTHeaderErr;

        if ( (CRead ( CFHandle, PackedStr.tcp_Name, IFFLength ) != IFFLength) )
          goto LoadTCTHeaderErr;

        IFFLength &= 1;

        break;
      }
      case 'THDR' :
      {
        if ( Invalid )
          goto TCTIFFDamaged;

        if ( (CRead ( CFHandle, &(THDR.thdr_Version), (sizeof (struct IFF_THDR) - sizeof (struct IFFChunk))) != (sizeof (struct IFF_THDR) - sizeof (struct IFFChunk))))
          goto LoadTCTHeaderErr;

        Warning = 0L;

        if ( (THDR.thdr_Version > THDR_VERSION) )
          Warning = ERROR_ACTION_NOT_KNOWN;

        memcpy ( &(PackedStr.tcp_Length), &(THDR.thdr_Length), (sizeof (struct IFF_THDR) - offsetof (struct IFF_THDR, thdr_Length)));

#ifndef MOTOROLA
        PackedStr.tcp_Flags = ReadMWord ( (UBYTE *) &(PackedStr.tcp_Flags) );
        PackedStr.tcp_Frames = ReadMWord ( (UBYTE *) &(PackedStr.tcp_Frames) );
        PackedStr.tcp_SPDSpeed = ReadMWord ( (UBYTE *) &(PackedStr.tcp_SPDSpeed) );
        PackedStr.tcp_BPMTempo = ReadMWord ( (UBYTE *) &(PackedStr.tcp_BPMTempo) );
        PackedStr.tcp_BPMSpeed = ReadMWord ( (UBYTE *) &(PackedStr.tcp_BPMSpeed) );
#endif

        Invalid = ~Invalid;

        IFFLength = IFFSize - (sizeof (struct IFF_THDR) - sizeof (struct IFFChunk));

        break;
      }
      case 'BODY' :
      {
        if ( PackedStr.tcp_Data )
          goto TCTIFFDamaged;

        if (!( PackedStr.tcp_Data = (UBYTE *) AllocPMem ( IFFLength + 1 )))
          goto LoadTCTHeaderErr;

        if ( (CRead ( CFHandle, PackedStr.tcp_Data, IFFLength )) != IFFLength )
          goto LoadTCTHeaderErr;

        PackedStr.tcp_PackLength = IFFLength;

        IFFLength &= 1;

        break;
      }
      default :
      {
        if (!( UnkChunk = AllocUnknownChunk ( NULL, ReadMLong ( (UBYTE *) &(IFF_Chunk.iff_Identifier[0])), IFFLength )))
          goto LoadTCTHeaderErr;

        AddNodeTail ( (struct List *) &(PackedStr.tcp_UnknownChunks), (struct Node *) UnkChunk );

        if ( (CRead ( CFHandle, &(UnkChunk->uc_Data), IFFLength ) != IFFLength ) )
          goto LoadTCTHeaderErr;

        IFFLength &= 1;

        break;
      }
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
      {
LoadTCTHeaderErr:

        FreePMem ( PackedStr.tcp_Data, PackedStr.tcp_PackLength + 1 );

        UnkChunk = (struct UnknownChunk *) &(PackedStr.tcp_UnknownChunks.mlh_Head);

        for (;;)
        {
          UnkChunk = (struct UnknownChunk *) UnkChunk->uc_Node.mln_Succ;

          if (!( UnkChunk->uc_Node.mln_Succ ))
            break;

          FreeUnknownChunk ( UnkChunk );
        }

        return ( NULL );
      }
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      goto TCTIFFDamaged;

  } while ( (Length -= IFFSize) );

  if (!( Invalid ))
  {
TCTIFFDamaged:

    SetIoErr ( ERROR_INVALID_RESIDENT_LIBRARY );

    FreePMem ( PackedStr.tcp_Data, PackedStr.tcp_PackLength + 1 );

    UnkChunk = (struct UnknownChunk *) &(PackedStr.tcp_UnknownChunks.mlh_Head);

    for (;;)
    {
      UnkChunk = (struct UnknownChunk *) UnkChunk->uc_Node.mln_Succ;

      if (!( UnkChunk->uc_Node.mln_Succ ))
        break;

      FreeUnknownChunk ( UnkChunk );
    }

    return ( NULL );
  }

  if (!( TrackStr = UnPackTrack ( (struct TuComposerPacked *) &PackedStr, TrackStr )))
    goto LoadTCTHeaderErr;

  FreePMem ( PackedStr.tcp_Data, PackedStr.tcp_PackLength + 1 );

  UnkChunk = (struct UnknownChunk *) &(PackedStr.tcp_UnknownChunks.mlh_Head);

  for (;;)
  {
    UnkChunk = (struct UnknownChunk *) UnkChunk->uc_Node.mln_Succ;

    if (!( UnkChunk->uc_Node.mln_Succ ))
      break;

    FreeUnknownChunk ( UnkChunk );
  }

  SetIoErr ( Warning );

  return ( TrackStr );
}

static ULONG SaveTCT ( struct CFHandle *CFHandle,
    struct TuComposerTrack *TrackStr, union FlagPass *FlagPassStr )
{
  struct IFFHeader IFF_Header;
  struct IFFChunk IFF_Chunk;
  struct IFF_THDR THDR;
  struct TuComposerPacked *PackedStr;
  struct UnknownChunk *UnkChunk;
  ULONG TCTSize = sizeof (ULONG), FORMSize;
  ULONG TCTLenPos;
  ULONG FaultCode;

  if ( (TCTLenPos = CSeek ( CFHandle, 0L, OFFSET_CURRENT )) == 0xFFFFFFFF )
    return ( ERROR_TRACK_IOERR );

  TCTLenPos += sizeof (ULONG);

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0]), 'FORM' );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), TCTSize );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_TypeId[0]), 'TCT1' );

  if ( (CWrite ( CFHandle, &IFF_Header, sizeof (struct IFFHeader )) != sizeof (struct IFFHeader )))
    return ( ERROR_TRACK_IOERR );

  if ( (FaultCode = SaveTCTName ( CFHandle, TrackStr->tct_FileName, 'FILE', (ULONG *) &TCTSize, ERROR_TRACK_IOERR )))
    return ( FaultCode );

  if ( (FaultCode = SaveTCTName ( CFHandle, TrackStr->tct_Name, 'NAME', (ULONG *) &TCTSize, ERROR_TRACK_IOERR )))
    return ( FaultCode );

  THDR.thdr_Version = THDR_VERSION;
  THDR.thdr_Revision = THDR_REVISION;

  memcpy ( &(THDR.thdr_Length), &(TrackStr->tct_Length), (sizeof (struct IFF_THDR) - sizeof (struct IFFChunk)));

#ifndef MOTOROLA
  WriteMWord ( (UBYTE *) &(THDR.thdr_Length), THDR.thdr_Length );
  WriteMWord ( (UBYTE *) &(THDR.thdr_Flags), THDR.thdr_Flags );
  WriteMWord ( (UBYTE *) &(THDR.thdr_Frames), THDR.thdr_Frames );
  WriteMWord ( (UBYTE *) &(THDR.thdr_SPDSpeed), THDR.thdr_SPDSpeed );
  WriteMWord ( (UBYTE *) &(THDR.thdr_BPMTempo), THDR.thdr_BPMTempo );
  WriteMWord ( (UBYTE *) &(THDR.thdr_BPMSpeed), THDR.thdr_BPMSpeed );
#endif

  if ( (FaultCode = SaveTCTChunk ( CFHandle, (struct IFFChunk *) &THDR, 'THDR', sizeof (struct IFF_THDR), (ULONG *) &TCTSize, ERROR_TRACK_IOERR )))
    return ( FaultCode );

  if (!( PackedStr = PackTrack ( TrackStr )))
    return ( ERROR_TRACK_NOMEM );

  if ( (FORMSize = (PackedStr->tcp_PackLength - 1)))
  {
    WriteMLong ( (UBYTE *) &(IFF_Chunk.iff_Identifier[0]), 'BODY' );
    WriteMLong ( (UBYTE *) &(IFF_Chunk.iff_Length[0]), FORMSize );

    if ( (CWrite ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk )) != sizeof (struct IFFChunk )))
    {
      FreePackedTrack ( PackedStr );

      return ( ERROR_TRACK_IOERR );
    }

    FORMSize += (FORMSize & 1);
    TCTSize += sizeof (struct IFFChunk) + FORMSize;

    if ( (CWrite ( CFHandle, PackedStr->tcp_Data, FORMSize )) != FORMSize )
    {
      FreePackedTrack ( PackedStr );

      return ( ERROR_TRACK_IOERR );
    }
  }

  FreePackedTrack ( PackedStr );

  UnkChunk = (struct UnknownChunk *) &(TrackStr->tct_UnknownChunks.mlh_Head);

  for (;;)
  {
    UnkChunk = (struct UnknownChunk *) UnkChunk->uc_Node.mln_Succ;

    if (!( UnkChunk->uc_Node.mln_Succ ))
      break;

    if ( (FaultCode = SaveTCTChunk ( CFHandle, (struct IFFChunk *) &UnkChunk, ReadMLong ( (UBYTE *) &(UnkChunk->uc_Identifier[0])), ReadMLong ( (UBYTE *) &(UnkChunk->uc_Length[0])), &TCTSize, ERROR_TRACK_IOERR )))
      return ( FaultCode );
  }

  if ( CSeek ( CFHandle, TCTLenPos, OFFSET_BEGINNING ) == 0xFFFFFFFF )
    return ( ERROR_TRACK_IOERR );

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), TCTSize );

  if ( CWrite ( CFHandle, &(IFF_Header.ifh_Length[0]), sizeof (ULONG)) != sizeof (ULONG))
    return ( ERROR_TRACK_IOERR );

  return ( 0L );
}

static ULONG SaveTCTChunk ( struct CFHandle *CFHandle, struct IFFChunk *IFF_Chunk, unsigned long ChunkName, unsigned long ChunkLen, ULONG *ChunkLenBuf, unsigned long ErrorCode )
{
  ChunkLen -= sizeof (struct IFFChunk );

  if ( ChunkLen )
  {
    WriteMLong ( (UBYTE *) &(IFF_Chunk->iff_Identifier[0]), ChunkName );
    WriteMLong ( (UBYTE *) &(IFF_Chunk->iff_Length[0]), ChunkLen );

    if ( (CWrite ( CFHandle, IFF_Chunk, (ChunkLen + sizeof (struct IFFChunk)))) != (ChunkLen + sizeof (struct IFFChunk)))
      return ( ErrorCode );

    if ( ChunkLen & 1 )
    {
      *(ULONG *) &(IFF_Chunk->iff_Identifier[0]) = 0L;

      if ( (CWrite ( CFHandle, &IFF_Chunk, 1 )) != 1 )
        return ( ErrorCode );

      ChunkLen++;
    }

    *ChunkLenBuf += (ChunkLen + sizeof (struct IFFChunk));
  }

  return ( 0L );
}

static ULONG SaveTCTName ( struct CFHandle *CFHandle, STRPTR Name, unsigned long ChunkName, ULONG *ChunkLenBuf, unsigned long ErrorCode )
{
  struct IFFChunk IFF_Chunk;

  if ( Name )
  {
    ULONG Length = strlen ( Name );

    WriteMLong ( (UBYTE *) &(IFF_Chunk.iff_Identifier[0]), ChunkName );
    WriteMLong ( (UBYTE *) &(IFF_Chunk.iff_Length[0]), Length );

    Length += (Length & 1);

    if ( Length )
    {
      if ( (CWrite ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk))) != sizeof (struct IFFChunk))
        return ( ErrorCode );

      if ( (CWrite ( CFHandle, Name, Length )) != Length )
        return ( ErrorCode );

      *ChunkLenBuf += (Length + sizeof (struct IFFChunk));
    }
  }

  return ( 0L );
}

static ULONG GetTCTInfo ( struct CFHandle *CFHandle,
    struct TrackInfo *TrackInfo, unsigned long NumTrack )
{
  struct IFFHeader IFF_Header;
  struct IFF_THDR THDR;
  ULONG Length, IFFSize;
  UBYTE ScanFlags = 0;

  for (;;)
  {
    if ( (CRead ( CFHandle, &IFF_Header, sizeof (struct IFFHeader)) != sizeof (struct IFFHeader)) )
      return ( 0L );

    if ( (ReadMLong ( (UBYTE *) &IFF_Header.ifh_FORMId[0] ) != 'FORM') || ( (Length = ReadMLong ( (UBYTE *) &IFF_Header.ifh_Length[0]) ) < 4) || ((ReadMLong ( (UBYTE *) &IFF_Header.ifh_TypeId[0] ) >> 8) != 'TCT') )
      return ( 0L );

    Length -= 4;

    if (!( NumTrack-- ))
      break;

    if ( CSeek ( CFHandle, Length, OFFSET_CURRENT ) == 0xFFFFFFFF )
      return ( 0L );
  }

  do
  {
    struct IFFChunk IFF_Chunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      return ( 0L );

    if ( (IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0] )) & 1 )
      IFFSize++;

    IFFLength = IFFSize;

    switch ( ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] ) )
    {
      case 'FILE' :
      {
        if ( ScanFlags & 0x01 )
          break;

        if (!( TrackInfo->trki_FileName = (STRPTR) AllocPMem ( ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0] ) + 1) ))
          goto GetTCTError;

        if ( (CRead ( CFHandle, TrackInfo->trki_FileName, IFFLength ) != IFFLength) )
          goto GetTCTError;

        IFFLength = 0L;
        ScanFlags |= 0x01;

        break;
      }
      case 'NAME' :
      {
        if ( ScanFlags & 0x02 )
          break;

        if (!( TrackInfo->trki_Name = (STRPTR) AllocPMem ( ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0] ) + 1) ))
          goto GetTCTError;

        if ( (CRead ( CFHandle, TrackInfo->trki_Name, IFFLength ) != IFFLength) )
          goto GetTCTError;

        IFFLength = 0L;
        ScanFlags |= 0x02;

        break;
      }
      case 'THDR' :
      {
        if ( ScanFlags & 0x04 )
          break;

        if ( (CRead ( CFHandle, &(THDR.thdr_Version), (sizeof (struct IFF_THDR) - sizeof (struct IFFChunk))) != (sizeof (struct IFF_THDR) - sizeof (struct IFFChunk))) )
          goto GetTCTError;

        TrackInfo->trki_Version = THDR.thdr_Version;
        TrackInfo->trki_Revision = THDR.thdr_Revision;

        memcpy ( &(TrackInfo->trki_Version), &(THDR.thdr_Version), (sizeof (struct IFF_THDR) - sizeof (struct IFFChunk)));

#ifndef MOTOROLA
        TrackInfo->trki_Length = ReadMWord ( (UBYTE *) &(TrackInfo->trki_Length) );
        TrackInfo->trki_Flags = ReadMWord ( (UBYTE *) &(TrackInfo->trki_Flags) );
        TrackInfo->trki_Frames = ReadMWord ( (UBYTE *) &(TrackInfo->trki_Frames) );
        TrackInfo->trki_SPDSpeed = ReadMWord ( (UBYTE *) &(TrackInfo->trki_SPDSpeed) );
        TrackInfo->trki_BPMTempo = ReadMWord ( (UBYTE *) &(TrackInfo->trki_BPMTempo) );
        TrackInfo->trki_BPMSpeed = ReadMWord ( (UBYTE *) &(TrackInfo->trki_BPMSpeed) );
#endif

        IFFLength -= (sizeof (struct IFF_THDR) - sizeof (struct IFFChunk));
        ScanFlags |= 0x04;

        break;
      }
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
      {
GetTCTError:
        FreeNameStr ( TrackInfo->trki_FileName );
        FreeNameStr ( TrackInfo->trki_Name );

        return ( 0L );
      }
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      goto GetTCTError;

  } while ( (Length -= IFFSize) );

  if (!( ScanFlags & 0x04 ))
    goto GetTCTError;

  return ( -1L );
}

static ULONG GetNumTCTs ( struct CFHandle *CFHandle )
{
  struct IFFHeader IFF_Header;
  ULONG NumTCTs = 0L;

  for (;;)
  {
    ULONG Length;

    if ( (CRead ( CFHandle, &IFF_Header, sizeof (struct IFFHeader)) != sizeof (struct IFFHeader)) )
      break;

    if ( (ReadMLong ( (UBYTE *) &IFF_Header.ifh_FORMId[0] ) != 'FORM') || ( (Length = ReadMLong ( (UBYTE *) &IFF_Header.ifh_Length[0]) ) < 4) )
      break;

    if ( CSeek ( CFHandle, Length - 4, OFFSET_CURRENT ) == 0xFFFFFFFF )
      break;

    if ( ((ReadMLong ( (UBYTE *) &IFF_Header.ifh_TypeId[0] ) >> 8) != 'TCT') )
      continue;

    NumTCTs++;
  }

  return ( NumTCTs );
}

#ifndef MOTOROLA

static UWORD ReadMWord ( UBYTE *Buffer )
{
  UWORD MotorolaWord;

  MotorolaWord = *Buffer++ << 8U;
  MotorolaWord += *Buffer;

  return ( MotorolaWord );
}

static void WriteMWord ( UBYTE *Buffer, UWORD Value )
{
  *Buffer++ = Value >> 8U;
  *Buffer = Value;
}

static ULONG ReadMLong ( UBYTE *Buffer )
{
  ULONG MotorolaLong;

  MotorolaLong = *Buffer++;
  MotorolaLong <<= 8UL;
  MotorolaLong += *Buffer++;
  MotorolaLong <<= 8UL;
  MotorolaLong += *Buffer++;
  MotorolaLong <<= 8UL;
  MotorolaLong += *Buffer;

  return ( MotorolaLong );
}

static void WriteMLong ( UBYTE *Buffer, ULONG Value )
{
  *Buffer++ = Value >> 24UL;
  *Buffer++ = Value >> 16UL;
  *Buffer++ = Value >> 8UL;
  *Buffer = Value;
}

#endif
