/* TuComposer AmigaOS dependent code
**
** $VER: InternalTCY.c 31.00 (8.2.00)
**
** (C) 2000 by Sebastian Vater (known as Basty/Seasons).
**
*/

#include <stddef.h>
#include <string.h>
#include <exec/types.h>
#include <exec/libraries.h>
#include <exec/lists.h>
#include <exec/nodes.h>
#include <tucomposer/tucomposer.h>
#include <tucomposer/synth.h>
#include <tucomposer/external/external.h>
#include <tucomposer/external/synth.h>
#include <proto/dos.h>
#include <proto/tucomposer.h>

#include "compiler.h"

#define SAVESAMPLE_BUFFER 4096

#ifdef __cplusplus
extern "C" {
#endif

#ifndef MOTOROLA
  static UWORD ReadMWord ( UBYTE *Buffer );
  static void WriteMWord ( UBYTE *Buffer, UWORD Value );

  static ULONG ReadMLong ( UBYTE *Buffer );
  static void WriteMLong ( UBYTE *Buffer, ULONG Value );
#else
  #define ReadMWord( Buffer ) (*(UWORD *) (Buffer))
  #define WriteMWord( Buffer, Value ) (*(UWORD *) (Buffer)) = Value

  #define ReadMLong( Buffer ) (*(ULONG *) (Buffer))
  #define WriteMLong( Buffer, Value ) (*(ULONG *) (Buffer)) = Value
#endif

static struct TuComposerExtLibBase *InternalTCYOpenExtLib ( void );
static BPTR InternalTCYCloseExtLib ( void );
static BPTR ExtLib ( void );
static ULONG IdentifyTCY ( struct CFHandle *CFHandle );
static struct TuComposerSynth *LoadTCY ( struct CFHandle *CFHandle,
    struct TuComposerSynth *SynthStr, union FlagPass *FlagPassStr,
    unsigned long NumSynth );
static ULONG SaveTCY ( struct CFHandle *CFHandle,
    struct TuComposerSynth *SynthStr, union FlagPass *FlagPassStr );
static ULONG GetTCYInfo ( struct CFHandle *CFHandle,
    struct SynthInfo *SynthInfo, unsigned long NumSynth );
static ULONG GetNumTCYs ( struct CFHandle *CFHandle );

static ULONG LoadTCYWfrm ( struct CFHandle *CFHandle, struct TuComposerSynth *SynthStr, unsigned long Length );
static ULONG LoadWfrmWave ( struct CFHandle *CFHandle, struct TuComposerSynth *SynthStr, unsigned long Length );
static ULONG LoadTCYStab ( struct CFHandle *CFHandle, struct TuComposerSynth *SynthStr, unsigned long Length );
static ULONG LoadStabSmbl ( struct CFHandle *CFHandle, struct TuComposerSynth *SynthStr, unsigned long Length );

static ULONG SaveTCYChunk ( struct CFHandle *CFHandle, struct IFFChunk *IFF_Chunk, unsigned long ChunkName, unsigned long ChunkLen, ULONG *ChunkLenBuf, unsigned long ErrorCode );
static ULONG SaveTCYName ( struct CFHandle *CFHandle, STRPTR Name, unsigned long ChunkName, ULONG *ChunkLenBuf, unsigned long ErrorCode );

static ULONG SaveTCYWave ( struct CFHandle *CFHandle, struct TuComposerSynthWave *WaveformStr, ULONG *ChunkLenBuf );
static ULONG SaveTCYSmbl ( struct CFHandle *CFHandle, struct TuComposerSynthSymbolTable *SymbolStr, ULONG *ChunkLenBuf );

#ifdef __cplusplus
}
#endif

extern struct SynthLibrary InternalTCYBase;
extern struct InternalTCYLoadFlagStr InternalTCYLoadFlags;
extern struct InternalTCYSaveFlagStr InternalTCYSaveFlags;

static UBYTE InternalTCYName[] = "internal_tcy.library\0";
static UBYTE InternalTCYLoadID[] = "TuComposer Synth (TCY)\0";
static UBYTE InternalTCYIDString[] = "Internal TCY Lib Version 1.00 (v31) (08 Feb 00) by Basty/Seasons.\0";

struct TuComposerExtList InternalTCY =
{
  {NULL, NULL},
  EXTLF_Intern|EXTLF_Loader|EXTLF_Saver, 0,
  (struct TuComposerExtLibBase *) &InternalTCYBase,
  NULL,
  (STRPTR) &InternalTCYName,
  (STRPTR) &InternalTCYLoadID,
  (struct TuComposerExtFlagStr *) &InternalTCYLoadFlags,
  (struct TuComposerExtFlagStr *) &InternalTCYSaveFlags,
  0
};

#include "InternalTCY_library.h"

static UBYTE LoadText0[] = "Load code lines\0";
static UBYTE LoadText1[] = "Load waveforms\0";
static UBYTE LoadText2[] = "Load symbol table\0";

static UBYTE SaveText0[] = "Save code lines\0";
static UBYTE SaveText1[] = "Save waveforms\0";
static UBYTE SaveText2[] = "Save symbol table\0";

static struct InternalTCYLoadFlagStr {
  UWORD  tefs_NumEntries;
  UWORD  tefs_StructSize;
  union FlagEntryBooleanStruct tefs_Flag0;
  union FlagEntryBooleanStruct tefs_Flag1;
  union FlagEntryBooleanStruct tefs_Flag2;
} InternalTCYLoadFlags =
{
  3, sizeof (InternalTCYLoadFlags),
  {(STRPTR) &LoadText0, EXTFLAG_BOOLEAN, 0, 0, 0L, 0L, 0xFFFFFFFFL},
  {(STRPTR) &LoadText1, EXTFLAG_BOOLEAN, 0, 0, 0L, 0L, 0xFFFFFFFFL},
  {(STRPTR) &LoadText2, EXTFLAG_BOOLEAN, 0, 0, 0L, 0L, 0xFFFFFFFFL}
};

static struct InternalTCYSaveFlagStr {
  UWORD  tefs_NumEntries;
  UWORD  tefs_StructSize;
  union FlagEntryBooleanStruct tefs_Flag0;
  union FlagEntryBooleanStruct tefs_Flag1;
  union FlagEntryBooleanStruct tefs_Flag2;
} InternalTCYSaveFlags =
{
  3, sizeof (InternalTCYSaveFlags),
  {(STRPTR) &SaveText0, EXTFLAG_BOOLEAN, 0, 0, 0L, 0L, 0xFFFFFFFFL},
  {(STRPTR) &SaveText1, EXTFLAG_BOOLEAN, 0, 0, 0L, 0L, 0xFFFFFFFFL},
  {(STRPTR) &SaveText2, EXTFLAG_BOOLEAN, 0, 0, 0L, 0L, 0xFFFFFFFFL}
};

static struct TuComposerExtLibBase *InternalTCYOpenExtLib ( void )
{
  InternalTCYBase.synl_Library.extb_LibNode.lib_OpenCnt++;

  return ( (struct TuComposerExtLibBase *) &InternalTCYBase );
}

static BPTR InternalTCYCloseExtLib ( void )
{
  InternalTCYBase.synl_Library.extb_LibNode.lib_OpenCnt--;

  return ( NULL );
}

static BPTR ExtLib ( void )
{
  return ( NULL );
}

static ULONG IdentifyTCY ( struct CFHandle *CFHandle )
{
  struct IFFHeader ID_IFF;

  if ( (CRead ( CFHandle, &ID_IFF, sizeof (struct IFFHeader)) != sizeof (struct IFFHeader)) )
    return ( 0L );

  if ( (ReadMLong ( (UBYTE *) &ID_IFF.ifh_FORMId[0] ) != 'FORM') || ( ReadMLong ( (UBYTE *) &ID_IFF.ifh_Length[0] ) < 4) || ( (ReadMLong ( (UBYTE *) &ID_IFF.ifh_TypeId[0]) >> 8) != 'TCY' ) )
    return ( 0L );

  return ( -1L );
}

static struct TuComposerSynth *LoadTCY ( struct CFHandle *CFHandle,
    struct TuComposerSynth *SynthStr, union FlagPass *FlagPassStr,
    ULONG NumSynth )
{
  struct IFFHeader IFF_Header;
  struct IFF_YHDR YHDR;
  struct TuComposerSynth *TmpSynth;
  ULONG Length, IFFSize;
  ULONG Warning, Invalid = 0L;

  if (!( TmpSynth = AllocSynth ( 1L, 0L, 0L )))
    return ( NULL );

  for (;;)
  {
    if ( (CRead ( CFHandle, &IFF_Header, sizeof (struct IFFHeader)) != sizeof (struct IFFHeader)) )
      return ( NULL );

    if ( (ReadMLong ( (UBYTE *) &IFF_Header.ifh_FORMId[0] ) != 'FORM') || ( (Length = ReadMLong ( (UBYTE *) &IFF_Header.ifh_Length[0]) ) < 4) || ((ReadMLong ( (UBYTE *) &IFF_Header.ifh_TypeId[0] ) >> 8) != 'TCY') )
      return ( NULL );

    Length -= 4;

    if (!( NumSynth-- ))
      break;

    if ( CSeek ( CFHandle, Length, OFFSET_CURRENT ) == 0xFFFFFFFF )
      return ( NULL );
  }

  if ( (ReadMLong ( (UBYTE *) &IFF_Header.ifh_TypeId[0] ) != 'TCY1' ) )
  {
    SetIoErr ( ERROR_ACTION_NOT_KNOWN );

    return ( NULL );
  }

  do
  {
    struct IFFChunk IFF_Chunk;
    struct UnknownChunk *UnkChunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      goto LoadTCYHeaderErr;

    IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0]);
    IFFLength = IFFSize;

    IFFSize += (IFFSize & 1);

    switch ( ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] ) )
    {
      case 'FILE' :
      {
        if ( TmpSynth->tcy_FileName )
          break;

        if (!( TmpSynth->tcy_FileName = (STRPTR) AllocPMem ( IFFLength + 1 )))
          goto LoadTCYHeaderErr;

        if ( (CRead ( CFHandle, TmpSynth->tcy_FileName, IFFLength ) != IFFLength) )
          goto LoadTCYHeaderErr;

        IFFLength &= 1;

        break;
      }
      case 'NAME' :
      {
        if ( TmpSynth->tcy_Name )
          break;

        if (!( TmpSynth->tcy_Name = (STRPTR) AllocPMem ( IFFLength + 1 )))
          goto LoadTCYHeaderErr;

        if ( (CRead ( CFHandle, TmpSynth->tcy_Name, IFFLength ) != IFFLength) )
          goto LoadTCYHeaderErr;

        IFFLength &= 1;

        break;
      }
      case 'YHDR' :
      {
        if ( Invalid & 1 )
          goto TCYIFFDamaged;

        if ( (CRead ( CFHandle, &(YHDR.yhdr_Version), (sizeof (struct IFF_YHDR) - sizeof (struct IFFChunk))) != (sizeof (struct IFF_YHDR) - sizeof (struct IFFChunk))))
          goto LoadTCYHeaderErr;

        Warning = 0L;

        if ( (YHDR.yhdr_Version > YHDR_VERSION) )
          Warning = ERROR_ACTION_NOT_KNOWN;

        memcpy ( &(TmpSynth->tcy_VolumePos), &(YHDR.yhdr_VolumePos), (sizeof (struct IFF_YHDR) - offsetof (struct IFF_YHDR, yhdr_VolumePos)));

#ifndef MOTOROLA
        TmpSynth->tcy_VolumePos = ReadMWord ( (UBYTE *) &(TmpSynth->tcy_VolumePos) );
        TmpSynth->tcy_PanningPos = ReadMWord ( (UBYTE *) &(TmpSynth->tcy_PanningPos) );
        TmpSynth->tcy_SlidePos = ReadMWord ( (UBYTE *) &(TmpSynth->tcy_SlidePos) );
        TmpSynth->tcy_SpecialPos = ReadMWord ( (UBYTE *) &(TmpSynth->tcy_SpecialPos) );
        TmpSynth->tcy_VolumeSusPos = ReadMWord ( (UBYTE *) &(TmpSynth->tcy_VolumeSusPos) );
        TmpSynth->tcy_PanningSusPos = ReadMWord ( (UBYTE *) &(TmpSynth->tcy_PanningSusPos) );
        TmpSynth->tcy_SlideSusPos = ReadMWord ( (UBYTE *) &(TmpSynth->tcy_SlideSusPos) );
        TmpSynth->tcy_SpecialSusPos = ReadMWord ( (UBYTE *) &(TmpSynth->tcy_SpecialSusPos) );
        TmpSynth->tcy_VolNNAPos = ReadMWord ( (UBYTE *) &(TmpSynth->tcy_VolNNAPos) );
        TmpSynth->tcy_PanNNAPos = ReadMWord ( (UBYTE *) &(TmpSynth->tcy_PanNNAPos) );
        TmpSynth->tcy_SlideNNAPos = ReadMWord ( (UBYTE *) &(TmpSynth->tcy_SlideNNAPos) );
        TmpSynth->tcy_SpecialNNAPos = ReadMWord ( (UBYTE *) &(TmpSynth->tcy_SpecialNNAPos) );
        TmpSynth->tcy_VolDNAPos = ReadMWord ( (UBYTE *) &(TmpSynth->tcy_VolDNAPos) );
        TmpSynth->tcy_PanDNAPos = ReadMWord ( (UBYTE *) &(TmpSynth->tcy_PanDNAPos) );
        TmpSynth->tcy_SlideDNAPos = ReadMWord ( (UBYTE *) &(TmpSynth->tcy_SlideDNAPos) );
        TmpSynth->tcy_SpecialDNAPos = ReadMWord ( (UBYTE *) &(TmpSynth->tcy_SpecialDNAPos) );
        TmpSynth->tcy_Variable0 = ReadMWord ( (UBYTE *) &(TmpSynth->tcy_Variable0) );
        TmpSynth->tcy_Variable1 = ReadMWord ( (UBYTE *) &(TmpSynth->tcy_Variable1) );
        TmpSynth->tcy_Variable2 = ReadMWord ( (UBYTE *) &(TmpSynth->tcy_Variable2) );
        TmpSynth->tcy_Variable3 = ReadMWord ( (UBYTE *) &(TmpSynth->tcy_Variable3) );
        TmpSynth->tcy_Variable4 = ReadMWord ( (UBYTE *) &(TmpSynth->tcy_Variable4) );
        TmpSynth->tcy_Variable5 = ReadMWord ( (UBYTE *) &(TmpSynth->tcy_Variable5) );
        TmpSynth->tcy_Variable6 = ReadMWord ( (UBYTE *) &(TmpSynth->tcy_Variable6) );
        TmpSynth->tcy_Variable7 = ReadMWord ( (UBYTE *) &(TmpSynth->tcy_Variable7) );
        TmpSynth->tcy_Variable8 = ReadMWord ( (UBYTE *) &(TmpSynth->tcy_Variable8) );
        TmpSynth->tcy_Variable9 = ReadMWord ( (UBYTE *) &(TmpSynth->tcy_Variable9) );
        TmpSynth->tcy_VariableA = ReadMWord ( (UBYTE *) &(TmpSynth->tcy_VariableA) );
        TmpSynth->tcy_VariableB = ReadMWord ( (UBYTE *) &(TmpSynth->tcy_VariableB) );
        TmpSynth->tcy_VariableC = ReadMWord ( (UBYTE *) &(TmpSynth->tcy_VariableC) );
        TmpSynth->tcy_VariableD = ReadMWord ( (UBYTE *) &(TmpSynth->tcy_VariableD) );
        TmpSynth->tcy_VariableE = ReadMWord ( (UBYTE *) &(TmpSynth->tcy_VariableE) );
        TmpSynth->tcy_VariableF = ReadMWord ( (UBYTE *) &(TmpSynth->tcy_VariableF) );
        TmpSynth->tcy_VolConVar = ReadMWord ( (UBYTE *) &(TmpSynth->tcy_VolConVar) );
        TmpSynth->tcy_PanConVar = ReadMWord ( (UBYTE *) &(TmpSynth->tcy_PanConVar) );
        TmpSynth->tcy_SlideConVar = ReadMWord ( (UBYTE *) &(TmpSynth->tcy_SlideConVar) );
        TmpSynth->tcy_SpecialConVar = ReadMWord ( (UBYTE *) &(TmpSynth->tcy_SpecialConVar) );
        TmpSynth->tcy_VarKeepMask = ReadMWord ( (UBYTE *) &(TmpSynth->tcy_VarKeepMask) );
#endif

        Invalid |= 1;

        IFFLength = IFFSize - (sizeof (struct IFF_YHDR) - sizeof (struct IFFChunk));

        break;
      }
      case 'CODE' :
      {
        struct TuComposerSynthCode *SynthCodeStr;

        if ( Invalid & 2 )
          goto TCYIFFDamaged;

        Invalid |= 2;

        if (!( FlagPassStr[0].flg_Value ))
          break;

        if (!( SynthCodeStr = AllocSynthCode ( IFFSize / sizeof (struct TuComposerSynthCode) )))
          goto LoadTCYHeaderErr;

        AssignSynthCode ( TmpSynth, SynthCodeStr, IFFSize / sizeof (struct TuComposerSynthCode));

        if ( CRead ( CFHandle, SynthCodeStr, IFFSize ) != IFFSize )
          goto LoadTCYHeaderErr;

#ifndef MOTOROLA
        for (IFFLength = TmpSynth->tcy_CodeEntries; IFFLength > 0; IFFLength--)
        {
          SynthCodeStr->tcyc_InstData = ReadMWord ( (UBYTE *) &(SynthCodeStr->tcyc_InstData) );

          SynthCodeStr++;
        }
#endif
        IFFLength &= 1;

        break;
      }
      case 'FORM' :
      {
        ULONG FaultCode;

        if ( (CRead ( CFHandle, &IFF_Header, 4 ) != 4 ) )
          goto LoadTCYHeaderErr;

        switch ( ReadMLong ( (UBYTE *) &IFF_Header.ifh_FORMId[0] ) )
        {
          case 'WFRM' :
          {
            if (!( FlagPassStr[1].flg_Value ))
              break;

            if ( (FaultCode = LoadTCYWfrm ( CFHandle, TmpSynth, IFFSize )))
            {
SetFaultCode:

              SetIoErr ( FaultCode );

              FreeSynth ( TmpSynth );

              return ( NULL );
            }

            IFFLength = 0L;

            break;
          }
          case 'STAB' :
          {
            if (!( FlagPassStr[2].flg_Value ))
              break;

            if ( (FaultCode = LoadTCYStab ( CFHandle, TmpSynth, IFFSize )))
              goto SetFaultCode;

            IFFLength = 0L;

            break;
          }
          default :
          {
            if ( CSeek ( CFHandle, -4, OFFSET_CURRENT ) == 0xFFFFFFFF )
              goto LoadTCYHeaderErr;

            goto AllocUnkChunk;
          }
        }

        break;
      }
      default :
      {
AllocUnkChunk:

        if (!( UnkChunk = AllocUnknownChunk ( NULL, ReadMLong ( (UBYTE *) &(IFF_Chunk.iff_Identifier[0])), IFFLength )))
          goto LoadTCYHeaderErr;

        AddSynthUnknownChunk ( TmpSynth, UnkChunk );

        if ( (CRead ( CFHandle, &(UnkChunk->uc_Data), IFFLength ) != IFFLength ) )
          goto LoadTCYHeaderErr;

        IFFLength &= 1;

        break;
      }
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
      {
LoadTCYHeaderErr:

        FreeSynth ( TmpSynth );

        return ( NULL );
      }
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      goto TCYIFFDamaged;

  } while ( (Length -= IFFSize) );

  if (!( Invalid ))
  {
TCYIFFDamaged:

    SetIoErr ( ERROR_INVALID_RESIDENT_LIBRARY );

    FreeSynth ( TmpSynth );

    return ( NULL );
  }

  if (!( SynthStr ))
    SynthStr = TmpSynth;

  ReMapSynth ( TmpSynth, SynthStr );

  SetIoErr ( Warning );

  return ( SynthStr );
}

static ULONG LoadTCYWfrm ( struct CFHandle *CFHandle, struct TuComposerSynth *SynthStr, unsigned long Length )
{
  struct IFFHeader IFF_Header;
  ULONG IFFSize;

  if ( Length < 4 )
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  Length -= 4;

  do
  {
    struct IFFChunk IFF_Chunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      return ( IoErr ());

    if ( (IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0] )) & 1 )
      IFFSize++;

    IFFLength = IFFSize;

    if ( (ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] ) ) == 'FORM' )
    {
      ULONG FaultCode;

      if ( (CRead ( CFHandle, &IFF_Header, 4 ) != 4 ) )
        return ( IoErr ());

      if ( (ReadMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0])) ) == 'WAVE' )
      {
        if ( (FaultCode = LoadWfrmWave ( CFHandle, SynthStr, IFFSize )))
          return ( FaultCode );

        IFFLength = 0L;
      }
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
        return ( IoErr ());
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      return ( ERROR_INVALID_RESIDENT_LIBRARY );

  } while ( (Length -= IFFSize) );

  return ( 0L );
}

static ULONG LoadWfrmWave ( struct CFHandle *CFHandle, struct TuComposerSynth *SynthStr, unsigned long Length )
{
  struct IFF_WHDR WHDR;
  struct TuComposerSynthWave *WaveformStr;
  ULONG IFFSize;
  UBYTE Invalid = 0;

  if ( Length < 4 )
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  Length -= 4;

  if (!( WaveformStr = AllocWaveform ( 1L )))
    return ( IoErr ());

  if ( AddWaveform ( SynthStr, WaveformStr ))
    return ( ERROR_TOO_MANY_LEVELS );

  do
  {
    struct IFFChunk IFF_Chunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      return ( IoErr ());

    IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0]);
    IFFLength = IFFSize;

    IFFSize += (IFFSize & 1);

    switch ( ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] ) )
    {
      case 'FILE' :
      {
        if ( WaveformStr->tcyw_FileName )
          break;

        if (!( WaveformStr->tcyw_FileName = (STRPTR) AllocPMem ( IFFLength + 1 )))
          return ( IoErr ());

        if ( (CRead ( CFHandle, WaveformStr->tcyw_FileName, IFFLength ) != IFFLength) )
          return ( IoErr ());

        IFFLength &= 1;

        break;
      }
      case 'NAME' :
      {
        if ( WaveformStr->tcyw_Name )
          break;

        if (!( WaveformStr->tcyw_Name = (STRPTR) AllocPMem ( IFFLength + 1 )))
          return ( IoErr ());

        if ( (CRead ( CFHandle, WaveformStr->tcyw_Name, IFFLength ) != IFFLength) )
          return ( IoErr ());

        IFFLength &= 1;

        break;
      }
      case 'WHDR' :
      {
        if ( Invalid & 1 )
          return ( ERROR_INVALID_RESIDENT_LIBRARY );

        if ( (CRead ( CFHandle, &(WHDR.whdr_Repeat), (sizeof (struct IFF_WHDR) - sizeof (struct IFFChunk))) != (sizeof (struct IFF_WHDR) - sizeof (struct IFFChunk))))
          return ( IoErr ());

        WaveformStr->tcyw_Repeat = ReadMLong ( (UBYTE *) &(WHDR.whdr_Repeat) );
        WaveformStr->tcyw_RepLen = ReadMLong ( (UBYTE *) &(WHDR.whdr_RepLen) );
        WaveformStr->tcyw_Flags = ReadMWord ( (UBYTE *) &(WHDR.whdr_Flags) );

        Invalid |= 1;

        IFFLength = IFFSize - (sizeof (struct IFF_WHDR) - sizeof (struct IFFChunk));

        break;
      }
      case 'BODY' :
      {
        WORD *WaveData;

        if ( Invalid & 2 )
          return ( ERROR_INVALID_RESIDENT_LIBRARY );

        if (!( WaveData = (WORD *) AllocPMem ( IFFLength )))
          return ( IoErr ());

        FreePMem ( WaveformStr->tcyw_WaveData, 2L );

        WaveformStr->tcyw_WaveData = WaveData;
        WaveformStr->tcyw_Length = IFFLength;

        if ( CRead ( CFHandle, WaveData, IFFLength ) != IFFLength )
          return ( IoErr ());

        Invalid |= 2;

        IFFLength &= 1;

        break;
      }
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
        return ( IoErr ());
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      return ( ERROR_INVALID_RESIDENT_LIBRARY );

  } while ( (Length -= IFFSize) );

  if ( Invalid < 3 )
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  Length = WaveformStr->tcyw_Length;

  if (!( WaveformStr->tcyw_Flags & TCYWF_8Bit ))
    Length >>= 1UL;

  WaveformStr->tcyw_Samples = Length;

#ifndef MOTOROLA
  if (!( WaveformStr->tcyw_Flags & TCYWF_8Bit ))
  {
    WORD *WaveData = WaveformStr->tcyw_WaveData;
    ULONG i;

    for (i = ((WaveformStr->tcyw_Samples + 3UL) >> 2UL); i > 0; i--)
    {
      *WaveData++ = ReadMWord ( (UBYTE *) WaveData);
      *WaveData++ = ReadMWord ( (UBYTE *) WaveData);
      *WaveData++ = ReadMWord ( (UBYTE *) WaveData);
      *WaveData++ = ReadMWord ( (UBYTE *) WaveData);
    }
  }
#endif

  return ( 0L );
}

static ULONG LoadTCYStab ( struct CFHandle *CFHandle, struct TuComposerSynth *SynthStr, unsigned long Length )
{
  struct IFFHeader IFF_Header;
  ULONG IFFSize;

  if ( Length < 4 )
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  Length -= 4;

  do
  {
    struct IFFChunk IFF_Chunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      return ( IoErr ());

    if ( (IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0] )) & 1 )
      IFFSize++;

    IFFLength = IFFSize;

    if ( (ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] ) ) == 'FORM' )
    {
      ULONG FaultCode;

      if ( (CRead ( CFHandle, &IFF_Header, 4 ) != 4 ) )
        return ( IoErr ());

      if ( (ReadMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0])) ) == 'SMBL' )
      {
        if ( (FaultCode = LoadStabSmbl ( CFHandle, SynthStr, IFFSize )))
          return ( FaultCode );

        IFFLength = 0L;
      }
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
        return ( IoErr ());
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      return ( ERROR_INVALID_RESIDENT_LIBRARY );

  } while ( (Length -= IFFSize) );

  return ( 0L );
}

static ULONG LoadStabSmbl ( struct CFHandle *CFHandle, struct TuComposerSynth *SynthStr, unsigned long Length )
{
  struct IFF_SREF SREF;
  struct TuComposerSynthSymbolTable *SymbolStr;
  ULONG Invalid = 0L, IFFSize;

  if ( Length < 4 )
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  Length -= 4;

  if (!( SymbolStr = AllocSymbol ( "UNNAMED" )))
    return ( IoErr ());

  AddSymbol ( SynthStr, SymbolStr );

  do
  {
    struct IFFChunk IFF_Chunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      return ( IoErr ());

    IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0]);
    IFFLength = IFFSize;

    IFFSize += (IFFSize & 1);

    switch ( ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] ) )
    {
      case 'NAME' :
      {
        STRPTR SymbolName;

        if (!( SymbolName = (STRPTR) AllocPMem ( IFFLength + 1 )))
          return ( IoErr ());

        if ( (CRead ( CFHandle, SymbolName, IFFLength ) != IFFLength) )
          return ( IoErr ());

        if ( AssignSymbol ( SymbolStr, SymbolName ))
          return ( IoErr ());

        IFFLength &= 1;

        break;
      }
      case 'SREF' :
      {
        if ( Invalid )
          return ( ERROR_INVALID_RESIDENT_LIBRARY );

        if ( (CRead ( CFHandle, &(SREF.sref_SymbolValue), (sizeof (struct IFF_SREF) - sizeof (struct IFFChunk))) != (sizeof (struct IFF_SREF) - sizeof (struct IFFChunk))))
          return ( IoErr ());

        SymbolStr->tsst_SymbolValue = ReadMWord ( (UBYTE *) &(SREF.sref_SymbolValue) );
        SymbolStr->tsst_FirstLine = ReadMWord ( (UBYTE *) &(SREF.sref_FirstLine) );
        SymbolStr->tsst_LastLine = ReadMWord ( (UBYTE *) &(SREF.sref_LastLine) );
        SymbolStr->tsst_Type = SREF.sref_Type;
        SymbolStr->tsst_Flags = SREF.sref_Flags;

        Invalid = ~Invalid;

        IFFLength = IFFSize - (sizeof (struct IFF_SREF) - sizeof (struct IFFChunk));

        break;
      }
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
        return ( IoErr ());
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      return ( ERROR_INVALID_RESIDENT_LIBRARY );

  } while ( (Length -= IFFSize) );

  if ( Invalid )
    return ( 0L );

  return ( ERROR_INVALID_RESIDENT_LIBRARY );
}

static ULONG SaveTCY ( struct CFHandle *CFHandle,
    struct TuComposerSynth *SynthStr, union FlagPass *FlagPassStr )
{
  struct IFFHeader IFF_Header;
  struct IFFChunk IFF_Chunk;
  struct IFF_YHDR YHDR;
  struct TuComposerSynthWave *WaveformStr;
  struct TuComposerSynthSymbolTable *SymbolStr;
  struct UnknownChunk *UnkChunk;
  ULONG TCYSize = sizeof (ULONG), FORMSize;
  ULONG TCYLenPos, FORMLenPos;
  ULONG FaultCode;

  if ( (TCYLenPos = CSeek ( CFHandle, 0L, OFFSET_CURRENT )) == 0xFFFFFFFF )
    return ( ERROR_SYNTH_IOERR );

  TCYLenPos += sizeof (ULONG);

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0]), 'FORM' );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), TCYSize );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_TypeId[0]), 'TCY1' );

  if ( (CWrite ( CFHandle, &IFF_Header, sizeof (struct IFFHeader )) != sizeof (struct IFFHeader )))
    return ( ERROR_SYNTH_IOERR );

  if ( (FaultCode = SaveTCYName ( CFHandle, SynthStr->tcy_FileName, 'FILE', (ULONG *) &TCYSize, ERROR_SYNTH_IOERR )))
    return ( FaultCode );

  if ( (FaultCode = SaveTCYName ( CFHandle, SynthStr->tcy_Name, 'NAME', (ULONG *) &TCYSize, ERROR_SYNTH_IOERR )))
    return ( FaultCode );

  YHDR.yhdr_Version = YHDR_VERSION;
  YHDR.yhdr_Revision = YHDR_REVISION;

  WriteMWord ( (UBYTE *) &(YHDR.yhdr_NumWaves), GetNumWaveforms ( SynthStr ));

  memcpy ( &(YHDR.yhdr_VolumePos), &(SynthStr->tcy_VolumePos), (sizeof (struct IFF_YHDR) - sizeof (struct IFFChunk)));

#ifndef MOTOROLA
  WriteMWord ( (UBYTE *) &(YHDR.yhdr_VolumePos), YHDR.yhdr_VolumePos );
  WriteMWord ( (UBYTE *) &(YHDR.yhdr_PanningPos), YHDR.yhdr_PanningPos );
  WriteMWord ( (UBYTE *) &(YHDR.yhdr_SlidePos), YHDR.yhdr_SlidePos );
  WriteMWord ( (UBYTE *) &(YHDR.yhdr_SpecialPos), YHDR.yhdr_SpecialPos );
  WriteMWord ( (UBYTE *) &(YHDR.yhdr_VolumeSusPos), YHDR.yhdr_VolumeSusPos );
  WriteMWord ( (UBYTE *) &(YHDR.yhdr_PanningSusPos), YHDR.yhdr_PanningSusPos );
  WriteMWord ( (UBYTE *) &(YHDR.yhdr_SlideSusPos), YHDR.yhdr_SlideSusPos );
  WriteMWord ( (UBYTE *) &(YHDR.yhdr_SpecialSusPos), YHDR.yhdr_SpecialSusPos );
  WriteMWord ( (UBYTE *) &(YHDR.yhdr_VolNNAPos), YHDR.yhdr_VolNNAPos );
  WriteMWord ( (UBYTE *) &(YHDR.yhdr_PanNNAPos), YHDR.yhdr_PanNNAPos );
  WriteMWord ( (UBYTE *) &(YHDR.yhdr_SlideNNAPos), YHDR.yhdr_SlideNNAPos );
  WriteMWord ( (UBYTE *) &(YHDR.yhdr_SpecialNNAPos), YHDR.yhdr_SpecialNNAPos );
  WriteMWord ( (UBYTE *) &(YHDR.yhdr_VolDNAPos), YHDR.yhdr_VolDNAPos );
  WriteMWord ( (UBYTE *) &(YHDR.yhdr_PanDNAPos), YHDR.yhdr_PanDNAPos );
  WriteMWord ( (UBYTE *) &(YHDR.yhdr_SlideDNAPos), YHDR.yhdr_SlideDNAPos );
  WriteMWord ( (UBYTE *) &(YHDR.yhdr_SpecialDNAPos), YHDR.yhdr_SpecialDNAPos );
  WriteMWord ( (UBYTE *) &(YHDR.yhdr_Variable0), YHDR.yhdr_Variable0 );
  WriteMWord ( (UBYTE *) &(YHDR.yhdr_Variable1), YHDR.yhdr_Variable1 );
  WriteMWord ( (UBYTE *) &(YHDR.yhdr_Variable2), YHDR.yhdr_Variable2 );
  WriteMWord ( (UBYTE *) &(YHDR.yhdr_Variable3), YHDR.yhdr_Variable3 );
  WriteMWord ( (UBYTE *) &(YHDR.yhdr_Variable4), YHDR.yhdr_Variable4 );
  WriteMWord ( (UBYTE *) &(YHDR.yhdr_Variable5), YHDR.yhdr_Variable5 );
  WriteMWord ( (UBYTE *) &(YHDR.yhdr_Variable6), YHDR.yhdr_Variable6 );
  WriteMWord ( (UBYTE *) &(YHDR.yhdr_Variable7), YHDR.yhdr_Variable7 );
  WriteMWord ( (UBYTE *) &(YHDR.yhdr_Variable8), YHDR.yhdr_Variable8 );
  WriteMWord ( (UBYTE *) &(YHDR.yhdr_Variable9), YHDR.yhdr_Variable9 );
  WriteMWord ( (UBYTE *) &(YHDR.yhdr_VariableA), YHDR.yhdr_VariableA );
  WriteMWord ( (UBYTE *) &(YHDR.yhdr_VariableB), YHDR.yhdr_VariableB );
  WriteMWord ( (UBYTE *) &(YHDR.yhdr_VariableC), YHDR.yhdr_VariableC );
  WriteMWord ( (UBYTE *) &(YHDR.yhdr_VariableD), YHDR.yhdr_VariableD );
  WriteMWord ( (UBYTE *) &(YHDR.yhdr_VariableE), YHDR.yhdr_VariableE );
  WriteMWord ( (UBYTE *) &(YHDR.yhdr_VariableF), YHDR.yhdr_VariableF );
  WriteMWord ( (UBYTE *) &(YHDR.yhdr_VolConVar), YHDR.yhdr_VolConVar );
  WriteMWord ( (UBYTE *) &(YHDR.yhdr_PanConVar), YHDR.yhdr_PanConVar );
  WriteMWord ( (UBYTE *) &(YHDR.yhdr_SlideConVar), YHDR.yhdr_SlideConVar );
  WriteMWord ( (UBYTE *) &(YHDR.yhdr_SpecialConVar), YHDR.yhdr_SpecialConVar );
  WriteMWord ( (UBYTE *) &(YHDR.yhdr_VarKeepMask), YHDR.yhdr_VarKeepMask );
#endif

  if ( (FaultCode = SaveTCYChunk ( CFHandle, (struct IFFChunk *) &YHDR, 'YHDR', sizeof (struct IFF_YHDR), (ULONG *) &TCYSize, ERROR_SYNTH_IOERR )))
    return ( FaultCode );

  if ( FlagPassStr[1].flg_Value )
  {
    WaveformStr = (struct TuComposerSynthWave *) &(SynthStr->tcy_Waves.mlh_Head);

    if ( WaveformStr->tcyw_Node.mln_Succ->mln_Succ )
    {
      TCYSize += (sizeof (struct IFFHeader) - sizeof (ULONG));
      FORMSize = sizeof (ULONG);

      if ( (FORMLenPos = CSeek ( CFHandle, 0L, OFFSET_CURRENT )) == 0xFFFFFFFF )
        return ( ERROR_SYNTH_IOERR );

      FORMLenPos += sizeof (ULONG);

      WriteMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0]), 'FORM' );
      WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), FORMSize );
      WriteMLong ( (UBYTE *) &(IFF_Header.ifh_TypeId[0]), 'WFRM' );

      if ( (CWrite ( CFHandle, &IFF_Header, sizeof (struct IFFHeader )) != sizeof (struct IFFHeader )))
        return ( ERROR_SYNTH_IOERR );

      for (;;)
      {
        WaveformStr = (struct TuComposerSynthWave *) WaveformStr->tcyw_Node.mln_Succ;

        if (!( WaveformStr->tcyw_Node.mln_Succ ))
          break;

        if ( (FaultCode = SaveTCYWave ( CFHandle, WaveformStr, (ULONG *) &FORMSize )))
          return ( FaultCode );
      }

      if ( (FORMLenPos = CSeek ( CFHandle, FORMLenPos, OFFSET_BEGINNING )) == 0xFFFFFFFF )
        return ( ERROR_SYNTH_IOERR );

      WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), FORMSize );

      if ( CWrite ( CFHandle, &(IFF_Header.ifh_Length[0]), sizeof (ULONG)) != sizeof (ULONG))
        return ( ERROR_SYNTH_IOERR );

      if ( CSeek ( CFHandle, FORMLenPos, OFFSET_BEGINNING ) == 0xFFFFFFFF )
        return ( ERROR_SYNTH_IOERR );

      TCYSize += FORMSize;
    }
  }

  if ( FlagPassStr[0].flg_Value )
  {
    FORMSize = SynthStr->tcy_CodeEntries * sizeof (struct TuComposerSynthCode);

    WriteMLong ( (UBYTE *) &(IFF_Chunk.iff_Identifier[0]), 'CODE' );
    WriteMLong ( (UBYTE *) &(IFF_Chunk.iff_Length[0]), FORMSize );

    if ( (CWrite ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk )) != sizeof (struct IFFChunk )))
      return ( ERROR_SYNTH_IOERR );

    FORMSize += (FORMSize & 1);
    TCYSize += sizeof (struct IFFChunk) + FORMSize;

    if ( (CWrite ( CFHandle, SynthStr->tcy_SynthCode, FORMSize )) != FORMSize )
      return ( ERROR_SYNTH_IOERR );
  }

  if ( FlagPassStr[2].flg_Value )
  {
    SymbolStr = (struct TuComposerSynthSymbolTable *) &(SynthStr->tcy_Symbol.mlh_Head);

    if ( SymbolStr->tsst_Node.mln_Succ->mln_Succ )
    {
      TCYSize += (sizeof (struct IFFHeader) - sizeof (ULONG));
      FORMSize = sizeof (ULONG);

      if ( (FORMLenPos = CSeek ( CFHandle, 0L, OFFSET_CURRENT )) == 0xFFFFFFFF )
        return ( ERROR_SYNTH_IOERR );

      FORMLenPos += sizeof (ULONG);

      WriteMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0]), 'FORM' );
      WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), FORMSize );
      WriteMLong ( (UBYTE *) &(IFF_Header.ifh_TypeId[0]), 'STAB' );

      if ( (CWrite ( CFHandle, &IFF_Header, sizeof (struct IFFHeader )) != sizeof (struct IFFHeader )))
        return ( ERROR_SYNTH_IOERR );

      for (;;)
      {
        SymbolStr = (struct TuComposerSynthSymbolTable *) SymbolStr->tsst_Node.mln_Succ;

        if (!( SymbolStr->tsst_Node.mln_Succ ))
          break;

        if ( (FaultCode = SaveTCYSmbl ( CFHandle, SymbolStr, (ULONG *) &FORMSize )))
          return ( FaultCode );
      }

      if ( (FORMLenPos = CSeek ( CFHandle, FORMLenPos, OFFSET_BEGINNING )) == 0xFFFFFFFF )
        return ( ERROR_SYNTH_IOERR );

      WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), FORMSize );

      if ( CWrite ( CFHandle, &(IFF_Header.ifh_Length[0]), sizeof (ULONG)) != sizeof (ULONG))
        return ( ERROR_SYNTH_IOERR );

      if ( CSeek ( CFHandle, FORMLenPos, OFFSET_BEGINNING ) == 0xFFFFFFFF )
        return ( ERROR_SYNTH_IOERR );

      TCYSize += FORMSize;
    }
  }

  UnkChunk = (struct UnknownChunk *) &(SynthStr->tcy_UnknownChunks.mlh_Head);

  for (;;)
  {
    UnkChunk = (struct UnknownChunk *) UnkChunk->uc_Node.mln_Succ;

    if (!( UnkChunk->uc_Node.mln_Succ ))
      break;

    if ( (FaultCode = SaveTCYChunk ( CFHandle, (struct IFFChunk *) &UnkChunk, ReadMLong ( (UBYTE *) &(UnkChunk->uc_Identifier[0])), ReadMLong ( (UBYTE *) &(UnkChunk->uc_Length[0])), &TCYSize, ERROR_SYNTH_IOERR )))
      return ( FaultCode );
  }

  if ( CSeek ( CFHandle, TCYLenPos, OFFSET_BEGINNING ) == 0xFFFFFFFF )
    return ( ERROR_SYNTH_IOERR );

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), TCYSize );

  if ( CWrite ( CFHandle, &(IFF_Header.ifh_Length[0]), sizeof (ULONG)) != sizeof (ULONG))
    return ( ERROR_SYNTH_IOERR );

  return ( 0L );
}

static ULONG SaveTCYChunk ( struct CFHandle *CFHandle, struct IFFChunk *IFF_Chunk, unsigned long ChunkName, unsigned long ChunkLen, ULONG *ChunkLenBuf, unsigned long ErrorCode )
{
  ChunkLen -= sizeof (struct IFFChunk );

  if ( ChunkLen )
  {
    WriteMLong ( (UBYTE *) &(IFF_Chunk->iff_Identifier[0]), ChunkName );
    WriteMLong ( (UBYTE *) &(IFF_Chunk->iff_Length[0]), ChunkLen );

    if ( (CWrite ( CFHandle, IFF_Chunk, (ChunkLen + sizeof (struct IFFChunk)))) != (ChunkLen + sizeof (struct IFFChunk)))
      return ( ErrorCode );

    if ( ChunkLen & 1 )
    {
      *(ULONG *) &(IFF_Chunk->iff_Identifier[0]) = 0L;

      if ( (CWrite ( CFHandle, &IFF_Chunk, 1 )) != 1 )
        return ( ErrorCode );

      ChunkLen++;
    }

    *ChunkLenBuf += (ChunkLen + sizeof (struct IFFChunk));
  }

  return ( 0L );
}

static ULONG SaveTCYName ( struct CFHandle *CFHandle, STRPTR Name, unsigned long ChunkName, ULONG *ChunkLenBuf, unsigned long ErrorCode )
{
  struct IFFChunk IFF_Chunk;

  if ( Name )
  {
    ULONG Length = strlen ( Name );

    WriteMLong ( (UBYTE *) &(IFF_Chunk.iff_Identifier[0]), ChunkName );
    WriteMLong ( (UBYTE *) &(IFF_Chunk.iff_Length[0]), Length );

    Length += (Length & 1);

    if ( Length )
    {
      if ( (CWrite ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk))) != sizeof (struct IFFChunk))
        return ( ErrorCode );

      if ( (CWrite ( CFHandle, Name, Length )) != Length )
        return ( ErrorCode );

      *ChunkLenBuf += (Length + sizeof (struct IFFChunk));
    }
  }

  return ( 0L );
}

static ULONG SaveTCYWave ( struct CFHandle *CFHandle, struct TuComposerSynthWave *WaveformStr, ULONG *ChunkLenBuf )
{
  struct IFFHeader IFF_Header;
  struct IFFChunk IFF_Chunk;
  struct IFF_WHDR WHDR;
  BYTE *WaveData = (BYTE *) WaveformStr->tcyw_WaveData;
  ULONG WaveSize = sizeof (ULONG), FORMSize;
  ULONG WaveLenPos;
  ULONG FaultCode, WaveLen;

  if ( (WaveLenPos = CSeek ( CFHandle, 0L, OFFSET_CURRENT )) == 0xFFFFFFFF )
    return ( ERROR_SYNTH_IOERR );

  WaveLenPos += sizeof (ULONG);

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0]), 'FORM' );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), WaveSize );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_TypeId[0]), 'WAVE' );

  if ( (CWrite ( CFHandle, &IFF_Header, sizeof (struct IFFHeader )) != sizeof (struct IFFHeader )))
    return ( ERROR_SYNTH_IOERR );

  if ( (FaultCode = SaveTCYName ( CFHandle, WaveformStr->tcyw_FileName, 'FILE', (ULONG *) &WaveSize, ERROR_SYNTH_IOERR )))
    return ( FaultCode );

  if ( (FaultCode = SaveTCYName ( CFHandle, WaveformStr->tcyw_Name, 'NAME', (ULONG *) &WaveSize, ERROR_SYNTH_IOERR )))
    return ( FaultCode );

   memcpy ( &(WHDR.whdr_Repeat), &(WaveformStr->tcyw_Repeat), (sizeof (struct IFF_WHDR) - sizeof (struct IFFChunk)));

#ifndef MOTOROLA
   WriteMLong ( (UBYTE *) &(WHDR.whdr_Repeat), WHDR.whdr_Repeat );
   WriteMLong ( (UBYTE *) &(WHDR.whdr_RepLen), WHDR.whdr_RepLen );
   WriteMWord ( (UBYTE *) &(WHDR.whdr_Flags), WHDR.whdr_Flags );
#endif

  if ( (FaultCode = SaveTCYChunk ( CFHandle, (struct IFFChunk *) &WHDR, 'WHDR', sizeof (struct IFF_WHDR), (ULONG *) &WaveSize, ERROR_SYNTH_IOERR )))
    return ( FaultCode );

  FORMSize = WaveformStr->tcyw_Length;
  WaveLen = WaveformStr->tcyw_Samples;

  WriteMLong ( (UBYTE *) &(IFF_Chunk.iff_Identifier[0]), 'BODY' );
  WriteMLong ( (UBYTE *) &(IFF_Chunk.iff_Length[0]), FORMSize );

  if ( (CWrite ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk )) != sizeof (struct IFFChunk )))
    return ( ERROR_SYNTH_IOERR );

  FORMSize += (FORMSize & 1);
  WaveSize += sizeof (struct IFFChunk) + FORMSize;

#ifndef MOTOROLA
  if ( WaveformStr->tcyw_Flags & TCYWF_8Bit )
  {
    WaveLen = WaveformStr->tcyw_Length;

    if ( WaveLen & 1 )
      WaveData[WaveLen] = 0;

    if ( (CWrite ( CFHandle, WaveData, FORMSize )) != FORMSize )
      return ( ERROR_SYNTH_IOERR );
  } else
  {
    WORD *CopyData = AllocSampleData ( SAVESAMPLE_BUFFER * sizeof (WORD));
    ULONG WriteLeft;

    do
    {
      WORD *Buffer = CopyData;
      ULONG i;

      WriteLeft = SAVESAMPLE_BUFFER;

      if ( WaveLen < SAVESAMPLE_BUFFER )
        WriteLeft = WaveLen;

      for (i = WriteLeft; i > 0; i--)
      {
        *Buffer++ = ReadMWord ( (UBYTE *) WaveData++ );
      }

      if ( CWrite ( CFHandle, CopyData, (WriteLeft * sizeof (WORD))) != (WriteLeft * sizeof (WORD)))
      {
        FreeSampleData ( CopyData, SAVESAMPLE_BUFFER * sizeof (WORD));

        return ( ERROR_SYNTH_IOERR );
      }
    } while ( (WaveLen -= WriteLeft) );

    FreeSampleData ( CopyData, SAVESAMPLE_BUFFER * sizeof (WORD));

    if ( WaveformStr->tcyw_Length & 1 )
    {
      *(ULONG *) &(IFF_Chunk.iff_Identifier[0]) = 0L;

      if ( (CWrite ( CFHandle, &IFF_Chunk, 1 )) != 1 )
        return ( ERROR_SYNTH_IOERR );
    }
  }
#else
  if ( (CWrite ( CFHandle, WaveData, FORMSize )) != FORMSize )
    return ( ERROR_SYNTH_IOERR );
#endif

  if ( (WaveLenPos = CSeek ( CFHandle, WaveLenPos, OFFSET_BEGINNING )) == 0xFFFFFFFF )
    return ( ERROR_SYNTH_IOERR );

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), WaveSize );

  if ( CWrite ( CFHandle, &(IFF_Header.ifh_Length[0]), sizeof (ULONG)) != sizeof (ULONG))
    return ( ERROR_SYNTH_IOERR );

  if ( CSeek ( CFHandle, WaveLenPos, OFFSET_BEGINNING ) == 0xFFFFFFFF )
    return ( ERROR_SYNTH_IOERR );

  *ChunkLenBuf += WaveSize + (sizeof (struct IFFHeader) - sizeof (ULONG));

  return ( 0L );
}

static ULONG SaveTCYSmbl ( struct CFHandle *CFHandle, struct TuComposerSynthSymbolTable *SymbolStr, ULONG *ChunkLenBuf )
{
  struct IFFHeader IFF_Header;
  struct IFF_SREF SREF;
  ULONG SmblSize = sizeof (ULONG);
  ULONG SmblLenPos;
  ULONG FaultCode;

  if ( (SmblLenPos = CSeek ( CFHandle, 0L, OFFSET_CURRENT )) == 0xFFFFFFFF )
    return ( ERROR_SYNTH_IOERR );

  SmblLenPos += sizeof (ULONG);

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0]), 'FORM' );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), SmblSize );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_TypeId[0]), 'SMBL' );

  if ( (CWrite ( CFHandle, &IFF_Header, sizeof (struct IFFHeader )) != sizeof (struct IFFHeader )))
    return ( ERROR_SYNTH_IOERR );

  if ( (FaultCode = SaveTCYName ( CFHandle, SymbolStr->tsst_SymbolName, 'NAME', (ULONG *) &SmblSize, ERROR_SYNTH_IOERR )))
    return ( FaultCode );

  memcpy ( &(SREF.sref_SymbolValue), &(SymbolStr->tsst_SymbolValue), (sizeof (struct IFF_SREF) - sizeof (struct IFFChunk)));

#ifndef MOTOROLA
  WriteMWord ( (UBYTE *) &(SREF.sref_SymbolValue), SREF.sref_SymbolValue );
  WriteMWord ( (UBYTE *) &(SREF.sref_FirstLine), SREF.sref_FirstLine );
  WriteMWord ( (UBYTE *) &(SREF.sref_LastLine), SREF.sref_LastLine );
#endif

  if ( (FaultCode = SaveTCYChunk ( CFHandle, (struct IFFChunk *) &SREF, 'SREF', sizeof (struct IFF_SREF), (ULONG *) &SmblSize, ERROR_SYNTH_IOERR )))
    return ( FaultCode );

  if ( (SmblLenPos = CSeek ( CFHandle, SmblLenPos, OFFSET_BEGINNING )) == 0xFFFFFFFF )
    return ( ERROR_SYNTH_IOERR );

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), SmblSize );

  if ( CWrite ( CFHandle, &(IFF_Header.ifh_Length[0]), sizeof (ULONG)) != sizeof (ULONG))
    return ( ERROR_SYNTH_IOERR );

  if ( CSeek ( CFHandle, SmblLenPos, OFFSET_BEGINNING ) == 0xFFFFFFFF )
    return ( ERROR_SYNTH_IOERR );

  *ChunkLenBuf += SmblSize + (sizeof (struct IFFHeader) - sizeof (ULONG));

  return ( 0L );
}

static ULONG GetTCYInfo ( struct CFHandle *CFHandle,
    struct SynthInfo *SynthInfo, unsigned long NumSynth )
{
  struct IFFHeader IFF_Header;
  struct IFF_YHDR YHDR;
  ULONG Length, IFFSize;
  UBYTE ScanFlags = 0;

  for (;;)
  {
    if ( (CRead ( CFHandle, &IFF_Header, sizeof (struct IFFHeader)) != sizeof (struct IFFHeader)) )
      return ( 0L );

    if ( (ReadMLong ( (UBYTE *) &IFF_Header.ifh_FORMId[0] ) != 'FORM') || ( (Length = ReadMLong ( (UBYTE *) &IFF_Header.ifh_Length[0]) ) < 4) || ((ReadMLong ( (UBYTE *) &IFF_Header.ifh_TypeId[0] ) >> 8) != 'TCY') )
      return ( 0L );

    Length -= 4;

    if (!( NumSynth-- ))
      break;

    if ( CSeek ( CFHandle, Length, OFFSET_CURRENT ) == 0xFFFFFFFF )
      return ( 0L );
  }

  do
  {
    struct IFFChunk IFF_Chunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      return ( 0L );

    if ( (IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0] )) & 1 )
      IFFSize++;

    IFFLength = IFFSize;

    switch ( ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] ) )
    {
      case 'FILE' :
      {
        if ( ScanFlags & 0x01 )
          break;

        if (!( SynthInfo->yi_FileName = (STRPTR) AllocPMem ( ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0] ) + 1) ))
          goto GetTCYError;

        if ( (CRead ( CFHandle, SynthInfo->yi_FileName, IFFLength ) != IFFLength) )
          goto GetTCYError;

        IFFLength = 0L;
        ScanFlags |= 0x01;

        break;
      }
      case 'NAME' :
      {
        if ( ScanFlags & 0x02 )
          break;

        if (!( SynthInfo->yi_Name = (STRPTR) AllocPMem ( ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0] ) + 1) ))
          goto GetTCYError;

        if ( (CRead ( CFHandle, SynthInfo->yi_Name, IFFLength ) != IFFLength) )
          goto GetTCYError;

        IFFLength = 0L;
        ScanFlags |= 0x02;

        break;
      }
      case 'YHDR' :
      {
        if ( ScanFlags & 0x04 )
          break;

        if ( (CRead ( CFHandle, &(YHDR.yhdr_Version), (sizeof (struct IFF_YHDR) - sizeof (struct IFFChunk))) != (sizeof (struct IFF_YHDR) - sizeof (struct IFFChunk))) )
          goto GetTCYError;

        SynthInfo->yi_Version = YHDR.yhdr_Version;
        SynthInfo->yi_Revision = YHDR.yhdr_Revision;
        SynthInfo->yi_NumWaves = ReadMWord ( (UBYTE *) &(YHDR.yhdr_NumWaves) );

        memcpy ( &(SynthInfo->yi_VolumePos), &(YHDR.yhdr_VolumePos), sizeof (struct IFF_YHDR) - offsetof (struct IFF_YHDR, yhdr_VolumePos ));

#ifndef MOTOROLA
        SynthInfo->yi_VolumePos = ReadMWord ( (UBYTE *) &(SynthInfo->yi_VolumePos) );
        SynthInfo->yi_PanningPos = ReadMWord ( (UBYTE *) &(SynthInfo->yi_PanningPos) );
        SynthInfo->yi_SlidePos = ReadMWord ( (UBYTE *) &(SynthInfo->yi_SlidePos) );
        SynthInfo->yi_SpecialPos = ReadMWord ( (UBYTE *) &(SynthInfo->yi_SpecialPos) );
        SynthInfo->yi_VolumeSusPos = ReadMWord ( (UBYTE *) &(SynthInfo->yi_VolumeSusPos) );
        SynthInfo->yi_PanningSusPos = ReadMWord ( (UBYTE *) &(SynthInfo->yi_PanningSusPos) );
        SynthInfo->yi_SlideSusPos = ReadMWord ( (UBYTE *) &(SynthInfo->yi_SlideSusPos) );
        SynthInfo->yi_SpecialSusPos = ReadMWord ( (UBYTE *) &(SynthInfo->yi_SpecialSusPos) );
        SynthInfo->yi_VolNNAPos = ReadMWord ( (UBYTE *) &(SynthInfo->yi_VolNNAPos) );
        SynthInfo->yi_PanNNAPos = ReadMWord ( (UBYTE *) &(SynthInfo->yi_PanNNAPos) );
        SynthInfo->yi_SlideNNAPos = ReadMWord ( (UBYTE *) &(SynthInfo->yi_SlideNNAPos) );
        SynthInfo->yi_SpecialNNAPos = ReadMWord ( (UBYTE *) &(SynthInfo->yi_SpecialNNAPos) );
        SynthInfo->yi_VolDNAPos = ReadMWord ( (UBYTE *) &(SynthInfo->yi_VolDNAPos) );
        SynthInfo->yi_PanDNAPos = ReadMWord ( (UBYTE *) &(SynthInfo->yi_PanDNAPos) );
        SynthInfo->yi_SlideDNAPos = ReadMWord ( (UBYTE *) &(SynthInfo->yi_SlideDNAPos) );
        SynthInfo->yi_SpecialDNAPos = ReadMWord ( (UBYTE *) &(SynthInfo->yi_SpecialDNAPos) );
        SynthInfo->yi_Variable0 = ReadMWord ( (UBYTE *) &(SynthInfo->yi_Variable0) );
        SynthInfo->yi_Variable1 = ReadMWord ( (UBYTE *) &(SynthInfo->yi_Variable1) );
        SynthInfo->yi_Variable2 = ReadMWord ( (UBYTE *) &(SynthInfo->yi_Variable2) );
        SynthInfo->yi_Variable3 = ReadMWord ( (UBYTE *) &(SynthInfo->yi_Variable3) );
        SynthInfo->yi_Variable4 = ReadMWord ( (UBYTE *) &(SynthInfo->yi_Variable4) );
        SynthInfo->yi_Variable5 = ReadMWord ( (UBYTE *) &(SynthInfo->yi_Variable5) );
        SynthInfo->yi_Variable6 = ReadMWord ( (UBYTE *) &(SynthInfo->yi_Variable6) );
        SynthInfo->yi_Variable7 = ReadMWord ( (UBYTE *) &(SynthInfo->yi_Variable7) );
        SynthInfo->yi_Variable8 = ReadMWord ( (UBYTE *) &(SynthInfo->yi_Variable8) );
        SynthInfo->yi_Variable9 = ReadMWord ( (UBYTE *) &(SynthInfo->yi_Variable9) );
        SynthInfo->yi_VariableA = ReadMWord ( (UBYTE *) &(SynthInfo->yi_VariableA) );
        SynthInfo->yi_VariableB = ReadMWord ( (UBYTE *) &(SynthInfo->yi_VariableB) );
        SynthInfo->yi_VariableC = ReadMWord ( (UBYTE *) &(SynthInfo->yi_VariableC) );
        SynthInfo->yi_VariableD = ReadMWord ( (UBYTE *) &(SynthInfo->yi_VariableD) );
        SynthInfo->yi_VariableE = ReadMWord ( (UBYTE *) &(SynthInfo->yi_VariableE) );
        SynthInfo->yi_VariableF = ReadMWord ( (UBYTE *) &(SynthInfo->yi_VariableF) );
        SynthInfo->yi_VolConVar = ReadMWord ( (UBYTE *) &(SynthInfo->yi_VolConVar) );
        SynthInfo->yi_PanConVar = ReadMWord ( (UBYTE *) &(SynthInfo->yi_PanConVar) );
        SynthInfo->yi_SlideConVar = ReadMWord ( (UBYTE *) &(SynthInfo->yi_SlideConVar) );
        SynthInfo->yi_SpecialConVar = ReadMWord ( (UBYTE *) &(SynthInfo->yi_SpecialConVar) );
        SynthInfo->yi_VarKeepMask = ReadMWord ( (UBYTE *) &(SynthInfo->yi_VarKeepMask) );
#endif

        IFFLength -= (sizeof (struct IFF_YHDR) - sizeof (struct IFFChunk));
        ScanFlags |= 0x04;

        break;
      }
      case 'CODE' :
      {
        if ( ScanFlags & 0x08 )
          break;

        SynthInfo->yi_CodeEntries = ReadMLong ( (UBYTE *) &(IFF_Chunk.iff_Length[0]) ) / sizeof (struct TuComposerSynthCode);

        ScanFlags |= 0x08;

        break;
      }
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
      {
GetTCYError:
        FreeNameStr ( SynthInfo->yi_FileName );
        FreeNameStr ( SynthInfo->yi_Name );

        return ( 0L );
      }
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      goto GetTCYError;

  } while ( (Length -= IFFSize) );

  if (!( ScanFlags & 0x04 ))
    goto GetTCYError;

  return ( -1L );
}

static ULONG GetNumTCYs ( struct CFHandle *CFHandle )
{
  struct IFFHeader IFF_Header;
  ULONG NumTCYs = 0L;

  for (;;)
  {
    ULONG Length;

    if ( (CRead ( CFHandle, &IFF_Header, sizeof (struct IFFHeader)) != sizeof (struct IFFHeader)) )
      break;

    if ( (ReadMLong ( (UBYTE *) &IFF_Header.ifh_FORMId[0] ) != 'FORM') || ( (Length = ReadMLong ( (UBYTE *) &IFF_Header.ifh_Length[0]) ) < 4) )
      break;

    if ( CSeek ( CFHandle, Length - 4, OFFSET_CURRENT ) == 0xFFFFFFFF )
      break;

    if ( ((ReadMLong ( (UBYTE *) &IFF_Header.ifh_TypeId[0] ) >> 8) != 'TCY') )
      continue;

    NumTCYs++;
  }

  return ( NumTCYs );
}

#ifndef MOTOROLA

static UWORD ReadMWord ( UBYTE *Buffer )
{
  UWORD MotorolaWord;

  MotorolaWord = *Buffer++ << 8U;
  MotorolaWord += *Buffer;

  return ( MotorolaWord );
}

static void WriteMWord ( UBYTE *Buffer, UWORD Value )
{
  *Buffer++ = Value >> 8U;
  *Buffer = Value;
}

static ULONG ReadMLong ( UBYTE *Buffer )
{
  ULONG MotorolaLong;

  MotorolaLong = *Buffer++;
  MotorolaLong <<= 8UL;
  MotorolaLong += *Buffer++;
  MotorolaLong <<= 8UL;
  MotorolaLong += *Buffer++;
  MotorolaLong <<= 8UL;
  MotorolaLong += *Buffer;

  return ( MotorolaLong );
}

static void WriteMLong ( UBYTE *Buffer, ULONG Value )
{
  *Buffer++ = Value >> 24UL;
  *Buffer++ = Value >> 16UL;
  *Buffer++ = Value >> 8UL;
  *Buffer = Value;
}

#endif
