/* TuComposer OS independent code
**
** $VER: DupTCModule.c 31.22 (4.8.99)
**
** (C) 1999 by Sebastian Vater (known as Basty/Seasons).
**
*/

#include <stddef.h>
#include <string.h>
#include <exec/types.h>
#include <tucomposer/tucomposer.h>
#include <clib/tucomposer_protos.h>

struct TuComposerModule *DoDupTCModule ( struct TuComposerModule *ModuleStr, struct TuComposerModule *CloneModule );

#ifndef MOTOROLA
  ULONG ReadMLong ( UBYTE *Buffer );
#else
  #define ReadMLong( Buffer ) (*(ULONG *) (Buffer))
#endif

struct TuComposerModule *DupTCModule ( struct TuComposerModule *ModuleStr )
{
  return ( DoDupTCModule ( ModuleStr, NULL ));
}

struct TuComposerModule *DoDupTCModule ( struct TuComposerModule *ModuleStr, struct TuComposerModule *CloneModule )
{
  if ( ModuleStr )
  {
    if ( CloneModule == NULL )
      CloneModule = AllocTCModule ( ModuleStr->tcm_Channels );

    if ( CloneModule )
    {
      struct TuComposerSong *SongStr;
      struct TuComposerSong *CloneSong;
      struct TuComposerInstr *InstrStr;
      struct TuComposerInstr *CloneInstr;
      struct TuComposerInstrEnvelope *EnvStr;
      struct TuComposerInstrEnvelope *CloneEnv;
      struct TuComposerInstrKeyboard *KeybrdStr;
      struct TuComposerInstrKeyboard *CloneKeybrd;
      struct TuComposerInstrArpeggio *ArpStr;
      struct TuComposerInstrArpeggio *CloneArp;
      struct UnknownChunk *ModuleUnkChunk;
      struct UnknownChunk *CloneUnkChunk;

      if ( ModuleStr->tcm_FileName )
      {
        if (!( SetString ( (STRPTR *) &(CloneModule->tcm_FileName), ModuleStr->tcm_FileName )))
          goto DupModuleErr;
      }

      if ( ModuleStr->tcm_Name )
      {
        if (!( SetString ( (STRPTR *) &(CloneModule->tcm_Name), ModuleStr->tcm_Name )))
          goto DupModuleErr;
      }

      if ( ModuleStr->tcm_Message )
      {
        if (!( SetString ( (STRPTR *) &(CloneModule->tcm_Message), ModuleStr->tcm_Message )))
          goto DupModuleErr;
      }

      if ( ModuleStr->tcm_Composer )
      {
        if (!( SetString ( (STRPTR *) &(CloneModule->tcm_Composer), ModuleStr->tcm_Composer )))
          goto DupModuleErr;
      }

      if ( ModuleStr->tcm_Style )
      {
        if (!( SetString ( (STRPTR *) &(CloneModule->tcm_Style), ModuleStr->tcm_Style )))
          goto DupModuleErr;
      }

      if ( ModuleStr->tcm_Comment )
      {
        if (!( SetString ( (STRPTR *) &(CloneModule->tcm_Comment), ModuleStr->tcm_Comment )))
          goto DupModuleErr;
      }

      SongStr = (struct TuComposerSong *) &(ModuleStr->tcm_SongList.mlh_Head);

      for (;;)
      {
        SongStr = (struct TuComposerSong *) SongStr->tcso_Node.mln_Succ;

        if (!( SongStr->tcso_Node.mln_Succ))
          break;

        if (!( CloneSong = DupSong ( SongStr )))
          goto DupModuleErr;

        AddSong ( CloneModule, CloneSong );
      }

      InstrStr = (struct TuComposerInstr *) &(ModuleStr->tcm_InstrList.mlh_Head);

      for (;;)
      {
        InstrStr = (struct TuComposerInstr *) InstrStr->tci_Node.mln_Succ;

        if (!( InstrStr->tci_Node.mln_Succ))
          break;

        if (!( CloneInstr = DupInstr ( InstrStr )))
          goto DupModuleErr;

        AddInstr ( CloneModule, CloneInstr );
      }

      EnvStr = (struct TuComposerInstrEnvelope *) &(ModuleStr->tcm_Envelopes.mlh_Head);

      for (;;)
      {
        EnvStr = (struct TuComposerInstrEnvelope *) EnvStr->tcie_Node.mln_Succ;

        if (!( EnvStr->tcie_Node.mln_Succ))
          break;

        if (!( CloneEnv = DupEnvelope ( EnvStr )))
          goto DupModuleErr;

        AddEnvelope ( CloneModule, CloneEnv );
      }

      KeybrdStr = (struct TuComposerInstrKeyboard *) &(ModuleStr->tcm_KeyboardDefs.mlh_Head);

      for (;;)
      {
        KeybrdStr = (struct TuComposerInstrKeyboard *) KeybrdStr->tcik_Node.mln_Succ;

        if (!( KeybrdStr->tcik_Node.mln_Succ))
          break;

        if (!( CloneKeybrd = DupKeyboard ( KeybrdStr )))
          goto DupModuleErr;

        AddKeyboard ( CloneModule, CloneKeybrd );
      }

      ArpStr = (struct TuComposerInstrArpeggio *) &(ModuleStr->tcm_ArpEnvList.mlh_Head);

      for (;;)
      {
        ArpStr = (struct TuComposerInstrArpeggio *) ArpStr->tcia_Node.mln_Succ;

        if (!( ArpStr->tcia_Node.mln_Succ))
          break;

        if (!( CloneArp = DupArpeggio ( ArpStr )))
          goto DupModuleErr;

        AddArpeggio ( CloneModule, CloneArp );
      }

      if ( SetTCMChannels ( CloneModule, ModuleStr->tcm_Channels ))
        goto DupModuleErr;

      memcpy ( &(CloneModule->tcm_BeginDate), &(ModuleStr->tcm_BeginDate), offsetof (struct TuComposerModule, tcm_SongList) - offsetof (struct TuComposerModule, tcm_BeginDate));

      CloneModule->tcm_CompFlags = ModuleStr->tcm_CompFlags;
      CloneModule->tcm_Flags = ModuleStr->tcm_Flags;

      ModuleUnkChunk = (struct UnknownChunk *) &(ModuleStr->tcm_UnknownChunks.mlh_Head);

      for (;;)
      {
        ModuleUnkChunk = (struct UnknownChunk *) ModuleUnkChunk->uc_Node.mln_Succ;

        if (!( ModuleUnkChunk->uc_Node.mln_Succ))
          break;

        if (!( CloneUnkChunk = AllocUnknownChunk ( (APTR) &(ModuleUnkChunk->uc_Data), ReadMLong ( (UBYTE *) &(ModuleUnkChunk->uc_Identifier[0])), ReadMLong ( (UBYTE *) &(ModuleUnkChunk->uc_Length[0])) )))
          goto DupModuleErr;

        AddNodeTail ( (struct List *) &(CloneModule->tcm_UnknownChunks), (struct Node *) CloneUnkChunk );
      }
    }
  } else
  {
    CloneModule = NULL;
  }

  return ( CloneModule );

DupModuleErr:

  FreeTCModule ( CloneModule );

  return ( NULL );
}
