/* TuComposer OS independent code
**
** $VER: AmplifySample.c 31.22 (24.7.99)
**
** (C) 1999 by Sebastian Vater (known as Basty/Seasons).
**
*/

#include <exec/types.h>
#include <tucomposer/tucomposer.h>

#ifdef __cplusplus
extern "C" {
#endif

void AmplifySample8 ( BYTE *SampleData, ULONG StartSmp, ULONG StartAmplify, ULONG NumSamples, LONG AmpDiv, ULONG AmpMod, LONG AmpDir );
void AmplifySample16 ( WORD *SampleData, ULONG StartSmp, ULONG StartAmplify, ULONG NumSamples, LONG AmpDiv, ULONG AmpMod, LONG AmpDir );
void AmplifySample32 ( LONG *SampleData, ULONG StartSmp, ULONG StartAmplify, ULONG NumSamples, LONG AmpDiv, ULONG AmpMod, LONG AmpDir );
void AmplifySampleX ( LONG *SampleData, ULONG StartSmp, ULONG StartAmplify, ULONG NumSamples, LONG AmpDiv, ULONG AmpMod, LONG AmpDir, UBYTE BitDepth );

#ifdef __cplusplus
}
#endif

ULONG AmplifySample ( struct TuComposerSample *SampleStr,
    unsigned long StartAmplify, unsigned long EndAmplify,
    unsigned long StartSmp, unsigned long LastSmp )
{
  if ( SampleStr )
  {
    WORD *SampleData;
    ULONG Samples, NumSamples, AmpDiv, AmpMod;
    LONG AmpChg, AmpDir = 1L;

    if ( (SampleData = SampleStr->tcs_Data ))
    {
      Samples = SampleStr->tcs_Samples;

      if ( StartSmp >= Samples )
        StartSmp = Samples - 1;

      if ( LastSmp >= Samples )
        LastSmp = Samples - 1;

      if ( (StartAmplify == 0x10000) && (StartAmplify == EndAmplify) )
        return ( 0L );

      if ( StartSmp > LastSmp )
      {
        NumSamples = LastSmp;
        LastSmp = StartSmp;
        StartSmp = NumSamples;
      }

      NumSamples = (LastSmp - StartSmp) + 1;

      if ( (AmpChg = (EndAmplify - StartAmplify)) < 0 )
      {
        AmpChg = -AmpChg;
        AmpDir = -1L;
      }

      AmpDiv = (ULONG) AmpChg / NumSamples;
      AmpMod = (ULONG) AmpChg % NumSamples;

      if ( AmpDir < 0 )
        AmpDiv = -AmpDiv;

      switch ( SampleStr->tcs_BitDepth )
      {
        case 8 :
        {
          AmplifySample8 ( (BYTE *) SampleData, StartSmp, StartAmplify, NumSamples, AmpDiv, AmpMod, AmpDir );

          break;
        }
        case 16 :
        {
          AmplifySample16 ( SampleData, StartSmp, StartAmplify, NumSamples, AmpDiv, AmpMod, AmpDir );

          break;
        }
        case 32 :
        {
          AmplifySample32 ( (LONG *) SampleData, StartSmp, StartAmplify, NumSamples, AmpDiv, AmpMod, AmpDir );

          break;
        }
        default :
        {
          AmplifySampleX ( (LONG *) SampleData, StartSmp, StartAmplify, NumSamples, AmpDiv, AmpMod, AmpDir, SampleStr->tcs_BitDepth );

          break;
        }
      }

      return ( 0L );
    }

    return ( ERROR_SAMPLE_NODATA );
  }

  return ( ERROR_SAMPLE_NOGIVEN );
}

void AmplifySample8 ( BYTE *SampleData, ULONG StartSmp, ULONG StartAmplify, ULONG NumSamples, LONG AmpDiv, ULONG AmpMod, LONG AmpDir )
{
  ULONG i, AmpCount = 0L;
  LONG Sample;
  long long TmpSample;

  SampleData += StartSmp;

  for (i = NumSamples; i > 0; i--)
  {
    TmpSample = (*SampleData * (long long) StartAmplify) >> 16;
    Sample = TmpSample;

    if ( Sample < -128 )
      Sample = -128;

    if ( Sample > 127 )
      Sample = 127;

    *SampleData++ = Sample;

    StartAmplify += AmpDiv;
    AmpCount += AmpMod;

    if ( AmpCount >= NumSamples )
    {
      AmpCount -= NumSamples;
      StartAmplify += AmpDir;
    }
  }
}

void AmplifySample16 ( WORD *SampleData, ULONG StartSmp, ULONG StartAmplify, ULONG NumSamples, LONG AmpDiv, ULONG AmpMod, LONG AmpDir )
{
  ULONG i, AmpCount = 0L;
  LONG Sample;
  long long TmpSample;

  SampleData += StartSmp;

  for (i = NumSamples; i > 0; i--)
  {
    TmpSample = (*SampleData * (long long) StartAmplify) >> 16;
    Sample = TmpSample;

    if ( Sample < -32768 )
      Sample = -32768;

    if ( Sample > 32767 )
      Sample = 32767;

    *SampleData++ = Sample;

    StartAmplify += AmpDiv;
    AmpCount += AmpMod;

    if ( AmpCount >= NumSamples )
    {
      AmpCount -= NumSamples;
      StartAmplify += AmpDir;
    }
  }
}

void AmplifySample32 ( LONG *SampleData, ULONG StartSmp, ULONG StartAmplify, ULONG NumSamples, LONG AmpDiv, ULONG AmpMod, LONG AmpDir )
{
  ULONG i, AmpCount = 0L;
  long long Sample;

  SampleData += StartSmp;

  for (i = NumSamples; i > 0; i--)
  {
    Sample = (*SampleData * (long long) StartAmplify) >> 16;

    if ( Sample < -2147483648LL )
      Sample = -2147483648LL;

    if ( Sample > 2147483647LL )
      Sample = 2147483647LL;

    *SampleData++ = Sample;

    StartAmplify += AmpDiv;
    AmpCount += AmpMod;

    if ( AmpCount >= NumSamples )
    {
      AmpCount -= NumSamples;
      StartAmplify += AmpDir;
    }
  }
}

void AmplifySampleX ( LONG *SampleData, ULONG StartSmp, ULONG StartAmplify, ULONG NumSamples, LONG AmpDiv, ULONG AmpMod, LONG AmpDir, UBYTE BitDepth )
{
  ULONG i, AmpCount = 0L, Bit;
  ULONG TmpSample;
  long long Sample;

  SampleData += (StartSmp * BitDepth) / 32;
  Bit = (StartSmp * BitDepth) % 32;

  for (i = NumSamples; i > 0; i--)
  {
    if ( (Bit + BitDepth) < 32 )
    {
      TmpSample = (*SampleData << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
    } else
    {
      TmpSample = *SampleData << Bit;
      TmpSample |= (*(SampleData + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
    }

    Sample = ((long long) TmpSample * StartAmplify) >> 16;

    if ( Sample < -2147483648LL )
      Sample = -2147483648LL;

    if ( Sample > 2147483647LL )
      Sample = 2147483647LL;

    TmpSample = Sample;
    TmpSample &= ~((1UL << (32UL - BitDepth)) - 1UL );

    if ( (Bit + BitDepth) < 32 )
    {
      *SampleData &= ~((1UL << (32UL - Bit)) - (1UL << (32UL - BitDepth - Bit)));
      *SampleData |= (TmpSample >> Bit);
    } else
    {
      *SampleData &= ~((1UL << (32UL - Bit)) - 1UL);
      *SampleData |= (TmpSample >> Bit);
      SampleData++;
      *SampleData &= ((1UL << (64UL - (Bit + BitDepth))) - 1UL);
      *SampleData |= (TmpSample << (32UL - Bit));

      Bit -= 32;
    }

    Bit += BitDepth;

    StartAmplify += AmpDiv;
    AmpCount += AmpMod;

    if ( AmpCount >= NumSamples )
    {
      AmpCount -= NumSamples;
      StartAmplify += AmpDir;
    }
  }
}
