/* TuComposer OS independent code
**
** $VER: Backwards.c 31.22 (25.7.99)
**
** (C) 1999 by Sebastian Vater (known as Basty/Seasons).
**
*/

#include <exec/types.h>
#include <tucomposer/tucomposer.h>
#include <clib/tucomposer_protos.h>

#ifdef __cplusplus
extern "C" {
#endif

void Backwards8 ( BYTE *SampleData, ULONG StartSmp, ULONG LastSmp, ULONG NumSamples );
void Backwards16 ( WORD *SampleData, ULONG StartSmp, ULONG LastSmp, ULONG NumSamples );
void Backwards32 ( LONG *SampleData, ULONG StartSmp, ULONG LastSmp, ULONG NumSamples );
void BackwardsX ( LONG *SampleData, ULONG StartSmp, ULONG LastSmp, ULONG NumSamples, UBYTE BitDepth );

#ifdef __cplusplus
}
#endif

ULONG Backwards ( struct TuComposerSample *SampleStr, unsigned long StartSmp,
    unsigned long LastSmp )
{
  if ( SampleStr )
  {
    WORD *SampleData;
    ULONG Samples, NumSamples;

   if ( (SampleData = SampleStr->tcs_Data) )
   {
     Samples = SampleStr->tcs_Samples;

     if ( StartSmp >= Samples )
       StartSmp = Samples - 1;

     if ( LastSmp >= Samples )
       LastSmp = Samples - 1;

     if ( StartSmp > LastSmp )
     {
       NumSamples = LastSmp;
       LastSmp = StartSmp;
       StartSmp = NumSamples;
     }

     NumSamples = ((LastSmp - StartSmp) + 1) >> 1;

     switch ( SampleStr->tcs_BitDepth )
     {
       case 8 :
       {
         Backwards8 ( (BYTE *) SampleData, StartSmp, LastSmp, NumSamples );

         break;
       }
       case 16 :
       {
         Backwards16 ( SampleData, StartSmp, LastSmp, NumSamples );

         break;
       }
       case 32 :
       {
         Backwards32 ( (LONG *) SampleData, StartSmp, LastSmp, NumSamples );

         break;
       }
       default :
       {
         BackwardsX ( (LONG *) SampleData, StartSmp, LastSmp, NumSamples, SampleStr->tcs_BitDepth );

         break;
       }
     }

     return ( 0L );
   }

   return ( ERROR_SAMPLE_NODATA );
  }

  return ( ERROR_SAMPLE_NOGIVEN );
}

void Backwards8 ( BYTE *SampleData, ULONG StartSmp, ULONG LastSmp, ULONG NumSamples )
{
  BYTE *BackSampleData;
  BYTE Sample;
  ULONG i;

  BackSampleData = SampleData + LastSmp;
  SampleData += StartSmp;

  for (i = NumSamples; i > 0; i--)
  {
    Sample = *SampleData;
    *SampleData++ = *BackSampleData;
    *BackSampleData = Sample;
    BackSampleData--;
  }
}

void Backwards16 ( WORD *SampleData, ULONG StartSmp, ULONG LastSmp, ULONG NumSamples )
{
  WORD *BackSampleData;
  WORD Sample;
  ULONG i;

  BackSampleData = SampleData + LastSmp;
  SampleData += StartSmp;

  for (i = NumSamples; i > 0; i--)
  {
    Sample = *SampleData;
    *SampleData++ = *BackSampleData;
    *BackSampleData = Sample;
    BackSampleData--;
  }
}

void Backwards32 ( LONG *SampleData, ULONG StartSmp, ULONG LastSmp, ULONG NumSamples )
{
  LONG *BackSampleData;
  LONG Sample;
  ULONG i;

  BackSampleData = SampleData + LastSmp;
  SampleData += StartSmp;

  for (i = NumSamples; i > 0; i--)
  {
    Sample = *SampleData;
    *SampleData++ = *BackSampleData;
    *BackSampleData = Sample;
    BackSampleData--;
  }
}

void BackwardsX ( LONG *SampleData, ULONG StartSmp, ULONG LastSmp, ULONG NumSamples, UBYTE BitDepth )
{
  LONG *BackSampleData;
  ULONG Sample, BackSample;
  ULONG i, Bit, BackBit;

  BackSampleData = SampleData + ((LastSmp * BitDepth) / 32);
  SampleData += (StartSmp * BitDepth) / 32;

  BackBit = (LastSmp * BitDepth) % 32;
  Bit = (StartSmp * BitDepth) % 32;

  for (i = NumSamples; i > 0; i--)
  {
    if ( (Bit + BitDepth) < 32 )
    {
      Sample = (*SampleData << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
    } else
    {
      Sample = *SampleData << Bit;
      Sample |= (*(SampleData + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
    }

    if ( (BackBit + BitDepth) < 32 )
    {
      BackSample = (*BackSampleData << BackBit) & ~((1UL << (32UL - BitDepth)) - 1UL);
    } else
    {
      BackSample = *BackSampleData << BackBit;
      BackSample |= (*(BackSampleData + 1) & ~((1UL << (64UL - (BackBit + BitDepth))) - 1UL)) >> (32UL - BackBit);
    }

    BackSample &= ~((1UL << (32UL - BitDepth)) - 1 );

    if ( (Bit + BitDepth) < 32 )
    {
      *SampleData &= ~((1UL << (32UL - Bit)) - (1UL << (32UL - BitDepth - Bit)));
      *SampleData |= (BackSample >> Bit);
    } else
    {
      *SampleData &= ~((1UL << (32UL - Bit)) - 1UL);
      *SampleData |= (BackSample >> Bit);
      SampleData++;
      *SampleData &= ((1UL << (64UL - (Bit + BitDepth))) - 1UL);
      *SampleData |= (BackSample << (32UL - Bit));

      Bit -= 32;
    }

    Bit += BitDepth;

    Sample &= ~((1UL << (32UL - BitDepth)) - 1 );

    if ( (BackBit + BitDepth) < 32 )
    {
      *BackSampleData &= ~((1UL << (32UL - BackBit)) - (1UL << (32UL - BitDepth - BackBit)));
      *BackSampleData |= (Sample >> BackBit);
    } else
    {
      *BackSampleData &= ~((1UL << (32UL - BackBit)) - 1UL);
      *BackSampleData |= (Sample >> BackBit);
      BackSampleData++;
      *BackSampleData &= ((1UL << (64UL - (BackBit + BitDepth))) - 1UL);
      *BackSampleData |= (Sample << (32UL - BackBit));
      BackSampleData--;
    }

    BackBit -= BitDepth;

    if ( (LONG) BackBit < 0 )
    {
      BackSampleData--;

      BackBit += 32;
    }
  }
}
