/* TuComposer OS independent code
**
** $VER: ClearSampleData.c 31.22 (25.7.99)
**
** (C) 1999 by Sebastian Vater (known as Basty/Seasons).
**
*/

#include <exec/types.h>
#include <tucomposer/tucomposer.h>
#include <clib/tucomposer_protos.h>

#ifdef __cplusplus
extern "C" {
#endif

void ClearSampleData8 ( BYTE *SampleData, ULONG FirstSmp, ULONG NumSamples );
void ClearSampleData16 ( WORD *SampleData, ULONG FirstSmp, ULONG NumSamples );
void ClearSampleData32 ( LONG *SampleData, ULONG FirstSmp, ULONG NumSamples );
void ClearSampleDataX ( LONG *SampleData, ULONG FirstSmp, ULONG NumSamples, UBYTE BitDepth );

#ifdef __cplusplus
}
#endif

ULONG ClearSampleData ( struct TuComposerSample *SampleStr,
    unsigned long FirstSmp, unsigned long LastSmp )
{
  if ( SampleStr )
  {
    WORD *SampleData;
    ULONG Samples, NumSamples;

   if ( (SampleData = SampleStr->tcs_Data) )
   {
     Samples = SampleStr->tcs_Samples;

     if ( FirstSmp >= Samples )
       FirstSmp = Samples - 1;

     if ( LastSmp >= Samples )
       LastSmp = Samples - 1;

     if ( FirstSmp > LastSmp )
     {
       NumSamples = LastSmp;
       LastSmp = FirstSmp;
       FirstSmp = NumSamples;
     }

     NumSamples = (LastSmp - FirstSmp) + 1;

     switch ( SampleStr->tcs_BitDepth )
     {
       case 8 :
       {
         ClearSampleData8 ( (BYTE *) SampleData, FirstSmp, NumSamples );

         break;
       }
       case 16 :
       {
         ClearSampleData16 ( SampleData, FirstSmp, NumSamples );

         break;
       }
       case 32 :
       {
         ClearSampleData32 ( (LONG *) SampleData, FirstSmp, NumSamples );

         break;
       }
       default :
       {
         ClearSampleDataX ( (LONG *) SampleData, FirstSmp, NumSamples, SampleStr->tcs_BitDepth );

         break;
       }
     }

     return ( 0L );
   }

   return ( ERROR_SAMPLE_NODATA );
  }

  return ( ERROR_SAMPLE_NOGIVEN );
}

void ClearSampleData8 ( BYTE *SampleData, ULONG FirstSmp, ULONG NumSamples )
{
  ULONG i;

  SampleData += FirstSmp;

  for (i = NumSamples; i > 0; i--)
  {
    *SampleData++ = 0;
  }
}

void ClearSampleData16 ( WORD *SampleData, ULONG FirstSmp, ULONG NumSamples )
{
  ULONG i;

  SampleData += FirstSmp;

  for (i = NumSamples; i > 0; i--)
  {
    *SampleData++ = 0;
  }
}

void ClearSampleData32 ( LONG *SampleData, ULONG FirstSmp, ULONG NumSamples )
{
  ULONG i;

  SampleData += FirstSmp;

  for (i = NumSamples; i > 0; i--)
  {
    *SampleData++ = 0L;
  }
}

void ClearSampleDataX ( LONG *SampleData, ULONG FirstSmp, ULONG NumSamples, UBYTE BitDepth )
{
  ULONG i, Bit;

  SampleData += (FirstSmp * BitDepth) / 32;
  Bit = (FirstSmp * BitDepth) % 32;

  for (i = NumSamples; i > 0; i--)
  {
    if ( (Bit + BitDepth) < 32 )
    {
      *SampleData &= ~((1UL << (32UL - Bit)) - (1UL << (32UL - BitDepth - Bit)));
    } else
    {
      *SampleData &= ~((1UL << (32UL - Bit)) - 1UL);
      SampleData++;
      *SampleData &= ((1UL << (64UL - (Bit + BitDepth))) - 1UL);

      Bit -= 32;
    }

    Bit += BitDepth;
  }
}
