/* TuComposer OS independent code
**
** $VER: ConvertBits.c 31.22 (22.7.99)
**
** (C) 1999 by Sebastian Vater (known as Basty/Seasons).
**
*/

#include <stddef.h>
#include <exec/types.h>
#include <tucomposer/tucomposer.h>
#include <clib/tucomposer_protos.h>

#ifdef __cplusplus
extern "C" {
#endif

void ConvertBits816 ( BYTE *SampleData, WORD *NewSmpData, ULONG Samples );
void ConvertBits832 ( BYTE *SampleData, LONG *NewSmpData, ULONG Samples );
void ConvertBits168 ( WORD *SampleData, BYTE *NewSmpData, ULONG Samples );
void ConvertBits1632 ( WORD *SampleData, LONG *NewSmpData, ULONG Samples );
void ConvertBits328 ( LONG *SampleData,BYTE *NewSmpData, ULONG Samples );
void ConvertBits3216 ( LONG *SampleData, WORD *NewSmpData, ULONG Samples );
void ConvertBitsXY ( LONG *SampleData, LONG *NewSmpData, ULONG Samples, ULONG SrcBitDepth, ULONG DestBitDepth );

#ifdef __cplusplus
}
#endif

ULONG ConvertBits ( struct TuComposerSample *SampleStr,
    unsigned long DestBitDepth, unsigned long Flags )
{
  if ( SampleStr = FindOriginSample ( SampleStr ))
  {
    WORD *SampleData;
    WORD *NewSmpData;
   ULONG SrcBitDepth, NewLength, Samples;

    if ( (SampleData = SampleStr->tcs_Data) )
    {
      struct TuComposerModule *ModuleStr;

      if ( DestBitDepth == 0 )
        return ( ERROR_SAMPLE_BITDEPTH );

      if ( DestBitDepth > MAXBITDEPTH )
        DestBitDepth = MAXBITDEPTH;

      SrcBitDepth = SampleStr->tcs_BitDepth;

      if ( SrcBitDepth != DestBitDepth )
      {
        Samples = SampleStr->tcs_Samples;

        if ( Flags & CBF_ChangeOnly )
        {
          SampleStr->tcs_Samples = (Samples * SrcBitDepth) / DestBitDepth;

          if ( SampleStr->tcs_Node.mln_Succ )
          {
            struct TuComposerInstr *InstrStr = (struct TuComposerInstr *) &(SampleStr->tcs_Node.mln_Succ);

            for (;;)
            {
              InstrStr = (struct TuComposerInstr *) InstrStr->tci_Node.mln_Pred;

              if (!( InstrStr->tci_Node.mln_Pred ))
                break;
            }

            (UBYTE *) InstrStr += -offsetof (struct TuComposerInstr, tci_Samples );

            if ( InstrStr->tci_Node.mln_Succ )
            {
              ModuleStr = (struct TuComposerModule *) &(InstrStr->tci_Node.mln_Succ);

              for (;;)
              {
                ModuleStr = (struct TuComposerModule *) ModuleStr->tcm_Node.mln_Pred;

                if (!( ModuleStr->tcm_Node.mln_Pred ))
                  break;
              }

              (UBYTE *) ModuleStr += -offsetof (struct TuComposerModule, tcm_InstrList );

              InstrStr = (struct TuComposerInstr *) &(ModuleStr->tcm_InstrList.mlh_Head);

              for (;;)
              {
                struct TuComposerSample *OrigSampleStr;

                InstrStr = (struct TuComposerInstr *) InstrStr->tci_Node.mln_Succ;

                if (!( InstrStr->tci_Node.mln_Succ ))
                  break;

                OrigSampleStr = (struct TuComposerSample *) &(InstrStr->tci_Samples.mlh_Head);

                for (;;)
                {
                  OrigSampleStr = (struct TuComposerSample *) OrigSampleStr->tcs_Node.mln_Succ;

                  if (!( OrigSampleStr->tcs_Node.mln_Succ ))
                    break;

                  if ( (!( OrigSampleStr->tcs_Flags & TCSF_Redirect )) || ( OrigSampleStr->tcs_BitDepth != SrcBitDepth ) )
                    continue;

                  if ( OrigSampleStr->tcs_Data == SampleStr->tcs_Data )
                    OrigSampleStr->tcs_Samples = (Samples * SrcBitDepth) / DestBitDepth;
                }
              }
            }
          }
        } else
        {
          NewLength = ((Samples * DestBitDepth) + SMP_BLOCKMASK) / SMP_BLOCKSIZE;

          if (!( NewSmpData = AllocSampleData ( NewLength )))
            return ( ERROR_SAMPLE_NOMEM );

          switch ( SrcBitDepth )
          {
            case 8 :
            {
              switch ( DestBitDepth )
              {
                case 16 :
                {
                  ConvertBits816 ( (BYTE *) SampleData, NewSmpData, Samples );

                  break;
                }
                case 32 :
                {
                  ConvertBits832 ( (BYTE *) SampleData, (LONG *) NewSmpData, Samples );

                  break;
                }
                default :
                {
                  goto OtherBits;

                  break;
                }
              }

              break;
            }
            case 16 :
            {
              switch ( DestBitDepth )
              {
                case 8 :
                {
                  ConvertBits168 ( SampleData, (BYTE *) NewSmpData, Samples );

                  break;
                }
                case 32 :
                {
                  ConvertBits1632 ( SampleData, (LONG *) NewSmpData, Samples );

                  break;
                }
                default :
                {
                  goto OtherBits;

                  break;
                }
              }

              break;
            }
            case 32 :
            {
              switch ( DestBitDepth )
              {
                case 8 :
                {
                  ConvertBits328 ( (LONG *) SampleData, (BYTE *) NewSmpData, Samples );

                  break;
                }
                case 16 :
                {
                  ConvertBits3216 ( (LONG *) SampleData, NewSmpData, Samples );

                  break;
                }
                default :
                {
                  goto OtherBits;

                  break;
                }
              }

              break;
            }
            default :
            {
OtherBits:

              ConvertBitsXY ( (LONG *) SampleData, (LONG *) NewSmpData, Samples, SrcBitDepth, DestBitDepth );

              break;
            }
          }

          FreeSampleData ( SampleData, SampleStr->tcs_Length );

          SampleStr->tcs_Data = NewSmpData;
          SampleStr->tcs_Length = NewLength;

          if ( SampleStr->tcs_Node.mln_Succ )
          {
            struct TuComposerInstr *InstrStr = (struct TuComposerInstr *) &(SampleStr->tcs_Node.mln_Succ);

            for (;;)
            {
              InstrStr = (struct TuComposerInstr *) InstrStr->tci_Node.mln_Pred;

              if (!( InstrStr->tci_Node.mln_Pred ))
                break;
            }

            (UBYTE *) InstrStr += -offsetof (struct TuComposerInstr, tci_Samples );

            if ( InstrStr->tci_Node.mln_Succ )
            {
              ModuleStr = (struct TuComposerModule *) &(InstrStr->tci_Node.mln_Succ);

              for (;;)
              {
                ModuleStr = (struct TuComposerModule *) ModuleStr->tcm_Node.mln_Pred;

                if (!( ModuleStr->tcm_Node.mln_Pred ))
                  break;
              }

              (UBYTE *) ModuleStr += -offsetof (struct TuComposerModule, tcm_InstrList );

              InstrStr = (struct TuComposerInstr *) &(ModuleStr->tcm_InstrList.mlh_Head);

              for (;;)
              {
                struct TuComposerSample *OrigSampleStr;

                InstrStr = (struct TuComposerInstr *) InstrStr->tci_Node.mln_Succ;

                if (!( InstrStr->tci_Node.mln_Succ ))
                  break;

                OrigSampleStr = (struct TuComposerSample *) &(InstrStr->tci_Samples.mlh_Head);

                for (;;)
                {
                  OrigSampleStr = (struct TuComposerSample *) OrigSampleStr->tcs_Node.mln_Succ;

                  if (!( OrigSampleStr->tcs_Node.mln_Succ ))
                    break;

                  if ( (!( OrigSampleStr->tcs_Flags & TCSF_Redirect )) || ( OrigSampleStr->tcs_BitDepth != SrcBitDepth ) )
                    continue;

                  if ( OrigSampleStr->tcs_Data == SampleData )
                  {
                    OrigSampleStr->tcs_Data = NewSmpData;
                    OrigSampleStr->tcs_Length = ((SampleStr->tcs_Samples * DestBitDepth) + SMP_BLOCKMASK) / SMP_BLOCKSIZE;

                    OrigSampleStr->tcs_BitDepth = DestBitDepth;
                  }
                }
              }
            }
          }
        }
      }

      SampleStr->tcs_BitDepth = DestBitDepth;
    }

    return ( 0L );
  }

  return ( ERROR_SAMPLE_NOGIVEN );
}

void ConvertBits816 ( BYTE *SampleData, WORD *NewSmpData, ULONG Samples )
{
  ULONG i;

  for (i = Samples; i > 0; i--)
  {
#ifndef MOTOROLA
    *NewSmpData++ = *SampleData++ << 8U;
#else
    *(BYTE *) NewSmpData = *SampleData++;
    NewSmpData++;
#endif
  }
}

void ConvertBits832 ( BYTE *SampleData, LONG *NewSmpData, ULONG Samples )
{
  ULONG i;

  for (i = Samples; i > 0; i--)
  {
#ifndef MOTOROLA
    *NewSmpData++ = *SampleData++ << 24UL;
#else
    *(BYTE *) NewSmpData = *SampleData++;
    NewSmpData++;
#endif
  }
}

void ConvertBits168 ( WORD *SampleData, BYTE *NewSmpData, ULONG Samples )
{
  ULONG i;

  for (i = Samples; i > 0; i--)
  {
#ifndef MOTOROLA
    *NewSmpData++ = *SampleData++ >> 8U;
#else
    *NewSmpData++ = *(BYTE *) SampleData;
    SampleData++;
#endif
  }
}

void ConvertBits1632 ( WORD *SampleData, LONG *NewSmpData, ULONG Samples )
{
  ULONG i;

  for (i = Samples; i > 0; i--)
  {
#ifndef MOTOROLA
    *NewSmpData++ = *SampleData++ << 16UL;
#else
    *(WORD *) NewSmpData = *SampleData++;
    NewSmpData++;
#endif
  }
}

void ConvertBits328 ( LONG *SampleData, BYTE *NewSmpData, ULONG Samples )
{
  ULONG i;

  for (i = Samples; i > 0; i--)
  {
#ifndef MOTOROLA
    *NewSmpData++ = *SampleData++ >> 24UL;
#else
    *NewSmpData++ = *(BYTE *) SampleData;
    SampleData++;
#endif
  }
}

void ConvertBits3216 ( LONG *SampleData, WORD *NewSmpData, ULONG Samples )
{
  ULONG i;

  for (i = Samples; i > 0; i--)
  {
#ifndef MOTOROLA
    *NewSmpData++ = *SampleData++ >> 16UL;
#else
    *NewSmpData++ = *(WORD *) SampleData;
    SampleData++;
#endif
  }
}

void ConvertBitsXY ( LONG *SampleData, LONG *NewSmpData, ULONG Samples, ULONG SrcBitDepth, ULONG DestBitDepth )
{
  ULONG SrcSample;
  ULONG SrcBit = 0L, DestBit = 0L, i;

  for (i = Samples; i > 0; i--)
  {
    if ( (SrcBit + SrcBitDepth) < 32 )
    {
      SrcSample = (*SampleData << SrcBit) & ~((1UL << (32UL - SrcBitDepth)) - 1UL);
    } else
    {
      SrcSample = *SampleData++ << SrcBit;
      SrcSample |= (*SampleData & ~((1UL << (64UL - (SrcBit + SrcBitDepth))) - 1UL)) >> (32UL - SrcBit);

      SrcBit -= 32;
    }

    SrcBit += SrcBitDepth;

    SrcSample &= ~((1UL << (32UL - DestBitDepth)) - 1UL );

    *NewSmpData |= (SrcSample >> DestBit);

    if ( (DestBit + DestBitDepth) >= 32 )
    {
      NewSmpData++;
      *NewSmpData = (SrcSample << (32UL - DestBit));

      DestBit -= 32;
    }

    DestBit += DestBitDepth;
  }
}
