/* TuComposer OS independent code
**
** $VER: ConvertSigned.c 31.22 (25.7.99)
**
** (C) 1999 by Sebastian Vater (known as Basty/Seasons).
**
*/

#include <exec/types.h>
#include <tucomposer/tucomposer.h>
#include <clib/tucomposer_protos.h>

#ifdef __cplusplus
extern "C" {
#endif

void ConvertSigned8 ( LONG *SampleData, ULONG Length );
void ConvertSigned16 ( LONG *SampleData, ULONG Length );
void ConvertSigned32 ( LONG *SampleData, ULONG Length );
void ConvertSignedX ( LONG *SampleData, ULONG Samples, UBYTE BitDepth );

#ifdef __cplusplus
}
#endif

ULONG ConvertSigned ( struct TuComposerSample *SampleStr )
{
  if ( SampleStr = FindOriginSample ( SampleStr ))
  {
    WORD *SampleData;

    if ( (SampleData = SampleStr->tcs_Data ))
    {
      ULONG Length;

      if ( (Length = (SampleStr->tcs_Length + SMP_BLOCKMASK) / SMP_BLOCKSIZE ))
      {
        switch ( SampleStr->tcs_BitDepth )
        {
          case 8 :
          {
            ConvertSigned8 ( (LONG *) SampleData, Length );

            break;
          }
          case 16 :
          {
            ConvertSigned16 ( (LONG *) SampleData, Length );

            break;
          }
          case 32 :
          {
            ConvertSigned32 ( (LONG *) SampleData, Length );

            break;
          }
          default :
          {
            ConvertSignedX ( (LONG *) SampleData, SampleStr->tcs_Samples, SampleStr->tcs_BitDepth );

            break;
          }
        }
      }

      return ( 0L );
    }

    return ( ERROR_SAMPLE_NODATA );
  }

  return ( ERROR_SAMPLE_NOGIVEN );
}

void ConvertSigned8 ( LONG *SampleData, ULONG Length )
{
  ULONG i, Mask = 0x80808080;

  for (i = Length; i > 0; i--)
  {
    *SampleData ^= Mask;
    SampleData++;
    *SampleData ^= Mask;
    SampleData++;
  }
}

void ConvertSigned16 ( LONG *SampleData, ULONG Length )
{
  ULONG i, Mask = 0x80008000;

  for (i = Length; i > 0; i--)
  {
    *SampleData ^= Mask;
    SampleData++;
    *SampleData ^= Mask;
    SampleData++;
  }
}

void ConvertSigned32 ( LONG *SampleData, ULONG Length )
{
  ULONG i, Mask = 0x80000000;

  for (i = Length; i > 0; i--)
  {
    *SampleData ^= Mask;
    SampleData++;
    *SampleData ^= Mask;
    SampleData++;
  }
}

void ConvertSignedX ( LONG *SampleData, ULONG Samples, UBYTE BitDepth )
{
  ULONG i, Bit = 0L, Mask = 0x80000000;
  ULONG Sample;

  for (i = Samples; i > 0; i--)
  {
    if ( (Bit + BitDepth) < 32 )
    {
      Sample = (*SampleData << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
    } else
    {
      Sample = *SampleData << Bit;
      Sample |= (*(SampleData + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
    }

    Sample ^= Mask;
    Sample &= ~((1UL << (32UL - BitDepth)) - 1UL );

    if ( (Bit + BitDepth) < 32 )
    {
      *SampleData &= ~((1UL << (32UL - Bit)) - (1UL << (32UL - BitDepth - Bit)));
      *SampleData |= (Sample >> Bit);
    } else
    {
      *SampleData &= ~((1UL << (32UL - Bit)) - 1UL);
      *SampleData |= (Sample >> Bit);
      SampleData++;
      *SampleData &= ((1UL << (64UL - (Bit + BitDepth))) - 1UL);
      *SampleData |= (Sample << (32UL - Bit));

      Bit -= 32;
    }

    Bit += BitDepth;
  }
}
