/* TuComposer OS independent code
**
** $VER: CopySampleData.c 31.22 (27.4.00)
**
** (C) 2000 by Sebastian Vater (known as Basty/Seasons).
**
*/

#include <string.h>
#include <exec/types.h>
#include <tucomposer/tucomposer.h>
#include <clib/exec_protos.h>
#include <clib/tucomposer_protos.h>

#ifdef __cplusplus
extern "C" {
#endif

void CopySampleDataX ( LONG *SampleData, LONG *NewSampleData, ULONG SrcSamplePos, ULONG DestSamplePos, ULONG NumSamples, UBYTE BitDepth );

#ifdef __cplusplus
}
#endif

struct TuComposerSample *DoDupSample ( struct TuComposerSample *SampleStr, struct TuComposerSample *CloneSample );

extern struct TuComposerBase *TuCBase;

struct TuComposerSample *CopySampleData ( struct TuComposerSample *SampleStr,
    struct TuComposerSample *SampleBuf,
    unsigned long FirstSmp, unsigned long LastSmp )
{
  struct TuComposerSample *SampleGiven = SampleBuf;
  ULONG SmpSize;

  if ( FirstSmp >= SampleStr->tcs_Samples )
    FirstSmp = SampleStr->tcs_Samples - 1;

  if ( LastSmp >= SampleStr->tcs_Samples )
    LastSmp = SampleStr->tcs_Samples - 1;

  if ( FirstSmp > LastSmp )
  {
    ULONG TmpSmp = FirstSmp;

    FirstSmp = LastSmp;
    LastSmp = TmpSmp;
  }

  SmpSize = (LastSmp - FirstSmp) + 1;

  if (!( SampleBuf ))
    SampleBuf = PushSample ( SampleStr );

  if ( SampleBuf )
  {
    ObtainSemaphore ( TuCBase->tcb_SampleSigSem );

    if (!( ReSizeSample ( SampleBuf, SmpSize )))
    {
      ClearSample ( SampleBuf );

      if (!( SampleBuf = DoDupSample ( SampleStr, SampleBuf )))
        goto CopySampleDataErr;

      switch ( SampleStr->tcs_BitDepth )
      {
        case 8 :
        {
          memcpy ( (BYTE *) SampleBuf->tcs_Data, (BYTE *) SampleStr->tcs_Data + FirstSmp, SmpSize );

          break;
        }
        case 16 :
        {
          memcpy ( SampleBuf->tcs_Data, SampleStr->tcs_Data + FirstSmp, SmpSize * sizeof (WORD));

          break;
        }
        case 32 :
        {
          memcpy ( (LONG *) SampleBuf->tcs_Data, (LONG *) SampleStr->tcs_Data + FirstSmp, SmpSize * sizeof (LONG));

          break;
        }
        default :
        {
          CopySampleDataX ( (LONG *) SampleStr->tcs_Data, (LONG *) SampleBuf->tcs_Data, FirstSmp, 0L, SmpSize, SampleStr->tcs_BitDepth );

          break;
        }
      }

      ReleaseSemaphore ( TuCBase->tcb_SampleSigSem );

      return ( SampleBuf );
    }

CopySampleDataErr:

    if (!( SampleGiven ))
      PopSample ( SampleBuf );

    ReleaseSemaphore ( TuCBase->tcb_SampleSigSem );
  }

  return ( NULL );
}

void CopySampleDataX ( LONG *SampleData, LONG *NewSampleData, ULONG SrcSamplePos, ULONG DestSamplePos, ULONG NumSamples, UBYTE BitDepth )
{
  ULONG i, SrcBit, DestBit = 0L;
  ULONG Sample;

  SampleData += (SrcSamplePos * BitDepth) / 32;
  SrcBit = (SrcSamplePos * BitDepth) % 32;

  NewSampleData += (DestSamplePos * BitDepth) / 32;
  SrcBit = (DestSamplePos * BitDepth) % 32;

  for (i = NumSamples; i > 0; i--)
  {
    if ( (SrcBit + BitDepth) < 32 )
    {
      Sample = (*SampleData << SrcBit) & ~((1UL << (32UL - BitDepth)) - 1UL);
    } else
    {
      Sample = *SampleData++ << SrcBit;
      Sample |= (*SampleData & ~((1UL << (64UL - (SrcBit + BitDepth))) - 1UL)) >> (32 - SrcBit);

      SrcBit -= 32;
    }

    SrcBit += BitDepth;

    Sample &= ~((1UL << (32UL - BitDepth)) - 1UL );

    if ( (DestBit + BitDepth) < 32UL )
    {
      *NewSampleData &= ~((1UL << (32UL - DestBit)) - (1UL << (32UL - BitDepth - DestBit)));
      *NewSampleData |= (Sample >> DestBit);
    } else
    {
      *NewSampleData &= ~((1UL << (32UL - DestBit)) - 1UL);
      *NewSampleData |= (Sample >> DestBit);
      NewSampleData++;
      *NewSampleData &= ((1UL << (64UL - (DestBit + BitDepth))) - 1UL);
      *NewSampleData |= (Sample << (32UL - DestBit));

      DestBit -= 32;
    }

    DestBit += BitDepth;
  }
}
