/* TuComposer OS independent code
**
** $VER: GetLowestAmplify.c 31.22 (25.7.99)
**
** (C) 1999 by Sebastian Vater (known as Basty/Seasons).
**
*/

#include <exec/types.h>
#include <tucomposer/tucomposer.h>
#include <clib/tucomposer_protos.h>

#ifdef __cplusplus
extern "C" {
#endif

ULONG GetLowestAmplify8 ( BYTE *SampleData, ULONG StartSmp, ULONG NumSamples );
ULONG GetLowestAmplify16 ( WORD *SampleData, ULONG StartSmp, ULONG NumSamples );
ULONG GetLowestAmplify32 ( LONG *SampleData, ULONG StartSmp, ULONG NumSamples );
ULONG GetLowestAmplifyX ( LONG *SampleData, ULONG StartSmp, ULONG NumSamples, UBYTE BitDepth );

#ifdef __cplusplus
}
#endif

ULONG GetLowestAmplify ( struct TuComposerSample *SampleStr,
    unsigned long StartSmp, unsigned long LastSmp )
{
  ULONG Amplify = 0L;

  if ( SampleStr )
  {
    WORD *SampleData;

    if ( (SampleData = SampleStr->tcs_Data ))
    {
      ULONG Samples, NumSamples;
      unsigned long long Sample;

      Samples = SampleStr->tcs_Samples;

      if ( StartSmp >= Samples )
        StartSmp = Samples - 1;

      if ( LastSmp >= Samples )
        LastSmp = Samples - 1;

      if ( StartSmp > LastSmp )
      {
        NumSamples = LastSmp;
        LastSmp = StartSmp;
        StartSmp = NumSamples;
      }

      NumSamples = (LastSmp - StartSmp) + 1;

      switch ( SampleStr->tcs_BitDepth )
      {
        case 8 :
        {
          Amplify = GetLowestAmplify8 ( (BYTE *) SampleData, StartSmp, NumSamples );

          break;
        }
        case 16 :
        {
          Amplify = GetLowestAmplify16 ( SampleData, StartSmp, NumSamples );

          break;
        }
        case 32 :
        {
          Amplify = GetLowestAmplify32 ( (LONG *) SampleData, StartSmp, NumSamples );

          break;
        }
        default :
        {
          Amplify = GetLowestAmplifyX ( (LONG *) SampleData, StartSmp, NumSamples, SampleStr->tcs_BitDepth );

          break;
        }
      }

      Sample = (1LL << (ULONG) (48 - SampleStr->tcs_BitDepth ));

      if ( Amplify )
        Sample /= Amplify;

      Amplify = (( Sample > 0x10000LL ) ? 0x10000 : Sample);
    }
  }

  return ( Amplify );
}

ULONG GetLowestAmplify8 ( BYTE *SampleData, ULONG StartSmp, ULONG NumSamples )
{
  ULONG i, Amplify = 0x80000000;
  LONG Sample;

  SampleData += StartSmp;

  for (i = NumSamples; i > 0; i--)
  {
    if ( ( Sample = (UBYTE) *SampleData++ ))
    {
      Sample <<= 24;

      if ( Sample < 0 )
        Sample = -Sample;

      if ( Sample < Amplify )
        Amplify = Sample;
    }
  }

  return ( Amplify );
}

ULONG GetLowestAmplify16 ( WORD *SampleData, ULONG StartSmp, ULONG NumSamples )
{
  ULONG i, Amplify = 0x80000000;
  LONG Sample;

  SampleData += StartSmp;

  for (i = NumSamples; i > 0; i--)
  {
    if ( ( Sample = (UWORD) *SampleData++ ))
    {
      Sample <<= 16;

      if ( Sample < 0 )
        Sample = -Sample;

      if ( Sample < Amplify )
        Amplify = Sample;
    }
  }

  return ( Amplify );
}

ULONG GetLowestAmplify32 ( LONG *SampleData, ULONG StartSmp, ULONG NumSamples )
{
  ULONG i, Amplify = 0x80000000;
  LONG Sample;

  SampleData += StartSmp;

  for (i = NumSamples; i > 0; i--)
  {
    if ( ( Sample = *SampleData++ ))
    {
      if ( Sample < 0 )
        Sample = -Sample;

      if ( Sample < Amplify )
        Amplify = Sample;
    }
  }

  return ( Amplify );
}

ULONG GetLowestAmplifyX ( LONG *SampleData, ULONG StartSmp, ULONG NumSamples, UBYTE BitDepth )
{
  ULONG i, Amplify = 0x80000000, Bit;
  LONG Sample;

  SampleData += (StartSmp * BitDepth) / 32;
  Bit = (StartSmp * BitDepth) % 32;

  for (i = NumSamples; i > 0; i--)
  {
    if ( (Bit + BitDepth) < 32 )
    {
      Sample = (*SampleData << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
    } else
    {
      Sample = *SampleData++ << Bit;
      Sample |= (*SampleData & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32 - Bit);

      Bit -= 32;
    }

    Bit += BitDepth;

    if ( Sample )
    {
      if ( Sample < 0 )
        Sample = -Sample;

      if ( Sample < Amplify )
        Amplify = Sample;
    }
  }

  return ( Amplify );
}
