/* TuComposer OS independent code
**
** $VER: GetOptimalBits.c 31.22 (5.2.00)
**
** (C) 2000 by Sebastian Vater (known as Basty/Seasons).
**
*/

#include <exec/types.h>
#include <tucomposer/tucomposer.h>
#include <clib/tucomposer_protos.h>

#ifdef __cplusplus
extern "C" {
#endif

ULONG GetOptimalBits8Y ( BYTE *SampleData, ULONG Samples, ULONG CheckBit );
ULONG GetOptimalBits168 ( WORD *SampleData, ULONG Samples );
ULONG GetOptimalBits16Y ( WORD *SampleData, ULONG Samples, ULONG CheckBit );
ULONG GetOptimalBits3216 ( LONG *SampleData, ULONG Samples );
ULONG GetOptimalBits32Y ( LONG *SampleData, ULONG Samples, ULONG CheckBit );
ULONG GetOptimalBitsX16 ( LONG *SampleData, ULONG Samples, ULONG BitDepth );
ULONG GetOptimalBitsXY ( LONG *SampleData, ULONG Samples, ULONG BitDepth, ULONG CheckBit );

#ifdef __cplusplus
}
#endif

ULONG GetOptimalBits ( struct TuComposerSample *SampleStr,
    unsigned long Flags )
{
  if ( SampleStr = FindOriginSample ( SampleStr ))
  {
    WORD *SampleData = SampleStr->tcs_Data;
    ULONG Samples = SampleStr->tcs_Samples;
    ULONG BitDepth = SampleStr->tcs_BitDepth;
    ULONG LastCheckBit = BitDepth, CheckBit = BitDepth;

    if ( Flags & GOBF_Check81632Only )
    {
      for (;;)
      {
        if ( CheckBit > 16 )
        {
          CheckBit = 16;
        } else if ( CheckBit > 8 )
        {
          CheckBit = 8;
        } else
        {
          break;
        }

        switch ( BitDepth )
        {
          case 16 :
          {
            if ( (GetOptimalBits168 ( SampleData, Samples )))
              LastCheckBit = CheckBit;

            break;
          }
          case 32 :
          {
            if ( CheckBit == 16 )
            {
              if ( (GetOptimalBits3216 ( (LONG *) SampleData, Samples )))
                LastCheckBit = CheckBit;
            } else
            {
              if ( (GetOptimalBits32Y ( (LONG *) SampleData, Samples, CheckBit )))
                LastCheckBit = CheckBit;
            }

            break;
          }
          default :
          {
            if ( CheckBit == 16 )
            {
              if ( (GetOptimalBitsX16 ( (LONG *) SampleData, Samples, BitDepth )))
                LastCheckBit = CheckBit;
            } else
            {
              if ( (GetOptimalBitsXY ( (LONG *) SampleData, Samples, BitDepth, CheckBit )))
                LastCheckBit = CheckBit;
            }

            break;
          }
        }

        if ( LastCheckBit == BitDepth )
        {
          break;
        } else
        {
          if ( Flags & GOBF_CheckOnce )
            break;
        }
      }
    } else
    {
      if ( LastCheckBit > 1 )
      {
        for (CheckBit = BitDepth - 1; CheckBit > 0; CheckBit--)
        {
          switch ( BitDepth )
          {
            case 8 :
            {
              if ( (GetOptimalBits8Y ( (BYTE *) SampleData, Samples, CheckBit )))
                LastCheckBit = CheckBit;

              break;
            }
            case 16 :
            {
              if ( CheckBit == 8 )
              {
                if ( (GetOptimalBits168 ( SampleData, Samples )))
                  LastCheckBit = CheckBit;
              } else
              {
                if ( (GetOptimalBits16Y ( SampleData, Samples, CheckBit )))
                  LastCheckBit = CheckBit;
              }

              break;
            }
            case 32 :
            {
              if ( CheckBit == 16 )
              {
                if ( (GetOptimalBits3216 ( (LONG *) SampleData, Samples )))
                  LastCheckBit = CheckBit;
              } else
              {
                if ( (GetOptimalBits32Y ( (LONG *) SampleData, Samples, CheckBit )))
                  LastCheckBit = CheckBit;
              }

              break;
            }
            default :
            {
              if ( CheckBit == 16 )
              {
                if ( (GetOptimalBitsX16 ( (LONG *) SampleData, Samples, BitDepth )))
                  LastCheckBit = CheckBit;
              } else
              {
                if ( (GetOptimalBitsXY ( (LONG *) SampleData, Samples, BitDepth, CheckBit )))
                  LastCheckBit = CheckBit;
              }

              break;
            }
          }

          if ( LastCheckBit == BitDepth )
          {
            break;
          } else
          {
            if ( Flags & GOBF_CheckOnce )
              break;
          }
        }
      }
    }

    return ( LastCheckBit );
  }

  return ( 0L );
}

ULONG GetOptimalBits8Y ( BYTE *SampleData, ULONG Samples, ULONG CheckBit )
{
  ULONG i;
  UBYTE Mask = (1U << (8U - CheckBit)) - 1U;

  for (i = Samples; i > 0; i--)
  {
    if ( *SampleData++ & Mask )
      return ( 0L );
  }

  return ( -1L );
}

ULONG GetOptimalBits168 ( WORD *SampleData, ULONG Samples )
{
  ULONG i;

  for ( i = Samples; i > 0; i--)
  {
    BYTE Sample = *SampleData++;

    if ( Sample )
      return ( 0L );
  }

  return ( -1L );
}

ULONG GetOptimalBits16Y ( WORD *SampleData, ULONG Samples, ULONG CheckBit )
{
  ULONG i;
  UWORD Mask = (1U << (16U - CheckBit)) - 1U;

  for (i = Samples; i > 0; i--)
  {
    if ( *SampleData++ & Mask )
      return ( 0L );
  }

  return ( -1L );
}

ULONG GetOptimalBits3216 ( LONG *SampleData, ULONG Samples )
{
  ULONG i;

  for ( i = Samples; i > 0; i--)
  {
    WORD Sample = *SampleData++;

    if ( Sample )
      return ( 0L );
  }

  return ( -1L );
}

ULONG GetOptimalBits32Y ( LONG *SampleData, ULONG Samples, ULONG CheckBit )
{
  ULONG i;
  ULONG Mask = (1UL << (32UL - CheckBit)) - 1UL;

  for (i = Samples; i > 0; i--)
  {
    if ( *SampleData++ & Mask )
      return ( 0L );
  }

  return ( -1L );
}

ULONG GetOptimalBitsX16 ( LONG *SampleData, ULONG Samples, ULONG BitDepth )
{
  ULONG Bit = 0L, i;

  for (i = Samples; i > 0; i--)
  {
    ULONG Sample;
    WORD  CheckSample;

    if ( (Bit + BitDepth) < 32 )
    {
      Sample = (*SampleData << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
    } else
    {
      Sample = *SampleData++ << Bit;
      Sample |= (*SampleData & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

      Bit -= 32;
    }

    Bit += BitDepth;

    CheckSample = Sample;

    if ( CheckSample )
      return ( 0L );
  }

  return ( -1L );
}

ULONG GetOptimalBitsXY ( LONG *SampleData, ULONG Samples, ULONG BitDepth, ULONG CheckBit )
{
  ULONG Bit = 0L, i;
  ULONG Mask = (1UL << (32UL - CheckBit)) - 1UL;

  for (i = Samples; i > 0; i--)
  {
    ULONG Sample;

    if ( (Bit + BitDepth) < 32 )
    {
      Sample = (*SampleData << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
    } else
    {
      Sample = *SampleData++ << Bit;
      Sample |= (*SampleData & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

      Bit -= 32;
    }

    Bit += BitDepth;

    if ( Sample & Mask )
      return ( 0L );
  }

  return ( -1L );
}
