/* TuComposer OS independent code
**
** $VER: GetSampleValueArray.c 31.22 (25.7.99)
**
** (C) 1999 by Sebastian Vater (known as Basty/Seasons).
**
*/

#include <string.h>
#include <exec/types.h>
#include <tucomposer/tucomposer.h>
#include <clib/tucomposer_protos.h>

#ifdef __cplusplus
extern "C" {
#endif

void GetSampleValueArray8 ( BYTE *SampleData, LONG *ArrayPtr, ULONG StartSmp, ULONG ArrayEntries );
void GetSampleValueArray16 ( WORD *SampleData, LONG *ArrayPtr, ULONG StartSmp, ULONG ArrayEntries );
void GetSampleValueArrayX ( LONG *SampleData, LONG *ArrayPtr, ULONG StartSmp, ULONG ArrayEntries, UBYTE BitDepth );

#ifdef __cplusplus
}
#endif

ULONG GetSampleValueArray ( struct TuComposerSample *SampleStr,
    LONG *ArrayPtr, unsigned long StartSmp, unsigned long ArrayEntries )
{
  if ( SampleStr )
  {
    WORD *SampleData;

   if ( (SampleData = SampleStr->tcs_Data ))
   {
     ULONG Samples;

     Samples = SampleStr->tcs_Samples;

     if ( StartSmp >= Samples )
       StartSmp = Samples - 1;

     if ( (StartSmp + ArrayEntries) >= Samples )
       ArrayEntries = Samples - StartSmp;

     switch ( SampleStr->tcs_BitDepth )
     {
       case 8 :
       {
         GetSampleValueArray8 ( (BYTE *) SampleData, ArrayPtr, StartSmp, ArrayEntries );

         break;
       }
       case 16 :
       {
         GetSampleValueArray16 ( SampleData, ArrayPtr, StartSmp, ArrayEntries );

         break;
       }
       case 32 :
       {
         memcpy ( ArrayPtr, (LONG *) SampleData + StartSmp, ArrayEntries * sizeof (LONG));

         break;
       }
       default :
       {
         GetSampleValueArrayX ( (LONG *) SampleData, ArrayPtr, StartSmp, ArrayEntries, SampleStr->tcs_BitDepth );

         break;
       }
     }
   }

   return ( ERROR_SAMPLE_NODATA );
  }

  return ( ERROR_SAMPLE_NOGIVEN );
}

void GetSampleValueArray8 ( BYTE *SampleData, LONG *ArrayPtr, ULONG StartSmp, ULONG ArrayEntries )
{
  ULONG i;

  SampleData += StartSmp;

  for (i = ArrayEntries; i > 0; i--)
  {
    *ArrayPtr++ = ( (UBYTE) *SampleData++ << 24);
  }
}

void GetSampleValueArray16 ( WORD *SampleData, LONG *ArrayPtr, ULONG StartSmp, ULONG ArrayEntries )
{
  ULONG i;

  SampleData += StartSmp;

  for (i = ArrayEntries; i > 0; i--)
  {
    *ArrayPtr++ = ( (UWORD) *SampleData++ << 16);
  }
}

void GetSampleValueArrayX ( LONG *SampleData, LONG *ArrayPtr, ULONG StartSmp, ULONG ArrayEntries, UBYTE BitDepth )
{
  ULONG i, Bit;
  LONG Sample;

  SampleData += (StartSmp * BitDepth) / 32;
  Bit = (StartSmp * BitDepth) % 32;

  for (i = ArrayEntries; i > 0; i--)
  {
    if ( (Bit + BitDepth) < 32 )
    {
      Sample = (*SampleData << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
    } else
    {
      Sample = *SampleData++ << Bit;
      Sample |= (*SampleData & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32 - Bit);

      Bit -= 32;
    }

    Bit += BitDepth;

    *ArrayPtr++ = Sample;
  }
}
