/* TuComposer OS independent code
**
** $VER: InsertSampleData.c 31.22 (24.4.00)
**
** (C) 2000 by Sebastian Vater (known as Basty/Seasons).
**
*/

#include <string.h>
#include <exec/types.h>
#include <tucomposer/tucomposer.h>
#include <clib/tucomposer_protos.h>

#ifdef __cplusplus
extern "C" {
#endif

void InsertSampleDataX ( LONG *SampleData, LONG *NewSampleData, ULONG NewSamples, ULONG SamplePos, ULONG NumSamples, UBYTE BitDepth );

#ifdef __cplusplus
}
#endif

ULONG InsertSampleData ( struct TuComposerSample *SampleStr,
    unsigned long SamplePos, unsigned long NumSamples )
{
  if ( SampleStr )
  {
    ULONG NewSamples, NewBuffers, Buffers;
    WORD *SampleData;
    WORD *NewSampleData;

    NewSamples = NumSamples + SampleStr->tcs_Samples;

    if ( NewSamples < SampleStr->tcs_Samples )
      return ( ERROR_SAMPLE_TOOBIG );

    NewBuffers = (( NewSamples * SampleStr->tcs_BitDepth) + SMP_BLOCKMASK) / SMP_BLOCKSIZE;

    if ( SampleStr->tcs_Flags & TCSF_Redirect )
    {
      struct TuComposerSample *OrigSampleStr = FindOriginSample ( SampleStr );

      if ( NewSamples > OrigSampleStr->tcs_Samples )
        NewSamples = OrigSampleStr->tcs_Samples;

      NewBuffers = (( NewSamples * SampleStr->tcs_BitDepth) + SMP_BLOCKMASK) / SMP_BLOCKSIZE;

      SampleStr->tcs_Samples = NewSamples;
      SampleStr->tcs_Length = NewBuffers;

      return ( 0L );
    } else
    {
      if ( (NewSampleData = (WORD *) AllocSampleData ( NewBuffers )))
      {
        Buffers = (SampleStr->tcs_Length + SMP_BLOCKMASK) / SMP_BLOCKSIZE;

        if ( Buffers )
        {
          if ( (SampleData = SampleStr->tcs_Data ))
          {
            switch ( SampleStr->tcs_BitDepth )
            {
              case 8 :
              {
                if ( SamplePos )
                  memcpy ( (BYTE *) NewSampleData, (BYTE *) SampleData, SamplePos );

                memcpy ( (BYTE *) NewSampleData + SamplePos + NumSamples, (BYTE *) SampleData + SamplePos, (NewSamples - (SamplePos + NumSamples)));

                break;
              }
              case 16 :
              {
                if ( SamplePos )
                  memcpy ( NewSampleData, SampleData, SamplePos * sizeof (WORD));

                memcpy ( NewSampleData + SamplePos + NumSamples, SampleData + SamplePos, (NewSamples - (SamplePos + NumSamples)) * sizeof (WORD));

                break;
              }
              case 32 :
              {
                if ( SamplePos )
                  memcpy ( (LONG *) NewSampleData, (LONG *) SampleData, SamplePos * sizeof (LONG));

                memcpy ( (LONG *) NewSampleData + SamplePos + NumSamples, (LONG *) SampleData + SamplePos, (NewSamples - (SamplePos + NumSamples)) * sizeof (LONG));

                break;
              }
              default :
              {
                InsertSampleDataX ( (LONG *) SampleData, (LONG *) NewSampleData, NewSamples, SamplePos, NumSamples, SampleStr->tcs_BitDepth );

                break;
              }
            }

            FreeSampleData ( SampleData, SampleStr->tcs_Length );
          }
        }

        SampleStr->tcs_Data = NewSampleData;
        SampleStr->tcs_Samples = NewSamples;
        SampleStr->tcs_Length = NewBuffers;

        return ( 0L );
      }

      return ( ERROR_SAMPLE_NOMEM );
    }
  }

  return ( ERROR_SAMPLE_NOGIVEN );
}

void InsertSampleDataX ( LONG *SampleData, LONG *NewSampleData, ULONG NewSamples, ULONG SamplePos, ULONG NumSamples, UBYTE BitDepth )
{
  LONG *SrcSampleData;
  LONG *DestSampleData;
  ULONG i, SrcBit = 0L, DestBit = 0L;
  ULONG Sample;

  if ( SamplePos )
  {
    SrcSampleData = SampleData;
    DestSampleData = NewSampleData;

    for (i = SamplePos; i > 0; i--)
    {
      if ( (SrcBit + BitDepth) < 32 )
      {
        Sample = (*SrcSampleData << SrcBit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        Sample = *SrcSampleData++ << SrcBit;
        Sample |= (*SrcSampleData & ~((1UL << (64UL - (SrcBit + BitDepth))) - 1UL)) >> (32 - SrcBit);

        SrcBit -= 32;
      }

      SrcBit += BitDepth;

      Sample &= ~((1UL << (32UL - BitDepth)) - 1UL );

      if ( (DestBit + BitDepth) < 32UL )
      {
        *DestSampleData &= ~((1UL << (32UL - DestBit)) - (1UL << (32UL - BitDepth - DestBit)));
        *DestSampleData |= (Sample >> DestBit);
      } else
      {
        *DestSampleData &= ~((1UL << (32UL - DestBit)) - 1UL);
        *DestSampleData |= (Sample >> DestBit);
        DestSampleData++;
        *DestSampleData &= ((1UL << (64UL - (DestBit + BitDepth))) - 1UL);
        *DestSampleData |= (Sample << (32UL - DestBit));

        DestBit -= 32;
      }

      DestBit += BitDepth;
    }
  }

  SampleData += (SamplePos * BitDepth) / 32;
  NewSampleData += ((SamplePos + NumSamples) * BitDepth) / 32;

  SrcBit = (SamplePos * BitDepth) % 32;
  DestBit = ((SamplePos + NumSamples) * BitDepth) % 32;

  for (i = NewSamples - (SamplePos + NumSamples); i > 0; i--)
  {
    if ( (SrcBit + BitDepth) < 32 )
    {
      Sample = (*SampleData << SrcBit) & ~((1UL << (32UL - BitDepth)) - 1UL);
    } else
    {
      Sample = *SampleData++ << SrcBit;
      Sample |= (*SampleData & ~((1UL << (64UL - (SrcBit + BitDepth))) - 1UL)) >> (32 - SrcBit);

      SrcBit -= 32;
    }

    SrcBit += BitDepth;

    Sample &= ~((1UL << (32UL - BitDepth)) - 1UL );

    if ( (DestBit + BitDepth) < 32UL )
    {
      *NewSampleData &= ~((1UL << (32UL - DestBit)) - (1UL << (32UL - BitDepth - DestBit)));
      *NewSampleData |= (Sample >> DestBit);
    } else
    {
      *NewSampleData &= ~((1UL << (32UL - DestBit)) - 1UL);
      *NewSampleData |= (Sample >> DestBit);
      NewSampleData++;
      *NewSampleData &= ((1UL << (64UL - (DestBit + BitDepth))) - 1UL);
      *NewSampleData |= (Sample << (32UL - DestBit));

      DestBit -= 32;
    }

    DestBit += BitDepth;
  }
}
