/* TuComposer OS independent code
**
** $VER: OctaveDown.c 31.22 (25.7.99)
**
** (C) 1999 by Sebastian Vater (known as Basty/Seasons).
**
*/

#include <stddef.h>
#include <exec/types.h>
#include <tucomposer/tucomposer.h>
#include <clib/tucomposer_protos.h>

#ifdef __cplusplus
extern "C" {
#endif

void OctaveDown8 ( BYTE *SampleData, BYTE *NewSmpData, ULONG Samples );
void OctaveDown16 ( WORD *SampleData, WORD *NewSmpData, ULONG Samples );
void OctaveDown32 ( LONG *SampleData, LONG *NewSmpData, ULONG Samples );
void OctaveDownX ( LONG *SampleData, LONG *NewSmpData, ULONG Samples, UBYTE BitDepth );

#ifdef __cplusplus
}
#endif

ULONG OctaveDown ( struct TuComposerSample *SampleStr )
{
  if ( SampleStr = FindOriginSample ( SampleStr ))
  {
    WORD *SampleData;
    WORD *NewSmpData;

    if ( ( SampleData = SampleStr->tcs_Data ))
    {
      ULONG Samples, Length;
      UBYTE BitDepth;

      BitDepth = SampleStr->tcs_BitDepth;

      Samples = SampleStr->tcs_Samples * 2;
      Length = ((Samples * BitDepth) + SMP_BLOCKMASK) / SMP_BLOCKSIZE;

      if (!( NewSmpData = (WORD *) AllocSampleData ( Length )))
        return ( ERROR_SAMPLE_NOMEM );

      SampleStr->tcs_Length = Length;
      SampleStr->tcs_Samples = Samples;

      Samples /= 2;

      switch ( BitDepth )
      {
        case 8 :
        {
          OctaveDown8 ( (BYTE *) SampleData, (BYTE *) NewSmpData, Samples );

          break;
        }
        case 16 :
        {
          OctaveDown16 ( SampleData, NewSmpData, Samples );

          break;
        }
        case 32 :
        {
          OctaveDown32 ( (LONG *) SampleData, (LONG *) NewSmpData, Samples );

          break;
        }
        default :
        {
          OctaveDownX ( (LONG *) SampleData, (LONG *) NewSmpData, Samples, BitDepth );

          break;
        }
      }

      SampleStr->tcs_Repeat *= 2;
      SampleStr->tcs_RepLen *= 2;
      SampleStr->tcs_SustainRepeat *= 2;
      SampleStr->tcs_SustainRepLen *= 2;

      if ( SampleStr->tcs_Node.mln_Succ )
      {
        struct TuComposerInstr *InstrStr = (struct TuComposerInstr *) &(SampleStr->tcs_Node.mln_Succ);

        for (;;)
        {
          InstrStr = (struct TuComposerInstr *) InstrStr->tci_Node.mln_Pred;

          if (!( InstrStr->tci_Node.mln_Pred ))
            break;
        }

        (UBYTE *) InstrStr += -offsetof (struct TuComposerInstr, tci_Samples );

        if ( InstrStr->tci_Node.mln_Succ )
        {
          struct TuComposerModule *ModuleStr = (struct TuComposerModule *) &(InstrStr->tci_Node.mln_Succ);

          for (;;)
          {
            ModuleStr = (struct TuComposerModule *) ModuleStr->tcm_Node.mln_Pred;

            if (!( ModuleStr->tcm_Node.mln_Pred ))
              break;
          }

          (UBYTE *) ModuleStr += -offsetof (struct TuComposerModule, tcm_InstrList );

          InstrStr = (struct TuComposerInstr *) &(ModuleStr->tcm_InstrList.mlh_Head);

          for (;;)
          {
            struct TuComposerSample *OrigSampleStr;

            InstrStr = (struct TuComposerInstr *) InstrStr->tci_Node.mln_Succ;

            if (!( InstrStr->tci_Node.mln_Succ ))
              break;

            OrigSampleStr = (struct TuComposerSample *) &(InstrStr->tci_Samples.mlh_Head);

            for (;;)
            {
              OrigSampleStr = (struct TuComposerSample *) OrigSampleStr->tcs_Node.mln_Succ;

              if (!( OrigSampleStr->tcs_Node.mln_Succ ))
                break;

              if ( (OrigSampleStr->tcs_Flags & TCSF_Redirect) && (OrigSampleStr->tcs_Data == SampleStr->tcs_Data) )
              {
                OrigSampleStr->tcs_Samples *= 2;
                OrigSampleStr->tcs_Length = ((OrigSampleStr->tcs_Samples * OrigSampleStr->tcs_BitDepth) + SMP_BLOCKMASK) / SMP_BLOCKSIZE;

                OrigSampleStr->tcs_Repeat *= 2;
                OrigSampleStr->tcs_RepLen *= 2;

                OrigSampleStr->tcs_SustainRepeat *= 2;
                OrigSampleStr->tcs_SustainRepLen *= 2;
              }
            }
          }
        }
      }
    }

    return ( 0L );
  }

  return ( ERROR_SAMPLE_NOGIVEN );
}

void OctaveDown8 ( BYTE *SampleData, BYTE *NewSmpData, ULONG Samples )
{
  ULONG i;
  BYTE Sample;

  for (i = Samples; i > 0; i--)
  {
    Sample = *SampleData++;
    *NewSmpData++ = Sample;
    *NewSmpData++ = Sample;
  }
}

void OctaveDown16 ( WORD *SampleData, WORD *NewSmpData, ULONG Samples )
{
  ULONG i;
  WORD Sample;

  for (i = Samples; i > 0; i--)
  {
    Sample = *SampleData++;
    *NewSmpData++ = Sample;
    *NewSmpData++ = Sample;
  }
}

void OctaveDown32 ( LONG *SampleData, LONG *NewSmpData, ULONG Samples )
{
  ULONG i;
  LONG Sample;

  for (i = Samples; i > 0; i--)
  {
    Sample = *SampleData++;
    *NewSmpData++ = Sample;
    *NewSmpData++ = Sample;
  }
}

void OctaveDownX ( LONG *SampleData, LONG *NewSmpData, ULONG Samples, UBYTE BitDepth )
{
  ULONG i, SrcBit = 0L, DestBit = 0L;
  ULONG Sample;

  for (i = Samples; i > 0; i--)
  {
    if ( (SrcBit + BitDepth) < 32 )
    {
      Sample = (*SampleData << SrcBit) & ~((1UL << (32UL - BitDepth)) - 1UL);
    } else
    {
      Sample = (*SampleData++) << SrcBit;
      Sample |= (*SampleData & ~((1UL << (64UL - (SrcBit + BitDepth))) - 1UL)) >> (32UL - SrcBit);

      SrcBit -= 32;
    }

    SrcBit += BitDepth;

    Sample &= ~((1UL << (32UL - BitDepth)) - 1UL );

    *NewSmpData |= (Sample >> DestBit);

    if ( (DestBit + BitDepth) >= 32 )
    {
      NewSmpData++;
      *NewSmpData = (Sample << (32UL - DestBit));

      DestBit -= 32;
    }

    DestBit += BitDepth;

    *NewSmpData |= (Sample >> DestBit);

    if ( (DestBit + BitDepth) >= 32 )
    {
      NewSmpData++;
      *NewSmpData = (Sample << (32UL - DestBit));

      DestBit -= 32;
    }

    DestBit += BitDepth;
  }
}
