/* TuComposer OS independent code
**
** $VER: ReSample.c 31.22 (26.7.99)
**
** (C) 1999 by Sebastian Vater (known as Basty/Seasons).
**
*/

#include <stddef.h>
#include <exec/types.h>
#include <tucomposer/tucomposer.h>
#include <clib/tucomposer_protos.h>

#ifdef __cplusplus
extern "C" {
#endif

void ReSample8 ( BYTE *SampleData, BYTE *NewSmpData, ULONG Samples, ULONG OldFrequency, ULONG FreqDiv, ULONG FreqMod );
void ReSample16 ( WORD *SampleData, WORD *NewSmpData, ULONG Samples, ULONG OldFrequency, ULONG FreqDiv, ULONG FreqMod );
void ReSample32 ( LONG *SampleData, LONG *NewSmpData, ULONG Samples, ULONG OldFrequency, ULONG FreqDiv, ULONG FreqMod );
void ReSampleX ( LONG *SampleData, LONG *NewSmpData, ULONG Samples, ULONG OldFrequency, ULONG FreqDiv, ULONG FreqMod, UBYTE BitDepth );
void ReSample8I ( BYTE *SampleData, BYTE *NewSmpData, ULONG Samples, ULONG OldFrequency, ULONG FreqDiv, ULONG FreqMod );
void ReSample16I ( WORD *SampleData, WORD *NewSmpData, ULONG Samples, ULONG OldFrequency, ULONG FreqDiv, ULONG FreqMod );
void ReSample32I ( LONG *SampleData, LONG *NewSmpData, ULONG Samples, ULONG OldFrequency, ULONG FreqDiv, ULONG FreqMod );
void ReSampleXI ( LONG *SampleData, LONG *NewSmpData, ULONG Samples, ULONG OldFrequency, ULONG FreqDiv, ULONG FreqMod, UBYTE BitDepth );

#ifdef __cplusplus
}
#endif

ULONG ReSample ( struct TuComposerSample *SampleStr,
    unsigned long NewFrequency, unsigned long OldFrequency,
    unsigned long Flags )
{
  if ( SampleStr = FindOriginSample ( SampleStr ))
  {
    WORD *SampleData;
    WORD *NewSmpData;
    ULONG Samples, NewSamples, NewLength, FreqDiv, FreqMod;
    UBYTE BitDepth;

    if ( (SampleData = SampleStr->tcs_Data ))
    {
      BitDepth = SampleStr->tcs_BitDepth;

      Samples = SampleStr->tcs_Samples;
      NewSamples = (((unsigned long long) Samples * NewFrequency) + (OldFrequency - 1)) / OldFrequency;

      NewLength = ((NewSamples * BitDepth) + SMP_BLOCKMASK) / SMP_BLOCKSIZE;

      if ( ( NewSmpData = (WORD *) AllocSampleData ( NewLength )))
      {
        if ( OldFrequency == 0L )
          OldFrequency = SampleStr->tcs_Rate;

        FreqDiv = NewFrequency / OldFrequency;
        FreqMod = NewFrequency % OldFrequency;

        if ( Flags & RSF_Interpolation )
        {
          switch ( BitDepth )
          {
            case 8 :
            {
              ReSample8I ( (BYTE *) SampleData, (BYTE *) NewSmpData, Samples, OldFrequency, FreqDiv, FreqMod );

              break;
            }
            case 16 :
            {
              ReSample16I ( SampleData, NewSmpData, Samples, OldFrequency, FreqDiv, FreqMod );

              break;
            }
            case 32 :
            {
              ReSample32I ( (LONG *) SampleData, (LONG *) NewSmpData, Samples, OldFrequency, FreqDiv, FreqMod );

              break;
            }
            default :
            {
              ReSampleXI ( (LONG *) SampleData, (LONG *) NewSmpData, Samples, OldFrequency, FreqDiv, FreqMod, BitDepth );

              break;
            }
          }
        } else
        {
          switch ( BitDepth )
          {
            case 8 :
            {
              ReSample8 ( (BYTE *) SampleData, (BYTE *) NewSmpData, Samples, OldFrequency, FreqDiv, FreqMod );

              break;
            }
            case 16 :
            {
              ReSample16 ( SampleData, NewSmpData, Samples, OldFrequency, FreqDiv, FreqMod );

              break;
            }
            case 32 :
            {
              ReSample32 ( (LONG *) SampleData, (LONG *) NewSmpData, Samples, OldFrequency, FreqDiv, FreqMod );

              break;
            }
            default :
            {
              ReSampleX ( (LONG *) SampleData, (LONG *) NewSmpData, Samples, OldFrequency, FreqDiv, FreqMod, BitDepth );

              break;
            }
          }
        }

        FreeSampleData ( SampleStr->tcs_Data, SampleStr->tcs_Length );

        SampleStr->tcs_Data = NewSmpData;
        SampleStr->tcs_Length = NewLength;
        SampleStr->tcs_Samples = NewSamples;

        SampleStr->tcs_Repeat = (((unsigned long long) SampleStr->tcs_Repeat * NewFrequency) + ( OldFrequency - 1 )) / OldFrequency;
        SampleStr->tcs_RepLen = (((unsigned long long) SampleStr->tcs_RepLen * NewFrequency) + ( OldFrequency - 1 )) / OldFrequency;

        SampleStr->tcs_SustainRepeat = (((unsigned long long) SampleStr->tcs_SustainRepeat * NewFrequency) + ( OldFrequency - 1 )) / OldFrequency;
        SampleStr->tcs_SustainRepLen = (((unsigned long long) SampleStr->tcs_SustainRepLen * NewFrequency) + ( OldFrequency - 1 )) / OldFrequency;

        if ( SampleStr->tcs_Node.mln_Succ )
        {
          struct TuComposerInstr *InstrStr = (struct TuComposerInstr *) &(SampleStr->tcs_Node.mln_Succ);

          for (;;)
          {
            InstrStr = (struct TuComposerInstr *) InstrStr->tci_Node.mln_Pred;

            if (!( InstrStr->tci_Node.mln_Pred ))
              break;
          }

          (UBYTE *) InstrStr += -offsetof (struct TuComposerInstr, tci_Samples );

          if ( InstrStr->tci_Node.mln_Succ )
          {
            struct TuComposerModule *ModuleStr = (struct TuComposerModule *) &(InstrStr->tci_Node.mln_Succ);

            for (;;)
            {
              ModuleStr = (struct TuComposerModule *) ModuleStr->tcm_Node.mln_Pred;

              if (!( ModuleStr->tcm_Node.mln_Pred ))
                break;
            }

            (UBYTE *) ModuleStr += -offsetof (struct TuComposerModule, tcm_InstrList );

            InstrStr = (struct TuComposerInstr *) &(ModuleStr->tcm_InstrList.mlh_Head);

            for (;;)
            {
              struct TuComposerSample *OrigSampleStr;

              InstrStr = (struct TuComposerInstr *) InstrStr->tci_Node.mln_Succ;

              if (!( InstrStr->tci_Node.mln_Succ ))
                break;

              OrigSampleStr = (struct TuComposerSample *) &(InstrStr->tci_Samples.mlh_Head);

              for (;;)
              {
                OrigSampleStr = (struct TuComposerSample *) OrigSampleStr->tcs_Node.mln_Succ;

                if (!( OrigSampleStr->tcs_Node.mln_Succ ))
                  break;

                if ( (OrigSampleStr->tcs_Flags & TCSF_Redirect) && (OrigSampleStr->tcs_Data == SampleStr->tcs_Data) )
                {
                  OrigSampleStr->tcs_Samples = (((unsigned long long) OrigSampleStr->tcs_Samples * NewFrequency) + ( OldFrequency - 1 )) / OldFrequency;
                  OrigSampleStr->tcs_Length = ((OrigSampleStr->tcs_Samples * OrigSampleStr->tcs_BitDepth) + SMP_BLOCKMASK) / SMP_BLOCKSIZE;

                  OrigSampleStr->tcs_Repeat = (((unsigned long long) OrigSampleStr->tcs_Repeat * NewFrequency) + ( OldFrequency - 1 )) / OldFrequency;
                  OrigSampleStr->tcs_RepLen = (((unsigned long long) OrigSampleStr->tcs_RepLen * NewFrequency) + ( OldFrequency - 1 )) / OldFrequency;

                  OrigSampleStr->tcs_SustainRepeat = (((unsigned long long) OrigSampleStr->tcs_SustainRepeat * NewFrequency) + ( OldFrequency - 1 )) / OldFrequency;
                  OrigSampleStr->tcs_SustainRepLen = (((unsigned long long) OrigSampleStr->tcs_SustainRepLen * NewFrequency) + ( OldFrequency - 1 )) / OldFrequency;
                }
              }
            }
          }
        }

        return ( 0L );
      }

      return ( ERROR_SAMPLE_NOMEM );
    }

    return ( ERROR_SAMPLE_NODATA );
  }

  return ( ERROR_SAMPLE_NOGIVEN );
}

void ReSample8 ( BYTE *SampleData, BYTE *NewSmpData, ULONG Samples, ULONG OldFrequency, ULONG FreqDiv, ULONG FreqMod )
{
  ULONG i, j, FreqCnt = 0L, FreqSub;
  BYTE Sample = *SampleData;

  for (i = Samples; i > 0; i--)
  {
    FreqSub = FreqDiv;

    FreqCnt += FreqMod;
    if ( FreqCnt >= OldFrequency )
    {
      FreqCnt -= OldFrequency;
      FreqSub++;
    }

    SampleData++;

    if ( FreqSub )
    {
      for (j = FreqSub; j > 0; j--)
      {
        *NewSmpData++ = Sample;
      }

      Sample = *SampleData;
    }
  }
}

void ReSample16 ( WORD *SampleData, WORD *NewSmpData, ULONG Samples, ULONG OldFrequency, ULONG FreqDiv, ULONG FreqMod )
{
  ULONG i, j, FreqCnt = 0L, FreqSub;
  WORD Sample = *SampleData;

  for (i = Samples; i > 0; i--)
  {
    FreqSub = FreqDiv;

    FreqCnt += FreqMod;
    if ( FreqCnt >= OldFrequency )
    {
      FreqCnt -= OldFrequency;
      FreqSub++;
    }

    SampleData++;

    if ( FreqSub )
    {
      for (j = FreqSub; j > 0; j--)
      {
        *NewSmpData++ = Sample;
      }

      Sample = *SampleData;
    }
  }
}

void ReSample32 ( LONG *SampleData, LONG *NewSmpData, ULONG Samples, ULONG OldFrequency, ULONG FreqDiv, ULONG FreqMod )
{
  ULONG i, j, FreqCnt = 0L, FreqSub;
  LONG Sample = *SampleData;

  for (i = Samples; i > 0; i--)
  {
    FreqSub = FreqDiv;

    FreqCnt += FreqMod;
    if ( FreqCnt >= OldFrequency )
    {
      FreqCnt -= OldFrequency;
      FreqSub++;
    }

    SampleData++;

    if ( FreqSub )
    {
      for (j = FreqSub; j > 0; j--)
      {
        *NewSmpData++ = Sample;
      }

      Sample = *SampleData;
    }
  }
}

void ReSampleX ( LONG *SampleData, LONG *NewSmpData, ULONG Samples, ULONG OldFrequency, ULONG FreqDiv, ULONG FreqMod, UBYTE BitDepth )
{
  ULONG i, j, FreqCnt = 0L, FreqSub, SrcBit = 0L, DestBit = 0L;
  ULONG Sample = (*SampleData) & ~((1UL << (32UL - BitDepth)) - 1UL);

  for (i = Samples; i > 0; i--)
  {
    FreqSub = FreqDiv;

    FreqCnt += FreqMod;
    if ( FreqCnt >= OldFrequency )
    {
      FreqCnt -= OldFrequency;
      FreqSub++;
    }

    if ( (SrcBit + BitDepth) >= 32 )
    {
      SampleData++;

      SrcBit -= 32;
    }

    SrcBit += BitDepth;

    if ( FreqSub )
    {
      Sample &= ~((1UL << (32UL - BitDepth)) - 1UL );

      for (j = FreqSub; j > 0; j--)
      {
        *NewSmpData |= (Sample >> DestBit);

        if ( (DestBit + BitDepth) >= 32 )
        {
          NewSmpData++;
          *NewSmpData = (Sample << (32UL - DestBit));

          DestBit -= 32;
        }

        DestBit += BitDepth;
      }

      if ( (SrcBit + BitDepth) < 32 )
      {
        Sample = (*SampleData << SrcBit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        Sample = *SampleData << SrcBit;
        Sample |= (*(SampleData + 1) & ~((1UL << (64UL - (SrcBit + BitDepth))) - 1UL)) >> (32UL - SrcBit);
      }
    }
  }
}

void ReSample8I ( BYTE *SampleData, BYTE *NewSmpData, ULONG Samples, ULONG OldFrequency, ULONG FreqDiv, ULONG FreqMod )
{
  ULONG i, j, FreqCnt = 0L, FreqSub, InterpolDiv = 0L;
  LONG Sample = 0L, MixSmp;

  for (i = Samples; i > 0; i--)
  {
    MixSmp = *SampleData++;
    Sample += MixSmp;
    InterpolDiv++;

    FreqSub = FreqDiv;

    FreqCnt += FreqMod;
    if ( FreqCnt >= OldFrequency )
    {
      FreqCnt -= OldFrequency;
      FreqSub++;
    }

    if ( FreqSub )
    {
      Sample /= InterpolDiv;

      for (j = FreqSub; j > 0; j--)
      {
        *NewSmpData++ = Sample;
      }

      Sample = 0;

      InterpolDiv = 0L;
    }
  }
}

void ReSample16I ( WORD *SampleData, WORD *NewSmpData, ULONG Samples, ULONG OldFrequency, ULONG FreqDiv, ULONG FreqMod )
{
  ULONG i, j, FreqCnt = 0L, FreqSub, InterpolDiv = 0L;
  LONG Sample = 0L, MixSmp;

  for (i = Samples; i > 0; i--)
  {
    MixSmp = *SampleData++;
    Sample += MixSmp;
    InterpolDiv++;

    FreqSub = FreqDiv;

    FreqCnt += FreqMod;
    if ( FreqCnt >= OldFrequency )
    {
      FreqCnt -= OldFrequency;
      FreqSub++;
    }

    if ( FreqSub )
    {
      Sample /= InterpolDiv;

      for (j = FreqSub; j > 0; j--)
      {
        *NewSmpData++ = Sample;
      }

      Sample = 0L;

      InterpolDiv = 0L;
    }
  }
}

void ReSample32I ( LONG *SampleData, LONG *NewSmpData, ULONG Samples, ULONG OldFrequency, ULONG FreqDiv, ULONG FreqMod )
{
  ULONG i, j, FreqCnt = 0L, FreqSub, InterpolDiv = 0L;
  long long Sample = 0LL, MixSmp;

  for (i = Samples; i > 0; i--)
  {
    MixSmp = *SampleData++;
    Sample += MixSmp;
    InterpolDiv++;

    FreqSub = FreqDiv;

    FreqCnt += FreqMod;
    if ( FreqCnt >= OldFrequency )
    {
      FreqCnt -= OldFrequency;
      FreqSub++;
    }

    if ( FreqSub )
    {
      Sample /= InterpolDiv;

      for (j = FreqSub; j > 0; j--)
      {
        *NewSmpData++ = Sample;
      }

      Sample = 0LL;

      InterpolDiv = 0L;
    }
  }
}

void ReSampleXI ( LONG *SampleData, LONG *NewSmpData, ULONG Samples, ULONG OldFrequency, ULONG FreqDiv, ULONG FreqMod, UBYTE BitDepth )
{
  ULONG i, j, FreqCnt = 0L, FreqSub, SrcBit = 0L, DestBit = 0L, SrcSample, InterpolDiv = 0L;
  long long Sample = 0LL, MixSmp;

  for (i = Samples; i > 0; i--)
  {
    if ( (SrcBit + BitDepth) < 32 )
    {
      SrcSample = (*SampleData << SrcBit) & ~((1UL << (32UL - BitDepth)) - 1UL);
    } else
    {
      SrcSample = *SampleData++ << SrcBit;
      SrcSample |= (*SampleData & ~((1UL << (64UL - (SrcBit + BitDepth))) - 1UL)) >> (32UL - SrcBit);

      SrcBit -= 32;
    }

    SrcBit += BitDepth;

    MixSmp = SrcSample;

    Sample += MixSmp;
    InterpolDiv++;

    FreqSub = FreqDiv;

    FreqCnt += FreqMod;
    if ( FreqCnt >= OldFrequency )
    {
      FreqCnt -= OldFrequency;
      FreqSub++;
    }

    if ( FreqSub )
    {
      Sample /= InterpolDiv;

      SrcSample = Sample;
      SrcSample &= ~((1UL << (32UL - BitDepth)) - 1UL );

      for (j = FreqSub; j > 0; j--)
      {
        *NewSmpData |= (SrcSample >> DestBit);

        if ( (DestBit + BitDepth) >= 32 )
        {
          NewSmpData++;
          *NewSmpData = (SrcSample << (32UL - DestBit));

          DestBit -= 32;
        }

        DestBit += BitDepth;
      }

      Sample = 0LL;

      InterpolDiv = 0L;
    }
  }
}
