/* TuComposer OS independent code
**
** $VER: CalcSongPlayTime.c 31.22 (19.7.99)
**
** (C) 1999 by Sebastian Vater (known as Basty/Seasons).
**
*/

#include <string.h>
#include <exec/types.h>
#include <tucomposer/tucomposer.h>
#include <tucomposer/external/external.h>
#include <tucomposer/external/mixer.h>
#include <clib/tucomposer_protos.h>

#define COUNT_MIXRATE 64000

unsigned long long CalcSongPlayTimeULL ( struct TuComposerSong *SongStr, struct PlayExecHandler *PlayExecStr );

struct ChannelInfo {
  struct ChannelBlock {
    WORD *chi_SampleStartAdr;
    ULONG chi_SampleLen;
    ULONG chi_Offset;
    ULONG chi_Fraction;
    ULONG chi_Advance;
    ULONG chi_AdvFract;
    void (*chi_MixRoutine)( ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, UWORD Length );
    ULONG chi_EndOffset;
    ULONG chi_RestartOffset;
    ULONG chi_Repeat;
    ULONG chi_RepLen;
    ULONG chi_CountRestart;
    ULONG chi_Counted;
    ULONG chi_SampleRate;
    void (*chi_AltMixRoutine)( ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, UWORD Length );
    UBYTE chi_BitDepth;
    UBYTE chi_Flags;
    UBYTE chi_Volume;
    UBYTE chi_Panning;
  } chi_Mix;
  struct ChannelBlock chi_New;
}; /* ChannelInfo */

struct CountMixerAllocStruct {
  struct MixerAllocStruct cntmas_AllocMixStruct;
  struct ChannelInfo *cntmas_ChInfo;
  ULONG  cntmas_Tempo;
  ULONG  cntmas_PassLength;
  ULONG  cntmas_PassFrac;
  ULONG  cntmas_CurrLeftFrac;
  UWORD  cntmas_Channels;
}; /* CountMixerAllocStruct */

#ifdef __cplusplus
extern "C" {
#endif

static BPTR CountExtLib ( void );
static ULONG ChangeCountMixTempo ( struct MixerAllocStruct *MixAllocStr,
    unsigned long NewTempo );
static void GetCountChannel ( struct MixerAllocStruct *MixAllocStr,
    struct MixerStruct *MixStr, unsigned long Channel );
static void SetCountChannel ( struct MixerAllocStruct *MixAllocStr,
    struct MixerStruct *MixStr, unsigned long Channel );
static void SetCountChannelVol ( struct MixerAllocStruct
    *MixAllocStr, struct MixerStruct *MixStr, unsigned long Channel );
static void SetCountChannelPos ( struct MixerAllocStruct
    *MixAllocStr, struct MixerStruct *MixStr, unsigned long Channel );
static void CountMix ( struct CountMixerAllocStruct *CntMixAllocStr );

static void SetSmpMixRate ( struct CountMixerAllocStruct *CntMixAllocStr, struct ChannelInfo *ChInfo, ULONG SampleRate );
static void AdvOk ( struct CountMixerAllocStruct *CntMixAllocStr, struct ChannelInfo *ChInfo );

static void Skip ( ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void Skipb ( ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );

#ifdef __cplusplus
}
#endif

extern APTR CountMixerFuncList [];

#include "CalcSongPlayTime_library.h"

ULONG CalcSongPlayTime ( struct TuComposerSong *SongStr,
    struct PlayExecHandler *PlayExecStr )
{
  return ( (CalcSongPlayTimeULL ( SongStr, PlayExecStr ) >> 32) );
}

unsigned long long CalcSongPlayTimeULL ( struct TuComposerSong *SongStr, struct PlayExecHandler *PlayExecStr )
{
  struct TuComposerBase *TuCBase;
  struct PlayerGlobals *PGStr;
  unsigned long long TotalTime;
  ULONG PlayTime = 0L;
  LONG PlayTimeFrac;

  if ( SongStr )
  {
    if ( (TuCBase = AllocSimulateMix ( SongStr )))
    {
      if ( PlayExecStr )
        EnqueueNode ( (struct List *) &(TuCBase->tcb_ExecList), (struct Node *) PlayExecStr );

      TuCBase->tcb_PlayMixerAlloc->mas_MixLibBase->mix_Library.extb_TuComposerBase = TuCBase;

      PGStr = TuCBase->tcb_PlaySong->tcso_GlobalData;

      for (;;)
      {
        PlayTime = PGStr->pg_PlayTime;
        PlayTimeFrac = PGStr->pg_PlayTimeFrac;

        CountMix ( (struct CountMixerAllocStruct *) TuCBase->tcb_PlayMixerAlloc );

        if ( PGStr->pg_PlayTime >= 100*60*60*100 )
        {
          PlayTime = 0L;
          PlayTimeFrac = 0L;
          break;
        }

        if ( PGStr->pg_Flags & PGF_SongEnd )
          break;
      }

      FreeSimulateMix ( TuCBase );

      TotalTime = ((unsigned long long) PlayTime << 32) + (ULONG) PlayTimeFrac;

      if ( PlayTimeFrac < 0 )
        PlayTime++;

      SongStr->tcso_PlayTimeHours = (PlayTime / 360000);
      SongStr->tcso_PlayTimeMins = (PlayTime / 6000) % 60;
      SongStr->tcso_PlayTimeSecs = (PlayTime / 100) % 60;
      SongStr->tcso_PlayTimeHundreds = PlayTime % 100;
    }
  }

  return ( TotalTime );
}

APTR CountMixerFuncList[] =
{
  CountExtLib,
  CountExtLib,
  CountExtLib,
  CountExtLib,
  ChangeCountMixTempo,
  CountExtLib,
  GetCountChannel,
  SetCountChannel,
  SetCountChannelVol,
  SetCountChannelPos,
  CountMix
};

static BPTR CountExtLib ( void )
{
  return ( NULL );
}

static ULONG ChangeCountMixTempo ( struct MixerAllocStruct *MixAllocStr,
    unsigned long NewTempo )
{
  struct CountMixerAllocStruct *CntMixAllocStr = (struct CountMixerAllocStruct *) MixAllocStr;
  unsigned long long PassValue;

  CntMixAllocStr->cntmas_AllocMixStruct.mas_Tempo = NewTempo;

  PassValue = ((unsigned long long) COUNT_MIXRATE * 655360ULL);

  CntMixAllocStr->cntmas_PassLength = (unsigned long long) PassValue / CntMixAllocStr->cntmas_AllocMixStruct.mas_Tempo;
  CntMixAllocStr->cntmas_PassFrac = (((unsigned long long) PassValue % CntMixAllocStr->cntmas_AllocMixStruct.mas_Tempo) << 32) / CntMixAllocStr->cntmas_AllocMixStruct.mas_Tempo;

  return ( 0L );
}

static void GetCountChannel ( struct MixerAllocStruct *MixAllocStr,
    struct MixerStruct *MixStr, unsigned long Channel )
{
  struct CountMixerAllocStruct *CntMixAllocStr = (struct CountMixerAllocStruct *) MixAllocStr;
  struct ChannelInfo *ChInfo;

  if ( Channel < CntMixAllocStr->cntmas_Channels )
  {
    ChInfo = CntMixAllocStr->cntmas_ChInfo + Channel;

    MixStr->ms_SamplePos = ChInfo->chi_Mix.chi_Offset;

    MixStr->ms_BitDepth = ChInfo->chi_Mix.chi_BitDepth;
    MixStr->ms_Flags = ChInfo->chi_Mix.chi_Flags;
    MixStr->ms_Volume = ChInfo->chi_Mix.chi_Volume;
    MixStr->ms_Panning = ChInfo->chi_Mix.chi_Panning;

    MixStr->ms_SampleData = ChInfo->chi_Mix.chi_SampleStartAdr;
    MixStr->ms_SampleLen = ChInfo->chi_Mix.chi_SampleLen;

    MixStr->ms_Repeat = ChInfo->chi_Mix.chi_Repeat;
    MixStr->ms_RepLen = ChInfo->chi_Mix.chi_RepLen;

    MixStr->ms_RepCount = ChInfo->chi_Mix.chi_CountRestart;
    MixStr->ms_RepCounted = ChInfo->chi_Mix.chi_Counted;

    MixStr->ms_SampleRate = ChInfo->chi_Mix.chi_SampleRate;
  }
}

static void SetCountChannel ( struct MixerAllocStruct *MixAllocStr,
    struct MixerStruct *MixStr, unsigned long Channel )
{
  struct CountMixerAllocStruct *CntMixAllocStr = (struct CountMixerAllocStruct *) MixAllocStr;
  struct ChannelInfo *ChInfo;

  if ( Channel < CntMixAllocStr->cntmas_Channels )
  {
    ULONG Repeat, RepLen;

    ChInfo = CntMixAllocStr->cntmas_ChInfo + Channel;

    ChInfo->chi_New.chi_SampleStartAdr = NULL;

    if ( MixStr->ms_Flags & MSF_Synth )
    {
      ChInfo->chi_Mix.chi_Flags |= MSF_Synth;

      (BYTE *) ChInfo += sizeof (struct ChannelBlock);
    }

    ChInfo->chi_Mix.chi_Offset = MixStr->ms_SamplePos;
    ChInfo->chi_Mix.chi_Fraction = 0L;

    ChInfo->chi_Mix.chi_BitDepth = MixStr->ms_BitDepth;
    ChInfo->chi_Mix.chi_Flags = MixStr->ms_Flags;
    ChInfo->chi_Mix.chi_Volume = MixStr->ms_Volume;
    ChInfo->chi_Mix.chi_Panning = MixStr->ms_Panning;

    ChInfo->chi_Mix.chi_SampleStartAdr = MixStr->ms_SampleData;
    ChInfo->chi_Mix.chi_SampleLen = MixStr->ms_SampleLen;

    Repeat = MixStr->ms_Repeat;
    RepLen = MixStr->ms_RepLen;

    ChInfo->chi_Mix.chi_Repeat = Repeat;
    ChInfo->chi_Mix.chi_RepLen = RepLen;

    if (!( ChInfo->chi_Mix.chi_Flags & MSF_Loop ))
    {
      Repeat = MixStr->ms_SampleLen;
      RepLen = 0L;
    }

    Repeat += RepLen;

    if ( ChInfo->chi_Mix.chi_Flags & MSF_Backwards )
    {
      Repeat -= RepLen;

      if (!( ChInfo->chi_Mix.chi_Flags & MSF_Loop ))
        Repeat = -1L;
    }

    ChInfo->chi_Mix.chi_EndOffset = Repeat;
    ChInfo->chi_Mix.chi_RestartOffset = RepLen;

    ChInfo->chi_Mix.chi_CountRestart = MixStr->ms_RepCount;
    ChInfo->chi_Mix.chi_Counted = MixStr->ms_RepCounted;

    SetSmpMixRate ( CntMixAllocStr, ChInfo, MixStr->ms_SampleRate );
  }
}

static void SetCountChannelVol ( struct MixerAllocStruct
    *MixAllocStr, struct MixerStruct *MixStr, unsigned long Channel )
{
  struct CountMixerAllocStruct *CntMixAllocStr = (struct CountMixerAllocStruct *) MixAllocStr;
  struct ChannelInfo *ChInfo;

  if ( Channel < CntMixAllocStr->cntmas_Channels )
  {
    ChInfo = CntMixAllocStr->cntmas_ChInfo + Channel;

    if ( (ChInfo->chi_Mix.chi_Volume == MixStr->ms_Volume) && (ChInfo->chi_Mix.chi_Panning == MixStr->ms_Panning) )
    {
      ULONG SmpRate = MixStr->ms_SampleRate, TmpStore;

      ChInfo->chi_Mix.chi_SampleRate = SmpRate;
      ChInfo->chi_New.chi_SampleRate = SmpRate;

      TmpStore = SmpRate / COUNT_MIXRATE;

      ChInfo->chi_Mix.chi_Advance = TmpStore;
      ChInfo->chi_New.chi_Advance = TmpStore;

      TmpStore = (((unsigned long long) SmpRate % COUNT_MIXRATE) << 32) / COUNT_MIXRATE;

      ChInfo->chi_Mix.chi_AdvFract = TmpStore;
      ChInfo->chi_New.chi_AdvFract = TmpStore;
    } else
    {
      ULONG SmpRate = MixStr->ms_SampleRate, TmpStore;
      UBYTE Volume = MixStr->ms_Volume;
      BYTE Panning = MixStr->ms_Panning;

      ChInfo->chi_Mix.chi_Volume = Volume;
      ChInfo->chi_New.chi_Volume = Volume;

      ChInfo->chi_Mix.chi_Panning = Panning;
      ChInfo->chi_New.chi_Panning = Panning;

      ChInfo->chi_Mix.chi_SampleRate = SmpRate;
      ChInfo->chi_New.chi_SampleRate = SmpRate;

      TmpStore = SmpRate / COUNT_MIXRATE;

      ChInfo->chi_Mix.chi_Advance = TmpStore;
      ChInfo->chi_New.chi_Advance = TmpStore;

      TmpStore = (((unsigned long long) SmpRate % COUNT_MIXRATE) << 32) / COUNT_MIXRATE;

      ChInfo->chi_Mix.chi_AdvFract = TmpStore;
      ChInfo->chi_New.chi_AdvFract = TmpStore;

      AdvOk ( CntMixAllocStr, ChInfo );

      AdvOk ( CntMixAllocStr, (struct ChannelInfo *) &(ChInfo->chi_New) );
    }
  }
}

static void SetCountChannelPos ( struct MixerAllocStruct
    *MixAllocStr, struct MixerStruct *MixStr, unsigned long Channel )
{
  struct CountMixerAllocStruct *CntMixAllocStr = (struct CountMixerAllocStruct *) MixAllocStr;
  struct ChannelInfo *ChInfo;

  if ( Channel < CntMixAllocStr->cntmas_Channels )
  {
    ChInfo = CntMixAllocStr->cntmas_ChInfo + Channel;

    if ( ChInfo->chi_Mix.chi_Flags == MixStr->ms_Flags )
    {
      ULONG Repeat, RepLen;

      Repeat = MixStr->ms_SamplePos;
      if ( Repeat != ChInfo->chi_Mix.chi_Offset )
      {
        ChInfo->chi_Mix.chi_Offset = Repeat;
        ChInfo->chi_Mix.chi_Fraction = 0L;
      }

      Repeat = MixStr->ms_Repeat;
      RepLen = MixStr->ms_RepLen;

      ChInfo->chi_Mix.chi_Repeat = Repeat;
      ChInfo->chi_Mix.chi_RepLen = RepLen;

      if (!( ChInfo->chi_Mix.chi_Flags & MSF_Loop ))
      {
        Repeat = MixStr->ms_SampleLen;
        RepLen = 0L;
      }

      Repeat += RepLen;

      if ( ChInfo->chi_Mix.chi_Flags & MSF_Backwards )
      {
        Repeat -= RepLen;

        if (!( ChInfo->chi_Mix.chi_Flags & MSF_Loop ))
          Repeat = -1L;
      }

      ChInfo->chi_Mix.chi_EndOffset = Repeat;
      ChInfo->chi_Mix.chi_RestartOffset = RepLen;

      ChInfo->chi_Mix.chi_CountRestart = MixStr->ms_RepCount;
      ChInfo->chi_Mix.chi_Counted = MixStr->ms_RepCounted;
    } else
    {
      ULONG Repeat, RepLen;

      ChInfo->chi_Mix.chi_Flags = MixStr->ms_Flags;

      Repeat = MixStr->ms_SamplePos;
      if ( Repeat != ChInfo->chi_Mix.chi_Offset )
      {
        ChInfo->chi_Mix.chi_Offset = Repeat;
        ChInfo->chi_Mix.chi_Fraction = 0L;
      }

      Repeat = MixStr->ms_Repeat;
      RepLen = MixStr->ms_RepLen;

      ChInfo->chi_Mix.chi_Repeat = Repeat;
      ChInfo->chi_Mix.chi_RepLen = RepLen;

      if (!( ChInfo->chi_Mix.chi_Flags & MSF_Loop ))
      {
        Repeat = MixStr->ms_SampleLen;
        RepLen = 0L;
      }

      Repeat += RepLen;

      if ( ChInfo->chi_Mix.chi_Flags & MSF_Backwards )
      {
        Repeat -= RepLen;

        if (!( ChInfo->chi_Mix.chi_Flags & MSF_Loop ))
          Repeat = -1L;
      }

      ChInfo->chi_Mix.chi_EndOffset = Repeat;
      ChInfo->chi_Mix.chi_RestartOffset = RepLen;

      ChInfo->chi_Mix.chi_CountRestart = MixStr->ms_RepCount;
      ChInfo->chi_Mix.chi_Counted = MixStr->ms_RepCounted;

      AdvOk ( CntMixAllocStr, ChInfo );
    }
  }
}

static void CountMix (struct CountMixerAllocStruct *CntMixAllocStr )
{
  struct ChannelInfo *ChInfo;
  UWORD Channels;
  ULONG Length, MixFrac, MixPassFrac;

  DoPlayerIRQ ( CntMixAllocStr->cntmas_AllocMixStruct.mas_MixLibBase->mix_Library.extb_TuComposerBase );

  Length = CntMixAllocStr->cntmas_PassLength;
  MixFrac = CntMixAllocStr->cntmas_CurrLeftFrac;
  MixPassFrac = CntMixAllocStr->cntmas_PassFrac;

  if ( (MixFrac += MixPassFrac ) < MixPassFrac )
    Length++;

  CntMixAllocStr->cntmas_CurrLeftFrac = MixFrac;

  ChInfo = CntMixAllocStr->cntmas_ChInfo;

  for (Channels = CntMixAllocStr->cntmas_Channels; Channels > 0; Channels--)
  {
    if ( ChInfo->chi_Mix.chi_Flags & MSF_Play )
    {
      void (*MixRoutine)( ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
      ULONG Offset = ChInfo->chi_Mix.chi_Offset;
      ULONG Fraction = ChInfo->chi_Mix.chi_Fraction;
      ULONG Advance = ChInfo->chi_Mix.chi_Advance;
      ULONG AdvFract = ChInfo->chi_Mix.chi_AdvFract;
      ULONG RemainLen = Length, RemainMix;
      ULONG Counted;
      ULONG CountRestart;
      unsigned long long CalcMix;

      (APTR) MixRoutine = (APTR) ChInfo->chi_Mix.chi_MixRoutine;

      if ( ChInfo->chi_Mix.chi_Flags & MSF_Backwards )
      {
MixSmp_Backwards:

        for (;;)
        {
          CalcMix = (((((unsigned long long) Advance << 32) + AdvFract) * RemainLen) + Fraction) >> 32;

          if ( (LONG) (RemainMix = Offset - ChInfo->chi_Mix.chi_EndOffset) > 0 )
          {
            if ( (ULONG) CalcMix < RemainMix )
            {
              MixRoutine ( (ULONG *) &Offset, (ULONG *) &Fraction, Advance, AdvFract, RemainLen );

              if ( (LONG) Offset <= (LONG) ChInfo->chi_Mix.chi_EndOffset )
              {
                RemainLen = 0L;
              } else
              {
                break;
              }
            } else
            {
              CalcMix = (((((unsigned long long) RemainMix << 32) - Fraction) - 1) / (((unsigned long long) Advance << 32) + AdvFract) + 1);

              RemainLen -= (ULONG) CalcMix;

              MixRoutine ( (ULONG *) &Offset, (ULONG *) &Fraction, Advance, AdvFract, (ULONG) CalcMix );

              if ( (LONG) Offset > (LONG) ChInfo->chi_Mix.chi_EndOffset )
              {
                if (!( RemainLen ))
                  break;
              }
            }
          }

          if ( ChInfo->chi_Mix.chi_Flags & MSF_Loop )
          {
            Counted = ChInfo->chi_Mix.chi_Counted++;

            if ( (CountRestart = ChInfo->chi_Mix.chi_CountRestart) && (CountRestart == Counted) )
            {
              ChInfo->chi_Mix.chi_Flags &= ~MSF_Loop;

              ChInfo->chi_Mix.chi_EndOffset = -1L;

              goto MixSmp_Synth;
            } else
            {
              if ( ChInfo->chi_Mix.chi_Flags & MSF_Pingpong )
              {
                APTR ChangeMixRoutine;

                if ( ChInfo->chi_New.chi_SampleStartAdr )
                {
                  memcpy ( ChInfo, &(ChInfo->chi_New), sizeof (struct ChannelBlock ));

                  ChInfo->chi_New.chi_SampleStartAdr = NULL;
                }

                (APTR) ChangeMixRoutine = (APTR) ChInfo->chi_Mix.chi_AltMixRoutine;
                (APTR) ChInfo->chi_Mix.chi_AltMixRoutine = MixRoutine;

                (APTR) MixRoutine = (APTR) ChangeMixRoutine;
                (APTR) ChInfo->chi_Mix.chi_MixRoutine = (APTR) MixRoutine;

                ChInfo->chi_Mix.chi_Flags ^= MSF_Backwards;

                RemainMix = ChInfo->chi_Mix.chi_EndOffset;

                Offset -= RemainMix;
                Offset = -Offset + RemainMix;

                RemainMix += ChInfo->chi_Mix.chi_RestartOffset;
                ChInfo->chi_Mix.chi_EndOffset = RemainMix;

                if ( (LONG) RemainLen > 0 )
                  goto MixSmp_Forwards;

                break;
              } else
              {
                Offset += ChInfo->chi_Mix.chi_RestartOffset;

                if ( ChInfo->chi_New.chi_SampleStartAdr )
                  goto MixSmp_Synth;

                if ( (LONG) RemainLen > 0 )
                  continue;

                break;
              }
            }
          } else
          {
            if ( ChInfo->chi_New.chi_SampleStartAdr )
            {
              goto MixSmp_Synth;
            } else
            {
              ChInfo->chi_Mix.chi_Flags &= ~MSF_Play;
            }

            break;
          }
        }
      } else
      {
MixSmp_Forwards:

        for (;;)
        {
          CalcMix = (((((unsigned long long) Advance << 32) + AdvFract) * RemainLen) + Fraction) >> 32;

          if ( (LONG) (RemainMix = ChInfo->chi_Mix.chi_EndOffset - Offset) > 0 )
          {
            if ( (ULONG) CalcMix < RemainMix )
            {
              MixRoutine ( (ULONG *) &Offset, (ULONG *) &Fraction, Advance, AdvFract, RemainLen );

              if ( Offset >= ChInfo->chi_Mix.chi_EndOffset )
              {
                RemainLen = 0L;
              } else
              {
                break;
              }
            } else
            {
              CalcMix = (((((unsigned long long) RemainMix << 32) - Fraction) - 1) / (((unsigned long long) Advance << 32) + AdvFract) + 1);

              RemainLen -= (ULONG) CalcMix;

              MixRoutine ( (ULONG *) &Offset, (ULONG *) &Fraction, Advance, AdvFract, (ULONG) CalcMix );

              if ( Offset < ChInfo->chi_Mix.chi_EndOffset )
              {
                if (!( RemainLen ))
                  break;
              }
            }
          }

          if ( ChInfo->chi_Mix.chi_Flags & MSF_Loop )
          {
            Counted = ChInfo->chi_Mix.chi_Counted++;

            if ( (CountRestart = ChInfo->chi_Mix.chi_CountRestart) && (CountRestart == Counted) )
            {
              ChInfo->chi_Mix.chi_Flags &= ~MSF_Loop;

              ChInfo->chi_Mix.chi_EndOffset = ChInfo->chi_Mix.chi_SampleLen;

              goto MixSmp_Synth;
            } else
            {
              if ( ChInfo->chi_Mix.chi_Flags & MSF_Pingpong )
              {
                APTR ChangeMixRoutine;

                if ( ChInfo->chi_New.chi_SampleStartAdr )
                {
                  memcpy ( ChInfo, &(ChInfo->chi_New), sizeof (struct ChannelBlock ));

                  ChInfo->chi_New.chi_SampleStartAdr = NULL;
                }

                (APTR) ChangeMixRoutine = (APTR) ChInfo->chi_Mix.chi_AltMixRoutine;
                (APTR) ChInfo->chi_Mix.chi_AltMixRoutine = MixRoutine;

                (APTR) MixRoutine = (APTR) ChangeMixRoutine;
                (APTR) ChInfo->chi_Mix.chi_MixRoutine = (APTR) MixRoutine;

                ChInfo->chi_Mix.chi_Flags ^= MSF_Backwards;

                RemainMix = ChInfo->chi_Mix.chi_EndOffset;

                Offset -= RemainMix;
                Offset = -Offset + RemainMix;

                RemainMix -= ChInfo->chi_Mix.chi_RestartOffset;
                ChInfo->chi_Mix.chi_EndOffset = RemainMix;

                if ( RemainLen )
                  goto MixSmp_Backwards;

                break;
              } else
              {
                Offset -= ChInfo->chi_Mix.chi_RestartOffset;

                if ( ChInfo->chi_New.chi_SampleStartAdr )
                {
                  memcpy ( ChInfo, &(ChInfo->chi_New), sizeof (struct ChannelBlock ));

                  ChInfo->chi_New.chi_SampleStartAdr = NULL;
                }

                if ( (LONG) RemainLen > 0 )
                  continue;

                break;
              }
            }
          } else
          {
            if ( ChInfo->chi_New.chi_SampleStartAdr )
            {
MixSmp_Synth:
              memcpy ( ChInfo, &(ChInfo->chi_New), sizeof (struct ChannelBlock ));

              ChInfo->chi_New.chi_SampleStartAdr = NULL;

              if ( (LONG) RemainLen > 0 )
                continue;

            } else
            {
              ChInfo->chi_Mix.chi_Flags &= ~MSF_Play;
            }

            break;
          }
        }
      }

      ChInfo->chi_Mix.chi_Offset = Offset;
      ChInfo->chi_Mix.chi_Fraction = Fraction;
    }

    ChInfo++;
  }
}

static void SetSmpMixRate ( struct CountMixerAllocStruct *CntMixAllocStr, struct ChannelInfo *ChInfo, unsigned long SampleRate )
{
  ChInfo->chi_Mix.chi_SampleRate = SampleRate;

  ChInfo->chi_Mix.chi_Advance = SampleRate / COUNT_MIXRATE;
  ChInfo->chi_Mix.chi_AdvFract = (((unsigned long long) SampleRate % COUNT_MIXRATE) << 32) / COUNT_MIXRATE;

  AdvOk ( CntMixAllocStr, ChInfo );
}

static void AdvOk ( struct CountMixerAllocStruct *CntMixAllocStr, struct ChannelInfo *ChInfo )
{
  if ( ChInfo->chi_Mix.chi_Flags & MSF_Backwards )
  {
    (APTR) ChInfo->chi_Mix.chi_MixRoutine = (APTR) Skipb;
    (APTR) ChInfo->chi_Mix.chi_AltMixRoutine = (APTR) Skip;
  } else
  {
    (APTR) ChInfo->chi_Mix.chi_MixRoutine = (APTR) Skip;
    (APTR) ChInfo->chi_Mix.chi_AltMixRoutine = (APTR) Skipb;
  }
}

static void Skip ( ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG SkipDiv;
  unsigned long long SkipLength;

  SkipLength = (((unsigned long long) Advance << 32) + AdvFract) * Length;

  SkipDiv = SkipLength >> 32;
  CurrOffset += SkipDiv;

  SkipDiv = SkipLength;
  CurrFraction += SkipDiv;

  if ( CurrFraction < SkipDiv )
    CurrOffset++;

  *Offset = CurrOffset;
  *Fraction = CurrFraction;
}

static void Skipb ( ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG SkipDiv;
  unsigned long long SkipLength;

  SkipLength = (((unsigned long long) Advance << 32) + AdvFract) * Length;

  SkipDiv = SkipLength >> 32;
  CurrOffset -= SkipDiv;

  SkipDiv = SkipLength;
  CurrFraction += SkipDiv;

  if ( CurrFraction < SkipDiv )
    CurrOffset--;

  *Offset = CurrOffset;
  *Fraction = CurrFraction;
}
