/* TuComposer OS independent code
**
** $VER: GetSongInstrs.c 31.22 (20.7.99)
**
** (C) 1999 by Sebastian Vater (known as Basty/Seasons).
**
*/

#include <stddef.h>
#include <string.h>
#include <exec/types.h>
#include <tucomposer/tucomposer.h>
#include <clib/tucomposer_protos.h>

ULONG CheckInstr ( struct TuComposerInstr **InstrList, struct TuComposerInstr *InstrStr );

struct TuComposerInstr **GetSongInstrs ( struct TuComposerSong *SongStr )
{
  struct TuComposerModule *ModuleStr = (struct TuComposerModule *) &(SongStr->tcso_Node.mln_Succ);
  struct TuComposerInstr **InstrList = NULL;
  struct TuComposerInstr **InstrTmp;
  ULONG NumInstrs;

  for (;;)
  {
    ModuleStr = (struct TuComposerModule *) ModuleStr->tcm_Node.mln_Pred;

    if (!( ModuleStr->tcm_Node.mln_Pred ))
      break;
  }

  (UBYTE *) ModuleStr += -offsetof (struct TuComposerModule, tcm_SongList );

  NumInstrs = GetNumInstrs ( ModuleStr ) + 1;

  if ( InstrTmp = (struct TuComposerInstr **) AllocPMem ( NumInstrs * sizeof ( struct TuComposerInstr *)))
  {
    struct TuComposerTrack *TrackStr = (struct TuComposerTrack *) &(SongStr->tcso_TrackList.mlh_Head);
    struct TuComposerOrders *PosStr;
    ULONG Count = 1L, i;
    UBYTE Flags = 0;

    for (;;)
    {
      struct TuComposerTrackData *TrackData;

      TrackStr = (struct TuComposerTrack *) TrackStr->tct_Node.mln_Succ;

      if (!( TrackStr->tct_Node.mln_Succ ))
        break;

      TrackData = TrackStr->tct_Data;

      for (i = (TrackStr->tct_Length + 1); i > 0; i--)
      {
        if ( (TrackData->tctd_Instrument == 0) && (TrackData->tctd_Octave != 0) && (TrackData->tctd_Note != 0) )
        {
          Flags |= 1;

          goto NullInstrsFound;
        }

        TrackData++;
      }
    }

NullInstrsFound:

    PosStr = SongStr->tcso_Position;

    for (i = SongStr->tcso_Channels; i > 0; i--)
    {
      struct TuComposerOrderList *PosListStr = (struct TuComposerOrderList *) &(PosStr->tco_OrderData.mlh_Head);

      for (;;)
      {
        struct TuComposerTrackData *TrackData;
        WORD InstrTranspose;
        ULONG j;

        PosListStr = (struct TuComposerOrderList *) PosListStr->tcol_Node.mln_Succ;

        if (!( PosListStr->tcol_Node.mln_Succ ))
          break;

        InstrTranspose = PosListStr->tcol_InstrTranspose;

        if (!( TrackStr = PosListStr->tcol_Track ))
          continue;

        TrackData = TrackStr->tct_Data;

        for (j = (TrackStr->tct_Length + 1); j > 0; j--)
        {
          UWORD CurrInstr;

          if ( (CurrInstr = TrackData->tctd_Instrument) )
          {
            struct TuComposerInstr *CurrInstrStr = GetInstrAddr ( ModuleStr, CurrInstr );

            if ( CurrInstrStr->tci_Flags & TCIF_NoInstrTranspose )
            {
              Count += CheckInstr ( InstrTmp, CurrInstrStr );
            } else
            {
              if ( Flags & 1 )
                Count += CheckInstr ( InstrTmp, CurrInstrStr );

              Count += CheckInstr ( InstrTmp, GetInstrAddr ( ModuleStr, CurrInstr + (ULONG) InstrTranspose ));
            }

            if ( CurrInstrStr->tci_CompFlags & TCICF_SeparateSamples )
            {
              struct TuComposerInstrKeyboard *KeybrdStr;

              if ( (KeybrdStr = CurrInstrStr->tci_KeyboardDefs ))
              {
                UWORD NoteIndex = (TrackData->tctd_Octave * 12) + (TrackData->tctd_Note - 1);

                if ( (CurrInstr = KeybrdStr->tcik_NoteEntry[NoteIndex].tcik_Sample_C_0 ))
                  Count += CheckInstr ( InstrTmp, GetInstrAddr ( ModuleStr, CurrInstr ));
              } else
              {
                Count += CheckInstr ( InstrTmp, GetInstrAddr ( ModuleStr, 1L ));
              }
            }
          }

          TrackData++;
        }
      }

      PosStr++;
    }

    if ( (InstrList = AllocPMem ( Count * sizeof (struct TuComposerInstr *))))
      memcpy ( InstrList, InstrTmp, Count * sizeof (struct TuComposerInstr *));

    FreePMem ( InstrTmp, NumInstrs * sizeof (struct TuComposerInstr *));
  }

  return ( InstrList );
}

ULONG CheckInstr ( struct TuComposerInstr **InstrList, struct TuComposerInstr *InstrStr )
{
  if ( InstrStr )
  {
    for (;;)
    {
      if (!( *InstrList ))
        break;

      if ( *InstrList++ == InstrStr )
        return ( 0L );
    }

    *InstrList = InstrStr;

    return ( 1L );
  }

  return ( 0L );
}
