/* TuComposer OS independent code
**
** $VER: CmpSynths.c 31.22 (10.2.00)
**
** (C) 2000 by Sebastian Vater (known as Basty/Seasons).
**
*/

#include <stddef.h>
#include <string.h>
#include <exec/types.h>
#include <tucomposer/tucomposer.h>
#include <clib/tucomposer_protos.h>

LONG CmpSynths ( struct TuComposerSynth *SrcSynthStr,
    struct TuComposerSynth *DestSynthStr )
{
  if ( SrcSynthStr && DestSynthStr )
  {
    struct TuComposerSynthWave *SrcWaveformStr = (struct TuComposerSynthWave *) &(SrcSynthStr->tcy_Waves.mlh_Head);
    struct TuComposerSynthWave *DestWaveformStr = (struct TuComposerSynthWave *) &(DestSynthStr->tcy_Waves.mlh_Head);
    struct TuComposerSynthSymbolTable *SrcSymbolStr = (struct TuComposerSynthSymbolTable *) &(SrcSynthStr->tcy_Symbol.mlh_Head);
    struct TuComposerSynthSymbolTable *DestSymbolStr = (struct TuComposerSynthSymbolTable *) &(DestSynthStr->tcy_Symbol.mlh_Head);

    if ( SrcSynthStr == DestSynthStr )
      goto SynthsEqual;

    if ( SrcSynthStr->tcy_FileName != DestSynthStr->tcy_FileName )
    {
      if ( (SrcSynthStr->tcy_FileName == NULL) || (DestSynthStr->tcy_FileName == NULL) )
        goto SynthsNotEqual;

      if ( strcmp ( SrcSynthStr->tcy_FileName, DestSynthStr->tcy_FileName ))
        goto SynthsNotEqual;
    }

    if ( SrcSynthStr->tcy_Name != DestSynthStr->tcy_Name )
    {
      if ( (SrcSynthStr->tcy_Name == NULL) || (DestSynthStr->tcy_Name == NULL) )
        goto SynthsNotEqual;

      if ( strcmp ( SrcSynthStr->tcy_Name, DestSynthStr->tcy_Name ))
        goto SynthsNotEqual;
    }

    if ( memcmp ( &(SrcSynthStr->tcy_NumWaves), &(DestSynthStr->tcy_NumWaves), offsetof (struct TuComposerSynth, tcy_UnknownChunks) - offsetof (struct TuComposerSynth, tcy_NumWaves )))
      goto SynthsNotEqual;

    if ( memcmp ( SrcSynthStr->tcy_SynthCode, DestSynthStr->tcy_SynthCode, SrcSynthStr->tcy_CodeEntries * sizeof (struct TuComposerSynthCode )))
      goto SynthsNotEqual;

    for (;;)
    {
      SrcWaveformStr = (struct TuComposerSynthWave *) SrcWaveformStr->tcyw_Node.mln_Succ;
      DestWaveformStr = (struct TuComposerSynthWave *) DestWaveformStr->tcyw_Node.mln_Succ;

      if (!( SrcWaveformStr->tcyw_Node.mln_Succ ))
        break;

      if (!( DestWaveformStr->tcyw_Node.mln_Succ ))
        goto SynthsNotEqual;

      if ( (CmpWaveforms ( SrcWaveformStr, DestWaveformStr )))
        goto SynthsNotEqual;
    }

    if ( DestWaveformStr->tcyw_Node.mln_Succ )
      goto SynthsNotEqual;

    for (;;)
    {
      SrcSymbolStr = (struct TuComposerSynthSymbolTable *) SrcSymbolStr->tsst_Node.mln_Succ;
      DestSymbolStr = (struct TuComposerSynthSymbolTable *) DestSymbolStr->tsst_Node.mln_Succ;

      if (!( SrcSymbolStr->tsst_Node.mln_Succ ))
        break;

      if (!( DestSymbolStr->tsst_Node.mln_Succ ))
        goto SynthsNotEqual;

      if ( SrcSymbolStr->tsst_SymbolName != DestSymbolStr->tsst_SymbolName )
      {
        if ( (SrcSymbolStr->tsst_SymbolName == NULL) || (DestSymbolStr->tsst_SymbolName == NULL) )
          goto SynthsNotEqual;

        if ( strcmp ( SrcSymbolStr->tsst_SymbolName, DestSymbolStr->tsst_SymbolName ))
          goto SynthsNotEqual;
      }

      if ( memcmp ( &(SrcSymbolStr->tsst_SymbolValue), &(DestSymbolStr->tsst_SymbolValue), offsetof (struct TuComposerSynthSymbolTable, tsst_Reserved) - offsetof (struct TuComposerSynthSymbolTable, tsst_SymbolValue )))
        goto SynthsNotEqual;
    }

    if ( DestSymbolStr->tsst_Node.mln_Succ )
      goto SynthsNotEqual;

SynthsEqual:
    return ( 0L );

SynthsNotEqual:
    return ( 1L );

  }

  return ( ERROR_SYNTH_NOGIVEN );
}
