/* TuComposer OS independent code
**
** $VER: ConvertWaveform.c 31.22 (7.8.99)
**
** (C) 1999 by Sebastian Vater (known as Basty/Seasons).
**
*/

#include <exec/types.h>
#include <tucomposer/tucomposer.h>
#include <clib/tucomposer_protos.h>

ULONG ConvertWaveform ( struct TuComposerSynthWave *WaveformStr,
    unsigned long DestBitDepth, unsigned long Flags )
{
  if (!( DestBitDepth ))
    DestBitDepth = 16;

  switch ( DestBitDepth )
  {
    case 8 :
    {
      goto Convert8BitWaveform;

      break;
    }
    case 16 :
    {
      goto Convert16BitWaveform;

      break;
    }
    default :
    {
      return ( ERROR_SYNTH_BADBITS );

      break;
    }
  }

Convert8BitWaveform:

  if (!( WaveformStr->tcyw_Flags & TCYWF_8Bit ))
  {
    WaveformStr->tcyw_Flags |= TCYWF_8Bit;

    if ( Flags & CWF_ChangeOnly )
    {
      WaveformStr->tcyw_Samples = WaveformStr->tcyw_Length;

      WaveformStr->tcyw_Repeat += WaveformStr->tcyw_Repeat;
      WaveformStr->tcyw_RepLen += WaveformStr->tcyw_RepLen;
    } else
    {
      WORD *WaveData;

      if ( (WaveData = (WORD *) AllocPMem ( WaveformStr->tcyw_Samples )))
      {
        WORD *OldWaveData = WaveformStr->tcyw_WaveData;
        WORD *OldWaveBuf = OldWaveData;
        BYTE *WaveBuf = (BYTE *) WaveData;
        WORD WavePoint;
        ULONG i;

        for (i = WaveformStr->tcyw_Samples; i > 0; i--)
        {
          WavePoint = *OldWaveBuf++;

          WavePoint >>= 8U;

          *WaveBuf++ = WavePoint;
        }

        WaveformStr->tcyw_WaveData = WaveData;
        WaveformStr->tcyw_Length = WaveformStr->tcyw_Samples;

        FreePMem ( OldWaveData, WaveformStr->tcyw_Samples * sizeof (WORD));
      } else
      {
        return ( ERROR_SYNTH_NOMEM );
      }
    }
  }

  return ( 0L );

Convert16BitWaveform:

  if ( WaveformStr->tcyw_Flags & TCYWF_8Bit )
  {
    WaveformStr->tcyw_Flags &= ~TCYWF_8Bit;

    if ( Flags & CWF_ChangeOnly )
    {
      WaveformStr->tcyw_Samples = WaveformStr->tcyw_Length / 2;

      WaveformStr->tcyw_Repeat = WaveformStr->tcyw_Repeat / 2;
      WaveformStr->tcyw_RepLen = WaveformStr->tcyw_RepLen / 2;
    } else
    {
      WORD *WaveData;

      if ( (WaveData = (WORD *) AllocPMem ( WaveformStr->tcyw_Samples * sizeof (WORD))))
      {
        WORD *OldWaveData = WaveformStr->tcyw_WaveData;
        WORD *WaveBuf = WaveData;
        BYTE *OldWaveBuf = (BYTE *) OldWaveData;
        WORD WavePoint;
        ULONG i;

        for (i = WaveformStr->tcyw_Samples; i > 0; i--)
        {
          WavePoint = (UWORD) *OldWaveBuf++;

          WavePoint <<= 8U;

          *WaveBuf++ = WavePoint;
        }

        WaveformStr->tcyw_WaveData = WaveData;
        WaveformStr->tcyw_Length += WaveformStr->tcyw_Samples;

        FreePMem ( OldWaveData, WaveformStr->tcyw_Samples );
      } else
      {
        return ( ERROR_SYNTH_NOMEM );
      }
    }
  }

  return ( 0L );
}
