/* TuComposer OS independent code
**
** $VER: DupSynth.c 31.22 (7.8.99)
**
** (C) 1999 by Sebastian Vater (known as Basty/Seasons).
**
*/

#include <stddef.h>
#include <string.h>
#include <exec/types.h>
#include <tucomposer/tucomposer.h>
#include <clib/tucomposer_protos.h>

struct TuComposerSynth *DoDupSynth ( struct TuComposerSynth *SynthStr, struct TuComposerSynth *CloneSynth );

#ifndef MOTOROLA
  ULONG ReadMLong ( UBYTE *Buffer );
#else
  #define ReadMLong( Buffer ) (*(ULONG *) (Buffer))
#endif

struct TuComposerSynth *DupSynth ( struct TuComposerSynth *SynthStr )
{
  return ( DoDupSynth ( SynthStr, NULL ));
}

struct TuComposerSynth *DoDupSynth ( struct TuComposerSynth *SynthStr, struct TuComposerSynth *CloneSynth )
{
  if ( SynthStr )
  {
    if ( CloneSynth == NULL )
      CloneSynth = AllocSynth ( SynthStr->tcy_CodeEntries, 0L, 0L );

    if ( CloneSynth )
    {
      struct TuComposerSynthWave *WaveformStr;
      struct TuComposerSynthWave *CloneWaveform;
      struct TuComposerSynthSymbolTable *SymbolStr;
      struct TuComposerSynthSymbolTable *CloneSymbol;
      struct UnknownChunk *SynthUnkChunk;
      struct UnknownChunk *CloneUnkChunk;

      if ( SynthStr->tcy_FileName )
      {
        if (!( SetString ( (STRPTR *) &(CloneSynth->tcy_FileName), SynthStr->tcy_FileName )))
          goto DupSynthErr;
      }

      if ( SynthStr->tcy_Name )
      {
        if (!( SetString ( (STRPTR *) &(CloneSynth->tcy_Name), SynthStr->tcy_Name )))
          goto DupSynthErr;
      }

      if ( ReSizeSynthCode ( CloneSynth, SynthStr->tcy_CodeEntries ))
        goto DupSynthErr;

      memcpy ( CloneSynth->tcy_SynthCode, SynthStr->tcy_SynthCode, SynthStr->tcy_CodeEntries * sizeof (struct TuComposerSynthCode ));

      memcpy ( &(CloneSynth->tcy_VolumePos), &(SynthStr->tcy_VolumePos), offsetof (struct TuComposerSynth, tcy_UnknownChunks) - offsetof (struct TuComposerSynth, tcy_VolumePos));

      WaveformStr = (struct TuComposerSynthWave *) &(SynthStr->tcy_Waves.mlh_Head);

      for (;;)
      {
        WaveformStr = (struct TuComposerSynthWave *) WaveformStr->tcyw_Node.mln_Succ;

        if (!( WaveformStr->tcyw_Node.mln_Succ))
          break;

        if (!( CloneWaveform = DupWaveform ( WaveformStr )))
          goto DupSynthErr;

        AddWaveform ( CloneSynth, CloneWaveform );
      }

      SymbolStr = (struct TuComposerSynthSymbolTable *) &(SynthStr->tcy_Symbol.mlh_Head);

      for (;;)
      {
        SymbolStr = (struct TuComposerSynthSymbolTable *) SymbolStr->tsst_Node.mln_Succ;

        if (!( SymbolStr->tsst_Node.mln_Succ))
          break;

        if (!( CloneSymbol = DupSymbol ( SymbolStr )))
          goto DupSynthErr;

        AddSymbol ( CloneSynth, CloneSymbol );
      }

      SynthUnkChunk = (struct UnknownChunk *) &(SynthStr->tcy_UnknownChunks.mlh_Head);

      for (;;)
      {
        SynthUnkChunk = (struct UnknownChunk *) SynthUnkChunk->uc_Node.mln_Succ;

        if (!( SynthUnkChunk->uc_Node.mln_Succ))
          break;

        if (!( CloneUnkChunk = AllocUnknownChunk ( (APTR) &(SynthUnkChunk->uc_Data), ReadMLong ( (UBYTE *) &(SynthUnkChunk->uc_Identifier[0])), ReadMLong ( (UBYTE *) &(SynthUnkChunk->uc_Length[0])) )))
          goto DupSynthErr;

        AddNodeTail ( (struct List *) &(CloneSynth->tcy_UnknownChunks), (struct Node *) CloneUnkChunk );
      }
    }
  } else
  {
    CloneSynth = NULL;
  }

  return ( CloneSynth );

DupSynthErr:

  FreeSynth ( CloneSynth );

  return ( NULL );
}
