/* TuComposer OS independent code
**
** $VER: FillWaveform.c 31.22 (8.8.99)
**
** (C) 1999 by Sebastian Vater (known as Basty/Seasons).
**
*/

#include <exec/types.h>
#include <tucomposer/tucomposer.h>
#include <clib/tucomposer_protos.h>

#ifdef __cplusplus
#extern "C" {
#endif

void CreateNoneWave16 ( WORD *WaveData, ULONG NumPoints, ULONG RepLen, ULONG Scale, ULONG YOffset, ULONG ScaleType );
void CreateNoneWave8 ( BYTE *WaveData, ULONG NumPoints, ULONG RepLen, ULONG Scale, ULONG YOffset, ULONG ScaleType );
void CreateSineWave16 ( WORD *WaveData, ULONG NumPoints, ULONG RepLen, ULONG Scale, ULONG YOffset, ULONG ScaleType );
void CreateSineWave8 ( BYTE *WaveData, ULONG NumPoints, ULONG RepLen, ULONG Scale, ULONG YOffset, ULONG ScaleType );
void CreateCoSineWave16 ( WORD *WaveData, ULONG NumPoints, ULONG RepLen, ULONG Scale, ULONG YOffset, ULONG ScaleType );
void CreateCoSineWave8 ( BYTE *WaveData, ULONG NumPoints, ULONG RepLen, ULONG Scale, ULONG YOffset, ULONG ScaleType );
void CreateRampWave16 ( WORD *WaveData, ULONG NumPoints, ULONG RepLen, ULONG Scale, ULONG YOffset, ULONG ScaleType );
void CreateRampWave8 ( BYTE *WaveData, ULONG NumPoints, ULONG RepLen, ULONG Scale, ULONG YOffset, ULONG ScaleType );
void CreateTriangleWave16 ( WORD *WaveData, ULONG NumPoints, ULONG RepLen, ULONG Scale, ULONG YOffset, ULONG ScaleType );
void CreateTriangleWave8 ( BYTE *WaveData, ULONG NumPoints, ULONG RepLen, ULONG Scale, ULONG YOffset, ULONG ScaleType );
void CreateSquareWave16 ( WORD *WaveData, ULONG NumPoints, ULONG RepLen, ULONG Scale, ULONG YOffset, ULONG ScaleType );
void CreateSquareWave8 ( BYTE *WaveData, ULONG NumPoints, ULONG RepLen, ULONG Scale, ULONG YOffset, ULONG ScaleType );
void CreateSawtoothWave16 ( WORD *WaveData, ULONG NumPoints, ULONG RepLen, ULONG Scale, ULONG YOffset, ULONG ScaleType );
void CreateSawtoothWave8 ( BYTE *WaveData, ULONG NumPoints, ULONG RepLen, ULONG Scale, ULONG YOffset, ULONG ScaleType );

#ifdef __cplusplus
}
#endif

extern struct TuComposerBase *TuCBase;

APTR WaveTypeTable[] = {
  CreateNoneWave16,
  CreateNoneWave8,
  CreateSineWave16,
  CreateSineWave8,
  CreateCoSineWave16,
  CreateCoSineWave8,
  CreateRampWave16,
  CreateRampWave8,
  CreateSquareWave16,
  CreateSquareWave8,
  CreateTriangleWave16,
  CreateTriangleWave8,
  CreateSawtoothWave16,
  CreateSawtoothWave8
};

ULONG FillWaveform ( struct TuComposerSynthWave *WaveformStr,
    unsigned long RepLen, unsigned long Type, unsigned long Scale,
    long YOffset )
{
  if ( WaveformStr )
  {
    void (**FillWaveType)( WORD *WaveData, ULONG NumPoints, ULONG RepLen, ULONG Scale, ULONG YOffset, ULONG ScaleType );
    ULONG ScaleType;

    if ( RepLen == 0L )
      RepLen = WaveformStr->tcyw_Samples;

    ScaleType = Scale & SCALE_INVERT;
    Scale &= ~(SCALE_INVERT);

    if ( Scale > MAX_SCALE )
      Scale = MAX_SCALE;

    if ( Type > ENVL_LAST )
      Type = ENVL_EMPTY;

    (APTR) FillWaveType = (APTR) &(WaveTypeTable);
    FillWaveType[Type * 2 + ((WaveformStr->tcyw_Flags & TCYWF_8Bit) ? 1 : 0)] ( WaveformStr->tcyw_WaveData, WaveformStr->tcyw_Samples, RepLen, Scale, YOffset, ScaleType );

    return ( 0L );
  }

  return ( ERROR_SYNTH_NODATA );
}

void CreateNoneWave16 ( WORD *WaveData, ULONG NumPoints, ULONG RepLen, ULONG Scale, ULONG YOffset, ULONG ScaleType )
{
  ULONG i;

  for (i = NumPoints; i > 0; i--)
  {
    *WaveData++ = YOffset;
  }
}

void CreateNoneWave8 ( BYTE *WaveData, ULONG NumPoints, ULONG RepLen, ULONG Scale, ULONG YOffset, ULONG ScaleType )
{
  ULONG i;

  YOffset >>= 8;

  for (i = NumPoints; i > 0; i--)
  {
    *WaveData++ = YOffset;
  }
}

void CreateSineWave16 ( WORD *WaveData, ULONG NumPoints, ULONG RepLen, ULONG Scale, ULONG YOffset, ULONG ScaleType )
{
  ULONG i, SineDiv, SineMod, SinePos = 0L, SineCnt = 0L;
  LONG SineVal = 0L;
  WORD *SineTable = TuCBase->tcb_SineTable;

  SineDiv = 360 / RepLen;
  SineMod = 360 % RepLen;

  for (i = NumPoints; i > 0; i--)
  {
    SineVal = SineTable[SinePos];

    if ( ScaleType )
      SineVal = -SineVal;

    SinePos += SineDiv;

    SineVal *= (LONG) Scale;
    SineVal /= 32767;
    SineVal += YOffset;

    SineCnt += SineMod;

    if ( SineCnt >= RepLen )
    {
      SineCnt -= RepLen;
      SinePos++;
    }

    if ( SinePos >= 360 )
      SinePos -= 360;

    *WaveData++ = SineVal;
  }
}

void CreateSineWave8 ( BYTE *WaveData, ULONG NumPoints, ULONG RepLen, ULONG Scale, ULONG YOffset, ULONG ScaleType )
{
  ULONG i, SineDiv, SineMod, SinePos = 0L, SineCnt = 0L;
  LONG SineVal = 0L;
  WORD *SineTable = TuCBase->tcb_SineTable;

  SineDiv = 360 / RepLen;
  SineMod = 360 % RepLen;

  for (i = NumPoints; i > 0; i--)
  {
    SineVal = SineTable[SinePos];

    if ( ScaleType )
      SineVal = -SineVal;

    SinePos += SineDiv;

    SineVal *= (LONG) Scale;
    SineVal /= 32767;
    SineVal += YOffset;

    SineCnt += SineMod;

    if ( SineCnt >= RepLen )
    {
      SineCnt -= RepLen;
      SinePos++;
    }

    if ( SinePos >= 360 )
      SinePos -= 360;

    SineVal >>= 8;

    *WaveData++ = SineVal;
  }
}

void CreateCoSineWave16 ( WORD *WaveData, ULONG NumPoints, ULONG RepLen, ULONG Scale, ULONG YOffset, ULONG ScaleType )
{
  ULONG i, SineDiv, SineMod, SineCnt = 0L;
  LONG SinePos = 90L, SineVal = 0L;
  WORD *SineTable = TuCBase->tcb_SineTable;

  SineDiv = 360 / RepLen;
  SineMod = 360 % RepLen;

  for (i = NumPoints; i > 0; i--)
  {
    SineVal = SineTable[SinePos];

    if ( ScaleType )
      SineVal = -SineVal;

    if ( (SinePos -= SineDiv) < 0)
      SinePos += 360;

    SineVal *= (LONG) Scale;
    SineVal /= 32767;
    SineVal += YOffset;

    SineCnt += SineMod;

    if ( SineCnt >= RepLen )
    {
      SineCnt -= RepLen;
      SinePos--;

      if ( SinePos < 0 )
        SinePos += 360;
    }

    *WaveData++ = SineVal;
  }
}

void CreateCoSineWave8 ( BYTE *WaveData, ULONG NumPoints, ULONG RepLen, ULONG Scale, ULONG YOffset, ULONG ScaleType )
{
  ULONG i, SineDiv, SineMod, SineCnt = 0L;
  LONG SinePos = 90L, SineVal = 0L;
  WORD *SineTable = TuCBase->tcb_SineTable;

  SineDiv = 360 / RepLen;
  SineMod = 360 % RepLen;

  for (i = NumPoints; i > 0; i--)
  {
    SineVal = SineTable[SinePos];

    if ( ScaleType )
      SineVal = -SineVal;

    if ( (SinePos -= SineDiv) < 0)
      SinePos += 360;

    SineVal *= (LONG) Scale;
    SineVal /= 32767;
    SineVal += YOffset;

    SineCnt += SineMod;

    if ( SineCnt >= RepLen )
    {
      SineCnt -= RepLen;
      SinePos--;

      if ( SinePos < 0 )
        SinePos += 360;
    }

    SineVal >>= 8;

    *WaveData++ = SineVal;
  }
}

void CreateRampWave16 ( WORD *WaveData, ULONG NumPoints, ULONG RepLen, ULONG Scale, ULONG YOffset, ULONG ScaleType )
{
  ULONG i, StartScale = -Scale, Points, ScaleDiv, ScaleMod, ScaleCnt = 0L, RampVal;

  if ( (Points = RepLen >> 1) == 0L )
    Points = 1;

  ScaleDiv = Scale / Points;
  ScaleMod = Scale % Points;

  for (i = NumPoints; i > 0; i--)
  {
    RampVal = StartScale;

    StartScale += ScaleDiv;
    ScaleCnt += ScaleMod;

    if ( ScaleCnt >= RepLen )
    {
      ScaleCnt -= RepLen;
      StartScale++;
    }

    if ( StartScale >= Scale )
      StartScale = -Scale;

    if ( ScaleType )
      RampVal = -RampVal;

    RampVal += YOffset;

    *WaveData++ = RampVal;
  }
}

void CreateRampWave8 ( BYTE *WaveData, ULONG NumPoints, ULONG RepLen, ULONG Scale, ULONG YOffset, ULONG ScaleType )
{
  ULONG i, StartScale = -Scale, Points, ScaleDiv, ScaleMod, ScaleCnt = 0L, RampVal;

  if ( (Points = RepLen >> 1) == 0L )
    Points = 1;

  ScaleDiv = Scale / Points;
  ScaleMod = Scale % Points;

  for (i = NumPoints; i > 0; i--)
  {
    RampVal = StartScale;

    StartScale += ScaleDiv;
    ScaleCnt += ScaleMod;

    if ( ScaleCnt >= RepLen )
    {
      ScaleCnt -= RepLen;
      StartScale++;
    }

    if ( StartScale >= Scale )
      StartScale = -Scale;

    if ( ScaleType )
      RampVal = -RampVal;

    RampVal += YOffset;

    RampVal >>= 8;

    *WaveData++ = RampVal;
  }
}

void CreateSquareWave16 ( WORD *WaveData, ULONG NumPoints, ULONG RepLen, ULONG Scale, ULONG YOffset, ULONG ScaleType )
{
  ULONG i, SquareVal;

  if ( ScaleType )
    Scale = -Scale;

  while ( NumPoints > 0 )
  {
    Scale = -Scale;

    SquareVal = (Scale + YOffset);

    for (i = RepLen / 2; i > 0; i--)
    {
      *WaveData++ = SquareVal;

      NumPoints--;

      if (!( NumPoints ))
        break;
    }
  }
}

void CreateSquareWave8 ( BYTE *WaveData, ULONG NumPoints, ULONG RepLen, ULONG Scale, ULONG YOffset, ULONG ScaleType )
{
  ULONG i, SquareVal;

  if ( ScaleType )
    Scale = -Scale;

  while ( NumPoints > 0 )
  {
    Scale = -Scale;

    SquareVal = (Scale + YOffset) >> 8;

    for (i = RepLen / 2; i > 0; i--)
    {
      *WaveData++ = SquareVal;

      NumPoints--;

      if (!( NumPoints ))
        break;
    }
  }
}

void CreateTriangleWave16 ( WORD *WaveData, ULONG NumPoints, ULONG RepLen, ULONG Scale, ULONG YOffset, ULONG ScaleType )
{
  ULONG i, TriangleVal, TrianglePos, TriangleDownPos, Points, ScaleDiv, ScaleMod, ScaleCnt;

  if ( (Points = RepLen >> 2) == 0L )
    Points = 1;

   ScaleDiv = Scale / Points;
   ScaleMod = Scale % Points;

  while ( NumPoints > 0 )
  {
    TrianglePos = 0L;
    ScaleCnt = 0L;
    TriangleDownPos = RepLen - Points;

    for (i = RepLen; i > 0; i--)
    {
      TriangleVal = TrianglePos;

      if ( TriangleDownPos >= i )
      {
        if ( TriangleDownPos == i )
        {
          ScaleCnt += ScaleMod;
          ScaleDiv = -ScaleDiv;
        }

        if ( Points >= i )
        {
          ScaleCnt += ScaleMod;
          ScaleDiv = -ScaleDiv;
          TriangleDownPos = 0L;
        }

        TrianglePos += ScaleDiv;
        ScaleCnt += ScaleMod;

        if ( ScaleCnt >= RepLen )
        {
          ScaleCnt -= RepLen;
          TrianglePos--;
        }
      } else
      {
        TrianglePos += ScaleDiv;
        ScaleCnt += ScaleMod;

        if ( ScaleCnt >= RepLen )
        {
          ScaleCnt -= RepLen;
          TrianglePos++;
        }
      }

      if ( ScaleType )
        TriangleVal = -TriangleVal;

      TriangleVal += YOffset;

      *WaveData++ = TriangleVal;

      NumPoints--;

      if (!( NumPoints ))
        break;
    }
  }
}

void CreateTriangleWave8 ( BYTE *WaveData, ULONG NumPoints, ULONG RepLen, ULONG Scale, ULONG YOffset, ULONG ScaleType )
{
  ULONG i, TriangleVal, TrianglePos, TriangleDownPos, Points, ScaleDiv, ScaleMod, ScaleCnt;

  if ( (Points = RepLen >> 2) == 0L )
    Points = 1;

   ScaleDiv = Scale / Points;
   ScaleMod = Scale % Points;

  while ( NumPoints > 0 )
  {
    TrianglePos = 0L;
    ScaleCnt = 0L;
    TriangleDownPos = RepLen - Points;

    for (i = RepLen; i > 0; i--)
    {
      TriangleVal = TrianglePos;

      if ( TriangleDownPos >= i )
      {
        if ( TriangleDownPos == i )
        {
          ScaleCnt += ScaleMod;
          ScaleDiv = -ScaleDiv;
        }

        if ( Points >= i )
        {
          ScaleCnt += ScaleMod;
          ScaleDiv = -ScaleDiv;
          TriangleDownPos = 0L;
        }

        TrianglePos += ScaleDiv;
        ScaleCnt += ScaleMod;

        if ( ScaleCnt >= RepLen )
        {
          ScaleCnt -= RepLen;
          TrianglePos--;
        }
      } else
      {
        TrianglePos += ScaleDiv;
        ScaleCnt += ScaleMod;

        if ( ScaleCnt >= RepLen )
        {
          ScaleCnt -= RepLen;
          TrianglePos++;
        }
      }

      if ( ScaleType )
        TriangleVal = -TriangleVal;

      TriangleVal += YOffset;

      TriangleVal >>= 8;

      *WaveData++ = TriangleVal;

      NumPoints--;

      if (!( NumPoints ))
        break;
    }
  }
}

void CreateSawtoothWave16 ( WORD *WaveData, ULONG NumPoints, ULONG RepLen, ULONG Scale, ULONG YOffset, ULONG ScaleType )
{
  ULONG i, SawtoothVal, SawtoothPos, SawtoothDownPos, Points, ScaleDiv, ScaleMod, ScaleCnt;

  if ( (Points = RepLen >> 2) == 0L )
    Points = 1;

  ScaleDiv = -(Scale / Points);
  ScaleMod = Scale % Points;

  while ( NumPoints > 0 )
  {
    SawtoothPos = Scale;
    ScaleCnt = 0L;
    SawtoothDownPos = RepLen >> 1;

    for (i = RepLen; i > 0; i--)
    {
      SawtoothVal = SawtoothPos;

      if ( SawtoothDownPos >= i )
      {
        if ( SawtoothDownPos == i )
        {
          ScaleCnt += ScaleMod;
          ScaleDiv = -ScaleDiv;
        }

        SawtoothPos += ScaleDiv;
        ScaleCnt += ScaleMod;

        if ( ScaleCnt >= RepLen )
        {
          ScaleCnt -= RepLen;
          SawtoothPos++;
        }
      } else
      {
        SawtoothPos += ScaleDiv;
        ScaleCnt += ScaleMod;

        if ( ScaleCnt >= RepLen )
        {
          ScaleCnt -= RepLen;
          SawtoothPos--;
        }
      }

      if ( ScaleType )
        SawtoothVal = -SawtoothVal;

      SawtoothVal += YOffset;

      *WaveData++ = SawtoothVal;

      NumPoints--;

      if (!( NumPoints ))
        break;
    }
  }
}

void CreateSawtoothWave8 ( BYTE *WaveData, ULONG NumPoints, ULONG RepLen, ULONG Scale, ULONG YOffset, ULONG ScaleType )
{
  ULONG i, SawtoothVal, SawtoothPos, SawtoothDownPos, Points, ScaleDiv, ScaleMod, ScaleCnt;

  if ( (Points = RepLen >> 2) == 0L )
    Points = 1;

  ScaleDiv = -(Scale / Points);
  ScaleMod = Scale % Points;

  while ( NumPoints > 0 )
  {
    SawtoothPos = Scale;
    ScaleCnt = 0L;
    SawtoothDownPos = RepLen >> 1;

    for (i = RepLen; i > 0; i--)
    {
      SawtoothVal = SawtoothPos;

      if ( SawtoothDownPos >= i )
      {
        if ( SawtoothDownPos == i )
        {
          ScaleCnt += ScaleMod;
          ScaleDiv = -ScaleDiv;
        }

        SawtoothPos += ScaleDiv;
        ScaleCnt += ScaleMod;

        if ( ScaleCnt >= RepLen )
        {
          ScaleCnt -= RepLen;
          SawtoothPos++;
        }
      } else
      {
        SawtoothPos += ScaleDiv;
        ScaleCnt += ScaleMod;

        if ( ScaleCnt >= RepLen )
        {
          ScaleCnt -= RepLen;
          SawtoothPos--;
        }
      }

      if ( ScaleType )
        SawtoothVal = -SawtoothVal;

      SawtoothVal += YOffset;

      SawtoothVal >>= 8;

      *WaveData++ = SawtoothVal;

      NumPoints--;

      if (!( NumPoints ))
        break;
    }
  }
}
