/* TuComposer OS independent code
**
** $VER: FlipTrack.c 31.22 (23.4.99)
**
** (C) 1999 by Sebastian Vater (known as Basty/Seasons).
**
*/

#include <exec/types.h>
#include <tucomposer/tucomposer.h>
#include <clib/tucomposer_protos.h>

ULONG FlipTrack ( struct TuComposerTrack *TrackStr, unsigned long Scale,
    unsigned long StartRow, unsigned long EndRow )
{
  if ( TrackStr )
  {
    if ( Scale )
    {
      struct TuComposerTrackData *TrackData = TrackStr->tct_Data;
      struct TuComposerTrackData *NewTrackData;
      ULONG TrackLen = (TrackStr->tct_Length) + 1;

      StartRow = (StartRow < TrackLen ? StartRow : TrackLen - 1);
      EndRow = (EndRow < TrackLen ? EndRow : TrackLen - 1);

      if (StartRow > EndRow)
      {
        ULONG TmpRow;
        TmpRow = StartRow;
        StartRow = EndRow;
        EndRow = TmpRow;
      }

      Scale = (((EndRow - StartRow) + 1) <= Scale ? 1 : Scale);

      if ((Scale >= TrackLen) || (StartRow == EndRow))
        return ( 0L );

      if ( NewTrackData = AllocTrackData ( TrackLen ))
      {
        struct TuComposerTrackData *SrcTrackData;
        struct TuComposerTrackData *DestTrackData;
        ULONG Row;
        ULONG ScaleCnt;

        if ( StartRow )
        {
          SrcTrackData = TrackData;
          DestTrackData = NewTrackData;

          for (Row = StartRow; Row > 0; Row--)
          {
            DestTrackData->tctd_Octave = SrcTrackData->tctd_Octave;
            DestTrackData->tctd_Note = SrcTrackData->tctd_Note;
            DestTrackData->tctd_Instrument = SrcTrackData->tctd_Instrument;

            ReMapNodeList ( (struct List *) &(SrcTrackData->tctd_Effects), (struct List *) &(DestTrackData->tctd_Effects));
            InitNodeList ( (struct List *) &(SrcTrackData->tctd_Effects));

            SrcTrackData++;
            DestTrackData++;
          }
        }

        SrcTrackData = TrackData + StartRow;
        DestTrackData = NewTrackData + (EndRow - Scale);

        for (Row = (EndRow - StartRow) + 1 - Scale; Row > 0;)
        {
          for (ScaleCnt = Scale; ScaleCnt > 0; ScaleCnt--)
          {
            DestTrackData->tctd_Octave = SrcTrackData->tctd_Octave;
            DestTrackData->tctd_Note = SrcTrackData->tctd_Note;
            DestTrackData->tctd_Instrument = SrcTrackData->tctd_Instrument;

            ReMapNodeList ( (struct List *) &(SrcTrackData->tctd_Effects), (struct List *) &(DestTrackData->tctd_Effects));
            InitNodeList ( (struct List *) &(SrcTrackData->tctd_Effects));

            Row--;
            if (Row == 0)
              break;

            SrcTrackData++;
            DestTrackData++;
          }

          DestTrackData -= Scale * 2;
        }

        ScaleCnt = (EndRow - (((EndRow - StartRow) + 1) % Scale) + 1);

        if (ScaleCnt < TrackLen)
        {
          SrcTrackData = TrackData + ScaleCnt;
          DestTrackData = NewTrackData + ScaleCnt;

          for (Row = TrackLen - ScaleCnt; Row > 0; Row--)
          {
            DestTrackData->tctd_Octave = SrcTrackData->tctd_Octave;
            DestTrackData->tctd_Note = SrcTrackData->tctd_Note;
            DestTrackData->tctd_Instrument = SrcTrackData->tctd_Instrument;

            ReMapNodeList ( (struct List *) &(SrcTrackData->tctd_Effects), (struct List *) &(DestTrackData->tctd_Effects));
            InitNodeList ( (struct List *) &(SrcTrackData->tctd_Effects));

            SrcTrackData++;
            DestTrackData++;
          }
        }

        FreeTrackData ( TrackData, TrackLen );

        TrackStr->tct_Data = NewTrackData;

        return ( 0L );
      }

      return ( ERROR_TRACK_NOMEM );
    }

    return ( ERROR_TRACK_NOSCALE );
  }

  return( ERROR_TRACK_NOGIVEN );
}
