/* TuComposer OS independent code
**
** $VER: PackTrack.c 31.22 (29.5.99)
**
** (C) 1999 by Sebastian Vater (known as Basty/Seasons).
**
*/

#include <stddef.h>
#include <string.h>
#include <exec/types.h>
#include <tucomposer/tucomposer.h>
#include <clib/tucomposer_protos.h>

#ifndef MOTOROLA
  ULONG ReadMLong ( UBYTE *Buffer );
#else
  #define ReadMLong( Buffer ) (*(ULONG *) (Buffer))
#endif

struct TuComposerPacked *PackTrack ( struct TuComposerTrack *TrackStr )
{
  struct TuComposerTrackData *TrackData;
  struct TuComposerPacked *PackedTrack;
  struct TuComposerTrackEffect *TrackFX;
  struct UnknownChunk *TrackUnkChunk;
  struct UnknownChunk *CloneUnkChunk;
  ULONG TrackLen,PackedLen = 0L, Row, RowLen, LastRow = 0L;
  UWORD InstrTmp;
  UBYTE *PackedTrackData;
  UBYTE PackTmp, InstrHI, RowHI, LastRowHI = 0;

  if ( TrackStr )
  {
    if ( TrackData = TrackStr->tct_Data )
    {
      if ( TrackLen = TrackStr->tct_Length )
      {
        TrackLen++;

        for ( Row = 0L; Row < TrackLen; Row++)
        {
          RowLen = 0L;

          if ( TrackData->tctd_Octave || TrackData->tctd_Note )
            RowLen++;

          InstrTmp = TrackData->tctd_Instrument;

          if ( InstrTmp >> 8U )
            RowLen++;

          if ( InstrTmp & 0xFF )
            RowLen++;

          TrackFX = (struct TuComposerTrackEffect *) TrackData->tctd_Effects.mlh_Head;

          if ( TrackFX->tcte_Node.mln_Succ )
          {
            if (!( TrackFX->tcte_EffectByte ))
            {
              if ( TrackFX->tcte_DataWord )
                goto CheckForNextEff;
            }

            RowLen++;
            goto NoEffByte;

CheckForNextEff:
            if ( (TrackFX->tcte_Node.mln_Succ)->mln_Succ )
              RowLen++;

NoEffByte:
            InstrTmp = TrackFX->tcte_DataWord;

            if ( InstrTmp >> 8U )
              RowLen++;

            if ( InstrTmp & 0xFF )
              RowLen++;

            for (;;)
            {
              TrackFX = (struct TuComposerTrackEffect *) TrackFX->tcte_Node.mln_Succ;

              if (!( (struct TuComposerTrackEffect *) TrackFX->tcte_Node.mln_Succ ))
                break;

              RowLen += 1+2;
            }
          }

          TrackData++;

          if ( RowLen )
          {
            if ( Row != LastRow )
            {
              if ( Row & 0xFF )
                RowLen++;

              RowHI = (Row >> 8);

              if ( RowHI != LastRowHI )
              {
                LastRowHI = RowHI;
                RowLen++;
              }

              LastRow = Row;
            }

            LastRow++;
            PackedLen += (RowLen + 1);
          }
        }

        PackedLen++;

        if ( PackedTrack = (struct TuComposerPacked *) AllocPMem ( sizeof (struct TuComposerPacked ) ))
        {
          InitNodeList ( (struct List *) &(PackedTrack->tcp_UnknownChunks));

          TrackUnkChunk = (struct UnknownChunk *) &(TrackStr->tct_UnknownChunks.mlh_Head);

          for (;;)
          {
            TrackUnkChunk = (struct UnknownChunk *) TrackUnkChunk->uc_Node.mln_Succ;

            if (!( TrackUnkChunk->uc_Node.mln_Succ))
              break;

            if (!( CloneUnkChunk = AllocUnknownChunk ( (APTR) &(TrackUnkChunk->uc_Data), ReadMLong ( (UBYTE *) &(TrackUnkChunk->uc_Identifier[0])), ReadMLong ( (UBYTE *) &(TrackUnkChunk->uc_Length[0])) )))
              goto PackTErr;

            AddNodeTail ( (struct List *) &(PackedTrack->tcp_UnknownChunks), (struct Node *) CloneUnkChunk );
          }

          if ( PackedTrackData = (UBYTE *) AllocPMem ( PackedLen + 1 ) )
          {
            if ( TrackStr->tct_FileName )
            {
              if (!( SetString ( (STRPTR *) &(PackedTrack->tcp_FileName), TrackStr->tct_FileName )))
                goto PackTErr;
            }

            if ( TrackStr->tct_Name )
            {
              if (!( SetString ( (STRPTR *) &(PackedTrack->tcp_Name), TrackStr->tct_Name )))
                goto PackTErr;
            }


            PackedTrack->tcp_FileName = TrackStr->tct_FileName;
            PackedTrack->tcp_Name = TrackStr->tct_Name;
            PackedTrack->tcp_Data = PackedTrackData;
            PackedTrack->tcp_PackLength = PackedLen;

            memcpy ( &(PackedTrack->tcp_Length), &(TrackStr->tct_Length), offsetof(struct TuComposerTrack, tct_UnknownChunks) - offsetof(struct TuComposerTrack, tct_Length));

            TrackData = TrackStr->tct_Data;

            LastRow = 0L;
            LastRowHI = 0;

            for (Row = 0L; Row < TrackLen; Row++)
            {
              PackTmp = 0;

              if ( TrackData->tctd_Octave || TrackData->tctd_Note )
                PackTmp |= TCPF_Note;

              InstrTmp = TrackData->tctd_Instrument;

              if ( InstrTmp >> 8U )
                PackTmp |= TCPF_InstrHI;

              if ( InstrTmp & 0xFF )
                PackTmp |= TCPF_InstrLO;

              TrackFX = (struct TuComposerTrackEffect *) TrackData->tctd_Effects.mlh_Head;

              if ( TrackFX->tcte_Node.mln_Succ )
              {
                if (!( TrackFX->tcte_EffectByte ))
                {
                  if ( TrackFX->tcte_DataWord )
                    goto NotPreProcessEffect;
                }

                PackTmp |= TCPF_Effect;

NotPreProcessEffect:
                if ( (TrackFX->tcte_Node.mln_Succ)->mln_Succ )
                  PackTmp |= TCPF_Effect;

                InstrTmp = TrackFX->tcte_DataWord;

                if ( InstrTmp >> 8U )
                  PackTmp |= TCPF_DataHI;

                if ( InstrTmp & 0xFF )
                  PackTmp |= TCPF_DataLO;
              }

              if ( PackTmp )
              {
                if ( LastRow == Row )
                {
                  *PackedTrackData++ = PackTmp;
                } else
                {
                  LastRow = Row;
                  RowHI = LastRow >> 8U;

                  if ( RowHI != LastRowHI )
                  {
                    LastRowHI = RowHI;
                    PackTmp |= TCPF_RowHI;
                  }

                  if ( (InstrHI = (Row & 0xFF)) )
                    PackTmp |= TCPF_RowLO;

                  *PackedTrackData++ = PackTmp;

                  if ( PackTmp & TCPF_RowHI )
                    *PackedTrackData++ = RowHI;

                  if ( InstrHI )
                    *PackedTrackData++ = InstrHI;
                }

                LastRow++;

                if ( (InstrHI = ((TrackData->tctd_Octave) << 4U) | (UBYTE) TrackData->tctd_Note ))
                  *PackedTrackData++ = InstrHI;

                if ( (InstrHI = (TrackData->tctd_Instrument) >> 8U ))
                  *PackedTrackData++ = InstrHI;

                if ( (InstrHI = (TrackData->tctd_Instrument) & 0xFF ))
                  *PackedTrackData++ = InstrHI;

                TrackFX = (struct TuComposerTrackEffect *) TrackData->tctd_Effects.mlh_Head;

                if ( TrackFX->tcte_Node.mln_Succ )
                {
                  if ( PackTmp & TCPF_Effect )
                  {
                    InstrHI = (TrackFX->tcte_EffectByte) & 0x7F;

                    if ( (TrackFX->tcte_Node.mln_Succ)->mln_Succ )
                      InstrHI |= 0x80;

                    *PackedTrackData++ = InstrHI;
                  }

                  InstrTmp = TrackFX->tcte_DataWord;

                  if ( PackTmp & TCPF_DataHI )
                    *PackedTrackData++ = (InstrTmp >> 8U);

                  if ( PackTmp & TCPF_DataLO )
                    *PackedTrackData++ = (InstrTmp & 0xFF);

                  for (;;)
                  {
                    TrackFX = (struct TuComposerTrackEffect *) TrackFX->tcte_Node.mln_Succ;

                    if (!( (struct TuComposerTrackEffect *) TrackFX->tcte_Node.mln_Succ ))
                      break;

                    InstrHI = (TrackFX->tcte_EffectByte) & 0x7F;

                    if ( (TrackFX->tcte_Node.mln_Succ)->mln_Succ )
                      InstrHI |= 0x80;

                    *PackedTrackData++ = InstrHI;

                    InstrTmp = TrackFX->tcte_DataWord;

                    *PackedTrackData++ = (InstrTmp >> 8);
                    *PackedTrackData++ = (InstrTmp & 0xFF);
                  }
                }
              }

              TrackData++;
            }

            *PackedTrackData = 0;

            return ( PackedTrack );
          }

PackTErr:
          FreePackedTrack ( PackedTrack );
        }
      }
    }
  }

  return ( NULL );
}
