/* TuComposer OS independent code
**
** $VER: UnPackTrack.c 31.22 (27.6.99)
**
** (C) 1999 by Sebastian Vater (known as Basty/Seasons).
**
*/

#include <stddef.h>
#include <string.h>
#include <exec/types.h>
#include <tucomposer/tucomposer.h>
#include <clib/tucomposer_protos.h>

#ifndef MOTOROLA
  ULONG ReadMLong ( UBYTE *Buffer );
#else
  #define ReadMLong( Buffer ) (*(ULONG *) (Buffer))
#endif

struct TuComposerTrack *UnPackTrack ( struct TuComposerPacked *PackedTrackStr,
     struct TuComposerTrack *TrackStr )
{
  if ( PackedTrackStr )
  {
    struct TuComposerTrack *TrackBuf;
    struct TuComposerTrackData *TrackData;
    struct TuComposerTrackEffect *TrackFX;
    struct UnknownChunk *TrackUnkChunk;
    struct UnknownChunk *CloneUnkChunk;
    UBYTE *PackedPtr;
    UBYTE TmpPackByte, PackType;
    UWORD TmpPackWord, TmpPackRow, LastPackRow;
    ULONG PackLength;

    PackedPtr = PackedTrackStr->tcp_Data;
    PackLength = PackedTrackStr->tcp_PackLength;

    if (!( TrackBuf = (struct TuComposerTrack *) AllocTrack ( (PackedTrackStr->tcp_Length) + 1 )))
      return ( NULL );

    TrackUnkChunk = (struct UnknownChunk *) &(PackedTrackStr->tcp_UnknownChunks.mlh_Head);

    for (;;)
    {
      TrackUnkChunk = (struct UnknownChunk *) TrackUnkChunk->uc_Node.mln_Succ;

      if (!( TrackUnkChunk->uc_Node.mln_Succ))
        break;

      if (!( CloneUnkChunk = AllocUnknownChunk ( (APTR) &(TrackUnkChunk->uc_Data), ReadMLong ( (UBYTE *) &(TrackUnkChunk->uc_Identifier[0])), ReadMLong ( (UBYTE *) &(TrackUnkChunk->uc_Length[0])) )))
      {

UnPackTrackErr:

        FreeTrack ( TrackBuf );

        return ( NULL );
      }

      AddNodeTail ( (struct List *) &(TrackBuf->tct_UnknownChunks), (struct Node *) CloneUnkChunk );
    }

    if ( PackedTrackStr->tcp_FileName )
    {
      if (!( SetString ( (STRPTR *) &(TrackBuf->tct_FileName), PackedTrackStr->tcp_FileName )))
        goto UnPackTrackErr;
    }

    if ( PackedTrackStr->tcp_Name )
    {
      if (!( SetString ( (STRPTR *) &(TrackBuf->tct_Name), PackedTrackStr->tcp_Name )))
        goto UnPackTrackErr;
    }

    memcpy ( &(TrackBuf->tct_Length), &(PackedTrackStr->tcp_Length), offsetof(struct TuComposerTrack, tct_UnknownChunks) - offsetof(struct TuComposerTrack, tct_Length));

    if ( PackLength )
    {
      LastPackRow = 0;
      TrackData = TrackBuf->tct_Data;

      for (;;)
      {
        if (!( (PackType = *PackedPtr++) ))
          break;

        TmpPackWord = 0;

        if ( PackType & TCPF_RowHI )
          TmpPackWord = (UBYTE) ((*PackedPtr++) << 8);

        if ( PackType & TCPF_RowLO )
          TmpPackWord |= (UBYTE) *PackedPtr++;

        if ( (TmpPackRow = TmpPackWord) )
        {
          if (!( (TmpPackRow >>= 8) ))
          {
            TmpPackRow = (UBYTE) TmpPackWord;
            TmpPackWord = (LastPackRow & 0xFF00) | TmpPackRow;
          }

          TmpPackRow = LastPackRow;
          LastPackRow = TmpPackWord;

          TrackData += (TmpPackWord - TmpPackRow);
        }

        if ( PackType & TCPF_Note )
        {
          if ( (TmpPackByte = *PackedPtr++) >= SPEC_NOTES)
          {
            TrackData->tctd_Note = TmpPackByte;
          } else
          {
            TrackData->tctd_Octave = (TmpPackByte >> 4);
            TrackData->tctd_Note = (TmpPackByte & 0xF);
          }
        }

        TmpPackWord = 0;

        if ( PackType & TCPF_InstrHI )
        {
          TmpPackWord = ((*PackedPtr++) << 8);
        }

        if ( PackType & TCPF_InstrLO )
          TmpPackWord |= *PackedPtr++;

        TrackData->tctd_Instrument = TmpPackWord;

        if ( PackType & (TCPF_Effect|TCPF_DataHI|TCPF_DataLO) )
        {
          for (;;)
          {
            TmpPackByte = 0;

            if ( PackType & TCPF_Effect )
              TmpPackByte = *PackedPtr++;

            TmpPackWord = 0;

            if ( PackType & TCPF_DataHI )
              TmpPackWord = ((*PackedPtr++) << 8);

            if ( PackType & TCPF_DataLO )
              TmpPackWord |= *PackedPtr++;

            if (!( (TrackFX = AllocEffect ( TmpPackByte, TmpPackWord ))))
              goto UnPackTrackErr;

            AddNodeTail ( (struct List *) &(TrackData->tctd_Effects), (struct Node *) TrackFX );

            PackType = 0xFF;

            if ( (BYTE) TmpPackByte >= 0 )
              break;
          }
        }

        LastPackRow++;
        TrackData++;
      }
    }

    if ( TrackStr )
    {
      FreeNameStr ( TrackStr->tct_FileName );
      FreeNameStr ( TrackStr->tct_Name );
      FreeTrackData ( TrackStr->tct_Data, (TrackStr->tct_Length) + 1);

      TrackStr->tct_FileName = TrackBuf->tct_FileName;
      TrackStr->tct_Name = TrackBuf->tct_Name;
      TrackStr->tct_Data = TrackBuf->tct_Data;

      memcpy ( &(TrackStr->tct_Length), &(TrackBuf->tct_Length), offsetof(struct TuComposerTrack, tct_UnknownChunks) - offsetof(struct TuComposerTrack, tct_Length));

      TrackUnkChunk = (struct UnknownChunk *) &(TrackStr->tct_UnknownChunks.mlh_Head);

      for (;;)
      {
        CloneUnkChunk = (struct UnknownChunk *) TrackUnkChunk->uc_Node.mln_Succ;

        if (!( (struct UnknownChunk *) CloneUnkChunk->uc_Node.mln_Succ ))
          break;

        FreeUnknownChunk ( CloneUnkChunk );
      }

      ReMapNodeList ( (struct List *) &(TrackBuf->tct_UnknownChunks), (struct List *) &(TrackStr->tct_UnknownChunks));

      FreePMem ( TrackBuf, sizeof (struct TuComposerTrack ));

      return ( TrackStr );
    }

    return ( TrackBuf );
  }

  return ( NULL );
}
