/* TuComposer.library main program to be called
** if not called as a library. */

#include <exec/types.h>
#include <tucomposer/tucomposer.h>
#include <clib/exec_protos.h>
#include <clib/tucomposer_protos.h>

#ifdef __cplusplus
extern "C" {
#endif

#ifndef MOTOROLA
  UWORD ReadMWord ( UBYTE *Buffer );
  void WriteMWord ( UBYTE *Buffer, UWORD Value );
  ULONG ReadMLong ( UBYTE *Buffer );
  void WriteMLong ( UBYTE *Buffer, ULONG Value );
#endif

#ifdef __cplusplus
}
#endif

struct TuComposerBase *Initialize ( struct TuComposerBase *TuCBase );

struct InternalInit {
  struct TuComposerExtList *inti_InternalInit;
  ULONG inti_InternalInitType;
};

extern struct ExecBase *SysBase;
extern struct TuComposerBase *TuCBase;
extern struct Resident *ROMTag;

extern struct TuComposerExtList *InternalSoundDevice;
extern struct TuComposerExtList *InternalMixer;
extern struct TuComposerExtList *InternalPostProc;
extern struct TuComposerExtList *InternalTCM;
extern struct TuComposerExtList *InternalTCS;
extern struct TuComposerExtList *InternalTCT;
extern struct TuComposerExtList *InternalTCI;
extern struct TuComposerExtList *InternalTCY;

extern UWORD *LinearFreqTable;
extern ULONG *PitchTable;
extern ULONG FXTable;
extern ULONG SETab;

/* Change the following '#if 0' into '#if 1' if you don't have coded
   any internal default drivers for your OS yet */

#if 0
struct {
  struct InternalInit inti_InitLib0;
  struct InternalInit inti_InitLib1;
  struct InternalInit inti_InitLib2;
  struct InternalInit inti_InitLib3;
  struct InternalInit inti_InitLib4;
  struct InternalInit inti_InitLib5;
  struct InternalInit inti_InitLib6;
  struct InternalInit inti_InitLib7;
  APTR inti_Terminator;
} InternalInitTable =
{
  {(struct TuComposerExtList *) &InternalSoundDevice, EXTTYPE_DEVICE},
  {(struct TuComposerExtList *) &InternalMixer, EXTTYPE_MIXER},
  {(struct TuComposerExtList *) &InternalPostProc, EXTTYPE_POSTPROC},
  {(struct TuComposerExtList *) &InternalTCM, EXTTYPE_MODULE},
  {(struct TuComposerExtList *) &InternalTCS, EXTTYPE_SONG},
  {(struct TuComposerExtList *) &InternalTCT, EXTTYPE_TRACK},
  {(struct TuComposerExtList *) &InternalTCI, EXTTYPE_INSTR},
  {(struct TuComposerExtList *) &InternalTCY, EXTTYPE_SYNTH},
  NULL
};
#else
  extern struct InternalInit *InternalInitTable;
#endif

/* main() is only called if the library is started
   from command line. Just exit here */

int main ( void )
{
  return ( -1 );
}

#ifndef MOTOROLA

UWORD ReadMWord ( UBYTE *Buffer )
{
  UWORD MotorolaWord;

  MotorolaWord = *Buffer++ << 8U;
  MotorolaWord += *Buffer;

  return ( MotorolaWord );
}

void WriteMWord ( UBYTE *Buffer, UWORD Value )
{
  *Buffer++ = Value >> 8U;
  *Buffer = Value;
}

ULONG ReadMLong ( UBYTE *Buffer )
{
  ULONG MotorolaLong;

  MotorolaLong = *Buffer++;
  MotorolaLong <<= 8UL;
  MotorolaLong += *Buffer++;
  MotorolaLong <<= 8UL;
  MotorolaLong += *Buffer++;
  MotorolaLong <<= 8UL;
  MotorolaLong += *Buffer;

  return ( MotorolaLong );
}

void WriteMLong ( UBYTE *Buffer, ULONG Value )
{
  *Buffer++ = Value >> 24UL;
  *Buffer++ = Value >> 16UL;
  *Buffer++ = Value >> 8UL;
  *Buffer = Value;
}

#endif

ULONG OldSTTable[] = {
  192345259,
  96192529,
  64123930,
  48096264,
  38475419,
  32061964,
  27482767,
  24048132,
  21687744,
  19240098
};

WORD Sin15Table[] =
{
  0, 571, 1143, 1714, 2285, 2855, 3425, 3993, 4560, 5125, 5689, 6252, 6812, 7370, 7927, 8480,
  9031, 9580, 10125, 10667, 11206, 11742, 12274, 12803, 13327, 13847, 14364, 14875, 15383, 15885, 16383, 16876,
  17363, 17846, 18323, 18794, 19259, 19719, 20173, 20620, 21062, 21497, 21925, 22347, 22761, 23169, 23570, 23964,
  24350, 24729, 25100, 25464, 25820, 26168, 26509, 26841, 27165, 27480, 27787, 28086, 28377, 28658, 28931, 29195,
  29450, 29696, 29934, 30162, 30381, 30590, 30790, 30981, 31163, 31335, 31497, 31650, 31793, 31927, 32050, 32164,
  32269, 32363, 32448, 32522, 32587, 32642, 32687, 32722, 32747, 32762, 32767, 32762, 32747, 32722, 32687, 32642,
  32587, 32522, 32448, 32363, 32269, 32164, 32050, 31927, 31793, 31650, 31497, 31335, 31163, 30981, 30790, 30590,
  30381, 30162, 29934, 29696, 29450, 29195, 28931, 28658, 28377, 28086, 27787, 27480, 27165, 26841, 26509, 26168,
  25820, 25464, 25100, 24729, 24350, 23964, 23570, 23169, 22761, 22347, 21925, 21497, 21062, 20620, 20173, 19719,
  19259, 18794, 18323, 17846, 17363, 16876, 16383, 15885, 15383, 14875, 14364, 13847, 13327, 12803, 12274, 11742,
  11206, 10667, 10125, 9580, 9031, 8480, 7927, 7370, 6812, 6252, 5689, 5125, 4560, 3993, 3425, 2855,
  2285, 1714, 1143, 571, 0, -571, -1143, -1714, -2285, -2855, -3425, -3993, -4560, -5125, -5689, -6252,
  -6812, -7370, -7927, -8480, -9031, -9580, -10125, -10667, -11206, -11742, -12274, -12803, -13327, -13847, -14364, -14875,
  -15383, -15885, -16383, -16876, -17363, -17846, -18323, -18794, -19259, -19719, -20173, -20620, -21062, -21497, -21925, -22347,
  -22761, -23169, -23570, -23964, -24350, -24729, -25100, -25464, -25820, -26168, -26509, -26841, -27165, -27480, -27787, -28086,
  -28377, -28658, -28931, -29195, -29450, -29696, -29934, -30162, -30381, -30590, -30790, -30981, -31163, -31335, -31497, -31650,
  -31793, -31927, -32050, -32164, -32269, -32363, -32448, -32522, -32587, -32642, -32687, -32722, -32747, -32762, -32767, -32762,
  -32747, -32722, -32687, -32642, -32587, -32522, -32448, -32363, -32269, -32164, -32050, -31927, -31793, -31650, -31497, -31335,
  -31163, -30981, -30790, -30590, -30381, -30162, -29934, -29696, -29450, -29195, -28931, -28658, -28377, -28086, -27787, -27480,
  -27165, -26841, -26509, -26168, -25820, -25464, -25100, -24729, -24350, -23964, -23570, -23169, -22761, -22347, -21925, -21497,
  -21062, -20620, -20173, -19719, -19259, -18794, -18323, -17846, -17363, -16876, -16383, -15885, -15383, -14875, -14364, -13847,
  -13327, -12803, -12274, -11742, -11206, -10667, -10125, -9580, -9031, -8480, -7927, -7370, -6812, -6252, -5689, -5125,
  -4560, -3993, -3425, -2855, -2285, -1714, -1143, -571
};

struct SynthTable SynthTable[] = {
  {"STOP", TCYINST_STOP, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_SrcNotReq},
  {"END", TCYINST_END, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_SrcNotReq},
  {"KILL", TCYINST_KILL, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData},
  {"WAIT", TCYINST_WAIT, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData},
  {"WAITVOL", TCYINST_WAITVOL, STABF_Source|STABF_SrcLine|STABF_NoSrcV0|STABF_NoSrcData},
  {"WAITPAN", TCYINST_WAITPAN, STABF_Source|STABF_SrcLine|STABF_NoSrcV0|STABF_NoSrcData},
  {"WAITSLD", TCYINST_WAITSLD, STABF_Source|STABF_SrcLine|STABF_NoSrcV0|STABF_NoSrcData},
  {"WAITSPC", TCYINST_WAITSPC, STABF_Source|STABF_SrcLine|STABF_NoSrcV0|STABF_NoSrcData},
  {"JUMP", TCYINST_JUMP, STABF_Source|STABF_SrcLine|STABF_NoSrcV0|STABF_NoSrcData},
  {"JUMPEQ", TCYINST_JUMPEQ, STABF_Source|STABF_SrcLine|STABF_NoSrcV0|STABF_NoSrcData},
  {"JUMPNE", TCYINST_JUMPNE, STABF_Source|STABF_SrcLine|STABF_NoSrcV0|STABF_NoSrcData},
  {"JUMPPL", TCYINST_JUMPPL, STABF_Source|STABF_SrcLine|STABF_NoSrcV0|STABF_NoSrcData},
  {"JUMPMI", TCYINST_JUMPMI, STABF_Source|STABF_SrcLine|STABF_NoSrcV0|STABF_NoSrcData},
  {"JUMPLT", TCYINST_JUMPLT, STABF_Source|STABF_SrcLine|STABF_NoSrcV0|STABF_NoSrcData},
  {"JUMPLE", TCYINST_JUMPLE, STABF_Source|STABF_SrcLine|STABF_NoSrcV0|STABF_NoSrcData},
  {"JUMPGT", TCYINST_JUMPGT, STABF_Source|STABF_SrcLine|STABF_NoSrcV0|STABF_NoSrcData},
  {"JUMPGE", TCYINST_JUMPGE, STABF_Source|STABF_SrcLine|STABF_NoSrcV0|STABF_NoSrcData},
  {"JUMPVS", TCYINST_JUMPVS, STABF_Source|STABF_SrcLine|STABF_NoSrcV0|STABF_NoSrcData},
  {"JUMPVC", TCYINST_JUMPVC, STABF_Source|STABF_SrcLine|STABF_NoSrcV0|STABF_NoSrcData},
  {"JUMPCS", TCYINST_JUMPCS, STABF_Source|STABF_SrcLine|STABF_NoSrcV0|STABF_NoSrcData},
  {"JUMPLO", TCYINST_JUMPLO, STABF_Source|STABF_SrcLine|STABF_NoSrcV0|STABF_NoSrcData},
  {"JUMPCC", TCYINST_JUMPCC, STABF_Source|STABF_SrcLine|STABF_NoSrcV0|STABF_NoSrcData},
  {"JUMPHS", TCYINST_JUMPHS, STABF_Source|STABF_SrcLine|STABF_NoSrcV0|STABF_NoSrcData},
  {"JUMPLS", TCYINST_JUMPLS, STABF_Source|STABF_SrcLine|STABF_NoSrcV0|STABF_NoSrcData},
  {"JUMPHI", TCYINST_JUMPHI, STABF_Source|STABF_SrcLine|STABF_NoSrcV0|STABF_NoSrcData},
  {"JUMPVOL", TCYINST_JUMPVOL, STABF_Source|STABF_SrcLine|STABF_NoSrcV0|STABF_NoSrcData},
  {"JUMPPAN", TCYINST_JUMPPAN, STABF_Source|STABF_SrcLine|STABF_NoSrcV0|STABF_NoSrcData},
  {"JUMPSLD", TCYINST_JUMPSLD, STABF_Source|STABF_SrcLine|STABF_NoSrcV0|STABF_NoSrcData},
  {"JUMPSPC", TCYINST_JUMPSPC, STABF_Source|STABF_SrcLine|STABF_NoSrcV0|STABF_NoSrcData},
  {"CALL", TCYINST_CALL, STABF_Source|STABF_SrcLine|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"RETURN", TCYINST_RETURN, STABF_Source|STABF_SrcLine|STABF_NoSrcData|STABF_Dest},
  {"POSVAR", TCYINST_POSVAR, STABF_Source|STABF_SrcLine|STABF_NoSrcData|STABF_Dest},
  {"LOAD", TCYINST_LOAD, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"ADD", TCYINST_ADD, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"ADDX", TCYINST_ADDX, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"SUB", TCYINST_SUB, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"SUBX", TCYINST_SUBX, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"CMP", TCYINST_CMP, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"MULU", TCYINST_MULU, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"MULS", TCYINST_MULS, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"DMULU", TCYINST_DMULU, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest|STABF_DestDbl},
  {"DMULS", TCYINST_DMULS, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest|STABF_DestDbl},
  {"DIVU", TCYINST_DIVU, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"DIVS", TCYINST_DIVS, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"MODU", TCYINST_MODU, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"MODS", TCYINST_MODS, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"DDIVU", TCYINST_DDIVU, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest|STABF_DestDbl},
  {"DDIVS", TCYINST_DDIVS, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest|STABF_DestDbl},
  {"ASHL", TCYINST_ASHL, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"ASHR", TCYINST_ASHR, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"LSHL", TCYINST_LSHL, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"LSHR", TCYINST_LSHR, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"ROL", TCYINST_ROL, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"ROR", TCYINST_ROR, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"ROLX", TCYINST_ROLX, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"RORX", TCYINST_RORX, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"OR", TCYINST_OR, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"AND", TCYINST_AND, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"XOR", TCYINST_XOR, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"NOT", TCYINST_NOT, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest|STABF_VarPri|STABF_SrcNotReq},
  {"NEG", TCYINST_NEG, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest|STABF_VarPri|STABF_SrcNotReq},
  {"NEGX", TCYINST_NEGX, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest|STABF_VarPri|STABF_SrcNotReq},
  {"EXTB", TCYINST_EXTB, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest|STABF_VarPri|STABF_SrcNotReq},
  {"EXT", TCYINST_EXT, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest|STABF_DestDbl|STABF_VarPri|STABF_SrcNotReq},
  {"XCHG", TCYINST_XCHG, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest|STABF_DestDbl|STABF_VarPri|STABF_SrcNotReq},
  {"SWAP", TCYINST_SWAP, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest|STABF_VarPri|STABF_SrcNotReq},
  {"GETWAVE", TCYINST_GETWAVE, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"GETWLEN", TCYINST_GETWLEN, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"GETWPOS", TCYINST_GETWPOS, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"GETCHAN", TCYINST_GETCHAN, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"GETNOTE", TCYINST_GETNOTE, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"GETRANS", TCYINST_GETRANS, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"GETPTCH", TCYINST_GETPTCH, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest|STABF_DestDbl},
  {"GETPER", TCYINST_GETPER, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest|STABF_DestDbl},
  {"GETFX", TCYINST_GETFX, STABF_Source|STABF_NoSrcData|STABF_Dest|STABF_VarPri},
  {"GETARPW", TCYINST_GETARPW, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"GETARPV", TCYINST_GETARPV, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"GETARPL", TCYINST_GETARPL, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"GETARPP", TCYINST_GETARPP, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"GETVIBW", TCYINST_GETVIBW, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"GETVIBV", TCYINST_GETVIBV, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"GETVIBL", TCYINST_GETVIBL, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"GETVIBP", TCYINST_GETVIBP, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"GETTRMW", TCYINST_GETTRMW, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"GETTRMV", TCYINST_GETTRMV, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"GETTRML", TCYINST_GETTRML, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"GETTRMP", TCYINST_GETTRMP, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"GETPANW", TCYINST_GETPANW, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"GETPANV", TCYINST_GETPANV, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"GETPANL", TCYINST_GETPANL, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"GETPANP", TCYINST_GETPANP, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest},
  {"GETRND", TCYINST_GETRND, STABF_Source|STABF_NoSrcV0|STABF_Dest},
  {"GETSINE", TCYINST_GETSINE, STABF_Source|STABF_NoSrcV0|STABF_Dest},
  {"PORTAUP", TCYINST_PORTAUP, STABF_Source|STABF_NoSrcV0},
  {"PORTADN", TCYINST_PORTADN, STABF_Source|STABF_NoSrcV0},
  {"VIBSPD", TCYINST_VIBSPD, STABF_Source|STABF_NoSrcV0},
  {"VIBDPTH", TCYINST_VIBDPTH, STABF_Source|STABF_NoSrcV0},
  {"VIBWAVE", TCYINST_VIBWAVE, STABF_Source|STABF_NoSrcV0},
  {"VIBWAVP", TCYINST_VIBWAVP, STABF_Source|STABF_NoSrcV0},
  {"VIBRATO", TCYINST_VIBRATO, STABF_Source|STABF_NoSrcV0},
  {"VIBVAL", TCYINST_VIBVAL, STABF_Source|STABF_NoSrcV0},
  {"ARPSPD", TCYINST_ARPSPD, STABF_Source|STABF_NoSrcV0},
  {"ARPWAVE", TCYINST_ARPWAVE, STABF_Source|STABF_NoSrcV0},
  {"ARPWAVP", TCYINST_ARPWAVP, STABF_Source|STABF_NoSrcV0},
  {"ARPEGIO", TCYINST_ARPEGIO, STABF_Source|STABF_NoSrcV0},
  {"ARPVAL", TCYINST_ARPVAL, STABF_Source|STABF_NoSrcV0},
  {"SETWAVE", TCYINST_SETWAVE, STABF_Source|STABF_NoSrcV0},
  {"ISETWAV", TCYINST_ISETWAV, STABF_Source|STABF_NoSrcV0},
  {"SETWAVP", TCYINST_SETWAVP, STABF_Source|STABF_NoSrcV0},
  {"SETRANS", TCYINST_SETRANS, STABF_Source|STABF_NoSrcV0},
  {"SETNOTE", TCYINST_SETNOTE, STABF_Source|STABF_NoSrcV0},
  {"SETPTCH", TCYINST_SETPTCH, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest|STABF_DestDbl},
  {"SETPER", TCYINST_SETPER, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_Dest|STABF_DestDbl},
  {"RESET", TCYINST_RESET, STABF_Source|STABF_NoSrcV0|STABF_NoSrcData|STABF_SrcNotReq},
  {"VOLSLUP", TCYINST_VOLSLUP, STABF_Source|STABF_NoSrcV0},
  {"VOLSLDN", TCYINST_VOLSLDN, STABF_Source|STABF_NoSrcV0},
  {"TRMSPD", TCYINST_TRMSPD, STABF_Source|STABF_NoSrcV0},
  {"TRMDPTH", TCYINST_TRMDPTH, STABF_Source|STABF_NoSrcV0},
  {"TRMWAVE", TCYINST_TRMWAVE, STABF_Source|STABF_NoSrcV0},
  {"TRMWAVP", TCYINST_TRMWAVP, STABF_Source|STABF_NoSrcV0},
  {"TREMOLO", TCYINST_TREMOLO, STABF_Source|STABF_NoSrcV0},
  {"TRMVAL", TCYINST_TRMVAL, STABF_Source|STABF_NoSrcV0},
  {"PANLEFT", TCYINST_PANLEFT, STABF_Source|STABF_NoSrcV0},
  {"PANRGHT", TCYINST_PANRGHT, STABF_Source|STABF_NoSrcV0},
  {"PANSPD", TCYINST_PANSPD, STABF_Source|STABF_NoSrcV0},
  {"PANDPTH", TCYINST_PANDPTH, STABF_Source|STABF_NoSrcV0},
  {"PANWAVE", TCYINST_PANWAVE, STABF_Source|STABF_NoSrcV0},
  {"PANWAVP", TCYINST_PANWAVP, STABF_Source|STABF_NoSrcV0},
  {"PANNOLO", TCYINST_PANNOLO, STABF_Source|STABF_NoSrcV0},
  {"PANVAL", TCYINST_PANVAL, STABF_Source|STABF_NoSrcV0},
  NULL
};

struct TuComposerBase *Initialize ( struct TuComposerBase *TuCBase )
{
  struct InternalInit *InternalInitLibs;
  struct TuComposerExtList *InternalLib;

  if (!( TuCBase->tcb_ModuleListSigSem = (struct SignalSemaphore *) AllocPMem ( sizeof ( struct SignalSemaphore))))
    goto PrepLibError;

  InitSemaphore ( TuCBase->tcb_ModuleListSigSem );

  if (!( TuCBase->tcb_PlaySigSem = (struct SignalSemaphore *) AllocPMem ( sizeof ( struct SignalSemaphore))))
    goto PrepLibError;

  InitSemaphore ( TuCBase->tcb_PlaySigSem );

  if (!( TuCBase->tcb_ModuleSigSem = (struct SignalSemaphore *) AllocPMem ( sizeof ( struct SignalSemaphore))))
    goto PrepLibError;

  InitSemaphore ( TuCBase->tcb_ModuleSigSem );

  if (!( TuCBase->tcb_SongSigSem = (struct SignalSemaphore *) AllocPMem ( sizeof ( struct SignalSemaphore))))
    goto PrepLibError;

  InitSemaphore ( TuCBase->tcb_SongSigSem );

  if (!( TuCBase->tcb_TrackSigSem = (struct SignalSemaphore *) AllocPMem ( sizeof ( struct SignalSemaphore))))
    goto PrepLibError;

  InitSemaphore ( TuCBase->tcb_TrackSigSem );

  if (!( TuCBase->tcb_InstrSigSem = (struct SignalSemaphore *) AllocPMem ( sizeof ( struct SignalSemaphore))))
    goto PrepLibError;

  InitSemaphore ( TuCBase->tcb_InstrSigSem );

  if (!( TuCBase->tcb_SampleSigSem = (struct SignalSemaphore *) AllocPMem ( sizeof ( struct SignalSemaphore))))
    goto PrepLibError;

  InitSemaphore ( TuCBase->tcb_SampleSigSem );

  if (!( TuCBase->tcb_SynthSigSem = (struct SignalSemaphore *) AllocPMem ( sizeof ( struct SignalSemaphore))))
    goto PrepLibError;

  InitSemaphore ( TuCBase->tcb_SynthSigSem );

  if (!( TuCBase->tcb_WavesSigSem = (struct SignalSemaphore *) AllocPMem ( sizeof ( struct SignalSemaphore))))
    goto PrepLibError;

  InitSemaphore ( TuCBase->tcb_WavesSigSem );

  if (!( TuCBase->tcb_ExternalSigSem = (struct SignalSemaphore *) AllocPMem ( sizeof ( struct SignalSemaphore))))
    goto PrepLibError;

  InitSemaphore ( TuCBase->tcb_ExternalSigSem );

  TuCBase->tcb_SineTable = (WORD *) &Sin15Table;
  TuCBase->tcb_LinearFreqTable = (UWORD *) &LinearFreqTable;
  TuCBase->tcb_FrequencyTable = (ULONG *) (&PitchTable) + 1;
  TuCBase->tcb_OldSTTable = (ULONG *) &OldSTTable;
  TuCBase->tcb_EffectsTable = (APTR) &FXTable;
  TuCBase->tcb_SynthInstrTable = (struct SynthTable *) &SynthTable;
  TuCBase->tcb_SynthExecTable = (ULONG *) &SETab;

  Randomize ( 0L );

  InitNodeList ( (struct List *) &(TuCBase->tcb_ModuleList));
  InitNodeList ( (struct List *) &(TuCBase->tcb_ExecList));
  InitNodeList ( (struct List *) &(TuCBase->tcb_ModuleBuf));
  InitNodeList ( (struct List *) &(TuCBase->tcb_SongBuf));
  InitNodeList ( (struct List *) &(TuCBase->tcb_TrackBuf));
  InitNodeList ( (struct List *) &(TuCBase->tcb_InstrBuf));
  InitNodeList ( (struct List *) &(TuCBase->tcb_SampleBuf));
  InitNodeList ( (struct List *) &(TuCBase->tcb_SynthBuf));
  InitNodeList ( (struct List *) &(TuCBase->tcb_WavesBuf));
  InitNodeList ( (struct List *) &(TuCBase->tcb_ExtCrunch));
  InitNodeList ( (struct List *) &(TuCBase->tcb_ExtDevice));
  InitNodeList ( (struct List *) &(TuCBase->tcb_ExtRecord));
  InitNodeList ( (struct List *) &(TuCBase->tcb_ExtMixer));
  InitNodeList ( (struct List *) &(TuCBase->tcb_ExtPostProc));
  InitNodeList ( (struct List *) &(TuCBase->tcb_ExtModule));
  InitNodeList ( (struct List *) &(TuCBase->tcb_ExtSong));
  InitNodeList ( (struct List *) &(TuCBase->tcb_ExtTrack));
  InitNodeList ( (struct List *) &(TuCBase->tcb_ExtInstr));
  InitNodeList ( (struct List *) &(TuCBase->tcb_ExtSample));
  InitNodeList ( (struct List *) &(TuCBase->tcb_ExtSynth));

  GetExternalLibs ( NULL );

  InternalInitLibs = (struct InternalInit *) &InternalInitTable;

  for (;;)
  {
    if (!( InternalLib = InternalInitLibs->inti_InternalInit ))
      break;

    AddExternalLib ( InternalLib, InternalInitLibs->inti_InternalInitType );
    InternalLib->extl_LibBase->extb_TuComposerBase = TuCBase;

    AddLibrary ( (struct Library *) InternalLib->extl_LibBase );

    InternalInitLibs++;
  }

  return ( TuCBase );

PrepLibError:
  FreePMem ( TuCBase->tcb_ExternalSigSem, sizeof ( struct SignalSemaphore ));
  FreePMem ( TuCBase->tcb_WavesSigSem, sizeof ( struct SignalSemaphore ));
  FreePMem ( TuCBase->tcb_SynthSigSem, sizeof ( struct SignalSemaphore ));
  FreePMem ( TuCBase->tcb_SampleSigSem, sizeof ( struct SignalSemaphore ));
  FreePMem ( TuCBase->tcb_InstrSigSem, sizeof ( struct SignalSemaphore ));
  FreePMem ( TuCBase->tcb_TrackSigSem, sizeof ( struct SignalSemaphore ));
  FreePMem ( TuCBase->tcb_SongSigSem, sizeof ( struct SignalSemaphore ));
  FreePMem ( TuCBase->tcb_ModuleSigSem, sizeof ( struct SignalSemaphore ));
  FreePMem ( TuCBase->tcb_PlaySigSem, sizeof ( struct SignalSemaphore ));
  FreePMem ( TuCBase->tcb_ModuleListSigSem, sizeof ( struct SignalSemaphore ));

  return ( NULL );
}
