#ifndef TUCOMPOSER_EXTERNAL_DEVICE_H
#define TUCOMPOSER_EXTERNAL_DEVICE_H
/*
**  $VER: device.h 31.22 (2.4.99)
**
**  Standard C header for tucomposer.library
**
**  (C) Copyright 1999 by Basty/Seasons
**      All Rights Reserved
*/

#ifndef EXEC_TYPES_H
#include <exec/types.h>
#endif /* EXEC_TYPES_H */

#ifndef EXEC_LIBRARIES_H
#include <exec/libraries.h>
#endif /* EXEC_LIBRARIES_H */

#ifndef EXEC_LISTS_H
#include <exec/lists.h>
#endif /* EXEC_LISTS_H */

#ifndef EXEC_TASKS_H
#include <exec/tasks.h>
#endif /* EXEC_TASKS_H */

#ifndef LIBRARIES_DOS_H
#include <libraries/dos.h>
#endif /* LIBRARIES_DOS_H */

#ifndef TUCOMPOSER_EXTERNAL_EXTERNAL_H
#include <tucomposer/external/external.h>
#endif /* TUCOMPOSER_EXTERNAL_EXTERNAL_H */

/* Sound device info information flags (read-only for you): */
#define SDIB_Allocated      0   /* Soundcard allocated */
#define SDIB_Playing        1   /* Soundcard is currently playing */
#define SDIB_Stereo         2   /* Soundcard is capable of stereo output */
#define SDIB_Interrupt      3   /* Soundcard is interrupt driven */
#define SDIB_NoMixer        4   /* Soundcard does not support
                                   software mixing */

#define SDIF_Allocated      (1<<SDIB_Allocated)
#define SDIF_Playing        (1<<SDIB_Playing)
#define SDIF_Stereo         (1<<SDIB_Stereo)
#define SDIF_Interrupt      (1<<SDIB_Interrupt)
#define SDIF_NoMixer        (1<<SDIB_NoMixer)

/* Channel flags (sdi_ChannelFlags) bit definitions: */
#define SDICB_Stereo        0   /* Mix buffer contains stereo data */

#define SDICF_Stereo        (1<<SDICB_Stereo)

/* Currently the structure is only coded for mixing devices, that means,
   you should always use an mixing device to output sound. I decided this,
   because the AMIGA currently has not a soundcard like the PC with 32
   hardware channels (like the GUS), so I don't plan to implement full sound-
   card drivers until the AMIGA gets such a soundcard. Maybe there will be
   some additions in v32 of the library. But don't count on it. Also remember
   that for any hardware related drivers, an exec device is a better
   solution. */

/* Sound device info structure (after library structure): */
struct SoundDeviceInfo {
  struct TuComposerExtLibBase sdi_Library;
  UBYTE  sdi_Flags;      /* Soundcard information flags (only readable) */
  UBYTE  sdi_Pad;        /* To longword align the following */
  ULONG  sdi_LowFrequency; /* Lowest possible frequency of soundcard */
  ULONG  sdi_HighFrequency; /* Highest possible frequency of soundcard */
  struct Task *sdi_Task; /* Ptr to the task using the soundcard or NULL if
                            allocated by an unknown resource */
  ULONG  sdi_ChannelBufLen; /* Buffer length of channels */
  ULONG  sdi_ChannelFreq; /* Sample frequency (in Hz) of channels */
  LONG   *sdi_ChannelBuf; /* Mixing buffer data */
  UBYTE  sdi_ChannelFlags; /* Flags of channels */
  UBYTE  sdi_Reserved[15]; /* Reserved for future expansions */
}; /* SoundDeviceInfo */

/* Error codes returned by soundcard devices. */

#define ERROR_EXTLIB_BUSY     ERROR_EXTLIB+0x8  /* Soundcard is busy by
                                                   another program */
#define ERROR_EXTLIB_NOALLOC  ERROR_EXTLIB+0x9  /* Soundcard not allocated */

#define EXTLIB_DEVICE_VERSION 31   /* Current external device library
                                      version */
#define EXTLIB_DEVICE_REVISION 0   /* Current external device library
                                      revision */

#endif /* TUCOMPOSER_EXTERNAL_DEVICE_H */
