#ifndef TUCOMPOSER_EXTERNAL_EXTERNAL_H
#define TUCOMPOSER_EXTERNAL_EXTERNAL_H
/*
**  $VER: external.h 31.22 (29.3.99)
**
**  Standard C header for tucomposer.library
**
**  (C) Copyright 1999 by Basty/Seasons
**      All Rights Reserved
*/

#ifndef EXEC_TYPES_H
#include <exec/types.h>
#endif /* EXEC_TYPES_H */

#ifndef EXEC_LIBRARIES_H
#include <exec/libraries.h>
#endif /* EXEC_LIBRARIES_H */

#ifndef EXEC_LISTS_H
#include <exec/lists.h>
#endif /* EXEC_LISTS_H */

#ifndef LIBRARIES_DOS_H
#include <libraries/dos.h>
#endif /* LIBRARIES_DOS_H */

#ifndef TUCOMPOSER_TUCOMPOSER_H
#include <tucomposer/tucomposer.h>
#endif /* TUCOMPOSER_TUCOMPOSER_H */

/* The external libraries ables tucomposer.library to load alien modules,
   tracks, samples, instruments, etc. So you can use an XM-loader, for
   example to import/export FastTracker ][ (from PC, but a great program !!!)
   XM modules. So tucomposer.library can act like a module converter, just
   add some external loaders, load the modules, and save them in an other
   format. Because TuComposer is much more powerful than any other trackers
   I know of, it shouldn't be a problem to import FastTracker ][,
   ScreamTracker ]I[, TFMX, StoneTracker, etc. modules. When exporting,
   don't forget the restrictions of some formats (e.g. ScreamTracker can't
   handle 8-bit panning, for example) and none of the above doesn't support
   8-bit volumes (maybe TFMX can do, as I don't know about the abilities of
   this format), and ProTracker (clones) just handle octaves 3-5 and only
   4 channels, while the others can handle up to 32. All except StoneTracker
   (and maybe TFMX) don't support track ordering, they use all pattern
   ordering, which is very unflexible compared to TCM. The hardest thing to
   do, is to import from TCM and export to ProTracker. */

/* extb_Flags field: */
#define EXTLB_Intern         0  /* If bit set, this is an internal library */
#define EXTLB_Disabled       1  /* If bit set, library has been disabled */
#define EXTLB_Loader         2  /* If bit is set, this is a loader */
#define EXTLB_Saver          3  /* If bit is set, this is a saver */

#define EXTLF_Intern         (1<<EXTLB_Intern)
#define EXTLF_Disabled       (1<<EXTLB_Disabled)
#define EXTLF_Loader         (1<<EXTLB_Loader)
#define EXTLF_Saver          (1<<EXTLB_Saver)

/* tefs_FlagxFlags field: */
#define TEFSB_Update         0  /* If set, app must call
                                   ExternalUpdateFlags() on flag change */
#define TEFSB_Hidden         1  /* Entering data is hidden (e.g. password) */
#define TEFSB_FixedValues    2  /* Integer values are fixed 16.16 integers,
                                   i.e. integers with 1/65536th accuracy. */
#define TEFSB_Disabled       7  /* Disabled function. Allows you to code
                                   limited shareware versions, or if it's
                                   not possible to set this (e.g. because you
                                   don't have the hardware) */

#define TEFSF_Update         (1<<TEFSB_Update)
#define TEFSF_Hidden         (1<<TEFSB_Hidden)
#define TEFSF_FixedValues    (1<<TEFSB_FixedValues)
#define TEFSF_Disabled       (1<<TEFSB_Disabled)

/* Flag structure. Types are usually represented by the desired intuition
   gadgets. It is on you to display these properly. I suggest you using
   MUI, because it does almost all for you. */
struct TuComposerExtFlagStr {
  UWORD  tefs_NumEntries;       /* Number of flags */
  UWORD  tefs_StructSize;       /* Size of the structure (for FreeMem()) */
  union FlagEntryStruct {
    struct FlagInteger {
      STRPTR tefs_FlagText;     /* Flag description text */
      UBYTE  tefs_FlagType;     /* Type of this flag */
      UBYTE  tefs_FlagFlags;    /* Flag special flags (look below) */
      UWORD  tefs_FlagLen;      /* Flag Length of int/etc. flag types.
                                   If 0, it's just a view gadget, which
                                   simply shows some informations */
      LONG   tefs_FlagLowest;   /* Flag lowest value (int/slider types) */
      LONG   tefs_FlagHighest;  /* Flag highest value (int/slider types) */
      LONG   tefs_FlagDefault;  /* Flag default value */
    } tefs_FlagInteger;
    struct FlagString {
      STRPTR tefs_FlagText;     /* Flag description text */
      UBYTE  tefs_FlagType;     /* Type of this flag */
      UBYTE  tefs_FlagFlags;    /* Flag special flags (look below) */
      UWORD  tefs_FlagLen;      /* Flag Length of string/file/etc. flag
                                   types. If 0, it's just a view gadget
                                   which simply shows some informations */
      LONG   tefs_FlagLowest;   /* Flag lowest value (unused by strings) */
      LONG   tefs_FlagHighest;  /* Flag highest value (unused by strings) */
      STRPTR tefs_FlagDefault;  /* Flag default string */
    } tefs_FlagString;
    struct FlagSlider {
      STRPTR tefs_FlagText;     /* Flag description text */
      UBYTE  tefs_FlagType;     /* Type of this flag */
      UBYTE  tefs_FlagFlags;    /* Flag special flags (look below) */
      UWORD  tefs_FlagLen;      /* Number of slider entries or 0 if it's
                                   a smooth slider including any values from
                                   tefs_FlagLowest and tefs_FlagHighest
                                   inclusive, e.g. a frequency slider
                                   allowing only 11, 22 and 44kHz. */
      LONG   tefs_FlagLowest;   /* Flag lowest value (if array is unused) */
      LONG   tefs_FlagHighest;  /* Flag highest value (if array is unused) */
      ULONG  tefs_FlagDefault;  /* Flag default position */
      LONG  *tefs_FlagValues;   /* Pointer to slider values when
                                   tefs_FlagLen is not zero */
    } tefs_FlagSlider;
    struct FlagArray {
      STRPTR *tefs_FlagText;    /* Flag array of the strings which can be
                                   selected. */
      UBYTE  tefs_FlagType;     /* Type of this flag */
      UBYTE  tefs_FlagFlags;    /* Flag special flags (look below) */
      UWORD  tefs_FlagLen;      /* Size of the string array. */
      LONG   tefs_FlagLowest;   /* Flag lowest value (unused by arrays) */
      LONG   tefs_FlagHighest;  /* Flag highest value (unused by arrays) */
      ULONG  tefs_FlagDefault;  /* Flag default position */
    } tefs_FlagArray;
  } tefs_Flag[0];
}; /* TuComposerExtFlagStr */

/* Use these for auto initialization when coding external libraries. */

union FlagEntryBooleanStruct {
  struct FlagInteger tefs_FlagInteger;
  struct FlagString tefs_FlagString;
  struct FlagSlider tefs_FlagSlider;
  struct FlagArray tefs_FlagArray;
};

union FlagEntryTextStruct {
  struct FlagString tefs_FlagString;
  struct FlagInteger tefs_FlagInteger;
  struct FlagSlider tefs_FlagSlider;
  struct FlagArray tefs_FlagArray;
};

union FlagEntryScreenStruct {
  struct FlagString tefs_FlagString;
  struct FlagInteger tefs_FlagInteger;
  struct FlagSlider tefs_FlagSlider;
  struct FlagArray tefs_FlagArray;
};

union FlagEntryWindowStruct {
  struct FlagString tefs_FlagString;
  struct FlagInteger tefs_FlagInteger;
  struct FlagSlider tefs_FlagSlider;
  struct FlagArray tefs_FlagArray;
};

union FlagEntryIntegerStruct {
  struct FlagInteger tefs_FlagInteger;
  struct FlagString tefs_FlagString;
  struct FlagSlider tefs_FlagSlider;
  struct FlagArray tefs_FlagArray;
};

union FlagEntrySliderStruct {
  struct FlagSlider tefs_FlagSlider;
  struct FlagInteger tefs_FlagInteger;
  struct FlagString tefs_FlagString;
  struct FlagArray tefs_FlagArray;
};

union FlagEntryStringStruct {
  struct FlagString tefs_FlagString;
  struct FlagInteger tefs_FlagInteger;
  struct FlagSlider tefs_FlagSlider;
  struct FlagArray tefs_FlagArray;
};

union FlagEntryFileNameStruct {
  struct FlagString tefs_FlagString;
  struct FlagInteger tefs_FlagInteger;
  struct FlagSlider tefs_FlagSlider;
  struct FlagArray tefs_FlagArray;
};

union FlagEntryDirectoryStruct {
  struct FlagString tefs_FlagString;
  struct FlagInteger tefs_FlagInteger;
  struct FlagSlider tefs_FlagSlider;
  struct FlagArray tefs_FlagArray;
};

union FlagEntrySaveFileStruct {
  struct FlagString tefs_FlagString;
  struct FlagInteger tefs_FlagInteger;
  struct FlagSlider tefs_FlagSlider;
  struct FlagArray tefs_FlagArray;
};

union FlagEntryButtonStruct {
  struct FlagArray tefs_FlagArray;
  struct FlagInteger tefs_FlagInteger;
  struct FlagString tefs_FlagString;
  struct FlagSlider tefs_FlagSlider;
};

union FlagEntryCycleStruct {
  struct FlagArray tefs_FlagArray;
  struct FlagInteger tefs_FlagInteger;
  struct FlagString tefs_FlagString;
  struct FlagSlider tefs_FlagSlider;
};

union FlagEntryListViewStruct {
  struct FlagArray tefs_FlagArray;
  struct FlagInteger tefs_FlagInteger;
  struct FlagString tefs_FlagString;
  struct FlagSlider tefs_FlagSlider;
};

/* Flag pass structure. This structure is passed to the external libraries */
union FlagPass {
  LONG flg_Value;
  APTR flg_Pointer;
}; /* FlagPass */

/* External library list base structure. Use this structure to access the
   External (and internal) libraries. */
struct TuComposerExtList {
  struct MinNode extl_Node;
  UBYTE  extl_Flags;            /* Flags (look bitdefs upward) */
  BYTE   extl_Pri;              /* Priority (for sorting) */
  struct TuComposerExtLibBase *extl_LibBase; /* Ptr to external library base */
  STRPTR extl_Directory;        /* Directory where external lib is placed */
  STRPTR extl_LibName;          /* Pointer to external library name */
  STRPTR extl_Descriptor;       /* Loader descriptor (e.g. for FastTracker ][
                                   instrs: "FastTracker ][ Instrument") */
  struct TuComposerExtFlagStr *extl_LoadFlagStr; /* List to flag structure
                                   for use with your apps, allowing to
                                   configure your external loaders or NULL
                                   if you don't have any load flags */
  struct TuComposerExtFlagStr *extl_SaveFlagStr; /* Same as LoadFlagStr
                                   except for the save flags */
  UBYTE  extl_Reserved[8];      /* Reserved for future expansions */
}; /* TuComposerExtList */

/* Flag types */
#define EXTFLAG_BOOLEAN      0  /* Boolean (contains TRUE or FALSE) */
#define EXTFLAG_TEXT         1  /* Gadget which contains text in it's press-
                                   able borders which can change. This gadget
                                   requires the TEFSF_Update bit to be set.
                                   The flag pass structure must contain an
                                   pointer to a buffer with a size of
                                   tefs_FlagLen bytes, in which the new text
                                   will be stored. The buffer should
                                   initially be filled with
                                   tefs_FlagDefault. */
#define EXTFLAG_SCREEN       2  /* This is just a signal that here's (in the
                                   flag pass structure position) a screen
                                   pointer needed. If it's a public screen,
                                   you have to lock/unlock it yourself. */
#define EXTFLAG_WINDOW       3  /* This is just a signal that here's (flag
                                   pass struct) a window pointer needed. */
#define EXTFLAG_INTEGER      4  /* Integer (contains integer value) */
#define EXTFLAG_SLIDER       5  /* An slider with an value range */
#define EXTFLAG_STRING       6  /* String (contains any legal string) */
#define EXTFLAG_FILENAME     7  /* File name string (read access) */
#define EXTFLAG_DIRECTORY    8  /* Directory name string */
#define EXTFLAG_SAVEFILE     9  /* File name string (write access) */
#define EXTFLAG_BUTTON      10  /* Button type (tefs_FlagText is an array
                                   with tefs_FlagHighest entries). The first
                                   entry is the heading text */
#define EXTFLAG_CYCLE       11  /* Cycling (FlagText is also an array) */
#define EXTFLAG_LISTVIEW    12  /* List view (FlagText is also an array) */
#define EXTFLAG_RESERVED    13  /* Reserved for future expansions */

/* The external library base structure. THESE MUST BE PRESENT IN EACH
   EXTERNAL LIBRARY !!! */
struct TuComposerExtLibBase {
  struct Library extb_LibNode;
  BPTR  extb_SegList;           /* BPTR to DOS segments to UnLoadSeg() */
  struct TuComposerBase *extb_TuComposerBase; /* Ptr to TuComposer Library */
  struct TuComposerExtList *extb_ExtLibList; /* Ptr to external lib list */
}; /* TuComposerExtLibBase */

/* Minium external library supported by TuComposer. Libraries below these
   will be ignored */
#define EXTLIB_VERSION       31
#define EXTLIB_REVISION      0

/* External types */
#define EXTTYPE_CRUNCHER     1  /* A file (de)cruncher (XPK, PP, etc.) */
#define EXTTYPE_DEVICE       2  /* A soundcard device (for Toccata, etc.) */
#define EXTTYPE_RECORDER     3  /* A recording device (Mic-In, MIDI, etc.) */
#define EXTTYPE_MIXER        4  /* Mix devices (High quality mixers, etc.) */
#define EXTTYPE_POSTPROC     5  /* Post processor for mixer (echoes, etc.) */
#define EXTTYPE_MODULE       6  /* Module loader (TCM, XM, S3M, MOD, etc.) */
#define EXTTYPE_SONG         7  /* Song loader (TCM, IT, XM, MOD, etc.) */
#define EXTTYPE_TRACK        8  /* A track loader (TCT, XT, etc.) */
#define EXTTYPE_INSTR        9  /* An instr loader (TCI, XI, ITI, etc.) */
#define EXTTYPE_SAMPLE      10  /* A sample loader (IFF, WAV, S3I, etc.) */
#define EXTTYPE_SYNTH       11  /* An synth loader (TCY, MED-Synth, etc.) */
#define EXTTYPE_RESERVED    12  /* Reserved for future expansions */

/* External library errors used by all external libraries */
#define ERROR_EXTLIB_NOGIVEN ERROR_EXTLIB+0x0 /* External lib not given */
#define ERROR_EXTLIB_NOOPEN  ERROR_EXTLIB+0x1 /* External lib didn't open */
#define ERROR_EXTLIB_NOMEM   ERROR_EXTLIB+0x2 /* Not enough memory for any
                                                 reason. */

#endif /* TUCOMPOSER_EXTERNAL_EXTERNAL_H */
