#ifndef TUCOMPOSER_EXTERNAL_INSTR_H
#define TUCOMPOSER_EXTERNAL_INSTR_H
/*
**  $VER: instr.h 31.22 (3.4.99)
**
**  Standard C header for tucomposer.library
**
**  (C) Copyright 1999 by Basty/Seasons
**      All Rights Reserved
*/

#ifndef EXEC_TYPES_H
#include <exec/types.h>
#endif /* EXEC_TYPES_H */

#ifndef EXEC_LIBRARIES_H
#include <exec/libraries.h>
#endif /* EXEC_LIBRARIES_H */

#ifndef EXEC_LISTS_H
#include <exec/lists.h>
#endif /* EXEC_LISTS_H */

#ifndef LIBRARIES_DOS_H
#include <libraries/dos.h>
#endif /* LIBRARIES_DOS_H */

#ifndef TUCOMPOSER_TUCOMPOSER_H
#include <tucomposer/tucomposer.h>
#endif /* TUCOMPOSER_TUCOMPOSER_H */

#ifndef TUCOMPOSER_EXTERNAL_CRUNCHER_H
#include <tucomposer/external/cruncher.h>
#endif /* TUCOMPOSER_EXTERNAL_CRUNCHER_H */

/* InstrInfo structure. Got by GetInstrInfo(). Version and revision will
   be the same as that of the library if the instrument doesn't support this.
   All values will be the same as the IHDR fields (i.e. Flags, etc.). */

struct InstrInfo {
  STRPTR ii_FileName;          /* File name of instrument if supported */
  STRPTR ii_Name;              /* Name of instrument if supported */
  UWORD  ii_VolumeEnvelope;    /* Size of volume envelope (NULL if none) */
  UWORD  ii_PanningEnvelope;   /* Size of panning envelope (NULL if none) */
  UWORD  ii_SlideEnvelope;     /* Size of slide envelope (NULL if none) */
  UWORD  ii_VibratoEnvelope;   /* Size of vibrato envelope (NULL if none) */
  UWORD  ii_TremoloEnvelope;   /* Size of tremolo envelope (NULL if none) */
  UWORD  ii_PannoloEnvelope;   /* Size of pannolo envelope (NULL if none) */
  UWORD  ii_ChannoloEnvelope;  /* Size of channolo envelope (0 for none) */
  UWORD  ii_SpenoloEnvelope;   /* Size of spenolo envelope (NULL if none) */
  UWORD  ii_ArpeggioStructure; /* Size of arpeggio structure (0 if none) */
  UWORD  ii_KeyboardDefs;      /* Size of keyboard defs (0 if none) */
  UBYTE  ii_Version;           /* Version number of TCI1 file */
  UBYTE  ii_Revision;          /* Revision number of TCI1 file */
  UBYTE  ii_NumSamples;        /* Number of samples */
  UBYTE  ii_GlobalVolume;      /* Global volume for instrument samples */
  UBYTE  ii_NNA;               /* New note action */
  UBYTE  ii_NoteSwing;         /* Random note swing (semitones) */
  UWORD  ii_VolumeSwing;       /* Volume swing */
  UWORD  ii_PanningSwing;      /* Panning swing */
  ULONG  ii_PitchSwing;        /* Pitch swing */
  WORD   ii_PitchPanSeparation; /* Pitch panning separation */
  UBYTE  ii_DefaultPanning;    /* Default panning for all samples */
  UBYTE  ii_DefaultSubPan;     /* Default sub panning for all samples */
  UBYTE  ii_DCT;               /* Duplicate note check type (see upwards) */
  UBYTE  ii_DNA;               /* Duplicate note check action (see below) */
  UBYTE  ii_CompFlags;         /* Flags for compatibility (instr.i) */
  UBYTE  ii_Flags;             /* Special flags (see instr.i) */
  UWORD  ii_EnvUsageFlags;     /* Envelope usage flags */
  UWORD  ii_EnvProcFlags;      /* Envelope processing flags */
  UWORD  ii_EnvRetrigFlags;    /* Envelope retrigger flags */
  UWORD  ii_EnvRandomFlags;    /* Envelope randomize flags */
  UWORD  ii_EnvRndDelayFlags;  /* Envelope random speed is delay flags */
  UWORD  ii_FadeOut;           /* Fade out value */
  UWORD  ii_Hold;              /* Hold value */
  UWORD  ii_Decay;             /* Decay value */
  UBYTE  ii_DCA;               /* Decay action when decay is off */
  UBYTE  ii_PitchPanCenter;    /* Pitch panning center (0 - C-0, etc.) */
  UBYTE  ii_MIDICh;            /* MIDI channel */
  UBYTE  ii_MIDIProgram;       /* MIDI program (instrument) */
  UBYTE  ii_MIDIFlags;         /* MIDI flags (look upwards) */
  BYTE   ii_MIDITranspose;     /* MIDI transpose (in halftones) */
  UBYTE  ii_MIDIAftertouch;    /* MIDI aftertouch (percentage) */
  UBYTE  ii_MIDIBender;        /* MIDI pitchbender (in halftones) */
}; /* InstrInfo */

/* Instrument library structure (after library structure): */
struct InstrLibrary {
  struct TuComposerExtLibBase insl_Library;
}; /* InstrLibrary */

#define EXTLIB_INSTR_VERSION  31   /* Current external instr library
                                      version */
#define EXTLIB_INSTR_REVISION  0   /* Current external instr library
                                      revision */

#endif /* TUCOMPOSER_EXTERNAL_INSTR_H */
