#ifndef TUCOMPOSER_EXTERNAL_MIXER_H
#define TUCOMPOSER_EXTERNAL_MIXER_H
/*
**  $VER: mixer.h 31.22 (1.4.99)
**
**  Standard C header for tucomposer.library
**
**  (C) Copyright 1999 by Basty/Seasons
**      All Rights Reserved
*/

#ifndef EXEC_TYPES_H
#include <exec/types.h>
#endif /* EXEC_TYPES_H */

#ifndef EXEC_LIBRARIES_H
#include <exec/libraries.h>
#endif /* EXEC_LIBRARIES_H */

#ifndef EXEC_LISTS_H
#include <exec/lists.h>
#endif /* EXEC_LISTS_H */

#ifndef EXEC_TASKS_H
#include <exec/tasks.h>
#endif /* EXEC_TASKS_H */

#ifndef LIBRARIES_DOS_H
#include <libraries/dos.h>
#endif /* LIBRARIES_DOS_H */

#ifndef TUCOMPOSER_EXTERNAL_EXTERNAL_H
#include <tucomposer/external/external.h>
#endif /* TUCOMPOSER_EXTERNAL_EXTERNAL_H */

#ifndef TUCOMPOSER_EXTERNAL_DEVICE_H
#include <tucomposer/external/device.h>
#endif /* TUCOMPOSER_EXTERNAL_DEVICE_H */

/* Flags for Mixer info: */
#define MIB_StereoSupport   3   /* Mixer supports stereo mixing (readable) */
#define MIB_Surround        4   /* Mixer supports surround (readable) */
#define MIB_PostProcs       5   /* Mixer supports post processors */

#define MIF_StereoSupport   (1<<MIB_StereoSupport)
#define MIF_Surround        (1<<MIB_Surround)
#define MIF_PostProcs       (1<<MIB_PostProcs)

/* Flags for Mixer allocation structure: */

#define MASB_Allocated      0   /* Mixer is allocated */
#define MASB_Mixing         1   /* Mixer is currently mixing (i.e. playing)
                                   This bit must be set from the soundcard
                                   driver when playing starts ! */
#define MASB_Stereo         2   /* Mixer is outputting in stereo */
#define MASB_Frozen         3   /* Mixer is frozen (this was invented because
                                   post processors, which may temporary
                                   stop the mixing due to some delays
                                   (e.g. disk writing) */

#define MASF_Allocated      (1<<MASB_Allocated)
#define MASF_Mixing         (1<<MASB_Mixing)
#define MASF_Stereo         (1<<MASB_Stereo)
#define MASF_Frozen         (1<<MASB_Frozen)

/* Flags for the Mixer structure (used by ExternalMixerxxxChannel()): */
#define MSB_Muted           0   /* Channel is muted (i.e. processed but not
                                   outputted) */
#define MSB_Synth           1   /* Start synthetic wave when old finished */
#define MSB_Loop            2   /* Loop being used */
#define MSB_Pingpong        3   /* Pingpong loop */
#define MSB_Backwards       4   /* Sample is currently playing backwards */
#define MSB_BackLoop        5   /* Backward looping */
#define MSB_Surround        6   /* Channel in surround mode */
#define MSB_Play            7   /* Sample currently playing */

#define MSF_Muted           (1<<MSB_Muted)
#define MSF_Synth           (1<<MSB_Synth)
#define MSF_Loop            (1<<MSB_Loop)
#define MSF_Pingpong        (1<<MSB_Pingpong)
#define MSF_Backwards       (1<<MSB_Backwards)
#define MSF_BackLoop        (1<<MSB_BackLoop)
#define MSF_Surround        (1<<MSB_Surround)
#define MSF_Play            (1<<MSB_Play)

/* The following structure is returned by AllocMixer() or
   ExternalMixerAlloc(): */

struct MixerAllocStruct {
  struct MixerInfo *mas_MixLibBase; /* Ptr to external mixer library base */
  struct SoundDeviceInfo *mas_SoundDevLibBase; /* Ptr to sound device lib
                                   base outputs the fuckin' sound ! */
  struct Task *mas_Task;        /* Ptr to task allocated the structure */
  struct MinList mas_PostProcs; /* Linked list of post processors */
  ULONG  mas_MixRate;           /* Current mixing rate in Hz (must be set by
                                   you before call init & change routines) */
  LONG  *mas_MixBuf;            /* APTR to mixing buffer */
  ULONG  mas_MixBufSize;        /* Size of mix buffer(s) (must set by you) */
  ULONG  mas_VolumeBoost;       /* Volume boost (65536 = 100%) (you set) */
  UWORD  mas_LeftVolume;        /* Left volume (256 = 100%) (set by you) */
  UWORD  mas_RightVolume;       /* Right volume (256 = 100%) (set by you) */
  ULONG  mas_Tempo;             /* Speed of player IRQ in 1/65536 seconds.
                                   131072 means to run the player twice a
                                   second, and so on...For BpM you can use
                                   the following formula (simple !):
                                   65536*RpB*BpM (RpB = Rows per Beat (4)) */
  UWORD  mas_MaxChannels;       /* Maximum number of channels selected */
  UBYTE  mas_Flags;             /* Mixer information flags */
  UBYTE  mas_Pad;               /* To longword align the following */
  UBYTE  mas_Reserved[16];      /* Reserved for use by TuComposer */
  APTR   mas_UserData;          /* Ptr to user data (for your own purposes) */
                                /* Mixer specific data follows here */
}; /* MixerAllocStruct */

/* The following structure is returned by ExternalMixerGetChannel() and set
   by ExternalMixerSetChannel(): */
struct MixerStruct {
  ULONG  ms_SamplePos;          /* Current sample position (in samples) */
  ULONG  ms_SampleLen;          /* Current sample length (in samples) */
  WORD  *ms_SampleData;         /* APTR to sample data currently playing */
  ULONG  ms_SampleRate;         /* Sample rate of sample */
  ULONG  ms_Repeat;             /* Repeat offset (in samples) */
  ULONG  ms_RepLen;             /* Repeat length (in samples) */
  ULONG  ms_RepCount;           /* Repeat counter (0 for unlimited) */
  ULONG  ms_RepCounted;         /* Counted repeats */
  UBYTE  ms_BitDepth;           /* The bit depth of the sample */
  UBYTE  ms_Flags;              /* Special flags */
  UBYTE  ms_Volume;             /* Volume of sample playing (0 - 0xFF) */
  UBYTE  ms_Panning;            /* Panning of sample playing (0 - 0xFF) */
  UBYTE  ms_Reserved[16];       /* Reserved for future expansions (resonance
                                   (filtering, ring modulation, etc.) */
}; /* MixerStruct */

/* Mixer library info structure (after library structure): */
struct MixerInfo {
  struct TuComposerExtLibBase mix_Library;
  UBYTE  mix_Flags;             /* Mixer information flags (only readable) */
  UBYTE  mix_Pad;               /* To longword align the following */
  ULONG  mix_DefaultMixRate;    /* Default mixing rate */
  ULONG  mix_DefaultBufSize;    /* Default mixing buffer size */
  ULONG  mix_DefaultBoost;      /* Default volume boost */
  ULONG  mix_MinMixRate;        /* Lowest mixing rate supported by mixer */
  ULONG  mix_MaxMixRate;        /* Highest mixing rate supported by mixer */
  ULONG  mix_MinBufSize;        /* Smallest buffer size supported */
  ULONG  mix_MaxBufSize;        /* Biggest buffer size supported */
  UWORD  mix_MaxChannels;       /* Maximum number of channels allowed */
  UBYTE  mix_Reserved[14];      /* Reserved ! Do not touch ! */
}; /* MixerInfo */

/* Error codes returned by mixers. */

#define ERROR_EXTLIB_MIXNOALLOC ERROR_EXTLIB+0x8 /* Mixer is not allocated */
#define EXTLIB_MIXER_VERSION 31 /* Current external mixer library version */
#define EXTLIB_MIXER_REVISION 0 /* Current external mixer library revision */

#endif /* TUCOMPOSER_EXTERNAL_MIXER_H */
