#ifndef TUCOMPOSER_EXTERNAL_SAMPLE_H
#define TUCOMPOSER_EXTERNAL_SAMPLE_H
/*
**  $VER: sample.h 31.22 (4.4.99)
**
**  Standard C header for tucomposer.library
**
**  (C) Copyright 1999 by Basty/Seasons
**      All Rights Reserved
*/

#ifndef EXEC_TYPES_H
#include <exec/types.h>
#endif /* EXEC_TYPES_H */

#ifndef EXEC_LIBRARIES_H
#include <exec/libraries.h>
#endif /* EXEC_LIBRARIES_H */

#ifndef EXEC_LISTS_H
#include <exec/lists.h>
#endif /* EXEC_LISTS_H */

#ifndef LIBRARIES_DOS_H
#include <libraries/dos.h>
#endif /* LIBRARIES_DOS_H */

#ifndef TUCOMPOSER_TUCOMPOSER_H
#include <tucomposer/tucomposer.h>
#endif /* TUCOMPOSER_TUCOMPOSER_H */

#ifndef TUCOMPOSER_EXTERNAL_CRUNCHER_H
#include <tucomposer/external/cruncher.h>
#endif /* TUCOMPOSER_EXTERNAL_CRUNCHER_H */

/* si_Flags are same as tcs_Flags except bit #0. */

#define SIB_Synth       0   /* If bit set sample contains synthetic set
                               (the tracker uses this bit for redirection) */

#define SIF_Synth       (1<<SIB_Synth)

/* SampleInfo structure. Got by GetSampleInfo(). */

struct SampleInfo {
  STRPTR si_FileName;       /* Pointer to sample file name
                               (e.g. C-Hihat.IFF). Remember
                               that TuComposer is also available for
                               MS-DOS and other platforms. So, please try
                               to use the limited 8x3 MS-DOS file/dir
                               name length. */
  STRPTR si_Name;           /* Pointer to sample name (e.g. Close Hihat) */
  ULONG  si_Length;         /* Sample length (in bytes). If this is a
                               synth this is just zero. */
  ULONG  si_Samples;        /* Number of samples (identical to tcs_Length
                               if 8 bit, and the halve if 16 bit) */
  ULONG  si_Repeat;         /* Sample repeat */
  ULONG  si_SustainRepeat;  /* Sample sustain repeat */
  ULONG  si_RepLen;         /* Sample repeat length */
  ULONG  si_SustainRepLen;  /* Sample sustain repeat length */
  ULONG  si_RepCount;       /* Sample count, NULL if no count */
  ULONG  si_SustainRepCount; /* Sustain sample count, NULL if no count */
  ULONG  si_Rate;           /* C-4 sample frequency in Hz */
  ULONG  si_StartOffset;    /* Start offset (if any) */
  ULONG  si_LowerRateLimit; /* Lower sample rate limit */
  ULONG  si_UpperRateLimit; /* Upper sample rate limit */
  UBYTE  si_BitDepth;       /* Sample bit resolution (currently 1 to 16 */
  BYTE   si_Transpose;      /* Sample transpose */
  BYTE   si_FineTune;       /* Sample fine tune value */
  UBYTE  si_CompFlags;      /* Flags for compatibility, see upwards. */
  UBYTE  si_Flags;          /* Special flags, see upwards. */
  UBYTE  si_RepMode;        /* Sample repeat mode (e.g. Pingpong mode) */
  UBYTE  si_SustainLoopMode; /* Sample sustain loop mode (e.g. Pingpong) */
  UBYTE  si_Volume;         /* 8-bit default volume of sample. */
  UBYTE  si_SubVol;         /* Sub volume (only used by vol-slide cmds) */
  BYTE   si_Panning;        /* 8-bit panning of instr if TCSB_InsPanning
                               is set. */
  UBYTE  si_SubPanning;     /* Sub panning (only used by pan-slide cmds) */
  UBYTE  si_EnvUsageFlags;  /* Auto vibrato/tremolo/etc. usage flags */
  UBYTE  si_EnvProcFlags;   /* Auto vibrato/etc. processing flags */
  UBYTE  si_EnvRetrigFlags; /* Auto vibrato/etc. retrigger flags */
  UBYTE  si_EnvRandomFlags; /* Auto vibrato/etc. randomize flags */
  UWORD  si_VibratoSweep;   /* Auto vibrato sweep */
  UWORD  si_TremoloSweep;   /* Auto tremolo sweep */
  UWORD  si_PannoloSweep;   /* Auto pannolo sweep */
  UBYTE  si_VibratoDepth;   /* Auto vibrato depth */
  UBYTE  si_VibratoRate;    /* Auto vibrato rate (speed) */
  UBYTE  si_TremoloDepth;   /* Auto tremolo depth */
  UBYTE  si_TremoloRate;    /* Auto tremolo rate (speed) */
  UBYTE  si_PannoloDepth;   /* Auto pannolo depth */
  UBYTE  si_PannoloRate;    /* Auto pannolo rate (speed) */
}; /* SampleInfo */

/* Sample library structure (after library structure): */
struct SampleLibrary {
  struct TuComposerExtLibBase sampl_Library;
}; /* SampleLibrary */

#define EXTLIB_SAMPLE_VERSION  31   /* Current external sample library
                                       version */
#define EXTLIB_SAMPLE_REVISION  0   /* Current external sample library
                                       revision */

#endif /* TUCOMPOSER_EXTERNAL_SAMPLE_H */
