#ifndef TUCOMPOSER_EXTERNAL_SYNTH_H
#define TUCOMPOSER_EXTERNAL_SYNTH_H
/*
**  $VER: synth.h 31.22 (4.4.99)
**
**  Standard C header for tucomposer.library
**
**  (C) Copyright 1999 by Basty/Seasons
**      All Rights Reserved
*/

#ifndef EXEC_TYPES_H
#include <exec/types.h>
#endif /* EXEC_TYPES_H */

#ifndef EXEC_LIBRARIES_H
#include <exec/libraries.h>
#endif /* EXEC_LIBRARIES_H */

#ifndef EXEC_LISTS_H
#include <exec/lists.h>
#endif /* EXEC_LISTS_H */

#ifndef LIBRARIES_DOS_H
#include <libraries/dos.h>
#endif /* LIBRARIES_DOS_H */

#ifndef TUCOMPOSER_TUCOMPOSER_H
#include <tucomposer/tucomposer.h>
#endif /* TUCOMPOSER_TUCOMPOSER_H */

#ifndef TUCOMPOSER_EXTERNAL_CRUNCHER_H
#include <tucomposer/external/cruncher.h>
#endif /* TUCOMPOSER_EXTERNAL_CRUNCHER_H */

/* SynthInfo structure. Got by GetSynthInfo(). Version and revision will
   be the same as that of the library if the synthetic doesn't support this.
   All values will be the same as the YHDR fields (i.e. Flags, etc.). */

struct SynthInfo {

  STRPTR yi_FileName;       /* Pointer to file name of synthetic */
  STRPTR yi_Name;           /* Pointer to name of synthetic */
  UBYTE  yi_Version;        /* Version number of synthetic */
  UBYTE  yi_Revision;       /* Revision number of synthetic */
  UWORD  yi_NumWaves;       /* Number of waveforms (if 0 this is a hybrid) */
  UWORD  yi_CodeEntries;    /* Number of synth code lines */
  UWORD  yi_VolumePos;      /* Entry position of volume handling code */
  UWORD  yi_PanningPos;     /* Entry position of panning handling code */
  UWORD  yi_SlidePos;       /* Entry position of slide handling code */
  UWORD  yi_SpecialPos;     /* Entry position of special handling code */
  UWORD  yi_VolumeSusPos;   /* Entry position of sustain volume code */
  UWORD  yi_PanningSusPos;  /* Entry position of sustain panning code */
  UWORD  yi_SlideSusPos;    /* Entry position of sustain slide code */
  UWORD  yi_SpecialSusPos;  /* Entry position of sustain special code */
  UWORD  yi_VolNNAPos;      /* Position of volume handling code when NNA
                               has been triggered. This allows custom NNA
                               handling ! */
  UWORD  yi_PanNNAPos;      /* Position of panning handling NNA code */
  UWORD  yi_SlideNNAPos;    /* Position of slide handling NNA code */
  UWORD  yi_SpecialNNAPos;  /* Position of special NNA handling code */
  UWORD  yi_VolDNAPos;      /* Position of volume handling code when DNA
                               has been triggered. This allows custom DNA
                               handling ! */
  UWORD  yi_PanDNAPos;      /* Position of panning handling DNA code */
  UWORD  yi_SlideDNAPos;    /* Position of slide handling DNA code */
  UWORD  yi_SpecialDNAPos;  /* Position of special DNA handling code */
  UWORD  yi_Variable0;      /* Initial contents of variable 0 */
  UWORD  yi_Variable1;      /* Initial contents of variable 1 */
  UWORD  yi_Variable2;      /* Initial contents of variable 2 */
  UWORD  yi_Variable3;      /* Initial contents of variable 3 */
  UWORD  yi_Variable4;      /* Initial contents of variable 4 */
  UWORD  yi_Variable5;      /* Initial contents of variable 5 */
  UWORD  yi_Variable6;      /* Initial contents of variable 6 */
  UWORD  yi_Variable7;      /* Initial contents of variable 7 */
  UWORD  yi_Variable8;      /* Initial contents of variable 8 */
  UWORD  yi_Variable9;      /* Initial contents of variable 9 */
  UWORD  yi_VariableA;      /* Initial contents of variable 10 */
  UWORD  yi_VariableB;      /* Initial contents of variable 11 */
  UWORD  yi_VariableC;      /* Initial contents of variable 12 */
  UWORD  yi_VariableD;      /* Initial contents of variable 13 */
  UWORD  yi_VariableE;      /* Initial contents of variable 14 */
  UWORD  yi_VariableF;      /* Initial contents of variable 15 */
  UWORD  yi_VolConVar;      /* Initial contents of volume condition var. */
  UWORD  yi_PanConVar;      /* Initial contents of panning condition var. */
  UWORD  yi_SlideConVar;    /* Initial contents of slide condition var. */
  UWORD  yi_SpecialConVar;  /* Initial contents of special condition var. */
  UBYTE  yi_UseNNAFlags;    /* Use NNA positions. See bit defs above */
  UBYTE  yi_UseSusFlags;    /* Use sustain flags */
  UBYTE  yi_PosKeepMask;    /* Position keep mask. All initial entry
                               positions will be keeped. See bit defs
                               above. */
  UBYTE  yi_NNAPosKeepMask; /* NNA trigger position keep mask. Nearly
                               the same as yi_PosKeepMask. */
  UWORD  yi_VarKeepMask;    /* Variable keep mask. All variables where bit
                               is set will be keeped (normally they will
                               be overwritten with the initial values),
                               e.g. bit 5 set will keep variable 5. */
}; /* SynthInfo */

/* Synth library structure (after library structure): */
struct SynthLibrary {
  struct TuComposerExtLibBase synl_Library;
}; /* SynthLibrary */

#define EXTLIB_SYNTH_VERSION   31   /* Current external synth library
                                       version */
#define EXTLIB_SYNTH_REVISION   0   /* Current external synth library
                                       revision */

#endif /* TUCOMPOSER_EXTERNAL_SYNTH_H */
