#ifndef TUCOMPOSER_INSTR_H
#define TUCOMPOSER_INSTR_H
/*
**  $VER: instr.h 31.22 (1.4.99)
**
**  Standard C header for tucomposer.library
**
**  (C) Copyright 1999 by Basty/Seasons
**      All Rights Reserved
*/

#ifndef EXEC_TYPES_H
#include <exec/types.h>
#endif /* EXEC_TYPES_H */

#ifndef EXEC_LISTS_H
#include <exec/lists.h>
#endif /* EXEC_LISTS_H */

#ifndef EXEC_NODES_H
#include <exec/nodes.h>
#endif /* EXEC_NODES_H */

#ifndef TUCOMPOSER_TUCOMPOSER_H
#include <tucomposer/tucomposer.h>
#endif /* TUCOMPOSER_TUCOMPOSER_H */

/* Bit definitions for tci_CompFlags: */
#define TCICB_LockInsWave       0   /* Instrument wave is locked as in MOD,
                                       but volume/panning/etc. is taken,
                                       if both bits are clear it will handle
                                       like S3M/IT, i.e. instr is changed */
#define TCICB_AffectChanPan     1   /* Instrument panning affects channel
                                       panning (like IT does) */
#define TCICB_PrevSample        2   /* If no sample is given in the keyboard
                                       table, the previous one is used */
#define TCICB_SeparateSamples   3   /* Samples are separate as in IT (else
                                       handles like XM) */

#define TCICF_LockInsWave       (1<<TCICB_LockInsWave)
#define TCICF_AffectChanPan     (1<<TCICB_AffectChanPan)
#define TCICF_PrevSample        (1<<TCICB_PrevSample)
#define TCICF_SeparateSamples   (1<<TCICB_SeparateSamples)

/* Bit definitions for tci_Flags: */
#define TCIB_NoTranspose        0   /* If bit is set, the instrument can't
                                       be transposed by the position list */
#define TCIB_PortaSlideEnv      1   /* If bit is set, the values in the
                                       slide envelope will be portamento
                                       slide values, else they will be a
                                       transpose & finetune */
#define TCIB_LinearSlideEnv     2   /* If bit is set, use linear frequence
                                       table for slide envelope if portamento
                                       mode is activated */
#define TCIB_DefaultPanning     4   /* If bit is set, default instrument
                                       panning is used and override any
                                       sample settings. */
#define TCIB_SurroundPanning    5   /* If bit is set, default instr pan is
                                       surround panning. */
#define TCIB_NoInstrTranspose   6   /* Instrument is not transposeable via
                                       the order's instrument transpose. */

#define TCIF_NoTranspose        (1<<TCIB_NoTranspose)
#define TCIF_PortaSlideEnv      (1<<TCIB_PortaSlideEnv)
#define TCIF_LinearSlideEnv     (1<<TCIB_LinearSlideEnv)
#define TCIF_DefaultPanning     (1<<TCIB_DefaultPanning)
#define TCIF_SurroundPanning    (1<<TCIB_SurroundPanning)
#define TCIF_NoInstrTranspose   (1<<TCIB_NoInstrTranspose)

/* Bit definitions for tci_EnvUsageFlags: */
#define TCIEUB_UseVolumeEnv     0   /* Use (reload) volume envelope */
#define TCIEUB_UsePanningEnv    1   /* Use (reload) panning envelope */
#define TCIEUB_UseSlideEnv      2   /* Use (reload) slide envelope */
#define TCIEUB_UseVibratoEnv    3   /* Use (reload) vibrato envelope */
#define TCIEUB_UseTremoloEnv    4   /* Use (reload) tremolo envelope */
#define TCIEUB_UsePannoloEnv    5   /* Use (reload) pannolo envelope */
#define TCIEUB_UseChannoloEnv   6   /* Use (reload) channolo envelope */
#define TCIEUB_UseSpenoloEnv    7   /* Use (reload) spenolo envelope */
#define TCIEUB_UseTTremolo      8   /* Use (reload) track tremolo envelope */
#define TCIEUB_UseTPannolo      9   /* Use (reload) track pannolo envelope */
#define TCIEUB_UseGTremolo     10   /* Use (reload) glob. tremolo envelope */
#define TCIEUB_UseGPannolo     11   /* Use (reload) glob. pannolo envelope */

#define TCIEUF_UseVolumeEnv     (1<<TCIEUB_UseVolumeEnv)
#define TCIEUF_UsePanningEnv    (1<<TCIEUB_UsePanningEnv)
#define TCIEUF_UseSlideEnv      (1<<TCIEUB_UseSlideEnv)
#define TCIEUF_UseVibratoEnv    (1<<TCIEUB_UseVibratoEnv)
#define TCIEUF_UseTremoloEnv    (1<<TCIEUB_UseTremoloEnv)
#define TCIEUF_UsePannoloEnv    (1<<TCIEUB_UsePannoloEnv)
#define TCIEUF_UseChannoloEnv   (1<<TCIEUB_UseChannoloEnv)
#define TCIEUF_UseSpenoloEnv    (1<<TCIEUB_UseSpenoloEnv)
#define TCIEUF_UseTTremolo      (1<<TCIEUB_UseTTremolo)
#define TCIEUF_UseTPannolo      (1<<TCIEUB_UseTPannolo)
#define TCIEUF_UseGTremolo      (1<<TCIEUB_UseGTremolo)
#define TCIEUF_UseGPannolo      (1<<TCIEUB_UseGPannolo)

/* Bit definitions for tci_EnvProcFlags: */
#define TCIEPB_ProcVolumeEnv    0   /* Add first, then get vol env. value */
#define TCIEPB_ProcPanningEnv   1   /* Add first, then get pan env. value */
#define TCIEPB_ProcSlideEnv     2   /* Add first, then get sld. env. value */
#define TCIEPB_ProcVibratoEnv   3   /* Add first then get vibrato env. val */
#define TCIEPB_ProcTremoloEnv   4   /* Add first then get tremolo env. val */
#define TCIEPB_ProcPannoloEnv   5   /* Add first then get pannolo env. val */
#define TCIEPB_ProcChannoloEnv  6   /* Add first then get channolo env val */
#define TCIEPB_ProcSpenoloEnv   7   /* Add first then get spenolo env. val */
#define TCIEPB_ProcTTremolo     8   /* Add 1st then get track tremolo val */
#define TCIEPB_ProcTPannolo     9   /* Add 1st then get track pannolo val */
#define TCIEPB_ProcGTremolo    10   /* Add first then get glob tremolo val */
#define TCIEPB_ProcGPannolo    11   /* Add first then get glob pannolo val */

#define TCIEPF_ProcVolumeEnv    (1<<TCIEPB_ProcVolumeEnv)
#define TCIEPF_ProcPanningEnv   (1<<TCIEPB_ProcPanningEnv)
#define TCIEPF_ProcSlideEnv     (1<<TCIEPB_ProcSlideEnv)
#define TCIEPF_ProcVibratoEnv   (1<<TCIEPB_ProcVibratoEnv)
#define TCIEPF_ProcTremoloEnv   (1<<TCIEPB_ProcTremoloEnv)
#define TCIEPF_ProcPannoloEnv   (1<<TCIEPB_ProcPannoloEnv)
#define TCIEPF_ProcChannoloEnv  (1<<TCIEPB_ProcChannoloEnv)
#define TCIEPF_ProcSpenoloEnv   (1<<TCIEPB_ProcSpenoloEnv)
#define TCIEPF_ProcTTremolo     (1<<TCIEPB_ProcTTremolo)
#define TCIEPF_ProcTPannolo     (1<<TCIEPB_ProcTPannolo)
#define TCIEPF_ProcGTremolo     (1<<TCIEPB_ProcGTremolo)
#define TCIEPF_ProcGPannolo     (1<<TCIEPB_ProcGPannolo)

/* Bit definitions for tci_EnvRetrigFlags: */
#define TCIERB_NotRetrigVolEnv  0   /* Not retrigger volume envelope */
#define TCIERB_NotRetrigPanEnv  1   /* Not retrigger panning envelope */
#define TCIERB_NotRetrigSldEnv  2   /* Not retrigger slide envelope */
#define TCIERB_NotRetrigVibEnv  3   /* Not retrigger vibrato envelope */
#define TCIERB_NotRetrigTrmEnv  4   /* Not retrigger tremolo envelope */
#define TCIERB_NotRetrigPnoEnv  5   /* Not retrigger pannolo envelope */
#define TCIERB_NotRetrigChnEnv  6   /* Not retrigger channolo envelope */
#define TCIERB_NotRetrigSpnEnv  7   /* Not retrigger spenolo envelope */
#define TCIERB_NotRetrigTTrm    8   /* Not retrig track tremolo envelope */
#define TCIERB_NotRetrigTPan    9   /* Not retrig track pannolo envelope */
#define TCIERB_NotRetrigGTrm   10   /* Not retrigger global tremolo envelope */
#define TCIERB_NotRetrigGPan   11   /* Not retrigger global pannolo envelope */

#define TCIERF_NotRetrigVolEnv  (1<<TCIERB_NotRetrigVolEnv)
#define TCIERF_NotRetrigPanEnv  (1L<<TCIERB_NotRetrigPanEnv)
#define TCIERF_NotRetrigSldEnv  (1<<TCIERB_NotRetrigSldEnv)
#define TCIERF_NotRetrigVibEnv  (1<<TCIERB_NotRetrigVibEnv)
#define TCIERF_NotRetrigTrmEnv  (1<<TCIERB_NotRetrigTrmEnv)
#define TCIERF_NotRetrigPnoEnv  (1<<TCIERB_NotRetrigPnoEnv)
#define TCIERF_NotRetrigChnEnv  (1<<TCIERB_NotRetrigChnEnv)
#define TCIERF_NotRetrigSpnEnv  (1<<TCIERB_NotRetrigSpnEnv)
#define TCIERF_NotRetrigTTrm    (1<<TCIERB_NotRetrigTTrm)
#define TCIERF_NotRetrigTPan    (1<<TCIERB_NotRetrigTPan)
#define TCIERF_NotRetrigGTrm    (1<<TCIERB_NotRetrigGTrm)
#define TCIERF_NotRetrigGPan    (1<<TCIERB_NotRetrigGPan)

/* Bit definitions for tci_EnvRandomFlags: */
#define TCIERB_RandomVolEnv     0   /* Random volume envelope, if bit set */
#define TCIERB_RandomPanEnv     1   /* Random panning envelope, if bit set */
#define TCIERB_RandomSlideEnv   2   /* Random slide envelope, if bit set */
#define TCIERB_RandomVibEnv     3   /* Random vibrato envelope, if bit set */
#define TCIERB_RandomTremEnv    4   /* Random tremolo envelope, if bit set */
#define TCIERB_RandomPannoEnv   5   /* Random pannolo envelope, if bit set */
#define TCIERB_RandomChanEnv    6   /* Random channolo envelope if bit set */
#define TCIERB_RandomSpenEnv    7   /* Random spenolo envelope, if bit set */
#define TCIERB_RandomTTrmEnv    8   /* Random track tremolo envelope */
#define TCIERB_RandomTPanEnv    9   /* Random track pannolo envelope */
#define TCIERB_RandomGTrmEnv   10   /* Random global tremolo envelope */
#define TCIERB_RandomGPanEnv   11   /* Random global pannolo envelope */

#define TCIERF_RandomVolEnv     (1<<TCIERB_RandomVolEnv)
#define TCIERF_RandomPanEnv     (1<<TCIERB_RandomPanEnv)
#define TCIERF_RandomSlideEnv   (1<<TCIERB_RandomSlideEnv)
#define TCIERF_RandomVibEnv     (1<<TCIERB_RandomVibEnv)
#define TCIERF_RandomTremEnv    (1<<TCIERB_RandomTremEnv)
#define TCIERF_RandomPannoEnv   (1<<TCIERB_RandomPannoEnv)
#define TCIERF_RandomChanEnv    (1<<TCIERB_RandomChanEnv)
#define TCIERF_RandomSpenEnv    (1<<TCIERB_RandomSpenEnv)
#define TCIERF_RandomTTrmEnv    (1<<TCIERB_RandomTTrmEnv)
#define TCIERF_RandomTPanEnv    (1<<TCIERB_RandomTPanEnv)
#define TCIERF_RandomGTrmEnv    (1<<TCIERB_RandomGTrmEnv)
#define TCIERF_RandomGPanEnv    (1<<TCIERB_RandomGPanEnv)

/* Bit definitions for tci_EnvRndDelayFlags: */
#define TCIERB_RndDelayVolEnv   0   /* Speed is delay random volume env. */
#define TCIERB_RndDelayPanEnv   1   /* Speed is delay random panning env. */
#define TCIERB_RndDelaySldEnv   2   /* Speed is delay random slide env. */
#define TCIERB_RndDelayVibEnv   3   /* Speed is delay random vibrato env. */
#define TCIERB_RndDelayTremEnv  4   /* Speed is delay random tremolo env. */
#define TCIERB_RndDelayPanoEnv  5   /* Speed is delay random pannolo env. */
#define TCIERB_RndDelayChanEnv  6   /* Speed is delay random channolo env. */
#define TCIERB_RndDelaySpenEnv  7   /* Speed is delay random spenolo env. */
#define TCIERB_RndDelayTTrmEnv  8   /* Spd is delay rnd. track tremolo env */
#define TCIERB_RndDelayTPanEnv  9   /* Spd is delay rnd. track pannolo env */
#define TCIERB_RndDelayGTrEnv  10   /* Spd is delay rnd. glob. tremolo env */
#define TCIERB_RndDelayGPnEnv  11   /* Spd is delay rnd. glob. pannolo env */

#define TCIERF_RndDelayVolEnv   (1<<TCIERB_RndDelayVolEnv)
#define TCIERF_RndDelayPanEnv   (1<<TCIERB_RndDelayPanEnv)
#define TCIERF_RndDelaySldEnv   (1<<TCIERB_RndDelaySldEnv)
#define TCIERF_RndDelayVibEnv   (1<<TCIERB_RndDelayVibEnv)
#define TCIERF_RndDelayTremEnv  (1<<TCIERB_RndDelayTremEnv)
#define TCIERF_RndDelayPanoEnv  (1<<TCIERB_RndDelayPanoEnv)
#define TCIERF_RndDelayChanEnv  (1<<TCIERB_RndDelayChanEnv)
#define TCIERF_RndDelaySpenEnv  (1<<TCIERB_RndDelaySpenEnv)
#define TCIERF_RndDelayTTrmEnv  (1<<TCIERB_RndDelayTTrmEnv)
#define TCIERF_RndDelayTPanEnv  (1<<TCIERB_RndDelayTPanEnv)
#define TCIERF_RndDelayGTrEnv   (1<<TCIERB_RndDelayGTrEnv)
#define TCIERF_RndDelayGPnEnv   (1<<TCIERB_RndDelayGPnEnv)

/* Bit definitions for tci_DCT: */
#define DCTB_InstrNoteOR        0   /* Check for duplicate OR instr notes */
#define DCTB_SampleNoteOR       1   /* Check for duplicate OR sample notes */
#define DCTB_InstrOR            2   /* Check for duplicate OR instruments */
#define DCTB_SampleOR           3   /* Check for duplicate OR samples */
#define DCTB_InstrNoteAND       4   /* Check for duplicate AND instr notes */
#define DCTB_SampleNoteAND      5   /* Check for duplicate AND samp. notes */
#define DCTB_InstrAND           6   /* Check for duplicate AND instruments */
#define DCTB_SampleAND          7   /* Check for duplicate AND samples */

#define DCTF_InstrNoteOR        (1<<DCTB_InstrNoteOR)
#define DCTF_SampleNoteOR       (1<<DCTB_SampleNoteOR)
#define DCTF_InstrOR            (1<<DCTB_InstrOR)
#define DCTF_SampleOR           (1<<DCTB_SampleOR)
#define DCTF_InstrNoteAND       (1<<DCTB_InstrNoteAND)
#define DCTF_SampleNoteAND      (1<<DCTB_SampleNoteAND)
#define DCTF_InstrAND           (1<<DCTB_InstrAND)
#define DCTF_SampleAND          (1<<DCTB_SampleAND)

/* Bit definitions for tci_MIDIFlags: */
#define MIDIB_TickQuantize      0   /* Tick quantize (insert note delays) */
#define MIDIB_NoteOff           1   /* Record Note-Off (keyoff note) */
#define MIDIB_Velocity          2   /* Record velocity */
#define MIDIB_Aftertouch        3   /* Record aftertouch */
#define MIDIB_ExtSync           4   /* External sync when recording */
#define MIDIB_Enable            7   /* MIDI enabled */

#define MIDIF_TickQuantize      (1<<MIDIB_TickQuantize)
#define MIDIF_NoteOff           (1<<MIDIB_NoteOff)
#define MIDIF_Velocity          (1<<MIDIB_Velocity)
#define MIDIF_Aftertouch        (1<<MIDIB_Aftertouch)
#define MIDIF_ExtSync           (1<<MIDIB_ExtSync)
#define MIDIF_Enable            (1<<MIDIB_Enable)

/* Bit definitions for SwapInstrInModule(): */
#define SIIMB_Swap              0L  /* If bit set, swap instrument data */
#define SIIMB_Update            1L  /* If bit set, update tracks */

#define SIIMF_Swap              (1L<<SIIMB_Swap)
#define SIIMF_Update            (1L<<SIIMB_Update)

/* Bit definitions for tcie_Flags: */
#define TCIEB_Loop              0   /* If bit is set, env will be looped */
#define TCIEB_Sustain           1   /* If bit is set, envelope will have a
                                       sustain point */
#define TCIEB_Pingpong          2   /* Loop in ping pong mode */
#define TCIEB_SustainPingpong   3   /* Sustain loop in ping pong mode */

#define TCIEF_Loop              (1<<TCIEB_Loop)
#define TCIEF_Sustain           (1<<TCIEB_Sustain)
#define TCIEF_Pingpong          (1<<TCIEB_Pingpong)
#define TCIEF_SustainPingpong   (1<<TCIEB_SustainPingpong)

/* Bit definitions for SwapEnvelopeInModule(): */
#define SEIMB_Swap              0L /* If bit set, swap envelope data */
#define SEIMB_Update            1L /* If bit set, update instruments */

#define SEIMF_Swap              (1L<<SEIMB_Swap)
#define SEIMF_Update            (1L<<SEIMB_Update)

/* Bit definitions for SwapKeyboardInModule(): */
#define SKIMB_Swap              0L /* If bit set, swap keyboard def data */
#define SKIMB_Update            1L /* If bit set, update instruments */

#define SKIMF_Swap              (1L<<SKIMB_Swap)
#define SKIMF_Update            (1L<<SKIMB_Update)

/* Bit definitions for tcia_Flags: */
#define TCIAB_Loop              0   /* If bit is set arpeggio loop is used */
#define TCIAB_Sustain           1   /* If bit is set, sustain tick is used */
#define TCIAB_Pingpong          2   /* Arpeggio structure will be played in
                                       ping pong mode (if loop enabled) */
#define TCIAB_SustainPingpong   3   /* If set, sustain ping pong is used */

#define TCIAF_Loop              (1<<TCIAB_Loop)
#define TCIAF_Sustain           (1<<TCIAB_Sustain)
#define TCIAF_Pingpong          (1<<TCIAB_Pingpong)
#define TCIAF_SustainPingpong   (1<<TCIAB_SustainPingpong)

/* Bit definitions for SwapArpeggioInModule(): */
#define SAIMB_Swap              0L /* If bit set, swap arpeggio def data */
#define SAIMB_Update            1L /* If bit set, update instruments */

#define SAIMF_Swap              (1L<<SAIMB_Swap)
#define SAIMF_Update            (1L<<SAIMB_Update)

/* Bit definitions for InsertEnvPoint() and DeleteEnvPoint() flags */
#define ENVB_PointsOnly         0L  /* If bit is set, only the points will be
                                       affected, else the envelope's length
                                       will be affected, too. */

#define ENVF_PointsOnly         (1L<<ENVB_PointsOnly)

/* Bit definitions for InsertArpeggioTick() and DeleteArpeggioTick() flags */
#define ENVB_TicksOnly          0L  /* If bit is set, only the ticks will be
                                       affected, else the arpeggio envelope's
                                       length will also be affected. */

#define ENVF_TicksOnly          (1L<<ENVB_TicksOnly)

/* Bit definitions for AddInstrEnvelopes(): */
#define ENVB_Tail               0L  /* Append envelopes else insert them
                                       at the beginning */

#define ENVF_Tail               (1L<<ENVB_Tail)

/* Envelope structure, all envelopes have this structure. */
struct TuComposerInstrEnvelope {
  struct MinNode tcie_Node;
  UWORD  tcie_Flags;                /* Envelope flags */
  WORD  *tcie_Data;                 /* Pointer to envelope data */
  UWORD  *tcie_NodePoints;          /* Ptr to nodes or NULL */
  UWORD  tcie_NumNodes;             /* Number of node points */
  UWORD  tcie_NumPoints;            /* Number of envelope points */
  UWORD  tcie_Tempo;                /* Envelope tempo in ticks (def is 1) */
  UWORD  tcie_SustainStart;         /* Sustain start point */
  UWORD  tcie_SustainEnd;           /* Sustain end point */
  UWORD  tcie_SustainCount;         /* Sustain loop counter */
  UWORD  tcie_LoopStart;            /* Loop start point */
  UWORD  tcie_LoopEnd;              /* Loop end point */
  UWORD  tcie_LoopCount;            /* Loop counter */
  WORD   tcie_LowestValue;          /* Lowest random value */
  WORD   tcie_HighestValue;         /* Highest random value */
  struct MinList tcie_UnknownChunks; /* Linked list of unknown chunks */
  UBYTE  tcie_Reserved[8];          /* Reserved for future expansions */
  APTR   tcie_UserData;             /* For your own purposes */
}; /* TuComposerInstrEnvelope */

/* Default values for envelopes */
#define DEFAULT_ENVPOINTS       64  /* Default number of points is 64 */
#define DEFAULT_ENVTEMPO        1   /* Default tempo is every tick */
#define DEFAULT_VOLUMESCALE     0x7FFF /* Default scale for volume env */
#define DEFAULT_PANNINGSCALE    0x1FFF /* Default scale for panning env */
#define DEFAULT_SLIDESCALE      0x0100 /* Default scale for slide envelope */
#define DEFAULT_VIBRATOSCALE    0x0100 /* Def scale for (auto) vibratos */
#define DEFAULT_ENVNODES        12     /* Defaults are 12 node points */

/* Waveform limitations */
#define MAX_SCALE               0x7FFF /* Maximum scale of waveform */
#define SCALE_INVALID           (MAX_SCALE+1) /* Invalid scale values */

/* Waveform creations */
#define SCALE_INVERT            0x80000000 /* If this is added, the outputted
									   graph will be inverted, e.g.
									   0x80000FFF will create an inverted
                                       cosine wave, if you would specify
                                       ENVL_COSINE */

/* Waveform (CreateEnvelope() & ClearEnvelope() & FillWaveform()) types */
#define ENVL_EMPTY              0L  /* Create empty envelope */
#define ENVL_SINE               1L  /* Create sine envelope */
#define ENVL_COSINE             2L  /* Create cosine envelope */
#define ENVL_RAMP               3L  /* Create ramping (down) envelope */
#define ENVL_SQUARE             4L  /* Create square envelope */
#define ENVL_TRIANGLE           5L  /* Create triangle envelope */
#define ENVL_SAWTOOTH           6L  /* Create sawtooth envelope */
#define ENVL_LAST               6L  /* Last valid number of envelope */
#define ENVL_INVALID            (ENVL_LAST+1) /* Numbers below are invalid
                                       and entering them cause an empty
                                       envelope to be generated. However, in
                                       future versions they may generate
                                       something. */

/* Keyboard definitions structure. C-0 is first entry. B-9 is 120th entry. */
struct TuComposerInstrKeyboard {
  struct MinNode tcik_Node;
  struct KeyboardEntry {
    UWORD  tcik_Sample_C_0;         /* Sample number for C-0 */
    UBYTE  tcik_Octave_C_0;         /* Octave number for C-0 */
    UBYTE  tcik_Note_C_0;           /* Note number for C-0 */
  } tcik_NoteEntry[1];
  UWORD  tcik_Sample_C_SHARP_0;     /* Sample number for C#0 */
  UBYTE  tcik_Octave_C_SHARP_0;     /* Octave number for C#0 */
  UBYTE  tcik_Note_C_SHARP_0;       /* Note number for C#0 */
  UWORD  tcik_Sample_D_0;           /* Sample number for D-0 */
  UBYTE  tcik_Octave_D_0;           /* Octave number for D-0 */
  UBYTE  tcik_Note_D_0;             /* Note number for D-0 */
  UWORD  tcik_Sample_D_SHARP_0;     /* Sample number for D#0 */
  UBYTE  tcik_Octave_D_SHARP_0;     /* Octave number for D#0 */
  UBYTE  tcik_Note_D_SHARP_0;       /* Note number for D#0 */
  UWORD  tcik_Sample_E_0;           /* Sample number for E-0 */
  UBYTE  tcik_Octave_E_0;           /* Octave number for E-0 */
  UBYTE  tcik_Note_E_0;             /* Note number for E-0 */
  UWORD  tcik_Sample_F_0;           /* Sample number for F-0 */
  UBYTE  tcik_Octave_F_0;           /* Octave number for F-0 */
  UBYTE  tcik_Note_F_0;             /* Note number for F-0 */
  UWORD  tcik_Sample_F_SHARP_0;     /* Sample number for F#0 */
  UBYTE  tcik_Octave_F_SHARP_0;     /* Octave number for F#0 */
  UBYTE  tcik_Note_F_SHARP_0;       /* Note number for F#0 */
  UWORD  tcik_Sample_G_0;           /* Sample number for G-0 */
  UBYTE  tcik_Octave_G_0;           /* Octave number for G-0 */
  UBYTE  tcik_Note_G_0;             /* Note number for G-0 */
  UWORD  tcik_Sample_G_SHARP_0;     /* Sample number for G#0 */
  UBYTE  tcik_Octave_G_SHARP_0;     /* Octave number for G#0 */
  UBYTE  tcik_Note_G_SHARP_0;       /* Note number for G#0 */
  UWORD  tcik_Sample_A_0;           /* Sample number for A-0 */
  UBYTE  tcik_Octave_A_0;           /* Octave number for A-0 */
  UBYTE  tcik_Note_A_0;             /* Note number for A-0 */
  UWORD  tcik_Sample_A_SHARP_0;     /* Sample number for A#0 */
  UBYTE  tcik_Octave_A_SHARP_0;     /* Octave number for A#0 */
  UBYTE  tcik_Note_A_SHARP_0;       /* Note number for A#0 */
  UWORD  tcik_Sample_B_0;           /* Sample number for B-0 */
  UBYTE  tcik_Octave_B_0;           /* Octave number for B-0 */
  UBYTE  tcik_Note_B_0;             /* Note number for B-0 */
  UWORD  tcik_Sample_C_1;           /* Sample number for C-1 */
  UBYTE  tcik_Octave_C_1;           /* Octave number for C-1 */
  UBYTE  tcik_Note_C_1;             /* Note number for C-1 */
  UWORD  tcik_Sample_C_SHARP_1;     /* Sample number for C#1 */
  UBYTE  tcik_Octave_C_SHARP_1;     /* Octave number for C#1 */
  UBYTE  tcik_Note_C_SHARP_1;       /* Note number for C#1 */
  UWORD  tcik_Sample_D_1;           /* Sample number for D-1 */
  UBYTE  tcik_Octave_D_1;           /* Octave number for D-1 */
  UBYTE  tcik_Note_D_1;             /* Note number for D-1 */
  UWORD  tcik_Sample_D_SHARP_1;     /* Sample number for D#1 */
  UBYTE  tcik_Octave_D_SHARP_1;     /* Octave number for D#1 */
  UBYTE  tcik_Note_D_SHARP_1;       /* Note number for D#1 */
  UWORD  tcik_Sample_E_1;           /* Sample number for E-1 */
  UBYTE  tcik_Octave_E_1;           /* Octave number for E-1 */
  UBYTE  tcik_Note_E_1;             /* Note number for E-1 */
  UWORD  tcik_Sample_F_1;           /* Sample number for F-1 */
  UBYTE  tcik_Octave_F_1;           /* Octave number for F-1 */
  UBYTE  tcik_Note_F_1;             /* Note number for F-1 */
  UWORD  tcik_Sample_F_SHARP_1;     /* Sample number for F#1 */
  UBYTE  tcik_Octave_F_SHARP_1;     /* Octave number for F#1 */
  UBYTE  tcik_Note_F_SHARP_1;       /* Note number for F#1 */
  UWORD  tcik_Sample_G_1;           /* Sample number for G-1 */
  UBYTE  tcik_Octave_G_1;           /* Octave number for G-1 */
  UBYTE  tcik_Note_G_1;             /* Note number for G-1 */
  UWORD  tcik_Sample_G_SHARP_1;     /* Sample number for G#1 */
  UBYTE  tcik_Octave_G_SHARP_1;     /* Octave number for G#1 */
  UBYTE  tcik_Note_G_SHARP_1;       /* Note number for G#1 */
  UWORD  tcik_Sample_A_1;           /* Sample number for A-1 */
  UBYTE  tcik_Octave_A_1;           /* Octave number for A-1 */
  UBYTE  tcik_Note_A_1;             /* Note number for A-1 */
  UWORD  tcik_Sample_A_SHARP_1;     /* Sample number for A#1 */
  UBYTE  tcik_Octave_A_SHARP_1;     /* Octave number for A#1 */
  UBYTE  tcik_Note_A_SHARP_1;       /* Note number for A#1 */
  UWORD  tcik_Sample_B_1;           /* Sample number for B-1 */
  UBYTE  tcik_Octave_B_1;           /* Octave number for B-1 */
  UBYTE  tcik_Note_B_1;             /* Note number for B-1 */
  UWORD  tcik_Sample_C_2;           /* Sample number for C-2 */
  UBYTE  tcik_Octave_C_2;           /* Octave number for C-2 */
  UBYTE  tcik_Note_C_2;             /* Note number for C-2 */
  UWORD  tcik_Sample_C_SHARP_2;     /* Sample number for C#2 */
  UBYTE  tcik_Octave_C_SHARP_2;     /* Octave number for C#2 */
  UBYTE  tcik_Note_C_SHARP_2;       /* Note number for C#2 */
  UWORD  tcik_Sample_D_2;           /* Sample number for D-2 */
  UBYTE  tcik_Octave_D_2;           /* Octave number for D-2 */
  UBYTE  tcik_Note_D_2;             /* Note number for D-2 */
  UWORD  tcik_Sample_D_SHARP_2;     /* Sample number for D#2 */
  UBYTE  tcik_Octave_D_SHARP_2;     /* Octave number for D#2 */
  UBYTE  tcik_Note_D_SHARP_2;       /* Note number for D#2 */
  UWORD  tcik_Sample_E_2;           /* Sample number for E-2 */
  UBYTE  tcik_Octave_E_2;           /* Octave number for E-2 */
  UBYTE  tcik_Note_E_2;             /* Note number for E-2 */
  UWORD  tcik_Sample_F_2;           /* Sample number for F-2 */
  UBYTE  tcik_Octave_F_2;           /* Octave number for F-2 */
  UBYTE  tcik_Note_F_2;             /* Note number for F-2 */
  UWORD  tcik_Sample_F_SHARP_2;     /* Sample number for F#2 */
  UBYTE  tcik_Octave_F_SHARP_2;     /* Octave number for F#2 */
  UBYTE  tcik_Note_F_SHARP_2;       /* Note number for F#2 */
  UWORD  tcik_Sample_G_2;           /* Sample number for G-2 */
  UBYTE  tcik_Octave_G_2;           /* Octave number for G-2 */
  UBYTE  tcik_Note_G_2;             /* Note number for G-2 */
  UWORD  tcik_Sample_G_SHARP_2;     /* Sample number for G#2 */
  UBYTE  tcik_Octave_G_SHARP_2;     /* Octave number for G#2 */
  UBYTE  tcik_Note_G_SHARP_2;       /* Note number for G#2 */
  UWORD  tcik_Sample_A_2;           /* Sample number for A-2 */
  UBYTE  tcik_Octave_A_2;           /* Octave number for A-2 */
  UBYTE  tcik_Note_A_2;             /* Note number for A-2 */
  UWORD  tcik_Sample_A_SHARP_2;     /* Sample number for A#2 */
  UBYTE  tcik_Octave_A_SHARP_2;     /* Octave number for A#2 */
  UBYTE  tcik_Note_A_SHARP_2;       /* Note number for A#2 */
  UWORD  tcik_Sample_B_2;           /* Sample number for B-2 */
  UBYTE  tcik_Octave_B_2;           /* Octave number for B-2 */
  UBYTE  tcik_Note_B_2;             /* Note number for B-2 */
  UWORD  tcik_Sample_C_3;           /* Sample number for C-3 */
  UBYTE  tcik_Octave_C_3;           /* Octave number for C-3 */
  UBYTE  tcik_Note_C_3;             /* Note number for C-3 */
  UWORD  tcik_Sample_C_SHARP_3;     /* Sample number for C#3 */
  UBYTE  tcik_Octave_C_SHARP_3;     /* Octave number for C#3 */
  UBYTE  tcik_Note_C_SHARP_3;       /* Note number for C#3 */
  UWORD  tcik_Sample_D_3;           /* Sample number for D-3 */
  UBYTE  tcik_Octave_D_3;           /* Octave number for D-3 */
  UBYTE  tcik_Note_D_3;             /* Note number for D-3 */
  UWORD  tcik_Sample_D_SHARP_3;     /* Sample number for D#3 */
  UBYTE  tcik_Octave_D_SHARP_3;     /* Octave number for D#3 */
  UBYTE  tcik_Note_D_SHARP_3;       /* Note number for D#3 */
  UWORD  tcik_Sample_E_3;           /* Sample number for E-3 */
  UBYTE  tcik_Octave_E_3;           /* Octave number for E-3 */
  UBYTE  tcik_Note_E_3;             /* Note number for E-3 */
  UWORD  tcik_Sample_F_3;           /* Sample number for F-3 */
  UBYTE  tcik_Octave_F_3;           /* Octave number for F-3 */
  UBYTE  tcik_Note_F_3;             /* Note number for F-3 */
  UWORD  tcik_Sample_F_SHARP_3;     /* Sample number for F#3 */
  UBYTE  tcik_Octave_F_SHARP_3;     /* Octave number for F#3 */
  UBYTE  tcik_Note_F_SHARP_3;       /* Note number for F#3 */
  UWORD  tcik_Sample_G_3;           /* Sample number for G-3 */
  UBYTE  tcik_Octave_G_3;           /* Octave number for G-3 */
  UBYTE  tcik_Note_G_3;             /* Note number for G-3 */
  UWORD  tcik_Sample_G_SHARP_3;     /* Sample number for G#3 */
  UBYTE  tcik_Octave_G_SHARP_3;     /* Octave number for G#3 */
  UBYTE  tcik_Note_G_SHARP_3;       /* Note number for G#3 */
  UWORD  tcik_Sample_A_3;           /* Sample number for A-3 */
  UBYTE  tcik_Octave_A_3;           /* Octave number for A-3 */
  UBYTE  tcik_Note_A_3;             /* Note number for A-3 */
  UWORD  tcik_Sample_A_SHARP_3;     /* Sample number for A#3 */
  UBYTE  tcik_Octave_A_SHARP_3;     /* Octave number for A#3 */
  UBYTE  tcik_Note_A_SHARP_3;       /* Note number for A#3 */
  UWORD  tcik_Sample_B_3;           /* Sample number for B-3 */
  UBYTE  tcik_Octave_B_3;           /* Octave number for B-3 */
  UBYTE  tcik_Note_B_3;             /* Note number for B-3 */
  UWORD  tcik_Sample_C_4;           /* Sample number for C-4 */
  UBYTE  tcik_Octave_C_4;           /* Octave number for C-4 */
  UBYTE  tcik_Note_C_4;             /* Note number for C-4 */
  UWORD  tcik_Sample_C_SHARP_4;     /* Sample number for C#4 */
  UBYTE  tcik_Octave_C_SHARP_4;     /* Octave number for C#4 */
  UBYTE  tcik_Note_C_SHARP_4;       /* Note number for C#4 */
  UWORD  tcik_Sample_D_4;           /* Sample number for D-4 */
  UBYTE  tcik_Octave_D_4;           /* Octave number for D-4 */
  UBYTE  tcik_Note_D_4;             /* Note number for D-4 */
  UWORD  tcik_Sample_D_SHARP_4;     /* Sample number for D#4 */
  UBYTE  tcik_Octave_D_SHARP_4;     /* Octave number for D#4 */
  UBYTE  tcik_Note_D_SHARP_4;       /* Note number for D#4 */
  UWORD  tcik_Sample_E_4;           /* Sample number for E-4 */
  UBYTE  tcik_Octave_E_4;           /* Octave number for E-4 */
  UBYTE  tcik_Note_E_4;             /* Note number for E-4 */
  UWORD  tcik_Sample_F_4;           /* Sample number for F-4 */
  UBYTE  tcik_Octave_F_4;           /* Octave number for F-4 */
  UBYTE  tcik_Note_F_4;             /* Note number for F-4 */
  UWORD  tcik_Sample_F_SHARP_4;     /* Sample number for F#4 */
  UBYTE  tcik_Octave_F_SHARP_4;     /* Octave number for F#4 */
  UBYTE  tcik_Note_F_SHARP_4;       /* Note number for F#4 */
  UWORD  tcik_Sample_G_4;           /* Sample number for G-4 */
  UBYTE  tcik_Octave_G_4;           /* Octave number for G-4 */
  UBYTE  tcik_Note_G_4;             /* Note number for G-4 */
  UWORD  tcik_Sample_G_SHARP_4;     /* Sample number for G#4 */
  UBYTE  tcik_Octave_G_SHARP_4;     /* Octave number for G#4 */
  UBYTE  tcik_Note_G_SHARP_4;       /* Note number for G#4 */
  UWORD  tcik_Sample_A_4;           /* Sample number for A-4 */
  UBYTE  tcik_Octave_A_4;           /* Octave number for A-4 */
  UBYTE  tcik_Note_A_4;             /* Note number for A-4 */
  UWORD  tcik_Sample_A_SHARP_4;     /* Sample number for A#4 */
  UBYTE  tcik_Octave_A_SHARP_4;     /* Octave number for A#4 */
  UBYTE  tcik_Note_A_SHARP_4;       /* Note number for A#4 */
  UWORD  tcik_Sample_B_4;           /* Sample number for B-4 */
  UBYTE  tcik_Octave_B_4;           /* Octave number for B-4 */
  UBYTE  tcik_Note_B_4;             /* Note number for B-4 */
  UWORD  tcik_Sample_C_5;           /* Sample number for C-5 */
  UBYTE  tcik_Octave_C_5;           /* Octave number for C-5 */
  UBYTE  tcik_Note_C_5;             /* Note number for C-5 */
  UWORD  tcik_Sample_C_SHARP_5;     /* Sample number for C#5 */
  UBYTE  tcik_Octave_C_SHARP_5;     /* Octave number for C#5 */
  UBYTE  tcik_Note_C_SHARP_5;       /* Note number for C#5 */
  UWORD  tcik_Sample_D_5;           /* Sample number for D-5 */
  UBYTE  tcik_Octave_D_5;           /* Octave number for D-5 */
  UBYTE  tcik_Note_D_5;             /* Note number for D-5 */
  UWORD  tcik_Sample_D_SHARP_5;     /* Sample number for D#5 */
  UBYTE  tcik_Octave_D_SHARP_5;     /* Octave number for D#5 */
  UBYTE  tcik_Note_D_SHARP_5;       /* Note number for D#5 */
  UWORD  tcik_Sample_E_5;           /* Sample number for E-5 */
  UBYTE  tcik_Octave_E_5;           /* Octave number for E-5 */
  UBYTE  tcik_Note_E_5;             /* Note number for E-5 */
  UWORD  tcik_Sample_F_5;           /* Sample number for F-5 */
  UBYTE  tcik_Octave_F_5;           /* Octave number for F-5 */
  UBYTE  tcik_Note_F_5;             /* Note number for F-5 */
  UWORD  tcik_Sample_F_SHARP_5;     /* Sample number for F#5 */
  UBYTE  tcik_Octave_F_SHARP_5;     /* Octave number for F#5 */
  UBYTE  tcik_Note_F_SHARP_5;       /* Note number for F#5 */
  UWORD  tcik_Sample_G_5;           /* Sample number for G-5 */
  UBYTE  tcik_Octave_G_5;           /* Octave number for G-5 */
  UBYTE  tcik_Note_G_5;             /* Note number for G-5 */
  UWORD  tcik_Sample_G_SHARP_5;     /* Sample number for G#5 */
  UBYTE  tcik_Octave_G_SHARP_5;     /* Octave number for G#5 */
  UBYTE  tcik_Note_G_SHARP_5;       /* Note number for G#5 */
  UWORD  tcik_Sample_A_5;           /* Sample number for A-5 */
  UBYTE  tcik_Octave_A_5;           /* Octave number for A-5 */
  UBYTE  tcik_Note_A_5;             /* Note number for A-5 */
  UWORD  tcik_Sample_A_SHARP_5;     /* Sample number for A#5 */
  UBYTE  tcik_Octave_A_SHARP_5;     /* Octave number for A#5 */
  UBYTE  tcik_Note_A_SHARP_5;       /* Note number for A#5 */
  UWORD  tcik_Sample_B_5;           /* Sample number for B-5 */
  UBYTE  tcik_Octave_B_5;           /* Octave number for B-5 */
  UBYTE  tcik_Note_B_5;             /* Note number for B-5 */
  UWORD  tcik_Sample_C_6;           /* Sample number for C-6 */
  UBYTE  tcik_Octave_C_6;           /* Octave number for C-6 */
  UBYTE  tcik_Note_C_6;             /* Note number for C-6 */
  UWORD  tcik_Sample_C_SHARP_6;     /* Sample number for C#6 */
  UBYTE  tcik_Octave_C_SHARP_6;     /* Octave number for C#6 */
  UBYTE  tcik_Note_C_SHARP_6;       /* Note number for C#6 */
  UWORD  tcik_Sample_D_6;           /* Sample number for D-6 */
  UBYTE  tcik_Octave_D_6;           /* Octave number for D-6 */
  UBYTE  tcik_Note_D_6;             /* Note number for D-6 */
  UWORD  tcik_Sample_D_SHARP_6;     /* Sample number for D#6 */
  UBYTE  tcik_Octave_D_SHARP_6;     /* Octave number for D#6 */
  UBYTE  tcik_Note_D_SHARP_6;       /* Note number for D#6 */
  UWORD  tcik_Sample_E_6;           /* Sample number for E-6 */
  UBYTE  tcik_Octave_E_6;           /* Octave number for E-6 */
  UBYTE  tcik_Note_E_6;             /* Note number for E-6 */
  UWORD  tcik_Sample_F_6;           /* Sample number for F-6 */
  UBYTE  tcik_Octave_F_6;           /* Octave number for F-6 */
  UBYTE  tcik_Note_F_6;             /* Note number for F-6 */
  UWORD  tcik_Sample_F_SHARP_6;     /* Sample number for F#6 */
  UBYTE  tcik_Octave_F_SHARP_6;     /* Octave number for F#6 */
  UBYTE  tcik_Note_F_SHARP_6;       /* Note number for F#6 */
  UWORD  tcik_Sample_G_6;           /* Sample number for G-6 */
  UBYTE  tcik_Octave_G_6;           /* Octave number for G-6 */
  UBYTE  tcik_Note_G_6;             /* Note number for G-6 */
  UWORD  tcik_Sample_G_SHARP_6;     /* Sample number for G#6 */
  UBYTE  tcik_Octave_G_SHARP_6;     /* Octave number for G#6 */
  UBYTE  tcik_Note_G_SHARP_6;       /* Note number for G#6 */
  UWORD  tcik_Sample_A_6;           /* Sample number for A-6 */
  UBYTE  tcik_Octave_A_6;           /* Octave number for A-6 */
  UBYTE  tcik_Note_A_6;             /* Note number for A-6 */
  UWORD  tcik_Sample_A_SHARP_6;     /* Sample number for A#6 */
  UBYTE  tcik_Octave_A_SHARP_6;     /* Octave number for A#6 */
  UBYTE  tcik_Note_A_SHARP_6;       /* Note number for A#6 */
  UWORD  tcik_Sample_B_6;           /* Sample number for B-6 */
  UBYTE  tcik_Octave_B_6;           /* Octave number for B-6 */
  UBYTE  tcik_Note_B_6;             /* Note number for B-6 */
  UWORD  tcik_Sample_C_7;           /* Sample number for C-7 */
  UBYTE  tcik_Octave_C_7;           /* Octave number for C-7 */
  UBYTE  tcik_Note_C_7;             /* Note number for C-7 */
  UWORD  tcik_Sample_C_SHARP_7;     /* Sample number for C#7 */
  UBYTE  tcik_Octave_C_SHARP_7;     /* Octave number for C#7 */
  UBYTE  tcik_Note_C_SHARP_7;       /* Note number for C#7 */
  UWORD  tcik_Sample_D_7;           /* Sample number for D-7 */
  UBYTE  tcik_Octave_D_7;           /* Octave number for D-7 */
  UBYTE  tcik_Note_D_7;             /* Note number for D-7 */
  UWORD  tcik_Sample_D_SHARP_7;     /* Sample number for D#7 */
  UBYTE  tcik_Octave_D_SHARP_7;     /* Octave number for D#7 */
  UBYTE  tcik_Note_D_SHARP_7;       /* Note number for D#7 */
  UWORD  tcik_Sample_E_7;           /* Sample number for E-7 */
  UBYTE  tcik_Octave_E_7;           /* Octave number for E-7 */
  UBYTE  tcik_Note_E_7;             /* Note number for E-7 */
  UWORD  tcik_Sample_F_7;           /* Sample number for F-7 */
  UBYTE  tcik_Octave_F_7;           /* Octave number for F-7 */
  UBYTE  tcik_Note_F_7;             /* Note number for F-7 */
  UWORD  tcik_Sample_F_SHARP_7;     /* Sample number for F#7 */
  UBYTE  tcik_Octave_F_SHARP_7;     /* Octave number for F#7 */
  UBYTE  tcik_Note_F_SHARP_7;       /* Note number for F#7 */
  UWORD  tcik_Sample_G_7;           /* Sample number for G-7 */
  UBYTE  tcik_Octave_G_7;           /* Octave number for G-7 */
  UBYTE  tcik_Note_G_7;             /* Note number for G-7 */
  UWORD  tcik_Sample_G_SHARP_7;     /* Sample number for G#7 */
  UBYTE  tcik_Octave_G_SHARP_7;     /* Octave number for G#7 */
  UBYTE  tcik_Note_G_SHARP_7;       /* Note number for G#7 */
  UWORD  tcik_Sample_A_7;           /* Sample number for A-7 */
  UBYTE  tcik_Octave_A_7;           /* Octave number for A-7 */
  UBYTE  tcik_Note_A_7;             /* Note number for A-7 */
  UWORD  tcik_Sample_A_SHARP_7;     /* Sample number for A#7 */
  UBYTE  tcik_Octave_A_SHARP_7;     /* Octave number for A#7 */
  UBYTE  tcik_Note_A_SHARP_7;       /* Note number for A#7 */
  UWORD  tcik_Sample_B_7;           /* Sample number for B-7 */
  UBYTE  tcik_Octave_B_7;           /* Octave number for B-7 */
  UBYTE  tcik_Note_B_7;             /* Note number for B-7 */
  UWORD  tcik_Sample_C_8;           /* Sample number for C-8 */
  UBYTE  tcik_Octave_C_8;           /* Octave number for C-8 */
  UBYTE  tcik_Note_C_8;             /* Note number for C-8 */
  UWORD  tcik_Sample_C_SHARP_8;     /* Sample number for C#8 */
  UBYTE  tcik_Octave_C_SHARP_8;     /* Octave number for C#8 */
  UBYTE  tcik_Note_C_SHARP_8;       /* Note number for C#8 */
  UWORD  tcik_Sample_D_8;           /* Sample number for D-8 */
  UBYTE  tcik_Octave_D_8;           /* Octave number for D-8 */
  UBYTE  tcik_Note_D_8;             /* Note number for D-8 */
  UWORD  tcik_Sample_D_SHARP_8;     /* Sample number for D#8 */
  UBYTE  tcik_Octave_D_SHARP_8;     /* Octave number for D#8 */
  UBYTE  tcik_Note_D_SHARP_8;       /* Note number for D#8 */
  UWORD  tcik_Sample_E_8;           /* Sample number for E-8 */
  UBYTE  tcik_Octave_E_8;           /* Octave number for E-8 */
  UBYTE  tcik_Note_E_8;             /* Note number for E-8 */
  UWORD  tcik_Sample_F_8;           /* Sample number for F-8 */
  UBYTE  tcik_Octave_F_8;           /* Octave number for F-8 */
  UBYTE  tcik_Note_F_8;             /* Note number for F-8 */
  UWORD  tcik_Sample_F_SHARP_8;     /* Sample number for F#8 */
  UBYTE  tcik_Octave_F_SHARP_8;     /* Octave number for F#8 */
  UBYTE  tcik_Note_F_SHARP_8;       /* Note number for F#8 */
  UWORD  tcik_Sample_G_8;           /* Sample number for G-8 */
  UBYTE  tcik_Octave_G_8;           /* Octave number for G-8 */
  UBYTE  tcik_Note_G_8;             /* Note number for G-8 */
  UWORD  tcik_Sample_G_SHARP_8;     /* Sample number for G#8 */
  UBYTE  tcik_Octave_G_SHARP_8;     /* Octave number for G#8 */
  UBYTE  tcik_Note_G_SHARP_8;       /* Note number for G#8 */
  UWORD  tcik_Sample_A_8;           /* Sample number for A-8 */
  UBYTE  tcik_Octave_A_8;           /* Octave number for A-8 */
  UBYTE  tcik_Note_A_8;             /* Note number for A-8 */
  UWORD  tcik_Sample_A_SHARP_8;     /* Sample number for A#8 */
  UBYTE  tcik_Octave_A_SHARP_8;     /* Octave number for A#8 */
  UBYTE  tcik_Note_A_SHARP_8;       /* Note number for A#8 */
  UWORD  tcik_Sample_B_8;           /* Sample number for B-8 */
  UBYTE  tcik_Octave_B_8;           /* Octave number for B-8 */
  UBYTE  tcik_Note_B_8;             /* Note number for B-8 */
  UWORD  tcik_Sample_C_9;           /* Sample number for C-9 */
  UBYTE  tcik_Octave_C_9;           /* Octave number for C-9 */
  UBYTE  tcik_Note_C_9;             /* Note number for C-9 */
  UWORD  tcik_Sample_C_SHARP_9;     /* Sample number for C#9 */
  UBYTE  tcik_Octave_C_SHARP_9;     /* Octave number for C#9 */
  UBYTE  tcik_Note_C_SHARP_9;       /* Note number for C#9 */
  UWORD  tcik_Sample_D_9;           /* Sample number for D-9 */
  UBYTE  tcik_Octave_D_9;           /* Octave number for D-9 */
  UBYTE  tcik_Note_D_9;             /* Note number for D-9 */
  UWORD  tcik_Sample_D_SHARP_9;     /* Sample number for D#9 */
  UBYTE  tcik_Octave_D_SHARP_9;     /* Octave number for D#9 */
  UBYTE  tcik_Note_D_SHARP_9;       /* Note number for D#9 */
  UWORD  tcik_Sample_E_9;           /* Sample number for E-9 */
  UBYTE  tcik_Octave_E_9;           /* Octave number for E-9 */
  UBYTE  tcik_Note_E_9;             /* Note number for E-9 */
  UWORD  tcik_Sample_F_9;           /* Sample number for F-9 */
  UBYTE  tcik_Octave_F_9;           /* Octave number for F-9 */
  UBYTE  tcik_Note_F_9;             /* Note number for F-9 */
  UWORD  tcik_Sample_F_SHARP_9;     /* Sample number for F#9 */
  UBYTE  tcik_Octave_F_SHARP_9;     /* Octave number for F#9 */
  UBYTE  tcik_Note_F_SHARP_9;       /* Note number for F#9 */
  UWORD  tcik_Sample_G_9;           /* Sample number for G-9 */
  UBYTE  tcik_Octave_G_9;           /* Octave number for G-9 */
  UBYTE  tcik_Note_G_9;             /* Note number for G-9 */
  UWORD  tcik_Sample_G_SHARP_9;     /* Sample number for G#9 */
  UBYTE  tcik_Octave_G_SHARP_9;     /* Octave number for G#9 */
  UBYTE  tcik_Note_G_SHARP_9;       /* Note number for G#9 */
  UWORD  tcik_Sample_A_9;           /* Sample number for A-9 */
  UBYTE  tcik_Octave_A_9;           /* Octave number for A-9 */
  UBYTE  tcik_Note_A_9;             /* Note number for A-9 */
  UWORD  tcik_Sample_A_SHARP_9;     /* Sample number for A#9 */
  UBYTE  tcik_Octave_A_SHARP_9;     /* Octave number for A#9 */
  UBYTE  tcik_Note_A_SHARP_9;       /* Note number for A#9 */
  UWORD  tcik_Sample_B_9;           /* Sample number for B-9 */
  UBYTE  tcik_Octave_B_9;           /* Octave number for B-9 */
  UBYTE  tcik_Note_B_9;             /* Note number for B-9 */
}; /* TuComposerInstrKeyboard */

/* Apreggio data structure. Used by arpeggio only. */
struct TuComposerInstrArpeggioData {
  UBYTE  tciad_Tone;                /* Packed note or NULL if arp note */
  BYTE   tciad_Transpose;           /* Transpose */
  UWORD  tciad_Instrument;          /* Instrument number or NULL for arpeggio
                                       instrument */
  UBYTE  tciad_EffectByte;          /* First effect byte */
  UBYTE  tciad_EffectByte2;         /* Second effect byte */
  UBYTE  tciad_EffectByte3;         /* Third effect byte */
  UBYTE  tciad_EffectByte4;         /* Fourth (last) effect byte */
  UWORD  tciad_DataWord;            /* Data word (for set volume, etc.) */
  UWORD  tciad_DataWord2;           /* The 2nd data word of this tick */
  UWORD  tciad_DataWord3;           /* The 3rd data word of this tick */
  UWORD  tciad_DataWord4;           /* The 4th data word of this tick */
  UBYTE  tciad_NextEntry[0];        /* Next entry */
}; /* TuComposerInstrArpeggioData */

/* Arpeggio structure. Used by command arpeggio only. */
struct TuComposerInstrArpeggio {
  struct MinNode tcia_Node;
  UWORD  tcia_Flags;                /* Special flags (look upwards) */
  struct TuComposerInstrArpeggioData *tcia_Data; /* Ptr to arpeggio data */
  UWORD  tcia_Entries;              /* Number of entries */
  UWORD  tcia_SustainStart;         /* Sustain start tick */
  UWORD  tcia_SustainEnd;           /* Sustain end tick */
  UWORD  tcia_SustainCount;         /* Sustain loop counter */
  UWORD  tcia_LoopStart;            /* Loop start tick */
  UWORD  tcia_LoopEnd;              /* Loop end tick */
  UWORD  tcia_LoopCount;            /* Loop counter */
  UBYTE  tcia_Reserved[8];          /* Reserved for future expansions */
  APTR   tcia_UserData;             /* Just for you */
}; /* TuComposerInstrArpeggio */

/* Default values for arpeggio structures */
#define DEFAULT_ARPPOINTS       3L  /* Default number of points is 3 */

/* Instrument structure used by all Instrument functions (e.g. AddInstr()) */
struct TuComposerInstr {
  struct MinNode tci_Node;
  STRPTR tci_FileName;              /* Pointer to sample file name
                                       (e.g. C-Hihat.TCI).
                                       Remember, that TuComposer is also
                                       available for MS-DOS and other
                                       platforms. So, please try to use the
                                       limited 8x3 MS-DOS file/dir name
                                       length. */
  STRPTR tci_Name;                  /* Pointer to the instrument name
                                       (e.g. Close-Hihat (Echo)) */
  struct MinList tci_Samples;       /* Linked list of samples */
  struct TuComposerInstrEnvelope *tci_VolumeEnvelope; /* Volume env used */
  struct TuComposerInstrEnvelope *tci_PanningEnvelope; /* Pan env used */
  struct TuComposerInstrEnvelope *tci_SlideEnvelope; /* Slide env used */
  struct TuComposerInstrEnvelope *tci_VibratoEnvelope; /* Vibrato env */
  struct TuComposerInstrEnvelope *tci_TremoloEnvelope; /* Tremolo env */
  struct TuComposerInstrEnvelope *tci_PannoloEnvelope; /* Pannolo env */
  struct TuComposerInstrEnvelope *tci_ChannoloEnvelope; /* Channolo env */
  struct TuComposerInstrEnvelope *tci_SpenoloEnvelope; /* Spenolo env */
  struct TuComposerInstrArpeggio *tci_ArpeggioStructure; /* Arp struct */
  struct TuComposerInstrKeyboard *tci_KeyboardDefs; /* Keyboard defs */
  UBYTE  tci_NumSamples;            /* Number of samples */
  UBYTE  tci_GlobalVolume;          /* Global volume for instr samples */
  UBYTE  tci_NNA;                   /* New note action */
  UBYTE  tci_NoteSwing;             /* Random note swing (semitones) */
  UWORD  tci_VolumeSwing;           /* Volume swing (in %, 256 = 100%) */
  UWORD  tci_PanningSwing;          /* Panning swing */
  ULONG  tci_PitchSwing;            /* Pitch swing (in %, 65536 = 100%) */
  WORD   tci_PitchPanSeparation;    /* Pitch panning separation */
  UBYTE  tci_DefaultPanning;        /* Default panning for all samples */
  UBYTE  tci_DefaultSubPan;         /* Default sub pan for all samples */
  UBYTE  tci_DCT;                   /* Dup note check type (see upwards) */
  UBYTE  tci_DNA;                   /* Dup note check action (see below) */
  UBYTE  tci_CompFlags;             /* Flags for compatibility (upwards) */
  UBYTE  tci_Flags;                 /* Special flags (see upwards) */
  UWORD  tci_EnvUsageFlags;         /* Envelope usage flags */
  UWORD  tci_EnvProcFlags;          /* Envelope processing flags */
  UWORD  tci_EnvRetrigFlags;        /* Envelope retrigger flags */
  UWORD  tci_EnvRandomFlags;        /* Envelope random flags */
  UWORD  tci_EnvRndDelayFlags;      /* Envelope random speed is delay flag */
  UWORD  tci_FadeOut;               /* Fade out value */
  UWORD  tci_Hold;                  /* Hold value */
  UWORD  tci_Decay;                 /* Decay value */
  UBYTE  tci_DCA;                   /* Decay action when decay is off */
  UBYTE  tci_PitchPanCenter;        /* Pitch pan center (0 - C-0, etc.) */
  UBYTE  tci_MIDICh;                /* MIDI channel */
  UBYTE  tci_MIDIProgram;           /* MIDI program (instrument) */
  UBYTE  tci_MIDIFlags;             /* MIDI flags (look upwards) */
  BYTE   tci_MIDITranspose;         /* MIDI transpose (in halftones) */
  UBYTE  tci_MIDIAftertouch;        /* MIDI aftertouch (percentage) */
  UBYTE  tci_MIDIBender;            /* MIDI pitchbender (in halftones) */
  struct MinList tci_UnknownChunks; /* Linked list of unknown chunks */
  UBYTE  tci_Reserved[8]; /* Reserved for future expansions */
  APTR   tci_UserData;              /* Free for any user data. */
}; /* TuComposerInstrument */

/* Instrument default volumes */
#define DEFAULTPITCHPANCENTER   (4*12) /* Default pitch pan center
                                          is C-4 */

/* tci_DNA definitions */
#define DNA_NOTE_CUT            0   /* Do note cut (as normal) */
#define DNA_NOTE_OFF            1   /* Perform keyoff */
#define DNA_NOTE_FADE           2   /* Fadeoff notes */
#define DNA_NOTE_CONTINUE       3   /* Nothing (only useful for synth DNA) */

/* Fade out related definitions */
#define FADE_OUT_START          65535L /* Default is 65535 as in FT2. */

/* tci_NNA definitions */
#define NNA_NOTE_CUT            0   /* Cut note (as normal) */
#define NNA_NOTE_CONTINUE       1   /* Cont note (allocate new channel) */
#define NNA_NOTE_OFF            2   /* Perform keyoff note */
#define NNA_NOTE_FADE           3   /* Fade out note (uses tci_FadeOut) */

#define ERROR_INSTR_NOGIVEN     (ERROR_INSTR+0x0) /* Instrument not given */
#define ERROR_INSTR_NODATA      (ERROR_INSTR+0x1) /* Instrument contains
                                       no data */
#define ERROR_INSTR_NOSMPL      (ERROR_INSTR+0x2) /* No samples in instr */
#define ERROR_INSTR_NOMEM       (ERROR_INSTR+0x3) /* No memory for
                                       operation */
#define ERROR_INSTR_NOFILE      (ERROR_INSTR+0x4) /* No file given */
#define ERROR_INSTR_IOERR       (ERROR_INSTR+0x5) /* I/O error occured */
#define ERROR_INSTR_BAD_IFF     (ERROR_INSTR+0x6) /* TCI1-IFF-file damaged */
/* Because LoadInstr() returns NULL on error, use DOS IoErr() and check
   ERROR_INVALID_RESIDENT_LIBRARY for it. */

#define ERROR_INSTR_BAD_VER     (ERROR_INSTR+0x7) /* Incorrect instrument
                                       version */
/* Because LoadInstr() returns NULL on error, use DOS IoErr() and check
   ERROR_ACTION_NOT_KNOWN for it. */

#define ERROR_INSTR_SAMPLES     (ERROR_INSTR+0x8) /* Too many smp in TCI1 */
/* Because LoadInstr() returns NULL on error, use DOS IoErr() and check
   ERROR_TOO_MANY_LEVELS for it. */

#define ERROR_INSTR_NOENVL      (ERROR_INSTR+0x9) /* Envelope not given */
#define ERROR_INSTR_ENVDATA     (ERROR_INSTR+0xA) /* Env contains no data */
#define ERROR_INSTR_ENVLMEM     (ERROR_INSTR+0xB) /* No mem for env op */
#define ERROR_INSTR_ENVOVFL     (ERROR_INSTR+0xC) /* Envelope entry
                                       overflow */
#define ERROR_INSTR_ENVLESS     (ERROR_INSTR+0xD) /* Envelope is too small */
#define ERROR_INSTR_ENVBIG      (ERROR_INSTR+0xE) /* Envelope is too big */
#define ERROR_INSTR_ENVSCLR     (ERROR_INSTR+0xF) /* Bad envelope scaler */

/* IFF-TCM1 IHDR structure. Used by disk operations
   (e.g. by LoadTCModule() & SaveTCModule()) */

struct MIFF_IHDR {
  struct IFFChunk mihdr_IFFHeader;
  UWORD  mihdr_VolumeEnvNum;        /* Number of volume envelopes (or 0) */
  UWORD  mihdr_PanningEnvNum;       /* Number of panning envs (or 0) */
  UWORD  mihdr_SlideEnvNum;         /* Number of slide envelope (or 0) */
  UWORD  mihdr_VibratoEnvNum;       /* Number of vibrato envelope (or 0) */
  UWORD  mihdr_TremoloEnvNum;       /* Number of tremolo envelope (or 0) */
  UWORD  mihdr_PannoloEnvNum;       /* Number of pannolo envelope (or 0) */
  UWORD  mihdr_ChannoloEnvNum;      /* Number of channolo envelope (or 0) */
  UWORD  mihdr_SpenoloEnvNum;       /* Number of spenolo envelope (or 0) */
  UWORD  mihdr_ArpeggioDefNum;      /* Number of arpeggio def (or 0) */
  UWORD  mihdr_KeyboardDefNum;      /* Number of keyboard def (or 0) */
  UBYTE  mihdr_NumSamples;          /* Number of samples */
  UBYTE  mihdr_GlobalVolume;        /* Global volume for instr samples */
  UBYTE  mihdr_NNA;                 /* New note action */
  UBYTE  mihdr_NoteSwing;           /* Random note swing (semitones) */
  UWORD  mihdr_VolumeSwing;         /* Volume swing */
  UWORD  mihdr_PanningSwing;        /* Panning swing */
  ULONG  mihdr_PitchSwing;          /* Pitch swing */
  WORD   mihdr_PitchPanSep;         /* Pitch panning separation */
  UBYTE  mihdr_DefaultPanning;      /* Default panning for all samples */
  UBYTE  mihdr_DefaultSubPan;       /* Default sub panning for all samples */
  UBYTE  mihdr_DCT;                 /* Dup note check type (see upwards) */
  UBYTE  mihdr_DNA;                 /* Dup note check action (see below) */
  UBYTE  mihdr_CompFlags;           /* Flags for compatibility (upwards) */
  UBYTE  mihdr_Flags;               /* Special flags (see upwards) */
  UWORD  mihdr_EnvUsageFlags;       /* Envelope usage flags */
  UWORD  mihdr_EnvProcFlags;        /* Envelope processing flags */
  UWORD  mihdr_EnvRetrigFlags;      /* Envelope retrigger flags */
  UWORD  mihdr_EnvRandomFlags;      /* Envelope randomize flags */
  UWORD  mihdr_EnvRndDelayFlags;    /* Env random speed is delay flags */
  UWORD  mihdr_FadeOut;             /* Fade out value */
  UWORD  mihdr_Hold;                /* Hold value */
  UWORD  mihdr_Decay;               /* Decay value */
  UBYTE  mihdr_DCA;                 /* Decay action when decay is off */
  UBYTE  mihdr_PitchPanCenter;      /* Pitch panning center (0 - C-0) */
  UBYTE  mihdr_MIDICh;              /* MIDI channel */
  UBYTE  mihdr_MIDIProgram;         /* MIDI program (instrument) */
  UBYTE  mihdr_MIDIFlags;           /* MIDI flags (look upwards) */
  BYTE   mihdr_MIDITranspose;       /* MIDI transpose (in halftones) */
  UBYTE  mihdr_MIDIAftertouch;      /* MIDI aftertouch (percentage) */
  UBYTE  mihdr_MIDIBender;          /* MIDI pitchbender (in halftones) */
}; /* MIFF_IHDR */

/* IFF-TCI1 IHDR structure. Used by disk operations
   (e.g. by LoadInstr() & SaveInstr()) */

struct IFF_IHDR {
  struct IFFChunk ihdr_IFFHeader;
  UBYTE  ihdr_Version;              /* Version number of TCI1 file */
  UBYTE  ihdr_Revision;             /* Revision number of TCI1 file */
  UWORD  ihdr_VolumeEnvNum;         /* Number of volume envelopes (or 0) */
  UWORD  ihdr_PanningEnvNum;        /* Number of panning envs (or 0) */
  UWORD  ihdr_SlideEnvNum;          /* Number of slide envelope (or 0) */
  UWORD  ihdr_VibratoEnvNum;        /* Number of vibrato envelope (or 0) */
  UWORD  ihdr_TremoloEnvNum;        /* Number of tremolo envelope (or 0) */
  UWORD  ihdr_PannoloEnvNum;        /* Number of pannolo envelope (or 0) */
  UWORD  ihdr_ChannoloEnvNum;       /* Number of channolo envelope (or 0) */
  UWORD  ihdr_SpenoloEnvNum;        /* Number of spenolo envelope (or 0) */
  UWORD  ihdr_ArpeggioDefNum;       /* Number of arpeggio def (or 0) */
  UWORD  ihdr_KeyboardDefNum;       /* Number of keyboard def (or 0) */
  UBYTE  ihdr_NumSamples;           /* Number of samples */
  UBYTE  ihdr_GlobalVolume;         /* Global volume for instr samples */
  UBYTE  ihdr_NNA;                  /* New note action */
  UBYTE  ihdr_NoteSwing;            /* Random note swing (semitones) */
  UWORD  ihdr_VolumeSwing;          /* Volume swing */
  UWORD  ihdr_PanningSwing;         /* Panning swing */
  ULONG  ihdr_PitchSwing;           /* Pitch swing */
  WORD   ihdr_PitchPanSep;          /* Pitch panning separation */
  UBYTE  ihdr_DefaultPanning;       /* Default panning for all samples */
  UBYTE  ihdr_DefaultSubPan;        /* Default sub panning for all samples */
  UBYTE  ihdr_DCT;                  /* Dup note check type (see upwards) */
  UBYTE  ihdr_DNA;                  /* Dup note check action (see below) */
  UBYTE  ihdr_CompFlags;            /* Flags for compatibility (upwards) */
  UBYTE  ihdr_Flags;                /* Special flags (see upwards) */
  UWORD  ihdr_EnvUsageFlags;        /* Envelope usage flags */
  UWORD  ihdr_EnvProcFlags;         /* Envelope processing flags */
  UWORD  ihdr_EnvRetrigFlags;       /* Envelope retrigger flags */
  UWORD  ihdr_EnvRandomFlags;       /* Envelope randomize flags */
  UWORD  ihdr_EnvRndDelayFlags;     /* Env random speed is delay flags */
  UWORD  ihdr_FadeOut;              /* Fade out value */
  UWORD  ihdr_Hold;                 /* Hold value */
  UWORD  ihdr_Decay;                /* Decay value */
  UBYTE  ihdr_DCA;                  /* Decay action when decay is off */
  UBYTE  ihdr_PitchPanCenter;       /* Pitch panning center (0 - C-0) */
  UBYTE  ihdr_MIDICh;               /* MIDI channel */
  UBYTE  ihdr_MIDIProgram;          /* MIDI program (instrument) */
  UBYTE  ihdr_MIDIFlags;            /* MIDI flags (look upwards) */
  BYTE   ihdr_MIDITranspose;        /* MIDI transpose (in halftones) */
  UBYTE  ihdr_MIDIAftertouch;       /* MIDI aftertouch (percentage) */
  UBYTE  ihdr_MIDIBender;           /* MIDI pitchbender (in halftones) */
}; /* IFF_IHDR */

/* IFF-TCM1/IFF-TCI1 EHDR structure. Used by disk operations.
   (e.g. by LoadModule() & SaveModule()) */

struct MIFF_EHDR {
  struct IFFChunk mehdr_IFFHeader;
  UWORD  mehdr_Flags;               /* Envelope flags (see upwards) */
  UWORD  mehdr_Tempo;               /* Envelope tempo in ticks (default 1) */
  UWORD  mehdr_SustainStart;        /* Sustain start point */
  UWORD  mehdr_SustainEnd;          /* Sustain end point */
  UWORD  mehdr_SustainCount;        /* Sustain counter */
  UWORD  mehdr_LoopStart;           /* Loop start point */
  UWORD  mehdr_LoopEnd;             /* Loop end point */
  UWORD  mehdr_LoopCount;           /* Loop counter */
  WORD   mehdr_LowestValue;         /* Random lowest value */
  WORD   mehdr_HighestValue;        /* Random highest value */
}; /* MIFF_EHDR */

/* IFF-TCE1 EHDR structure. Used by disk operations.
   (e.g. by LoadEnvelope() & SaveEnvelope()) */

struct IFF_EHDR {
  struct IFFChunk ehdr_IFFHeader;
  UBYTE  ehdr_Version;              /* Version number (currently 1) */
  UBYTE  ehdr_Revision;             /* Revision number (currently 0) */
  UWORD  ehdr_Flags;                /* Envelope flags (see upwards) */
  UWORD  ehdr_Tempo;                /* Envelope tempo in ticks (default 1) */
  UWORD  ehdr_SustainStart;         /* Sustain start point */
  UWORD  ehdr_SustainEnd;           /* Sustain end point */
  UWORD  ehdr_SustainCount;         /* Sustain counter */
  UWORD  ehdr_LoopStart;            /* Loop start point */
  UWORD  ehdr_LoopEnd;              /* Loop end point */
  UWORD  ehdr_LoopCount;            /* Loop counter */
  WORD   ehdr_LowestValue;          /* Random lowest value */
  WORD   ehdr_HighestValue;         /* Random highest value */
}; /* IFF_EHDR */

/* IFF-TCI1 KBRD structure. Used by disk operations.
   Here is stored the keyboard table (play at which note is which sample) */

struct IFF_KBRD {
  struct IFFChunk kbrd_IFFHeader;
  struct KeyboardEntry kbrd_Keys[120];
}; /* IFF_KBRD */

/* IFF-TCM1/IFF-TCI1 AHDR structure. Used by disk operations.
   (e.g. by LoadModule() & SaveModule()) */

struct IFF_AHDR {
  struct IFFChunk ahdr_IFFHeader;
  UWORD  ahdr_Length;               /* Number of arpeggio ticks */
  UWORD  ahdr_Flags;                /* Special flags look upwards */
  UWORD  ahdr_SustainStart;         /* Sustain start tick */
  UWORD  ahdr_SustainEnd;           /* Sustain end tick */
  UWORD  ahdr_SustainCount;         /* Sustain counter */
  UWORD  ahdr_LoopStart;            /* Loop start tick */
  UWORD  ahdr_LoopEnd;              /* Loop end tick */
  UWORD  ahdr_LoopCount;            /* Loop counter */
}; /* IFF_AHDR */

/* IFF-TCM1/IFF-TCI1 ARPE structure. Used by disk operations.
   (e.g. by LoadModule() & SaveModule()) */

struct IFF_ARPE {
  struct IFFChunk arpe_IFFHeader;
  UBYTE  arpe_Tone;                 /* Packed note or NULL if arp note */
  BYTE   arpe_Transpose;            /* Transpose */
  UWORD  arpe_Instrument;           /* Instrument number or NULL for arpeggio
                                       instrument */
  UBYTE  arpe_EffectByte;           /* First effect byte */
  UBYTE  arpe_EffectByte2;          /* Second effect byte */
  UBYTE  arpe_EffectByte3;          /* Third effect byte */
  UBYTE  arpe_EffectByte4;          /* Fourth (last) effect byte */
  UWORD  arpe_DataWord;             /* Data word (for set volume, etc.) */
  UWORD  arpe_DataWord2;            /* The 2nd data word of this tick */
  UWORD  arpe_DataWord3;            /* The 3rd data word of this tick */
  UWORD  arpe_DataWord4;            /* The 4th data word of this tick */
}; /* IFF_ARPE */

#define IHDR_VERSION            1   /* Current instr version: 1 */
#define IHDR_REVISION           0   /* Current instr revision: 0 */
#define EHDR_VERSION            1   /* Current envelope version: 1 */
#define EHDR_REVISION           0   /* Current envelope revision: 0 */

#endif /* TUCOMPOSER_INSTR_H */
