#ifndef TUCOMPOSER_MODULE_H
#define TUCOMPOSER_MODULE_H
/*
**  $VER: module.h 31.22 (29.3.99)
**
**  Standard C header for tucomposer.library
**
**  (C) Copyright 1999 by Basty/Seasons
**      All Rights Reserved
*/

#ifndef EXEC_TYPES_H
#include <exec/types.h>
#endif /* EXEC_TYPES_H */

#ifndef EXEC_LISTS_H
#include <exec/lists.h>
#endif /* EXEC_LISTS_H */

#ifndef EXEC_NODES_H
#include <exec/nodes.h>
#endif /* EXEC_NODES_H */

#ifndef EXEC_TASKS_H
#include <exec/tasks.h>
#endif /* EXEC_TASKS_H */

#ifndef TUCOMPOSER_TUCOMPOSER_H
#include <tucomposer/tucomposer.h>
#endif /* TUCOMPOSER_TUCOMPOSER_H */

/* No bit definitions for tcm_CompFlags & tcm_Flags yet. */

/* Module structure, used by all modules functions (e.g. PlayTCModule()) */
struct TuComposerModule {
  struct MinNode tcm_Node;
  struct Task *tcm_Task;               /* Owner task of this module */
  STRPTR tcm_FileName;                 /* File name of current module */
  STRPTR tcm_Name;                     /* Name of current module */
  STRPTR tcm_Message;                  /* Message to the whole module (ANSI
                                          & multiple lines allowed */
  STRPTR tcm_Composer;                 /* Module composer(s) */
  STRPTR tcm_Style;                    /* Module style(s) */
  STRPTR tcm_Comment;                  /* A short comment from the composer
                                          to the whole module */
  struct PlayerChannel *tcm_ChannelData; /* Pointer to virtual chan. data */
  struct StampDate tcm_BeginDate;      /* Begin date of composing */
  struct StampDate tcm_EndDate;        /* Finish date of composing */
  UBYTE  tcm_PlayTimeHours;            /* Play time of whole module in h */
  UBYTE  tcm_PlayTimeMins;             /* Play time of whole module in m */
  UBYTE  tcm_PlayTimeSecs;             /* Play time of whole module in s */
  UBYTE  tcm_PlayTimeHundreds;         /* Play time of whole module 1/100s */
  struct MinList tcm_SongList;         /* Song list of current module */
  struct MinList tcm_InstrList;        /* Linked list of instruments in
                                          the whole module */
  struct MinList tcm_Envelopes;        /* Linked list of envelopes used */
  struct MinList tcm_KeyboardDefs;     /* Linked list of keyboard defs */
  struct MinList tcm_ArpEnvList;       /* Linked list of arpeggio envs */
  UWORD  tcm_Channels;                 /* Maximum channels (incl. NNA back-
                                          ground channels) to be allocated
                                          by the mix routine (default 16) */
  UBYTE  tcm_CompFlags;                /* Compatibility flags (see
                                          upwards) */
  UBYTE  tcm_Flags;                    /* Module flags (see upwards) */
  struct MinList tcm_UnknownChunks;    /* Linked list of unknown chunks */
  UBYTE  tcm_Reserved[8];              /* Reserved for future expansions. */
  APTR   tcm_UserData;                 /* This field is just for you ! */
}; /* TuComposerModule */

#define DEFAULTNNACHANNELS   16        /* 16 channels are used by defaults */

/* Module error codes */
#define ERROR_MODULE_NOGIVEN (ERROR_MODULE+0x0) /* Set if no MOD was given */
#define ERROR_MODULE_NODATA  (ERROR_MODULE+0x1) /* Module contains no data */
#define ERROR_MODULE_NOMEM   (ERROR_MODULE+0x2) /* Returned on mem fail */
#define ERROR_MODULE_NOFILE  (ERROR_MODULE+0x4) /* No file given */
#define ERROR_MODULE_IOERR   (ERROR_MODULE+0x5) /* I/O error occured on */
#define ERROR_MODULE_BAD_IFF (ERROR_MODULE+0x6) /* TCM1-IFF-file damaged */

/* Because LoadTCModule() returns NULL on error, use DOS IoErr() and check
   ERROR_INVALID_RESIDENT_LIBRARY for it. */

#define ERROR_MODULE_BAD_VER (ERROR_MODULE+0x7) /* Incorrect module
                                          version */
#define ERROR_MODULE_NOKEYBOARD (ERROR_MODULE+0x8) /* If no keyboard def
                                          was set, this is returned */
#define ERROR_MODULE_BADOCTAVE (ERROR_MODULE+0x9) /* Bad octave given to
                                          keyboard function */
#define ERROR_MODULE_BADNOTE (ERROR_MODULE+0xA) /* Bad note given to
                                          keyboard function */

/* TuComposer Module Header. Used by disk operations
   (e.g. by LoadTCModule() & SaveTCModule()) */

struct IFF_MHDR {
  struct IFFChunk mhdr_IFFHeader;
  UBYTE  mhdr_Version;                 /* Module version (currently 01) */
  UBYTE  mhdr_Revision;                /* Module revision (currently 00) */
  struct StampDate mhdr_BeginDate;     /* Begin date of composing */
  struct StampDate mhdr_EndDate;       /* Finish date of composing */
  UBYTE  mhdr_PlayTimeHours;           /* Play time of whole MOD in hours */
  UBYTE  mhdr_PlayTimeMins;            /* Play time of whole MOD in mins */
  UBYTE  mhdr_PlayTimeSecs;            /* Play time of whole MOD in secs */
  UBYTE  mhdr_PlayTimeHundreds;        /* Play time of whole MOD in 1/100s */
  UWORD  mhdr_Songs;                   /* Number of songs in this module */
  ULONG  mhdr_Tracks;                  /* Number of total tracks */
  UWORD  mhdr_Instruments;             /* Number of instruments */
  ULONG  mhdr_Samples;                 /* Number of total samples */
  ULONG  mhdr_Synths;                  /* Number of total synths */
  UWORD  mhdr_Envelopes;               /* Number of envelopes */
  UWORD  mhdr_KeyboardDefs;            /* Number of keyboard definitions */
  UWORD  mhdr_ArpeggioDefs;            /* Number of arpeggio definitions */
  UWORD  mhdr_Channels;                /* Maximum channels allowed */
  UBYTE  mhdr_CompFlags;               /* Compatibility flags (upwards) */
  UBYTE  mhdr_Flags;                   /* Flags (see upwards) */
}; /* IFF_MHDR */

#define CURRENT_VERSION      1         /* Current version of MOD format. */
#define CURRENT_REVISION     0         /* Current revision of MOD format. */

#endif /* TUCOMPOSER_MODULE_H */
