#ifndef TUCOMPOSER_PLAYER_H
#define TUCOMPOSER_PLAYER_H
/*
**  $VER: player.h 31.22 (29.3.99)
**
**  Standard C header for tucomposer.library
**
**  (C) Copyright 1999 by Basty/Seasons
**      All Rights Reserved
*/

#ifndef EXEC_TYPES_H
#include <exec/types.h>
#endif /* EXEC_TYPES_H */

#ifndef EXEC_LISTS_H
#include <exec/lists.h>
#endif /* EXEC_LISTS_H */

#ifndef EXEC_NODES_H
#include <exec/nodes.h>
#endif /* EXEC_NODES_H */

#ifndef TUCOMPOSER_TUCOMPOSER_H
#include <tucomposer/tucomposer.h>
#endif /* TUCOMPOSER_TUCOMPOSER_H */

#ifndef TUCOMPOSER_POSITION_H
#include <tucomposer/position.h>
#endif /* TUCOMPOSER_POSITION_H */

#ifndef TUCOMPOSER_TRACK_H
#include <tucomposer/track.h>
#endif /* TUCOMPOSER_TRACK_H */

#ifndef TUCOMPOSER_INSTR_H
#include <tucomposer/instr.h>
#endif /* TUCOMPOSER_INSTR_H */

#ifndef TUCOMPOSER_SAMPLE_H
#include <tucomposer/sample.h>
#endif /* TUCOMPOSER_SAMPLE_H */

#ifndef TUCOMPOSER_SYNTH_H
#include <tucomposer/synth.h>
#endif /* TUCOMPOSER_SYNTH_H */

#ifndef TUCOMPOSER_EXTERNAL_EXTERNAL_H
#include <tucomposer/external/external.h>
#endif /* TUCOMPOSER_EXTERNAL_EXTERNAL_H */

#ifndef TUCOMPOSER_EXTERNAL_MIXER_H
#include <tucomposer/external/mixer.h>
#endif /* TUCOMPOSER_EXTERNAL_MIXER_H */

/* Player envelope structure flags: */
#define PESB_FirstAdd              0   /* First add envelope then get val */
#define PESB_NoRetrig              1   /* Envelope won't be retriggered on
                                          new note play */
#define PESB_Random                2   /* Envelope returns random values */
#define PESB_RndDelay              3   /* If random enabled, speed will be
                                          interpreted as a delay */
#define PESB_Backwards             4   /* Playing currently backwards */
#define PESB_Looping               5   /* Currently looping (either sustain
                                          or normal loop) */
#define PESB_Pingpong              6   /* Current loop is pingpong */

#define PESF_FirstAdd              (1<<PESB_FirstAdd)
#define PESF_NoRetrig              (1<<PESB_NoRetrig)
#define PESF_Random                (1<<PESB_Random)
#define PESF_RndDelay              (1<<PESB_RndDelay)
#define PESF_Backwards             (1<<PESB_Backwards)
#define PESF_Looping               (1<<PESB_Looping)
#define PESF_Pingpong              (1<<PESB_Pingpong)

/* Player envelope structure repeat flags: */
#define PESRB_Loop                 0   /* Loop points used */
#define PESRB_Sustain              1   /* Sustain points used */
#define PESRB_Pingpong             2   /* Ping pong loop */
#define PESRB_SustainPingpong      3   /* Sustain ping pong loop */

#define PESRF_Loop                 (1<<PESRB_Loop)
#define PESRF_Sustain              (1<<PESRB_Sustain)
#define PESRF_Pingpong             (1<<PESRB_Pingpong)
#define PESRF_SustainPingpong      (1<<PESRB_SustainPingpong)

/* Player globals flags: */
#define PGB_PlayOnce               0   /* Play song in once mode, i.e.
                                          stop after played it */
#define PGB_NoProcPattern          1   /* Don't process patterns (i.e.
                                          process instrument data only) */
#define PGB_PlayPattern            2   /* Play pattern only (don't break) */
#define PGB_Surround               3   /* Global panning is surround pan */
#define PGB_SongEnd                4   /* Song end found. This bit is used
                                          by the play exec handlers and the
                                          play time calculation routine. */
#define PGB_SPD                    5   /* SPD timing used */
#define PGB_TraceMode              6   /* Single step mode enabled */

#define PGF_PlayOnce               (1<<PGB_PlayOnce)
#define PGF_NoProcPattern          (1<<PGB_NoProcPattern)
#define PGF_PlayPattern            (1<<PGB_PlayPattern)
#define PGF_Surround               (1<<PGB_Surround)
#define PGF_SongEnd                (1<<PGB_SongEnd)
#define PGF_SPD                    (1<<PGB_SPD)
#define PGF_TraceMode              (1<<PGB_TraceMode)

/* Player host channel flags: */
#define PHCB_LinearFreq            0L  /* Use linear frequence table */
#define PHCB_Backwards             1L  /* Playing track backwards */
#define PHCB_PatternBreak          2L  /* Pattern break */
#define PHCB_SmpOffsetRel          3L  /* Sample offset relative */
#define PHCB_TrackSurPan           4L  /* Track surround panning used */
#define PHCB_AffectChanPan         5L  /* Channel panning is affected */
#define PHCB_ChannelSurPan         6L  /* Channel surround panning used */
#define PHCB_ExecFX                7L  /* Exec FX called */
#define PHCB_TonePorta             8L  /* Tone portamento executed */
#define PHCB_SetTranspose          9L  /* Set transpose/finetune executed */
#define PHCB_SubSlideRetrig       10L  /* Allow sub-slides in multi retrig */
#define PHCB_TremorExec           11L  /* Tremor executed (i.e. holded) */
#define PHCB_TremorOff            12L  /* Tremor off bit */
#define PHCB_RetrigNote           13L  /* Note retrig bit, set by a new note,
                                          cleared by multi retrig and after
                                          each row */
#define PHCB_Vibrato              14L  /* Vibrato hold (i.e. executed) */
#define PHCB_Tremolo              15L  /* Tremolo hold (i.e. executed) */
#define PHCB_ChgPattern           16L  /* Pattern changed */
#define PHCB_PattLoop             17L  /* Performing pattern loop */
#define PHCB_PattLoopJmp          18L  /* Pattern loop jumped */
#define PHCB_PattReset            19L  /* Pattern loop needs to be reset */
#define PHCB_SetInstr             20L  /* Flag set by PlayInstr(), cleared
                                          afterwards. Instrument to be played
                                          is put into phc_Instrument */
#define PHCB_SetSample            21L  /* Flag set by PlaySample(), cleared
                                          afterwards. Sample to be played
                                          is put into phc_Sample. If this
										  and previous is set, the track
										  at phc_Instrument will be played
										  at row phc_Sample */
#define PHCB_SongEnd              31L  /* Song end found for this channel */

#define PHCF_LinearFreq            (1L<<PHCB_LinearFreq)
#define PHCF_Backwards             (1L<<PHCB_Backwards)
#define PHCF_PatternBreak          (1L<<PHCB_PatternBreak)
#define PHCF_SmpOffsetRel          (1L<<PHCB_SmpOffsetRel)
#define PHCF_TrackSurPan           (1L<<PHCB_TrackSurPan)
#define PHCF_AffectChanPan         (1L<<PHCB_AffectChanPan)
#define PHCF_ChannelSurPan         (1L<<PHCB_ChannelSurPan)
#define PHCF_ExecFX                (1L<<PHCB_ExecFX)
#define PHCF_TonePorta             (1L<<PHCB_TonePorta)
#define PHCF_SetTranspose          (1L<<PHCB_SetTranspose)
#define PHCF_SubSlideRetrig        (1L<<PHCB_SubSlideRetrig)
#define PHCF_TremorExec            (1L<<PHCB_TremorExec)
#define PHCF_TremorOff             (1L<<PHCB_TremorOff)
#define PHCF_RetrigNote            (1L<<PHCB_RetrigNote)
#define PHCF_Vibrato               (1L<<PHCB_Vibrato)
#define PHCF_Tremolo               (1L<<PHCB_Tremolo)
#define PHCF_ChgPattern            (1L<<PHCB_ChgPattern)
#define PHCF_PattLoop              (1L<<PHCB_PattLoop)
#define PHCF_PattLoopJmp           (1L<<PHCB_PattLoopJmp)
#define PHCF_PattReset             (1L<<PHCB_PattReset)
#define PHCF_SetInstr              (1L<<PHCB_SetInstr)
#define PHCF_SetSample             (1L<<PHCB_SetSample)
#define PHCF_SongEnd               (1L<<PHCB_SongEnd)

/* Player fine slide flags: */
#define FSB_FinePortaDown          0L  /* Fine portamento is directed down */
#define FSB_PortaOnceDown          1L  /* Portamento once is directed down */
#define FSB_FinePortaOnceDown      2L  /* Fine porta once is directed down */
#define FSB_FinePorta              3L  /* Fine portamento used */
#define FSB_PortaOnce              4L  /* Portamento once used */
#define FSB_FineTonePorta          5L  /* Fine tone portamento used */
#define FSB_TonePortaOnce          6L  /* Tone portamento once used */
#define FSB_VolSlideDown           7L  /* Volume slide directed downwards */
#define FSB_FineVolSlideDown       8L  /* Fine volume slide directed down */
#define FSB_FineVolSlide           9L  /* Fine volume slide used */
#define FSB_TrackVolSlideDown     10L  /* Track volume slide directed down */
#define FSB_FineTrackVolSlideDown 11L  /* Fine track vol slide downwards */
#define FSB_FineTrackVolSlide     12L  /* Fine track volume slide used */
#define FSB_PanSlideRight         13L  /* Panning slide is directed right */
#define FSB_FinePanSlideRight     14L  /* Fine pan slide is directed right */
#define FSB_FinePanSlide          15L  /* Fine panning slide is used */
#define FSB_TrackPanSlideRight    16L  /* Track panning slide is right */
#define FSB_FineTrkPanSlideRight  17L  /* Fine track pan slide is right */
#define FSB_FineTrackPanSlide     18L  /* Fine track panning slide is used */
#define FSB_SpeedSlideSlower      19L  /* Speed slide is directed faster */
#define FSB_FineSpeedSlideSlower  20L  /* Fine speed slide directed faster */
#define FSB_FineSpeedSlide        21L  /* Fine speed slide is used */
#define FSB_GlobalVolSlideDown    22L  /* Global volume slide is downed */
#define FSB_FineGVolSlideDown     23L  /* Fine global volume slide downed */
#define FSB_FineGlobalVolSlide    24L  /* Fine global volume slide is used */
#define FSB_GlobalPanSlideRight   25L  /* Global panning slide is righted */
#define FSB_FineGPanSlideRight    26L  /* Fine global pan slide is righted */
#define FSB_FineGlobalPanSlide    27L  /* Fine global panning slide used */

#define FSF_FinePortaDown          (1L<<FSB_FinePortaDown)
#define FSF_PortaOnceDown          (1L<<FSB_PortaOnceDown)
#define FSF_FinePortaOnceDown      (1L<<FSB_FinePortaOnceDown)
#define FSF_FinePorta              (1L<<FSB_FinePorta)
#define FSF_PortaOnce              (1L<<FSB_PortaOnce)
#define FSF_FineTonePorta          (1L<<FSB_FineTonePorta)
#define FSF_TonePortaOnce          (1L<<FSB_TonePortaOnce)
#define FSF_VolSlideDown           (1L<<FSB_VolSlideDown)
#define FSF_FineVolSlideDown       (1L<<FSB_FineVolSlideDown)
#define FSF_FineVolSlide           (1L<<FSB_FineVolSlide)
#define FSF_TrackVolSlideDown      (1L<<FSB_TrackVolSlideDown)
#define FSF_FineTrackVolSlideDown  (1L<<FSB_FineTrackVolSlideDown)
#define FSF_FineTrackVolSlide      (1L<<FSB_FineTrackVolSlide)
#define FSF_PanSlideRight          (1L<<FSB_PanSlideRight)
#define FSF_FinePanSlideRight      (1L<<FSB_FinePanSlideRight)
#define FSF_FinePanSlide           (1L<<FSB_FinePanSlide)
#define FSF_TrackPanSlideRight     (1L<<FSB_TrackPanSlideRight)
#define FSF_FineTrkPanSlideRight   (1L<<FSB_FineTrkPanSlideRight)
#define FSF_FineTrackPanSlide      (1L<<FSB_FineTrackPanSlide)
#define FSF_SpeedSlideSlower       (1L<<FSB_SpeedSlideSlower)
#define FSF_FineSpeedSlideSlower   (1L<<FSB_FineSpeedSlideSlower)
#define FSF_FineSpeedSlide         (1L<<FSB_FineSpeedSlide)
#define FSF_GlobalVolSlideDown     (1L<<FSB_GlobalVolSlideDown)
#define FSF_FineGVolSlideDown      (1L<<FSB_FineGVolSlideDown)
#define FSF_FineGlobalVolSlide     (1L<<FSB_FineGlobalVolSlide)
#define FSF_GlobalPanSlideRight    (1L<<FSB_GlobalPanSlideRight)
#define FSF_FineGPanSlideRight     (1L<<FSB_FineGPanSlideRight)
#define FSF_FineGlobalPanSlide     (1L<<FSB_FineGlobalPanSlide)

/* Player channel flags: */
#define PCB_Sustain                0   /* Sustain points released */
#define PCB_Fading                 1   /* Fading out */
#define PCB_Decay                  2   /* Decay running */
#define PCB_TrackPan               3   /* Channel uses track panning */
#define PCB_SmpSurPan              4   /* Sample panning is surround pan */
#define PCB_GlobalSurPan           5   /* Global panning is surround pan */
#define PCB_Surround               6   /* Surround output */
#define PCB_Background             7   /* Channel playing in background */
#define PCB_PortaSlideEnv          8   /* If bit is set, the values in the
                                          slide envelope will be portamento
                                          slide values, else they will be a
                                          transpose & finetune */
#define PCB_LinearSlideEnv         9   /* If bit is set, use linear frequence
                                          table for slide envelope if
                                          portamento mode is activated */
#define PCB_LinearFreqAutoVib     10   /* If bit is set, linear frequence tab
                                          for auto vibrato will be used */
#define PCB_Allocated             15   /* Channel allocated but no playing */

#define PCF_Sustain                (1<<PCB_Sustain)
#define PCF_Fading                 (1<<PCB_Fading)
#define PCF_Decay                  (1<<PCB_Decay)
#define PCF_TrackPan               (1<<PCB_TrackPan)
#define PCF_SmpSurPan              (1<<PCB_SmpSurPan)
#define PCF_GlobalSurPan           (1<<PCB_GlobalSurPan)
#define PCF_Surround               (1<<PCB_Surround)
#define PCF_Background             (1<<PCB_Background)
#define PCF_PortaSlideEnv          (1<<PCB_PortaSlideEnv)
#define PCF_LinearSlideEnv         (1<<PCB_LinearSlideEnv)
#define PCF_LinearFreqAutoVib      (1<<PCB_LinearFreqAutoVib)
#define PCF_Allocated              (1<<PCB_Allocated)

/* Player channel synth flags: */
#define PCYB_VolumeKill            0   /* Volume handle code kill running */
#define PCYB_PanningKill           1   /* Panning handle code kill running */
#define PCYB_SlideKill             2   /* Slide handle code kill running */
#define PCYB_SpecialKill           3   /* Special handle code kill running */

#define PCYF_VolumeKill            (1<<PCYB_VolumeKill)
#define PCYF_PanningKil            (1<<PCYB_PanningKill)
#define PCYF_SlideKill             (1<<PCYB_SlideSkill)
#define PCYF_SpecialKil            (1<<PCYB_SpecialKill)

/* Player channel control affect bits: */
#define PCAB_Notes                 0   /* Affect note related effects
                                          volume, panning, etc. */
#define PCAB_NoNotes               1   /* Affect non-note related effects
                                          patt loops, patt breaks, etc. */

#define PCAF_Notes                 (1<<PCAB_Notes)
#define PCAF_NoNotes               (1<<PCAB_NoNotes)

/* Play types as used by pg_PlayType: */
#define PTYPB_Song                 7   /* Flag set by PlayTCModule() and
                                          if PlayTrack() is in continous
                                          mode to indicate that the whole
                                          song is played. Disk writers can
                                          check this bit in order to write
                                          sample data only if the song is
                                          being played */

#define PTYPF_Song                 (1<<PTYPB_Song)

/* Player envelope structure. The player envelope routine evaluates this */
struct Envelope {
  struct TuComposerInstrEnvelope *pes_EnvelopeStructure; /* Ptr to env */
  WORD   pes_EnvelopeValue;            /* Current envelope value */
  UWORD  pes_EnvelopePos;              /* Envelope position (0 is first) */
  UWORD  pes_EnvelopeRestart;          /* Envelope cur. loop restart point */
  UWORD  pes_EnvelopeEnd;              /* Envelope current loop end point */
  UWORD  pes_EnvelopeSustainCounted;   /* Envelope sustain counter counted */
  UWORD  pes_EnvelopeLoopCounted;      /* Envelope loop counter counted */
  UWORD  pes_EnvelopeTempoCount;       /* Envelope tempo counter */
  UWORD  pes_EnvelopeTempo;            /* Envelope tempo */
  UWORD  pes_EnvelopeSustainStart;     /* Envelope sustain start point */
  UWORD  pes_EnvelopeSustainEnd;       /* Envelope sustain end point */
  UWORD  pes_EnvelopeSustainCount;     /* Envelope sustain counter */
  UWORD  pes_EnvelopeLoopStart;        /* Envelope loop start point */
  UWORD  pes_EnvelopeLoopEnd;          /* Envelope loop end point */
  UWORD  pes_EnvelopeLoopCount;        /* Envelope loop counter */
  WORD   pes_EnvelopeLowestValue;      /* Data lowest value (used by random) */
  WORD   pes_EnvelopeHighestValue;     /* Data highest value (by random) */
  UBYTE  pes_EnvelopeFlags;            /* Envelope flags (see upwards) */
  UBYTE  pes_EnvelopeRepFlags;         /* Envelope repeat flags */
  UBYTE  pes_Reserved[10];             /* Reserved for future expansions */
}; /* Envelope */

/* Player global structure (Player global settings, like BpM, etc.) */
struct PlayerGlobals {
  UBYTE  pg_Flags;                     /* Special flags */
  UBYTE  pg_SpeedSlideToValue;         /* Speed slide to value */
  UBYTE  pg_SpeedMul;                  /* Speed multiplier (allowing 2/3) */
  UBYTE  pg_SpeedDiv;                  /* Speed divider (divs the mul) */
  ULONG  pg_RelativeSpeed;             /* Relative speed (65536 is 100%) */
  ULONG  pg_RelativePitch;             /* Relative pitch (65536 is 100%) */
  ULONG  pg_PlayTime;                  /* Playing time in 1/100 secs */
  ULONG  pg_PlayTimeFrac;              /* Playing time 32-bit fraction */
  ULONG  pg_PlayTics;                  /* Playing tics in 1/100 secs. This
                                          increases depending on Tempo,
                                          while PlayTime depends on
                                          Tempo+RelativeSpeed. This is
                                          important if you want do things
                                          like percentage display of played
                                          module. Just use the formula
                                          (PlayTics/Duration of song)*100 */
  ULONG  pg_PlayTicsFrac;              /* Playing tics 32-bit fraction */
  ULONG  pg_Tempo;                     /* Current tempo in 1/655360 secs */
  UWORD  pg_SPDSpeed;                  /* Current SPD value */
  UWORD  pg_BPMTempo;                  /* Current BPM tempo (rows/beat) */
  UWORD  pg_BPMSpeed;                  /* Current BPM speed (default 125) */
  UBYTE  pg_GlobalVolumeSlideToValue;  /* Global volume slide to value */
  UBYTE  pg_GlobalPanSlideToValue;     /* Global panning slide to value */
  UBYTE  pg_GlobalVolume;              /* Global volume */
  UBYTE  pg_GlobalSubVol;              /* Global sub-volume */
  UBYTE  pg_GlobalPanning;             /* Global panning */
  UBYTE  pg_GlobalSubPan;              /* Global sub-panning */
  UWORD  pg_SpeedSlideFasterValue;     /* Speed slide faster value */
  UWORD  pg_SpeedSlideSlowerValue;     /* Speed slide slower value */
  UWORD  pg_FineSpeedSlideFastValue;   /* Fine speed slide faster value */
  UWORD  pg_FineSpeedSlideSlowValue;   /* Fine speed slide slower value */
  UWORD  pg_SpeedSlideToSlide;         /* Speed slide to slide value */
  UWORD  pg_SpeedSlideToSpeed;         /* Speed slide to speed */
  WORD   pg_SpenoloSlideValue;         /* Spenolo relative slide value */
  UBYTE  pg_SpenoloDepth;              /* Spenolo depth */
  UBYTE  pg_SpenoloRate;               /* Spenolo rate */
  UWORD  pg_GlobalVolSlideUpValue;     /* Global volume slide up value */
  UWORD  pg_GlobalVolSlideDownValue;   /* Global volume slide down value */
  UWORD  pg_FineGlobalVolSlUpValue;    /* Fine global vol. slide up value */
  UWORD  pg_FineGlobalVolSlDownValue;  /* Fine glob. vol. slide down value */
  UWORD  pg_GlobalVolumeSlideToSlide;  /* Global vol. slide to slide value */
  UBYTE  pg_GlobalVolumeSlToVolume;    /* Global volume slide to volume */
  UBYTE  pg_GlobalVolumeSlToSubVol;    /* Global volume slide to
                                          sub volume */
  WORD   pg_TremoloSlideValue;         /* Global tremolo rel. slide value */
  UBYTE  pg_TremoloDepth;              /* Global tremolo depth */
  UBYTE  pg_TremoloRate;               /* Global tremolo rate */
  UWORD  pg_GlobalPanSlideLeftValue;   /* Global panning slide left value */
  UWORD  pg_GlobalPanSlideRightValue;  /* Global panning slide right value */
  UWORD  pg_FineGlobalPanSlLeftValue;  /* Fine global pan slide left value */
  UWORD  pg_FineGlobalPanSlRightVal;   /* Fine glob. pan slide right value */
  UWORD  pg_GlobalPanSlideToSlide;     /* Global pan slide to slide value */
  UBYTE  pg_GlobalPanSlideToPanning;   /* Global panning slide to panning */
  UBYTE  pg_GlobalPanSlideToSubPan;    /* Global panning slide to
                                          sub panning */
  WORD   pg_PannoloSlideValue;         /* Global pan relative slide value */
  UBYTE  pg_PannoloDepth;              /* Global pannolo depth */
  UBYTE  pg_PannoloRate;               /* Global pannolo rate */
  UWORD  pg_Channels;                  /* Channels currently active */
  UWORD  pg_MaxChannels;               /* Highest number of chans played */
  struct Envelope pg_SpenoloEnv;       /* Spenolo envelope */
  struct Envelope pg_TremoloEnv;       /* Global tremolo envelope */
  struct Envelope pg_PannoloEnv;       /* Global pannolo envelope */
  UBYTE  pg_SpeedType;                 /* Speed timing type (look Set Speed
                                          (track.h) for details) */
  UBYTE  pg_PlayType;                  /* Play type. See above for
                                          details. */
  UWORD  pg_TraceCount;                /* Trace counter */
  UBYTE  pg_Reserved[12];              /* Reserved for future */
}; /* PlayerGlobals */

/* Host player channel structure (Used by player) */
struct PlayerHostChannel {
  struct TuComposerOrderList *phc_Order; /* Pointer to order playing */
  struct TuComposerTrack *phc_Track;   /* Pointer to current play track */
  struct TuComposerTrackEffect *phc_Effect; /* Pointer to fx to be process */
  struct TuComposerInstr *phc_Instrument;  /* Pointer to current instr */
  struct TuComposerSample *phc_Sample; /* Pointer to current sample */
  UWORD  phc_Row;                      /* Row currently playing */
  UWORD  phc_FinePatternDelay;         /* Fine pattern delay value */
  ULONG  phc_TempoCounter;             /* Tempo counter (current tick) */
  ULONG  phc_MaxRow;                   /* Last row before new pattern */
  ULONG  phc_Flags;                    /* Track flags (look upwards) */
  ULONG  phc_FineSlideFlags;           /* Fine slide flags (look upwards) */
  UWORD  phc_Tempo;                    /* Track tempo (in cycles) */
  WORD   phc_FinalNote;                /* Final note playing (C-0 is 0) */
  UBYTE  phc_InstrNote;                /* Current instr note (C-0 is 1) */
  UBYTE  phc_SampleNote;               /* Current sample note (C-0 is 1) */
  UBYTE  phc_TrackVolume;              /* Track volume (or channel volume) */
  UBYTE  phc_TrackSubVol;              /* Track sub-volume */
  UBYTE  phc_TrackPanning;             /* Track panning position */
  UBYTE  phc_TrackSubPan;              /* Track sub-panning position */
  UBYTE  phc_TrackNotePan;             /* Track note panning */
  UBYTE  phc_TrackNoteSubPan;          /* Track note sub-panning */
  UBYTE  phc_ChannelPanning;           /* Track channel panning */
  UBYTE  phc_ChannelSubPan;            /* Track channel sub-panning */
  UBYTE  phc_FineTune;                 /* Current sample finetune */
  UBYTE  phc_ArpeggioTick;             /* Arpeggio tick */
  LONG   phc_ArpeggioFrequency;        /* Arpeggio frequency */
  BYTE   phc_ArpeggioFirst;            /* Arpeggio first value */
  BYTE   phc_ArpeggioSecond;           /* Arpeggio second value */
  UWORD  phc_SmpOffsetHI;              /* High 16 bits of smp offset cmd */
  UBYTE  phc_FirstChannelSync;         /* 1st channel to be sync */
  UBYTE  phc_SecondChannelSync;        /* 2nd channel to be sync */
  UBYTE  phc_ThirdChannelSync;         /* 3rd channel to be sync */
  UBYTE  phc_FourthChannelSync;        /* 4th channel to be sync */
  UWORD  phc_PortaUpValue;             /* Portamento up slide value */
  UWORD  phc_PortaDownValue;           /* Portamento down slide value */
  UWORD  phc_FinePortaUpValue;         /* Fine portamento up slide value */
  UWORD  phc_FinePortaDownValue;       /* Fine portamento down slide value */
  UWORD  phc_PortaUpOnceValue;         /* Portamento up once slide value */
  UWORD  phc_PortaDownOnceValue;       /* Portamento down once slide value */
  UWORD  phc_FinePortaUpOnceValue;     /* Fine porta up once slide value */
  UWORD  phc_FinePortaDownOnceValue;   /* Fine porta down once slide value */
  UWORD  phc_TonePortaValue;           /* Tone portamento slide value */
  UWORD  phc_FineTonePortaValue;       /* Fine tone portamento value */
  UWORD  phc_TonePortaOnceValue;       /* Tone portamento once slide value */
  UWORD  phc_FineTonePortaOnceValue;   /* Fine tone porta once slide value */
  ULONG  phc_TonePortaTargetPitch;     /* Tone portamento target pitch */
  UBYTE  phc_SubSlideValue;            /* Current subslide (pitch) value */
  UBYTE  phc_NoteSlideType;            /* Note slide type */
  UBYTE  phc_NoteSlideValue;           /* Note slide value */
  UBYTE  phc_GlissandoValue;           /* Glissando value */
  LONG   phc_VibratoSlideValue;        /* Relative vibrato slide value */
  UWORD  phc_VibratoRate;              /* Vibrato rate */
  UWORD  phc_VibratoDepth;             /* Vibrato depth */
  UWORD  phc_NoteDelay;                /* Note delay tick */
  UBYTE  phc_TremorOnTicks;            /* Tremor on ticks */
  UBYTE  phc_TremorOffTicks;           /* Tremor off ticks */
  UBYTE  phc_TremorCount;              /* Tremor count */
  UBYTE  phc_SubSlideBits;             /* Sub-slide bits */
  UWORD  phc_RetrigTickCount;          /* Retrigger tick counter */
  UBYTE  phc_MultiRetrigTick;          /* Multi retrigger tick */
  UBYTE  phc_MultiRetrigVolChg;        /* Multi retrigger volume change */
  UBYTE  phc_MultiRetrigScale;         /* Multi retrig scale (01-04) */
  UBYTE  phc_InvertCount;              /* Invert loop count */
  UWORD  phc_InvertSpeed;              /* Invert loop speed */
  UWORD  phc_ExecFX;                   /* Execute FX at tick */
  UBYTE  phc_VolumeSlideToValue;       /* Volume slide to speed value */
  UBYTE  phc_TrackVolSlideToValue;     /* Track vol slide to speed value */
  UBYTE  phc_PanningSlideToValue;      /* Panning slide to speed value */
  UBYTE  phc_TrackPanSlideToValue;     /* Track pan slide to speed value */
  UWORD  phc_VolSlideUpValue;          /* Volume slide up value */
  UWORD  phc_VolSlideDownValue;        /* Volume slide down value */
  UWORD  phc_FineVolSlideUpValue;      /* Fine volume slide up value */
  UWORD  phc_FineVolSlideDownValue;    /* Fine volume slide down value */
  UWORD  phc_VolumeSlideToSlide;       /* Volume slide to slide value */
  UBYTE  phc_VolumeSlideToVolume;      /* Volume slide to volume */
  UBYTE  phc_VolumeSlideToSubVol;      /* Volume slide to sub volume */
  WORD   phc_TremoloSlideValue;        /* Tremolo relative slide value */
  UBYTE  phc_TremoloDepth;             /* Tremolo depth */
  UBYTE  phc_TremoloRate;              /* Tremolo rate */
  UWORD  phc_TrackVolSlideUpValue;     /* Track volume slide up value */
  UWORD  phc_TrackVolSlideDownValue;   /* Track volume slide down value */
  UWORD  phc_FineTrkVolSlideUpValue;   /* Fine track volume slide up value */
  UWORD  phc_FineTrkVolSlideDnValue;   /* Fine track vol slide down value */
  UWORD  phc_TrackVolSlideToSlide;     /* Track vol slide to slide value */
  UBYTE  phc_TrackVolSlideToVolume;    /* Track volume slide to volume */
  UBYTE  phc_TrackVolSlideToSubVol;    /* Track volume slide to sub volume */
  WORD   phc_TrackTremSlideValue;      /* Track relative slide value */
  UBYTE  phc_TrackTremDepth;           /* Track tremolo depth */
  UBYTE  phc_TrackTremRate;            /* Track tremolo rate */
  UWORD  phc_PanSlideLeftValue;        /* Panning slide left value */
  UWORD  phc_PanSlideRightValue;       /* Panning slide right value */
  UWORD  phc_FinePanSlideLeftValue;    /* Fine panning slide left value */
  UWORD  phc_FinePanSlideRightValue;   /* Fine panning slide right value */
  UWORD  phc_PanningSlideToSlide;      /* Panning slide to slide value */
  UBYTE  phc_PanningSlideToPanning;    /* Panning slide to panning */
  UBYTE  phc_PanningSlideToSubPan;     /* Panning slide to sub panning */
  WORD   phc_PannoloSlideValue;        /* Pannolo relative slide value */
  UBYTE  phc_PannoloDepth;             /* Pannolo depth */
  UBYTE  phc_PannoloRate;              /* Pannolo rate */
  UWORD  phc_TrackPanSlideLeftValue;   /* Track panning slide left value */
  UWORD  phc_TrackPanSlideRightValue;  /* Track panning slide right value */
  UWORD  phc_FineTrkPanSldLeftValue;   /* Fine track pan slide left value */
  UWORD  phc_FineTrkPanSldRightValue;  /* Fine track pan slide right value */
  UWORD  phc_TrackPanSlideToSlide;     /* Track pan slide to slide value */
  UBYTE  phc_TrackPanSlideToPanning;   /* Track panning slide to panning */
  UBYTE  phc_TrackPanSlideToSubPan;    /* Track panning slide to sub
                                          panning */
  UWORD  phc_TrackPanSlideValue;       /* Track pan relative slide value */
  UBYTE  phc_TrackPanDepth;            /* Track pannolo depth */
  UBYTE  phc_TrackPanRate;             /* Track pannolo rate */
  UWORD  phc_BreakRow;                 /* Pattern break row */
  UWORD  phc_PosJump;                  /* Position to jump to */
  UWORD  phc_ChgPattern;               /* Change pattern number */
  UWORD  phc_PatternDelayCount;        /* Pattern delay current count */
  UWORD  phc_PatternDelay;             /* Pattern delay (0 for nothing) */
  UWORD  phc_PatternLoopDepth;         /* Pattern loop stack depth */
  UWORD  phc_GoSub;                    /* GoSub order number */
  UWORD  phc_GoSubDepth;               /* GoSub stack depth */
  UWORD  phc_VirtualChannel;           /* Number of virtual channel */
  UWORD  phc_NumVirtual;               /* Virtual channels allocated */
  UBYTE  phc_Transpose;                /* Set transpose */
  BYTE   phc_TransFineTune;            /* Set transpose fine tune */
  UBYTE  phc_EnvCtrlKind;              /* Kind of envelope to be changed */
  UBYTE  phc_EnvCtrlChange;            /* Envelope control change */
  UWORD  phc_EnvCtrlValue;             /* Envelope control value */
  UBYTE  phc_SynthCtrlCount;           /* How many items to be changed */
  UBYTE  phc_SynthCtrlChange;          /* Synth item to be changed */
  UWORD  phc_SynthCtrlValue;           /* Synth control value */
  UBYTE  phc_DCT;                      /* Current Duplicate Check Type */
  UBYTE  phc_DNA;                      /* Current Duplicate Check Action */
  UBYTE  phc_NNA;                      /* Current New Note Action */
  UBYTE  phc_ChControlFlags;           /* Channel control flags */
  UBYTE  phc_ChControlType;            /* Channel ctrl type (global, etc.) */
  UBYTE  phc_ChControlMode;            /* Ctrl mode (normal, a row, etc.) */
  UBYTE  phc_ChControlAffect;          /* Ctrl affect (notes, non-notes) */
  UBYTE  phc_ChControlChannel;         /* Control channel (single type) */
  UBYTE  phc_SlideFXChLeftValue;       /* Slide effect channel left value */
  UBYTE  phc_SlideFXChRightValue;      /* Slide effect channel right value */
  UBYTE  phc_FineSlideFXChLeftValue;   /* Fine slide effect chan. left val */
  UBYTE  phc_FineSlideFXChRightValue;  /* Fine slide fx chan. right value */
  UBYTE  phc_SlideFXChannelToValue;    /* Slide effect channel to value */
  UBYTE  phc_SlideFXChannelToChannel;  /* Slide effect channel to channel */
  WORD   phc_ChannoloChannel;          /* Channolo channel */
  UBYTE  phc_ChannoloDepth;            /* Channolo depth */
  UBYTE  phc_ChannoloRate;             /* Channolo rate */
  struct Envelope phc_VibratoEnv;      /* Vibrato envelope */
  struct Envelope phc_TremoloEnv;      /* Pannolo envelope */
  struct Envelope phc_PannoloEnv;      /* Tremolo envelope */
  struct Envelope phc_ChannoloEnv;     /* Channolo envelope */
  struct Envelope phc_ArpStructure;    /* Arpeggio definition to be used */
  struct Envelope phc_TrackTremEnv;    /* Track tremolo envelope */
  struct Envelope phc_TrackPanEnv;     /* Track pannolo envelope */
  struct TuComposerInstrEnvelope *phc_PrevVolumeEnv;  /* Previous volume
                                          envelope pointer */
  struct TuComposerInstrEnvelope *phc_PrevPanningEnv; /* Previous pan
                                          envelope pointer */
  struct TuComposerInstrEnvelope *phc_PrevSlideEnv;   /* Previous slide
                                          envelope pointer */
  struct TuComposerInstrEnvelope *phc_PrevAutoVibEnv; /* Previous
                                          auto vibrato env ptr */
  struct TuComposerInstrEnvelope *phc_PrevAutoTremEnv; /* Previous
                                          auto tremolo env ptr */
  struct TuComposerInstrEnvelope *phc_PrevAutoPanEnv; /* Previous
                                          auto pannolo env ptr */
  struct TuComposerSynthWave *phc_WaveformList; /* Pointer to list of
                                          synth waveforms */
  struct TuComposerSynth *phc_Synth;   /* Pointer to synth structure for
                                          obtaining the synth code struct */
  UWORD  phc_VolumePos;                /* Entry pos. of vol. handling code */
  UWORD  phc_PanningPos;               /* Entry pos. of pan. handling code */
  UWORD  phc_SlidePos;                 /* Entry pos. of slide handle code */
  UWORD  phc_SpecialPos;               /* Entry pos. of spec. handle code */
  UWORD  phc_VolumeSusPos;             /* Entry pos. of sustain vol. code */
  UWORD  phc_PanningSusPos;            /* Entry pos. of sustain pan. code */
  UWORD  phc_SlideSusPos;              /* Entry pos. of sustain slide code */
  UWORD  phc_SpecialSusPos;            /* Entry pos. of sustain spec. code */
  UWORD  phc_VolNNAPos;                /* Position of volume handling code
                                          when NNA has been triggered. This
                                          allows custom NNA handling ! */
  UWORD  phc_PanNNAPos;                /* Pos. of panning handle NNA code */
  UWORD  phc_SlideNNAPos;              /* Pos. of slide handle NNA code */
  UWORD  phc_SpecialNNAPos;            /* Pos. of special NNA handle code */
  UWORD  phc_VolDNAPos;                /* Position of volume handle code
                                          when DNA has been triggered. This
                                          allows custom DNA handling ! */
  UWORD  phc_PanDNAPos;                /* Pos. of panning handle DNA code */
  UWORD  phc_SlideDNAPos;              /* Pos. of slide handling DNA code */
  UWORD  phc_SpecialDNAPos;            /* Pos. of special DNA handle code */
  UWORD  phc_Variable0;                /* Initial contents of variable 0 */
  UWORD  phc_Variable1;                /* Initial contents of variable 1 */
  UWORD  phc_Variable2;                /* Initial contents of variable 2 */
  UWORD  phc_Variable3;                /* Initial contents of variable 3 */
  UWORD  phc_Variable4;                /* Initial contents of variable 4 */
  UWORD  phc_Variable5;                /* Initial contents of variable 5 */
  UWORD  phc_Variable6;                /* Initial contents of variable 6 */
  UWORD  phc_Variable7;                /* Initial contents of variable 7 */
  UWORD  phc_Variable8;                /* Initial contents of variable 8 */
  UWORD  phc_Variable9;                /* Initial contents of variable 9 */
  UWORD  phc_VariableA;                /* Initial contents of variable 10 */
  UWORD  phc_VariableB;                /* Initial contents of variable 11 */
  UWORD  phc_VariableC;                /* Initial contents of variable 12 */
  UWORD  phc_VariableD;                /* Initial contents of variable 13 */
  UWORD  phc_VariableE;                /* Initial contents of variable 14 */
  UWORD  phc_VariableF;                /* Initial contents of variable 15 */
  UWORD  phc_VolConVar;                /* Contents of vol condition var. */
  UWORD  phc_PanConVar;                /* Contents of pan condition var. */
  UWORD  phc_SlideConVar;              /* Contents of slide condition var. */
  UWORD  phc_SpecialConVar;            /* Contents of spec. condition var. */
  UBYTE  phc_ControlChannels[256/8];   /* Bits for control channels 0-255 */
  UBYTE  phc_EffectsUsed[128/8];       /* Bits for used effects (0-7F) */
  UBYTE  phc_Reserved[24];             /* Reserved for future versions ! */
                                       /* Next host channel */
}; /* PlayerHostChannel */

/* Channel control types */
#define CH_CTRL_OFF                0   /* Channel control turned off */
#define CH_CTRL_NORMAL             1   /* Channel control normal */
#define CH_CTRL_MULTI              2   /* Channel control multi channels */
#define CH_CTRL_GLOBAL             3   /* Channel control all channels */

/* Channel control modes */
#define CH_MODE_NORMAL             0   /* Channel control mode one effect */
#define CH_MODE_TICK               1   /* Channel control one tick */
#define CH_MODE_ROW                2   /* Channel control one row */
#define CH_MODE_TRACK              3   /* Channel control one track */
#define CH_MODE_SONG               4   /* Channel control whole song */

/* Player channel structure (Used by player for NNAs mainly) */
struct PlayerChannel {
  struct MixerStruct pc_ChannelData;   /* Play data for channel */
  struct TuComposerInstr *pc_Instrument; /* Ptr to instrument structure */
  struct TuComposerSample *pc_Sample;  /* Pointer to sample structure */
  ULONG  pc_Frequency;                 /* Output frequency in Hz */
  UBYTE  pc_Volume;                    /* Volume */
  UBYTE  pc_SubVol;                    /* Sub-volume */
  UWORD  pc_InstrVolume;               /* Instrument/sample global volume */
  UBYTE  pc_Panning;                   /* Panning position */
  UBYTE  pc_SubPan;                    /* Sub-panning position */
  UBYTE  pc_FinalVolume;               /* Final volume */
  UBYTE  pc_FinalPanning;              /* Final panning */
  UBYTE  pc_GlobalVolume;              /* Global volume */
  UBYTE  pc_GlobalSubVol;              /* Global sub volume */
  UBYTE  pc_GlobalPanning;             /* Global panning */
  UBYTE  pc_GlobalSubPan;              /* Global sub panning */
  UWORD  pc_VolumeSwing;               /* Current volume swing */
  UWORD  pc_PanningSwing;              /* Current panning swing */
  ULONG  pc_PitchSwing;                /* Current pitch swing */
  UWORD  pc_HostChannel;               /* Host channel used (0 - 255) */
  UWORD  pc_Flags;                     /* Special flags (see upwards) */
  struct Envelope pc_VolEnvelope;      /* Volume envelope */
  struct Envelope pc_PanEnvelope;      /* Panning envelope */
  struct Envelope pc_SlideEnvelope;    /* Slide envelope */
  struct Envelope pc_AutoVibEnv;       /* Auto vibrato envelope */
  struct Envelope pc_AutoTremoEnv;     /* Auto tremolo envelope */
  struct Envelope pc_AutoPanEnv;       /* Auto pannolo envelope */
  LONG   pc_SlideEnvelopeFrequency;    /* Slide envelope relative freq */
  LONG   pc_AutoVibFrequency;          /* Auto vibrato frequency */
  WORD   pc_AutoTremoVolume;           /* Auto tremolo volume */
  WORD   pc_AutoPanPanning;            /* Auto pannolo panning */
  UWORD  pc_AutoVibCount;              /* Auto vibrato counter */
  UWORD  pc_AutoTremoCount;            /* Auto tremolo counter */
  UWORD  pc_AutoPanCount;              /* Auto pannolo counter */
  UWORD  pc_FadeOut;                   /* Fade out value */
  UWORD  pc_FadeOutCount;              /* Current fade out count */
  WORD   pc_PitchPanSeparation;        /* Current pitch panning separation */
  UBYTE  pc_PitchPanCenter;            /* Current pitch panning center */
  UBYTE  pc_DCA;                       /* Decay action */
  UWORD  pc_Hold;                      /* Hold value */
  UWORD  pc_Decay;                     /* Decay value */
  UWORD  pc_AutoVibSweep;              /* Auto vibrato sweep */
  UWORD  pc_AutoTremoSweep;            /* Auto tremolo sweep */
  UWORD  pc_AutoPanSweep;              /* Auto pannolo sweep */
  UBYTE  pc_AutoVibDepth;              /* Auto vibrato depth */
  UBYTE  pc_AutoVibRate;               /* Auto vibrato rate */
  UBYTE  pc_AutoTremoDepth;            /* Auto tremolo depth */
  UBYTE  pc_AutoTremoRate;             /* Auto tremolo rate */
  UBYTE  pc_AutoPanDepth;              /* Auto pannolo depth */
  UBYTE  pc_AutoPanRate;               /* Auto pannolo rate */
  UBYTE  pc_InstrNote;                 /* Instrument note (C-0 is 1) */
  UBYTE  pc_SampleNote;                /* Sample note (C-0 is 1) */
  struct TuComposerSynthWave *pc_WaveformList; /* APTR to waveform list */
  struct TuComposerSynth *pc_Synth;    /* Pointer to synth structure for
                                          obtaining the synth code struct */
  struct TuComposerSynthWave *pc_SampleWaveform; /* Sample waveform struct */
  struct TuComposerSynthWave *pc_VibratoWaveform; /* Vibrato wave struct */
  struct TuComposerSynthWave *pc_TremoloWaveform; /* Tremolo wave struct */
  struct TuComposerSynthWave *pc_PannoloWaveform; /* Pannolo wave struct */
  struct TuComposerSynthWave *pc_ArpeggioWaveform; /* Arpeggio wave struct */
  UWORD  pc_SynthPos[0];
  UWORD  pc_VolumePos;                 /* Current volume position (line) */
  UWORD  pc_PanningPos;                /* Current panning position (line) */
  UWORD  pc_SlidePos;                  /* Current slide position (line) */
  UWORD  pc_SpecialPos;                /* Current special position (line) */
  UWORD  pc_SynthSusPos[0];
  UWORD  pc_VolumeSusPos;              /* Current vol sustain pos. entry */
  UWORD  pc_PanningSusPos;             /* Current pan sustain pos. entry */
  UWORD  pc_SlideSusPos;               /* Current slide sustain pos. entry */
  UWORD  pc_SpecialSusPos;             /* Current spec. sustain pos. entry */
  UWORD  pc_SynthNNAPos[0];
  UWORD  pc_VolNNAPos;                 /* Current NNA volume pos. trigger */
  UWORD  pc_PanNNAPos;                 /* Current NNA panning pos. trigger */
  UWORD  pc_SlideNNAPos;               /* Current NNA slide pos. trigger */
  UWORD  pc_SpecialNNAPos;             /* Current NNA special pos. trigger */
  UWORD  pc_SynthDNAPos[0];
  UWORD  pc_VolDNAPos;                 /* Current DNA volume pos. trigger */
  UWORD  pc_PanDNAPos;                 /* Current DNA panning pos. trigger */
  UWORD  pc_SlideDNAPos;               /* Current DNA slide pos. trigger */
  UWORD  pc_SpecialDNAPos;             /* Current DNA special pos. trigger */
  UWORD  pc_Variables[0];
  UWORD  pc_Variable0;                 /* Current contents of variable 0 */
  UWORD  pc_Variable1;                 /* Current contents of variable 1 */
  UWORD  pc_Variable2;                 /* Current contents of variable 2 */
  UWORD  pc_Variable3;                 /* Current contents of variable 3 */
  UWORD  pc_Variable4;                 /* Current contents of variable 4 */
  UWORD  pc_Variable5;                 /* Current contents of variable 5 */
  UWORD  pc_Variable6;                 /* Current contents of variable 6 */
  UWORD  pc_Variable7;                 /* Current contents of variable 7 */
  UWORD  pc_Variable8;                 /* Current contents of variable 8 */
  UWORD  pc_Variable9;                 /* Current contents of variable 9 */
  UWORD  pc_VariableA;                 /* Current contents of variable 10 */
  UWORD  pc_VariableB;                 /* Current contents of variable 11 */
  UWORD  pc_VariableC;                 /* Current contents of variable 12 */
  UWORD  pc_VariableD;                 /* Current contents of variable 13 */
  UWORD  pc_VariableE;                 /* Current contents of variable 14 */
  UWORD  pc_VariableF;                 /* Current contents of variable 15 */
  UWORD  pc_SynthConVar[0];
  UWORD  pc_VolConVar;                 /* Contents of vol condition var. */
  UWORD  pc_PanConVar;                 /* Contents of pan condition var. */
  UWORD  pc_SlideConVar;               /* Contents of slide condition var. */
  UWORD  pc_SpecialConVar;             /* Contents of spec. condition var. */
  UBYTE  pc_UseNNAFlags;               /* Use NNA positions. See synth.h */
  UBYTE  pc_UseSusFlags;               /* Use sustain flags. See synth.h */
  WORD   pc_FinalNote;                 /* Final note playing (C-0 is 0) */
  BYTE   pc_FineTune;                  /* Sample fine tune */
  UBYTE  pc_StopForbidMask;            /* STOP forbid/permit mask */
  UWORD  pc_VibratoPos;                /* Current vibrato wave position */
  UWORD  pc_TremoloPos;                /* Current tremolo wave position */
  UWORD  pc_PannoloPos;                /* Current pannolo wave position */
  UWORD  pc_ArpeggioPos;               /* Current arpeggio wave position */
  UWORD  pc_SynthFlags;                /* Synth flags. See above */
  UWORD  pc_KillSynthCount[0];
  UWORD  pc_KillVolCount;              /* Current vol KILL count in ticks */
  UWORD  pc_KillPanCount;              /* Current pan KILL count in ticks */
  UWORD  pc_KillSldCount;              /* Current slide KILL count */
  UWORD  pc_KillSpcCount;              /* Current special KILL count */
  UWORD  pc_WaitSynthCount[0];
  UWORD  pc_WaitVolCount;              /* Current vol WAIT count in ticks */
  UWORD  pc_WaitPanCount;              /* Current pan WAIT count in ticks */
  UWORD  pc_WaitSldCount;              /* Current slide WAIT count */
  UWORD  pc_WaitSpcCount;              /* Current special WAIT count */
  UWORD  pc_WaitSynthLine[0];
  UWORD  pc_WaitVolLine;               /* Volume handling line to wait */
  UWORD  pc_WaitPanLine;               /* Panning handling line for wait */
  UWORD  pc_WaitSldLine;               /* Slide handling line for wait */
  UWORD  pc_WaitSpcLine;               /* Special handling line for wait */
  UBYTE  pc_WaitSynthType[0];
  UBYTE  pc_WaitVolType;               /* Volume handling type to wait
                                          (0 is WAITVOL, 1 is WAITPAN, 2 is
                                          WAITSLD and 3 is WAITSPC) */
  UBYTE  pc_WaitPanType;               /* Panning handling type to wait */
  UBYTE  pc_WaitSldType;               /* Slide handling type to wait */
  UBYTE  pc_WaitSpcType;               /* Special handling type to wait */
  UWORD  pc_PortaUpValue;              /* PORTAUP memory */
  UWORD  pc_PortaDnValue;              /* PORTADN memory */
  LONG   pc_PortamentoValue;           /* PORTAUP/PORTADN total value */
  LONG   pc_VibratoSlideValue;         /* VIBRATO relative slide value */
  UWORD  pc_VibratoRate;               /* VIBRATO rate */
  UWORD  pc_VibratoDepth;              /* VIBRATO depth */
  LONG   pc_ArpeggioSlideValue;        /* ARPEGIO relative slide value */
  UWORD  pc_ArpeggioSpeed;             /* ARPEGIO speed value */
  BYTE   pc_ArpeggioTranspose;         /* APREGIO transpose */
  UBYTE  pc_ArpeggioFineTune;          /* ARPEGIO fine tune */
  UWORD  pc_VolSlUpValue;              /* VOLSLUP memory */
  UWORD  pc_VolSlDnValue;              /* VOLSLDN memory */
  WORD   pc_TremoloSlideValue;         /* TREMOLO relative slide value */
  UWORD  pc_TremoloDepth;              /* TREMOLO depth */
  UWORD  pc_TremoloRate;               /* TREMOLO rate */
  UWORD  pc_PanSlLeftValue;            /* PANLEFT slide left value */
  UWORD  pc_PanSlRightValue;           /* PANRGHT slide right value */
  WORD   pc_PannoloSlideValue;         /* PANNOLO relative slide value */
  UWORD  pc_PannoloDepth;              /* PANNOLO depth */
  UWORD  pc_PannoloRate;               /* PANNOLO rate */
  UBYTE  pc_Reserved[16];              /* Reserved for future versions */
                                       /* Next Channel */
}; /* PlayerChannel */

#endif /* TUCOMPOSER_PLAYER_H */
