#ifndef TUCOMPOSER_POSITION_H
#define TUCOMPOSER_POSITION_H
/*
**  $VER: position.h 31.22 (31.3.99)
**
**  Standard C header for tucomposer.library
**
**  (C) Copyright 1999 by Basty/Seasons
**      All Rights Reserved
*/

#ifndef EXEC_TYPES_H
#include <exec/types.h>
#endif /* EXEC_TYPES_H */

#ifndef EXEC_NODES_H
#include <exec/nodes.h>
#endif /* EXEC_NODES_H */

#ifndef EXEC_LISTS_H
#include <exec/lists.h>
#endif /* EXEC_LISTS_H */

#ifndef TUCOMPOSER_TRACK_H
#include <tucomposer/track.h>
#endif /* TUCOMPOSER_TRACK_H */

/* No bits defined yet for tco_CompFlags */

/* Bit definitions for tco_Flags: */
#define TCOB_Surround              0   /* Default surround panning */
#define TCOB_TrackSurround         1   /* Default track surround panning */
#define TCOB_Muted                 2   /* Channel initially muted */

#define TCOF_Surround              (1<<TCOB_Surround)
#define TCOF_TrackSurround         (1<<TCOB_TrackSurround)
#define TCOF_Muted                 (1<<TCOB_Muted)

/* Bit definitions for tcol_Flags: */
#define TCOLB_EndOrder             0   /* Position indicates end of order */
#define TCOLB_EndSong              1   /* Position indicates end of whole
                                          song */
#define TCOLB_NotInOnce            2   /* If bit is set, this order entry
                                          will be skipped if you're playing
                                          the module in song once mode. If
                                          clear, it will be played in once
                                          mode. */
#define TCOLB_NotInRepeat          3   /* If this is set, the order entry
                                          will be skipped if you're in
                                          song repeat mode. If cleared, it
                                          will be played in repeat mode.
                                          NOTE: Setting TCOLB_NotInOnce, and
                                          TCOLB_NotInRepeat at same time, is
                                          a little bit silly :-). If both
                                          are cleared, this entry will be
                                          played in both cases, of course. */
#define TCOLB_TrackSync            4   /* If set, this order entry causes
                                          to be synchronized with up to 4
                                          others. tcol_StartRow and
                                          tcol_EndRow decide up to 4 tracks
                                          to be synchronized with
                                          If tcol_StartRow and tcol_EndRow
                                          are $0000, sync with all channels.
                                          The hi-byte of tcol_StartRow is 1st
                                          track to be sync, lo-byte the 2nd,
                                          hi-byte tcol_EndRow is 3th track,
                                          and the lo-byte the 4th track. */
#define TCOLB_SetVolume            5   /* Bit set takes advantage of the
                                          position volume set */

#define TCOLF_EndOrder             (1<<TCOLB_EndOrder)
#define TCOLF_EndSong              (1<<TCOLB_EndSong)
#define TCOLF_NotInOnce            (1<<TCOLB_NotInOnce)
#define TCOLF_NotInRepeat          (1<<TCOLB_NotInRepeat)
#define TCOLF_TrackSync            (1<<TCOLB_TrackSync)
#define TCOLF_SetVolume            (1<<TCOLB_SetVolume)

/* Orders structure, used by all position functions (e.g. AddPosition()) */
struct TuComposerOrders {
  struct MinList tco_OrderData;        /* Order list data of channel 0 */
  UWORD  tco_Length;                   /* Order list entries of channel 0 */
  UWORD  tco_RepStart;                 /* Repeat start of channel 0 */
  UBYTE  tco_Volume;                   /* Volume of channel 0 */
  UBYTE  tco_SubVol;                   /* Sub volume of channel 0 */
  BYTE   tco_TrackPanning;             /* Track panning of channel 0 */
  UBYTE  tco_TrackSubPan;              /* Track sub panning of channel 0 */
  BYTE   tco_Panning;                  /* Channel panning of channel 0 */
  UBYTE  tco_SubPan;                   /* Channel sub panning of channel 0 */
  UBYTE  tco_CompFlags;                /* Compatilibity flags for ch. 0 */
  UBYTE  tco_Flags;                    /* Special flags for channel 0 */
  UBYTE  tco_Reserved[8];              /* Reserved for channel 0 */
                                       /* Linked list to channel x repeat
                                          start and order list. This comes
                                          as often as many channels are used
                                          in the song. */
}; /* TuComposerOrders */

/* Order structure default values */
#define DEFAULTPOSVOLUME           0xFF /* Full volume by default */
#define DEFAULTPOSSUBVOL           0x0  /* No sub volume by default */
#define DEFAULTPOSPANNING         -0x80 /* Full panning by default */
#define DEFAULTPOSSUBPAN           0x0  /* No sub panning by default */

/* Position error codes */

#define ERROR_POSITION_NOGIVEN     (ERROR_POSITION) /* If no position
                                           given */
#define ERROR_POSITION_NOMEM       (ERROR_POSITION+0x1) /* Memory allocation
                                           has failed */
#define ERROR_POSLIST_NOGIVEN      (ERROR_POSITION+0x8) /* No position list
                                           given */

/* Order List structure, this contains actual order list data. */
struct TuComposerOrderList {
  struct MinNode tcol_Node;
  struct TuComposerTrack *tcol_Track;  /* Pointer to playing track. */
  struct TuComposerOrderList *tcol_NextPos; /* Next position pointer if you
                                          skip forward in a player */
  struct TuComposerOrderList *tcol_PrevPos; /* Previous position ptr if you
                                          skip backward in a player */
  UWORD  tcol_NextRow;                 /* Number of next row if you skip
                                          forward in a module player */
  UWORD  tcol_PrevRow;                 /* Number of next row if you skip
                                          backward in a module player */
  UWORD  tcol_FirstRow;                /* Beginning row of this track. */
  UWORD  tcol_LastRow;                 /* Last row of this track. If set to
                                          0xFFFF this feature is disabled. */
  UBYTE  tcol_Flags;                   /* Special flags, see upwards. */
  BYTE   tcol_Transpose;               /* Relative note transpose for
                                          full track. Allows playing several
                                          tracks some half-tones up/down */
  WORD   tcol_InstrTranspose;          /* Instrument transpose. All
                                          instruments will be mapped to this
                                          if this is non-zero. */
  UWORD  tcol_Tempo;                   /* Tempo change or NULL to for keep */
  UBYTE  tcol_Volume;                  /* Track volume (this overrides
                                          the track settings). To enable
                                          this, TCOLB_Volume must be set. */
  UBYTE  tcol_SubVol;                  /* Track sub volume (this overrides
                                          track). */
  UWORD  tcol_Played;                  /* Played nesting level (nesting
                                          level is gosub depth). */
  UBYTE  tcol_Reserved[8];             /* Reserved for future expansions */
  APTR   tcol_UserData;                /* Extra for you ! */
}; /* TuComposerOrderList */

/* TuComposer Position Header. Used by disk operations
   (e.g. by LoadTCModule() & SaveTCModule()) */

struct IFF_PHDR {
  struct IFFChunk phdr_IFFHeader;
  UWORD  phdr_Length;                  /* Order list entries */
  UWORD  phdr_RepStart;                /* Repeat start */
  UBYTE  phdr_Volume;                  /* Channel volume */
  UBYTE  phdr_SubVol;                  /* Sub volume */
  BYTE   phdr_TrackPanning;            /* Track panning */
  UBYTE  phdr_TrackSubPan;             /* Track sub panning */
  BYTE   phdr_Panning;                 /* Channel panning */
  UBYTE  phdr_SubPan;                  /* Channel sub panning */
  UBYTE  phdr_CompFlags;               /* Compatilibity flags */
  UBYTE  phdr_Flags;                   /* Special flags */
}; /* IFF_PHDR */

/* TuComposer Position Data. Used by disk operations
   (e.g. by LoadTCModule() & SaveTCModule()) */

struct IFF_PDAT {
  struct IFFChunk pdat_IFFHeader;
  UWORD  pdat_TrackNum;                /* Track number to be used */
  UWORD  pdat_NextPos;                 /* Next position number to be used */
  UWORD  pdat_PrevPos;                 /* Previous position to be used */
  UWORD  pdat_NextRow;                 /* Start row of next order entry */
  UWORD  pdat_PrevRow;                 /* Start row of prev order entry */
  UWORD  pdat_FirstRow;                /* First row to play */
  UWORD  pdat_LastRow;                 /* Last row to play */
  UBYTE  pdat_Flags;                   /* Flags */
  BYTE   pdat_Transpose;               /* Note transpose for full track */
  WORD   pdat_InstrTranspose;          /* Instr transpose for full track */
  UWORD  pdat_Tempo;                   /* Tempo change or NULL to for keep */
  UBYTE  pdat_Volume;                  /* Order vol (overrides track vol) */
  UBYTE  pdat_SubVol;                  /* Order sub volume */
}; /* IFF_PDAT */

#endif /* TUCOMPOSER_POSITION_H */
