#ifndef TUCOMPOSER_SAMPLE_H
#define TUCOMPOSER_SAMPLE_H
/*
**  $VER: sample.h 31.22 (2.4.99)
**
**  Standard C header for tucomposer.library
**
**  (C) Copyright 1999 by Basty/Seasons
**      All Rights Reserved
*/

#ifndef EXEC_TYPES_H
#include <exec/types.h>
#endif /* EXEC_TYPES_H */

#ifndef EXEC_NODES_H
#include <exec/nodes.h>
#endif /* EXEC_NODES_H */

#ifndef EXEC_LISTS_H
#include <exec/lists.h>
#endif /* EXEC_LISTS_H */

#ifndef TUCOMPOSER_TUCOMPOSER_H
#include <tucomposer/tucomposer.h>
#endif /* TUCOMPOSER_TUCOMPOSER_H */

#ifndef TUCOMPOSER_INSTR_H
#include <tucomposer/instr.h>
#endif /* TUCOMPOSER_INSTR_H */

#ifndef TUCOMPOSER_SYNTH_H
#include <tucomposer/synth.h>
#endif /* TUCOMPOSER_SYNTH_H */

/* Bit definitions for tcs_CompFlags: */
#define TCSCB_AffectChanPan     0   /* Sample panning affects channel panning
                                       (like IT does) */
#define TCSCB_VolumeOnly        1   /* If a note without a sample is played,
                                       only the sample volume will be left
                                       unchanged */
#define TCSCB_StartTonePorta    2   /* If a tone porta with a note is
                                       executed but no note is currently
                                       played, the tone porta will be ignored
                                       and start playing the note normally */
#define TCSCB_PlayBegin         3   /* If you change a sample within a tone
                                       portamento the samples will be played
                                       from it's beginning */

#define TCSCF_AffectChanPan     (1<<TCSCB_AffectChanPan)
#define TCSCF_VolumeOnly        (1<<TCSCB_VolumeOnly)
#define TCSCF_StartTonePorta    (1<<TCSCB_StartTonePorta)
#define TCSCF_PlayBegin         (1<<TCSCB_PlayBegin)

/* Bit definitions for tcs_Flags: */
#define TCSB_Redirect           0   /* Set, this is a redirection sample */
#define TCSB_Loop               1   /* If bit set, sample uses looping */
#define TCSB_SustainLoop        2   /* If set, sample uses sustain loop */
#define TCSB_SmpPanning         3   /* If bit set, sample panning will be
                                       used */
#define TCSB_SurroundPan        4   /* If set, sample pan is surround pan */
#define TCSB_Reverse            6   /* If bit set, sample will be played
                                       backwards initially */

#define TCSF_Redirect           (1<<TCSB_Redirect)
#define TCSF_Loop               (1<<TCSB_Loop)
#define TCSF_SustainLoop        (1<<TCSB_SustainLoop)
#define TCSF_SmpPanning         (1<<TCSB_SmpPanning)
#define TCSF_SurroundPan        (1<<TCSB_SurroundPan)
#define TCSF_Reverse            (1<<TCSB_Reverse)

/* Bit definitions for tcs_RepMode and tcs_SustainRepMode: */
#define TCSRB_Backwards         0   /* If set backward loop, else forward */
#define TCSRB_Pingpong          1   /* If set ping pong loop, else not */

#define TCSRF_Backwards         (1<<TCSRB_Backwards)
#define TCSRF_Pingpong          (1<<TCSRB_Pingpong)

/* Bit definitions for tcs_EnvUsageFlags: */
#define TCSEUB_UseAutoVibEnv    0   /* Use (reload) auto vibrato envelope */
#define TCSEUB_UseAutoTremEnv   1   /* Use (reload) auto tremolo envelope */
#define TCSEUB_UseAutoPanEnv    2   /* Use (reload) auto pannolo envelope */

#define TCSEUF_UseAutoVibEnv    (1<<TCSEUB_UseAutoVibEnv)
#define TCSEUF_UseAutoTremEnv   (1<<TCSEUB_UseAutoTremEnv)
#define TCSEUF_UseAutoPanEnv    (1<<TCSEUB_UseAutoPanEnv)

/* Bit definitions for tcs_EnvProcFlags: */
#define TCSEPB_ProcAutoVibEnv   0   /* Add 1st, then get auto vib. env val */
#define TCSEPB_ProcAutoTremEnv  1   /* Add 1st, then get auto trm. env val */
#define TCSEPB_ProcAutoPanEnv   2   /* Add 1st, then get auto pan. env val */
#define TCSEPB_LinearAutoVib    7   /* If bit is set, linear frequence table
                                      for auto vibrato will be used */

#define TCSEPF_ProcAutoVibEnv   (1<<TCSEPB_ProcAutoVibEnv)
#define TCSEPF_ProcAutoTremEnv  (1<<TCSEPB_ProcAutoTremEnv)
#define TCSEPF_ProcAutoPanEnv   (1<<TCSEPB_ProcAutoPanEnv)
#define TCSEPF_LinearAutoVib    (1<<TCSEPB_LinearAutoVib)

/* Bit definitions for tcs_EnvRetrigFlags: */
#define TCSERB_NotRetrigAVibEnv  0  /* Not retrigger auto vibrato envelope */
#define TCSERB_NotRetrigATremEnv 1  /* Not retrigger auto tremolo envelope */
#define TCSERB_NotRetrigAPanEnv  2  /* Not retrigger auto pannolo envelope */

#define TCSERF_NotRetrigAVibEnv  (1L<<TCSERB_NotRetrigAVibEnv)
#define TCSERF_NotRetrigATremEnv (1<<TCSERB_NotRetrigATremEnv)
#define TCSERF_NotRetrigAPanEnv  (1<<TCSERB_NotRetrigAPanEnv)

/* Bit definitions for tcs_EnvRandomFlags: */
#define TCSERB_RandomAVibEnv    0   /* Randomize auto vibrato env, if set */
#define TCSERB_RandomATremEnv   1   /* Randomize auto tremolo env, if set */
#define TCSERB_RandomAPanEnv    2   /* Randomize auto pannolo env, if set */

#define TCSERF_RandomAVibEnv    (1<<TCSERB_RandomAVibEnv)
#define TCSERF_RandomATremEnv   (1<<TCSERB_RandomATremEnv)
#define TCSERF_RandomAPanEnv    (1<<TCSERB_RandomAPanEnv)

/* Bit definitions for smpl_Flags & smph_Flags: */
#define SMPLB_Delta             0   /* If bit is set, this is a delta sample.
                                       All other bits are identically to
                                       tcs_Flags. */

#define SMPLF_Delta             (1<<SMPLB_Delta)

/* Bit definitions for SwapSampleInInstr(): */
#define SSIIB_Swap              0L /* If bit set, swap sample data */
#define SSIIB_Update            1L /* If bit set, update keyboard defs */

#define SSIIF_Swap              (1L<<SSIIB_Swap)
#define SSIIF_Update            (1L<<SSIIB_Update)

/* Bit definitions for ConvertBits(): */
#define CBB_ChangeOnly          0L  /* If bit set, change bit resolution only
                                       and update tcs_Samples else do a
                                       sample bit conversion. */

#define CBF_ChangeOnly          (1L<<CBB_ChangeOnly)

/* Bit definitions for GetOptimalBits(): */
#define GOBB_Check81632Only     0L  /* If bit set, only 8, 16 and 32 bits
                                       are checked.*/
#define GOBB_CheckOnce          1L  /* Do not find mininum but check instead
                                       if the current bit depth can be
                                       reduced further */

#define GOBF_Check81632Only     (1L<<GOBB_Check81632Only)
#define GOBF_CheckOnce          (1L<<GOBB_CheckOnce)

/* Bit definitions for ReSample(): */
#define RSB_Interpolation       0L  /* Resample with interpolation if set */

#define RSF_Interpolation       (1L<<RSB_Interpolation)

/* Sample structure, used by all Sample functions (e.g. SaveSampleRaw()) */
struct TuComposerSample {
  struct MinNode tcs_Node;
  STRPTR tcs_FileName;              /* Pointer to sample file name
                                       (e.g. C-Hihat.IFF). Remember that
                                       TuComposer is also available for
                                       MS-DOS and other platforms. So, please
                                       try to use the limited 8x3 MS-DOS
                                       file/dir name length. */
  STRPTR tcs_Name;                  /* Ptr to smp name (e.g. Close Hihat) */
  struct TuComposerSynth *tcs_Synth; /* Ptr to synth structure if
                                        synth/hybrid else zero (see synth.h
                                        for details). */
  WORD  *tcs_Data;                  /* Pointer to raw sample data. If this is
                                       a synth, this is zero. */
  ULONG  tcs_Length;                /* Sample length (in bytes). If this is a
                                       synth this is just zero. */
  ULONG  tcs_Samples;               /* Num of smps (identical to tcs_Length
                                       if 8 bit, and the halve if 16 bit) */
  ULONG  tcs_Repeat;                /* Sample repeat */
  ULONG  tcs_RepLen;                /* Sample repeat length */
  ULONG  tcs_RepCount;              /* Sample count, NULL if no count */
  ULONG  tcs_SustainRepeat;         /* Sample sustain repeat */
  ULONG  tcs_SustainRepLen;         /* Sample sustain repeat length */
  ULONG  tcs_SustainRepCount;       /* Sustain sample count, 0 if no count */
  ULONG  tcs_Rate;                  /* C-4 Sample frequency (in Hz) */
  ULONG  tcs_StartOffset;           /* Start offset of playing (usually 0) */
  ULONG  tcs_LowerRateLimit;        /* Lower sample rate limit (will not
                                       exceed this during playing) */
  ULONG  tcs_UpperRateLimit;        /* Upper sample rate limit (will not
                                       exceed this during playing) */
  UBYTE  tcs_BitDepth;              /* Sample bit resolution (currently 1 to
                                       16 bit samples are supported) */
  BYTE   tcs_Transpose;             /* Sample transpose */
  BYTE   tcs_FineTune;              /* Sample fine tune value */
  UBYTE  tcs_CompFlags;             /* Compatibility flags, see upwards. */
  UBYTE  tcs_Flags;                 /* Special flags, see upwards. */
  UBYTE  tcs_RepMode;               /* Sample repeat mode (e.g. Pingpong) */
  UBYTE  tcs_SustainRepMode;        /* Sample sustain loop (e.g. Pingpong) */
  UBYTE  tcs_GlobalVolume;          /* 8-bit global volume of sample. */
  UBYTE  tcs_Volume;                /* 8-bit default volume of sample. */
  UBYTE  tcs_SubVol;                /* Sub volume (used by vol-slide cmds) */
  BYTE   tcs_Panning;               /* 8-bit panning of instr if
                                       TCSB_InsPanning is set. */
  UBYTE  tcs_SubPanning;            /* Sub pan (used by pan-slide cmds) */
  struct TuComposerInstrEnvelope *tcs_AutoVibratoEnv; /* Ptr to auto vibrato
                                       envelope (or NULL) */
  struct TuComposerInstrEnvelope *tcs_AutoTremoloEnv; /* Ptr to auto tremolo
                                       envelope (or NULL) */
  struct TuComposerInstrEnvelope *tcs_AutoPannoloEnv; /* Ptr to auto pannolo
                                       envelope (or NULL) */
  UBYTE  tcs_EnvUsageFlags;         /* Auto vib/tremo/pannolo usage flags */
  UBYTE  tcs_EnvProcFlags;          /* Auto vibrato/etc. processing flags */
  UBYTE  tcs_EnvRetrigFlags;        /* Auto vibrato/etc. retrigger flags */
  UBYTE  tcs_EnvRandomFlags;        /* Auto vibrato/etc. randomize flags */
  UWORD  tcs_VibratoSweep;          /* Auto vibrato sweep */
  UWORD  tcs_TremoloSweep;          /* Auto tremolo sweep */
  UWORD  tcs_PannoloSweep;          /* Auto pannolo sweep */
  UBYTE  tcs_VibratoDepth;          /* Auto vibrato depth */
  UBYTE  tcs_VibratoRate;           /* Auto vibrato rate (speed) */
  UBYTE  tcs_TremoloDepth;          /* Auto tremolo depth */
  UBYTE  tcs_TremoloRate;           /* Auto tremolo rate (speed) */
  UBYTE  tcs_PannoloDepth;          /* Auto pannolo depth */
  UBYTE  tcs_PannoloRate;           /* Auto pannolo rate (speed) */
  struct MinList tcs_UnknownChunks; /* Linked list of unknown chunks */
  UBYTE  tcs_Reserved[8];           /* Reserved for future expansions */
  APTR   tcs_UserData;              /* Free for any user data. */
}; /* TuComposerSample */

/* Default settings for new sample. */

#define DEFAULTSAMPLEBITDEPTH    8  /* 8 bit sample by default */
#define DEFAULTSAMPLERATE     8363L /* Default sample rate (C-4) */
#define DEFAULTSAMPLETRANSPOSE   0  /* No default transpose */
#define DEFAULTSAMPLEFINETUNE    0  /* No default fine tune */
#define DEFAULTSAMPLEVOLUME   0xFF  /* Full volume for default */
#define DEFAULTSAMPLEPANNING -0x80  /* Middle panning */

/* Sample limitation definitions */
#define MAXSAMPLESPERINSTR    256   /* Maximum up to 256 samples per instr */
#define LASTSAMPLEPERINSTR  (MAXSAMPLESPERINSTR-1) /* Last is the 255th */
#define MINBITDEPTH             1   /* The smallest bit depth is 1 of a
                                       sample */
#define MAXBITDEPTH            32   /* Up to 32 bit samples are supported by
                                       now */

/* Sample data allocation alignment rules */
#define SMP_BLOCKSIZE           8L  /* The sample memory length must be
                                       divideable through 8 by now. This is
                                       for faster processing within some
                                       functions. */
#define SMP_BLOCKMASK (SMP_BLOCKSIZE-1L) /* Just for AND, OR, etc. */

/* Sample error codes */

#define ERROR_SAMPLE_NOGIVEN    ERROR_SAMPLE+0x0 /* If no sample given */
#define ERROR_SAMPLE_NODATA     ERROR_SAMPLE+0x1 /* Smp contains no data */
#define ERROR_SAMPLE_NOMEM      ERROR_SAMPLE+0x2 /* Returned on mem fail */
#define ERROR_SAMPLE_TOOMANY    ERROR_SAMPLE+0x3 /* Too many samples (255) */
#define ERROR_SAMPLE_NOFILE     ERROR_SAMPLE+0x4 /* No filename specified */
#define ERROR_SAMPLE_IOERR      ERROR_SAMPLE+0x5 /* I/O error occured */
#define ERROR_SAMPLE_TOOSMALL   ERROR_SAMPLE+0x8 /* Sample too small */
#define ERROR_SAMPLE_TOOBIG     ERROR_SAMPLE+0x9 /* Too big for operate */
#define ERROR_SAMPLE_BITDEPTH   ERROR_SAMPLE+0xA /* Bit resolution invalid */
#define ERROR_SAMPLE_NO_LOOP    ERROR_SAMPLE+0xB /* Returned if no loop */
#define ERROR_SAMPLE_NOARRAY    ERROR_SAMPLE+0xC /* No array given */

/* IFF-TCM1/IFF-TCI1 SMPH structure. Used by disk operations.
   (e.g. by LoadModule() & SaveModule()) */

struct IFF_SMPH {
  struct IFFChunk smph_IFFHeader;
  ULONG  smph_Samples;              /* Number of samples */
  ULONG  smph_Repeat;               /* Sample repeat */
  ULONG  smph_RepLen;               /* Sample repeat length */
  ULONG  smph_RepCount;             /* Sample count, NULL if no count */
  ULONG  smph_SustainRepeat;        /* Sample sustain repeat */
  ULONG  smph_SustainRepLen;        /* Sample sustain repeat length */
  ULONG  smph_SustainRepCount;      /* Sustain sample count, 0 if no count */
  ULONG  smph_Rate;                 /* C-4 Sample frequency (in Hz) */
  ULONG  smph_StartOffset;          /* Start offset of playing (usually 0) */
  ULONG  smph_LowerRateLimit;       /* Lower sample rate limit (will not
                                       exceed this during playing) */
  ULONG  smph_UpperRateLimit;       /* Upper sample rate limit (will not
                                       exceed this during playing) */
  UBYTE  smph_BitDepth;             /* Sample bit resolution (currently 1 to
                                       16 bit samples are supported) */
  BYTE   smph_Transpose;            /* Sample transpose */
  BYTE   smph_FineTune;             /* Sample fine tune value */
  UBYTE  smph_CompFlags;            /* Compatibility flags, see upwards. */
  UBYTE  smph_Flags;                /* Special flags, see upwards. */
  UBYTE  smph_RepMode;              /* Sample repeat mode (e.g. Pingpong) */
  UBYTE  smph_SustainRepMode;       /* Sample sustain repeat mode */
  UBYTE  smph_GlobalVolume;         /* 8-bit global volume of sample. */
  UBYTE  smph_Volume;               /* 8-bit default volume of sample. */
  UBYTE  smph_SubVol;               /* Sub volume (used by vol-slide cmds) */
  BYTE   smph_Panning;              /* 8-bit panning if TCSB_InsPanning is
                                       is set. */
  UBYTE  smph_SubPanning;           /* Sub panning (by pan-slide cmds) */
  UWORD  smph_AutoVibratoNum;       /* Num of auto vibrato envelope (or 0) */
  UWORD  smph_AutoTremoloNum;       /* Num of auto tremolo envelope (or 0) */
  UWORD  smph_AutoPannoloNum;       /* Num of auto pannolo envelope (or 0) */
  UBYTE  smph_EnvUsageFlags;        /* Auto vib/tremolo/etc. usage flags */
  UBYTE  smph_EnvProcFlags;         /* Auto vibrato/etc. processing flags */
  UBYTE  smph_EnvRetrigFlags;       /* Auto vibrato/etc. retrigger flags */
  UBYTE  smph_EnvRandomFlags;       /* Auto vibrato/etc. randomize flags */
  UWORD  smph_VibratoSweep;         /* Auto vibrato sweep */
  UWORD  smph_TremoloSweep;         /* Auto tremolo sweep */
  UWORD  smph_PannoloSweep;         /* Auto pannolo sweep */
  UBYTE  smph_VibratoDepth;         /* Auto vibrato depth */
  UBYTE  smph_VibratoRate;          /* Auto vibrato rate (speed) */
  UBYTE  smph_TremoloDepth;         /* Auto tremolo depth */
  UBYTE  smph_TremoloRate;          /* Auto tremolo rate (speed) */
  UBYTE  smph_PannoloDepth;         /* Auto pannolo depth */
  UBYTE  smph_PannoloRate;          /* Auto pannolo rate (speed) */
}; /* IFF_SMPH */

/* IFF-TCM1 SMPR structure. Used by disk operations.
   (e.g. by LoadModule() & SaveModule()) */

struct IFF_SMPR {
  struct IFFChunk smpr_IFFHeader;
  ULONG  smpr_SampleNum;            /* Sample number that contains the wave-
                                       form (1 is the first). The number is
                                       ABSOLUTE from the first sample of the
                                       first instrument. */
}; /* IFF_SMPR */

#endif /* TUCOMPOSER_SAMPLE_H */
