#ifndef TUCOMPOSER_SONG_H
#define TUCOMPOSER_SONG_H
/*
**  $VER: song.h 31.22 (29.3.99)
**
**  Standard C header for tucomposer.library
**
**  (C) Copyright 1999 by Basty/Seasons
**      All Rights Reserved
*/

#ifndef EXEC_TYPES_H
#include <exec/types.h>
#endif /* EXEC_TYPES_H */

#ifndef EXEC_LISTS_H
#include <exec/lists.h>
#endif /* EXEC_LISTS_H */

#ifndef EXEC_NODES_H
#include <exec/nodes.h>
#endif /* EXEC_NODES_H */

#ifndef TUCOMPOSER_TUCOMPOSER_H
#include <tucomposer/tucomposer.h>
#endif /* TUCOMPOSER_TUCOMPOSER_H */

#ifndef TUCOMPOSER_PLAYER_H
#include <tucomposer/player.h>
#endif /* TUCOMPOSER_PLAYER_H */

/* Bit definitions for tcso_CompFlags: */

#define TCSOCB_Sync          0      /* Tracks are synchronous, not
                                       asynchronous (for MOD, S3M, etc.) */
#define TCSOCB_GlobalLoop    1      /* Global pattern loop memory */
#define TCSOCB_AmigaLimits   2      /* Flags indicates that the AMIGA
                                       sound hardware is capable of
                                       playback this song without mixing */
#define TCSOCB_OldVolumes    3      /* If set, all volume related cmds are
                                       handled as old volume cmds (i.e.
                                       they range from 00-40, not 00-FF) */
#define TCSOCB_GlobalNewOnly 4      /* Global volume/panning changes
                                       affect new notes only (as S3M) */

#define TCSOCF_Sync          (1<<TCSOCB_Sync)
#define TCSOCF_GlobalLoop    (1<<TCSOCB_GlobalLoop)
#define TCSOCF_AmigaLimits   (1<<TCSOCB_AmigaLimits)
#define TCSOCF_OldVolumes    (1<<TCSOCB_OldVolumes)
#define TCSOCF_GlobalNewOnly (1<<TCSOCB_GlobalNewOnly)

/* Bit definitions for tcso_Flags: */
#define TCSOB_LinearFreqTable 0     /* Bit set means, linear frequency
                                       table is used, else Amiga freq
                                       table is used. */
#define TCSOB_UseSPD          1     /* Use SPD timing instead BpM */
#define TCSOB_Mono            2     /* If bit is set, the song will be
                                       played in mono, that means that
                                       a stereo mix routine must set pan
                                       always to middle or just activate
                                       mono mixing (faster) */
#define TCSOB_Surround        3     /* Initial surround global panning */

#define TCSOF_LinearFreqTable (1<<TCSOB_LinearFreqTable)
#define TCSOF_UseSPD          (1<<TCSOB_UseSPD)
#define TCSOF_Mono            (1<<TCSOB_Mono)
#define TCSOF_Surround        (1<<TCSOB_Surround)

/* Song structure, used by all song related functions (e.g. PlaySong()) */
struct TuComposerSong {
  struct MinNode tcso_Node;
  STRPTR tcso_FileName;                  /* File name of this song */
  STRPTR tcso_Name;                      /* Song name. */
  STRPTR tcso_Message;                   /* Song message (may span multiple
                                            lines and contain ANSI codes) */
  STRPTR tcso_Composer;                  /* Ptr to name of composer. If NULL
                                            composer is same as
                                            TuComposerModule.Composer. */
  STRPTR tcso_Style;                     /* Pointer to style string (techno,
                                            rave, trance, pop, etc.) */
  STRPTR tcso_Comment;                   /* A short comment from composer */
  struct PlayerGlobals *tcso_GlobalData; /* Ptr to ch. glob. data (player) */
  struct PlayerHostChannel *tcso_ChannelData; /* Ptr to ch. data (player) */
  struct StampDate tcso_BeginDate;       /* Begin composing date */
  struct StampDate tcso_EndDate;         /* Finish composing date */
  UBYTE  tcso_PlayTimeHours;             /* Playing time of song in hours */
  UBYTE  tcso_PlayTimeMins;              /* Playing time of song in mins */
  UBYTE  tcso_PlayTimeSecs;              /* Playing time of song in secs */
  UBYTE  tcso_PlayTimeHundreds;          /* Playing time of song in 1/100s */
  struct TuComposerOrders *tcso_Position; /* Pointer to order structure */
  struct MinList tcso_TrackList;         /* Linked list of used tracks */
  UWORD  *tcso_GoSubStackPtr;            /* Stack ptr for GoSub cmd */
  UWORD  *tcso_LoopStackPtr;             /* Stack ptr for pattern loop cmd */
  UWORD  tcso_GoSubStackSize;            /* Stack size for GoSub cmd */
  UWORD  tcso_LoopStackSize;             /* Stack size for patt loop cmd */
  UBYTE  tcso_CompFlags;                 /* Compatibility flags. Upwards */
  UBYTE  tcso_Flags;                     /* Flags. See upwards */
  UWORD  tcso_Channels;                  /* Channels used (default 8) */
  UWORD  tcso_Frames;                    /* Initial tempo (6) */
  UBYTE  tcso_SpeedMul;                  /* Initial speed multiplier (0) */
  UBYTE  tcso_SpeedDiv;                  /* Initial speed divider (0) */
  UWORD  tcso_SPDSpeed;                  /* Initial SPD speed (33) */
  UWORD  tcso_BPMTempo;                  /* Initial BPM tempo (4) */
  UWORD  tcso_BPMSpeed;                  /* Initial BPM speed (125) */
  UWORD  tcso_FramesLowerLimit;          /* Frames lower limit */
  UWORD  tcso_FramesUpperLimit;          /* Frames upper limit */
  UWORD  tcso_SPDLowerLimit;             /* SPD lower limit */
  UWORD  tcso_SPDUpperLimit;             /* SPD upper limit */
  UWORD  tcso_BPMTempoLowerLimit;        /* BPM tempo lower limit */
  UWORD  tcso_BPMTempoUpperLimit;        /* BPM tempo upper limit */
  UWORD  tcso_BPMSpeedLowerLimit;        /* BPM speed lower limit */
  UWORD  tcso_BPMSpeedUpperLimit;        /* BPM speed upper limit */
  UBYTE  tcso_GlobalVolume;              /* Default global volume (255) */
  UBYTE  tcso_GlobalSubVol;              /* Global sub volume (0) */
  UBYTE  tcso_GlobalPanning;             /* Default global panning (0) */
  UBYTE  tcso_GlobalSubPan;              /* Default global sub panning (0) */
  struct MinList tcso_UnknownChunks;     /* Linked list of unknown chunks */
  UBYTE  tcso_Reserved[8];               /* Reserved for future expansions */
  APTR   tcso_UserData;                  /* This entry is for you ! */
}; /* TuComposerSong */

#define FIRSTSONGCHANNEL    0            /* First channel is #0 */
#define LASTSONGCHANNEL     255          /* Last channel is #255 */
#define MINSONGCHANNELS     (FIRSTSONGCHANNEL+1) /* Minium 1 channel
                                            allowed */
#define MAXSONGCHANNELS     (LASTSONGCHANNEL+1) /* Maximum number of channels
                                             allowed are 256 */
#define DEFAULTSONGCHANNELS 8            /* 8 channels are used by default */
#define DEFAULTGOSUBSTACK   4            /* 4 GoSubs can be stacked by def */
#define DEFAULTPATTERNLOOPSTACK 1        /* 1 pattern loop stacked by def */
#define DEFAULTSONGVOLUME   0xFF         /* Default is full global volume */
#define DEFAULTSONGPANNING  0x00         /* Default is left master panning
                                            (full stereo separation) */
#define DEFAULTSONGTEMPO    6            /* Delay one row 6/50 secs. */
#define DEFAULTSONGSPEEDMUL 0            /* Disable multipler and */
#define DEFAULTSONGSPEEDDIV 0            /* the divider */
#define DEFAULTSONGSPDSPEED 33           /* 33 as in OctaMED */
#define DEFAULTSONGBPMTEMPO 4            /* 4 rows are a beat. */
#define DEFAULTSONGBPMSPEED 125          /* Beats per minute (50 Hz) */

/* Song errors */
#define ERROR_SONG_NOGIVEN  (ERROR_SONG+0x0) /* If no song structure
                                            was given */
#define ERROR_SONG_NODATA   (ERROR_SONG+0x1) /* Song contains no data */
#define ERROR_SONG_NOMEM    (ERROR_SONG+0x2) /* Returned on mem fail */
#define ERROR_SONG_NOFILE   (ERROR_SONG+0x4) /* No file given */
#define ERROR_SONG_IOERR    (ERROR_SONG+0x5) /* I/O error occured on */
#define ERROR_SONG_BAD_IFF  (ERROR_SONG+0x6) /* TCM1-IFF-file damaged */
/* Because LoadSong() returns NULL on error, use DOS IoErr() and check
   ERROR_INVALID_RESIDENT_LIBRARY for it. */

#define ERROR_SONG_BAD_VER  (ERROR_SONG+0x7) /* Incorrect song version */

/* TuComposer Song Header. Used by disk operations
   (e.g. by LoadSong() & SaveSong()) */

struct IFF_SHDR {
  struct IFFChunk shdr_IFFHeader;
  struct StampDate shdr_BeginDate;       /* Begin composing date */
  struct StampDate shdr_EndDate;         /* Finish composing date */
  UBYTE  shdr_PlayTimeHours;             /* Playing time of song in hours */
  UBYTE  shdr_PlayTimeMins;              /* Playing time of song in mins */
  UBYTE  shdr_PlayTimeSecs;              /* Playing time of song in secs */
  UBYTE  shdr_PlayTimeHundreds;          /* Playing time of song in 1/100s */
  UWORD  shdr_Tracks;                    /* Number of tracks in this song */

  UWORD  shdr_GoSubStackSize;            /* Stack size for GoSub cmd */
  UWORD  shdr_LoopStackSize;             /* Stack size for patt loop cmd */
  UBYTE  shdr_CompFlags;                 /* Compatibility flags. Upwards */
  UBYTE  shdr_Flags;                     /* Flags. See upwards */
  UWORD  shdr_Channels;                  /* Channels used (default 8) */
  UWORD  shdr_Frames;                    /* Initial tempo (6) */
  UBYTE  shdr_SpeedMul;                  /* Initial speed multiplier (0) */
  UBYTE  shdr_SpeedDiv;                  /* Initial speed divider (0) */
  UWORD  shdr_SPDSpeed;                  /* Initial SPD speed (33) */
  UWORD  shdr_BPMTempo;                  /* Initial BPM tempo (4) */
  UWORD  shdr_BPMSpeed;                  /* Initial BPM speed (125) */
  UWORD  shdr_FramesLowerLimit;          /* Frames lower limit */
  UWORD  shdr_FramesUpperLimit;          /* Frames upper limit */
  UWORD  shdr_SPDLowerLimit;             /* SPD lower limit */
  UWORD  shdr_SPDUpperLimit;             /* SPD upper limit */
  UWORD  shdr_BPMTempoLowerLimit;        /* BPM tempo lower limit */
  UWORD  shdr_BPMTempoUpperLimit;        /* BPM tempo upper limit */
  UWORD  shdr_BPMSpeedLowerLimit;        /* BPM speed lower limit */
  UWORD  shdr_BPMSpeedUpperLimit;        /* BPM speed upper limit */
  UBYTE  shdr_GlobalVolume;              /* Default global volume (255) */
  UBYTE  shdr_GlobalSubVol;              /* Global sub volume (0) */
  UBYTE  shdr_GlobalPanning;             /* Default global panning (0) */
  UBYTE  shdr_GlobalSubPan;              /* Default global sub panning (0) */
}; /* IFF_SHDR */

#endif /* TUCOMPOSER_SONG_H */
