#ifndef TUCOMPOSER_SYNTH_H
#define TUCOMPOSER_SYNTH_H
/*
**  $VER: synth.h 31.22 (2.4.99)
**
**  Standard C header for tucomposer.library
**
**  (C) Copyright 1999 by Basty/Seasons
**      All Rights Reserved
*/

#ifndef EXEC_TYPES_H
#include <exec/types.h>
#endif /* EXEC_TYPES_H */

#ifndef EXEC_NODES_H
#include <exec/nodes.h>
#endif /* EXEC_NODES_H */

#ifndef EXEC_LISTS_H
#include <exec/lists.h>
#endif /* EXEC_LISTS_H */

#ifndef TUCOMPOSER_TUCOMPOSER_H
#include <tucomposer/tucomposer.h>
#endif /* TUCOMPOSER_TUCOMPOSER_H */

/* Flags for synth table are: */

#define STABB_Source         0   /* Source parameter required */
#define STABB_SrcLine        1   /* Source parameter is line number */
#define STABB_NoSrcV0        2   /* Don't display source variable if v0. */
#define STABB_NoSrcData      3   /* Don't display source data if 0x0000. */
#define STABB_Dest           4   /* Destination parameter required */
#define STABB_DestDbl        5   /* Destination double register required */
#define STABB_VarPri         6   /* If both variable is and data are zero,
                                    the variable has higher priority of
                                    being displayed */
#define STABB_SrcNotReq      7   /* Source parameter needn't to be passed */

#define STABF_Source         (1<<STABB_Source)
#define STABF_SrcLine        (1<<STABB_SrcLine)
#define STABF_NoSrcV0        (1<<STABB_NoSrcV0)
#define STABF_NoSrcData      (1<<STABB_NoSrcData)
#define STABF_Dest           (1<<STABB_Dest)
#define STABF_DestDbl        (1<<STABB_DestDbl)
#define STABF_VarPri         (1<<STABB_VarPri)
#define STABF_SrcNotReq      (1<<STABB_SrcNotReq)

/* Bit definitions for tcy_VolConVar, tcy_PanConVar and tcy_xxxConVar: */
#define TCYCB_Carry     0   /* Carry. If set, e.g. JUMPCS jumps. */
#define TCYCB_Overflow  1   /* Overflow. If set, e.g. JUMPVS jumps. */
#define TCYCB_Zero      2   /* Zero. If set, e.g. JUMPEQ jumps. */
#define TCYCB_Negative  3   /* Negative. If set, e.g. JUMPMI jumps. */
#define TCYCB_Extend    4   /* Extend. Used by ADDX, SUBX and NEGX. */

#define TCYCF_Carry     (1<<TCYCB_Carry)
#define TCYCF_Overflow  (1<<TCYCB_Overflow)
#define TCYCF_Zero      (1<<TCYCB_Zero)
#define TCYCF_Negative  (1<<TCYCB_Negative)
#define TCYCF_Extend    (1<<TCYCB_Extend)

/* Bit definitions for tcy_UseNNAFlags: */
#define TCYUNB_VolNNA   0   /* Use volume NNA trigger entry field */
#define TCYUNB_PanNNA   1   /* Use panning NNA trigger entry field */
#define TCYUNB_SlideNNA 2   /* Use slide NNA trigger entry field */
#define TCYUNB_SpecialNNA 3 /* Use special NNA trigger entry field */
#define TCYUNB_VolDNA   4   /* Use volume DNA trigger entry field */
#define TCYUNB_PanDNA   5   /* Use panning DNA trigger entry field */
#define TCYUNB_SlideDNA 6   /* Use slide DNA trigger entry field */
#define TCYUNB_SpecialDNA 7 /* Use special DNA trigger entry field */

#define TCYUNF_VolNNA   (1<<TCYUNB_VolNNA)
#define TCYUNF_PanNNA   (1<<TCYUNB_PanNNA)
#define TCYUNF_SlideNNA (1<<TCYUNB_SlideNNA)
#define TCYUNF_SpecialNNA (1<<TCYUNB_SpecialNNA)
#define TCYUNF_VolDNA   (1<<TCYUNB_VolDNA)
#define TCYUNF_PanDNA   (1<<TCYUNB_PanDNA)
#define TCYUNF_SlideDNA (1<<TCYUNB_SlideDNA)
#define TCYUNF_SpecialDNA (1<<TCYUNB_SpecialDNA)

/* Bit definitions for tcy_UseSusFlags: */
#define TCYSB_VolSus    0   /* Use sustain entry volume position field */
#define TCYSB_PanSus    1   /* Use sustain entry panning position field */
#define TCYSB_SlideSus  2   /* Use sustain entry slide position field */
#define TCYSB_SpecialSus 3  /* Use sustain entry special position field */
#define TCYSB_VolSusKeep 4  /* Keep sustain entry volume position */
#define TCYSB_PanSusKeep 5  /* Keep sustain entry panning position */
#define TCYSB_SlideSusKeep 6 /* Keep sustain entry slide position */
#define TCYSB_SpecSusKeep 7 /* Keep sustain entry special position */

#define TCYSF_VolSus    (1<<TCYSB_VolSus)
#define TCYSF_PanSus    (1<<TCYSB_PanSus)
#define TCYSF_SlideSus  (1<<TCYSB_SlideSus)
#define TCYSF_SpecialSus (1<<TCYSB_SpecialSus)
#define TCYSF_VolSusKeep (1<<TCYSB_VolSusKeep)
#define TCYSF_PanSusKeep (1<<TCYSB_PanSusKeep)
#define TCYSF_SlideSusKeep (1<<TCYSB_SlideSusKeep)
#define TCYSF_SpecSusKeep (1<<TCYSB_SpecSusKeep)

/* Bit definitions for tcy_PosKeepMask: */
#define TCYPB_Volume    0   /* Keep volume position */
#define TCYPB_Panning   1   /* Keep panning position */
#define TCYPB_Slide     2   /* Keep slide position */
#define TCYPB_Special   3   /* Keep special position */
#define TCYPB_Waveforms 6   /* Keep synthetic waveforms */
#define TCYPB_Code      7   /* Keep synth programming code */

#define TCYPF_Volume    (1<<TCYPB_Volume)
#define TCYPF_Panning   (1<<TCYPB_Panning)
#define TCYPF_Slide     (1<<TCYPB_Slide)
#define TCYPF_Special   (1<<TCYPB_Special)
#define TCYPF_Waveforms (1<<TCYPB_Waveforms)
#define TCYPF_Code      (1<<TCYPB_Code)

/* Bit definitions for tcy_NNAPosKeepMask: */
#define TCYNB_VolNNA    0   /* Keep volume NNA trigger position */
#define TCYNB_PanNNA    1   /* Keep panning NNA trigger position */
#define TCYNB_SlideNNA  2   /* Keep slide NNA trigger position */
#define TCYNB_SpecialNNA 3  /* Keep special NNA trigger position */
#define TCYNB_VolDNA    4   /* Keep volume DNA trigger position */
#define TCYNB_PanDNA    5   /* Keep panning DNA trigger position */
#define TCYNB_SlideDNA  6   /* Keep slide DNA trigger position */
#define TCYNB_SpecialDNA 7  /* Keep special DNA trigger position */

#define TCYNF_VolNNA    (1<<TCYNB_VolNNA)
#define TCYNF_PanNNA    (1<<TCYNB_PanNNA)
#define TCYNF_SlideNNA  (1<<TCYNB_SlideNNA)
#define TCYNF_SpecialNNA (1<<TCYNB_SpecialNNA)
#define TCYNF_VolDNA    (1<<TCYNB_VolDNA)
#define TCYNF_PanDNA    (1<<TCYNB_PanDNA)
#define TCYNF_SlideDNA  (1<<TCYNB_SlideDNA)
#define TCYNF_SpecialDNA (1<<TCYNB_SpecialDNA)

/* Bit definitions for tcyw_Flags: */
#define TCYWB_8Bit     15   /* If bit set, this is an 8-bit waveform,
                               else it's an 16-bit waveform. This is
                               only useful for sample using waveforms,
                               because the mixing routines handle 8-bit
                               sound faster than 16-bit ones. For some
                               waveform types it's also useful, e.g. for
                               Ramp up/down waves. The synth code GETxxxW
                               instructions always return an 16-bit value,
                               i.e. an 8-bit waveform will be shifted left
                               by 8 bits before passed to the destination
                               variable. */
#define TCYWB_NoLoop    7   /* If bit is set, no looping for waveform */

#define TCYWF_8Bit      (1<<TCYWB_8Bit)
#define TCYWF_NoLoop    (1<<TCYWB_NoLoop)

/* Bit definitions for tsst_Flags: */
#define TCYSTB_Unused   7   /* If bit is set, symbol is unused currently,
                               i.e. not evaluated. This makes it possible
                               to temporary remove any symbols without
                               deleting them. */

#define TCYSTF_Unused   (1<<TCYSTB_Unused)

/* Bit defs for InsertWaveformPoint() and DeleteWaveformPoint() flags */
#define WAVB_PointsOnly 0L  /* If bit is set, only the points will be
                               affected, else the waveform's length
                               will be affected, too. */


#define WAVF_PointsOnly (1L<<WAVB_PointsOnly)
/* Bit definitions for ConvertWaveform(): */
#define CWB_ChangeOnly  0L  /* If bit is set, change bit resolution only
                               and update tcyw_Samples else do a waveform
                               bit conversion. */

#define CWF_ChangeOnly  (1L<<CWB_ChangeOnly)

/* Bit defs for InsertSynthCodeLine() and DeleteSynthCodeLine() flags */
#define CODB_LinesOnly  0L  /* If bit is set, only the lines will be
                               affected, else the code's length
                               will be affected, too. */
#define CODB_CorrectRefs 1L /* Correct line references, e.g. of JUMPs */

#define CODF_LinesOnly  (1L<<CODB_LinesOnly)
#define CODF_CorrectRefs (1L<<CODB_CorrectRefs)

/* Bit definitions for AssembleSynthCodeLine() */
#define ASCB_GenLabels  0L  /* Generate new defined labels if bit is set */
#define ASCB_AllowDupLabel 1L /* Allow duplicate label. */

#define ASCF_GenLabels  (1L<<ASCB_GenLabels)
#define ASCF_AllowDupLabel (1L<<ASCB_AllowDupLabel)

/* Bit definitions for DisassembleSynthCodeLine() */
#define DASCB_StopIsEnd 0L  /* Substitute STOP instruction with END */
#define DASCB_JmpCCIsJmpHS 1L /* Substitute JUMPCC with JUMPHS */
#define DASCB_JmpCSIsJmpLO 2L /* Substitute JUMPCS with JUMPLO */
#define DASCB_Decimal   3L  /* Decimal source parameter, else hexadecimal */
#define DASCB_LowerCase 4L  /* Instructions are lower case */
#define DASCB_VarUpperCase 5L /* Variables are upper case */
#define DASCB_SrcParSigned 6L /* Source parameter is a signed word */
#define DASCB_NotEvalSyms 7L /* Don't evaluate symbols */
#define DASCB_AlwaysSrcV0 8L /* Always display variable 0 in source */
#define DASCB_AlwaysSrcPar 9L /* Always display source parameter (even 0) */

#define DASCF_StopIsEnd (1L<<DASCB_StopIsEnd)
#define DASCF_JmpCCIsJmpHS (1L<<DASCB_JmpCCIsJmpHS)
#define DASCF_JmpCSIsJmpLO (1L<<DASCB_JmpCSIsJmpLO)
#define DASCF_Decimal (1L<<DASCB_Decimal)
#define DASCF_LowerCase (1L<<DASCB_LowerCase)
#define DASCF_VarUpperCase (1L<<DASCB_VarUpperCase)
#define DASCF_SrcParSigned (1L<<DASCB_SrcParSigned)
#define DASCF_NotEvalSyms (1L<<DASCB_NotEvalSyms)
#define DASCF_AlwaysSrcV0 (1L<<DASCB_AlwaysSrcV0)
#define DASCF_AlwaysSrcPar (1L<<DASCB_AlwaysSrcPar)

/* Synth table. Used for both assembling and disassembling.
   Structure is build up as follows: */

struct SynthTable {
  UBYTE  stab_InstName[8];       /* Instruction name with trailing zero */
  UBYTE  stab_InstCode;          /* Instruction code */
  UBYTE  stab_InstFlags;         /* Instruction flags */
};

/* Synth waveform structure, used by all synth waveform functions (e.g.
   AddSynthWaveform()). */
struct TuComposerSynthWave {
  struct MinNode tcyw_Node;
  STRPTR tcyw_FileName;     /* File name of waveform (e.g. SineWave.TCW) */
  STRPTR tcyw_Name;         /* Name of waveform (e.g. Sine Wave) */
  WORD  *tcyw_WaveData;     /* Pointer to waveform data. The waveforms are
                               an array of WORDs (i.e. they're threated
                               as signed 16-bit samples), in most cases. */
  ULONG  tcyw_Length;       /* Length of waveform in bytes. */
  ULONG  tcyw_Samples;      /* Length of waveform in samples. */
  ULONG  tcyw_Repeat;       /* Repeat start of waveform */
  ULONG  tcyw_RepLen;       /* Repeat length of waveform */
  UWORD  tcyw_Flags;        /* Some flags. See below */
  UBYTE  tcyw_Reserved[18]; /* Reserved for future expansions. */
  APTR   tcyw_UserData;     /* Use for any data you like */
}; /* TuComposerSynthWave */

/* Synth programming code structure, used by all synth all synth programming
   functions (e.g. AddSynth()) */
struct TuComposerSynthCode {
  BYTE  tcyc_Instruction;   /* Instruction to execute on this line. For
                               a list, see below. */
  UBYTE  tcyc_SrcDestVar;   /* This are actually two nibbles. The higher
                               nibble is the source variable to take
                               value from. The lower nibble is the
                               destination variable where to put result. */
  UWORD  tcyc_InstData;     /* Instruction data. Instruction specific. */
                            /* Please note that the effects receive this
                               value + the value of the source variable
                               (all except between NOT and SWAP) ! */
                            /* The same for code line 0x0001 - .... */
}; /* TuComposerSynthCode */

/* Instruction list (positive numbers mean normal effect execution): */
#define TCYINST_STOP    0x00    /* STOP or END: synth execution here !
                                   tcyc_InstData points to an
                                   external influence forbid
                                   mask, which is defined as
                                   following (can be combined):
                                   Mask | Meanings
                                   0001 | Forbid external JUMPVOL
                                   0002 | Forbid external JUMPPAN
                                   0004 | Forbid external JUMPSLD
                                   0008 | Forbid external JUMPSPC
                                   If bit 15 is set, the desired
                                   influence will be permitted. */
#define TCYINST_END TCYINST_STOP /* The same as above... */
#define TCYINST_KILL    0x01   /* KILL: stop and free current channel, most
                                  used in xxxNNAPos code. tcyc_InstData
                                  is amount of ticks to wait before
                                  killing the channel. Synth code
                                  processing continues as normally
                                  until the wait counter has been
                                  reached. Please note that even with
                                  tcyc_InstData set to zero, all
                                  instructions executing in the same
                                  tick as the KILL instruction will
                                  continue do so. If you don't want
                                  this, just place a STOP instruction
                                  straight afterward. */
#define TCYINST_WAIT    0x02   /* WAIT: Wait before going on. tcyc_InstData
                                  is amount of ticks to wait. */
#define TCYINST_WAITVOL 0x03   /* WAITVOL: Waits until volume handling code
                                           has reached line tcyc_InstData */
#define TCYINST_WAITPAN 0x04   /* WAITPAN: Same as WAITVOL but guards the
                                           panning handling code. */
#define TCYINST_WAITSLD 0x05   /* WAITSLD: Same as WAITPAN but guards the
                                           slide handling code. */
#define TCYINST_WAITSPC 0x06   /* WAITSLD: Same as WAITSLD but guards the
                                           slide handling code. */
#define TCYINST_JUMP    0x07   /* JUMP: Jump to line tcyc_InstData. */
#define TCYINST_JUMPEQ  0x08   /* JUMPEQ: Jump on zero to line
                                          tcyc_InstData. */
#define TCYINST_JUMPNE  0x09   /* JUMPNE: Jump on not zero to line
                                          tcyc_InstData. */
#define TCYINST_JUMPPL  0x0A   /* JUMPPL: Jump on positive to line
                                          tcyc_InstData. */
#define TCYINST_JUMPMI  0x0B   /* JUMPPL: Jump on negative to line
                                          tcyc_InstData. */
#define TCYINST_JUMPLT  0x0C   /* JUMPLT: Jump if signed less than to line
                                          tcyc_InstData. */
#define TCYINST_JUMPLE  0x0D   /* JUMPLE: Jump if signed less or equal than
                                          to tcyc_InstData. */
#define TCYINST_JUMPGT  0x0E   /* JUMPGT: Jump if signed greater than to
                                          line tcyc_InstData. */
#define TCYINST_JUMPGE  0x0F   /* JUMPGE: Jump if signed greater or equal
                                          than to line tcyc_InstData. */
#define TCYINST_JUMPVS  0x10   /* JUMPVS: Jump if signed overflow to line
                                          tcyc_InstData. */
#define TCYINST_JUMPVC  0x11   /* JUMPVC: Jump if no signed overflow to line
                                          tcyc_InstData. */
#define TCYINST_JUMPCS  0x12   /* JUMPCS or JUMPLO: Jump if carry generated
                                                    (unsigned lower than). */
#define TCYINST_JUMPLO  TCYINST_JUMPCS /* The same as above... */
#define TCYINST_JUMPCC  0x13   /* JUMPCC or JUMPHS: Jump if no carry caused
                                                (unsigned higher or same). */
#define TCYINST_JUMPHS  TCYINST_JUMPCC /* The same as above... */
#define TCYINST_JUMPLS  0x14   /* JUMPLS: Jump if unsigned less or equal. */
#define TCYINST_JUMPHI  0x15   /* JUMPHI: Jump if unsigned greater than. */
#define TCYINST_JUMPVOL 0x16   /* JUMPVOL: Causes the volume handling code
                                           to jump to the line
                                           tcyc_InstData. */
#define TCYINST_JUMPPAN 0x17   /* JUMPPAN: Causes the panning handling code
                                           to jump to the line
                                           tcyc_InstData. */
#define TCYINST_JUMPSLD 0x18   /* JUMPSLD: Causes the slide handling code
                                           to jump to the line
                                           tcyc_InstData. */
#define TCYINST_JUMPSPC 0x19   /* JUMPSPC: Causes the special handling code
                                           to jump to the line
                                           tcyc_InstData. */
#define TCYINST_CALL    0x1A   /* CALL: Pushes current position to the
                                        destination variable and continues
                                        execution at tcyc_InstData. */
#define TCYINST_RETURN  0x1B   /* RETURN: Pops position to jump to from the
                                          destination variable and adds
                                          tcyc_InstData to it. */
#define TCYINST_POSVAR  0x1C   /* POSVAR: Pushes position + tcyc_InstData to
                                          the destination variable. */
#define TCYINST_LOAD    0x1D   /* LOAD: Loads the destination variable with
                                        the source variable and adds
                                        tcyc_InstData (ABSOLUTE) to it. */
#define TCYINST_ADD     0x1E   /* ADD: Adds tcyc_InstData to destination
                                       variable. */
#define TCYINST_ADDX    0x1F   /* ADDX: Adds tcyc_InstData + extend flag
                                        to the destination variable. */
#define TCYINST_SUB     0x20   /* SUB: Like ADD but subtracts instead. */
#define TCYINST_SUBX    0x21   /* SUBX: Like ADDX but subtracts instead. */
#define TCYINST_CMP     0x22   /* CMP: Like SUB but doesn't store it in
                                       the destination variable. */
#define TCYINST_MULU    0x23   /* MULU: Multiplies tcyc_InstData with the
                                        destination variable (unsigned). */
#define TCYINST_MULS    0x24   /* MULS: Like MULU but uses signed
                                        arithmetic. */
#define TCYINST_DMULU   0x25L  /* DMULU: Like MULU but places the upper word
                                         result in the destination variable
                                         the lower word result in the
                                         destination variable + 1 (or stores
                                         only the lower word if destination
                                         variable is 15) */
#define TCYINST_DMULS   0x26   /* DMULS: Like DMULS but calculates
                                         signed. */
#define TCYINST_DIVU    0x27   /* DIVU: Divides the destination variable
                                        with tcyc_InstData (unsigned). */
#define TCYINST_DIVS    0x28   /* DIVS: Like DIVU but calculates signed. */
#define TCYINST_MODU    0x29   /* MODU: Like DIVU but puts the remainder in
                                        in the destination register
                                        instead. */
#define TCYINST_MODS    0x2A   /* MODS: Like MODU but calculates signed. */
#define TCYINST_DDIVU   0x2B   /* DDIVU: Like DIVU but divides the longword
                                         destination variable + destination
                                         variable + 1 with tcyc_InstData. The
                                         quotient is stored in destination
                                         variable + 1 and the remainder in
                                         destination variable. If the
                                         destination variable is 15, the
                                         quotient will be placed at it. */
#define TCYINST_DDIVS   0x2C   /* DDIVS: Like DDIVU but calculates signed. */
#define TCYINST_ASHL    0x2D   /* ASHL: Arithmetical shifts left the dest.
                                        variable tcyc_InstData bits. */
#define TCYINST_ASHR    0x2E   /* ASHR: Arithmetical shifts right the dest.
                                        variable tcyc_InstData bits. */
#define TCYINST_LSHL    0x2F   /* LSHL: Like ASHL but logical shifts
                                        instead. */
#define TCYINST_LSHR    0x30   /* LSHR: Like ASHR but logical shifts instead. */
#define TCYINST_ROL     0x31   /* ROL: Like LSHL but rotates instead. */
#define TCYINST_ROR     0x32   /* ROR: Like LSHR but rotates instead. */
#define TCYINST_ROLX    0x33   /* ROLX: Like ROL but rotates with extend. */
#define TCYINST_RORX    0x34   /* RORX: Like ROR but rotates with extend. */
#define TCYINST_OR      0x35   /* OR: Logical OR tcyc_InstData with the
                                      destination variable. */
#define TCYINST_AND     0x36   /* AND: Like OR but uses logical AND
                                       instead. */
#define TCYINST_XOR     0x37   /* XOR: Like OR but uses eXclusive OR. */
#define TCYINST_NOT     0x38   /* NOT: Inverts the destination variable and
                                       adds tcyc_InstData to it
                                       afterwards. */
#define TCYINST_NEG     0x39   /* NEG: Negates the destination variable and
                                       adds tcyc_InstData to it
                                       afterwards. */
#define TCYINST_NEGX    0x3A   /* NEGX: Subtracts the destination variable +
                                        the extend flag from zero and adds
                                        tcyc_InstData to it afterwards. */
#define TCYINST_EXTB    0x3B   /* EXTB: Copies bit 7 to bit 8-15 in the
                                        the destination variable and adds
                                        tcyc_InstData to it afterwards. */
#define TCYINST_EXT     0x3C   /* EXT: Copies bit 15 of destination variable
                                       + 1 to bits 0 - 15 of destination
                                       variable and adds tcyc_InstData to it
                                       afterwards. */
#define TCYINST_XCHG    0x3D   /* XCHG: Exchanges the source variable with
                                        the destination variable and adds
                                        tcyc_InstData (ABSOLUTE) to it. */
#define TCYINST_SWAP    0x3E   /* SWAP: Swaps the upper byte of the
                                        destination variable with the lower
                                        byte and adds tcyc_InstData to it. */
#define TCYINST_GETWAVE 0x3F   /* GETWAVE: Gets the current sample waveform +
                                           tcyc_InstData to the destination
                                           variable. */
#define TCYINST_GETWLEN 0x40   /* GETWLEN: Gets the current sample waveform
                                           length + tcyc_InstData to the
                                           destination variable. */
#define TCYINST_GETWPOS 0x41   /* GETWPOS: Gets the current sample waveform
                                           position + tcyc_InstData to the
                                           destination variable. */
#define TCYINST_GETCHAN 0x42   /* GETCHAN: Stores the current host channel +
                                           tcyc_InstData to the destination
                                           variable. */
#define TCYINST_GETNOTE 0x43   /* GETNOTE: Store the current note playing +
                                           tcyc_InstData to the destination
                                           variable. */
#define TCYINST_GETRANS 0x44   /* GETRANS: Like GETNOTE but stores the
                                           current sample transpose value */
#define TCYINST_GETPTCH 0x45   /* GETPTCH: Stores the current frequency in
                                           the destination variable (upper
                                           word) and destination variable + 1
                                           (lower word). If the destination
                                           variable is 15 only the lower word
                                           will be stored. */
#define TCYINST_GETPER 0x46    /* GETPER: Equal to GETPTCH except that the
                                          received value is the period
                                          value. */
#define TCYINST_GETFX  0x47    /* GETFX: This effect stores the current
                                         effect's value given in the upper
                                         byte of tcyc_InstData in the
                                         destination variable. This is the
                                         value passed to the effect at last,
                                         e.g. GETFX with a upper byte value
                                         of 20 will return the current
                                         volume. If the effect memorizes his
                                         value somehow, this value will be
                                         returned. The result of GETFX is
                                         directly passable to SETFXxx. */
#define TCYINST_GETARPW 0x48   /* GETARPW: Stores the arpeggio waveform
                                           + tcy_InstData to the
                                           destination. */
#define TCYINST_GETARPV 0x49   /* GETARPV: Stores the arpeggio waveform value
                                           of tcy_InstData point */
#define TCYINST_GETARPL 0x4A   /* GETARPL: Like GETARPW but stores the
                                           waveform length instead. */
#define TCYINST_GETARPP 0x4B   /* GETARPP: Like GETARPW but stores the
                                           waveform position instead. */
#define TCYINST_GETVIBW 0x4C   /* GETVIBW: Like GETARPW but stores vibrato
                                           waveform number instead. */
#define TCYINST_GETVIBV 0x4D   /* GETVIBV: Like GETARPV but stores vibrato
                                           waveform value instead. */
#define TCYINST_GETVIBL 0x4E   /* GETVIBL: Like GETARPL but stores
                                           vibrato length instead. */
#define TCYINST_GETVIBP 0x4F   /* GETVIBP: Like GETARPP but stores current
                                           vibrato position instead. */
#define TCYINST_GETTRMW 0x50   /* GETTRMW: Like GETVIBW but stores tremolo
                                           waveform number instead. */
#define TCYINST_GETTRMV 0x51   /* GETTRMV: Like GETVIBV but stores tremolo
                                           waveform value instead. */
#define TCYINST_GETTRML 0x52   /* GETTRML: Like GETVIBL but stores tremolo
                                           waveform length instead. */
#define TCYINST_GETTRMP 0x53   /* GETTRMP: Like GETVIBP but stores current
                                           tremolo position instead. */
#define TCYINST_GETPANW 0x54   /* GETPANW: Like GETTRMW but stores pannolo
                                           waveform number instead. */
#define TCYINST_GETPANV 0x55   /* GETPANV: Like GETTRMV but stores pannolo
                                           waveform value instead. */
#define TCYINST_GETPANL 0x56   /* GETPANL: Like GETTRML but stores tremolo
                                           waveform length instead. */
#define TCYINST_GETPANP 0x57   /* GETPANP: Like GETTRMP but stores current
                                           pannolo position instead. */
#define TCYINST_GETRND  0x58   /* GETRND: Gets a random value between 0 and
                                          tcyc_InstData. */
#define TCYINST_GETSINE 0x59   /* GETSINE: Gets the sine of tcyc_InstData.
                                           The parameter are degrees. Use
                                           DMULS and DDIVS to scale to the
                                           required value. */
#define TCYINST_PORTAUP 0x5A   /* PORTAUP: Slides the pitch up with the
                                           amount given in tcyc_InstData. */
#define TCYINST_PORTADN 0x5B   /* PORTADN: Slides the pitch down with the
                                           amount given in tcyc_InstData. */
#define TCYINST_VIBSPD  0x5C   /* VIBSPD: Sets vibrato speed to
                                          tcyc_InstData. */
#define TCYINST_VIBDPTH 0x5D   /* VIBDPTH: Sets vibrato depth to
                                           tcyc_InstData. */
#define TCYINST_VIBWAVE 0x5E   /* VIBWAVE: Sets vibrato waveform to
                                           tcyc_InstData. */
#define TCYINST_VIBWAVP 0x5F   /* VIBWAVP: Sets vibrato position to
                                           tcyc_InstData. */
#define TCYINST_VIBRATO 0x60   /* VIBRATO: Executes the vibrato. The upper
                                           byte contains the speed or NULL
                                           to use the previous values. The
                                           lower byte contains the depth or
                                           NULL for use the previous
                                           values. */
#define TCYINST_VIBVAL  0x61   /* VIBVAL: Executes the vibrato with the
                                          given period in tcyc_InstData. */
#define TCYINST_ARPSPD  0x62   /* ARPSPD: Sets arpeggio speed to
                                          tcyc_InstData. */
#define TCYINST_ARPWAVE 0x63   /* ARPWAVE: Sets arpeggio waveform to
                                           tcyc_InstData. */
#define TCYINST_ARPWAVP 0x64   /* ARPWAVP: Sets arpeggio position to
                                           tcyc_InstData. */
#define TCYINST_ARPEGIO 0x65   /* ARPEGIO: Executes the arpeggio. The upper
                                           byte of tcyc_InstData contains
                                           the arpeggio speed and the low
                                           byte a finetune modifier. */
#define TCYINST_ARPVAL  0x66   /* ARPVAL: Like ARPEGIO with the exception
                                          that the value is taken from
                                          tcyc_InstData (upper byte is
                                          transpose, lower byte is
                                          finetune) */
#define TCYINST_SETWAVE 0x67   /* SETWAVE: Sets the sample waveform to src +
                                           tcyc_InstData. The current wave-
                                           form will finish playing. */
#define TCYINST_ISETWAV 0x68   /* ISETWAV: Set the sample waveform to
                                           tcyc_InstData immediately. */
#define TCYINST_SETWAVP 0x69   /* SETWAVP: Sets the sample waveform position
                                           to tcy_InstData. */
#define TCYINST_SETRANS 0x6A   /* SETRANS: Sets the transpose number to
                                           tcyc_InstData. The current
                                           transpose will be overwritten. */
#define TCYINST_SETNOTE 0x6B   /* SETNOTE: Sets note but doesn't update note
                                           field. */
#define TCYINST_SETPTCH 0x6C   /* SETPTCH: Sets the current frequency taken
                                           from the destination variable
                                           (upper word) and the destination
                                           variable + 1 (lower word) and adds
                                           tcyc_InstData to it. */
#define TCYINST_SETPER  0x6D   /* SETPER: Like SETPTCH with the exception
                                          that the destination will be
                                          period instead of frequency. */
#define TCYINST_RESET   0x6E   /* RESET: Resets vibrato/tremolo/pannolo
                                         counters. tcyc_InstData is a mask
                                         which disables affection.
                                         Mask | Meanings
                                         0001 | Disables arpeggio reset
                                         0002 | Disables vibrato reset
                                         0004 | Disables tremolo reset
                                         0008 | Disables pannolo reset
                                         0010 | Disables redo of portamento
                                         0020 | Disables portamento reset */
#define TCYINST_VOLSLUP 0x6F   /* VOLSLUP: Slides volume up by
                                           tcyc_InstData. */
#define TCYINST_VOLSLDN 0x70   /* VOLSLDN: Like VOLSLUP but slides down. */
#define TCYINST_TRMSPD  0x71   /* TRMSPD: Like VIBSPD but for tremolo. */
#define TCYINST_TRMDPTH 0x72   /* TRMDPTH: Like VIBDPTH but for tremolo. */
#define TCYINST_TRMWAVE 0x73   /* TRMWAVE: Like VIBWAVE but for tremolo. */
#define TCYINST_TRMWAVP 0x74   /* TRMWAVP: Like VIBWAVP but for tremolo. */
#define TCYINST_TREMOLO 0x75   /* TREMOLO: Executes the tremolo (see VIBRATO
                                           for more informations). */
#define TCYINST_TRMVAL  0x76   /* TRMVAL: Executes the tremolo with the given
                                          volume in source */
#define TCYINST_PANLEFT 0x77   /* PANLEFT: Slides the panning left by
                                           tcyc_InstData. */
#define TCYINST_PANRGHT 0x78   /* PANRGHT: Like PANLEFT but slides right. */
#define TCYINST_PANSPD  0x79   /* PANSPD: Like TRMSPD but for pannolo. */
#define TCYINST_PANDPTH 0x7A   /* PANDPTH: Like TRMDPTH but for pannolo. */
#define TCYINST_PANWAVE 0x7B   /* PANWAVE: Like TRMWAVE but for pannolo. */
#define TCYINST_PANWAVP 0x7C   /* PANWAVP: Like TRMWAVP but for pannolo. */
#define TCYINST_PANNOLO 0x7D   /* PANNOLO: Executes the pannolo (see TREMOLO
                                           for more informations). */
#define TCYINST_PANVAL  0x7E   /* PANVAL: Executes the pannolo with the given
                                          panning in source */

/* Synth code symbol table. It has the same purpose as a symbol table of a
   linker symbol table: It replaces values by symbols. This enhances the
   readability of complex synth codes. */
struct TuComposerSynthSymbolTable {
  struct MinNode tsst_Node;
  STRPTR tsst_SymbolName;      /* Name of symbol (e.g. MAXVOL) */
  UWORD  tsst_SymbolValue;     /* Symbol value */
  UWORD  tsst_FirstLine;       /* First code line the symbol affects */
  UWORD  tsst_LastLine;        /* Last code line the symbol affects */
  UBYTE  tsst_Type;            /* Symbol type. See below */
  UBYTE  tsst_Flags;           /* Symbol flags. See upwards */
  UBYTE  tsst_Reserved[16];    /* Never touch this ! */
  APTR   tsst_UserData;        /* Any user data */
}; /* TuComposerSynthSymbolTable */

/* Symbol types */
#define SYMBOL_PARAMETER   0   /* Symbol is an ordinary instruction parameter
                                  value. tsst_SymbolValue points to the value
                                  to be referenced. */
#define SYMBOL_SRC_VAR     1   /* Symbol is a source variable reference.
                                  tsst_SymbolValue points to variable number
                                  to be referenced (0 - 15) */
#define SYMBOL_DEST_VAR    2   /* Symbol is a destination variable reference.
                                  tsst_SymbolValue is as above. */
#define SYMBOL_VARIABLE    3   /* Symbol is a variable reference (both source
                                  and destination). tsst_SymbolValue is as
                                  above. */
#define SYMBOL_LABEL       4   /* Symbol is a label reference.
                                  tsst_SymbolValue points to the line number
                                  to be referenced. */
#define SYMBOL_ANY        -1   /* Any symbol type (neg. values only used by
                                  MatchSymbol()) */
#define SYMBOL_NAME       -2   /* Just scan for symbol name */
#define SYMBOL_LABELID    -4   /* Search for label definition */
#define SYMBOL_LABELLINE  -5   /* Search for defined label on specified
                                  line */
#define SYMBOL_LABELDBL   -6   /* Search for duplicate label def, i.e. any
                                  matching line except giving will return
                                  label */

/* Synth structure, used by all Synth functions (e.g. AddSynth()) */
struct TuComposerSynth {
  struct MinNode tcy_Node;
  STRPTR tcy_FileName;      /* File name of synth (e.g. SynPiano.TCY) */
  STRPTR tcy_Name;          /* Name of synth (e.g. Synth Piano) */
  struct MinList tcy_Waves; /* Linked list of waveforms or empty if hybrid */
  struct MinList tcy_Symbol; /* Linked list of symbols */
  struct TuComposerSynthCode *tcy_SynthCode; /* Ptr to synth programming
                               code */
  UWORD  tcy_NumWaves;      /* Number of waveforms (if 0 this is a hybrid) */
  UWORD  tcy_CodeEntries;   /* Length of synth code in 2 UWORDs. */
  UWORD  tcy_VolumePos;     /* Entry position of volume handling code */
  UWORD  tcy_PanningPos;    /* Entry position of panning handling code */
  UWORD  tcy_SlidePos;      /* Entry position of slide handling code */
  UWORD  tcy_SpecialPos;    /* Entry position of special handling code */
  UWORD  tcy_VolumeSusPos;  /* Entry position of sustain volume code */
  UWORD  tcy_PanningSusPos; /* Entry position of sustain panning code */
  UWORD  tcy_SlideSusPos;   /* Entry position of sustain slide code */
  UWORD  tcy_SpecialSusPos; /* Entry position of sustain special code */
  UWORD  tcy_VolNNAPos;     /* Position of volume handling code when NNA
                               has been triggered. This allows custom NNA
                               handling ! */
  UWORD  tcy_PanNNAPos;     /* Position of panning handling NNA code */
  UWORD  tcy_SlideNNAPos;   /* Position of slide handling NNA code */
  UWORD  tcy_SpecialNNAPos; /* Position of special NNA handling code */
  UWORD  tcy_VolDNAPos;     /* Position of volume handling code when DNA
                               has been triggered. This allows custom DNA
                               handling ! */
  UWORD  tcy_PanDNAPos;     /* Position of panning handling DNA code */
  UWORD  tcy_SlideDNAPos;   /* Position of slide handling DNA code */
  UWORD  tcy_SpecialDNAPos; /* Position of special DNA handling code */
  UWORD  tcy_Variable0;     /* Initial contents of variable 0 */
  UWORD  tcy_Variable1;     /* Initial contents of variable 1 */
  UWORD  tcy_Variable2;     /* Initial contents of variable 2 */
  UWORD  tcy_Variable3;     /* Initial contents of variable 3 */
  UWORD  tcy_Variable4;     /* Initial contents of variable 4 */
  UWORD  tcy_Variable5;     /* Initial contents of variable 5 */
  UWORD  tcy_Variable6;     /* Initial contents of variable 6 */
  UWORD  tcy_Variable7;     /* Initial contents of variable 7 */
  UWORD  tcy_Variable8;     /* Initial contents of variable 8 */
  UWORD  tcy_Variable9;     /* Initial contents of variable 9 */
  UWORD  tcy_VariableA;     /* Initial contents of variable 10 */
  UWORD  tcy_VariableB;     /* Initial contents of variable 11 */
  UWORD  tcy_VariableC;     /* Initial contents of variable 12 */
  UWORD  tcy_VariableD;     /* Initial contents of variable 13 */
  UWORD  tcy_VariableE;     /* Initial contents of variable 14 */
  UWORD  tcy_VariableF;     /* Initial contents of variable 15 */
  UWORD  tcy_VolConVar;     /* Initial contents of volume condition var. */
  UWORD  tcy_PanConVar;     /* Initial contents of panning condition var. */
  UWORD  tcy_SlideConVar;   /* Initial contents of slide condition var. */
  UWORD  tcy_SpecialConVar; /* Initial contents of special condition var. */
  UBYTE  tcy_UseNNAFlags;   /* Use NNA positions. See bit defs above */
  UBYTE  tcy_UseSusFlags;   /* Use sustain flags */
  UBYTE  tcy_PosKeepMask;   /* Position keep mask. All initial entry
                               positions will be keeped. See bit defs
                               above. */
  UBYTE  tcy_NNAPosKeepMask; /* NNA trigger position keep mask. Nearly same
                                as tcy_PosKeepMask. Look there for
                                details. */
  UWORD  tcy_VarKeepMask;   /* Variable keep mask. All variables where bit
                               is set will be keeped (normally they will
                               be overwritten with the initial values),
                               e.g. bit 5 set will keep variable 5. */
  struct MinList tcy_UnknownChunks; /* Linked list of unknown chunks */
  UBYTE  tcy_Reserved[18];  /* Reserved for future expansions */
  APTR   tcy_UserData;      /* Use for any data you like */
}; /* TuComposerSynth */

/* Default settings for new synth. */

#define DEFAULTSYNTHBITDEPTH   16L /* 16 bit waveforms by default */
#define DEFAULTSYNTHWAVEFORMS   1  /* Default is one waveform */
#define DEFAULTSYNTHWAVELENGTH 64L /* Default size is 64 WORDs. */
#define DEFAULTSYNTHCODELINES   1  /* Default is one line */

/* Synth limitation definitions */
#define MAXWAVEFORMSPERSYNTH 65535  /* Maximum up to 65535 waveforms per
                                       synth. */

/* Pseudo op-codes for synth assembler */

#define InstrInst "INSTR\0\0\0"
#define SetFXInst "SETFX\0\0\0"

/* Synth error codes */

#define ERROR_SYNTH_NOGIVEN  (ERROR_SYNTH+0x0) /* If no synth was given */
#define ERROR_SYNTH_NODATA   (ERROR_SYNTH+0x1) /* Synth contains no data */
#define ERROR_SYNTH_NOMEM    (ERROR_SYNTH+0x2) /* Out of memory */
#define ERROR_SYNTH_TOOMANY  (ERROR_SYNTH+0x3) /* Too many waveforms (max.
                                      65535) */
#define ERROR_SYNTH_NOFILE   (ERROR_SYNTH+0x4) /* No filename specified */
#define ERROR_SYNTH_IOERR    (ERROR_SYNTH+0x5) /* I/O error occured */
#define ERROR_SYNTH_TOOSMALL (ERROR_SYNTH+0x8) /* Synth waveform too small */
#define ERROR_SYNTH_TOOBIG   (ERROR_SYNTH+0x9) /* Synth waveform too big */
#define ERROR_SYNTH_SIZE     (ERROR_SYNTH+0xA) /* Selected waves are not
                                      equal in size */
#define ERROR_SYNTH_BADBITS  (ERROR_SYNTH+0xB) /* ConvertWaveform() bit depth
                                      is not supported (currently only 8
                                      and 16-bit waveforms are supported) */
#define ERROR_SYNTH_SYMNAME  (ERROR_SYNTH+0xC) /* Invalid symbol name,
                                      returned by AssignSynth(). Symbols may
                                      not begin with a decimal character.
                                      Otherwise the only allowed characters
                                      are: 0-9, A-Z, a-z, _, ., @. */

/* IFF-TCM1/IFF-TCI1 YHDR structure. Used by disk operations.
   (e.g. by LoadModule() & SaveModule()) */

struct MIFF_YHDR {
  struct IFFChunk myhdr_IFFHeader;
  UWORD  myhdr_NumWaves;    /* Number of waveforms (if 0 this is a hybrid) */
  UWORD  myhdr_VolumePos;   /* Entry position of volume handling code */
  UWORD  myhdr_PanningPos;  /* Entry position of panning handling code */
  UWORD  myhdr_SlidePos;    /* Entry position of slide handling code */
  UWORD  myhdr_SpecialPos;  /* Entry position of special handling code */
  UWORD  myhdr_VolumeSusPos; /* Entry position of sustain volume code */
  UWORD  myhdr_PanningSusPos; /* Entry position of sustain panning code */
  UWORD  myhdr_SlideSusPos; /* Entry position of sustain slide code */
  UWORD  myhdr_SpecialSusPos; /* Entry position of sustain special code */
  UWORD  myhdr_VolNNAPos;   /* Position of volume handling code when NNA
                               has been triggered. This allows custom NNA
                               handling ! */
  UWORD  myhdr_PanNNAPos;   /* Position of panning handling NNA code */
  UWORD  myhdr_SlideNNAPos; /* Position of slide handling NNA code */
  UWORD  myhdr_SpecialNNAPos; /* Position of special NNA handling code */
  UWORD  myhdr_VolDNAPos;   /* Position of volume handling code when DNA
                               has been triggered. This allows custom DNA
                               handling ! */
  UWORD  myhdr_PanDNAPos;   /* Position of panning handling DNA code */
  UWORD  myhdr_SlideDNAPos; /* Position of slide handling DNA code */
  UWORD  myhdr_SpecialDNAPos; /* Position of special DNA handling code */
  UWORD  myhdr_Variable0;   /* Initial contents of variable 0 */
  UWORD  myhdr_Variable1;   /* Initial contents of variable 1 */
  UWORD  myhdr_Variable2;   /* Initial contents of variable 2 */
  UWORD  myhdr_Variable3;   /* Initial contents of variable 3 */
  UWORD  myhdr_Variable4;   /* Initial contents of variable 4 */
  UWORD  myhdr_Variable5;   /* Initial contents of variable 5 */
  UWORD  myhdr_Variable6;   /* Initial contents of variable 6 */
  UWORD  myhdr_Variable7;   /* Initial contents of variable 7 */
  UWORD  myhdr_Variable8;   /* Initial contents of variable 8 */
  UWORD  myhdr_Variable9;   /* Initial contents of variable 9 */
  UWORD  myhdr_VariableA;   /* Initial contents of variable 10 */
  UWORD  myhdr_VariableB;   /* Initial contents of variable 11 */
  UWORD  myhdr_VariableC;   /* Initial contents of variable 12 */
  UWORD  myhdr_VariableD;   /* Initial contents of variable 13 */
  UWORD  myhdr_VariableE;   /* Initial contents of variable 14 */
  UWORD  myhdr_VariableF;   /* Initial contents of variable 15 */
  UWORD  myhdr_VolConVar;   /* Initial contents of volume condition var. */
  UWORD  myhdr_PanConVar;   /* Initial contents of panning condition var. */
  UWORD  myhdr_SlideConVar; /* Initial contents of slide condition var. */
  UWORD  myhdr_SpecialConVar; /* Initial contents of special condition
                                 var. */
  UBYTE  myhdr_UseNNAFlags; /* Use NNA positions. See bit defs above */
  UBYTE  myhdr_UseSusFlags; /* Use sustain flags */
  UBYTE  myhdr_PosKeepMask; /* Position keep mask. All initial entry
                               positions will be keeped. See bit defs
                               above. */
  UBYTE  myhdr_NNAPosKeepMask; /* NNA trigger position keep mask. Nearly
                                  the same as myhdr_PosKeepMask. */
  UWORD  myhdr_VarKeepMask; /* Variable keep mask. All variables where bit
                               is set will be keeped (normally they will
                               be overwritten with the initial values),
                               e.g. bit 5 set will keep variable 5. */
}; /* MIFF_YHDR */  

/* IFF-TCY1 YHDR structure. Used by disk operations.
   (e.g. by LoadSynth() & SaveSynth()) */

struct IFF_YHDR {
  struct IFFChunk yhdr_IFFHeader;
  UBYTE  yhdr_Version;      /* Version number of TCY1 file */
  UBYTE  yhdr_Revision;     /* Revision number of TCY1 file */
  UWORD  yhdr_NumWaves;     /* Number of waveforms (if 0 this is a hybrid) */
  UWORD  yhdr_VolumePos;    /* Entry position of volume handling code */
  UWORD  yhdr_PanningPos;   /* Entry position of panning handling code */
  UWORD  yhdr_SlidePos;     /* Entry position of slide handling code */
  UWORD  yhdr_SpecialPos;   /* Entry position of special handling code */
  UWORD  yhdr_VolumeSusPos; /* Entry position of sustain volume code */
  UWORD  yhdr_PanningSusPos; /* Entry position of sustain panning code */
  UWORD  yhdr_SlideSusPos;  /* Entry position of sustain slide code */
  UWORD  yhdr_SpecialSusPos; /* Entry position of sustain special code */
  UWORD  yhdr_VolNNAPos;    /* Position of volume handling code when NNA
                               has been triggered. This allows custom NNA
                                handling ! */
  UWORD  yhdr_PanNNAPos;    /* Position of panning handling NNA code */
  UWORD  yhdr_SlideNNAPos;  /* Position of slide handling NNA code */
  UWORD  yhdr_SpecialNNAPos; /* Position of special NNA handling code */
  UWORD  yhdr_VolDNAPos;    /* Position of volume handling code when DNA
                               has been triggered. This allows custom DNA
                               handling ! */
  UWORD  yhdr_PanDNAPos;    /* Position of panning handling DNA code */
  UWORD  yhdr_SlideDNAPos;  /* Position of slide handling DNA code */
  UWORD  yhdr_SpecialDNAPos; /* Position of special DNA handling code */
  UWORD  yhdr_Variable0;    /* Initial contents of variable 0 */
  UWORD  yhdr_Variable1;    /* Initial contents of variable 1 */
  UWORD  yhdr_Variable2;    /* Initial contents of variable 2 */
  UWORD  yhdr_Variable3;    /* Initial contents of variable 3 */
  UWORD  yhdr_Variable4;    /* Initial contents of variable 4 */
  UWORD  yhdr_Variable5;    /* Initial contents of variable 5 */
  UWORD  yhdr_Variable6;    /* Initial contents of variable 6 */
  UWORD  yhdr_Variable7;    /* Initial contents of variable 7 */
  UWORD  yhdr_Variable8;    /* Initial contents of variable 8 */
  UWORD  yhdr_Variable9;    /* Initial contents of variable 9 */
  UWORD  yhdr_VariableA;    /* Initial contents of variable 10 */
  UWORD  yhdr_VariableB;    /* Initial contents of variable 11 */
  UWORD  yhdr_VariableC;    /* Initial contents of variable 12 */
  UWORD  yhdr_VariableD;    /* Initial contents of variable 13 */
  UWORD  yhdr_VariableE;    /* Initial contents of variable 14 */
  UWORD  yhdr_VariableF;    /* Initial contents of variable 15 */
  UWORD  yhdr_VolConVar;    /* Initial contents of volume condition var. */
  UWORD  yhdr_PanConVar;    /* Initial contents of panning condition var. */
  UWORD  yhdr_SlideConVar;  /* Initial contents of slide condition var. */
  UWORD  yhdr_SpecialConVar; /* Initial contents of special condition var. */
  UBYTE  yhdr_UseNNAFlags;  /* Use NNA positions. See bit defs above */
  UBYTE  yhdr_UseSusFlags;  /* Use sustain flags */
  UBYTE  yhdr_PosKeepMask;  /* Position keep mask. All initial entry
                               positions will be keeped. See bit defs
                               above. */
  UBYTE  yhdr_NNAPosKeepMask; /* NNA trigger position keep mask. Nearly same
                                 as yhdr_PosKeepMask. Look there for
                                 details. */
  UWORD  yhdr_VarKeepMask;  /* Variable keep mask. All variables where bit
                               is set will be keeped (normally they will
                               be overwritten with the initial values),
                               e.g. bit 5 set will keep variable 5. */
}; /* IFF_YHDR */

/* IFF-TCM1/IFF-TCI1/IFF-TCY1 WHDR structure. Used by disk operations.
   (e.g. by LoadSynth() & SaveSynth()) */

struct IFF_WHDR {
  struct IFFChunk whdr_IFFHeader;
  ULONG  whdr_Repeat;       /* Repeat start of waveform */
  ULONG  whdr_RepLen;       /* Repeat length of waveform */
  UWORD  whdr_Flags;        /* Some flags. See below */
}; /* IFF_WHDR */

/* IFF-TCM1/IFF-TCI1/IFF-TCY1 SREF structure. Used by disk operations.
   (e.g. by LoadSynth() & SaveSynth()) */

struct IFF_SREF {
  struct IFFChunk sref_IFFHeader;
  UWORD  sref_SymbolValue;  /* Symbol value */
  UWORD  sref_FirstLine;    /* First code line the symbol affects */
  UWORD  sref_LastLine;     /* Last code line the symbol affects */
  UBYTE  sref_Type;         /* Symbol type. See above */
  UBYTE  sref_Flags;        /* Symbol flags. See upwards */
}; /* IFF_SREF */

#define YHDR_VERSION            1   /* Current synth version: 1 */
#define YHDR_REVISION           0   /* Current synth revision: 0 */

#endif /* TUCOMPOSER_SYNTH_H */
