#ifndef TUCOMPOSER_TRACK_H
#define TUCOMPOSER_TRACK_H
/*
**  $VER: track.h 31.22 (31.3.99)
**
**  Standard C header for tucomposer.library
**
**  (C) Copyright 1999 by Basty/Seasons
**      All Rights Reserved
*/

#ifndef EXEC_TYPES_H
#include <exec/types.h>
#endif /* EXEC_TYPES_H */

#ifndef EXEC_NODES_H
#include <exec/nodes.h>
#endif /* EXEC_NODES_H */

#ifndef EXEC_LISTS_H
#include <exec/lists.h>
#endif /* EXEC_LISTS_H */

#ifndef TUCOMPOSER_TUCOMPOSER_H
#include <tucomposer/tucomposer.h>
#endif /* TUCOMPOSER_TUCOMPOSER_H */

/* Bit definitions for tct_CompFlags: */
#define TCTCB_SOffset       0  /* Sample offset beyond end of sample will be
                                  ignored (IT compatibility) */
#define TCTCB_TPorta        1  /* Share tone porta memory with portamentoes
                                  (again for IT compatibility), also unlocks
                                  tone porta samples and adjusts freq to:
                                  NewFreq = Frequency * NewRate / OldRate
                                  If an instrument number is given the envelope
                                  will be retriggered */
#define TCTCB_Slide         2  /* Portas of same type share the same memory
                                  (e.g. porta up/fine porta up/etc.) */
#define TCTCB_VolSlide      3  /* All except portamento slides share the same
                                  memory (e.g. volume/panning slides) */
#define TCTCB_OpSlide       4  /* Oppositional portamento directions don't
                                  share the same memory (e.g. porta up and
                                  porta down) */
#define TCTCB_OpVSlide      5  /* Oppositional non-portamento slide
                                  directions don't share the same memory */
#define TCTCB_VolPitch      6  /* Volume & Pitch slides share same memory
                                  (for S3M compatibility) */

#define TCTCF_SOffset       (1<<TCTCB_SOffset)
#define TCTCF_TPorta        (1<<TCTCB_TPorta)
#define TCTCF_Slide         (1<<TCTCB_Slide)
#define TCTCF_VolSlide      (1<<TCTCB_VolSlide)
#define TCTCF_OpSlide       (1<<TCTCB_OpSlide)
#define TCTCF_OpVSlide      (1<<TCTCB_OpVSlide)
#define TCTCF_VolPitch      (1<<TCTCB_VolPitch)

/* Bit definitions for tct_Flags */
#define TCTB_UseTiming      1  /* Bit set means, use track timing fields */
#define TCTB_SPDTiming      2  /* Bit set means, SPD speed timing else BPM */
#define TCTB_TrackPan       3  /* Bit set means, track pan will be used */
#define TCTB_SurPan         4  /* Bit set means, surround pan for track */
#define TCTB_Reverse        5  /* Means, Track will be played backwards */

#define TCTF_UseTiming      (1<<TCTB_UseTiming)
#define TCTF_SPDTiming      (1<<TCTB_SPDTiming)
#define TCTF_TrackPan       (1<<TCTB_TrackPan)
#define TCTF_SurPan         (1<<TCTB_SurPan)
#define TCTF_Reverse        (1<<TCTB_Reverse)

/* Bit definitions for SwapTrackInSong(): */
#define STISB_Swap          0L /* If bit set, swap track data */
#define STISB_Update        1L /* If bit set, update order list entries. */

#define STISF_Swap          (1L<<STISB_Swap)
#define STISF_Update        (1L<<STISB_Update)

/* Bit definitions for GetNote...Str() flags */
#define NSB_HDisplay        0L /* If bit is set B- will be displayed as H- */
#define NSB_bSharp          1L /* If set C# will be displayed as Db, etc. */

#define NSF_HDisplay        (1L<<NSB_HDisplay)
#define NSF_bSharp          (1L<<NSB_bSharp)

/* Bit definitions for InsertRow() and DeleteRow() flags */
#define ROWB_NotesOnly      0L /* If bit is set, only the notes will be
                                  else track length will be affected, too. */

#define ROWF_NotesOnly      (1L<<ROWB_NotesOnly)

/* Bit definitions for packed tracks.
   To extract a packed track by hand, read a packed byte, check the table
   below, and read the desired bytes and assign them. If TCPB_RowHI, and
   TCPB_RowLO are clear the next row is assumed, else continue extracting at
   the given row. Example: If bits 2, 4, 5, and 7 are set, read note byte,
   then instr lo-byte (hi-byte isn't affected until new readed), then read
   effect byte,  and at last read data lo-byte (again clear hi-byte). All
   others are cleared. Do this until packed byte is $00 (The IFF-TCT1 (the
   others also) doesn't need this, so before call UnPackTrack(), clear the
   last byte or you may get garbage or even a system crash. The PackTrack()
   routine does this automatically when you pack but SaveTrack() drops the
   NULL-byte, so you have to add it yourself when loading a track by hand. */

#define TCPB_RowHI          0  /* Bit set, used row hi-byte follows, else
                                  next one is supposed. */
#define TCPB_RowLO          1  /* Bit set, used row lo-byte follows, else
                                  next one is supposed. */
#define TCPB_Note           2  /* Bit set, octave (hi-nibble), note
                                  (lo-nibble) follows (0xFx are special notes
                                  (keyoff, etc.)) */
#define TCPB_InstrHI        3  /* Bit set, hi-byte of instr follows */
#define TCPB_InstrLO        4  /* Bit set, lo-byte of instr follows */
#define TCPB_Effect         5  /* Bit set, first effect byte follows */
#define TCPB_DataHI         6  /* Set, first hi-byte of data word follows */
#define TCPB_DataLO         7  /* Set, first lo-byte of data word follows */

#define TCPF_RowHI          (1<<TCPB_RowHI)
#define TCPF_RowLO          (1<<TCPB_RowLO)
#define TCPF_Note           (1<<TCPB_Note)
#define TCPF_InstrHI        (1<<TCPB_InstrHI)
#define TCPF_InstrLO        (1<<TCPB_InstrLO)
#define TCPF_Effect         (1<<TCPB_Effect)
#define TCPF_DataHI         (1<<TCPB_DataHI)
#define TCPF_DataLO         (1<<TCPB_DataLO)

struct TuComposerTrackData {
  UBYTE  tctd_Octave;              /* On which octave note should be played
                                      NOTE: Do not check octave for keyoff,
                                      etc. ! */
  BYTE  tctd_Note;                 /* Note (0 is none, 1 is C-, 12 is B-,
                                      0xFD is keyoff). */
  UWORD  tctd_Instrument;          /* Number of instrument to be played */
  struct MinList tctd_Effects;     /* Pointer to effects */
                                   /* Now the same comes except for
                                      next row. This comes as often as
                                      track len. */
}; /* TuComposerTrackData */

/* Note names. */

#define NOTE_NONE             0    /* --- */
#define NOTE_C_               1    /* C-x */
#define NOTE_C_SHARP          2    /* C#x = Dbx */
#define NOTE_D_               3    /* D-x */
#define NOTE_D_SHARP          4    /* D#x = Ebx */
#define NOTE_E_               5    /* E-x */
#define NOTE_F_               6    /* F-x */
#define NOTE_F_SHARP          7    /* F#x = Gbx */
#define NOTE_G_               8    /* G-x */
#define NOTE_G_SHARP          9    /* G#x = Abx */
#define NOTE_A_               10   /* A-x */
#define NOTE_A_SHARP          11   /* A#x = Bbx */
#define NOTE_B_               12   /* B-x */
#define NOTE_H_               NOTE_B_ /* H-x */
#define NOTE_END              0xF0 /* END (pattern end marker) */
#define NOTE_FADE             0xFB /* -\- (note fade) */
#define NOTE_HOLD_DELAY       0xFC /* -|- (hold delay) */
#define NOTE_KEYOFF           0xFD /* === (keyoff note) */
#define NOTE_OFF              0xFE /* ^^- (Note off) */
#define NOTE_KILL             0xFF /* ^^^ (Kill note) */

/* Default note, octave, tune. */

#define DEFAULTOCTAVE         4    /* Default octave is 4 */
#define DEFAULTNOTE           NOTE_C_ /* Default note is C- */
#define DEFAULTTONE           (4<<8|NOTE_C_) /* Default tone is C-4 */

/* Octave, note ranges */

#define FIRSTOCTAVE           0                 /* 1st octave is 0, not 1 */
#define FIRSTNOTE             NOTE_C_           /* First note is C- */
#define LASTOCTAVE            9                 /* Octaves ranges up to 9 */
#define LASTNOTE              NOTE_B_           /* Notes range from C- to B- */
#define NUMOCTAVES            (LASTOCTAVE+1)    /* Number of octaves */
#define NUMNOTES              NOTE_B_           /* Number of notes */
#define SPEC_NOTES            0xF0              /* All above this is reserved
                                                   for special notes (like
                                                   keyoff and similar) */

/* Track structure, used by all Track functions (e.g. SplitTrack()) */
struct TuComposerTrack {
  struct MinNode tct_Node;
  STRPTR tct_FileName;     /* Pointer to track file name (e.g. MainPart.TRK).
                              Remember that TuComposer is also available for
                              MS-DOS and other platforms. So, please try
                              to use the limited 8x3 MS-DOS file/dir
                              name length. */
  STRPTR tct_Name;         /* Pointer to track info, e.g. (Main Part) */
  struct TuComposerTrackData *tct_Data; /* Pointer to track data */
  UWORD  tct_Length;       /* Last row of track (default is 63 (0x3F)) */
  UBYTE  tct_Volume;       /* Track global volume */
  UBYTE  tct_SubVolume;    /* Sub volume (used when you slide) */
  BYTE   tct_Panning;      /* Panning for track if ins panning is off */
  UBYTE  tct_SubPanning;   /* Sub Panning (used when you slide) */
  BYTE   tct_Transpose;    /* Track transpose */
  UBYTE  tct_CompFlags;    /* Compatibility flags (look bit def table) */
  UWORD  tct_Flags;        /* Track flags, see bit defs */
  UWORD  tct_Frames;       /* Number of frames per row (default 6) */
  UBYTE  tct_SpeedMul;     /* Speed multiplier (allowing you e.g. 2/3) */
  UBYTE  tct_SpeedDiv;     /* Speed divider (the div for the multiplier) */
  UWORD  tct_SPDSpeed;     /* Track SPD */
  UWORD  tct_BPMTempo;     /* Track BPM tempo (number of rows of a beat). */
  UWORD  tct_BPMSpeed;     /* Track BPM speed. */
  struct MinList tct_UnknownChunks; /* Linked list of unknown chunks */
  UBYTE  tct_Reserved[8];  /* Reserved for future expansions */
  APTR   tct_UserData;     /* This field is ignored by TuComposer.Library. */
}; /* TuComposerTrack */

#define DEFAULTTRACKROWS      0x40 /* Default rows of track (64) */
#define DEFAULTTRACKLENGTH    (DEFAULTTRACKROWS-1) /* Length entry is -1 */
#define DEFAULTTRACKVOLUME    0xFF /* Default is full volume */
#define DEFAULTTRACKPANNING  -0x80 /* Default is middle panning */
#define DEFAULTTRACKTRANSPOSE 0 /* No default transpose */
#define DEFAULTTRACKFRAMES    6 /* Delay one row 6/50 secs. */
#define DEFAULTTRACKSPEEDMUL  0 /* Disable multiplier and */
#define DEFAULTTRACKSPEEDDIV  0 /* the divider */
#define DEFAULTTRACKSPDSPEED  33 /* 33 as in OctaMED */
#define DEFAULTTRACKBPMTEMPO  4 /* 4 rows are a beat. */
#define DEFAULTTRACKBPMSPEED  125 /* Beats per minute (50 Hz) */

/* Maximum values */

#define MAXTRACKROWS          0x10000 /* Maximum number of rows possible */
#define TRACKLASTROW          (MAXTRACKROWS-1) /* Last valid row to
                                                  address */

/* Track error codes */

#define ERROR_TRACK_NOGIVEN   (ERROR_TRACK+0x0) /* If no track given */
#define ERROR_TRACK_NODATA    (ERROR_TRACK+0x1) /* Track contains no data */
#define ERROR_TRACK_NOMEM     (ERROR_TRACK+0x2) /* Returned on mem fail */
#define ERROR_TRACK_NOFILE    (ERROR_TRACK+0x4) /* No file given */
#define ERROR_TRACK_IOERR     (ERROR_TRACK+0x5) /* I/O error occured on */
#define ERROR_TRACK_BAD_IFF   (ERROR_TRACK+0x6) /* TCT1-IFF-file damaged */
/* Because LoadTrack() returns NULL on error, use DOS IoErr() and check
   ERROR_INVALID_RESIDENT_LIBRARY for it. */

#define ERROR_TRACK_BAD_VER   (ERROR_TRACK+0x7) /* Incorrect track version */
/* Because LoadTrack() returns NULL on error, use DOS IoErr() and check
   ERROR_ACTION_NOT_KNOWN for it. */

#define ERROR_TRACK_TOO_BIG   (ERROR_TRACK+0x8) /* If rows > 0x10000 */
#define ERROR_TRACK_TOO_LOW   (ERROR_TRACK+0x9) /* If rows < 0x1 */
#define ERROR_TRACK_NOSCALE   (ERROR_TRACK+0xA) /* Invalid scale (=<0) */
#define ERROR_TRACK_INVALID_OCTAVE  (ERROR_TRACK+0xB) /* Too much
                              octaves ! */
#define ERROR_TRACK_INVALID_NOTE    (ERROR_TRACK+0xC) /* Invalid note ! */

struct TuComposerTrackEffect {
  struct MinNode tcte_Node;
  UBYTE  tcte_Reserved; /* Unused now, leave it zero ! */
  UBYTE  tcte_EffectByte; /* Effect byte, look table below */
  UWORD  tcte_DataWord; /* Data word, same as old PT data byte. */
  UBYTE  tcte_NextEffect[0]; /* Next effect, until it reaches zero */
}; /* TuComposerTrackEffect */

/* Effect types. */

#define NOTE_EFFECTS          0x00 /* Note effects */
#define VOLUME_EFFECTS        0x20 /* Volume related effects */
#define PANNING_EFFECTS       0x30 /* Panning related effects */
#define TRACK_EFFECTS         0x40 /* Track/Channel effects */
#define INSTR_EFFECTS         0x50 /* Instrument related effects */
#define GLOBAL_EFFECTS        0x60 /* Global effects */
#define USER_EFFECTS          0x7F /* User effect for demos, etc. */

/* Note effects: */

#define FX_ARPEGGIO           0x00 /* Arpeggio. Data word = xxyy */
#define FX_PORTA_UP           0x01 /* Portamento up. Data word = xxyy */
#define FX_PORTA_DOWN         0x02 /* Portamento down. Data word = xxyy */
#define FX_F_PORTA_UP         0x03 /* Fine portamento up. Data word = xxyy */
#define FX_F_PORTA_DOWN       0x04 /* Fine portamento dn. Data word = xxyy */
#define FX_O_PORTA_UP         0x05 /* Portamento up once. Data word = xxyy */
#define FX_O_PORTA_DOWN       0x06 /* Portamento dn once. Data word = xxyy */
#define FX_OFPORTA_UP         0x07 /* Fine porta up once. Data word = xxyy */
#define FX_OFPORTA_DOWN       0x08 /* Fine porta dn once. Data word = xxyy */
#define FX_TONE_PORTA         0x09 /* Tone portamento. Data word = xxyy */
#define FX_F_TONE_PORTA       0x0A /* Fine tone porta. Data word = xxyy */
#define FX_O_TONE_PORTA       0x0B /* Tone porta once. Data word = xxyy */
#define FX_OFTONE_PORTA       0x0C /* Fine tone porta once. Data = xxyy */
#define FX_NOTE_SLIDE         0x0D /* (Fine) note slide up/down. xxyy */
#define FX_VIBRATO            0x0E /* Vibrato. Data word = xxyy */
#define FX_FINE_VIBRATO       0x0F /* Fine vibrato. Data word = xxyy */
#define FX_VIBRATO_ONCE       0x10 /* Vibrato once. Data word = xxyy */
#define FX_F_VIB_ONCE         0x11 /* Fine vibrato once. Data word = xxyy */
#define FX_DO_KEYOFF          0x12 /* Do a keyoff. Data word = xxxx */
#define FX_DO_HOLD            0x13 /* Do a hold delay. Data word = xxxx */
#define FX_NOTE_FADE          0x14 /* Do a fadeout. Data word = xxxx */
#define FX_NOTE_CUT           0x15 /* Note cut. Data word = xxyy */
#define FX_NOTE_DELAY         0x16 /* Note delay. Data word = xxxx */
#define FX_TREMOR             0x17 /* Tremor. Data word = xxyy */
#define FX_RETRIG_NOTE        0x18 /* Note retrigger. Data word = xxxx */
#define FX_MULTI_RETRIG       0x19 /* Multi retrig note. Data word = xxyz */
#define FX_EXTEND_CTRL        0x1A /* Extended control. Data word = xyyy */
#define FX_INVERT_LOOP        0x1B /* Invert loop. Data word = xxxx */
#define FX_EXECUTE_FX         0x1C /* Execute FX at Tick. Data word = xxxx */
#define FX_STOP_FX            0x1D /* Stop FX at Tick. Data word = xxyy */

/* Extended control x values */

#define EXC_FREQ_TABLE        0x0 /* Set frequence table use for slides */
#define EXC_GLISS_CTRL        0x1 /* Glissando control */
#define EXC_MULTI_CTRL        0x2 /* Multi retrig control */
#define EXC_PAST_CUT          0x3 /* Past note cut */
#define EXC_PAST_FADE         0x4 /* Past note fade out */
#define EXC_PAST_OFF          0x5 /* Past note off */

/* Volume effects: */

#define FX_SET_VOLUME         0x20 /* Set volume. Data word = xxyy */
#define FX_VOL_SLD_UP         0x21 /* Volume slide up. Data word = xxyy */
#define FX_VOL_SLD_DOWN       0x22 /* Volume slide down. Data word = xxyy */
#define FX_F_VOLSL_UP         0x23 /* Fine vol slide up. Data word = xxyy */
#define FX_F_VOLSL_DOWN       0x24 /* Fine volslide down. Data word = xxyy */
#define FX_VOL_SLD_TO         0x25 /* Volume slide to. Data word = xxyy */
#define FX_TREMOLO            0x26 /* Tremolo. Data word = xxyy */
#define FX_TREMOLO_ONCE       0x27 /* Tremolo once. Data word = xxyy */
#define FX_SET_TRK_VOL        0x28 /* Set track volume. Data word = xxyy */
#define FX_TVOL_SL_UP         0x29 /* Track vol slide up. Data word = xxyy */
#define FX_TVOL_SL_DOWN       0x2A /* Track vol slide dn. Data word = xxyy */
#define FX_F_TVOL_SL_UP       0x2B /* Fine track vol slide up. Data = xxyy */
#define FX_F_TVOL_SL_DN       0x2C /* Fine track vol slide dn. Data = xxyy */
#define FX_TVOL_SLD_TO        0x2D /* Track vol slide to. Data word = xxyy */
#define FX_TRK_TREMOLO        0x2E /* Track tremolo. Data word = xxyy */
#define FX_T_TREMO_ONCE       0x2F /* Track tremolo once. Data word = xxyy */

/* Panning effects: */

#define FX_SET_PANNING        0x30 /* Set pan position. Data word = xxyy */
#define FX_PAN_SL_LEFT        0x31 /* Panning slide left. Data word = xxyy */
#define FX_PAN_SL_RIGHT       0x32 /* Pan slide right. Data word = xxyy */
#define FX_F_P_SL_LEFT        0x33 /* Fine pan slide left. Data = xxyy */
#define FX_F_P_SL_RIGHT       0x34 /* Fine pan slide right. Data = xxyy */
#define FX_PAN_SLD_TO         0x35 /* Panning slide to. Data word = xxyy */
#define FX_PANNOLO            0x36 /* Tremolo for pan. Data word = xxyy */
#define FX_PANNOLO_ONCE       0x37 /* Pannolo once. Data word = xxyy */
#define FX_SET_TRK_PAN        0x38 /* Set track panning. Data word = xxyy */
#define FX_TPAN_SL_LEFT       0x39 /* Track pan slide up. Data word = xxyy */
#define FX_TPAN_SL_RGHT       0x3A /* Track pan slide dn. Data word = xxyy */
#define FX_F_TP_SL_LEFT       0x3B /* Fine track pan slide up. Data = xxyy */
#define FX_F_TP_SL_RGHT       0x3C /* Fine track pan slide dn. Data = xxyy */
#define FX_TPAN_SLD_TO        0x3D /* Track pan slide to. Data word = xxyy */
#define FX_TRK_PANNOLO        0x3E /* Track pannolo. Data word = xxyy */
#define FX_TPANOLO_ONCE       0x3F /* Track pannolo once. Data word = xxyy */

/* Track effects: */

#define FX_SET_TEMPO          0x40 /* Set channel tempo. Data word = xxxx */
#define FX_SET_REL_TMPO       0x41 /* Set relative tempo. Data word = xxxx */
#define FX_PATT_BREAK         0x42 /* Pattern break. Data word = xxxx */
#define FX_POS_JUMP           0x43 /* Position jump. Data word = xxxx */
#define FX_REL_POS_JUMP       0x44 /* Relative pos jump. Data word = xxxx */
#define FX_CHG_PATTERN        0x45 /* Change pattern. Data word = xxxx */
#define FX_REVERSE_PLAY       0x46 /* Reverse play ctrl. Data word = xxxx */
#define FX_PATT_DELAY         0x47 /* Pattern delay. Data word = xxxx */
#define FX_F_PATT_DELAY       0x48 /* Fine pattern delay. Data word = xxxx */
#define FX_PATT_LOOP          0x49 /* Pattern loop. Data word = xxxx */
#define FX_GOSUB              0x4A /* GoSub. Data word = xxxx */
#define FX_RETURN             0x4B /* Return from GoSub. Data word = xxxx */
#define FX_CHANNEL_SYNC       0x4C /* Channel sync. Data word = xxyy */
#define FX_SET_SUBSLIDE       0x4D /* Set target sub-slide to. Data = xxyy */

/* Set target sub-slide to xx bits */
#define TSSB_VOLUME           0L   /* Set volume sub-slide to yy */
#define TSSB_TRACKVOLUME      1L   /* Set track volume sub-slide to yy */
#define TSSB_GLOBALVOLUME     2L   /* Set global volume sub-slide to yy */
#define TSSB_PANNING          3L   /* Set panning sub-slide to yy */
#define TSSB_TRACKPANNING     4L   /* Set track panning sub-slide to yy */
#define TSSB_GLOBALPANNING    5L   /* Set global panning sub-slide to yy */

#define TSSF_VOLUME           (1L<<TSSB_VOLUME)
#define TSSF_TRACKVOLUME      (1L<<TSSB_TRACKVOLUME)
#define TSSF_GLOBALVOLUME     (1L<<TSSB_GLOBALVOLUME)
#define TSSF_PANNING          (1L<<TSSB_PANNING)
#define TSSF_TRACKPANNING     (1L<<TSSB_TRACKPANNING)
#define TSSF_GLOBALPANNING    (1L<<TSSB_GLOBALPANNING)

/* Instrument effects: */

#define FX_SMP_OFF_HIGH       0x50 /* Sample offset HI-Word. Data = xxxx */
#define FX_SMP_OFF_LOW        0x51 /* Sample offset LO-Word. Data = xxxx */
#define FX_SET_HOLD           0x52 /* Set hold. Data word = xxxx */
#define FX_SET_DECAY          0x53 /* Set decay. Data word = xxxx */
#define FX_SET_TRANSP         0x54 /* Set transpose. Data word = xxyy */
#define FX_INS_CONTROL        0x55 /* Instrument control. Data word = xxyy */
#define FX_INS_CHANGE         0x56 /* Instrument change. Data word = xyyy */
#define FX_SYNTH_CTRL         0x57 /* Synth control. Data word = xxyy */
#define FX_SET_SYN_VAL        0x58 /* Set synth value. Data word = xxxx */
#define FX_ENV_CONTROL        0x59 /* Envelope control. Data word = xxyy */
#define FX_SET_ENV_VAL        0x5A /* Set envelope value. Data word = xxxx */
#define FX_NNA_CONTROL        0x5B /* Set new note action. Data = xxxx */
#define FX_SET_LOOP           0x5C /* Loop control. Data word = xyyy */

/* Instrument control xx (on-bits) and yy (off-bits) bits */
#define ICB_PANNING           0L /* Instrument panning on if bit set in xx,
                                    when in yy it's off */
#define ICB_TRANSPOSE         1L /* Position transposeable if bit set in xx,
                                    when in yy it's off */
#define ICB_SLIDENVPORTA      3L /* Slide envelope portamento bit */
#define ICB_SLIDENVLINEAR     4L /* Slide envelope linear frequence bit */
#define ICB_AUTOVIBLINEAR     7L /* Auto vibrato linear frequence bit */

#define ICF_PANNING           (1L<<ICB_PANNING)
#define ICF_TRANSPOSE         (1L<<ICB_TRANSPOSE)
#define ICF_SLIDENVPORTA      (1L<<ICB_SLIDENVPORTA)
#define ICF_SLIDENVLINEAR     (1L<<ICB_SLIDENVLINEAR)
#define ICF_AUTOVIBLINEAR     (1L<<ICB_AUTOVIBLINEAR)

/* Instrument change x values */
#define IC_GLOBAL_VOL         0x0 /* Change instr global volume to 0yy */
#define IC_VOL_SWING          0x1 /* Change instr volume swing to yyy */
#define IC_PAN_SWING          0x2 /* Change instr panning swing to yyy */
#define IC_PITCH_SWING        0x3 /* Change instr pitch swing to yyy */
#define IC_FADEOUT            0x4 /* Change instr fadeout to yyy0 */
#define IC_FADE_COUNT         0x5 /* Change fadeout count to yyy0 */
#define IC_A_VIB_CTRL         0x6 /* Set Auto Vibrato/Tremolo/Pannolo wave */
#define IC_A_VIB_SWEEP        0x60 /* Set auto vibrato sweep to yy */
#define IC_A_VIB_DEPTH        0x61 /* Set auto vibrato depth to yy */
#define IC_A_VIB_RATE         0x62 /* Set auto vibrato rate to yy */
#define IC_A_TREM_SWEEP       0x64 /* Set auto tremolo sweep to yy */
#define IC_A_TREM_DEPTH       0x65 /* Set auto tremolo depth to yy */
#define IC_A_TREM_RATE        0x66 /* Set auto tremolo rate to yy */
#define IC_A_PAN_SWEEP        0x68 /* Set auto pannolo sweep to yy */
#define IC_A_PAN_DEPTH        0x69 /* Set auto pannolo depth to yy */
#define IC_A_PAN_RATE         0x6A /* Set auto pannolo rate to yy */
#define IC_PCH_PAN_SEP        0x7 /* Set pitch panning separation to yyy */
#define IC_PCH_PAN_CNTR       0x8 /* Set pitch panning center to 0yy */

/* Global effects: */

#define FX_SET_SPEED          0x60 /* Set speed. Data word = xyyy */
#define FX_SPD_SLD_FAST       0x61 /* Speed slide faster. Data word = xxyy */
#define FX_SPD_SLD_SLOW       0x62 /* Speed slide slower. Data word = xxyy */
#define FX_F_S_SLD_FAST       0x63 /* Fine speed slide faster. Data = xxyy */
#define FX_F_S_SLD_SLOW       0x64 /* Fine speed slide slower. Data = xxyy */
#define FX_SPD_SLD_TO         0x65 /* Set speed slide to. Data word = xxyy */
#define FX_SPENOLO            0x66 /* Spenolo (Vib for speed). Data = xxyy */
#define FX_SPENOLO_ONCE       0x67 /* Spenolo once. Data word = xxyy */
#define FX_CHANNEL_CTRL       0x68 /* Channel control. Data word = xyzz */
#define FX_SET_G_VOLUME       0x69 /* Set global volume. Data word = xxyy */
#define FX_G_VOLSL_UP         0x6A /* Glob. vol slide up. Data word = xxyy */
#define FX_G_VOLSL_DOWN       0x6B /* Glob. vol slide dn. Data word = xxyy */
#define FX_F_G_VOL_UP         0x6C /* Fine glob. vol slide up. Data = xxyy */
#define FX_F_G_VOL_DOWN       0x6D /* Fine glob. vol slide dn. Data = xxyy */
#define FX_G_VOLSL_TO         0x6E /* Set glob. vol slide to. Data = xxyy */
#define FX_G_TREMOLO          0x6F /* Global Tremolo. Data word = xxyy */
#define FX_G_TREMO_ONCE       0x70 /* Global Tremolo once. Data = xxyy */
#define FX_SET_G_PAN          0x71 /* Set global pan pos. Data word = xxyy */
#define FX_GPANSL_LEFT        0x72 /* Global pan slide left. Data = xxyy */
#define FX_GPANSL_RIGHT       0x73 /* Global pan slide right. Data = xxyy */
#define FX_FGP_SL_LEFT        0x74 /* Global fine pan slide l. Data = xxyy */
#define FX_FGP_SL_RIGHT       0x75 /* Global fine pan slide r. Data = xxyy */
#define FX_GPANSL_TO          0x76 /* Set global pan slide to. Data = xxyy */
#define FX_G_PANNOLO          0x77 /* Global pannolo. Data word = xxyy */
#define FX_G_PANNOLO_O        0x78 /* Global pannolo once. Data = xxyy */

/* Channel control x contents: */
#define CC_EFFECT_CHANNEL     0x0 /* Effect channel related control. */
#define CC_OUTPUT_CONTROL     0x1 /* Channel output control. */

/* Effect channel control (xy): */

#define EC_SET_CC_MODE        0x00 /* Set channel control mode to zz. */
#define EC_SET_O_EFFECT       0x01 /* Set once effect channel to zz. */
#define EC_SET_EFFECT         0x02 /* Set effect channel to zz. */
#define EC_SLD_E_LEFT         0x03 /* Slide effect channel to left by zz. */
#define EC_SLD_E_RIGHT        0x04 /* Slide effect channel to right by zz. */
#define EC_F_S_E_LEFT         0x05 /* F. slide fx channel to left by zz. */
#define EC_F_S_E_RIGHT        0x06 /* F. slide fx channel to right by zz. */
#define EC_F_SO_E_LEFT        0x07 /* F. slide fx channel left once by zz. */
#define EC_F_SO_E_RIGHT       0x08 /* F. sl. fx channel right once by zz. */
#define EC_CHANNOLO           0x09 /* Channolo (Vib for channels) to zz. */
#define EC_F_CHANNOLO         0x0A /* Fine channolo to zz. */
#define EC_F_CHANNOLO_O       0x0B /* Fine channolo once to zz. */
#define OC_SURROUND_CHN       0x10 /* If zz == 00 then disable surround, else
                                      enable it. */
#define OC_MUTE_CHANNEL       0x11 /* If zz == 00 then MUTE channel else play
                                      it. */

/* Track packed structure. Used by PackTrack(), UnPackTrack(), LoadTrack(),
   and SaveTrack(). */

struct TuComposerPacked {
  STRPTR tcp_FileName;      /* Pointer to packed file name
                               (e.g. MainPart.TRK). Remember that TuComposer
                               is also available for MS-DOS and other
                               platforms. So, please try to use the limited
                               8x3 MS-DOS file/dir name length. */
  STRPTR tcp_Name;          /* Pointer to track info, e.g. (Main Part) */
  UBYTE  *tcp_Data;         /* Pointer to packed track data (notes, etc.) */
  ULONG  tcp_PackLength;    /* Track packed size */
  UWORD  tcp_Length;        /* Track length in rows - 1 (default 63) */
  UBYTE  tcp_Volume;        /* Track global volume */
  UBYTE  tcp_SubVolume;     /* Sub volume (used when you slide) */
  BYTE   tcp_Panning;       /* Panning for track if ins panning is off */
  UBYTE  tcp_SubPanning;    /* Sub Panning (used when you slide) */
  BYTE   tcp_Transpose;     /* Track transpose */
  UBYTE  tcp_CompFlags;     /* Compatibility flags (look bit def table) */
  UWORD  tcp_Flags;         /* Track flags, see bit defs */
  UWORD  tcp_Frames;        /* Number of frames per row (default 6) */
  UBYTE  tcp_SpeedMul;      /* Speed multiplier (allowing you e.g. 2/3) */
  UBYTE  tcp_SpeedDiv;      /* Speed divider (the div for the multiplier) */
  UWORD  tcp_SPDSpeed;      /* Track SPD */
  UWORD  tcp_BPMTempo;      /* Track BPM tempo (number of rows of a beat). */
  UWORD  tcp_BPMSpeed;      /* Track BPM speed. */
  struct MinList tcp_UnknownChunks; /* Linked list of unknown chunks */
  UBYTE  tcp_Reserved[8];   /* Reserved for future expansions */
  APTR   tcp_UserData;      /* This field is ignored by TuComposer. */
}; /* TuComposerPacked */

/* IFF-TCT1 THDR structure. Used by disk operations
   (e.g. by LoadTrack() & SaveTrack()) */

struct IFF_THDR {
  struct IFFChunk thdr_IFFHeader;
  UBYTE  thdr_Version;      /* Version number of TCT1 file */
  UBYTE  thdr_Revision;     /* Revision number of TCT1 file */
  UWORD  thdr_Length;       /* Last row of track (default 63 (0x3F)) */
  UBYTE  thdr_Volume;       /* Track global volume */
  UBYTE  thdr_SubVolume;    /* Sub volume */
  BYTE   thdr_Panning;      /* Track panning */
  UBYTE  thdr_SubPanning;   /* Sub panning */
  BYTE   thdr_Transpose;    /* Transposing of track */
  UBYTE  thdr_CompFlags;    /* Compatibility flags (look bit def table) */
  UWORD  thdr_Flags;        /* Track flags, see bit defs */
  UWORD  thdr_Frames;       /* Number of frames per row (default 6) */
  UBYTE  thdr_SpeedMul;     /* Speed multiplier (allowing you e.g. 2/3) */
  UBYTE  thdr_SpeedDiv;     /* Speed divider (the divider for the multiplier) */
  UWORD  thdr_SPDSpeed;     /* Track SPD */
  UWORD  thdr_BPMTempo;     /* Track BPM tempo (number of rows of a beat). */
  UWORD  thdr_BPMSpeed;     /* Track BPM speed. */
}; /* IFF_THDR */

/* IFF-TCM1 THDR structure. Used by disk operations
  (e.g. by LoadTCModule() & SaveTCModule()) */

struct MIFF_THDR {
  struct IFFChunk mthdr_IFFHeader;
  UWORD  mthdr_Length;      /* Last row of track (default 63 (0x3F)) */
  UBYTE  mthdr_Volume;      /* Track global volume */
  UBYTE  mthdr_SubVolume;   /* Sub volume */
  BYTE   mthdr_Panning;     /* Track panning */
  UBYTE  mthdr_SubPan;      /* Sub panning */
  BYTE   mthdr_Transpose;   /* Transposing of track */
  UBYTE  mthdr_CompFlags;   /* Compatibility flags (look bit def table) */
  UWORD  mthdr_Flags;       /* Track flags, see bit defs */
  UWORD  mthdr_Frames;      /* Number of frames per row (default 6) */
  UBYTE  mthdr_SpeedMul;    /* Speed multiplier (allowing you e.g. 2/3) */
  UBYTE  mthdr_SpeedDiv;    /* Speed divider (the div for the multiplier) */
  UWORD  mthdr_SPDSpeed;    /* Track SPD */
  UWORD  mthdr_BPMTempo;    /* Track BPM tempo (number of rows of a beat). */
  UWORD  mthdr_BPMSpeed;    /* Track BPM speed. */
}; /* MIFF_THDR */

#define THDR_VERSION          1  /* Current version 1 */
#define THDR_REVISION         0  /* Current revision 0 */

#endif /* TUCOMPOSER_TRACK_H */
