#ifndef TUCOMPOSER_TUCOMPOSER_H
#define TUCOMPOSER_TUCOMPOSER_H
/*
**  $VER: tucomposer.h 31.22 (29.3.99)
**
**  Standard C header for tucomposer.library
**
**  (C) Copyright 1999 by Basty/Seasons
**      All Rights Reserved
*/

#ifndef EXEC_TYPES_H
#include <exec/types.h>
#endif /* EXEC_TYPES_H */

#ifndef EXEC_LIBRARIES_H
#include <exec/libraries.h>
#endif /* EXEC_LIBRARIES_H */

#ifndef EXEC_LISTS_H
#include <exec/lists.h>
#endif /* EXEC_LISTS_H */

#ifndef EXEC_NODES_H
#include <exec/nodes.h>
#endif /* EXEC_NODES_H */

#ifndef EXEC_SEMAPHORES_H
#include <exec/semaphores.h>
#endif /* EXEC_SEMAPHORES_H */

#ifndef EXEC_TASKS_H
#include <exec/tasks.h>
#endif /* EXEC_TASKS_H */

#ifndef LIBRARIES_DOS_H
#include <libraries/dos.h>
#endif /* LIBRARIES_DOS_H */

/* Bit definitions for peh_Flags: */

#define PEHB_SongEnd        0   /* Signal song ends only */
#define PEHB_Beginning      1   /* Signal before executing player code only
                                   useful if given exec code */

#define PEHF_SongEnd        (1<<PEHB_SongEnd)
#define PEHF_Beginning      (1<<PEHB_Beginning)

/* The library structure. */
struct TuComposerBase {
  struct Library tcb_LibNode;
  BPTR   tcb_SegList;           /* LoadSeg() array of the library */
  struct DosLibrary *tcb_DOSBase; /* dos.library base */
  struct IntuitionBase *tcb_IntBase; /* intuition.library base */
  APTR   tcb_MemPool;           /* Memory pool (OS3.0+) */
  struct SignalSemaphore *tcb_MemSigSem; /* Memory pool signal semaphore */
  struct TuComposerModule *tcb_PlayModule; /* Currently playing MOD or 0 */
  struct TuComposerSong *tcb_PlaySong; /* Currently playing song or zero */
  struct SoundDeviceInfo *tcb_PlaySoundDeviceBase; /* Currently playing
                                   sound device lib base */
  struct MixerAllocStruct *tcb_PlayMixerAlloc; /* Ptr to playing mixer
                                   alloc structure */
  WORD  *tcb_SineTable;         /* Ptr to sine table */
  UWORD *tcb_LinearFreqTable;   /* Ptr to linear frequency table */
  ULONG *tcb_FrequencyTable;    /* Ptr to frequency table */
  ULONG *tcb_OldSTTable;        /* Ptr to old SoundTracker tempos table */
  APTR  tcb_EffectsTable;       /* Ptr to player effects table */
  struct SynthTable *tcb_SynthInstrTable; /* Ptr to synth code instruction
                                   table */
  APTR  tcb_SynthExecTable;     /* Ptr to synth execution table */
  ULONG tcb_RandomSeed;         /* Current randomize seed */
    
  struct MinList tcb_ModuleList; /* Linked list of loaded MODs */
  struct MinList tcb_ExecList;  /* Linked list of signalling tasks on each
                                   tick. */
  struct MinList tcb_ModuleBuf; /* Buffered MODs (cut, copy, etc.) */
  struct MinList tcb_SongBuf;   /* Linked list of buffered songs. */
  struct MinList tcb_TrackBuf;  /* Linked list of buffered track. */
  struct MinList tcb_InstrBuf;  /* Linked list of buffered instruments. */
  struct MinList tcb_SampleBuf; /* Linked list of buffered samples. */
  struct MinList tcb_SynthBuf;  /* Linked list of buffered synthetics. */
  struct MinList tcb_WavesBuf;  /* Linked list of buffered waveforms. */
  struct MinList tcb_ExtCrunch; /* Linked list of external crunchers. */
  struct MinList tcb_ExtDevice; /* Linked list of external devices. */
  struct MinList tcb_ExtRecord; /* Linked list of external recorders. */
  struct MinList tcb_ExtMixer;  /* Linked list of external mixers. */
  struct MinList tcb_ExtPostProc; /* Linked list of external mixer post
                                   processors. */
  struct MinList tcb_ExtModule; /* Linked list of external modules. */
  struct MinList tcb_ExtSong;   /* Linked list of external songs. */
  struct MinList tcb_ExtTrack;  /* Linked list of external tracks. */
  struct MinList tcb_ExtInstr;  /* Linked list of external instruments. */
  struct MinList tcb_ExtSample; /* Linked list of external samples. */
  struct MinList tcb_ExtSynth;  /* Linked list of external synths. */
  struct SignalSemaphore *tcb_ModuleListSigSem; /* Module list signal sem. */
  struct SignalSemaphore *tcb_PlaySigSem; /* Playing access signal sem. */
  struct SignalSemaphore *tcb_ModuleSigSem; /* Module protection sig. sem. */
  struct SignalSemaphore *tcb_SongSigSem; /* Song protection sig. sem. */
  struct SignalSemaphore *tcb_TrackSigSem; /* Track protection sig. sem. */
  struct SignalSemaphore *tcb_InstrSigSem; /* Instrument protect sig. sem. */
  struct SignalSemaphore *tcb_SampleSigSem; /* Sample protection sig. sem. */
  struct SignalSemaphore *tcb_SynthSigSem; /* Synth protection sig. sem. */
  struct SignalSemaphore *tcb_WavesSigSem; /* Waveform protect sig. sem. */
  struct SignalSemaphore *tcb_ExternalSigSem; /* External libraries sem. */
}; /* TuComposerBase */

/* PlayExec structure for AddPlayExecHandler() & RemPlayExecHandler() */
struct PlayExecHandler {
  struct MinNode peh_Node;
  UBYTE  peh_Flags;             /* Special flags (see above) */
  BYTE   peh_Pri;               /* Sorting priority */
  union {
    struct Task *peh_Task;      /* Pointer to signalling task */
    void   (*peh_PlayExecCode)( struct TuComposerBase *TuCBase,
        struct TuComposerModule *ModuleStr,
        struct TuComposerSong *SongStr,
        APTR UserData );        /* If peh_SigBit is 0xFF, this is used */
  } peh_Action;
  UBYTE  peh_SigBit;            /* Signalling bit */
  UBYTE  peh_Pad;               /* Just for even alignment */
  APTR   peh_Data;              /* Any data value put in A0 when peh_Task
                                   points to any code to be executed */
  UBYTE  peh_Reserved[10];      /* Reserved for future expansions */
}; /* PlayExecHandler */

/* Date structure for Stamp2Date() */
struct StampDate {
  UBYTE  sds_Day;               /* Day (01 - 07) */
  UBYTE  sds_Month;             /* Month (01 - 12) */
  UWORD  sds_Year;              /* Year (0from 0 A.D.) */
  UBYTE  sds_TimeHours;         /* Hours (0 - 23) */
  UBYTE  sds_TimeMins;          /* Mins (0 - 59) */
  UBYTE  sds_TimeSecs;          /* Seconds (0 - 59) */
  UBYTE  sds_TimeHundreds;      /* 1/100 seconds (0 - 99) */
}; /* StampDate */

/* Unknown chunk structure. This structure is used by all chunks stated as
   unknown. */
struct UnknownChunk {
  struct MinNode uc_Node;
  UBYTE  uc_Identifier[4];      /* Unknown IFF-Chunk Identifier */
  UBYTE  uc_Length[4];          /* Unknown IFF-Chunk length */
  ULONG  uc_Data[0];            /* Here follows the unknown data */
}; /* UnknownChunks */

/* IFF-Header structure */
struct IFFHeader {
  UBYTE  ifh_FORMId[4];         /* FORM id */
  UBYTE  ifh_Length[4];         /* Length of IFF file from TypeId */
  UBYTE  ifh_TypeId[4];         /* Type id (TCM1 in this case) */
  UBYTE  ifh_Data[0];           /* Data, can also be byte boundary */
}; /* IFFHeader */

/* IFF Chunk structure */

struct IFFChunk {
  UBYTE  iff_Identifier[4];     /* IFF-Chunk identifier */
  UBYTE  iff_Length[4];         /* Length */
  UBYTE  iff_Data[0];           /* IFF-Chunk data (also byte boundary !) */
}; /* IFFChunk */

#define LINEAR_FINE_UP   0      /* Fine up linear frequence table */
#define LINEAR_FINE_DOWN 1      /* Fine down linear frequence table */
#define LINEAR_UP        2      /* Up linear frequence table */
#define LINEAR_DOWN      3      /* Down linear frequence table */

/* Global error codes */

#define ERROR_MODULE 0xFFFFFFF0 /* Module related errors */
#define ERROR_SONG   0xFFFFFFE0 /* Song related errors */
#define ERROR_TRACK  0xFFFFFFD0 /* Track related errors */
#define ERROR_POSITION 0xFFFFFFC0 /* Position/order list related errors */
#define ERROR_INSTR  0xFFFFFFB0 /* Instrument related errors */
#define ERROR_SAMPLE 0xFFFFFFA0 /* Sample related errors */
#define ERROR_SYNTH  0xFFFFFF90 /* Synthetic related errors */
#define ERROR_EXTLIB 0xFFFFFF80 /* External library related errors */

#define TuCName "tucomposer.library"
#define TuCVer 31
#define TuCRev 0

#ifndef TUCOMPOSER_MODULE_H
#include "tucomposer/module.h"
#endif /* TUCOMPOSER_MODULE_H */

#ifndef TUCOMPOSER_SONG_H
#include "tucomposer/song.h"
#endif /* TUCOMPOSER_SONG_H */

#ifndef TUCOMPOSER_EXTERNAL_EXTERNAL_H
#include "tucomposer/external/external.h"
#endif /* TUCOMPOSER_EXTERNAL_EXTERNAL_H */

#endif /* TUCOMPOSER_TUCOMPOSER_H */
