/* TuComposer OS independent code
**
** $VER: InternalLQMix.c 31.00 (3.8.99)
**
** (C) 1999 by Sebastian Vater (known as Basty/Seasons).
**
*/

#include <string.h>
#include <exec/types.h>
#include <exec/libraries.h>
#include <tucomposer/tucomposer.h>
#include <tucomposer/external/external.h>
#include <tucomposer/external/mixer.h>
#include <tucomposer/external/postproc.h>
#include <proto/exec.h>
#include <proto/tucomposer.h>
#include <clib/ext_devicebase_protos.h>
#include <clib/ext_postprocbase_protos.h>

struct LQMixerAllocStruct {
  struct MixerAllocStruct lqmas_AllocMixStruct;
  LONG  *lqmas_MixBuffer;
  ULONG lqmas_BuffSize;
  ULONG lqmas_MixBufSize;
  LONG  *lqmas_VolTable;
  struct ChannelInfo *lqmas_ChInfo;
  ULONG lqmas_Amplify;
  ULONG lqmas_MixRate;
  ULONG lqmas_MixFract;
  ULONG lqmas_CurrLeft;
  ULONG lqmas_CurrLeftFrac;
  ULONG lqmas_PassLength;
  ULONG lqmas_PassFrac;
  UWORD lqmas_Channels;
  UBYTE lqmas_Interpolation;
  UBYTE lqmas_Real16Bit;
  UBYTE lqmas_Stereo;
  UBYTE lqmas_PrevStereo;
}; /* LQMixerAllocStruct */

struct ChannelInfo {
  struct ChannelBlock {
    WORD *chi_SampleStartAdr;
    ULONG chi_SampleLen;
    ULONG chi_Offset;
    ULONG chi_Fraction;
    ULONG chi_Advance;
    ULONG chi_AdvFract;
    void (*chi_MixRoutine)( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, UWORD Length );
    ULONG chi_EndOffset;
    ULONG chi_RestartOffset;
    ULONG chi_Repeat;
    ULONG chi_RepLen;
    ULONG chi_CountRestart;
    ULONG chi_Counted;
    ULONG chi_SampleRate;
    LONG *chi_VolTable_L;
    LONG *chi_VolTable_R;
    ULONG chi_Multiply_L;
    ULONG chi_Divider;
    ULONG chi_Multiply_R;
    void (*chi_AltMixRoutine)( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, UWORD Length );
    UBYTE chi_BitDepth;
    UBYTE chi_Flags;
    UBYTE chi_Volume;
    UBYTE chi_Panning;
  } chi_Mix;
  struct ChannelBlock chi_New;
}; /* ChannelInfo */

#ifdef __cplusplus
extern "C" {
#endif

static struct TuComposerExtLibBase *InternalLQMixOpenExtLib ( struct TuComposerExtLibBase *ExtLibBase );
static BPTR InternalLQMixCloseExtLib ( struct TuComposerExtLibBase *ExtLibBase );
static BPTR ExtLib ( void );
static ULONG InternalMixerUpdate ( union FlagPass *FlagPassStr,
    unsigned long FlagPassType, unsigned long FlagNum );
static struct MixerAllocStruct *InternalMixerAlloc ( void );
static ULONG InternalMixerFree ( struct MixerAllocStruct *MixAllocStr );
static ULONG InternalMixerInit ( struct MixerAllocStruct *MixAllocStr,
    struct SoundDeviceInfo *ExtSoundDevStr, union FlagPass *FlagPassStr );
static ULONG InternalMixerChangeRate ( struct MixerAllocStruct *MixAllocStr,
    unsigned long NewMixRate );
static ULONG InternalMixerChangeTempo ( struct MixerAllocStruct *MixAllocStr,
    unsigned long NewTempo );
static ULONG InternalMixerChangeVolTable ( struct MixerAllocStruct *MixAllocStr,
    unsigned long Amplify, unsigned long LeftVol, unsigned long RightVol,
    unsigned long NumChannels );
static void InternalMixerGetChannel ( struct MixerAllocStruct *MixAllocStr,
    struct MixerStruct *MixStr, unsigned long Channel );
static void InternalMixerSetChannel ( struct MixerAllocStruct *MixAllocStr,
    struct MixerStruct *MixStr, unsigned long Channel );
static void InternalMixerSetChannelVolPanPitch ( struct MixerAllocStruct
    *MixAllocStr, struct MixerStruct *MixStr, unsigned long Channel );
static void InternalMixerSetChannelPosRepFlags ( struct MixerAllocStruct
    *MixAllocStr, struct MixerStruct *MixStr, unsigned long Channel );
static void InternalMixerDoMix ( struct MixerAllocStruct *MixAllocStr,
    LONG *MixBuffer );

static void MixSample ( struct LQMixerAllocStruct *LQMixAllocStr, LONG *MixBuffer, ULONG Length );

static void SetSmpMixRate ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, ULONG SampleRate );
static void AdvOk ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo );

static void mstv_Skip ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, UWORD Volume, UWORD Panning );
static void mstv_Stereo8 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, UWORD Volume, UWORD Panning );
static void mstv_Stereo8L ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, UWORD Volume, UWORD Panning );
static void mstv_Stereo8R ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, UWORD Volume, UWORD Panning );
static void mstv_Stereo8C ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, UWORD Volume, UWORD Panning );
static void mstv_Stereo16 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, UWORD Volume, UWORD Panning );
static void mstv_Stereo16L ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, UWORD Volume, UWORD Panning );
static void mstv_Stereo16R ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, UWORD Volume, UWORD Panning );
static void mstv_Stereo16C ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, UWORD Volume, UWORD Panning );
static void mstv_Stereo32 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, UWORD Volume, UWORD Panning );
static void mstv_Stereo32L ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, UWORD Volume, UWORD Panning );
static void mstv_Stereo32R ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, UWORD Volume, UWORD Panning );
static void mstv_Stereo32C ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, UWORD Volume, UWORD Panning );

static void Skip ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void Skipb ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );

static void MIX_Mono8 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIXb_Mono8 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIX_Mono16 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIXb_Mono16 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIX_Mono32 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIXb_Mono32 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIX_MonoX ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIXb_MonoX ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIX_Mono168 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIXb_Mono168 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIX_Mono328 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIXb_Mono328 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIX_MonoX8 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIXb_MonoX8 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );

static void MIX_Stereo8 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIXb_Stereo8 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIX_Stereo16 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIXb_Stereo16 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIX_Stereo32 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIXb_Stereo32 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIX_StereoX ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIXb_StereoX ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIX_Stereo168 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIXb_Stereo168 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIX_Stereo328 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIXb_Stereo328 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIX_StereoX8 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIXb_StereoX8 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );

static void MIX_Stereo8L ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIXb_Stereo8L ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIX_Stereo16L ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIXb_Stereo16L ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIX_Stereo32L ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIXb_Stereo32L ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIX_StereoXL ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIXb_StereoXL ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIX_Stereo168L ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIXb_Stereo168L ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIX_Stereo328L ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIXb_Stereo328L ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIX_StereoX8L ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIXb_StereoX8L ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );

static void MIX_Stereo8R ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIXb_Stereo8R ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIX_Stereo16R ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIXb_Stereo16R ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIX_Stereo32R ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIXb_Stereo32R ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIX_StereoXR ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIXb_StereoXR ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIX_Stereo168R ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIXb_Stereo168R ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIX_Stereo328R ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIXb_Stereo328R ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIX_StereoX8R ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIXb_StereoX8R ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );

static void MIX_Stereo8C ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIXb_Stereo8C ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIX_Stereo16C ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIXb_Stereo16C ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIX_Stereo32C ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIXb_Stereo32C ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIX_StereoXC ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIXb_StereoXC ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIX_Stereo168C ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIXb_Stereo168C ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIX_Stereo328C ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIXb_Stereo328C ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIX_StereoX8C ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIXb_StereoX8C ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );

static void MIX_Stereo8Sur ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIXb_Stereo8Sur ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIX_Stereo16Sur ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIXb_Stereo16Sur ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIX_Stereo32Sur ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIXb_Stereo32Sur ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIX_StereoXSur ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIXb_StereoXSur ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIX_Stereo168Sur ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIXb_Stereo168Sur ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIX_Stereo328Sur ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIXb_Stereo328Sur ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIX_StereoX8Sur ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
static void MIXb_StereoX8Sur ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );

#ifdef __cplusplus
}
#endif

extern struct MixerInfo InternalMixerBase;
extern struct InternalMixerFlagStr InternalMixerSaveFlags;
extern STRPTR SaveButton2 [];

static UBYTE InternalMixerName[] = "internal_lqmix.library\0";
static UBYTE InternalMixerLoadID[] = "Low Quality Mixer\0";
static UBYTE InternalMixerIDString[] = "Internal Low Quality Mixer Lib Version 1.00 (v31) (03 Aug 99) by Basty/Seasons.\0";

struct TuComposerExtList InternalMixer =
{
  {NULL, NULL},
  EXTLF_Intern|EXTLF_Loader|EXTLF_Saver, 0,
  (struct TuComposerExtLibBase *) &InternalMixerBase,
  NULL,
  (STRPTR) &InternalMixerName,
  (STRPTR) &InternalMixerLoadID,
  NULL,
  (struct TuComposerExtFlagStr *) &InternalMixerSaveFlags,
  0
};

#include "InternalLQMix_library.h"

static UBYTE SaveText0[] = "True 16-bit mixing\0";
static UBYTE SaveText1[] = "Stereo output\0";
static UBYTE SaveText2[] = "Interpolation\0";
static UBYTE SaveText21[] = "Disabled\0";
static UBYTE SaveText22[] = "Fast\0";
static UBYTE SaveText23[] = "Effective\0";
static UBYTE SaveText24[] = "Full\0";

static struct InternalMixerFlagStr {
  UWORD  tefs_NumEntries;
  UWORD  tefs_StructSize;
  union FlagEntryBooleanStruct tefs_Flag0;
  union FlagEntryBooleanStruct tefs_Flag1;
  union FlagEntryButtonStruct tefs_Flag2;
} InternalMixerSaveFlags =
{
  3, sizeof (InternalMixerSaveFlags),
  {(STRPTR) &SaveText0, EXTFLAG_BOOLEAN, TEFSF_Update, 0, 0L, 0L, 0xFFFFFFFFL},
  {(STRPTR) &SaveText1, EXTFLAG_BOOLEAN, TEFSF_Update, 0, 0L, 0L, 0xFFFFFFFFL},
  {(STRPTR *) &SaveButton2, EXTFLAG_BUTTON, TEFSF_Update, 1+4, 0L, 0L, 1L}
};

static STRPTR SaveButton2[] =
{
  SaveText2,
  SaveText21,
  SaveText22,
  SaveText23,
  SaveText24
};

static APTR Mixr_Skip[] =
{
  mstv_Skip, mstv_Skip, mstv_Skip, Skip, Skip, Skip, Skip, Skipb, Skipb, Skipb, Skipb
};

static APTR Mixr_Mono[] =
{
  mstv_Stereo8C, mstv_Stereo16C, mstv_Stereo32C, MIX_Mono8, MIX_Mono16, MIX_Mono32, MIX_MonoX, MIXb_Mono8, MIXb_Mono16, MIXb_Mono32, MIXb_MonoX
};

static APTR Mixr_Stereo[] =
{
  mstv_Stereo8, mstv_Stereo16, mstv_Stereo32, MIX_Stereo8, MIX_Stereo16, MIX_Stereo32, MIX_StereoX, MIXb_Stereo8, MIXb_Stereo16, MIXb_Stereo32, MIXb_StereoX
};

static APTR Mixr_StereoL[] =
{
  mstv_Stereo8L, mstv_Stereo16L, mstv_Stereo32L, MIX_Stereo8L, MIX_Stereo16L, MIX_Stereo32L, MIX_StereoXL, MIXb_Stereo8L, MIXb_Stereo16L, MIXb_Stereo32L, MIXb_StereoXL
};

static APTR Mixr_StereoR[] =
{
  mstv_Stereo8R, mstv_Stereo16R, mstv_Stereo32R, MIX_Stereo8R, MIX_Stereo16R, MIX_Stereo32R, MIX_StereoXR, MIXb_Stereo8R, MIXb_Stereo16R, MIXb_Stereo32R, MIXb_StereoXR
};

static APTR Mixr_StereoC[] =
{
  mstv_Stereo8C, mstv_Stereo16C, mstv_Stereo32C, MIX_Stereo8C, MIX_Stereo16C, MIX_Stereo32C, MIX_StereoXC, MIXb_Stereo8C, MIXb_Stereo16C, MIXb_Stereo32C, MIXb_StereoXC
};

static APTR Mixr_StereoSur[] =
{
  mstv_Stereo8C, mstv_Stereo16C, mstv_Stereo32C, MIX_Stereo8Sur, MIX_Stereo16Sur, MIX_Stereo32Sur, MIX_StereoXSur, MIXb_Stereo8Sur, MIXb_Stereo16Sur, MIXb_Stereo32Sur, MIXb_StereoXSur
};

static APTR Mixr_Skip168[] =
{
  mstv_Skip, mstv_Skip, mstv_Skip, Skip, Skip, Skip, Skip, Skipb, Skipb, Skipb, Skipb
};

static APTR Mixr_Mono168[] =
{
  mstv_Stereo8C, mstv_Stereo8C, mstv_Stereo8C, MIX_Mono8, MIX_Mono168, MIX_Mono328, MIX_MonoX8, MIXb_Mono8, MIXb_Mono168, MIXb_Mono328, MIXb_MonoX8
};

static APTR Mixr_Stereo168[] =
{
  mstv_Stereo8, mstv_Stereo8, mstv_Stereo8, MIX_Stereo8, MIX_Stereo168, MIX_Stereo328, MIX_StereoX8, MIXb_Stereo8, MIXb_Stereo168, MIXb_Stereo328, MIXb_StereoX8
};

static APTR Mixr_StereoL168[] =
{
  mstv_Stereo8L, mstv_Stereo8L, mstv_Stereo8L, MIX_Stereo8L, MIX_Stereo168L, MIX_Stereo328L, MIX_StereoX8L, MIXb_Stereo8L, MIXb_Stereo168L, MIXb_Stereo328, MIXb_StereoX8L
};

static APTR Mixr_StereoR168[] =
{
  mstv_Stereo8R, mstv_Stereo8R, mstv_Stereo8R, MIX_Stereo8R, MIX_Stereo168R, MIX_Stereo328R, MIX_StereoX8R, MIXb_Stereo8R, MIXb_Stereo168R, MIXb_Stereo328R, MIXb_StereoX8R
};

static APTR Mixr_StereoC168[] =
{
  mstv_Stereo8C, mstv_Stereo8C, mstv_Stereo8C, MIX_Stereo8C, MIX_Stereo168C, MIX_Stereo328C, MIX_StereoX8C, MIXb_Stereo8C, MIXb_Stereo168C, MIXb_Stereo328C, MIXb_StereoX8C
};

static APTR Mixr_StereoSur168[] =
{
  mstv_Stereo8C, mstv_Stereo8C, mstv_Stereo8C, MIX_Stereo8Sur, MIX_Stereo168Sur, MIX_Stereo328Sur, MIX_StereoX8Sur, MIXb_Stereo8Sur, MIXb_Stereo168Sur, MIXb_Stereo328Sur, MIXb_StereoX8Sur
};

static struct TuComposerExtLibBase *InternalLQMixOpenExtLib ( struct TuComposerExtLibBase *ExtLibBase )
{
  InternalMixerBase.mix_Library.extb_LibNode.lib_OpenCnt++;

  return ( (struct TuComposerExtLibBase *) &InternalMixerBase );
}

static BPTR InternalLQMixCloseExtLib ( struct TuComposerExtLibBase *ExtLibBase )
{
  InternalMixerBase.mix_Library.extb_LibNode.lib_OpenCnt--;

  return ( NULL );
}

static BPTR ExtLib ( void )
{
  return ( NULL );
}

static ULONG InternalMixerUpdate ( union FlagPass *FlagPassStr,
    unsigned long FlagPassType, unsigned long FlagNum )
{
  return ( 0L );
}

static struct MixerAllocStruct *InternalMixerAlloc ( void )
{
  struct LQMixerAllocStruct *MixAllocStr;

  if ( MixAllocStr = (struct LQMixerAllocStruct *) AllocPMem ( sizeof (struct LQMixerAllocStruct )))
  {
    MixAllocStr->lqmas_AllocMixStruct.mas_MixLibBase = (struct MixerInfo *) &InternalMixerBase;

    if (!( MixAllocStr->lqmas_VolTable = (LONG *) AllocPMem ( (256 * 256 * sizeof (LONG)) )))
    {
      FreePMem ( MixAllocStr, sizeof (struct LQMixerAllocStruct ));

      return ( NULL );
    }

    MixAllocStr->lqmas_AllocMixStruct.mas_Task = FindTask ( NULL );

    InitNodeList ( (struct List *) &(MixAllocStr->lqmas_AllocMixStruct.mas_PostProcs));
  }

  return ( (struct MixerAllocStruct *) MixAllocStr );
}

static ULONG InternalMixerFree ( struct MixerAllocStruct *MixAllocStr )
{
  struct LQMixerAllocStruct *LQMixAllocStr = (struct LQMixerAllocStruct *) MixAllocStr;

  if ( LQMixAllocStr )
  {
    if ( LQMixAllocStr->lqmas_ChInfo )
      FreePMem ( LQMixAllocStr->lqmas_ChInfo, LQMixAllocStr->lqmas_AllocMixStruct.mas_MaxChannels * sizeof (struct ChannelInfo ));

    if ( LQMixAllocStr->lqmas_MixBuffer )
      FreePMem ( LQMixAllocStr->lqmas_MixBuffer, LQMixAllocStr->lqmas_MixBufSize );

    if ( LQMixAllocStr->lqmas_VolTable )
      FreePMem ( LQMixAllocStr->lqmas_VolTable, (256 * 256 * sizeof (LONG)) );

    FreePMem ( LQMixAllocStr, sizeof (struct LQMixerAllocStruct ));

    return ( 0L );
  }

  return ( ERROR_EXTLIB_MIXNOALLOC );
}

static ULONG InternalMixerInit ( struct MixerAllocStruct *MixAllocStr,
    struct SoundDeviceInfo *ExtSoundDevStr, union FlagPass *FlagPassStr )
{
  struct LQMixerAllocStruct *LQMixAllocStr = (struct LQMixerAllocStruct *) MixAllocStr;
  struct ChannelInfo *ChInfo;
  ULONG MixBufSize = LQMixAllocStr->lqmas_AllocMixStruct.mas_MixBufSize, MixBufMemSize, ChannelFreq;
  UWORD Channels = LQMixAllocStr->lqmas_AllocMixStruct.mas_MaxChannels, i;
  UBYTE Stereo;
  unsigned long long PassValue;

  if ( (Stereo = FlagPassStr[1].flg_Value) )
  {
    if (!( ExtSoundDevStr->sdi_Flags & SDIF_Stereo ))
      Stereo = 0;
  }

  if ( LQMixAllocStr->lqmas_AllocMixStruct.mas_SoundDevLibBase != ExtSoundDevStr )
  {
    if ( LQMixAllocStr->lqmas_AllocMixStruct.mas_Flags & MASF_Mixing )
      ExternalDeviceStopBase ( LQMixAllocStr->lqmas_AllocMixStruct.mas_SoundDevLibBase );

    LQMixAllocStr->lqmas_AllocMixStruct.mas_SoundDevLibBase = ExtSoundDevStr;
  }

  LQMixAllocStr->lqmas_AllocMixStruct.mas_MixRate = ExternalDeviceCheckRateBase ( LQMixAllocStr->lqmas_AllocMixStruct.mas_MixRate,ExtSoundDevStr );
  LQMixAllocStr->lqmas_MixFract = ExternalDeviceCheckRateFractionBase ( LQMixAllocStr->lqmas_AllocMixStruct.mas_MixRate, ExtSoundDevStr );

  if ( (Channels != LQMixAllocStr->lqmas_Channels) || (LQMixAllocStr->lqmas_AllocMixStruct.mas_VolumeBoost != LQMixAllocStr->lqmas_Amplify) || (Stereo != LQMixAllocStr->lqmas_Stereo ))
  {
    LONG *VolTable = LQMixAllocStr->lqmas_VolTable;
    ULONG VolTabMul = 0L, VolTabDiv = LQMixAllocStr->lqmas_AllocMixStruct.mas_MaxChannels << 8;
    ULONG Amplify = LQMixAllocStr->lqmas_AllocMixStruct.mas_VolumeBoost;
    UBYTE i = 0, j = 0;

    LQMixAllocStr->lqmas_Amplify = Amplify;
    LQMixAllocStr->lqmas_PrevStereo = LQMixAllocStr->lqmas_Stereo;
    LQMixAllocStr->lqmas_Stereo = Stereo;

    do
    {
      do
      {
        LONG VolTabVal = (BYTE) j << 8;

        *VolTable++ = (long long) VolTabVal * VolTabMul / VolTabDiv;
      } while ( ++j );

      VolTabMul += Amplify;

    } while ( ++i );
  }

  if ( LQMixAllocStr->lqmas_AllocMixStruct.mas_Flags & MASF_Mixing )
    ExternalDeviceStopBase ( ExtSoundDevStr );

  if ( (LQMixAllocStr->lqmas_ChInfo == NULL) || (Channels != LQMixAllocStr->lqmas_Channels) )
  {
    struct ChannelInfo *OldChInfo;

    if (!( ChInfo = (struct ChannelInfo *) AllocPMem ( Channels * sizeof (struct ChannelInfo ))))
      return ( ERROR_EXTLIB_MIXNOALLOC );

    if ( (OldChInfo = LQMixAllocStr->lqmas_ChInfo) )
    {
      UWORD CopyCh = Channels;
      UWORD OldCh = LQMixAllocStr->lqmas_Channels;

      if ( CopyCh > OldCh )
        CopyCh = OldCh;

      memcpy ( ChInfo, OldChInfo, CopyCh * sizeof (struct ChannelInfo ));

      FreePMem ( OldChInfo, OldCh * sizeof (struct ChannelInfo ));
    }

    LQMixAllocStr->lqmas_ChInfo = ChInfo;
    LQMixAllocStr->lqmas_Channels = Channels;
  }

  LQMixAllocStr->lqmas_AllocMixStruct.mas_MaxChannels = LQMixAllocStr->lqmas_Channels;

  MixBufMemSize = MixBufSize << 2;

  if ( Stereo )
    MixBufMemSize += MixBufMemSize;

  if ( (LQMixAllocStr->lqmas_MixBuffer == NULL) || (LQMixAllocStr->lqmas_MixBufSize != MixBufMemSize) )
  {
    LONG *MixBuffer;

    if (!( MixBuffer = (LONG *) AllocPMem ( MixBufMemSize )))
    {
      if ( LQMixAllocStr->lqmas_ChInfo )
        FreePMem ( LQMixAllocStr->lqmas_ChInfo, Channels * sizeof (struct ChannelInfo ));

      return ( ERROR_EXTLIB_MIXNOALLOC );
    }

    if ( LQMixAllocStr->lqmas_MixBuffer )
      FreePMem ( LQMixAllocStr->lqmas_MixBuffer, LQMixAllocStr->lqmas_MixBufSize );

    LQMixAllocStr->lqmas_MixBufSize = MixBufMemSize;

    LQMixAllocStr->lqmas_MixBuffer = MixBuffer;
    LQMixAllocStr->lqmas_BuffSize = MixBufSize;

    ExtSoundDevStr->sdi_ChannelBufLen = MixBufSize;
  }

  LQMixAllocStr->lqmas_AllocMixStruct.mas_MixBufSize = LQMixAllocStr->lqmas_BuffSize;

  ExtSoundDevStr->sdi_ChannelBuf = LQMixAllocStr->lqmas_MixBuffer;

  LQMixAllocStr->lqmas_AllocMixStruct.mas_MixBuf = LQMixAllocStr->lqmas_MixBuffer;

  ChannelFreq = LQMixAllocStr->lqmas_AllocMixStruct.mas_MixRate;
  LQMixAllocStr->lqmas_MixRate = ChannelFreq;
  ExtSoundDevStr->sdi_ChannelFreq = ChannelFreq;

  ExtSoundDevStr->sdi_ChannelFlags &= ~SDICF_Stereo;
  LQMixAllocStr->lqmas_AllocMixStruct.mas_Flags &= ~MASF_Stereo;

  if ( Stereo )
  {
    ExtSoundDevStr->sdi_ChannelFlags |= SDICF_Stereo;
    LQMixAllocStr->lqmas_AllocMixStruct.mas_Flags |= MASF_Stereo;
  }

  LQMixAllocStr->lqmas_AllocMixStruct.mas_SoundDevLibBase = ExtSoundDevStr;

  ChannelFreq *= 10;
  PassValue = ((unsigned long long) ChannelFreq << 16) + ((unsigned long long) LQMixAllocStr->lqmas_MixFract >> 16);

  LQMixAllocStr->lqmas_PassLength = (unsigned long long) PassValue / LQMixAllocStr->lqmas_AllocMixStruct.mas_Tempo;
  LQMixAllocStr->lqmas_PassFrac = (((unsigned long long) PassValue % LQMixAllocStr->lqmas_AllocMixStruct.mas_Tempo) << 32) / LQMixAllocStr->lqmas_AllocMixStruct.mas_Tempo;

  LQMixAllocStr->lqmas_Real16Bit = 0;

  if ( FlagPassStr[0].flg_Value != 0L )
    LQMixAllocStr->lqmas_Real16Bit = -1;

  LQMixAllocStr->lqmas_Interpolation = FlagPassStr[2].flg_Value - 1;

  ChInfo = LQMixAllocStr->lqmas_ChInfo;

  for (i = Channels; i > 0; i--)
  {
    SetSmpMixRate ( LQMixAllocStr, ChInfo, ChInfo->chi_Mix.chi_SampleRate );

    ChInfo++;
  }

  if ( LQMixAllocStr->lqmas_AllocMixStruct.mas_Flags & MASF_Mixing )
    ExternalDevicePlayBase ( (struct MixerAllocStruct *) LQMixAllocStr, NULL, ExtSoundDevStr );

  return ( 0L );
}
static ULONG InternalMixerChangeRate ( struct MixerAllocStruct *MixAllocStr,
    unsigned long NewMixRate )
{
  struct LQMixerAllocStruct *LQMixAllocStr = (struct LQMixerAllocStruct *) MixAllocStr;
  struct PostProcAllocStruct *PostAllocStr;
  struct SoundDeviceInfo *ExtSoundDevStr;
  ULONG MixRate, MixFrac;

  LQMixAllocStr->lqmas_AllocMixStruct.mas_MixRate = NewMixRate;

  if ( LQMixAllocStr->lqmas_AllocMixStruct.mas_Flags & MASF_Mixing )
  {
    ExtSoundDevStr = LQMixAllocStr->lqmas_AllocMixStruct.mas_SoundDevLibBase;

    MixRate = ExternalDeviceCheckRateBase ( NewMixRate, ExtSoundDevStr );
    MixFrac = ExternalDeviceCheckRateFractionBase ( NewMixRate, ExtSoundDevStr );

    if ( (ExternalDeviceUpdateRateBase ( MixRate, ExtSoundDevStr ) == 0L) && (LQMixAllocStr->lqmas_MixRate != MixRate) )
    {
      struct ChannelInfo *ChInfo = LQMixAllocStr->lqmas_ChInfo;
      UWORD i;

      LQMixAllocStr->lqmas_MixRate = MixRate;
      LQMixAllocStr->lqmas_MixFract = MixFrac;

      InternalMixerChangeTempo ( (struct MixerAllocStruct *) LQMixAllocStr, LQMixAllocStr->lqmas_AllocMixStruct.mas_Tempo );

      for (i = LQMixAllocStr->lqmas_Channels; i > 0; i--)
      {
        ChInfo->chi_Mix.chi_Advance = ChInfo->chi_Mix.chi_SampleRate / MixRate;
        ChInfo->chi_Mix.chi_AdvFract = (((unsigned long long) ChInfo->chi_Mix.chi_SampleRate % MixRate) << 32) / MixRate;

        ChInfo->chi_New.chi_Advance = ChInfo->chi_New.chi_SampleRate / MixRate;
        ChInfo->chi_Mix.chi_AdvFract = (((unsigned long long) ChInfo->chi_New.chi_SampleRate % MixRate) << 32) / MixRate;

        ChInfo++;
      }
    }
  }

  PostAllocStr = (struct PostProcAllocStruct *) &(MixAllocStr->mas_PostProcs.mlh_Head);

  for (;;)
  {
    PostAllocStr = (struct PostProcAllocStruct *) PostAllocStr->pas_Node.mln_Succ;

    if (!( PostAllocStr->pas_Node.mln_Succ ))
      break;

    if ( PostAllocStr->pas_Flags & PASF_Update )
      ExternalPostProcChangeBase ( PostAllocStr, (struct MixerAllocStruct *) LQMixAllocStr, PostAllocStr->pas_PostProcLibBase );
  }

  return ( 0L );
}

static ULONG InternalMixerChangeTempo ( struct MixerAllocStruct *MixAllocStr,
    unsigned long NewTempo )
{
  struct LQMixerAllocStruct *LQMixAllocStr = (struct LQMixerAllocStruct *) MixAllocStr;
  ULONG ChannelFreq;
  unsigned long long PassValue;

  LQMixAllocStr->lqmas_AllocMixStruct.mas_Tempo = NewTempo;

  ChannelFreq = LQMixAllocStr->lqmas_MixRate * 10;
  PassValue = ((unsigned long long) ChannelFreq << 16) + ((unsigned long long) LQMixAllocStr->lqmas_MixFract >> 16);

  LQMixAllocStr->lqmas_PassLength = (unsigned long long) PassValue / LQMixAllocStr->lqmas_AllocMixStruct.mas_Tempo;
  LQMixAllocStr->lqmas_PassFrac = (((unsigned long long) PassValue % LQMixAllocStr->lqmas_AllocMixStruct.mas_Tempo) << 32) / LQMixAllocStr->lqmas_AllocMixStruct.mas_Tempo;

  return ( 0L );
}

static ULONG InternalMixerChangeVolTable ( struct MixerAllocStruct *MixAllocStr,
    unsigned long Amplify, unsigned long LeftVol, unsigned long RightVol,
    unsigned long NumChannels )
{
  struct LQMixerAllocStruct *LQMixAllocStr = (struct LQMixerAllocStruct *) MixAllocStr;
  struct ChannelInfo *ChInfo;
  struct ChannelInfo *OldChInfo = LQMixAllocStr->lqmas_ChInfo;
  ULONG OldChannels, i;

  if ( (OldChannels = LQMixAllocStr->lqmas_Channels) != NumChannels )
  {
    if (!( ChInfo = (struct ChannelInfo *) AllocPMem ( NumChannels * sizeof (struct ChannelInfo ))))
      return ( ERROR_EXTLIB_MIXNOALLOC );
  }

  LQMixAllocStr->lqmas_AllocMixStruct.mas_VolumeBoost = Amplify;

  LQMixAllocStr->lqmas_AllocMixStruct.mas_LeftVolume = LeftVol;
  LQMixAllocStr->lqmas_AllocMixStruct.mas_RightVolume = RightVol;

  LQMixAllocStr->lqmas_AllocMixStruct.mas_MaxChannels = NumChannels;

  if ( (OldChannels != NumChannels) || (LQMixAllocStr->lqmas_Amplify != Amplify) )
  {
    LONG *VolTable = LQMixAllocStr->lqmas_VolTable;
    ULONG VolTabMul = 0L, VolTabDiv = NumChannels << 8;
    UBYTE i = 0, j = 0;

    LQMixAllocStr->lqmas_Amplify = Amplify;

    do
    {
      do
      {
        LONG VolTabVal = (BYTE) j << 8;

        *VolTable++ = (long long) VolTabVal * VolTabMul / VolTabDiv;
      } while ( ++j );

      VolTabMul += Amplify;

    } while ( ++i );
  }

  if ( OldChannels != NumChannels )
  {
    ULONG CopyChannels = OldChannels;

    Disable ();

    if ( CopyChannels > NumChannels )
      CopyChannels = NumChannels;

    memcpy ( ChInfo, OldChInfo, CopyChannels * sizeof (struct ChannelInfo ));

    LQMixAllocStr->lqmas_ChInfo = ChInfo;
    LQMixAllocStr->lqmas_Channels = NumChannels;

    Enable ();

    FreePMem ( OldChInfo, OldChannels * sizeof (struct ChannelInfo ));
  }

  Disable ();

  ChInfo = LQMixAllocStr->lqmas_ChInfo;

  for (i = NumChannels; i > 0; i--)
  {
    SetSmpMixRate ( LQMixAllocStr, ChInfo, ChInfo->chi_Mix.chi_SampleRate );

    ChInfo++;
  }

  Enable ();

  return ( 0L );
}

static void InternalMixerGetChannel ( struct MixerAllocStruct *MixAllocStr,
    struct MixerStruct *MixStr, unsigned long Channel )
{
  struct LQMixerAllocStruct *LQMixAllocStr = (struct LQMixerAllocStruct *) MixAllocStr;
  struct ChannelInfo *ChInfo;

  if ( Channel < LQMixAllocStr->lqmas_Channels )
  {
    ChInfo = LQMixAllocStr->lqmas_ChInfo + Channel;

    MixStr->ms_SamplePos = ChInfo->chi_Mix.chi_Offset;

    MixStr->ms_BitDepth = ChInfo->chi_Mix.chi_BitDepth;
    MixStr->ms_Flags = ChInfo->chi_Mix.chi_Flags;
    MixStr->ms_Volume = ChInfo->chi_Mix.chi_Volume;
    MixStr->ms_Panning = ChInfo->chi_Mix.chi_Panning;

    MixStr->ms_SampleData = ChInfo->chi_Mix.chi_SampleStartAdr;
    MixStr->ms_SampleLen = ChInfo->chi_Mix.chi_SampleLen;

    MixStr->ms_Repeat = ChInfo->chi_Mix.chi_Repeat;
    MixStr->ms_RepLen = ChInfo->chi_Mix.chi_RepLen;

    MixStr->ms_RepCount = ChInfo->chi_Mix.chi_CountRestart;
    MixStr->ms_RepCounted = ChInfo->chi_Mix.chi_Counted;

    MixStr->ms_SampleRate = ChInfo->chi_Mix.chi_SampleRate;
  }
}

static void InternalMixerSetChannel ( struct MixerAllocStruct *MixAllocStr,
    struct MixerStruct *MixStr, unsigned long Channel )
{
  struct LQMixerAllocStruct *LQMixAllocStr = (struct LQMixerAllocStruct *) MixAllocStr;
  struct ChannelInfo *ChInfo;

  if ( Channel < LQMixAllocStr->lqmas_Channels )
  {
    ULONG Repeat, RepLen;

    ChInfo = LQMixAllocStr->lqmas_ChInfo + Channel;

    ChInfo->chi_New.chi_SampleStartAdr = NULL;

    if ( MixStr->ms_Flags & MSF_Synth )
    {
      ChInfo->chi_Mix.chi_Flags |= MSF_Synth;

      (BYTE *) ChInfo += sizeof (struct ChannelBlock);
    }

    ChInfo->chi_Mix.chi_Offset = MixStr->ms_SamplePos;
    ChInfo->chi_Mix.chi_Fraction = 0L;

    ChInfo->chi_Mix.chi_BitDepth = MixStr->ms_BitDepth;
    ChInfo->chi_Mix.chi_Flags = MixStr->ms_Flags;
    ChInfo->chi_Mix.chi_Volume = MixStr->ms_Volume;
    ChInfo->chi_Mix.chi_Panning = MixStr->ms_Panning;

    ChInfo->chi_Mix.chi_SampleStartAdr = MixStr->ms_SampleData;
    ChInfo->chi_Mix.chi_SampleLen = MixStr->ms_SampleLen;

    Repeat = MixStr->ms_Repeat;
    RepLen = MixStr->ms_RepLen;

    ChInfo->chi_Mix.chi_Repeat = Repeat;
    ChInfo->chi_Mix.chi_RepLen = RepLen;

    if (!( ChInfo->chi_Mix.chi_Flags & MSF_Loop ))
    {
      Repeat = MixStr->ms_SampleLen;
      RepLen = 0L;
    }

    Repeat += RepLen;

    if ( ChInfo->chi_Mix.chi_Flags & MSF_Backwards )
    {
      Repeat -= RepLen;

      if (!( ChInfo->chi_Mix.chi_Flags & MSF_Loop ))
        Repeat = -1L;
    }

    ChInfo->chi_Mix.chi_EndOffset = Repeat;
    ChInfo->chi_Mix.chi_RestartOffset = RepLen;

    ChInfo->chi_Mix.chi_CountRestart = MixStr->ms_RepCount;
    ChInfo->chi_Mix.chi_Counted = MixStr->ms_RepCounted;

    SetSmpMixRate ( LQMixAllocStr, ChInfo, MixStr->ms_SampleRate );
  }
}

static void InternalMixerSetChannelVolPanPitch ( struct MixerAllocStruct
    *MixAllocStr, struct MixerStruct *MixStr, unsigned long Channel )
{
  struct LQMixerAllocStruct *LQMixAllocStr = (struct LQMixerAllocStruct *) MixAllocStr;
  struct ChannelInfo *ChInfo;

  if ( Channel < LQMixAllocStr->lqmas_Channels )
  {
    ChInfo = LQMixAllocStr->lqmas_ChInfo + Channel;

    if ( (ChInfo->chi_Mix.chi_Volume == MixStr->ms_Volume) && (ChInfo->chi_Mix.chi_Panning == MixStr->ms_Panning) )
    {
      ULONG SmpRate = MixStr->ms_SampleRate, MixRate = LQMixAllocStr->lqmas_MixRate, TmpStore;

      ChInfo->chi_Mix.chi_SampleRate = SmpRate;
      ChInfo->chi_New.chi_SampleRate = SmpRate;

      TmpStore = SmpRate / MixRate;

      ChInfo->chi_Mix.chi_Advance = TmpStore;
      ChInfo->chi_New.chi_Advance = TmpStore;

      TmpStore = (((unsigned long long) SmpRate % MixRate) << 32) / MixRate;

      ChInfo->chi_Mix.chi_AdvFract = TmpStore;
      ChInfo->chi_New.chi_AdvFract = TmpStore;
    } else
    {
      ULONG SmpRate = MixStr->ms_SampleRate, MixRate = LQMixAllocStr->lqmas_MixRate, TmpStore;
      UBYTE Volume = MixStr->ms_Volume;
      BYTE Panning = MixStr->ms_Panning;

      ChInfo->chi_Mix.chi_Volume = Volume;
      ChInfo->chi_New.chi_Volume = Volume;

      ChInfo->chi_Mix.chi_Panning = Panning;
      ChInfo->chi_New.chi_Panning = Panning;

      ChInfo->chi_Mix.chi_SampleRate = SmpRate;
      ChInfo->chi_New.chi_SampleRate = SmpRate;

      TmpStore = SmpRate / MixRate;

      ChInfo->chi_Mix.chi_Advance = TmpStore;
      ChInfo->chi_New.chi_Advance = TmpStore;

      TmpStore = (((unsigned long long) SmpRate % MixRate) << 32) / MixRate;

      ChInfo->chi_Mix.chi_AdvFract = TmpStore;
      ChInfo->chi_New.chi_AdvFract = TmpStore;

      AdvOk ( LQMixAllocStr, ChInfo );

      AdvOk ( LQMixAllocStr, (struct ChannelInfo *) &(ChInfo->chi_New) );
    }
  }
}

static void InternalMixerSetChannelPosRepFlags ( struct MixerAllocStruct
    *MixAllocStr, struct MixerStruct *MixStr, unsigned long Channel )
{
  struct LQMixerAllocStruct *LQMixAllocStr = (struct LQMixerAllocStruct *) MixAllocStr;
  struct ChannelInfo *ChInfo;

  if ( Channel < LQMixAllocStr->lqmas_Channels )
  {
    ChInfo = LQMixAllocStr->lqmas_ChInfo + Channel;

    if ( ChInfo->chi_Mix.chi_Flags == MixStr->ms_Flags )
    {
      ULONG Repeat, RepLen;

      Repeat = MixStr->ms_SamplePos;
      if ( Repeat != ChInfo->chi_Mix.chi_Offset )
      {
        ChInfo->chi_Mix.chi_Offset = Repeat;
        ChInfo->chi_Mix.chi_Fraction = 0L;
      }

      Repeat = MixStr->ms_Repeat;
      RepLen = MixStr->ms_RepLen;

      ChInfo->chi_Mix.chi_Repeat = Repeat;
      ChInfo->chi_Mix.chi_RepLen = RepLen;

      if (!( ChInfo->chi_Mix.chi_Flags & MSF_Loop ))
      {
        Repeat = MixStr->ms_SampleLen;
        RepLen = 0L;
      }

      Repeat += RepLen;

      if ( ChInfo->chi_Mix.chi_Flags & MSF_Backwards )
      {
        Repeat -= RepLen;

        if (!( ChInfo->chi_Mix.chi_Flags & MSF_Loop ))
          Repeat = -1L;
      }

      ChInfo->chi_Mix.chi_EndOffset = Repeat;
      ChInfo->chi_Mix.chi_RestartOffset = RepLen;

      ChInfo->chi_Mix.chi_CountRestart = MixStr->ms_RepCount;
      ChInfo->chi_Mix.chi_Counted = MixStr->ms_RepCounted;
    } else
    {
      ULONG Repeat, RepLen;

      ChInfo->chi_Mix.chi_Flags = MixStr->ms_Flags;

      Repeat = MixStr->ms_SamplePos;
      if ( Repeat != ChInfo->chi_Mix.chi_Offset )
      {
        ChInfo->chi_Mix.chi_Offset = Repeat;
        ChInfo->chi_Mix.chi_Fraction = 0L;
      }

      Repeat = MixStr->ms_Repeat;
      RepLen = MixStr->ms_RepLen;

      ChInfo->chi_Mix.chi_Repeat = Repeat;
      ChInfo->chi_Mix.chi_RepLen = RepLen;

      if (!( ChInfo->chi_Mix.chi_Flags & MSF_Loop ))
      {
        Repeat = MixStr->ms_SampleLen;
        RepLen = 0L;
      }

      Repeat += RepLen;

      if ( ChInfo->chi_Mix.chi_Flags & MSF_Backwards )
      {
        Repeat -= RepLen;

        if (!( ChInfo->chi_Mix.chi_Flags & MSF_Loop ))
          Repeat = -1L;
      }

      ChInfo->chi_Mix.chi_EndOffset = Repeat;
      ChInfo->chi_Mix.chi_RestartOffset = RepLen;

      ChInfo->chi_Mix.chi_CountRestart = MixStr->ms_RepCount;
      ChInfo->chi_Mix.chi_Counted = MixStr->ms_RepCounted;

      AdvOk ( LQMixAllocStr, ChInfo );
    }
  }
}

static void InternalMixerDoMix ( struct MixerAllocStruct *MixAllocStr,
    LONG *MixBuffer )
{
  struct LQMixerAllocStruct *LQMixAllocStr = (struct LQMixerAllocStruct *) MixAllocStr;
  struct PostProcAllocStruct *PostAllocStr;
  LONG *PostMixBuf = MixBuffer;
  ULONG MixRate, CurrLeft, CurrLeftFrac, BuffSize;

  if (!( LQMixAllocStr->lqmas_AllocMixStruct.mas_Flags & MASF_Frozen ))
  {
    MixRate = LQMixAllocStr->lqmas_MixRate;
    CurrLeft = LQMixAllocStr->lqmas_CurrLeft;
    CurrLeftFrac = LQMixAllocStr->lqmas_CurrLeftFrac;
    BuffSize = LQMixAllocStr->lqmas_BuffSize;

    if ( LQMixAllocStr->lqmas_Stereo )
    {
      memset ( MixBuffer, 0, BuffSize << 3UL );
    } else
    {
      memset ( MixBuffer, 0, BuffSize << 2UL );
    }

    while ( BuffSize )
    {
      if ( CurrLeft )
      {
        ULONG MixLength;

        MixLength = BuffSize;

        if ( BuffSize > CurrLeft )
          MixLength = CurrLeft;

        CurrLeft -= MixLength;
        BuffSize -= MixLength;

        MixSample ( LQMixAllocStr, MixBuffer, MixLength );

        if ( LQMixAllocStr->lqmas_Stereo )
          MixLength += MixLength;

        MixBuffer += MixLength;
      }

      if ( CurrLeft )
        continue;

      DoPlayerIRQ ( LQMixAllocStr->lqmas_AllocMixStruct.mas_MixLibBase->mix_Library.extb_TuComposerBase );

      CurrLeft = LQMixAllocStr->lqmas_PassLength;
      CurrLeftFrac += LQMixAllocStr->lqmas_PassFrac;

      if ( CurrLeftFrac < LQMixAllocStr->lqmas_PassFrac )
        CurrLeft++;
    }

    LQMixAllocStr->lqmas_CurrLeft = CurrLeft;
    LQMixAllocStr->lqmas_CurrLeftFrac = CurrLeftFrac;
  }

  PostAllocStr = (struct PostProcAllocStruct *) &(LQMixAllocStr->lqmas_AllocMixStruct.mas_PostProcs.mlh_Head);

  for (;;)
  {
    PostAllocStr = (struct PostProcAllocStruct *) PostAllocStr->pas_Node.mln_Succ;

    if (!( PostAllocStr->pas_Node.mln_Succ ))
      break;

    ExternalPostProcDoMixBase ( PostAllocStr, PostMixBuf, PostAllocStr->pas_PostProcLibBase );
  }
}

static void MixSample ( struct LQMixerAllocStruct *LQMixAllocStr, LONG *MixBuffer, ULONG Length )
{
  struct ChannelInfo *ChInfo = LQMixAllocStr->lqmas_ChInfo;
  UWORD i;

  for (i = LQMixAllocStr->lqmas_Channels; i > 0; i--)
  {
    if ( ChInfo->chi_Mix.chi_Flags & MSF_Play )
    {
      void (*MixRoutine)( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length );
      LONG *MixBuf = MixBuffer;
      ULONG Offset = ChInfo->chi_Mix.chi_Offset;
      ULONG Fraction = ChInfo->chi_Mix.chi_Fraction;
      ULONG Advance = ChInfo->chi_Mix.chi_Advance;
      ULONG AdvFract = ChInfo->chi_Mix.chi_AdvFract;
      ULONG RemainLen = Length, RemainMix;
      ULONG Counted;
      ULONG CountRestart;
      unsigned long long CalcMix;

      (APTR) MixRoutine = (APTR) ChInfo->chi_Mix.chi_MixRoutine;

      if ( ChInfo->chi_Mix.chi_Flags & MSF_Backwards )
      {
MixSmp_Backwards:

        for (;;)
        {
          CalcMix = (((((unsigned long long) Advance << 32) + AdvFract) * RemainLen) + Fraction) >> 32;

          if ( (LONG) (RemainMix = Offset - ChInfo->chi_Mix.chi_EndOffset) > 0 )
          {
            if ( (ULONG) CalcMix < RemainMix )
            {
              MixRoutine ( LQMixAllocStr, ChInfo, (LONG **) &MixBuf, (ULONG *) &Offset, (ULONG *) &Fraction, Advance, AdvFract, RemainLen );

              if ( (LONG) Offset <= (LONG) ChInfo->chi_Mix.chi_EndOffset )
              {
                RemainLen = 0L;
              } else
              {
                break;
              }
            } else
            {
              CalcMix = (((((unsigned long long) RemainMix << 32) - Fraction) - 1) / (((unsigned long long) Advance << 32) + AdvFract) + 1);

              RemainLen -= (ULONG) CalcMix;

              MixRoutine ( LQMixAllocStr, ChInfo, (LONG **) &MixBuf, (ULONG *) &Offset, (ULONG *) &Fraction, Advance, AdvFract, (ULONG) CalcMix );

              if ( (LONG) Offset > (LONG) ChInfo->chi_Mix.chi_EndOffset )
              {
                if (!( RemainLen ))
                  break;
              }
            }
          }

          if ( ChInfo->chi_Mix.chi_Flags & MSF_Loop )
          {
            Counted = ChInfo->chi_Mix.chi_Counted++;

            if ( (CountRestart = ChInfo->chi_Mix.chi_CountRestart) && (CountRestart == Counted) )
            {
              ChInfo->chi_Mix.chi_Flags &= ~MSF_Loop;

              ChInfo->chi_Mix.chi_EndOffset = -1L;

              goto MixSmp_Synth;
            } else
            {
              if ( ChInfo->chi_Mix.chi_Flags & MSF_Pingpong )
              {
                APTR ChangeMixRoutine;

                if ( ChInfo->chi_New.chi_SampleStartAdr )
                {
                  memcpy ( ChInfo, &(ChInfo->chi_New), sizeof (struct ChannelBlock ));

                  ChInfo->chi_New.chi_SampleStartAdr = NULL;
                }

                (APTR) ChangeMixRoutine = (APTR) ChInfo->chi_Mix.chi_AltMixRoutine;
                (APTR) ChInfo->chi_Mix.chi_AltMixRoutine = MixRoutine;

                (APTR) MixRoutine = (APTR) ChangeMixRoutine;
                (APTR) ChInfo->chi_Mix.chi_MixRoutine = (APTR) MixRoutine;

                ChInfo->chi_Mix.chi_Flags ^= MSF_Backwards;

                RemainMix = ChInfo->chi_Mix.chi_EndOffset;

                Offset -= RemainMix;
                Offset = -Offset + RemainMix;

                RemainMix += ChInfo->chi_Mix.chi_RestartOffset;
                ChInfo->chi_Mix.chi_EndOffset = RemainMix;

                if ( (LONG) RemainLen > 0 )
                  goto MixSmp_Forwards;

                break;
              } else
              {
                Offset += ChInfo->chi_Mix.chi_RestartOffset;

                if ( ChInfo->chi_New.chi_SampleStartAdr )
                  goto MixSmp_Synth;

                if ( (LONG) RemainLen > 0 )
                  continue;

                break;
              }
            }
          } else
          {
            if ( ChInfo->chi_New.chi_SampleStartAdr )
            {
              goto MixSmp_Synth;
            } else
            {
              ChInfo->chi_Mix.chi_Flags &= ~MSF_Play;
            }

            break;
          }
        }
      } else
      {
MixSmp_Forwards:

        for (;;)
        {
          CalcMix = (((((unsigned long long) Advance << 32) + AdvFract) * RemainLen) + Fraction) >> 32;

          if ( (LONG) (RemainMix = ChInfo->chi_Mix.chi_EndOffset - Offset) > 0 )
          {
            if ( (ULONG) CalcMix < RemainMix )
            {
              MixRoutine ( LQMixAllocStr, ChInfo, (LONG **) &MixBuf, (ULONG *) &Offset, (ULONG *) &Fraction, Advance, AdvFract, RemainLen );

              if ( Offset >= ChInfo->chi_Mix.chi_EndOffset )
              {
                RemainLen = 0L;
              } else
              {
                break;
              }
            } else
            {
              CalcMix = (((((unsigned long long) RemainMix << 32) - Fraction) - 1) / (((unsigned long long) Advance << 32) + AdvFract) + 1);

              RemainLen -= (ULONG) CalcMix;

              MixRoutine ( LQMixAllocStr, ChInfo, (LONG **) &MixBuf, (ULONG *) &Offset, (ULONG *) &Fraction, Advance, AdvFract, (ULONG) CalcMix );

              if ( Offset < ChInfo->chi_Mix.chi_EndOffset )
              {
                if (!( RemainLen ))
                  break;
              }
            }
          }

          if ( ChInfo->chi_Mix.chi_Flags & MSF_Loop )
          {
            Counted = ChInfo->chi_Mix.chi_Counted++;

            if ( (CountRestart = ChInfo->chi_Mix.chi_CountRestart) && (CountRestart == Counted) )
            {
              ChInfo->chi_Mix.chi_Flags &= ~MSF_Loop;

              ChInfo->chi_Mix.chi_EndOffset = ChInfo->chi_Mix.chi_SampleLen;

              goto MixSmp_Synth;
            } else
            {
              if ( ChInfo->chi_Mix.chi_Flags & MSF_Pingpong )
              {
                APTR ChangeMixRoutine;

                if ( ChInfo->chi_New.chi_SampleStartAdr )
                {
                  memcpy ( ChInfo, &(ChInfo->chi_New), sizeof (struct ChannelBlock ));

                  ChInfo->chi_New.chi_SampleStartAdr = NULL;
                }

                (APTR) ChangeMixRoutine = (APTR) ChInfo->chi_Mix.chi_AltMixRoutine;
                (APTR) ChInfo->chi_Mix.chi_AltMixRoutine = MixRoutine;

                (APTR) MixRoutine = (APTR) ChangeMixRoutine;
                (APTR) ChInfo->chi_Mix.chi_MixRoutine = (APTR) MixRoutine;

                ChInfo->chi_Mix.chi_Flags ^= MSF_Backwards;

                RemainMix = ChInfo->chi_Mix.chi_EndOffset;

                Offset -= RemainMix;
                Offset = -Offset + RemainMix;

                RemainMix -= ChInfo->chi_Mix.chi_RestartOffset;
                ChInfo->chi_Mix.chi_EndOffset = RemainMix;

                if ( RemainLen )
                  goto MixSmp_Backwards;

                break;
              } else
              {
                Offset -= ChInfo->chi_Mix.chi_RestartOffset;

                if ( ChInfo->chi_New.chi_SampleStartAdr )
                {
                  memcpy ( ChInfo, &(ChInfo->chi_New), sizeof (struct ChannelBlock ));

                  ChInfo->chi_New.chi_SampleStartAdr = NULL;
                }

                if ( (LONG) RemainLen > 0 )
                  continue;

                break;
              }
            }
          } else
          {
            if ( ChInfo->chi_New.chi_SampleStartAdr )
            {
MixSmp_Synth:
              memcpy ( ChInfo, &(ChInfo->chi_New), sizeof (struct ChannelBlock ));

              ChInfo->chi_New.chi_SampleStartAdr = NULL;

              if ( (LONG) RemainLen > 0 )
                continue;

            } else
            {
              ChInfo->chi_Mix.chi_Flags &= ~MSF_Play;
            }

            break;
          }
        }
      }

      ChInfo->chi_Mix.chi_Offset = Offset;
      ChInfo->chi_Mix.chi_Fraction = Fraction;
    }

    ChInfo++;
  }
}

static void SetSmpMixRate ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, unsigned long SampleRate )
{
  ULONG MixRate;

  ChInfo->chi_Mix.chi_SampleRate = SampleRate;

  MixRate = LQMixAllocStr->lqmas_MixRate;

  ChInfo->chi_Mix.chi_Advance = SampleRate / MixRate;
  ChInfo->chi_Mix.chi_AdvFract = (((unsigned long long) SampleRate % MixRate) << 32) / MixRate;

  AdvOk ( LQMixAllocStr, ChInfo );
}

static void AdvOk ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo )
{
  APTR *MixRoutine;
  void (*MixInit)( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, UWORD Volume, UWORD Panning );
  UWORD Panning;

  if ( (ChInfo->chi_Mix.chi_Flags & MSF_Muted) || (ChInfo->chi_Mix.chi_Volume == 0) )
  {
    (APTR) MixRoutine = (APTR) &Mixr_Skip;
  } else if (!( LQMixAllocStr->lqmas_Stereo ))
  {
    (APTR) MixRoutine = (APTR) &Mixr_Mono;
  } else if ( ChInfo->chi_Mix.chi_Flags & MSF_Surround )
  {
    Panning = 0x80;

    (APTR) MixRoutine = (APTR) &Mixr_Stereo;

    if ( LQMixAllocStr->lqmas_AllocMixStruct.mas_LeftVolume == LQMixAllocStr->lqmas_AllocMixStruct.mas_RightVolume )
      (APTR) MixRoutine = (APTR) &Mixr_StereoSur;
  } else
  {
    switch ( (Panning = ChInfo->chi_Mix.chi_Panning) )
    {
      case 0 :
      {
        (APTR) MixRoutine = (APTR) &Mixr_Skip;

        if ( LQMixAllocStr->lqmas_AllocMixStruct.mas_LeftVolume )
          (APTR) MixRoutine = (APTR) &Mixr_StereoL;

        break;
      }
      case 0xFF :
      {
        (APTR) MixRoutine = (APTR) &Mixr_Skip;

        if ( LQMixAllocStr->lqmas_AllocMixStruct.mas_RightVolume )
          (APTR) MixRoutine = (APTR) &Mixr_StereoR;

        break;
      }
      case 0x80 :
      {
        (APTR) MixRoutine = (APTR) &Mixr_Stereo;

        if ( LQMixAllocStr->lqmas_AllocMixStruct.mas_LeftVolume == LQMixAllocStr->lqmas_AllocMixStruct.mas_RightVolume )
          (APTR) MixRoutine = (APTR) &Mixr_StereoC;

        break;
      }
      default :
      {
        (APTR) MixRoutine = (APTR) &Mixr_Stereo;

        break;
      }
    }
  }

  if ( (ChInfo->chi_Mix.chi_BitDepth < 9) || (LQMixAllocStr->lqmas_Real16Bit == 0) )
    (BYTE *) MixRoutine += ((BYTE *) &Mixr_Skip168 - (BYTE *) &Mixr_Skip);

  switch ( ChInfo->chi_Mix.chi_BitDepth )
  {
    case 8 :
    {
      if ( ChInfo->chi_Mix.chi_Flags & MSF_Backwards )
      {
        (APTR) ChInfo->chi_Mix.chi_MixRoutine = (APTR *) MixRoutine[7];
        (APTR) ChInfo->chi_Mix.chi_AltMixRoutine = (APTR *) MixRoutine[3];
      } else
      {
        (APTR) ChInfo->chi_Mix.chi_MixRoutine = (APTR *) MixRoutine[3];
        (APTR) ChInfo->chi_Mix.chi_AltMixRoutine = (APTR *) MixRoutine[7];
      }

      (APTR) MixInit = (APTR *) MixRoutine[0];

      break;
    }

    case 16 :
    {
      if ( ChInfo->chi_Mix.chi_Flags & MSF_Backwards )
      {
        (APTR) ChInfo->chi_Mix.chi_MixRoutine = (APTR *) MixRoutine[8];
        (APTR) ChInfo->chi_Mix.chi_AltMixRoutine = (APTR *) MixRoutine[4];
      } else
      {
        (APTR) ChInfo->chi_Mix.chi_MixRoutine = (APTR *) MixRoutine[4];
        (APTR) ChInfo->chi_Mix.chi_AltMixRoutine = (APTR *) MixRoutine[8];
      }

      (APTR) MixInit = (APTR *) MixRoutine[1];

      break;
    }

    case 32 :
    {
      if ( ChInfo->chi_Mix.chi_Flags & MSF_Backwards )
      {
        (APTR) ChInfo->chi_Mix.chi_MixRoutine = (APTR *) MixRoutine[9];
        (APTR) ChInfo->chi_Mix.chi_AltMixRoutine = (APTR *) MixRoutine[5];
      } else
      {
        (APTR) ChInfo->chi_Mix.chi_MixRoutine = (APTR *) MixRoutine[5];
        (APTR) ChInfo->chi_Mix.chi_AltMixRoutine = (APTR *) MixRoutine[9];
      }

      (APTR) MixInit = (APTR *) MixRoutine[2];

      break;
    }

    default :
    {
      if ( ChInfo->chi_Mix.chi_Flags & MSF_Backwards )
      {
        (APTR) ChInfo->chi_Mix.chi_MixRoutine = (APTR *) MixRoutine[10];
        (APTR) ChInfo->chi_Mix.chi_AltMixRoutine = (APTR *) MixRoutine[6];
      } else
      {
        (APTR) ChInfo->chi_Mix.chi_MixRoutine = (APTR *) MixRoutine[6];
        (APTR) ChInfo->chi_Mix.chi_AltMixRoutine = (APTR *) MixRoutine[10];
      }

      (APTR) MixInit = (APTR *) MixRoutine[2];

      break;
    }
  }

  MixInit ( LQMixAllocStr, ChInfo, ChInfo->chi_Mix.chi_Volume, Panning );
}

static void mstv_Skip ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, UWORD Volume, UWORD Panning )
{
}

static void mstv_Stereo8 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, UWORD Volume, UWORD Panning )
{
  ULONG LeftVolume = 255 - Panning;

  LeftVolume *= LQMixAllocStr->lqmas_AllocMixStruct.mas_LeftVolume * Volume;
  LeftVolume >>= 8;
  LeftVolume &= 0xFF00;

  ChInfo->chi_Mix.chi_VolTable_L = LQMixAllocStr->lqmas_VolTable + LeftVolume;

  LeftVolume = ((Panning * LQMixAllocStr->lqmas_AllocMixStruct.mas_RightVolume * Volume) >> 8) & 0xFF00;

  ChInfo->chi_Mix.chi_VolTable_R = LQMixAllocStr->lqmas_VolTable + LeftVolume;
}

static void mstv_Stereo8L ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, UWORD Volume, UWORD Panning )
{
  Volume *= LQMixAllocStr->lqmas_AllocMixStruct.mas_LeftVolume;
  Volume &= 0xFF00;

  ChInfo->chi_Mix.chi_VolTable_L = LQMixAllocStr->lqmas_VolTable + Volume;
}

static void mstv_Stereo8R ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, UWORD Volume, UWORD Panning )
{
  Volume *= LQMixAllocStr->lqmas_AllocMixStruct.mas_RightVolume;
  Volume &= 0xFF00;

  ChInfo->chi_Mix.chi_VolTable_R = LQMixAllocStr->lqmas_VolTable + Volume;
}

static void mstv_Stereo8C ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, UWORD Volume, UWORD Panning )
{
  Volume *= LQMixAllocStr->lqmas_AllocMixStruct.mas_LeftVolume;
  Volume >>= 1;
  Volume &= 0xFF00;

  ChInfo->chi_Mix.chi_VolTable_L = LQMixAllocStr->lqmas_VolTable + Volume;
}

static void mstv_Stereo16 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, UWORD Volume, UWORD Panning )
{
  ULONG LeftVolume = 255 - Panning;

  LeftVolume *= LQMixAllocStr->lqmas_AllocMixStruct.mas_LeftVolume * Volume;
  LeftVolume >>= 16;
  ChInfo->chi_Mix.chi_Multiply_L = LeftVolume * LQMixAllocStr->lqmas_Amplify;

  LeftVolume = ((Panning * LQMixAllocStr->lqmas_AllocMixStruct.mas_RightVolume * Volume) >> 16) * LQMixAllocStr->lqmas_Amplify;
  ChInfo->chi_Mix.chi_Multiply_R = LeftVolume * LQMixAllocStr->lqmas_Amplify;

  ChInfo->chi_Mix.chi_Divider = (ULONG) LQMixAllocStr->lqmas_Channels << 8;
}

static void mstv_Stereo16L ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, UWORD Volume, UWORD Panning )
{
  Volume *= LQMixAllocStr->lqmas_AllocMixStruct.mas_LeftVolume;
  Volume >>= 8;

  ChInfo->chi_Mix.chi_Multiply_L = (ULONG) Volume * LQMixAllocStr->lqmas_Amplify;
  ChInfo->chi_Mix.chi_Divider = (ULONG) LQMixAllocStr->lqmas_Channels << 8;
}

static void mstv_Stereo16R ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, UWORD Volume, UWORD Panning )
{
  Volume *= LQMixAllocStr->lqmas_AllocMixStruct.mas_RightVolume;
  Volume >>= 8;

  ChInfo->chi_Mix.chi_Multiply_R = (ULONG) Volume * LQMixAllocStr->lqmas_Amplify;
  ChInfo->chi_Mix.chi_Divider = (ULONG) LQMixAllocStr->lqmas_Channels << 8;
}

static void mstv_Stereo16C ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, UWORD Volume, UWORD Panning )
{
  Volume *= LQMixAllocStr->lqmas_AllocMixStruct.mas_LeftVolume;
  Volume >>= 9;

  ChInfo->chi_Mix.chi_Multiply_L = (ULONG) Volume * LQMixAllocStr->lqmas_Amplify;
  ChInfo->chi_Mix.chi_Divider = (ULONG) LQMixAllocStr->lqmas_Channels << 8;
}

static void mstv_Stereo32 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, UWORD Volume, UWORD Panning )
{
  ULONG LeftVolume = 255 - Panning;

  LeftVolume *= LQMixAllocStr->lqmas_AllocMixStruct.mas_LeftVolume * Volume;
  LeftVolume >>= 16;
  ChInfo->chi_Mix.chi_Multiply_L = (LeftVolume * LQMixAllocStr->lqmas_Amplify) >> 8;

  LeftVolume = ((Panning * LQMixAllocStr->lqmas_AllocMixStruct.mas_RightVolume * Volume) >> 16) * LQMixAllocStr->lqmas_Amplify;
  ChInfo->chi_Mix.chi_Multiply_R = (LeftVolume * LQMixAllocStr->lqmas_Amplify) >> 8;

  ChInfo->chi_Mix.chi_Divider = (ULONG) LQMixAllocStr->lqmas_Channels << 16;
}

static void mstv_Stereo32L ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, UWORD Volume, UWORD Panning )
{
  Volume *= LQMixAllocStr->lqmas_AllocMixStruct.mas_LeftVolume;
  Volume >>= 8;

  ChInfo->chi_Mix.chi_Multiply_L = ((ULONG) Volume * LQMixAllocStr->lqmas_Amplify) >> 8;
  ChInfo->chi_Mix.chi_Divider = (ULONG) LQMixAllocStr->lqmas_Channels << 16;
}

static void mstv_Stereo32R ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, UWORD Volume, UWORD Panning )
{
  Volume *= LQMixAllocStr->lqmas_AllocMixStruct.mas_RightVolume;
  Volume >>= 8;

  ChInfo->chi_Mix.chi_Multiply_R = ((ULONG) Volume * LQMixAllocStr->lqmas_Amplify) >> 8;
  ChInfo->chi_Mix.chi_Divider = (ULONG) LQMixAllocStr->lqmas_Channels << 16;
}

static void mstv_Stereo32C ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, UWORD Volume, UWORD Panning )
{
  Volume *= LQMixAllocStr->lqmas_AllocMixStruct.mas_LeftVolume;
  Volume >>= 9;

  ChInfo->chi_Mix.chi_Multiply_L = ((ULONG) Volume * LQMixAllocStr->lqmas_Amplify) >> 8;
  ChInfo->chi_Mix.chi_Divider = (ULONG) LQMixAllocStr->lqmas_Channels << 16;
}

static void Skip ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG SkipDiv;
  unsigned long long SkipLength;

  SkipLength = (((unsigned long long) Advance << 32) + AdvFract) * Length;

  SkipDiv = SkipLength >> 32;
  CurrOffset += SkipDiv;

  SkipDiv = SkipLength;
  CurrFraction += SkipDiv;

  if ( CurrFraction < SkipDiv )
    CurrOffset++;

  *Offset = CurrOffset;
  *Fraction = CurrFraction;
}

static void Skipb ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG SkipDiv;
  unsigned long long SkipLength;

  SkipLength = (((unsigned long long) Advance << 32) + AdvFract) * Length;

  SkipDiv = SkipLength >> 32;
  CurrOffset -= SkipDiv;

  SkipDiv = SkipLength;
  CurrFraction += SkipDiv;

  if ( CurrFraction < SkipDiv )
    CurrOffset--;

  *Offset = CurrOffset;
  *Fraction = CurrFraction;
}

static void MIX_Mono8 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  BYTE *SmpArray = (BYTE *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable = ChInfo->chi_Mix.chi_VolTable_L;
  LONG *MixBuf = *MixBuffer;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      BYTE *SmpArrayPos = SmpArray;
      LONG Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += VolTable[(UBYTE) Sample];

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( (CurrFraction + AdvFract) < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += VolTable[(UBYTE) Sample];
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        *MixBuf++ += VolTable[(UBYTE) SmpArray[CurrOffset]];

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

GetSecondAdvSample:
        *MixBuf++ += VolTable[(UBYTE) SmpArray[CurrOffset]];

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    BYTE *SmpArrayPos = SmpArray;
    LONG Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *SmpArray++;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        *MixBuf++ += VolTable[(UBYTE) Sample];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        *MixBuf++ += VolTable[(UBYTE) Sample];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = VolTable[(UBYTE) *SmpArray++];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[(UBYTE) *SmpArray++];

GetSecondSample:
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[(UBYTE) *SmpArray++];

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIXb_Mono8 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  BYTE *SmpArray = (BYTE *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable = ChInfo->chi_Mix.chi_VolTable_L;
  LONG *MixBuf = *MixBuffer;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      BYTE *SmpArrayPos = SmpArray;
      LONG Sample;
      ULONG InterpolDiv;

      SmpArray += (CurrOffset + 1);

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += VolTable[(UBYTE) Sample];

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += VolTable[(UBYTE) Sample];
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        *MixBuf++ += VolTable[(UBYTE) SmpArray[CurrOffset]];

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

GetSecondAdvSample:
        *MixBuf++ += VolTable[(UBYTE) SmpArray[CurrOffset]];

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    BYTE *SmpArrayPos = SmpArray;
    LONG Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += (CurrOffset + 1);

      Sample = *--SmpArray;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        *MixBuf++ += VolTable[(UBYTE) Sample];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        *MixBuf++ += VolTable[(UBYTE) Sample];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += (CurrOffset + 1);

      Sample = VolTable[(UBYTE) *--SmpArray];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[(UBYTE) *--SmpArray];

GetSecondSample:
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[(UBYTE) *--SmpArray];

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIX_Mono16 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  WORD *SmpArray = (WORD *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *MixBuf = *MixBuffer;
  LONG  Mul_L = ChInfo->chi_Mix.chi_Multiply_L, Div = ChInfo->chi_Mix.chi_Divider;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      WORD *SmpArrayPos = SmpArray;
      LONG Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += (((long long) Sample * Mul_L) / Div);

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += (((long long) Sample * Mul_L) / Div);
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        *MixBuf++ += (((long long) SmpArray[CurrOffset] * Mul_L) / Div);

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

GetSecondAdvSample:
        *MixBuf++ += (((long long) SmpArray[CurrOffset] * Mul_L) / Div);

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    WORD *SmpArrayPos = SmpArray;
    LONG Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *SmpArray++;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        *MixBuf++ += (((long long) Sample * Mul_L) / Div);

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        *MixBuf++ += (((long long) Sample * Mul_L) / Div);

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = (((long long) *SmpArray++ * Mul_L) / Div);

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = (((long long) *SmpArray++ * Mul_L) / Div);

GetSecondSample:
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = (((long long) *SmpArray++ * Mul_L) / Div);

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIXb_Mono16 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  WORD *SmpArray = (WORD *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *MixBuf = *MixBuffer;
  LONG  Mul_L = ChInfo->chi_Mix.chi_Multiply_L, Div = ChInfo->chi_Mix.chi_Divider;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      WORD *SmpArrayPos = SmpArray;
      LONG Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += (((long long) Sample * Mul_L) / Div);

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += (((long long) Sample * Mul_L) / Div);
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        *MixBuf++ += (((long long) SmpArray[CurrOffset] * Mul_L) / Div);

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

GetSecondAdvSample:
        *MixBuf++ += (((long long) SmpArray[CurrOffset] * Mul_L) / Div);

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    WORD *SmpArrayPos = SmpArray;
    LONG Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *--SmpArray;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        *MixBuf++ += (((long long) Sample * Mul_L) / Div);

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        *MixBuf++ += (((long long) Sample * Mul_L) / Div);

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = (((long long) *--SmpArray * Mul_L) / Div);

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = (((long long) *--SmpArray * Mul_L) / Div);

GetSecondSample:
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = (((long long) *--SmpArray * Mul_L) / Div);

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIX_Mono32 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *MixBuf = *MixBuffer;
  LONG  Mul_L = ChInfo->chi_Mix.chi_Multiply_L, Div = ChInfo->chi_Mix.chi_Divider;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      LONG *SmpArrayPos = SmpArray;
      long long Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += (((long long) Sample * Mul_L) / Div);

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += (((long long) Sample * Mul_L) / Div);
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        *MixBuf++ += (((long long) SmpArray[CurrOffset] * Mul_L) / Div);

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

GetSecondAdvSample:
        *MixBuf++ += (((long long) SmpArray[CurrOffset] * Mul_L) / Div);

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    LONG *SmpArrayPos = SmpArray;
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *SmpArray++;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        *MixBuf++ += (((long long) Sample * Mul_L) / Div);

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        *MixBuf++ += (((long long) Sample * Mul_L) / Div);

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = (((long long) *SmpArray++ * Mul_L) / Div);

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = (((long long) *SmpArray++ * Mul_L) / Div);

GetSecondSample:
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = (((long long) *SmpArray++ * Mul_L) / Div);

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIXb_Mono32 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *MixBuf = *MixBuffer;
  LONG  Mul_L = ChInfo->chi_Mix.chi_Multiply_L, Div = ChInfo->chi_Mix.chi_Divider;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      LONG *SmpArrayPos = SmpArray;
      long long Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += (((long long) Sample * Mul_L) / Div);

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += (((long long) Sample * Mul_L) / Div);
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        *MixBuf++ += (((long long) SmpArray[CurrOffset] * Mul_L) / Div);

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

GetSecondAdvSample:
        *MixBuf++ += (((long long) SmpArray[CurrOffset] * Mul_L) / Div);

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    LONG *SmpArrayPos = SmpArray;
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *--SmpArray;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        *MixBuf++ += (((long long) Sample * Mul_L) / Div);

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        *MixBuf++ += (((long long) Sample * Mul_L) / Div);

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = (((long long) *--SmpArray * Mul_L) / Div);

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = (((long long) *--SmpArray * Mul_L) / Div);

GetSecondSample:
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = (((long long) *--SmpArray * Mul_L) / Div);

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIX_MonoX ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *MixBuf = *MixBuffer;
  LONG  Mul_L = ChInfo->chi_Mix.chi_Multiply_L, Div = ChInfo->chi_Mix.chi_Divider;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i, Bit, SampleData, BitDepth = ChInfo->chi_Mix.chi_BitDepth;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      long long Sample;
      ULONG InterpolDiv, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset += SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;

          Sample += SampleData;

          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += (((long long) Sample * Mul_L) / Div);

GetSecondAdvInterpolSample:
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset += SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;

          Sample += SampleData;
          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += (((long long) Sample * Mul_L) / Div);
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      ULONG RealAdv = Advance * BitDepth, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpOffset = (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        *MixBuf++ += (((long long) SampleData * Mul_L) / Div);

        CurrFraction += AdvFract;
        CurrOffset += Advance;
        Bit += RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset++;
          Bit += BitDepth;
        }

        SmpOffset += (Bit >> 5UL);
        Bit &= 31;

GetSecondAdvSample:
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        *MixBuf++ += (((long long) SampleData * Mul_L) / Div);

        CurrFraction += AdvFract;
        CurrOffset += Advance;
        Bit += RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset++;
          Bit += BitDepth;
        }

        SmpOffset += (Bit >> 5UL);
        Bit &= 31;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray++ << Bit;
        SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

        Bit &= 31;
      }

      Bit += BitDepth;
      CurrOffset++;

      Sample = (LONG) SampleData;
      SampleValue = 0LL;

      if ( Length )
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) *SmpArray << Bit;
          SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
      }

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        *MixBuf++ += (((long long) Sample * Mul_L) / Div);

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        *MixBuf++ += (((long long) Sample * Mul_L) / Div);

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray++ << Bit;
        SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

        Bit &= 31;
      }

      Bit += BitDepth;
      CurrOffset++;

      Sample = (((long long) SampleData * Mul_L) / Div);

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = (((long long) SampleData * Mul_L) / Div);
        }

GetSecondSample:
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = (((long long) SampleData * Mul_L) / Div);
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIXb_MonoX ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *MixBuf = *MixBuffer;
  LONG  Mul_L = ChInfo->chi_Mix.chi_Multiply_L, Div = ChInfo->chi_Mix.chi_Divider;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i, Bit, SampleData, BitDepth = ChInfo->chi_Mix.chi_BitDepth;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      long long Sample;
      ULONG InterpolDiv, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset -= SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample += SampleData;
          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += (((long long) Sample * Mul_L) / Div);

GetSecondAdvInterpolSample:
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset -= SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample += SampleData;
          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += (((long long) Sample * Mul_L) / Div);
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      ULONG RealAdv = Advance * BitDepth, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpOffset = (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        *MixBuf++ += (((long long) SampleData * Mul_L) / Div);

        CurrFraction += AdvFract;
        CurrOffset -= Advance;
        Bit -= RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;
        }

        SmpOffset += ((LONG) Bit >> 5L);
        Bit &= 31;

GetSecondAdvSample:
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        *MixBuf++ += (((long long) SampleData * Mul_L) / Div);

        CurrFraction += AdvFract;
        CurrOffset -= Advance;
        Bit -= RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;
        }

        SmpOffset += ((LONG) Bit >> 5L);
        Bit &= 31;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      CurrOffset--;
      Bit -= BitDepth;

      if ( (LONG) Bit < 0 )
      {
        SmpArray--;

        Bit &= 31;
      }

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray << Bit;
        SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
      }

      Sample = (LONG) SampleData;
      SampleValue = 0LL;

      if ( Length )
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) *SmpArray << Bit;
          SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
      }

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        *MixBuf++ += (((long long) Sample * Mul_L) / Div);

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        *MixBuf++ += (((long long) Sample * Mul_L) / Div);

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      CurrOffset--;
      Bit -= BitDepth;

      if ( (LONG) Bit < 0 )
      {
        SmpArray--;

        Bit &= 31;
      }

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray << Bit;
        SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
      }

      Sample = (((long long) SampleData * Mul_L) / Div);

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = (((long long) SampleData * Mul_L) / Div);
        }

GetSecondSample:
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = (((long long) SampleData * Mul_L) / Div);
        }

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIX_Mono168 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  WORD *SmpArray = (WORD *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable = ChInfo->chi_Mix.chi_VolTable_L;
  LONG *MixBuf = *MixBuffer;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      WORD *SmpArrayPos = SmpArray;
      LONG Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += VolTable[((UWORD) Sample >> 8U)];

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += VolTable[((UWORD) Sample >> 8U)];
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        *MixBuf++ += VolTable[((UWORD) SmpArray[CurrOffset] >> 8U)];

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

GetSecondAdvSample:
        *MixBuf++ += VolTable[((UWORD) SmpArray[CurrOffset] >> 8U)];

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    WORD *SmpArrayPos = SmpArray;
    LONG Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *SmpArray++;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        *MixBuf++ += VolTable[((UWORD) Sample >> 8U)];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        *MixBuf++ += VolTable[((UWORD) Sample >> 8U)];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = VolTable[(((UWORD) *SmpArray++ >> 8U))];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[((UWORD) *SmpArray++ >> 8U)];

GetSecondSample:
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[((UWORD) *SmpArray++ >> 8U)];

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIXb_Mono168 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  WORD *SmpArray = (WORD *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable = ChInfo->chi_Mix.chi_VolTable_L;
  LONG *MixBuf = *MixBuffer;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      WORD *SmpArrayPos = SmpArray;
      LONG Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += VolTable[((UWORD) Sample >> 8U)];

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += VolTable[((UWORD) Sample >> 8U)];
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        *MixBuf++ += VolTable[((UWORD) SmpArray[CurrOffset] >> 8U)];

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

GetSecondAdvSample:
        *MixBuf++ += VolTable[((UWORD) SmpArray[CurrOffset] >> 8U)];

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    WORD *SmpArrayPos = SmpArray;
    LONG Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *--SmpArray;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        *MixBuf++ += VolTable[((UWORD) Sample >> 8U)];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        *MixBuf++ += VolTable[((UWORD) Sample >> 8U)];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = VolTable[(((UWORD) *--SmpArray >> 8U))];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[((UWORD) *--SmpArray >> 8U)];

GetSecondSample:
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[((UWORD) *--SmpArray >> 8U)];

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIX_Mono328 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable = ChInfo->chi_Mix.chi_VolTable_L;
  LONG *MixBuf = *MixBuffer;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      LONG *SmpArrayPos = SmpArray;
      long long Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += VolTable[((ULONG) Sample >> 24UL)];

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += VolTable[((ULONG) Sample >> 24UL)];
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        *MixBuf++ += VolTable[((ULONG) SmpArray[CurrOffset] >> 24UL)];

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

GetSecondAdvSample:
        *MixBuf++ += VolTable[((ULONG) SmpArray[CurrOffset] >> 24UL)];

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    LONG *SmpArrayPos = SmpArray;
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *SmpArray++;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        *MixBuf++ += VolTable[((ULONG) Sample >> 24UL)];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        *MixBuf++ += VolTable[((ULONG) Sample >> 24UL)];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = VolTable[(((ULONG) *SmpArray++ >> 24UL))];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[((ULONG) *SmpArray++ >> 24UL)];

GetSecondSample:
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[((ULONG) *SmpArray++ >> 24UL)];

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIXb_Mono328 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable = ChInfo->chi_Mix.chi_VolTable_L;
  LONG *MixBuf = *MixBuffer;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      LONG *SmpArrayPos = SmpArray;
      long long Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += VolTable[((ULONG) Sample >> 24UL)];

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += VolTable[((ULONG) Sample >> 24UL)];
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        *MixBuf++ += VolTable[((ULONG) SmpArray[CurrOffset] >> 24UL)];

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

GetSecondAdvSample:
        *MixBuf++ += VolTable[((ULONG) SmpArray[CurrOffset] >> 24UL)];

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    LONG *SmpArrayPos = SmpArray;
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *--SmpArray;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        *MixBuf++ += VolTable[((ULONG) Sample >> 24UL)];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        *MixBuf++ += VolTable[((ULONG) Sample >> 24UL)];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = VolTable[(((ULONG) *SmpArray++ >> 24UL))];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[((ULONG) *--SmpArray >> 24UL)];

GetSecondSample:
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[((ULONG) *--SmpArray >> 24UL)];

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIX_MonoX8 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable = ChInfo->chi_Mix.chi_VolTable_L;
  LONG *MixBuf = *MixBuffer;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i, Bit, SampleData, BitDepth = ChInfo->chi_Mix.chi_BitDepth;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      long long Sample;
      ULONG InterpolDiv, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset += SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;

          Sample += SampleData;

          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += VolTable[((ULONG) Sample >> 24UL)];

GetSecondAdvInterpolSample:
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset += SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;

          Sample += SampleData;
          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += VolTable[((ULONG) Sample >> 24UL)];
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      ULONG RealAdv = Advance * BitDepth, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpOffset = (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        *MixBuf++ += VolTable[((ULONG) SampleData >> 24UL)];

        CurrFraction += AdvFract;
        CurrOffset += Advance;
        Bit += RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset++;
          Bit += BitDepth;
        }

        SmpOffset += (Bit >> 5UL);
        Bit &= 31;

GetSecondAdvSample:
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        *MixBuf++ += VolTable[((ULONG) SampleData >> 24UL)];

        CurrFraction += AdvFract;
        CurrOffset += Advance;
        Bit += RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset++;
          Bit += BitDepth;
        }

        SmpOffset += (Bit >> 5UL);
        Bit &= 31;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray++ << Bit;
        SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

        Bit &= 31;
      }

      Bit += BitDepth;
      CurrOffset++;

      Sample = (LONG) SampleData;
      SampleValue = 0LL;

      if ( Length )
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) *SmpArray << Bit;
          SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
      }

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        *MixBuf++ += VolTable[((ULONG) Sample >> 24UL)];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        *MixBuf++ += VolTable[((ULONG) Sample >> 24UL)];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray++ << Bit;
        SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

        Bit &= 31;
      }

      Bit += BitDepth;
      CurrOffset++;

      Sample = VolTable[((ULONG) SampleData >> 24UL)];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = VolTable[((ULONG) SampleData >> 24UL)];
        }

GetSecondSample:
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = VolTable[((ULONG) SampleData >> 24UL)];
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIXb_MonoX8 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable = ChInfo->chi_Mix.chi_VolTable_L;
  LONG *MixBuf = *MixBuffer;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i, Bit, SampleData, BitDepth = ChInfo->chi_Mix.chi_BitDepth;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      long long Sample;
      ULONG InterpolDiv, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset -= SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample += SampleData;
          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += VolTable[((ULONG) Sample >> 24UL)];

GetSecondAdvInterpolSample:
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset -= SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample += SampleData;
          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += VolTable[((ULONG) Sample >> 24UL)];
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      ULONG RealAdv = Advance * BitDepth, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpOffset = (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        *MixBuf++ += VolTable[((ULONG) SampleData >> 24UL)];

        CurrFraction += AdvFract;
        CurrOffset -= Advance;
        Bit -= RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;
        }

        SmpOffset += ((LONG) Bit >> 5L);
        Bit &= 31;

GetSecondAdvSample:
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        *MixBuf++ += VolTable[((ULONG) SampleData >> 24UL)];

        CurrFraction += AdvFract;
        CurrOffset -= Advance;
        Bit -= RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;
        }

        SmpOffset += ((LONG) Bit >> 5L);
        Bit &= 31;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      CurrOffset--;
      Bit -= BitDepth;

      if ( (LONG) Bit < 0 )
      {
        SmpArray--;

        Bit &= 31;
      }

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray << Bit;
        SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
      }

      Sample = (LONG) SampleData;
      SampleValue = 0LL;

      if ( Length )
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) *SmpArray << Bit;
          SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
      }

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        *MixBuf++ += VolTable[((ULONG) Sample >> 24UL)];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        *MixBuf++ += VolTable[((ULONG) Sample >> 24UL)];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      CurrOffset--;
      Bit -= BitDepth;

      if ( (LONG) Bit < 0 )
      {
        SmpArray--;

        Bit &= 31;
      }

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray << Bit;
        SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
      }

      Sample = VolTable[((ULONG) SampleData >> 24UL)];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = VolTable[((ULONG) SampleData >> 24UL)];
        }

GetSecondSample:
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = VolTable[((ULONG) SampleData >> 24UL)];
        }

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIX_Stereo8 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  BYTE *SmpArray = (BYTE *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable_L = ChInfo->chi_Mix.chi_VolTable_L;
  LONG *VolTable_R = ChInfo->chi_Mix.chi_VolTable_R;
  LONG *MixBuf = *MixBuffer;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      BYTE *SmpArrayPos = SmpArray;
      LONG Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += VolTable_L[(UBYTE) Sample];
        *MixBuf++ += VolTable_R[(UBYTE) Sample];

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += VolTable_L[(UBYTE) Sample];
        *MixBuf++ += VolTable_R[(UBYTE) Sample];
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        *MixBuf++ += VolTable_L[(UBYTE) SmpArray[CurrOffset]];
        *MixBuf++ += VolTable_R[(UBYTE) SmpArray[CurrOffset]];

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

GetSecondAdvSample:
        *MixBuf++ += VolTable_L[(UBYTE) SmpArray[CurrOffset]];
        *MixBuf++ += VolTable_R[(UBYTE) SmpArray[CurrOffset]];

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    BYTE *SmpArrayPos = SmpArray;
    LONG Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *SmpArray++;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        *MixBuf++ += VolTable_L[(UBYTE) Sample];
        *MixBuf++ += VolTable_R[(UBYTE) Sample];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        *MixBuf++ += VolTable_L[(UBYTE) Sample];
        *MixBuf++ += VolTable_R[(UBYTE) Sample];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      BYTE WriteSample;
      LONG Sample_R;

      SmpArray += CurrOffset;

      WriteSample = *SmpArray++;
      Sample = VolTable_L[(UBYTE) WriteSample];
      Sample_R = VolTable_R[(UBYTE) WriteSample];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;
        *MixBuf++ += Sample_R;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          WriteSample = *SmpArray++;
          Sample = VolTable_L[(UBYTE) WriteSample];
          Sample_R = VolTable_R[(UBYTE) WriteSample];
        }

GetSecondSample:
        *MixBuf++ += Sample;
        *MixBuf++ += Sample_R;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          WriteSample = *SmpArray++;
          Sample = VolTable_L[(UBYTE) WriteSample];
          Sample_R = VolTable_R[(UBYTE) WriteSample];
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIXb_Stereo8 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  BYTE *SmpArray = (BYTE *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable_L = ChInfo->chi_Mix.chi_VolTable_L;
  LONG *VolTable_R = ChInfo->chi_Mix.chi_VolTable_R;
  LONG *MixBuf = *MixBuffer;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      BYTE *SmpArrayPos = SmpArray;
      LONG Sample;
      ULONG InterpolDiv;

      SmpArray += (CurrOffset + 1);

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += VolTable_L[(UBYTE) Sample];
        *MixBuf++ += VolTable_R[(UBYTE) Sample];

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += VolTable_L[(UBYTE) Sample];
        *MixBuf++ += VolTable_R[(UBYTE) Sample];
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {

        *MixBuf++ += VolTable_L[(UBYTE) SmpArray[CurrOffset]];
        *MixBuf++ += VolTable_R[(UBYTE) SmpArray[CurrOffset]];

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

GetSecondAdvSample:
        *MixBuf++ += VolTable_L[(UBYTE) SmpArray[CurrOffset]];
        *MixBuf++ += VolTable_R[(UBYTE) SmpArray[CurrOffset]];

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    BYTE *SmpArrayPos = SmpArray;
    LONG Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += (CurrOffset + 1);

      Sample = *--SmpArray;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        *MixBuf++ += VolTable_L[(UBYTE) Sample];
        *MixBuf++ += VolTable_R[(UBYTE) Sample];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        *MixBuf++ += VolTable_L[(UBYTE) Sample];
        *MixBuf++ += VolTable_R[(UBYTE) Sample];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      BYTE WriteSample;
      LONG Sample_R;

      SmpArray += (CurrOffset + 1);

      WriteSample = *--SmpArray;
      Sample = VolTable_L[(UBYTE) WriteSample];
      Sample_R = VolTable_R[(UBYTE) WriteSample];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;
        *MixBuf++ += Sample_R;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          WriteSample = *--SmpArray;
          Sample = VolTable_L[(UBYTE) WriteSample];
          Sample_R = VolTable_R[(UBYTE) WriteSample];
        }

GetSecondSample:
        *MixBuf++ += Sample;
        *MixBuf++ += Sample_R;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          WriteSample = *--SmpArray;
          Sample = VolTable_L[(UBYTE) WriteSample];
          Sample_R = VolTable_R[(UBYTE) WriteSample];
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIX_Stereo16 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  WORD *SmpArray = (WORD *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *MixBuf = *MixBuffer;
  LONG  Mul_L = ChInfo->chi_Mix.chi_Multiply_L, Mul_R = ChInfo->chi_Mix.chi_Multiply_R, Div = ChInfo->chi_Mix.chi_Divider;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      WORD *SmpArrayPos = SmpArray;
      LONG Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        *MixBuf++ += (((long long) SmpArray[CurrOffset] * Mul_L) / Div);
        *MixBuf++ += (((long long) SmpArray[CurrOffset] * Mul_R) / Div);

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

GetSecondAdvSample:
        *MixBuf++ += (((long long) SmpArray[CurrOffset] * Mul_L) / Div);
        *MixBuf++ += (((long long) SmpArray[CurrOffset] * Mul_R) / Div);

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    WORD *SmpArrayPos = SmpArray;
    LONG Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *SmpArray++;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        *MixBuf++ += (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        *MixBuf++ += (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      WORD WriteSample;
      LONG Sample_R;

      SmpArray += CurrOffset;

      WriteSample = *SmpArray++;
      Sample = (((long long) WriteSample * Mul_L) / Div);
      Sample_R = (((long long) WriteSample * Mul_R) / Div);

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;
        *MixBuf++ += Sample_R;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          WriteSample = *SmpArray++;
          Sample = (((long long) WriteSample * Mul_L) / Div);
          Sample_R = (((long long) WriteSample * Mul_R) / Div);
        }

GetSecondSample:
        *MixBuf++ += Sample;
        *MixBuf++ += Sample_R;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          WriteSample = *SmpArray++;
          Sample = (((long long) WriteSample * Mul_L) / Div);
          Sample_R = (((long long) WriteSample * Mul_R) / Div);
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIXb_Stereo16 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  WORD *SmpArray = (WORD *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *MixBuf = *MixBuffer;
  LONG  Mul_L = ChInfo->chi_Mix.chi_Multiply_L, Mul_R = ChInfo->chi_Mix.chi_Multiply_R, Div = ChInfo->chi_Mix.chi_Divider;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      WORD *SmpArrayPos = SmpArray;
      LONG Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        *MixBuf++ += (((long long) SmpArray[CurrOffset] * Mul_L) / Div);
        *MixBuf++ += (((long long) SmpArray[CurrOffset] * Mul_R) / Div);

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

GetSecondAdvSample:
        *MixBuf++ += (((long long) SmpArray[CurrOffset] * Mul_L) / Div);
        *MixBuf++ += (((long long) SmpArray[CurrOffset] * Mul_R) / Div);

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    WORD *SmpArrayPos = SmpArray;
    LONG Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *--SmpArray;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        *MixBuf++ += (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        *MixBuf++ += (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      WORD WriteSample;
      LONG Sample_R;

      SmpArray += CurrOffset;

      WriteSample = *--SmpArray;
      Sample = (((long long) WriteSample * Mul_L) / Div);
      Sample_R = (((long long) WriteSample * Mul_R) / Div);

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;
        *MixBuf++ += Sample_R;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          WriteSample = *--SmpArray;
          Sample = (((long long) WriteSample * Mul_L) / Div);
          Sample_R = (((long long) WriteSample * Mul_R) / Div);
        }

GetSecondSample:
        *MixBuf++ += Sample;
        *MixBuf++ += Sample_R;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          WriteSample = *--SmpArray;
          Sample = (((long long) WriteSample * Mul_L) / Div);
          Sample_R = (((long long) WriteSample * Mul_R) / Div);
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIX_Stereo32 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *MixBuf = *MixBuffer;
  LONG  Mul_L = ChInfo->chi_Mix.chi_Multiply_L, Mul_R = ChInfo->chi_Mix.chi_Multiply_R, Div = ChInfo->chi_Mix.chi_Divider;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      LONG *SmpArrayPos = SmpArray;
      long long Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        *MixBuf++ += (((long long) SmpArray[CurrOffset] * Mul_L) / Div);
        *MixBuf++ += (((long long) SmpArray[CurrOffset] * Mul_R) / Div);

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

GetSecondAdvSample:
        *MixBuf++ += (((long long) SmpArray[CurrOffset] * Mul_L) / Div);
        *MixBuf++ += (((long long) SmpArray[CurrOffset] * Mul_R) / Div);

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    LONG *SmpArrayPos = SmpArray;
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *SmpArray++;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        *MixBuf++ += (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        *MixBuf++ += (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      LONG WriteSample;
      LONG Sample_R;

      SmpArray += CurrOffset;

      WriteSample = *SmpArray++;
      Sample = (((long long) WriteSample * Mul_L) / Div);
      Sample_R = (((long long) WriteSample * Mul_R) / Div);

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;
        *MixBuf++ += Sample_R;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          WriteSample = *SmpArray++;
          Sample = (((long long) WriteSample * Mul_L) / Div);
          Sample_R = (((long long) WriteSample * Mul_R) / Div);
        }

GetSecondSample:
        *MixBuf++ += Sample;
        *MixBuf++ += Sample_R;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          WriteSample = *SmpArray++;
          Sample = (((long long) WriteSample * Mul_L) / Div);
          Sample_R = (((long long) WriteSample * Mul_R) / Div);
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIXb_Stereo32 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *MixBuf = *MixBuffer;
  LONG  Mul_L = ChInfo->chi_Mix.chi_Multiply_L, Mul_R = ChInfo->chi_Mix.chi_Multiply_R, Div = ChInfo->chi_Mix.chi_Divider;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      LONG *SmpArrayPos = SmpArray;
      long long Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        *MixBuf++ += (((long long) SmpArray[CurrOffset] * Mul_L) / Div);
        *MixBuf++ += (((long long) SmpArray[CurrOffset] * Mul_R) / Div);

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

GetSecondAdvSample:
        *MixBuf++ += (((long long) SmpArray[CurrOffset] * Mul_L) / Div);
        *MixBuf++ += (((long long) SmpArray[CurrOffset] * Mul_R) / Div);

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    LONG *SmpArrayPos = SmpArray;
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *--SmpArray;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        *MixBuf++ += (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        *MixBuf++ += (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      LONG WriteSample;
      LONG Sample_R;

      SmpArray += CurrOffset;

      WriteSample = *--SmpArray;
      Sample = (((long long) WriteSample * Mul_L) / Div);
      Sample_R = (((long long) WriteSample * Mul_R) / Div);

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;
        *MixBuf++ += Sample_R;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          WriteSample = *--SmpArray;
          Sample = (((long long) WriteSample * Mul_L) / Div);
          Sample_R = (((long long) WriteSample * Mul_R) / Div);
        }

GetSecondSample:
        *MixBuf++ += Sample;
        *MixBuf++ += Sample_R;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          WriteSample = *--SmpArray;
          Sample = (((long long) WriteSample * Mul_L) / Div);
          Sample_R = (((long long) WriteSample * Mul_R) / Div);
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIX_StereoX ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *MixBuf = *MixBuffer;
  LONG  Mul_L = ChInfo->chi_Mix.chi_Multiply_L, Mul_R = ChInfo->chi_Mix.chi_Multiply_R, Div = ChInfo->chi_Mix.chi_Divider;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i, Bit, SampleData, BitDepth = ChInfo->chi_Mix.chi_BitDepth;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      long long Sample;
      ULONG InterpolDiv, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset += SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;

          Sample += SampleData;

          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);

GetSecondAdvInterpolSample:
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset += SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;

          Sample += SampleData;
          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      ULONG RealAdv = Advance * BitDepth, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpOffset = (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        *MixBuf++ += (((long long) SampleData * Mul_L) / Div);
        *MixBuf++ += (((long long) SampleData * Mul_R) / Div);

        CurrFraction += AdvFract;
        CurrOffset += Advance;
        Bit += RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset++;
          Bit += BitDepth;
        }

        SmpOffset += (Bit >> 5UL);
        Bit &= 31;

GetSecondAdvSample:
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        *MixBuf++ += (((long long) SampleData * Mul_L) / Div);
        *MixBuf++ += (((long long) SampleData * Mul_R) / Div);

        CurrFraction += AdvFract;
        CurrOffset += Advance;
        Bit += RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset++;
          Bit += BitDepth;
        }

        SmpOffset += (Bit >> 5UL);
        Bit &= 31;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray++ << Bit;
        SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

        Bit &= 31;
      }

      Bit += BitDepth;
      CurrOffset++;

      Sample = (LONG) SampleData;
      SampleValue = 0LL;

      if ( Length )
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) *SmpArray << Bit;
          SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
      }

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        *MixBuf++ += (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        *MixBuf++ += (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      LONG Sample_R;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray++ << Bit;
        SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

        Bit &= 31;
      }

      Bit += BitDepth;
      CurrOffset++;

      Sample = (((long long) SampleData * Mul_L) / Div);
      Sample_R = (((long long) SampleData * Mul_R) / Div);

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;
        *MixBuf++ += Sample_R;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = (((long long) SampleData * Mul_L) / Div);
          Sample_R = (((long long) SampleData * Mul_R) / Div);
        }

GetSecondSample:
        *MixBuf++ += Sample;
        *MixBuf++ += Sample_R;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = (((long long) SampleData * Mul_L) / Div);
          Sample_R = (((long long) SampleData * Mul_R) / Div);
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIXb_StereoX ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *MixBuf = *MixBuffer;
  LONG  Mul_L = ChInfo->chi_Mix.chi_Multiply_L, Mul_R = ChInfo->chi_Mix.chi_Multiply_R, Div = ChInfo->chi_Mix.chi_Divider;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i, Bit, SampleData, BitDepth = ChInfo->chi_Mix.chi_BitDepth;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      long long Sample;
      ULONG InterpolDiv, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset -= SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample += SampleData;
          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);

GetSecondAdvInterpolSample:
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset -= SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample += SampleData;
          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        *MixBuf++ += (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      ULONG RealAdv = Advance * BitDepth, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpOffset = (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        *MixBuf++ += (((long long) SampleData * Mul_L) / Div);
        *MixBuf++ += (((long long) SampleData * Mul_R) / Div);

        CurrFraction += AdvFract;
        CurrOffset -= Advance;
        Bit -= RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;
        }

        SmpOffset += ((LONG) Bit >> 5L);
        Bit &= 31;

GetSecondAdvSample:
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        *MixBuf++ += (((long long) SampleData * Mul_L) / Div);
        *MixBuf++ += (((long long) SampleData * Mul_R) / Div);

        CurrFraction += AdvFract;
        CurrOffset -= Advance;
        Bit -= RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;
        }

        SmpOffset += ((LONG) Bit >> 5L);
        Bit &= 31;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      CurrOffset--;
      Bit -= BitDepth;

      if ( (LONG) Bit < 0 )
      {
        SmpArray--;

        Bit &= 31;
      }

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray << Bit;
        SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
      }

      Sample = (LONG) SampleData;
      SampleValue = 0LL;

      if ( Length )
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) *SmpArray << Bit;
          SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
      }

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        *MixBuf++ += (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        *MixBuf++ += (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      LONG Sample_R;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      CurrOffset--;
      Bit -= BitDepth;

      if ( (LONG) Bit < 0 )
      {
        SmpArray--;

        Bit &= 31;
      }

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray << Bit;
        SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
      }

      Sample = (((long long) SampleData * Mul_L) / Div);
      Sample_R = (((long long) SampleData * Mul_R) / Div);

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;
        *MixBuf++ += Sample_R;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = (((long long) SampleData * Mul_L) / Div);
          Sample_R = (((long long) SampleData * Mul_R) / Div);
        }

GetSecondSample:
        *MixBuf++ += Sample;
        *MixBuf++ += Sample_R;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = (((long long) SampleData * Mul_L) / Div);
          Sample_R = (((long long) SampleData * Mul_R) / Div);
        }

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIX_Stereo168 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  WORD *SmpArray = (WORD *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable_L = ChInfo->chi_Mix.chi_VolTable_L;
  LONG *VolTable_R = ChInfo->chi_Mix.chi_VolTable_R;
  LONG *MixBuf = *MixBuffer;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      WORD *SmpArrayPos = SmpArray;
      LONG Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        Sample >>= 8U;
        *MixBuf++ += VolTable_L[(UWORD) Sample];
        *MixBuf++ += VolTable_R[(UWORD) Sample];

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        Sample >>= 8U;
        *MixBuf++ += VolTable_L[(UWORD) Sample];
        *MixBuf++ += VolTable_R[(UWORD) Sample];
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      LONG Sample;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        Sample = ((UWORD) SmpArray[CurrOffset] >> 8U);
        *MixBuf++ += VolTable_L[(UWORD) Sample];
        *MixBuf++ += VolTable_R[(UWORD) Sample];

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

GetSecondAdvSample:
        Sample = ((UWORD) SmpArray[CurrOffset] >> 8U);
        *MixBuf++ += VolTable_L[(UWORD) Sample];
        *MixBuf++ += VolTable_R[(UWORD) Sample];

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    UWORD WriteSample;
    WORD *SmpArrayPos = SmpArray;
    LONG Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *SmpArray++;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        WriteSample = ((UWORD) Sample >> 8U);
        *MixBuf++ += VolTable_L[(UWORD) WriteSample];
        *MixBuf++ += VolTable_R[(UWORD) WriteSample];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        WriteSample = ((UWORD) Sample >> 8U);
        *MixBuf++ += VolTable_L[(UWORD) WriteSample];
        *MixBuf++ += VolTable_R[(UWORD) WriteSample];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      LONG Sample_R;

      SmpArray += CurrOffset;

      WriteSample = (((UWORD) *SmpArray++ >> 8U));
      Sample = VolTable_L[(UWORD) WriteSample];
      Sample_R = VolTable_R[(UWORD) WriteSample];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;
        *MixBuf++ += Sample_R;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          WriteSample = (((UWORD) *SmpArray++ >> 8U));
          Sample = VolTable_L[(UWORD) WriteSample];
          Sample_R = VolTable_R[(UWORD) WriteSample];
        }

GetSecondSample:
        *MixBuf++ += Sample;
        *MixBuf++ += Sample_R;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          WriteSample = (((UWORD) *SmpArray++ >> 8U));
          Sample = VolTable_L[(UWORD) WriteSample];
          Sample_R = VolTable_R[(UWORD) WriteSample];
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIXb_Stereo168 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  WORD *SmpArray = (WORD *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable_L = ChInfo->chi_Mix.chi_VolTable_L;
  LONG *VolTable_R = ChInfo->chi_Mix.chi_VolTable_R;
  LONG *MixBuf = *MixBuffer;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      WORD *SmpArrayPos = SmpArray;
      LONG Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        Sample >>= 8U;
        *MixBuf++ += VolTable_L[(UWORD) Sample];
        *MixBuf++ += VolTable_R[(UWORD) Sample];
GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        Sample >>= 8U;
        *MixBuf++ += VolTable_L[(UWORD) Sample];
        *MixBuf++ += VolTable_R[(UWORD) Sample];
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      LONG Sample;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        Sample = ((UWORD) SmpArray[CurrOffset] >> 8U);
        *MixBuf++ += VolTable_L[(UWORD) Sample];
        *MixBuf++ += VolTable_R[(UWORD) Sample];

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

GetSecondAdvSample:
        Sample = ((UWORD) SmpArray[CurrOffset] >> 8U);
        *MixBuf++ += VolTable_L[(UWORD) Sample];
        *MixBuf++ += VolTable_R[(UWORD) Sample];

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    UWORD WriteSample;
    WORD *SmpArrayPos = SmpArray;
    LONG Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *--SmpArray;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        WriteSample = (UWORD) Sample >> 8U;
        *MixBuf++ += VolTable_L[(UWORD) WriteSample];
        *MixBuf++ += VolTable_R[(UWORD) WriteSample];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        WriteSample = (UWORD) Sample >> 8U;
        *MixBuf++ += VolTable_L[(UWORD) WriteSample];
        *MixBuf++ += VolTable_R[(UWORD) WriteSample];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      LONG Sample_R;

      SmpArray += CurrOffset;

      WriteSample = ((UWORD) *--SmpArray >> 8U);
      Sample = VolTable_L[(UWORD) WriteSample];
      Sample_R = VolTable_R[(UWORD) WriteSample];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;
        *MixBuf++ += Sample_R;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          WriteSample = ((UWORD) *--SmpArray >> 8U);
          Sample = VolTable_L[(UWORD) WriteSample];
          Sample_R = VolTable_R[(UWORD) WriteSample];
        }

GetSecondSample:
        *MixBuf++ += Sample;
        *MixBuf++ += Sample_R;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          WriteSample = ((UWORD) *--SmpArray >> 8U);
          Sample = VolTable_L[(UWORD) WriteSample];
          Sample_R = VolTable_R[(UWORD) WriteSample];
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIX_Stereo328 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable_L = ChInfo->chi_Mix.chi_VolTable_L;
  LONG *VolTable_R = ChInfo->chi_Mix.chi_VolTable_R;
  LONG *MixBuf = *MixBuffer;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      LONG *SmpArrayPos = SmpArray;
      long long Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        Sample >>= 24UL;
        *MixBuf++ += VolTable_L[(ULONG) Sample];
        *MixBuf++ += VolTable_R[(ULONG) Sample];

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        Sample >>= 24UL;
        *MixBuf++ += VolTable_L[(ULONG) Sample];
        *MixBuf++ += VolTable_R[(ULONG) Sample];
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      LONG Sample;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        Sample = ((ULONG) SmpArray[CurrOffset] >> 24UL);
        *MixBuf++ += VolTable_L[(ULONG) Sample];
        *MixBuf++ += VolTable_R[(ULONG) Sample];

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

GetSecondAdvSample:
        Sample = ((ULONG) SmpArray[CurrOffset] >> 24UL);
        *MixBuf++ += VolTable_L[(ULONG) Sample];
        *MixBuf++ += VolTable_R[(ULONG) Sample];

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    ULONG WriteSample;
    LONG *SmpArrayPos = SmpArray;
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *SmpArray++;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        WriteSample = (ULONG) Sample >> 24UL;
        *MixBuf++ += VolTable_L[(ULONG) WriteSample];
        *MixBuf++ += VolTable_R[(ULONG) WriteSample];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        WriteSample = (ULONG) Sample >> 24UL;
        *MixBuf++ += VolTable_L[(ULONG) WriteSample];
        *MixBuf++ += VolTable_R[(ULONG) WriteSample];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      LONG Sample_R;

      SmpArray += CurrOffset;

      WriteSample = ((ULONG) *SmpArray++ >> 24UL);
      Sample = VolTable_L[(ULONG) WriteSample];
      Sample_R = VolTable_R[(ULONG) WriteSample];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;
        *MixBuf++ += Sample_R;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          WriteSample = ((ULONG) *SmpArray++ >> 24UL);
          Sample = VolTable_L[(ULONG) WriteSample];
          Sample_R = VolTable_R[(ULONG) WriteSample];
        }

GetSecondSample:
        *MixBuf++ += Sample;
        *MixBuf++ += Sample_R;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          WriteSample = ((ULONG) *SmpArray++ >> 24UL);
          Sample = VolTable_L[(ULONG) WriteSample];
          Sample_R = VolTable_R[(ULONG) WriteSample];
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIXb_Stereo328 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable_L = ChInfo->chi_Mix.chi_VolTable_L;
  LONG *VolTable_R = ChInfo->chi_Mix.chi_VolTable_R;
  LONG *MixBuf = *MixBuffer;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      LONG *SmpArrayPos = SmpArray;
      long long Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        Sample >>= 24UL;
        *MixBuf++ += VolTable_L[(ULONG) Sample];
        *MixBuf++ += VolTable_R[(ULONG) Sample];

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        Sample >>= 24UL;
        *MixBuf++ += VolTable_L[(ULONG) Sample];
        *MixBuf++ += VolTable_R[(ULONG) Sample];
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      LONG Sample;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        Sample = ((ULONG) SmpArray[CurrOffset] >> 24UL);
        *MixBuf++ += VolTable_L[(ULONG) Sample];
        *MixBuf++ += VolTable_R[(ULONG) Sample];

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

GetSecondAdvSample:
        Sample = ((ULONG) SmpArray[CurrOffset] >> 24UL);
        *MixBuf++ += VolTable_L[(ULONG) Sample];
        *MixBuf++ += VolTable_R[(ULONG) Sample];

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    ULONG WriteSample;
    LONG *SmpArrayPos = SmpArray;
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *--SmpArray;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        WriteSample = ((ULONG) Sample >> 24UL);
        *MixBuf++ += VolTable_L[(ULONG) WriteSample];
        *MixBuf++ += VolTable_R[(ULONG) WriteSample];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        WriteSample = ((ULONG) Sample >> 24UL);
        *MixBuf++ += VolTable_L[(ULONG) WriteSample];
        *MixBuf++ += VolTable_R[(ULONG) WriteSample];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      LONG Sample_R;

      SmpArray += CurrOffset;

      WriteSample = (ULONG) *SmpArray++ >> 24UL;
      Sample = VolTable_L[(ULONG) WriteSample];
      Sample_R = VolTable_R[(ULONG) WriteSample];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;
        *MixBuf++ += Sample_R;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          WriteSample = (ULONG) *SmpArray++ >> 24UL;
          Sample = VolTable_L[(ULONG) WriteSample];
          Sample_R = VolTable_R[(ULONG) WriteSample];
        }

GetSecondSample:
        *MixBuf++ += Sample;
        *MixBuf++ += Sample_R;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          WriteSample = (ULONG) *SmpArray++ >> 24UL;
          Sample = VolTable_L[(ULONG) WriteSample];
          Sample_R = VolTable_R[(ULONG) WriteSample];
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIX_StereoX8 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable_L = ChInfo->chi_Mix.chi_VolTable_L;
  LONG *VolTable_R = ChInfo->chi_Mix.chi_VolTable_R;
  LONG *MixBuf = *MixBuffer;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i, Bit, SampleData, BitDepth = ChInfo->chi_Mix.chi_BitDepth;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      long long Sample;
      ULONG InterpolDiv, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset += SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;

          Sample += SampleData;

          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        Sample >>= 24UL;
        *MixBuf++ += VolTable_L[(ULONG) Sample];
        *MixBuf++ += VolTable_R[(ULONG) Sample];

GetSecondAdvInterpolSample:
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset += SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;

          Sample += SampleData;
          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        Sample >>= 24UL;
        *MixBuf++ += VolTable_L[(ULONG) Sample];
        *MixBuf++ += VolTable_R[(ULONG) Sample];
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      ULONG RealAdv = Advance * BitDepth, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpOffset = (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        SampleData >>= 24UL;
        *MixBuf++ += VolTable_L[(ULONG) SampleData];
        *MixBuf++ += VolTable_R[(ULONG) SampleData];

        CurrFraction += AdvFract;
        CurrOffset += Advance;
        Bit += RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset++;
          Bit += BitDepth;
        }

        SmpOffset += (Bit >> 5UL);
        Bit &= 31;

GetSecondAdvSample:
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }


        SampleData >>= 24UL;
        *MixBuf++ += VolTable_L[(ULONG) SampleData];
        *MixBuf++ += VolTable_R[(ULONG) SampleData];

        CurrFraction += AdvFract;
        CurrOffset += Advance;
        Bit += RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset++;
          Bit += BitDepth;
        }

        SmpOffset += (Bit >> 5UL);
        Bit &= 31;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    ULONG WriteSample;
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray++ << Bit;
        SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

        Bit &= 31;
      }

      Bit += BitDepth;
      CurrOffset++;

      Sample = (LONG) SampleData;
      SampleValue = 0LL;

      if ( Length )
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) *SmpArray << Bit;
          SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
      }

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        WriteSample = ((ULONG) Sample >> 24UL);
        *MixBuf++ += VolTable_L[(ULONG) WriteSample];
        *MixBuf++ += VolTable_R[(ULONG) WriteSample];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:

        WriteSample = ((ULONG) Sample >> 24UL);
        *MixBuf++ += VolTable_L[(ULONG) WriteSample];
        *MixBuf++ += VolTable_R[(ULONG) WriteSample];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      LONG Sample_R;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray++ << Bit;
        SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

        Bit &= 31;
      }

      Bit += BitDepth;
      CurrOffset++;

      WriteSample = (ULONG) SampleData >> 24UL;
      Sample = VolTable_L[(ULONG) WriteSample];
      Sample_R = VolTable_R[(ULONG) WriteSample];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;
        *MixBuf++ += Sample_R;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          WriteSample = (ULONG) SampleData >> 24UL;
          Sample = VolTable_L[(ULONG) WriteSample];
          Sample_R = VolTable_R[(ULONG) WriteSample];
        }

GetSecondSample:
        *MixBuf++ += Sample;
        *MixBuf++ += Sample_R;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          WriteSample = (ULONG) SampleData >> 24UL;
          Sample = VolTable_L[(ULONG) WriteSample];
          Sample_R = VolTable_R[(ULONG) WriteSample];
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIXb_StereoX8 ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable_L = ChInfo->chi_Mix.chi_VolTable_L;
  LONG *VolTable_R = ChInfo->chi_Mix.chi_VolTable_R;
  LONG *MixBuf = *MixBuffer;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i, Bit, SampleData, BitDepth = ChInfo->chi_Mix.chi_BitDepth;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      long long Sample;
      ULONG InterpolDiv, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset -= SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample += SampleData;
          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        Sample >>= 24UL;
        *MixBuf++ += VolTable_L[(ULONG) Sample];
        *MixBuf++ += VolTable_R[(ULONG) Sample];

GetSecondAdvInterpolSample:
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset -= SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample += SampleData;
          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        Sample >>= 24UL;
        *MixBuf++ += VolTable_L[(ULONG) Sample];
        *MixBuf++ += VolTable_R[(ULONG) Sample];
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      ULONG RealAdv = Advance * BitDepth, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpOffset = (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        SampleData >>= 24UL;
        *MixBuf++ += VolTable_L[(ULONG) SampleData];
        *MixBuf++ += VolTable_R[(ULONG) SampleData];

        CurrFraction += AdvFract;
        CurrOffset -= Advance;
        Bit -= RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;
        }

        SmpOffset += ((LONG) Bit >> 5L);
        Bit &= 31;

GetSecondAdvSample:
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        SampleData >>= 24UL;
        *MixBuf++ += VolTable_L[(ULONG) SampleData];
        *MixBuf++ += VolTable_R[(ULONG) SampleData];

        CurrFraction += AdvFract;
        CurrOffset -= Advance;
        Bit -= RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;
        }

        SmpOffset += ((LONG) Bit >> 5L);
        Bit &= 31;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    ULONG WriteSample;
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      CurrOffset--;
      Bit -= BitDepth;

      if ( (LONG) Bit < 0 )
      {
        SmpArray--;

        Bit &= 31;
      }

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray << Bit;
        SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
      }

      Sample = (LONG) SampleData;
      SampleValue = 0LL;

      if ( Length )
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) *SmpArray << Bit;
          SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
      }

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        WriteSample = ((ULONG) Sample >> 24UL);
        *MixBuf++ += VolTable_L[(ULONG) WriteSample];
        *MixBuf++ += VolTable_R[(ULONG) WriteSample];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        WriteSample = ((ULONG) Sample >> 24UL);
        *MixBuf++ += VolTable_L[(ULONG) WriteSample];
        *MixBuf++ += VolTable_R[(ULONG) WriteSample];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      LONG Sample_R;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      CurrOffset--;
      Bit -= BitDepth;

      if ( (LONG) Bit < 0 )
      {
        SmpArray--;

        Bit &= 31;
      }

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray << Bit;
        SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
      }

      WriteSample = (ULONG) SampleData >> 24UL;
      Sample = VolTable_L[(ULONG) WriteSample];
      Sample_R = VolTable_R[(ULONG) WriteSample];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;
        *MixBuf++ += Sample_R;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          WriteSample = (ULONG) SampleData >> 24UL;
          Sample = VolTable_L[(ULONG) WriteSample];
          Sample_R = VolTable_R[(ULONG) WriteSample];
        }

GetSecondSample:
        *MixBuf++ += Sample;
        *MixBuf++ += Sample_R;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          WriteSample = (ULONG) SampleData >> 24UL;
          Sample = VolTable_L[(ULONG) WriteSample];
          Sample_R = VolTable_R[(ULONG) WriteSample];
        }

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIX_Stereo8L ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  BYTE *SmpArray = (BYTE *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable = ChInfo->chi_Mix.chi_VolTable_L;
  LONG *MixBuf = *MixBuffer;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      BYTE *SmpArrayPos = SmpArray;
      LONG Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf += VolTable[(UBYTE) Sample];
        MixBuf += 2;

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf += VolTable[(UBYTE) Sample];
        MixBuf += 2;
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        *MixBuf += VolTable[(UBYTE) SmpArray[CurrOffset]];
        MixBuf += 2;

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

GetSecondAdvSample:
        *MixBuf += VolTable[(UBYTE) SmpArray[CurrOffset]];
        MixBuf += 2;

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    BYTE *SmpArrayPos = SmpArray;
    LONG Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *SmpArray++;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        *MixBuf += VolTable[(UBYTE) Sample];
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        *MixBuf += VolTable[(UBYTE) Sample];
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = VolTable[(UBYTE) *SmpArray++];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf += Sample;
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[(UBYTE) *SmpArray++];

GetSecondSample:
        *MixBuf += Sample;
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[(UBYTE) *SmpArray++];

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIXb_Stereo8L ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  BYTE *SmpArray = (BYTE *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable = ChInfo->chi_Mix.chi_VolTable_L;
  LONG *MixBuf = *MixBuffer;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      BYTE *SmpArrayPos = SmpArray;
      LONG Sample;
      ULONG InterpolDiv;

      SmpArray += (CurrOffset + 1);

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf += VolTable[(UBYTE) Sample];
        MixBuf += 2;

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf += VolTable[(UBYTE) Sample];
        MixBuf += 2;
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        *MixBuf += VolTable[(UBYTE) SmpArray[CurrOffset]];
        MixBuf += 2;

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

GetSecondAdvSample:
        *MixBuf += VolTable[(UBYTE) SmpArray[CurrOffset]];
        MixBuf += 2;

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    BYTE *SmpArrayPos = SmpArray;
    LONG Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += (CurrOffset + 1);

      Sample = *--SmpArray;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        *MixBuf += VolTable[(UBYTE) Sample];
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        *MixBuf += VolTable[(UBYTE) Sample];
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += (CurrOffset + 1);

      Sample = VolTable[(UBYTE) *--SmpArray];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf += Sample;
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[(UBYTE) *--SmpArray];

GetSecondSample:
        *MixBuf += Sample;
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[(UBYTE) *--SmpArray];

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIX_Stereo16L ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  WORD *SmpArray = (WORD *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *MixBuf = *MixBuffer;
  LONG  Mul_L = ChInfo->chi_Mix.chi_Multiply_L, Div = ChInfo->chi_Mix.chi_Divider;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      WORD *SmpArrayPos = SmpArray;
      LONG Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf += (((long long) Sample * Mul_L) / Div);
        MixBuf += 2;

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf += (((long long) Sample * Mul_L) / Div);
        MixBuf += 2;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        *MixBuf += (((long long) SmpArray[CurrOffset] * Mul_L) / Div);
        MixBuf += 2;

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

GetSecondAdvSample:
        *MixBuf += (((long long) SmpArray[CurrOffset] * Mul_L) / Div);
        MixBuf += 2;

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    WORD *SmpArrayPos = SmpArray;
    LONG Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *SmpArray++;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        *MixBuf += (((long long) Sample * Mul_L) / Div);
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        *MixBuf += (((long long) Sample * Mul_L) / Div);
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = (((long long) *SmpArray++ * Mul_L) / Div);

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf += Sample;
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = (((long long) *SmpArray++ * Mul_L) / Div);

GetSecondSample:
        *MixBuf += Sample;
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = (((long long) *SmpArray++ * Mul_L) / Div);

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIXb_Stereo16L ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  WORD *SmpArray = (WORD *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *MixBuf = *MixBuffer;
  LONG  Mul_L = ChInfo->chi_Mix.chi_Multiply_L, Div = ChInfo->chi_Mix.chi_Divider;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      WORD *SmpArrayPos = SmpArray;
      LONG Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf += (((long long) Sample * Mul_L) / Div);
        MixBuf += 2;

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf += (((long long) Sample * Mul_L) / Div);
        MixBuf += 2;
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        *MixBuf += (((long long) SmpArray[CurrOffset] * Mul_L) / Div);
        MixBuf += 2;

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

GetSecondAdvSample:
        *MixBuf += (((long long) SmpArray[CurrOffset] * Mul_L) / Div);
        MixBuf += 2;

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    WORD *SmpArrayPos = SmpArray;
    LONG Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *--SmpArray;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        *MixBuf += (((long long) Sample * Mul_L) / Div);
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        *MixBuf += (((long long) Sample * Mul_L) / Div);
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = (((long long) *--SmpArray * Mul_L) / Div);

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf += Sample;
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = (((long long) *--SmpArray * Mul_L) / Div);

GetSecondSample:
        *MixBuf += Sample;
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = (((long long) *--SmpArray * Mul_L) / Div);

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIX_Stereo32L ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *MixBuf = *MixBuffer;
  LONG  Mul_L = ChInfo->chi_Mix.chi_Multiply_L, Div = ChInfo->chi_Mix.chi_Divider;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      LONG *SmpArrayPos = SmpArray;
      long long Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf += (((long long) Sample * Mul_L) / Div);
        MixBuf += 2;

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf += (((long long) Sample * Mul_L) / Div);
        MixBuf += 2;
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        *MixBuf += (((long long) SmpArray[CurrOffset] * Mul_L) / Div);
        MixBuf += 2;

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

GetSecondAdvSample:
        *MixBuf += (((long long) SmpArray[CurrOffset] * Mul_L) / Div);
        MixBuf += 2;

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    LONG *SmpArrayPos = SmpArray;
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *SmpArray++;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        *MixBuf += (((long long) Sample * Mul_L) / Div);
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        *MixBuf += (((long long) Sample * Mul_L) / Div);
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = (((long long) *SmpArray++ * Mul_L) / Div);

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf += Sample;
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = (((long long) *SmpArray++ * Mul_L) / Div);

GetSecondSample:
        *MixBuf += Sample;
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = (((long long) *SmpArray++ * Mul_L) / Div);

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIXb_Stereo32L ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *MixBuf = *MixBuffer;
  LONG  Mul_L = ChInfo->chi_Mix.chi_Multiply_L, Div = ChInfo->chi_Mix.chi_Divider;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      LONG *SmpArrayPos = SmpArray;
      long long Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf += (((long long) Sample * Mul_L) / Div);
        MixBuf += 2;

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf += (((long long) Sample * Mul_L) / Div);
        MixBuf += 2;
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        *MixBuf += (((long long) SmpArray[CurrOffset] * Mul_L) / Div);
        MixBuf += 2;

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

GetSecondAdvSample:
        *MixBuf += (((long long) SmpArray[CurrOffset] * Mul_L) / Div);
        MixBuf += 2;

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    LONG *SmpArrayPos = SmpArray;
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *--SmpArray;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        *MixBuf += (((long long) Sample * Mul_L) / Div);
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        *MixBuf += (((long long) Sample * Mul_L) / Div);
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = (((long long) *--SmpArray * Mul_L) / Div);

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf += Sample;
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = (((long long) *--SmpArray * Mul_L) / Div);

GetSecondSample:
        *MixBuf += Sample;
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = (((long long) *--SmpArray * Mul_L) / Div);

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIX_StereoXL ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *MixBuf = *MixBuffer;
  LONG  Mul_L = ChInfo->chi_Mix.chi_Multiply_L, Div = ChInfo->chi_Mix.chi_Divider;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i, Bit, SampleData, BitDepth = ChInfo->chi_Mix.chi_BitDepth;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      long long Sample;
      ULONG InterpolDiv, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset += SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;

          Sample += SampleData;

          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        *MixBuf += (((long long) Sample * Mul_L) / Div);
        MixBuf += 2;

GetSecondAdvInterpolSample:
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset += SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;

          Sample += SampleData;
          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        *MixBuf += (((long long) Sample * Mul_L) / Div);
        MixBuf += 2;
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      ULONG RealAdv = Advance * BitDepth, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpOffset = (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        *MixBuf += (((long long) SampleData * Mul_L) / Div);
        MixBuf += 2;

        CurrFraction += AdvFract;
        CurrOffset += Advance;
        Bit += RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset++;
          Bit += BitDepth;
        }

        SmpOffset += (Bit >> 5UL);
        Bit &= 31;

GetSecondAdvSample:
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        *MixBuf += (((long long) SampleData * Mul_L) / Div);
        MixBuf += 2;

        CurrFraction += AdvFract;
        CurrOffset += Advance;
        Bit += RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset++;
          Bit += BitDepth;
        }

        SmpOffset += (Bit >> 5UL);
        Bit &= 31;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray++ << Bit;
        SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

        Bit &= 31;
      }

      Bit += BitDepth;
      CurrOffset++;

      Sample = (LONG) SampleData;
      SampleValue = 0LL;

      if ( Length )
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) *SmpArray << Bit;
          SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
      }

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        *MixBuf += (((long long) Sample * Mul_L) / Div);
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        *MixBuf += (((long long) Sample * Mul_L) / Div);
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray++ << Bit;
        SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

        Bit &= 31;
      }

      Bit += BitDepth;
      CurrOffset++;

      Sample = (((long long) SampleData * Mul_L) / Div);

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf += Sample;
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = (((long long) SampleData * Mul_L) / Div);
        }

GetSecondSample:
        *MixBuf += Sample;
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = (((long long) SampleData * Mul_L) / Div);
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIXb_StereoXL ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *MixBuf = *MixBuffer;
  LONG  Mul_L = ChInfo->chi_Mix.chi_Multiply_L, Div = ChInfo->chi_Mix.chi_Divider;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i, Bit, SampleData, BitDepth = ChInfo->chi_Mix.chi_BitDepth;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      long long Sample;
      ULONG InterpolDiv, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset -= SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample += SampleData;
          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        *MixBuf += (((long long) Sample * Mul_L) / Div);
        MixBuf += 2;

GetSecondAdvInterpolSample:
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset -= SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample += SampleData;
          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        *MixBuf += (((long long) Sample * Mul_L) / Div);
        MixBuf += 2;
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      ULONG RealAdv = Advance * BitDepth, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpOffset = (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        *MixBuf += (((long long) SampleData * Mul_L) / Div);
        MixBuf += 2;

        CurrFraction += AdvFract;
        CurrOffset -= Advance;
        Bit -= RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;
        }

        SmpOffset += ((LONG) Bit >> 5L);
        Bit &= 31;

GetSecondAdvSample:
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        *MixBuf += (((long long) SampleData * Mul_L) / Div);
        MixBuf += 2;

        CurrFraction += AdvFract;
        CurrOffset -= Advance;
        Bit -= RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;
        }

        SmpOffset += ((LONG) Bit >> 5L);
        Bit &= 31;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      CurrOffset--;
      Bit -= BitDepth;

      if ( (LONG) Bit < 0 )
      {
        SmpArray--;

        Bit &= 31;
      }

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray << Bit;
        SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
      }

      Sample = (LONG) SampleData;
      SampleValue = 0LL;

      if ( Length )
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) *SmpArray << Bit;
          SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
      }

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        *MixBuf += (((long long) Sample * Mul_L) / Div);
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        *MixBuf += (((long long) Sample * Mul_L) / Div);
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      CurrOffset--;
      Bit -= BitDepth;

      if ( (LONG) Bit < 0 )
      {
        SmpArray--;

        Bit &= 31;
      }

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray << Bit;
        SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
      }

      Sample = (((long long) SampleData * Mul_L) / Div);

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf += Sample;
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = (((long long) SampleData * Mul_L) / Div);
        }

GetSecondSample:
        *MixBuf += Sample;
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = (((long long) SampleData * Mul_L) / Div);
        }

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIX_Stereo168L ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  WORD *SmpArray = (WORD *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable = ChInfo->chi_Mix.chi_VolTable_L;
  LONG *MixBuf = *MixBuffer;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      WORD *SmpArrayPos = SmpArray;
      LONG Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf += VolTable[((UWORD) Sample >> 8U)];
        MixBuf += 2;

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf += VolTable[((UWORD) Sample >> 8U)];
        MixBuf += 2;
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        *MixBuf += VolTable[((UWORD) SmpArray[CurrOffset] >> 8U)];
        MixBuf += 2;

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

GetSecondAdvSample:
        *MixBuf += VolTable[((UWORD) SmpArray[CurrOffset] >> 8U)];
        MixBuf += 2;

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    WORD *SmpArrayPos = SmpArray;
    LONG Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *SmpArray++;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        *MixBuf += VolTable[((UWORD) Sample >> 8U)];
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        *MixBuf += VolTable[((UWORD) Sample >> 8U)];
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = VolTable[(((UWORD) *SmpArray++ >> 8U))];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf += Sample;
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[((UWORD) *SmpArray++ >> 8U)];

GetSecondSample:
        *MixBuf += Sample;
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[((UWORD) *SmpArray++ >> 8U)];

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIXb_Stereo168L ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  WORD *SmpArray = (WORD *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable = ChInfo->chi_Mix.chi_VolTable_L;
  LONG *MixBuf = *MixBuffer;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      WORD *SmpArrayPos = SmpArray;
      LONG Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf += VolTable[((UWORD) Sample >> 8U)];
        MixBuf += 2;

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf += VolTable[((UWORD) Sample >> 8U)];
        MixBuf += 2;
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        *MixBuf += VolTable[((UWORD) SmpArray[CurrOffset] >> 8U)];
        MixBuf += 2;

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

GetSecondAdvSample:
        *MixBuf += VolTable[((UWORD) SmpArray[CurrOffset] >> 8U)];
        MixBuf += 2;

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    WORD *SmpArrayPos = SmpArray;
    LONG Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *--SmpArray;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        *MixBuf += VolTable[((UWORD) Sample >> 8U)];
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        *MixBuf += VolTable[((UWORD) Sample >> 8U)];
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = VolTable[(((UWORD) *--SmpArray >> 8U))];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf += Sample;
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[((UWORD) *--SmpArray >> 8U)];

GetSecondSample:
        *MixBuf += Sample;
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[((UWORD) *--SmpArray >> 8U)];

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIX_Stereo328L ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable = ChInfo->chi_Mix.chi_VolTable_L;
  LONG *MixBuf = *MixBuffer;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      LONG *SmpArrayPos = SmpArray;
      long long Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf += VolTable[((ULONG) Sample >> 24UL)];
        MixBuf += 2;

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf += VolTable[((ULONG) Sample >> 24UL)];
        MixBuf += 2;
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        *MixBuf += VolTable[((ULONG) SmpArray[CurrOffset] >> 24UL)];
        MixBuf += 2;

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

GetSecondAdvSample:
        *MixBuf += VolTable[((ULONG) SmpArray[CurrOffset] >> 24UL)];
        MixBuf += 2;

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    LONG *SmpArrayPos = SmpArray;
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *SmpArray++;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        *MixBuf += VolTable[((ULONG) Sample >> 24UL)];
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        *MixBuf += VolTable[((ULONG) Sample >> 24UL)];
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = VolTable[(((ULONG) *SmpArray++ >> 24UL))];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf += Sample;
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[((ULONG) *SmpArray++ >> 24UL)];

GetSecondSample:
        *MixBuf += Sample;
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[((ULONG) *SmpArray++ >> 24UL)];

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIXb_Stereo328L ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable = ChInfo->chi_Mix.chi_VolTable_L;
  LONG *MixBuf = *MixBuffer;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      LONG *SmpArrayPos = SmpArray;
      long long Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf += VolTable[((ULONG) Sample >> 24UL)];
        MixBuf += 2;

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        *MixBuf += VolTable[((ULONG) Sample >> 24UL)];
        MixBuf += 2;
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        *MixBuf += VolTable[((ULONG) SmpArray[CurrOffset] >> 24UL)];
        MixBuf += 2;

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

GetSecondAdvSample:
        *MixBuf += VolTable[((ULONG) SmpArray[CurrOffset] >> 24UL)];
        MixBuf += 2;

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    LONG *SmpArrayPos = SmpArray;
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *--SmpArray;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        *MixBuf += VolTable[((ULONG) Sample >> 24UL)];
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        *MixBuf += VolTable[((ULONG) Sample >> 24UL)];
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = VolTable[(((ULONG) *SmpArray++ >> 24UL))];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf += Sample;
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[((ULONG) *--SmpArray >> 24UL)];

GetSecondSample:
        *MixBuf += Sample;
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[((ULONG) *--SmpArray >> 24UL)];

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIX_StereoX8L ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable = ChInfo->chi_Mix.chi_VolTable_L;
  LONG *MixBuf = *MixBuffer;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i, Bit, SampleData, BitDepth = ChInfo->chi_Mix.chi_BitDepth;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      long long Sample;
      ULONG InterpolDiv, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset += SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;

          Sample += SampleData;

          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        *MixBuf += VolTable[((ULONG) Sample >> 24UL)];
        MixBuf += 2;

GetSecondAdvInterpolSample:
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset += SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;

          Sample += SampleData;
          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        *MixBuf += VolTable[((ULONG) Sample >> 24UL)];
        MixBuf += 2;
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      ULONG RealAdv = Advance * BitDepth, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpOffset = (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        *MixBuf += VolTable[((ULONG) SampleData >> 24UL)];
        MixBuf += 2;

        CurrFraction += AdvFract;
        CurrOffset += Advance;
        Bit += RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset++;
          Bit += BitDepth;
        }

        SmpOffset += (Bit >> 5UL);
        Bit &= 31;

GetSecondAdvSample:
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        *MixBuf += VolTable[((ULONG) SampleData >> 24UL)];
        MixBuf += 2;

        CurrFraction += AdvFract;
        CurrOffset += Advance;
        Bit += RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset++;
          Bit += BitDepth;
        }

        SmpOffset += (Bit >> 5UL);
        Bit &= 31;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray++ << Bit;
        SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

        Bit &= 31;
      }

      Bit += BitDepth;
      CurrOffset++;

      Sample = (LONG) SampleData;
      SampleValue = 0LL;

      if ( Length )
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) *SmpArray << Bit;
          SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
      }

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        *MixBuf += VolTable[((ULONG) Sample >> 24UL)];
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        *MixBuf += VolTable[((ULONG) Sample >> 24UL)];
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray++ << Bit;
        SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

        Bit &= 31;
      }

      Bit += BitDepth;
      CurrOffset++;

      Sample = VolTable[((ULONG) SampleData >> 24UL)];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf += Sample;
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = VolTable[((ULONG) SampleData >> 24UL)];
        }

GetSecondSample:
        *MixBuf += Sample;
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = VolTable[((ULONG) SampleData >> 24UL)];
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIXb_StereoX8L ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable = ChInfo->chi_Mix.chi_VolTable_L;
  LONG *MixBuf = *MixBuffer;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i, Bit, SampleData, BitDepth = ChInfo->chi_Mix.chi_BitDepth;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      long long Sample;
      ULONG InterpolDiv, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset -= SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample += SampleData;
          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        *MixBuf += VolTable[((ULONG) Sample >> 24UL)];
        MixBuf += 2;

GetSecondAdvInterpolSample:
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset -= SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample += SampleData;
          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        *MixBuf += VolTable[((ULONG) Sample >> 24UL)];
        MixBuf += 2;
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      ULONG RealAdv = Advance * BitDepth, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpOffset = (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        *MixBuf += VolTable[((ULONG) SampleData >> 24UL)];
        MixBuf += 2;

        CurrFraction += AdvFract;
        CurrOffset -= Advance;
        Bit -= RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;
        }

        SmpOffset += ((LONG) Bit >> 5L);
        Bit &= 31;

GetSecondAdvSample:
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        *MixBuf += VolTable[((ULONG) SampleData >> 24UL)];
        MixBuf += 2;

        CurrFraction += AdvFract;
        CurrOffset -= Advance;
        Bit -= RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;
        }

        SmpOffset += ((LONG) Bit >> 5L);
        Bit &= 31;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      CurrOffset--;
      Bit -= BitDepth;

      if ( (LONG) Bit < 0 )
      {
        SmpArray--;

        Bit &= 31;
      }

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray << Bit;
        SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
      }

      Sample = (LONG) SampleData;
      SampleValue = 0LL;

      if ( Length )
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) *SmpArray << Bit;
          SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
      }

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        *MixBuf += VolTable[((ULONG) Sample >> 24UL)];
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        *MixBuf += VolTable[((ULONG) Sample >> 24UL)];
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      CurrOffset--;
      Bit -= BitDepth;

      if ( (LONG) Bit < 0 )
      {
        SmpArray--;

        Bit &= 31;
      }

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray << Bit;
        SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
      }

      Sample = VolTable[((ULONG) SampleData >> 24UL)];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf += Sample;
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = VolTable[((ULONG) SampleData >> 24UL)];
        }

GetSecondSample:
        *MixBuf += Sample;
        MixBuf += 2;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = VolTable[((ULONG) SampleData >> 24UL)];
        }

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIX_Stereo8R ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  BYTE *SmpArray = (BYTE *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable = ChInfo->chi_Mix.chi_VolTable_R;
  LONG *MixBuf = *MixBuffer;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      BYTE *SmpArrayPos = SmpArray;
      LONG Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        MixBuf++;
        *MixBuf++ += VolTable[(UBYTE) Sample];

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        MixBuf++;
        *MixBuf++ += VolTable[(UBYTE) Sample];
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        MixBuf++;
        *MixBuf++ += VolTable[(UBYTE) SmpArray[CurrOffset]];

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

GetSecondAdvSample:
        MixBuf++;
        *MixBuf++ += VolTable[(UBYTE) SmpArray[CurrOffset]];

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    BYTE *SmpArrayPos = SmpArray;
    LONG Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *SmpArray++;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        MixBuf++;
        *MixBuf++ += VolTable[(UBYTE) Sample];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        MixBuf++;
        *MixBuf++ += VolTable[(UBYTE) Sample];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = VolTable[(UBYTE) *SmpArray++];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        MixBuf++;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[(UBYTE) *SmpArray++];

GetSecondSample:
        MixBuf++;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[(UBYTE) *SmpArray++];

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIXb_Stereo8R ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  BYTE *SmpArray = (BYTE *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable = ChInfo->chi_Mix.chi_VolTable_R;
  LONG *MixBuf = *MixBuffer;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      BYTE *SmpArrayPos = SmpArray;
      LONG Sample;
      ULONG InterpolDiv;

      SmpArray += (CurrOffset + 1);

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        MixBuf++;
        *MixBuf++ += VolTable[(UBYTE) Sample];

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        MixBuf++;
        *MixBuf++ += VolTable[(UBYTE) Sample];
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        MixBuf++;
        *MixBuf++ += VolTable[(UBYTE) SmpArray[CurrOffset]];

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

GetSecondAdvSample:
        MixBuf++;
        *MixBuf++ += VolTable[(UBYTE) SmpArray[CurrOffset]];

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    BYTE *SmpArrayPos = SmpArray;
    LONG Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += (CurrOffset + 1);

      Sample = *--SmpArray;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        MixBuf++;
        *MixBuf++ += VolTable[(UBYTE) Sample];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        MixBuf++;
        *MixBuf++ += VolTable[(UBYTE) Sample];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += (CurrOffset + 1);

      Sample = VolTable[(UBYTE) *--SmpArray];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        MixBuf++;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[(UBYTE) *--SmpArray];

GetSecondSample:
        MixBuf++;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[(UBYTE) *--SmpArray];

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIX_Stereo16R ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  WORD *SmpArray = (WORD *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *MixBuf = *MixBuffer;
  LONG  Mul_R = ChInfo->chi_Mix.chi_Multiply_R, Div = ChInfo->chi_Mix.chi_Divider;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      WORD *SmpArrayPos = SmpArray;
      LONG Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        MixBuf++;
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        MixBuf++;
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        MixBuf++;
        *MixBuf++ += (((long long) SmpArray[CurrOffset] * Mul_R) / Div);

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

GetSecondAdvSample:
        MixBuf++;
        *MixBuf++ += (((long long) SmpArray[CurrOffset] * Mul_R) / Div);

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    WORD *SmpArrayPos = SmpArray;
    LONG Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *SmpArray++;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        MixBuf++;
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        MixBuf++;
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = (((long long) *SmpArray++ * Mul_R) / Div);

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        MixBuf++;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = (((long long) *SmpArray++ * Mul_R) / Div);

GetSecondSample:
        MixBuf++;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = (((long long) *SmpArray++ * Mul_R) / Div);

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIXb_Stereo16R ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  WORD *SmpArray = (WORD *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *MixBuf = *MixBuffer;
  LONG  Mul_R = ChInfo->chi_Mix.chi_Multiply_R, Div = ChInfo->chi_Mix.chi_Divider;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      WORD *SmpArrayPos = SmpArray;
      LONG Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        MixBuf++;
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        MixBuf++;
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        MixBuf++;
        *MixBuf++ += (((long long) SmpArray[CurrOffset] * Mul_R) / Div);

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

GetSecondAdvSample:
        MixBuf++;
        *MixBuf++ += (((long long) SmpArray[CurrOffset] * Mul_R) / Div);

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    WORD *SmpArrayPos = SmpArray;
    LONG Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *--SmpArray;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        MixBuf++;
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        MixBuf++;
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = (((long long) *--SmpArray * Mul_R) / Div);

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        MixBuf++;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = (((long long) *--SmpArray * Mul_R) / Div);

GetSecondSample:
        MixBuf++;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = (((long long) *--SmpArray * Mul_R) / Div);

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIX_Stereo32R ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *MixBuf = *MixBuffer;
  LONG  Mul_R = ChInfo->chi_Mix.chi_Multiply_R, Div = ChInfo->chi_Mix.chi_Divider;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      LONG *SmpArrayPos = SmpArray;
      long long Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        MixBuf++;
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        MixBuf++;
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        MixBuf++;
        *MixBuf++ += (((long long) SmpArray[CurrOffset] * Mul_R) / Div);

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

GetSecondAdvSample:
        MixBuf++;
        *MixBuf++ += (((long long) SmpArray[CurrOffset] * Mul_R) / Div);

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    LONG *SmpArrayPos = SmpArray;
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *SmpArray++;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        MixBuf++;
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        MixBuf++;
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = (((long long) *SmpArray++ * Mul_R) / Div);

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        MixBuf++;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = (((long long) *SmpArray++ * Mul_R) / Div);

GetSecondSample:
        MixBuf++;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = (((long long) *SmpArray++ * Mul_R) / Div);

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIXb_Stereo32R ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *MixBuf = *MixBuffer;
  LONG  Mul_R = ChInfo->chi_Mix.chi_Multiply_R, Div = ChInfo->chi_Mix.chi_Divider;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      LONG *SmpArrayPos = SmpArray;
      long long Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        MixBuf++;
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        MixBuf++;
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        MixBuf++;
        *MixBuf++ += (((long long) SmpArray[CurrOffset] * Mul_R) / Div);

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

GetSecondAdvSample:
        MixBuf++;
        *MixBuf++ += (((long long) SmpArray[CurrOffset] * Mul_R) / Div);

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    LONG *SmpArrayPos = SmpArray;
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *--SmpArray;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        MixBuf++;
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        MixBuf++;
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = (((long long) *--SmpArray * Mul_R) / Div);

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        MixBuf++;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = (((long long) *--SmpArray * Mul_R) / Div);

GetSecondSample:
        MixBuf++;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = (((long long) *--SmpArray * Mul_R) / Div);

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIX_StereoXR ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *MixBuf = *MixBuffer;
  LONG  Mul_R = ChInfo->chi_Mix.chi_Multiply_R, Div = ChInfo->chi_Mix.chi_Divider;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i, Bit, SampleData, BitDepth = ChInfo->chi_Mix.chi_BitDepth;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      long long Sample;
      ULONG InterpolDiv, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset += SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;

          Sample += SampleData;

          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        MixBuf++;
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);

GetSecondAdvInterpolSample:
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset += SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;

          Sample += SampleData;
          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        MixBuf++;
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      ULONG RealAdv = Advance * BitDepth, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpOffset = (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        MixBuf++;
        *MixBuf++ += (((long long) SampleData * Mul_R) / Div);

        CurrFraction += AdvFract;
        CurrOffset += Advance;
        Bit += RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset++;
          Bit += BitDepth;
        }

        SmpOffset += (Bit >> 5UL);
        Bit &= 31;

GetSecondAdvSample:
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        MixBuf++;
        *MixBuf++ += (((long long) SampleData * Mul_R) / Div);

        CurrFraction += AdvFract;
        CurrOffset += Advance;
        Bit += RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset++;
          Bit += BitDepth;
        }

        SmpOffset += (Bit >> 5UL);
        Bit &= 31;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray++ << Bit;
        SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

        Bit &= 31;
      }

      Bit += BitDepth;
      CurrOffset++;

      Sample = (LONG) SampleData;
      SampleValue = 0LL;

      if ( Length )
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) *SmpArray << Bit;
          SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
      }

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        MixBuf++;
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        MixBuf++;
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray++ << Bit;
        SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

        Bit &= 31;
      }

      Bit += BitDepth;
      CurrOffset++;

      Sample = (((long long) SampleData * Mul_R) / Div);

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        MixBuf++;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = (((long long) SampleData * Mul_R) / Div);
        }

GetSecondSample:
        MixBuf++;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = (((long long) SampleData * Mul_R) / Div);
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIXb_StereoXR ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *MixBuf = *MixBuffer;
  LONG  Mul_R = ChInfo->chi_Mix.chi_Multiply_R, Div = ChInfo->chi_Mix.chi_Divider;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i, Bit, SampleData, BitDepth = ChInfo->chi_Mix.chi_BitDepth;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      long long Sample;
      ULONG InterpolDiv, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset -= SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample += SampleData;
          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        MixBuf++;
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);

GetSecondAdvInterpolSample:
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset -= SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample += SampleData;
          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        MixBuf++;
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      ULONG RealAdv = Advance * BitDepth, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpOffset = (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        MixBuf++;
        *MixBuf++ += (((long long) SampleData * Mul_R) / Div);

        CurrFraction += AdvFract;
        CurrOffset -= Advance;
        Bit -= RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;
        }

        SmpOffset += ((LONG) Bit >> 5L);
        Bit &= 31;

GetSecondAdvSample:
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        MixBuf++;
        *MixBuf++ += (((long long) SampleData * Mul_R) / Div);

        CurrFraction += AdvFract;
        CurrOffset -= Advance;
        Bit -= RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;
        }

        SmpOffset += ((LONG) Bit >> 5L);
        Bit &= 31;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      CurrOffset--;
      Bit -= BitDepth;

      if ( (LONG) Bit < 0 )
      {
        SmpArray--;

        Bit &= 31;
      }

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray << Bit;
        SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
      }

      Sample = (LONG) SampleData;
      SampleValue = 0LL;

      if ( Length )
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) *SmpArray << Bit;
          SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
      }

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        MixBuf++;
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        MixBuf++;
        *MixBuf++ += (((long long) Sample * Mul_R) / Div);

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      CurrOffset--;
      Bit -= BitDepth;

      if ( (LONG) Bit < 0 )
      {
        SmpArray--;

        Bit &= 31;
      }

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray << Bit;
        SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
      }

      Sample = (((long long) SampleData * Mul_R) / Div);

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        MixBuf++;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = (((long long) SampleData * Mul_R) / Div);
        }

GetSecondSample:
        MixBuf++;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = (((long long) SampleData * Mul_R) / Div);
        }

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIX_Stereo168R ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  WORD *SmpArray = (WORD *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable = ChInfo->chi_Mix.chi_VolTable_R;
  LONG *MixBuf = *MixBuffer;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      WORD *SmpArrayPos = SmpArray;
      LONG Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        MixBuf++;
        *MixBuf++ += VolTable[((UWORD) Sample >> 8U)];

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        MixBuf++;
        *MixBuf++ += VolTable[((UWORD) Sample >> 8U)];
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        MixBuf++;
        *MixBuf++ += VolTable[((UWORD) SmpArray[CurrOffset] >> 8U)];

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

GetSecondAdvSample:
        MixBuf++;
        *MixBuf++ += VolTable[((UWORD) SmpArray[CurrOffset] >> 8U)];

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    WORD *SmpArrayPos = SmpArray;
    LONG Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *SmpArray++;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        MixBuf++;
        *MixBuf++ += VolTable[((UWORD) Sample >> 8U)];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        MixBuf++;
        *MixBuf++ += VolTable[((UWORD) Sample >> 8U)];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = VolTable[(((UWORD) *SmpArray++ >> 8U))];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        MixBuf++;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[((UWORD) *SmpArray++ >> 8U)];

GetSecondSample:
        MixBuf++;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[((UWORD) *SmpArray++ >> 8U)];

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIXb_Stereo168R ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  WORD *SmpArray = (WORD *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable = ChInfo->chi_Mix.chi_VolTable_R;
  LONG *MixBuf = *MixBuffer;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      WORD *SmpArrayPos = SmpArray;
      LONG Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        MixBuf++;
        *MixBuf++ += VolTable[((UWORD) Sample >> 8U)];

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        MixBuf++;
        *MixBuf++ += VolTable[((UWORD) Sample >> 8U)];
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        MixBuf++;
        *MixBuf++ += VolTable[((UWORD) SmpArray[CurrOffset] >> 8U)];

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

GetSecondAdvSample:
        MixBuf++;
        *MixBuf++ += VolTable[((UWORD) SmpArray[CurrOffset] >> 8U)];

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    WORD *SmpArrayPos = SmpArray;
    LONG Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *--SmpArray;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        MixBuf++;
        *MixBuf++ += VolTable[((UWORD) Sample >> 8U)];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        MixBuf++;
        *MixBuf++ += VolTable[((UWORD) Sample >> 8U)];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = VolTable[(((UWORD) *--SmpArray >> 8U))];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        MixBuf++;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[((UWORD) *--SmpArray >> 8U)];

GetSecondSample:
        MixBuf++;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[((UWORD) *--SmpArray >> 8U)];

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIX_Stereo328R ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable = ChInfo->chi_Mix.chi_VolTable_R;
  LONG *MixBuf = *MixBuffer;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      LONG *SmpArrayPos = SmpArray;
      long long Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        MixBuf++;
        *MixBuf++ += VolTable[((ULONG) Sample >> 24UL)];

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        MixBuf++;
        *MixBuf++ += VolTable[((ULONG) Sample >> 24UL)];
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        MixBuf++;
        *MixBuf++ += VolTable[((ULONG) SmpArray[CurrOffset] >> 24UL)];

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

GetSecondAdvSample:
        MixBuf++;
        *MixBuf++ += VolTable[((ULONG) SmpArray[CurrOffset] >> 24UL)];

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    LONG *SmpArrayPos = SmpArray;
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *SmpArray++;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        MixBuf++;
        *MixBuf++ += VolTable[((ULONG) Sample >> 24UL)];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        MixBuf++;
        *MixBuf++ += VolTable[((ULONG) Sample >> 24UL)];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = VolTable[(((ULONG) *SmpArray++ >> 24UL))];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        MixBuf++;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[((ULONG) *SmpArray++ >> 24UL)];

GetSecondSample:
        MixBuf++;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[((ULONG) *SmpArray++ >> 24UL)];

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIXb_Stereo328R ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable = ChInfo->chi_Mix.chi_VolTable_R;
  LONG *MixBuf = *MixBuffer;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      LONG *SmpArrayPos = SmpArray;
      long long Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        MixBuf++;
        *MixBuf++ += VolTable[((ULONG) Sample >> 24UL)];

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        MixBuf++;
        *MixBuf++ += VolTable[((ULONG) Sample >> 24UL)];
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        MixBuf++;
        *MixBuf++ += VolTable[((ULONG) SmpArray[CurrOffset] >> 24UL)];

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

GetSecondAdvSample:
        MixBuf++;
        *MixBuf++ += VolTable[((ULONG) SmpArray[CurrOffset] >> 24UL)];

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    LONG *SmpArrayPos = SmpArray;
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *--SmpArray;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        MixBuf++;
        *MixBuf++ += VolTable[((ULONG) Sample >> 24UL)];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        MixBuf++;
        *MixBuf++ += VolTable[((ULONG) Sample >> 24UL)];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = VolTable[(((ULONG) *SmpArray++ >> 24UL))];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        MixBuf++;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[((ULONG) *--SmpArray >> 24UL)];

GetSecondSample:
        MixBuf++;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[((ULONG) *--SmpArray >> 24UL)];

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIX_StereoX8R ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable = ChInfo->chi_Mix.chi_VolTable_R;
  LONG *MixBuf = *MixBuffer;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i, Bit, SampleData, BitDepth = ChInfo->chi_Mix.chi_BitDepth;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      long long Sample;
      ULONG InterpolDiv, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset += SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;

          Sample += SampleData;

          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        MixBuf++;
        *MixBuf++ += VolTable[((ULONG) Sample >> 24UL)];

GetSecondAdvInterpolSample:
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset += SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;

          Sample += SampleData;
          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        MixBuf++;
        *MixBuf++ += VolTable[((ULONG) Sample >> 24UL)];
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      ULONG RealAdv = Advance * BitDepth, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpOffset = (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        MixBuf++;
        *MixBuf++ += VolTable[((ULONG) SampleData >> 24UL)];

        CurrFraction += AdvFract;
        CurrOffset += Advance;
        Bit += RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset++;
          Bit += BitDepth;
        }

        SmpOffset += (Bit >> 5UL);
        Bit &= 31;

GetSecondAdvSample:
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        MixBuf++;
        *MixBuf++ += VolTable[((ULONG) SampleData >> 24UL)];

        CurrFraction += AdvFract;
        CurrOffset += Advance;
        Bit += RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset++;
          Bit += BitDepth;
        }

        SmpOffset += (Bit >> 5UL);
        Bit &= 31;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray++ << Bit;
        SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

        Bit &= 31;
      }

      Bit += BitDepth;
      CurrOffset++;

      Sample = (LONG) SampleData;
      SampleValue = 0LL;

      if ( Length )
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) *SmpArray << Bit;
          SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
      }

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        MixBuf++;
        *MixBuf++ += VolTable[((ULONG) Sample >> 24UL)];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        MixBuf++;
        *MixBuf++ += VolTable[((ULONG) Sample >> 24UL)];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray++ << Bit;
        SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

        Bit &= 31;
      }

      Bit += BitDepth;
      CurrOffset++;

      Sample = VolTable[((ULONG) SampleData >> 24UL)];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        MixBuf++;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = VolTable[((ULONG) SampleData >> 24UL)];
        }

GetSecondSample:
        MixBuf++;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = VolTable[((ULONG) SampleData >> 24UL)];
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIXb_StereoX8R ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable = ChInfo->chi_Mix.chi_VolTable_R;
  LONG *MixBuf = *MixBuffer;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i, Bit, SampleData, BitDepth = ChInfo->chi_Mix.chi_BitDepth;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      long long Sample;
      ULONG InterpolDiv, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset -= SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample += SampleData;
          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        MixBuf++;
        *MixBuf++ += VolTable[((ULONG) Sample >> 24UL)];

GetSecondAdvInterpolSample:
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset -= SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample += SampleData;
          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        MixBuf++;
        *MixBuf++ += VolTable[((ULONG) Sample >> 24UL)];
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      ULONG RealAdv = Advance * BitDepth, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpOffset = (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        MixBuf++;
        *MixBuf++ += VolTable[((ULONG) SampleData >> 24UL)];

        CurrFraction += AdvFract;
        CurrOffset -= Advance;
        Bit -= RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;
        }

        SmpOffset += ((LONG) Bit >> 5L);
        Bit &= 31;

GetSecondAdvSample:
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        MixBuf++;
        *MixBuf++ += VolTable[((ULONG) SampleData >> 24UL)];

        CurrFraction += AdvFract;
        CurrOffset -= Advance;
        Bit -= RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;
        }

        SmpOffset += ((LONG) Bit >> 5L);
        Bit &= 31;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      CurrOffset--;
      Bit -= BitDepth;

      if ( (LONG) Bit < 0 )
      {
        SmpArray--;

        Bit &= 31;
      }

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray << Bit;
        SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
      }

      Sample = (LONG) SampleData;
      SampleValue = 0LL;

      if ( Length )
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) *SmpArray << Bit;
          SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
      }

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        MixBuf++;
        *MixBuf++ += VolTable[((ULONG) Sample >> 24UL)];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        MixBuf++;
        *MixBuf++ += VolTable[((ULONG) Sample >> 24UL)];

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      CurrOffset--;
      Bit -= BitDepth;

      if ( (LONG) Bit < 0 )
      {
        SmpArray--;

        Bit &= 31;
      }

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray << Bit;
        SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
      }

      Sample = VolTable[((ULONG) SampleData >> 24UL)];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        MixBuf++;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = VolTable[((ULONG) SampleData >> 24UL)];
        }

GetSecondSample:
        MixBuf++;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = VolTable[((ULONG) SampleData >> 24UL)];
        }

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIX_Stereo8C ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  BYTE *SmpArray = (BYTE *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable = ChInfo->chi_Mix.chi_VolTable_L;
  LONG *MixBuf = *MixBuffer;
  LONG  WriteSample;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      BYTE *SmpArrayPos = SmpArray;
      LONG Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        WriteSample = VolTable[(UBYTE) Sample];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        WriteSample = VolTable[(UBYTE) Sample];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        WriteSample = VolTable[(UBYTE) SmpArray[CurrOffset]];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

GetSecondAdvSample:
        WriteSample = VolTable[(UBYTE) SmpArray[CurrOffset]];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    BYTE *SmpArrayPos = SmpArray;
    LONG Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *SmpArray++;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        WriteSample = VolTable[(UBYTE) Sample];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        WriteSample = VolTable[(UBYTE) Sample];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = VolTable[(UBYTE) *SmpArray++];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[(UBYTE) *SmpArray++];

GetSecondSample:
        *MixBuf++ += Sample;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[(UBYTE) *SmpArray++];

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIXb_Stereo8C ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  BYTE *SmpArray = (BYTE *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable = ChInfo->chi_Mix.chi_VolTable_L;
  LONG *MixBuf = *MixBuffer;
  LONG  WriteSample;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      BYTE *SmpArrayPos = SmpArray;
      LONG Sample;
      ULONG InterpolDiv;

      SmpArray += (CurrOffset + 1);

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        WriteSample = VolTable[(UBYTE) Sample];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        WriteSample = VolTable[(UBYTE) Sample];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        WriteSample = VolTable[(UBYTE) SmpArray[CurrOffset]];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

GetSecondAdvSample:
        WriteSample = VolTable[(UBYTE) SmpArray[CurrOffset]];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    BYTE *SmpArrayPos = SmpArray;
    LONG Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += (CurrOffset + 1);

      Sample = *--SmpArray;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        WriteSample = VolTable[(UBYTE) Sample];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        WriteSample = VolTable[(UBYTE) Sample];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += (CurrOffset + 1);

      Sample = VolTable[(UBYTE) *--SmpArray];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[(UBYTE) *--SmpArray];

GetSecondSample:
        *MixBuf++ += Sample;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[(UBYTE) *--SmpArray];

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIX_Stereo16C ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  WORD *SmpArray = (WORD *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *MixBuf = *MixBuffer;
  LONG  Mul_L = ChInfo->chi_Mix.chi_Multiply_L, Div = ChInfo->chi_Mix.chi_Divider;
  LONG  WriteSample;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      WORD *SmpArrayPos = SmpArray;
      LONG Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        WriteSample = (((long long) SmpArray[CurrOffset] * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

GetSecondAdvSample:
        WriteSample = (((long long) SmpArray[CurrOffset] * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    WORD *SmpArrayPos = SmpArray;
    LONG Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *SmpArray++;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = (((long long) *SmpArray++ * Mul_L) / Div);

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = (((long long) *SmpArray++ * Mul_L) / Div);

GetSecondSample:
        *MixBuf++ += Sample;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = (((long long) *SmpArray++ * Mul_L) / Div);

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIXb_Stereo16C ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  WORD *SmpArray = (WORD *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *MixBuf = *MixBuffer;
  LONG  Mul_L = ChInfo->chi_Mix.chi_Multiply_L, Div = ChInfo->chi_Mix.chi_Divider;
  LONG  WriteSample;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      WORD *SmpArrayPos = SmpArray;
      LONG Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        WriteSample = (((long long) SmpArray[CurrOffset] * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

GetSecondAdvSample:
        WriteSample = (((long long) SmpArray[CurrOffset] * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    WORD *SmpArrayPos = SmpArray;
    LONG Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *--SmpArray;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = (((long long) *--SmpArray * Mul_L) / Div);

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = (((long long) *--SmpArray * Mul_L) / Div);

GetSecondSample:
        *MixBuf++ += Sample;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = (((long long) *--SmpArray * Mul_L) / Div);

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIX_Stereo32C ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *MixBuf = *MixBuffer;
  LONG  Mul_L = ChInfo->chi_Mix.chi_Multiply_L, Div = ChInfo->chi_Mix.chi_Divider;
  LONG  WriteSample;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      LONG *SmpArrayPos = SmpArray;
      long long Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        WriteSample = (((long long) SmpArray[CurrOffset] * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

GetSecondAdvSample:
        WriteSample = (((long long) SmpArray[CurrOffset] * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    LONG *SmpArrayPos = SmpArray;
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *SmpArray++;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = (((long long) *SmpArray++ * Mul_L) / Div);

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = (((long long) *SmpArray++ * Mul_L) / Div);

GetSecondSample:
        *MixBuf++ += Sample;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = (((long long) *SmpArray++ * Mul_L) / Div);

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIXb_Stereo32C ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *MixBuf = *MixBuffer;
  LONG  Mul_L = ChInfo->chi_Mix.chi_Multiply_L, Div = ChInfo->chi_Mix.chi_Divider;
  LONG  WriteSample;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      LONG *SmpArrayPos = SmpArray;
      long long Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        WriteSample = (((long long) SmpArray[CurrOffset] * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

GetSecondAdvSample:
        WriteSample = (((long long) SmpArray[CurrOffset] * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    LONG *SmpArrayPos = SmpArray;
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *--SmpArray;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = (((long long) *--SmpArray * Mul_L) / Div);

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = (((long long) *--SmpArray * Mul_L) / Div);

GetSecondSample:
        *MixBuf++ += Sample;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = (((long long) *--SmpArray * Mul_L) / Div);

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIX_StereoXC ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *MixBuf = *MixBuffer;
  LONG  Mul_L = ChInfo->chi_Mix.chi_Multiply_L, Div = ChInfo->chi_Mix.chi_Divider;
  LONG  WriteSample;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i, Bit, SampleData, BitDepth = ChInfo->chi_Mix.chi_BitDepth;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      long long Sample;
      ULONG InterpolDiv, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset += SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;

          Sample += SampleData;

          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

GetSecondAdvInterpolSample:
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset += SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;

          Sample += SampleData;
          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      ULONG RealAdv = Advance * BitDepth, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpOffset = (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        WriteSample = (((long long) SampleData * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;
        CurrOffset += Advance;
        Bit += RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset++;
          Bit += BitDepth;
        }

        SmpOffset += (Bit >> 5UL);
        Bit &= 31;

GetSecondAdvSample:
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        WriteSample = (((long long) SampleData * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;
        CurrOffset += Advance;
        Bit += RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset++;
          Bit += BitDepth;
        }

        SmpOffset += (Bit >> 5UL);
        Bit &= 31;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray++ << Bit;
        SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

        Bit &= 31;
      }

      Bit += BitDepth;
      CurrOffset++;

      Sample = (LONG) SampleData;
      SampleValue = 0LL;

      if ( Length )
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) *SmpArray << Bit;
          SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
      }

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray++ << Bit;
        SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

        Bit &= 31;
      }

      Bit += BitDepth;
      CurrOffset++;

      Sample = (((long long) SampleData * Mul_L) / Div);

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = (((long long) SampleData * Mul_L) / Div);
        }

GetSecondSample:
        *MixBuf++ += Sample;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = (((long long) SampleData * Mul_L) / Div);
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIXb_StereoXC ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *MixBuf = *MixBuffer;
  LONG  Mul_L = ChInfo->chi_Mix.chi_Multiply_L, Div = ChInfo->chi_Mix.chi_Divider;
  LONG  WriteSample;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i, Bit, SampleData, BitDepth = ChInfo->chi_Mix.chi_BitDepth;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      long long Sample;
      ULONG InterpolDiv, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset -= SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample += SampleData;
          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

GetSecondAdvInterpolSample:
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset -= SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample += SampleData;
          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      ULONG RealAdv = Advance * BitDepth, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpOffset = (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        WriteSample = (((long long) SampleData * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;
        CurrOffset -= Advance;
        Bit -= RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;
        }

        SmpOffset += ((LONG) Bit >> 5L);
        Bit &= 31;

GetSecondAdvSample:
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        WriteSample = (((long long) SampleData * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;
        CurrOffset -= Advance;
        Bit -= RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;
        }

        SmpOffset += ((LONG) Bit >> 5L);
        Bit &= 31;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      CurrOffset--;
      Bit -= BitDepth;

      if ( (LONG) Bit < 0 )
      {
        SmpArray--;

        Bit &= 31;
      }

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray << Bit;
        SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
      }

      Sample = (LONG) SampleData;
      SampleValue = 0LL;

      if ( Length )
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) *SmpArray << Bit;
          SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
      }

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      CurrOffset--;
      Bit -= BitDepth;

      if ( (LONG) Bit < 0 )
      {
        SmpArray--;

        Bit &= 31;
      }

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray << Bit;
        SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
      }

      Sample = (((long long) SampleData * Mul_L) / Div);

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = (((long long) SampleData * Mul_L) / Div);
        }

GetSecondSample:
        *MixBuf++ += Sample;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = (((long long) SampleData * Mul_L) / Div);
        }

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIX_Stereo168C ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  WORD *SmpArray = (WORD *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable = ChInfo->chi_Mix.chi_VolTable_L;
  LONG *MixBuf = *MixBuffer;
  LONG  WriteSample;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      WORD *SmpArrayPos = SmpArray;
      LONG Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        WriteSample = VolTable[((UWORD) Sample >> 8U)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        WriteSample = VolTable[((UWORD) Sample >> 8U)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        WriteSample = VolTable[((UWORD) SmpArray[CurrOffset] >> 8U)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

GetSecondAdvSample:
        WriteSample = VolTable[((UWORD) SmpArray[CurrOffset] >> 8U)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    WORD *SmpArrayPos = SmpArray;
    LONG Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *SmpArray++;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        WriteSample = VolTable[((UWORD) Sample >> 8U)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        WriteSample = VolTable[((UWORD) Sample >> 8U)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = VolTable[(((UWORD) *SmpArray++ >> 8U))];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[((UWORD) *SmpArray++ >> 8U)];

GetSecondSample:
        *MixBuf++ += Sample;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[((UWORD) *SmpArray++ >> 8U)];

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIXb_Stereo168C ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  WORD *SmpArray = (WORD *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable = ChInfo->chi_Mix.chi_VolTable_L;
  LONG *MixBuf = *MixBuffer;
  LONG  WriteSample;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      WORD *SmpArrayPos = SmpArray;
      LONG Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        WriteSample = VolTable[((UWORD) Sample >> 8U)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        WriteSample = VolTable[((UWORD) Sample >> 8U)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        WriteSample = VolTable[((UWORD) SmpArray[CurrOffset] >> 8U)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

GetSecondAdvSample:
        WriteSample = VolTable[((UWORD) SmpArray[CurrOffset] >> 8U)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    WORD *SmpArrayPos = SmpArray;
    LONG Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *--SmpArray;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        WriteSample = VolTable[((UWORD) Sample >> 8U)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        WriteSample = VolTable[((UWORD) Sample >> 8U)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = VolTable[(((UWORD) *--SmpArray >> 8U))];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[((UWORD) *--SmpArray >> 8U)];

GetSecondSample:
        *MixBuf++ += Sample;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[((UWORD) *--SmpArray >> 8U)];

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIX_Stereo328C ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable = ChInfo->chi_Mix.chi_VolTable_L;
  LONG *MixBuf = *MixBuffer;
  LONG  WriteSample;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      LONG *SmpArrayPos = SmpArray;
      long long Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        WriteSample = VolTable[((ULONG) Sample >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        WriteSample = VolTable[((ULONG) Sample >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        WriteSample = VolTable[((ULONG) SmpArray[CurrOffset] >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

GetSecondAdvSample:
        WriteSample = VolTable[((ULONG) SmpArray[CurrOffset] >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    LONG *SmpArrayPos = SmpArray;
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *SmpArray++;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        WriteSample = VolTable[((ULONG) Sample >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        WriteSample = VolTable[((ULONG) Sample >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = VolTable[(((ULONG) *SmpArray++ >> 24UL))];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[((ULONG) *SmpArray++ >> 24UL)];

GetSecondSample:
        *MixBuf++ += Sample;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[((ULONG) *SmpArray++ >> 24UL)];

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIXb_Stereo328C ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable = ChInfo->chi_Mix.chi_VolTable_L;
  LONG *MixBuf = *MixBuffer;
  LONG  WriteSample;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      LONG *SmpArrayPos = SmpArray;
      long long Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        WriteSample = VolTable[((ULONG) Sample >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        WriteSample = VolTable[((ULONG) Sample >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        WriteSample = VolTable[((ULONG) SmpArray[CurrOffset] >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

GetSecondAdvSample:
        WriteSample = VolTable[((ULONG) SmpArray[CurrOffset] >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    LONG *SmpArrayPos = SmpArray;
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *--SmpArray;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        WriteSample = VolTable[((ULONG) Sample >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        WriteSample = VolTable[((ULONG) Sample >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = VolTable[(((ULONG) *SmpArray++ >> 24UL))];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[((ULONG) *--SmpArray >> 24UL)];

GetSecondSample:
        *MixBuf++ += Sample;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[((ULONG) *--SmpArray >> 24UL)];

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIX_StereoX8C ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable = ChInfo->chi_Mix.chi_VolTable_L;
  LONG *MixBuf = *MixBuffer;
  LONG  WriteSample;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i, Bit, SampleData, BitDepth = ChInfo->chi_Mix.chi_BitDepth;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      long long Sample;
      ULONG InterpolDiv, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset += SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;

          Sample += SampleData;

          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        WriteSample = VolTable[((ULONG) Sample >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

GetSecondAdvInterpolSample:
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset += SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;

          Sample += SampleData;
          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        WriteSample = VolTable[((ULONG) Sample >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      ULONG RealAdv = Advance * BitDepth, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpOffset = (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        WriteSample = VolTable[((ULONG) SampleData >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;
        CurrOffset += Advance;
        Bit += RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset++;
          Bit += BitDepth;
        }

        SmpOffset += (Bit >> 5UL);
        Bit &= 31;

GetSecondAdvSample:
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        WriteSample = VolTable[((ULONG) SampleData >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;
        CurrOffset += Advance;
        Bit += RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset++;
          Bit += BitDepth;
        }

        SmpOffset += (Bit >> 5UL);
        Bit &= 31;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray++ << Bit;
        SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

        Bit &= 31;
      }

      Bit += BitDepth;
      CurrOffset++;

      Sample = (LONG) SampleData;
      SampleValue = 0LL;

      if ( Length )
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) *SmpArray << Bit;
          SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
      }

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        WriteSample = VolTable[((ULONG) Sample >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        WriteSample = VolTable[((ULONG) Sample >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray++ << Bit;
        SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

        Bit &= 31;
      }

      Bit += BitDepth;
      CurrOffset++;

      Sample = VolTable[((ULONG) SampleData >> 24UL)];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = VolTable[((ULONG) SampleData >> 24UL)];
        }

GetSecondSample:
        *MixBuf++ += Sample;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = VolTable[((ULONG) SampleData >> 24UL)];
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIXb_StereoX8C ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable = ChInfo->chi_Mix.chi_VolTable_L;
  LONG *MixBuf = *MixBuffer;
  LONG  WriteSample;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i, Bit, SampleData, BitDepth = ChInfo->chi_Mix.chi_BitDepth;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      long long Sample;
      ULONG InterpolDiv, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset -= SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample += SampleData;
          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        WriteSample = VolTable[((ULONG) Sample >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

GetSecondAdvInterpolSample:
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset -= SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample += SampleData;
          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        WriteSample = VolTable[((ULONG) Sample >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      ULONG RealAdv = Advance * BitDepth, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpOffset = (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        WriteSample = VolTable[((ULONG) SampleData >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;
        CurrOffset -= Advance;
        Bit -= RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;
        }

        SmpOffset += ((LONG) Bit >> 5L);
        Bit &= 31;

GetSecondAdvSample:
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        WriteSample = VolTable[((ULONG) SampleData >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;
        CurrOffset -= Advance;
        Bit -= RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;
        }

        SmpOffset += ((LONG) Bit >> 5L);
        Bit &= 31;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      CurrOffset--;
      Bit -= BitDepth;

      if ( (LONG) Bit < 0 )
      {
        SmpArray--;

        Bit &= 31;
      }

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray << Bit;
        SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
      }

      Sample = (LONG) SampleData;
      SampleValue = 0LL;

      if ( Length )
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) *SmpArray << Bit;
          SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
      }

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        WriteSample = VolTable[((ULONG) Sample >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        WriteSample = VolTable[((ULONG) Sample >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      CurrOffset--;
      Bit -= BitDepth;

      if ( (LONG) Bit < 0 )
      {
        SmpArray--;

        Bit &= 31;
      }

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray << Bit;
        SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
      }

      Sample = VolTable[((ULONG) SampleData >> 24UL)];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = VolTable[((ULONG) SampleData >> 24UL)];
        }

GetSecondSample:
        *MixBuf++ += Sample;
        *MixBuf++ += Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = VolTable[((ULONG) SampleData >> 24UL)];
        }

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIX_Stereo8Sur ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  BYTE *SmpArray = (BYTE *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable = ChInfo->chi_Mix.chi_VolTable_L;
  LONG *MixBuf = *MixBuffer;
  LONG  WriteSample;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      BYTE *SmpArrayPos = SmpArray;
      LONG Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        WriteSample = VolTable[(UBYTE) Sample];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        WriteSample = VolTable[(UBYTE) Sample];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        WriteSample = VolTable[(UBYTE) SmpArray[CurrOffset]];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

GetSecondAdvSample:
        WriteSample = VolTable[(UBYTE) SmpArray[CurrOffset]];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    BYTE *SmpArrayPos = SmpArray;
    LONG Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *SmpArray++;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        WriteSample = VolTable[(UBYTE) Sample];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        WriteSample = VolTable[(UBYTE) Sample];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = VolTable[(UBYTE) *SmpArray++];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;
        *MixBuf++ += ~(LONG) Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[(UBYTE) *SmpArray++];

GetSecondSample:
        *MixBuf++ += Sample;
        *MixBuf++ += ~(LONG) Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[(UBYTE) *SmpArray++];

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIXb_Stereo8Sur ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  BYTE *SmpArray = (BYTE *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable = ChInfo->chi_Mix.chi_VolTable_L;
  LONG *MixBuf = *MixBuffer;
  LONG  WriteSample;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      BYTE *SmpArrayPos = SmpArray;
      LONG Sample;
      ULONG InterpolDiv;

      SmpArray += (CurrOffset + 1);

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        WriteSample = VolTable[(UBYTE) Sample];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        WriteSample = VolTable[(UBYTE) Sample];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        WriteSample = VolTable[(UBYTE) SmpArray[CurrOffset]];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

GetSecondAdvSample:
        WriteSample = VolTable[(UBYTE) SmpArray[CurrOffset]];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    BYTE *SmpArrayPos = SmpArray;
    LONG Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += (CurrOffset + 1);

      Sample = *--SmpArray;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        WriteSample = VolTable[(UBYTE) Sample];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        WriteSample = VolTable[(UBYTE) Sample];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += (CurrOffset + 1);

      Sample = VolTable[(UBYTE) *--SmpArray];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;
        *MixBuf++ += ~(LONG) Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[(UBYTE) *--SmpArray];

GetSecondSample:
        *MixBuf++ += Sample;
        *MixBuf++ += ~(LONG) Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[(UBYTE) *--SmpArray];

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIX_Stereo16Sur ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  WORD *SmpArray = (WORD *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *MixBuf = *MixBuffer;
  LONG  Mul_L = ChInfo->chi_Mix.chi_Multiply_L, Div = ChInfo->chi_Mix.chi_Divider;
  LONG  WriteSample;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      WORD *SmpArrayPos = SmpArray;
      LONG Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        WriteSample = (((long long) SmpArray[CurrOffset] * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

GetSecondAdvSample:
        WriteSample = (((long long) SmpArray[CurrOffset] * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    WORD *SmpArrayPos = SmpArray;
    LONG Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *SmpArray++;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = (((long long) *SmpArray++ * Mul_L) / Div);

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;
        *MixBuf++ += ~(LONG) Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = (((long long) *SmpArray++ * Mul_L) / Div);

GetSecondSample:
        *MixBuf++ += Sample;
        *MixBuf++ += ~(LONG) Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = (((long long) *SmpArray++ * Mul_L) / Div);

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIXb_Stereo16Sur ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  WORD *SmpArray = (WORD *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *MixBuf = *MixBuffer;
  LONG  Mul_L = ChInfo->chi_Mix.chi_Multiply_L, Div = ChInfo->chi_Mix.chi_Divider;
  LONG  WriteSample;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      WORD *SmpArrayPos = SmpArray;
      LONG Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        WriteSample = (((long long) SmpArray[CurrOffset] * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

GetSecondAdvSample:
        WriteSample = (((long long) SmpArray[CurrOffset] * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    WORD *SmpArrayPos = SmpArray;
    LONG Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *--SmpArray;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = (((long long) *--SmpArray * Mul_L) / Div);

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;
        *MixBuf++ += ~(LONG) Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = (((long long) *--SmpArray * Mul_L) / Div);

GetSecondSample:
        *MixBuf++ += Sample;
        *MixBuf++ += ~(LONG) Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = (((long long) *--SmpArray * Mul_L) / Div);

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIX_Stereo32Sur ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *MixBuf = *MixBuffer;
  LONG  Mul_L = ChInfo->chi_Mix.chi_Multiply_L, Div = ChInfo->chi_Mix.chi_Divider;
  LONG  WriteSample;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      LONG *SmpArrayPos = SmpArray;
      long long Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        WriteSample = (((long long) SmpArray[CurrOffset] * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

GetSecondAdvSample:
        WriteSample = (((long long) SmpArray[CurrOffset] * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    LONG *SmpArrayPos = SmpArray;
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *SmpArray++;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = (((long long) *SmpArray++ * Mul_L) / Div);

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;
        *MixBuf++ += ~(LONG) Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = (((long long) *SmpArray++ * Mul_L) / Div);

GetSecondSample:
        *MixBuf++ += Sample;
        *MixBuf++ += ~(LONG) Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = (((long long) *SmpArray++ * Mul_L) / Div);

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIXb_Stereo32Sur ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *MixBuf = *MixBuffer;
  LONG  Mul_L = ChInfo->chi_Mix.chi_Multiply_L, Div = ChInfo->chi_Mix.chi_Divider;
  LONG  WriteSample;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      LONG *SmpArrayPos = SmpArray;
      long long Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        WriteSample = (((long long) SmpArray[CurrOffset] * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

GetSecondAdvSample:
        WriteSample = (((long long) SmpArray[CurrOffset] * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    LONG *SmpArrayPos = SmpArray;
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *--SmpArray;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = (((long long) *--SmpArray * Mul_L) / Div);

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;
        *MixBuf++ += ~(LONG) Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = (((long long) *--SmpArray * Mul_L) / Div);

GetSecondSample:
        *MixBuf++ += Sample;
        *MixBuf++ += ~(LONG) Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = (((long long) *--SmpArray * Mul_L) / Div);

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIX_StereoXSur ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *MixBuf = *MixBuffer;
  LONG  Mul_L = ChInfo->chi_Mix.chi_Multiply_L, Div = ChInfo->chi_Mix.chi_Divider;
  LONG  WriteSample;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i, Bit, SampleData, BitDepth = ChInfo->chi_Mix.chi_BitDepth;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      long long Sample;
      ULONG InterpolDiv, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset += SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;

          Sample += SampleData;

          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

GetSecondAdvInterpolSample:
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset += SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;

          Sample += SampleData;
          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      ULONG RealAdv = Advance * BitDepth, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpOffset = (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        WriteSample = (((long long) SampleData * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;
        CurrOffset += Advance;
        Bit += RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset++;
          Bit += BitDepth;
        }

        SmpOffset += (Bit >> 5UL);
        Bit &= 31;

GetSecondAdvSample:
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        WriteSample = (((long long) SampleData * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;
        CurrOffset += Advance;
        Bit += RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset++;
          Bit += BitDepth;
        }

        SmpOffset += (Bit >> 5UL);
        Bit &= 31;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray++ << Bit;
        SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

        Bit &= 31;
      }

      Bit += BitDepth;
      CurrOffset++;

      Sample = (LONG) SampleData;
      SampleValue = 0LL;

      if ( Length )
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) *SmpArray << Bit;
          SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
      }

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray++ << Bit;
        SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

        Bit &= 31;
      }

      Bit += BitDepth;
      CurrOffset++;

      Sample = (((long long) SampleData * Mul_L) / Div);

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;
        *MixBuf++ += ~(LONG) Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = (((long long) SampleData * Mul_L) / Div);
        }

GetSecondSample:
        *MixBuf++ += Sample;
        *MixBuf++ += ~(LONG) Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = (((long long) SampleData * Mul_L) / Div);
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIXb_StereoXSur ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *MixBuf = *MixBuffer;
  LONG  Mul_L = ChInfo->chi_Mix.chi_Multiply_L, Div = ChInfo->chi_Mix.chi_Divider;
  LONG  WriteSample;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i, Bit, SampleData, BitDepth = ChInfo->chi_Mix.chi_BitDepth;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      long long Sample;
      ULONG InterpolDiv, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset -= SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample += SampleData;
          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

GetSecondAdvInterpolSample:
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset -= SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample += SampleData;
          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      ULONG RealAdv = Advance * BitDepth, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpOffset = (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        WriteSample = (((long long) SampleData * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;
        CurrOffset -= Advance;
        Bit -= RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;
        }

        SmpOffset += ((LONG) Bit >> 5L);
        Bit &= 31;

GetSecondAdvSample:
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        WriteSample = (((long long) SampleData * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;
        CurrOffset -= Advance;
        Bit -= RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;
        }

        SmpOffset += ((LONG) Bit >> 5L);
        Bit &= 31;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      CurrOffset--;
      Bit -= BitDepth;

      if ( (LONG) Bit < 0 )
      {
        SmpArray--;

        Bit &= 31;
      }

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray << Bit;
        SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
      }

      Sample = (LONG) SampleData;
      SampleValue = 0LL;

      if ( Length )
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) *SmpArray << Bit;
          SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
      }

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        WriteSample = (((long long) Sample * Mul_L) / Div);
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      CurrOffset--;
      Bit -= BitDepth;

      if ( (LONG) Bit < 0 )
      {
        SmpArray--;

        Bit &= 31;
      }

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray << Bit;
        SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
      }

      Sample = (((long long) SampleData * Mul_L) / Div);

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;
        *MixBuf++ += ~(LONG) Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = (((long long) SampleData * Mul_L) / Div);
        }

GetSecondSample:
        *MixBuf++ += Sample;
        *MixBuf++ += ~(LONG) Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = (((long long) SampleData * Mul_L) / Div);
        }

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIX_Stereo168Sur ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  WORD *SmpArray = (WORD *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable = ChInfo->chi_Mix.chi_VolTable_L;
  LONG *MixBuf = *MixBuffer;
  LONG  WriteSample;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      WORD *SmpArrayPos = SmpArray;
      LONG Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        WriteSample = VolTable[((UWORD) Sample >> 8U)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        WriteSample = VolTable[((UWORD) Sample >> 8U)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        WriteSample = VolTable[((UWORD) SmpArray[CurrOffset] >> 8U)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

GetSecondAdvSample:
        WriteSample = VolTable[((UWORD) SmpArray[CurrOffset] >> 8U)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    WORD *SmpArrayPos = SmpArray;
    LONG Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *SmpArray++;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        WriteSample = VolTable[((UWORD) Sample >> 8U)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        WriteSample = VolTable[((UWORD) Sample >> 8U)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = VolTable[(((UWORD) *SmpArray++ >> 8U))];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;
        *MixBuf++ += ~(LONG) Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[((UWORD) *SmpArray++ >> 8U)];

GetSecondSample:
        *MixBuf++ += Sample;
        *MixBuf++ += ~(LONG) Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[((UWORD) *SmpArray++ >> 8U)];

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIXb_Stereo168Sur ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  WORD *SmpArray = (WORD *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable = ChInfo->chi_Mix.chi_VolTable_L;
  LONG *MixBuf = *MixBuffer;
  LONG  WriteSample;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      WORD *SmpArrayPos = SmpArray;
      LONG Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        WriteSample = VolTable[((UWORD) Sample >> 8U)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0L;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        WriteSample = VolTable[((UWORD) Sample >> 8U)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        WriteSample = VolTable[((UWORD) SmpArray[CurrOffset] >> 8U)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

GetSecondAdvSample:
        WriteSample = VolTable[((UWORD) SmpArray[CurrOffset] >> 8U)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    WORD *SmpArrayPos = SmpArray;
    LONG Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *--SmpArray;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        WriteSample = VolTable[((UWORD) Sample >> 8U)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        WriteSample = VolTable[((UWORD) Sample >> 8U)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = VolTable[(((UWORD) *--SmpArray >> 8U))];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;
        *MixBuf++ += ~(LONG) Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[((UWORD) *--SmpArray >> 8U)];

GetSecondSample:
        *MixBuf++ += Sample;
        *MixBuf++ += ~(LONG) Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[((UWORD) *--SmpArray >> 8U)];

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIX_Stereo328Sur ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable = ChInfo->chi_Mix.chi_VolTable_L;
  LONG *MixBuf = *MixBuffer;
  LONG  WriteSample;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      LONG *SmpArrayPos = SmpArray;
      long long Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        WriteSample = VolTable[((ULONG) Sample >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *SmpArray++;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        WriteSample = VolTable[((ULONG) Sample >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        WriteSample = VolTable[((ULONG) SmpArray[CurrOffset] >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

GetSecondAdvSample:
        WriteSample = VolTable[((ULONG) SmpArray[CurrOffset] >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;
        CurrOffset += Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    LONG *SmpArrayPos = SmpArray;
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *SmpArray++;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        WriteSample = VolTable[((ULONG) Sample >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        WriteSample = VolTable[((ULONG) Sample >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *SmpArray++;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = VolTable[(((ULONG) *SmpArray++ >> 24UL))];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;
        *MixBuf++ += ~(LONG) Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[((ULONG) *SmpArray++ >> 24UL)];

GetSecondSample:
        *MixBuf++ += Sample;
        *MixBuf++ += ~(LONG) Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[((ULONG) *SmpArray++ >> 24UL)];

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIXb_Stereo328Sur ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable = ChInfo->chi_Mix.chi_VolTable_L;
  LONG *MixBuf = *MixBuffer;
  LONG  WriteSample;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      LONG *SmpArrayPos = SmpArray;
      long long Sample;
      ULONG InterpolDiv;

      SmpArray += CurrOffset;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        WriteSample = VolTable[((ULONG) Sample >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

GetSecondAdvInterpolSample:
        CurrOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          CurrOffset++;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Sample += *--SmpArray;
          InterpolDiv++;
        } while ( --CurrOffset );

        Sample /= InterpolDiv;

        WriteSample = VolTable[((ULONG) Sample >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        WriteSample = VolTable[((ULONG) SmpArray[CurrOffset] >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

GetSecondAdvSample:
        WriteSample = VolTable[((ULONG) SmpArray[CurrOffset] >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;
        CurrOffset -= Advance;

        if ( CurrFraction < AdvFract )
          CurrOffset--;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    LONG *SmpArrayPos = SmpArray;
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      SmpArray += CurrOffset;

      Sample = *--SmpArray;
      SampleValue = 0LL;

      if ( Length )
        SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        WriteSample = VolTable[((ULONG) Sample >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        WriteSample = VolTable[((ULONG) Sample >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          Sample = *--SmpArray;
          SampleValue = 0LL;

          if ( Length )
            SampleValue = (*SmpArray - Sample) * (long long) AdvFract;

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    } else
    {
      SmpArray += CurrOffset;

      Sample = VolTable[(((ULONG) *SmpArray++ >> 24UL))];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;
        *MixBuf++ += ~(LONG) Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[((ULONG) *--SmpArray >> 24UL)];

GetSecondSample:
        *MixBuf++ += Sample;
        *MixBuf++ += ~(LONG) Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          Sample = VolTable[((ULONG) *--SmpArray >> 24UL)];

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = (SmpArray - SmpArrayPos) - 1;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIX_StereoX8Sur ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable = ChInfo->chi_Mix.chi_VolTable_L;
  LONG *MixBuf = *MixBuffer;
  LONG  WriteSample;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i, Bit, SampleData, BitDepth = ChInfo->chi_Mix.chi_BitDepth;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      long long Sample;
      ULONG InterpolDiv, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset += SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;

          Sample += SampleData;

          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        WriteSample = VolTable[((ULONG) Sample >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

GetSecondAdvInterpolSample:
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset += SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;

          Sample += SampleData;
          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        WriteSample = VolTable[((ULONG) Sample >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      ULONG RealAdv = Advance * BitDepth, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpOffset = (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        WriteSample = VolTable[((ULONG) SampleData >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;
        CurrOffset += Advance;
        Bit += RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset++;
          Bit += BitDepth;
        }

        SmpOffset += (Bit >> 5UL);
        Bit &= 31;

GetSecondAdvSample:
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        WriteSample = VolTable[((ULONG) SampleData >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;
        CurrOffset += Advance;
        Bit += RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset++;
          Bit += BitDepth;
        }

        SmpOffset += (Bit >> 5UL);
        Bit &= 31;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray++ << Bit;
        SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

        Bit &= 31;
      }

      Bit += BitDepth;
      CurrOffset++;

      Sample = (LONG) SampleData;
      SampleValue = 0LL;

      if ( Length )
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) *SmpArray << Bit;
          SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
      }

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        WriteSample = VolTable[((ULONG) Sample >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        WriteSample = VolTable[((ULONG) Sample >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray++ << Bit;
        SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

        Bit &= 31;
      }

      Bit += BitDepth;
      CurrOffset++;

      Sample = VolTable[((ULONG) SampleData >> 24UL)];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;
        *MixBuf++ += ~(LONG) Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = VolTable[((ULONG) SampleData >> 24UL)];
        }

GetSecondSample:
        *MixBuf++ += Sample;
        *MixBuf++ += ~(LONG) Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray++ << Bit;
            SampleData |= ((ULONG) *SmpArray & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);

            Bit &= 31;
          }

          Bit += BitDepth;
          CurrOffset++;

          Sample = VolTable[((ULONG) SampleData >> 24UL)];
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  }
}

static void MIXb_StereoX8Sur ( struct LQMixerAllocStruct *LQMixAllocStr, struct ChannelInfo *ChInfo, LONG **MixBuffer, ULONG *Offset, ULONG *Fraction, ULONG Advance, ULONG AdvFract, ULONG Length )
{
  LONG *SmpArray = (LONG *) ChInfo->chi_Mix.chi_SampleStartAdr;
  LONG *VolTable = ChInfo->chi_Mix.chi_VolTable_L;
  LONG *MixBuf = *MixBuffer;
  LONG  WriteSample;
  ULONG CurrOffset = *Offset;
  ULONG CurrFraction = *Fraction;
  ULONG i, Bit, SampleData, BitDepth = ChInfo->chi_Mix.chi_BitDepth;

  if ( Advance )
  {
    if ( LQMixAllocStr->lqmas_Interpolation )
    {
      long long Sample;
      ULONG InterpolDiv, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvInterpolSample;

      i--;

      do
      {
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset -= SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample += SampleData;
          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        WriteSample = VolTable[((ULONG) Sample >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

GetSecondAdvInterpolSample:
        SmpOffset = Advance;
        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
          SmpOffset++;

        CurrOffset -= SmpOffset;

        Sample = 0LL;
        InterpolDiv = 0L;

        do
        {
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample += SampleData;
          InterpolDiv++;
        } while ( --SmpOffset );

        Sample /= InterpolDiv;

        WriteSample = VolTable[((ULONG) Sample >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      ULONG RealAdv = Advance * BitDepth, SmpOffset;

      Bit = CurrOffset * BitDepth;
      SmpOffset = (Bit >> 5UL);
      Bit &= 31;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondAdvSample;

      i--;

      do
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        WriteSample = VolTable[((ULONG) SampleData >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;
        CurrOffset -= Advance;
        Bit -= RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;
        }

        SmpOffset += ((LONG) Bit >> 5L);
        Bit &= 31;

GetSecondAdvSample:
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) SmpArray[SmpOffset] << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) SmpArray[SmpOffset] << Bit;
          SampleData |= ((ULONG) SmpArray[SmpOffset+1] & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        WriteSample = VolTable[((ULONG) SampleData >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;
        CurrOffset -= Advance;
        Bit -= RealAdv;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;
        }

        SmpOffset += ((LONG) Bit >> 5L);
        Bit &= 31;

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  } else
  {
    long long Sample;

    if ( LQMixAllocStr->lqmas_Interpolation > 1 )
    {
      ULONG InterpolFrac, InterpolCnt;
      LONG InterpolDiv;
      long long SampleValue;

      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      CurrOffset--;
      Bit -= BitDepth;

      if ( (LONG) Bit < 0 )
      {
        SmpArray--;

        Bit &= 31;
      }

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray << Bit;
        SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
      }

      Sample = (LONG) SampleData;
      SampleValue = 0LL;

      if ( Length )
      {
        if ( (Bit + BitDepth) < 32 )
        {
          SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
        } else
        {
          SampleData = (ULONG) *SmpArray << Bit;
          SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
        }

        SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
      }

      InterpolDiv = SampleValue >> 32;
      InterpolFrac = SampleValue;

      InterpolCnt = 0L;

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondInterpolSample;

      i--;

      do
      {
        WriteSample = VolTable[((ULONG) Sample >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }

GetSecondInterpolSample:
        WriteSample = VolTable[((ULONG) Sample >> 24UL)];
        *MixBuf++ += WriteSample;
        *MixBuf++ += ~WriteSample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = (LONG) SampleData;
          SampleValue = 0LL;

          if ( Length )
          {
            if ( (Bit + BitDepth) < 32 )
            {
              SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
            } else
            {
              SampleData = (ULONG) *SmpArray << Bit;
              SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
            }

            SampleValue = ((LONG) SampleData - Sample) * (long long) AdvFract;
          }

          InterpolDiv = SampleValue >> 32;
          InterpolFrac = SampleValue;

          InterpolCnt = 0L;
        } else
        {
          Sample += InterpolDiv;

          InterpolCnt += InterpolFrac;

          if ( InterpolCnt < InterpolFrac )
          {
            Sample++;

            if ( InterpolDiv < 0 )
              Sample -= 2;
          }
        }
      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    } else
    {
      Bit = CurrOffset * BitDepth;
      SmpArray += (Bit >> 5UL);
      Bit &= 31;

      CurrOffset--;
      Bit -= BitDepth;

      if ( (LONG) Bit < 0 )
      {
        SmpArray--;

        Bit &= 31;
      }

      if ( (Bit + BitDepth) < 32 )
      {
        SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
      } else
      {
        SampleData = (ULONG) *SmpArray << Bit;
        SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
      }

      Sample = VolTable[((ULONG) SampleData >> 24UL)];

      i = (Length >> 1) + 1;

      if ( Length & 1 )
        goto GetSecondSample;

      i--;

      do
      {
        *MixBuf++ += Sample;
        *MixBuf++ += ~(LONG) Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = VolTable[((ULONG) SampleData >> 24UL)];
        }

GetSecondSample:
        *MixBuf++ += Sample;
        *MixBuf++ += ~(LONG) Sample;

        CurrFraction += AdvFract;

        if ( CurrFraction < AdvFract )
        {
          CurrOffset--;
          Bit -= BitDepth;

          if ( (LONG) Bit < 0 )
          {
            SmpArray--;

            Bit &= 31;
          }

          if ( (Bit + BitDepth) < 32 )
          {
            SampleData = ((ULONG) *SmpArray << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
          } else
          {
            SampleData = (ULONG) *SmpArray << Bit;
            SampleData |= ((ULONG) *(SmpArray + 1) & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
          }

          Sample = VolTable[((ULONG) SampleData >> 24UL)];
        }

      } while ( --i );

      *MixBuffer = MixBuf;
      *Offset = CurrOffset;
      *Fraction = CurrFraction;

      return;
    }
  }
}
