/* TuComposer AmigaOS dependent code
**
** $VER: InternalTCS.c 31.00 (10.2.00)
**
** (C) 2000 by Sebastian Vater (known as Basty/Seasons).
**
*/

#include <stddef.h>
#include <string.h>
#include <exec/types.h>
#include <exec/libraries.h>
#include <exec/lists.h>
#include <exec/nodes.h>
#include <tucomposer/tucomposer.h>
#include <tucomposer/synth.h>
#include <tucomposer/external/external.h>
#include <tucomposer/external/song.h>
#include <proto/dos.h>
#include <proto/tucomposer.h>

#include "compiler.h"

#define SAVESAMPLE_BUFFER 4096

#ifdef __cplusplus
extern "C" {
#endif

#ifndef MOTOROLA
  static UWORD ReadMWord ( UBYTE *Buffer );
  static void WriteMWord ( UBYTE *Buffer, UWORD Value );

  static ULONG ReadMLong ( UBYTE *Buffer );
  static void WriteMLong ( UBYTE *Buffer, ULONG Value );
#else
  #define ReadMWord( Buffer ) (*(UWORD *) (Buffer))
  #define WriteMWord( Buffer, Value ) (*(UWORD *) (Buffer)) = Value

  #define ReadMLong( Buffer ) (*(ULONG *) (Buffer))
  #define WriteMLong( Buffer, Value ) (*(ULONG *) (Buffer)) = Value
#endif

static struct TuComposerExtLibBase *InternalTCSOpenExtLib ( void );
static BPTR InternalTCSCloseExtLib ( void );
static BPTR ExtLib ( void );
static ULONG IdentifyTCS ( struct CFHandle *CFHandle );
static struct TuComposerSong *LoadTCS ( struct CFHandle *CFHandle,
    struct TuComposerSong *SongStr, union FlagPass *FlagPassStr,
    unsigned long NumSong );
static ULONG SaveTCS ( struct CFHandle *CFHandle,
    struct TuComposerSong *SongStr, union FlagPass *FlagPassStr );
static ULONG GetTCSInfo ( struct CFHandle *CFHandle,
    struct SongInfo *SongInfo, unsigned long NumSong );
static ULONG GetNumTCSs ( struct CFHandle *CFHandle );

static ULONG LoadTCSSong ( struct CFHandle *CFHandle, struct TuComposerModule *ModuleStr, unsigned long Length );
static ULONG LoadSongPatt ( struct CFHandle *CFHandle, struct TuComposerSong *SongStr, unsigned long Length );
static ULONG LoadPattTrak ( struct CFHandle *CFHandle, struct TuComposerSong *SongStr, unsigned long Length );
static ULONG LoadSongPosi ( struct CFHandle *CFHandle, struct TuComposerSong *SongStr, unsigned long Length );
static ULONG LoadPosiPost ( struct CFHandle *CFHandle, struct TuComposerSong *SongStr, unsigned long Channel, unsigned long Length );
static ULONG LoadPostPosl ( struct CFHandle *CFHandle, struct TuComposerOrders *PosStr, unsigned long Length );

static ULONG LoadTCSInsl ( struct CFHandle *CFHandle, struct TuComposerModule *ModuleStr, union FlagPass *FlagPassStr, unsigned long Length );
static ULONG LoadInslInst ( struct CFHandle *CFHandle, struct TuComposerModule *ModuleStr, union FlagPass *FlagPassStr, unsigned long Length );
static ULONG LoadInstSamp ( struct CFHandle *CFHandle, struct TuComposerInstr *InstrStr, union FlagPass *FlagPassStr, unsigned long Length );
static ULONG LoadSampSmpl ( struct CFHandle *CFHandle, struct TuComposerInstr *InstrStr, union FlagPass *FlagPassStr, unsigned long Length );
static ULONG LoadSmplSnth ( struct CFHandle *CFHandle, struct TuComposerSample *SampleStr, union FlagPass *FlagPassStr, unsigned long Length );
static ULONG LoadSnthWfrm ( struct CFHandle *CFHandle, struct TuComposerSynth *SynthStr, unsigned long Length );
static ULONG LoadWfrmWave ( struct CFHandle *CFHandle, struct TuComposerSynth *SynthStr, unsigned long Length );
static ULONG LoadSnthStab ( struct CFHandle *CFHandle, struct TuComposerSynth *SynthStr, unsigned long Length );
static ULONG LoadStabSmbl ( struct CFHandle *CFHandle, struct TuComposerSynth *SynthStr, unsigned long Length );

static ULONG LoadTCSEnvl ( struct CFHandle *CFHandle, struct TuComposerModule *ModuleStr, unsigned long Length );
static ULONG LoadEnvlEnvd ( struct CFHandle *CFHandle, struct TuComposerModule *ModuleStr, unsigned long Length );

static ULONG LoadTCSKeyb ( struct CFHandle *CFHandle, struct TuComposerModule *ModuleStr, unsigned long Length );

static ULONG LoadTCSArpl ( struct CFHandle *CFHandle, struct TuComposerModule *ModuleStr, unsigned long Length );
static ULONG LoadArplArpg ( struct CFHandle *CFHandle, struct TuComposerModule *ModuleStr, unsigned long Length );
static ULONG LoadArpgArpe ( struct CFHandle *CFHandle, struct TuComposerInstrArpeggio *ArpStr, unsigned long Length );

static ULONG SaveTCSChunk ( struct CFHandle *CFHandle, struct IFFChunk *IFF_Chunk, unsigned long ChunkName, unsigned long ChunkLen, ULONG *ChunkLenBuf, unsigned long ErrorCode );
static ULONG SaveTCSName ( struct CFHandle *CFHandle, STRPTR Name, unsigned long ChunkName, ULONG *ChunkLenBuf, unsigned long ErrorCode );

static ULONG SaveTCSSong ( struct CFHandle *CFHandle, struct TuComposerSong *SongStr, struct TuComposerInstr **InstrList, ULONG *ChunkLenBuf );
static ULONG SaveSongTrak ( struct CFHandle *CFHandle, struct TuComposerTrack *TrackStr, struct TuComposerSong *SongStr, struct TuComposerInstr **InstrList, ULONG *ChunkLenBuf );
static ULONG SaveSongPost ( struct CFHandle *CFHandle, struct TuComposerSong *SongStr, ULONG *ChunkLenBuf );

static ULONG SaveTCSInst ( struct CFHandle *CFHandle, struct TuComposerModule *ModuleStr, struct TuComposerInstr *InstrStr, ULONG *ChunkLenBuf, union FlagPass *FlagPassStr );
static ULONG SaveInstSmpl ( struct CFHandle *CFHandle, struct TuComposerModule *ModuleStr, struct TuComposerSample *SampleStr, ULONG *ChunkLenBuf, union FlagPass *FlagPassStr );
static ULONG SaveSmplSnth ( struct CFHandle *CFHandle, struct TuComposerSynth *SynthStr, ULONG *ChunkLenBuf, union FlagPass *FlagPassStr );
static ULONG SaveSnthWave ( struct CFHandle *CFHandle, struct TuComposerSynthWave *WaveformStr, ULONG *ChunkLenBuf );
static ULONG SaveSnthSmbl ( struct CFHandle *CFHandle, struct TuComposerSynthSymbolTable *SymbolStr, ULONG *ChunkLenBuf );

static ULONG SaveTCSEnvd ( struct CFHandle *CFHandle, struct TuComposerInstrEnvelope *EnvStr, ULONG *ChunkLenBuf );

static ULONG SaveTCSKbrd ( struct CFHandle *CFHandle, struct TuComposerInstrKeyboard *KeybrdStr, ULONG *ChunkLenBuf );

static ULONG SaveTCSArpg ( struct CFHandle *CFHandle, struct TuComposerInstrArpeggio *ArpStr, ULONG *ChunkLenBuf );
static ULONG SaveArpgArpe ( struct CFHandle *CFHandle, struct TuComposerInstrArpeggio *ArpStr, ULONG *ChunkLenBuf );

static ULONG GetTCSSongInfo ( struct CFHandle *CFHandle, struct SongInfo *SongInfo, UBYTE *ScanFlagsPos, ULONG Length );

static void DeltaCrunch8 ( BYTE **SampleDataPos, BYTE *Buffer, BYTE *DeltaPos, ULONG Length );
static void DeltaCrunch16 ( WORD **SampleDataPos, WORD *Buffer, WORD *DeltaPos, ULONG Length );
static void DeltaCrunch32 ( LONG **SampleDataPos, LONG *Buffer, LONG *DeltaPos, ULONG Length );
static void DeltaCrunchX ( LONG **SampleDataPos, LONG *Buffer, LONG *DeltaPos, ULONG BitDepth, ULONG Length );

#ifdef __cplusplus
}
#endif

extern struct SongLibrary InternalTCSBase;
extern struct InternalTCSLoadFlagStr InternalTCSLoadFlags;
extern struct InternalTCSSaveFlagStr InternalTCSSaveFlags;
extern STRPTR LoadButton3 [];
extern STRPTR LoadButton4 [];
extern STRPTR SaveButton1 [];

static UBYTE InternalTCSName[] = "internal_tcs.library\0";
static UBYTE InternalTCSLoadID[] = "TuComposer Module Song (TCS)\0";
static UBYTE InternalTCSIDString[] = "Internal TCS Lib Version 1.00 (v31) (28 Jan 00) by Basty/Seasons.\0";

struct TuComposerExtList InternalTCS =
{
  {NULL, NULL},
  EXTLF_Intern|EXTLF_Loader|EXTLF_Saver, 0,
  (struct TuComposerExtLibBase *) &InternalTCSBase,
  NULL,
  (STRPTR) &InternalTCSName,
  (STRPTR) &InternalTCSLoadID,
  (struct TuComposerExtFlagStr *) &InternalTCSLoadFlags,
  (struct TuComposerExtFlagStr *) &InternalTCSSaveFlags,
  0
};

#include "InternalTCS_library.h"

static UBYTE LoadText0[] = "Load samples\0";
static UBYTE LoadText1[] = "Sample directory\0";
static UBYTE LoadText1Default[] = "ST-01:\0";
static UBYTE LoadText2[] = "Load synth code symbols\0";
static UBYTE LoadText3[] = "Instruments\0";
static UBYTE LoadText31[] = "Add to end\0";
static UBYTE LoadText32[] = "Add to beginning\0";
static UBYTE LoadText33[] = "Ignore\0";
static UBYTE LoadText4[] = "Envelopes\0";
static UBYTE LoadText41[] = "Add to end\0";
static UBYTE LoadText42[] = "Add to beginning\0";
static UBYTE LoadText43[] = "Ignore\0";
static UBYTE LoadText5[] = "Flush identical resources\0";

static UBYTE SaveText0[] = "Save samples\0";
static UBYTE SaveText1[] = "Delta packing\0";
static UBYTE SaveText11[] = "All\0";
static UBYTE SaveText12[] = "None\0";
static UBYTE SaveText13[] = "Partial (4, 8, 12, 16, 20, 24, 28 & 32 bit)\0";
static UBYTE SaveText2[] = "Save synth code symbols\0";
static UBYTE SaveText3[] = "Save instruments\0";
static UBYTE SaveText4[] = "Save envelopes\0";

static struct InternalTCSLoadFlagStr {
  UWORD  tefs_NumEntries;
  UWORD  tefs_StructSize;
  union FlagEntryBooleanStruct tefs_Flag0;
  union FlagEntryDirectoryStruct tefs_Flag1;
  union FlagEntryBooleanStruct tefs_Flag2;
  union FlagEntryButtonStruct tefs_Flag3;
  union FlagEntryButtonStruct tefs_Flag4;
  union FlagEntryBooleanStruct tefs_Flag5;
} InternalTCSLoadFlags =
{
  6, sizeof (InternalTCSLoadFlags),
  {(STRPTR) &LoadText0, EXTFLAG_BOOLEAN, 0, 0, 0L, 0L, 0xFFFFFFFFL},
  {(STRPTR) &LoadText1, EXTFLAG_DIRECTORY, 0, 256, 0L, 0L, (STRPTR) &LoadText1Default},
  {(STRPTR) &LoadText2, EXTFLAG_BOOLEAN, 0, 0, 0L, 0L, 0xFFFFFFFFL},
  {(STRPTR *) &LoadButton3, EXTFLAG_BUTTON, 0, 1+3, 0L, 0L, 1L},
  {(STRPTR *) &LoadButton4, EXTFLAG_BUTTON, 0, 1+3, 0L, 0L, 1L},
  {(STRPTR) &LoadText5, EXTFLAG_BOOLEAN, 0, 0, 0L, 0L, 0xFFFFFFFFL}
};

static struct InternalTCSSaveFlagStr {
  UWORD  tefs_NumEntries;
  UWORD  tefs_StructSize;
  union FlagEntryBooleanStruct tefs_Flag0;
  union FlagEntryButtonStruct tefs_Flag1;
  union FlagEntryBooleanStruct tefs_Flag2;
  union FlagEntryBooleanStruct tefs_Flag3;
  union FlagEntryBooleanStruct tefs_Flag4;
} InternalTCSSaveFlags =
{
  5, sizeof (InternalTCSSaveFlags),
  {(STRPTR) &SaveText0, EXTFLAG_BOOLEAN, 0, 0, 0L, 0L, 0xFFFFFFFFL},
  {(STRPTR *) &SaveText1, EXTFLAG_BUTTON, 0, 1+3, 0L, 0L, 1L},
  {(STRPTR) &SaveText2, EXTFLAG_BOOLEAN, 0, 0, 0L, 0L, 0xFFFFFFFFL},
  {(STRPTR) &SaveText3, EXTFLAG_BOOLEAN, 0, 0, 0L, 0L, 0xFFFFFFFFL},
  {(STRPTR) &SaveText4, EXTFLAG_BOOLEAN, 0, 0, 0L, 0L, 0xFFFFFFFFL}
};

static STRPTR LoadButton3[] =
{
  LoadText3,
  LoadText31,
  LoadText32,
  LoadText33,
};

static STRPTR LoadButton4[] =
{
  LoadText4,
  LoadText41,
  LoadText42,
  LoadText43,
};

static STRPTR SaveButton1[] =
{
  SaveText1,
  SaveText11,
  SaveText12,
  SaveText13,
};

static struct TuComposerExtLibBase *InternalTCSOpenExtLib ( void )
{
  InternalTCSBase.songl_Library.extb_LibNode.lib_OpenCnt++;

  return ( (struct TuComposerExtLibBase *) &InternalTCSBase );
}

static BPTR InternalTCSCloseExtLib ( void )
{
  InternalTCSBase.songl_Library.extb_LibNode.lib_OpenCnt--;

  return ( NULL );
}

static BPTR ExtLib ( void )
{
  return ( NULL );
}

static ULONG IdentifyTCS ( struct CFHandle *CFHandle )
{
  struct IFFHeader ID_IFF;

  if ( (CRead ( CFHandle, &ID_IFF, sizeof (struct IFFHeader)) != sizeof (struct IFFHeader)) )
    return ( 0L );

  if ( (ReadMLong ( (UBYTE *) &ID_IFF.ifh_FORMId[0] ) != 'FORM') || ( ReadMLong ( (UBYTE *) &ID_IFF.ifh_Length[0] ) < 4) || ( (ReadMLong ( (UBYTE *) &ID_IFF.ifh_TypeId[0]) >> 8) != 'TCM' ) )
    return ( 0L );

  return ( -1L );
}

static struct TuComposerSong *LoadTCS ( struct CFHandle *CFHandle,
    struct TuComposerSong *SongStr, union FlagPass *FlagPassStr,
    ULONG NumSong )
{
  struct IFFHeader IFF_Header;
  struct TuComposerModule *ModuleStr;
  struct TuComposerSong *TmpSong;
  struct TuComposerInstr *InstrStr;
  struct TuComposerOrders *PosStr;
  ULONG i, Channel = 0L;
  ULONG Length, IFFSize;

  if ( (CRead ( CFHandle, &IFF_Header, sizeof (struct IFFHeader)) != sizeof (struct IFFHeader)) )
    return ( NULL );

  if ( (ReadMLong ( (UBYTE *) &IFF_Header.ifh_FORMId[0] ) != 'FORM') || ( (Length = ReadMLong ( (UBYTE *) &IFF_Header.ifh_Length[0]) ) < 4) || ((ReadMLong ( (UBYTE *) &IFF_Header.ifh_TypeId[0] ) >> 8) != 'TCM') )
    return ( NULL );

  Length -= 4;

  if ( (ReadMLong ( (UBYTE *) &IFF_Header.ifh_TypeId[0] ) != 'TCM1' ) )
  {
    SetIoErr ( ERROR_ACTION_NOT_KNOWN );

    return ( NULL );
  }

  if (!( ModuleStr = AllocTCModule ( 1L )) )
    return ( NULL );

  do
  {
    struct IFFChunk IFF_Chunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      goto LoadTCSHeaderErr;

    IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0]);
    IFFLength = IFFSize;

    IFFSize += (IFFSize & 1);

    if ( (ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] )) == 'FORM' )
    {
      ULONG FaultCode;

      if ( (CRead ( CFHandle, &IFF_Header, 4 ) != 4 ) )
        goto LoadTCSHeaderErr;

      switch ( ReadMLong ( (UBYTE *) &IFF_Header.ifh_FORMId[0] ) )
      {
        case 'SONG' :
        {
          if (!( --NumSong ))
          {
            if ( (FaultCode = LoadTCSSong ( CFHandle, ModuleStr, IFFSize )))
            {
SetFaultCode:
              SetIoErr ( FaultCode );

              FreeTCModule ( ModuleStr );

              return ( NULL );
            }

            IFFLength = 0L;
          }

          break;
        }
        case 'INSL' :
        {
          if ( FlagPassStr[3].flg_Value != 3 )
          {
            if ( (FaultCode = LoadTCSInsl ( CFHandle, ModuleStr, FlagPassStr, IFFSize )))
              goto SetFaultCode;

            IFFLength = 0L;
          }

          break;
        }
        case 'ENVL' :
        {
          if ( FlagPassStr[4].flg_Value != 3 )
          {
            if ( (FaultCode = LoadTCSEnvl ( CFHandle, ModuleStr, IFFSize )))
              goto SetFaultCode;

            IFFLength = 0L;
          }

          break;
        }
        case 'KEYB' :
        {
          if ( FlagPassStr[4].flg_Value != 3 )
          {
            if ( (FaultCode = LoadTCSKeyb ( CFHandle, ModuleStr, IFFSize )))
              goto SetFaultCode;

            IFFLength = 0L;
          }

          break;
        }
        case 'ARPL' :
        {
          if ( FlagPassStr[4].flg_Value != 3 )
          {
            if ( (FaultCode = LoadTCSArpl ( CFHandle, ModuleStr, IFFSize )))
              goto SetFaultCode;

            IFFLength = 0L;
          }

          break;
        }
      }
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
      {
LoadTCSHeaderErr:

        FreeTCModule ( ModuleStr );

        return ( NULL );
      }
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      goto TCSIFFDamaged;

  } while ( (Length -= IFFSize) );

  TmpSong = (struct TuComposerSong *) ModuleStr->tcm_SongList.mlh_Head;

  if (!( TmpSong->tcso_Node.mln_Succ ))
  {
TCSIFFDamaged:

    SetIoErr ( ERROR_INVALID_RESIDENT_LIBRARY );

    FreeTCModule ( ModuleStr );

    return ( NULL );
  }

  DeleteSong ( TmpSong );

  PosStr = TmpSong->tcso_Position;

  for (i = TmpSong->tcso_Channels; i > 0; i--)
  {
    struct TuComposerOrderList *PosListStr;

    PosListStr = (struct TuComposerOrderList *) PosStr;

    for (;;)
    {
      PosListStr = (struct TuComposerOrderList *) PosListStr->tcol_Node.mln_Succ;

      if (!( PosListStr->tcol_Node.mln_Succ ))
        break;

      PosListStr->tcol_Track = GetTrackAddr ( SongStr, *(ULONG *) &(PosListStr->tcol_Track) );
      PosListStr->tcol_NextPos = GetPositionAddr ( SongStr->tcso_Position, Channel, *(ULONG *) &(PosListStr->tcol_NextPos));
      PosListStr->tcol_PrevPos = GetPositionAddr ( SongStr->tcso_Position, Channel, *(ULONG *) &(PosListStr->tcol_PrevPos));
    }

    Channel++;
    PosStr++;
  }

  InstrStr = (struct TuComposerInstr *) &(ModuleStr->tcm_InstrList.mlh_Head);

  for (;;)
  {
    struct TuComposerSample *SampleStr;

    InstrStr = (struct TuComposerInstr *) InstrStr->tci_Node.mln_Succ;

    if (!( InstrStr->tci_Node.mln_Succ ))
      break;

    InstrStr->tci_VolumeEnvelope = GetEnvelopeAddr ( ModuleStr, *(ULONG *) &(InstrStr->tci_VolumeEnvelope));
    InstrStr->tci_PanningEnvelope = GetEnvelopeAddr ( ModuleStr, *(ULONG *) &(InstrStr->tci_PanningEnvelope));
    InstrStr->tci_SlideEnvelope = GetEnvelopeAddr ( ModuleStr, *(ULONG *) &(InstrStr->tci_SlideEnvelope));
    InstrStr->tci_VibratoEnvelope = GetEnvelopeAddr ( ModuleStr, *(ULONG *) &(InstrStr->tci_VibratoEnvelope));
    InstrStr->tci_TremoloEnvelope = GetEnvelopeAddr ( ModuleStr, *(ULONG *) &(InstrStr->tci_TremoloEnvelope));
    InstrStr->tci_PannoloEnvelope = GetEnvelopeAddr ( ModuleStr, *(ULONG *) &(InstrStr->tci_PannoloEnvelope));
    InstrStr->tci_ChannoloEnvelope = GetEnvelopeAddr ( ModuleStr, *(ULONG *) &(InstrStr->tci_ChannoloEnvelope));
    InstrStr->tci_SpenoloEnvelope = GetEnvelopeAddr ( ModuleStr, *(ULONG *) &(InstrStr->tci_SpenoloEnvelope));
    InstrStr->tci_ArpeggioStructure = GetArpeggioAddr ( ModuleStr, *(ULONG *) &(InstrStr->tci_ArpeggioStructure));
    InstrStr->tci_KeyboardDefs = GetKeyboardAddr ( ModuleStr, *(ULONG *) &(InstrStr->tci_KeyboardDefs));

    SampleStr = (struct TuComposerSample *) &(InstrStr->tci_Samples.mlh_Head);

    for (;;)
    {
      SampleStr = (struct TuComposerSample *) SampleStr->tcs_Node.mln_Succ;

      if (!( SampleStr->tcs_Node.mln_Succ ))
        break;

      if ( SampleStr->tcs_Flags & TCSF_Redirect )
      {
        struct TuComposerInstr *InstrScan = (struct TuComposerInstr *) &(ModuleStr->tcm_InstrList.mlh_Head);
        struct TuComposerSample *SampleScan;
        ULONG Cnt = *(ULONG *) &(SampleStr->tcs_UserData);

        for (;;)
        {
          InstrScan = (struct TuComposerInstr *) InstrScan->tci_Node.mln_Succ;

          if (!( InstrScan->tci_Node.mln_Succ ))
            goto TCSIFFDamaged;

          SampleScan = (struct TuComposerSample *) &(InstrStr->tci_Samples.mlh_Head);

          for (;;)
          {
            SampleScan = (struct TuComposerSample *) SampleScan->tcs_Node.mln_Succ;

            if (!( SampleScan->tcs_Node.mln_Succ ))
              break;

            if ( --Cnt )
              continue;

            SampleStr->tcs_Data = SampleScan->tcs_Data;
            SampleStr->tcs_Length = ((SampleStr->tcs_Samples * SampleStr->tcs_BitDepth) + 7UL) >> 3UL;
            SampleStr->tcs_UserData = NULL;

            goto GetSampleAddrDone;
          }
        }

GetSampleAddrDone:

      }

      SampleStr->tcs_AutoVibratoEnv = GetEnvelopeAddr ( ModuleStr, *(ULONG *) &(SampleStr->tcs_AutoVibratoEnv));
      SampleStr->tcs_AutoTremoloEnv = GetEnvelopeAddr ( ModuleStr, *(ULONG *) &(SampleStr->tcs_AutoTremoloEnv));
      SampleStr->tcs_AutoPannoloEnv = GetEnvelopeAddr ( ModuleStr, *(ULONG *) &(SampleStr->tcs_AutoPannoloEnv));
    }
  }

  if (!( SongStr ))
    SongStr = TmpSong;

  if ( SongStr->tcso_Node.mln_Succ )
  {
    struct TuComposerModule *OrigModuleStr = (struct TuComposerModule *) &(SongStr->tcso_Node.mln_Succ);
    struct TuComposerTrack *TrackStr = (struct TuComposerTrack *) &(SongStr->tcso_TrackList.mlh_Head);
    ULONG NumInstrs;

    for (;;)
    {
      OrigModuleStr = (struct TuComposerModule *) OrigModuleStr->tcm_Node.mln_Pred;

      if (!( OrigModuleStr->tcm_Node.mln_Pred ))
        break;
    }

    (UBYTE *) OrigModuleStr += -offsetof (struct TuComposerModule, tcm_SongList);

    NumInstrs = GetNumInstrs ( OrigModuleStr ) + 1;

    for (;;)
    {
      TrackStr = (struct TuComposerTrack *) TrackStr->tct_Node.mln_Succ;

      if (!( TrackStr->tct_Node.mln_Succ ))
        break;

      SetBaseSongInstrs ( SongStr, TrackStr, NumInstrs );
    }

    if ( FlagPassStr[3].flg_Value != 3 )
    {
      struct TuComposerInstr *InstrStr = (struct TuComposerInstr *) &(ModuleStr->tcm_InstrList.mlh_Head);

      for (;;)
      {
        InstrStr = (struct TuComposerInstr *) InstrStr->tci_Node.mln_Succ;

        if (!( InstrStr->tci_Node.mln_Succ ))
          break;

        if ( FlagPassStr[3].flg_Value == 1 )
        {
          AddInstr ( OrigModuleStr, InstrStr );
        } else
        {
          AddNodeHead ( (struct List *) &(OrigModuleStr->tcm_InstrList), (struct Node *) InstrStr );
        }

        if ( (InstrStr->tci_CompFlags & TCICF_SeparateSamples) && (NumInstrs > 1) )
        {
          struct TuComposerInstrKeyboard *KeybrdStr = InstrStr->tci_KeyboardDefs;
          ULONG i;

          if ( KeybrdStr )
          {
            struct KeyboardEntry *KeybrdEntry = (struct KeyboardEntry *) &(KeybrdStr->tcik_NoteEntry[0]);

            if (!( KeybrdEntry->tcik_Octave_C_0 & 0x80 ))
            {
              KeybrdEntry->tcik_Octave_C_0 |= 0x80;

              for (i = (NUMOCTAVES * NUMNOTES); i > 0; i--)
              {
                if ( KeybrdEntry->tcik_Sample_C_0 )
                  KeybrdEntry->tcik_Sample_C_0 += NumInstrs - 1;

                KeybrdEntry++;
              }
            }
          } else
          {
            InstrStr->tci_CompFlags &= ~TCICF_SeparateSamples;
          }
        }
      }
    }

    if ( FlagPassStr[4].flg_Value != 3 )
    {
      struct TuComposerInstrEnvelope *EnvStr = (struct TuComposerInstrEnvelope *) &(ModuleStr->tcm_Envelopes.mlh_Head);
      struct TuComposerInstrKeyboard *KeybrdStr;
      struct TuComposerInstrArpeggio *ArpStr;

      for (;;)
      {
        EnvStr = (struct TuComposerInstrEnvelope *) EnvStr->tcie_Node.mln_Succ;

        if (!( EnvStr->tcie_Node.mln_Succ ))
          break;

        if ( FlagPassStr[4].flg_Value == 1 )
        {
          AddEnvelope ( OrigModuleStr, EnvStr );
        } else
        {
          AddNodeHead ( (struct List *) &(OrigModuleStr->tcm_Envelopes), (struct Node *) EnvStr );
        }
      }

      KeybrdStr = (struct TuComposerInstrKeyboard *) &(ModuleStr->tcm_KeyboardDefs.mlh_Head);

      for (;;)
      {
        KeybrdStr = (struct TuComposerInstrKeyboard *) KeybrdStr->tcik_Node.mln_Succ;

        if (!( KeybrdStr->tcik_Node.mln_Succ ))
          break;

        KeybrdStr->tcik_NoteEntry[0].tcik_Octave_C_0 &= 0x7F;

        if ( FlagPassStr[4].flg_Value == 1 )
        {
          AddKeyboard ( OrigModuleStr, KeybrdStr );
        } else
        {
          AddNodeHead ( (struct List *) &(OrigModuleStr->tcm_KeyboardDefs), (struct Node *) KeybrdStr );
        }
      }

      ArpStr = (struct TuComposerInstrArpeggio *) &(ModuleStr->tcm_ArpEnvList.mlh_Head);

      for (;;)
      {
        ArpStr = (struct TuComposerInstrArpeggio *) ArpStr->tcia_Node.mln_Succ;

        if (!( ArpStr->tcia_Node.mln_Succ ))
          break;

        if ( FlagPassStr[4].flg_Value == 1 )
        {
          AddArpeggio ( OrigModuleStr, ArpStr );
        } else
        {
          AddNodeHead ( (struct List *) &(OrigModuleStr->tcm_ArpEnvList), (struct Node *) ArpStr );
        }
      }
    }

    if ( FlagPassStr[5].flg_Value )
    {
      struct TuComposerInstr *InstrStr = (struct TuComposerInstr *) &(OrigModuleStr->tcm_InstrList.mlh_Head);
      struct TuComposerInstrEnvelope *EnvStr;
      struct TuComposerInstrKeyboard *KeybrdStr;
      struct TuComposerInstrArpeggio *ArpStr;

      for (;;)
      {
        struct TuComposerInstr *InstrScan = (struct TuComposerInstr *) &(OrigModuleStr->tcm_InstrList.mlh_Head);

        InstrStr = (struct TuComposerInstr *) InstrStr->tci_Node.mln_Succ;

        if (!( InstrStr->tci_Node.mln_Succ ))
          break;

        for (;;)
        {
          InstrScan = (struct TuComposerInstr *) InstrScan->tci_Node.mln_Succ;

          if (!( InstrScan->tci_Node.mln_Succ ))
            break;

          if ( InstrStr == InstrScan )
            continue;

          if (!( CmpInstrs ( InstrStr, InstrScan )))
          {
            ReMapInstr ( InstrScan, InstrStr );

            InstrStr = (struct TuComposerInstr *) &(OrigModuleStr->tcm_InstrList.mlh_Head);

            break;
          }
        }
      }

      EnvStr = (struct TuComposerInstrEnvelope *) &(OrigModuleStr->tcm_Envelopes.mlh_Head);

      for (;;)
      {
        struct TuComposerInstrEnvelope *EnvScan = (struct TuComposerInstrEnvelope *) &(OrigModuleStr->tcm_Envelopes.mlh_Head);

        EnvStr = (struct TuComposerInstrEnvelope *) EnvStr->tcie_Node.mln_Succ;

        if (!( EnvStr->tcie_Node.mln_Succ ))
          break;

        for (;;)
        {
          EnvScan = (struct TuComposerInstrEnvelope *) EnvScan->tcie_Node.mln_Succ;

          if (!( EnvScan->tcie_Node.mln_Succ ))
            break;

          if ( EnvStr == EnvScan )
            continue;

          if (!( CmpEnvelopes ( EnvStr, EnvScan )))
          {
            ReMapEnvelope ( EnvScan, EnvStr );

            EnvStr = (struct TuComposerInstrEnvelope *) &(OrigModuleStr->tcm_Envelopes.mlh_Head);

            break;
          }
        }
      }

      KeybrdStr = (struct TuComposerInstrKeyboard *) &(OrigModuleStr->tcm_KeyboardDefs.mlh_Head);

      for (;;)
      {
        struct TuComposerInstrKeyboard *KeybrdScan = (struct TuComposerInstrKeyboard *) &(OrigModuleStr->tcm_KeyboardDefs.mlh_Head);

        KeybrdStr = (struct TuComposerInstrKeyboard *) KeybrdStr->tcik_Node.mln_Succ;

        if (!( KeybrdStr->tcik_Node.mln_Succ ))
          break;

        for (;;)
        {
          KeybrdScan = (struct TuComposerInstrKeyboard *) KeybrdScan->tcik_Node.mln_Succ;

          if (!( KeybrdScan->tcik_Node.mln_Succ ))
            break;

          if ( KeybrdStr == KeybrdScan )
            continue;

          if (!( CmpKeyboards ( KeybrdStr, KeybrdScan )))
          {
            ReMapKeyboard ( KeybrdScan, KeybrdStr );

            KeybrdStr = (struct TuComposerInstrKeyboard *) &(OrigModuleStr->tcm_KeyboardDefs.mlh_Head);

            break;
          }
        }
      }

      ArpStr = (struct TuComposerInstrArpeggio *) &(OrigModuleStr->tcm_ArpEnvList.mlh_Head);

      for (;;)
      {
        struct TuComposerInstrArpeggio *ArpScan = (struct TuComposerInstrArpeggio *) &(OrigModuleStr->tcm_ArpEnvList.mlh_Head);

        ArpStr = (struct TuComposerInstrArpeggio *) ArpStr->tcia_Node.mln_Succ;

        if (!( ArpStr->tcia_Node.mln_Succ ))
          break;

        for (;;)
        {
          ArpScan = (struct TuComposerInstrArpeggio *) ArpScan->tcia_Node.mln_Succ;

          if (!( ArpScan->tcia_Node.mln_Succ ))
            break;

          if ( ArpStr == ArpScan )
            continue;

          if (!( CmpArpeggios ( ArpStr, ArpScan )))
          {
            ReMapArpeggio ( ArpScan, ArpStr );

            ArpStr = (struct TuComposerInstrArpeggio *) &(OrigModuleStr->tcm_ArpEnvList.mlh_Head);

            break;
          }
        }
      }
    }
  }

  ReMapSong ( TmpSong, SongStr );

  InitNodeList ( (struct List *) &(ModuleStr->tcm_InstrList));
  InitNodeList ( (struct List *) &(ModuleStr->tcm_Envelopes));
  InitNodeList ( (struct List *) &(ModuleStr->tcm_KeyboardDefs));
  InitNodeList ( (struct List *) &(ModuleStr->tcm_ArpEnvList));

  FreeTCModule ( ModuleStr );

  return ( SongStr );
}

static ULONG LoadTCSSong ( struct CFHandle *CFHandle, struct TuComposerModule *ModuleStr, unsigned long Length )
{
  struct IFFHeader IFF_Header;
  struct IFF_SHDR SHDR;
  struct TuComposerSong *SongStr;
  ULONG Invalid = 0L, IFFSize;

  if ( Length < 4 )
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  Length -= 4;

  if (!( SongStr = AllocSong ( 1L, 1L, 1L )) )
    return ( IoErr ());

  AddSong ( ModuleStr, SongStr );

  do
  {
    struct IFFChunk IFF_Chunk;
    struct UnknownChunk *UnkChunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      return ( IoErr ());

    IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0]);
    IFFLength = IFFSize;

    IFFSize += (IFFSize & 1);

    switch ( ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] ) )
    {
      case 'FILE' :
      {
        if ( SongStr->tcso_FileName )
          break;

        if (!( SongStr->tcso_FileName = (STRPTR) AllocPMem ( IFFLength + 1 )))
          return ( IoErr ());

        if ( (CRead ( CFHandle, SongStr->tcso_FileName, IFFLength ) != IFFLength) )
          return ( IoErr ());

        IFFLength &= 1;

        break;
      }
      case 'NAME' :
      {
        if ( SongStr->tcso_Name )
          break;

        if (!( SongStr->tcso_Name = (STRPTR) AllocPMem ( IFFLength + 1 )))
          return ( IoErr ());

        if ( (CRead ( CFHandle, SongStr->tcso_Name, IFFLength ) != IFFLength) )
          return ( IoErr ());

        IFFLength &= 1;

        break;
      }
      case 'SMSG' :
      {
        if ( SongStr->tcso_Message )
          break;

        if (!( SongStr->tcso_Message = (STRPTR) AllocPMem ( IFFLength + 1 )))
          return ( IoErr ());

        if ( (CRead ( CFHandle, SongStr->tcso_Message, IFFLength ) != IFFLength) )
          return ( IoErr ());

        IFFLength &= 1;

        break;
      }
      case 'AUTH' :
      {
        if ( SongStr->tcso_Composer )
          break;

        if (!( SongStr->tcso_Composer = (STRPTR) AllocPMem ( IFFLength + 1 )))
          return ( IoErr ());

        if ( (CRead ( CFHandle, SongStr->tcso_Composer, IFFLength ) != IFFLength) )
          return ( IoErr ());

        IFFLength &= 1;

        break;
      }
      case 'STIL' :
      {
        if ( SongStr->tcso_Style )
          break;

        if (!( SongStr->tcso_Style = (STRPTR) AllocPMem ( IFFLength + 1 )))
          return ( IoErr ());

        if ( (CRead ( CFHandle, SongStr->tcso_Style, IFFLength ) != IFFLength) )
          return ( IoErr ());

        IFFLength &= 1;

        break;
      }
      case 'CMNT' :
      {
        if ( SongStr->tcso_Comment )
          break;

        if (!( SongStr->tcso_Comment = (STRPTR) AllocPMem ( IFFLength + 1 )))
          return ( IoErr ());

        if ( (CRead ( CFHandle, SongStr->tcso_Comment, IFFLength ) != IFFLength) )
          return ( IoErr ());

        IFFLength &= 1;

        break;
      }
      case 'SHDR' :
      {
        if ( Invalid )
          return ( ERROR_INVALID_RESIDENT_LIBRARY );

        if ( (CRead ( CFHandle, &(SHDR.shdr_BeginDate), (sizeof (struct IFF_SHDR) - sizeof (struct IFFChunk))) != (sizeof (struct IFF_SHDR) - sizeof (struct IFFChunk))) )
          return ( IoErr ());

        memcpy ( &(SongStr->tcso_BeginDate), &(SHDR.shdr_BeginDate), offsetof (struct IFF_SHDR, shdr_Tracks) - sizeof (struct IFFChunk));

#ifndef MOTOROLA
        SongStr->tcso_BeginDate.sds_Year = ReadMWord ( (UBYTE *) &(SongStr->tcso_BeginDate.sds_Year));
        SongStr->tcso_EndDate.sds_Year = ReadMWord ( (UBYTE *) &(SongStr->tcso_EndDate.sds_Year));
#endif

        SongStr->tcso_CompFlags = SHDR.shdr_CompFlags;
        SongStr->tcso_Flags = SHDR.shdr_Flags;

        memcpy ( &(SongStr->tcso_Frames), &(SHDR.shdr_Frames), sizeof (struct IFF_SHDR) - offsetof ( struct IFF_SHDR, shdr_Frames ));

#ifndef MOTOROLA
        SongStr->tcso_Frames = ReadMWord ( (UBYTE *) &(SHDR.shdr_Frames) );
        SongStr->tcso_SPDSpeed = ReadMWord ( (UBYTE *) &(SHDR.shdr_SPDSpeed) );
        SongStr->tcso_BPMTempo = ReadMWord ( (UBYTE *) &(SHDR.shdr_BPMTempo) );
        SongStr->tcso_BPMSpeed = ReadMWord ( (UBYTE *) &(SHDR.shdr_BPMSpeed) );
        SongStr->tcso_FramesLowerLimit = ReadMWord ( (UBYTE *) &(SHDR.shdr_FramesLowerLimit) );
        SongStr->tcso_FramesUpperLimit = ReadMWord ( (UBYTE *) &(SHDR.shdr_FramesUpperLimit) );
        SongStr->tcso_SPDLowerLimit = ReadMWord ( (UBYTE *) &(SHDR.shdr_SPDLowerLimit) );
        SongStr->tcso_SPDUpperLimit = ReadMWord ( (UBYTE *) &(SHDR.shdr_SPDUpperLimit) );
        SongStr->tcso_BPMTempoLowerLimit = ReadMWord ( (UBYTE *) &(SHDR.shdr_BPMTempoLowerLimit) );
        SongStr->tcso_BPMTempoUpperLimit = ReadMWord ( (UBYTE *) &(SHDR.shdr_BPMTempoUpperLimit) );
        SongStr->tcso_BPMSpeedLowerLimit = ReadMWord ( (UBYTE *) &(SHDR.shdr_BPMSpeedLowerLimit) );
        SongStr->tcso_BPMSpeedUpperLimit = ReadMWord ( (UBYTE *) &(SHDR.shdr_BPMSpeedUpperLimit) );
#endif

        if ( ( SetSongChannels ( SongStr, ReadMWord ( (UBYTE *) &(SHDR.shdr_Channels)))) )
          return ( IoErr ());

        if ( ( SetStack ( SongStr, ReadMWord ( (UBYTE *) &(SHDR.shdr_GoSubStackSize)), ReadMWord ( (UBYTE *) &(SHDR.shdr_LoopStackSize)))) )
          return ( IoErr ());

        Invalid = ~Invalid;

        IFFLength -= sizeof (struct IFF_SHDR) - sizeof (struct IFFChunk);

        break;
      }
      case 'FORM' :
      {
        ULONG FaultCode;

        if ( (CRead ( CFHandle, &IFF_Header, 4 ) != 4 ) )
          return ( IoErr ());

        switch ( ReadMLong ( (UBYTE *) &IFF_Header.ifh_FORMId[0] ) )
        {
          case 'PATT' :
          {
            if ( (FaultCode = LoadSongPatt ( CFHandle, SongStr, IFFSize )))
              return ( FaultCode );

            IFFLength = 0L;

            break;
          }
          case 'POSI' :
          {
            if ( (FaultCode = LoadSongPosi ( CFHandle, SongStr, IFFSize )))
              return ( FaultCode );

            IFFLength = 0L;

            break;
          }
          default :
          {
            if ( CSeek ( CFHandle, -4, OFFSET_CURRENT ) == 0xFFFFFFFF )
              return ( FaultCode );

            goto AllocUnkChunk;
          }
        }

        break;
      }
      default :
      {
AllocUnkChunk:

        if (!( UnkChunk = AllocUnknownChunk ( NULL, ReadMLong ( (UBYTE *) &(IFF_Chunk.iff_Identifier[0])), IFFLength )))
          return ( IoErr ());

        AddSongUnknownChunk ( SongStr, UnkChunk );

        if ( (CRead ( CFHandle, &(UnkChunk->uc_Data), IFFLength ) != IFFLength ) )
          return ( IoErr ());

        IFFLength &= 1;

        break;
      }
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
        return ( IoErr ());
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      return ( ERROR_INVALID_RESIDENT_LIBRARY );

  } while ( (Length -= IFFSize) );

  if ( Invalid )
    return ( 0L );

  return ( ERROR_INVALID_RESIDENT_LIBRARY );
}

static ULONG LoadSongPatt ( struct CFHandle *CFHandle, struct TuComposerSong *SongStr, unsigned long Length )
{
  struct IFFHeader IFF_Header;
  ULONG IFFSize;

  if ( Length < 4 )
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  Length -= 4;

  do
  {
    struct IFFChunk IFF_Chunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      return ( IoErr ());

    if ( (IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0])) & 1 )
      IFFSize++;

    IFFLength = IFFSize;

    if ( (ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] ) ) == 'FORM' )
    {
      if ( (CRead ( CFHandle, &IFF_Header, 4 ) != 4 ) )
        return ( IoErr ());

      if ( (ReadMLong ( (UBYTE *) &IFF_Header.ifh_FORMId[0] )) == 'TRAK' )
      {
        ULONG FaultCode;

        if ( (FaultCode = LoadPattTrak ( CFHandle, SongStr, IFFSize )))
         return ( FaultCode );

        IFFLength = 0L;
      }
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
        return ( IoErr ());
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      return ( ERROR_INVALID_RESIDENT_LIBRARY );

  } while ( (Length -= IFFSize) );

  return ( 0L );
}

static ULONG LoadPattTrak ( struct CFHandle *CFHandle, struct TuComposerSong *SongStr, unsigned long Length )
{
  struct MIFF_THDR THDR;
  struct TuComposerPacked PackedStr;
  struct TuComposerTrack *TrackStr;
  struct UnknownChunk *UnkChunk;
  ULONG Invalid = 0L, IFFSize;

  memset ( &(PackedStr.tcp_FileName), 0, sizeof (struct TuComposerPacked));

  InitNodeList ( (struct List *) &(PackedStr.tcp_UnknownChunks));

  if ( Length < 4 )
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  Length -= 4;

  do
  {
    struct IFFChunk IFF_Chunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
    {
LoadTCSTrakErr:
      FreePMem ( PackedStr.tcp_Data, PackedStr.tcp_PackLength + 1 );

      UnkChunk = (struct UnknownChunk *) &(PackedStr.tcp_UnknownChunks.mlh_Head);

      for (;;)
      {
        UnkChunk = (struct UnknownChunk *) UnkChunk->uc_Node.mln_Succ;

        if (!( UnkChunk->uc_Node.mln_Succ ))
          break;

        FreeUnknownChunk ( UnkChunk );
      }

      return ( IoErr ());
    }

    IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0]);
    IFFLength = IFFSize;

    IFFSize += (IFFSize & 1);

    switch ( ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] ) )
    {
      case 'FILE' :
      {
        if ( PackedStr.tcp_FileName )
          break;

        if (!( PackedStr.tcp_FileName = (STRPTR) AllocPMem ( IFFLength + 1 )))
          goto LoadTCSTrakErr;

        if ( (CRead ( CFHandle, PackedStr.tcp_FileName, IFFLength ) != IFFLength) )
          goto LoadTCSTrakErr;

        IFFLength &= 1;

        break;
      }
      case 'NAME' :
      {
        if ( PackedStr.tcp_Name )
          break;

        if (!( PackedStr.tcp_Name = (STRPTR) AllocPMem ( IFFLength + 1 )))
          goto LoadTCSTrakErr;

        if ( (CRead ( CFHandle, PackedStr.tcp_Name, IFFLength ) != IFFLength) )
          goto LoadTCSTrakErr;

        IFFLength &= 1;

        break;
      }
      case 'THDR' :
      {
        if ( Invalid )
          goto TCSTrackIFFDamaged;

        if ( (CRead ( CFHandle, &(THDR.mthdr_Length), (sizeof (struct MIFF_THDR) - sizeof (struct IFFChunk))) != (sizeof (struct MIFF_THDR) - sizeof (struct IFFChunk))))
          goto LoadTCSTrakErr;

        memcpy ( &(PackedStr.tcp_Length), &(THDR.mthdr_Length), sizeof (struct MIFF_THDR ) - sizeof (struct IFFChunk));

#ifndef MOTOROLA
        PackedStr.tcp_Length = ReadMWord ( (UBYTE *) &(PackedStr.tcp_Length));
        PackedStr.tcp_Flags = ReadMWord ( (UBYTE *) &(PackedStr.tcp_Flags));
        PackedStr.tcp_Frames = ReadMWord ( (UBYTE *) &(PackedStr.tcp_Frames));
        PackedStr.tcp_SPDSpeed = ReadMWord ( (UBYTE *) &(PackedStr.tcp_SPDSpeed));
        PackedStr.tcp_BPMTempo = ReadMWord ( (UBYTE *) &(PackedStr.tcp_BPMTempo));
        PackedStr.tcp_BPMSpeed = ReadMWord ( (UBYTE *) &(PackedStr.tcp_BPMSpeed));
#endif

        Invalid = ~Invalid;

        IFFLength = IFFSize - (sizeof (struct MIFF_THDR) - sizeof (struct IFFChunk));

        break;
      }
      case 'BODY' :
      {
        if ( PackedStr.tcp_Data )
          goto TCSTrackIFFDamaged;

        if (!( PackedStr.tcp_Data = (UBYTE *) AllocPMem ( IFFLength + 1 )))
          goto LoadTCSTrakErr;

        if ( (CRead ( CFHandle, PackedStr.tcp_Data, IFFLength )) != IFFLength )
          goto LoadTCSTrakErr;

        PackedStr.tcp_PackLength = IFFLength;

        IFFLength &= 1;

        break;
      }
      default :
      {
        if (!( UnkChunk = AllocUnknownChunk ( NULL, ReadMLong ( (UBYTE *) &(IFF_Chunk.iff_Identifier[0])), IFFLength )))
          goto LoadTCSTrakErr;

        AddNodeTail ( (struct List *) &(PackedStr.tcp_UnknownChunks), (struct Node *) UnkChunk );

        if ( (CRead ( CFHandle, &(UnkChunk->uc_Data), IFFLength ) != IFFLength ) )
          goto LoadTCSTrakErr;

        IFFLength &= 1;

        break;
      }
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
        goto LoadTCSTrakErr;
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
    {
TCSTrackIFFDamaged:

      FreePMem ( PackedStr.tcp_Data, PackedStr.tcp_PackLength + 1 );

      UnkChunk = (struct UnknownChunk *) &(PackedStr.tcp_UnknownChunks.mlh_Head);

      for (;;)
      {
        UnkChunk = (struct UnknownChunk *) UnkChunk->uc_Node.mln_Succ;

        if (!( UnkChunk->uc_Node.mln_Succ ))
          break;

        FreeUnknownChunk ( UnkChunk );
      }

      return ( ERROR_INVALID_RESIDENT_LIBRARY );
    }

  } while ( (Length -= IFFSize) );

  if (!( Invalid ))
    goto TCSTrackIFFDamaged;

  if (!( TrackStr = UnPackTrack ( (struct TuComposerPacked *) &PackedStr, NULL )))
    goto LoadTCSTrakErr;

  AddTrack ( SongStr, TrackStr );

  FreePMem ( PackedStr.tcp_Data, PackedStr.tcp_PackLength + 1 );

  UnkChunk = (struct UnknownChunk *) &(PackedStr.tcp_UnknownChunks.mlh_Head);

  for (;;)
  {
    UnkChunk = (struct UnknownChunk *) UnkChunk->uc_Node.mln_Succ;

    if (!( UnkChunk->uc_Node.mln_Succ ))
      break;

    FreeUnknownChunk ( UnkChunk );
  }

  return ( 0L );
}

static ULONG LoadSongPosi ( struct CFHandle *CFHandle, struct TuComposerSong *SongStr, unsigned long Length )
{
  struct IFFHeader IFF_Header;
  ULONG Channel = 0L, IFFSize;

  if ( Length < 4 )
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  Length -= 4;

  do
  {
    struct IFFChunk IFF_Chunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      return ( IoErr ());

    if ( (IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0])) & 1)
      IFFSize++;

    IFFLength = IFFSize;

    if ( (ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] ) ) == 'FORM' )
    {
      if ( (CRead ( CFHandle, &IFF_Header, 4 ) != 4 ) )
        return ( IoErr ());

      if ( (ReadMLong ( (UBYTE *) &IFF_Header.ifh_FORMId[0] )) == 'POST' )
      {
        ULONG FaultCode;

        if ( Channel >= SongStr->tcso_Channels )
        {
          if ( (SetSongChannels ( SongStr, Channel + 1 )))
            return ( IoErr ());
        }

        if ( (FaultCode = LoadPosiPost ( CFHandle, SongStr, Channel, IFFSize )))
         return ( FaultCode );

        Channel++;

        IFFLength = 0L;
      }
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
        return ( IoErr ());
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      return ( ERROR_INVALID_RESIDENT_LIBRARY );

  } while ( (Length -= IFFSize) );

  return ( 0L );
}

static ULONG LoadPosiPost ( struct CFHandle *CFHandle, struct TuComposerSong *SongStr, unsigned long Channel, unsigned long Length )
{
  struct IFFHeader IFF_Header;
  struct TuComposerOrders *PosStr = (SongStr->tcso_Position) + Channel;
  ULONG Invalid = 0L, IFFSize;

  if ( Length < 4 )
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  Length -= 4;

  do
  {
    struct IFFChunk IFF_Chunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      return ( IoErr ());

    if ( (IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0])) & 1 )
      IFFSize++;

    IFFLength = IFFSize;

    switch ( ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] ) )
    {
      case 'PHDR' :
      {
        if ( Invalid )
          return ( ERROR_INVALID_RESIDENT_LIBRARY );

        if ( (CRead ( CFHandle, &(PosStr->tco_Length), (sizeof (struct IFF_PHDR) - sizeof (struct IFFChunk))) != (sizeof (struct IFF_PHDR) - sizeof (struct IFFChunk))))
          return ( IoErr ());

#ifndef MOTOROLA
        PosStr->tco_Length = ReadMWord ( (UBYTE *) &(PosStr->tco_Length));
        PosStr->tco_RepStart = ReadMWord ( (UBYTE *) &(PosStr->tco_RepStart));
#endif

        Invalid = ~Invalid;

        IFFLength -= sizeof (struct IFF_PHDR) - sizeof (struct IFFChunk);

        break;
      }
      case 'FORM' :
      {
        ULONG FaultCode;

        if ( (CRead ( CFHandle, &IFF_Header, 4 ) != 4 ) )
          return ( IoErr ());

        if ( (ReadMLong ( (UBYTE *) &IFF_Header.ifh_FORMId[0] ) == 'POSL' ))
        {
          if ( (FaultCode = LoadPostPosl ( CFHandle, PosStr, IFFSize )))
            return ( FaultCode );

          IFFLength = 0L;
        }

        break;
      }
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
        return ( IoErr ());
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      return ( ERROR_INVALID_RESIDENT_LIBRARY );

  } while ( (Length -= IFFSize) );

  if ( Invalid )
    return ( 0L );

  return ( ERROR_INVALID_RESIDENT_LIBRARY );
}

static ULONG LoadPostPosl ( struct CFHandle *CFHandle, struct TuComposerOrders *PosStr, unsigned long Length )
{
  struct IFF_PDAT PDAT;
  struct TuComposerOrderList *PosListStr;
  ULONG IFFSize;

  if ( Length < 4 )
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  Length -= 4;

  do
  {
    struct IFFChunk IFF_Chunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      return ( IoErr ());

    if ( (IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0])) & 1 )
      IFFSize++;

    IFFLength = IFFSize;

    if ( (ReadMLong ( (UBYTE *) &(IFF_Chunk.iff_Identifier[0])) == 'PDAT') )
    {
      if ( CRead ( CFHandle, &(PDAT.pdat_TrackNum), (sizeof (struct IFF_PDAT ) - sizeof (struct IFFChunk))) != (sizeof (struct IFF_PDAT) - sizeof (struct IFFChunk)))
        return ( IoErr ());

      if (!( PosListStr = AllocPositionList ()))
        return ( IoErr ());

      memcpy ( &(PosListStr->tcol_NextRow), &(PDAT.pdat_NextRow), (sizeof (struct IFF_PDAT) - offsetof (struct IFF_PDAT, pdat_NextRow )));

      *(ULONG *) &(PosListStr->tcol_Track) = ReadMWord ( (UBYTE *) &(PDAT.pdat_TrackNum) );
      *(ULONG *) &(PosListStr->tcol_NextPos) = ReadMWord ( (UBYTE *) &(PDAT.pdat_NextPos) );
      *(ULONG *) &(PosListStr->tcol_PrevPos) = ReadMWord ( (UBYTE *) &(PDAT.pdat_PrevPos) );

#ifndef MOTOROLA
      PosListStr->tcol_NextRow = ReadMWord ( (UBYTE *) &(PosListStr->tcol_NextRow) );
      PosListStr->tcol_PrevRow = ReadMWord ( (UBYTE *) &(PosListStr->tcol_PrevRow) );
      PosListStr->tcol_FirstRow = ReadMWord ( (UBYTE *) &(PosListStr->tcol_FirstRow) );
      PosListStr->tcol_LastRow = ReadMWord ( (UBYTE *) &(PosListStr->tcol_LastRow) );
      PosListStr->tcol_InstrTranspose = ReadMWord ( (UBYTE *) &(PosListStr->tcol_InstrTranspose) );
      PosListStr->tcol_Tempo = ReadMWord ( (UBYTE *) &(PosListStr->tcol_Tempo) );
#endif

      AddNodeTail ( (struct List *) PosStr, (struct Node *) PosListStr );

      IFFLength -= sizeof (struct IFF_PDAT ) - sizeof (struct IFFChunk);
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
        return ( IoErr ());
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      return ( ERROR_INVALID_RESIDENT_LIBRARY );

  } while ( (Length -= IFFSize) );

  return ( 0L );
}

static ULONG LoadTCSInsl ( struct CFHandle *CFHandle, struct TuComposerModule *ModuleStr, union FlagPass *FlagPassStr, unsigned long Length )
{
  struct IFFHeader IFF_Header;
  ULONG IFFSize;

  if ( Length < 4 )
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  Length -= 4;

  do
  {
    struct IFFChunk IFF_Chunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      return ( IoErr ());

    if ( (IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0])) & 1 )
      IFFSize++;

    IFFLength = IFFSize;

    if ( (ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] ) ) == 'FORM' )
    {
      ULONG FaultCode;

      if ( (CRead ( CFHandle, &IFF_Header, 4 ) != 4 ) )
        return ( IoErr ());

      if ( (ReadMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0])) ) == 'INST' )
      {
        if ( (FaultCode = LoadInslInst ( CFHandle, ModuleStr, FlagPassStr, IFFSize )))
          return ( FaultCode );

        IFFLength = 0L;
      }
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
        return ( IoErr ());
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      return ( ERROR_INVALID_RESIDENT_LIBRARY );

  } while ( (Length -= IFFSize) );

  return ( 0L );
}

static ULONG LoadInslInst ( struct CFHandle *CFHandle, struct TuComposerModule *ModuleStr, union FlagPass *FlagPassStr, unsigned long Length )
{
  struct IFFHeader IFF_Header;
  struct MIFF_IHDR IHDR;
  struct TuComposerInstr *InstrStr;
  ULONG Invalid = 0L, IFFSize;

  if ( Length < 4 )
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  Length -= 4;

  if (!( InstrStr = AllocInstr ( 0L )))
    return ( IoErr ());

  AddInstr ( ModuleStr, InstrStr );

  do
  {
    struct IFFChunk IFF_Chunk;
    struct UnknownChunk *UnkChunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      return ( IoErr ());

    IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0]);
    IFFLength = IFFSize;

    IFFSize += (IFFSize & 1);

    switch ( ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] ) )
    {
      case 'FILE' :
      {
        if ( InstrStr->tci_FileName )
          break;

        if (!( InstrStr->tci_FileName = (STRPTR) AllocPMem ( IFFLength + 1 )))
          return ( IoErr ());

        if ( (CRead ( CFHandle, InstrStr->tci_FileName, IFFLength ) != IFFLength) )
          return ( IoErr ());

        IFFLength &= 1;

        break;
      }
      case 'NAME' :
      {
        if ( InstrStr->tci_Name )
          break;

        if (!( InstrStr->tci_Name = (STRPTR) AllocPMem ( IFFLength + 1 )))
          return ( IoErr ());

        if ( (CRead ( CFHandle, InstrStr->tci_Name, IFFLength ) != IFFLength) )
          return ( IoErr ());

        IFFLength &= 1;

        break;
      }
      case 'IHDR' :
      {
        if ( Invalid )
          return ( ERROR_INVALID_RESIDENT_LIBRARY );

        if ( (CRead ( CFHandle, &(IHDR.mihdr_VolumeEnvNum), (sizeof (struct MIFF_IHDR) - sizeof (struct IFFChunk))) != (sizeof (struct MIFF_IHDR) - sizeof (struct IFFChunk))))
          return ( IoErr ());

        *(ULONG *) &(InstrStr->tci_VolumeEnvelope) = ReadMWord ( (UBYTE *) &(IHDR.mihdr_VolumeEnvNum));
        *(ULONG *) &(InstrStr->tci_PanningEnvelope) = ReadMWord ( (UBYTE *) &(IHDR.mihdr_PanningEnvNum));
        *(ULONG *) &(InstrStr->tci_SlideEnvelope) = ReadMWord ( (UBYTE *) &(IHDR.mihdr_SlideEnvNum));
        *(ULONG *) &(InstrStr->tci_VibratoEnvelope) = ReadMWord ( (UBYTE *) &(IHDR.mihdr_VibratoEnvNum));
        *(ULONG *) &(InstrStr->tci_TremoloEnvelope) = ReadMWord ( (UBYTE *) &(IHDR.mihdr_TremoloEnvNum));
        *(ULONG *) &(InstrStr->tci_PannoloEnvelope) = ReadMWord ( (UBYTE *) &(IHDR.mihdr_PannoloEnvNum));
        *(ULONG *) &(InstrStr->tci_ChannoloEnvelope) = ReadMWord ( (UBYTE *) &(IHDR.mihdr_ChannoloEnvNum));
        *(ULONG *) &(InstrStr->tci_SpenoloEnvelope) = ReadMWord ( (UBYTE *) &(IHDR.mihdr_SpenoloEnvNum));
        *(ULONG *) &(InstrStr->tci_ArpeggioStructure) = ReadMWord ( (UBYTE *) &(IHDR.mihdr_ArpeggioDefNum));
        *(ULONG *) &(InstrStr->tci_KeyboardDefs) = ReadMWord ( (UBYTE *) &(IHDR.mihdr_KeyboardDefNum));

        memcpy ( &(InstrStr->tci_NumSamples), &(IHDR.mihdr_NumSamples), (sizeof (struct MIFF_IHDR) - offsetof (struct MIFF_IHDR, mihdr_NumSamples)));

#ifndef MOTOROLA
        InstrStr->tci_VolumeSwing = ReadMWord ( (UBYTE *) &(InstrStr->tci_VolumeSwing) );
        InstrStr->tci_PanningSwing = ReadMWord ( (UBYTE *) &(InstrStr->tci_PanningSwing) );
        InstrStr->tci_PitchSwing = ReadMLong ( (UBYTE *) &(InstrStr->tci_PitchSwing) );
        InstrStr->tci_PitchPanSeparation = ReadMWord ( (UBYTE *) &(InstrStr->tci_PitchPanSeparation) );
        InstrStr->tci_EnvUsageFlags = ReadMWord ( (UBYTE *) &(InstrStr->tci_EnvUsageFlags) );
        InstrStr->tci_EnvProcFlags = ReadMWord ( (UBYTE *) &(InstrStr->tci_EnvProcFlags) );
        InstrStr->tci_EnvRetrigFlags = ReadMWord ( (UBYTE *) &(InstrStr->tci_EnvRetrigFlags) );
        InstrStr->tci_EnvRandomFlags = ReadMWord ( (UBYTE *) &(InstrStr->tci_EnvRandomFlags) );
        InstrStr->tci_EnvRndDelayFlags = ReadMWord ( (UBYTE *) &(InstrStr->tci_EnvRndDelayFlags) );
        InstrStr->tci_FadeOut = ReadMWord ( (UBYTE *) &(InstrStr->tci_FadeOut) );
        InstrStr->tci_Hold = ReadMWord ( (UBYTE *) &(InstrStr->tci_Hold) );
        InstrStr->tci_Decay = ReadMWord ( (UBYTE *) &(InstrStr->tci_Decay) );
#endif

        Invalid = ~Invalid;

        IFFLength = IFFSize - (sizeof (struct MIFF_IHDR) - sizeof (struct IFFChunk));

        break;
      }
      case 'FORM' :
      {
        ULONG FaultCode;

        if ( (CRead ( CFHandle, &IFF_Header, 4 ) != 4 ) )
          return ( IoErr ());

        if ( (ReadMLong ( (UBYTE *) &IFF_Header.ifh_FORMId[0] ) == 'SAMP' ))
        {
          if ( (FaultCode = LoadInstSamp ( CFHandle, InstrStr, FlagPassStr, IFFSize )))
            return ( FaultCode );

          IFFLength = 0L;
        }
      }
      default :
      {
        if (!( UnkChunk = AllocUnknownChunk ( NULL, ReadMLong ( (UBYTE *) &(IFF_Chunk.iff_Identifier[0])), IFFLength )))
          return ( IoErr ());

        AddInstrUnknownChunk ( InstrStr, UnkChunk );

        if ( (CRead ( CFHandle, &(UnkChunk->uc_Data), IFFLength ) != IFFLength ) )
          return ( IoErr ());

        IFFLength &= 1;

        break;
      }
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
        return ( IoErr ());
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      return ( ERROR_INVALID_RESIDENT_LIBRARY );

  } while ( (Length -= IFFSize) );

  if ( Invalid )
    return ( 0L );

  return ( ERROR_INVALID_RESIDENT_LIBRARY );
}

static ULONG LoadInstSamp ( struct CFHandle *CFHandle, struct TuComposerInstr *InstrStr, union FlagPass *FlagPassStr, unsigned long Length )
{
  struct IFFHeader IFF_Header;
  ULONG IFFSize;

  if ( Length < 4 )
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  Length -= 4;

  do
  {
    struct IFFChunk IFF_Chunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      return ( IoErr ());

    if ( (IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0] )) & 1 )
      IFFSize++;

    IFFLength = IFFSize;

    if ( (ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] ) ) == 'FORM' )
    {
      ULONG FaultCode;

      if ( (CRead ( CFHandle, &IFF_Header, 4 ) != 4 ) )
        return ( IoErr ());

      if ( (ReadMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0])) ) == 'SMPL' )
      {
        if ( (FaultCode = LoadSampSmpl ( CFHandle, InstrStr, FlagPassStr, IFFSize )))
          return ( FaultCode );

        IFFLength = 0L;
      }
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
        return ( IoErr ());
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      return ( ERROR_INVALID_RESIDENT_LIBRARY );

  } while ( (Length -= IFFSize) );

  return ( 0L );
}

static ULONG LoadSampSmpl ( struct CFHandle *CFHandle, struct TuComposerInstr *InstrStr, union FlagPass *FlagPassStr, unsigned long Length )
{
  struct IFFHeader IFF_Header;
  struct IFF_SMPH SMPH;
  struct IFF_SMPR SMPR;
  struct TuComposerSample *SampleStr;
  ULONG Invalid = 0L, IFFSize;

  if ( Length < 4 )
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  Length -= 4;

  if (!( SampleStr = AllocSample ( NULL, 0L )))
    return ( IoErr ());

  if ( AddSample ( InstrStr, SampleStr ))
    return ( ERROR_TOO_MANY_LEVELS );


  do
  {
    struct IFFChunk IFF_Chunk;
    struct UnknownChunk *UnkChunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      return ( IoErr ());

    IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0]);
    IFFLength = IFFSize;

    IFFSize += (IFFSize & 1);

    switch ( ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] ) )
    {
      case 'FILE' :
      {
        if ( SampleStr->tcs_FileName )
          break;

        if (!( SampleStr->tcs_FileName = (STRPTR) AllocPMem ( IFFLength + 1 )))
          return ( IoErr ());

        if ( (CRead ( CFHandle, SampleStr->tcs_FileName, IFFLength ) != IFFLength) )
          return ( IoErr ());

        IFFLength &= 1;

        break;
      }
      case 'NAME' :
      {
        if ( SampleStr->tcs_Name )
          break;

        if (!( SampleStr->tcs_Name = (STRPTR) AllocPMem ( IFFLength + 1 )))
          return ( IoErr ());

        if ( (CRead ( CFHandle, SampleStr->tcs_Name, IFFLength ) != IFFLength) )
          return ( IoErr ());

        IFFLength &= 1;

        break;
      }
      case 'SMPH' :
      {
        if ( Invalid )
          return ( ERROR_INVALID_RESIDENT_LIBRARY );

        if ( (CRead ( CFHandle, &(SMPH.smph_Samples), (sizeof (struct IFF_SMPH) - sizeof (struct IFFChunk))) != (sizeof (struct IFF_SMPH) - sizeof (struct IFFChunk))))
          return ( IoErr ());

        *(ULONG *) &(SampleStr->tcs_AutoVibratoEnv) = ReadMWord ( (UBYTE *) &(SMPH.smph_AutoVibratoNum));
        *(ULONG *) &(SampleStr->tcs_AutoTremoloEnv) = ReadMWord ( (UBYTE *) &(SMPH.smph_AutoTremoloNum));
        *(ULONG *) &(SampleStr->tcs_AutoPannoloEnv) = ReadMWord ( (UBYTE *) &(SMPH.smph_AutoPannoloNum));

        memcpy ( &(SampleStr->tcs_Samples), &(SMPH.smph_Samples), (offsetof (struct IFF_SMPH, smph_AutoVibratoNum) - sizeof (struct IFFChunk)));

#ifndef MOTOROLA
        SampleStr->tcs_Samples = ReadMLong ( (UBYTE *) &(SampleStr->tcs_Samples) );
        SampleStr->tcs_Repeat = ReadMLong ( (UBYTE *) &(SampleStr->tcs_Repeat) );
        SampleStr->tcs_RepLen = ReadMLong ( (UBYTE *) &(SampleStr->tcs_RepLen) );
        SampleStr->tcs_RepCount = ReadMLong ( (UBYTE *) &(SampleStr->tcs_RepCount) );
        SampleStr->tcs_SustainRepeat = ReadMLong ( (UBYTE *) &(SampleStr->tcs_SustainRepeat) );
        SampleStr->tcs_SustainRepLen = ReadMLong ( (UBYTE *) &(SampleStr->tcs_SustainRepLen) );
        SampleStr->tcs_SustainRepCount = ReadMLong ( (UBYTE *) &(SampleStr->tcs_SustainRepCount) );
        SampleStr->tcs_Rate = ReadMLong ( (UBYTE *) &(SampleStr->tcs_Rate) );
        SampleStr->tcs_StartOffset = ReadMLong ( (UBYTE *) &(SampleStr->tcs_StartOffset) );
        SampleStr->tcs_LowerRateLimit = ReadMLong ( (UBYTE *) &(SampleStr->tcs_LowerRateLimit) );
        SampleStr->tcs_UpperRateLimit = ReadMLong ( (UBYTE *) &(SampleStr->tcs_UpperRateLimit) );
#endif

        memcpy ( &(SampleStr->tcs_EnvUsageFlags), &(SMPH.smph_EnvUsageFlags), (sizeof (struct IFF_SMPH) - offsetof (struct IFF_SMPH, smph_EnvUsageFlags)));

#ifndef MOTOROLA
        SampleStr->tcs_VibratoSweep = ReadMWord ( (UBYTE *) &(SampleStr->tcs_VibratoSweep) );
        SampleStr->tcs_TremoloSweep = ReadMWord ( (UBYTE *) &(SampleStr->tcs_TremoloSweep) );
        SampleStr->tcs_PannoloSweep = ReadMWord ( (UBYTE *) &(SampleStr->tcs_PannoloSweep) );
#endif

        Invalid = ~Invalid;

        IFFLength = IFFSize - (sizeof (struct IFF_SMPH) - sizeof (struct IFFChunk));

        break;
      }
      case 'BODY' :
      {
        if ( SampleStr->tcs_Length )
          return ( ERROR_INVALID_RESIDENT_LIBRARY );

        SampleStr->tcs_Length = ~SampleStr->tcs_Length;

        if ( FlagPassStr[0].flg_Value )
        {
          SampleStr->tcs_Length = IFFLength;

          if (!( SampleStr->tcs_Data = AllocSampleData ( IFFLength )))
            return ( IoErr ());

          if ( CRead ( CFHandle, SampleStr->tcs_Data, IFFLength ) != IFFLength )
            return ( IoErr ());

          IFFLength &= 1;
        }

        break;
      }
      case 'SMPR' :
      {
        if ( SampleStr->tcs_Length )
          return ( ERROR_INVALID_RESIDENT_LIBRARY );

        SampleStr->tcs_Length = ~SampleStr->tcs_Length;

        if ( CRead ( CFHandle, &(SMPR.smpr_SampleNum), (sizeof (struct IFF_SMPR) - sizeof (struct IFFChunk))) != (sizeof (struct IFF_SMPR) - sizeof (struct IFFChunk)))
          return ( IoErr ());

        *(ULONG *) &(SampleStr->tcs_UserData) = ReadMLong ( (UBYTE *) &(SMPR.smpr_SampleNum));

        IFFLength &= 1;

        break;
      }
      case 'FORM' :
      {
        ULONG FaultCode;

        if ( (CRead ( CFHandle, &IFF_Header, 4 ) != 4 ) )
          return ( IoErr ());

        if ( (ReadMLong ( (UBYTE *) &IFF_Header.ifh_FORMId[0] ) == 'SNTH' ))
        {
          if ( (FaultCode = LoadSmplSnth ( CFHandle, SampleStr, FlagPassStr, IFFSize )))
            return ( FaultCode );

          IFFLength = 0L;
        } else
        {
          if ( CSeek ( CFHandle, -4, OFFSET_CURRENT ) == 0xFFFFFFFF )
            return ( FaultCode );

          goto AllocUnkChunk;
        }

        break;
      }
      default :
      {
AllocUnkChunk:

        if (!( UnkChunk = AllocUnknownChunk ( NULL, ReadMLong ( (UBYTE *) &(IFF_Chunk.iff_Identifier[0])), IFFLength )))
          return ( IoErr ());

        AddSampleUnknownChunk ( SampleStr, UnkChunk );

        if ( (CRead ( CFHandle, &(UnkChunk->uc_Data), IFFLength ) != IFFLength ) )
          return ( IoErr ());

        IFFLength &= 1;

        break;
      }
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
        return ( IoErr ());
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      return ( ERROR_INVALID_RESIDENT_LIBRARY );

  } while ( (Length -= IFFSize) );

  if (!( Invalid ))
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  if (!(*(ULONG *) &(SampleStr->tcs_UserData)) )
  {
    if ( SampleStr->tcs_Samples && ( SampleStr->tcs_Length == 0L ) )
    {
      struct CFHandle *LoadSmpHandle;
      struct TuComposerExtList *SmpExtList;
      union FlagPass *SmpFlagPassStr = NULL;
      BPTR OldLock;
      BPTR SmpDirLock;
      ULONG SmpDefaultSize = 0L;
      ULONG Error;

      if (!( SmpDirLock = Lock ( (STRPTR) FlagPassStr[1].flg_Pointer, SHARED_LOCK )))
        return ( IoErr ());

      if (!( OldLock = CurrentDir ( SmpDirLock )))
      {
        Error = IoErr ();

        UnLock ( SmpDirLock );

        return ( Error );
      }

      if (!( LoadSmpHandle = COpen ( SampleStr->tcs_FileName, MODE_OLDFILE, NULL )))
      {
        Error = IoErr ();

        CurrentDir ( OldLock );
        UnLock ( SmpDirLock );

        return ( Error );
      }

      if ( ( SmpExtList = IdentifySample ( LoadSmpHandle )))
      {
        if ( SmpExtList->extl_LoadFlagStr )
          SmpDefaultSize = SmpExtList->extl_LoadFlagStr->tefs_NumEntries;

        if ( SmpDefaultSize )
        {
          union FlagEntryStruct *FlagEntry = (union FlagEntryStruct *) &(SmpExtList->extl_LoadFlagStr->tefs_Flag);
          union FlagPass *SmpStoreFlagPass = SmpFlagPassStr;
          ULONG i;

          if (!( SmpFlagPassStr = (union FlagPass *) AllocPMem ( SmpDefaultSize * sizeof (LONG))))
          {
            Error = IoErr ();

            CClose ( LoadSmpHandle );

            CurrentDir ( OldLock );
            UnLock ( SmpDirLock );

            return ( Error );
          }

          for (i = SmpDefaultSize; i > 0; i--)
          {
            (*SmpStoreFlagPass++).flg_Value = FlagEntry->tefs_FlagInteger.tefs_FlagDefault;

            FlagEntry++;
          }
        }

        SampleStr = LoadSample ( LoadSmpHandle, SampleStr, SmpExtList, SmpFlagPassStr, 0L );
      } else
      {
        SampleStr = LoadSampleData ( LoadSmpHandle, SampleStr, SampleStr->tcs_Length );
      }

      Error = IoErr ();

      FreePMem ( SmpFlagPassStr, SmpDefaultSize );

      CClose ( LoadSmpHandle );

      CurrentDir ( OldLock );
      UnLock ( SmpDirLock );

      if (!( SampleStr ))
        return ( Error );

    } else
    {
#ifndef MOTOROLA
      if ( SampleStr->tcs_BitDepth != 8 )
      {
        ULONG i;

        if ( SampleStr->tcs_BitDepth == 16 )
        {
          WORD *SampleData = (WORD *) SampleStr->tcs_Data;

          for ( i = ((SampleStr->tcs_Samples + 3UL) >> 2UL); i > 0; i--)
          {
            *SampleData++ = ReadMWord ( (UBYTE *) SampleData );
            *SampleData++ = ReadMWord ( (UBYTE *) SampleData );
            *SampleData++ = ReadMWord ( (UBYTE *) SampleData );
            *SampleData++ = ReadMWord ( (UBYTE *) SampleData );
          }
        } else
        {
          LONG *SampleData = (LONG *) SampleStr->tcs_Data;

          for ( i = ((SampleStr->tcs_Samples + 1UL) >> 1UL); i > 0; i--)
          {
            *SampleData++ = ReadMLong ( (UBYTE *) SampleData );
            *SampleData++ = ReadMLong ( (UBYTE *) SampleData );
          }
        }
      }
#endif

      if ( SampleStr->tcs_Flags & SMPLF_Delta )
        DecrunchSample ( SampleStr, 0L );
    }

    SampleStr->tcs_Flags &= ~(TCSF_Redirect);
  }

  return ( 0L );
}

static ULONG LoadSmplSnth ( struct CFHandle *CFHandle, struct TuComposerSample *SampleStr, union FlagPass *FlagPassStr, unsigned long Length )
{
  struct IFFHeader IFF_Header;
  struct MIFF_YHDR YHDR;
  struct TuComposerSynth *SynthStr;
  ULONG IFFSize;
  UBYTE Invalid = 0;

  if ( Length < 4 )
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  Length -= 4;

  if (!( SynthStr = AllocSynth ( 1L, 0L, 0L )))
    return ( IoErr ());

  AssignSynth ( SampleStr, SynthStr );

  do
  {
    struct IFFChunk IFF_Chunk;
    struct UnknownChunk *UnkChunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      return ( IoErr ());

    IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0]);
    IFFLength = IFFSize;

    IFFSize += (IFFSize & 1);

    switch ( ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] ) )
    {
      case 'FILE' :
      {
        if ( SynthStr->tcy_FileName )
          break;

        if (!( SynthStr->tcy_FileName = (STRPTR) AllocPMem ( IFFLength + 1 )))
          return ( IoErr ());

        if ( (CRead ( CFHandle, SynthStr->tcy_FileName, IFFLength ) != IFFLength) )
          return ( IoErr ());

        IFFLength &= 1;

        break;
      }
      case 'NAME' :
      {
        if ( SynthStr->tcy_Name )
          break;

        if (!( SynthStr->tcy_Name = (STRPTR) AllocPMem ( IFFLength + 1 )))
          return ( IoErr ());

        if ( (CRead ( CFHandle, SynthStr->tcy_Name, IFFLength ) != IFFLength) )
          return ( IoErr ());

        IFFLength &= 1;

        break;
      }
      case 'YHDR' :
      {
        if ( Invalid & 1 )
          return ( ERROR_INVALID_RESIDENT_LIBRARY );

        if ( (CRead ( CFHandle, &(YHDR.myhdr_NumWaves), (sizeof (struct MIFF_YHDR) - sizeof (struct IFFChunk))) != (sizeof (struct MIFF_YHDR) - sizeof (struct IFFChunk))))
          return ( IoErr ());

        memcpy ( &(SynthStr->tcy_VolumePos), &(YHDR.myhdr_VolumePos), (sizeof (struct MIFF_YHDR) - offsetof (struct MIFF_YHDR, myhdr_VolumePos)));

#ifndef MOTOROLA
        SynthStr->tcy_VolumePos = ReadMWord ( (UBYTE *) &(SynthStr->tcy_VolumePos) );
        SynthStr->tcy_PanningPos = ReadMWord ( (UBYTE *) &(SynthStr->tcy_PanningPos) );
        SynthStr->tcy_SlidePos = ReadMWord ( (UBYTE *) &(SynthStr->tcy_SlidePos) );
        SynthStr->tcy_SpecialPos = ReadMWord ( (UBYTE *) &(SynthStr->tcy_SpecialPos) );
        SynthStr->tcy_VolumeSusPos = ReadMWord ( (UBYTE *) &(SynthStr->tcy_VolumeSusPos) );
        SynthStr->tcy_PanningSusPos = ReadMWord ( (UBYTE *) &(SynthStr->tcy_PanningSusPos) );
        SynthStr->tcy_SlideSusPos = ReadMWord ( (UBYTE *) &(SynthStr->tcy_SlideSusPos) );
        SynthStr->tcy_SpecialSusPos = ReadMWord ( (UBYTE *) &(SynthStr->tcy_SpecialSusPos) );
        SynthStr->tcy_VolNNAPos = ReadMWord ( (UBYTE *) &(SynthStr->tcy_VolNNAPos) );
        SynthStr->tcy_PanNNAPos = ReadMWord ( (UBYTE *) &(SynthStr->tcy_PanNNAPos) );
        SynthStr->tcy_SlideNNAPos = ReadMWord ( (UBYTE *) &(SynthStr->tcy_SlideNNAPos) );
        SynthStr->tcy_SpecialNNAPos = ReadMWord ( (UBYTE *) &(SynthStr->tcy_SpecialNNAPos) );
        SynthStr->tcy_VolDNAPos = ReadMWord ( (UBYTE *) &(SynthStr->tcy_VolDNAPos) );
        SynthStr->tcy_PanDNAPos = ReadMWord ( (UBYTE *) &(SynthStr->tcy_PanDNAPos) );
        SynthStr->tcy_SlideDNAPos = ReadMWord ( (UBYTE *) &(SynthStr->tcy_SlideDNAPos) );
        SynthStr->tcy_SpecialDNAPos = ReadMWord ( (UBYTE *) &(SynthStr->tcy_SpecialDNAPos) );
        SynthStr->tcy_Variable0 = ReadMWord ( (UBYTE *) &(SynthStr->tcy_Variable0) );
        SynthStr->tcy_Variable1 = ReadMWord ( (UBYTE *) &(SynthStr->tcy_Variable1) );
        SynthStr->tcy_Variable2 = ReadMWord ( (UBYTE *) &(SynthStr->tcy_Variable2) );
        SynthStr->tcy_Variable3 = ReadMWord ( (UBYTE *) &(SynthStr->tcy_Variable3) );
        SynthStr->tcy_Variable4 = ReadMWord ( (UBYTE *) &(SynthStr->tcy_Variable4) );
        SynthStr->tcy_Variable5 = ReadMWord ( (UBYTE *) &(SynthStr->tcy_Variable5) );
        SynthStr->tcy_Variable6 = ReadMWord ( (UBYTE *) &(SynthStr->tcy_Variable6) );
        SynthStr->tcy_Variable7 = ReadMWord ( (UBYTE *) &(SynthStr->tcy_Variable7) );
        SynthStr->tcy_Variable8 = ReadMWord ( (UBYTE *) &(SynthStr->tcy_Variable8) );
        SynthStr->tcy_Variable9 = ReadMWord ( (UBYTE *) &(SynthStr->tcy_Variable9) );
        SynthStr->tcy_VariableA = ReadMWord ( (UBYTE *) &(SynthStr->tcy_VariableA) );
        SynthStr->tcy_VariableB = ReadMWord ( (UBYTE *) &(SynthStr->tcy_VariableB) );
        SynthStr->tcy_VariableC = ReadMWord ( (UBYTE *) &(SynthStr->tcy_VariableC) );
        SynthStr->tcy_VariableD = ReadMWord ( (UBYTE *) &(SynthStr->tcy_VariableD) );
        SynthStr->tcy_VariableE = ReadMWord ( (UBYTE *) &(SynthStr->tcy_VariableE) );
        SynthStr->tcy_VariableF = ReadMWord ( (UBYTE *) &(SynthStr->tcy_VariableF) );
        SynthStr->tcy_VolConVar = ReadMWord ( (UBYTE *) &(SynthStr->tcy_VolConVar) );
        SynthStr->tcy_PanConVar = ReadMWord ( (UBYTE *) &(SynthStr->tcy_PanConVar) );
        SynthStr->tcy_SlideConVar = ReadMWord ( (UBYTE *) &(SynthStr->tcy_SlideConVar) );
        SynthStr->tcy_SpecialConVar = ReadMWord ( (UBYTE *) &(SynthStr->tcy_SpecialConVar) );
        SynthStr->tcy_VarKeepMask = ReadMWord ( (UBYTE *) &(SynthStr->tcy_VarKeepMask) );
#endif

        Invalid |= 1;

        IFFLength = IFFSize - (sizeof (struct MIFF_YHDR) - sizeof (struct IFFChunk));

        break;
      }
      case 'CODE' :
      {
        struct TuComposerSynthCode *SynthCodeStr;

        if ( Invalid & 2 )
          return ( ERROR_INVALID_RESIDENT_LIBRARY );

        if (!( SynthCodeStr = AllocSynthCode ( IFFSize / sizeof (struct TuComposerSynthCode) )))
          return ( IoErr ());

        AssignSynthCode ( SynthStr, SynthCodeStr, IFFSize / sizeof (struct TuComposerSynthCode));

        if ( CRead ( CFHandle, SynthCodeStr, IFFSize ) != IFFSize )
          return ( IoErr ());

#ifndef MOTOROLA
        for (IFFLength = SynthStr->tcy_CodeEntries; IFFLength > 0; IFFLength--)
        {
          SynthCodeStr->tcyc_InstData = ReadMWord ( (UBYTE *) &(SynthCodeStr->tcyc_InstData) );

          SynthCodeStr++;
        }
#endif
        IFFLength &= 1;

        Invalid |= 2;

        break;
      }
      case 'FORM' :
      {
        ULONG FaultCode;

        if ( (CRead ( CFHandle, &IFF_Header, 4 ) != 4 ) )
          return ( IoErr ());

        switch ( ReadMLong ( (UBYTE *) &IFF_Header.ifh_FORMId[0] ) )
        {
          case 'WFRM' :
          {
            if ( (FaultCode = LoadSnthWfrm ( CFHandle, SynthStr, IFFSize )))
              return ( FaultCode );

            IFFLength = 0L;

            break;
          }
          case 'STAB' :
          {
            if (!( FlagPassStr[2].flg_Value ))
              break;

            if ( (FaultCode = LoadSnthStab ( CFHandle, SynthStr, IFFSize )))
              return ( FaultCode );

            IFFLength = 0L;

            break;
          }
          default :
          {
            if ( CSeek ( CFHandle, -4, OFFSET_CURRENT ) == 0xFFFFFFFF )
              return ( FaultCode );

            goto AllocUnkChunk;
          }
        }

        break;
      }
      default :
      {
AllocUnkChunk:

        if (!( UnkChunk = AllocUnknownChunk ( NULL, ReadMLong ( (UBYTE *) &(IFF_Chunk.iff_Identifier[0])), IFFLength )))
          return ( IoErr ());

        AddSynthUnknownChunk ( SynthStr, UnkChunk );

        if ( (CRead ( CFHandle, &(UnkChunk->uc_Data), IFFLength ) != IFFLength ) )
          return ( IoErr ());

        IFFLength &= 1;

        break;
      }
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
        return ( IoErr ());
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      return ( ERROR_INVALID_RESIDENT_LIBRARY );

  } while ( (Length -= IFFSize) );

  if ( Invalid < 3 )
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  return ( 0L );
}

static ULONG LoadSnthWfrm ( struct CFHandle *CFHandle, struct TuComposerSynth *SynthStr, unsigned long Length )
{
  struct IFFHeader IFF_Header;
  ULONG IFFSize;

  if ( Length < 4 )
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  Length -= 4;

  do
  {
    struct IFFChunk IFF_Chunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      return ( IoErr ());

    if ( (IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0] )) & 1 )
      IFFSize++;

    IFFLength = IFFSize;

    if ( (ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] ) ) == 'FORM' )
    {
      ULONG FaultCode;

      if ( (CRead ( CFHandle, &IFF_Header, 4 ) != 4 ) )
        return ( IoErr ());

      if ( (ReadMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0])) ) == 'WAVE' )
      {
        if ( (FaultCode = LoadWfrmWave ( CFHandle, SynthStr, IFFSize )))
          return ( FaultCode );

        IFFLength = 0L;
      }
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
        return ( IoErr ());
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      return ( ERROR_INVALID_RESIDENT_LIBRARY );

  } while ( (Length -= IFFSize) );

  return ( 0L );
}

static ULONG LoadWfrmWave ( struct CFHandle *CFHandle, struct TuComposerSynth *SynthStr, unsigned long Length )
{
  struct IFF_WHDR WHDR;
  struct TuComposerSynthWave *WaveformStr;
  ULONG IFFSize;
  UBYTE Invalid = 0;

  if ( Length < 4 )
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  Length -= 4;

  if (!( WaveformStr = AllocWaveform ( 1L )))
    return ( IoErr ());

  if ( AddWaveform ( SynthStr, WaveformStr ))
    return ( ERROR_TOO_MANY_LEVELS );

  do
  {
    struct IFFChunk IFF_Chunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      return ( IoErr ());

    IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0]);
    IFFLength = IFFSize;

    IFFSize += (IFFSize & 1);

    switch ( ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] ) )
    {
      case 'FILE' :
      {
        if ( WaveformStr->tcyw_FileName )
          break;

        if (!( WaveformStr->tcyw_FileName = (STRPTR) AllocPMem ( IFFLength + 1 )))
          return ( IoErr ());

        if ( (CRead ( CFHandle, WaveformStr->tcyw_FileName, IFFLength ) != IFFLength) )
          return ( IoErr ());

        IFFLength &= 1;

        break;
      }
      case 'NAME' :
      {
        if ( WaveformStr->tcyw_Name )
          break;

        if (!( WaveformStr->tcyw_Name = (STRPTR) AllocPMem ( IFFLength + 1 )))
          return ( IoErr ());

        if ( (CRead ( CFHandle, WaveformStr->tcyw_Name, IFFLength ) != IFFLength) )
          return ( IoErr ());

        IFFLength &= 1;

        break;
      }
      case 'WHDR' :
      {
        if ( Invalid & 1 )
          return ( ERROR_INVALID_RESIDENT_LIBRARY );

        if ( (CRead ( CFHandle, &(WHDR.whdr_Repeat), (sizeof (struct IFF_WHDR) - sizeof (struct IFFChunk))) != (sizeof (struct IFF_WHDR) - sizeof (struct IFFChunk))))
          return ( IoErr ());

        WaveformStr->tcyw_Repeat = ReadMLong ( (UBYTE *) &(WHDR.whdr_Repeat) );
        WaveformStr->tcyw_RepLen = ReadMLong ( (UBYTE *) &(WHDR.whdr_RepLen) );
        WaveformStr->tcyw_Flags = ReadMWord ( (UBYTE *) &(WHDR.whdr_Flags) );

        Invalid |= 1;

        IFFLength = IFFSize - (sizeof (struct IFF_WHDR) - sizeof (struct IFFChunk));

        break;
      }
      case 'BODY' :
      {
        WORD *WaveData;

        if ( Invalid & 2 )
          return ( ERROR_INVALID_RESIDENT_LIBRARY );

        if (!( WaveData = (WORD *) AllocPMem ( IFFLength )))
          return ( IoErr ());

        FreePMem ( WaveformStr->tcyw_WaveData, 2L );

        WaveformStr->tcyw_WaveData = WaveData;
        WaveformStr->tcyw_Length = IFFLength;

        if ( CRead ( CFHandle, WaveData, IFFLength ) != IFFLength )
          return ( IoErr ());

        Invalid |= 2;

        IFFLength &= 1;

        break;
      }
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
        return ( IoErr ());
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      return ( ERROR_INVALID_RESIDENT_LIBRARY );

  } while ( (Length -= IFFSize) );

  if ( Invalid < 3 )
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  Length = WaveformStr->tcyw_Length;

  if (!( WaveformStr->tcyw_Flags & TCYWF_8Bit ))
    Length >>= 1UL;

  WaveformStr->tcyw_Samples = Length;

#ifndef MOTOROLA
  if (!( WaveformStr->tcyw_Flags & TCYWF_8Bit ))
  {
    WORD *WaveData = WaveformStr->tcyw_WaveData;
    ULONG i;

    for (i = ((WaveformStr->tcyw_Samples + 3UL) >> 2UL); i > 0; i--)
    {
      *WaveData++ = ReadMWord ( (UBYTE *) WaveData);
      *WaveData++ = ReadMWord ( (UBYTE *) WaveData);
      *WaveData++ = ReadMWord ( (UBYTE *) WaveData);
      *WaveData++ = ReadMWord ( (UBYTE *) WaveData);
    }
  }
#endif

  return ( 0L );
}

static ULONG LoadSnthStab ( struct CFHandle *CFHandle, struct TuComposerSynth *SynthStr, unsigned long Length )
{
  struct IFFHeader IFF_Header;
  ULONG IFFSize;

  if ( Length < 4 )
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  Length -= 4;

  do
  {
    struct IFFChunk IFF_Chunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      return ( IoErr ());

    if ( (IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0] )) & 1 )
      IFFSize++;

    IFFLength = IFFSize;

    if ( (ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] ) ) == 'FORM' )
    {
      ULONG FaultCode;

      if ( (CRead ( CFHandle, &IFF_Header, 4 ) != 4 ) )
        return ( IoErr ());

      if ( (ReadMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0])) ) == 'SMBL' )
      {
        if ( (FaultCode = LoadStabSmbl ( CFHandle, SynthStr, IFFSize )))
          return ( FaultCode );

        IFFLength = 0L;
      }
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
        return ( IoErr ());
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      return ( ERROR_INVALID_RESIDENT_LIBRARY );

  } while ( (Length -= IFFSize) );

  return ( 0L );
}

static ULONG LoadStabSmbl ( struct CFHandle *CFHandle, struct TuComposerSynth *SynthStr, unsigned long Length )
{
  struct IFF_SREF SREF;
  struct TuComposerSynthSymbolTable *SymbolStr;
  ULONG Invalid = 0L, IFFSize;

  if ( Length < 4 )
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  Length -= 4;

  if (!( SymbolStr = AllocSymbol ( "UNNAMED" )))
    return ( IoErr ());

  AddSymbol ( SynthStr, SymbolStr );

  do
  {
    struct IFFChunk IFF_Chunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      return ( IoErr ());

    IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0]);
    IFFLength = IFFSize;

    IFFSize += (IFFSize & 1);

    switch ( ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] ) )
    {
      case 'NAME' :
      {
        STRPTR SymbolName;

        if (!( SymbolName = (STRPTR) AllocPMem ( IFFLength + 1 )))
          return ( IoErr ());

        if ( (CRead ( CFHandle, SymbolName, IFFLength ) != IFFLength) )
          return ( IoErr ());

        if ( AssignSymbol ( SymbolStr, SymbolName ))
          return ( IoErr ());

        IFFLength &= 1;

        break;
      }
      case 'SREF' :
      {
        if ( Invalid )
          return ( ERROR_INVALID_RESIDENT_LIBRARY );

        if ( (CRead ( CFHandle, &(SREF.sref_SymbolValue), (sizeof (struct IFF_SREF) - sizeof (struct IFFChunk))) != (sizeof (struct IFF_SREF) - sizeof (struct IFFChunk))))
          return ( IoErr ());

        SymbolStr->tsst_SymbolValue = ReadMWord ( (UBYTE *) &(SREF.sref_SymbolValue) );
        SymbolStr->tsst_FirstLine = ReadMWord ( (UBYTE *) &(SREF.sref_FirstLine) );
        SymbolStr->tsst_LastLine = ReadMWord ( (UBYTE *) &(SREF.sref_LastLine) );
        SymbolStr->tsst_Type = SREF.sref_Type;
        SymbolStr->tsst_Flags = SREF.sref_Flags;

        Invalid = ~Invalid;

        IFFLength = IFFSize - (sizeof (struct IFF_SREF) - sizeof (struct IFFChunk));

        break;
      }
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
        return ( IoErr ());
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      return ( ERROR_INVALID_RESIDENT_LIBRARY );

  } while ( (Length -= IFFSize) );

  if ( Invalid )
    return ( 0L );

  return ( ERROR_INVALID_RESIDENT_LIBRARY );
}

static ULONG LoadTCSEnvl ( struct CFHandle *CFHandle, struct TuComposerModule *ModuleStr, unsigned long Length )
{
  struct IFFHeader IFF_Header;
  ULONG IFFSize;

  if ( Length < 4 )
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  Length -= 4;

  do
  {
    struct IFFChunk IFF_Chunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      return ( IoErr ());

    if ( (IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0] )) & 1 )
      IFFSize++;

    IFFLength = IFFSize;

    if ( (ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] ) ) == 'FORM' )
    {
      ULONG FaultCode;

      if ( (CRead ( CFHandle, &IFF_Header, 4 ) != 4 ) )
        return ( IoErr ());

      if ( (ReadMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0])) ) == 'ENVD' )
      {
        if ( (FaultCode = LoadEnvlEnvd ( CFHandle, ModuleStr, IFFSize )))
          return ( FaultCode );

        IFFLength = 0L;
      }
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
        return ( IoErr ());
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      return ( ERROR_INVALID_RESIDENT_LIBRARY );

  } while ( (Length -= IFFSize) );

  return ( 0L );
}

static ULONG LoadEnvlEnvd ( struct CFHandle *CFHandle, struct TuComposerModule *ModuleStr, unsigned long Length )
{
  struct MIFF_EHDR EHDR;
  struct TuComposerInstrEnvelope *EnvStr;

#ifndef MOTOROLA
  WORD *EnvData;
  UWORD *EnvNode;
#endif

  ULONG Invalid = 0L, IFFSize;

  if ( Length < 4 )
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  Length -= 4;

  if (!( EnvStr = CreateEnvelope ( 1L, ENVL_EMPTY, 0L, 0L, 0L )))
    return ( IoErr ());

  AddEnvelope ( ModuleStr, EnvStr );

  do
  {
    struct IFFChunk IFF_Chunk;
    struct UnknownChunk *UnkChunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      return ( IoErr ());

    IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0]);
    IFFLength = IFFSize;

    IFFSize += (IFFSize & 1);

    switch ( ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] ) )
    {
      case 'EHDR' :
      {
        if ( Invalid )
          return ( ERROR_INVALID_RESIDENT_LIBRARY );

        if ( (CRead ( CFHandle, &(EHDR.mehdr_Flags), (sizeof (struct MIFF_EHDR) - sizeof (struct IFFChunk))) != (sizeof (struct MIFF_EHDR) - sizeof (struct IFFChunk))))
          return ( IoErr ());

        EnvStr->tcie_Flags = ReadMWord ( (UBYTE *) &(EHDR.mehdr_Flags) );
        EnvStr->tcie_Tempo = ReadMWord ( (UBYTE *) &(EHDR.mehdr_Tempo) );
        EnvStr->tcie_SustainStart = ReadMWord ( (UBYTE *) &(EHDR.mehdr_SustainStart) );
        EnvStr->tcie_SustainEnd = ReadMWord ( (UBYTE *) &(EHDR.mehdr_SustainEnd) );
        EnvStr->tcie_SustainCount = ReadMWord ( (UBYTE *) &(EHDR.mehdr_SustainCount) );
        EnvStr->tcie_LoopStart = ReadMWord ( (UBYTE *) &(EHDR.mehdr_LoopStart) );
        EnvStr->tcie_LoopEnd = ReadMWord ( (UBYTE *) &(EHDR.mehdr_LoopEnd) );
        EnvStr->tcie_LoopCount = ReadMWord ( (UBYTE *) &(EHDR.mehdr_LoopCount) );
        EnvStr->tcie_LowestValue = ReadMWord ( (UBYTE *) &(EHDR.mehdr_LowestValue) );
        EnvStr->tcie_HighestValue = ReadMWord ( (UBYTE *) &(EHDR.mehdr_HighestValue) );

        Invalid = ~Invalid;

        IFFLength = IFFSize - (sizeof (struct MIFF_EHDR) - sizeof (struct IFFChunk));

        break;
      }
      case 'BODY' :
      {
        if ( ReSizeEnvelope ( EnvStr, IFFLength / sizeof (WORD)))
          return ( IoErr ());

        if ( CRead ( CFHandle, EnvStr->tcie_Data, IFFLength ) != IFFLength )
          return ( IoErr ());

        IFFLength &= 1;

        break;
      }
      case 'NODE' :
      {
        if ( EnvStr->tcie_NodePoints )
          return ( ERROR_INVALID_RESIDENT_LIBRARY );

        if (!( EnvStr->tcie_NodePoints = (UWORD *) AllocPMem ( IFFLength )))
          return ( IoErr ());

        EnvStr->tcie_NumNodes = IFFLength / sizeof (UWORD);

        if ( CRead ( CFHandle, EnvStr->tcie_NodePoints, IFFLength ) != IFFLength )
          return ( IoErr ());

        IFFLength &= 1;

        break;
      }
      default :
      {
        if (!( UnkChunk = AllocUnknownChunk ( NULL, ReadMLong ( (UBYTE *) &(IFF_Chunk.iff_Identifier[0])), IFFLength )))
          return ( IoErr ());

        AddEnvelopeUnknownChunk ( EnvStr, UnkChunk );

        if ( (CRead ( CFHandle, &(UnkChunk->uc_Data), IFFSize ) != IFFSize ) )
          return ( IoErr ());

        IFFLength &= 1;

        break;
      }
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
        return ( IoErr ());
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      return ( ERROR_INVALID_RESIDENT_LIBRARY );

  } while ( (Length -= IFFSize) );

  if (!( Invalid ))
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

#ifndef MOTOROLA
  EnvData = EnvStr->tcie_Data;

  for ( Length = EnvStr->tcie_NumPoints; Length > 0; Length--)
  {
    *EnvData++ = ReadMWord ( (UBYTE *) EnvData);
  }

  EnvNode = EnvStr->tcie_NodePoints;

  for ( Length = EnvStr->tcie_NumNodes; Length > 0; Length--)
  {
    *EnvNode++ = ReadMWord ( (UBYTE *) EnvNode);
  }
#endif

  return ( 0L );
}

static ULONG LoadTCSKeyb ( struct CFHandle *CFHandle, struct TuComposerModule *ModuleStr, unsigned long Length )
{
  ULONG IFFSize;

  if ( Length < 4 )
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  Length -= 4;

  do
  {
    struct IFFChunk IFF_Chunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      return ( IoErr ());

    if ( (IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0] )) & 1 )
      IFFSize++;

    IFFLength = IFFSize;

    if ( (ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0])) == 'KBRD' )
    {
      struct TuComposerInstrKeyboard *KeybrdStr;

#ifndef MOTOROLA
      struct KeyboardEntry *KeybrdEntry;
      ULONG i;
#endif

      if (!( KeybrdStr = AllocKeyboard ()))
        return ( IoErr ());

      AddKeyboard ( ModuleStr, KeybrdStr );

      if ( CRead ( CFHandle, &(KeybrdStr->tcik_NoteEntry[0]), (sizeof (struct IFF_KBRD) - sizeof (struct IFFChunk))) != (sizeof (struct IFF_KBRD) - sizeof (struct IFFChunk)))
        return ( IoErr ());

#ifndef MOTOROLA
      KeybrdEntry = (struct KeyboardEntry *) &(KeybrdStr->tcik_NoteEntry[0]);

      for (i = NUMOCTAVES * NUMNOTES; i > 0; i--)
      {
        KeybrdEntry->tcik_Sample_C_0 = ReadMWord ( (UBYTE *) &(KeybrdEntry->tcik_Sample_C_0));

        KeybrdEntry++;
      }
#endif
      IFFLength = IFFSize - (sizeof (struct IFF_KBRD) - sizeof (struct IFFChunk));
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
        return ( IoErr ());
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      return ( ERROR_INVALID_RESIDENT_LIBRARY );

  } while ( (Length -= IFFSize) );

  return ( 0L );
}

static ULONG LoadTCSArpl ( struct CFHandle *CFHandle, struct TuComposerModule *ModuleStr, unsigned long Length )
{
  struct IFFHeader IFF_Header;
  ULONG IFFSize;

  if ( Length < 4 )
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  Length -= 4;

  do
  {
    struct IFFChunk IFF_Chunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      return ( IoErr ());

    if ( (IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0] )) & 1 )
      IFFSize++;

    IFFLength = IFFSize;

    if ( (ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] ) ) == 'FORM' )
    {
      ULONG FaultCode;

      if ( (CRead ( CFHandle, &IFF_Header, 4 ) != 4 ) )
        return ( IoErr ());

      if ( (ReadMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0])) ) == 'ARPG' )
      {
        if ( (FaultCode = LoadArplArpg ( CFHandle, ModuleStr, IFFSize )))
          return ( FaultCode );

        IFFLength = 0L;
      }
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
        return ( IoErr ());
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      return ( ERROR_INVALID_RESIDENT_LIBRARY );

  } while ( (Length -= IFFSize) );

  return ( 0L );
}

static ULONG LoadArplArpg ( struct CFHandle *CFHandle, struct TuComposerModule *ModuleStr, unsigned long Length )
{
  struct IFFHeader IFF_Header;
  struct IFF_AHDR AHDR;
  struct TuComposerInstrArpeggio *ArpStr;

#ifndef MOTOROLA
  struct TuComposerInstrArpeggioData *ArpData;
#endif

  ULONG Invalid = 0L, IFFSize;

  if ( Length < 4 )
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  Length -= 4;

  if (!( ArpStr = AllocArpeggio ( 1L )))
    return ( IoErr ());

  AddArpeggio ( ModuleStr, ArpStr );

  do
  {
    struct IFFChunk IFF_Chunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      return ( IoErr ());

    IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0]);
    IFFLength = IFFSize;

    IFFSize += (IFFSize & 1);

    switch ( ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] ) )
    {
      case 'AHDR' :
      {
        if ( Invalid )
          return ( ERROR_INVALID_RESIDENT_LIBRARY );

        if ( (CRead ( CFHandle, &(AHDR.ahdr_Length), (sizeof (struct IFF_AHDR) - sizeof (struct IFFChunk))) != (sizeof (struct IFF_AHDR) - sizeof (struct IFFChunk))))
          return ( IoErr ());

        ArpStr->tcia_Flags = ReadMWord ( (UBYTE *) &(AHDR.ahdr_Flags) );
        ArpStr->tcia_SustainStart = ReadMWord ( (UBYTE *) &(AHDR.ahdr_SustainStart) );
        ArpStr->tcia_SustainEnd = ReadMWord ( (UBYTE *) &(AHDR.ahdr_SustainEnd) );
        ArpStr->tcia_SustainCount = ReadMWord ( (UBYTE *) &(AHDR.ahdr_SustainCount) );
        ArpStr->tcia_LoopStart = ReadMWord ( (UBYTE *) &(AHDR.ahdr_LoopStart) );
        ArpStr->tcia_LoopEnd = ReadMWord ( (UBYTE *) &(AHDR.ahdr_LoopEnd) );
        ArpStr->tcia_LoopCount = ReadMWord ( (UBYTE *) &(AHDR.ahdr_LoopCount) );

        if ( ReSizeArpeggio ( ArpStr, ReadMWord ( (UBYTE *) &(AHDR.ahdr_Length))))
          return ( IoErr ());

        Invalid = ~Invalid;

        IFFLength = IFFSize - (sizeof (struct IFF_AHDR) - sizeof (struct IFFChunk));

        break;
      }
      case 'FORM' :
      {
        ULONG FaultCode;

        if ( (CRead ( CFHandle, &IFF_Header, 4 ) != 4 ) )
          return ( IoErr ());

        if ( (ReadMLong ( (UBYTE *) &IFF_Header.ifh_FORMId[0] ) == 'ARPE' ))
        {
          if ( (FaultCode = LoadArpgArpe ( CFHandle, ArpStr, IFFSize )))
            return ( FaultCode );

          IFFLength = 0L;
        }

        break;
      }
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
        return ( IoErr ());
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      return ( ERROR_INVALID_RESIDENT_LIBRARY );

  } while ( (Length -= IFFSize) );

  if (!( Invalid ))
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

#ifndef MOTOROLA
  ArpData = ArpStr->tcia_Data;

  for ( Length = ArpStr->tcia_Entries; Length > 0; Length--)
  {
    ArpData->tciad_DataWord = ReadMWord ( (UBYTE *) &(ArpData->tciad_DataWord));
    ArpData->tciad_DataWord2 = ReadMWord ( (UBYTE *) &(ArpData->tciad_DataWord2));
    ArpData->tciad_DataWord3 = ReadMWord ( (UBYTE *) &(ArpData->tciad_DataWord3));
    ArpData->tciad_DataWord4 = ReadMWord ( (UBYTE *) &(ArpData->tciad_DataWord4));

    ArpData++;
  }
#endif

  return ( 0L );
}

static ULONG LoadArpgArpe ( struct CFHandle *CFHandle, struct TuComposerInstrArpeggio *ArpStr, unsigned long Length )
{
  ULONG IFFSize, NumTick = 0L;

  if ( Length < 4 )
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  Length -= 4;

  do
  {
    struct IFFChunk IFF_Chunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      return ( IoErr ());

    if ( (IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0] )) & 1 )
      IFFSize++;

    IFFLength = IFFSize;

    if ( (ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0])) == 'ARPE' )
    {
      if ( NumTick >= ArpStr->tcia_Entries )
      {
        if ( AddArpeggioTick ( ArpStr, 1L ))
          return ( IoErr ());
      }

      if ( CRead ( CFHandle, &(ArpStr->tcia_Data[NumTick]), sizeof (struct TuComposerInstrArpeggioData)) != sizeof (struct TuComposerInstrArpeggioData))
        return ( IoErr ());

      NumTick++;

      IFFLength = IFFSize - sizeof (struct TuComposerInstrArpeggioData);
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
        return ( IoErr ());
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      return ( ERROR_INVALID_RESIDENT_LIBRARY );

  } while ( (Length -= IFFSize) );

  return ( 0L );
}

static ULONG SaveTCS ( struct CFHandle *CFHandle,
    struct TuComposerSong *SongStr, union FlagPass *FlagPassStr )
{
  struct IFFHeader IFF_Header;
  struct IFF_MHDR MHDR;
  struct TuComposerModule *ModuleStr = NULL;
  struct TuComposerSample *SampleStr;
  struct TuComposerInstr **InstrList = NULL;
  struct TuComposerInstrEnvelope **EnvList = NULL;
  struct TuComposerInstrKeyboard **KeybrdList = NULL;
  struct TuComposerInstrArpeggio **ArpList = NULL;
  struct TuComposerInstr **InstrScan;
  struct TuComposerInstrEnvelope **EnvScan;
  struct TuComposerInstrKeyboard **KeybrdScan;
  struct TuComposerInstrArpeggio **ArpScan;
  ULONG TCSSize = sizeof (ULONG), FORMSize;
  ULONG TCSLenPos, FORMLenPos;
  ULONG FaultCode, SaveFlags = 0L;
  ULONG NumSamples, NumSynths;
  UWORD NumInstrs, NumEnvelopes, NumKeyboards, NumArpeggios;

  if ( SongStr->tcso_Node.mln_Succ )
  {
    ModuleStr = (struct TuComposerModule *) &(SongStr->tcso_Node.mln_Succ);

    for (;;)
    {
      ModuleStr = (struct TuComposerModule *) ModuleStr->tcm_Node.mln_Pred;

      if (!( ModuleStr->tcm_Node.mln_Pred ))
        break;
    }

    (UBYTE *) ModuleStr += -offsetof (struct TuComposerModule, tcm_SongList);
  } else
  {
    if (!( ModuleStr = AllocTCModule ( SongStr->tcso_Channels )))
      return ( ERROR_SONG_NOMEM );

    SaveFlags = ~SaveFlags;
  }

  if (!( EnvList = GetSongEnvelopes ( SongStr )))
  {
SaveTCSNoMem:

    FreeSongInstrs ( InstrList );
    FreeSongArpeggios ( ArpList );
    FreeSongKeyboards ( KeybrdList );
    FreeSongEnvelopes ( EnvList );

    if ( SaveFlags )
      FreeTCModule ( ModuleStr );

    return ( ERROR_SONG_NOMEM );
  }

  if (!( KeybrdList = GetSongKeyboards ( SongStr )))
    goto SaveTCSNoMem;

  if (!( ArpList = GetSongArpeggios ( SongStr )))
    goto SaveTCSNoMem;

  if (!( InstrList = GetSongInstrs ( SongStr )))
    goto SaveTCSNoMem;

  if ( (TCSLenPos = CSeek ( CFHandle, 0L, OFFSET_CURRENT )) == 0xFFFFFFFF )
  {
SaveTCSIOErr:

    FreeSongInstrs ( InstrList );
    FreeSongArpeggios ( ArpList );
    FreeSongKeyboards ( KeybrdList );
    FreeSongEnvelopes ( EnvList );

    if ( SaveFlags )
      FreeTCModule ( ModuleStr );

    return ( ERROR_SONG_IOERR );
  }

  TCSLenPos += sizeof (ULONG);

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0]), 'FORM' );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), TCSSize );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_TypeId[0]), 'TCM1' );

  if ( (CWrite ( CFHandle, &IFF_Header, sizeof (struct IFFHeader )) != sizeof (struct IFFHeader )))
    goto SaveTCSIOErr;

  if ( (FaultCode = SaveTCSName ( CFHandle, ModuleStr->tcm_FileName, 'FILE', (ULONG *) &TCSSize, ERROR_SONG_IOERR )))
  {
SaveTCSFault:

    FreeSongInstrs ( InstrList );
    FreeSongArpeggios ( ArpList );
    FreeSongKeyboards ( KeybrdList );
    FreeSongEnvelopes ( EnvList );

    if ( SaveFlags )
      FreeTCModule ( ModuleStr );

    return ( FaultCode );
  }

  if ( (FaultCode = SaveTCSName ( CFHandle, ModuleStr->tcm_Name, 'NAME', (ULONG *) &TCSSize, ERROR_SONG_IOERR )))
    goto SaveTCSFault;

  if ( (FaultCode = SaveTCSName ( CFHandle, ModuleStr->tcm_Message, 'MMSG', (ULONG *) &TCSSize, ERROR_SONG_IOERR )))
    goto SaveTCSFault;

  if ( (FaultCode = SaveTCSName ( CFHandle, ModuleStr->tcm_Composer, 'AUTH', (ULONG *) &TCSSize, ERROR_SONG_IOERR )))
    goto SaveTCSFault;

  if ( (FaultCode = SaveTCSName ( CFHandle, ModuleStr->tcm_Style, 'STIL', (ULONG *) &TCSSize, ERROR_SONG_IOERR )))
    goto SaveTCSFault;

  if ( (FaultCode = SaveTCSName ( CFHandle, ModuleStr->tcm_Comment, 'CMNT', (ULONG *) &TCSSize, ERROR_SONG_IOERR )))
    goto SaveTCSFault;

  MHDR.mhdr_Version = CURRENT_VERSION;
  MHDR.mhdr_Revision = CURRENT_REVISION;

  memcpy ( &(MHDR.mhdr_BeginDate), &(ModuleStr->tcm_BeginDate), (offsetof (struct IFF_MHDR, mhdr_Songs) - offsetof (struct IFF_MHDR, mhdr_BeginDate )));

  if (!( *(ULONG *) &(ModuleStr->tcm_EndDate) ))
    Stamp2Date ( &(MHDR.mhdr_EndDate), NULL );

#ifndef MOTOROLA
  WriteMWord ( (UBYTE *) &(MHDR.mhdr_BeginDate.sds_Year), MHDR.mhdr_BeginDate.sds_Year );
  WriteMWord ( (UBYTE *) &(MHDR.mhdr_EndDate.sds_Year), MHDR.mhdr_EndDate.sds_Year );
#endif

  WriteMWord ( (UBYTE *) &(MHDR.mhdr_Songs), 1 );
  WriteMLong ( (UBYTE *) &(MHDR.mhdr_Tracks), GetNumTracks ( SongStr ));

  InstrScan = InstrList;
  NumInstrs = -1;
  NumSamples = 0L;
  NumSynths = 0L;

  for (;;)
  {
    struct TuComposerInstr *InstrStr;

    NumInstrs++;

    if (!( InstrStr = *InstrScan++ ))
      break;

    SampleStr = (struct TuComposerSample *) &(InstrStr->tci_Samples.mlh_Head);

    for (;;)
    {
      SampleStr = (struct TuComposerSample *) SampleStr->tcs_Node.mln_Succ;

      if (!( SampleStr->tcs_Node.mln_Succ ))
        break;

      NumSamples++;

      if ( SampleStr->tcs_Synth )
        NumSynths++;
    }
  }

  WriteMWord ( (UBYTE *) &(MHDR.mhdr_Instruments), NumInstrs );
  WriteMLong ( (UBYTE *) &(MHDR.mhdr_Samples), NumSamples );
  WriteMLong ( (UBYTE *) &(MHDR.mhdr_Synths), NumSynths );

  EnvScan = EnvList;
  NumEnvelopes = -1;
  NumKeyboards = -1;
  NumArpeggios = -1;

  for (;;)
  {
    NumEnvelopes++;

    if (!( *EnvScan++ ))
      break;
  }

  KeybrdScan = KeybrdList;

  for (;;)
  {
    NumKeyboards++;

    if (!( *KeybrdScan++ ))
      break;
  }

  ArpScan = ArpList;

  for (;;)
  {
    NumArpeggios++;

    if (!( *ArpScan++ ))
      break;
  }

  WriteMWord ( (UBYTE *) &(MHDR.mhdr_Envelopes), NumEnvelopes );
  WriteMWord ( (UBYTE *) &(MHDR.mhdr_KeyboardDefs), NumKeyboards );
  WriteMWord ( (UBYTE *) &(MHDR.mhdr_ArpeggioDefs), NumArpeggios );

  WriteMWord ( (UBYTE *) &(MHDR.mhdr_Channels), ModuleStr->tcm_Channels );
  MHDR.mhdr_CompFlags = ModuleStr->tcm_CompFlags;
  MHDR.mhdr_Flags = ModuleStr->tcm_Flags;

  if ( (FaultCode = SaveTCSChunk ( CFHandle, (struct IFFChunk *) &MHDR, 'MHDR', sizeof (struct IFF_MHDR), (ULONG *) &TCSSize, ERROR_SONG_IOERR )))
    goto SaveTCSFault;

  if ( (FaultCode = SaveTCSSong ( CFHandle, SongStr, InstrList, (ULONG *) &TCSSize )))
    goto SaveTCSFault;

  InstrScan = InstrList;

  if ( *InstrScan )
  {
    TCSSize += (sizeof (struct IFFHeader) - sizeof (ULONG));
    FORMSize = sizeof (ULONG);

    if ( (FORMLenPos = CSeek ( CFHandle, 0L, OFFSET_CURRENT )) == 0xFFFFFFFF )
      goto SaveTCSIOErr;

    FORMLenPos += sizeof (ULONG);

    WriteMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0]), 'FORM' );
    WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), FORMSize );
    WriteMLong ( (UBYTE *) &(IFF_Header.ifh_TypeId[0]), 'INSL' );

    if ( (CWrite ( CFHandle, &IFF_Header, sizeof (struct IFFHeader )) != sizeof (struct IFFHeader )))
      goto SaveTCSIOErr;

    for (;;)
    {
      struct TuComposerInstr *InstrStr;

      if (!( InstrStr = *InstrScan++ ))
        break;

      if ( (FaultCode = SaveTCSInst ( CFHandle, ModuleStr, InstrStr, (ULONG *) &FORMSize, FlagPassStr )))
        goto SaveTCSFault;
    }

    if ( (FORMLenPos = CSeek ( CFHandle, FORMLenPos, OFFSET_BEGINNING )) == 0xFFFFFFFF )
      goto SaveTCSIOErr;

    WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), FORMSize );

    if ( CWrite ( CFHandle, &(IFF_Header.ifh_Length[0]), sizeof (ULONG)) != sizeof (ULONG))
      goto SaveTCSIOErr;

    if ( CSeek ( CFHandle, FORMLenPos, OFFSET_BEGINNING ) == 0xFFFFFFFF )
      goto SaveTCSIOErr;

    TCSSize += FORMSize;
  }

  EnvScan = EnvList;

  if ( *EnvScan )
  {
    TCSSize += (sizeof (struct IFFHeader) - sizeof (ULONG));
    FORMSize = sizeof (ULONG);

    if ( (FORMLenPos = CSeek ( CFHandle, 0L, OFFSET_CURRENT )) == 0xFFFFFFFF )
      goto SaveTCSIOErr;

    FORMLenPos += sizeof (ULONG);

    WriteMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0]), 'FORM' );
    WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), FORMSize );
    WriteMLong ( (UBYTE *) &(IFF_Header.ifh_TypeId[0]), 'ENVL' );

    if ( (CWrite ( CFHandle, &IFF_Header, sizeof (struct IFFHeader )) != sizeof (struct IFFHeader )))
      goto SaveTCSIOErr;

    for (;;)
    {
      struct TuComposerInstrEnvelope *EnvStr;

      if (!( EnvStr = *EnvScan++ ))
        break;

      if ( (FaultCode = SaveTCSEnvd ( CFHandle, EnvStr, (ULONG *) &FORMSize )))
        goto SaveTCSFault;
    }

    if ( (FORMLenPos = CSeek ( CFHandle, FORMLenPos, OFFSET_BEGINNING )) == 0xFFFFFFFF )
      goto SaveTCSIOErr;

    WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), FORMSize );

    if ( CWrite ( CFHandle, &(IFF_Header.ifh_Length[0]), sizeof (ULONG)) != sizeof (ULONG))
      goto SaveTCSIOErr;

    if ( CSeek ( CFHandle, FORMLenPos, OFFSET_BEGINNING ) == 0xFFFFFFFF )
      goto SaveTCSIOErr;

    TCSSize += FORMSize;
  }

  KeybrdScan = KeybrdList;

  if ( *KeybrdScan )
  {
    TCSSize += (sizeof (struct IFFHeader) - sizeof (ULONG));
    FORMSize = sizeof (ULONG);

    if ( (FORMLenPos = CSeek ( CFHandle, 0L, OFFSET_CURRENT )) == 0xFFFFFFFF )
      goto SaveTCSIOErr;

    FORMLenPos += sizeof (ULONG);

    WriteMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0]), 'FORM' );
    WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), FORMSize );
    WriteMLong ( (UBYTE *) &(IFF_Header.ifh_TypeId[0]), 'KEYB' );

    if ( (CWrite ( CFHandle, &IFF_Header, sizeof (struct IFFHeader )) != sizeof (struct IFFHeader )))
      goto SaveTCSIOErr;

    for (;;)
    {
      struct TuComposerInstrKeyboard *KeybrdStr;

      if (!( KeybrdStr = *KeybrdScan++ ))
        break;

      if ( (FaultCode = SaveTCSKbrd ( CFHandle, KeybrdStr, (ULONG *) &FORMSize )))
        goto SaveTCSFault;
    }

    if ( (FORMLenPos = CSeek ( CFHandle, FORMLenPos, OFFSET_BEGINNING )) == 0xFFFFFFFF )
      goto SaveTCSIOErr;

    WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), FORMSize );

    if ( CWrite ( CFHandle, &(IFF_Header.ifh_Length[0]), sizeof (ULONG)) != sizeof (ULONG))
      goto SaveTCSIOErr;

    if ( CSeek ( CFHandle, FORMLenPos, OFFSET_BEGINNING ) == 0xFFFFFFFF )
      goto SaveTCSIOErr;

    TCSSize += FORMSize;
  }

  ArpScan = ArpList;

  if ( *ArpScan )
  {
    TCSSize += (sizeof (struct IFFHeader) - sizeof (ULONG));
    FORMSize = sizeof (ULONG);

    if ( (FORMLenPos = CSeek ( CFHandle, 0L, OFFSET_CURRENT )) == 0xFFFFFFFF )
      goto SaveTCSIOErr;

    FORMLenPos += sizeof (ULONG);

    WriteMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0]), 'FORM' );
    WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), FORMSize );
    WriteMLong ( (UBYTE *) &(IFF_Header.ifh_TypeId[0]), 'ARPL' );

    if ( (CWrite ( CFHandle, &IFF_Header, sizeof (struct IFFHeader )) != sizeof (struct IFFHeader )))
      goto SaveTCSIOErr;

    for (;;)
    {
      struct TuComposerInstrArpeggio *ArpStr;

      if (!( ArpStr = *ArpScan++ ))
        break;

      if ( (FaultCode = SaveTCSArpg ( CFHandle, ArpStr, (ULONG *) &FORMSize )))
        goto SaveTCSFault;
    }

    if ( (FORMLenPos = CSeek ( CFHandle, FORMLenPos, OFFSET_BEGINNING )) == 0xFFFFFFFF )
      goto SaveTCSIOErr;

    WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), FORMSize );

    if ( CWrite ( CFHandle, &(IFF_Header.ifh_Length[0]), sizeof (ULONG)) != sizeof (ULONG))
      goto SaveTCSIOErr;

    if ( CSeek ( CFHandle, FORMLenPos, OFFSET_BEGINNING ) == 0xFFFFFFFF )
      goto SaveTCSIOErr;

    TCSSize += FORMSize;
  }

  FreeSongInstrs ( InstrList );
  FreeSongArpeggios ( ArpList );
  FreeSongKeyboards ( KeybrdList );
  FreeSongEnvelopes ( EnvList );

  if ( SaveFlags )
    FreeTCModule ( ModuleStr );

  if ( CSeek ( CFHandle, TCSLenPos, OFFSET_BEGINNING ) == 0xFFFFFFFF )
    return ( ERROR_SONG_IOERR );

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), TCSSize );

  if ( CWrite ( CFHandle, &(IFF_Header.ifh_Length[0]), sizeof (ULONG)) != sizeof (ULONG))
    return ( ERROR_SONG_IOERR );

  return ( 0L );
}

static ULONG SaveTCSChunk ( struct CFHandle *CFHandle, struct IFFChunk *IFF_Chunk, unsigned long ChunkName, unsigned long ChunkLen, ULONG *ChunkLenBuf, unsigned long ErrorCode )
{
  ChunkLen -= sizeof (struct IFFChunk );

  if ( ChunkLen )
  {
    WriteMLong ( (UBYTE *) &(IFF_Chunk->iff_Identifier[0]), ChunkName );
    WriteMLong ( (UBYTE *) &(IFF_Chunk->iff_Length[0]), ChunkLen );

    if ( (CWrite ( CFHandle, IFF_Chunk, (ChunkLen + sizeof (struct IFFChunk)))) != (ChunkLen + sizeof (struct IFFChunk)))
      return ( ErrorCode );

    if ( ChunkLen & 1 )
    {
      *(ULONG *) &(IFF_Chunk->iff_Identifier[0]) = 0L;

      if ( (CWrite ( CFHandle, &IFF_Chunk, 1 )) != 1 )
        return ( ErrorCode );

      ChunkLen++;
    }

    *ChunkLenBuf += (ChunkLen + sizeof (struct IFFChunk));
  }

  return ( 0L );
}

static ULONG SaveTCSName ( struct CFHandle *CFHandle, STRPTR Name, unsigned long ChunkName, ULONG *ChunkLenBuf, unsigned long ErrorCode )
{
  struct IFFChunk IFF_Chunk;

  if ( Name )
  {
    ULONG Length = strlen ( Name );

    WriteMLong ( (UBYTE *) &(IFF_Chunk.iff_Identifier[0]), ChunkName );
    WriteMLong ( (UBYTE *) &(IFF_Chunk.iff_Length[0]), Length );

    Length += (Length & 1);

    if ( Length )
    {
      if ( (CWrite ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk))) != sizeof (struct IFFChunk))
        return ( ErrorCode );

      if ( (CWrite ( CFHandle, Name, Length )) != Length )
        return ( ErrorCode );

      *ChunkLenBuf += (Length + sizeof (struct IFFChunk));
    }
  }

  return ( 0L );
}

static ULONG SaveTCSSong ( struct CFHandle *CFHandle, struct TuComposerSong *SongStr, struct TuComposerInstr **InstrList, ULONG *ChunkLenBuf )
{
  struct IFFHeader IFF_Header;
  struct IFF_SHDR SHDR;
  struct TuComposerTrack *TrackStr;
  struct UnknownChunk *UnkChunk;
  ULONG SongSize = sizeof (ULONG), FORMSize;
  ULONG SongLenPos, FORMLenPos;
  ULONG FaultCode;

  if ( (SongLenPos = CSeek ( CFHandle, 0L, OFFSET_CURRENT )) == 0xFFFFFFFF )
    return ( ERROR_SONG_IOERR );

  SongLenPos += sizeof (ULONG);

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0]), 'FORM' );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), SongSize );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_TypeId[0]), 'SONG' );

  if ( (CWrite ( CFHandle, &IFF_Header, sizeof (struct IFFHeader )) != sizeof (struct IFFHeader )))
    return ( ERROR_SONG_IOERR );

  if ( (FaultCode = SaveTCSName ( CFHandle, SongStr->tcso_FileName, 'FILE', (ULONG *) &SongSize, ERROR_SONG_IOERR )))
    return ( FaultCode );

  if ( (FaultCode = SaveTCSName ( CFHandle, SongStr->tcso_Name, 'NAME', (ULONG *) &SongSize, ERROR_SONG_IOERR )))
    return ( FaultCode );

  if ( (FaultCode = SaveTCSName ( CFHandle, SongStr->tcso_Message, 'SMSG', (ULONG *) &SongSize, ERROR_SONG_IOERR )))
    return ( FaultCode );

  if ( (FaultCode = SaveTCSName ( CFHandle, SongStr->tcso_Composer, 'AUTH', (ULONG *) &SongSize, ERROR_SONG_IOERR )))
    return ( FaultCode );

  if ( (FaultCode = SaveTCSName ( CFHandle, SongStr->tcso_Style, 'STIL', (ULONG *) &SongSize, ERROR_SONG_IOERR )))
    return ( FaultCode );

  if ( (FaultCode = SaveTCSName ( CFHandle, SongStr->tcso_Comment, 'CMNT', (ULONG *) &SongSize, ERROR_SONG_IOERR )))
    return ( FaultCode );

  memcpy ( &(SHDR.shdr_BeginDate), &(SongStr->tcso_BeginDate), (offsetof (struct IFF_SHDR, shdr_Tracks) - offsetof (struct IFF_SHDR, shdr_BeginDate )));

  if (!( *(ULONG *) &(SongStr->tcso_EndDate) ))
    Stamp2Date ( &(SHDR.shdr_EndDate), NULL );

#ifndef MOTOROLA
  WriteMWord ( (UBYTE *) &(SHDR.shdr_BeginDate.sds_Year), SHDR.shdr_BeginDate.sds_Year );
  WriteMWord ( (UBYTE *) &(SHDR.shdr_EndDate.sds_Year), SHDR.shdr_EndDate.sds_Year );
#endif

  WriteMWord ( (UBYTE *) &(SHDR.shdr_Tracks), GetNumTracks ( SongStr ));

  memcpy ( &(SHDR.shdr_GoSubStackSize), &(SongStr->tcso_GoSubStackSize), (sizeof (struct IFF_SHDR) - offsetof (struct IFF_SHDR, shdr_GoSubStackSize )));

#ifndef MOTOROLA
  WriteMWord ( (UBYTE *) &(SHDR.shdr_GoSubStackSize), SHDR.shdr_GoSubStackSize );
  WriteMWord ( (UBYTE *) &(SHDR.shdr_LoopStackSize), SHDR.shdr_LoopStackSize );
  WriteMWord ( (UBYTE *) &(SHDR.shdr_Channels), SHDR.shdr_Channels );
  WriteMWord ( (UBYTE *) &(SHDR.shdr_Frames), SHDR.shdr_Frames );
  WriteMWord ( (UBYTE *) &(SHDR.shdr_SPDSpeed), SHDR.shdr_SPDSpeed );
  WriteMWord ( (UBYTE *) &(SHDR.shdr_BPMTempo), SHDR.shdr_BPMTempo );
  WriteMWord ( (UBYTE *) &(SHDR.shdr_BPMSpeed), SHDR.shdr_BPMSpeed );
  WriteMWord ( (UBYTE *) &(SHDR.shdr_FramesLowerLimit), SHDR.shdr_FramesLowerLimit );
  WriteMWord ( (UBYTE *) &(SHDR.shdr_FramesUpperLimit), SHDR.shdr_FramesUpperLimit );
  WriteMWord ( (UBYTE *) &(SHDR.shdr_SPDLowerLimit), SHDR.shdr_SPDLowerLimit );
  WriteMWord ( (UBYTE *) &(SHDR.shdr_SPDUpperLimit), SHDR.shdr_SPDUpperLimit );
  WriteMWord ( (UBYTE *) &(SHDR.shdr_BPMTempoLowerLimit), SHDR.shdr_BPMTempoLowerLimit );
  WriteMWord ( (UBYTE *) &(SHDR.shdr_BPMTempoUpperLimit), SHDR.shdr_BPMTempoUpperLimit );
  WriteMWord ( (UBYTE *) &(SHDR.shdr_BPMSpeedLowerLimit), SHDR.shdr_BPMSpeedLowerLimit );
  WriteMWord ( (UBYTE *) &(SHDR.shdr_BPMSpeedUpperLimit), SHDR.shdr_BPMSpeedUpperLimit );
#endif

  if ( (FaultCode = SaveTCSChunk ( CFHandle, (struct IFFChunk *) &SHDR, 'SHDR', sizeof (struct IFF_SHDR), (ULONG *) &SongSize, ERROR_SONG_IOERR )))
    return ( FaultCode );

  TrackStr = (struct TuComposerTrack *) &(SongStr->tcso_TrackList.mlh_Head);

  if ( TrackStr->tct_Node.mln_Succ->mln_Succ )
  {
    SongSize += (sizeof (struct IFFHeader) - sizeof (ULONG));
    FORMSize = sizeof (ULONG);

    if ( (FORMLenPos = CSeek ( CFHandle, 0L, OFFSET_CURRENT )) == 0xFFFFFFFF )
      return ( ERROR_SONG_IOERR );

    FORMLenPos += sizeof (ULONG);

    WriteMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0]), 'FORM' );
    WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), FORMSize );
    WriteMLong ( (UBYTE *) &(IFF_Header.ifh_TypeId[0]), 'PATT' );

    if ( (CWrite ( CFHandle, &IFF_Header, sizeof (struct IFFHeader )) != sizeof (struct IFFHeader )))
      return ( ERROR_SONG_IOERR );

    for (;;)
    {
      TrackStr = (struct TuComposerTrack *) TrackStr->tct_Node.mln_Succ;

      if (!( TrackStr->tct_Node.mln_Succ ))
        break;

      if ( (FaultCode = SaveSongTrak ( CFHandle, TrackStr, SongStr, InstrList, (ULONG *) &FORMSize )))
        return ( FaultCode );
    }

    if ( (FORMLenPos = CSeek ( CFHandle, FORMLenPos, OFFSET_BEGINNING )) == 0xFFFFFFFF )
      return ( ERROR_SONG_IOERR );

    WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), FORMSize );

    if ( CWrite ( CFHandle, &(IFF_Header.ifh_Length[0]), sizeof (ULONG)) != sizeof (ULONG))
      return ( ERROR_SONG_IOERR );

    if ( CSeek ( CFHandle, FORMLenPos, OFFSET_BEGINNING ) == 0xFFFFFFFF )
      return ( ERROR_SONG_IOERR );

    SongSize += FORMSize;
  }

  SongSize += (sizeof (struct IFFHeader) - sizeof (ULONG));
  FORMSize = sizeof (ULONG);

  if ( (FORMLenPos = CSeek ( CFHandle, 0L, OFFSET_CURRENT )) == 0xFFFFFFFF )
    return ( ERROR_SONG_IOERR );

  FORMLenPos += sizeof (ULONG);

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0]), 'FORM' );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), FORMSize );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_TypeId[0]), 'POSI' );

  if ( (CWrite ( CFHandle, &IFF_Header, sizeof (struct IFFHeader )) != sizeof (struct IFFHeader )))
    return ( ERROR_SONG_IOERR );

  if ( (FaultCode = SaveSongPost ( CFHandle, SongStr, (ULONG *) &FORMSize )))
    return ( FaultCode );

  if ( (FORMLenPos = CSeek ( CFHandle, FORMLenPos, OFFSET_BEGINNING )) == 0xFFFFFFFF )
    return ( ERROR_SONG_IOERR );

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), FORMSize );

  if ( CWrite ( CFHandle, &(IFF_Header.ifh_Length[0]), sizeof (ULONG)) != sizeof (ULONG))
    return ( ERROR_SONG_IOERR );

  if ( CSeek ( CFHandle, FORMLenPos, OFFSET_BEGINNING ) == 0xFFFFFFFF )
    return ( ERROR_SONG_IOERR );

  SongSize += FORMSize;

  UnkChunk = (struct UnknownChunk *) &(SongStr->tcso_UnknownChunks.mlh_Head);

  for (;;)
  {
    UnkChunk = (struct UnknownChunk *) UnkChunk->uc_Node.mln_Succ;

    if (!( UnkChunk->uc_Node.mln_Succ ))
      break;

    if ( (FaultCode = SaveTCSChunk ( CFHandle, (struct IFFChunk *) &UnkChunk, ReadMLong ( (UBYTE *) &(UnkChunk->uc_Identifier[0])), ReadMLong ( (UBYTE *) &(UnkChunk->uc_Length[0])), &SongSize, ERROR_SONG_IOERR )))
      return ( FaultCode );
  }

  if ( (SongLenPos = CSeek ( CFHandle, SongLenPos, OFFSET_BEGINNING )) == 0xFFFFFFFF )
    return ( ERROR_SONG_IOERR );

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), SongSize );

  if ( CWrite ( CFHandle, &(IFF_Header.ifh_Length[0]), sizeof (ULONG)) != sizeof (ULONG))
    return ( ERROR_SONG_IOERR );

  if ( CSeek ( CFHandle, SongLenPos, OFFSET_BEGINNING ) == 0xFFFFFFFF )
    return ( ERROR_SONG_IOERR );

  *ChunkLenBuf += SongSize + (sizeof (struct IFFHeader) - sizeof (ULONG));

  return ( 0L );
}

static ULONG SaveSongTrak ( struct CFHandle *CFHandle, struct TuComposerTrack *TrackStr, struct TuComposerSong *SongStr, struct TuComposerInstr **InstrList, ULONG *ChunkLenBuf )
{
  struct IFFHeader IFF_Header;
  struct IFFChunk IFF_Chunk;
  struct MIFF_THDR THDR;
  struct TuComposerPacked *PackedStr;
  struct UnknownChunk *UnkChunk;
  ULONG TrakSize = sizeof (ULONG), FORMSize;
  ULONG TrakLenPos;
  ULONG FaultCode;

  if ( (TrakLenPos = CSeek ( CFHandle, 0L, OFFSET_CURRENT )) == 0xFFFFFFFF )
    return ( ERROR_TRACK_IOERR );

  TrakLenPos += sizeof (ULONG);

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0]), 'FORM' );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), TrakSize );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_TypeId[0]), 'TRAK' );

  if ( (CWrite ( CFHandle, &IFF_Header, sizeof (struct IFFHeader )) != sizeof (struct IFFHeader )))
    return ( ERROR_TRACK_IOERR );

  if ( (FaultCode = SaveTCSName ( CFHandle, TrackStr->tct_FileName, 'FILE', (ULONG *) &TrakSize, ERROR_TRACK_IOERR )))
    return ( FaultCode );

  if ( (FaultCode = SaveTCSName ( CFHandle, TrackStr->tct_Name, 'NAME', (ULONG *) &TrakSize, ERROR_TRACK_IOERR )))
    return ( FaultCode );

  memcpy ( &(THDR.mthdr_Length), &(TrackStr->tct_Length), (sizeof (struct MIFF_THDR) - sizeof (struct IFFChunk)));

#ifndef MOTOROLA
  WriteMWord ( (UBYTE *) &(THDR.mthdr_Length), THDR.mthdr_Length );
  WriteMWord ( (UBYTE *) &(THDR.mthdr_Flags), THDR.mthdr_Flags );
  WriteMWord ( (UBYTE *) &(THDR.mthdr_Frames), THDR.mthdr_Frames );
  WriteMWord ( (UBYTE *) &(THDR.mthdr_SPDSpeed), THDR.mthdr_SPDSpeed );
  WriteMWord ( (UBYTE *) &(THDR.mthdr_BPMTempo), THDR.mthdr_BPMTempo );
  WriteMWord ( (UBYTE *) &(THDR.mthdr_BPMSpeed), THDR.mthdr_BPMSpeed );
#endif

  if ( (FaultCode = SaveTCSChunk ( CFHandle, (struct IFFChunk *) &THDR, 'THDR', sizeof (struct MIFF_THDR), (ULONG *) &TrakSize, ERROR_TRACK_IOERR )))
    return ( FaultCode );

  if (!( TrackStr = ReMapSongInstrs ( SongStr, TrackStr, InstrList )))
    return ( ERROR_TRACK_NOMEM );

  PackedStr = PackTrack ( TrackStr );

  FreeTrack ( TrackStr );

  if (!( PackedStr ))
    return ( ERROR_TRACK_NOMEM );

  if ( (FORMSize = (PackedStr->tcp_PackLength - 1)))
  {
    WriteMLong ( (UBYTE *) &(IFF_Chunk.iff_Identifier[0]), 'BODY' );
    WriteMLong ( (UBYTE *) &(IFF_Chunk.iff_Length[0]), FORMSize );

    if ( (CWrite ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk )) != sizeof (struct IFFChunk )))
    {
      FreePackedTrack ( PackedStr );

      return ( ERROR_TRACK_IOERR );
    }

    FORMSize += (FORMSize & 1);
    TrakSize += sizeof (struct IFFChunk) + FORMSize;

    if ( (CWrite ( CFHandle, PackedStr->tcp_Data, FORMSize )) != FORMSize )
    {
      FreePackedTrack ( PackedStr );

      return ( ERROR_TRACK_IOERR );
    }
  }

  FreePackedTrack ( PackedStr );

  UnkChunk = (struct UnknownChunk *) &(PackedStr->tcp_UnknownChunks.mlh_Head);

  for (;;)
  {
    UnkChunk = (struct UnknownChunk *) UnkChunk->uc_Node.mln_Succ;

    if (!( UnkChunk->uc_Node.mln_Succ ))
      break;

    if ( (FaultCode = SaveTCSChunk ( CFHandle, (struct IFFChunk *) &UnkChunk, ReadMLong ( (UBYTE *) &(UnkChunk->uc_Identifier[0])), ReadMLong ( (UBYTE *) &(UnkChunk->uc_Length[0])), &TrakSize, ERROR_TRACK_IOERR )))
      return ( FaultCode );
  }

  if ( (TrakLenPos = CSeek ( CFHandle, TrakLenPos, OFFSET_BEGINNING )) == 0xFFFFFFFF )
    return ( ERROR_TRACK_IOERR );

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), TrakSize );

  if ( CWrite ( CFHandle, &(IFF_Header.ifh_Length[0]), sizeof (ULONG)) != sizeof (ULONG))
    return ( ERROR_TRACK_IOERR );

  if ( CSeek ( CFHandle, TrakLenPos, OFFSET_BEGINNING ) == 0xFFFFFFFF )
    return ( ERROR_TRACK_IOERR );

  *ChunkLenBuf += TrakSize + (sizeof (struct IFFHeader) - sizeof (ULONG));

  return ( 0L );
}

static ULONG SaveSongPost ( struct CFHandle *CFHandle, struct TuComposerSong *SongStr, ULONG *ChunkLenBuf )
{
  struct IFFHeader IFF_Header;
  struct IFF_PHDR PHDR;
  struct TuComposerOrders *PosStr = SongStr->tcso_Position;
  ULONG i;

  for (i = SongStr->tcso_Channels; i > 0; i--)
  {
    struct TuComposerOrderList *PosListStr;
    ULONG PostSize = sizeof (ULONG), FORMSize;
    ULONG PostLenPos, FORMLenPos;
    ULONG FaultCode;

    if ( (PostLenPos = CSeek ( CFHandle, 0L, OFFSET_CURRENT )) == 0xFFFFFFFF )
      return ( ERROR_SONG_IOERR );

    PostLenPos += sizeof (ULONG);

    WriteMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0]), 'FORM' );
    WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), PostSize );
    WriteMLong ( (UBYTE *) &(IFF_Header.ifh_TypeId[0]), 'POST' );

    if ( (CWrite ( CFHandle, &IFF_Header, sizeof (struct IFFHeader )) != sizeof (struct IFFHeader )))
      return ( ERROR_SONG_IOERR );

    memcpy ( &(PHDR.phdr_Length), &(PosStr->tco_Length), (sizeof (struct IFF_PHDR) - sizeof (struct IFFChunk )));

#ifndef MOTOROLA
    WriteMWord ( (UBYTE *) &(PHDR.phdr_Length), PHDR.phdr_Length );
    WriteMWord ( (UBYTE *) &(PHDR.phdr_RepStart), PHDR.phdr_RepStart );
#endif

    if ( (FaultCode = SaveTCSChunk ( CFHandle, (struct IFFChunk *) &PHDR, 'PHDR', sizeof (struct IFF_PHDR), (ULONG *) &PostSize, ERROR_SONG_IOERR )))
      return ( FaultCode );

    PosListStr = (struct TuComposerOrderList *) &(PosStr->tco_OrderData.mlh_Head);

    if ( PosListStr->tcol_Node.mln_Succ->mln_Succ )
    {
      PostSize += (sizeof (struct IFFHeader) - sizeof (ULONG));
      FORMSize = sizeof (ULONG);

      if ( (FORMLenPos = CSeek ( CFHandle, 0L, OFFSET_CURRENT )) == 0xFFFFFFFF )
        return ( ERROR_SONG_IOERR );

      FORMLenPos += sizeof (ULONG);

      WriteMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0]), 'FORM' );
      WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), FORMSize );
      WriteMLong ( (UBYTE *) &(IFF_Header.ifh_TypeId[0]), 'POSL' );

      if ( (CWrite ( CFHandle, &IFF_Header, sizeof (struct IFFHeader )) != sizeof (struct IFFHeader )))
        return ( ERROR_SONG_IOERR );

      for (;;)
      {
        struct IFF_PDAT PDAT;

        PosListStr = (struct TuComposerOrderList *) PosListStr->tcol_Node.mln_Succ;

        if (!( PosListStr->tcol_Node.mln_Succ ))
          break;

        WriteMWord ( (UBYTE *) &(PDAT.pdat_TrackNum), GetTrackNum ( SongStr, PosListStr->tcol_Track ));
        WriteMWord ( (UBYTE *) &(PDAT.pdat_NextPos), GetPositionNum ( PosStr, PosListStr->tcol_NextPos, 0L ));
        WriteMWord ( (UBYTE *) &(PDAT.pdat_PrevPos), GetPositionNum ( PosStr, PosListStr->tcol_PrevPos, 0L ));

        memcpy ( &(PDAT.pdat_NextRow), &(PosListStr->tcol_NextRow), (sizeof (struct IFF_PDAT) - offsetof (struct IFF_PDAT, pdat_NextRow )));

#ifndef MOTOROLA
        WriteMWord ( (UBYTE *) &(PDAT.pdat_NextRow), PDAT.pdat_NextRow );
        WriteMWord ( (UBYTE *) &(PDAT.pdat_PrevRow), PDAT.pdat_PrevRow );
        WriteMWord ( (UBYTE *) &(PDAT.pdat_FirstRow), PDAT.pdat_FirstRow );
        WriteMWord ( (UBYTE *) &(PDAT.pdat_LastRow), PDAT.pdat_LastRow );
        WriteMWord ( (UBYTE *) &(PDAT.pdat_InstrTranspose), PDAT.pdat_InstrTranspose );
        WriteMWord ( (UBYTE *) &(PDAT.pdat_Tempo), PDAT.pdat_Tempo );
#endif

        if ( (FaultCode = SaveTCSChunk ( CFHandle, (struct IFFChunk *) &PDAT, 'PDAT', sizeof (struct IFF_PDAT), &FORMSize, ERROR_SONG_IOERR )))
          return ( FaultCode );
      }

      if ( (FORMLenPos = CSeek ( CFHandle, FORMLenPos, OFFSET_BEGINNING )) == 0xFFFFFFFF )
        return ( ERROR_SONG_IOERR );

      WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), FORMSize );

      if ( CWrite ( CFHandle, &(IFF_Header.ifh_Length[0]), sizeof (ULONG)) != sizeof (ULONG))
        return ( ERROR_SONG_IOERR );

      if ( CSeek ( CFHandle, FORMLenPos, OFFSET_BEGINNING ) == 0xFFFFFFFF )
        return ( ERROR_SONG_IOERR );

      PostSize += FORMSize;
    }

    if ( (PostLenPos = CSeek ( CFHandle, PostLenPos, OFFSET_BEGINNING )) == 0xFFFFFFFF )
      return ( ERROR_SONG_IOERR );

    WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), PostSize );

    if ( CWrite ( CFHandle, &(IFF_Header.ifh_Length[0]), sizeof (ULONG)) != sizeof (ULONG))
      return ( ERROR_SONG_IOERR );

    if ( CSeek ( CFHandle, PostLenPos, OFFSET_BEGINNING ) == 0xFFFFFFFF )
      return ( ERROR_SONG_IOERR );

    *ChunkLenBuf += PostSize + (sizeof (struct IFFHeader) - sizeof (ULONG));

    PosStr++;
  }

  return ( 0L );
}

static ULONG SaveTCSInst ( struct CFHandle *CFHandle, struct TuComposerModule *ModuleStr, struct TuComposerInstr *InstrStr, ULONG *ChunkLenBuf, union FlagPass *FlagPassStr )
{
  struct IFFHeader IFF_Header;
  struct MIFF_IHDR IHDR;
  struct TuComposerSample *SampleStr;
  struct UnknownChunk *UnkChunk;
  ULONG InstSize = sizeof (ULONG), FORMSize;
  ULONG InstLenPos, FORMLenPos;
  ULONG FaultCode;

  if ( (InstLenPos = CSeek ( CFHandle, 0L, OFFSET_CURRENT )) == 0xFFFFFFFF )
    return ( ERROR_INSTR_IOERR );

  InstLenPos += sizeof (ULONG);

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0]), 'FORM' );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), InstSize );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_TypeId[0]), 'INST' );

  if ( (CWrite ( CFHandle, &IFF_Header, sizeof (struct IFFHeader )) != sizeof (struct IFFHeader )))
    return ( ERROR_INSTR_IOERR );

  if ( (FaultCode = SaveTCSName ( CFHandle, InstrStr->tci_FileName, 'FILE', (ULONG *) &InstSize, ERROR_INSTR_IOERR )))
    return ( FaultCode );

  if ( (FaultCode = SaveTCSName ( CFHandle, InstrStr->tci_Name, 'NAME', (ULONG *) &InstSize, ERROR_INSTR_IOERR )))
    return ( FaultCode );

  WriteMWord ( (UBYTE *) &(IHDR.mihdr_VolumeEnvNum), GetEnvelopeNum ( ModuleStr, InstrStr->tci_VolumeEnvelope ));
  WriteMWord ( (UBYTE *) &(IHDR.mihdr_PanningEnvNum), GetEnvelopeNum ( ModuleStr, InstrStr->tci_PanningEnvelope ));
  WriteMWord ( (UBYTE *) &(IHDR.mihdr_SlideEnvNum), GetEnvelopeNum ( ModuleStr, InstrStr->tci_SlideEnvelope ));
  WriteMWord ( (UBYTE *) &(IHDR.mihdr_VibratoEnvNum), GetEnvelopeNum ( ModuleStr, InstrStr->tci_VibratoEnvelope ));
  WriteMWord ( (UBYTE *) &(IHDR.mihdr_TremoloEnvNum), GetEnvelopeNum ( ModuleStr, InstrStr->tci_TremoloEnvelope ));
  WriteMWord ( (UBYTE *) &(IHDR.mihdr_PannoloEnvNum), GetEnvelopeNum ( ModuleStr, InstrStr->tci_PannoloEnvelope ));
  WriteMWord ( (UBYTE *) &(IHDR.mihdr_ChannoloEnvNum), GetEnvelopeNum ( ModuleStr, InstrStr->tci_ChannoloEnvelope ));
  WriteMWord ( (UBYTE *) &(IHDR.mihdr_SpenoloEnvNum), GetEnvelopeNum ( ModuleStr, InstrStr->tci_SpenoloEnvelope ));
  WriteMWord ( (UBYTE *) &(IHDR.mihdr_ArpeggioDefNum), GetArpeggioNum ( ModuleStr, InstrStr->tci_ArpeggioStructure ));
  WriteMWord ( (UBYTE *) &(IHDR.mihdr_KeyboardDefNum), GetKeyboardNum ( ModuleStr, InstrStr->tci_KeyboardDefs ));

  memcpy ( &(IHDR.mihdr_NumSamples), &(InstrStr->tci_NumSamples), (sizeof (struct MIFF_IHDR) - offsetof (struct MIFF_IHDR, mihdr_NumSamples )));

#ifndef MOTOROLA
  WriteMWord ( (UBYTE *) &(IHDR.mihdr_VolumeSwing), IHDR.mihdr_VolumeSwing );
  WriteMWord ( (UBYTE *) &(IHDR.mihdr_PanningSwing), IHDR.mihdr_PanningSwing );
  WriteMLong ( (UBYTE *) &(IHDR.mihdr_PitchSwing), IHDR.mihdr_PitchSwing );
  WriteMWord ( (UBYTE *) &(IHDR.mihdr_PitchPanSep), IHDR.mihdr_PitchPanSep );
  WriteMWord ( (UBYTE *) &(IHDR.mihdr_EnvUsageFlags), IHDR.mihdr_EnvUsageFlags );
  WriteMWord ( (UBYTE *) &(IHDR.mihdr_EnvProcFlags), IHDR.mihdr_EnvProcFlags );
  WriteMWord ( (UBYTE *) &(IHDR.mihdr_EnvRetrigFlags), IHDR.mihdr_EnvRetrigFlags );
  WriteMWord ( (UBYTE *) &(IHDR.mihdr_EnvRandomFlags), IHDR.mihdr_EnvRandomFlags );
  WriteMWord ( (UBYTE *) &(IHDR.mihdr_EnvRndDelayFlags), IHDR.mihdr_EnvRndDelayFlags );
  WriteMWord ( (UBYTE *) &(IHDR.mihdr_FadeOut), IHDR.mihdr_FadeOut );
  WriteMWord ( (UBYTE *) &(IHDR.mihdr_Hold), IHDR.mihdr_Hold );
  WriteMWord ( (UBYTE *) &(IHDR.mihdr_Decay), IHDR.mihdr_Decay );
#endif

  if ( (FaultCode = SaveTCSChunk ( CFHandle, (struct IFFChunk *) &IHDR, 'IHDR', sizeof (struct MIFF_IHDR), (ULONG *) &InstSize, ERROR_INSTR_IOERR )))
    return ( FaultCode );

  SampleStr = (struct TuComposerSample *) &(InstrStr->tci_Samples.mlh_Head);

  if ( SampleStr->tcs_Node.mln_Succ->mln_Succ )
  {
    InstSize += (sizeof (struct IFFHeader) - sizeof (ULONG));
    FORMSize = sizeof (ULONG);

    if ( (FORMLenPos = CSeek ( CFHandle, 0L, OFFSET_CURRENT )) == 0xFFFFFFFF )
      return ( ERROR_INSTR_IOERR );

    FORMLenPos += sizeof (ULONG);

    WriteMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0]), 'FORM' );
    WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), FORMSize );
    WriteMLong ( (UBYTE *) &(IFF_Header.ifh_TypeId[0]), 'SAMP' );

    if ( (CWrite ( CFHandle, &IFF_Header, sizeof (struct IFFHeader )) != sizeof (struct IFFHeader )))
      return ( ERROR_INSTR_IOERR );

    for (;;)
    {
      SampleStr = (struct TuComposerSample *) SampleStr->tcs_Node.mln_Succ;

      if (!( SampleStr->tcs_Node.mln_Succ ))
        break;

      if ( (FaultCode = SaveInstSmpl ( CFHandle, ModuleStr, SampleStr, (ULONG *) &FORMSize, FlagPassStr )))
        return ( FaultCode );
    }

    if ( (FORMLenPos = CSeek ( CFHandle, FORMLenPos, OFFSET_BEGINNING )) == 0xFFFFFFFF )
      return ( ERROR_INSTR_IOERR );

    WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), FORMSize );

    if ( CWrite ( CFHandle, &(IFF_Header.ifh_Length[0]), sizeof (ULONG)) != sizeof (ULONG))
      return ( ERROR_INSTR_IOERR );

    if ( CSeek ( CFHandle, FORMLenPos, OFFSET_BEGINNING ) == 0xFFFFFFFF )
      return ( ERROR_INSTR_IOERR );

    InstSize += FORMSize;
  }

  UnkChunk = (struct UnknownChunk *) &(InstrStr->tci_UnknownChunks.mlh_Head);

  for (;;)
  {
    UnkChunk = (struct UnknownChunk *) UnkChunk->uc_Node.mln_Succ;

    if (!( UnkChunk->uc_Node.mln_Succ ))
      break;

    if ( (FaultCode = SaveTCSChunk ( CFHandle, (struct IFFChunk *) &UnkChunk, ReadMLong ( (UBYTE *) &(UnkChunk->uc_Identifier[0])), ReadMLong ( (UBYTE *) &(UnkChunk->uc_Length[0])), &InstSize, ERROR_INSTR_IOERR )))
      return ( FaultCode );
  }

  if ( (InstLenPos = CSeek ( CFHandle, InstLenPos, OFFSET_BEGINNING )) == 0xFFFFFFFF )
    return ( ERROR_INSTR_IOERR );

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), InstSize );

  if ( CWrite ( CFHandle, &(IFF_Header.ifh_Length[0]), sizeof (ULONG)) != sizeof (ULONG))
    return ( ERROR_INSTR_IOERR );

  if ( CSeek ( CFHandle, InstLenPos, OFFSET_BEGINNING ) == 0xFFFFFFFF )
    return ( ERROR_INSTR_IOERR );

  *ChunkLenBuf += InstSize + (sizeof (struct IFFHeader) - sizeof (ULONG));

  return ( 0L );
}

static ULONG SaveInstSmpl ( struct CFHandle *CFHandle, struct TuComposerModule *ModuleStr, struct TuComposerSample *SampleStr, ULONG *ChunkLenBuf, union FlagPass *FlagPassStr )
{
  struct IFFHeader IFF_Header;
  struct IFFChunk IFF_Chunk;
  struct IFF_SMPH SMPH;
  struct IFF_SMPR SMPR;
  struct UnknownChunk *UnkChunk;
  ULONG SmplSize = sizeof (ULONG), FORMSize;
  ULONG SmplLenPos;
  ULONG FaultCode;

  if ( (SmplLenPos = CSeek ( CFHandle, 0L, OFFSET_CURRENT )) == 0xFFFFFFFF )
    return ( ERROR_SAMPLE_IOERR );

  SmplLenPos += sizeof (ULONG);

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0]), 'FORM' );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), SmplSize );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_TypeId[0]), 'SMPL' );

  if ( (CWrite ( CFHandle, &IFF_Header, sizeof (struct IFFHeader )) != sizeof (struct IFFHeader )))
    return ( ERROR_SAMPLE_IOERR );

  if ( (FaultCode = SaveTCSName ( CFHandle, SampleStr->tcs_FileName, 'FILE', (ULONG *) &SmplSize, ERROR_SAMPLE_IOERR )))
    return ( FaultCode );

  if ( (FaultCode = SaveTCSName ( CFHandle, SampleStr->tcs_Name, 'NAME', (ULONG *) &SmplSize, ERROR_SAMPLE_IOERR )))
    return ( FaultCode );

  memcpy ( &(SMPH.smph_Samples), &(SampleStr->tcs_Samples), (offsetof (struct IFF_SMPH, smph_AutoVibratoNum) - sizeof (struct IFFChunk)));

#ifndef MOTOROLA
  WriteMLong ( (UBYTE *) &(SMPH.smph_Samples), SMPH.smph_Samples );
  WriteMLong ( (UBYTE *) &(SMPH.smph_Repeat), SMPH.smph_Repeat );
  WriteMLong ( (UBYTE *) &(SMPH.smph_RepLen), SMPH.smph_RepLen );
  WriteMLong ( (UBYTE *) &(SMPH.smph_RepCount), SMPH.smph_RepCount );
  WriteMLong ( (UBYTE *) &(SMPH.smph_SustainRepeat), SMPH.smph_SustainRepeat );
  WriteMLong ( (UBYTE *) &(SMPH.smph_SustainRepLen), SMPH.smph_SustainRepLen );
  WriteMLong ( (UBYTE *) &(SMPH.smph_SustainRepCount), SMPH.smph_SustainRepCount );
  WriteMLong ( (UBYTE *) &(SMPH.smph_Rate), SMPH.smph_Rate );
  WriteMLong ( (UBYTE *) &(SMPH.smph_StartOffset), SMPH.smph_StartOffset );
  WriteMLong ( (UBYTE *) &(SMPH.smph_LowerRateLimit), SMPH.smph_LowerRateLimit );
  WriteMLong ( (UBYTE *) &(SMPH.smph_UpperRateLimit), SMPH.smph_UpperRateLimit );
#endif

  WriteMWord ( (UBYTE *) &(SMPH.smph_AutoVibratoNum), GetEnvelopeNum ( ModuleStr, SampleStr->tcs_AutoVibratoEnv ));
  WriteMWord ( (UBYTE *) &(SMPH.smph_AutoTremoloNum), GetEnvelopeNum ( ModuleStr, SampleStr->tcs_AutoTremoloEnv ));
  WriteMWord ( (UBYTE *) &(SMPH.smph_AutoPannoloNum), GetEnvelopeNum ( ModuleStr, SampleStr->tcs_AutoPannoloEnv ));

  memcpy ( &(SMPH.smph_EnvUsageFlags), &(SampleStr->tcs_EnvUsageFlags), (sizeof (struct IFF_SMPH) - offsetof (struct IFF_SMPH, smph_EnvUsageFlags )));

#ifndef MOTOROLA
  WriteMWord ( (UBYTE *) &(SMPH.smph_VibratoSweep), SMPH.smph_VibratoSweep );
  WriteMWord ( (UBYTE *) &(SMPH.smph_TremoloSweep), SMPH.smph_TremoloSweep );
  WriteMWord ( (UBYTE *) &(SMPH.smph_PannoloSweep), SMPH.smph_PannoloSweep );
#endif

  SMPH.smph_Flags &= ~TCSF_Redirect;

  if ( SampleStr->tcs_Flags & TCSF_Redirect )
  {
    struct TuComposerInstr *InstrStr = (struct TuComposerInstr *) &(ModuleStr->tcm_InstrList.mlh_Head);
    struct TuComposerSample *OrigSampleStr;
    ULONG NumSample = 0L;

    if ( (FaultCode = SaveTCSChunk ( CFHandle, (struct IFFChunk *) &SMPH, 'SMPH', sizeof (struct IFF_SMPH), (ULONG *) &SmplSize, ERROR_SAMPLE_IOERR )))
      return ( FaultCode );

    for (;;)
    {
      InstrStr = (struct TuComposerInstr *) InstrStr->tci_Node.mln_Succ;

      if (!( InstrStr->tci_Node.mln_Succ ))
        break;

      OrigSampleStr = (struct TuComposerSample *) &(InstrStr->tci_Samples.mlh_Head);

      for (;;)
      {
        OrigSampleStr = (struct TuComposerSample *) OrigSampleStr->tcs_Node.mln_Succ;

        if (!( OrigSampleStr->tcs_Node.mln_Succ ))
          break;

        NumSample++;

        if ( OrigSampleStr->tcs_Flags & TCSF_Redirect )
          continue;

        if ( OrigSampleStr->tcs_Data == SampleStr->tcs_Data )
          goto FoundOriginSample;
      }
    }

    NumSample = 0L;

FoundOriginSample:

    WriteMLong ( (UBYTE *) &(SMPR.smpr_SampleNum), NumSample );

    if ( (FaultCode = SaveTCSChunk ( CFHandle, (struct IFFChunk *) &SMPR, 'SMPR', sizeof (struct IFF_SMPR), (ULONG *) &SmplSize, ERROR_SAMPLE_IOERR )))
      return ( FaultCode );
  } else
  {
    if ( SampleStr->tcs_Samples && FlagPassStr[0].flg_Value )
    {
      switch ( FlagPassStr[1].flg_Value )
      {
        case 1 :
        {
          SMPH.smph_Flags |= SMPLF_Delta;

          break;
        }
        case 3 :
        {
          if (!( SampleStr->tcs_BitDepth & 4 ))
            SMPH.smph_Flags |= SMPLF_Delta;

          break;
        }
      }

      if ( (FaultCode = SaveTCSChunk ( CFHandle, (struct IFFChunk *) &SMPH, 'SMPH', sizeof (struct IFF_SMPH), (ULONG *) &SmplSize, ERROR_SAMPLE_IOERR )))
        return ( FaultCode );

      FORMSize = SampleStr->tcs_Length;

      WriteMLong ( (UBYTE *) &(IFF_Chunk.iff_Identifier[0]), 'BODY' );
      WriteMLong ( (UBYTE *) &(IFF_Chunk.iff_Length[0]), FORMSize );

      if ( (CWrite ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk )) != sizeof (struct IFFChunk )))
        return ( ERROR_SAMPLE_IOERR );

      FORMSize += (FORMSize & 1);
      SmplSize += sizeof (struct IFFChunk) + FORMSize;

      if ( SMPH.smph_Flags & SMPLF_Delta )
      {
        WORD *SampleData = SampleStr->tcs_Data;
        WORD *CopyData = AllocSampleData ( SAVESAMPLE_BUFFER * sizeof (LONG));
        ULONG WriteLeft, WriteChunk = (((SAVESAMPLE_BUFFER * SampleStr->tcs_BitDepth) + 7UL) >> 3UL), WriteSmp = SAVESAMPLE_BUFFER;
        ULONG SmpLen = SampleStr->tcs_Length, SmpSize = SampleStr->tcs_Samples;
        LONG  Delta = 0L;

        do
        {
          WriteLeft = WriteChunk;

          if ( SmpLen < WriteChunk )
          {
            WriteLeft = (((SmpSize * SampleStr->tcs_BitDepth) + 7UL) >> 3UL);
            WriteSmp = SmpSize;
          }

          switch ( SampleStr->tcs_BitDepth )
          {
            case 8 :
            {
              DeltaCrunch8 ( (BYTE **) &SampleData, (BYTE *) CopyData, (BYTE *) &Delta, WriteSmp );

              break;
            }
            case 16 :
            {
              DeltaCrunch16 ( (WORD **) &SampleData, CopyData, (WORD *) &Delta, WriteSmp );

              break;
            }
            case 32 :
            {
              DeltaCrunch32 ( (LONG **) &SampleData, (LONG *) CopyData, (LONG *) &Delta, WriteSmp );

              break;
            }
            default :
            {
              DeltaCrunchX ( (LONG **) &SampleData, (LONG *) CopyData, (LONG *) &Delta, SampleStr->tcs_BitDepth, WriteSmp );

              break;
            }
          }

          if ( CWrite ( CFHandle, CopyData, WriteLeft ) != WriteLeft )
          {
            FreeSampleData ( CopyData, SAVESAMPLE_BUFFER * sizeof (LONG));

            return ( ERROR_SAMPLE_IOERR );
          }

          SmpSize -= WriteSmp;
        } while ( (SmpLen -= WriteLeft) );

        FreeSampleData ( CopyData, SAVESAMPLE_BUFFER * sizeof (LONG));

        if ( SampleStr->tcs_Length & 1 )
        {
          *(ULONG *) &(IFF_Chunk.iff_Identifier[0]) = 0L;

          if ( (CWrite ( CFHandle, &IFF_Chunk, 1 )) != 1 )
            return ( ERROR_SAMPLE_IOERR );
        }
      } else
      {
        BYTE *SampleData = (BYTE *) SampleStr->tcs_Data;
        ULONG SmpLen = SampleStr->tcs_Samples;

#ifndef MOTOROLA
        if ( SampleStr->tcs_BitDepth == 8 )
        {
          if ( SmpLen & 1 )
            SampleData[SmpLen] = 0;

          if ( (CWrite ( CFHandle, SampleData, FORMSize )) != FORMSize )
            return ( ERROR_SAMPLE_IOERR );
        } else
        {
          WORD *CopyData = AllocSampleData ( SAVESAMPLE_BUFFER * sizeof (ULONG));
          ULONG WriteLeft;

          do
          {
            ULONG i;

            WriteLeft = SAVESAMPLE_BUFFER;

            if ( SmpLen < SAVESAMPLE_BUFFER )
              WriteLeft = SmpLen;

            if ( SampleStr->tcs_BitDepth == 16 )
            {
              WORD *Buffer = CopyData;

              for (i = WriteLeft; i > 0; i--)
              {
                *Buffer++ = ReadMWord ( (UBYTE *) SampleData++ );
              }

              if ( CWrite ( CFHandle, CopyData, (WriteLeft * sizeof (WORD))) != (WriteLeft * sizeof (WORD)))
              {
                FreeSampleData ( CopyData, SAVESAMPLE_BUFFER * sizeof (ULONG));

                return ( ERROR_SAMPLE_IOERR );
              }
            } else
            {
              LONG *Buffer = (LONG *) CopyData;

              for (i = WriteLeft; i > 0; i--)
              {
                *Buffer++ = ReadMLong ( (UBYTE *) SampleData++ );
              }

              if ( CWrite ( CFHandle, CopyData, (WriteLeft * sizeof (LONG))) != (WriteLeft * sizeof (LONG)))
              {
                FreeSampleData ( CopyData, SAVESAMPLE_BUFFER * sizeof (ULONG));

                return ( ERROR_SAMPLE_IOERR );
              }
            }
          } while ( (SmpLen -= WriteLeft) );

          FreeSampleData ( CopyData, SAVESAMPLE_BUFFER * sizeof (ULONG));

          if ( SampleStr->tcs_Length & 1 )
          {
            *(ULONG *) &(IFF_Chunk.iff_Identifier[0]) = 0L;

            if ( (CWrite ( CFHandle, &IFF_Chunk, 1 )) != 1 )
              return ( ERROR_SAMPLE_IOERR );
          }
        }
#else
        if ( (CWrite ( CFHandle, SampleData, FORMSize )) != FORMSize )
          return ( ERROR_SAMPLE_IOERR );
#endif
      }
    } else
    {
      if ( (FaultCode = SaveTCSChunk ( CFHandle, (struct IFFChunk *) &SMPH, 'SMPH', sizeof (struct IFF_SMPH), (ULONG *) &SmplSize, ERROR_SAMPLE_IOERR )))
        return ( FaultCode );
    }
  }

  if ( SampleStr->tcs_Synth )
  {
    if ( (FaultCode = SaveSmplSnth ( CFHandle, SampleStr->tcs_Synth, (ULONG *) &SmplSize, FlagPassStr )))
      return ( FaultCode );
  }

  UnkChunk = (struct UnknownChunk *) &(SampleStr->tcs_UnknownChunks.mlh_Head);

  for (;;)
  {
    UnkChunk = (struct UnknownChunk *) UnkChunk->uc_Node.mln_Succ;

    if (!( UnkChunk->uc_Node.mln_Succ ))
      break;

    if ( (FaultCode = SaveTCSChunk ( CFHandle, (struct IFFChunk *) &UnkChunk, ReadMLong ( (UBYTE *) &(UnkChunk->uc_Identifier[0])), ReadMLong ( (UBYTE *) &(UnkChunk->uc_Length[0])), &SmplSize, ERROR_SAMPLE_IOERR )))
      return ( FaultCode );
  }

  if ( (SmplLenPos = CSeek ( CFHandle, SmplLenPos, OFFSET_BEGINNING )) == 0xFFFFFFFF )
    return ( ERROR_SAMPLE_IOERR );

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), SmplSize );

  if ( CWrite ( CFHandle, &(IFF_Header.ifh_Length[0]), sizeof (ULONG)) != sizeof (ULONG))
    return ( ERROR_SAMPLE_IOERR );

  if ( CSeek ( CFHandle, SmplLenPos, OFFSET_BEGINNING ) == 0xFFFFFFFF )
    return ( ERROR_SAMPLE_IOERR );

  *ChunkLenBuf += SmplSize + (sizeof (struct IFFHeader) - sizeof (ULONG));

  return ( 0L );
}

static ULONG SaveSmplSnth ( struct CFHandle *CFHandle, struct TuComposerSynth *SynthStr, ULONG *ChunkLenBuf, union FlagPass *FlagPassStr )
{
  struct IFFHeader IFF_Header;
  struct IFFChunk IFF_Chunk;
  struct MIFF_YHDR YHDR;
  struct TuComposerSynthWave *WaveformStr;
  struct TuComposerSynthSymbolTable *SymbolStr;
  struct UnknownChunk *UnkChunk;
  ULONG SnthSize = sizeof (ULONG), FORMSize;
  ULONG SnthLenPos, FORMLenPos;
  ULONG FaultCode;

  if ( (SnthLenPos = CSeek ( CFHandle, 0L, OFFSET_CURRENT )) == 0xFFFFFFFF )
    return ( ERROR_SYNTH_IOERR );

  SnthLenPos += sizeof (ULONG);

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0]), 'FORM' );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), SnthSize );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_TypeId[0]), 'SNTH' );

  if ( (CWrite ( CFHandle, &IFF_Header, sizeof (struct IFFHeader )) != sizeof (struct IFFHeader )))
    return ( ERROR_SYNTH_IOERR );

  if ( (FaultCode = SaveTCSName ( CFHandle, SynthStr->tcy_FileName, 'FILE', (ULONG *) &SnthSize, ERROR_SYNTH_IOERR )))
    return ( FaultCode );

  if ( (FaultCode = SaveTCSName ( CFHandle, SynthStr->tcy_Name, 'NAME', (ULONG *) &SnthSize, ERROR_SYNTH_IOERR )))
    return ( FaultCode );

  WriteMWord ( (UBYTE *) &(YHDR.myhdr_NumWaves), GetNumWaveforms ( SynthStr ));

  memcpy ( &(YHDR.myhdr_VolumePos), &(SynthStr->tcy_VolumePos), (sizeof (struct MIFF_YHDR) - sizeof (struct IFFChunk)));

#ifndef MOTOROLA
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_VolumePos), YHDR.myhdr_VolumePos );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_PanningPos), YHDR.myhdr_PanningPos );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_SlidePos), YHDR.myhdr_SlidePos );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_SpecialPos), YHDR.myhdr_SpecialPos );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_VolumeSusPos), YHDR.myhdr_VolumeSusPos );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_PanningSusPos), YHDR.myhdr_PanningSusPos );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_SlideSusPos), YHDR.myhdr_SlideSusPos );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_SpecialSusPos), YHDR.myhdr_SpecialSusPos );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_VolNNAPos), YHDR.myhdr_VolNNAPos );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_PanNNAPos), YHDR.myhdr_PanNNAPos );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_SlideNNAPos), YHDR.myhdr_SlideNNAPos );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_SpecialNNAPos), YHDR.myhdr_SpecialNNAPos );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_VolDNAPos), YHDR.myhdr_VolDNAPos );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_PanDNAPos), YHDR.myhdr_PanDNAPos );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_SlideDNAPos), YHDR.myhdr_SlideDNAPos );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_SpecialDNAPos), YHDR.myhdr_SpecialDNAPos );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_Variable0), YHDR.myhdr_Variable0 );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_Variable1), YHDR.myhdr_Variable1 );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_Variable2), YHDR.myhdr_Variable2 );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_Variable3), YHDR.myhdr_Variable3 );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_Variable4), YHDR.myhdr_Variable4 );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_Variable5), YHDR.myhdr_Variable5 );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_Variable6), YHDR.myhdr_Variable6 );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_Variable7), YHDR.myhdr_Variable7 );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_Variable8), YHDR.myhdr_Variable8 );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_Variable9), YHDR.myhdr_Variable9 );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_VariableA), YHDR.myhdr_VariableA );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_VariableB), YHDR.myhdr_VariableB );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_VariableC), YHDR.myhdr_VariableC );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_VariableD), YHDR.myhdr_VariableD );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_VariableE), YHDR.myhdr_VariableE );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_VariableF), YHDR.myhdr_VariableF );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_VolConVar), YHDR.myhdr_VolConVar );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_PanConVar), YHDR.myhdr_PanConVar );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_SlideConVar), YHDR.myhdr_SlideConVar );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_SpecialConVar), YHDR.myhdr_SpecialConVar );
  WriteMWord ( (UBYTE *) &(YHDR.myhdr_VarKeepMask), YHDR.myhdr_VarKeepMask );
#endif

  if ( (FaultCode = SaveTCSChunk ( CFHandle, (struct IFFChunk *) &YHDR, 'YHDR', sizeof (struct MIFF_YHDR), (ULONG *) &SnthSize, ERROR_SYNTH_IOERR )))
    return ( FaultCode );

  WaveformStr = (struct TuComposerSynthWave *) &(SynthStr->tcy_Waves.mlh_Head);

  if ( WaveformStr->tcyw_Node.mln_Succ->mln_Succ )
  {
    SnthSize += (sizeof (struct IFFHeader) - sizeof (ULONG));
    FORMSize = sizeof (ULONG);

    if ( (FORMLenPos = CSeek ( CFHandle, 0L, OFFSET_CURRENT )) == 0xFFFFFFFF )
      return ( ERROR_SYNTH_IOERR );

    FORMLenPos += sizeof (ULONG);

    WriteMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0]), 'FORM' );
    WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), FORMSize );
    WriteMLong ( (UBYTE *) &(IFF_Header.ifh_TypeId[0]), 'WFRM' );

    if ( (CWrite ( CFHandle, &IFF_Header, sizeof (struct IFFHeader )) != sizeof (struct IFFHeader )))
      return ( ERROR_SYNTH_IOERR );

    for (;;)
    {
      WaveformStr = (struct TuComposerSynthWave *) WaveformStr->tcyw_Node.mln_Succ;

      if (!( WaveformStr->tcyw_Node.mln_Succ ))
        break;

      if ( (FaultCode = SaveSnthWave ( CFHandle, WaveformStr, (ULONG *) &FORMSize )))
        return ( FaultCode );
    }

    if ( (FORMLenPos = CSeek ( CFHandle, FORMLenPos, OFFSET_BEGINNING )) == 0xFFFFFFFF )
      return ( ERROR_SYNTH_IOERR );

    WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), FORMSize );

    if ( CWrite ( CFHandle, &(IFF_Header.ifh_Length[0]), sizeof (ULONG)) != sizeof (ULONG))
      return ( ERROR_SYNTH_IOERR );

    if ( CSeek ( CFHandle, FORMLenPos, OFFSET_BEGINNING ) == 0xFFFFFFFF )
      return ( ERROR_SYNTH_IOERR );

    SnthSize += FORMSize;
  }

  FORMSize = SynthStr->tcy_CodeEntries * sizeof (struct TuComposerSynthCode);

  WriteMLong ( (UBYTE *) &(IFF_Chunk.iff_Identifier[0]), 'CODE' );
  WriteMLong ( (UBYTE *) &(IFF_Chunk.iff_Length[0]), FORMSize );

  if ( (CWrite ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk )) != sizeof (struct IFFChunk )))
    return ( ERROR_SYNTH_IOERR );

  FORMSize += (FORMSize & 1);
  SnthSize += sizeof (struct IFFChunk) + FORMSize;

  if ( (CWrite ( CFHandle, SynthStr->tcy_SynthCode, FORMSize )) != FORMSize )
    return ( ERROR_SYNTH_IOERR );

  if ( FlagPassStr[2].flg_Value )
  {
    SymbolStr = (struct TuComposerSynthSymbolTable *) &(SynthStr->tcy_Symbol.mlh_Head);

    if ( SymbolStr->tsst_Node.mln_Succ->mln_Succ )
    {
      SnthSize += (sizeof (struct IFFHeader) - sizeof (ULONG));
      FORMSize = sizeof (ULONG);

      if ( (FORMLenPos = CSeek ( CFHandle, 0L, OFFSET_CURRENT )) == 0xFFFFFFFF )
        return ( ERROR_SYNTH_IOERR );

      FORMLenPos += sizeof (ULONG);

      WriteMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0]), 'FORM' );
      WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), FORMSize );
      WriteMLong ( (UBYTE *) &(IFF_Header.ifh_TypeId[0]), 'STAB' );

      if ( (CWrite ( CFHandle, &IFF_Header, sizeof (struct IFFHeader )) != sizeof (struct IFFHeader )))
        return ( ERROR_SYNTH_IOERR );

      for (;;)
      {
        SymbolStr = (struct TuComposerSynthSymbolTable *) SymbolStr->tsst_Node.mln_Succ;

        if (!( SymbolStr->tsst_Node.mln_Succ ))
          break;

        if ( (FaultCode = SaveSnthSmbl ( CFHandle, SymbolStr, (ULONG *) &FORMSize )))
          return ( FaultCode );
      }

      if ( (FORMLenPos = CSeek ( CFHandle, FORMLenPos, OFFSET_BEGINNING )) == 0xFFFFFFFF )
        return ( ERROR_SYNTH_IOERR );

      WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), FORMSize );

      if ( CWrite ( CFHandle, &(IFF_Header.ifh_Length[0]), sizeof (ULONG)) != sizeof (ULONG))
        return ( ERROR_SYNTH_IOERR );

      if ( CSeek ( CFHandle, FORMLenPos, OFFSET_BEGINNING ) == 0xFFFFFFFF )
        return ( ERROR_SYNTH_IOERR );

      SnthSize += FORMSize;
    }
  }

  UnkChunk = (struct UnknownChunk *) &(SynthStr->tcy_UnknownChunks.mlh_Head);

  for (;;)
  {
    UnkChunk = (struct UnknownChunk *) UnkChunk->uc_Node.mln_Succ;

    if (!( UnkChunk->uc_Node.mln_Succ ))
      break;

    if ( (FaultCode = SaveTCSChunk ( CFHandle, (struct IFFChunk *) &UnkChunk, ReadMLong ( (UBYTE *) &(UnkChunk->uc_Identifier[0])), ReadMLong ( (UBYTE *) &(UnkChunk->uc_Length[0])), &SnthSize, ERROR_SYNTH_IOERR )))
      return ( FaultCode );
  }

  if ( (SnthLenPos = CSeek ( CFHandle, SnthLenPos, OFFSET_BEGINNING )) == 0xFFFFFFFF )
    return ( ERROR_SYNTH_IOERR );

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), SnthSize );

  if ( CWrite ( CFHandle, &(IFF_Header.ifh_Length[0]), sizeof (ULONG)) != sizeof (ULONG))
    return ( ERROR_SYNTH_IOERR );

  if ( CSeek ( CFHandle, SnthLenPos, OFFSET_BEGINNING ) == 0xFFFFFFFF )
    return ( ERROR_SYNTH_IOERR );

  *ChunkLenBuf += SnthSize + (sizeof (struct IFFHeader) - sizeof (ULONG));

  return ( 0L );
}

static ULONG SaveSnthWave ( struct CFHandle *CFHandle, struct TuComposerSynthWave *WaveformStr, ULONG *ChunkLenBuf )
{
  struct IFFHeader IFF_Header;
  struct IFFChunk IFF_Chunk;
  struct IFF_WHDR WHDR;
  BYTE *WaveData = (BYTE *) WaveformStr->tcyw_WaveData;
  ULONG WaveSize = sizeof (ULONG), FORMSize;
  ULONG WaveLenPos;
  ULONG FaultCode, WaveLen;

  if ( (WaveLenPos = CSeek ( CFHandle, 0L, OFFSET_CURRENT )) == 0xFFFFFFFF )
    return ( ERROR_SYNTH_IOERR );

  WaveLenPos += sizeof (ULONG);

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0]), 'FORM' );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), WaveSize );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_TypeId[0]), 'WAVE' );

  if ( (CWrite ( CFHandle, &IFF_Header, sizeof (struct IFFHeader )) != sizeof (struct IFFHeader )))
    return ( ERROR_SYNTH_IOERR );

  if ( (FaultCode = SaveTCSName ( CFHandle, WaveformStr->tcyw_FileName, 'FILE', (ULONG *) &WaveSize, ERROR_SYNTH_IOERR )))
    return ( FaultCode );

  if ( (FaultCode = SaveTCSName ( CFHandle, WaveformStr->tcyw_Name, 'NAME', (ULONG *) &WaveSize, ERROR_SYNTH_IOERR )))
    return ( FaultCode );

   memcpy ( &(WHDR.whdr_Repeat), &(WaveformStr->tcyw_Repeat), (sizeof (struct IFF_WHDR) - sizeof (struct IFFChunk)));

#ifndef MOTOROLA
   WriteMLong ( (UBYTE *) &(WHDR.whdr_Repeat), WHDR.whdr_Repeat );
   WriteMLong ( (UBYTE *) &(WHDR.whdr_RepLen), WHDR.whdr_RepLen );
   WriteMWord ( (UBYTE *) &(WHDR.whdr_Flags), WHDR.whdr_Flags );
#endif

  if ( (FaultCode = SaveTCSChunk ( CFHandle, (struct IFFChunk *) &WHDR, 'WHDR', sizeof (struct IFF_WHDR), (ULONG *) &WaveSize, ERROR_SYNTH_IOERR )))
    return ( FaultCode );

  FORMSize = WaveformStr->tcyw_Length;
  WaveLen = WaveformStr->tcyw_Samples;

  WriteMLong ( (UBYTE *) &(IFF_Chunk.iff_Identifier[0]), 'BODY' );
  WriteMLong ( (UBYTE *) &(IFF_Chunk.iff_Length[0]), FORMSize );

  if ( (CWrite ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk )) != sizeof (struct IFFChunk )))
    return ( ERROR_SYNTH_IOERR );

  FORMSize += (FORMSize & 1);
  WaveSize += sizeof (struct IFFChunk) + FORMSize;

#ifndef MOTOROLA
  if ( WaveformStr->tcyw_Flags & TCYWF_8Bit )
  {
    WaveLen = WaveformStr->tcyw_Length;

    if ( WaveLen & 1 )
      WaveData[WaveLen] = 0;

    if ( (CWrite ( CFHandle, WaveData, FORMSize )) != FORMSize )
      return ( ERROR_SYNTH_IOERR );
  } else
  {
    WORD *CopyData = AllocSampleData ( SAVESAMPLE_BUFFER * sizeof (WORD));
    ULONG WriteLeft;

    do
    {
      WORD *Buffer = CopyData;
      ULONG i;

      WriteLeft = SAVESAMPLE_BUFFER;

      if ( WaveLen < SAVESAMPLE_BUFFER )
        WriteLeft = WaveLen;

      for (i = WriteLeft; i > 0; i--)
      {
        *Buffer++ = ReadMWord ( (UBYTE *) WaveData++ );
      }

      if ( CWrite ( CFHandle, CopyData, (WriteLeft * sizeof (WORD))) != (WriteLeft * sizeof (WORD)))
      {
        FreeSampleData ( CopyData, SAVESAMPLE_BUFFER * sizeof (WORD));

        return ( ERROR_SYNTH_IOERR );
      }
    } while ( (WaveLen -= WriteLeft) );

    FreeSampleData ( CopyData, SAVESAMPLE_BUFFER * sizeof (WORD));

    if ( WaveformStr->tcyw_Length & 1 )
    {
      *(ULONG *) &(IFF_Chunk.iff_Identifier[0]) = 0L;

      if ( (CWrite ( CFHandle, &IFF_Chunk, 1 )) != 1 )
        return ( ERROR_SYNTH_IOERR );
    }
  }
#else
  if ( (CWrite ( CFHandle, WaveData, FORMSize )) != FORMSize )
    return ( ERROR_SYNTH_IOERR );
#endif

  if ( (WaveLenPos = CSeek ( CFHandle, WaveLenPos, OFFSET_BEGINNING )) == 0xFFFFFFFF )
    return ( ERROR_SYNTH_IOERR );

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), WaveSize );

  if ( CWrite ( CFHandle, &(IFF_Header.ifh_Length[0]), sizeof (ULONG)) != sizeof (ULONG))
    return ( ERROR_SYNTH_IOERR );

  if ( CSeek ( CFHandle, WaveLenPos, OFFSET_BEGINNING ) == 0xFFFFFFFF )
    return ( ERROR_SYNTH_IOERR );

  *ChunkLenBuf += WaveSize + (sizeof (struct IFFHeader) - sizeof (ULONG));

  return ( 0L );
}

static ULONG SaveSnthSmbl ( struct CFHandle *CFHandle, struct TuComposerSynthSymbolTable *SymbolStr, ULONG *ChunkLenBuf )
{
  struct IFFHeader IFF_Header;
  struct IFF_SREF SREF;
  ULONG SmblSize = sizeof (ULONG);
  ULONG SmblLenPos;
  ULONG FaultCode;

  if ( (SmblLenPos = CSeek ( CFHandle, 0L, OFFSET_CURRENT )) == 0xFFFFFFFF )
    return ( ERROR_SYNTH_IOERR );

  SmblLenPos += sizeof (ULONG);

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0]), 'FORM' );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), SmblSize );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_TypeId[0]), 'SMBL' );

  if ( (CWrite ( CFHandle, &IFF_Header, sizeof (struct IFFHeader )) != sizeof (struct IFFHeader )))
    return ( ERROR_SYNTH_IOERR );

  if ( (FaultCode = SaveTCSName ( CFHandle, SymbolStr->tsst_SymbolName, 'NAME', (ULONG *) &SmblSize, ERROR_SYNTH_IOERR )))
    return ( FaultCode );

  memcpy ( &(SREF.sref_SymbolValue), &(SymbolStr->tsst_SymbolValue), (sizeof (struct IFF_SREF) - sizeof (struct IFFChunk)));

#ifndef MOTOROLA
  WriteMWord ( (UBYTE *) &(SREF.sref_SymbolValue), SREF.sref_SymbolValue );
  WriteMWord ( (UBYTE *) &(SREF.sref_FirstLine), SREF.sref_FirstLine );
  WriteMWord ( (UBYTE *) &(SREF.sref_LastLine), SREF.sref_LastLine );
#endif

  if ( (FaultCode = SaveTCSChunk ( CFHandle, (struct IFFChunk *) &SREF, 'SREF', sizeof (struct IFF_SREF), (ULONG *) &SmblSize, ERROR_SYNTH_IOERR )))
    return ( FaultCode );

  if ( (SmblLenPos = CSeek ( CFHandle, SmblLenPos, OFFSET_BEGINNING )) == 0xFFFFFFFF )
    return ( ERROR_SYNTH_IOERR );

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), SmblSize );

  if ( CWrite ( CFHandle, &(IFF_Header.ifh_Length[0]), sizeof (ULONG)) != sizeof (ULONG))
    return ( ERROR_SYNTH_IOERR );

  if ( CSeek ( CFHandle, SmblLenPos, OFFSET_BEGINNING ) == 0xFFFFFFFF )
    return ( ERROR_SYNTH_IOERR );

  *ChunkLenBuf += SmblSize + (sizeof (struct IFFHeader) - sizeof (ULONG));

  return ( 0L );
}

static ULONG SaveTCSEnvd ( struct CFHandle *CFHandle, struct TuComposerInstrEnvelope *EnvStr, ULONG *ChunkLenBuf )
{
  struct IFFHeader IFF_Header;
  struct IFFChunk IFF_Chunk;
  struct MIFF_EHDR EHDR;
  struct UnknownChunk *UnkChunk;
  WORD *EnvData = EnvStr->tcie_Data;
  UWORD *EnvNode = EnvStr->tcie_NodePoints;
#ifndef MOTOROLA
  WORD *CopyData;
  UWORD *CopyNode;
  ULONG WriteLeft;
#endif
  ULONG EnvdSize = sizeof (ULONG), FORMSize;
  ULONG EnvdLenPos;
  ULONG FaultCode, EnvLen;

  if ( (EnvdLenPos = CSeek ( CFHandle, 0L, OFFSET_CURRENT )) == 0xFFFFFFFF )
    return ( ERROR_INSTR_IOERR );

  EnvdLenPos += sizeof (ULONG);

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0]), 'FORM' );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), EnvdSize );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_TypeId[0]), 'ENVD' );

  if ( (CWrite ( CFHandle, &IFF_Header, sizeof (struct IFFHeader )) != sizeof (struct IFFHeader )))
    return ( ERROR_INSTR_IOERR );

  WriteMWord ( (UBYTE *) &(EHDR.mehdr_Flags), EnvStr->tcie_Flags );
  WriteMWord ( (UBYTE *) &(EHDR.mehdr_Tempo), EnvStr->tcie_Tempo );
  WriteMWord ( (UBYTE *) &(EHDR.mehdr_SustainStart), EnvStr->tcie_SustainStart );
  WriteMWord ( (UBYTE *) &(EHDR.mehdr_SustainEnd), EnvStr->tcie_SustainEnd );
  WriteMWord ( (UBYTE *) &(EHDR.mehdr_SustainCount), EnvStr->tcie_SustainCount );
  WriteMWord ( (UBYTE *) &(EHDR.mehdr_LoopStart), EnvStr->tcie_LoopStart );
  WriteMWord ( (UBYTE *) &(EHDR.mehdr_LoopEnd), EnvStr->tcie_LoopEnd );
  WriteMWord ( (UBYTE *) &(EHDR.mehdr_LoopCount), EnvStr->tcie_LoopCount );
  WriteMWord ( (UBYTE *) &(EHDR.mehdr_LowestValue), EnvStr->tcie_LowestValue );
  WriteMWord ( (UBYTE *) &(EHDR.mehdr_HighestValue), EnvStr->tcie_HighestValue );

  if ( (FaultCode = SaveTCSChunk ( CFHandle, (struct IFFChunk *) &EHDR, 'EHDR', sizeof (struct MIFF_EHDR), (ULONG *) &EnvdSize, ERROR_INSTR_IOERR )))
    return ( FaultCode );

  EnvLen = EnvStr->tcie_NumPoints;
  FORMSize = EnvLen * sizeof (WORD);

  WriteMLong ( (UBYTE *) &(IFF_Chunk.iff_Identifier[0]), 'BODY' );
  WriteMLong ( (UBYTE *) &(IFF_Chunk.iff_Length[0]), FORMSize );

  if ( (CWrite ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk )) != sizeof (struct IFFChunk )))
    return ( ERROR_INSTR_IOERR );

  FORMSize += (FORMSize & 1);
  EnvdSize += sizeof (struct IFFChunk) + FORMSize;

#ifndef MOTOROLA
  CopyData = AllocSampleData ( SAVESAMPLE_BUFFER * sizeof (WORD));

  do
  {
    WORD *Buffer = CopyData;
    ULONG i;

    WriteLeft = SAVESAMPLE_BUFFER;

    if ( EnvLen < SAVESAMPLE_BUFFER )
      WriteLeft = EnvLen;

    for (i = WriteLeft; i > 0; i--)
    {
      *Buffer++ = ReadMWord ( (UBYTE *) EnvData++ );
    }

    if ( CWrite ( CFHandle, CopyData, (WriteLeft * sizeof (WORD))) != (WriteLeft * sizeof (WORD)))
    {
      FreeSampleData ( CopyData, SAVESAMPLE_BUFFER * sizeof (WORD));

      return ( ERROR_INSTR_IOERR );
    }
  } while ( (EnvLen -= WriteLeft) );

  FreeSampleData ( CopyData, SAVESAMPLE_BUFFER * sizeof (WORD));

  if ( (EnvStr->tcie_NumPoints * sizeof (WORD)) & 1 )
  {
    *(ULONG *) &(IFF_Chunk.iff_Identifier[0]) = 0L;

    if ( (CWrite ( CFHandle, &IFF_Chunk, 1 )) != 1 )
      return ( ERROR_INSTR_IOERR );
  }
#else
  if ( (CWrite ( CFHandle, EnvStr->tcie_Data, FORMSize )) != FORMSize )
    return ( ERROR_INSTR_IOERR );
#endif

  if ( EnvNode )
  {
    EnvLen = EnvStr->tcie_NumNodes;
    FORMSize = EnvLen * sizeof (UWORD);

    WriteMLong ( (UBYTE *) &(IFF_Chunk.iff_Identifier[0]), 'NODE' );
    WriteMLong ( (UBYTE *) &(IFF_Chunk.iff_Length[0]), FORMSize );

    if ( (CWrite ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk )) != sizeof (struct IFFChunk )))
      return ( ERROR_INSTR_IOERR );

    FORMSize += (FORMSize & 1);
    EnvdSize += sizeof (struct IFFChunk) + FORMSize;

#ifndef MOTOROLA
    CopyNode = (UWORD *) AllocSampleData ( SAVESAMPLE_BUFFER * sizeof (UWORD));

    do
    {
      UWORD *Buffer = CopyNode;
      ULONG i;

      WriteLeft = SAVESAMPLE_BUFFER;

      if ( EnvLen < SAVESAMPLE_BUFFER )
        WriteLeft = EnvLen;

      for (i = WriteLeft; i > 0; i--)
      {
        *Buffer++ = ReadMWord ( (UBYTE *) EnvNode++ );
      }

      if ( CWrite ( CFHandle, CopyData, (WriteLeft * sizeof (WORD))) != (WriteLeft * sizeof (UWORD)))
      {
        FreeSampleData ( CopyData, SAVESAMPLE_BUFFER * sizeof (UWORD));

        return ( ERROR_INSTR_IOERR );
      }
    } while ( (EnvLen -= WriteLeft) );

    FreeSampleData ( CopyData, SAVESAMPLE_BUFFER * sizeof (UWORD));

    if ( (EnvStr->tcie_NumNodes * sizeof (UWORD)) & 1 )
    {
      *(ULONG *) &(IFF_Chunk.iff_Identifier[0]) = 0L;

      if ( (CWrite ( CFHandle, &IFF_Chunk, 1 )) != 1 )
        return ( ERROR_INSTR_IOERR );
    }
#else
    if ( (CWrite ( CFHandle, EnvStr->tcie_NodePoints, FORMSize )) != FORMSize )
      return ( ERROR_INSTR_IOERR );
#endif
  }

  UnkChunk = (struct UnknownChunk *) &(EnvStr->tcie_UnknownChunks.mlh_Head);

  for (;;)
  {
    UnkChunk = (struct UnknownChunk *) UnkChunk->uc_Node.mln_Succ;

    if (!( UnkChunk->uc_Node.mln_Succ ))
      break;

    if ( (FaultCode = SaveTCSChunk ( CFHandle, (struct IFFChunk *) &UnkChunk, ReadMLong ( (UBYTE *) &(UnkChunk->uc_Identifier[0])), ReadMLong ( (UBYTE *) &(UnkChunk->uc_Length[0])), &EnvdSize, ERROR_INSTR_IOERR )))
      return ( FaultCode );
  }

  if ( (EnvdLenPos = CSeek ( CFHandle, EnvdLenPos, OFFSET_BEGINNING )) == 0xFFFFFFFF )
    return ( ERROR_INSTR_IOERR );

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), EnvdSize );

  if ( CWrite ( CFHandle, &(IFF_Header.ifh_Length[0]), sizeof (ULONG)) != sizeof (ULONG))
    return ( ERROR_INSTR_IOERR );

  if ( CSeek ( CFHandle, EnvdLenPos, OFFSET_BEGINNING ) == 0xFFFFFFFF )
    return ( ERROR_INSTR_IOERR );

  *ChunkLenBuf += EnvdSize + (sizeof (struct IFFHeader) - sizeof (ULONG));

  return ( 0L );
}

static ULONG SaveTCSKbrd ( struct CFHandle *CFHandle, struct TuComposerInstrKeyboard *KeybrdStr, ULONG *ChunkLenBuf )
{
  struct IFF_KBRD KBRD;
#ifndef MOTOROLA
  struct KeyboardEntry *KeybrdEntry;
  ULONG i;
#endif

  memcpy ( &(KBRD.kbrd_Keys[0]), &(KeybrdStr->tcik_NoteEntry[0]), (sizeof (struct IFF_KBRD) - sizeof (struct IFFChunk)));

#ifndef MOTOROLA
  KeybrdEntry = (struct KeyboardEntry *) &(KBRD.kbrd_Keys[0]);

  for (i = NUMOCTAVES * NUMNOTES; i > 0; i--)
  {
    WriteMWord ( (UBYTE *) &(KeybrdEntry->tcik_Sample_C_0), KeybrdEntry->tcik_Sample_C_0 );

    KeybrdEntry++;
  }
#endif

  return ( SaveTCSChunk ( CFHandle, (struct IFFChunk *) &KBRD, 'KBRD', sizeof (struct IFF_KBRD), (ULONG *) ChunkLenBuf, ERROR_INSTR_IOERR ));
}

static ULONG SaveTCSArpg ( struct CFHandle *CFHandle, struct TuComposerInstrArpeggio *ArpStr, ULONG *ChunkLenBuf )
{
  struct IFFHeader IFF_Header;
  struct IFF_AHDR AHDR;
  ULONG ArpgSize = sizeof (ULONG), FORMSize;
  ULONG ArpgLenPos, FORMLenPos;
  ULONG FaultCode;

  if ( (ArpgLenPos = CSeek ( CFHandle, 0L, OFFSET_CURRENT )) == 0xFFFFFFFF )
    return ( ERROR_INSTR_IOERR );

  ArpgLenPos += sizeof (ULONG);

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0]), 'FORM' );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), ArpgSize );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_TypeId[0]), 'ARPG' );

  if ( (CWrite ( CFHandle, &IFF_Header, sizeof (struct IFFHeader )) != sizeof (struct IFFHeader )))
    return ( ERROR_INSTR_IOERR );

  WriteMWord ( (UBYTE *) &(AHDR.ahdr_Length), ArpStr->tcia_Entries );
  WriteMWord ( (UBYTE *) &(AHDR.ahdr_Flags), ArpStr->tcia_Flags );
  WriteMWord ( (UBYTE *) &(AHDR.ahdr_SustainStart), ArpStr->tcia_SustainStart );
  WriteMWord ( (UBYTE *) &(AHDR.ahdr_SustainEnd), ArpStr->tcia_SustainEnd );
  WriteMWord ( (UBYTE *) &(AHDR.ahdr_SustainCount), ArpStr->tcia_SustainCount );
  WriteMWord ( (UBYTE *) &(AHDR.ahdr_LoopStart), ArpStr->tcia_LoopStart );
  WriteMWord ( (UBYTE *) &(AHDR.ahdr_LoopEnd), ArpStr->tcia_LoopEnd );

  if ( (FaultCode = SaveTCSChunk ( CFHandle, (struct IFFChunk *) &AHDR, 'AHDR', sizeof (struct IFF_AHDR), (ULONG *) &ArpgSize, ERROR_INSTR_IOERR )))
    return ( FaultCode );

  ArpgSize += (sizeof (struct IFFHeader) - sizeof (ULONG));
  FORMSize = sizeof (ULONG);

  if ( (FORMLenPos = CSeek ( CFHandle, 0L, OFFSET_CURRENT )) == 0xFFFFFFFF )
    return ( ERROR_INSTR_IOERR );

  FORMLenPos += sizeof (ULONG);

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_FORMId[0]), 'FORM' );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), FORMSize );
  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_TypeId[0]), 'ARPE' );

  if ( (CWrite ( CFHandle, &IFF_Header, sizeof (struct IFFHeader )) != sizeof (struct IFFHeader )))
    return ( ERROR_INSTR_IOERR );

  if ( (FaultCode = SaveArpgArpe ( CFHandle, ArpStr, (ULONG *) &FORMSize )))
    return ( FaultCode );

  if ( (FORMLenPos = CSeek ( CFHandle, FORMLenPos, OFFSET_BEGINNING )) == 0xFFFFFFFF )
    return ( ERROR_INSTR_IOERR );

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), FORMSize );

  if ( CWrite ( CFHandle, &(IFF_Header.ifh_Length[0]), sizeof (ULONG)) != sizeof (ULONG))
    return ( ERROR_INSTR_IOERR );

  if ( CSeek ( CFHandle, FORMLenPos, OFFSET_BEGINNING ) == 0xFFFFFFFF )
    return ( ERROR_INSTR_IOERR );

  ArpgSize += FORMSize;

  if ( (ArpgLenPos = CSeek ( CFHandle, ArpgLenPos, OFFSET_BEGINNING )) == 0xFFFFFFFF )
    return ( ERROR_INSTR_IOERR );

  WriteMLong ( (UBYTE *) &(IFF_Header.ifh_Length[0]), ArpgSize );

  if ( CWrite ( CFHandle, &(IFF_Header.ifh_Length[0]), sizeof (ULONG)) != sizeof (ULONG))
    return ( ERROR_INSTR_IOERR );

  if ( CSeek ( CFHandle, ArpgLenPos, OFFSET_BEGINNING ) == 0xFFFFFFFF )
    return ( ERROR_INSTR_IOERR );

  *ChunkLenBuf += ArpgSize + (sizeof (struct IFFHeader) - sizeof (ULONG));

  return ( 0L );
}

static ULONG SaveArpgArpe ( struct CFHandle *CFHandle, struct TuComposerInstrArpeggio *ArpStr, ULONG *ChunkLenBuf )
{
  struct IFF_ARPE ARPE;
  struct TuComposerInstrArpeggioData *ArpData = ArpStr->tcia_Data;
  ULONG FaultCode, i;

  for (i = ArpStr->tcia_Entries; i > 0; i--)
  {
    memcpy ( &(ARPE.arpe_Tone), ArpData, (sizeof (struct IFF_ARPE) - sizeof (struct IFFChunk )));

#ifndef MOTOROLA
    WriteMWord ( (UBYTE *) &(ARPE.arpe_DataWord), ARPE.arpe_DataWord );
    WriteMWord ( (UBYTE *) &(ARPE.arpe_DataWord2), ARPE.arpe_DataWord2 );
    WriteMWord ( (UBYTE *) &(ARPE.arpe_DataWord3), ARPE.arpe_DataWord3 );
    WriteMWord ( (UBYTE *) &(ARPE.arpe_DataWord4), ARPE.arpe_DataWord4 );
#endif

    if ( (FaultCode = SaveTCSChunk ( CFHandle, (struct IFFChunk *) &ARPE, 'ARPE', sizeof (struct IFF_ARPE), (ULONG *) ChunkLenBuf, ERROR_INSTR_IOERR )))
      return ( FaultCode );

    ArpData++;
  }

  return ( 0L );
}

static ULONG GetTCSInfo ( struct CFHandle *CFHandle,
    struct SongInfo *SongInfo, unsigned long NumSong )
{
  struct IFFHeader IFF_Header;
  struct IFF_MHDR MHDR;
  ULONG Length, IFFSize;
  UBYTE ScanFlags = 0;

  if ( (CRead ( CFHandle, &IFF_Header, sizeof (struct IFFHeader)) != sizeof (struct IFFHeader)) )
    return ( 0L );

  if ( (ReadMLong ( (UBYTE *) &IFF_Header.ifh_FORMId[0] ) != 'FORM') || ( (Length = ReadMLong ( (UBYTE *) &IFF_Header.ifh_Length[0]) ) < 4) || ((ReadMLong ( (UBYTE *) &IFF_Header.ifh_TypeId[0] ) >> 8) != 'TCM') )
    return ( 0L );

  Length -= 4;

  do
  {
    struct IFFChunk IFF_Chunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      goto GetTCSError;

    if ( (IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0] )) & 1 )
      IFFSize++;

    IFFLength = IFFSize;

    switch ( ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] ) )
    {
      case 'MHDR' :
      {
        if ( ScanFlags & 0x80 )
          return ( 0L );

        if ( (CRead ( CFHandle, &(MHDR.mhdr_Version), (sizeof (struct IFF_MHDR) - sizeof (struct IFFChunk))) != (sizeof (struct IFF_MHDR) - sizeof (struct IFFChunk))) )
          return ( 0L );

        SongInfo->soi_Version = MHDR.mhdr_Version;
        SongInfo->soi_Revision = MHDR.mhdr_Revision;

        memcpy ( &(SongInfo->soi_Instruments), &(MHDR.mhdr_Instruments), offsetof (struct SongInfo, soi_GoSubStackSize) - offsetof (struct SongInfo, soi_Instruments ));

#ifndef MOTOROLA
        SongInfo->soi_Instruments = ReadMWord ( (UBYTE *) &(SongInfo->soi_Instruments));
        SongInfo->soi_Samples = ReadMLong ( (UBYTE *) &(SongInfo->soi_Samples));
        SongInfo->soi_Synths = ReadMLong ( (UBYTE *) &(SongInfo->soi_Synths));
        SongInfo->soi_Envelopes = ReadMWord ( (UBYTE *) &(SongInfo->soi_Envelopes));
        SongInfo->soi_KeyboardDefs = ReadMWord ( (UBYTE *) &(SongInfo->soi_KeyboardDefs));
        SongInfo->soi_ArpeggioDefs = ReadMWord ( (UBYTE *) &(SongInfo->soi_ArpeggioDefs));
#endif

        IFFLength = IFFSize;
        IFFLength -= sizeof (struct IFF_MHDR) - sizeof (struct IFFChunk);
        ScanFlags |= 0x80;

        break;
      }
      case 'FORM' :
      {
        if ( (CRead ( CFHandle, &IFF_Header, 4 ) != 4 ) )
          goto GetTCSError;

        if ( (ReadMLong ( (UBYTE *) &IFF_Header.ifh_FORMId[0] )) == 'SONG' )
        {
          ULONG FaultCode;

          if (!( --NumSong ))
          {
            if ( (FaultCode = GetTCSSongInfo ( CFHandle, SongInfo, (UBYTE *) &ScanFlags, IFFSize )))
              goto GetTCSError;

            IFFLength = 0L;
          }
        }
      }
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
      {
GetTCSError:
        FreeNameStr ( SongInfo->soi_FileName );
        FreeNameStr ( SongInfo->soi_Name );
        FreeNameStr ( SongInfo->soi_Message );
        FreeNameStr ( SongInfo->soi_Composer );
        FreeNameStr ( SongInfo->soi_Style );
        FreeNameStr ( SongInfo->soi_Comment );

        return ( 0L );
      }
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      goto GetTCSError;

  } while ( (Length -= IFFSize) );

  if ( ScanFlags < 0xC0 )
    goto GetTCSError;

  return ( -1L );
}

static ULONG GetTCSSongInfo ( struct CFHandle *CFHandle, struct SongInfo *SongInfo, UBYTE *ScanFlagsPos, ULONG Length )
{
  struct IFF_SHDR SHDR;
  ULONG Invalid = 0L, IFFSize;
  UBYTE ScanFlags = *ScanFlagsPos;

  if ( Length < 4 )
    return ( ERROR_INVALID_RESIDENT_LIBRARY );

  Length -= 4;

  do
  {
    struct IFFChunk IFF_Chunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      return ( IoErr ());

    IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0]);
    IFFLength = IFFSize;

    IFFSize += (IFFSize & 1);

    switch ( ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] ) )
    {
      case 'FILE' :
      {
        if ( ScanFlags & 0x01 )
          break;

        if (!( SongInfo->soi_FileName = (STRPTR) AllocPMem ( IFFLength + 1 )))
          return ( IoErr ());

        if ( (CRead ( CFHandle, SongInfo->soi_FileName, IFFLength ) != IFFLength) )
          return ( IoErr ());

        IFFLength &= 1;
        ScanFlags |= 0x01;

        break;
      }
      case 'NAME' :
      {
        if ( ScanFlags & 0x02 )
          break;

        if (!( SongInfo->soi_Name = (STRPTR) AllocPMem ( IFFLength + 1 )))
          return ( IoErr ());

        if ( (CRead ( CFHandle, SongInfo->soi_Name, IFFLength ) != IFFLength) )
          return ( IoErr ());

        IFFLength &= 1;
        ScanFlags |= 0x02;

        break;
      }
      case 'SMSG' :
      {
        if ( ScanFlags & 0x04 )
          break;

        if (!( SongInfo->soi_Message = (STRPTR) AllocPMem ( IFFLength + 1 )))
          return ( IoErr ());

        if ( (CRead ( CFHandle, SongInfo->soi_Message, IFFLength ) != IFFLength) )
          return ( IoErr ());

        IFFLength &= 1;
        ScanFlags |= 0x04;

        break;
      }
      case 'AUTH' :
      {
        if ( ScanFlags & 0x08 )
          break;

        if (!( SongInfo->soi_Composer = (STRPTR) AllocPMem ( IFFLength + 1 )))
          return ( IoErr ());

        if ( (CRead ( CFHandle, SongInfo->soi_Composer, IFFLength ) != IFFLength) )
          return ( IoErr ());

        IFFLength &= 1;
        ScanFlags |= 0x08;

        break;
      }
      case 'STIL' :
      {
        if ( ScanFlags & 0x10 )
          break;

        if (!( SongInfo->soi_Style = (STRPTR) AllocPMem ( IFFLength + 1 )))
          return ( IoErr ());

        if ( (CRead ( CFHandle, SongInfo->soi_Style, IFFLength ) != IFFLength) )
          return ( IoErr ());

        IFFLength &= 1;
        ScanFlags |= 0x10;

        break;
      }
      case 'CMNT' :
      {
        if ( ScanFlags & 0x20 )
          break;

        if (!( SongInfo->soi_Comment = (STRPTR) AllocPMem ( IFFLength + 1 )))
          return ( IoErr ());

        if ( (CRead ( CFHandle, SongInfo->soi_Comment, IFFLength ) != IFFLength) )
          return ( IoErr ());

        IFFLength &= 1;
        ScanFlags |= 0x20;

        break;
      }
      case 'SHDR' :
      {
        if ( ScanFlags & 0x40 )
          return ( ERROR_INVALID_RESIDENT_LIBRARY );

        if ( (CRead ( CFHandle, &(SHDR.shdr_BeginDate), (sizeof (struct IFF_SHDR) - sizeof (struct IFFChunk))) != (sizeof (struct IFF_SHDR) - sizeof (struct IFFChunk))) )
          return ( IoErr ());

        memcpy ( &(SongInfo->soi_BeginDate), &(SHDR.shdr_BeginDate), offsetof (struct IFF_SHDR, shdr_GoSubStackSize) - sizeof (struct IFFChunk));

#ifndef MOTOROLA
        SongInfo->soi_BeginDate.sds_Year = ReadMWord ( (UBYTE *) &(SongInfo->soi_BeginDate.sds_Year));
        SongInfo->soi_EndDate.sds_Year = ReadMWord ( (UBYTE *) &(SongInfo->soi_EndDate.sds_Year));
        SongInfo->soi_Tracks = ReadMWord ( (UBYTE *) &(SongInfo->soi_Tracks));
#endif

        SongInfo->soi_CompFlags = SHDR.shdr_CompFlags;
        SongInfo->soi_Flags = SHDR.shdr_Flags;

        memcpy ( &(SongInfo->soi_Channels), &(SHDR.shdr_Channels), sizeof (struct IFF_SHDR) - offsetof ( struct IFF_SHDR, shdr_Channels ));

#ifndef MOTOROLA
        SongInfo->soi_Channels = ReadMWord ( (UBYTE *) &(SHDR.shdr_Channels) );
        SongInfo->soi_Frames = ReadMWord ( (UBYTE *) &(SHDR.shdr_Frames) );
        SongInfo->soi_SPDSpeed = ReadMWord ( (UBYTE *) &(SHDR.shdr_SPDSpeed) );
        SongInfo->soi_BPMTempo = ReadMWord ( (UBYTE *) &(SHDR.shdr_BPMTempo) );
        SongInfo->soi_BPMSpeed = ReadMWord ( (UBYTE *) &(SHDR.shdr_BPMSpeed) );
        SongInfo->soi_FramesLowerLimit = ReadMWord ( (UBYTE *) &(SHDR.shdr_FramesLowerLimit) );
        SongInfo->soi_FramesUpperLimit = ReadMWord ( (UBYTE *) &(SHDR.shdr_FramesUpperLimit) );
        SongInfo->soi_SPDLowerLimit = ReadMWord ( (UBYTE *) &(SHDR.shdr_SPDLowerLimit) );
        SongInfo->soi_SPDUpperLimit = ReadMWord ( (UBYTE *) &(SHDR.shdr_SPDUpperLimit) );
        SongInfo->soi_BPMTempoLowerLimit = ReadMWord ( (UBYTE *) &(SHDR.shdr_BPMTempoLowerLimit) );
        SongInfo->soi_BPMTempoUpperLimit = ReadMWord ( (UBYTE *) &(SHDR.shdr_BPMTempoUpperLimit) );
        SongInfo->soi_BPMSpeedLowerLimit = ReadMWord ( (UBYTE *) &(SHDR.shdr_BPMSpeedLowerLimit) );
        SongInfo->soi_BPMSpeedUpperLimit = ReadMWord ( (UBYTE *) &(SHDR.shdr_BPMSpeedUpperLimit) );
#endif

        IFFLength -= sizeof (struct IFF_SHDR) - sizeof (struct IFFChunk);
        ScanFlags |= 0x40;

        break;
      }
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
        return ( IoErr ());
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      return ( ERROR_INVALID_RESIDENT_LIBRARY );

  } while ( (Length -= IFFSize) );

  *ScanFlagsPos = ScanFlags;

  return ( 0L );
}

static ULONG GetNumTCSs ( struct CFHandle *CFHandle )
{
  struct IFFHeader IFF_Header;
  ULONG Length, IFFSize;
  ULONG NumTCSs = 0L;

  if ( (CRead ( CFHandle, &IFF_Header, sizeof (struct IFFHeader)) != sizeof (struct IFFHeader)) )
    return ( 0L );

  if ( (ReadMLong ( (UBYTE *) &IFF_Header.ifh_FORMId[0] ) != 'FORM') || ( (Length = ReadMLong ( (UBYTE *) &IFF_Header.ifh_Length[0]) ) < 4) || ((ReadMLong ( (UBYTE *) &IFF_Header.ifh_TypeId[0] ) >> 8) != 'TCM') )
    return ( 0L );

  Length -= 4;

  do
  {
    struct IFFChunk IFF_Chunk;
    ULONG IFFLength;

    if ( (CRead ( CFHandle, &IFF_Chunk, sizeof (struct IFFChunk)) != sizeof (struct IFFChunk )) )
      return ( 0L );

    if ( (IFFSize = ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Length[0] )) & 1 )
      IFFSize++;

    IFFLength = IFFSize;

    if ( (ReadMLong ( (UBYTE *) &IFF_Chunk.iff_Identifier[0] )) == 'FORM' )
    {
      if ( (CRead ( CFHandle, &IFF_Header, 4 ) != 4 ) )
        return ( 0L );

      if ( (ReadMLong ( (UBYTE *) &IFF_Header.ifh_FORMId[0] )) == 'SONG' )
        NumTCSs++;
    }

    if ( IFFLength )
    {
      if ( CSeek ( CFHandle, IFFLength, OFFSET_CURRENT ) == 0xFFFFFFFF )
        return ( 0L );
    }

    IFFSize += sizeof (struct IFFChunk );

    if ( Length < IFFSize )
      return ( 0L );

  } while ( (Length -= IFFSize) );

  return ( NumTCSs );
}

static void DeltaCrunch8 ( BYTE **SampleDataPos, BYTE *Buffer, BYTE *DeltaPos, ULONG Length )
{
  BYTE *SampleData = *SampleDataPos;
  ULONG i;
  BYTE Delta = *DeltaPos;

  for (i = Length; i > 0; i--)
  {
    BYTE Sample = *SampleData++;

    Sample -= Delta;
    *Buffer++ = Sample;

    Delta += Sample;
  }

  *SampleDataPos = SampleData;
  *DeltaPos = Delta;
}

static void DeltaCrunch16 ( WORD **SampleDataPos, WORD *Buffer, WORD *DeltaPos, ULONG Length )
{
  WORD *SampleData = *SampleDataPos;
  ULONG i;
  WORD Delta = *DeltaPos;

  for (i = Length; i > 0; i--)
  {
    WORD Sample = *SampleData++;

    Sample -= Delta;
    WriteMWord ( (UBYTE *) Buffer++, Sample );

    Delta += Sample;
  }

  *SampleDataPos = SampleData;
  *DeltaPos = Delta;
}

static void DeltaCrunch32 ( LONG **SampleDataPos, LONG *Buffer, LONG *DeltaPos, ULONG Length )
{
  LONG *SampleData = *SampleDataPos;
  ULONG i;
  LONG  Delta = *DeltaPos;

  for (i = Length; i > 0; i--)
  {
    LONG Sample = *SampleData++;

    Sample -= Delta;
    WriteMLong ( (UBYTE *) Buffer++, Sample );

    Delta += Sample;
  }

  *SampleDataPos = SampleData;
  *DeltaPos = Delta;
}

static void DeltaCrunchX ( LONG **SampleDataPos, LONG *Buffer, LONG *DeltaPos, ULONG BitDepth, ULONG Length )
{
  LONG *SampleData = *SampleDataPos;
  ULONG i;
  LONG  Delta = *DeltaPos;
  ULONG Bit = 0L;

  for (i = Length; i > 0; i--)
  {
    ULONG Sample;

    if ( (Bit + BitDepth) < 32 )
    {
      Sample = (*SampleData << Bit) & ~((1UL << (32UL - BitDepth)) - 1UL);
    } else
    {
      Sample = *SampleData++ << Bit;
      Sample |= (*SampleData & ~((1UL << (64UL - (Bit + BitDepth))) - 1UL)) >> (32UL - Bit);
    }

    Sample -= Delta;

    Sample &= ~((1UL << (32UL - BitDepth)) - 1UL );

    WriteMLong ( (UBYTE *) Buffer, ReadMLong ( (UBYTE *) Buffer ) | (Sample >> Bit));

    if ( (Bit + BitDepth) >= 32 )
    {
      Buffer++;
      WriteMLong ( (UBYTE *) Buffer, (Sample << (32UL - Bit)));

      Bit -= 32;
    }

    Bit += BitDepth;

    Delta += Sample;
  }

  *SampleDataPos = SampleData;
  *DeltaPos = Delta;
}

#ifndef MOTOROLA

static UWORD ReadMWord ( UBYTE *Buffer )
{
  UWORD MotorolaWord;

  MotorolaWord = *Buffer++ << 8U;
  MotorolaWord += *Buffer;

  return ( MotorolaWord );
}

static void WriteMWord ( UBYTE *Buffer, UWORD Value )
{
  *Buffer++ = Value >> 8U;
  *Buffer = Value;
}

static ULONG ReadMLong ( UBYTE *Buffer )
{
  ULONG MotorolaLong;

  MotorolaLong = *Buffer++;
  MotorolaLong <<= 8UL;
  MotorolaLong += *Buffer++;
  MotorolaLong <<= 8UL;
  MotorolaLong += *Buffer++;
  MotorolaLong <<= 8UL;
  MotorolaLong += *Buffer;

  return ( MotorolaLong );
}

static void WriteMLong ( UBYTE *Buffer, ULONG Value )
{
  *Buffer++ = Value >> 24UL;
  *Buffer++ = Value >> 16UL;
  *Buffer++ = Value >> 8UL;
  *Buffer = Value;
}

#endif
