/* TuComposer OS independent code
**
** $VER: DoPlayerIRQ.c 31.22 (28.4.00)
**
** (C) 2000 by Sebastian Vater (known as Basty/Seasons).
**
*/

#include <stddef.h>
#include <string.h>
#include <exec/types.h>
#include <tucomposer/tucomposer.h>
#include <tucomposer/synth.h>
#include <tucomposer/player.h>
#include <tucomposer/external/mixer.h>
#include <clib/exec_protos.h>
#include <clib/tucomposer_protos.h>
#include <clib/ext_mixerbase_protos.h>

#define RANDOM_CONST -1153374675
#define SLIDE_CONST (8363*1712*4)

/* Bit definitions of fx_Flags: */
#define FXB_ExecWholeRow           7L  /* If set, the effect will be
                                          executed the whole row, else
                                          only once */

#define FXF_ExecWholeRow           (1L<<FXB_ExecWholeRow)

struct EffectsTable {
  void (*fx_EffectPtr)( struct TuComposerBase *TuCBase,
    struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr,
    UWORD Channel, UWORD EffectByte, UWORD DataWord );
                                       /* Effect execution function ptr */
  void (*fx_PrePatternFXPtr)( struct TuComposerBase *TuCBase,
    struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr,
    UWORD Channel, UWORD DataWord );   /* Pre-pattern evaluation func ptr */
  void (*fx_CheckFXPtr)( struct TuComposerBase *TuCBase,
    struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr,
    UWORD Channel, UWORD *EffectByte, UWORD *DataWord, UWORD *Flags );
                                       /* Check parameter function ptr */
  UBYTE  fx_Flags;                     /* Special flags. See upwards */
  UBYTE  fx_ANDMaskCtrlFX;             /* AND-Mask for control effects */
  UWORD  fx_StdExecTick;               /* Standard exec. tick (mostly 0/1) */
}; /* EffectsTable */

#ifdef __cplusplus
  extern "C" {
#endif

static void ProcessRow ( struct TuComposerSong *SongStr, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel );
static void GetEffects ( struct TuComposerBase *TuCBase, struct TuComposerModule *ModuleStr, struct TuComposerSong *SongStr, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel );
static ULONG GetNote ( struct TuComposerBase *TuCBase, struct TuComposerModule *ModuleStr, struct TuComposerSong *SongStr, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel );
static void RunEffects ( struct TuComposerBase *TuCBase, struct TuComposerModule *ModuleStr, struct TuComposerSong *SongStr, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel );

static WORD GetKeyTableNote ( struct TuComposerModule *ModuleStr, struct TuComposerInstr *InstrStr, struct PlayerHostChannel *PHCStr, UWORD Octave, UWORD Note );
static WORD GetKeyTable ( struct TuComposerModule *ModuleStr, struct TuComposerInstr *InstrStr, struct PlayerHostChannel *PHCStr, UWORD Note );
static ULONG GetTonePitch ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, WORD Note );

static struct PlayerChannel *PlayNote ( struct TuComposerBase *TuCBase, struct TuComposerModule *ModuleStr, struct TuComposerInstr *InstrStr, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Octave, UWORD Note, ULONG Channel );
static struct PlayerChannel *PlayNoteGot ( struct TuComposerBase *TuCBase, struct TuComposerModule *ModuleStr, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Note, ULONG Channel );

static void InitNewInstr ( struct TuComposerBase *TuCBase, struct TuComposerSong *SongStr, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr );
static void InitNewSample ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr );

static struct PlayerChannel *TriggerNNA ( struct TuComposerModule *ModuleStr, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, ULONG Channel, UWORD *VirtualCh );
static ULONG TriggerDCT ( struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, ULONG DCT );

static void PlayKeyOff ( struct PlayerChannel *PCStr );

static void SetEnvelope ( struct PlayerChannel *PCStr, struct Envelope *EnvStr, UWORD EnvPos );
static WORD RunEnvelope ( struct TuComposerBase *TuCBase, struct Envelope *PlayEnvStr, UWORD TempoMul, UWORD ValAdjust );
static WORD StepEnvelope ( struct TuComposerBase *TuCBase, struct Envelope *PlayEnvStr, WORD *EnvData, UWORD EnvPos, UWORD TempoMul, UWORD ValAdjust );

static struct TuComposerInstrEnvelope *AssignVolumeEnv ( struct TuComposerSong *SongStr, struct TuComposerInstr *InstrStr, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, struct TuComposerInstrEnvelope **EnvStr, struct Envelope **PlayEnvStr );
static struct TuComposerInstrEnvelope *AssignPanningEnv ( struct TuComposerSong *SongStr, struct TuComposerInstr *InstrStr, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, struct TuComposerInstrEnvelope **EnvStr, struct Envelope **PlayEnvStr );
static struct TuComposerInstrEnvelope *AssignSlideEnv ( struct TuComposerSong *SongStr, struct TuComposerInstr *InstrStr, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, struct TuComposerInstrEnvelope **EnvStr, struct Envelope **PlayEnvStr );
static struct TuComposerInstrEnvelope *AssignVibratoEnv ( struct TuComposerSong *SongStr, struct TuComposerInstr *InstrStr, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, struct TuComposerInstrEnvelope **EnvStr, struct Envelope **PlayEnvStr );
static struct TuComposerInstrEnvelope *AssignTremoloEnv ( struct TuComposerSong *SongStr, struct TuComposerInstr *InstrStr, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, struct TuComposerInstrEnvelope **EnvStr, struct Envelope **PlayEnvStr );
static struct TuComposerInstrEnvelope *AssignPannoloEnv ( struct TuComposerSong *SongStr, struct TuComposerInstr *InstrStr, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, struct TuComposerInstrEnvelope **EnvStr, struct Envelope **PlayEnvStr );
static struct TuComposerInstrEnvelope *AssignChannoloEnv ( struct TuComposerSong *SongStr, struct TuComposerInstr *InstrStr, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, struct TuComposerInstrEnvelope **EnvStr, struct Envelope **PlayEnvStr );
static struct TuComposerInstrEnvelope *AssignSpenoloEnv ( struct TuComposerSong *SongStr, struct TuComposerInstr *InstrStr, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, struct TuComposerInstrEnvelope **EnvStr, struct Envelope **PlayEnvStr );
static struct TuComposerInstrEnvelope *AssignTTremoloEnv ( struct TuComposerSong *SongStr, struct TuComposerInstr *InstrStr, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, struct TuComposerInstrEnvelope **EnvStr, struct Envelope **PlayEnvStr );
static struct TuComposerInstrEnvelope *AssignTPannoloEnv ( struct TuComposerSong *SongStr, struct TuComposerInstr *InstrStr, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, struct TuComposerInstrEnvelope **EnvStr, struct Envelope **PlayEnvStr );
static struct TuComposerInstrEnvelope *AssignGTremoloEnv ( struct TuComposerSong *SongStr, struct TuComposerInstr *InstrStr, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, struct TuComposerInstrEnvelope **EnvStr, struct Envelope **PlayEnvStr );
static struct TuComposerInstrEnvelope *AssignGPannoloEnv ( struct TuComposerSong *SongStr, struct TuComposerInstr *InstrStr, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, struct TuComposerInstrEnvelope **EnvStr, struct Envelope **PlayEnvStr );

static struct TuComposerInstrEnvelope *AssignAutoVibEnv ( struct TuComposerSample *SampleStr, struct PlayerChannel *PCStr, struct Envelope **PlayEnvStr );
static struct TuComposerInstrEnvelope *AssignAutoTremEnv ( struct TuComposerSample *SampleStr, struct PlayerChannel *PCStr, struct Envelope **PlayEnvStr );
static struct TuComposerInstrEnvelope *AssignAutoPanEnv ( struct TuComposerSample *SampleStr, struct PlayerChannel *PCStr, struct Envelope **PlayEnvStr );

static struct Envelope *UseVolEnv ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr );
static struct Envelope *UsePanEnv ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr );
static struct Envelope *UseSlideEnv ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr );
static struct Envelope *UseVibEnv ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr );
static struct Envelope *UseTremoEnv ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr );
static struct Envelope *UsePannoEnv ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr );
static struct Envelope *UseChanEnv ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr );
static struct Envelope *UseSpenEnv ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr );
static struct Envelope *UseAutoVibEnv ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr );
static struct Envelope *UseAutoTremEnv ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr );
static struct Envelope *UseAutoPanEnv ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr );
static struct Envelope *UseTrkTremEnv ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr );
static struct Envelope *UseTrkPanEnv ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr );
static struct Envelope *UseGlobTremEnv ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr );
static struct Envelope *UseGlobPanEnv ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr );
static struct Envelope *UseArpEnv ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr );

static ULONG LinearSlideUp ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, ULONG Frequency, ULONG SlideValue );
static ULONG AmigaSlideUp ( struct PlayerChannel *PCStr, ULONG Frequency, ULONG SlideValue );

static ULONG LinearSlideDn ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, ULONG Frequency, ULONG SlideValue );
static ULONG AmigaSlideDn ( struct PlayerChannel *PCStr, ULONG Frequency, ULONG SlideValue );

static void PortaSlideUp ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, ULONG DataWord, ULONG CarryAdd, ULONG PortaShift, UWORD Channel );
static void PortaSlideDn ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, ULONG DataWord, ULONG CarryAdd, ULONG PortaShift, UWORD Channel );
static void PortaUpOk ( struct PlayerHostChannel *PHCStr, UWORD DataWord );
static void PortaDnOk ( struct PlayerHostChannel *PHCStr, UWORD DataWord );
static void PortaUpOnceOk ( struct PlayerHostChannel *PHCStr, UWORD DataWord );
static void PortaDnOnceOk ( struct PlayerHostChannel *PHCStr, UWORD DataWord );

static void DoVibrato ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD VibratoRate, WORD VibratoDepth );

static ULONG CheckOldVolume ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD *DataWord, UWORD Channel );
static void DoVolumeSlide ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD DataWord, UWORD Channel );
static void DoVolumeSlideDn ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD DataWord, UWORD Channel );
static void VolSlUpOk ( struct PlayerHostChannel *PHCStr, UWORD DataWord );
static void VolSlDnOk ( struct PlayerHostChannel *PHCStr, UWORD DataWord );
static void FVolSlUpOk ( struct PlayerHostChannel *PHCStr, UWORD DataWord );
static void FVolSlDnOk ( struct PlayerHostChannel *PHCStr, UWORD DataWord );

static ULONG CheckOldTVol ( struct TuComposerBase *TuCBase, UWORD *DataWord );
static void DoTVolumeSlide ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, UWORD DataWord );
static void DoTVolumeSlideDn ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, UWORD DataWord );

static void DoPanSlide ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD DataWord, UWORD Channel );
static void DoPanSlideRight ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD DataWord, UWORD Channel );

static void DoTPanSlide ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, UWORD DataWord );
static void DoTPanSlideRight ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, UWORD DataWord );

static ULONG CheckSurTPan ( struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UBYTE ChCtrlByte );

static UWORD *GetSpeedAdr ( struct TuComposerSong *SongStr, UWORD SpeedType, UWORD *SpeedLowerLimit, UWORD *SpeedUpperLimit );
static void SpeedValOk ( struct TuComposerBase *TuCBase, UWORD *SpeedAdr, UWORD SpeedValue, UBYTE SpeedType, UWORD SpeedLowerLimit, UWORD SpeedUpperLimit );
static void DoSpeedSlide ( struct TuComposerBase *TuCBase, UWORD DataWord );
static void DoSpeedSlideSlow ( struct TuComposerBase *TuCBase, UWORD DataWord );

static void DoGVolumeSlide ( struct TuComposerBase *TuCBase, struct PlayerGlobals *PGStr, UWORD DataWord );
static void DoGVolumeSlideDn ( struct TuComposerBase *TuCBase, struct PlayerGlobals *PGStr, UWORD DataWord );

static void DoGPanSlide ( struct PlayerGlobals *PGStr, UWORD DataWord );
static void DoGPanSlideRight ( struct PlayerGlobals *PGStr, UWORD DataWord );

static void PresetTonePorta ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD DataWord );
static void PresetVibrato ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD DataWord );
static void PresetNoteDelay ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD DataWord );
static void PresetTremolo ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD DataWord );
static void PresetSetTransp ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD DataWord );

static void CheckPorta ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD *EffectByte, UWORD *DataWord, UWORD *Flags );
static void CheckTonePorta ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD *EffectByte, UWORD *DataWord, UWORD *Flags );
static void CheckNoteSlide ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD *EffectByte, UWORD *DataWord, UWORD *Flags );
static void CheckVolSlide ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD *EffectByte, UWORD *DataWord, UWORD *Flags );
static void CheckVolSlTo ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD *EffectByte, UWORD *DataWord, UWORD *Flags );
static void CheckTVolSlide ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD *EffectByte, UWORD *DataWord, UWORD *Flags );
static void CheckPanSlide ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD *EffectByte, UWORD *DataWord, UWORD *Flags );
static void CheckTPanSlide ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD *EffectByte, UWORD *DataWord, UWORD *Flags );
static void CheckSpeedSlide ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD *EffectByte, UWORD *DataWord, UWORD *Flags );
static void CheckChCtrl ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD *EffectByte, UWORD *DataWord, UWORD *Flags );
static void CheckGVolSlide ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD *EffectByte, UWORD *DataWord, UWORD *Flags );
static void CheckGPanSlide ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD *EffectByte, UWORD *DataWord, UWORD *Flags );

static void Arpeggio ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void PortaUp ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void PortaDn ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void FPortaUp ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void FPortaDn ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void PortaUpOnce ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void PortaDnOnce ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void FPortaUpOnce ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void FPortaDnOnce ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void TonePorta ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void FTonePorta ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void TPortaOnce ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void FTPortaOnce ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void NoteSlide ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void Vibrato ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void FVibrato ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void DoKeyOff ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void HoldDelay ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void NoteFade ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void NoteCut ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void NoteDelay ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void Tremor ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void Retrig ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void MulRetrig ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void ExtCtrl ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void InvLoop ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void ExecFX ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void StopFX ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );

static void SetVol ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void VolSlUp ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void VolSlDn ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void FVolSlUp ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void FVolSlDn ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void VolSlTo ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void Tremo ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void SetTVol ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void TVolSlUp ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void TVolSlDn ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void FTVolSlUp ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void FTVolSlDn ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void TVolSlTo ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void TTremo ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );

static void SetPan ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void PanSlLeft ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void PanSlRight ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void FPanSlLeft ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void FPanSlRight ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void PanSlTo ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void Pannolo ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void SetTPan ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void TPanSlLeft ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void TPanSlRight ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void FTPanSlLeft ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void FTPanSlRight ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void TPanSlTo ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void TPannolo ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );

static void SetTempo ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void SetRelTempo ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void PattBrk ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void PosJump ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void RelPosJmp ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void ChgPattern ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void RevPattPlay ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void PattDelay ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void FPattDelay ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void PattLoop ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void GoSub ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void Return ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void ChanSync ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void SetSubSl ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );

static void SmpOffHI ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void SmpOffLO ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void SetHold ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void SetDecay ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void SetTransp ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void InsCtrl ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void InsChg ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void SynthCtrl ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void SetSynthVal ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void EnvCtrl ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void SetEnvVal ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void NNACtrl ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void LoopCtrl ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );

static void SetSpeed ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void SpdSlFast ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void SpdSlSlow ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void FSpdSlFast ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void FSpdSlSlow ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void SpdSlTo ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void Spenolo ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void ChCtrl ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void SetGVol ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void GVolSlUp ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void GVolSlDn ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void FGVolSlUp ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void FGVolSlDn ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void GVolSlTo ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void GTremo ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void SetGPan ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void GPanSlLeft ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void GPanSlRight ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void FGPanSlLeft ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void FGPanSlRight ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void GPanSlTo ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );
static void GPannolo ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );

static void UserSync ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord );

static ULONG ExecuteSynth ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, ULONG Channel, ULONG SynthType );

static void SE_VIBRATO_Do ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, LONG VibratoSlideValue );
static void SE_ARPEGIO_Do ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, WORD ArpeggioTranspose, UWORD ArpeggioFineTune );
static void SE_TREMOLO_Do ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, LONG TremoloSlideValue );
static void SE_PANNOLO_Do ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, LONG PannoloSlideValue );

static UWORD SE_STOP ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_KILL ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_WAIT ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_WAITVOL ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_WAITPAN ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_WAITSLD ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_WAITSPC ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_JUMP ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_JUMPEQ ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_JUMPNE ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_JUMPPL ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_JUMPMI ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_JUMPLT ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_JUMPLE ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_JUMPGT ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_JUMPGE ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_JUMPVS ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_JUMPVC ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_JUMPCS ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_JUMPCC ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_JUMPLS ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_JUMPHI ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_JUMPVOL ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_JUMPPAN ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_JUMPSLD ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_JUMPSPC ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_CALL ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_RETURN ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_POSVAR ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_LOAD ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_ADD ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_ADDX ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_SUB ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_SUBX ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_CMP ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_MULU ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_MULS ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_DMULU ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_DMULS ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_DIVU ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_DIVS ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_MODU ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_MODS ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_DDIVU ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_DDIVS ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_ASHL ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_ASHR ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_LSHL ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_LSHR ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_ROL ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_ROR ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_ROLX ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_RORX ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_OR ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_AND ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_XOR ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_NOT ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_NEG ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_NEGX ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_EXTB ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_EXT ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_XCHG ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_SWAP ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_GETWAVE ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_GETWLEN ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_GETWPOS ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_GETCHAN ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_GETNOTE ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_GETRANS ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_GETPTCH ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_GETPER ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_GETFX ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_GETARPW ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_GETARPV ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_GETARPL ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_GETARPP ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_GETVIBW ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_GETVIBV ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_GETVIBL ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_GETVIBP ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_GETTRMW ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_GETTRMV ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_GETTRML ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_GETTRMP ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_GETPANW ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_GETPANV ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_GETPANL ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_GETPANP ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_GETRND ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_GETSINE ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_PORTAUP ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_PORTADN ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_VIBSPD ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_VIBDPTH ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_VIBWAVE ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_VIBWAVP ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_VIBRATO ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_VIBVAL ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_ARPSPD ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_ARPWAVE ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_ARPWAVP ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_ARPEGIO ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_ARPVAL ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_SETWAVE ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_ISETWAV ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_SETWAVP ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_SETRANS ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_SETNOTE ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_SETPTCH ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_SETPER ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_RESET ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_VOLSLUP ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_VOLSLDN ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_TRMSPD ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_TRMDPTH ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_TRMWAVE ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_TRMWAVP ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_TREMOLO ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_TRMVAL ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_PANLEFT ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_PANRGHT ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_PANSPD ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_PANDPTH ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_PANWAVE ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_PANWAVP ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_PANNOLO ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_PANVAL ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );
static UWORD SE_NOP ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );

#ifdef __cplusplus
}
#endif

static APTR AssignEnvTab[] = {
  AssignVolumeEnv,
  AssignPanningEnv,
  AssignSlideEnv,
  AssignVibratoEnv,
  AssignTremoloEnv,
  AssignPannoloEnv,
  AssignChannoloEnv,
  AssignSpenoloEnv,
  AssignTTremoloEnv,
  AssignTPannoloEnv,
  AssignGTremoloEnv,
  AssignGPannoloEnv
};

static APTR AssignAutoEnvTab[] = {
  AssignAutoVibEnv,
  AssignAutoTremEnv,
  AssignAutoPanEnv
};

static APTR EnvCtrlTypeTab[] = {
  UseVolEnv,
  UsePanEnv,
  UseSlideEnv,
  UseVibEnv,
  UseTremoEnv,
  UsePannoEnv,
  UseChanEnv,
  UseSpenEnv,
  UseAutoVibEnv,
  UseAutoTremEnv,
  UseAutoPanEnv,
  UseTrkTremEnv,
  UseTrkPanEnv,
  UseGlobTremEnv,
  UseGlobPanEnv,
  UseArpEnv
};

static UBYTE Msg[] = "Kristina Kratochvilova, I love you !!! Signed, Basty/CDGS.\0";

static BYTE EmptyWaveform[256];

void DoPlayerIRQ ( struct TuComposerBase *TuCBase )
{
  struct TuComposerModule *ModuleStr;
  struct TuComposerSong *SongStr;
  struct PlayerGlobals *PGStr;
  struct PlayerHostChannel *PHCStr;
  struct PlayerChannel *PCStr;
  struct MixerAllocStruct *MixStr = TuCBase->tcb_PlayMixerAlloc;
  struct PlayExecHandler *ExecList;
  ULONG PlayTimeCalc, PlayTimeAdv, PlayTimeFrac;
  UWORD Channel, VirtualCh;

  if (!( ModuleStr = TuCBase->tcb_PlayModule ))
    return;

  Channel = 0;
  PCStr = ModuleStr->tcm_ChannelData;

  do
  {
    ExternalMixerGetChannelBase ( MixStr, (struct MixerStruct *) &(PCStr->pc_ChannelData), Channel, MixStr->mas_MixLibBase );

    PCStr++;
  } while ( ++Channel < ModuleStr->tcm_Channels );

  SongStr = TuCBase->tcb_PlaySong;
  PGStr = SongStr->tcso_GlobalData;

  if ( PGStr->pg_Flags & PGF_TraceMode )
  {
    if (!( PGStr->pg_TraceCount-- ))
      PGStr->pg_TraceCount = 0;

    return;
  }

  ExecList = (struct PlayExecHandler *) &(TuCBase->tcb_ExecList.mlh_Head);

  for (;;)
  {
    struct Task *CurrTask;

    ExecList = (struct PlayExecHandler *) ExecList->peh_Node.mln_Succ;

    if (!( ExecList->peh_Node.mln_Succ ))
      break;

    if ( (ExecList->peh_Flags & PEHF_Beginning) && ( CurrTask = ExecList->peh_Action.peh_Task ) )
    {
      if ( (ExecList->peh_Flags & PEHF_SongEnd) && (!( PGStr->pg_Flags & PGF_SongEnd )) )
        continue;

      if ( ExecList->peh_SigBit != 255 )
      {
        Signal ( CurrTask, 1UL<<ExecList->peh_SigBit );
      } else
      {
        ExecList->peh_Action.peh_PlayExecCode ( TuCBase, ModuleStr, SongStr, ExecList->peh_Data );
      }
    }
  }

  if ( PGStr->pg_PlayType & PTYPF_Song )
  {
    PlayTimeCalc = ((unsigned long long) PGStr->pg_Tempo * PGStr->pg_RelativeSpeed) >> 16UL;
    PlayTimeAdv = (65536UL*1000UL) / PlayTimeCalc;
    PlayTimeFrac = ((unsigned long long) ((65536UL*1000UL) % PlayTimeCalc) << 32ULL) / PlayTimeCalc;

    PGStr->pg_PlayTimeFrac += PlayTimeFrac;

    if ( PGStr->pg_PlayTimeFrac < PlayTimeFrac )
      PlayTimeAdv++;

    PGStr->pg_PlayTime += PlayTimeAdv;

    PlayTimeCalc = PGStr->pg_Tempo;
    PlayTimeAdv = (65536UL*1000UL) / PlayTimeCalc;
    PlayTimeFrac = ((unsigned long long) ((65536UL*1000UL) % PlayTimeCalc) << 32ULL) / PlayTimeCalc;

    PGStr->pg_PlayTicsFrac += PlayTimeFrac;

    if ( PGStr->pg_PlayTicsFrac < PlayTimeFrac )
      PlayTimeAdv++;

    PGStr->pg_PlayTics += PlayTimeAdv;
  }

  Channel = 0;
  PHCStr = SongStr->tcso_ChannelData;

  do
  {
    PCStr = ModuleStr->tcm_ChannelData + PHCStr->phc_VirtualChannel;

    if ( (PHCStr->phc_Flags & PHCF_SetInstr) && (PHCStr->phc_Flags & PHCF_SetSample) )
    {
      struct TuComposerTrack *OldTrackStr = PHCStr->phc_Track;
      struct TuComposerTrackEffect *OldFXStr = PHCStr->phc_Effect;
      ULONG OldTempoCounter = PHCStr->phc_TempoCounter;
      UWORD OldRow = PHCStr->phc_Row;

      PHCStr->phc_Flags &= ~(PHCF_SetInstr|PHCF_SetSample);

      PHCStr->phc_Track = (struct TuComposerTrack *) PHCStr->phc_Instrument;
      PHCStr->phc_Effect = NULL;
      PHCStr->phc_Row = *(ULONG *) &(PHCStr->phc_Sample);

      PHCStr->phc_Instrument = NULL;
      PHCStr->phc_Sample = NULL;

      GetEffects ( TuCBase, ModuleStr, SongStr, PHCStr, PCStr, Channel );

      PHCStr->phc_TempoCounter = PHCStr->phc_NoteDelay;

      GetNote ( TuCBase, ModuleStr, SongStr, PHCStr, PCStr, Channel );
      RunEffects ( TuCBase, ModuleStr, SongStr, PHCStr, PCStr, Channel );

      PHCStr->phc_Track = OldTrackStr;
      PHCStr->phc_Effect = OldFXStr;
      PHCStr->phc_TempoCounter = OldTempoCounter;
      PHCStr->phc_Row = OldRow;
    }

    if ( PHCStr->phc_Flags & PHCF_SetInstr )
    {
      UWORD Note = (UBYTE) PHCStr->phc_InstrNote;

      PHCStr->phc_Flags &= ~PHCF_SetInstr;

      if ( (WORD) Note < 0 )
      {
        switch ( Note )
        {
          case NOTE_FADE :
          {
            PCStr->pc_Flags |= PCF_Fading;

            break;
          }
          case NOTE_HOLD_DELAY :
          {
            break;
          }
          case NOTE_KEYOFF :
          {
            PlayKeyOff ( PCStr );

            break;
          }
          case NOTE_OFF :
          {
            PCStr->pc_Volume = 0;

            break;
          }
          case NOTE_KILL :
          {
            PHCStr->phc_Instrument = NULL;
            PHCStr->phc_Sample = NULL;
            PHCStr->phc_InstrNote = 0;

            if ( PCStr->pc_HostChannel == Channel )
              PCStr->pc_ChannelData.ms_Flags = 0;

            break;
          }
        }
      } else
      {
        struct TuComposerInstr *InstrStr = PHCStr->phc_Instrument;
        struct TuComposerSample *SampleStr;
        struct PlayerChannel *NewPCStr;

        if ( (NewPCStr = PlayNote ( TuCBase, ModuleStr, InstrStr, PHCStr, PCStr, Note / NUMNOTES, Note % NUMNOTES, Channel )))
          PCStr = NewPCStr;

        SampleStr = PHCStr->phc_Sample;

        PCStr->pc_Volume = PHCStr->phc_SampleNote;
        PCStr->pc_SubVol = 0;

        InitNewInstr ( TuCBase, SongStr, PHCStr, PCStr );
        InitNewSample ( TuCBase, PHCStr, PCStr );
      }
    }

    if ( PHCStr->phc_Flags & PHCF_SetSample )
    {
      struct TuComposerInstr *InstrStr;
      struct TuComposerSample *SampleStr = PHCStr->phc_Sample;
      ULONG Frequency = *(ULONG *) &(PHCStr->phc_Instrument);
      UWORD VirtualCh;

      PHCStr->phc_Flags &= ~PHCF_SetSample;

      PHCStr->phc_DCT = 0;
      PHCStr->phc_NNA = NNA_NOTE_CUT;

      PHCStr->phc_FineTune = SampleStr->tcs_FineTune;

      PHCStr->phc_PrevAutoVibEnv = PCStr->pc_AutoVibEnv.pes_EnvelopeStructure;
      PHCStr->phc_PrevAutoTremEnv = PCStr->pc_AutoTremoEnv.pes_EnvelopeStructure;
      PHCStr->phc_PrevAutoPanEnv = PCStr->pc_AutoPanEnv.pes_EnvelopeStructure;

      PCStr = TriggerNNA ( ModuleStr, PHCStr, PCStr, Channel, (UWORD *) &VirtualCh );

      PCStr->pc_ChannelData.ms_SamplePos = SampleStr->tcs_StartOffset;

      PHCStr->phc_VirtualChannel = VirtualCh;
      PCStr->pc_HostChannel = Channel;

      PHCStr->phc_Instrument = NULL;
      PCStr->pc_Sample = SampleStr;

      PCStr->pc_Frequency = Frequency;

      PCStr->pc_Volume = PHCStr->phc_InstrNote;
      PCStr->pc_SubVol = 0;

      PHCStr->phc_InstrNote = 0;

      InitNewInstr ( TuCBase, SongStr, PHCStr, PCStr );

      InstrStr = (struct TuComposerInstr *) &(ModuleStr->tcm_InstrList.mlh_Head);

      for (;;)
      {
        InstrStr = (struct TuComposerInstr *) InstrStr->tci_Node.mln_Succ;

        if (!( InstrStr->tci_Node.mln_Succ ))
          break;

        SampleStr = (struct TuComposerSample *) &(InstrStr->tci_Samples.mlh_Head);

        for (;;)
        {
          SampleStr = (struct TuComposerSample *) SampleStr->tcs_Node.mln_Succ;

          if (!( SampleStr->tcs_Node.mln_Succ ))
            break;

          if ( SampleStr == PCStr->pc_Sample )
          {
            PHCStr->phc_Instrument = InstrStr;

            goto InstrFound;
          }
        }
      }

InstrFound:

      PCStr->pc_Instrument = PHCStr->phc_Instrument;

      InitNewSample ( TuCBase, PHCStr, PCStr );
    }

    if ( (!( PGStr->pg_Flags & PGF_NoProcPattern )) && PHCStr->phc_Tempo )
    {
ReScanRow:

      ProcessRow ( SongStr, PHCStr, PCStr, Channel );
      GetEffects ( TuCBase, ModuleStr, SongStr, PHCStr, PCStr, Channel );

      if ( PCStr->pc_HostChannel == Channel )
      {
        if (!( PHCStr->phc_Flags & PHCF_Vibrato ))
        {
          ULONG SlideValue = PHCStr->phc_VibratoSlideValue;

          PHCStr->phc_VibratoSlideValue = 0L;
          PCStr->pc_Frequency -= SlideValue;
        }

        if (!( PHCStr->phc_Flags & PHCF_Tremolo ))
        {
          WORD SlideValue = PHCStr->phc_TremoloSlideValue;

          PHCStr->phc_TremoloSlideValue = 0;

          if ( (SlideValue = ( PCStr->pc_Volume - SlideValue )) < 0 )
            SlideValue = 0;

          if ( (UWORD) SlideValue >= 255 )
            SlideValue = -1;

          PCStr->pc_Volume = SlideValue;
        }
      }

      if ( GetNote ( TuCBase, ModuleStr, SongStr, PHCStr, PCStr, Channel ))
        goto ReScanRow;
    }

    PHCStr->phc_NumVirtual = 0;

    PHCStr++;
  } while ( ++Channel < SongStr->tcso_Channels );

  Channel = 0;
  PHCStr = SongStr->tcso_ChannelData;

  do
  {
    if ( (!( PGStr->pg_Flags & PGF_NoProcPattern )) && PHCStr->phc_Tempo )
    {
      PCStr = ModuleStr->tcm_ChannelData + PHCStr->phc_VirtualChannel;

      RunEffects ( TuCBase, ModuleStr, SongStr, PHCStr, PCStr, Channel );
    }

    PHCStr->phc_NumVirtual = 0;

    PHCStr++;
  } while ( ++Channel < SongStr->tcso_Channels );

  VirtualCh = 0;
  Channel = 0;
  PCStr = ModuleStr->tcm_ChannelData;

  do
  {
    struct Envelope *PlayEnvStr;

    if ( PCStr->pc_Flags & PCF_Allocated )
      PCStr->pc_ChannelData.ms_Flags &= ~MSF_Play;

    if ( PCStr->pc_ChannelData.ms_Flags & MSF_Play )
    {
      struct TuComposerSample *SampleStr;
      ULONG Frequency, HostVol, VirtVol;
      ULONG AutoVibDepth, AutoVibCount;
      UWORD Flags, SlideEnvVal;
      WORD AutoVibVal, Pan, AbsPan, PanEnvVal;

      PHCStr = SongStr->tcso_ChannelData + PCStr->pc_HostChannel;

      PlayEnvStr = (struct Envelope *) &(PCStr->pc_VolEnvelope);

      if ( PlayEnvStr->pes_EnvelopeTempo )
      {
        UWORD Volume;

        Volume = RunEnvelope ( TuCBase, PlayEnvStr, 1, 0x8000 );

        if (!( PlayEnvStr->pes_EnvelopeTempo ))
        {
          if (!( Volume >> 8U ))
            goto TurnNoteOff;

          PCStr->pc_Flags |= PCF_Fading;
        }
      }

      RunEnvelope ( TuCBase, (struct Envelope *) &(PCStr->pc_PanEnvelope), 1, 0 );
      SlideEnvVal = RunEnvelope ( TuCBase, (struct Envelope *) &(PCStr->pc_SlideEnvelope), 1, 0 );

      if ( PCStr->pc_Flags & PCF_PortaSlideEnv )
      {
        ULONG OldFreq = PCStr->pc_Frequency;

        PCStr->pc_Frequency += PCStr->pc_SlideEnvelopeFrequency;

        if ( (Frequency = PCStr->pc_Frequency) )
        {
          if ( (WORD) SlideEnvVal < 0 )
          {
            SlideEnvVal = -SlideEnvVal;

            if ( PCStr->pc_Flags & PCF_LinearSlideEnv )
            {
              Frequency = LinearSlideDn ( TuCBase, PCStr, Frequency, SlideEnvVal );
            } else
            {
              Frequency = AmigaSlideDn ( PCStr, Frequency, SlideEnvVal );
            }
          } else
          {
            if ( PCStr->pc_Flags & PCF_LinearSlideEnv )
            {
              Frequency = LinearSlideUp ( TuCBase, PCStr, Frequency, SlideEnvVal );
            } else
            {
              Frequency = AmigaSlideUp ( PCStr, Frequency, SlideEnvVal );
            }
          }

          OldFreq -= Frequency;

          PCStr->pc_SlideEnvelopeFrequency += OldFreq;
        }
      } else
      {
        ULONG *FreqTable;
        ULONG Frequency, NextFreq, SlideEnvFreq, OldFreq;
        WORD Octave, Note;
        WORD SlideNote = (WORD) SlideEnvVal >> 8;
        WORD FineTune = SlideEnvVal & 0xFF;

        Octave = SlideNote / NUMNOTES;
        Note = SlideNote % NUMNOTES;

        if ( Note < 0 )
        {
          Octave--;
          Note += NUMNOTES;

          FineTune = -FineTune;
        }

        FreqTable = TuCBase->tcb_FrequencyTable + Note;

        Frequency = *FreqTable++;
        NextFreq = *FreqTable - Frequency;
        Frequency += (LONG) (FineTune * (WORD) NextFreq) >> 8;

        if ( (WORD) ( Octave ) < 0 )
        {
          Octave = -Octave;
          Frequency >>= Octave;
        } else
        {
          Frequency <<= Octave;
        }

        SlideEnvFreq = PCStr->pc_SlideEnvelopeFrequency;
        OldFreq = PCStr->pc_Frequency;

        SlideEnvFreq += OldFreq;

        PCStr->pc_Frequency = Frequency = ((unsigned long long) Frequency * SlideEnvFreq) >> 16ULL;

        PCStr->pc_SlideEnvelopeFrequency += OldFreq - Frequency;
      }

      if ( PCStr->pc_Flags & PCF_Fading )
      {
        LONG FadeOut = (ULONG) PCStr->pc_FadeOutCount;

        if ( ( FadeOut -= PCStr->pc_FadeOut ) <= 0 )
          goto TurnNoteOff;

        PCStr->pc_FadeOutCount = FadeOut;
      }

      AutoVibVal = RunEnvelope ( TuCBase, (struct Envelope *) &(PCStr->pc_AutoVibEnv), PCStr->pc_AutoVibRate, 0 );
      AutoVibDepth = PCStr->pc_AutoVibDepth << 8UL;
      AutoVibCount = PCStr->pc_AutoVibCount;

      AutoVibCount += PCStr->pc_AutoVibSweep;

      if ( AutoVibCount > AutoVibDepth )
        AutoVibCount = AutoVibDepth;

      PCStr->pc_AutoVibCount = AutoVibCount;

      AutoVibCount >>= 8U;

      if ( ( AutoVibVal *= (WORD) -AutoVibCount ) )
      {
        ULONG OldFreq = PCStr->pc_Frequency;

        AutoVibVal >>= 7-2;

        PCStr->pc_Frequency -= PCStr->pc_AutoVibFrequency;

        if ( (Frequency = PCStr->pc_Frequency) )
        {
          if ( (WORD) AutoVibVal < 0 )
          {
            AutoVibVal = -AutoVibVal;

            if ( PCStr->pc_Flags & PCF_LinearSlideEnv )
            {
              Frequency = LinearSlideUp ( TuCBase, PCStr, Frequency, AutoVibVal );
            } else
            {
              Frequency = AmigaSlideUp ( PCStr, Frequency, AutoVibVal );
            }
          } else
          {
            if ( PCStr->pc_Flags & PCF_LinearSlideEnv )
            {
              Frequency = LinearSlideDn ( TuCBase, PCStr, Frequency, AutoVibVal );
            } else
            {
              Frequency = AmigaSlideDn ( PCStr, Frequency, AutoVibVal );
            }
          }

          OldFreq -= Frequency;

          PCStr->pc_AutoVibFrequency -= OldFreq;
        }
      }

      SampleStr = PCStr->pc_Sample;

      if ( SampleStr->tcs_Synth )
      {
        if (!( ExecuteSynth ( TuCBase, PHCStr, PCStr, Channel, 0L )))
          goto TurnNoteOff;

        if (!( ExecuteSynth ( TuCBase, PHCStr, PCStr, Channel, 1L )))
          goto TurnNoteOff;

        if (!( ExecuteSynth ( TuCBase, PHCStr, PCStr, Channel, 2L )))
          goto TurnNoteOff;

        if (!( ExecuteSynth ( TuCBase, PHCStr, PCStr, Channel, 3L )))
          goto TurnNoteOff;
      }

      if ( ((!( PCStr->pc_ChannelData.ms_SampleData )) || (!( PCStr->pc_ChannelData.ms_BitDepth ))) && ( PCStr->pc_ChannelData.ms_Flags & MSF_Play ))
      {
        PCStr->pc_ChannelData.ms_SamplePos = 0L;
        PCStr->pc_ChannelData.ms_SampleLen = (sizeof (EmptyWaveform) / sizeof (EmptyWaveform[0]));
        PCStr->pc_ChannelData.ms_SampleData = (WORD *) &(EmptyWaveform);
        PCStr->pc_ChannelData.ms_Repeat = 0L;
        PCStr->pc_ChannelData.ms_RepLen = (sizeof (EmptyWaveform) / sizeof (EmptyWaveform[0]));
        PCStr->pc_ChannelData.ms_RepCount = 0L;
        PCStr->pc_ChannelData.ms_RepCounted = 0L;
        PCStr->pc_ChannelData.ms_BitDepth = (sizeof (EmptyWaveform[0]) * 8);
        PCStr->pc_ChannelData.ms_Flags = MSF_Loop|MSF_Play;
      }

      Frequency = PCStr->pc_Frequency;

      if ( Frequency < SampleStr->tcs_LowerRateLimit )
        Frequency = SampleStr->tcs_LowerRateLimit;

      if ( Frequency > SampleStr->tcs_UpperRateLimit )
        Frequency = SampleStr->tcs_UpperRateLimit;

      if (!( PCStr->pc_Frequency = Frequency ))
      {

TurnNoteOff:

        PCStr->pc_ChannelData.ms_Flags = 0;

        goto NotCalcNoPlaying;
      }

      if (!( PCStr->pc_ChannelData.ms_SampleRate = ((unsigned long long) Frequency * PGStr->pg_RelativePitch) >> 16ULL ))
        goto TurnNoteOff;

      if (!( SongStr->tcso_CompFlags & TCSOCF_GlobalNewOnly ))
      {
        PCStr->pc_GlobalVolume = PGStr->pg_GlobalVolume;
        PCStr->pc_GlobalSubVol = PGStr->pg_GlobalSubVol;

        PCStr->pc_GlobalPanning = PGStr->pg_GlobalPanning;
        PCStr->pc_GlobalSubPan = PGStr->pg_GlobalSubPan;
      }

      HostVol = PCStr->pc_Volume;

      PHCStr->phc_NumVirtual++;
      VirtualCh++;

      if ( (!( PCStr->pc_Flags & PCF_Background )) && ( PHCStr->phc_VirtualChannel == Channel ) && ( PHCStr->phc_Flags & PHCF_TremorExec ) && ( PHCStr->phc_Flags & PHCF_TremorOff ) )
        HostVol = 0;

      HostVol *= (UWORD) PHCStr->phc_TrackVolume * (UWORD) PCStr->pc_InstrVolume;
      VirtVol = (((UWORD) PCStr->pc_VolEnvelope.pes_EnvelopeValue >> 8U) * (UWORD) PCStr->pc_GlobalVolume) * (UWORD) PCStr->pc_FadeOutCount;

      PCStr->pc_ChannelData.ms_Volume = PCStr->pc_FinalVolume = ((unsigned long long) HostVol * VirtVol) / (255ULL*255ULL*255ULL*255ULL*65535ULl*255ULL);

      Flags = 0;

      PCStr->pc_Flags &= ~PCF_Surround;
      PCStr->pc_ChannelData.ms_Flags &= ~PCF_Surround;

      if ( PCStr->pc_Flags & PCF_SmpSurPan )
        Flags = PCF_Surround;

      Pan = PCStr->pc_Panning;

      if ( PCStr->pc_Flags & PCF_TrackPan )
      {
        Pan = PHCStr->phc_TrackPanning;
        Flags = 0;

        if ( (PHCStr->phc_Flags & PHCF_TrackSurPan) || (PHCStr->phc_Flags & PHCF_ChannelSurPan) )
          Flags = PCF_Surround;
      }

      PCStr->pc_Flags |= Flags;

      if (!( SongStr->tcso_Flags & TCSOF_Mono ))
        PCStr->pc_ChannelData.ms_Flags |= Flags;

      if ( Pan == 255 )
        Pan++;

      PanEnvVal = Pan;

      if ( (WORD) ( Pan = (128 - Pan) ) < 0 )
        Pan = -Pan;

      AbsPan = 128 - Pan;

      Pan = PCStr->pc_PanEnvelope.pes_EnvelopeValue >> 8;

      if ( Pan == 127 )
        Pan++;

      Pan = 128 - (((Pan * AbsPan) >> 7) + PanEnvVal);

      AbsPan = (UWORD) PHCStr->phc_ChannelPanning;

      if ( AbsPan == 255 )
        AbsPan++;

      AbsPan -= 128;
      PanEnvVal = AbsPan = ((Pan * AbsPan) >> 7) + 128;

      if ( PanEnvVal > 255 )
        PanEnvVal = 255;

      PCStr->pc_FinalPanning = PanEnvVal;

      Pan = 128;

      if (!( SongStr->tcso_Flags & TCSOF_Mono ))
      {
        if ( PCStr->pc_Flags & PCF_GlobalSurPan )
          PCStr->pc_ChannelData.ms_Flags |= MSF_Surround;

        Pan -= AbsPan;

        AbsPan = (UWORD) PCStr->pc_GlobalPanning;

        if ( AbsPan == 255 )
          AbsPan++;

        AbsPan -= 128;
        Pan = ((Pan * AbsPan) >> 7) + 128;

        if ( Pan == 256 )
          Pan--;
      }

      PCStr->pc_ChannelData.ms_Panning = Pan;

      ExternalMixerSetChannelVolPanPitchBase ( MixStr, (struct MixerStruct *) &(PCStr->pc_ChannelData), Channel, MixStr->mas_MixLibBase );
    }

NotCalcNoPlaying:

    ExternalMixerSetChannelPosRepFlagsBase ( MixStr, (struct MixerStruct *) &(PCStr->pc_ChannelData), Channel, MixStr->mas_MixLibBase );

    PCStr++;
  } while ( ++Channel < ModuleStr->tcm_Channels );

  PGStr->pg_Channels = VirtualCh;

  if ( VirtualCh > PGStr->pg_MaxChannels )
    PGStr->pg_MaxChannels = VirtualCh;

  Channel = 0;
  PHCStr = SongStr->tcso_ChannelData;

  do
  {
    if (!( PHCStr->phc_Flags & PHCF_SongEnd ))
      goto CheckSongEndDone;

    PHCStr++;
  } while ( ++Channel < SongStr->tcso_Channels );

  PGStr->pg_Flags |= PGF_SongEnd;

CheckSongEndDone:

  ExecList = (struct PlayExecHandler *) &(TuCBase->tcb_ExecList.mlh_Head);

  for (;;)
  {
    struct Task *CurrTask;

    ExecList = (struct PlayExecHandler *) ExecList->peh_Node.mln_Succ;

    if (!( ExecList->peh_Node.mln_Succ ))
      break;

    if ( (!( ExecList->peh_Flags & PEHF_Beginning )) && ( CurrTask = ExecList->peh_Action.peh_Task ) )
    {
      if ( (ExecList->peh_Flags & PEHF_SongEnd) && (!( PGStr->pg_Flags & PGF_SongEnd )) )
        continue;

      if ( ExecList->peh_SigBit != 255 )
      {
        Signal ( CurrTask, 1UL<<ExecList->peh_SigBit );
      } else
      {
        ExecList->peh_Action.peh_PlayExecCode ( TuCBase, ModuleStr, SongStr, ExecList->peh_Data );
      }
    }
  }

  if ( PGStr->pg_Flags & PGF_SongEnd )
  {
    PHCStr = SongStr->tcso_ChannelData;

    if ( PHCStr->phc_Flags & PHCF_SongEnd )
    {
      struct TuComposerOrders *PosStr = SongStr->tcso_Position;
      Channel = SongStr->tcso_Channels;

      do
      {
        struct TuComposerOrderList *PosListStr = (struct TuComposerOrderList *) &(PosStr->tco_OrderData.mlh_Head);

        if ( PHCStr->phc_Tempo )
          PHCStr->phc_Flags &= ~PHCF_SongEnd;

        for (;;)
        {
          PosListStr = (struct TuComposerOrderList *) PosListStr->tcol_Node.mln_Succ;

          if (!( PosListStr->tcol_Node.mln_Succ ))
            break;

          if ( PosListStr != PHCStr->phc_Order )
            PosListStr->tcol_Played = 0;
        }

        PosStr++;
        PHCStr++;
      } while ( --Channel );
    }
  }
}

static void ProcessRow ( struct TuComposerSong *SongStr, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel )
{
  ULONG CurrTick;
  UWORD Counted = 0;

  PHCStr->phc_Flags &= ~PHCF_TremorExec;

  CurrTick = PHCStr->phc_TempoCounter;
  CurrTick++;

  if ( CurrTick >= ( PHCStr->phc_FinePatternDelay + PHCStr->phc_Tempo ) )
    CurrTick = 0L;

  if (!( PHCStr->phc_TempoCounter = CurrTick ))
  {
    struct TuComposerTrack *TrackStr;
    struct TuComposerOrders *PosStr;
    struct TuComposerOrderList *PosListStr;
    struct PlayerGlobals *PGStr = SongStr->tcso_GlobalData;
    ULONG *EffectsUsed;
    UWORD PattDelay, Row;
    UWORD TrackLen, LastRow;

    if ( PCStr->pc_HostChannel == Channel )
    {
      ULONG SlideValue = PHCStr->phc_ArpeggioFrequency;

      PHCStr->phc_ArpeggioFrequency = 0L;
      PCStr->pc_Frequency += SlideValue;
    }

    PHCStr->phc_Flags &= ~(PHCF_ExecFX|PHCF_TonePorta|PHCF_SetTranspose|PHCF_Vibrato|PHCF_Tremolo);

    EffectsUsed = (ULONG *) &(PHCStr->phc_EffectsUsed);

    *EffectsUsed++ = 0L;
    *EffectsUsed++ = 0L;
    *EffectsUsed++ = 0L;
    *EffectsUsed = 0L;

    PHCStr->phc_Effect = NULL;
    PHCStr->phc_ArpeggioTick = 0;
    PHCStr->phc_NoteDelay = 0;
    PHCStr->phc_RetrigTickCount = 0;

    if ( (PattDelay = PHCStr->phc_PatternDelay ))
    {
      if ( (PattDelay > PHCStr->phc_PatternDelayCount++) )
        return;
    }

    PHCStr->phc_PatternDelayCount = 0;
    PHCStr->phc_PatternDelay = 0;

    Row = PHCStr->phc_Row;

    if ( PHCStr->phc_Flags & PHCF_PattLoop )
    {
      PHCStr->phc_Flags &= ~PHCF_PattLoop;

      PosListStr = PHCStr->phc_Order;
      TrackStr = PHCStr->phc_Track;

      goto LoopToRow;
    }

    PHCStr->phc_Flags &= ~PHCF_PattLoopJmp;

    if ( PHCStr->phc_Flags & PHCF_ChgPattern )
    {
      ULONG i;

      PHCStr->phc_Flags &= ~PHCF_ChgPattern;

      PosListStr = PHCStr->phc_Order;
      TrackStr = (struct TuComposerTrack *) &(SongStr->tcso_TrackList.mlh_Head);

      i = PHCStr->phc_ChgPattern + 1;

      do
      {
        TrackStr = (struct TuComposerTrack *) TrackStr->tct_Node.mln_Succ;

        if (!( TrackStr->tct_Node.mln_Succ ))
          break;
      } while ( --i );

      if ( TrackStr->tct_Node.mln_Succ )
      {
        if (!( SongStr->tcso_GlobalData->pg_Flags & PGF_PlayPattern ))
          PHCStr->phc_Track = TrackStr;

        goto LoopToRow;
      }
    }

    if ( PHCStr->phc_Flags & PHCF_PatternBreak )
      goto GetNewPattern;

    if ( PHCStr->phc_Flags & PHCF_Backwards )
    {
      if (!( Row-- ))
        goto GetNewPattern;
    } else
    {
      if ( ( ++Row >= PHCStr->phc_MaxRow ) )
      {
GetNewPattern:

        PosListStr = PHCStr->phc_Order;

        if ( SongStr->tcso_GlobalData->pg_Flags & PGF_PlayPattern )
        {
          TrackStr = PHCStr->phc_Track;

          goto LoopToRow;
        }

CheckNextEmptyOrder:

        do
        {
          PosListStr = (struct TuComposerOrderList *) PosListStr->tcol_Node.mln_Succ;

          if (!( PosListStr->tcol_Node.mln_Succ ))
          {
            ULONG i;

SongEndFound:

            PHCStr->phc_Flags |= PHCF_SongEnd;

            PosStr = (struct TuComposerOrders *) SongStr->tcso_Position + Channel;
            PosListStr = (struct TuComposerOrderList *) &(PosStr->tco_OrderData.mlh_Head);

            i = PosStr->tco_RepStart + 1;

            do
            {
              PosListStr = (struct TuComposerOrderList *) PosListStr->tcol_Node.mln_Succ;

              if (!( PosListStr->tcol_Node.mln_Succ ))
              {
DisableChannel:

                PHCStr->phc_Flags |= PHCF_SongEnd;
                PHCStr->phc_Tempo = 0;

                return;
              }
            } while ( --i );

            if ( PosListStr->tcol_Flags & (TCOLF_EndOrder|TCOLF_EndSong) )
              goto DisableChannel;

            Row = 0;

            if ( ((PGStr->pg_Flags & PGF_PlayOnce) && (PosListStr->tcol_Flags & TCOLF_NotInOnce)) || ((!( PGStr->pg_Flags & PGF_PlayOnce )) && (PosListStr->tcol_Flags & TCOLF_NotInRepeat )))
              goto DisableChannel;

            if ( (TrackStr = PosListStr->tcol_Track) )
              break;
          }

          if ( PosListStr->tcol_Flags & TCOLF_EndOrder )
            goto SongEndFound;

          if ( PosListStr->tcol_Flags & TCOLF_EndSong )
          {
            if ( PGStr->pg_Flags & PGF_PlayOnce )
              goto DisableChannel;

            goto SongEndFound;
          }
        } while ( ((PGStr->pg_Flags & PGF_PlayOnce) && (PosListStr->tcol_Flags & TCOLF_NotInOnce)) || ((!( PGStr->pg_Flags & PGF_PlayOnce )) && (PosListStr->tcol_Flags & TCOLF_NotInRepeat )) || (!( TrackStr = PosListStr->tcol_Track )) );

        PHCStr->phc_Order = PosListStr;
        PHCStr->phc_Track = TrackStr;

        if ( PHCStr->phc_GoSubDepth < PosListStr->tcol_Played )
        {
          PHCStr->phc_Flags |= PHCF_SongEnd;

          if ( PGStr->pg_Flags & PGF_PlayOnce )
            PHCStr->phc_Tempo = 0;
        }

        PosListStr->tcol_Played++;

        PHCStr->phc_Flags &= ~PHCF_PattReset;

LoopToRow:

        TrackLen = TrackStr->tct_Length;
        Row = PosListStr->tcol_FirstRow;
        LastRow = PosListStr->tcol_LastRow;

        if ( PHCStr->phc_Flags & PHCF_PatternBreak )
        {
          PHCStr->phc_Flags &= ~PHCF_PatternBreak;

          Row = PHCStr->phc_BreakRow;

          if ( TrackLen < Row )
            Row = PosListStr->tcol_FirstRow;
        }

        if ( TrackLen < Row )
          goto CheckNextEmptyOrder;

        if ( TrackLen < LastRow )
          LastRow = TrackLen;

        PHCStr->phc_MaxRow = LastRow + 1;

        if ( (PattDelay = PosListStr->tcol_Tempo) )
          PHCStr->phc_Tempo = PattDelay;

        if ( PHCStr->phc_Flags & PHCF_Backwards )
          Row = LastRow - Row;
      }
    }

    PHCStr->phc_Row = Row;

    if ( (UBYTE) ((PHCStr->phc_Track->tct_Data) + Row)->tctd_Note == NOTE_END )
    {
      if ( ++Counted )
        goto GetNewPattern;

      goto DisableChannel;
    }
  }
}

static void GetEffects ( struct TuComposerBase *TuCBase, struct TuComposerModule *ModuleStr, struct TuComposerSong *SongStr, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel )
{
  struct TuComposerTrack *TrackStr;

  if ( (TrackStr = PHCStr->phc_Track) )
  {
    struct TuComposerTrackData *TrackData = TrackStr->tct_Data;
    struct TuComposerTrackEffect *TrackFX;

    if (!( TrackFX = PHCStr->phc_Effect ))
    {
      TrackData += PHCStr->phc_Row;

      TrackFX = (struct TuComposerTrackEffect *) TrackData->tctd_Effects.mlh_Head;
    }

    PHCStr->phc_Effect = TrackFX;

    if ( TrackFX->tcte_Node.mln_Succ )
    {
      for (;;)
      {
        UBYTE FXByte = TrackFX->tcte_EffectByte;

        if ( FXByte == FX_EXECUTE_FX )
        {
          PHCStr->phc_Flags |= PHCF_ExecFX;

          PHCStr->phc_ExecFX = TrackFX->tcte_DataWord;

          if ( PHCStr->phc_TempoCounter < PHCStr->phc_ExecFX )
            break;
        }

        TrackFX = (struct TuComposerTrackEffect *) TrackFX->tcte_Node.mln_Succ;

        if (!( TrackFX->tcte_Node.mln_Succ ))
          break;
      }

      if ( PHCStr->phc_Effect != TrackFX )
      {
        ULONG *EffectsUsed = (ULONG *) &(PHCStr->phc_EffectsUsed);

        PHCStr->phc_Effect = TrackFX;

        *EffectsUsed++ = 0L;
        *EffectsUsed++ = 0L;
        *EffectsUsed++ = 0L;
        *EffectsUsed = 0;
      }

      TrackData = TrackStr->tct_Data + PHCStr->phc_Row;

      TrackFX = (struct TuComposerTrackEffect *) &(TrackData->tctd_Effects.mlh_Head);

      for (;;)
      {
        struct EffectsTable *FXTable;
        void (*PreFXFunction)( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD DataWord );
        UWORD FXByte;

        TrackFX = (struct TuComposerTrackEffect *) TrackFX->tcte_Node.mln_Succ;

        if (!( TrackFX->tcte_Node.mln_Succ ))
          break;

        FXByte = TrackFX->tcte_EffectByte;

        FXTable = (struct EffectsTable *) TuCBase->tcb_EffectsTable + FXByte;

        if ( (PreFXFunction = FXTable->fx_PrePatternFXPtr) )
          PreFXFunction ( TuCBase, PHCStr, PCStr, Channel, TrackFX->tcte_DataWord );
      }
    }
  }
}

static ULONG GetNote ( struct TuComposerBase *TuCBase, struct TuComposerModule *ModuleStr, struct TuComposerSong *SongStr, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel )
{
  struct TuComposerTrack *TrackStr;
  struct TuComposerTrackData *TrackData;
  struct TuComposerInstr *InstrStr;
  struct PlayerChannel *NewPCStr;
  UWORD OctaveNote, Instr, i;
  UBYTE Octave;
  BYTE Note;

  if ( PHCStr->phc_PatternDelayCount || ( PHCStr->phc_TempoCounter != PHCStr->phc_NoteDelay ) || (!( TrackStr = PHCStr->phc_Track)) )
    return ( 0L );

  TrackData = TrackStr->tct_Data + PHCStr->phc_Row;

  if (!( *(ULONG *) &(TrackData->tctd_Octave) ))
    return ( 0L );

  OctaveNote = *(UWORD *) &(TrackData->tctd_Octave);
  Octave = TrackData->tctd_Octave;

  if ( (Note = TrackData->tctd_Note) < 0 )
  {
    switch ( (UBYTE) Note )
    {
      case NOTE_END :
      {
        if (!( PHCStr->phc_Flags & PHCF_PattLoop ))
        {
          PHCStr->phc_Flags |= PHCF_PatternBreak;

          PHCStr->phc_BreakRow = 0;
        }

        return ( 1L );
      }
      case NOTE_FADE :
      {
        PCStr->pc_Flags |= PCF_Fading;

        break;
      }
      case NOTE_HOLD_DELAY :
      {
        break;
      }
      case NOTE_KEYOFF :
      {
        PlayKeyOff ( PCStr );

        break;
      }
      case NOTE_OFF :
      {
        PCStr->pc_Volume = 0;

        break;
      }
      case NOTE_KILL :
      {
        PHCStr->phc_Instrument = NULL;
        PHCStr->phc_Sample = NULL;
        PHCStr->phc_InstrNote = 0;

        if ( PCStr->pc_HostChannel == Channel )
          PCStr->pc_ChannelData.ms_Flags = 0;

        break;
      }
    }

    return ( 0L );
  }

  if ( (Instr = TrackData->tctd_Instrument) )
  {
    struct TuComposerSample *SampleStr;

    InstrStr = (struct TuComposerInstr *) &(ModuleStr->tcm_InstrList.mlh_Head);
    i = Instr;

    do
    {
      InstrStr = (struct TuComposerInstr *) InstrStr->tci_Node.mln_Succ;

      if (!( InstrStr->tci_Node.mln_Succ ))
        return ( 0L );
    } while ( --i );

    if (!( InstrStr->tci_Flags & TCIF_NoInstrTranspose ))
    {
      struct TuComposerOrderList *PosListStr = PHCStr->phc_Order;

      if ( PosListStr->tcol_InstrTranspose )
      {
        struct TuComposerInstr *InstrScan = (struct TuComposerInstr *) &(ModuleStr->tcm_InstrList.mlh_Head);

        Instr += PosListStr->tcol_InstrTranspose;

        i = Instr;

        do
        {
          InstrScan = (struct TuComposerInstr *) InstrScan->tci_Node.mln_Succ;

          if (!( InstrScan->tci_Node.mln_Succ ))
            goto NotAllowInstrTrsp;
        } while ( --i );

        InstrStr = InstrScan;

NotAllowInstrTrsp:
      }
    }

    if ( PHCStr->phc_Flags & PHCF_TonePorta )
    {
      PHCStr->phc_TonePortaTargetPitch = GetTonePitch ( TuCBase, PHCStr, PCStr, GetKeyTableNote ( ModuleStr, InstrStr, PHCStr, Octave, Note ));

      return ( 0L );
    }

    if ( OctaveNote )
    {
      if ( (NewPCStr = PlayNote ( TuCBase, ModuleStr, InstrStr, PHCStr, PCStr, Octave, Note, Channel )))
        PCStr = NewPCStr;

      SampleStr = PHCStr->phc_Sample;

      PCStr->pc_Volume = SampleStr->tcs_Volume;
      PCStr->pc_SubVol = SampleStr->tcs_SubVol;

      InitNewInstr ( TuCBase, SongStr, PHCStr, PCStr );
      InitNewSample ( TuCBase, PHCStr, PCStr );
    } else
    {
      UWORD Note;

      if (!( InstrStr ))
        return ( 0L );

      if ( (Note = PHCStr->phc_InstrNote) )
      {
        if ( (Note = GetKeyTable ( ModuleStr, InstrStr, PHCStr, Note )) == 0x8000 )
          return ( 0L );

        if ( (PCStr->pc_HostChannel != Channel) || (PHCStr->phc_Instrument != InstrStr) )
        {
          if ( (NewPCStr = PlayNoteGot ( TuCBase, ModuleStr, PHCStr, PCStr, Note, Channel )))
            PCStr = NewPCStr;
        }
      } else
      {
        Note = GetKeyTable ( ModuleStr, InstrStr, PHCStr, 1L );

        PHCStr->phc_InstrNote = 0;
        PHCStr->phc_SampleNote = 0;

        if ( (NewPCStr = PlayNoteGot ( TuCBase, ModuleStr, PHCStr, PCStr, Note, Channel )))
          PCStr = NewPCStr;

        PCStr->pc_Flags |= PCF_Allocated;
      }

      SampleStr = PHCStr->phc_Sample;

      PCStr->pc_Volume = SampleStr->tcs_Volume;
      PCStr->pc_SubVol = SampleStr->tcs_SubVol;

      InitNewInstr ( TuCBase, SongStr, PHCStr, PCStr );

      if (!( InstrStr->tci_CompFlags & TCICF_LockInsWave ))
        InitNewSample ( TuCBase, PHCStr, PCStr );
    }
  } else
  {
    if ( (InstrStr = PHCStr->phc_Instrument) )
    {
      if (!( InstrStr->tci_Flags & TCIF_NoInstrTranspose ))
      {
        struct TuComposerOrderList *PosListStr = PHCStr->phc_Order;

        if ( PosListStr->tcol_InstrTranspose )
        {
          struct TuComposerInstr *InstrScan = (struct TuComposerInstr *) &(ModuleStr->tcm_InstrList.mlh_Head);

          Instr += PosListStr->tcol_InstrTranspose;

          i = Instr;

          do
          {
            InstrScan = (struct TuComposerInstr *) InstrScan->tci_Node.mln_Succ;

            if (!( InstrScan->tci_Node.mln_Succ ))
              goto NotAllowInstrTrsp2;
          } while ( --i );

          InstrStr = InstrScan;

NotAllowInstrTrsp2:
        }
      }

      if ( (NewPCStr = PlayNote ( TuCBase, ModuleStr, InstrStr, PHCStr, PCStr, Octave, Note, Channel )) )
      {
        struct TuComposerSample *SampleStr = PHCStr->phc_Sample;

        NewPCStr->pc_ChannelData.ms_SamplePos = SampleStr->tcs_StartOffset;

        if ( SampleStr->tcs_Flags & TCSCF_VolumeOnly )
        {
          NewPCStr->pc_Volume = PCStr->pc_Volume;
          NewPCStr->pc_SubVol = PCStr->pc_SubVol;

          InitNewInstr ( TuCBase, SongStr, PHCStr, NewPCStr );
          InitNewSample ( TuCBase, PHCStr, NewPCStr );
        } else
        {
          if ( PCStr != NewPCStr )
          {
            memcpy ( &(NewPCStr->pc_Volume), &(PCStr->pc_Volume), (offsetof (struct PlayerChannel, pc_InstrNote) - offsetof (struct PlayerChannel, pc_Volume)));

            NewPCStr->pc_HostChannel = Channel;
          }

          InitNewInstr ( TuCBase, SongStr, PHCStr, NewPCStr );
          InitNewSample ( TuCBase, PHCStr, NewPCStr );
        }
      }
    }
  }

  return ( 0L );
}

static WORD GetKeyTableNote ( struct TuComposerModule *ModuleStr, struct TuComposerInstr *InstrStr, struct PlayerHostChannel *PHCStr, UWORD Octave, UWORD Note )
{
  return ( GetKeyTable ( ModuleStr, InstrStr, PHCStr, (( Octave * NUMNOTES ) + Note )));
}

static WORD GetKeyTable ( struct TuComposerModule *ModuleStr, struct TuComposerInstr *InstrStr, struct PlayerHostChannel *PHCStr, UWORD Note )
{
  struct KeyboardEntry *KeybrdStr;
  struct TuComposerSample *SampleStr;
  UWORD Sample = 1, i;
  BYTE Transpose;

  if (!( PHCStr->phc_Instrument ))
    PHCStr->phc_NNA = InstrStr->tci_NNA;

  PHCStr->phc_InstrNote = Note;
  PHCStr->phc_SampleNote = Note;

  PHCStr->phc_Instrument = InstrStr;

  if (!( KeybrdStr = (struct KeyboardEntry *) InstrStr->tci_KeyboardDefs ))
    goto NotPlayKeyDef;

  KeybrdStr += ++Note;

  Note = ((UWORD) (KeybrdStr->tcik_Octave_C_0 & 0x7F) * NUMNOTES) + KeybrdStr->tcik_Note_C_0;

  PHCStr->phc_SampleNote = Note;

  if ( (Sample = KeybrdStr->tcik_Sample_C_0) )
  {

NotPlayKeyDef:

    if (!( InstrStr->tci_CompFlags & TCICF_SeparateSamples ))
    {
      SampleStr = (struct TuComposerSample *) &(InstrStr->tci_Samples.mlh_Head);

      i = Sample;

      do
      {
        SampleStr = (struct TuComposerSample *) SampleStr->tcs_Node.mln_Succ;

        if (!( SampleStr->tcs_Node.mln_Succ ))
          return ( 0x8000 );
      } while ( --i );
    } else
    {
      InstrStr = (struct TuComposerInstr *) &(ModuleStr->tcm_InstrList.mlh_Head);

      i = Sample;

      do
      {
        InstrStr = (struct TuComposerInstr *) InstrStr->tci_Node.mln_Succ;

        if (!( InstrStr->tci_Node.mln_Succ ))
          return ( 0x8000 );
      } while ( --i );

      SampleStr = (struct TuComposerSample *) InstrStr->tci_Samples.mlh_Head;

      if (!( SampleStr->tcs_Node.mln_Succ ))
        return ( 0x8000 );
    }
  } else
  {
    SampleStr = PHCStr->phc_Sample;

    if ( ( InstrStr->tci_CompFlags & TCICF_PrevSample ) || (!( SampleStr )))
      return ( 0x8000 );
  }

  PHCStr->phc_Sample = SampleStr;

  Transpose = SampleStr->tcs_Transpose;

  if ( PHCStr->phc_Flags & PHCF_SetTranspose )
    Transpose = PHCStr->phc_Transpose;

  Note += Transpose;

  if (!( InstrStr->tci_Flags & TCIF_NoTranspose ))
    Note += PHCStr->phc_Order->tcol_Transpose;

  Note += PHCStr->phc_Track->tct_Transpose;

  return ( Note - 1 );
}

static ULONG GetTonePitch ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, WORD Note )
{
  struct TuComposerSample *SampleStr = PHCStr->phc_Sample;
  ULONG *FreqTable;
  ULONG Frequency, NextFreq;
  UWORD Octave;
  BYTE FineTune;

  Octave = Note / NUMNOTES;
  Note = Note % NUMNOTES;

  if ( Note < 0 )
  {
    Octave--;

    Note += NUMNOTES;
  }

  if ( (FineTune = PHCStr->phc_FineTune) < 0 )
  {
    FineTune += -0x80;

    Note--;
  }

  FreqTable = TuCBase->tcb_FrequencyTable + Note;

  Frequency = *FreqTable++;
  NextFreq = *FreqTable - Frequency;
  Frequency += (LONG) (FineTune * (WORD) NextFreq) >> 7;

  if ( (WORD) ( Octave -= 4 ) < 0 )
  {
    Octave = -Octave;

    return ( ((unsigned long long) Frequency * SampleStr->tcs_Rate) >> (16UL + Octave) );
  } else
  {
    Frequency <<= Octave;

    return ( ((unsigned long long) Frequency * SampleStr->tcs_Rate) >> 16ULL );
  }
}

static struct PlayerChannel *PlayNote ( struct TuComposerBase *TuCBase, struct TuComposerModule *ModuleStr, struct TuComposerInstr *InstrStr, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Octave, UWORD Note, ULONG Channel )
{
  PHCStr->phc_Flags |= PHCF_RetrigNote;

  if ( (Note = GetKeyTableNote ( ModuleStr, InstrStr, PHCStr, Octave, Note )) == 0x8000 )
    return ( NULL );

  return ( PlayNoteGot ( TuCBase, ModuleStr, PHCStr, PCStr, Note, Channel ));
}

static struct PlayerChannel *PlayNoteGot ( struct TuComposerBase *TuCBase, struct TuComposerModule *ModuleStr, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Note, ULONG Channel )
{
  struct TuComposerInstr *InstrStr = PHCStr->phc_Instrument;
  struct TuComposerSample *SampleStr = PHCStr->phc_Sample;
  ULONG NoteSwing, PitchSwing, Frequency = 0L;
  ULONG Seed;
  UWORD VirtualCh;

  PHCStr->phc_DCT = InstrStr->tci_DCT;
  PHCStr->phc_DNA = InstrStr->tci_DNA;

  NoteSwing = (InstrStr->tci_NoteSwing << 1UL) + 1;

  TuCBase->tcb_RandomSeed = Seed = ((LONG) TuCBase->tcb_RandomSeed * RANDOM_CONST) + 1;

  NoteSwing = ((unsigned long long) Seed * NoteSwing) >> 32ULL;
  NoteSwing -= InstrStr->tci_NoteSwing;
  Note += NoteSwing;
  PHCStr->phc_FinalNote = Note;

  PHCStr->phc_FineTune = SampleStr->tcs_FineTune;

  if ( PHCStr->phc_Flags & PHCF_SetTranspose )
    PHCStr->phc_FineTune = PHCStr->phc_TransFineTune;

  PHCStr->phc_PrevVolumeEnv = PCStr->pc_VolEnvelope.pes_EnvelopeStructure;
  PHCStr->phc_PrevPanningEnv = PCStr->pc_PanEnvelope.pes_EnvelopeStructure;
  PHCStr->phc_PrevSlideEnv = PCStr->pc_SlideEnvelope.pes_EnvelopeStructure;
  PHCStr->phc_PrevAutoVibEnv = PCStr->pc_AutoVibEnv.pes_EnvelopeStructure;
  PHCStr->phc_PrevAutoTremEnv = PCStr->pc_AutoTremoEnv.pes_EnvelopeStructure;
  PHCStr->phc_PrevAutoPanEnv = PCStr->pc_AutoPanEnv.pes_EnvelopeStructure;

  PCStr = TriggerNNA ( ModuleStr, PHCStr, PCStr, Channel, (UWORD *) &VirtualCh );

  PCStr->pc_ChannelData.ms_SamplePos = SampleStr->tcs_StartOffset;

  PHCStr->phc_VirtualChannel = VirtualCh;
  PCStr->pc_HostChannel = Channel;

  PCStr->pc_Instrument = InstrStr;
  PCStr->pc_Sample = SampleStr;

  if ( (PCStr->pc_InstrNote = PHCStr->phc_InstrNote) )
  {
    WORD FinalNote;

    PCStr->pc_SampleNote = PHCStr->phc_SampleNote;

    PCStr->pc_FinalNote = FinalNote = PHCStr->phc_FinalNote;

    Frequency = GetTonePitch ( TuCBase, PHCStr, PCStr, FinalNote );
  }

  NoteSwing = PitchSwing = ((unsigned long long) Frequency * InstrStr->tci_PitchSwing) >> 16ULL;
  PitchSwing <<= 1UL;

  if ( PitchSwing < NoteSwing )
    PitchSwing = 0xFFFFFFFE;

  NoteSwing = PitchSwing++ >> 1UL;

  TuCBase->tcb_RandomSeed = Seed = ((LONG) TuCBase->tcb_RandomSeed * RANDOM_CONST) + 1;

  PitchSwing = ((unsigned long long) Seed * PitchSwing) >> 32ULL;
  PitchSwing -= NoteSwing;

  if ( (LONG) ( Frequency += PitchSwing ) < 0 )
    Frequency = 0L;

  PCStr->pc_Frequency = Frequency;

  return ( PCStr );
}

static void InitNewInstr ( struct TuComposerBase *TuCBase, struct TuComposerSong *SongStr, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr )
{
  struct TuComposerInstr *InstrStr = PHCStr->phc_Instrument;
  struct TuComposerSample *SampleStr = PHCStr->phc_Sample;
  struct PlayerGlobals *PGStr;
  struct TuComposerInstrEnvelope * (**AssignEnv)( struct TuComposerSong *SongStr, struct TuComposerInstr *InstrStr, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, struct TuComposerInstrEnvelope **EnvStr, struct Envelope **PlayEnvStr );
  struct TuComposerInstrEnvelope * (**AssignAutoEnv)( struct TuComposerSample *SampleStr, struct PlayerChannel *PCStr, struct Envelope **PlayEnvStr );
  ULONG Volume, VolumeSwing, VolSwingAbs, Seed, Pan, PanSep, i;

  if ( InstrStr )
  {
    Volume = SampleStr->tcs_GlobalVolume * InstrStr->tci_GlobalVolume;
    VolumeSwing = (Volume * InstrStr->tci_VolumeSwing) >> 8UL;
    VolSwingAbs = (VolumeSwing << 1UL) + 1;

    TuCBase->tcb_RandomSeed = Seed = ((LONG) TuCBase->tcb_RandomSeed * RANDOM_CONST) + 1;

    VolSwingAbs = ((unsigned long long) Seed * VolSwingAbs) >> 32ULL;
    VolSwingAbs -= VolumeSwing;

    if ( (LONG) ( Volume += VolSwingAbs ) < 0 )
      Volume = 0;

    if ( Volume > ( 255*255 ) )
      Volume = 255*255;
  } else
  {
    Volume = SampleStr->tcs_GlobalVolume * 255;
  }

  PCStr->pc_InstrVolume = Volume;

  PGStr = SongStr->tcso_GlobalData;

  PCStr->pc_GlobalVolume = PGStr->pg_GlobalVolume;
  PCStr->pc_GlobalSubVol = PGStr->pg_GlobalSubVol;
  PCStr->pc_GlobalPanning = PGStr->pg_GlobalPanning;
  PCStr->pc_GlobalSubPan = PGStr->pg_GlobalSubPan;

  if ( InstrStr )
  {
    PCStr->pc_FadeOut = InstrStr->tci_FadeOut;
    PCStr->pc_FadeOutCount = FADE_OUT_START;

    PHCStr->phc_NNA = InstrStr->tci_NNA;
  }

  PCStr->pc_AutoVibSweep = SampleStr->tcs_VibratoSweep;
  PCStr->pc_AutoTremoSweep = SampleStr->tcs_TremoloSweep;
  PCStr->pc_AutoPanSweep = SampleStr->tcs_PannoloSweep;
  PCStr->pc_AutoVibDepth = SampleStr->tcs_VibratoDepth;
  PCStr->pc_AutoVibRate = SampleStr->tcs_VibratoRate;
  PCStr->pc_AutoTremoDepth = SampleStr->tcs_TremoloDepth;
  PCStr->pc_AutoTremoRate = SampleStr->tcs_TremoloRate;
  PCStr->pc_AutoPanDepth = SampleStr->tcs_PannoloDepth;
  PCStr->pc_AutoPanRate = SampleStr->tcs_PannoloRate;

  PCStr->pc_SlideEnvelopeFrequency = 0L;

  PCStr->pc_AutoVibCount = 0;
  PCStr->pc_AutoTremoCount = 0;
  PCStr->pc_AutoPanCount = 0;

  PCStr->pc_Flags &= PCF_Allocated;

  PHCStr->phc_VibratoSlideValue = 0L;
  PHCStr->phc_TremoloSlideValue = 0;

  PCStr->pc_AutoVibFrequency = 0L;
  PCStr->pc_AutoTremoVolume = 0;
  PCStr->pc_AutoPanPanning = 0;

  PCStr->pc_Flags &= ~(PCF_LinearFreqAutoVib|PCF_PortaSlideEnv|PCF_LinearSlideEnv);

  if ( SampleStr->tcs_EnvProcFlags & TCSEPF_LinearAutoVib )
    PCStr->pc_Flags |= PCF_LinearFreqAutoVib;

  if ( InstrStr->tci_Flags & TCIF_PortaSlideEnv )
    PCStr->pc_Flags |= PCF_PortaSlideEnv;

  if ( InstrStr->tci_Flags & TCIF_LinearSlideEnv )
    PCStr->pc_Flags |= PCF_LinearSlideEnv;

  (APTR) AssignEnv = (APTR) &(AssignEnvTab);

  i = 0;

  do
  {
    struct TuComposerInstrEnvelope *EnvStr;
    struct Envelope *PlayEnvStr;
    UWORD Mask = 1U << i;

    if ( InstrStr )
    {
      if ( AssignEnv[i] ( SongStr, InstrStr, PHCStr, PCStr, (struct TuComposerInstrEnvelope **) &EnvStr, (struct Envelope **) &PlayEnvStr ))
      {
        if ( InstrStr->tci_EnvUsageFlags & Mask )
          continue;
      }

      if ( (PlayEnvStr->pes_EnvelopeStructure = EnvStr) )
      {
        UBYTE Flags = 0;
        UWORD EnvPos = 0, EnvValue = 0;

        if ( InstrStr->tci_EnvProcFlags & Mask )
          Flags = PESF_FirstAdd;

        if ( InstrStr->tci_EnvRetrigFlags & Mask )
        {
          Flags += PESF_NoRetrig;

          EnvPos = PlayEnvStr->pes_EnvelopePos;
          EnvValue = PlayEnvStr->pes_EnvelopeValue;
        }

        if ( InstrStr->tci_EnvRandomFlags & Mask )
          Flags += PESF_Random;

        if ( InstrStr->tci_EnvRndDelayFlags & Mask )
          Flags += PESF_RndDelay;

        PlayEnvStr->pes_EnvelopeValue = EnvValue;

        PlayEnvStr->pes_EnvelopeTempo = EnvStr->tcie_Tempo;
        PlayEnvStr->pes_EnvelopeTempoCount = 0;

        PlayEnvStr->pes_EnvelopeSustainCounted = 0;
        PlayEnvStr->pes_EnvelopeLoopCounted = 0;

        PlayEnvStr->pes_EnvelopeSustainStart = EnvStr->tcie_SustainStart;
        PlayEnvStr->pes_EnvelopeSustainEnd = EnvStr->tcie_SustainEnd;
        PlayEnvStr->pes_EnvelopeSustainCount = EnvStr->tcie_SustainCount;

        PlayEnvStr->pes_EnvelopeLoopStart = EnvStr->tcie_LoopStart;
        PlayEnvStr->pes_EnvelopeLoopEnd = EnvStr->tcie_LoopEnd;
        PlayEnvStr->pes_EnvelopeLoopCount = EnvStr->tcie_LoopCount;

        PlayEnvStr->pes_EnvelopeLowestValue = EnvStr->tcie_LowestValue;
        PlayEnvStr->pes_EnvelopeHighestValue = EnvStr->tcie_HighestValue;

        PlayEnvStr->pes_EnvelopeRepFlags = EnvStr->tcie_Flags;

        SetEnvelope ( PCStr, PlayEnvStr, EnvPos );

        PlayEnvStr->pes_EnvelopeFlags |= Flags;
      }
    } else
    {
      AssignEnv[i] ( SongStr, InstrStr, PHCStr, PCStr, (struct TuComposerInstrEnvelope **) &EnvStr, (struct Envelope **) &PlayEnvStr );

      PlayEnvStr->pes_EnvelopeStructure = NULL;

      PCStr->pc_VolEnvelope.pes_EnvelopeValue = 0;
    }
  } while ( ++i < (sizeof (AssignEnvTab) / sizeof (APTR)));

  PCStr->pc_VolEnvelope.pes_EnvelopeValue = -1;

  (APTR) AssignAutoEnv = (APTR) &(AssignAutoEnvTab);

  i = 0;

  do
  {
    struct TuComposerInstrEnvelope *EnvStr;
    struct Envelope *PlayEnvStr;
    UWORD Mask = 1U << i;

    EnvStr = AssignAutoEnv[i] ( SampleStr, PCStr, (struct Envelope **) &PlayEnvStr );

    if ( PlayEnvStr->pes_EnvelopeStructure && (SampleStr->tcs_EnvUsageFlags & Mask) )
      continue;

    if ( (PlayEnvStr->pes_EnvelopeStructure = EnvStr) )
    {
      UBYTE Flags = 0;
      UWORD EnvPos = 0, EnvValue = 0;

      if ( SampleStr->tcs_EnvProcFlags & Mask )
        Flags = PESF_FirstAdd;

      if ( SampleStr->tcs_EnvRetrigFlags & Mask )
      {
        Flags += PESF_NoRetrig;

        EnvPos = PlayEnvStr->pes_EnvelopePos;
        EnvValue = PlayEnvStr->pes_EnvelopeValue;
      }

      if ( SampleStr->tcs_EnvRandomFlags & Mask )
        Flags += PESF_Random;

      PlayEnvStr->pes_EnvelopeValue = EnvValue;

      PlayEnvStr->pes_EnvelopeTempo = EnvStr->tcie_Tempo;
      PlayEnvStr->pes_EnvelopeTempoCount = 0;

      PlayEnvStr->pes_EnvelopeSustainCounted = 0;
      PlayEnvStr->pes_EnvelopeLoopCounted = 0;

      PlayEnvStr->pes_EnvelopeSustainStart = EnvStr->tcie_SustainStart;
      PlayEnvStr->pes_EnvelopeSustainEnd = EnvStr->tcie_SustainEnd;
      PlayEnvStr->pes_EnvelopeSustainCount = EnvStr->tcie_SustainCount;

      PlayEnvStr->pes_EnvelopeLoopStart = EnvStr->tcie_LoopStart;
      PlayEnvStr->pes_EnvelopeLoopEnd = EnvStr->tcie_LoopEnd;
      PlayEnvStr->pes_EnvelopeLoopCount = EnvStr->tcie_LoopCount;

      PlayEnvStr->pes_EnvelopeLowestValue = EnvStr->tcie_LowestValue;
      PlayEnvStr->pes_EnvelopeHighestValue = EnvStr->tcie_HighestValue;

      PlayEnvStr->pes_EnvelopeRepFlags = EnvStr->tcie_Flags;

      SetEnvelope ( PCStr, PlayEnvStr, EnvPos );

      PlayEnvStr->pes_EnvelopeFlags |= Flags;
    }
  } while ( ++i < (sizeof (AssignAutoEnvTab) / sizeof (APTR)));

  Pan = PHCStr->phc_TrackNotePan;

  PCStr->pc_Flags |= PCF_TrackPan;

  if ( SampleStr->tcs_Flags & TCSF_SmpPanning )
  {
    PCStr->pc_Flags &= ~(PCF_SmpSurPan|PCF_TrackPan);

    if ( SampleStr->tcs_Flags & TCSF_SurroundPan )
      PCStr->pc_Flags |= PCF_SmpSurPan;

    PCStr->pc_Panning = SampleStr->tcs_Panning;
    PCStr->pc_SubPan = SampleStr->tcs_SubPanning;
    PHCStr->phc_PannoloSlideValue = 0;

    Pan = PCStr->pc_Panning;

    if ( SampleStr->tcs_CompFlags & TCSCF_AffectChanPan )
    {
      PHCStr->phc_TrackPanning = PCStr->pc_Panning;
      PHCStr->phc_TrackSubPan = PCStr->pc_SubPan;

      PHCStr->phc_TrackNotePan = PCStr->pc_Panning;
      PHCStr->phc_TrackNoteSubPan = PCStr->pc_SubPan;

      PHCStr->phc_Flags &= ~PHCF_TrackSurPan;

      if ( PCStr->pc_Flags & PCF_SmpSurPan )
        PHCStr->phc_Flags |= PHCF_TrackSurPan;
    }
  } else
  {
    PCStr->pc_Panning = PHCStr->phc_TrackPanning;
    PCStr->pc_SubPan = PHCStr->phc_TrackSubPan;

    PCStr->pc_Flags &= ~(PCF_SmpSurPan);
    PCStr->pc_Flags |= (PHCStr->phc_Flags & PHCF_TrackSurPan);
  }

  PHCStr->phc_Flags |= PHCF_AffectChanPan;

  if ( InstrStr )
  {
    if ( (InstrStr->tci_CompFlags & TCICF_AffectChanPan) && (SampleStr->tcs_CompFlags & TCSCF_AffectChanPan) )
      PHCStr->phc_Flags &= ~PHCF_AffectChanPan;

    if ( InstrStr->tci_Flags & TCIF_DefaultPanning )
    {
      PCStr->pc_Flags &= ~(PCF_SmpSurPan|PCF_TrackPan);

      if ( InstrStr->tci_Flags & TCIF_SurroundPanning )
        PCStr->pc_Flags |= PCF_SmpSurPan;

      PCStr->pc_Panning = InstrStr->tci_DefaultPanning;
      PCStr->pc_SubPan = InstrStr->tci_DefaultSubPan;
    }

    PHCStr->phc_PannoloSlideValue = 0;

    Pan = PCStr->pc_Panning;

    if ( InstrStr->tci_CompFlags & TCICF_AffectChanPan )
    {
      PHCStr->phc_TrackPanning = PCStr->pc_Panning;
      PHCStr->phc_TrackSubPan = PCStr->pc_SubPan;

      PHCStr->phc_Flags &= ~(PHCF_TrackSurPan);

      if ( PCStr->pc_Flags & PCF_SmpSurPan )
        PHCStr->phc_Flags |= PHCF_TrackSurPan;
    }

    PanSep = ((WORD) (PHCStr->phc_InstrNote - (1 + InstrStr->tci_PitchPanCenter)) * (WORD) InstrStr->tci_PitchPanSeparation) >> 8L;

    VolumeSwing = (InstrStr->tci_PanningSwing << 1UL) + 1;

    TuCBase->tcb_RandomSeed = Seed = ((LONG) TuCBase->tcb_RandomSeed * RANDOM_CONST) + 1;

    VolumeSwing = ((unsigned long long) Seed * VolumeSwing) >> 32ULL;
    VolumeSwing -= InstrStr->tci_VolumeSwing;
    Pan += VolumeSwing;

    if ( (LONG) ( Pan += PanSep ) < 0 )
      Pan = 0;

    if ( Pan > 255 )
      Pan = 255;
  }

  if ( PCStr->pc_Flags & PCF_TrackPan )
  {
    PHCStr->phc_TrackPanning = Pan;
  } else
  {
    PCStr->pc_Panning = Pan;
  }

  if ( PHCStr->phc_Flags & PHCF_AffectChanPan )
  {
    PHCStr->phc_TrackPanning = Pan;
    PCStr->pc_Panning = Pan;
  }
}

static void InitNewSample ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr )
{
  struct TuComposerSample *SampleStr = PHCStr->phc_Sample;
  struct TuComposerSynth *SynthStr;
  struct MixerAllocStruct *MixStr;
  ULONG Samples;

  if ( (Samples = SampleStr->tcs_Samples) )
  {
    UBYTE Flags, RepMode, PlayFlags;

    PCStr->pc_ChannelData.ms_SampleLen = Samples;
    PCStr->pc_ChannelData.ms_SampleData = SampleStr->tcs_Data;
    PCStr->pc_ChannelData.ms_SampleRate = PCStr->pc_Frequency;

    Flags = SampleStr->tcs_Flags;

    if ( Flags & TCSF_SustainLoop )
    {
      PCStr->pc_ChannelData.ms_Repeat = SampleStr->tcs_SustainRepeat;
      PCStr->pc_ChannelData.ms_RepLen = SampleStr->tcs_SustainRepLen;
      PCStr->pc_ChannelData.ms_RepCount = SampleStr->tcs_SustainRepCount;

      RepMode = SampleStr->tcs_SustainRepMode;

      Flags >>= 1U;
    } else
    {
      PCStr->pc_ChannelData.ms_Repeat = SampleStr->tcs_Repeat;
      PCStr->pc_ChannelData.ms_RepLen = SampleStr->tcs_RepLen;
      PCStr->pc_ChannelData.ms_RepCount = SampleStr->tcs_RepCount;

      RepMode = SampleStr->tcs_RepMode;
    }

    PCStr->pc_ChannelData.ms_RepCounted = 0L;
    PCStr->pc_ChannelData.ms_BitDepth = SampleStr->tcs_BitDepth;

    PlayFlags = MSF_Play;

    if ( SampleStr->tcs_Flags & TCSF_Reverse )
      PlayFlags |= MSF_Backwards;

    if ( (Flags & TCSF_Loop) && Samples )
    {
      PlayFlags += MSF_Loop;

      if ( RepMode & TCSRF_Pingpong )
        PlayFlags += MSF_Pingpong;

      if ( RepMode & TCSRF_Backwards )
        PlayFlags |= MSF_Backwards;
    }

    PCStr->pc_ChannelData.ms_Flags = PlayFlags;
  }

  if ( (!( SynthStr = SampleStr->tcs_Synth )) || (!( PHCStr->phc_Synth )) || (!( SynthStr->tcy_PosKeepMask & TCYPF_Code )))
    PHCStr->phc_Synth = SynthStr;

  if ( (PCStr->pc_Synth = PHCStr->phc_Synth) )
  {
    UWORD *SrcVar;
    UWORD *DestVar;
    UWORD KeepFlags, i;

    PCStr->pc_ChannelData.ms_Flags |= MSF_Play;

    if ( (!( PHCStr->phc_WaveformList )) || (!( SynthStr->tcy_PosKeepMask & TCYPF_Waveforms )))
    {
      struct TuComposerSynthWave *WaveformList = (struct TuComposerSynthWave *) &(SynthStr->tcy_Waves.mlh_Head);
      struct TuComposerSynthWave *WaveformStr = NULL;

      PHCStr->phc_WaveformList = WaveformList;

      WaveformList = (struct TuComposerSynthWave *) WaveformList->tcyw_Node.mln_Succ;

      if ( WaveformList->tcyw_Node.mln_Succ )
        WaveformStr = WaveformList;

      PCStr->pc_VibratoWaveform = WaveformStr;
      PCStr->pc_TremoloWaveform = WaveformStr;
      PCStr->pc_PannoloWaveform = WaveformStr;
      PCStr->pc_ArpeggioWaveform = WaveformStr;
    }

    PCStr->pc_WaveformList = PHCStr->phc_WaveformList;

    KeepFlags = SynthStr->tcy_PosKeepMask;

    if (!( SynthStr->tcy_PosKeepMask & TCYPF_Volume ))
      PHCStr->phc_VolumePos = SynthStr->tcy_VolumePos;

    if (!( SynthStr->tcy_PosKeepMask & TCYPF_Panning ))
      PHCStr->phc_PanningPos = SynthStr->tcy_PanningPos;

    if (!( SynthStr->tcy_PosKeepMask & TCYPF_Slide ))
      PHCStr->phc_SlidePos = SynthStr->tcy_SlidePos;

    if (!( SynthStr->tcy_PosKeepMask & TCYPF_Special ))
      PHCStr->phc_SpecialPos = SynthStr->tcy_SpecialPos;

    PCStr->pc_UseSusFlags = SynthStr->tcy_UseSusFlags;

    if (!( PCStr->pc_UseSusFlags & TCYSF_VolSusKeep ))
      PHCStr->phc_VolumeSusPos = SynthStr->tcy_VolumeSusPos;

    if (!( PCStr->pc_UseSusFlags & TCYSF_PanSusKeep ))
      PHCStr->phc_PanningSusPos = SynthStr->tcy_PanningSusPos;

    if (!( PCStr->pc_UseSusFlags & TCYSF_SlideSusKeep ))
      PHCStr->phc_SlideSusPos = SynthStr->tcy_SlideSusPos;

    if (!( PCStr->pc_UseSusFlags & TCYSF_SpecSusKeep ))
      PHCStr->phc_SpecialSusPos = SynthStr->tcy_SpecialSusPos;

    PCStr->pc_UseNNAFlags = SynthStr->tcy_UseNNAFlags;

    if (!( SynthStr->tcy_NNAPosKeepMask & TCYNF_VolNNA ))
      PHCStr->phc_VolNNAPos = SynthStr->tcy_VolNNAPos;

    if (!( SynthStr->tcy_NNAPosKeepMask & TCYNF_PanNNA ))
      PHCStr->phc_PanNNAPos = SynthStr->tcy_PanNNAPos;

    if (!( SynthStr->tcy_NNAPosKeepMask & TCYNF_SlideNNA ))
      PHCStr->phc_SlideNNAPos = SynthStr->tcy_SlideNNAPos;

    if (!( SynthStr->tcy_NNAPosKeepMask & TCYNF_SpecialNNA ))
      PHCStr->phc_SpecialNNAPos = SynthStr->tcy_SpecialNNAPos;

    if (!( SynthStr->tcy_NNAPosKeepMask & TCYNF_VolDNA ))
      PHCStr->phc_VolDNAPos = SynthStr->tcy_VolDNAPos;

    if (!( SynthStr->tcy_NNAPosKeepMask & TCYNF_PanDNA ))
      PHCStr->phc_PanDNAPos = SynthStr->tcy_PanDNAPos;

    if (!( SynthStr->tcy_NNAPosKeepMask & TCYNF_SlideDNA ))
      PHCStr->phc_SlideDNAPos = SynthStr->tcy_SlideDNAPos;

    if (!( SynthStr->tcy_NNAPosKeepMask & TCYNF_SpecialDNA ))
      PHCStr->phc_SpecialDNAPos = SynthStr->tcy_SpecialDNAPos;

    KeepFlags = 1;

    SrcVar = (UWORD *) &(SynthStr->tcy_Variable0);
    DestVar = (UWORD *) &(PHCStr->phc_Variable0);

    i = 16;

    do
    {
      if (!( SynthStr->tcy_VarKeepMask & KeepFlags ))
        *DestVar = *SrcVar;

      KeepFlags <<= 1U;

      SrcVar++;
      DestVar++;
    } while ( --i );

    PHCStr->phc_VolConVar = SynthStr->tcy_VolConVar;
    PHCStr->phc_PanConVar = SynthStr->tcy_PanConVar;
    PHCStr->phc_SlideConVar = SynthStr->tcy_SlideConVar;
    PHCStr->phc_SpecialConVar = SynthStr->tcy_SpecialConVar;

    memcpy ( &(PCStr->pc_VolumePos), &(PHCStr->phc_VolumePos), (offsetof (struct PlayerChannel, pc_UseNNAFlags) - offsetof (struct PlayerChannel, pc_VolumePos)));
    memset ( &(PCStr->pc_FineTune), 0, (offsetof (struct PlayerChannel, pc_Reserved) - offsetof (struct PlayerChannel, pc_FineTune)));
  }

  PCStr->pc_FineTune = PHCStr->phc_FineTune;

  MixStr = TuCBase->tcb_PlayMixerAlloc;
  ExternalMixerSetChannelBase ( MixStr, (struct MixerStruct *) &(PCStr->pc_ChannelData), PHCStr->phc_VirtualChannel, MixStr->mas_MixLibBase );
}

static struct PlayerChannel *TriggerNNA ( struct TuComposerModule *ModuleStr, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, ULONG Channel, UWORD *VirtualCh )
{
  struct PlayerChannel *NewPCStr = PCStr;
  struct PlayerChannel *ScanPCStr;
  UWORD NNACh, NNAMaxVol, NNAVol;
  UBYTE NNA;

  *VirtualCh = PHCStr->phc_VirtualChannel;

  if ( PCStr->pc_HostChannel != Channel )
  {
    NewPCStr = ModuleStr->tcm_ChannelData;

    NNACh = 0;

    do
    {
      if ( NewPCStr->pc_HostChannel == Channel )
        goto PrevNNAFound;

      NewPCStr++;
    } while ( ++NNACh < ModuleStr->tcm_Channels );

    goto FindNNA;

PrevNNAFound:

    *VirtualCh = NNACh;
  }

  NNAVol = NewPCStr->pc_Flags & PCF_Allocated;
  NewPCStr->pc_Flags &= ~PCF_Allocated;

  if ( ( NNAVol ) || (!( NNA = PHCStr->phc_NNA )) )
    goto NNAFound;

  if ( NewPCStr->pc_UseNNAFlags & TCYUNF_VolNNA )
    NewPCStr->pc_VolumePos = NewPCStr->pc_VolNNAPos;

  if ( NewPCStr->pc_UseNNAFlags & TCYUNF_PanNNA )
    NewPCStr->pc_PanningPos = NewPCStr->pc_PanNNAPos;

  if ( NewPCStr->pc_UseNNAFlags & TCYUNF_SlideNNA )
    NewPCStr->pc_SlidePos = NewPCStr->pc_SlideNNAPos;

  if ( NewPCStr->pc_UseNNAFlags & TCYUNF_SpecialNNA )
    NewPCStr->pc_SpecialPos = NewPCStr->pc_SpecialNNAPos;

  NewPCStr->pc_Flags |= PCF_Background;

  switch ( NNA )
  {
    case NNA_NOTE_OFF :
    {
      PlayKeyOff ( NewPCStr );

      break;
    }
    case NNA_NOTE_FADE :
    {
      NewPCStr->pc_Flags |= PCF_Fading;

      break;
    }
  }

  if ( (!( PHCStr->phc_DCT )) || PHCStr->phc_DNA )
    goto FindNNA;

  ScanPCStr = ModuleStr->tcm_ChannelData;

  NNACh = 0;

  do
  {
    if ( ScanPCStr->pc_HostChannel == Channel )
    {
      if ( TriggerDCT ( PHCStr, ScanPCStr, PHCStr->phc_DCT ))
      {
        *VirtualCh = NNACh;
        NewPCStr = ScanPCStr;

        goto NNAFound;
      }
    }

    ScanPCStr++;
  } while ( ++NNACh < ModuleStr->tcm_Channels );

FindNNA:

  ScanPCStr = ModuleStr->tcm_ChannelData;
  NewPCStr = NULL;

  NNACh = 0;

  do
  {
    if (!( (ScanPCStr->pc_Flags & PCF_Allocated) || (ScanPCStr->pc_ChannelData.ms_Flags & MSF_Play )))
    {
      *VirtualCh = NNACh;
      NewPCStr = ScanPCStr;

      goto NNAFound;
    }

    ScanPCStr++;
  } while ( ++NNACh < ModuleStr->tcm_Channels );

  NNAMaxVol = 256;

  ScanPCStr = ModuleStr->tcm_ChannelData;

  NNACh = 0;

  do
  {
    if ( ScanPCStr->pc_Flags & PCF_Background )
    {
      NNAVol = PCStr->pc_FinalVolume;

      if ( NNAMaxVol > NNAVol )
      {
        NNAMaxVol = NNAVol;

        *VirtualCh = NNACh;
        NewPCStr = ScanPCStr;

        break;
      }
    }

    ScanPCStr++;
  } while ( ++NNACh < ModuleStr->tcm_Channels );

  if (!( NewPCStr ))
    NewPCStr = PCStr;

NNAFound:

  if ( PHCStr->phc_DCT && ( NewPCStr != PCStr ) )
  {
    ScanPCStr = ModuleStr->tcm_ChannelData;

    NNACh = 0;

    do
    {
      if ( ScanPCStr->pc_HostChannel == Channel )
      {
        if ( TriggerDCT ( PHCStr, ScanPCStr, PHCStr->phc_DCT ))
        {
          if ( ScanPCStr->pc_UseNNAFlags & TCYUNF_VolDNA )
            ScanPCStr->pc_VolumePos = ScanPCStr->pc_VolDNAPos;

          if ( ScanPCStr->pc_UseNNAFlags & TCYUNF_PanDNA )
            ScanPCStr->pc_PanningPos = ScanPCStr->pc_PanDNAPos;

          if ( ScanPCStr->pc_UseNNAFlags & TCYUNF_SlideDNA )
            ScanPCStr->pc_SlidePos = ScanPCStr->pc_SlideDNAPos;

          if ( ScanPCStr->pc_UseNNAFlags & TCYUNF_SpecialNNA )
            ScanPCStr->pc_SpecialPos = ScanPCStr->pc_SpecialDNAPos;

          switch ( PHCStr->phc_DNA )
          {
            case DNA_NOTE_CUT :
            {
              PCStr->pc_ChannelData.ms_Flags = 0;

              break;
            }
            case DNA_NOTE_OFF :
            {
              PlayKeyOff ( ScanPCStr );

              break;
            }
            case DNA_NOTE_FADE :
            {
              PCStr->pc_Flags |= PCF_Fading;
            }
          }
        }
      }
    } while ( ++NNACh < ModuleStr->tcm_Channels );
  }

  PCStr->pc_Flags &= ~PCF_Allocated;

  return ( NewPCStr );
}

static ULONG TriggerDCT ( struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, ULONG DCT )
{
  ULONG Trigger = 0L;

  if ( DCT & DCTF_InstrNoteOR )
  {
    if ( PHCStr->phc_InstrNote == PCStr->pc_InstrNote )
      Trigger++;
  }

  if ( DCT & DCTF_SampleNoteOR )
  {
    if ( PHCStr->phc_SampleNote == PCStr->pc_SampleNote )
      Trigger++;
  }

  if ( DCT & DCTF_InstrOR )
  {
    if ( PHCStr->phc_Instrument == PCStr->pc_Instrument )
      Trigger++;
  }

  if ( DCT & DCTF_SampleOR )
  {
    if ( PHCStr->phc_Sample == PCStr->pc_Sample )
      Trigger++;
  }

  if ( DCT & DCTF_InstrNoteAND )
  {
    if ( PHCStr->phc_InstrNote != PCStr->pc_InstrNote )
      return ( 0L );

    Trigger++;
  }

  if ( DCT & DCTF_SampleNoteAND )
  {
    if ( PHCStr->phc_SampleNote != PCStr->pc_SampleNote )
      return ( 0L );

    Trigger++;
  }

  if ( DCT & DCTF_InstrAND )
  {
    if ( PHCStr->phc_Instrument != PCStr->pc_Instrument )
      return ( 0L );

    Trigger++;
  }

  if ( DCT & DCTF_SampleAND )
  {
    if ( PHCStr->phc_Sample != PCStr->pc_Sample )
      return ( 0L );

    Trigger++;
  }

  return ( Trigger );
}

static void PlayKeyOff ( struct PlayerChannel *PCStr )
{
  struct TuComposerSample *SampleStr;
  ULONG Repeat, RepLen, RepCount;
  UBYTE Flags;

  if ( PCStr->pc_Flags & PCF_Sustain )
    return;

  PCStr->pc_Flags |= PCF_Sustain;

  SetEnvelope ( PCStr, (struct Envelope *) &(PCStr->pc_VolEnvelope), PCStr->pc_VolEnvelope.pes_EnvelopePos );
  SetEnvelope ( PCStr, (struct Envelope *) &(PCStr->pc_PanEnvelope), PCStr->pc_PanEnvelope.pes_EnvelopePos );
  SetEnvelope ( PCStr, (struct Envelope *) &(PCStr->pc_SlideEnvelope), PCStr->pc_SlideEnvelope.pes_EnvelopePos );
  SetEnvelope ( PCStr, (struct Envelope *) &(PCStr->pc_AutoVibEnv), PCStr->pc_AutoVibEnv.pes_EnvelopePos );
  SetEnvelope ( PCStr, (struct Envelope *) &(PCStr->pc_AutoTremoEnv), PCStr->pc_AutoTremoEnv.pes_EnvelopePos );
  SetEnvelope ( PCStr, (struct Envelope *) &(PCStr->pc_AutoPanEnv), PCStr->pc_AutoPanEnv.pes_EnvelopePos );

  if ( (!( PCStr->pc_VolEnvelope.pes_EnvelopeStructure )) || (!( PCStr->pc_VolEnvelope.pes_EnvelopeTempo )) || ( PCStr->pc_VolEnvelope.pes_EnvelopeFlags & PESF_Looping ))
    PCStr->pc_Flags |= PCF_Fading;

  SampleStr = PCStr->pc_Sample;

  if (!( SampleStr->tcs_Flags & TCSF_SustainLoop ))
    return;

  Repeat = SampleStr->tcs_Repeat;
  RepLen = SampleStr->tcs_RepLen;
  RepCount = SampleStr->tcs_RepCount;

  PCStr->pc_ChannelData.ms_Repeat = Repeat;
  PCStr->pc_ChannelData.ms_RepLen = RepLen;
  PCStr->pc_ChannelData.ms_RepCount = RepCount;

  Flags = PCStr->pc_ChannelData.ms_Flags & ~(MSF_Loop|MSF_Pingpong|MSF_Backwards);

  if ( (SampleStr->tcs_Flags & TCSF_Loop) && RepLen )
  {
    Flags |= MSF_Loop;

    if ( SampleStr->tcs_RepMode & TCSRF_Pingpong )
      Flags |= MSF_Pingpong;

    if ( SampleStr->tcs_RepMode & TCSRF_Backwards )
      Flags |= MSF_Backwards;
  }

  PCStr->pc_ChannelData.ms_Flags = Flags;

  if ( PCStr->pc_UseSusFlags & TCYSF_VolSus )
    PCStr->pc_VolumePos = PCStr->pc_VolumeSusPos;

  if ( PCStr->pc_UseSusFlags & TCYSF_PanSus )
    PCStr->pc_PanningPos = PCStr->pc_PanningSusPos;

  if ( PCStr->pc_UseSusFlags & TCYSF_SlideSus )
    PCStr->pc_SlidePos = PCStr->pc_SlideSusPos;

  if ( PCStr->pc_UseSusFlags & TCYSF_SpecialSus )
    PCStr->pc_SpecialPos = PCStr->pc_SpecialSusPos;
}

static void SetEnvelope ( struct PlayerChannel *PCStr, struct Envelope *EnvStr, UWORD EnvPos )
{
  struct TuComposerInstrEnvelope *InstrEnvStr;
  UBYTE EnvFlags;
  UWORD EnvLoopStart, EnvLoopEnd;

  if (!( InstrEnvStr = EnvStr->pes_EnvelopeStructure ))
    return;

  EnvFlags = PESF_Looping;
  EnvLoopStart = EnvStr->pes_EnvelopeLoopStart;
  EnvLoopEnd = EnvStr->pes_EnvelopeLoopEnd;

  if ( ( EnvStr->pes_EnvelopeRepFlags & PESRF_Sustain ) && (!( PCStr->pc_Flags & PCF_Sustain )))
  {
    EnvLoopStart = EnvStr->pes_EnvelopeSustainStart;
    EnvLoopEnd = EnvStr->pes_EnvelopeSustainEnd;
  } else if (!( EnvStr->pes_EnvelopeRepFlags & PESRF_Loop ))
  {
    EnvFlags = 0;

    EnvLoopEnd = InstrEnvStr->tcie_NumPoints - 1;
  }

  if ( EnvLoopStart > EnvLoopEnd )
    EnvLoopStart = EnvLoopEnd;

  if ( EnvPos > EnvLoopEnd )
    EnvPos = EnvLoopEnd;

  EnvStr->pes_EnvelopePos = EnvPos;
  EnvStr->pes_EnvelopeRestart = EnvLoopStart;
  EnvStr->pes_EnvelopeEnd = EnvLoopEnd;
  EnvStr->pes_EnvelopeFlags = EnvFlags;
}

static WORD RunEnvelope ( struct TuComposerBase *TuCBase, struct Envelope *PlayEnvStr, UWORD TempoMul, UWORD ValAdjust )
{
  struct TuComposerInstrEnvelope *EnvStr;
  WORD Value = PlayEnvStr->pes_EnvelopeValue;

  if ( (EnvStr = PlayEnvStr->pes_EnvelopeStructure) )
  {
    WORD *EnvData = EnvStr->tcie_Data;
    UWORD EnvPos = PlayEnvStr->pes_EnvelopePos;

    if ( PlayEnvStr->pes_EnvelopeTempo )
    {
      UWORD EnvCount;

      if (!( EnvCount = PlayEnvStr->pes_EnvelopeTempoCount ))
        PlayEnvStr->pes_EnvelopeValue = Value = StepEnvelope ( TuCBase, PlayEnvStr, EnvData, EnvPos, TempoMul, ValAdjust );

      PlayEnvStr->pes_EnvelopeTempoCount = ++EnvCount;

      if ( (PlayEnvStr->pes_EnvelopeFlags & PESF_Random) && (PlayEnvStr->pes_EnvelopeFlags & PESF_RndDelay) )
      {
        TempoMul *= PlayEnvStr->pes_EnvelopeTempo;
      } else
      {
        TempoMul = PlayEnvStr->pes_EnvelopeTempo;
      }

      if ( EnvCount >= TempoMul )
      {
        PlayEnvStr->pes_EnvelopeTempoCount = 0;
      } else
      {
        if ( (PlayEnvStr->pes_EnvelopeFlags & PESF_FirstAdd) && (!( PlayEnvStr->pes_EnvelopeFlags & PESF_Random )))
          Value = EnvData[EnvPos] + ValAdjust;
      }
    }
  }

  return ( Value );
}

static WORD StepEnvelope ( struct TuComposerBase *TuCBase, struct Envelope *PlayEnvStr, WORD *EnvData, UWORD EnvPos, UWORD TempoMul, UWORD ValAdjust )
{
  ULONG Seed, RandomVal;
  UWORD EnvRestart = PlayEnvStr->pes_EnvelopeRestart;
  WORD Value;

  Value = EnvData[EnvPos];

  if ( PlayEnvStr->pes_EnvelopeFlags & PESF_Random )
  {
    TuCBase->tcb_RandomSeed = Seed = ((LONG) TuCBase->tcb_RandomSeed * RANDOM_CONST) + 1;

    RandomVal = ((UWORD) PlayEnvStr->pes_EnvelopeHighestValue - (UWORD) PlayEnvStr->pes_EnvelopeLowestValue) + 1;

    Value = ((unsigned long long) Seed * RandomVal) >> 32ULL;
    Value += PlayEnvStr->pes_EnvelopeLowestValue;
  }

  Value += ValAdjust;

  if ( PlayEnvStr->pes_EnvelopeFlags & PESF_Backwards )
  {
    if ( PlayEnvStr->pes_EnvelopeFlags & PESF_Looping )
    {
      EnvPos += TempoMul;

      if ( EnvPos < TempoMul )
        goto LoopEnvOverBack;

      for (;;)
      {
CheckBackEnvLoop:

        if ( EnvPos <= EnvRestart )
          break;

LoopEnvOverBack:

        if ( EnvRestart == PlayEnvStr->pes_EnvelopeEnd )
          goto RunEnvCheckPPWait;

        if ( PlayEnvStr->pes_EnvelopeFlags & PESF_Pingpong )
        {
          EnvPos -= PlayEnvStr->pes_EnvelopePos;
          EnvPos += -EnvPos + EnvRestart;

          if ( EnvPos < EnvRestart )
            goto CheckEnvLoop;

          goto LoopEnvOver;
        } else
        {
          EnvPos += PlayEnvStr->pes_EnvelopeEnd - EnvRestart;
        }
      }
    } else
    {
      if ( EnvPos < TempoMul )
        PlayEnvStr->pes_EnvelopeTempo = 0;

      EnvPos -= TempoMul;
    }
  } else
  {
    if ( PlayEnvStr->pes_EnvelopeFlags & PESF_Looping )
    {
      EnvPos += TempoMul;

      if ( EnvPos < TempoMul )
        goto LoopEnvOver;

      for (;;)
      {
CheckEnvLoop:

        if ( EnvPos <= PlayEnvStr->pes_EnvelopeEnd )
          break;

LoopEnvOver:

        if ( EnvRestart == PlayEnvStr->pes_EnvelopeEnd )
        {
RunEnvCheckPPWait:

          EnvPos = EnvRestart;

          if ( PlayEnvStr->pes_EnvelopeFlags & PESF_Pingpong )
            PlayEnvStr->pes_EnvelopeFlags ^= PESF_Backwards;

          break;
        }

        if ( PlayEnvStr->pes_EnvelopeFlags & PESF_Pingpong )
        {
          PlayEnvStr->pes_EnvelopeFlags ^= PESF_Backwards;

          EnvPos -= PlayEnvStr->pes_EnvelopePos;
          EnvPos += -EnvPos + PlayEnvStr->pes_EnvelopeEnd;

          if ( EnvPos < PlayEnvStr->pes_EnvelopeEnd )
            goto CheckBackEnvLoop;

          goto LoopEnvOverBack;
        } else
        {
          EnvPos += EnvRestart - PlayEnvStr->pes_EnvelopeEnd;
        }
      }
    } else
    {
      EnvPos += TempoMul;

      if ( (EnvPos < TempoMul) || (EnvPos > PlayEnvStr->pes_EnvelopeEnd) )
        PlayEnvStr->pes_EnvelopeTempo = 0;
    }
  }

  PlayEnvStr->pes_EnvelopePos = EnvPos;

  if ( (PlayEnvStr->pes_EnvelopeFlags & PESF_FirstAdd) && (!( PlayEnvStr->pes_EnvelopeFlags & PESF_Random )))
    Value = EnvData[EnvPos] + ValAdjust;

  return ( Value );
}

static struct TuComposerInstrEnvelope *AssignVolumeEnv ( struct TuComposerSong *SongStr, struct TuComposerInstr *InstrStr, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, struct TuComposerInstrEnvelope **EnvStr, struct Envelope **PlayEnvStr )
{
  if ( InstrStr )
    *EnvStr = InstrStr->tci_VolumeEnvelope;

  *PlayEnvStr = (struct Envelope *) &(PCStr->pc_VolEnvelope);

  return ( PHCStr->phc_PrevVolumeEnv );
}

static struct TuComposerInstrEnvelope *AssignPanningEnv ( struct TuComposerSong *SongStr, struct TuComposerInstr *InstrStr, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, struct TuComposerInstrEnvelope **EnvStr, struct Envelope **PlayEnvStr )
{
  if ( InstrStr )
    *EnvStr = InstrStr->tci_PanningEnvelope;

  *PlayEnvStr = (struct Envelope *) &(PCStr->pc_PanEnvelope);

  return ( PHCStr->phc_PrevPanningEnv );
}

static struct TuComposerInstrEnvelope *AssignSlideEnv ( struct TuComposerSong *SongStr, struct TuComposerInstr *InstrStr, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, struct TuComposerInstrEnvelope **EnvStr, struct Envelope **PlayEnvStr )
{
  if ( InstrStr )
    *EnvStr = InstrStr->tci_SlideEnvelope;

  *PlayEnvStr = (struct Envelope *) &(PCStr->pc_SlideEnvelope);

  return ( PHCStr->phc_PrevSlideEnv );
}

static struct TuComposerInstrEnvelope *AssignVibratoEnv ( struct TuComposerSong *SongStr, struct TuComposerInstr *InstrStr, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, struct TuComposerInstrEnvelope **EnvStr, struct Envelope **PlayEnvStr )
{
  if ( InstrStr )
    *EnvStr = InstrStr->tci_VibratoEnvelope;

  *PlayEnvStr = (struct Envelope *) &(PHCStr->phc_VibratoEnv);

  return ( (*PlayEnvStr)->pes_EnvelopeStructure );
}

static struct TuComposerInstrEnvelope *AssignTremoloEnv ( struct TuComposerSong *SongStr, struct TuComposerInstr *InstrStr, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, struct TuComposerInstrEnvelope **EnvStr, struct Envelope **PlayEnvStr )
{
  if ( InstrStr )
    *EnvStr = InstrStr->tci_TremoloEnvelope;

  *PlayEnvStr = (struct Envelope *) &(PHCStr->phc_TremoloEnv);

  return ( (*PlayEnvStr)->pes_EnvelopeStructure );
}

static struct TuComposerInstrEnvelope *AssignPannoloEnv ( struct TuComposerSong *SongStr, struct TuComposerInstr *InstrStr, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, struct TuComposerInstrEnvelope **EnvStr, struct Envelope **PlayEnvStr )
{
  if ( InstrStr )
    *EnvStr = InstrStr->tci_PannoloEnvelope;

  *PlayEnvStr = (struct Envelope *) &(PHCStr->phc_PannoloEnv);

  return ( (*PlayEnvStr)->pes_EnvelopeStructure );
}

static struct TuComposerInstrEnvelope *AssignChannoloEnv ( struct TuComposerSong *SongStr, struct TuComposerInstr *InstrStr, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, struct TuComposerInstrEnvelope **EnvStr, struct Envelope **PlayEnvStr )
{
  if ( InstrStr )
    *EnvStr = InstrStr->tci_ChannoloEnvelope;

  *PlayEnvStr = (struct Envelope *) &(PHCStr->phc_ChannoloEnv);

  return ( (*PlayEnvStr)->pes_EnvelopeStructure );
}

static struct TuComposerInstrEnvelope *AssignSpenoloEnv ( struct TuComposerSong *SongStr, struct TuComposerInstr *InstrStr, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, struct TuComposerInstrEnvelope **EnvStr, struct Envelope **PlayEnvStr )
{
  if ( InstrStr )
    *EnvStr = InstrStr->tci_SpenoloEnvelope;

  *PlayEnvStr = (struct Envelope *) &(SongStr->tcso_GlobalData->pg_SpenoloEnv);

  return ( (*PlayEnvStr)->pes_EnvelopeStructure );
}

static struct TuComposerInstrEnvelope *AssignTTremoloEnv ( struct TuComposerSong *SongStr, struct TuComposerInstr *InstrStr, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, struct TuComposerInstrEnvelope **EnvStr, struct Envelope **PlayEnvStr )
{
  if ( InstrStr )
    *EnvStr = InstrStr->tci_TremoloEnvelope;

  *PlayEnvStr = (struct Envelope *) &(PHCStr->phc_TrackTremEnv);

  return ( (*PlayEnvStr)->pes_EnvelopeStructure );
}

static struct TuComposerInstrEnvelope *AssignTPannoloEnv ( struct TuComposerSong *SongStr, struct TuComposerInstr *InstrStr, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, struct TuComposerInstrEnvelope **EnvStr, struct Envelope **PlayEnvStr )
{
  if ( InstrStr )
    *EnvStr = InstrStr->tci_PannoloEnvelope;

  *PlayEnvStr = (struct Envelope *) &(PHCStr->phc_TrackPanEnv);

  return ( (*PlayEnvStr)->pes_EnvelopeStructure );
}

static struct TuComposerInstrEnvelope *AssignGTremoloEnv ( struct TuComposerSong *SongStr, struct TuComposerInstr *InstrStr, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, struct TuComposerInstrEnvelope **EnvStr, struct Envelope **PlayEnvStr )
{
  if ( InstrStr )
    *EnvStr = InstrStr->tci_TremoloEnvelope;

  *PlayEnvStr = (struct Envelope *) &(SongStr->tcso_GlobalData->pg_TremoloEnv);

  return ( (*PlayEnvStr)->pes_EnvelopeStructure );
}

static struct TuComposerInstrEnvelope *AssignGPannoloEnv ( struct TuComposerSong *SongStr, struct TuComposerInstr *InstrStr, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, struct TuComposerInstrEnvelope **EnvStr, struct Envelope **PlayEnvStr )
{
  if ( InstrStr )
    *EnvStr = InstrStr->tci_PannoloEnvelope;

  *PlayEnvStr = (struct Envelope *) &(SongStr->tcso_GlobalData->pg_PannoloEnv);

  return ( (*PlayEnvStr)->pes_EnvelopeStructure );
}

static struct TuComposerInstrEnvelope *AssignAutoVibEnv ( struct TuComposerSample *SampleStr, struct PlayerChannel *PCStr, struct Envelope **PlayEnvStr )
{
  *PlayEnvStr = (struct Envelope *) &(PCStr->pc_AutoVibEnv);

  return ( SampleStr->tcs_AutoVibratoEnv );
}

static struct TuComposerInstrEnvelope *AssignAutoTremEnv ( struct TuComposerSample *SampleStr, struct PlayerChannel *PCStr, struct Envelope **PlayEnvStr )
{
  *PlayEnvStr = (struct Envelope *) &(PCStr->pc_AutoTremoEnv);

  return ( SampleStr->tcs_AutoTremoloEnv );
}

static struct TuComposerInstrEnvelope *AssignAutoPanEnv ( struct TuComposerSample *SampleStr, struct PlayerChannel *PCStr, struct Envelope **PlayEnvStr )
{
  *PlayEnvStr = (struct Envelope *) &(PCStr->pc_AutoPanEnv);

  return ( SampleStr->tcs_AutoPannoloEnv );
}

static struct Envelope *UseVolEnv ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr )
{
  return ( (struct Envelope *) &(PCStr->pc_VolEnvelope) );
}

static struct Envelope *UsePanEnv ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr )
{
  return ( (struct Envelope *) &(PCStr->pc_PanEnvelope) );
}

static struct Envelope *UseSlideEnv ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr )
{
  return ( (struct Envelope *) &(PCStr->pc_SlideEnvelope) );
}

static struct Envelope *UseVibEnv ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr )
{
  return ( (struct Envelope *) &(PHCStr->phc_VibratoEnv) );
}

static struct Envelope *UseTremoEnv ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr )
{
  return ( (struct Envelope *) &(PHCStr->phc_TremoloEnv) );
}

static struct Envelope *UsePannoEnv ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr )
{
  return ( (struct Envelope *) &(PHCStr->phc_PannoloEnv) );
}

static struct Envelope *UseChanEnv ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr )
{
  return ( (struct Envelope *) &(PHCStr->phc_ChannoloEnv) );
}

static struct Envelope *UseSpenEnv ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr )
{
  return ( (struct Envelope *) &(TuCBase->tcb_PlaySong->tcso_GlobalData->pg_SpenoloEnv) );
}

static struct Envelope *UseAutoVibEnv ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr )
{
  return ( (struct Envelope *) &(PCStr->pc_AutoVibEnv) );
}

static struct Envelope *UseAutoTremEnv ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr )
{
  return ( (struct Envelope *) &(PCStr->pc_AutoTremoEnv) );
}

static struct Envelope *UseAutoPanEnv ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr )
{
  return ( (struct Envelope *) &(PCStr->pc_AutoPanEnv) );
}

static struct Envelope *UseTrkTremEnv ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr )
{
  return ( (struct Envelope *) &(PHCStr->phc_TrackTremEnv) );
}

static struct Envelope *UseTrkPanEnv ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr )
{
  return ( (struct Envelope *) &(PHCStr->phc_TrackPanEnv) );
}

static struct Envelope *UseGlobTremEnv ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr )
{
  return ( (struct Envelope *) &(TuCBase->tcb_PlaySong->tcso_GlobalData->pg_TremoloEnv) );
}

static struct Envelope *UseGlobPanEnv ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr )
{
  return ( (struct Envelope *) &(TuCBase->tcb_PlaySong->tcso_GlobalData->pg_PannoloEnv) );
}

static struct Envelope *UseArpEnv ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr )
{
  return ( (struct Envelope *) &(PHCStr->phc_ArpStructure) );
}

static void RunEffects ( struct TuComposerBase *TuCBase, struct TuComposerModule *ModuleStr, struct TuComposerSong *SongStr, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel )
{
  struct TuComposerTrack *TrackStr;

  if ( (TrackStr = PHCStr->phc_Track) && PHCStr->phc_Effect )
  {
    struct TuComposerTrackData *TrackData = TrackStr->tct_Data + PHCStr->phc_Row;
    struct TuComposerTrackEffect *TrackFX = (struct TuComposerTrackEffect *) &(TrackData->tctd_Effects.mlh_Head);

    for (;;)
    {
      struct EffectsTable *FXTable;
      void (*CheckFunction)( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD *EffectByte, UWORD *DataWord, UWORD *Flags );
      UWORD FXByte, DataWord, Flags;
      UBYTE ChCtrlType;

      TrackFX = (struct TuComposerTrackEffect *) TrackFX->tcte_Node.mln_Succ;

      if ( PHCStr->phc_Effect == TrackFX )
        break;

      FXByte = TrackFX->tcte_EffectByte;

      FXTable = (struct EffectsTable *) TuCBase->tcb_EffectsTable + FXByte;

      DataWord = TrackFX->tcte_DataWord;
      Flags = FXTable->fx_Flags;

      if ( (CheckFunction = FXTable->fx_CheckFXPtr) )
      {
        CheckFunction ( TuCBase, PHCStr, PCStr, Channel, &FXByte, &DataWord, &Flags );

        FXTable = (struct EffectsTable *) TuCBase->tcb_EffectsTable + FXByte;
      }

      if ( Flags & FXF_ExecWholeRow )
        continue;

      Flags = PHCStr->phc_ExecFX;

      if (!( PHCStr->phc_Flags & PHCF_ExecFX ))
        Flags = FXTable->fx_StdExecTick;

      if ( Flags != PHCStr->phc_TempoCounter )
        continue;

      if ( PHCStr->phc_EffectsUsed[(FXByte >> 3U)] & (1 << (7 - (FXByte & 7))) )
        continue;

      FXTable->fx_EffectPtr ( TuCBase, PHCStr, PCStr, Channel, FXByte, DataWord );

      if ( (ChCtrlType = PHCStr->phc_ChControlType) )
      {
        if ( PHCStr->phc_ChControlAffect & FXTable->fx_ANDMaskCtrlFX )
        {
          struct PlayerHostChannel *NewPHCStr;
          struct PlayerChannel *NewPCStr;
          UWORD CtrlFXByte, CtrlDataWord, CtrlFlags, CtrlChannel;

          switch ( ChCtrlType )
          {
            case CH_CTRL_NORMAL :
            {
              if ( (CtrlChannel = PHCStr->phc_ChControlChannel) != Channel )
              {
                NewPHCStr = SongStr->tcso_ChannelData + CtrlChannel;
                NewPCStr = ModuleStr->tcm_ChannelData + NewPHCStr->phc_VirtualChannel;

                CtrlFXByte = FXByte;
                CtrlDataWord = DataWord;
                CtrlFlags = Flags;

                if ( (CheckFunction = FXTable->fx_CheckFXPtr) )
                {
                  CheckFunction ( TuCBase, PHCStr, PCStr, Channel, &CtrlFXByte, &CtrlDataWord, &CtrlFlags );

                  FXTable = (struct EffectsTable *) TuCBase->tcb_EffectsTable + CtrlFXByte;
                }


                FXTable->fx_EffectPtr ( TuCBase, NewPHCStr, NewPCStr, CtrlChannel, CtrlFXByte, CtrlDataWord );
              }

              break;
            }
            case CH_CTRL_MULTI :
            {
              CtrlChannel = 0;

              do
              {
                if ( (CtrlChannel != Channel) && (PHCStr->phc_ControlChannels[(CtrlChannel >> 3U)] & (1 << (7 - (CtrlChannel & 7)))) )
                {
                  NewPHCStr = SongStr->tcso_ChannelData + CtrlChannel;
                  NewPCStr = ModuleStr->tcm_ChannelData + NewPHCStr->phc_VirtualChannel;

                  CtrlFXByte = FXByte;
                  CtrlDataWord = DataWord;
                  CtrlFlags = Flags;

                  if ( (CheckFunction = FXTable->fx_CheckFXPtr) )
                  {
                    CheckFunction ( TuCBase, PHCStr, PCStr, Channel, &CtrlFXByte, &CtrlDataWord, &CtrlFlags );

                    FXTable = (struct EffectsTable *) TuCBase->tcb_EffectsTable + CtrlFXByte;
                  }

                  FXTable->fx_EffectPtr ( TuCBase, NewPHCStr, NewPCStr, CtrlChannel, CtrlFXByte, CtrlDataWord );
                }
              } while ( ++CtrlChannel < SongStr->tcso_Channels );

              break;
            }
            default :
            {
              CtrlChannel = 0;

              do
              {
                if ( CtrlChannel != Channel )
                {
                  NewPHCStr = SongStr->tcso_ChannelData + CtrlChannel;
                  NewPCStr = ModuleStr->tcm_ChannelData + NewPHCStr->phc_VirtualChannel;

                  CtrlFXByte = FXByte;
                  CtrlDataWord = DataWord;
                  CtrlFlags = Flags;

                  if ( (CheckFunction = FXTable->fx_CheckFXPtr) )
                  {
                    CheckFunction ( TuCBase, PHCStr, PCStr, Channel, &CtrlFXByte, &CtrlDataWord, &CtrlFlags );

                    FXTable = (struct EffectsTable *) TuCBase->tcb_EffectsTable + CtrlFXByte;
                  }

                  FXTable->fx_EffectPtr ( TuCBase, NewPHCStr, NewPCStr, CtrlChannel, CtrlFXByte, CtrlDataWord );
                }
              } while ( ++CtrlChannel < SongStr->tcso_Channels );

              break;
            }
          }
        }
      }
    }

    TrackFX = (struct TuComposerTrackEffect *) &(TrackData->tctd_Effects.mlh_Head);

    for (;;)
    {
      struct EffectsTable *FXTable;
      void (*CheckFunction)( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD *EffectByte, UWORD *DataWord, UWORD *Flags );
      UWORD FXByte, DataWord, Flags;
      UBYTE ChCtrlType;

      TrackFX = (struct TuComposerTrackEffect *) TrackFX->tcte_Node.mln_Succ;

      if ( PHCStr->phc_Effect == TrackFX )
        break;

      FXByte = TrackFX->tcte_EffectByte;

      FXTable = (struct EffectsTable *) TuCBase->tcb_EffectsTable + FXByte;

      DataWord = TrackFX->tcte_DataWord;
      Flags = FXTable->fx_Flags;

      if ( (CheckFunction = FXTable->fx_CheckFXPtr) )
      {
        CheckFunction ( TuCBase, PHCStr, PCStr, Channel, &FXByte, &DataWord, &Flags );

        FXTable = (struct EffectsTable *) TuCBase->tcb_EffectsTable + FXByte;
      }

      if (!( Flags & FXF_ExecWholeRow ))
        continue;

      Flags = PHCStr->phc_ExecFX;

      if (!( PHCStr->phc_Flags & PHCF_ExecFX ))
        Flags = FXTable->fx_StdExecTick;

      if ( PHCStr->phc_TempoCounter < Flags )
        continue;

      if ( PHCStr->phc_EffectsUsed[(FXByte >> 3U)] & (1 << (7 - (FXByte & 7))) )
        continue;

      FXTable->fx_EffectPtr ( TuCBase, PHCStr, PCStr, Channel, FXByte, DataWord );

      if ( (ChCtrlType = PHCStr->phc_ChControlType) )
      {
        if ( PHCStr->phc_ChControlAffect & FXTable->fx_ANDMaskCtrlFX )
        {
          struct PlayerHostChannel *NewPHCStr;
          struct PlayerChannel *NewPCStr;
          UWORD CtrlFXByte, CtrlDataWord, CtrlFlags, CtrlChannel;

          switch ( ChCtrlType )
          {
            case CH_CTRL_NORMAL :
            {
              if ( (CtrlChannel = PHCStr->phc_ChControlChannel) != Channel )
              {
                NewPHCStr = SongStr->tcso_ChannelData + CtrlChannel;
                NewPCStr = ModuleStr->tcm_ChannelData + NewPHCStr->phc_VirtualChannel;

                CtrlFXByte = FXByte;
                CtrlDataWord = DataWord;
                CtrlFlags = Flags;

                if ( (CheckFunction = FXTable->fx_CheckFXPtr) )
                {
                  CheckFunction ( TuCBase, PHCStr, PCStr, Channel, &CtrlFXByte, &CtrlDataWord, &CtrlFlags );

                  FXTable = (struct EffectsTable *) TuCBase->tcb_EffectsTable + CtrlFXByte;
                }

                if ( NewPHCStr->phc_EffectsUsed[(CtrlFXByte >> 3U)] & (1 << (7 - (CtrlFXByte & 7))) )
                  continue;

                FXTable->fx_EffectPtr ( TuCBase, NewPHCStr, NewPCStr, CtrlChannel, CtrlFXByte, CtrlDataWord );
              }

              break;
            }
            case CH_CTRL_MULTI :
            {
              CtrlChannel = 0;

              do
              {
                if ( (CtrlChannel != Channel) && (PHCStr->phc_ControlChannels[(CtrlChannel >> 3U)] & (1 << (7 - (CtrlChannel & 7)))) )
                {
                  NewPHCStr = SongStr->tcso_ChannelData + CtrlChannel;
                  NewPCStr = ModuleStr->tcm_ChannelData + NewPHCStr->phc_VirtualChannel;

                  CtrlFXByte = FXByte;
                  CtrlDataWord = DataWord;
                  CtrlFlags = Flags;

                  if ( (CheckFunction = FXTable->fx_CheckFXPtr) )
                  {
                    CheckFunction ( TuCBase, PHCStr, PCStr, Channel, &CtrlFXByte, &CtrlDataWord, &CtrlFlags );

                    FXTable = (struct EffectsTable *) TuCBase->tcb_EffectsTable + CtrlFXByte;
                  }

                  if ( NewPHCStr->phc_EffectsUsed[(CtrlFXByte >> 3U)] & (1 << (7 - (CtrlFXByte & 7))) )
                    continue;

                  FXTable->fx_EffectPtr ( TuCBase, NewPHCStr, NewPCStr, CtrlChannel, CtrlFXByte, CtrlDataWord );
                }
              } while ( ++CtrlChannel < SongStr->tcso_Channels );

              break;
            }
            default :
            {
              CtrlChannel = 0;

              do
              {
                if ( CtrlChannel != Channel )
                {
                  NewPHCStr = SongStr->tcso_ChannelData + CtrlChannel;
                  NewPCStr = ModuleStr->tcm_ChannelData + NewPHCStr->phc_VirtualChannel;

                  CtrlFXByte = FXByte;
                  CtrlDataWord = DataWord;
                  CtrlFlags = Flags;

                  if ( (CheckFunction = FXTable->fx_CheckFXPtr) )
                  {
                    CheckFunction ( TuCBase, PHCStr, PCStr, Channel, &CtrlFXByte, &CtrlDataWord, &CtrlFlags );

                    FXTable = (struct EffectsTable *) TuCBase->tcb_EffectsTable + CtrlFXByte;
                  }

                  if ( NewPHCStr->phc_EffectsUsed[(CtrlFXByte >> 3U)] & (1 << (7 - (CtrlFXByte & 7))) )
                    continue;

                  FXTable->fx_EffectPtr ( TuCBase, NewPHCStr, NewPCStr, CtrlChannel, CtrlFXByte, CtrlDataWord );
                }
              } while ( ++CtrlChannel < SongStr->tcso_Channels );

              break;
            }
          }
        }
      }
    }
  }
}

static ULONG LinearSlideUp ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, ULONG Frequency, ULONG SlideValue )
{
  ULONG LinearSlideDiv = SlideValue / 3072;
  ULONG LinearSlideMod = SlideValue % 3072;
  ULONG LinearMul = (0x10000 + TuCBase->tcb_LinearFreqTable[LinearSlideMod]) << LinearSlideDiv;
  unsigned long long SlideFreq = ((unsigned long long) LinearMul * Frequency) >> 16ULL;

  if ( SlideFreq > 0xFFFFFFFFULL )
    SlideFreq = -1LL;

  return ( (PCStr->pc_Frequency = SlideFreq) );
}

static ULONG AmigaSlideUp ( struct PlayerChannel *PCStr, ULONG Frequency, ULONG SlideValue )
{
  ULONG Period = SLIDE_CONST / Frequency;

  if ( Period <= SlideValue )
    return ( 1L );

  Period -= SlideValue;

  return ( (PCStr->pc_Frequency = (SLIDE_CONST / Period)) );
}

static ULONG LinearSlideDn ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, ULONG Frequency, ULONG SlideValue )
{
  ULONG LinearSlideDiv = SlideValue / 3072;
  ULONG LinearSlideMod = SlideValue % 3072;
  ULONG LinearMul = 0x10000;

  if (!( LinearSlideMod ))
    LinearMul <<= 1UL;

  LinearMul += TuCBase->tcb_LinearFreqTable[-LinearSlideMod + 3072U];

  return ( (PCStr->pc_Frequency = ((unsigned long long) LinearMul * Frequency) >> (17UL + LinearSlideDiv)));
}

static ULONG AmigaSlideDn ( struct PlayerChannel *PCStr, ULONG Frequency, ULONG SlideValue )
{
  ULONG Period = SLIDE_CONST / Frequency;
  ULONG NewFrequency;

  Period += SlideValue;

  if ( Period < SlideValue )
    return ( 0L );

  NewFrequency = SLIDE_CONST / Period;

  if ( Frequency == NewFrequency )
  {
    if (!( NewFrequency-- ))
      NewFrequency = 0L;
  }

  return ( (PCStr->pc_Frequency = NewFrequency) );
}

static void PortaSlideUp ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, ULONG DataWord, ULONG CarryAdd, ULONG PortaShift, UWORD Channel )
{
  if ( PCStr->pc_HostChannel == Channel )
  {
    ULONG PortaSlideVal;
    UBYTE SubSlideVal = DataWord;

    if ( (PortaSlideVal = ( DataWord & 0xFFFFFF00 ) >> PortaShift) )
    {
      PHCStr->phc_SubSlideValue += SubSlideVal;

      if ( PHCStr->phc_SubSlideValue < SubSlideVal )
        PortaSlideVal += CarryAdd;

      if ( PHCStr->phc_Flags & PHCF_LinearFreq )
      {
        LinearSlideUp ( TuCBase, PCStr, PCStr->pc_Frequency, PortaSlideVal );
      } else
      {
        AmigaSlideUp ( PCStr, PCStr->pc_Frequency, PortaSlideVal );
      }
    }
  }
}

static void PortaSlideDn ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, ULONG DataWord, ULONG CarryAdd, ULONG PortaShift, UWORD Channel )
{
  if ( PCStr->pc_HostChannel == Channel )
  {
    ULONG PortaSlideVal;
    UBYTE SubSlideVal = DataWord;

    if ( (LONG) DataWord < 0 )
    {
      PortaSlideUp ( TuCBase, PHCStr, PCStr, -DataWord, CarryAdd, PortaShift, Channel );

      return;
    }

    if ( (PortaSlideVal = ( DataWord & 0xFFFFFF00 ) >> PortaShift) )
    {
      if ( PHCStr->phc_SubSlideValue < SubSlideVal )
      {
        PortaSlideVal += CarryAdd;

        if ( PortaSlideVal < CarryAdd )
          PortaSlideVal = -1L;
      }

      PHCStr->phc_SubSlideValue -= SubSlideVal;

      if ( PHCStr->phc_Flags & PHCF_LinearFreq )
      {
        LinearSlideDn ( TuCBase, PCStr, PCStr->pc_Frequency, PortaSlideVal );
      } else
      {
        AmigaSlideDn ( PCStr, PCStr->pc_Frequency, PortaSlideVal );
      }
    }
  }
}

static void PortaUpOk ( struct PlayerHostChannel *PHCStr, UWORD DataWord )
{
  struct TuComposerTrack *TrackStr = PHCStr->phc_Track;
  UWORD v0, v1, v3, v4, v5, v8;

  v0 = PHCStr->phc_FinePortaUpValue;
  v1 = PHCStr->phc_FinePortaDownValue;
  v3 = PHCStr->phc_PortaUpOnceValue;
  v4 = PHCStr->phc_PortaDownOnceValue;
  v5 = PHCStr->phc_FinePortaUpOnceValue;
  v8 = PHCStr->phc_FinePortaDownOnceValue;

  if ( TrackStr->tct_CompFlags & TCTCF_Slide )
  {
    v0 = DataWord;
    v3 = DataWord;
    v5 = DataWord;
  }

  if (!( TrackStr->tct_CompFlags & TCTCF_OpSlide ))
  {
    PHCStr->phc_PortaDownValue = DataWord;
    v1 = v0;
    v4 = v3;
    v8 = v5;
  }

  PHCStr->phc_PortaUpValue = DataWord;

  PHCStr->phc_FinePortaUpValue = v0;
  PHCStr->phc_FinePortaDownValue = v1;
  PHCStr->phc_PortaUpOnceValue = v3;
  PHCStr->phc_PortaDownOnceValue = v4;
  PHCStr->phc_FinePortaUpOnceValue = v5;
  PHCStr->phc_FinePortaDownOnceValue = v8;

  if ( TrackStr->tct_CompFlags & TCTCF_TPorta )
  {
    v1 = PHCStr->phc_FineTonePortaValue;
    v4 = PHCStr->phc_TonePortaOnceValue;
    v8 = PHCStr->phc_FineTonePortaOnceValue;

    if ( TrackStr->tct_CompFlags & TCTCF_Slide )
    {
      v1 = v0;
      v4 = v3;
      v8 = v5;
    }

    PHCStr->phc_TonePortaValue = DataWord;
    PHCStr->phc_FineTonePortaValue = v1;
    PHCStr->phc_TonePortaOnceValue = v4;
    PHCStr->phc_FineTonePortaOnceValue = v8;
  }
}

static void PortaDnOk ( struct PlayerHostChannel *PHCStr, UWORD DataWord )
{
  struct TuComposerTrack *TrackStr = PHCStr->phc_Track;
  UWORD v0, v1, v3, v4, v5, v8;

  v0 = PHCStr->phc_FinePortaUpValue;
  v1 = PHCStr->phc_FinePortaDownValue;
  v3 = PHCStr->phc_PortaUpOnceValue;
  v4 = PHCStr->phc_PortaDownOnceValue;
  v5 = PHCStr->phc_FinePortaUpOnceValue;
  v8 = PHCStr->phc_FinePortaDownOnceValue;

  if ( TrackStr->tct_CompFlags & TCTCF_Slide )
  {
    v1 = DataWord;
    v4 = DataWord;
    v8 = DataWord;
  }

  if (!( TrackStr->tct_CompFlags & TCTCF_OpSlide ))
  {
    PHCStr->phc_PortaUpValue = DataWord;
    v0 = v1;
    v3 = v4;
    v5 = v8;
  }

  PHCStr->phc_PortaDownValue = DataWord;

  PHCStr->phc_FinePortaUpValue = v0;
  PHCStr->phc_FinePortaDownValue = v1;
  PHCStr->phc_PortaUpOnceValue = v3;
  PHCStr->phc_PortaDownOnceValue = v4;
  PHCStr->phc_FinePortaUpOnceValue = v5;
  PHCStr->phc_FinePortaDownOnceValue = v8;

  if ( TrackStr->tct_CompFlags & TCTCF_TPorta )
  {
    v0 = PHCStr->phc_FineTonePortaValue;
    v3 = PHCStr->phc_TonePortaOnceValue;
    v5 = PHCStr->phc_FineTonePortaOnceValue;

    if ( TrackStr->tct_CompFlags & TCTCF_Slide )
    {
      v0 = v1;
      v3 = v4;
      v5 = v8;
    }

    PHCStr->phc_TonePortaValue = DataWord;
    PHCStr->phc_FineTonePortaValue = v0;
    PHCStr->phc_TonePortaOnceValue = v3;
    PHCStr->phc_FineTonePortaOnceValue = v5;
  }
}

static void PortaUpOnceOk ( struct PlayerHostChannel *PHCStr, UWORD DataWord )
{
  struct TuComposerTrack *TrackStr = PHCStr->phc_Track;
  UWORD v0, v1, v3, v4, v5, v8;

  v0 = PHCStr->phc_PortaUpValue;
  v1 = PHCStr->phc_PortaDownValue;
  v3 = PHCStr->phc_FinePortaUpValue;
  v4 = PHCStr->phc_FinePortaDownValue;
  v5 = PHCStr->phc_FinePortaUpOnceValue;
  v8 = PHCStr->phc_FinePortaDownOnceValue;

  if ( TrackStr->tct_CompFlags & TCTCF_Slide )
  {
    v0 = DataWord;
    v3 = DataWord;
    v5 = DataWord;
  }

  if (!( TrackStr->tct_CompFlags & TCTCF_OpSlide ))
  {
    PHCStr->phc_PortaDownOnceValue = DataWord;
    v1 = v0;
    v4 = v3;
    v8 = v5;
  }

  PHCStr->phc_PortaUpOnceValue = DataWord;

  PHCStr->phc_PortaUpValue = v0;
  PHCStr->phc_PortaDownValue = v1;
  PHCStr->phc_FinePortaUpValue = v3;
  PHCStr->phc_FinePortaDownValue = v4;
  PHCStr->phc_FinePortaUpOnceValue = v5;
  PHCStr->phc_FinePortaDownOnceValue = v8;

  if ( TrackStr->tct_CompFlags & TCTCF_TPorta )
  {
    v1 = PHCStr->phc_TonePortaValue;
    v4 = PHCStr->phc_FineTonePortaValue;
    v8 = PHCStr->phc_FineTonePortaOnceValue;

    if ( TrackStr->tct_CompFlags & TCTCF_Slide )
    {
      v1 = v0;
      v4 = v3;
      v8 = v5;
    }

    PHCStr->phc_TonePortaValue = v1;
    PHCStr->phc_FineTonePortaValue = v4;
    PHCStr->phc_TonePortaOnceValue = DataWord;
    PHCStr->phc_FineTonePortaOnceValue = v8;
  }
}

static void PortaDnOnceOk ( struct PlayerHostChannel *PHCStr, UWORD DataWord )
{
  struct TuComposerTrack *TrackStr = PHCStr->phc_Track;
  UWORD v0, v1, v3, v4, v5, v8;

  v0 = PHCStr->phc_PortaUpValue;
  v1 = PHCStr->phc_PortaDownValue;
  v3 = PHCStr->phc_FinePortaUpValue;
  v4 = PHCStr->phc_FinePortaDownValue;
  v5 = PHCStr->phc_FinePortaUpOnceValue;
  v8 = PHCStr->phc_FinePortaDownOnceValue;

  if ( TrackStr->tct_CompFlags & TCTCF_Slide )
  {
    v1 = DataWord;
    v4 = DataWord;
    v8 = DataWord;
  }

  if (!( TrackStr->tct_CompFlags & TCTCF_OpSlide ))
  {
    PHCStr->phc_PortaUpOnceValue = DataWord;
    v0 = v1;
    v3 = v4;
    v5 = v8;
  }

  PHCStr->phc_PortaDownOnceValue = DataWord;

  PHCStr->phc_PortaUpValue = v0;
  PHCStr->phc_PortaDownValue = v1;
  PHCStr->phc_FinePortaUpValue = v3;
  PHCStr->phc_FinePortaDownValue = v4;
  PHCStr->phc_FinePortaUpOnceValue = v5;
  PHCStr->phc_FinePortaDownOnceValue = v8;

  if ( TrackStr->tct_CompFlags & TCTCF_TPorta )
  {
    v0 = PHCStr->phc_TonePortaValue;
    v3 = PHCStr->phc_FineTonePortaValue;
    v5 = PHCStr->phc_FineTonePortaOnceValue;

    if ( TrackStr->tct_CompFlags & TCTCF_Slide )
    {
      v0 = v1;
      v3 = v4;
      v5 = v8;
    }

    PHCStr->phc_TonePortaValue = v0;
    PHCStr->phc_FineTonePortaValue = v3;
    PHCStr->phc_TonePortaOnceValue = DataWord;
    PHCStr->phc_FineTonePortaOnceValue = v5;
  }
}

static void DoVibrato ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD VibratoRate, WORD VibratoDepth )
{
  LONG VibratoSlideValue;

  if (!( VibratoDepth ))
    VibratoDepth = PHCStr->phc_VibratoDepth;

  PHCStr->phc_VibratoDepth = VibratoDepth;

  VibratoSlideValue = ((-VibratoDepth * RunEnvelope ( TuCBase, (struct Envelope *) &(PHCStr->phc_VibratoEnv), VibratoRate, 0 )) >> (7L-2L)) << 8L;

  if ( PCStr->pc_HostChannel == Channel )
  {
    ULONG OldFrequency = PCStr->pc_Frequency;

    PCStr->pc_Frequency -= PHCStr->phc_VibratoSlideValue;

    PortaSlideDn ( TuCBase, PHCStr, PCStr, VibratoSlideValue, 1, 8, Channel );

    PHCStr->phc_VibratoSlideValue -= OldFrequency - PCStr->pc_Frequency;
  }
}

static ULONG CheckOldVolume ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD *DataWord, UWORD Channel )
{
  struct TuComposerSong *SongStr;

  if ( Channel != PCStr->pc_HostChannel )
    return ( 0L );

  SongStr = TuCBase->tcb_PlaySong;

  if ( SongStr->tcso_CompFlags & TCSOCF_OldVolumes )
  {
    if ( *DataWord < 0x4000 )
    {
      *DataWord = ((*DataWord & 0xFF00) << 2U) | (*DataWord & 0xFF);
    } else
    {
      *DataWord = 0xFFFF;
    }
  }

  return ( 1L );
}

static void DoVolumeSlide ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD DataWord, UWORD Channel )
{
  if ( CheckOldVolume ( TuCBase, PCStr, &DataWord, Channel ))
  {
    UWORD SlideVol = (PCStr->pc_Volume << 8U) + PCStr->pc_SubVol;

    if ( (SlideVol += DataWord) < DataWord )
      SlideVol = 0xFFFF;

    PCStr->pc_Volume = SlideVol >> 8U;
    PCStr->pc_SubVol = SlideVol;
  }
}

static void DoVolumeSlideDn ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD DataWord, UWORD Channel )
{
  if ( CheckOldVolume ( TuCBase, PCStr, &DataWord, Channel ))
  {
    UWORD SlideVol = (PCStr->pc_Volume << 8U) + PCStr->pc_SubVol;

    if ( SlideVol < DataWord )
      DataWord = SlideVol;

    SlideVol -= DataWord;

    PCStr->pc_Volume = SlideVol >> 8U;
    PCStr->pc_SubVol = SlideVol;
  }
}

static void VolSlUpOk ( struct PlayerHostChannel *PHCStr, UWORD DataWord )
{
  struct TuComposerTrack *TrackStr = PHCStr->phc_Track;
  UWORD v3, v4, v5;

  v3 = PHCStr->phc_VolSlideDownValue;
  v4 = PHCStr->phc_FineVolSlideUpValue;
  v5 = PHCStr->phc_FineVolSlideDownValue;

  if ( TrackStr->tct_Flags & TCTCF_VolSlide )
    v4 = DataWord;

  if (!( TrackStr->tct_Flags & TCTCF_OpVSlide ))
  {
    v3 = DataWord;
    v5 = v4;
  }

  PHCStr->phc_VolSlideUpValue = DataWord;
  PHCStr->phc_VolSlideDownValue = v3;
  PHCStr->phc_FineVolSlideUpValue = v4;
  PHCStr->phc_FineVolSlideDownValue = v5;
}

static void VolSlDnOk ( struct PlayerHostChannel *PHCStr, UWORD DataWord )
{
  struct TuComposerTrack *TrackStr = PHCStr->phc_Track;
  UWORD v0, v3, v4;

  v0 = PHCStr->phc_VolSlideUpValue;
  v3 = PHCStr->phc_FineVolSlideUpValue;
  v4 = PHCStr->phc_FineVolSlideDownValue;

  if ( TrackStr->tct_Flags & TCTCF_VolSlide )
    v4 = DataWord;

  if (!( TrackStr->tct_Flags & TCTCF_OpVSlide ))
  {
    v0 = DataWord;
    v3 = v4;
  }

  PHCStr->phc_VolSlideUpValue = v0;
  PHCStr->phc_VolSlideDownValue = DataWord;
  PHCStr->phc_FineVolSlideUpValue = v3;
  PHCStr->phc_FineVolSlideDownValue = v4;
}

static void FVolSlUpOk ( struct PlayerHostChannel *PHCStr, UWORD DataWord )
{
  struct TuComposerTrack *TrackStr = PHCStr->phc_Track;
  UWORD v0, v1, v4;

  v0 = PHCStr->phc_VolSlideUpValue;
  v1 = PHCStr->phc_VolSlideDownValue;
  v4 = PHCStr->phc_FineVolSlideDownValue;

  if ( TrackStr->tct_Flags & TCTCF_VolSlide )
    v0 = DataWord;

  if (!( TrackStr->tct_Flags & TCTCF_OpVSlide ))
  {
    v1 = v0;
    v4 = DataWord;
  }

  PHCStr->phc_VolSlideUpValue = v0;
  PHCStr->phc_VolSlideDownValue = v1;
  PHCStr->phc_FineVolSlideUpValue = DataWord;
  PHCStr->phc_FineVolSlideDownValue = v4;
}

static void FVolSlDnOk ( struct PlayerHostChannel *PHCStr, UWORD DataWord )
{
  struct TuComposerTrack *TrackStr = PHCStr->phc_Track;
  UWORD v0, v1, v3;

  v0 = PHCStr->phc_VolSlideUpValue;
  v1 = PHCStr->phc_VolSlideDownValue;
  v3 = PHCStr->phc_FineVolSlideUpValue;

  if ( TrackStr->tct_Flags & TCTCF_VolSlide )
    v1 = DataWord;

  if (!( TrackStr->tct_Flags & TCTCF_OpVSlide ))
  {
    v0 = v1;
    v3 = DataWord;
  }

  PHCStr->phc_VolSlideUpValue = v0;
  PHCStr->phc_VolSlideDownValue = v1;
  PHCStr->phc_FineVolSlideUpValue = DataWord;
  PHCStr->phc_FineVolSlideDownValue = v3;
}

static ULONG CheckOldTVol ( struct TuComposerBase *TuCBase, UWORD *DataWord )
{
  struct TuComposerSong *SongStr = TuCBase->tcb_PlaySong;

  if ( SongStr->tcso_CompFlags & TCSOCF_OldVolumes )
  {
    if ( *DataWord < 0x4000 )
    {
      *DataWord = ((*DataWord & 0xFF00) << 2U) | (*DataWord & 0xFF);
    } else
    {
      *DataWord = 0xFFFF;
    }
  }

  return ( 1L );
}

static void DoTVolumeSlide ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, UWORD DataWord )
{
  if ( CheckOldTVol ( TuCBase, &DataWord ) )
  {
    UWORD TrackVolume = (PHCStr->phc_TrackVolume << 8U) + PHCStr->phc_TrackSubVol;

    if ( (TrackVolume += DataWord) < DataWord )
      TrackVolume = 0xFFFF;

    PHCStr->phc_TrackVolume = TrackVolume >> 8U;
    PHCStr->phc_TrackSubVol = TrackVolume;
  }
}

static void DoTVolumeSlideDn ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, UWORD DataWord )
{
  if ( CheckOldTVol ( TuCBase, &DataWord ) )
  {
    UWORD TrackVolume = (PHCStr->phc_TrackVolume << 8U) + PHCStr->phc_TrackSubVol;

    if ( TrackVolume < DataWord )
      DataWord = TrackVolume;

    TrackVolume -= DataWord;

    PHCStr->phc_TrackVolume = TrackVolume >> 8U;
    PHCStr->phc_TrackSubVol = TrackVolume;
  }
}

static void DoPanSlide ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD DataWord, UWORD Channel )
{
  if ( PCStr->pc_HostChannel == Channel )
  {
    UWORD Panning = (PCStr->pc_Panning << 8U) + PCStr->pc_SubPan;

    if ( Panning < DataWord )
      DataWord = Panning;

    Panning -= DataWord;

    PHCStr->phc_TrackPanning = PCStr->pc_Panning = Panning >> 8U;
    PHCStr->phc_TrackSubPan = PCStr->pc_SubPan = Panning;
  } else
  {
    UWORD TrackPanning = (PHCStr->phc_TrackPanning << 8U) + PHCStr->phc_TrackSubPan;

    if ( TrackPanning < DataWord )
      DataWord = TrackPanning;

    TrackPanning -= DataWord;

    PHCStr->phc_TrackPanning = TrackPanning >> 8U;
    PHCStr->phc_TrackSubPan = TrackPanning;
  }

  PHCStr->phc_TrackNotePan = PHCStr->phc_TrackPanning;
  PHCStr->phc_TrackNoteSubPan = PHCStr->phc_TrackSubPan;
}

static void DoPanSlideRight ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD DataWord, UWORD Channel )
{
  if ( PCStr->pc_HostChannel == Channel )
  {
    UWORD Panning = (PCStr->pc_Panning << 8U) + PCStr->pc_SubPan;

    if ( (Panning += DataWord) < DataWord )
      Panning = 0xFFFF;

    PHCStr->phc_TrackPanning = PCStr->pc_Panning = Panning >> 8U;
    PHCStr->phc_TrackSubPan = PCStr->pc_SubPan = Panning;
  } else
  {
    UWORD TrackPanning = (PHCStr->phc_TrackPanning << 8U) + PHCStr->phc_TrackSubPan;

    if ( (TrackPanning += DataWord) < DataWord )
      TrackPanning = 0xFFFF;

    PHCStr->phc_TrackPanning = TrackPanning >> 8U;
    PHCStr->phc_TrackSubPan = TrackPanning;
  }

  PHCStr->phc_TrackNotePan = PHCStr->phc_TrackPanning;
  PHCStr->phc_TrackNoteSubPan = PHCStr->phc_TrackSubPan;
}

static void DoTPanSlide ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, UWORD DataWord )
{
  UWORD ChannelPanning = (PHCStr->phc_ChannelPanning << 8U) + PHCStr->phc_ChannelSubPan;

  if ( ChannelPanning < DataWord )
    DataWord = ChannelPanning;

  ChannelPanning -= DataWord;

  PHCStr->phc_ChannelPanning = ChannelPanning >> 8U;
  PHCStr->phc_ChannelSubPan = ChannelPanning;
}

static void DoTPanSlideRight ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, UWORD DataWord )
{
  UWORD ChannelPanning = (PHCStr->phc_ChannelPanning << 8U) + PHCStr->phc_ChannelSubPan;

  if ( (ChannelPanning += DataWord) < DataWord )
    ChannelPanning = 0xFFFF;

  PHCStr->phc_ChannelPanning = ChannelPanning >> 8U;
  PHCStr->phc_ChannelSubPan = ChannelPanning;
}

static ULONG CheckSurTPan ( struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UBYTE ChCtrlByte )
{
  if ( PCStr->pc_HostChannel == Channel )
  {
    if ( PHCStr->phc_Flags & PHCF_AffectChanPan )
      return ( 1L );

    if ( ChCtrlByte )
    {
      PCStr->pc_Flags |= PCF_SmpSurPan;
    } else
    {
      PCStr->pc_Flags &= ~PCF_SmpSurPan;
    }
  } else
  {
    if ( PHCStr->phc_Flags & PHCF_AffectChanPan )
    {
      PHCStr->phc_Flags &= ~PHCF_TrackSurPan;

      if ( ChCtrlByte )
        PHCStr->phc_Flags |= PHCF_TrackSurPan;
    }
  }

  return ( 0L );
}

static UWORD *GetSpeedAdr ( struct TuComposerSong *SongStr, UWORD SpeedType, UWORD *SpeedLowerLimit, UWORD *SpeedUpperLimit )
{
  struct PlayerGlobals *PGStr = SongStr->tcso_GlobalData;
  UWORD *SpeedAdr = NULL;

  switch ( SpeedType & 0x07 )
  {
    case 0x00 :
    {
      *SpeedLowerLimit = SongStr->tcso_BPMSpeedLowerLimit;
      *SpeedUpperLimit = SongStr->tcso_BPMSpeedUpperLimit;

      SpeedAdr = (UWORD *) &(PGStr->pg_BPMSpeed);

      break;
    }
    case 0x01 :
    {
      *SpeedLowerLimit = SongStr->tcso_BPMTempoLowerLimit;
      *SpeedUpperLimit = SongStr->tcso_BPMTempoUpperLimit;

      SpeedAdr = (UWORD *) &(PGStr->pg_BPMTempo);

      break;
    }
    case 0x02 :
    {
      *SpeedLowerLimit = SongStr->tcso_SPDLowerLimit;
      *SpeedUpperLimit = SongStr->tcso_SPDUpperLimit;

      SpeedAdr = (UWORD *) &(PGStr->pg_SPDSpeed);

      break;
    }
    case 0x07 :
    {
      *SpeedLowerLimit = 1;
      *SpeedUpperLimit = 0xFFFF;

      SpeedAdr = (UWORD *) &(PGStr->pg_SpeedMul);

      break;
    }
  }

  return ( SpeedAdr );
}

static void SpeedValOk ( struct TuComposerBase *TuCBase, UWORD *SpeedAdr, UWORD SpeedValue, UBYTE SpeedType, UWORD SpeedLowerLimit, UWORD SpeedUpperLimit )
{
  struct PlayerGlobals *PGStr = TuCBase->tcb_PlaySong->tcso_GlobalData;

  if ( SpeedValue < SpeedLowerLimit )
    SpeedValue = SpeedLowerLimit;

  if ( SpeedValue > SpeedUpperLimit )
    SpeedValue = SpeedUpperLimit;

  if ( (SpeedType & 0x07) == 0x07 )
  {
    PGStr->pg_SpeedMul = SpeedValue >> 8U;
    PGStr->pg_SpeedDiv = SpeedValue;
  } else
  {
    *SpeedAdr = SpeedValue;
  }

  if (!( (PGStr->pg_SpeedType = SpeedType) & 0x08 ))
  {
    struct MixerAllocStruct *MixStr = TuCBase->tcb_PlayMixerAlloc;
    unsigned long long Tempo = 0LL;
    UBYTE SpeedMul;

    switch ( SpeedType & 0x07 )
    {
      case 0x00 :
      {
        PGStr->pg_Flags &= ~PGF_SPD;

        break;
      }
      case 0x02 :
      {
        PGStr->pg_Flags |= PGF_SPD;

        break;
      }
    }

    if ( PGStr->pg_Flags & PGF_SPD )
    {
      if ( PGStr->pg_SPDSpeed > 10 )
      {
        Tempo = (ULONG) 989156 * PGStr->pg_SPDSpeed;

        if ( (SpeedMul = PGStr->pg_SpeedMul ))
          Tempo *= SpeedMul;

        if ( (SpeedMul = PGStr->pg_SpeedDiv ))
          Tempo /= SpeedMul;
      } else
      {
        Tempo = TuCBase->tcb_OldSTTable[PGStr->pg_SPDSpeed];
      }
    } else
    {
      Tempo = (PGStr->pg_BPMSpeed) * (PGStr->pg_BPMTempo) << 16;

      if ( (SpeedMul = PGStr->pg_SpeedMul ))
        Tempo *= SpeedMul;

      if ( (SpeedMul = PGStr->pg_SpeedDiv ))
        Tempo /= SpeedMul;
    }

    PGStr->pg_Tempo = Tempo;

    Tempo *= PGStr->pg_RelativeSpeed;
    Tempo >>= 16;

    ExternalMixerChangeTempoBase ( MixStr, Tempo, MixStr->mas_MixLibBase );
  }
}

static void DoSpeedSlide ( struct TuComposerBase *TuCBase, UWORD DataWord )
{
  struct TuComposerSong *SongStr = TuCBase->tcb_PlaySong;
  struct PlayerGlobals *PGStr = SongStr->tcso_GlobalData;
  UWORD *SpeedPtr;
  UWORD SpeedLowerLimit, SpeedUpperLimit;

  if ( (SpeedPtr = GetSpeedAdr ( SongStr, PGStr->pg_SpeedType, &SpeedLowerLimit, &SpeedUpperLimit )))
  {
    UWORD SpeedValue;

    if ( (PGStr->pg_SpeedType & 0x07) == 0x07 )
    {
      SpeedValue = (PGStr->pg_SpeedMul << 8U) + PGStr->pg_SpeedDiv;
    } else
    {
      SpeedValue = *SpeedPtr;
    }

    if ( (SpeedValue += DataWord) < DataWord )
      SpeedValue = 0xFFFF;

    SpeedValOk ( TuCBase, SpeedPtr, SpeedValue, PGStr->pg_SpeedType, SpeedLowerLimit, SpeedUpperLimit );
  }
}

static void DoSpeedSlideSlow ( struct TuComposerBase *TuCBase, UWORD DataWord )
{
  struct TuComposerSong *SongStr = TuCBase->tcb_PlaySong;
  struct PlayerGlobals *PGStr = SongStr->tcso_GlobalData;
  UWORD *SpeedPtr;
  UWORD SpeedLowerLimit, SpeedUpperLimit;

  if ( (SpeedPtr = GetSpeedAdr ( SongStr, PGStr->pg_SpeedType, &SpeedLowerLimit, &SpeedUpperLimit )))
  {
    UWORD SpeedValue;

    if ( (PGStr->pg_SpeedType & 0x07) == 0x07 )
    {
      SpeedValue = (PGStr->pg_SpeedMul << 8U) + PGStr->pg_SpeedDiv;
    } else
    {
      SpeedValue = *SpeedPtr;
    }

    if ( SpeedValue < DataWord )
      DataWord = SpeedValue;

    SpeedValue -= DataWord;

    SpeedValOk ( TuCBase, SpeedPtr, SpeedValue, PGStr->pg_SpeedType, SpeedLowerLimit, SpeedUpperLimit );
  }
}

static void DoGVolumeSlide ( struct TuComposerBase *TuCBase, struct PlayerGlobals *PGStr, UWORD DataWord )
{
  if ( CheckOldTVol ( TuCBase, &DataWord ) )
  {
    UWORD GlobalVolume = (PGStr->pg_GlobalVolume << 8U) + PGStr->pg_GlobalSubVol;

    if ( (GlobalVolume += DataWord) < DataWord )
      GlobalVolume = 0xFFFF;

    PGStr->pg_GlobalVolume = GlobalVolume >> 8U;
    PGStr->pg_GlobalSubVol = GlobalVolume;
  }
}

static void DoGVolumeSlideDn ( struct TuComposerBase *TuCBase, struct PlayerGlobals *PGStr, UWORD DataWord )
{
  if ( CheckOldTVol ( TuCBase, &DataWord ) )
  {
    UWORD GlobalVolume = (PGStr->pg_GlobalVolume << 8U) + PGStr->pg_GlobalSubVol;

    if ( GlobalVolume < DataWord )
      DataWord = GlobalVolume;

    GlobalVolume -= DataWord;

    PGStr->pg_GlobalVolume = GlobalVolume >> 8U;
    PGStr->pg_GlobalSubVol = GlobalVolume;
  }
}

static void DoGPanSlide ( struct PlayerGlobals *PGStr, UWORD DataWord )
{
  UWORD GlobalPanning = (PGStr->pg_GlobalPanning << 8U) + PGStr->pg_GlobalSubPan;

  PGStr->pg_Flags &= ~PGF_Surround;

  if ( (GlobalPanning += DataWord) < DataWord )
    GlobalPanning = 0xFFFF;

  PGStr->pg_GlobalPanning = GlobalPanning >> 8U;
  PGStr->pg_GlobalSubPan = GlobalPanning;
}

static void DoGPanSlideRight ( struct PlayerGlobals *PGStr, UWORD DataWord )
{
  UWORD GlobalPanning = (PGStr->pg_GlobalPanning << 8U) + PGStr->pg_GlobalSubPan;

  PGStr->pg_Flags &= ~PGF_Surround;

  if ( GlobalPanning < DataWord )
    DataWord = GlobalPanning;

  GlobalPanning -= DataWord;

  PGStr->pg_GlobalPanning = GlobalPanning >> 8U;
  PGStr->pg_GlobalSubPan = GlobalPanning;
}

static void PresetTonePorta ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD DataWord )
{
  PHCStr->phc_Flags |= PHCF_TonePorta;
}

static void PresetVibrato ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD DataWord )
{
  PHCStr->phc_Flags |= PHCF_Vibrato;
}

static void PresetNoteDelay ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD DataWord )
{
  if ( (PHCStr->phc_NoteDelay = DataWord) )
  {
    if (!( PHCStr->phc_Flags & PHCF_ExecFX ))
    {
      PHCStr->phc_Flags |= PHCF_ExecFX;

      PHCStr->phc_ExecFX = PHCStr->phc_NoteDelay;
    }
  }
}

static void PresetTremolo ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD DataWord )
{
  PHCStr->phc_Flags |= PHCF_Tremolo;
}

static void PresetSetTransp ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD DataWord )
{
  PHCStr->phc_Flags |= PHCF_SetTranspose;

  PHCStr->phc_Transpose = DataWord >> 8U;
  PHCStr->phc_TransFineTune = DataWord;
}

static ULONG PortaMask[] =
{
  0L, 0L, FSF_FinePorta, FSF_FinePortaDown|FSF_FinePorta, FSF_PortaOnce, FSF_PortaOnceDown|FSF_PortaOnce, FSF_FinePorta|FSF_PortaOnce, FSF_FinePortaDown|FSF_FinePorta|FSF_PortaOnceDown|FSF_PortaOnce
};

static ULONG PortaTrigMask[] =
{
  FSF_FinePortaDown, FSF_FinePortaDown, FSF_PortaOnceDown, FSF_PortaOnceDown, FSF_FinePortaOnceDown, FSF_FinePortaOnceDown
};

static void CheckPorta ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD *EffectByte, UWORD *DataWord, UWORD *Flags )
{
  if ( *DataWord )
  {
    PHCStr->phc_FineSlideFlags &= ~(FSF_FinePortaDown|FSF_PortaOnceDown|FSF_FinePortaOnceDown|FSF_FinePorta|FSF_PortaOnce);

    PHCStr->phc_FineSlideFlags |= PortaMask[(*EffectByte - FX_PORTA_UP)];
  } else
  {
    struct TuComposerTrack *TrackStr = PHCStr->phc_Track;

    if ( TrackStr->tct_CompFlags & TCTCF_Slide )
    {
      if ( (*EffectByte <= FX_PORTA_DOWN) && (!( PHCStr->phc_FineSlideFlags & (FSF_FinePorta|FSF_PortaOnce))) )
        goto TriggerPortaDone;

      *EffectByte = FX_PORTA_UP;

      if ( PHCStr->phc_FineSlideFlags & FSF_FinePorta )
        *EffectByte = FX_F_PORTA_UP;

      if ( PHCStr->phc_FineSlideFlags & FSF_PortaOnce )
        *EffectByte += FX_O_PORTA_UP - FX_PORTA_UP;
    }

    if ( (!( TrackStr->tct_CompFlags & TCTCF_OpSlide )) && (*EffectByte > FX_PORTA_DOWN) )
    {
      UWORD MaskVolFX = *EffectByte;

      *EffectByte -= (FX_PORTA_UP - FX_ARPEGGIO);
      *EffectByte &= -2;

      if ( PHCStr->phc_FineSlideFlags & PortaTrigMask[(MaskVolFX - FX_PORTA_DOWN - 1)] )
        *EffectByte |= 1;

      *EffectByte += FX_PORTA_UP - FX_ARPEGGIO;
    }

TriggerPortaDone:

    *Flags |= FXF_ExecWholeRow;

    if ( *EffectByte >= FX_O_PORTA_UP )
      *Flags &= ~FXF_ExecWholeRow;
  }
}

static void CheckTonePorta ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD *EffectByte, UWORD *DataWord, UWORD *Flags )
{
}

static void CheckNoteSlide ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD *EffectByte, UWORD *DataWord, UWORD *Flags )
{
  UBYTE NoteSlideType;

  if (!( NoteSlideType = *DataWord >> 8U ))
    NoteSlideType = PHCStr->phc_NoteSlideType;

  if ( NoteSlideType & 0xF )
    *Flags &= ~FXF_ExecWholeRow;
}

static ULONG VolSlideMask[] =
{
  0L, FSF_VolSlideDown, FSF_FineVolSlide, FSF_FineVolSlideDown|FSF_FineVolSlide
};

static ULONG VolSlideTrigMask[] =
{
  FSF_VolSlideDown, FSF_VolSlideDown, FSF_FineVolSlideDown, FSF_FineVolSlideDown
};

static void CheckVolSlide ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD *EffectByte, UWORD *DataWord, UWORD *Flags )
{
  if ( *DataWord )
  {
    PHCStr->phc_FineSlideFlags &= ~(FSF_VolSlideDown|FSF_FineVolSlideDown|FSF_FineVolSlide);

    PHCStr->phc_FineSlideFlags |= VolSlideMask[(*EffectByte - FX_VOL_SLD_UP)];
  } else
  {
    struct TuComposerTrack *TrackStr = PHCStr->phc_Track;

    if ( TrackStr->tct_CompFlags & TCTCF_VolSlide )
    {
      *EffectByte = FX_VOL_SLD_UP;

      if ( PHCStr->phc_FineSlideFlags & FSF_FineVolSlide )
        *EffectByte = FX_F_VOLSL_UP;
    }

    if (!( TrackStr->tct_CompFlags & TCTCF_OpVSlide ))
    {
      UWORD MaskVolFX = *EffectByte;

      *EffectByte -= (FX_VOL_SLD_UP - FX_SET_VOLUME);
      *EffectByte &= -2;

      if ( VolSlideTrigMask[(MaskVolFX - FX_VOL_SLD_UP)] & PHCStr->phc_FineSlideFlags )
        *EffectByte |= 1;

      *EffectByte += FX_VOL_SLD_UP - FX_SET_VOLUME;
    }

    *Flags |= FXF_ExecWholeRow;

    if ( *EffectByte >= FX_F_VOLSL_UP )
      *Flags &= ~FXF_ExecWholeRow;
  }
}

static void CheckVolSlTo ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD *EffectByte, UWORD *DataWord, UWORD *Flags )
{
  if ( (*DataWord >> 8U) == 0xFF )
    *Flags &= ~FXF_ExecWholeRow;
}

static ULONG TVolSlideMask[] =
{
  0L, FSF_TrackVolSlideDown, FSF_FineTrackVolSlide, FSF_FineTrackVolSlideDown|FSF_FineTrackVolSlide
};

static ULONG TVolSlideTrigMask[] =
{
  FSF_TrackVolSlideDown, FSF_TrackVolSlideDown, FSF_FineTrackVolSlideDown, FSF_FineTrackVolSlideDown
};

static void CheckTVolSlide ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD *EffectByte, UWORD *DataWord, UWORD *Flags )
{
  if ( *DataWord )
  {
    PHCStr->phc_FineSlideFlags &= ~(FSF_TrackVolSlideDown|FSF_FineTrackVolSlideDown|FSF_FineTrackVolSlide);

    PHCStr->phc_FineSlideFlags |= TVolSlideMask[(*EffectByte - FX_TVOL_SL_UP)];
  } else
  {
    struct TuComposerTrack *TrackStr = PHCStr->phc_Track;

    if ( TrackStr->tct_CompFlags & TCTCF_VolSlide )
    {
      *EffectByte = FX_TVOL_SL_UP;

      if ( PHCStr->phc_FineSlideFlags & FSF_FineTrackVolSlide )
        *EffectByte = FX_F_TVOL_SL_UP;
    }

    if (!( TrackStr->tct_CompFlags & TCTCF_OpVSlide ))
    {
      UWORD MaskVolFX = *EffectByte;

      *EffectByte -= (FX_TVOL_SL_UP - FX_SET_TRK_VOL);
      *EffectByte &= -2;

      if ( TVolSlideTrigMask[(MaskVolFX - FX_TVOL_SL_UP)] & PHCStr->phc_FineSlideFlags )
        *EffectByte |= 1;

      *EffectByte += FX_TVOL_SL_UP - FX_SET_TRK_VOL;
    }

    *Flags |= FXF_ExecWholeRow;

    if ( *EffectByte >= FX_F_TVOL_SL_UP )
      *Flags &= ~FXF_ExecWholeRow;
  }
}

static ULONG PanSlideMask[] =
{
  0L, FSF_PanSlideRight, FSF_FinePanSlide, FSF_FinePanSlideRight|FSF_FinePanSlide
};

static ULONG PanSlideTrigMask[] =
{
  FSF_PanSlideRight, FSF_PanSlideRight, FSF_FinePanSlideRight, FSF_FinePanSlideRight
};

static void CheckPanSlide ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD *EffectByte, UWORD *DataWord, UWORD *Flags )
{
  if ( *DataWord )
  {
    PHCStr->phc_FineSlideFlags &= ~(FSF_PanSlideRight|FSF_FinePanSlideRight|FSF_FinePanSlide);

    PHCStr->phc_FineSlideFlags |= PanSlideMask[(*EffectByte - FX_PAN_SL_LEFT)];
  } else
  {
    struct TuComposerTrack *TrackStr = PHCStr->phc_Track;

    if ( TrackStr->tct_CompFlags & TCTCF_VolSlide )
    {
      *EffectByte = FX_PAN_SL_LEFT;

      if ( PHCStr->phc_FineSlideFlags & FSF_FinePanSlide )
        *EffectByte = FX_F_P_SL_LEFT;
    }

    if (!( TrackStr->tct_CompFlags & TCTCF_OpVSlide ))
    {
      UWORD MaskVolFX = *EffectByte;

      *EffectByte -= (FX_PAN_SL_LEFT - FX_SET_PANNING);
      *EffectByte &= -2;

      if ( PanSlideTrigMask[(MaskVolFX - FX_PAN_SL_LEFT)] & PHCStr->phc_FineSlideFlags )
        *EffectByte |= 1;

      *EffectByte += FX_PAN_SL_LEFT - FX_SET_PANNING;
    }

    *Flags |= FXF_ExecWholeRow;

    if ( *EffectByte >= FX_F_P_SL_LEFT )
      *Flags &= ~FXF_ExecWholeRow;
  }
}

static ULONG TPanSlideMask[] =
{
  0L, FSF_TrackPanSlideRight, FSF_FineTrackPanSlide, FSF_FineTrkPanSlideRight|FSF_FineTrackPanSlide
};

static ULONG TPanSlideTrigMask[] =
{
  FSF_TrackPanSlideRight, FSF_TrackPanSlideRight, FSF_FineTrkPanSlideRight, FSF_FineTrkPanSlideRight
};

static void CheckTPanSlide ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD *EffectByte, UWORD *DataWord, UWORD *Flags )
{
  if ( *DataWord )
  {
    PHCStr->phc_FineSlideFlags &= ~(FSF_TrackPanSlideRight|FSF_FineTrkPanSlideRight|FSF_FineTrackPanSlide);

    PHCStr->phc_FineSlideFlags |= TPanSlideMask[(*EffectByte - FX_TPAN_SL_LEFT)];
  } else
  {
    struct TuComposerTrack *TrackStr = PHCStr->phc_Track;

    if ( TrackStr->tct_CompFlags & TCTCF_VolSlide )
    {
      *EffectByte = FX_TPAN_SL_LEFT;

      if ( PHCStr->phc_FineSlideFlags & FSF_FineTrackPanSlide )
        *EffectByte = FX_F_TP_SL_LEFT;
    }

    if (!( TrackStr->tct_CompFlags & TCTCF_OpVSlide ))
    {
      UWORD MaskVolFX = *EffectByte;

      *EffectByte -= (FX_TPAN_SL_LEFT - FX_SET_TRK_PAN);
      *EffectByte &= -2;

      if ( TPanSlideTrigMask[(MaskVolFX - FX_TPAN_SL_LEFT)] & PHCStr->phc_FineSlideFlags )
        *EffectByte |= 1;

      *EffectByte += FX_TPAN_SL_LEFT - FX_SET_TRK_PAN;
    }

    *Flags |= FXF_ExecWholeRow;

    if ( *EffectByte >= FX_F_TP_SL_LEFT )
      *Flags &= ~FXF_ExecWholeRow;
  }
}

static ULONG SpeedSlideMask[] =
{
  0L, FSF_SpeedSlideSlower, FSF_FineSpeedSlide, FSF_FineSpeedSlideSlower|FSF_FineSpeedSlide
};

static ULONG SpeedSlideTrigMask[] =
{
  FSF_SpeedSlideSlower, FSF_SpeedSlideSlower, FSF_FineSpeedSlideSlower, FSF_FineSpeedSlideSlower
};

static void CheckSpeedSlide ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD *EffectByte, UWORD *DataWord, UWORD *Flags )
{
  if ( *DataWord )
  {
    PHCStr->phc_FineSlideFlags &= ~(FSF_SpeedSlideSlower|FSF_FineSpeedSlideSlower|FSF_FineSpeedSlide);

    PHCStr->phc_FineSlideFlags |= SpeedSlideMask[(*EffectByte - FX_SPD_SLD_FAST)];
  } else
  {
    struct TuComposerTrack *TrackStr = PHCStr->phc_Track;

    if ( TrackStr->tct_CompFlags & TCTCF_VolSlide )
    {
      *EffectByte = FX_SPD_SLD_FAST;

      if ( PHCStr->phc_FineSlideFlags & FSF_FineSpeedSlide )
        *EffectByte = FX_F_S_SLD_FAST;
    }

    if (!( TrackStr->tct_CompFlags & TCTCF_OpVSlide ))
    {
      UWORD MaskVolFX = *EffectByte;

      *EffectByte -= (FX_SPD_SLD_FAST - FX_SET_SPEED);
      *EffectByte &= -2;

      if ( SpeedSlideTrigMask[(MaskVolFX - FX_SPD_SLD_FAST)] & PHCStr->phc_FineSlideFlags )
        *EffectByte |= 1;

      *EffectByte += FX_SPD_SLD_FAST - FX_SET_SPEED;
    }

    *Flags |= FXF_ExecWholeRow;

    if ( *EffectByte >= FX_F_S_SLD_FAST )
      *Flags &= ~FXF_ExecWholeRow;
  }
}

static void CheckChCtrl ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD *EffectByte, UWORD *DataWord, UWORD *Flags )
{
}

static ULONG GVolSlideMask[] =
{
  0L, FSF_GlobalVolSlideDown, FSF_FineGlobalVolSlide, FSF_FineGVolSlideDown|FSF_FineGlobalVolSlide
};

static ULONG GVolSlideTrigMask[] =
{
  FSF_GlobalVolSlideDown, FSF_GlobalVolSlideDown, FSF_FineGVolSlideDown, FSF_FineGVolSlideDown
};

static void CheckGVolSlide ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD *EffectByte, UWORD *DataWord, UWORD *Flags )
{
  if ( *DataWord )
  {
    PHCStr->phc_FineSlideFlags &= ~(FSF_GlobalVolSlideDown|FSF_FineGVolSlideDown|FSF_FineGlobalVolSlide);

    PHCStr->phc_FineSlideFlags |= GVolSlideMask[(*EffectByte - FX_G_VOLSL_UP)];
  } else
  {
    struct TuComposerTrack *TrackStr = PHCStr->phc_Track;

    if ( TrackStr->tct_CompFlags & TCTCF_VolSlide )
    {
      *EffectByte = FX_G_VOLSL_UP;

      if ( PHCStr->phc_FineSlideFlags & FSF_FineGlobalVolSlide )
        *EffectByte = FX_F_G_VOL_UP;
    }

    if (!( TrackStr->tct_CompFlags & TCTCF_OpVSlide ))
    {
      UWORD MaskVolFX = *EffectByte;

      *EffectByte &= -2;

      if ( GVolSlideTrigMask[(MaskVolFX - FX_G_VOLSL_UP)] & PHCStr->phc_FineSlideFlags )
        *EffectByte |= 1;
    }

    *Flags |= FXF_ExecWholeRow;

    if ( *EffectByte >= FX_F_G_VOL_UP )
      *Flags &= ~FXF_ExecWholeRow;
  }
}

static ULONG GPanSlideMask[] =
{
  0L, FSF_GlobalPanSlideRight, FSF_FineGlobalPanSlide, FSF_FineGPanSlideRight|FSF_FineGlobalPanSlide
};

static ULONG GPanSlideTrigMask[] =
{
  FSF_GlobalPanSlideRight, FSF_GlobalPanSlideRight, FSF_FineGPanSlideRight, FSF_FineGPanSlideRight
};

static void CheckGPanSlide ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD *EffectByte, UWORD *DataWord, UWORD *Flags )
{
  if ( *DataWord )
  {
    PHCStr->phc_FineSlideFlags &= ~(FSF_GlobalPanSlideRight|FSF_FineGPanSlideRight|FSF_FineGlobalPanSlide);

    PHCStr->phc_FineSlideFlags |= GPanSlideMask[(*EffectByte - FX_GPANSL_LEFT)];
  } else
  {
    struct TuComposerTrack *TrackStr = PHCStr->phc_Track;

    if ( TrackStr->tct_CompFlags & TCTCF_VolSlide )
    {
      *EffectByte = FX_GPANSL_LEFT;

      if ( PHCStr->phc_FineSlideFlags & FSF_FineGlobalPanSlide )
        *EffectByte = FX_FGP_SL_LEFT;
    }

    if (!( TrackStr->tct_CompFlags & TCTCF_OpVSlide ))
    {
      UWORD MaskVolFX = *EffectByte;

      *EffectByte &= -2;

      if ( GPanSlideTrigMask[(MaskVolFX - FX_GPANSL_LEFT)] & PHCStr->phc_FineSlideFlags )
        *EffectByte |= 1;
    }

    *Flags |= FXF_ExecWholeRow;

    if ( *EffectByte >= FX_FGP_SL_LEFT )
      *Flags &= ~FXF_ExecWholeRow;
  }
}

static void Arpeggio ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  BYTE ArpFirst, ArpSecond;
  WORD ArpValue;

  if (!( DataWord ))
    DataWord = (PHCStr->phc_ArpeggioFirst << 8U) + PHCStr->phc_ArpeggioSecond;

  PHCStr->phc_ArpeggioFirst = ArpFirst = DataWord >> 8U;
  PHCStr->phc_ArpeggioSecond = ArpSecond = DataWord;

  switch ( PHCStr->phc_ArpeggioTick )
  {
    case 0 :
    {
      ArpValue = 0;

      break;
    }
    case 1 :
    {
      ArpValue = ArpFirst;

      break;
    }
    default :
    {
      ArpValue = ArpSecond;

      PHCStr->phc_ArpeggioTick -= 3;

      break;
    }
  }

  if ( PCStr->pc_HostChannel == Channel )
  {
    ULONG Frequency, ArpFreq, OldFreq;
    UWORD Octave;
    WORD Note;

    Octave = ArpValue / NUMNOTES;
    Note = ArpValue % NUMNOTES;

    if ( Note < 0 )
    {
      Octave--;

      Note += NUMNOTES;
    }

    OldFreq = PCStr->pc_Frequency;
    Frequency = OldFreq + PHCStr->phc_ArpeggioFrequency;

    ArpFreq = TuCBase->tcb_FrequencyTable[Note];

    if ( (WORD) Octave < 0 )
    {
      Octave = -Octave;

      ArpFreq = ((unsigned long long) Frequency * ArpFreq) >> (16UL + Octave);
    } else
    {
      ArpFreq <<= Octave;

      ArpFreq = ((unsigned long long) Frequency * ArpFreq) >> 16ULL;
    }

    PHCStr->phc_ArpeggioFrequency += OldFreq - ArpFreq;

    PCStr->pc_Frequency = ArpFreq;
  }

  PHCStr->phc_ArpeggioTick++;
}

static void PortaUp ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct TuComposerTrack *TrackStr;

  if (!( DataWord ))
    DataWord = PHCStr->phc_PortaUpValue;

  PortaSlideUp ( TuCBase, PHCStr, PCStr, DataWord, 16, 8-4, Channel );

  TrackStr = PHCStr->phc_Track;

  if ( TrackStr->tct_CompFlags & TCTCF_VolPitch )
    VolSlUpOk ( PHCStr, DataWord );

  PortaUpOk ( PHCStr, DataWord );
}

static void PortaDn ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct TuComposerTrack *TrackStr;

  if (!( DataWord ))
    DataWord = PHCStr->phc_PortaDownValue;

  PortaSlideDn ( TuCBase, PHCStr, PCStr, DataWord, 16, 8-4, Channel );

  TrackStr = PHCStr->phc_Track;

  if ( TrackStr->tct_CompFlags & TCTCF_VolPitch )
    VolSlDnOk ( PHCStr, DataWord );

  PortaDnOk ( PHCStr, DataWord );
}

static void FPortaUp ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct TuComposerTrack *TrackStr;
  UWORD v0, v1, v3, v4, v5, v8;

  if (!( DataWord ))
    DataWord = PHCStr->phc_FinePortaUpValue;

  PortaSlideUp ( TuCBase, PHCStr, PCStr, DataWord, 1, 8, Channel );

  TrackStr = PHCStr->phc_Track;

  if ( TrackStr->tct_CompFlags & TCTCF_VolPitch )
    VolSlUpOk ( PHCStr, DataWord );

  v0 = PHCStr->phc_PortaUpValue;
  v1 = PHCStr->phc_PortaDownValue;
  v3 = PHCStr->phc_PortaUpOnceValue;
  v4 = PHCStr->phc_PortaDownOnceValue;
  v5 = PHCStr->phc_FinePortaUpOnceValue;
  v8 = PHCStr->phc_FinePortaDownOnceValue;

  if ( TrackStr->tct_CompFlags & TCTCF_Slide )
  {
    v0 = DataWord;
    v3 = DataWord;
    v5 = DataWord;
  }

  if (!( TrackStr->tct_CompFlags & TCTCF_OpSlide ))
  {
    PHCStr->phc_FinePortaDownValue = DataWord;
    v1 = v0;
    v4 = v3;
    v8 = v5;
  }

  PHCStr->phc_PortaUpValue = v0;
  PHCStr->phc_PortaDownValue = v1;
  PHCStr->phc_FinePortaUpValue = DataWord;

  PHCStr->phc_PortaUpOnceValue = v3;
  PHCStr->phc_PortaDownOnceValue = v4;
  PHCStr->phc_FinePortaUpOnceValue = v5;
  PHCStr->phc_FinePortaDownOnceValue = v8;

  if ( TrackStr->tct_CompFlags & TCTCF_TPorta )
  {
    v1 = PHCStr->phc_TonePortaValue;
    v4 = PHCStr->phc_TonePortaOnceValue;
    v8 = PHCStr->phc_FineTonePortaOnceValue;

    if ( TrackStr->tct_CompFlags & TCTCF_Slide )
    {
      v1 = v0;
      v4 = v3;
      v8 = v5;
    }

    PHCStr->phc_TonePortaValue = v1;
    PHCStr->phc_FineTonePortaValue = DataWord;
    PHCStr->phc_TonePortaOnceValue = v4;
    PHCStr->phc_FineTonePortaOnceValue = v8;
  }
}

static void FPortaDn ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct TuComposerTrack *TrackStr;
  UWORD v0, v1, v3, v4, v5, v8;

  if (!( DataWord ))
    DataWord = PHCStr->phc_FinePortaDownValue;

  PortaSlideDn ( TuCBase, PHCStr, PCStr, DataWord, 1, 8, Channel );

  TrackStr = PHCStr->phc_Track;

  if ( TrackStr->tct_CompFlags & TCTCF_VolPitch )
    VolSlDnOk ( PHCStr, DataWord );

  v0 = PHCStr->phc_PortaUpValue;
  v1 = PHCStr->phc_PortaDownValue;
  v3 = PHCStr->phc_PortaUpOnceValue;
  v4 = PHCStr->phc_PortaDownOnceValue;
  v5 = PHCStr->phc_FinePortaUpOnceValue;
  v8 = PHCStr->phc_FinePortaDownOnceValue;

  if ( TrackStr->tct_CompFlags & TCTCF_Slide )
  {
    v1 = DataWord;
    v4 = DataWord;
    v8 = DataWord;
  }

  if (!( TrackStr->tct_CompFlags & TCTCF_OpSlide ))
  {
    PHCStr->phc_FinePortaUpValue = DataWord;
    v0 = v1;
    v3 = v4;
    v5 = v8;
  }

  PHCStr->phc_PortaUpValue = v0;
  PHCStr->phc_PortaDownValue = v1;
  PHCStr->phc_FinePortaDownValue = DataWord;
  PHCStr->phc_PortaUpOnceValue = v3;
  PHCStr->phc_PortaDownOnceValue = v4;
  PHCStr->phc_FinePortaUpOnceValue = v5;
  PHCStr->phc_FinePortaDownOnceValue = v8;

  if ( TrackStr->tct_CompFlags & TCTCF_TPorta )
  {
    v0 = PHCStr->phc_TonePortaValue;
    v3 = PHCStr->phc_TonePortaOnceValue;
    v5 = PHCStr->phc_FineTonePortaOnceValue;

    if ( TrackStr->tct_CompFlags & TCTCF_Slide )
    {
      v0 = v1;
      v3 = v4;
      v5 = v8;
    }

    PHCStr->phc_TonePortaValue = v0;
    PHCStr->phc_FineTonePortaValue = DataWord;
    PHCStr->phc_TonePortaOnceValue = v3;
    PHCStr->phc_FineTonePortaOnceValue = v5;
  }
}

static void PortaUpOnce ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct TuComposerTrack *TrackStr;

  if (!( DataWord ))
    DataWord = PHCStr->phc_PortaUpOnceValue;

  PortaSlideUp ( TuCBase, PHCStr, PCStr, DataWord, 16, 8-4, Channel );

  TrackStr = PHCStr->phc_Track;

  if ( TrackStr->tct_CompFlags & TCTCF_VolPitch )
    FVolSlUpOk ( PHCStr, DataWord );

  PortaUpOnceOk ( PHCStr, DataWord );
}

static void PortaDnOnce ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct TuComposerTrack *TrackStr;

  if (!( DataWord ))
    DataWord = PHCStr->phc_PortaDownOnceValue;

  PortaSlideDn ( TuCBase, PHCStr, PCStr, DataWord, 16, 8-4, Channel );

  TrackStr = PHCStr->phc_Track;

  if ( TrackStr->tct_CompFlags & TCTCF_VolPitch )
    FVolSlDnOk ( PHCStr, DataWord );

  PortaDnOnceOk ( PHCStr, DataWord );
}

static void FPortaUpOnce ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct TuComposerTrack *TrackStr;
  UWORD v0, v1, v3, v4, v5, v8;

  if (!( DataWord ))
    DataWord = PHCStr->phc_FinePortaUpOnceValue;

  PortaSlideUp ( TuCBase, PHCStr, PCStr, DataWord, 1, 8, Channel );

  TrackStr = PHCStr->phc_Track;

  if ( TrackStr->tct_CompFlags & TCTCF_VolPitch )
    FVolSlUpOk ( PHCStr, DataWord );

  v0 = PHCStr->phc_PortaUpValue;
  v1 = PHCStr->phc_PortaDownValue;
  v3 = PHCStr->phc_FinePortaUpValue;
  v4 = PHCStr->phc_FinePortaDownValue;
  v5 = PHCStr->phc_PortaUpOnceValue;
  v8 = PHCStr->phc_PortaDownOnceValue;

  if ( TrackStr->tct_CompFlags & TCTCF_Slide )
  {
    v0 = DataWord;
    v3 = DataWord;
    v5 = DataWord;
  }

  if (!( TrackStr->tct_CompFlags & TCTCF_OpSlide ))
  {
    PHCStr->phc_FinePortaDownOnceValue = DataWord;
    v1 = v0;
    v4 = v3;
    v8 = v5;
  }

  PHCStr->phc_FinePortaUpOnceValue = DataWord;

  PHCStr->phc_PortaUpValue = v0;
  PHCStr->phc_PortaDownValue = v1;
  PHCStr->phc_FinePortaUpValue = v3;
  PHCStr->phc_FinePortaDownValue = v4;
  PHCStr->phc_PortaUpOnceValue = v5;
  PHCStr->phc_PortaDownOnceValue = v8;

  if ( TrackStr->tct_CompFlags & TCTCF_TPorta )
  {
    v1 = PHCStr->phc_TonePortaValue;
    v4 = PHCStr->phc_FineTonePortaValue;
    v8 = PHCStr->phc_FineTonePortaOnceValue;

    if ( TrackStr->tct_CompFlags & TCTCF_Slide )
    {
      v1 = v0;
      v4 = v3;
      v8 = v5;
    }

    PHCStr->phc_TonePortaValue = v1;
    PHCStr->phc_FineTonePortaValue = v4;
    PHCStr->phc_TonePortaOnceValue = v8;
    PHCStr->phc_FineTonePortaOnceValue = DataWord;
  }
}

static void FPortaDnOnce ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct TuComposerTrack *TrackStr;
  UWORD v0, v1, v3, v4, v5, v8;

  if (!( DataWord ))
    DataWord = PHCStr->phc_FinePortaDownOnceValue;

  PortaSlideDn ( TuCBase, PHCStr, PCStr, DataWord, 1, 8, Channel );

  TrackStr = PHCStr->phc_Track;

  if ( TrackStr->tct_CompFlags & TCTCF_VolPitch )
    FVolSlDnOk ( PHCStr, DataWord );

  v0 = PHCStr->phc_PortaUpValue;
  v1 = PHCStr->phc_PortaDownValue;
  v3 = PHCStr->phc_FinePortaUpValue;
  v4 = PHCStr->phc_FinePortaDownValue;
  v5 = PHCStr->phc_PortaUpOnceValue;
  v8 = PHCStr->phc_PortaDownOnceValue;

  if ( TrackStr->tct_CompFlags & TCTCF_Slide )
  {
    v0 = DataWord;
    v3 = DataWord;
    v5 = DataWord;
  }

  if (!( TrackStr->tct_CompFlags & TCTCF_OpSlide ))
  {
    PHCStr->phc_FinePortaUpOnceValue = DataWord;
    v1 = v0;
    v4 = v3;
    v8 = v5;
  }

  PHCStr->phc_FinePortaDownOnceValue = DataWord;

  PHCStr->phc_PortaUpValue = v0;
  PHCStr->phc_PortaDownValue = v1;
  PHCStr->phc_FinePortaUpValue = v3;
  PHCStr->phc_FinePortaDownValue = v4;
  PHCStr->phc_PortaUpOnceValue = v5;
  PHCStr->phc_PortaDownOnceValue = v8;

  if ( TrackStr->tct_CompFlags & TCTCF_TPorta )
  {
    v0 = PHCStr->phc_TonePortaValue;
    v3 = PHCStr->phc_FineTonePortaValue;
    v5 = PHCStr->phc_FineTonePortaOnceValue;

    if ( TrackStr->tct_CompFlags & TCTCF_Slide )
    {
      v0 = v1;
      v3 = v4;
      v5 = v8;
    }

    PHCStr->phc_TonePortaValue = v0;
    PHCStr->phc_FineTonePortaValue = v3;
    PHCStr->phc_TonePortaOnceValue = v5;
    PHCStr->phc_FineTonePortaOnceValue = DataWord;
  }
}

static void TonePorta ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  ULONG TPortaTargetPitch;

  if (!( DataWord ))
    DataWord = PHCStr->phc_TonePortaValue;

  if ( (TPortaTargetPitch = PHCStr->phc_TonePortaTargetPitch) )
  {
    struct TuComposerTrack *TrackStr = PHCStr->phc_Track;
    UWORD v0, v1, v3;

    if ( PCStr->pc_HostChannel == Channel )
    {
      if ( TPortaTargetPitch <= PCStr->pc_Frequency )
      {
        PortaSlideDn ( TuCBase, PHCStr, PCStr, DataWord, 16, 8-4, Channel );

        if ( TPortaTargetPitch >= PCStr->pc_Frequency )
        {
          PCStr->pc_Frequency = TPortaTargetPitch;

          PHCStr->phc_TonePortaTargetPitch = 0L;
        }
      } else
      {
        PortaSlideUp ( TuCBase, PHCStr, PCStr, DataWord, 16, 8-4, Channel );

        if ( (!( PCStr->pc_Frequency )) || (TPortaTargetPitch <= PCStr->pc_Frequency) )
        {
          PCStr->pc_Frequency = TPortaTargetPitch;

          PHCStr->phc_TonePortaTargetPitch = 0L;
        }
      }
    }

    v0 = PHCStr->phc_FineTonePortaValue;
    v1 = PHCStr->phc_TonePortaOnceValue;
    v3 = PHCStr->phc_FineTonePortaOnceValue;

    if ( TrackStr->tct_CompFlags & TCTCF_Slide )
    {
      v0 = DataWord;
      v1 = DataWord;
      v3 = DataWord;
    }

    PHCStr->phc_TonePortaValue = DataWord;
    PHCStr->phc_FineTonePortaValue = v0;
    PHCStr->phc_TonePortaOnceValue = v1;
    PHCStr->phc_FineTonePortaOnceValue = v3;

    if ( TrackStr->tct_CompFlags & TCTCF_TPorta )
    {
      PHCStr->phc_PortaUpValue = DataWord;
      PHCStr->phc_PortaDownValue = DataWord;
      PHCStr->phc_FinePortaUpValue = DataWord;
      PHCStr->phc_FinePortaDownValue = DataWord;
      PHCStr->phc_PortaUpOnceValue = DataWord;
      PHCStr->phc_PortaDownOnceValue = DataWord;
      PHCStr->phc_FinePortaUpOnceValue = DataWord;
      PHCStr->phc_FinePortaDownOnceValue = DataWord;
    }
  }
}

static void FTonePorta ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  ULONG TPortaTargetPitch;

  if (!( DataWord ))
    DataWord = PHCStr->phc_FineTonePortaValue;

  if ( (TPortaTargetPitch = PHCStr->phc_TonePortaTargetPitch) )
  {
    struct TuComposerTrack *TrackStr = PHCStr->phc_Track;
    UWORD v0, v1, v3;

    if ( PCStr->pc_HostChannel == Channel )
    {
      if ( TPortaTargetPitch <= PCStr->pc_Frequency )
      {
        PortaSlideDn ( TuCBase, PHCStr, PCStr, DataWord, 1, 8, Channel );

        if ( TPortaTargetPitch >= PCStr->pc_Frequency )
        {
          PCStr->pc_Frequency = TPortaTargetPitch;

          PHCStr->phc_TonePortaTargetPitch = 0L;
        }
      } else
      {
        PortaSlideUp ( TuCBase, PHCStr, PCStr, DataWord, 1, 8, Channel );

        if ( (!( PCStr->pc_Frequency )) || (TPortaTargetPitch <= PCStr->pc_Frequency) )
        {
          PCStr->pc_Frequency = TPortaTargetPitch;

          PHCStr->phc_TonePortaTargetPitch = 0L;
        }
      }
    }

    v0 = PHCStr->phc_TonePortaValue;
    v1 = PHCStr->phc_TonePortaOnceValue;
    v3 = PHCStr->phc_FineTonePortaOnceValue;

    if ( TrackStr->tct_CompFlags & TCTCF_Slide )
    {
      v0 = DataWord;
      v1 = DataWord;
      v3 = DataWord;
    }

    PHCStr->phc_TonePortaValue = v0;
    PHCStr->phc_FineTonePortaValue = v1;
    PHCStr->phc_TonePortaOnceValue = DataWord;
    PHCStr->phc_FineTonePortaOnceValue = v3;

    if ( TrackStr->tct_CompFlags & TCTCF_TPorta )
    {
      PHCStr->phc_PortaUpValue = DataWord;
      PHCStr->phc_PortaDownValue = DataWord;
      PHCStr->phc_FinePortaUpValue = DataWord;
      PHCStr->phc_FinePortaDownValue = DataWord;
      PHCStr->phc_PortaUpOnceValue = DataWord;
      PHCStr->phc_PortaDownOnceValue = DataWord;
      PHCStr->phc_FinePortaUpOnceValue = DataWord;
      PHCStr->phc_FinePortaDownOnceValue = DataWord;
    }
  }
}

static void TPortaOnce ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  ULONG TPortaTargetPitch;

  if (!( DataWord ))
    DataWord = PHCStr->phc_TonePortaOnceValue;

  if ( (TPortaTargetPitch = PHCStr->phc_TonePortaTargetPitch) )
  {
    struct TuComposerTrack *TrackStr = PHCStr->phc_Track;
    UWORD v0, v1, v3;

    if ( PCStr->pc_HostChannel == Channel )
    {
      if ( TPortaTargetPitch <= PCStr->pc_Frequency )
      {
        PortaSlideDn ( TuCBase, PHCStr, PCStr, DataWord, 16, 8-4, Channel );

        if ( TPortaTargetPitch >= PCStr->pc_Frequency )
        {
          PCStr->pc_Frequency = TPortaTargetPitch;

          PHCStr->phc_TonePortaTargetPitch = 0L;
        }
      } else
      {
        PortaSlideUp ( TuCBase, PHCStr, PCStr, DataWord, 16, 8-4, Channel );

        if ( (!( PCStr->pc_Frequency )) || (TPortaTargetPitch <= PCStr->pc_Frequency) )
        {
          PCStr->pc_Frequency = TPortaTargetPitch;

          PHCStr->phc_TonePortaTargetPitch = 0L;
        }
      }
    }

    v0 = PHCStr->phc_TonePortaValue;
    v1 = PHCStr->phc_FineTonePortaValue;
    v3 = PHCStr->phc_FineTonePortaOnceValue;

    if ( TrackStr->tct_CompFlags & TCTCF_Slide )
    {
      v0 = DataWord;
      v1 = DataWord;
      v3 = DataWord;
    }

    PHCStr->phc_TonePortaValue = v0;
    PHCStr->phc_FineTonePortaValue = v1;
    PHCStr->phc_TonePortaOnceValue = DataWord;
    PHCStr->phc_FineTonePortaOnceValue = v3;

    if ( TrackStr->tct_CompFlags & TCTCF_TPorta )
    {
      PHCStr->phc_PortaUpValue = DataWord;
      PHCStr->phc_PortaDownValue = DataWord;
      PHCStr->phc_FinePortaUpValue = DataWord;
      PHCStr->phc_FinePortaDownValue = DataWord;
      PHCStr->phc_PortaUpOnceValue = DataWord;
      PHCStr->phc_PortaDownOnceValue = DataWord;
      PHCStr->phc_FinePortaUpOnceValue = DataWord;
      PHCStr->phc_FinePortaDownOnceValue = DataWord;
    }
  }
}

static void FTPortaOnce ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  ULONG TPortaTargetPitch;

  if (!( DataWord ))
    DataWord = PHCStr->phc_FineTonePortaOnceValue;

  if ( (TPortaTargetPitch = PHCStr->phc_TonePortaTargetPitch) )
  {
    struct TuComposerTrack *TrackStr = PHCStr->phc_Track;
    UWORD v0, v1, v3;

    if ( PCStr->pc_HostChannel == Channel )
    {
      if ( TPortaTargetPitch <= PCStr->pc_Frequency )
      {
        PortaSlideDn ( TuCBase, PHCStr, PCStr, DataWord, 1, 8, Channel );

        if ( TPortaTargetPitch >= PCStr->pc_Frequency )
        {
          PCStr->pc_Frequency = TPortaTargetPitch;

          PHCStr->phc_TonePortaTargetPitch = 0L;
        }
      } else
      {
        PortaSlideUp ( TuCBase, PHCStr, PCStr, DataWord, 1, 8, Channel );

        if ( (!( PCStr->pc_Frequency )) || (TPortaTargetPitch <= PCStr->pc_Frequency) )
        {
          PCStr->pc_Frequency = TPortaTargetPitch;

          PHCStr->phc_TonePortaTargetPitch = 0L;
        }
      }
    }

    v0 = PHCStr->phc_TonePortaValue;
    v1 = PHCStr->phc_FineTonePortaValue;
    v3 = PHCStr->phc_TonePortaOnceValue;

    if ( TrackStr->tct_CompFlags & TCTCF_Slide )
    {
      v0 = DataWord;
      v1 = DataWord;
      v3 = DataWord;
    }

    PHCStr->phc_TonePortaValue = v0;
    PHCStr->phc_FineTonePortaValue = v1;
    PHCStr->phc_TonePortaOnceValue = v3;
    PHCStr->phc_FineTonePortaOnceValue = DataWord;

    if ( TrackStr->tct_CompFlags & TCTCF_TPorta )
    {
      PHCStr->phc_PortaUpValue = DataWord;
      PHCStr->phc_PortaDownValue = DataWord;
      PHCStr->phc_FinePortaUpValue = DataWord;
      PHCStr->phc_FinePortaDownValue = DataWord;
      PHCStr->phc_PortaUpOnceValue = DataWord;
      PHCStr->phc_PortaDownOnceValue = DataWord;
      PHCStr->phc_FinePortaUpOnceValue = DataWord;
      PHCStr->phc_FinePortaDownOnceValue = DataWord;
    }
  }
}

static void NoteSlide ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  UWORD NoteSlideValue, NoteSlideType;

  if (!( NoteSlideValue = DataWord & 0xFF ))
    NoteSlideValue = PHCStr->phc_NoteSlideValue;

  PHCStr->phc_NoteSlideValue = NoteSlideValue;

  if (!( NoteSlideType = ( DataWord >> 8U )))
    NoteSlideType = PHCStr->phc_NoteSlideType;

  PHCStr->phc_NoteSlideType = NoteSlideType;

  if (!( NoteSlideType & 0x10 ))
    NoteSlideValue = -NoteSlideValue;

  NoteSlideValue += PHCStr->phc_FinalNote;
  PHCStr->phc_FinalNote = NoteSlideValue;

  if ( PCStr->pc_HostChannel == Channel )
    PCStr->pc_Frequency = GetTonePitch ( TuCBase, PHCStr, PCStr, NoteSlideValue );
}

static void Vibrato ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  UWORD VibratoRate;
  BYTE VibratoDepth;

  if (!( VibratoRate = ( DataWord >> 8U )))
    VibratoRate = PHCStr->phc_VibratoRate;

  PHCStr->phc_VibratoRate = VibratoRate;

  VibratoDepth = DataWord;

  DoVibrato ( TuCBase, PHCStr, PCStr, Channel, VibratoRate, ((WORD) VibratoDepth << 2) );
}

static void FVibrato ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  UWORD VibratoRate;

  if (!( VibratoRate = ( DataWord >> 8U )))
    VibratoRate = PHCStr->phc_VibratoRate;

  PHCStr->phc_VibratoRate = VibratoRate;

  DoVibrato ( TuCBase, PHCStr, PCStr, Channel, VibratoRate, (BYTE) DataWord );
}

static void DoKeyOff ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  if ( DataWord >= PHCStr->phc_TempoCounter )
    PlayKeyOff ( PCStr );
}

static void HoldDelay ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{

}

static void NoteFade ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  if ( DataWord >= PHCStr->phc_TempoCounter )
  {
    PHCStr->phc_EffectsUsed[(EffectByte >> 3U)] |= (1 << (7 - (EffectByte & 7)));

    if ( PCStr->pc_HostChannel == Channel )
      PCStr->pc_Flags |= PCF_Fading;
  }
}

static void NoteCut ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  if ( (DataWord & 0xFFF) >= PHCStr->phc_TempoCounter )
  {
    PHCStr->phc_EffectsUsed[(EffectByte >> 3U)] |= (1 << (7 - (EffectByte & 7)));

    if ( PCStr->pc_HostChannel == Channel )
    {
      PCStr->pc_Volume = 0;
      PCStr->pc_SubVol = 0;

      if ( DataWord & 0xF000 )
      {
        PHCStr->phc_Instrument = NULL;
        PHCStr->phc_Sample = NULL;
        PCStr->pc_Flags = 0;
      }
    }
  }
}

static void NoteDelay ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
}

static void Tremor ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  UBYTE TremorOff, TremorOn;

  PHCStr->phc_Flags |= PHCF_TremorExec;

  if (!( TremorOff = DataWord ))
    TremorOff = PHCStr->phc_TremorOffTicks;

  PHCStr->phc_TremorOffTicks = TremorOff;

  if (!( TremorOn = ( DataWord >> 8U )))
    TremorOn = PHCStr->phc_TremorOnTicks;

  PHCStr->phc_TremorOnTicks = TremorOn;

  if (!( PHCStr->phc_Flags & PHCF_TremorOff ))
    TremorOff = TremorOn;

  if ( TremorOff <= PHCStr->phc_TremorCount )
  {
    PHCStr->phc_Flags ^= PHCF_TremorOff;

    PHCStr->phc_TremorCount = 0;
  }

  PHCStr->phc_TremorCount++;
}

static void Retrig ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  UWORD RetrigTick = DataWord & 0x7FFF, RetrigTickCount;

  if ( (DataWord & 0x8000) && DataWord )
    RetrigTick = PHCStr->phc_Tempo / RetrigTick;

  if ( (RetrigTickCount = PHCStr->phc_RetrigTickCount) && --RetrigTick )
  {
    if ( RetrigTick <= RetrigTickCount )
      RetrigTickCount = -1;
  } else
  {
    if ( PCStr->pc_HostChannel == Channel )
      PCStr->pc_ChannelData.ms_SamplePos = 0L;
  }

  PHCStr->phc_RetrigTickCount = ++RetrigTickCount;
}

static void MulRetrig ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  UBYTE MultiRetrigTick, MultiRetrigVolChg;
  UWORD RetrigTickCount;
  ULONG Volume;

  if (!( MultiRetrigTick = ( DataWord >> 8U )))
    MultiRetrigTick = PHCStr->phc_MultiRetrigTick;

  PHCStr->phc_MultiRetrigTick = MultiRetrigTick;

  if (!( MultiRetrigVolChg = DataWord ))
    MultiRetrigVolChg = PHCStr->phc_MultiRetrigVolChg;

  PHCStr->phc_MultiRetrigVolChg = MultiRetrigVolChg;

  if ( (RetrigTickCount = PHCStr->phc_RetrigTickCount) || (PHCStr->phc_Flags & PHCF_RetrigNote) || (PCStr->pc_HostChannel != Channel) )
  {
    PHCStr->phc_Flags &= ~PHCF_RetrigNote;
  } else
  {
    if ( (BYTE) MultiRetrigVolChg < 0 )
    {
      UBYTE MultiRetrigScale = 4;

      if (!( TuCBase->tcb_PlaySong->tcso_CompFlags & TCSOCF_OldVolumes ))
        MultiRetrigScale = PHCStr->phc_MultiRetrigScale;

      if ( (BYTE) (MultiRetrigVolChg -= 0xBF) >= 0 )
      {
        Volume = MultiRetrigVolChg * MultiRetrigScale;

        if ( PCStr->pc_Volume >= Volume )
        {
          PCStr->pc_Volume -= Volume;
        } else
        {
          PCStr->pc_Volume = 0;
          PCStr->pc_SubVol = 0;
        }
      } else
      {
        Volume = ((MultiRetrigVolChg + 0x40) * MultiRetrigScale) + PCStr->pc_Volume;

        if ( Volume < 0x100 )
        {
          PCStr->pc_Volume = Volume;
        } else
        {
          PCStr->pc_Volume = 0xFF;
          PCStr->pc_SubVol = 0xFF;
        }
      }
    } else
    {
      UBYTE MulVol, DivVol;

      Volume = (PCStr->pc_Volume << 8UL);

      if ( PHCStr->phc_Flags & PHCF_SubSlideRetrig )
        Volume += PCStr->pc_SubVol;

      if ( (MulVol = (MultiRetrigVolChg >> 4)) )
        Volume *= MulVol;

      if ( (DivVol = (MultiRetrigVolChg & 0xF)) )
        Volume /= DivVol;

      if ( Volume > 0xFFFF )
        Volume = 0xFFFF;

      PCStr->pc_Volume = Volume >> 8UL;

      if ( PHCStr->phc_Flags & PHCF_SubSlideRetrig )
        PCStr->pc_SubVol = Volume;
    }
  }

  if ( (RetrigTickCount = PHCStr->phc_RetrigTickCount) && --MultiRetrigTick )
  {
    if ( MultiRetrigTick <= RetrigTickCount )
      RetrigTickCount = -1;
  } else
  {
    if ( PCStr->pc_HostChannel == Channel )
      PCStr->pc_ChannelData.ms_SamplePos = 0L;
  }

  PHCStr->phc_RetrigTickCount = ++RetrigTickCount;
}

static void ExtCtrl ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  UWORD ControlWord = DataWord & 0x0FFF;

  switch ( DataWord >> 12U )
  {
    case 0 :
    {
      PHCStr->phc_Flags &= ~PHCF_LinearFreq;

      if (!( ControlWord ))
        PHCStr->phc_Flags |= PHCF_LinearFreq;

      break;
    }
    case 1 :
    {
      PHCStr->phc_GlissandoValue = ControlWord;

      break;
    }
    case 2 :
    {
      UBYTE ControlByte = ControlWord;

      switch ( ControlWord >> 8U )
      {
        case 0 :
        {
          if (!( ControlByte ))
            ControlByte = 1;

          if ( ControlByte > 4 )
            ControlByte = 4;

          PHCStr->phc_MultiRetrigScale = ControlByte;

          break;
        }
        case 1 :
        {
          PHCStr->phc_Flags &= ~PHCF_SubSlideRetrig;

          if ( ControlByte )
            PHCStr->phc_Flags |= PHCF_SubSlideRetrig;

          break;
        }
        case 2 :
        {
          if ( ControlByte )
            PHCStr->phc_MultiRetrigTick = PHCStr->phc_Tempo / ControlByte;

          break;
        }
      }

      break;
    }
    case 3 :
    {
      struct TuComposerModule *ModuleStr = TuCBase->tcb_PlayModule;
      struct PlayerChannel *ScanPCStr = ModuleStr->tcm_ChannelData;
      UWORD VCh = 0;

      do
      {
        if ( (ScanPCStr->pc_HostChannel == Channel) && (ScanPCStr->pc_Flags & PCF_Background) )
          ScanPCStr->pc_Flags = 0;

        ScanPCStr++;
      } while ( ++VCh < ModuleStr->tcm_Channels );

      break;
    }
    case 4 :
    {
      struct TuComposerModule *ModuleStr = TuCBase->tcb_PlayModule;
      struct PlayerChannel *ScanPCStr = ModuleStr->tcm_ChannelData;
      UWORD VCh = 0;

      do
      {
        if ( (ScanPCStr->pc_HostChannel == Channel) && (ScanPCStr->pc_Flags & PCF_Background) )
          ScanPCStr->pc_Flags |= PCF_Fading;

        ScanPCStr++;
      } while ( ++VCh < ModuleStr->tcm_Channels );

      break;
    }
    case 5 :
    {
      struct TuComposerModule *ModuleStr = TuCBase->tcb_PlayModule;
      struct PlayerChannel *ScanPCStr = ModuleStr->tcm_ChannelData;
      UWORD VCh = 0;

      do
      {
        if ( (ScanPCStr->pc_HostChannel == Channel) && (ScanPCStr->pc_Flags & PCF_Background) )
          PlayKeyOff ( ScanPCStr );


        ScanPCStr++;
      } while ( ++VCh < ModuleStr->tcm_Channels );

      break;
    }
    case 6 :
    {
      PHCStr->phc_SubSlideValue = ControlWord;

      break;
    }
  }
}

static void InvLoop ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{

}

static void ExecFX ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
}

static void StopFX ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  UBYTE StopFX = DataWord;

  if ( (BYTE) StopFX < 0 )
    StopFX = 127;

  if (!( DataWord >>= 8U ))
    DataWord = PHCStr->phc_ExecFX;

  if ( DataWord >= PHCStr->phc_TempoCounter )
    PHCStr->phc_EffectsUsed[(StopFX >> 3U)] |= (1 << (7 - (StopFX & 7)));
}

static void SetVol ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  PHCStr->phc_TremoloSlideValue = 0;

  if ( CheckOldVolume ( TuCBase, PCStr, &DataWord, Channel ) )
  {
    PCStr->pc_Volume = DataWord >> 8U;
    PCStr->pc_SubVol = DataWord;
  }
}

static void VolSlUp ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct TuComposerTrack *TrackStr;

  if (!( DataWord ))
    DataWord = PHCStr->phc_VolSlideUpValue;

  DoVolumeSlide ( TuCBase, PCStr, DataWord, Channel );

  TrackStr = PHCStr->phc_Track;

  if ( TrackStr->tct_CompFlags & TCTCF_VolPitch )
    PortaUpOk ( PHCStr, DataWord );

  VolSlUpOk ( PHCStr, DataWord );
}

static void VolSlDn ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct TuComposerTrack *TrackStr;

  if (!( DataWord ))
    DataWord = PHCStr->phc_VolSlideDownValue;

  DoVolumeSlideDn ( TuCBase, PCStr, DataWord, Channel );

  TrackStr = PHCStr->phc_Track;

  if ( TrackStr->tct_CompFlags & TCTCF_VolPitch )
    PortaDnOk ( PHCStr, DataWord );

  VolSlDnOk ( PHCStr, DataWord );
}

static void FVolSlUp ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct TuComposerTrack *TrackStr;

  if (!( DataWord ))
    DataWord = PHCStr->phc_FineVolSlideUpValue;

  DoVolumeSlide ( TuCBase, PCStr, DataWord, Channel );

  TrackStr = PHCStr->phc_Track;

  if ( TrackStr->tct_CompFlags & TCTCF_VolPitch )
    PortaUpOnceOk ( PHCStr, DataWord );

  FVolSlUpOk ( PHCStr, DataWord );
}

static void FVolSlDn ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct TuComposerTrack *TrackStr;

  if (!( DataWord ))
    DataWord = PHCStr->phc_VolSlideDownValue;

  DoVolumeSlideDn ( TuCBase, PCStr, DataWord, Channel );

  TrackStr = PHCStr->phc_Track;

  if ( TrackStr->tct_CompFlags & TCTCF_VolPitch )
    PortaDnOnceOk ( PHCStr, DataWord );

  FVolSlDnOk ( PHCStr, DataWord );
}

static void VolSlTo ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  UBYTE VolumeSlideToValue, VolumeSlideToVolume;

  if (!( VolumeSlideToValue = DataWord ))
    VolumeSlideToValue = PHCStr->phc_VolumeSlideToValue;

  PHCStr->phc_VolumeSlideToValue = VolumeSlideToValue;
  PHCStr->phc_VolumeSlideToSlide &= 0x00FF;
  PHCStr->phc_VolumeSlideToSlide += VolumeSlideToValue << 8U;

  VolumeSlideToVolume = DataWord >> 8U;

  if ( VolumeSlideToVolume && (VolumeSlideToVolume < 0xFF) )
  {
    PHCStr->phc_VolumeSlideToVolume = VolumeSlideToVolume;
  } else
  {
    if ( VolumeSlideToVolume && (PCStr->pc_HostChannel == Channel) )
    {
      UWORD VolumeSlideTarget = (VolumeSlideToVolume << 8U) + PHCStr->phc_VolumeSlideToSubVol;
      UWORD Volume = (PCStr->pc_Volume << 8U) + PCStr->pc_SubVol;

      if ( Volume < VolumeSlideTarget )
      {
        DoVolumeSlide ( TuCBase, PCStr, PHCStr->phc_VolumeSlideToSlide, Channel );

        Volume = (PCStr->pc_Volume << 8U) + PCStr->pc_SubVol;

        if ( VolumeSlideTarget <= Volume )
        {
          PCStr->pc_Volume = VolumeSlideTarget >> 8U;
          PCStr->pc_SubVol = VolumeSlideTarget;
        }
      } else
      {
        DoVolumeSlideDn ( TuCBase, PCStr, PHCStr->phc_VolumeSlideToSlide, Channel );

        Volume = (PCStr->pc_Volume << 8U) + PCStr->pc_SubVol;

        if ( VolumeSlideTarget >= Volume )
        {
          PCStr->pc_Volume = VolumeSlideTarget >> 8U;
          PCStr->pc_SubVol = VolumeSlideTarget;
        }
      }
    }
  }
}

static void Tremo ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct TuComposerSong *SongStr = TuCBase->tcb_PlaySong;
  LONG TremoloSlideValue;
  UBYTE TremoloRate;
  WORD TremoloDepth;

  if (!( TremoloRate = (DataWord >> 8U)))
    TremoloRate = PHCStr->phc_TremoloRate;

  PHCStr->phc_TremoloRate = TremoloRate;

  if (!( TremoloDepth = DataWord ))
    TremoloDepth = PHCStr->phc_TremoloDepth;

  PHCStr->phc_TremoloDepth = TremoloDepth;

  if ( SongStr->tcso_CompFlags & TCSOCF_OldVolumes )
  {
    if ( TremoloDepth > 63 )
      TremoloDepth = 63;

    if ( TremoloDepth < -63 )
      TremoloDepth = -63;
  }

  TremoloSlideValue = (-TremoloDepth * RunEnvelope ( TuCBase, (struct Envelope *) &(PHCStr->phc_TremoloEnv), TremoloRate, 0 )) >> 7L;

  if ( SongStr->tcso_CompFlags & TCSOCF_OldVolumes )
    TremoloSlideValue <<= 2;

  if ( PCStr->pc_HostChannel == Channel )
  {
    UWORD Volume = PCStr->pc_Volume;

    TremoloSlideValue -= PHCStr->phc_TremoloSlideValue;

    if ( (TremoloSlideValue += Volume) < 0 )
      TremoloSlideValue = 0;

    if ( TremoloSlideValue > 255 )
      TremoloSlideValue = 255;

    PCStr->pc_Volume = TremoloSlideValue;

    PHCStr->phc_TremoloSlideValue -= TremoloSlideValue - Volume;
  }
}

static void SetTVol ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  if ( CheckOldTVol ( TuCBase, &DataWord ) )
  {
    PHCStr->phc_TrackVolume = DataWord >> 8U;
    PHCStr->phc_TrackSubVol = DataWord;
  }
}

static void TVolSlUp ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct TuComposerTrack *TrackStr;
  UWORD v3, v4, v5;

  if (!( DataWord ))
    DataWord = PHCStr->phc_TrackVolSlideUpValue;

  DoTVolumeSlide ( TuCBase, PHCStr, DataWord );

  TrackStr = PHCStr->phc_Track;

  v3 = PHCStr->phc_TrackVolSlideDownValue;
  v4 = PHCStr->phc_FineTrkVolSlideUpValue;
  v5 = PHCStr->phc_FineTrkVolSlideDnValue;

  if ( TrackStr->tct_CompFlags & TCTCF_VolSlide )
    v4 = DataWord;

  if (!( TrackStr->tct_CompFlags & TCTCF_OpVSlide ))
  {
    v3 = DataWord;
    v5 = v4;
  }

  PHCStr->phc_TrackVolSlideUpValue = DataWord;
  PHCStr->phc_TrackVolSlideDownValue = v3;
  PHCStr->phc_FineTrkVolSlideUpValue = v4;
  PHCStr->phc_FineTrkVolSlideDnValue = v5;
}

static void TVolSlDn ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct TuComposerTrack *TrackStr;
  UWORD v0, v3, v4;

  if (!( DataWord ))
    DataWord = PHCStr->phc_TrackVolSlideDownValue;

  DoTVolumeSlideDn ( TuCBase, PHCStr, DataWord );

  TrackStr = PHCStr->phc_Track;

  v0 = PHCStr->phc_TrackVolSlideUpValue;
  v3 = PHCStr->phc_FineTrkVolSlideUpValue;
  v4 = PHCStr->phc_FineTrkVolSlideDnValue;

  if ( TrackStr->tct_CompFlags & TCTCF_VolSlide )
    v4 = DataWord;

  if (!( TrackStr->tct_CompFlags & TCTCF_OpVSlide ))
  {
    v0 = DataWord;
    v3 = v4;
  }

  PHCStr->phc_TrackVolSlideUpValue = v0;
  PHCStr->phc_TrackVolSlideDownValue = DataWord;
  PHCStr->phc_FineTrkVolSlideUpValue = v3;
  PHCStr->phc_FineTrkVolSlideDnValue = v4;
}

static void FTVolSlUp ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct TuComposerTrack *TrackStr;
  UWORD v0, v1, v4;

  if (!( DataWord ))
    DataWord = PHCStr->phc_FineTrkVolSlideUpValue;

  DoTVolumeSlide ( TuCBase, PHCStr, DataWord );

  TrackStr = PHCStr->phc_Track;

  v0 = PHCStr->phc_TrackVolSlideUpValue;
  v1 = PHCStr->phc_TrackVolSlideDownValue;
  v4 = PHCStr->phc_FineTrkVolSlideDnValue;

  if ( TrackStr->tct_CompFlags & TCTCF_VolSlide )
    v0 = DataWord;

  if (!( TrackStr->tct_CompFlags & TCTCF_OpVSlide ))
  {
    v1 = v0;
    v4 = DataWord;
  }

  PHCStr->phc_TrackVolSlideUpValue = v0;
  PHCStr->phc_TrackVolSlideDownValue = v1;
  PHCStr->phc_FineTrkVolSlideUpValue = DataWord;
  PHCStr->phc_FineTrkVolSlideDnValue = v4;
}

static void FTVolSlDn ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct TuComposerTrack *TrackStr;
  UWORD v0, v1, v3;

  if (!( DataWord ))
    DataWord = PHCStr->phc_FineTrkVolSlideDnValue;

  DoTVolumeSlideDn ( TuCBase, PHCStr, DataWord );

  TrackStr = PHCStr->phc_Track;

  v0 = PHCStr->phc_TrackVolSlideUpValue;
  v1 = PHCStr->phc_TrackVolSlideDownValue;
  v3 = PHCStr->phc_FineTrkVolSlideUpValue;

  if ( TrackStr->tct_CompFlags & TCTCF_VolSlide )
    v1 = DataWord;

  if (!( TrackStr->tct_CompFlags & TCTCF_OpVSlide ))
  {
    v0 = v1;
    v3 = DataWord;
  }

  PHCStr->phc_TrackVolSlideUpValue = v0;
  PHCStr->phc_TrackVolSlideDownValue = v1;
  PHCStr->phc_FineTrkVolSlideUpValue = v3;
  PHCStr->phc_FineTrkVolSlideDnValue = DataWord;
}

static void TVolSlTo ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  UBYTE TrackVolSlideToValue, TrackVolSlideToVolume;

  if (!( TrackVolSlideToValue = DataWord ))
    TrackVolSlideToValue = PHCStr->phc_TrackVolSlideToValue;

  PHCStr->phc_TrackVolSlideToValue = TrackVolSlideToValue;
  PHCStr->phc_TrackVolSlideToSlide &= 0x00FF;
  PHCStr->phc_TrackVolSlideToSlide += TrackVolSlideToValue << 8U;

  TrackVolSlideToVolume = DataWord >> 8U;

  if ( TrackVolSlideToVolume && (TrackVolSlideToVolume < 0xFF) )
  {
    PHCStr->phc_TrackVolSlideToVolume = TrackVolSlideToVolume;
  } else
  {
    if ( TrackVolSlideToVolume )
    {
      UWORD TrackVolSlideTarget = (TrackVolSlideToVolume << 8U) + PHCStr->phc_TrackVolSlideToSubVol;
      UWORD TrackVolume = (PHCStr->phc_TrackVolume << 8U) + PHCStr->phc_TrackSubVol;

      if ( TrackVolume < TrackVolSlideTarget )
      {
        DoTVolumeSlide ( TuCBase, PHCStr, PHCStr->phc_TrackVolSlideToSlide );

        TrackVolume = (PHCStr->phc_TrackVolume << 8U) + PHCStr->phc_TrackSubVol;

        if ( TrackVolSlideTarget <= TrackVolume )
        {
          PHCStr->phc_TrackVolume = TrackVolSlideTarget >> 8U;
          PHCStr->phc_TrackSubVol = TrackVolSlideTarget;
        }
      } else
      {
        DoTVolumeSlideDn ( TuCBase, PHCStr, PHCStr->phc_TrackVolSlideToSlide );

        TrackVolume = (PHCStr->phc_TrackVolume << 8U) + PHCStr->phc_TrackSubVol;

        if ( TrackVolSlideTarget >= TrackVolume )
        {
          PHCStr->phc_TrackVolume = TrackVolSlideTarget >> 8U;
          PHCStr->phc_TrackSubVol = TrackVolSlideTarget;
        }
      }
    }
  }
}

static void TTremo ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct TuComposerSong *SongStr = TuCBase->tcb_PlaySong;
  LONG TrackTremSlideValue;
  UBYTE TrackTremRate;
  WORD TrackTremDepth;
  UWORD TrackVolume;

  if (!( TrackTremRate = (DataWord >> 8U)))
    TrackTremRate = PHCStr->phc_TrackTremRate;

  PHCStr->phc_TrackTremRate = TrackTremRate;

  if (!( TrackTremDepth = DataWord ))
    TrackTremDepth = PHCStr->phc_TrackTremDepth;

  PHCStr->phc_TrackTremDepth = TrackTremDepth;

  if ( SongStr->tcso_CompFlags & TCSOCF_OldVolumes )
  {
    if ( TrackTremDepth > 63 )
      TrackTremDepth = 63;

    if ( TrackTremDepth < -63 )
      TrackTremDepth = -63;
  }

  TrackTremSlideValue = (-TrackTremDepth * RunEnvelope ( TuCBase, (struct Envelope *) &(PHCStr->phc_TrackTremEnv), TrackTremRate, 0 )) >> 7L;

  if ( SongStr->tcso_CompFlags & TCSOCF_OldVolumes )
    TrackTremSlideValue <<= 2;

  TrackVolume = PHCStr->phc_TrackVolume;

  TrackTremSlideValue -= PHCStr->phc_TrackTremSlideValue;

  if ( (TrackTremSlideValue += TrackVolume) < 0 )
    TrackTremSlideValue = 0;

  if ( TrackTremSlideValue > 255 )
    TrackTremSlideValue = 255;

  PHCStr->phc_TrackVolume = TrackTremSlideValue;

  PHCStr->phc_TrackTremSlideValue -= TrackTremSlideValue - TrackVolume;
}

static void SetPan ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  UBYTE Panning = DataWord >> 8U;

  if ( PCStr->pc_HostChannel == Channel )
  {
    PCStr->pc_Panning = Panning;
    PCStr->pc_SubPan = DataWord;
  }

  PHCStr->phc_Flags &= ~PHCF_TrackSurPan;

  PHCStr->phc_TrackPanning = Panning;
  PHCStr->phc_TrackSubPan = DataWord;

  PHCStr->phc_TrackNotePan = Panning;
  PHCStr->phc_TrackNoteSubPan = DataWord;
}

static void PanSlLeft ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct TuComposerTrack *TrackStr;
  UWORD v3, v4, v5, v8;

  if (!( DataWord ))
    DataWord = PHCStr->phc_PanSlideLeftValue;

  DoPanSlide ( TuCBase, PHCStr, PCStr, DataWord, Channel );

  TrackStr = PHCStr->phc_Track;

  v3 = PHCStr->phc_PanSlideRightValue;
  v4 = PHCStr->phc_FinePanSlideLeftValue;
  v5 = PHCStr->phc_FinePanSlideRightValue;
  v8 = PHCStr->phc_PanningSlideToSlide;

  if ( TrackStr->tct_CompFlags & TCTCF_VolSlide )
  {
    v4 = DataWord;
    v8 = DataWord;
  }

  if (!( TrackStr->tct_CompFlags & TCTCF_OpVSlide ))
  {
    v3 = DataWord;
    v5 = v4;
  }

  PHCStr->phc_PanSlideLeftValue = DataWord;
  PHCStr->phc_PanSlideRightValue = v3;
  PHCStr->phc_FinePanSlideLeftValue = v4;
  PHCStr->phc_FinePanSlideRightValue = v5;
  PHCStr->phc_PanningSlideToSlide = v8;
}

static void PanSlRight ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct TuComposerTrack *TrackStr;
  UWORD v0, v3, v4, v5;

  if (!( DataWord ))
    DataWord = PHCStr->phc_PanSlideRightValue;

  DoPanSlideRight ( TuCBase, PHCStr, PCStr, DataWord, Channel );

  TrackStr = PHCStr->phc_Track;

  v0 = PHCStr->phc_PanSlideLeftValue;
  v3 = PHCStr->phc_FinePanSlideLeftValue;
  v4 = PHCStr->phc_FinePanSlideRightValue;
  v5 = PHCStr->phc_PanningSlideToSlide;

  if ( TrackStr->tct_CompFlags & TCTCF_VolSlide )
  {
    v4 = DataWord;
    v5 = DataWord;
  }

  if (!( TrackStr->tct_CompFlags & TCTCF_OpVSlide ))
  {
    v0 = DataWord;
    v3 = v4;
  }

  PHCStr->phc_PanSlideLeftValue = v0;
  PHCStr->phc_PanSlideRightValue = DataWord;
  PHCStr->phc_FinePanSlideLeftValue = v3;
  PHCStr->phc_FinePanSlideRightValue = v4;
  PHCStr->phc_PanningSlideToSlide = v5;
}

static void FPanSlLeft ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct TuComposerTrack *TrackStr;
  UWORD v0, v1, v4, v5;

  if (!( DataWord ))
    DataWord = PHCStr->phc_FinePanSlideLeftValue;

  DoPanSlide ( TuCBase, PHCStr, PCStr, DataWord, Channel );

  TrackStr = PHCStr->phc_Track;

  v0 = PHCStr->phc_PanSlideLeftValue;
  v1 = PHCStr->phc_PanSlideRightValue;
  v4 = PHCStr->phc_FinePanSlideRightValue;
  v5 = PHCStr->phc_PanningSlideToSlide;

  if ( TrackStr->tct_CompFlags & TCTCF_VolSlide )
  {
    v0 = DataWord;
    v5 = DataWord;
  }

  if (!( TrackStr->tct_CompFlags & TCTCF_OpVSlide ))
  {
    v1 = v0;
    v4 = DataWord;
  }

  PHCStr->phc_PanSlideLeftValue = v0;
  PHCStr->phc_PanSlideRightValue = v1;
  PHCStr->phc_FinePanSlideLeftValue = DataWord;
  PHCStr->phc_FinePanSlideRightValue = v4;
  PHCStr->phc_PanningSlideToSlide = v5;
}

static void FPanSlRight ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct TuComposerTrack *TrackStr;
  UWORD v0, v1, v3, v5;

  if (!( DataWord ))
    DataWord = PHCStr->phc_FinePanSlideRightValue;

  DoPanSlideRight ( TuCBase, PHCStr, PCStr, DataWord, Channel );

  TrackStr = PHCStr->phc_Track;

  v0 = PHCStr->phc_PanSlideLeftValue;
  v1 = PHCStr->phc_PanSlideRightValue;
  v3 = PHCStr->phc_FinePanSlideLeftValue;
  v5 = PHCStr->phc_PanningSlideToSlide;

  if ( TrackStr->tct_CompFlags & TCTCF_VolSlide )
  {
    v1 = DataWord;
    v5 = DataWord;
  }

  if (!( TrackStr->tct_CompFlags & TCTCF_OpVSlide ))
  {
    v0 = v1;
    v3 = DataWord;
  }

  PHCStr->phc_PanSlideLeftValue = v0;
  PHCStr->phc_PanSlideRightValue = v1;
  PHCStr->phc_FinePanSlideLeftValue = v3;
  PHCStr->phc_FinePanSlideRightValue = DataWord;
  PHCStr->phc_PanningSlideToSlide = v5;
}

static void PanSlTo ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  UBYTE PanningSlideToValue, PanningSlideToPanning;

  if (!( PanningSlideToValue = DataWord ))
    PanningSlideToValue = PHCStr->phc_PanningSlideToValue;

  PHCStr->phc_PanningSlideToValue = PanningSlideToValue;
  PHCStr->phc_PanningSlideToSlide &= 0x00FF;
  PHCStr->phc_PanningSlideToSlide += PanningSlideToValue << 8U;

  PanningSlideToPanning = DataWord >> 8U;

  if ( PanningSlideToPanning && (PanningSlideToPanning < 0xFF) )
  {
    PHCStr->phc_PanningSlideToPanning = PanningSlideToPanning;
  } else
  {
    if ( PanningSlideToPanning && (PCStr->pc_HostChannel == Channel) )
    {
      UWORD PanningSlideTarget = (PanningSlideToPanning << 8U) + PHCStr->phc_PanningSlideToSubPan;
      UWORD Panning = (PCStr->pc_Panning << 8U) + PCStr->pc_SubPan;

      if ( Panning < PanningSlideTarget )
      {
        DoPanSlideRight ( TuCBase, PHCStr, PCStr, PHCStr->phc_PanningSlideToSlide, Channel );

        Panning = (PCStr->pc_Panning << 8U) + PCStr->pc_SubPan;

        if ( PanningSlideTarget <= Panning )
        {
          PCStr->pc_Panning = PanningSlideTarget >> 8U;
          PCStr->pc_SubPan = PanningSlideTarget;

          PHCStr->phc_PanningSlideToPanning = 0;

          PHCStr->phc_TrackNotePan = PHCStr->phc_TrackPanning = PHCStr->phc_PanningSlideToSlide >> 8U;
          PHCStr->phc_TrackNoteSubPan = PHCStr->phc_TrackSubPan = PHCStr->phc_PanningSlideToSlide;
        }
      } else
      {
        DoPanSlide ( TuCBase, PHCStr, PCStr, PHCStr->phc_PanningSlideToSlide, Channel );

        Panning = (PCStr->pc_Panning << 8U) + PCStr->pc_SubPan;

        if ( PanningSlideTarget >= Panning )
        {
          PCStr->pc_Panning = PanningSlideTarget >> 8U;
          PCStr->pc_SubPan = PanningSlideTarget;

          PHCStr->phc_PanningSlideToPanning = 0;

          PHCStr->phc_TrackNotePan = PHCStr->phc_TrackPanning = PHCStr->phc_PanningSlideToSlide >> 8U;
          PHCStr->phc_TrackNoteSubPan = PHCStr->phc_TrackSubPan = PHCStr->phc_PanningSlideToSlide;
        }
      }
    }
  }
}

static void Pannolo ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  LONG PannoloSlideValue;
  UBYTE PannoloRate;
  WORD PannoloDepth;

  if (!( PannoloRate = (DataWord >> 8U)))
    PannoloRate = PHCStr->phc_PannoloRate;

  PHCStr->phc_PannoloRate = PannoloRate;

  if (!( PannoloDepth = DataWord ))
    PannoloDepth = PHCStr->phc_PannoloDepth;

  PHCStr->phc_PannoloDepth = PannoloDepth;

  PannoloSlideValue = (-PannoloDepth * RunEnvelope ( TuCBase, (struct Envelope *) &(PHCStr->phc_PannoloEnv), PannoloRate, 0 )) >> 7L;

  if ( PCStr->pc_HostChannel == Channel )
  {
    UWORD Panning = PCStr->pc_Panning;

    PannoloSlideValue -= PHCStr->phc_PannoloSlideValue;

    if ( (PannoloSlideValue += Panning) < 0 )
      PannoloSlideValue = 0;

    if ( PannoloSlideValue > 255 )
      PannoloSlideValue = 255;

    PCStr->pc_Panning = PannoloSlideValue;

    PHCStr->phc_PannoloSlideValue -= PannoloSlideValue - Panning;

    PHCStr->phc_TrackNotePan = PHCStr->phc_TrackPanning = Panning;
    PHCStr->phc_TrackNoteSubPan = PHCStr->phc_TrackSubPan = PCStr->pc_SubPan;
  }
}

static void SetTPan ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  PHCStr->phc_ChannelPanning = DataWord >> 8U;
  PHCStr->phc_ChannelSubPan = DataWord;

  PHCStr->phc_Flags &= ~PHCF_ChannelSurPan;
}

static void TPanSlLeft ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct TuComposerTrack *TrackStr;
  UWORD v3, v4, v5, v8;

  if (!( DataWord ))
    DataWord = PHCStr->phc_TrackPanSlideLeftValue;

  DoTPanSlide ( TuCBase, PHCStr, DataWord );

  TrackStr = PHCStr->phc_Track;

  v3 = PHCStr->phc_TrackPanSlideRightValue;
  v4 = PHCStr->phc_FineTrkPanSldLeftValue;
  v5 = PHCStr->phc_FineTrkPanSldRightValue;
  v8 = PHCStr->phc_TrackPanSlideToSlide;

  if ( TrackStr->tct_CompFlags & TCTCF_VolSlide )
  {
    v4 = DataWord;
    v8 = DataWord;
  }

  if (!( TrackStr->tct_CompFlags & TCTCF_OpVSlide ))
  {
    v3 = DataWord;
    v5 = v4;
  }

  PHCStr->phc_TrackPanSlideLeftValue = DataWord;
  PHCStr->phc_TrackPanSlideRightValue = v3;
  PHCStr->phc_FineTrkPanSldLeftValue = v4;
  PHCStr->phc_FineTrkPanSldRightValue = v5;
  PHCStr->phc_PanningSlideToSlide = v8;
}

static void TPanSlRight ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct TuComposerTrack *TrackStr;
  UWORD v0, v3, v4, v5;

  if (!( DataWord ))
    DataWord = PHCStr->phc_TrackPanSlideRightValue;

  DoTPanSlideRight ( TuCBase, PHCStr, DataWord );

  TrackStr = PHCStr->phc_Track;

  v0 = PHCStr->phc_TrackPanSlideLeftValue;
  v3 = PHCStr->phc_FineTrkPanSldLeftValue;
  v4 = PHCStr->phc_FineTrkPanSldRightValue;
  v5 = PHCStr->phc_TrackPanSlideToSlide;

  if ( TrackStr->tct_CompFlags & TCTCF_VolSlide )
  {
    v4 = DataWord;
    v5 = DataWord;
  }

  if (!( TrackStr->tct_CompFlags & TCTCF_OpVSlide ))
  {
    v0 = DataWord;
    v3 = v4;
  }

  PHCStr->phc_TrackPanSlideLeftValue = v0;
  PHCStr->phc_TrackPanSlideRightValue = DataWord;
  PHCStr->phc_FineTrkPanSldLeftValue = v3;
  PHCStr->phc_FineTrkPanSldRightValue = v4;
  PHCStr->phc_PanningSlideToSlide = v5;
}

static void FTPanSlLeft ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct TuComposerTrack *TrackStr;
  UWORD v0, v1, v4, v5;

  if (!( DataWord ))
    DataWord = PHCStr->phc_FineTrkPanSldLeftValue;

  DoTPanSlide ( TuCBase, PHCStr, DataWord );

  TrackStr = PHCStr->phc_Track;

  v0 = PHCStr->phc_TrackPanSlideLeftValue;
  v1 = PHCStr->phc_TrackPanSlideRightValue;
  v4 = PHCStr->phc_FineTrkPanSldRightValue;
  v5 = PHCStr->phc_TrackPanSlideToSlide;

  if ( TrackStr->tct_CompFlags & TCTCF_VolSlide )
  {
    v0 = DataWord;
    v5 = DataWord;
  }

  if (!( TrackStr->tct_CompFlags & TCTCF_OpVSlide ))
  {
    v1 = v0;
    v4 = DataWord;
  }

  PHCStr->phc_TrackPanSlideLeftValue = v0;
  PHCStr->phc_TrackPanSlideRightValue = v1;
  PHCStr->phc_FineTrkPanSldLeftValue = DataWord;
  PHCStr->phc_FineTrkPanSldRightValue = v4;
  PHCStr->phc_PanningSlideToSlide = v5;
}

static void FTPanSlRight ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct TuComposerTrack *TrackStr;
  UWORD v0, v1, v3, v5;

  if (!( DataWord ))
    DataWord = PHCStr->phc_FineTrkPanSldRightValue;

  DoTPanSlideRight ( TuCBase, PHCStr, DataWord );

  TrackStr = PHCStr->phc_Track;

  v0 = PHCStr->phc_TrackPanSlideLeftValue;
  v1 = PHCStr->phc_TrackPanSlideRightValue;
  v3 = PHCStr->phc_FineTrkPanSldLeftValue;
  v5 = PHCStr->phc_TrackPanSlideToSlide;

  if ( TrackStr->tct_CompFlags & TCTCF_VolSlide )
  {
    v1 = DataWord;
    v5 = DataWord;
  }

  if (!( TrackStr->tct_CompFlags & TCTCF_OpVSlide ))
  {
    v0 = v1;
    v3 = DataWord;
  }

  PHCStr->phc_TrackPanSlideLeftValue = v0;
  PHCStr->phc_TrackPanSlideRightValue = v1;
  PHCStr->phc_FineTrkPanSldLeftValue = v3;
  PHCStr->phc_FineTrkPanSldRightValue = DataWord;
  PHCStr->phc_PanningSlideToSlide = v5;
}

static void TPanSlTo ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  UBYTE TrackPanSlideToValue, TrackPanSlideToPanning;

  if (!( TrackPanSlideToValue = DataWord ))
    TrackPanSlideToValue = PHCStr->phc_TrackPanSlideToValue;

  PHCStr->phc_TrackPanSlideToValue = TrackPanSlideToValue;
  PHCStr->phc_TrackPanSlideToSlide &= 0x00FF;
  PHCStr->phc_TrackPanSlideToSlide += TrackPanSlideToValue << 8U;

  TrackPanSlideToPanning = DataWord >> 8U;

  if ( TrackPanSlideToPanning && (TrackPanSlideToPanning < 0xFF) )
  {
    PHCStr->phc_TrackPanSlideToPanning = TrackPanSlideToPanning;
  } else
  {
    if ( TrackPanSlideToPanning )
    {
      UWORD TrackPanSlideTarget = (TrackPanSlideToPanning << 8U) + PHCStr->phc_TrackPanSlideToSubPan;
      UWORD TrackPanning = (PHCStr->phc_TrackPanning << 8U) + PHCStr->phc_TrackSubPan;

      if ( TrackPanning < TrackPanSlideTarget )
      {
        DoTPanSlideRight ( TuCBase, PHCStr, PHCStr->phc_TrackPanSlideToSlide );

        TrackPanning = (PHCStr->phc_TrackPanning << 8U) + PHCStr->phc_TrackSubPan;

        if ( TrackPanSlideTarget <= TrackPanning )
        {
          PHCStr->phc_TrackPanning = TrackPanSlideTarget >> 8U;
          PHCStr->phc_TrackSubPan = TrackPanSlideTarget;
        }
      } else
      {
        DoTPanSlide ( TuCBase, PHCStr, PHCStr->phc_TrackPanSlideToSlide );

        TrackPanning = (PHCStr->phc_TrackPanning << 8U) + PHCStr->phc_TrackSubPan;

        if ( TrackPanSlideTarget >= TrackPanning )
        {
          PHCStr->phc_TrackPanning = TrackPanSlideTarget >> 8U;
          PHCStr->phc_TrackSubPan = TrackPanSlideTarget;
        }
      }
    }
  }
}

static void TPannolo ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  LONG TrackPanSlideValue;
  UBYTE TrackPanRate;
  WORD TrackPanDepth;
  UWORD TrackPanning;

  if (!( TrackPanRate = (DataWord >> 8U)))
    TrackPanRate = PHCStr->phc_TrackPanRate;

  PHCStr->phc_TrackPanRate = TrackPanRate;

  if (!( TrackPanDepth = DataWord ))
    TrackPanDepth = PHCStr->phc_TrackPanDepth;

  PHCStr->phc_TrackPanDepth = TrackPanDepth;

  TrackPanSlideValue = (-TrackPanDepth * RunEnvelope ( TuCBase, (struct Envelope *) &(PHCStr->phc_TrackPanEnv), TrackPanRate, 0 )) >> 7L;

  TrackPanning = PHCStr->phc_TrackPanning;

  TrackPanSlideValue -= PHCStr->phc_TrackPanSlideValue;

  if ( (TrackPanSlideValue += TrackPanning) < 0 )
    TrackPanSlideValue = 0;

  if ( TrackPanSlideValue > 255 )
    TrackPanSlideValue = 255;

  PHCStr->phc_TrackPanning = TrackPanSlideValue;

  PHCStr->phc_TrackPanSlideValue -= TrackPanSlideValue - TrackPanning;
}

static void SetTempo ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  if (!( PHCStr->phc_Tempo = DataWord ))
    PHCStr->phc_Flags |= PHCF_SongEnd;
}

static void SetRelTempo ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  if (!( PHCStr->phc_Tempo += DataWord ))
    PHCStr->phc_Flags |= PHCF_SongEnd;
}

static void PattBrk ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  if (!( PHCStr->phc_Flags & PHCF_PattLoop ))
  {
    PHCStr->phc_Flags |= PHCF_PatternBreak;

    PHCStr->phc_BreakRow = DataWord;
  }
}

static void PosJump ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  if (!( PHCStr->phc_Flags & PHCF_PattLoop ))
  {
    struct TuComposerOrders *PosStr;
    struct TuComposerOrderList *PosListStr;

    PosStr = TuCBase->tcb_PlaySong->tcso_Position + Channel;
    PosListStr = (struct TuComposerOrderList *) &(PosStr->tco_OrderData.mlh_Head);

    if ( DataWord )
    {
      struct TuComposerOrderList *ScanPosListStr = PosListStr;

      do
      {
        struct TuComposerOrderList *PrevPosListStr;

        if (!( PrevPosListStr = (struct TuComposerOrderList *) PosListStr->tcol_Node.mln_Succ ))
          break;

        PosListStr = PrevPosListStr;
      } while ( --DataWord );

      if (!( PosListStr->tcol_Node.mln_Succ ))
        PosListStr = ScanPosListStr;
    }

    PHCStr->phc_Order = PosListStr;

    PattBrk ( TuCBase, PHCStr, PCStr, Channel, FX_PATT_BREAK, 0 );
  }
}

static void RelPosJmp ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct TuComposerOrders *PosStr = TuCBase->tcb_PlaySong->tcso_Position + Channel;
  struct TuComposerOrderList *PosListStr = (struct TuComposerOrderList *) &(PosStr->tco_OrderData.mlh_Head);

  if (!( DataWord ))
    DataWord = PHCStr->phc_PosJump;

  if ( (PHCStr->phc_PosJump = DataWord) )
  {
    struct TuComposerOrderList *ScanPosListStr = (struct TuComposerOrderList *) &(PosStr->tco_OrderData.mlh_Head);

    if ( (WORD) DataWord < 0 )
    {
      DataWord = -DataWord;

      PosListStr = (struct TuComposerOrderList *) PosListStr->tcol_Node.mln_Succ;

      do
      {
        struct TuComposerOrderList *PrevPosListStr;

        if (!( PrevPosListStr = (struct TuComposerOrderList *) PosListStr->tcol_Node.mln_Pred ))
          break;

        PosListStr = PrevPosListStr;
      } while ( --DataWord );
    } else
    {
      PosListStr = (struct TuComposerOrderList *) PosListStr->tcol_Node.mln_Pred;

      do
      {
        struct TuComposerOrderList *PrevPosListStr;

        if (!( PrevPosListStr = (struct TuComposerOrderList *) PosListStr->tcol_Node.mln_Succ ))
          break;

        PosListStr = PrevPosListStr;
      } while ( --DataWord );
    }

    if (!( PosListStr->tcol_Node.mln_Succ ))
      PosListStr = ScanPosListStr;

    PHCStr->phc_Order = PosListStr;

    PattBrk ( TuCBase, PHCStr, PCStr, Channel, FX_PATT_BREAK, 0 );
  }
}

static void ChgPattern ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  PHCStr->phc_ChgPattern = DataWord;

  PHCStr->phc_Flags |= PHCF_ChgPattern;
}

static void RevPattPlay ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  if (!( DataWord ))
  {
    PHCStr->phc_Flags ^= PHCF_Backwards;
  } else if ( DataWord & 0x8000 )
  {
    PHCStr->phc_Flags |= PHCF_Backwards;
  } else
  {
    PHCStr->phc_Flags &= ~PHCF_Backwards;
  }
}

static void PattDelay ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  PHCStr->phc_PatternDelay = DataWord;
}

static void FPattDelay ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  PHCStr->phc_FinePatternDelay = DataWord;
}

static void PattLoop ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct TuComposerSong *SongStr = TuCBase->tcb_PlaySong;

  UWORD *LoopStackPtr;
  UWORD LoopLength = PHCStr->phc_PatternLoopDepth;

  (UBYTE *) LoopStackPtr = (UBYTE *) SongStr->tcso_LoopStackPtr + (((SongStr->tcso_LoopStackSize * Channel) * sizeof (UWORD[2])) + (LoopLength * sizeof(UWORD[2])));

  if ( DataWord )
  {
    if ( DataWord == *LoopStackPtr )
    {
      *LoopStackPtr = 0;

      if ( LoopLength-- )
      {
        PHCStr->phc_PatternLoopDepth = LoopLength;
      } else
      {
        PHCStr->phc_Flags |= PHCF_PattReset;
      }
    } else
    {
      (*LoopStackPtr++)++;

      PHCStr->phc_Flags |= PHCF_PatternBreak|PHCF_PattLoop|PHCF_PattLoopJmp;

      PHCStr->phc_BreakRow = *LoopStackPtr;

      if ( PHCStr->phc_Flags & PHCF_PattReset )
        PHCStr->phc_Flags |= PHCF_SongEnd;
    }
  } else
  {
    if (!( PHCStr->phc_Flags & PHCF_PattLoopJmp ))
    {
      PHCStr->phc_Flags &= ~PHCF_PattReset;

      *LoopStackPtr++ = 0;
      *LoopStackPtr = PHCStr->phc_Row;

      if ( ++LoopLength != SongStr->tcso_LoopStackSize )
        PHCStr->phc_PatternLoopDepth = LoopLength;
    }
  }
}

static void GoSub ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{

}

static void Return ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{

}

static void ChanSync ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{

}

static void SetSubSl ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct PlayerGlobals *PGStr = TuCBase->tcb_PlaySong->tcso_GlobalData;
  UBYTE SubSlideBits;

  if (!( SubSlideBits = ( DataWord >> 8U )))
    SubSlideBits = PHCStr->phc_SubSlideBits;

  if ( SubSlideBits & 0x01 )
    PHCStr->phc_VolumeSlideToSubVol = DataWord;

  if ( SubSlideBits & 0x02 )
    PHCStr->phc_TrackVolSlideToSubVol = DataWord;

  if ( SubSlideBits & 0x04 )
    PGStr->pg_GlobalVolumeSlToSubVol = DataWord;

  if ( SubSlideBits & 0x08 )
    PHCStr->phc_PanningSlideToSubPan = DataWord;

  if ( SubSlideBits & 0x10 )
    PHCStr->phc_TrackPanSlideToSubPan = DataWord;

  if ( SubSlideBits & 0x20 )
    PGStr->pg_GlobalPanSlideToSubPan = DataWord;
}

static void SmpOffHI ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  PHCStr->phc_SmpOffsetHI = DataWord;
}

static void SmpOffLO ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  if ( PCStr->pc_HostChannel == Channel )
  {
    ULONG SmpOffset = (PHCStr->phc_SmpOffsetHI << 16UL) + DataWord;

    if (!( PHCStr->phc_Flags & PHCF_SmpOffsetRel ))
    {
      struct TuComposerTrack *TrackStr = PHCStr->phc_Track;

      if ( TrackStr->tct_CompFlags & TCTCF_SOffset )
      {
        struct TuComposerSample *SampleStr = PCStr->pc_Sample;

        if ( SmpOffset >= SampleStr->tcs_Samples )
          return;
      }

      PCStr->pc_ChannelData.ms_SamplePos = 0L;

      if ( PCStr->pc_ChannelData.ms_Flags & MSF_Loop )
      {
        ULONG RepEnd = PCStr->pc_ChannelData.ms_Repeat + PCStr->pc_ChannelData.ms_RepLen;

        if ( RepEnd < SmpOffset )
          SmpOffset = RepEnd;
      }
    }

    PCStr->pc_ChannelData.ms_SamplePos += SmpOffset;
  }
}

static void SetHold ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{

}

static void SetDecay ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{

}

static void SetTransp ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
}

static void InsCtrl ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{

}

static void InsChg ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{

}

static void SynthCtrl ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  PHCStr->phc_SynthCtrlCount = DataWord >> 8U;
  PHCStr->phc_SynthCtrlChange = DataWord;

  if ( DataWord & 0x80 )
    SetSynthVal ( TuCBase, PHCStr, PCStr, Channel, FX_SET_SYN_VAL, PHCStr->phc_SynthCtrlValue );
}

static void SetSynthVal ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct TuComposerSynthWave **WaveformStr;
  UWORD *SynthChange;
  UBYTE SynthCtrlCount = PHCStr->phc_SynthCtrlCount;
  UWORD SynthCtrlChange = PHCStr->phc_SynthCtrlChange & 0x7F;

  PHCStr->phc_SynthCtrlValue = DataWord;

  WaveformStr = (struct TuComposerSynthWave **) &(PCStr->pc_SampleWaveform) + (SynthCtrlChange - 0x24);

  SynthChange = (UWORD *) &(PCStr->pc_VolumePos) + SynthCtrlChange;

  do
  {
    if ( (SynthCtrlChange >= 0x24) )
    {
      struct TuComposerSynthWave *ScanWaveformStr = PCStr->pc_WaveformList;
      UWORD i = DataWord;

      for (;;)
      {
        ScanWaveformStr = (struct TuComposerSynthWave *) ScanWaveformStr->tcyw_Node.mln_Succ;

        if (!( ScanWaveformStr->tcyw_Node.mln_Succ ))
          break;

        if ( i-- )
        {
          *WaveformStr = ScanWaveformStr;

          break;
        }
      }

      WaveformStr++;
    } else if ( (SynthCtrlChange < 0x24) )
    {
      *SynthChange++ = DataWord;
    } else
    {
      break;
    }
  } while ( SynthCtrlCount-- );
}

static void EnvCtrl ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  UBYTE EnvCtrlKind = (DataWord >> 8U) & 0x7F;

  if ( EnvCtrlKind <= 0x0F )
  {
    UBYTE EnvCtrlType = DataWord;

    PHCStr->phc_EnvCtrlKind = EnvCtrlKind;

    if ( EnvCtrlType <= 0x32 )
    {
      struct Envelope *EnvStr;
      struct Envelope * (*GetEnvKind)( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr );

      (APTR) GetEnvKind = (APTR) EnvCtrlTypeTab[EnvCtrlKind];

      EnvStr = GetEnvKind ( TuCBase, PHCStr, PCStr );

      switch ( EnvCtrlType )
      {
        case 0x10 :
        {
          struct TuComposerInstrEnvelope *InstrEnvStr;

          if ( (InstrEnvStr = EnvStr->pes_EnvelopeStructure) )
          {
            EnvStr->pes_EnvelopeTempo = InstrEnvStr->tcie_Tempo;

            EnvStr->pes_EnvelopeSustainCounted = 0;
            EnvStr->pes_EnvelopeLoopCounted = 0;

            EnvStr->pes_EnvelopeTempoCount = 0;

            EnvStr->pes_EnvelopeSustainStart = InstrEnvStr->tcie_SustainStart;
            EnvStr->pes_EnvelopeSustainEnd = InstrEnvStr->tcie_SustainEnd;
            EnvStr->pes_EnvelopeSustainCount = InstrEnvStr->tcie_SustainCount;

            EnvStr->pes_EnvelopeLoopStart = InstrEnvStr->tcie_LoopStart;
            EnvStr->pes_EnvelopeLoopEnd = InstrEnvStr->tcie_LoopEnd;
            EnvStr->pes_EnvelopeLoopCount = InstrEnvStr->tcie_LoopCount;

            EnvStr->pes_EnvelopeLowestValue = InstrEnvStr->tcie_LowestValue;
            EnvStr->pes_EnvelopeHighestValue = InstrEnvStr->tcie_HighestValue;
            EnvStr->pes_EnvelopeRepFlags = InstrEnvStr->tcie_Flags;

            SetEnvelope ( PCStr, EnvStr, EnvStr->pes_EnvelopePos );
          }

          break;
        }
        case 0x01 :
        {
          EnvStr->pes_EnvelopeFlags |= PESF_NoRetrig;

          break;
        }
        case 0x11 :
        {
          EnvStr->pes_EnvelopeFlags &= ~PESF_NoRetrig;

          break;
        }
        case 0x02 :
        {
          EnvStr->pes_EnvelopeFlags &= ~PESF_Random;

          break;
        }
        case 0x12 :
        {
          EnvStr->pes_EnvelopeFlags |= PESF_Random;

          break;
        }
        case 0x22 :
        {
          EnvStr->pes_EnvelopeFlags &= ~PESF_RndDelay;

          break;
        }
        case 0x32 :
        {
          EnvStr->pes_EnvelopeFlags |= PESF_RndDelay;

          break;
        }
        case 0x03 :
        {
          EnvStr->pes_EnvelopeFlags &= ~PESF_FirstAdd;

          break;
        }
        case 0x13 :
        {
          EnvStr->pes_EnvelopeFlags |= PESF_FirstAdd;

          break;
        }
        default :
        {
          PHCStr->phc_EnvCtrlChange = EnvCtrlType;

          if ( DataWord & 0x8000 )
            SetEnvVal ( TuCBase, PHCStr, PCStr, Channel, FX_SET_ENV_VAL, PHCStr->phc_EnvCtrlValue );

          break;
        }
      }
    }
  }
}

static void SetEnvVal ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct Envelope *EnvStr;
  struct Envelope * (*GetEnvKind)( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr );

  PHCStr->phc_EnvCtrlValue = DataWord;

  (APTR) GetEnvKind = (APTR) EnvCtrlTypeTab[PHCStr->phc_EnvCtrlKind];

  EnvStr = GetEnvKind ( TuCBase, PHCStr, PCStr );

  switch ( PHCStr->phc_EnvCtrlChange )
  {
    case 0x00 :
    {
      struct TuComposerInstrEnvelope *InstrEnvStr = (struct TuComposerInstrEnvelope *) &(TuCBase->tcb_PlayModule->tcm_Envelopes.mlh_Head);
      struct TuComposerInstrEnvelope *PutEnvStr = NULL;

      for (;;)
      {
        InstrEnvStr = (struct TuComposerInstrEnvelope *) InstrEnvStr->tcie_Node.mln_Succ;

        if (!( InstrEnvStr->tcie_Node.mln_Succ ))
          break;

        if ( DataWord-- )
        {
          PutEnvStr = InstrEnvStr;

          break;
        }
      }

      EnvStr->pes_EnvelopeStructure = PutEnvStr;

      break;
    }
    case 0x04 :
    {
      EnvStr->pes_EnvelopePos = DataWord;

      break;
    }
    case 0x14 :
    {
      struct TuComposerInstrEnvelope *InstrEnvStr;

      if ( (InstrEnvStr = EnvStr->pes_EnvelopeStructure) )
      {
        if ( ++DataWord > InstrEnvStr->tcie_NumNodes )
          DataWord = InstrEnvStr->tcie_NumNodes;

        EnvStr->pes_EnvelopePos = InstrEnvStr->tcie_NodePoints[DataWord - 1];
      }

      break;
    }
    case 0x05 :
    {
      EnvStr->pes_EnvelopeTempo = DataWord;

      break;
    }
    case 0x15 :
    {
      EnvStr->pes_EnvelopeTempo += DataWord;

      break;
    }
    case 0x25 :
    {
      EnvStr->pes_EnvelopeTempoCount = DataWord;

      break;
    }
    case 0x06 :
    {
      EnvStr->pes_EnvelopeSustainStart = DataWord;

      break;
    }
    case 0x07 :
    {
      EnvStr->pes_EnvelopeSustainEnd = DataWord;

      break;
    }
    case 0x08 :
    {
      EnvStr->pes_EnvelopeSustainCount = DataWord;

      break;
    }
    case 0x09 :
    {
      EnvStr->pes_EnvelopeSustainCounted = DataWord;

      break;
    }
    case 0x0A :
    {
      EnvStr->pes_EnvelopeLoopStart = DataWord;

      break;
    }
    case 0x1A :
    {
      EnvStr->pes_EnvelopeRestart = DataWord;

      break;
    }
    case 0x0B :
    {
      EnvStr->pes_EnvelopeLoopEnd = DataWord;

      break;
    }
    case 0x1B :
    {
      EnvStr->pes_EnvelopeEnd = DataWord;

      break;
    }
    case 0x0C :
    {
      EnvStr->pes_EnvelopeLoopCount = DataWord;

      break;
    }
    case 0x0D :
    {
      EnvStr->pes_EnvelopeLoopCounted = DataWord;

      break;
    }
    case 0x0E :
    {
      EnvStr->pes_EnvelopeLowestValue = DataWord;

      break;
    }
    case 0x0F :
    {
      EnvStr->pes_EnvelopeHighestValue = DataWord;

      break;
    }
  }
}

static void NNACtrl ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  UBYTE ControlType = (DataWord >> 8U);
  UBYTE ControlAction = DataWord;

  switch ( ControlType )
  {
    case 0x00 :
    {
      switch ( ControlAction )
      {
        case 0x00 :
        {
          PHCStr->phc_NNA = NNA_NOTE_CUT;

          break;
        }
        case 0x01 :
        {
          PHCStr->phc_NNA = NNA_NOTE_OFF;

          break;
        }
        case 0x02 :
        {
          PHCStr->phc_NNA = NNA_NOTE_CONTINUE;

          break;
        }
        case 0x03 :
        {
          PHCStr->phc_NNA = NNA_NOTE_FADE;

          break;
        }
      }

      break;
    }
    case 0x11 :
    {
      if (!( ControlAction ))
        ControlAction = 0xFF;

      PHCStr->phc_DCT |= ControlAction;

      break;
    }
    case 0x01 :
    {
      if (!( ControlAction ))
        ControlAction = 0xFF;

      PHCStr->phc_DCT &= ~ControlAction;

      break;
    }
    case 0x02 :
    {
      PHCStr->phc_DNA = ControlAction;

      break;
    }
  }
}

static void LoopCtrl ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{

}

static void SetSpeed ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct TuComposerSong *SongStr = TuCBase->tcb_PlaySong;
  struct PlayerGlobals *PGStr = SongStr->tcso_GlobalData;
  UWORD *SpeedPtr;
  UWORD SpeedValue, SpeedLowerLimit, SpeedUpperLimit;
  UBYTE SpeedType = (DataWord >> 12U);

  if ( (SpeedPtr = GetSpeedAdr ( SongStr, SpeedType, &SpeedLowerLimit, &SpeedUpperLimit )))
  {
    if (!( SpeedValue = (DataWord & 0xFFF )))
    {
      if ( (DataWord & 0x7000) == 0x7000 )
      {
        SpeedValue = (PGStr->pg_SpeedMul << 8U) + PGStr->pg_SpeedDiv;
      } else
      {
        SpeedValue = *SpeedPtr;
      }
    }

    SpeedValOk ( TuCBase, SpeedPtr, SpeedValue, SpeedType, SpeedLowerLimit, SpeedUpperLimit );
  }
}

static void SpdSlFast ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct PlayerGlobals *PGStr = TuCBase->tcb_PlaySong->tcso_GlobalData;
  struct TuComposerTrack *TrackStr;
  UWORD v3, v4, v5;

  if (!( DataWord ))
    DataWord = PGStr->pg_SpeedSlideFasterValue;

  DoSpeedSlide ( TuCBase, DataWord );

  TrackStr = PHCStr->phc_Track;

  v3 = PGStr->pg_SpeedSlideSlowerValue;
  v4 = PGStr->pg_FineSpeedSlideFastValue;
  v5 = PGStr->pg_FineSpeedSlideSlowValue;

  if ( TrackStr->tct_CompFlags & TCTCF_VolSlide )
    v4 = DataWord;

  if (!( TrackStr->tct_CompFlags & TCTCF_OpVSlide ))
  {
    v3 = DataWord;
    v5 = v4;
  }

  PGStr->pg_SpeedSlideFasterValue = DataWord;
  PGStr->pg_SpeedSlideSlowerValue = v3;
  PGStr->pg_FineSpeedSlideFastValue = v4;
  PGStr->pg_FineSpeedSlideSlowValue = v5;
}

static void SpdSlSlow ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct PlayerGlobals *PGStr = TuCBase->tcb_PlaySong->tcso_GlobalData;
  struct TuComposerTrack *TrackStr;
  UWORD v0, v3, v4;

  if (!( DataWord ))
    DataWord = PGStr->pg_SpeedSlideSlowerValue;

  DoSpeedSlideSlow ( TuCBase, DataWord );

  TrackStr = PHCStr->phc_Track;

  v0 = PGStr->pg_SpeedSlideFasterValue;
  v3 = PGStr->pg_FineSpeedSlideFastValue;
  v4 = PGStr->pg_FineSpeedSlideSlowValue;

  if ( TrackStr->tct_CompFlags & TCTCF_VolSlide )
    v4 = DataWord;

  if (!( TrackStr->tct_CompFlags & TCTCF_OpVSlide ))
  {
    v0 = DataWord;
    v3 = v4;
  }

  PGStr->pg_SpeedSlideFasterValue = v0;
  PGStr->pg_SpeedSlideSlowerValue = DataWord;
  PGStr->pg_FineSpeedSlideFastValue = v3;
  PGStr->pg_FineSpeedSlideSlowValue = v4;
}

static void FSpdSlFast ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct PlayerGlobals *PGStr = TuCBase->tcb_PlaySong->tcso_GlobalData;
  struct TuComposerTrack *TrackStr;
  UWORD v0, v1, v4;

  if (!( DataWord ))
    DataWord = PGStr->pg_FineSpeedSlideFastValue;

  DoSpeedSlide ( TuCBase, DataWord );

  TrackStr = PHCStr->phc_Track;

  v0 = PGStr->pg_SpeedSlideFasterValue;
  v1 = PGStr->pg_SpeedSlideSlowerValue;
  v4 = PGStr->pg_FineSpeedSlideSlowValue;

  if ( TrackStr->tct_CompFlags & TCTCF_VolSlide )
    v0 = DataWord;

  if (!( TrackStr->tct_CompFlags & TCTCF_OpVSlide ))
  {
    v1 = v0;
    v4 = DataWord;
  }

  PGStr->pg_SpeedSlideFasterValue = v0;
  PGStr->pg_SpeedSlideSlowerValue = v1;
  PGStr->pg_FineSpeedSlideFastValue = DataWord;
  PGStr->pg_FineSpeedSlideSlowValue = v4;
}

static void FSpdSlSlow ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct PlayerGlobals *PGStr = TuCBase->tcb_PlaySong->tcso_GlobalData;
  struct TuComposerTrack *TrackStr;
  UWORD v0, v1, v3;

  if (!( DataWord ))
    DataWord = PGStr->pg_FineSpeedSlideSlowValue;

  DoSpeedSlideSlow ( TuCBase, DataWord );

  TrackStr = PHCStr->phc_Track;

  v0 = PGStr->pg_SpeedSlideFasterValue;
  v1 = PGStr->pg_SpeedSlideSlowerValue;
  v3 = PGStr->pg_FineSpeedSlideFastValue;

  if ( TrackStr->tct_CompFlags & TCTCF_VolSlide )
    v1 = DataWord;

  if (!( TrackStr->tct_CompFlags & TCTCF_OpVSlide ))
  {
    v0 = v1;
    v3 = DataWord;
  }

  PGStr->pg_SpeedSlideFasterValue = v0;
  PGStr->pg_SpeedSlideSlowerValue = v1;
  PGStr->pg_FineSpeedSlideFastValue = v3;
  PGStr->pg_FineSpeedSlideSlowValue = DataWord;
}

static void SpdSlTo ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{

}

static void Spenolo ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{

}

static void ChCtrl ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  UBYTE ChCtrlByte = DataWord;

  switch ( (DataWord >> 8U) )
  {
    case 0x00 :
    {


      break;
    }
    case 0x01 :
    {


      break;
    }
    case 0x02 :
    {


      break;
    }
    case 0x03 :
    {


      break;
    }
    case 0x04 :
    {


      break;
    }
    case 0x05 :
    {


      break;
    }
    case 0x06 :
    {


      break;
    }
    case 0x07 :
    {


      break;
    }
    case 0x08 :
    {


      break;
    }
    case 0x09 :
    {


      break;
    }
    case 0x0A :
    {


      break;
    }
    case 0x10 :
    {
      switch ( ChCtrlByte )
      {
        case 0x00 :
        {
          if ( CheckSurTPan ( PHCStr, PCStr, Channel, 0 ))
          {
            PHCStr->phc_Flags &= ~PHCF_TrackSurPan;
            PCStr->pc_Flags &= ~PCF_SmpSurPan;
          }

          break;
        }
        case 0x01 :
        {
          if ( CheckSurTPan ( PHCStr, PCStr, Channel, 1 ))
          {
            PHCStr->phc_Flags |= PHCF_TrackSurPan;
            PCStr->pc_Flags |= PCF_SmpSurPan;
          }

          break;
        }
        case 0x10 :
        {
          PHCStr->phc_Flags &= ~PHCF_ChannelSurPan;

          break;
        }
        case 0x11 :
        {
          PHCStr->phc_Flags |= PHCF_ChannelSurPan;

          break;
        }
        case 0x20 :
        {
          TuCBase->tcb_PlaySong->tcso_GlobalData->pg_Flags &= ~PGF_Surround;

          break;
        }
        case 0x21 :
        {
          TuCBase->tcb_PlaySong->tcso_GlobalData->pg_Flags |= PGF_Surround;

          break;
        }
      }

      break;
    }
    case 0x11 :
    {


      break;
    }
  }
}

static void SetGVol ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct PlayerGlobals *PGStr = TuCBase->tcb_PlaySong->tcso_GlobalData;

  if ( CheckOldTVol ( TuCBase, &DataWord ) )
  {
    PGStr->pg_GlobalVolume = DataWord >> 8U;
    PGStr->pg_GlobalSubVol = DataWord;
  }
}

static void GVolSlUp ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct PlayerGlobals *PGStr = TuCBase->tcb_PlaySong->tcso_GlobalData;
  struct TuComposerTrack *TrackStr;
  UWORD v3, v4, v5;

  if (!( DataWord ))
    DataWord = PGStr->pg_GlobalVolSlideUpValue;

  DoGVolumeSlide ( TuCBase, PGStr, DataWord );

  TrackStr = PHCStr->phc_Track;

  v3 = PGStr->pg_GlobalVolSlideDownValue;
  v4 = PGStr->pg_FineGlobalVolSlUpValue;
  v5 = PGStr->pg_FineGlobalVolSlDownValue;

  if ( TrackStr->tct_CompFlags & TCTCF_VolSlide )
    v4 = DataWord;

  if (!( TrackStr->tct_CompFlags & TCTCF_OpVSlide ))
  {
    v3 = DataWord;
    v5 = v4;
  }

  PGStr->pg_GlobalVolSlideUpValue = DataWord;
  PGStr->pg_GlobalVolSlideDownValue = v3;
  PGStr->pg_FineGlobalVolSlUpValue = v4;
  PGStr->pg_FineGlobalVolSlDownValue = v5;
}

static void GVolSlDn ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct PlayerGlobals *PGStr = TuCBase->tcb_PlaySong->tcso_GlobalData;
  struct TuComposerTrack *TrackStr;
  UWORD v0, v3, v4;

  if (!( DataWord ))
    DataWord = PGStr->pg_GlobalVolSlideDownValue;

  DoGVolumeSlideDn ( TuCBase, PGStr, DataWord );

  TrackStr = PHCStr->phc_Track;

  v0 = PGStr->pg_GlobalVolSlideUpValue;
  v3 = PGStr->pg_FineGlobalVolSlUpValue;
  v4 = PGStr->pg_FineGlobalVolSlDownValue;

  if ( TrackStr->tct_CompFlags & TCTCF_VolSlide )
    v4 = DataWord;

  if (!( TrackStr->tct_CompFlags & TCTCF_OpVSlide ))
  {
    v0 = DataWord;
    v3 = v4;
  }

  PGStr->pg_GlobalVolSlideUpValue = v0;
  PGStr->pg_GlobalVolSlideDownValue = DataWord;
  PGStr->pg_FineGlobalVolSlUpValue = v3;
  PGStr->pg_FineGlobalVolSlDownValue = v4;
}

static void FGVolSlUp ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct PlayerGlobals *PGStr = TuCBase->tcb_PlaySong->tcso_GlobalData;
  struct TuComposerTrack *TrackStr;
  UWORD v0, v1, v4;

  if (!( DataWord ))
    DataWord = PGStr->pg_FineGlobalVolSlUpValue;

  DoGVolumeSlide ( TuCBase, PGStr, DataWord );

  TrackStr = PHCStr->phc_Track;

  v0 = PGStr->pg_GlobalVolSlideUpValue;
  v1 = PGStr->pg_GlobalVolSlideDownValue;
  v4 = PGStr->pg_FineGlobalVolSlDownValue;

  if ( TrackStr->tct_CompFlags & TCTCF_VolSlide )
    DataWord = v0;

  if (!( TrackStr->tct_CompFlags & TCTCF_OpVSlide ))
  {
    v1 = v0;
    v4 = DataWord;
  }

  PGStr->pg_GlobalVolSlideUpValue = v0;
  PGStr->pg_GlobalVolSlideDownValue = v1;
  PGStr->pg_FineGlobalVolSlUpValue = DataWord;
  PGStr->pg_FineGlobalVolSlDownValue = v4;
}

static void FGVolSlDn ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct PlayerGlobals *PGStr = TuCBase->tcb_PlaySong->tcso_GlobalData;
  struct TuComposerTrack *TrackStr;
  UWORD v0, v1, v3;

  if (!( DataWord ))
    DataWord = PGStr->pg_GlobalVolSlideDownValue;

  DoGVolumeSlideDn ( TuCBase, PGStr, DataWord );

  TrackStr = PHCStr->phc_Track;

  v0 = PGStr->pg_GlobalVolSlideUpValue;
  v1 = PGStr->pg_GlobalVolSlideDownValue;
  v3 = PGStr->pg_FineGlobalVolSlUpValue;

  if ( TrackStr->tct_CompFlags & TCTCF_VolSlide )
    v1 = DataWord;

  if (!( TrackStr->tct_CompFlags & TCTCF_OpVSlide ))
  {
    v0 = v1;
    v3 = DataWord;
  }

  PGStr->pg_GlobalVolSlideUpValue = v0;
  PGStr->pg_GlobalVolSlideDownValue = v1;
  PGStr->pg_FineGlobalVolSlUpValue = v3;
  PGStr->pg_FineGlobalVolSlDownValue = DataWord;
}

static void GVolSlTo ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct PlayerGlobals *PGStr = TuCBase->tcb_PlaySong->tcso_GlobalData;
  UBYTE GlobalVolSlideToValue, GlobalVolSlideToVolume;

  if (!( GlobalVolSlideToValue = DataWord ))
    GlobalVolSlideToValue = PGStr->pg_GlobalVolumeSlideToValue;

  PGStr->pg_GlobalVolumeSlideToValue = GlobalVolSlideToValue;
  PGStr->pg_GlobalVolumeSlideToSlide &= 0x00FF;
  PGStr->pg_GlobalVolumeSlideToSlide += GlobalVolSlideToValue << 8U;

  GlobalVolSlideToVolume = DataWord >> 8U;

  if ( GlobalVolSlideToVolume && (GlobalVolSlideToVolume < 0xFF) )
  {
    PGStr->pg_GlobalVolumeSlToVolume = GlobalVolSlideToVolume;
  } else
  {
    if ( GlobalVolSlideToVolume )
    {
      UWORD GlobalVolumeSlTarget = (GlobalVolSlideToVolume << 8U) + PGStr->pg_GlobalVolumeSlToSubVol;
      UWORD GlobalVolume = (PGStr->pg_GlobalVolume << 8U) + PGStr->pg_GlobalSubVol;

      if ( GlobalVolume < GlobalVolumeSlTarget )
      {
        DoGVolumeSlide ( TuCBase, PGStr, PGStr->pg_GlobalVolumeSlideToSlide );

        GlobalVolume = (PGStr->pg_GlobalVolume << 8U) + PGStr->pg_GlobalSubVol;

        if ( GlobalVolumeSlTarget <= GlobalVolume )
        {
          PGStr->pg_GlobalVolume = GlobalVolumeSlTarget >> 8U;
          PGStr->pg_GlobalSubVol = GlobalVolumeSlTarget;
        }
      } else
      {
        DoGVolumeSlideDn ( TuCBase, PGStr, PGStr->pg_GlobalVolumeSlideToSlide );

        GlobalVolume = (PGStr->pg_GlobalVolume << 8U) + PGStr->pg_GlobalSubVol;

        if ( GlobalVolumeSlTarget >= GlobalVolume )
        {
          PGStr->pg_GlobalVolume = GlobalVolumeSlTarget >> 8U;
          PGStr->pg_GlobalSubVol = GlobalVolumeSlTarget;
        }
      }
    }
  }
}

static void GTremo ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct TuComposerSong *SongStr = TuCBase->tcb_PlaySong;
  struct PlayerGlobals *PGStr = SongStr->tcso_GlobalData;
  LONG GlobalTremSlideValue;
  UBYTE GlobalTremRate;
  WORD GlobalTremDepth;
  UWORD GlobalVolume;

  if (!( GlobalTremRate = (DataWord >> 8U)))
    GlobalTremRate = PGStr->pg_TremoloRate;

  PGStr->pg_TremoloRate = GlobalTremRate;

  if (!( GlobalTremDepth = DataWord ))
    GlobalTremDepth = PGStr->pg_TremoloDepth;

  PGStr->pg_TremoloDepth = GlobalTremDepth;

  if ( SongStr->tcso_CompFlags & TCSOCF_OldVolumes )
  {
    if ( GlobalTremDepth > 63 )
      GlobalTremDepth = 63;

    if ( GlobalTremDepth < -63 )
      GlobalTremDepth = -63;
  }

  GlobalTremSlideValue = (-GlobalTremDepth * RunEnvelope ( TuCBase, (struct Envelope *) &(PGStr->pg_TremoloEnv), GlobalTremRate, 0 )) >> 7L;

  if ( SongStr->tcso_CompFlags & TCSOCF_OldVolumes )
    GlobalTremSlideValue <<= 2;

  GlobalVolume = PGStr->pg_GlobalVolume;

  GlobalTremSlideValue -= PGStr->pg_TremoloSlideValue;

  if ( (GlobalTremSlideValue += GlobalVolume) < 0 )
    GlobalTremSlideValue = 0;

  if ( GlobalTremSlideValue > 255 )
    GlobalTremSlideValue = 255;

  PGStr->pg_GlobalVolume = GlobalTremSlideValue;

  PGStr->pg_TremoloSlideValue -= GlobalTremSlideValue - GlobalVolume;
}

static void SetGPan ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct PlayerGlobals *PGStr = TuCBase->tcb_PlaySong->tcso_GlobalData;

  PGStr->pg_GlobalPanning = DataWord >> 8U;
  PGStr->pg_GlobalSubPan = DataWord;

  PGStr->pg_Flags &= ~PGF_Surround;
}

static void GPanSlLeft ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct PlayerGlobals *PGStr = TuCBase->tcb_PlaySong->tcso_GlobalData;
  struct TuComposerTrack *TrackStr;
  UWORD v3, v4, v5, v8;

  if (!( DataWord ))
    DataWord = PGStr->pg_GlobalPanSlideLeftValue;

  DoGPanSlide ( PGStr, DataWord );

  TrackStr = PHCStr->phc_Track;

  v3 = PGStr->pg_GlobalPanSlideRightValue;
  v4 = PGStr->pg_FineGlobalPanSlLeftValue;
  v5 = PGStr->pg_FineGlobalPanSlRightVal;
  v8 = PGStr->pg_GlobalPanSlideToSlide;

  if ( TrackStr->tct_CompFlags & TCTCF_VolSlide )
  {
    v4 = DataWord;
    v8 = DataWord;
  }

  if (!( TrackStr->tct_CompFlags & TCTCF_OpVSlide ))
  {
    v3 = DataWord;
    v5 = v4;
  }

  PGStr->pg_GlobalPanSlideLeftValue = DataWord;
  PGStr->pg_GlobalPanSlideRightValue = v3;
  PGStr->pg_FineGlobalPanSlLeftValue = v4;
  PGStr->pg_FineGlobalPanSlRightVal = v5;
  PGStr->pg_GlobalPanSlideToSlide = v8;
}

static void GPanSlRight ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct PlayerGlobals *PGStr = TuCBase->tcb_PlaySong->tcso_GlobalData;
  struct TuComposerTrack *TrackStr;
  UWORD v0, v3, v4, v5;

  if (!( DataWord ))
    DataWord = PGStr->pg_GlobalPanSlideRightValue;

  DoGPanSlideRight ( PGStr, DataWord );

  TrackStr = PHCStr->phc_Track;

  v0 = PGStr->pg_GlobalPanSlideLeftValue;
  v3 = PGStr->pg_FineGlobalPanSlLeftValue;
  v4 = PGStr->pg_FineGlobalPanSlRightVal;
  v5 = PGStr->pg_GlobalPanSlideToSlide;

  if ( TrackStr->tct_CompFlags & TCTCF_VolSlide )
  {
    v4 = DataWord;
    v5 = DataWord;
  }

  if (!( TrackStr->tct_CompFlags & TCTCF_OpVSlide ))
  {
    v0 = DataWord;
    v3 = v4;
  }

  PGStr->pg_GlobalPanSlideLeftValue = v0;
  PGStr->pg_GlobalPanSlideRightValue = DataWord;
  PGStr->pg_FineGlobalPanSlLeftValue = v3;
  PGStr->pg_FineGlobalPanSlRightVal = v4;
  PGStr->pg_GlobalPanSlideToSlide = v5;
}

static void FGPanSlLeft ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct PlayerGlobals *PGStr = TuCBase->tcb_PlaySong->tcso_GlobalData;
  struct TuComposerTrack *TrackStr;
  UWORD v0, v1, v4, v5;

  if (!( DataWord ))
    DataWord = PGStr->pg_FineGlobalPanSlLeftValue;

  DoGPanSlide ( PGStr, DataWord );

  TrackStr = PHCStr->phc_Track;

  v0 = PGStr->pg_GlobalPanSlideLeftValue;
  v1 = PGStr->pg_GlobalPanSlideRightValue;
  v4 = PGStr->pg_FineGlobalPanSlRightVal;
  v5 = PGStr->pg_GlobalPanSlideToSlide;

  if ( TrackStr->tct_CompFlags & TCTCF_VolSlide )
  {
    v0 = DataWord;
    v5 = DataWord;
  }

  if (!( TrackStr->tct_CompFlags & TCTCF_OpVSlide ))
  {
    v1 = v0;
    v4 = DataWord;
  }

  PGStr->pg_GlobalPanSlideLeftValue = v0;
  PGStr->pg_GlobalPanSlideRightValue = v1;
  PGStr->pg_FineGlobalPanSlLeftValue = DataWord;
  PGStr->pg_FineGlobalPanSlRightVal = v4;
  PGStr->pg_GlobalPanSlideToSlide = v5;
}

static void FGPanSlRight ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct PlayerGlobals *PGStr = TuCBase->tcb_PlaySong->tcso_GlobalData;
  struct TuComposerTrack *TrackStr;
  UWORD v0, v1, v3, v5;

  if (!( DataWord ))
    DataWord = PGStr->pg_FineGlobalPanSlRightVal;

  DoGPanSlideRight ( PGStr, DataWord );

  TrackStr = PHCStr->phc_Track;

  v0 = PGStr->pg_GlobalPanSlideLeftValue;
  v1 = PGStr->pg_GlobalPanSlideRightValue;
  v3 = PGStr->pg_FineGlobalPanSlLeftValue;
  v5 = PGStr->pg_GlobalPanSlideToSlide;

  if ( TrackStr->tct_CompFlags & TCTCF_VolSlide )
  {
    v1 = DataWord;
    v5 = DataWord;
  }

  if (!( TrackStr->tct_CompFlags & TCTCF_OpVSlide ))
  {
    v0 = v1;
    v3 = DataWord;
  }

  PGStr->pg_GlobalPanSlideLeftValue = v0;
  PGStr->pg_GlobalPanSlideRightValue = v1;
  PGStr->pg_FineGlobalPanSlLeftValue = v3;
  PGStr->pg_FineGlobalPanSlRightVal = DataWord;
  PGStr->pg_GlobalPanSlideToSlide = v5;
}

static void GPanSlTo ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct PlayerGlobals *PGStr = TuCBase->tcb_PlaySong->tcso_GlobalData;
  UBYTE GlobalPanSlideToValue, GlobalPanSlideToPanning;

  if (!( GlobalPanSlideToValue = DataWord ))
    GlobalPanSlideToValue = PGStr->pg_GlobalPanSlideToValue;

  PGStr->pg_GlobalPanSlideToValue = GlobalPanSlideToValue;
  PGStr->pg_GlobalPanSlideToSlide &= 0x00FF;
  PGStr->pg_GlobalPanSlideToSlide += GlobalPanSlideToValue << 8U;

  GlobalPanSlideToPanning = DataWord >> 8U;

  if ( GlobalPanSlideToPanning && (GlobalPanSlideToPanning < 0xFF) )
  {
    PGStr->pg_GlobalPanSlideToPanning = GlobalPanSlideToPanning;
  } else
  {
    if ( GlobalPanSlideToPanning )
    {
      UWORD GlobalPanSlTarget = (GlobalPanSlideToPanning << 8U) + PGStr->pg_GlobalPanSlideToSubPan;
      UWORD GlobalPanning = (PGStr->pg_GlobalPanning << 8U) + PGStr->pg_GlobalSubPan;

      if ( GlobalPanning < GlobalPanSlTarget )
      {
        DoGPanSlideRight ( PGStr, PGStr->pg_GlobalPanSlideToSlide );

        GlobalPanning = (PGStr->pg_GlobalPanning << 8U) + PGStr->pg_GlobalSubPan;

        if ( GlobalPanSlTarget <= GlobalPanning )
        {
          PGStr->pg_GlobalPanning = GlobalPanSlTarget >> 8U;
          PGStr->pg_GlobalSubPan = GlobalPanSlTarget;
        }
      } else
      {
        DoGPanSlide ( PGStr, PGStr->pg_GlobalPanSlideToSlide );

        GlobalPanning = (PGStr->pg_GlobalPanning << 8U) + PGStr->pg_GlobalSubPan;

        if ( GlobalPanSlTarget >= GlobalPanning )
        {
          PGStr->pg_GlobalPanning = GlobalPanSlTarget >> 8U;
          PGStr->pg_GlobalSubPan = GlobalPanSlTarget;
        }
      }
    }
  }
}

static void GPannolo ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{
  struct PlayerGlobals *PGStr = TuCBase->tcb_PlaySong->tcso_GlobalData;
  LONG GlobalPanSlideValue;
  UBYTE GlobalPanRate;
  WORD GlobalPanDepth;
  UWORD GlobalPanning;

  if (!( GlobalPanRate = (DataWord >> 8U)))
    GlobalPanRate = PGStr->pg_PannoloRate;

  PGStr->pg_PannoloRate = GlobalPanRate;

  if (!( GlobalPanDepth = DataWord ))
    GlobalPanDepth = PGStr->pg_PannoloDepth;

  PGStr->pg_PannoloDepth = GlobalPanDepth;

  GlobalPanSlideValue = (-GlobalPanDepth * RunEnvelope ( TuCBase, (struct Envelope *) &(PGStr->pg_PannoloEnv), GlobalPanRate, 0 )) >> 7L;

  GlobalPanning = PGStr->pg_GlobalPanning;

  GlobalPanSlideValue -= PGStr->pg_PannoloSlideValue;

  if ( (GlobalPanSlideValue += GlobalPanning) < 0 )
    GlobalPanSlideValue = 0;

  if ( GlobalPanSlideValue > 255 )
    GlobalPanSlideValue = 255;

  PGStr->pg_GlobalPanning = GlobalPanSlideValue;

  PGStr->pg_PannoloSlideValue -= GlobalPanSlideValue - GlobalPanning;
}

static void UserSync ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD EffectByte, UWORD DataWord )
{

}

static ULONG ExecuteSynth ( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, ULONG Channel, ULONG SynthType )
{
  UWORD SynthCount = 0, BitMask = 1U << SynthType;

  do
  {
    struct TuComposerSynth *SynthStr = PCStr->pc_Synth;
    struct TuComposerSynthCode *SynthCodeStr = SynthStr->tcy_SynthCode;
    UWORD SynthLine = PCStr->pc_SynthPos[SynthType], InstData, i;
    BYTE Instruction;
    UBYTE SrcVar, DestVar;

    SynthCodeStr += SynthLine;

    if ( PCStr->pc_WaitSynthCount[SynthType]-- )
    {
ExecSynthDone:

      if ( ( PCStr->pc_SynthFlags & BitMask ) && (!( PCStr->pc_KillSynthCount[SynthType]-- )) )
        return ( 0L );

      return ( 1L );
    }

    PCStr->pc_WaitSynthCount[SynthType] = 0;

    if ( (SynthLine >= SynthStr->tcy_CodeEntries) || ((BYTE) PCStr->pc_WaitSynthType[SynthType] < 0) )
      goto ExecSynthDone;

    i = 4-1;

    do
    {
      BYTE WaitVolType;

      if ( ((WaitVolType = ~PCStr->pc_WaitSynthType[SynthType]) >= 0) && (WaitVolType == i) && (PCStr->pc_WaitSynthLine[SynthType] == SynthLine) )
        PCStr->pc_WaitSynthType[SynthType] = 0;

    } while ( i-- );

    Instruction = SynthCodeStr->tcyc_Instruction;
    DestVar = SynthCodeStr->tcyc_SrcDestVar;
    InstData = SynthCodeStr->tcyc_InstData;

    if ( (!( Instruction )) && (!( DestVar )) && (!( InstData )) )
      goto ExecSynthDone;

    SrcVar = DestVar >> 4U;
    DestVar &= 0x0F;

    SynthLine++;

    if ( Instruction < 0 )
    {
      struct EffectsTable *FXTable;
      void (*CheckFunction)( struct TuComposerBase *TuCBase, struct PlayerHostChannel *PHCStr, struct PlayerChannel *PCStr, UWORD Channel, UWORD *EffectByte, UWORD *DataWord, UWORD *Flags );
      UWORD FXByte, DataWord, Flags;

      FXByte = ~Instruction;

      FXTable = (struct EffectsTable *) TuCBase->tcb_EffectsTable + FXByte;

      DataWord = InstData + PCStr->pc_Variables[SrcVar];
      Flags = FXTable->fx_Flags;

      if ( (CheckFunction = FXTable->fx_CheckFXPtr) )
      {
        CheckFunction ( TuCBase, PHCStr, PCStr, PCStr->pc_HostChannel, &FXByte, &DataWord, &Flags );

        FXTable = (struct EffectsTable *) TuCBase->tcb_EffectsTable + FXByte;
      }

      if (!( FXTable->fx_PrePatternFXPtr ))
      {
        InstData = PHCStr->phc_VirtualChannel;

        PHCStr->phc_VirtualChannel = Channel;
        FXTable->fx_EffectPtr ( TuCBase, PHCStr, PCStr, PCStr->pc_HostChannel, FXByte, DataWord );

        PHCStr->phc_VirtualChannel = InstData;
      }

      PCStr->pc_SynthPos[SynthType] = SynthLine;
    } else
    {
      unsigned short (**FX_Execute)( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType );

      (APTR) FX_Execute = (APTR) TuCBase->tcb_SynthExecTable;

      PCStr->pc_SynthPos[SynthType] = FX_Execute[(UBYTE) Instruction] ( TuCBase, PCStr, Channel, SynthLine, SrcVar, DestVar, InstData, SynthType );
    }
  } while ( ++SynthCount );

  return ( 0L );
}

static void SE_VIBRATO_Do ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, LONG VibratoSlideValue )
{
  struct PlayerHostChannel *PHCStr = TuCBase->tcb_PlaySong->tcso_ChannelData + PCStr->pc_HostChannel;
  ULONG OldFrequency = PCStr->pc_Frequency;

  PCStr->pc_Frequency -= PCStr->pc_VibratoSlideValue;

  if ( VibratoSlideValue < 0 )
  {
    VibratoSlideValue = -VibratoSlideValue;

    if ( PHCStr->phc_Flags & PHCF_LinearFreq )
    {
      LinearSlideUp ( TuCBase, PCStr, PCStr->pc_Frequency, VibratoSlideValue );
    } else
    {
      AmigaSlideUp ( PCStr, PCStr->pc_Frequency, VibratoSlideValue );
    }
  } else
  {
    if ( PHCStr->phc_Flags & PHCF_LinearFreq )
    {
      LinearSlideDn ( TuCBase, PCStr, PCStr->pc_Frequency, VibratoSlideValue );
    } else
    {
      AmigaSlideDn ( PCStr, PCStr->pc_Frequency, VibratoSlideValue );
    }
  }

  PCStr->pc_VibratoSlideValue -= OldFrequency - PCStr->pc_Frequency;
}

static void SE_ARPEGIO_Do ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, WORD ArpeggioTranspose, UWORD ArpeggioFineTune )
{
  ULONG *FreqTable;
  ULONG Frequency, NextFreq, SlideFreq, OldFreq;
  UWORD Octave;
  WORD Note;

  Octave = ArpeggioTranspose / NUMNOTES;
  Note = ArpeggioTranspose % NUMNOTES;

  if ( Note < 0 )
  {
    Octave--;
    Note += NUMNOTES;

    ArpeggioFineTune = -ArpeggioFineTune;
  }

  FreqTable = TuCBase->tcb_FrequencyTable + Note;

  Frequency = *FreqTable++;
  NextFreq = *FreqTable - Frequency;
  Frequency += (LONG) (ArpeggioFineTune * (WORD) NextFreq) >> 8;

  if ( (WORD) ( Octave ) < 0 )
  {
    Octave = -Octave;
    Frequency >>= Octave;
  } else
  {
    Frequency <<= Octave;
  }

  OldFreq = PCStr->pc_Frequency;

  SlideFreq = PCStr->pc_ArpeggioSlideValue + OldFreq;

  PCStr->pc_Frequency = Frequency = ((unsigned long long) Frequency * SlideFreq) >> 16ULL;

  PCStr->pc_ArpeggioSlideValue += OldFreq - Frequency;
}

static void SE_TREMOLO_Do ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, LONG TremoloSlideValue )
{
  UWORD Volume = PCStr->pc_Volume;

  TremoloSlideValue -= PCStr->pc_TremoloSlideValue;

  if ( (TremoloSlideValue += Volume) < 0 )
    TremoloSlideValue = 0;

  if ( TremoloSlideValue > 255 )
    TremoloSlideValue = 255;

  PCStr->pc_Volume = TremoloSlideValue;

  PCStr->pc_TremoloSlideValue -= TremoloSlideValue - Volume;
}

static void SE_PANNOLO_Do ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, LONG PannoloSlideValue )
{
  UWORD Panning = PCStr->pc_Panning;

  PannoloSlideValue -= PCStr->pc_PannoloSlideValue;

  if ( (PannoloSlideValue += Panning) < 0 )
    PannoloSlideValue = 0;

  if ( PannoloSlideValue > 255 )
    PannoloSlideValue = 255;

  PCStr->pc_Panning = PannoloSlideValue;

  PCStr->pc_PannoloSlideValue -= PannoloSlideValue - Panning;
}

static UWORD SE_STOP ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  InstData += PCStr->pc_Variables[SrcVar];

  if ( InstData & 0x8000 )
  {
    PCStr->pc_StopForbidMask &= ~InstData;
  } else
  {
    PCStr->pc_StopForbidMask |= InstData;
  }

  return ( SynthLine );
}

static UWORD SE_KILL ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  InstData += PCStr->pc_Variables[SrcVar];

  PCStr->pc_KillSynthCount[SynthType] = InstData;

  PCStr->pc_SynthFlags |= 1<<SynthType;

  return ( SynthLine );
}

static UWORD SE_WAIT ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  InstData += PCStr->pc_Variables[SrcVar];

  PCStr->pc_WaitSynthCount[SynthType] = InstData;

  return ( SynthLine );
}

static UWORD SE_WAITVOL ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  InstData += PCStr->pc_Variables[SrcVar];

  PCStr->pc_WaitSynthLine[SynthType] = InstData;
  PCStr->pc_WaitSynthType[SynthType] = ~0;

  return ( SynthLine );
}

static UWORD SE_WAITPAN ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  InstData += PCStr->pc_Variables[SrcVar];

  PCStr->pc_WaitSynthLine[SynthType] = InstData;
  PCStr->pc_WaitSynthType[SynthType] = ~1;

  return ( SynthLine );
}

static UWORD SE_WAITSLD ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  InstData += PCStr->pc_Variables[SrcVar];

  PCStr->pc_WaitSynthLine[SynthType] = InstData;
  PCStr->pc_WaitSynthType[SynthType] = ~2;

  return ( SynthLine );
}

static UWORD SE_WAITSPC ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  InstData += PCStr->pc_Variables[SrcVar];

  PCStr->pc_WaitSynthLine[SynthType] = InstData;
  PCStr->pc_WaitSynthType[SynthType] = ~3;

  return ( SynthLine );
}

static UWORD SE_JUMP ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  InstData += PCStr->pc_Variables[SrcVar];

  return ( InstData );
}

static UWORD SE_JUMPEQ ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  if ( PCStr->pc_SynthConVar[SynthType] & TCYCF_Zero )
  {
    InstData += PCStr->pc_Variables[SrcVar];

    return ( InstData );
  }

  return ( SynthLine );
}

static UWORD SE_JUMPNE ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
 if (!( PCStr->pc_SynthConVar[SynthType] & TCYCF_Zero ))
  {
    InstData += PCStr->pc_Variables[SrcVar];

    return ( InstData );
  }

  return ( SynthLine );
}

static UWORD SE_JUMPPL ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
 if (!( PCStr->pc_SynthConVar[SynthType] & TCYCF_Negative ))
  {
    InstData += PCStr->pc_Variables[SrcVar];

    return ( InstData );
  }

  return ( SynthLine );
}

static UWORD SE_JUMPMI ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
 if ( PCStr->pc_SynthConVar[SynthType] & TCYCF_Negative )
  {
    InstData += PCStr->pc_Variables[SrcVar];

    return ( InstData );
  }

  return ( SynthLine );
}

static UWORD SE_JUMPLT ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD SynthConVar = PCStr->pc_SynthConVar[SynthType] & (TCYCF_Overflow|TCYCF_Negative);

  if ( (SynthConVar == TCYCF_Overflow) || (SynthConVar == TCYCF_Negative) )
  {
    InstData += PCStr->pc_Variables[SrcVar];

    return ( InstData );
  }

  return ( SynthLine );
}

static UWORD SE_JUMPLE ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD SynthConVar = PCStr->pc_SynthConVar[SynthType];

  if ( SynthConVar & TCYCF_Zero )
  {
    SynthConVar = PCStr->pc_SynthConVar[SynthType] & (TCYCF_Overflow|TCYCF_Negative);

    if ( (SynthConVar == TCYCF_Overflow) || (SynthConVar == TCYCF_Negative) )
    {
      InstData += PCStr->pc_Variables[SrcVar];

      return ( InstData );
    }
  }

  return ( SynthLine );
}

static UWORD SE_JUMPGT ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD SynthConVar = PCStr->pc_SynthConVar[SynthType];

  if (!( SynthConVar & TCYCF_Zero ))
  {
    SynthConVar = PCStr->pc_SynthConVar[SynthType] & (TCYCF_Overflow|TCYCF_Negative);

    if (!( (SynthConVar == TCYCF_Overflow) || (SynthConVar == TCYCF_Negative) ))
    {
      InstData += PCStr->pc_Variables[SrcVar];

      return ( InstData );
    }
  }

  return ( SynthLine );
}

static UWORD SE_JUMPGE ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD SynthConVar = PCStr->pc_SynthConVar[SynthType] & (TCYCF_Overflow|TCYCF_Negative);

  if (!( (SynthConVar == TCYCF_Overflow) || (SynthConVar == TCYCF_Negative) ))
  {
    InstData += PCStr->pc_Variables[SrcVar];

    return ( InstData );
  }

  return ( SynthLine );
}

static UWORD SE_JUMPVS ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
 if ( PCStr->pc_SynthConVar[SynthType] & TCYCF_Overflow )
  {
    InstData += PCStr->pc_Variables[SrcVar];

    return ( InstData );
  }

  return ( SynthLine );
}

static UWORD SE_JUMPVC ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
 if (!( PCStr->pc_SynthConVar[SynthType] & TCYCF_Overflow ))
  {
    InstData += PCStr->pc_Variables[SrcVar];

    return ( InstData );
  }

  return ( SynthLine );
}

static UWORD SE_JUMPCS ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
 if ( PCStr->pc_SynthConVar[SynthType] & TCYCF_Carry )
  {
    InstData += PCStr->pc_Variables[SrcVar];

    return ( InstData );
  }

  return ( SynthLine );
}

static UWORD SE_JUMPCC ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
 if (!( PCStr->pc_SynthConVar[SynthType] & TCYCF_Carry ))
  {
    InstData += PCStr->pc_Variables[SrcVar];

    return ( InstData );
  }

  return ( SynthLine );
}

static UWORD SE_JUMPLS ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD SynthConVar = PCStr->pc_SynthConVar[SynthType];

  if ( (SynthConVar & TCYCF_Zero) && (SynthConVar & TCYCF_Carry) )
  {
    InstData += PCStr->pc_Variables[SrcVar];

    return ( InstData );
  }

  return ( SynthLine );
}

static UWORD SE_JUMPHI ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD SynthConVar = PCStr->pc_SynthConVar[SynthType];

  if (!( (SynthConVar & TCYCF_Zero) && (SynthConVar & TCYCF_Carry) ))
  {
    InstData += PCStr->pc_Variables[SrcVar];

    return ( InstData );
  }

  return ( SynthLine );
}

static UWORD SE_JUMPVOL ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  if (!( PCStr->pc_StopForbidMask & 1 ))
  {
    InstData += PCStr->pc_Variables[SrcVar];

    PCStr->pc_VolumePos = InstData;
  }

  return ( SynthLine );
}

static UWORD SE_JUMPPAN ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  if (!( PCStr->pc_StopForbidMask & 2 ))
  {
    InstData += PCStr->pc_Variables[SrcVar];

    PCStr->pc_PanningPos = InstData;
  }

  return ( SynthLine );
}

static UWORD SE_JUMPSLD ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  if (!( PCStr->pc_StopForbidMask & 4 ))
  {
    InstData += PCStr->pc_Variables[SrcVar];

    PCStr->pc_SlidePos = InstData;
  }

  return ( SynthLine );
}

static UWORD SE_JUMPSPC ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  if (!( PCStr->pc_StopForbidMask & 8 ))
  {
    InstData += PCStr->pc_Variables[SrcVar];

    PCStr->pc_SpecialPos = InstData;
  }

  return ( SynthLine );
}

static UWORD SE_CALL ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  PCStr->pc_Variables[DestVar] = SynthLine;

  InstData += PCStr->pc_Variables[SrcVar];

  return ( InstData );
}

static UWORD SE_RETURN ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  InstData += PCStr->pc_Variables[SrcVar];

  PCStr->pc_Variables[DestVar] = --SynthLine;

  return ( InstData );
}

static UWORD SE_POSVAR ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  PCStr->pc_Variables[SrcVar] += SynthLine + --InstData;

  return ( SynthLine );
}

static UWORD SE_LOAD ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  InstData += PCStr->pc_Variables[SrcVar];

  PCStr->pc_Variables[DestVar] = InstData;

  return ( SynthLine );
}

static UWORD SE_ADD ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD Flags = 0;
  LONG AddData;

  InstData += PCStr->pc_Variables[SrcVar];

  AddData = (WORD) PCStr->pc_Variables[DestVar] + (WORD) InstData;
  PCStr->pc_Variables[DestVar] = AddData;

  if ( PCStr->pc_Variables[DestVar] < InstData )
    Flags += TCYCF_Carry|TCYCF_Extend;

  if ( (AddData < -0x8000) || (AddData >= 0x8000 ) )
    Flags += TCYCF_Overflow;

  if (!( AddData ))
    Flags += TCYCF_Zero;

  if ( AddData < 0 )
    Flags += TCYCF_Negative;

  PCStr->pc_SynthConVar[SynthType] = Flags;

  return ( SynthLine );
}

static UWORD SE_ADDX ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD Flags = PCStr->pc_SynthConVar[SynthType] & TCYCF_Zero;
  ULONG AddUData;
  LONG AddData;

  InstData += PCStr->pc_Variables[SrcVar];

  AddData = (WORD) PCStr->pc_Variables[DestVar] + (WORD) InstData;
  AddUData = InstData;

  if ( PCStr->pc_SynthConVar[SynthType] & TCYCF_Extend )
  {
    AddData++;
    AddUData++;
  }

  PCStr->pc_Variables[DestVar] = AddData;

  if ( (PCStr->pc_Variables[DestVar] < AddUData) )
    Flags += TCYCF_Carry|TCYCF_Extend;

  if ( (AddData < -0x8000) || (AddData >= 0x8000 ) )
    Flags += TCYCF_Overflow;

  if ( AddData )
    Flags &= ~TCYCF_Zero;

  if ( AddData < 0 )
    Flags += TCYCF_Negative;

  PCStr->pc_SynthConVar[SynthType] = Flags;

  return ( SynthLine );
}

static UWORD SE_SUB ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD Flags = 0;
  LONG SubData;

  InstData += PCStr->pc_Variables[SrcVar];

  SubData = (WORD) PCStr->pc_Variables[DestVar] - (WORD) InstData;

  if ( PCStr->pc_Variables[DestVar] < InstData )
    Flags += TCYCF_Carry|TCYCF_Extend;

  PCStr->pc_Variables[DestVar] = SubData;

  if ( (SubData < -0x8000) || (SubData >= 0x8000 ) )
    Flags += TCYCF_Overflow;

  if (!( SubData ))
    Flags += TCYCF_Zero;

  if ( SubData < 0 )
    Flags += TCYCF_Negative;

  PCStr->pc_SynthConVar[SynthType] = Flags;

  return ( SynthLine );
}

static UWORD SE_SUBX ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD Flags = PCStr->pc_SynthConVar[SynthType] & TCYCF_Zero;
  ULONG SubUData;
  LONG SubData;

  InstData += PCStr->pc_Variables[SrcVar];

  SubData = (WORD) PCStr->pc_Variables[DestVar] - (WORD) InstData;
  SubUData = InstData;

  if ( PCStr->pc_SynthConVar[SynthType] & TCYCF_Extend )
  {
    SubData--;
    SubUData++;
  }

  if ( PCStr->pc_Variables[DestVar] < SubUData )
    Flags += TCYCF_Carry|TCYCF_Extend;

  PCStr->pc_Variables[DestVar] = SubData;

  if ( (SubData < -0x8000) || (SubData >= 0x8000 ) )
    Flags += TCYCF_Overflow;

  if ( SubData )
    Flags &= ~TCYCF_Zero;

  if ( SubData < 0 )
    Flags += TCYCF_Negative;

  PCStr->pc_SynthConVar[SynthType] = Flags;

  return ( SynthLine );
}

static UWORD SE_CMP ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD Flags = PCStr->pc_SynthConVar[SynthType] & TCYCF_Extend;
  LONG SubData;

  InstData += PCStr->pc_Variables[SrcVar];

  SubData = (WORD) PCStr->pc_Variables[DestVar] - (WORD) InstData;

  if ( PCStr->pc_Variables[DestVar] < InstData )
    Flags += TCYCF_Carry;

  if ( (SubData < -0x8000) || (SubData >= 0x8000 ) )
    Flags += TCYCF_Overflow;

  if (!( SubData ))
    Flags += TCYCF_Zero;

  if ( SubData < 0 )
    Flags += TCYCF_Negative;

  PCStr->pc_SynthConVar[SynthType] = Flags;

  return ( SynthLine );
}

static UWORD SE_MULU ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD UMultResult, Flags = PCStr->pc_SynthConVar[SynthType] & TCYCF_Extend;

  InstData += PCStr->pc_Variables[SrcVar];

  PCStr->pc_Variables[DestVar] = UMultResult = (UBYTE) PCStr->pc_Variables[DestVar] * (UBYTE) InstData;

  if (!( UMultResult ))
    Flags += TCYCF_Zero;

  if ( (WORD) UMultResult < 0 )
    Flags += TCYCF_Negative;

  PCStr->pc_SynthConVar[SynthType] = Flags;

  return ( SynthLine );
}

static UWORD SE_MULS ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD Flags = PCStr->pc_SynthConVar[SynthType] & TCYCF_Extend;
  WORD SMultResult;

  InstData += PCStr->pc_Variables[SrcVar];

  PCStr->pc_Variables[DestVar] = SMultResult = (BYTE) PCStr->pc_Variables[DestVar] * (BYTE) InstData;

  if (!( SMultResult ))
    Flags += TCYCF_Zero;

  if ( SMultResult < 0 )
    Flags += TCYCF_Negative;

  PCStr->pc_SynthConVar[SynthType] = Flags;

  return ( SynthLine );
}

static UWORD SE_DMULU ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  ULONG UMultResult;
  UWORD Flags = PCStr->pc_SynthConVar[SynthType] & TCYCF_Extend;

  InstData += PCStr->pc_Variables[SrcVar];

  UMultResult = PCStr->pc_Variables[DestVar] * InstData;

  if ( DestVar == 15 )
  {
    PCStr->pc_Variables[DestVar] = UMultResult;

    if ( UMultResult >= 0x10000 )
      Flags += TCYCF_Overflow;

    UMultResult <<= 16UL;
  } else
  {
    PCStr->pc_Variables[DestVar++] = UMultResult >> 16UL;
    PCStr->pc_Variables[DestVar] = UMultResult;
  }

  if (!( UMultResult ))
    Flags += TCYCF_Zero;

  if ( (LONG) UMultResult < 0 )
    Flags += TCYCF_Negative;

  PCStr->pc_SynthConVar[SynthType] = Flags;

  return ( SynthLine );
}

static UWORD SE_DMULS ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  LONG SMultResult;
  UWORD Flags = PCStr->pc_SynthConVar[SynthType] & TCYCF_Extend;

  InstData += PCStr->pc_Variables[SrcVar];

  SMultResult = (WORD) PCStr->pc_Variables[DestVar] * (WORD) InstData;

  if ( DestVar == 15 )
  {
    PCStr->pc_Variables[DestVar] = SMultResult;

    if ( (SMultResult <= -0x10000) || (SMultResult >= 0x10000) )
      Flags += TCYCF_Overflow;

    SMultResult <<= 16L;
  } else
  {
    PCStr->pc_Variables[DestVar++] = SMultResult >> 16L;
    PCStr->pc_Variables[DestVar] = SMultResult;
  }

  if (!( SMultResult ))
    Flags += TCYCF_Zero;

  if ( SMultResult < 0 )
    Flags += TCYCF_Negative;

  PCStr->pc_SynthConVar[SynthType] = Flags;

  return ( SynthLine );
}

static UWORD SE_DIVU ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD UDivResult, Flags = PCStr->pc_SynthConVar[SynthType] & TCYCF_Extend;

  if ( (InstData += PCStr->pc_Variables[SrcVar] ))
  {
    PCStr->pc_Variables[DestVar] = UDivResult = PCStr->pc_Variables[DestVar] / InstData;

    if (!( UDivResult ))
      Flags += TCYCF_Zero;

    if ( (WORD) UDivResult < 0 )
      Flags += TCYCF_Negative;
  } else
  {
    Flags |= TCYCF_Carry|TCYCF_Overflow|TCYCF_Zero|TCYCF_Negative;
  }

  PCStr->pc_SynthConVar[SynthType] = Flags;

  return ( SynthLine );
}

static UWORD SE_DIVS ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD Flags = PCStr->pc_SynthConVar[SynthType] & TCYCF_Extend;
  WORD SDivResult;

  if ( (InstData += PCStr->pc_Variables[SrcVar] ))
  {
    PCStr->pc_Variables[DestVar] = SDivResult = (WORD) PCStr->pc_Variables[DestVar] / (WORD) InstData;

    if (!( SDivResult ))
      Flags += TCYCF_Zero;

    if ( SDivResult < 0 )
      Flags += TCYCF_Negative;
  } else
  {
    Flags |= TCYCF_Carry|TCYCF_Overflow|TCYCF_Zero|TCYCF_Negative;
  }

  PCStr->pc_SynthConVar[SynthType] = Flags;

  return ( SynthLine );
}

static UWORD SE_MODU ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD UModResult, Flags = PCStr->pc_SynthConVar[SynthType] & TCYCF_Extend;

  if ( (InstData += PCStr->pc_Variables[SrcVar] ))
  {
    PCStr->pc_Variables[DestVar] = UModResult = PCStr->pc_Variables[DestVar] % InstData;

    if (!( UModResult ))
      Flags += TCYCF_Zero;

    if ( (WORD) UModResult < 0 )
      Flags += TCYCF_Negative;
  } else
  {
    Flags |= TCYCF_Carry|TCYCF_Overflow|TCYCF_Zero|TCYCF_Negative;
  }

  PCStr->pc_SynthConVar[SynthType] = Flags;

  return ( SynthLine );
}

static UWORD SE_MODS ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD Flags = PCStr->pc_SynthConVar[SynthType] & TCYCF_Extend;
  WORD SModResult;

  if ( (InstData += PCStr->pc_Variables[SrcVar] ))
  {
    PCStr->pc_Variables[DestVar] = SModResult = (WORD) PCStr->pc_Variables[DestVar] % (WORD) InstData;

    if (!( SModResult ))
      Flags += TCYCF_Zero;

    if ( SModResult < 0 )
      Flags += TCYCF_Negative;
  } else
  {
    Flags |= TCYCF_Carry|TCYCF_Overflow|TCYCF_Zero|TCYCF_Negative;
  }

  PCStr->pc_SynthConVar[SynthType] = Flags;

  return ( SynthLine );
}

static UWORD SE_DDIVU ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  ULONG UDivResult;
  UWORD Flags = PCStr->pc_SynthConVar[SynthType] & TCYCF_Extend;

  if ( (InstData += PCStr->pc_Variables[SrcVar] ))
  {
    if ( DestVar == 15 )
    {
      PCStr->pc_Variables[DestVar] = UDivResult = (PCStr->pc_Variables[DestVar] << 16UL) / InstData;

      if ( UDivResult < 0x10000 )
      {
        if (!( UDivResult ))
          Flags += TCYCF_Zero;

        if ( (LONG) UDivResult < 0 )
          Flags += TCYCF_Negative;
      } else
      {
        Flags |= TCYCF_Overflow;
      }
    } else
    {
      ULONG Dividend = (PCStr->pc_Variables[DestVar++] << 16UL) + PCStr->pc_Variables[DestVar];

      if ( (UDivResult = (Dividend / InstData)) < 0x10000 )
      {
        PCStr->pc_Variables[DestVar--] = UDivResult;
        PCStr->pc_Variables[DestVar] = Dividend % InstData;

        if (!( UDivResult ))
          Flags += TCYCF_Zero;

        if ( (LONG) UDivResult < 0 )
          Flags += TCYCF_Negative;
      } else
      {
        Flags |= TCYCF_Overflow;
      }
    }
  } else
  {
    Flags |= TCYCF_Carry|TCYCF_Overflow|TCYCF_Zero|TCYCF_Negative;
  }

  PCStr->pc_SynthConVar[SynthType] = Flags;

  return ( SynthLine );
}

static UWORD SE_DDIVS ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  LONG SDivResult;
  UWORD Flags = PCStr->pc_SynthConVar[SynthType] & TCYCF_Extend;

  if ( (InstData += PCStr->pc_Variables[SrcVar] ))
  {
    if ( DestVar == 15 )
    {
      PCStr->pc_Variables[DestVar] = SDivResult = ((WORD) PCStr->pc_Variables[DestVar] << 16L) / (WORD) InstData;

      if ( (SDivResult < -0x10000) || (SDivResult < 0x10000) )
      {
        if (!( SDivResult ))
          Flags += TCYCF_Zero;

        if ( (LONG) SDivResult < 0 )
          Flags += TCYCF_Negative;
      } else
      {
        Flags |= TCYCF_Overflow;
      }
    } else
    {
      LONG Dividend = ((WORD) PCStr->pc_Variables[DestVar++] << 16UL) + (WORD) PCStr->pc_Variables[DestVar];

      SDivResult = Dividend / InstData;

      if ( (SDivResult < -0x10000) || (SDivResult < 0x10000) )
      {
        PCStr->pc_Variables[DestVar--] = SDivResult;
        PCStr->pc_Variables[DestVar] = Dividend % InstData;

        if (!( SDivResult ))
          Flags += TCYCF_Zero;

        if ( (LONG) SDivResult < 0 )
          Flags += TCYCF_Negative;
      } else
      {
        Flags |= TCYCF_Overflow;
      }
    }
  } else
  {
    Flags |= TCYCF_Carry|TCYCF_Overflow|TCYCF_Zero|TCYCF_Negative;
  }

  PCStr->pc_SynthConVar[SynthType] = Flags;

  return ( SynthLine );
}

static UWORD SE_ASHL ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD Flags = PCStr->pc_SynthConVar[SynthType] & TCYCF_Extend, HiBit;
  WORD ShiftValue = PCStr->pc_Variables[DestVar];

  InstData += PCStr->pc_Variables[SrcVar];
  InstData &= 0x3F;

  HiBit = ShiftValue & 0x8000;

  while ( InstData-- )
  {
    Flags &= ~(TCYCF_Carry|TCYCF_Extend);

    if ( ShiftValue & 0x8000 )
      Flags |= TCYCF_Carry|TCYCF_Extend;

    ShiftValue <<= 1;

    if ( HiBit != (ShiftValue & 0x8000) )
      Flags |= TCYCF_Overflow;
  }

  if ( ShiftValue < 0 )
    Flags |= TCYCF_Negative;

  if (!( ShiftValue ))
    Flags |= TCYCF_Zero;

  PCStr->pc_Variables[DestVar] = ShiftValue;
  PCStr->pc_SynthConVar[SynthType] = Flags;

  return ( SynthLine );
}

static UWORD SE_ASHR ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD Flags = PCStr->pc_SynthConVar[SynthType] & TCYCF_Extend;
  WORD ShiftValue = PCStr->pc_Variables[DestVar];

  InstData += PCStr->pc_Variables[SrcVar];
  InstData &= 0x3F;

  while ( InstData-- )
  {
    Flags &= ~(TCYCF_Carry|TCYCF_Extend);

    if ( ShiftValue & 0x1 )
      Flags |= TCYCF_Carry|TCYCF_Extend;

    ShiftValue >>= 1;
  }

  if ( ShiftValue < 0 )
    Flags |= TCYCF_Negative;

  if (!( ShiftValue ))
    Flags |= TCYCF_Zero;

  PCStr->pc_Variables[DestVar] = ShiftValue;
  PCStr->pc_SynthConVar[SynthType] = Flags;

  return ( SynthLine );
}

static UWORD SE_LSHL ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD Flags = PCStr->pc_SynthConVar[SynthType] & TCYCF_Extend;
  UWORD ShiftValue = PCStr->pc_Variables[DestVar];

  InstData += PCStr->pc_Variables[SrcVar];
  InstData &= 0x3F;

  while ( InstData-- )
  {
    Flags &= ~(TCYCF_Carry|TCYCF_Extend);

    if ( ShiftValue & 0x8000 )
      Flags |= TCYCF_Carry|TCYCF_Extend;

    ShiftValue <<= 1U;
  }

  if ( (WORD) ShiftValue < 0 )
    Flags |= TCYCF_Negative;

  if (!( ShiftValue ))
    Flags |= TCYCF_Zero;

  PCStr->pc_Variables[DestVar] = ShiftValue;
  PCStr->pc_SynthConVar[SynthType] = Flags;

  return ( SynthLine );
}

static UWORD SE_LSHR ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD Flags = PCStr->pc_SynthConVar[SynthType] & TCYCF_Extend;
  UWORD ShiftValue = PCStr->pc_Variables[DestVar];

  InstData += PCStr->pc_Variables[SrcVar];
  InstData &= 0x3F;

  while ( InstData-- )
  {
    Flags &= ~(TCYCF_Carry|TCYCF_Extend);

    if ( ShiftValue & 0x1 )
      Flags |= TCYCF_Carry|TCYCF_Extend;

    ShiftValue >>= 1U;
  }

  if ( (WORD) ShiftValue < 0 )
    Flags |= TCYCF_Negative;

  if (!( ShiftValue ))
    Flags |= TCYCF_Zero;

  PCStr->pc_Variables[DestVar] = ShiftValue;
  PCStr->pc_SynthConVar[SynthType] = Flags;

  return ( SynthLine );
}

static UWORD SE_ROL ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD Flags = PCStr->pc_SynthConVar[SynthType] & TCYCF_Extend;
  UWORD ShiftValue = PCStr->pc_Variables[DestVar];

  InstData += PCStr->pc_Variables[SrcVar];
  InstData &= 0x3F;

  while ( InstData-- )
  {
    Flags &= ~(TCYCF_Carry);

    if ( ShiftValue & 0x8000 )
      Flags |= TCYCF_Carry;

    ShiftValue <<= 1U;

    if ( Flags & TCYCF_Carry )
      ShiftValue++;
  }

  if ( (WORD) ShiftValue < 0 )
    Flags |= TCYCF_Negative;

  if (!( ShiftValue ))
    Flags |= TCYCF_Zero;

  PCStr->pc_Variables[DestVar] = ShiftValue;
  PCStr->pc_SynthConVar[SynthType] = Flags;

  return ( SynthLine );
}

static UWORD SE_ROR ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD Flags = PCStr->pc_SynthConVar[SynthType] & TCYCF_Extend;
  UWORD ShiftValue = PCStr->pc_Variables[DestVar];

  InstData += PCStr->pc_Variables[SrcVar];
  InstData &= 0x3F;

  while ( InstData-- )
  {
    Flags &= ~(TCYCF_Carry);

    if ( ShiftValue & 0x1 )
      Flags |= TCYCF_Carry;

    ShiftValue >>= 1U;

    if ( Flags & TCYCF_Carry )
      ShiftValue += 0x8000;
  }

  if ( (WORD) ShiftValue < 0 )
    Flags |= TCYCF_Negative;

  if (!( ShiftValue ))
    Flags |= TCYCF_Zero;

  PCStr->pc_Variables[DestVar] = ShiftValue;
  PCStr->pc_SynthConVar[SynthType] = Flags;

  return ( SynthLine );
}

static UWORD SE_ROLX ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD Flags = PCStr->pc_SynthConVar[SynthType] & (TCYCF_Carry|TCYCF_Extend);
  UWORD ShiftValue = PCStr->pc_Variables[DestVar];

  InstData += PCStr->pc_Variables[SrcVar];
  InstData &= 0x3F;

  while ( InstData-- )
  {
    Flags &= ~(TCYCF_Carry);

    if ( ShiftValue & 0x8000 )
      Flags |= TCYCF_Carry;

    ShiftValue <<= 1U;

    if ( Flags & TCYCF_Extend )
    {
      if (!( Flags & TCYCF_Carry ))
        Flags &= ~(TCYCF_Extend);

      ShiftValue++;
    } else
    {
      if ( Flags & TCYCF_Carry )
        Flags |= TCYCF_Extend;
    }
  }

  if ( (WORD) ShiftValue < 0 )
    Flags |= TCYCF_Negative;

  if (!( ShiftValue ))
    Flags |= TCYCF_Zero;

  PCStr->pc_Variables[DestVar] = ShiftValue;
  PCStr->pc_SynthConVar[SynthType] = Flags;

  return ( SynthLine );
}

static UWORD SE_RORX ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD Flags = PCStr->pc_SynthConVar[SynthType] & (TCYCF_Carry|TCYCF_Extend);
  UWORD ShiftValue = PCStr->pc_Variables[DestVar];

  InstData += PCStr->pc_Variables[SrcVar];
  InstData &= 0x3F;

  while ( InstData-- )
  {
    Flags &= ~(TCYCF_Carry);

    if ( ShiftValue & 0x1 )
      Flags |= TCYCF_Carry;

    ShiftValue >>= 1U;

    if ( Flags & TCYCF_Extend )
    {
      if (!( Flags & TCYCF_Carry ))
        Flags &= ~(TCYCF_Extend);

      ShiftValue += 0x8000;
    } else
    {
      if ( Flags & TCYCF_Carry )
        Flags |= TCYCF_Extend;
    }
  }

  if ( (WORD) ShiftValue < 0 )
    Flags |= TCYCF_Negative;

  if (!( ShiftValue ))
    Flags |= TCYCF_Zero;

  PCStr->pc_Variables[DestVar] = ShiftValue;
  PCStr->pc_SynthConVar[SynthType] = Flags;

  return ( SynthLine );
}

static UWORD SE_OR ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD Flags = PCStr->pc_SynthConVar[SynthType] & TCYCF_Extend;
  UWORD LogicValue;

  InstData += PCStr->pc_Variables[SrcVar];

  LogicValue = (PCStr->pc_Variables[DestVar] |= InstData );

  if ( (WORD) LogicValue < 0 )
    Flags += TCYCF_Negative;

  if (!( LogicValue ))
    Flags += TCYCF_Zero;

  PCStr->pc_SynthConVar[SynthType] = Flags;

  return ( SynthLine );
}

static UWORD SE_AND ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD Flags = PCStr->pc_SynthConVar[SynthType] & TCYCF_Extend;
  UWORD LogicValue;

  InstData += PCStr->pc_Variables[SrcVar];

  LogicValue = (PCStr->pc_Variables[DestVar] &= InstData );

  if ( (WORD) LogicValue < 0 )
    Flags += TCYCF_Negative;

  if (!( LogicValue ))
    Flags += TCYCF_Zero;

  PCStr->pc_SynthConVar[SynthType] = Flags;

  return ( SynthLine );
}

static UWORD SE_XOR ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD Flags = PCStr->pc_SynthConVar[SynthType] & TCYCF_Extend;
  UWORD LogicValue;

  InstData += PCStr->pc_Variables[SrcVar];

  LogicValue = (PCStr->pc_Variables[DestVar] ^= InstData );

  if ( (WORD) LogicValue < 0 )
    Flags += TCYCF_Negative;

  if (!( LogicValue ))
    Flags += TCYCF_Zero;

  PCStr->pc_SynthConVar[SynthType] = Flags;

  return ( SynthLine );
}

static UWORD SE_NOT ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD Flags = PCStr->pc_SynthConVar[SynthType] & TCYCF_Extend;
  UWORD LogicValue;

  InstData += PCStr->pc_Variables[SrcVar];

  LogicValue = ~PCStr->pc_Variables[DestVar];
  PCStr->pc_Variables[DestVar] = LogicValue + InstData;

  if (!( LogicValue ))
    Flags += TCYCF_Zero;

  if ( (WORD) LogicValue < 0 )
    Flags += TCYCF_Negative;

  PCStr->pc_SynthConVar[SynthType] = Flags;

  return ( SynthLine );
}

static UWORD SE_NEG ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD Flags = 0;
  WORD SubData;

  InstData += PCStr->pc_Variables[SrcVar];

  SubData = -PCStr->pc_Variables[DestVar];
  PCStr->pc_Variables[DestVar] = SubData + InstData;

  if ( SubData )
    Flags += TCYCF_Carry|TCYCF_Extend;

  if ( SubData == -0x8000 )
    Flags += TCYCF_Overflow;

  if (!( SubData ))
    Flags += TCYCF_Zero;

  if ( SubData < 0 )
    Flags += TCYCF_Negative;

  PCStr->pc_SynthConVar[SynthType] = Flags;

  return ( SynthLine );
}

static UWORD SE_NEGX ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD Flags = PCStr->pc_SynthConVar[SynthType] & TCYCF_Zero;
  WORD SubData;

  InstData += PCStr->pc_Variables[SrcVar];

  SubData = -PCStr->pc_Variables[DestVar];

  if ( Flags & TCYCF_Extend )
    SubData--;

  PCStr->pc_Variables[DestVar] = SubData + InstData;

  if ( (SubData == -0x8000) && (!( Flags & TCYCF_Extend )) )
    Flags += TCYCF_Overflow;

  Flags &= ~(TCYCF_Carry);

  if ( SubData || (Flags & TCYCF_Extend) )
    Flags |= TCYCF_Carry|TCYCF_Extend;

  if ( SubData )
    Flags &= ~(TCYCF_Zero);

  if ( SubData < 0 )
    Flags += TCYCF_Negative;

  PCStr->pc_SynthConVar[SynthType] = Flags;

  return ( SynthLine );
}

static UWORD SE_EXTB ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD Flags = PCStr->pc_SynthConVar[SynthType] & TCYCF_Extend;
  WORD ExtValue;

  InstData += PCStr->pc_Variables[SrcVar];

  ExtValue = (BYTE) PCStr->pc_Variables[DestVar];
  PCStr->pc_Variables[DestVar] = ExtValue + InstData;

  if (!( ExtValue ))
    Flags += TCYCF_Zero;

  if ( ExtValue < 0 )
    Flags += TCYCF_Negative;

  PCStr->pc_SynthConVar[SynthType] = Flags;

  return ( SynthLine );
}

static UWORD SE_EXT ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD Flags = PCStr->pc_SynthConVar[SynthType] & TCYCF_Extend;
  LONG ExtValue = 0;

  InstData += PCStr->pc_Variables[SrcVar];

  if ( DestVar != 15 )
    ExtValue = (WORD) PCStr->pc_Variables[DestVar + 1];

  PCStr->pc_Variables[DestVar] = (ExtValue >> 16UL) + InstData;

  if (!( ExtValue ))
    Flags += TCYCF_Zero;

  if ( ExtValue < 0 )
    Flags += TCYCF_Negative;

  PCStr->pc_SynthConVar[SynthType] = Flags;

  return ( SynthLine );
}

static UWORD SE_XCHG ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD Flags = PCStr->pc_SynthConVar[SynthType] & TCYCF_Extend;
  ULONG ExchangeValue;

  ExchangeValue = (PCStr->pc_Variables[DestVar] << 16UL) + PCStr->pc_Variables[SrcVar];

  PCStr->pc_Variables[DestVar] = ExchangeValue + InstData;
  PCStr->pc_Variables[SrcVar] = ExchangeValue >> 16UL;

  if (!( ExchangeValue ))
    Flags += TCYCF_Zero;

  if ( (LONG) ExchangeValue < 0 )
    Flags += TCYCF_Negative;

  PCStr->pc_SynthConVar[SynthType] = Flags;

  return ( SynthLine );
}

static UWORD SE_SWAP ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD Flags = PCStr->pc_SynthConVar[SynthType] & TCYCF_Extend;
  UWORD SwapValue;

  InstData += PCStr->pc_Variables[SrcVar];

  PCStr->pc_Variables[DestVar] = SwapValue = (PCStr->pc_Variables[DestVar] << 8U) + (PCStr->pc_Variables[DestVar] >> 8U);

  if (!( SwapValue ))
    Flags += TCYCF_Zero;

  if ( (WORD) SwapValue < 0 )
    Flags += TCYCF_Negative;

  PCStr->pc_SynthConVar[SynthType] = Flags;

  return ( SynthLine );
}

static UWORD SE_GETWAVE ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  struct TuComposerSynthWave *WaveformList = PCStr->pc_WaveformList;
  struct TuComposerSynthWave *WaveformStr = PCStr->pc_SampleWaveform;
  ULONG WaveNum = -1;

  InstData += PCStr->pc_Variables[SrcVar];

  for (;;)
  {
    WaveNum++;

    WaveformList = (struct TuComposerSynthWave *) WaveformList->tcyw_Node.mln_Succ;

    if (!( WaveformList->tcyw_Node.mln_Succ ))
      break;

    if ( WaveformList == WaveformStr )
    {
      InstData += WaveNum;

      break;
    }
  }

  PCStr->pc_Variables[DestVar] = InstData;

  return ( SynthLine );
}

static UWORD SE_GETWLEN ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD SmpLen = -1;

  if ( PCStr->pc_ChannelData.ms_SampleLen < 0x10000 )
    SmpLen = PCStr->pc_ChannelData.ms_SampleLen;

  InstData += PCStr->pc_Variables[SrcVar] + SmpLen;

  PCStr->pc_Variables[DestVar] = InstData;

  return ( SynthLine );
}

static UWORD SE_GETWPOS ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD SmpPos = -1;

  if ( PCStr->pc_ChannelData.ms_SamplePos < 0x10000 )
    SmpPos = PCStr->pc_ChannelData.ms_SamplePos;

  InstData += PCStr->pc_Variables[SrcVar] + SmpPos;

  PCStr->pc_Variables[DestVar] = InstData;

  return ( SynthLine );
}

static UWORD SE_GETCHAN ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  InstData += PCStr->pc_Variables[SrcVar] + PCStr->pc_HostChannel;

  PCStr->pc_Variables[DestVar] = InstData;

  return ( SynthLine );
}

static UWORD SE_GETNOTE ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD Note;

  Note = PCStr->pc_SampleNote;

  InstData += PCStr->pc_Variables[SrcVar] + --Note;

  PCStr->pc_Variables[DestVar] = InstData;

  return ( SynthLine );
}

static UWORD SE_GETRANS ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD Note;

  Note = PCStr->pc_SampleNote;

  InstData += PCStr->pc_Variables[SrcVar] + PCStr->pc_FinalNote - --Note;

  PCStr->pc_Variables[DestVar] = InstData;

  return ( SynthLine );
}

static UWORD SE_GETPTCH ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  ULONG Frequency = PCStr->pc_Frequency;

  InstData += PCStr->pc_Variables[SrcVar];
  Frequency += InstData;

  if ( DestVar != 15 )
    PCStr->pc_Variables[DestVar++] = Frequency >> 16UL;

  PCStr->pc_Variables[DestVar] = Frequency;

  return ( SynthLine );
}

static UWORD SE_GETPER ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  ULONG Frequency = PCStr->pc_Frequency, Period = 0L;

  if ( Frequency )
    Period = SLIDE_CONST / Frequency;

  InstData += PCStr->pc_Variables[SrcVar];
  Period += InstData;

  if ( DestVar != 15 )
    PCStr->pc_Variables[DestVar++] = Period >> 16UL;

  PCStr->pc_Variables[DestVar] = Period;

  return ( SynthLine );
}

static UWORD SE_GETFX ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{


  return ( SynthLine );
}

static UWORD SE_GETARPW ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  struct TuComposerSynthWave *WaveformList = PCStr->pc_WaveformList;
  struct TuComposerSynthWave *WaveformStr = PCStr->pc_ArpeggioWaveform;
  ULONG WaveNum = -1;

  InstData += PCStr->pc_Variables[SrcVar];

  for (;;)
  {
    WaveNum++;

    WaveformList = (struct TuComposerSynthWave *) WaveformList->tcyw_Node.mln_Succ;

    if (!( WaveformList->tcyw_Node.mln_Succ ))
      break;

    if ( WaveformList == WaveformStr )
    {
      InstData += WaveNum;

      break;
    }
  }

  PCStr->pc_Variables[DestVar] = InstData;

  return ( SynthLine );
}

static UWORD SE_GETARPV ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  struct TuComposerSynthWave *WaveformStr;

  InstData += PCStr->pc_Variables[SrcVar];

  if ( (WaveformStr = PCStr->pc_ArpeggioWaveform) )
  {
    ULONG WavePos = InstData % WaveformStr->tcyw_Samples;

    if ( WaveformStr->tcyw_Flags & TCYWF_8Bit )
    {
      PCStr->pc_Variables[DestVar] = ((UBYTE *) WaveformStr->tcyw_WaveData)[WavePos] << 8U;
    } else
    {
      PCStr->pc_Variables[DestVar] = WaveformStr->tcyw_WaveData[WavePos];
    }
  }

  return ( SynthLine );
}

static UWORD SE_GETARPL ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  struct TuComposerSynthWave *WaveformStr;

  if ( (WaveformStr = PCStr->pc_ArpeggioWaveform) )
  {
    UWORD WaveLen = -1;

    if ( WaveformStr->tcyw_Samples < 0x10000 )
      WaveLen = WaveformStr->tcyw_Samples;

    InstData += PCStr->pc_Variables[SrcVar] + WaveLen;

    PCStr->pc_Variables[DestVar] = InstData;
  }

  return ( SynthLine );
}

static UWORD SE_GETARPP ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  InstData += PCStr->pc_Variables[SrcVar] + PCStr->pc_ArpeggioPos;

  PCStr->pc_Variables[DestVar] = InstData;

  return ( SynthLine );
}

static UWORD SE_GETVIBW ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  struct TuComposerSynthWave *WaveformList = PCStr->pc_WaveformList;
  struct TuComposerSynthWave *WaveformStr = PCStr->pc_VibratoWaveform;
  ULONG WaveNum = -1;

  InstData += PCStr->pc_Variables[SrcVar];

  for (;;)
  {
    WaveNum++;

    WaveformList = (struct TuComposerSynthWave *) WaveformList->tcyw_Node.mln_Succ;

    if (!( WaveformList->tcyw_Node.mln_Succ ))
      break;

    if ( WaveformList == WaveformStr )
    {
      InstData += WaveNum;

      break;
    }
  }

  PCStr->pc_Variables[DestVar] = InstData;

  return ( SynthLine );
}

static UWORD SE_GETVIBV ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  struct TuComposerSynthWave *WaveformStr;

  InstData += PCStr->pc_Variables[SrcVar];

  if ( (WaveformStr = PCStr->pc_VibratoWaveform) )
  {
    ULONG WavePos = InstData % WaveformStr->tcyw_Samples;

    if ( WaveformStr->tcyw_Flags & TCYWF_8Bit )
    {
      PCStr->pc_Variables[DestVar] = ((UBYTE *) WaveformStr->tcyw_WaveData)[WavePos] << 8U;
    } else
    {
      PCStr->pc_Variables[DestVar] = WaveformStr->tcyw_WaveData[WavePos];
    }
  }

  return ( SynthLine );
}

static UWORD SE_GETVIBL ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  struct TuComposerSynthWave *WaveformStr;

  if ( (WaveformStr = PCStr->pc_VibratoWaveform) )
  {
    UWORD WaveLen = -1;

    if ( WaveformStr->tcyw_Samples < 0x10000 )
      WaveLen = WaveformStr->tcyw_Samples;

    InstData += PCStr->pc_Variables[SrcVar] + WaveLen;

    PCStr->pc_Variables[DestVar] = InstData;
  }

  return ( SynthLine );
}

static UWORD SE_GETVIBP ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  InstData += PCStr->pc_Variables[SrcVar] + PCStr->pc_VibratoPos;

  PCStr->pc_Variables[DestVar] = InstData;

  return ( SynthLine );
}

static UWORD SE_GETTRMW ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  struct TuComposerSynthWave *WaveformList = PCStr->pc_WaveformList;
  struct TuComposerSynthWave *WaveformStr = PCStr->pc_TremoloWaveform;
  ULONG WaveNum = -1;

  InstData += PCStr->pc_Variables[SrcVar];

  for (;;)
  {
    WaveNum++;

    WaveformList = (struct TuComposerSynthWave *) WaveformList->tcyw_Node.mln_Succ;

    if (!( WaveformList->tcyw_Node.mln_Succ ))
      break;

    if ( WaveformList == WaveformStr )
    {
      InstData += WaveNum;

      break;
    }
  }

  PCStr->pc_Variables[DestVar] = InstData;

  return ( SynthLine );
}

static UWORD SE_GETTRMV ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  struct TuComposerSynthWave *WaveformStr;

  InstData += PCStr->pc_Variables[SrcVar];

  if ( (WaveformStr = PCStr->pc_TremoloWaveform) )
  {
    ULONG WavePos = InstData % WaveformStr->tcyw_Samples;

    if ( WaveformStr->tcyw_Flags & TCYWF_8Bit )
    {
      PCStr->pc_Variables[DestVar] = ((UBYTE *) WaveformStr->tcyw_WaveData)[WavePos] << 8U;
    } else
    {
      PCStr->pc_Variables[DestVar] = WaveformStr->tcyw_WaveData[WavePos];
    }
  }

  return ( SynthLine );
}

static UWORD SE_GETTRML ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  struct TuComposerSynthWave *WaveformStr;

  if ( (WaveformStr = PCStr->pc_TremoloWaveform) )
  {
    UWORD WaveLen = -1;

    if ( WaveformStr->tcyw_Samples < 0x10000 )
      WaveLen = WaveformStr->tcyw_Samples;

    InstData += PCStr->pc_Variables[SrcVar] + WaveLen;

    PCStr->pc_Variables[DestVar] = InstData;
  }

  return ( SynthLine );
}

static UWORD SE_GETTRMP ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  InstData += PCStr->pc_Variables[SrcVar] + PCStr->pc_TremoloPos;

  PCStr->pc_Variables[DestVar] = InstData;

  return ( SynthLine );
}

static UWORD SE_GETPANW ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  struct TuComposerSynthWave *WaveformList = PCStr->pc_WaveformList;
  struct TuComposerSynthWave *WaveformStr = PCStr->pc_PannoloWaveform;
  ULONG WaveNum = -1;

  InstData += PCStr->pc_Variables[SrcVar];

  for (;;)
  {
    WaveNum++;

    WaveformList = (struct TuComposerSynthWave *) WaveformList->tcyw_Node.mln_Succ;

    if (!( WaveformList->tcyw_Node.mln_Succ ))
      break;

    if ( WaveformList == WaveformStr )
    {
      InstData += WaveNum;

      break;
    }
  }

  PCStr->pc_Variables[DestVar] = InstData;

  return ( SynthLine );
}

static UWORD SE_GETPANV ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  struct TuComposerSynthWave *WaveformStr;

  InstData += PCStr->pc_Variables[SrcVar];

  if ( (WaveformStr = PCStr->pc_PannoloWaveform) )
  {
    ULONG WavePos = InstData % WaveformStr->tcyw_Samples;

    if ( WaveformStr->tcyw_Flags & TCYWF_8Bit )
    {
      PCStr->pc_Variables[DestVar] = ((UBYTE *) WaveformStr->tcyw_WaveData)[WavePos] << 8U;
    } else
    {
      PCStr->pc_Variables[DestVar] = WaveformStr->tcyw_WaveData[WavePos];
    }
  }

  return ( SynthLine );
}

static UWORD SE_GETPANL ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  struct TuComposerSynthWave *WaveformStr;

  if ( (WaveformStr = PCStr->pc_PannoloWaveform) )
  {
    UWORD WaveLen = -1;

    if ( WaveformStr->tcyw_Samples < 0x10000 )
      WaveLen = WaveformStr->tcyw_Samples;

    InstData += PCStr->pc_Variables[SrcVar] + WaveLen;

    PCStr->pc_Variables[DestVar] = InstData;
  }

  return ( SynthLine );
}

static UWORD SE_GETPANP ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  InstData += PCStr->pc_Variables[SrcVar] + PCStr->pc_PannoloPos;

  PCStr->pc_Variables[DestVar] = InstData;

  return ( SynthLine );
}

static UWORD SE_GETRND ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  ULONG Seed;

  InstData += PCStr->pc_Variables[SrcVar];

  TuCBase->tcb_RandomSeed = Seed = ((LONG) TuCBase->tcb_RandomSeed * RANDOM_CONST) + 1;

  PCStr->pc_Variables[DestVar] = ((unsigned long long) Seed * InstData) >> 32ULL;

  return ( SynthLine );
}

static UWORD SE_GETSINE ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  WORD Degrees;

  InstData += PCStr->pc_Variables[SrcVar];

  Degrees = (WORD) InstData % 360;

  if ( Degrees < 0 )
    Degrees += 360;

  PCStr->pc_Variables[DestVar] = TuCBase->tcb_SineTable[Degrees];

  return ( SynthLine );
}

static UWORD SE_PORTAUP ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  struct PlayerHostChannel *PHCStr = TuCBase->tcb_PlaySong->tcso_ChannelData + PCStr->pc_HostChannel;

  if (!( InstData += PCStr->pc_Variables[SrcVar] ))
    InstData = PCStr->pc_PortaUpValue;

  PCStr->pc_PortaUpValue = InstData;
  PCStr->pc_PortamentoValue += InstData;

  if ( PHCStr->phc_Flags & PHCF_LinearFreq )
  {
    LinearSlideUp ( TuCBase, PCStr, PCStr->pc_Frequency, InstData );
  } else
  {
    AmigaSlideUp ( PCStr, PCStr->pc_Frequency, InstData );
  }

  return ( SynthLine );
}

static UWORD SE_PORTADN ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  struct PlayerHostChannel *PHCStr = TuCBase->tcb_PlaySong->tcso_ChannelData + PCStr->pc_HostChannel;

  if (!( InstData += PCStr->pc_Variables[SrcVar] ))
    InstData = PCStr->pc_PortaDnValue;

  PCStr->pc_PortaDnValue = InstData;
  PCStr->pc_PortamentoValue -= InstData;

  if ( PHCStr->phc_Flags & PHCF_LinearFreq )
  {
    LinearSlideDn ( TuCBase, PCStr, PCStr->pc_Frequency, InstData );
  } else
  {
    AmigaSlideDn ( PCStr, PCStr->pc_Frequency, InstData );
  }

  return ( SynthLine );
}

static UWORD SE_VIBSPD ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  InstData += PCStr->pc_Variables[SrcVar];

  PCStr->pc_VibratoRate = InstData;

  return ( SynthLine );
}

static UWORD SE_VIBDPTH ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  InstData += PCStr->pc_Variables[SrcVar];

  PCStr->pc_VibratoDepth = InstData;

  return ( SynthLine );
}

static UWORD SE_VIBWAVE ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  struct TuComposerSynthWave *WaveformStr = PCStr->pc_WaveformList;
  ULONG WaveNum = -1;

  InstData += PCStr->pc_Variables[SrcVar];

  PCStr->pc_VibratoWaveform = NULL;

  for (;;)
  {
    WaveNum++;

    WaveformStr = (struct TuComposerSynthWave *) WaveformStr->tcyw_Node.mln_Succ;

    if (!( WaveformStr->tcyw_Node.mln_Succ ))
      break;

    if ( WaveNum == InstData )
    {
      PCStr->pc_VibratoWaveform = WaveformStr;

      break;
    }
  }

  return ( SynthLine );
}

static UWORD SE_VIBWAVP ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  struct TuComposerSynthWave *WaveformStr;

  InstData += PCStr->pc_Variables[SrcVar];

  if ( (WaveformStr = PCStr->pc_VibratoWaveform) )
  {
    PCStr->pc_VibratoPos = InstData % WaveformStr->tcyw_Samples;
  } else
  {
    PCStr->pc_VibratoPos = InstData;
  }

  return ( SynthLine );
}

static UWORD SE_VIBRATO ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  struct TuComposerSynthWave *WaveformStr;
  UWORD VibratoRate;
  WORD VibratoDepth;

  InstData += PCStr->pc_Variables[SrcVar];

  if (!( VibratoRate = ( InstData >> 8U )))
    VibratoRate = PCStr->pc_VibratoRate;

  PCStr->pc_VibratoRate = VibratoRate;

  VibratoDepth = (InstData & 0xFF) << 2;

  if (!( VibratoDepth ))
    VibratoDepth = PCStr->pc_VibratoDepth;

  PCStr->pc_VibratoDepth = VibratoDepth;

  if ( (WaveformStr = PCStr->pc_VibratoWaveform) )
  {
    ULONG WavePos;
    LONG VibratoSlideValue;

    WavePos = PCStr->pc_VibratoPos % WaveformStr->tcyw_Samples;

    if ( WaveformStr->tcyw_Flags & TCYWF_8Bit )
    {
      VibratoSlideValue = ((BYTE *) WaveformStr->tcyw_WaveData)[WavePos] << 8;
    } else
    {
      VibratoSlideValue = WaveformStr->tcyw_WaveData[WavePos];
    }

    VibratoSlideValue *= -VibratoDepth;
    VibratoSlideValue >>= (7L-2L);

    PCStr->pc_VibratoPos = (WavePos + VibratoRate) % WaveformStr->tcyw_Samples;

    SE_VIBRATO_Do ( TuCBase, PCStr, VibratoSlideValue );
  }

  return ( SynthLine );
}

static UWORD SE_VIBVAL ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  LONG VibratoSlideValue;

  InstData += PCStr->pc_Variables[SrcVar];

  VibratoSlideValue = (WORD) InstData;

  SE_VIBRATO_Do ( TuCBase, PCStr, VibratoSlideValue );

  return ( SynthLine );
}

static UWORD SE_ARPSPD ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  InstData += PCStr->pc_Variables[SrcVar];

  PCStr->pc_ArpeggioSpeed = InstData;

  return ( SynthLine );
}

static UWORD SE_ARPWAVE ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  struct TuComposerSynthWave *WaveformStr = PCStr->pc_WaveformList;
  ULONG WaveNum = -1;

  InstData += PCStr->pc_Variables[SrcVar];

  PCStr->pc_ArpeggioWaveform = NULL;

  for (;;)
  {
    WaveNum++;

    WaveformStr = (struct TuComposerSynthWave *) WaveformStr->tcyw_Node.mln_Succ;

    if (!( WaveformStr->tcyw_Node.mln_Succ ))
      break;

    if ( WaveNum == InstData )
    {
      PCStr->pc_ArpeggioWaveform = WaveformStr;

      break;
    }
  }

  return ( SynthLine );
}

static UWORD SE_ARPWAVP ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  struct TuComposerSynthWave *WaveformStr;

  InstData += PCStr->pc_Variables[SrcVar];

  if ( (WaveformStr = PCStr->pc_ArpeggioWaveform) )
  {
    PCStr->pc_ArpeggioPos = InstData % WaveformStr->tcyw_Samples;
  } else
  {
    PCStr->pc_ArpeggioPos = InstData;
  }

  return ( SynthLine );
}

static UWORD SE_ARPEGIO ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  struct TuComposerSynthWave *WaveformStr;
  UWORD ArpeggioSpeed;

  InstData += PCStr->pc_Variables[SrcVar];

  if (!( ArpeggioSpeed = ( InstData >> 8U )))
    ArpeggioSpeed = PCStr->pc_ArpeggioSpeed;

  PCStr->pc_ArpeggioSpeed = ArpeggioSpeed;

  if ( (WaveformStr = PCStr->pc_ArpeggioWaveform) )
  {
    ULONG WavePos;
    BYTE ArpeggioTranspose;
    UBYTE ArpeggioFineTune;

    WavePos = PCStr->pc_ArpeggioPos % WaveformStr->tcyw_Samples;

    if ( WaveformStr->tcyw_Flags & TCYWF_8Bit )
    {
      ArpeggioTranspose = ((BYTE *) WaveformStr->tcyw_WaveData)[WavePos] << 8;
    } else
    {
      ArpeggioTranspose = WaveformStr->tcyw_WaveData[WavePos];
    }

    PCStr->pc_ArpeggioFineTune = ArpeggioFineTune = ArpeggioTranspose;
    PCStr->pc_ArpeggioTranspose = (ArpeggioTranspose >>= 8U);

    PCStr->pc_ArpeggioPos = (WavePos + ArpeggioSpeed) % WaveformStr->tcyw_Samples;

    SE_ARPEGIO_Do ( TuCBase, PCStr, ArpeggioTranspose, ArpeggioFineTune );
  }

  return ( SynthLine );
}

static UWORD SE_ARPVAL ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  BYTE ArpeggioTranspose;
  UBYTE ArpeggioFineTune;

  InstData += PCStr->pc_Variables[SrcVar];

  PCStr->pc_ArpeggioFineTune = ArpeggioFineTune = InstData;
  PCStr->pc_ArpeggioTranspose = ArpeggioTranspose = (ArpeggioTranspose >>= 8U);

  SE_ARPEGIO_Do ( TuCBase, PCStr, ArpeggioTranspose, ArpeggioFineTune );

  return ( SynthLine );
}

static UWORD SE_SETWAVE ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  struct MixerAllocStruct *MixStr;
  struct TuComposerSynthWave *WaveformStr = PCStr->pc_WaveformList;
  ULONG WaveNum = -1;
  UBYTE Flags;

  InstData += PCStr->pc_Variables[SrcVar];

  for (;;)
  {
    WaveNum++;

    WaveformStr = (struct TuComposerSynthWave *) WaveformStr->tcyw_Node.mln_Succ;

    if (!( WaveformStr->tcyw_Node.mln_Succ ))
      return ( SynthLine );

    if ( WaveNum == InstData )
      break;
  }

  PCStr->pc_SampleWaveform = WaveformStr;

  PCStr->pc_ChannelData.ms_SamplePos = 0;
  PCStr->pc_ChannelData.ms_SampleLen = WaveformStr->tcyw_Samples;
  PCStr->pc_ChannelData.ms_SampleData = WaveformStr->tcyw_WaveData;

  PCStr->pc_ChannelData.ms_Repeat = WaveformStr->tcyw_Repeat;
  PCStr->pc_ChannelData.ms_RepLen = WaveformStr->tcyw_RepLen;
  PCStr->pc_ChannelData.ms_RepCount = 0;
  PCStr->pc_ChannelData.ms_RepCounted = 0;

  if ( WaveformStr->tcyw_Flags & TCYWF_8Bit )
  {
    PCStr->pc_ChannelData.ms_BitDepth = 8;
  } else
  {
    PCStr->pc_ChannelData.ms_BitDepth = 16;
  }

  Flags = PCStr->pc_ChannelData.ms_Flags & (MSF_Surround|MSF_Play);

  if ( (!( WaveformStr->tcyw_Flags & TCYWF_NoLoop )) && (WaveformStr->tcyw_RepLen) )
    Flags += MSF_Loop;

  Flags += MSF_Synth;

  PCStr->pc_ChannelData.ms_Flags = Flags;

  MixStr = TuCBase->tcb_PlayMixerAlloc;
  ExternalMixerSetChannelBase ( MixStr, (struct MixerStruct *) &(PCStr->pc_ChannelData), VirtualChannel, MixStr->mas_MixLibBase );

  ExternalMixerGetChannelBase ( MixStr, (struct MixerStruct *) &(PCStr->pc_ChannelData), VirtualChannel, MixStr->mas_MixLibBase );

  return ( SynthLine );
}

static UWORD SE_ISETWAV ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  struct MixerAllocStruct *MixStr;
  struct TuComposerSynthWave *WaveformStr = PCStr->pc_WaveformList;
  ULONG WaveNum = -1;
  UBYTE Flags;

  InstData += PCStr->pc_Variables[SrcVar];

  for (;;)
  {
    WaveNum++;

    WaveformStr = (struct TuComposerSynthWave *) WaveformStr->tcyw_Node.mln_Succ;

    if (!( WaveformStr->tcyw_Node.mln_Succ ))
      return ( SynthLine );

    if ( WaveNum == InstData )
      break;
  }

  PCStr->pc_SampleWaveform = WaveformStr;

  PCStr->pc_ChannelData.ms_SamplePos = 0;
  PCStr->pc_ChannelData.ms_SampleLen = WaveformStr->tcyw_Samples;
  PCStr->pc_ChannelData.ms_SampleData = WaveformStr->tcyw_WaveData;

  PCStr->pc_ChannelData.ms_Repeat = WaveformStr->tcyw_Repeat;
  PCStr->pc_ChannelData.ms_RepLen = WaveformStr->tcyw_RepLen;
  PCStr->pc_ChannelData.ms_RepCount = 0;
  PCStr->pc_ChannelData.ms_RepCounted = 0;

  if ( WaveformStr->tcyw_Flags & TCYWF_8Bit )
  {
    PCStr->pc_ChannelData.ms_BitDepth = 8;
  } else
  {
    PCStr->pc_ChannelData.ms_BitDepth = 16;
  }

  Flags = PCStr->pc_ChannelData.ms_Flags & (MSF_Surround|MSF_Play);

  if ( (!( WaveformStr->tcyw_Flags & TCYWF_NoLoop )) && (WaveformStr->tcyw_RepLen) )
    Flags += MSF_Loop;

  PCStr->pc_ChannelData.ms_Flags = Flags;

  MixStr = TuCBase->tcb_PlayMixerAlloc;
  ExternalMixerSetChannelBase ( MixStr, (struct MixerStruct *) &(PCStr->pc_ChannelData), VirtualChannel, MixStr->mas_MixLibBase );

  return ( SynthLine );
}

static UWORD SE_SETWAVP ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  InstData += PCStr->pc_Variables[SrcVar];

  PCStr->pc_ChannelData.ms_SamplePos = InstData;

  return ( SynthLine );
}

static UWORD SE_SETRANS ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  ULONG *FreqTable;
  ULONG Frequency, NextFreq;
  UWORD Octave;
  WORD Note;
  BYTE FineTune;

  PCStr->pc_FinalNote = (InstData += PCStr->pc_Variables[SrcVar] + PCStr->pc_SampleNote);

  Octave = (WORD) InstData / NUMNOTES;
  Note = (WORD) InstData % NUMNOTES;

  if ( Note < 0 )
  {
    Octave--;

    Note += NUMNOTES;
  }

  if ( (FineTune = PCStr->pc_FineTune) < 0 )
  {
    FineTune += -0x80;

    Note--;
  }

  FreqTable = TuCBase->tcb_FrequencyTable + Note;

  Frequency = *FreqTable++;
  NextFreq = *FreqTable - Frequency;
  Frequency += (LONG) (FineTune * (WORD) NextFreq) >> 7;

  if ( (WORD) ( Octave -= 4 ) < 0 )
  {
    Octave = -Octave;

    PCStr->pc_Frequency = ((unsigned long long) Frequency * PCStr->pc_Sample->tcs_Rate) >> (16UL + Octave);
  } else
  {
    Frequency <<= Octave;

    PCStr->pc_Frequency = ((unsigned long long) Frequency * PCStr->pc_Sample->tcs_Rate) >> 16ULL;
  }

  return ( SynthLine );
}

static UWORD SE_SETNOTE ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  ULONG *FreqTable;
  ULONG Frequency, NextFreq;
  UWORD Octave;
  WORD Note;
  BYTE FineTune;

  InstData += PCStr->pc_Variables[SrcVar];

  Octave = (WORD) InstData / NUMNOTES;
  Note = (WORD) InstData % NUMNOTES;

  if ( Note < 0 )
  {
    Octave--;

    Note += NUMNOTES;
  }

  if ( (FineTune = PCStr->pc_FineTune) < 0 )
  {
    FineTune += -0x80;

    Note--;
  }

  FreqTable = TuCBase->tcb_FrequencyTable + Note;

  Frequency = *FreqTable++;
  NextFreq = *FreqTable - Frequency;
  Frequency += (LONG) (FineTune * (WORD) NextFreq) >> 7;

  if ( (WORD) ( Octave -= 4 ) < 0 )
  {
    Octave = -Octave;

    PCStr->pc_Frequency = ((unsigned long long) Frequency * PCStr->pc_Sample->tcs_Rate) >> (16UL + Octave);
  } else
  {
    Frequency <<= Octave;

    PCStr->pc_Frequency = ((unsigned long long) Frequency * PCStr->pc_Sample->tcs_Rate) >> 16ULL;
  }

  return ( SynthLine );
}

static UWORD SE_SETPTCH ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  ULONG Frequency;

  Frequency = InstData + PCStr->pc_Variables[SrcVar];

  if ( DestVar == 15 )
  {
    Frequency += PCStr->pc_Variables[DestVar];
  } else
  {
    Frequency += (PCStr->pc_Variables[DestVar++] << 16UL) + PCStr->pc_Variables[DestVar];
  }

  PCStr->pc_Frequency = Frequency;

  return ( SynthLine );
}

static UWORD SE_SETPER ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  ULONG Period;

  Period = InstData + PCStr->pc_Variables[SrcVar];

  if ( DestVar == 15 )
  {
    Period += PCStr->pc_Variables[DestVar];
  } else
  {
    Period += (PCStr->pc_Variables[DestVar++] << 16UL) + PCStr->pc_Variables[DestVar];
  }

  if ( Period )
  {
    PCStr->pc_Frequency = SLIDE_CONST / Period;
  } else
  {
    PCStr->pc_Frequency = 0L;
  }

  return ( SynthLine );
}

static UWORD SE_RESET ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  InstData += PCStr->pc_Variables[SrcVar];

  if (!( InstData & 0x01 ))
    PCStr->pc_ArpeggioSlideValue = 0L;

  if (!( InstData & 0x02 ))
    PCStr->pc_VibratoSlideValue = 0L;

  if (!( InstData & 0x04 ))
    PCStr->pc_TremoloSlideValue = 0;

  if (!( InstData & 0x08 ))
    PCStr->pc_PannoloSlideValue = 0;

  if (!( InstData & 0x10 ))
  {
    struct PlayerHostChannel *PHCStr = TuCBase->tcb_PlaySong->tcso_ChannelData + PCStr->pc_HostChannel;
    LONG PortaValue = PCStr->pc_PortamentoValue;

    if ( PortaValue < 0 )
    {
      PortaValue = -PortaValue;

      if ( PHCStr->phc_Flags & PHCF_LinearFreq )
      {
        LinearSlideDn ( TuCBase, PCStr, PCStr->pc_Frequency, PortaValue );
      } else
      {
        AmigaSlideDn ( PCStr, PCStr->pc_Frequency, PortaValue );
      }
    } else
    {
      if ( PHCStr->phc_Flags & PHCF_LinearFreq )
      {
        LinearSlideUp ( TuCBase, PCStr, PCStr->pc_Frequency, PortaValue );
      } else
      {
        AmigaSlideUp ( PCStr, PCStr->pc_Frequency, PortaValue );
      }
    }
  }

  if (!( InstData & 0x20 ))
    PCStr->pc_PortamentoValue = 0L;

  return ( SynthLine );
}

static UWORD SE_VOLSLUP ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD SlideVol = (PCStr->pc_Volume << 8U) + PCStr->pc_SubVol;

  if (!( InstData += PCStr->pc_Variables[SrcVar] ))
    InstData = PCStr->pc_VolSlUpValue;

  PCStr->pc_VolSlUpValue = InstData;

  if ( (SlideVol += InstData) < InstData )
    SlideVol = 0xFFFF;

  PCStr->pc_Volume = SlideVol >> 8U;
  PCStr->pc_SubVol = SlideVol;

  return ( SynthLine );
}

static UWORD SE_VOLSLDN ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD SlideVol = (PCStr->pc_Volume << 8U) + PCStr->pc_SubVol;

  if (!( InstData += PCStr->pc_Variables[SrcVar] ))
    InstData = PCStr->pc_VolSlDnValue;

  PCStr->pc_VolSlDnValue = InstData;

  if ( SlideVol < InstData )
    InstData = SlideVol;

  SlideVol -= InstData;

  PCStr->pc_Volume = SlideVol >> 8U;
  PCStr->pc_SubVol = SlideVol;

  return ( SynthLine );
}

static UWORD SE_TRMSPD ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  InstData += PCStr->pc_Variables[SrcVar];

  PCStr->pc_TremoloRate = InstData;

  return ( SynthLine );
}

static UWORD SE_TRMDPTH ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  InstData += PCStr->pc_Variables[SrcVar];

  PCStr->pc_TremoloRate = InstData;

  return ( SynthLine );
}

static UWORD SE_TRMWAVE ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  struct TuComposerSynthWave *WaveformStr = PCStr->pc_WaveformList;
  ULONG WaveNum = -1;

  InstData += PCStr->pc_Variables[SrcVar];

  PCStr->pc_TremoloWaveform = NULL;

  for (;;)
  {
    WaveNum++;

    WaveformStr = (struct TuComposerSynthWave *) WaveformStr->tcyw_Node.mln_Succ;

    if (!( WaveformStr->tcyw_Node.mln_Succ ))
      break;

    if ( WaveNum == InstData )
    {
      PCStr->pc_TremoloWaveform = WaveformStr;

      break;
    }
  }

  return ( SynthLine );
}

static UWORD SE_TRMWAVP ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  struct TuComposerSynthWave *WaveformStr;

  InstData += PCStr->pc_Variables[SrcVar];

  if ( (WaveformStr = PCStr->pc_TremoloWaveform) )
  {
    PCStr->pc_TremoloPos = InstData % WaveformStr->tcyw_Samples;
  } else
  {
    PCStr->pc_TremoloPos = InstData;
  }

  return ( SynthLine );
}

static UWORD SE_TREMOLO ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  struct TuComposerSynthWave *WaveformStr;
  UWORD TremoloRate;
  WORD TremoloDepth;

  InstData += PCStr->pc_Variables[SrcVar];

  if (!( TremoloRate = ( InstData >> 8U )))
    TremoloRate = PCStr->pc_TremoloRate;

  PCStr->pc_TremoloRate = TremoloRate;

  TremoloDepth = (InstData & 0xFF) << 2;

  if (!( TremoloDepth ))
    TremoloDepth = PCStr->pc_TremoloDepth;

  PCStr->pc_TremoloDepth = TremoloDepth;

  if ( (WaveformStr = PCStr->pc_VibratoWaveform) )
  {
    ULONG WavePos;
    LONG TremoloSlideValue;

    WavePos = PCStr->pc_TremoloPos % WaveformStr->tcyw_Samples;

    if ( WaveformStr->tcyw_Flags & TCYWF_8Bit )
    {
      TremoloSlideValue = ((BYTE *) WaveformStr->tcyw_WaveData)[WavePos] << 8;
    } else
    {
      TremoloSlideValue = WaveformStr->tcyw_WaveData[WavePos];
    }

    TremoloSlideValue *= -TremoloDepth;
    TremoloSlideValue >>= (7L-2L);

    PCStr->pc_TremoloPos = (WavePos + TremoloRate) % WaveformStr->tcyw_Samples;

    SE_TREMOLO_Do ( TuCBase, PCStr, TremoloSlideValue );
  }

  return ( SynthLine );
}

static UWORD SE_TRMVAL ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  LONG TremoloSlideValue;

  InstData += PCStr->pc_Variables[SrcVar];

  TremoloSlideValue = (WORD) InstData;

  SE_TREMOLO_Do ( TuCBase, PCStr, TremoloSlideValue );

  return ( SynthLine );
}

static UWORD SE_PANLEFT ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD Panning = (PCStr->pc_Panning << 8U) + PCStr->pc_SubPan;

  if (!( InstData += PCStr->pc_Variables[SrcVar] ))
    InstData = PCStr->pc_PanSlLeftValue;

  PCStr->pc_PanSlLeftValue = InstData;

  if ( Panning < InstData )
    InstData = Panning;

  Panning -= InstData;

  PCStr->pc_Panning = Panning >> 8U;
  PCStr->pc_SubPan = Panning;

  return ( SynthLine );
}

static UWORD SE_PANRGHT ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  UWORD Panning = (PCStr->pc_Panning << 8U) + PCStr->pc_SubPan;

  if (!( InstData += PCStr->pc_Variables[SrcVar] ))
    InstData = PCStr->pc_PanSlRightValue;

  PCStr->pc_PanSlRightValue = InstData;

  if ( (Panning += InstData) < InstData )
    Panning = 0xFFFF;

  PCStr->pc_Panning = Panning >> 8U;
  PCStr->pc_SubPan = Panning;

  return ( SynthLine );
}

static UWORD SE_PANSPD ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  InstData += PCStr->pc_Variables[SrcVar];

  PCStr->pc_PannoloRate = InstData;

  return ( SynthLine );
}

static UWORD SE_PANDPTH ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  InstData += PCStr->pc_Variables[SrcVar];

  PCStr->pc_PannoloDepth = InstData;

  return ( SynthLine );
}

static UWORD SE_PANWAVE ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  struct TuComposerSynthWave *WaveformStr = PCStr->pc_WaveformList;
  ULONG WaveNum = -1;

  InstData += PCStr->pc_Variables[SrcVar];

  PCStr->pc_PannoloWaveform = NULL;

  for (;;)
  {
    WaveNum++;

    WaveformStr = (struct TuComposerSynthWave *) WaveformStr->tcyw_Node.mln_Succ;

    if (!( WaveformStr->tcyw_Node.mln_Succ ))
      break;

    if ( WaveNum == InstData )
    {
      PCStr->pc_PannoloWaveform = WaveformStr;

      break;
    }
  }

  return ( SynthLine );
}

static UWORD SE_PANWAVP ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  struct TuComposerSynthWave *WaveformStr;

  InstData += PCStr->pc_Variables[SrcVar];

  if ( (WaveformStr = PCStr->pc_PannoloWaveform) )
  {
    PCStr->pc_PannoloPos = InstData % WaveformStr->tcyw_Samples;
  } else
  {
    PCStr->pc_PannoloPos = InstData;
  }

  return ( SynthLine );
}

static UWORD SE_PANNOLO ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  struct TuComposerSynthWave *WaveformStr;
  UWORD PannoloRate;
  WORD PannoloDepth;

  InstData += PCStr->pc_Variables[SrcVar];

  if (!( PannoloRate = ( InstData >> 8U )))
    PannoloRate = PCStr->pc_PannoloRate;

  PCStr->pc_PannoloRate = PannoloRate;

  PannoloDepth = (InstData & 0xFF) << 2;

  if (!( PannoloDepth ))
    PannoloDepth = PCStr->pc_PannoloDepth;

  PCStr->pc_PannoloDepth = PannoloDepth;

  if ( (WaveformStr = PCStr->pc_VibratoWaveform) )
  {
    ULONG WavePos;
    LONG PannoloSlideValue;

    WavePos = PCStr->pc_PannoloPos % WaveformStr->tcyw_Samples;

    if ( WaveformStr->tcyw_Flags & TCYWF_8Bit )
    {
      PannoloSlideValue = ((BYTE *) WaveformStr->tcyw_WaveData)[WavePos] << 8;
    } else
    {
      PannoloSlideValue = WaveformStr->tcyw_WaveData[WavePos];
    }

    PannoloSlideValue *= -PannoloDepth;
    PannoloSlideValue >>= (7L-2L);

    PCStr->pc_PannoloPos = (WavePos + PannoloRate) % WaveformStr->tcyw_Samples;

    SE_PANNOLO_Do ( TuCBase, PCStr, PannoloSlideValue );
  }

  return ( SynthLine );
}

static UWORD SE_PANVAL ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  LONG PannoloSlideValue;

  InstData += PCStr->pc_Variables[SrcVar];

  PannoloSlideValue = (WORD) InstData;

  SE_PANNOLO_Do ( TuCBase, PCStr, PannoloSlideValue );

  return ( SynthLine );
}

static UWORD SE_NOP ( struct TuComposerBase *TuCBase, struct PlayerChannel *PCStr, UWORD VirtualChannel, UWORD SynthLine, ULONG SrcVar, ULONG DestVar, UWORD InstData, ULONG SynthType )
{
  return ( SynthLine );
}

/* Note frequency table. Value is 65536*2^(x/12)
   Please note that the pointer actually points to the 2nd element.
   So the base C-4 is PitchTable[0], not PitchTable[-1]. B-3 is
   PitchTable[-1] */

ULONG PitchTable[] = {
  0x0000F1A2, /* B-3 [-1] */
  0x00010000, /* C-4 [0] */
  0x00010F39, /* C#4 [1] */
  0x00011F5A, /* D-4 [2] */
  0x00013070, /* D#4 [3] */
  0x0001428A, /* E-4 [4] */
  0x000155B8, /* F-4 [5] */
  0x00016A0A, /* F#4 [6] */
  0x00017F91, /* G-4 [7] */
  0x00019660, /* G#4 [8] */
  0x0001AE8A, /* A-4 [9] */
  0x0001C824, /* A#4 [10] */
  0x0001E343, /* B-4 [11] */
  0x00020000  /* C-5 [12] */
};

/* Linear frequence table (value is 65536*2^(x/3072)) */

UWORD LinearFreqTable[] = {
  0, 15, 30, 44, 59, 74, 89, 104, 118, 133, 148, 163, 178, 193, 207, 222,
  237, 252, 267, 282, 296, 311, 326, 341, 356, 371, 386, 400, 415, 430, 445, 460,
  475, 490, 505, 520, 535, 549, 564, 579, 594, 609, 624, 639, 654, 669, 684, 699,
  714, 729, 744, 758, 773, 788, 803, 818, 833, 848, 863, 878, 893, 908, 923, 938,
  953, 968, 983, 998, 1013, 1028, 1043, 1058, 1073, 1088, 1103, 1118, 1134, 1149, 1164, 1179,
  1194, 1209, 1224, 1239, 1254, 1269, 1284, 1299, 1314, 1329, 1344, 1360, 1375, 1390, 1405, 1420,
  1435, 1450, 1465, 1480, 1496, 1511, 1526, 1541, 1556, 1571, 1586, 1601, 1617, 1632, 1647, 1662,
  1677, 1692, 1708, 1723, 1738, 1753, 1768, 1784, 1799, 1814, 1829, 1844, 1859, 1875, 1890, 1905,
  1920, 1936, 1951, 1966, 1981, 1996, 2012, 2027, 2042, 2057, 2073, 2088, 2103, 2119, 2134, 2149,
  2164, 2180, 2195, 2210, 2225, 2241, 2256, 2271, 2287, 2302, 2317, 2333, 2348, 2363, 2379, 2394,
  2409, 2425, 2440, 2455, 2471, 2486, 2501, 2517, 2532, 2547, 2563, 2578, 2593, 2609, 2624, 2640,
  2655, 2670, 2686, 2701, 2716, 2732, 2747, 2763, 2778, 2794, 2809, 2824, 2840, 2855, 2871, 2886,
  2902, 2917, 2932, 2948, 2963, 2979, 2994, 3010, 3025, 3041, 3056, 3072, 3087, 3103, 3118, 3134,
  3149, 3165, 3180, 3196, 3211, 3227, 3242, 3258, 3273, 3289, 3304, 3320, 3335, 3351, 3366, 3382,
  3397, 3413, 3429, 3444, 3460, 3475, 3491, 3506, 3522, 3538, 3553, 3569, 3584, 3600, 3616, 3631,
  3647, 3662, 3678, 3694, 3709, 3725, 3740, 3756, 3772, 3787, 3803, 3819, 3834, 3850, 3866, 3881,
  3897, 3913, 3928, 3944, 3960, 3975, 3991, 4007, 4022, 4038, 4054, 4070, 4085, 4101, 4117, 4132,
  4148, 4164, 4180, 4195, 4211, 4227, 4242, 4258, 4274, 4290, 4305, 4321, 4337, 4353, 4369, 4384,
  4400, 4416, 4432, 4447, 4463, 4479, 4495, 4511, 4526, 4542, 4558, 4574, 4590, 4606, 4621, 4637,
  4653, 4669, 4685, 4701, 4716, 4732, 4748, 4764, 4780, 4796, 4812, 4827, 4843, 4859, 4875, 4891,
  4907, 4923, 4939, 4955, 4971, 4986, 5002, 5018, 5034, 5050, 5066, 5082, 5098, 5114, 5130, 5146,
  5162, 5178, 5194, 5210, 5226, 5241, 5257, 5273, 5289, 5305, 5321, 5337, 5353, 5369, 5385, 5401,
  5417, 5433, 5449, 5465, 5481, 5497, 5513, 5530, 5546, 5562, 5578, 5594, 5610, 5626, 5642, 5658,
  5674, 5690, 5706, 5722, 5738, 5754, 5770, 5787, 5803, 5819, 5835, 5851, 5867, 5883, 5899, 5915,
  5932, 5948, 5964, 5980, 5996, 6012, 6028, 6044, 6061, 6077, 6093, 6109, 6125, 6141, 6158, 6174,
  6190, 6206, 6222, 6239, 6255, 6271, 6287, 6303, 6320, 6336, 6352, 6368, 6384, 6401, 6417, 6433,
  6449, 6466, 6482, 6498, 6514, 6531, 6547, 6563, 6579, 6596, 6612, 6628, 6645, 6661, 6677, 6693,
  6710, 6726, 6742, 6759, 6775, 6791, 6808, 6824, 6840, 6857, 6873, 6889, 6906, 6922, 6938, 6955,
  6971, 6987, 7004, 7020, 7037, 7053, 7069, 7086, 7102, 7118, 7135, 7151, 7168, 7184, 7200, 7217,
  7233, 7250, 7266, 7283, 7299, 7315, 7332, 7348, 7365, 7381, 7398, 7414, 7431, 7447, 7463, 7480,
  7496, 7513, 7529, 7546, 7562, 7579, 7595, 7612, 7628, 7645, 7661, 7678, 7694, 7711, 7728, 7744,
  7761, 7777, 7794, 7810, 7827, 7843, 7860, 7876, 7893, 7910, 7926, 7943, 7959, 7976, 7992, 8009,
  8026, 8042, 8059, 8075, 8092, 8109, 8125, 8142, 8159, 8175, 8192, 8208, 8225, 8242, 8258, 8275,
  8292, 8308, 8325, 8342, 8358, 8375, 8392, 8408, 8425, 8442, 8458, 8475, 8492, 8509, 8525, 8542,
  8559, 8575, 8592, 8609, 8626, 8642, 8659, 8676, 8693, 8709, 8726, 8743, 8760, 8776, 8793, 8810,
  8827, 8843, 8860, 8877, 8894, 8911, 8927, 8944, 8961, 8978, 8995, 9012, 9028, 9045, 9062, 9079,
  9096, 9112, 9129, 9146, 9163, 9180, 9197, 9214, 9230, 9247, 9264, 9281, 9298, 9315, 9332, 9349,
  9366, 9382, 9399, 9416, 9433, 9450, 9467, 9484, 9501, 9518, 9535, 9552, 9569, 9586, 9603, 9620,
  9636, 9653, 9670, 9687, 9704, 9721, 9738, 9755, 9772, 9789, 9806, 9823, 9840, 9857, 9874, 9891,
  9908, 9925, 9942, 9959, 9976, 9993, 10011, 10028, 10045, 10062, 10079, 10096, 10113, 10130, 10147, 10164,
  10181, 10198, 10215, 10232, 10250, 10267, 10284, 10301, 10318, 10335, 10352, 10369, 10386, 10404, 10421, 10438,
  10455, 10472, 10489, 10506, 10524, 10541, 10558, 10575, 10592, 10610, 10627, 10644, 10661, 10678, 10695, 10713,
  10730, 10747, 10764, 10782, 10799, 10816, 10833, 10850, 10868, 10885, 10902, 10919, 10937, 10954, 10971, 10988,
  11006, 11023, 11040, 11058, 11075, 11092, 11109, 11127, 11144, 11161, 11179, 11196, 11213, 11231, 11248, 11265,
  11283, 11300, 11317, 11335, 11352, 11369, 11387, 11404, 11421, 11439, 11456, 11473, 11491, 11508, 11526, 11543,
  11560, 11578, 11595, 11613, 11630, 11647, 11665, 11682, 11700, 11717, 11735, 11752, 11769, 11787, 11804, 11822,
  11839, 11857, 11874, 11892, 11909, 11927, 11944, 11961, 11979, 11996, 12014, 12031, 12049, 12066, 12084, 12102,
  12119, 12137, 12154, 12172, 12189, 12207, 12224, 12242, 12259, 12277, 12294, 12312, 12330, 12347, 12365, 12382,
  12400, 12417, 12435, 12453, 12470, 12488, 12505, 12523, 12541, 12558, 12576, 12594, 12611, 12629, 12646, 12664,
  12682, 12699, 12717, 12735, 12752, 12770, 12788, 12805, 12823, 12841, 12858, 12876, 12894, 12912, 12929, 12947,
  12965, 12982, 13000, 13018, 13036, 13053, 13071, 13089, 13106, 13124, 13142, 13160, 13177, 13195, 13213, 13231,
  13249, 13266, 13284, 13302, 13320, 13337, 13355, 13373, 13391, 13409, 13427, 13444, 13462, 13480, 13498, 13516,
  13533, 13551, 13569, 13587, 13605, 13623, 13641, 13658, 13676, 13694, 13712, 13730, 13748, 13766, 13784, 13802,
  13819, 13837, 13855, 13873, 13891, 13909, 13927, 13945, 13963, 13981, 13999, 14017, 14035, 14053, 14071, 14088,
  14106, 14124, 14142, 14160, 14178, 14196, 14214, 14232, 14250, 14268, 14286, 14304, 14322, 14340, 14358, 14376,
  14394, 14413, 14431, 14449, 14467, 14485, 14503, 14521, 14539, 14557, 14575, 14593, 14611, 14629, 14647, 14665,
  14684, 14702, 14720, 14738, 14756, 14774, 14792, 14810, 14829, 14847, 14865, 14883, 14901, 14919, 14937, 14956,
  14974, 14992, 15010, 15028, 15046, 15065, 15083, 15101, 15119, 15137, 15156, 15174, 15192, 15210, 15228, 15247,
  15265, 15283, 15301, 15320, 15338, 15356, 15374, 15393, 15411, 15429, 15447, 15466, 15484, 15502, 15521, 15539,
  15557, 15575, 15594, 15612, 15630, 15649, 15667, 15685, 15704, 15722, 15740, 15759, 15777, 15795, 15814, 15832,
  15850, 15869, 15887, 15906, 15924, 15942, 15961, 15979, 15997, 16016, 16034, 16053, 16071, 16089, 16108, 16126,
  16145, 16163, 16182, 16200, 16218, 16237, 16255, 16274, 16292, 16311, 16329, 16348, 16366, 16385, 16403, 16422,
  16440, 16459, 16477, 16496, 16514, 16533, 16551, 16570, 16588, 16607, 16625, 16644, 16662, 16681, 16700, 16718,
  16737, 16755, 16774, 16792, 16811, 16830, 16848, 16867, 16885, 16904, 16922, 16941, 16960, 16978, 16997, 17016,
  17034, 17053, 17071, 17090, 17109, 17127, 17146, 17165, 17183, 17202, 17221, 17239, 17258, 17277, 17295, 17314,
  17333, 17352, 17370, 17389, 17408, 17426, 17445, 17464, 17483, 17501, 17520, 17539, 17557, 17576, 17595, 17614,
  17633, 17651, 17670, 17689, 17708, 17726, 17745, 17764, 17783, 17802, 17820, 17839, 17858, 17877, 17896, 17914,
  17933, 17952, 17971, 17990, 18009, 18028, 18046, 18065, 18084, 18103, 18122, 18141, 18160, 18179, 18197, 18216,
  18235, 18254, 18273, 18292, 18311, 18330, 18349, 18368, 18387, 18405, 18424, 18443, 18462, 18481, 18500, 18519,
  18538, 18557, 18576, 18595, 18614, 18633, 18652, 18671, 18690, 18709, 18728, 18747, 18766, 18785, 18804, 18823,
  18842, 18861, 18880, 18899, 18918, 18937, 18957, 18976, 18995, 19014, 19033, 19052, 19071, 19090, 19109, 19128,
  19147, 19167, 19186, 19205, 19224, 19243, 19262, 19281, 19300, 19320, 19339, 19358, 19377, 19396, 19415, 19435,
  19454, 19473, 19492, 19511, 19530, 19550, 19569, 19588, 19607, 19626, 19646, 19665, 19684, 19703, 19723, 19742,
  19761, 19780, 19800, 19819, 19838, 19857, 19877, 19896, 19915, 19934, 19954, 19973, 19992, 20012, 20031, 20050,
  20070, 20089, 20108, 20128, 20147, 20166, 20186, 20205, 20224, 20244, 20263, 20282, 20302, 20321, 20340, 20360,
  20379, 20399, 20418, 20437, 20457, 20476, 20496, 20515, 20534, 20554, 20573, 20593, 20612, 20632, 20651, 20670,
  20690, 20709, 20729, 20748, 20768, 20787, 20807, 20826, 20846, 20865, 20885, 20904, 20924, 20943, 20963, 20982,
  21002, 21021, 21041, 21060, 21080, 21099, 21119, 21139, 21158, 21178, 21197, 21217, 21236, 21256, 21276, 21295,
  21315, 21334, 21354, 21374, 21393, 21413, 21432, 21452, 21472, 21491, 21511, 21531, 21550, 21570, 21589, 21609,
  21629, 21648, 21668, 21688, 21708, 21727, 21747, 21767, 21786, 21806, 21826, 21845, 21865, 21885, 21905, 21924,
  21944, 21964, 21984, 22003, 22023, 22043, 22063, 22082, 22102, 22122, 22142, 22161, 22181, 22201, 22221, 22241,
  22260, 22280, 22300, 22320, 22340, 22360, 22379, 22399, 22419, 22439, 22459, 22479, 22498, 22518, 22538, 22558,
  22578, 22598, 22618, 22638, 22658, 22677, 22697, 22717, 22737, 22757, 22777, 22797, 22817, 22837, 22857, 22877,
  22897, 22917, 22937, 22957, 22977, 22996, 23016, 23036, 23056, 23076, 23096, 23116, 23136, 23156, 23176, 23196,
  23216, 23237, 23257, 23277, 23297, 23317, 23337, 23357, 23377, 23397, 23417, 23437, 23457, 23477, 23497, 23517,
  23537, 23558, 23578, 23598, 23618, 23638, 23658, 23678, 23698, 23719, 23739, 23759, 23779, 23799, 23819, 23839,
  23860, 23880, 23900, 23920, 23940, 23961, 23981, 24001, 24021, 24041, 24062, 24082, 24102, 24122, 24142, 24163,
  24183, 24203, 24223, 24244, 24264, 24284, 24304, 24325, 24345, 24365, 24386, 24406, 24426, 24446, 24467, 24487,
  24507, 24528, 24548, 24568, 24589, 24609, 24629, 24650, 24670, 24690, 24711, 24731, 24752, 24772, 24792, 24813,
  24833, 24853, 24874, 24894, 24915, 24935, 24956, 24976, 24996, 25017, 25037, 25058, 25078, 25099, 25119, 25139,
  25160, 25180, 25201, 25221, 25242, 25262, 25283, 25303, 25324, 25344, 25365, 25385, 25406, 25426, 25447, 25467,
  25488, 25508, 25529, 25550, 25570, 25591, 25611, 25632, 25652, 25673, 25694, 25714, 25735, 25755, 25776, 25797,
  25817, 25838, 25858, 25879, 25900, 25920, 25941, 25962, 25982, 26003, 26023, 26044, 26065, 26085, 26106, 26127,
  26148, 26168, 26189, 26210, 26230, 26251, 26272, 26292, 26313, 26334, 26355, 26375, 26396, 26417, 26438, 26458,
  26479, 26500, 26521, 26541, 26562, 26583, 26604, 26625, 26645, 26666, 26687, 26708, 26729, 26749, 26770, 26791,
  26812, 26833, 26854, 26874, 26895, 26916, 26937, 26958, 26979, 27000, 27021, 27041, 27062, 27083, 27104, 27125,
  27146, 27167, 27188, 27209, 27230, 27251, 27271, 27292, 27313, 27334, 27355, 27376, 27397, 27418, 27439, 27460,
  27481, 27502, 27523, 27544, 27565, 27586, 27607, 27628, 27649, 27670, 27691, 27712, 27733, 27754, 27775, 27796,
  27818, 27839, 27860, 27881, 27902, 27923, 27944, 27965, 27986, 28007, 28028, 28049, 28071, 28092, 28113, 28134,
  28155, 28176, 28197, 28219, 28240, 28261, 28282, 28303, 28324, 28346, 28367, 28388, 28409, 28430, 28452, 28473,
  28494, 28515, 28536, 28558, 28579, 28600, 28621, 28643, 28664, 28685, 28706, 28728, 28749, 28770, 28791, 28813,
  28834, 28855, 28877, 28898, 28919, 28941, 28962, 28983, 29005, 29026, 29047, 29069, 29090, 29111, 29133, 29154,
  29175, 29197, 29218, 29240, 29261, 29282, 29304, 29325, 29346, 29368, 29389, 29411, 29432, 29454, 29475, 29496,
  29518, 29539, 29561, 29582, 29604, 29625, 29647, 29668, 29690, 29711, 29733, 29754, 29776, 29797, 29819, 29840,
  29862, 29883, 29905, 29926, 29948, 29969, 29991, 30012, 30034, 30056, 30077, 30099, 30120, 30142, 30164, 30185,
  30207, 30228, 30250, 30272, 30293, 30315, 30336, 30358, 30380, 30401, 30423, 30445, 30466, 30488, 30510, 30531,
  30553, 30575, 30596, 30618, 30640, 30661, 30683, 30705, 30727, 30748, 30770, 30792, 30814, 30835, 30857, 30879,
  30900, 30922, 30944, 30966, 30988, 31009, 31031, 31053, 31075, 31097, 31118, 31140, 31162, 31184, 31206, 31227,
  31249, 31271, 31293, 31315, 31337, 31359, 31380, 31402, 31424, 31446, 31468, 31490, 31512, 31534, 31555, 31577,
  31599, 31621, 31643, 31665, 31687, 31709, 31731, 31753, 31775, 31797, 31819, 31841, 31863, 31885, 31907, 31929,
  31951, 31973, 31995, 32017, 32039, 32061, 32083, 32105, 32127, 32149, 32171, 32193, 32215, 32237, 32259, 32281,
  32303, 32325, 32347, 32369, 32392, 32414, 32436, 32458, 32480, 32502, 32524, 32546, 32568, 32591, 32613, 32635,
  32657, 32679, 32701, 32724, 32746, 32768, 32790, 32812, 32834, 32857, 32879, 32901, 32923, 32945, 32968, 32990,
  33012, 33034, 33057, 33079, 33101, 33123, 33146, 33168, 33190, 33213, 33235, 33257, 33279, 33302, 33324, 33346,
  33369, 33391, 33413, 33436, 33458, 33480, 33503, 33525, 33547, 33570, 33592, 33614, 33637, 33659, 33682, 33704,
  33726, 33749, 33771, 33794, 33816, 33838, 33861, 33883, 33906, 33928, 33951, 33973, 33995, 34018, 34040, 34063,
  34085, 34108, 34130, 34153, 34175, 34198, 34220, 34243, 34265, 34288, 34310, 34333, 34355, 34378, 34400, 34423,
  34446, 34468, 34491, 34513, 34536, 34558, 34581, 34604, 34626, 34649, 34671, 34694, 34717, 34739, 34762, 34785,
  34807, 34830, 34852, 34875, 34898, 34920, 34943, 34966, 34988, 35011, 35034, 35057, 35079, 35102, 35125, 35147,
  35170, 35193, 35216, 35238, 35261, 35284, 35307, 35329, 35352, 35375, 35398, 35420, 35443, 35466, 35489, 35512,
  35534, 35557, 35580, 35603, 35626, 35648, 35671, 35694, 35717, 35740, 35763, 35785, 35808, 35831, 35854, 35877,
  35900, 35923, 35946, 35969, 35991, 36014, 36037, 36060, 36083, 36106, 36129, 36152, 36175, 36198, 36221, 36244,
  36267, 36290, 36313, 36336, 36359, 36382, 36405, 36428, 36451, 36474, 36497, 36520, 36543, 36566, 36589, 36612,
  36635, 36658, 36681, 36704, 36727, 36750, 36773, 36796, 36820, 36843, 36866, 36889, 36912, 36935, 36958, 36981,
  37004, 37028, 37051, 37074, 37097, 37120, 37143, 37167, 37190, 37213, 37236, 37259, 37282, 37306, 37329, 37352,
  37375, 37399, 37422, 37445, 37468, 37491, 37515, 37538, 37561, 37584, 37608, 37631, 37654, 37678, 37701, 37724,
  37747, 37771, 37794, 37817, 37841, 37864, 37887, 37911, 37934, 37957, 37981, 38004, 38028, 38051, 38074, 38098,
  38121, 38144, 38168, 38191, 38215, 38238, 38261, 38285, 38308, 38332, 38355, 38379, 38402, 38426, 38449, 38472,
  38496, 38519, 38543, 38566, 38590, 38613, 38637, 38660, 38684, 38707, 38731, 38754, 38778, 38802, 38825, 38849,
  38872, 38896, 38919, 38943, 38966, 38990, 39014, 39037, 39061, 39084, 39108, 39132, 39155, 39179, 39202, 39226,
  39250, 39273, 39297, 39321, 39344, 39368, 39392, 39415, 39439, 39463, 39486, 39510, 39534, 39558, 39581, 39605,
  39629, 39652, 39676, 39700, 39724, 39747, 39771, 39795, 39819, 39843, 39866, 39890, 39914, 39938, 39961, 39985,
  40009, 40033, 40057, 40081, 40104, 40128, 40152, 40176, 40200, 40224, 40248, 40271, 40295, 40319, 40343, 40367,
  40391, 40415, 40439, 40463, 40486, 40510, 40534, 40558, 40582, 40606, 40630, 40654, 40678, 40702, 40726, 40750,
  40774, 40798, 40822, 40846, 40870, 40894, 40918, 40942, 40966, 40990, 41014, 41038, 41062, 41086, 41110, 41134,
  41158, 41182, 41207, 41231, 41255, 41279, 41303, 41327, 41351, 41375, 41399, 41424, 41448, 41472, 41496, 41520,
  41544, 41568, 41593, 41617, 41641, 41665, 41689, 41714, 41738, 41762, 41786, 41810, 41835, 41859, 41883, 41907,
  41932, 41956, 41980, 42004, 42029, 42053, 42077, 42101, 42126, 42150, 42174, 42199, 42223, 42247, 42272, 42296,
  42320, 42345, 42369, 42393, 42418, 42442, 42466, 42491, 42515, 42539, 42564, 42588, 42613, 42637, 42661, 42686,
  42710, 42735, 42759, 42784, 42808, 42833, 42857, 42881, 42906, 42930, 42955, 42979, 43004, 43028, 43053, 43077,
  43102, 43126, 43151, 43175, 43200, 43224, 43249, 43274, 43298, 43323, 43347, 43372, 43396, 43421, 43446, 43470,
  43495, 43519, 43544, 43569, 43593, 43618, 43642, 43667, 43692, 43716, 43741, 43766, 43790, 43815, 43840, 43864,
  43889, 43914, 43938, 43963, 43988, 44013, 44037, 44062, 44087, 44111, 44136, 44161, 44186, 44210, 44235, 44260,
  44285, 44310, 44334, 44359, 44384, 44409, 44434, 44458, 44483, 44508, 44533, 44558, 44583, 44607, 44632, 44657,
  44682, 44707, 44732, 44757, 44781, 44806, 44831, 44856, 44881, 44906, 44931, 44956, 44981, 45006, 45031, 45056,
  45081, 45106, 45131, 45155, 45180, 45205, 45230, 45255, 45280, 45305, 45330, 45355, 45381, 45406, 45431, 45456,
  45481, 45506, 45531, 45556, 45581, 45606, 45631, 45656, 45681, 45706, 45731, 45757, 45782, 45807, 45832, 45857,
  45882, 45907, 45932, 45958, 45983, 46008, 46033, 46058, 46083, 46109, 46134, 46159, 46184, 46209, 46235, 46260,
  46285, 46310, 46336, 46361, 46386, 46411, 46437, 46462, 46487, 46512, 46538, 46563, 46588, 46614, 46639, 46664,
  46690, 46715, 46740, 46766, 46791, 46816, 46842, 46867, 46892, 46918, 46943, 46968, 46994, 47019, 47045, 47070,
  47095, 47121, 47146, 47172, 47197, 47223, 47248, 47273, 47299, 47324, 47350, 47375, 47401, 47426, 47452, 47477,
  47503, 47528, 47554, 47579, 47605, 47630, 47656, 47681, 47707, 47733, 47758, 47784, 47809, 47835, 47860, 47886,
  47912, 47937, 47963, 47988, 48014, 48040, 48065, 48091, 48117, 48142, 48168, 48194, 48219, 48245, 48271, 48296,
  48322, 48348, 48373, 48399, 48425, 48450, 48476, 48502, 48528, 48553, 48579, 48605, 48631, 48656, 48682, 48708,
  48734, 48759, 48785, 48811, 48837, 48863, 48888, 48914, 48940, 48966, 48992, 49018, 49044, 49069, 49095, 49121,
  49147, 49173, 49199, 49225, 49251, 49276, 49302, 49328, 49354, 49380, 49406, 49432, 49458, 49484, 49510, 49536,
  49562, 49588, 49614, 49640, 49666, 49692, 49718, 49744, 49770, 49796, 49822, 49848, 49874, 49900, 49926, 49952,
  49978, 50004, 50030, 50056, 50082, 50108, 50135, 50161, 50187, 50213, 50239, 50265, 50291, 50317, 50343, 50370,
  50396, 50422, 50448, 50474, 50500, 50527, 50553, 50579, 50605, 50631, 50658, 50684, 50710, 50736, 50763, 50789,
  50815, 50841, 50868, 50894, 50920, 50946, 50973, 50999, 51025, 51052, 51078, 51104, 51131, 51157, 51183, 51210,
  51236, 51262, 51289, 51315, 51341, 51368, 51394, 51420, 51447, 51473, 51500, 51526, 51552, 51579, 51605, 51632,
  51658, 51685, 51711, 51738, 51764, 51790, 51817, 51843, 51870, 51896, 51923, 51949, 51976, 52002, 52029, 52056,
  52082, 52109, 52135, 52162, 52188, 52215, 52241, 52268, 52295, 52321, 52348, 52374, 52401, 52428, 52454, 52481,
  52507, 52534, 52561, 52587, 52614, 52641, 52667, 52694, 52721, 52747, 52774, 52801, 52827, 52854, 52881, 52908,
  52934, 52961, 52988, 53015, 53041, 53068, 53095, 53122, 53148, 53175, 53202, 53229, 53256, 53282, 53309, 53336,
  53363, 53390, 53416, 53443, 53470, 53497, 53524, 53551, 53578, 53605, 53631, 53658, 53685, 53712, 53739, 53766,
  53793, 53820, 53847, 53874, 53901, 53928, 53955, 53981, 54008, 54035, 54062, 54089, 54116, 54143, 54170, 54197,
  54224, 54251, 54278, 54306, 54333, 54360, 54387, 54414, 54441, 54468, 54495, 54522, 54549, 54576, 54603, 54630,
  54658, 54685, 54712, 54739, 54766, 54793, 54820, 54848, 54875, 54902, 54929, 54956, 54983, 55011, 55038, 55065,
  55092, 55119, 55147, 55174, 55201, 55228, 55256, 55283, 55310, 55337, 55365, 55392, 55419, 55447, 55474, 55501,
  55529, 55556, 55583, 55611, 55638, 55665, 55693, 55720, 55747, 55775, 55802, 55829, 55857, 55884, 55912, 55939,
  55966, 55994, 56021, 56049, 56076, 56104, 56131, 56158, 56186, 56213, 56241, 56268, 56296, 56323, 56351, 56378,
  56406, 56433, 56461, 56488, 56516, 56543, 56571, 56599, 56626, 56654, 56681, 56709, 56736, 56764, 56792, 56819,
  56847, 56874, 56902, 56930, 56957, 56985, 57013, 57040, 57068, 57096, 57123, 57151, 57179, 57206, 57234, 57262,
  57289, 57317, 57345, 57373, 57400, 57428, 57456, 57484, 57511, 57539, 57567, 57595, 57622, 57650, 57678, 57706,
  57734, 57761, 57789, 57817, 57845, 57873, 57901, 57929, 57956, 57984, 58012, 58040, 58068, 58096, 58124, 58152,
  58179, 58207, 58235, 58263, 58291, 58319, 58347, 58375, 58403, 58431, 58459, 58487, 58515, 58543, 58571, 58599,
  58627, 58655, 58683, 58711, 58739, 58767, 58795, 58823, 58851, 58879, 58907, 58935, 58964, 58992, 59020, 59048,
  59076, 59104, 59132, 59160, 59189, 59217, 59245, 59273, 59301, 59329, 59357, 59386, 59414, 59442, 59470, 59498,
  59527, 59555, 59583, 59611, 59640, 59668, 59696, 59724, 59753, 59781, 59809, 59837, 59866, 59894, 59922, 59951,
  59979, 60007, 60036, 60064, 60092, 60121, 60149, 60177, 60206, 60234, 60263, 60291, 60319, 60348, 60376, 60405,
  60433, 60461, 60490, 60518, 60547, 60575, 60604, 60632, 60661, 60689, 60717, 60746, 60774, 60803, 60831, 60860,
  60889, 60917, 60946, 60974, 61003, 61031, 61060, 61088, 61117, 61146, 61174, 61203, 61231, 61260, 61289, 61317,
  61346, 61374, 61403, 61432, 61460, 61489, 61518, 61546, 61575, 61604, 61632, 61661, 61690, 61718, 61747, 61776,
  61805, 61833, 61862, 61891, 61920, 61948, 61977, 62006, 62035, 62063, 62092, 62121, 62150, 62179, 62208, 62236,
  62265, 62294, 62323, 62352, 62381, 62409, 62438, 62467, 62496, 62525, 62554, 62583, 62612, 62641, 62670, 62698,
  62727, 62756, 62785, 62814, 62843, 62872, 62901, 62930, 62959, 62988, 63017, 63046, 63075, 63104, 63133, 63162,
  63191, 63220, 63249, 63278, 63308, 63337, 63366, 63395, 63424, 63453, 63482, 63511, 63540, 63569, 63599, 63628,
  63657, 63686, 63715, 63744, 63774, 63803, 63832, 63861, 63890, 63919, 63949, 63978, 64007, 64036, 64066, 64095,
  64124, 64153, 64183, 64212, 64241, 64270, 64300, 64329, 64358, 64388, 64417, 64446, 64476, 64505, 64534, 64564,
  64593, 64622, 64652, 64681, 64711, 64740, 64769, 64799, 64828, 64858, 64887, 64916, 64946, 64975, 65005, 65034,
  65064, 65093, 65123, 65152, 65182, 65211, 65241, 65270, 65300, 65329, 65359, 65388, 65418, 65447, 65477, 65506,
  0
};

struct EffectsTable FXTable[128] = {
  {Arpeggio, NULL, NULL, FXF_ExecWholeRow, 0x01, 0x0000},
  {PortaUp, NULL, CheckPorta, FXF_ExecWholeRow, 0x01, 0x0001},
  {PortaDn, NULL, CheckPorta, FXF_ExecWholeRow, 0x01, 0x0001},
  {FPortaUp, NULL, CheckPorta, FXF_ExecWholeRow, 0x01, 0x0001},
  {FPortaDn, NULL, CheckPorta, FXF_ExecWholeRow, 0x01, 0x0001},
  {PortaUpOnce, NULL, CheckPorta, 0x00, 0x01, 0x0000},
  {PortaDnOnce, NULL, CheckPorta, 0x00, 0x01, 0x0000},
  {FPortaUpOnce, NULL, CheckPorta, 0x00, 0x01, 0x0000},
  {FPortaDnOnce, NULL, CheckPorta, 0x00, 0x01, 0x0000},
  {TonePorta, PresetTonePorta, CheckTonePorta, FXF_ExecWholeRow, 0x01, 0x0001},
  {FTonePorta, PresetTonePorta, CheckTonePorta, FXF_ExecWholeRow, 0x01, 0x0001},
  {TPortaOnce, PresetTonePorta, CheckTonePorta, 0x00, 0x00, 0x0000},
  {FTPortaOnce, PresetTonePorta, CheckTonePorta, 0x00, 0x00, 0x0000},
  {NoteSlide, NULL, CheckNoteSlide, FXF_ExecWholeRow, 0x01, 0x0001},
  {Vibrato, PresetVibrato, NULL, FXF_ExecWholeRow, 0x01, 0x0001},
  {FVibrato, PresetVibrato, NULL, FXF_ExecWholeRow, 0x01, 0x0001},
  {Vibrato, PresetVibrato, NULL, 0x00, 0x01, 0x0000},
  {FVibrato, PresetVibrato, NULL, 0x00, 0x01, 0x0000},
  {DoKeyOff, NULL, NULL, FXF_ExecWholeRow, 0x01, 0x0000},
  {HoldDelay, NULL, NULL, FXF_ExecWholeRow, 0x01, 0x0000},
  {NoteFade, NULL, NULL, FXF_ExecWholeRow, 0x01, 0x0000},
  {NoteCut, NULL, NULL, FXF_ExecWholeRow, 0x01, 0x0000},
  {NoteDelay, PresetNoteDelay, NULL, 0x00, 0x00, 0x0000},
  {Tremor, NULL, NULL, FXF_ExecWholeRow, 0x01, 0x0000},
  {Retrig, NULL, NULL, FXF_ExecWholeRow, 0x01, 0x0000},
  {MulRetrig, NULL, NULL, FXF_ExecWholeRow, 0x01, 0x0000},
  {ExtCtrl, NULL, NULL, 0x00, 0x01, 0x0000},
  {InvLoop, NULL, NULL, FXF_ExecWholeRow, 0x01, 0x0001},
  {ExecFX, NULL, NULL, 0x00, 0x01, 0x0000},
  {StopFX, NULL, NULL, 0x00, 0x01, 0x0000},
  {NULL, NULL, NULL, 0x00, 0x00, 0x0000},
  {NULL, NULL, NULL, 0x00, 0x00, 0x0000},

  {SetVol, NULL, NULL, 0x00, 0x01, 0x0000},
  {VolSlUp, NULL, CheckVolSlide, FXF_ExecWholeRow, 0x01, 0x0001},
  {VolSlDn, NULL, CheckVolSlide, FXF_ExecWholeRow, 0x01, 0x0001},
  {FVolSlUp, NULL, CheckVolSlide, 0x00, 0x01, 0x0000},
  {FVolSlDn, NULL, CheckVolSlide, 0x00, 0x01, 0x0000},
  {VolSlTo, NULL, CheckVolSlTo, FXF_ExecWholeRow, 0x01, 0x0001},
  {Tremo, PresetTremolo, NULL, FXF_ExecWholeRow, 0x01, 0x0001},
  {Tremo, PresetTremolo, NULL, 0x00, 0x01, 0x0000},
  {SetTVol, NULL, NULL, 0x00, 0x01, 0x0000},
  {TVolSlUp, NULL, CheckTVolSlide, FXF_ExecWholeRow, 0x01, 0x0001},
  {TVolSlDn, NULL, CheckTVolSlide, FXF_ExecWholeRow, 0x01, 0x0001},
  {FTVolSlUp, NULL, CheckTVolSlide, 0x00, 0x01, 0x0000},
  {FTVolSlDn, NULL, CheckTVolSlide, 0x00, 0x01, 0x0000},
  {TVolSlTo, NULL, CheckVolSlTo, FXF_ExecWholeRow, 0x01, 0x0001},
  {TTremo, NULL, NULL, FXF_ExecWholeRow, 0x01, 0x0001},
  {TTremo, NULL, NULL, 0x00, 0x01, 0x0000},

  {SetPan, NULL, NULL, 0x00, 0x01, 0x0000},
  {PanSlLeft, NULL, CheckPanSlide, FXF_ExecWholeRow, 0x01, 0x0001},
  {PanSlRight, NULL, CheckPanSlide, FXF_ExecWholeRow, 0x01, 0x0001},
  {FPanSlLeft, NULL, CheckPanSlide, 0x00, 0x01, 0x0000},
  {FPanSlRight, NULL, CheckPanSlide, 0x00, 0x01, 0x0000},
  {PanSlTo, NULL, CheckVolSlTo, FXF_ExecWholeRow, 0x01, 0x0001},
  {Pannolo, NULL, NULL, FXF_ExecWholeRow, 0x01, 0x0001},
  {Pannolo, NULL, NULL, 0x00, 0x01, 0x0000},
  {SetTPan, NULL, NULL, 0x00, 0x01, 0x0000},
  {TPanSlLeft, NULL, CheckTPanSlide, FXF_ExecWholeRow, 0x01, 0x0001},
  {TPanSlRight, NULL, CheckTPanSlide, FXF_ExecWholeRow, 0x01, 0x0001},
  {FTPanSlLeft, NULL, CheckTPanSlide, 0x00, 0x01, 0x0000},
  {FTPanSlRight, NULL, CheckTPanSlide, 0x00, 0x01, 0x0000},
  {TPanSlTo, NULL, CheckVolSlTo, FXF_ExecWholeRow, 0x01, 0x0001},
  {TPannolo, NULL, NULL, FXF_ExecWholeRow, 0x01, 0x0001},
  {TPannolo, NULL, NULL, 0x00, 0x01, 0x0000},

  {SetTempo, NULL, NULL, 0x00, 0x02, 0x0000},
  {SetRelTempo, NULL, NULL, 0x00, 0x02, 0x0000},
  {PattBrk, NULL, NULL, 0x00, 0x02, 0x0000},
  {PosJump, NULL, NULL, 0x00, 0x02, 0x0000},
  {RelPosJmp, NULL, NULL, 0x00, 0x02, 0x0000},
  {ChgPattern, NULL, NULL, 0x00, 0x02, 0x0000},
  {RevPattPlay, NULL, NULL, 0x00, 0x02, 0x0000},
  {PattDelay, NULL, NULL, 0x00, 0x02, 0x0000},
  {FPattDelay, NULL, NULL, 0x00, 0x02, 0x0000},
  {PattLoop, NULL, NULL, 0x00, 0x02, 0x0000},
  {GoSub, NULL, NULL, 0x00, 0x02, 0x0000},
  {Return, NULL, NULL, 0x00, 0x02, 0x0000},
  {ChanSync, NULL, NULL, 0x00, 0x02, 0x0000},
  {SetSubSl, NULL, NULL, 0x00, 0x02, 0x0000},
  {NULL, NULL, NULL, 0x00, 0x00, 0x0000},
  {NULL, NULL, NULL, 0x00, 0x00, 0x0000},

  {SmpOffHI, NULL, NULL, 0x00, 0x01, 0x0000},
  {SmpOffLO, NULL, NULL, 0x00, 0x01, 0x0000},
  {SetHold, NULL, NULL, 0x00, 0x01, 0x0000},
  {SetDecay, NULL, NULL, 0x00, 0x01, 0x0000},
  {SetTransp, PresetSetTransp, NULL, 0x00, 0x01, 0x0000},
  {InsCtrl, NULL, NULL, 0x00, 0x01, 0x0000},
  {InsChg, NULL, NULL, 0x00, 0x01, 0x0000},
  {SynthCtrl, NULL, NULL, 0x00, 0x01, 0x0000},
  {SetSynthVal, NULL, NULL, 0x00, 0x01, 0x0000},
  {EnvCtrl, NULL, NULL, 0x00, 0x01, 0x0000},
  {SetEnvVal, NULL, NULL, 0x00, 0x01, 0x0000},
  {NNACtrl, NULL, NULL, 0x00, 0x01, 0x0000},
  {LoopCtrl, NULL, NULL, 0x00, 0x01, 0x0000},
  {NULL, NULL, NULL, 0x00, 0x00, 0x0000},
  {NULL, NULL, NULL, 0x00, 0x00, 0x0000},
  {NULL, NULL, NULL, 0x00, 0x00, 0x0000},

  {SetSpeed, NULL, NULL, 0x00, 0x00, 0x0000},
  {SpdSlFast, NULL, CheckSpeedSlide, FXF_ExecWholeRow, 0x00, 0x0001},
  {SpdSlSlow, NULL, CheckSpeedSlide, FXF_ExecWholeRow, 0x00, 0x0001},
  {FSpdSlFast, NULL, CheckSpeedSlide, 0x00, 0x00, 0x0000},
  {FSpdSlSlow, NULL, CheckSpeedSlide, 0x00, 0x00, 0x0000},
  {SpdSlTo, NULL, CheckVolSlTo, FXF_ExecWholeRow, 0x01, 0x0001},
  {Spenolo, NULL, NULL, FXF_ExecWholeRow, 0x00, 0x0001},
  {Spenolo, NULL, NULL, 0x00, 0x00, 0x0000},
  {ChCtrl, NULL, CheckChCtrl, 0x00, 0x00, 0x0000},
  {SetGVol, NULL, NULL, 0x00, 0x00, 0x0000},
  {GVolSlUp, NULL, CheckGVolSlide, FXF_ExecWholeRow, 0x00, 0x0001},
  {GVolSlDn, NULL, CheckGVolSlide, FXF_ExecWholeRow, 0x00, 0x0001},
  {FGVolSlUp, NULL, CheckGVolSlide, 0x00, 0x00, 0x0000},
  {FGVolSlDn, NULL, CheckGVolSlide, 0x00, 0x00, 0x0000},
  {GVolSlTo, NULL, CheckVolSlTo, FXF_ExecWholeRow, 0x00, 0x0001},
  {GTremo, NULL, NULL, FXF_ExecWholeRow, 0x00, 0x0001},
  {GTremo, NULL, NULL, 0x00, 0x00, 0x0000},
  {SetGPan, NULL, NULL, 0x00, 0x00, 0x0000},
  {GPanSlLeft, NULL, CheckGPanSlide, FXF_ExecWholeRow, 0x00, 0x0001},
  {GPanSlRight, NULL, CheckGPanSlide, FXF_ExecWholeRow, 0x00, 0x0001},
  {FGPanSlLeft, NULL, CheckGPanSlide, 0x00, 0x00, 0x0000},
  {FGPanSlRight, NULL, CheckGPanSlide, 0x00, 0x00, 0x0000},
  {GPanSlTo, NULL, CheckVolSlTo, FXF_ExecWholeRow, 0x01, 0x0000},
  {GPannolo, NULL, NULL, FXF_ExecWholeRow, 0x00, 0x0001},
  {GPannolo, NULL, NULL, 0x00, 0x00, 0x0000},

  {NULL, NULL, NULL, 0x00, 0x00, 0x0000},
  {NULL, NULL, NULL, 0x00, 0x00, 0x0000},
  {NULL, NULL, NULL, 0x00, 0x00, 0x0000},
  {NULL, NULL, NULL, 0x00, 0x00, 0x0000},
  {NULL, NULL, NULL, 0x00, 0x00, 0x0000},
  {NULL, NULL, NULL, 0x00, 0x00, 0x0000},

  {UserSync, NULL, NULL, FXF_ExecWholeRow, 0x00, 0x0000}
};

APTR SETab[128] = {
  SE_STOP, SE_KILL, SE_WAIT, SE_WAITVOL, SE_WAITPAN, SE_WAITSLD, SE_WAITSPC, SE_JUMP,
  SE_JUMPEQ, SE_JUMPNE, SE_JUMPPL, SE_JUMPMI, SE_JUMPLT, SE_JUMPLE, SE_JUMPGT, SE_JUMPGE,
  SE_JUMPVS, SE_JUMPVC, SE_JUMPCS, SE_JUMPCC, SE_JUMPLS, SE_JUMPHI, SE_JUMPVOL, SE_JUMPPAN,
  SE_JUMPSLD, SE_JUMPSPC, SE_CALL, SE_RETURN, SE_POSVAR, SE_LOAD, SE_ADD, SE_ADDX,
  SE_SUB, SE_SUBX, SE_CMP, SE_MULU, SE_MULS, SE_DMULU, SE_DMULS, SE_DIVU,
  SE_DIVS, SE_MODU, SE_MODS, SE_DDIVU, SE_DDIVS, SE_ASHL, SE_ASHR, SE_LSHL,
  SE_LSHR, SE_ROL, SE_ROR, SE_ROLX, SE_RORX, SE_OR, SE_AND, SE_XOR,
  SE_NOT, SE_NEG, SE_NEGX, SE_EXTB, SE_EXT, SE_XCHG, SE_SWAP, SE_GETWAVE,
  SE_GETWLEN, SE_GETWPOS, SE_GETCHAN, SE_GETNOTE, SE_GETRANS, SE_GETPTCH, SE_GETPER, SE_GETFX,
  SE_GETARPW, SE_GETARPV, SE_GETARPL, SE_GETARPP, SE_GETVIBW, SE_GETVIBV, SE_GETVIBL, SE_GETVIBP,
  SE_GETTRMW, SE_GETTRMV, SE_GETTRML, SE_GETTRMP, SE_GETPANW, SE_GETPANV, SE_GETPANL, SE_GETPANP,
  SE_GETRND, SE_GETSINE, SE_PORTAUP, SE_PORTADN, SE_VIBSPD, SE_VIBDPTH, SE_VIBWAVE, SE_VIBWAVP,
  SE_VIBRATO, SE_VIBVAL, SE_ARPSPD, SE_ARPWAVE, SE_ARPWAVP, SE_ARPEGIO, SE_ARPVAL, SE_SETWAVE,
  SE_ISETWAV, SE_SETWAVP, SE_SETRANS, SE_SETNOTE, SE_SETPTCH, SE_SETPER, SE_RESET, SE_VOLSLUP,
  SE_VOLSLDN, SE_TRMSPD, SE_TRMDPTH, SE_TRMWAVE, SE_TRMWAVP, SE_TREMOLO, SE_TRMVAL, SE_PANLEFT,
  SE_PANRGHT, SE_PANSPD, SE_PANDPTH, SE_PANWAVE, SE_PANWAVP, SE_PANNOLO, SE_PANVAL, SE_NOP
};
